package csbase.algorithms.flows;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Window;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.geom.AffineTransform;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.IOException;
import java.util.Collection;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import tecgraf.javautils.vix.Filter;
import tecgraf.javautils.vix.TypeMessage;
import tecgraf.javautils.vix.TypeVO;
import tecgraf.javautils.vix.TypeVS;
import csbase.algorithms.flows.configurator.FlowAlgorithmConfiguratorView;
import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.graph.GraphElement;
import csbase.algorithms.flows.graph.GraphFileDescriptor;
import csbase.algorithms.flows.graph.GraphLink;
import csbase.algorithms.flows.graph.GraphListener;
import csbase.algorithms.flows.graph.GraphNode;
import csbase.algorithms.flows.graph.Grid;
import csbase.algorithms.flows.messages.ChangeCursorMessage;
import csbase.algorithms.flows.messages.ErrorMessage;
import csbase.algorithms.flows.messages.HideHintMessage;
import csbase.algorithms.flows.messages.SelectElementMessage;
import csbase.algorithms.flows.messages.SelectElementsMessage;
import csbase.algorithms.flows.messages.ShowHintMessage;
import csbase.algorithms.flows.messages.ShowPopupMessage;
import csbase.algorithms.flows.zoom.AbstractZoomModel;
import csbase.algorithms.flows.zoom.ZoomModel;
import csbase.client.algorithms.AlgorithmConfiguratorFactory;
import csbase.client.algorithms.AlgorithmConfiguratorView;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.algorithms.AlgorithmVersionInfo;

/**
 * <p>
 * rea de trabalho.
 * </p>
 * 
 * <p>
 * A rea de trabalho  a regio onde so desenhados os elementos do grafo que
 * representa o fluxo de algoritmos. Ela  responsvel pelo controle de zoom e o
 * controle das barras de rolagem.
 * </p>
 * 
 * @author lmoreira
 */
public final class Workspace extends JPanel implements TypeVS {
  /**
   * Largura-padro.
   */
  private static final double DEFAULT_WIDTH = 400;

  /**
   * Altura-padro.
   */
  private static final double DEFAULT_HEIGHT = 300;

  /**
   * Margem.
   */
  private static final int MARGIN = 10;

  /** O grafo. */
  private Graph graph;

  /** O grid. */
  private final Grid grid;

  /** A matriz de transformao. */
  private final AffineTransform matrix;

  /** O vo. */
  private TypeVO vo;

  /** O modelo de zoom. */
  private final ZoomModel zoomModel;

  /**
   * Cria a rea de trabalho.
   * 
   * @param window A janela.
   * @param useGrid Indica se o grid deve ser utilizado.
   */
  public Workspace(Window window, boolean useGrid) {
    this(new Graph(window), useGrid);
  }

  /**
   * Cria a rea de trabalho.
   * 
   * @param graph grafo.
   * @param useGrid .
   */
  public Workspace(Graph graph, boolean useGrid) {
    this.matrix = new AffineTransform();
    this.graph = graph;
    this.zoomModel = new VSZoomModel();
    if (useGrid) {
      setBackground(Color.WHITE);
    }
    this.grid = new Grid(this);
    this.grid.setEnabled(useGrid);
    this.graph.changeVS(null, this.grid);
    changeVO(null, this.grid);
    this.grid.changeVO(null, this.graph);
    addComponentListener(new VSComponentListener());
    addMouseListener(new VSMouseListener());
    addMouseMotionListener(new VSMouseMotionListener());
    addKeyListener(new VSKeyListener());
    this.graph.addGraphListener(new VSGraphListener());
    new DropTarget(this, new VSDropTargetListener());
  }

  /**
   * Coloca o filtro fornecido como o vo atual da rea de trabalho.
   * 
   * @param filter O filtro.
   */
  public void attachFilter(Filter filter) {
    filter.changeVO(null, this.vo);
    filter.changeVS(null, this);
    this.vo.changeVS(null, filter);
    changeVO(null, filter);
  }

  /**
   * {@inheritDoc}
   */
  public boolean changeVO(TypeVO oldvo, TypeVO newvo) {
    if (oldvo == null) {
      this.vo = newvo;
      return true;
    }
    if (!oldvo.equals(this.vo)) {
      return false;
    }
    this.vo = newvo;
    return true;
  }

  /**
   * Obtm o grafo.
   * 
   * @return O grafo.
   */
  public Graph getGraph() {
    return this.graph;
  }

  /**
   * {@inheritDoc}
   */
  public Graphics2D getGraphics2D() {
    Graphics2D g = (Graphics2D) getGraphics();
    g.transform(this.matrix);
    return g;
  }

  /**
   * Obtm o modelo de zoom.
   * 
   * @return O modelo de zoom.
   */
  public ZoomModel getZoomModel() {
    return this.zoomModel;
  }

  /**
   * {@inheritDoc}
   */
  public boolean msgHandlerVS(TypeMessage msg) {
    if (msg instanceof ErrorMessage) {
      handleErrorMessage((ErrorMessage) msg);
      return true;
    }
    if (msg instanceof ShowPopupMessage) {
      handleShowPopupMessage((ShowPopupMessage) msg);
      return true;
    }
    if (msg instanceof ShowHintMessage) {
      handleShowHintMessage((ShowHintMessage) msg);
      return true;
    }
    if (msg instanceof HideHintMessage) {
      handleHideHintMessage();
      return true;
    }
    if (msg instanceof ChangeCursorMessage) {
      handleChangeCursorMessage((ChangeCursorMessage) msg);
      return true;
    }
    return false;
  }

  /**
   * Ajusta o tamanho da rea de trabalho para abrigar o grafo.
   */
  public void pack() {
    double width;
    double height;
    Rectangle2D bounds = this.graph.getBounds2D();
    if (bounds != null) {
      double zoomValue = getZoomModel().getValue();
      double tx = -bounds.getMinX() * zoomValue;
      double ty = -bounds.getMinY() * zoomValue;
      this.matrix.setTransform(zoomValue, 0.0, 0.0, zoomValue, tx, ty);
      width = bounds.getWidth() * zoomValue;
      height = bounds.getHeight() * zoomValue;
    }
    else {
      width = DEFAULT_WIDTH;
      height = DEFAULT_HEIGHT;
    }
    Dimension dimension = new Dimension();
    dimension.setSize(width, height);
    setPreferredSize(dimension);
  }

  /**
   * <p>
   * Envia uma mensagem ao vo atual desta rea de trabalho.
   * </p>
   * <p>
   * Seno existe um vo atual, ele no faz nada.
   * </p>
   * 
   * @param msg A mensagem.
   * 
   * @return <code>true</code> se a mensagem foi recebida, ou <code>false</code>
   *         se a mensagem no foi recebida (inclusive se no existir um vo
   *         atual).
   */
  public boolean sendVO(TypeMessage msg) {
    if (this.vo == null) {
      return false;
    }
    return msg.sendVO(this.vo);
  }

  /**
   * Ajusta o grafo.
   * 
   * @param graph O grafo.
   */
  public void setGraph(Graph graph) {
    graph.addGraphListeners(this.graph.getGraphListenerList());
    this.graph.clearGraphListeners();
    this.graph = graph;
    this.graph.changeVS(null, this.grid);
    this.grid.changeVO(null, this.graph);
    this.graph.setFullValidation(true);
    this.graph.validate(false);
    this.graph.setFullValidation(false);
    this.graph.notifyChangedWorkspace();
    this.graph.addGraphListener(new VSGraphListener());
    updateSize();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void paintChildren(Graphics g) {
    if (this.vo != null) {
      Graphics2D g2D = (Graphics2D) g;
      g2D.transform(this.matrix);
      this.vo.callbackRepaint((Graphics2D) g);
    }
  }

  /**
   * Obtm um texto no mecanismo de internacionalizao.
   * 
   * @param msgKey A chave da mensagem.
   * 
   * @return A mensagem.
   */
  private String getString(String msgKey) {
    return LNG.get(Workspace.class.getName() + "." + msgKey);
  }

  /**
   * Trata mensagens mudar cursor.
   * 
   * @param message A mensagem de mudana de cursor.
   */
  private void handleChangeCursorMessage(ChangeCursorMessage message) {
    setCursor(message.getCursor());
  }

  /**
   * Trata mensagens erro.
   * 
   * @param message A mensagem de erro.
   */
  private void handleErrorMessage(ErrorMessage message) {
    String key = message.getKey();
    if (key != null) {
      String errorMessage;
      Object[] arguments = message.getArguments();
      if (arguments == null) {
        errorMessage = LNG.get(key);
      }
      else {
        errorMessage = LNG.get(key, arguments);
      }
      showWarningDialog(errorMessage);
    }
    else {
      showErrorDialog(message.getException());
    }
  }

  /**
   * Trata mensagens ocultar dica.
   */
  private void handleHideHintMessage() {
    if (getToolTipText() != null) {
      setToolTipText(null);
    }
  }

  /**
   * Trata mensagens exibir dica.
   * 
   * @param message A mensagem.
   */
  private void handleShowHintMessage(ShowHintMessage message) {
    if (!message.getHint().equals(getToolTipText())) {
      setToolTipText(message.getHint());
    }
  }

  /**
   * Trata mensagens exibir menu popup.
   * 
   * @param message A mensagem.
   */
  private void handleShowPopupMessage(ShowPopupMessage message) {
    JPopupMenu popupMenu = new JPopupMenu();
    List<JComponent> components = message.getPopupComponentList();
    if (!components.isEmpty()) {
      for (JComponent component : components) {
        popupMenu.add(component);
      }
      Point2D point = message.getPoint();
      this.matrix.transform(point, point);
      popupMenu.show(this, (int) point.getX(), (int) point.getY());
    }
  }

  private void importAlgorithmConfigurator(
    AlgorithmConfiguratorView algorithmConfiguratorView, Point point) {
    GraphNode node =
      this.graph.createGraphNode(algorithmConfiguratorView, point, false);
    new SelectElementMessage(node, point).sendVO(this.vo);
  }

  private void importFlowAlgorithmConfigurator(
    FlowAlgorithmConfiguratorView flowAlgorithmConfiguratorView,
    Point dropPosition) {
    Graph graph = flowAlgorithmConfiguratorView.getGraph();
    Collection<GraphElement> elements = this.graph.merge(graph);
    new SelectElementsMessage(elements).sendVO(this.vo);
    Rectangle2D bounds = graph.getBounds2D();
    if (bounds != null) {
      double centerX = bounds.getCenterX();
      double centerY = bounds.getCenterY();
      double dx = dropPosition.getX() - centerX;
      double dy = dropPosition.getY() - centerY;
      for (GraphElement element : elements) {
        Rectangle2D elementBounds = element.getBounds2D();
        if (dx < 0) {
          dx = Math.max(dx, -1 * elementBounds.getMinX());
        }
        if (dy < 0) {
          dy = Math.max(dy, -1 * elementBounds.getMinY());
        }
      }
      for (GraphElement element : elements) {
        element.drag(dx, dy);
      }
      for (GraphElement element : elements) {
        element.drop();
      }
      repaint();
    }
  }

  /**
   * <p>
   * Inverte o ponto usando a matriz de transformao.
   * </p>
   * <p>
   * Ou seja, converte um ponto de mundo (grafo) para tela (rea de trabalho).
   * </p>
   * 
   * @param pt O ponto.
   */
  private void inverse(Point2D pt) {
    try {
      Workspace.this.matrix.inverseTransform(pt, pt);
    }
    catch (NoninvertibleTransformException e) {
      e.printStackTrace();
    }
  }

  /**
   * Atualiza as dimenses do grid.
   */
  private void resizeGrid() {
    if (this.grid != null) {
      double minX = 0.0;
      double minY = 0.0;
      double maxX = getBounds().getMaxX();
      double maxY = getBounds().getMaxY();
      Rectangle2D bounds = this.graph.getBounds2D();
      if (bounds != null) {
        maxX = Math.max(maxX, bounds.getMaxX() + MARGIN);
        maxY = Math.max(maxY, bounds.getMaxY() + MARGIN);
      }
      maxX *= 1 / this.zoomModel.getValue();
      maxY *= 1 / this.zoomModel.getValue();
      this.grid.setBounds2D(minX, minY, maxX, maxY);
      repaint();
    }
  }

  /**
   * Exibe um erro.
   * 
   * @param exception A exceo (No aceita {@code null}).
   */
  private void showErrorDialog(Exception exception) {
    Window window = SwingUtilities.getWindowAncestor(this);
    StandardErrorDialogs.showErrorDialog(window, getString("error"), exception);
  }

  /**
   * Exibe um aviso.
   * 
   * @param message A mensagem.
   */
  private void showWarningDialog(String message) {
    Window window = SwingUtilities.getWindowAncestor(this);
    StandardDialogs.showWarningDialog(window, getString("error"), message);
  }

  /**
   * Atualiza o tamanho da rea de trabalho.
   */
  private void updateSize() {
    Rectangle2D bounds = this.graph.getBounds2D();
    if (bounds != null) {
      Dimension newPreferredSize = new Dimension();
      double newWidth =
        (bounds.getX() + bounds.getWidth() + MARGIN)
          * this.zoomModel.getValue();
      double newHeight =
        (bounds.getY() + bounds.getHeight() + MARGIN)
          * this.zoomModel.getValue();
      newPreferredSize.setSize(newWidth, newHeight);
      setPreferredSize(newPreferredSize);
      revalidate();
    }
    resizeGrid();
  }

  /**
   * Listener que notifica eventos genricos de componente  rea de trabalho.
   */
  private final class VSComponentListener extends ComponentAdapter {
    /**
     * {@inheritDoc}
     */
    @Override
    public void componentResized(ComponentEvent e) {
      resizeGrid();
      repaint();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void componentShown(ComponentEvent e) {
      resizeGrid();
      repaint();
    }
  }

  /**
   * Observador de {@link DropTarget}.
   */
  private final class VSDropTargetListener implements DropTargetListener {
    /**
     * {@inheritDoc}
     */
    public void dragEnter(DropTargetDragEvent dtde) {
    }

    /**
     * {@inheritDoc}
     */
    public void dragExit(DropTargetEvent dte) {
    }

    /**
     * {@inheritDoc}
     */
    public void dragOver(DropTargetDragEvent dtde) {
      Transferable transferable = dtde.getTransferable();
      if (transferable
        .isDataFlavorSupported(AlgorithmVersionTransferable.DATA_FLAVOR)) {
        dtde.acceptDrag(DnDConstants.ACTION_COPY);
      }
      else {
        dtde.rejectDrag();
      }
    }

    /**
     * {@inheritDoc}
     */
    public void drop(DropTargetDropEvent dtde) {
      requestFocusInWindow();
      Point dropPosition = dtde.getLocation();
      inverse(dropPosition);
      Transferable transferable = dtde.getTransferable();
      if (!transferable
        .isDataFlavorSupported(AlgorithmVersionTransferable.DATA_FLAVOR)) {
        return;
      }
      AlgorithmVersionInfo algorithmVersionInfo;
      try {
        algorithmVersionInfo =
          (AlgorithmVersionInfo) transferable
            .getTransferData(AlgorithmVersionTransferable.DATA_FLAVOR);
      }
      catch (UnsupportedFlavorException e) {
        throw new IllegalStateException(e);
      }
      catch (IOException e) {
        showErrorDialog(e);
        return;
      }
      if (algorithmVersionInfo == null) {
        return;
      }
      Window window = SwingUtilities.getWindowAncestor(Workspace.this);
      AlgorithmConfiguratorView algorithmConfiguratorView =
        AlgorithmConfiguratorFactory.getInstance().createConfigurationView(
          window, algorithmVersionInfo);
      if (algorithmConfiguratorView == null) {
        return;
      }
      if (algorithmConfiguratorView instanceof FlowAlgorithmConfiguratorView) {
        FlowAlgorithmConfiguratorView flowAlgorithmConfiguratorView =
          (FlowAlgorithmConfiguratorView) algorithmConfiguratorView;
        importFlowAlgorithmConfigurator(flowAlgorithmConfiguratorView,
          dropPosition);
      }
      else {
        importAlgorithmConfigurator(algorithmConfiguratorView, dropPosition);
      }
    }

    /**
     * {@inheritDoc}
     */
    public void dropActionChanged(DropTargetDragEvent dtde) {
    }
  }

  /**
   * Observador de grafo.
   */
  private final class VSGraphListener implements GraphListener {
    /**
     * @param <V>
     * @param node
     * @param parameterName
     * @param parameterValue
     */
    public <V> void parameterValueWasChanged(GraphNode node,
      String parameterName, V parameterValue) {
    }

    /**
     * {@inheritDoc}
     */
    public void wasChangedWorkspace(Graph graph) {
    }

    /**
     * {@inheritDoc}
     */
    public void wasElementCreated(Graph graph, GraphElement element) {
      updateSize();
    }

    /**
     * {@inheritDoc}
     */
    public void wasElementDragged(Graph graph, GraphElement element, double tx,
      double ty) {
    }

    /**
     * {@inheritDoc}
     */
    public void wasElementDragged(Graph graph, GraphElement element,
      Point2D startPoint, Point2D endPoint) {
    }

    /**
     * {@inheritDoc}
     */
    public void wasElementDropped(Graph graph, GraphElement element,
      Point2D point) {
      updateSize();
    }

    /**
     * {@inheritDoc}
     */
    public void wasElementParametrized(Graph graph, GraphElement element) {
    }

    /**
     * {@inheritDoc}
     */
    public void wasElementRemoved(Graph graph, GraphElement element) {
      updateSize();
    }

    /**
     * {@inheritDoc}
     */
    public void wasElementSelected(Graph graph, GraphElement element) {
    }

    /**
     * {@inheritDoc}
     */
    public void wasLinkAnchored(Graph graph, GraphLink link,
      GraphFileDescriptor fileDescriptor) {
    }

    /**
     * {@inheritDoc}
     */
    public void wasLinkIncreased(Graph graph, GraphLink link) {
      updateSize();
    }

    /**
     * {@inheritDoc}
     */
    public void wasLinkStatusChanged(Graph graph, GraphLink link) {
    }

    /**
     * {@inheritDoc}
     */
    public void wasLinkUnanchored(Graph graph, GraphLink link,
      GraphFileDescriptor fileDescriptor) {
    }

    /**
     * {@inheritDoc}
     */
    public void wasNodeResized(Graph graph, GraphNode node) {
      updateSize();
    }

    /**
     * {@inheritDoc}
     */
    public void wasParameterSetEnabled(Graph graph, GraphNode node,
      String parameterName, boolean isEnabled) {
    }

    /**
     * {@inheritDoc}
     */
    public void wasReseted(Graph graph) {
    }

    /**
     * {@inheritDoc}
     */
    public void wasParameterSetVisible(Graph graph, GraphNode node,
      String parameterName, boolean isVisible) {

    }
  }

  /**
   * Listener que notifica eventos de teclas  rea de trabalho.
   */
  private final class VSKeyListener implements KeyListener {
    /**
     * {@inheritDoc}
     */
    public void keyPressed(KeyEvent ev) {
      if (Workspace.this.vo != null) {
        Workspace.this.vo.callbackKey(ev);
      }
    }

    /**
     * {@inheritDoc}
     */
    public void keyReleased(KeyEvent ev) {
      if (Workspace.this.vo != null) {
        Workspace.this.vo.callbackKey(ev);
      }
    }

    /**
     * {@inheritDoc}
     */
    public void keyTyped(KeyEvent ev) {
      if (Workspace.this.vo != null) {
        Workspace.this.vo.callbackKey(ev);
      }
    }
  }

  /**
   * Listener que notifica eventos de mouse  rea de trabalho.
   */
  private final class VSMouseListener extends MouseAdapter {
    /**
     * {@inheritDoc}
     */
    @Override
    public void mouseClicked(MouseEvent ev) {
      handleMouseButtonEvent(ev);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void mouseEntered(MouseEvent ev) {

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void mousePressed(MouseEvent ev) {
      handleMouseButtonEvent(ev);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void mouseReleased(MouseEvent ev) {
      handleMouseButtonEvent(ev);
    }

    /**
     * Trata eventos de boto do mouse.
     * 
     * @param ev O evento (No aceita {@code null}).
     */
    private void handleMouseButtonEvent(MouseEvent ev) {
      requestFocusInWindow();
      if (Workspace.this.vo != null) {
        Point2D pt = new Point2D.Float(ev.getX(), ev.getY());
        inverse(pt);
        Workspace.this.vo.callbackButton(pt, ev);
      }
    }
  }

  /**
   * Listener que notifica eventos de movimento de mouse  rea de trabalho.
   */
  private final class VSMouseMotionListener implements MouseMotionListener {
    /**
     * {@inheritDoc}
     */
    public void mouseDragged(MouseEvent ev) {
      final boolean isLeft = SwingUtilities.isLeftMouseButton(ev);
      if (isLeft && Workspace.this.vo != null) {
        Point2D pt = new Point2D.Double(ev.getX(), ev.getY());
        inverse(pt);
        Workspace.this.vo.callbackDrag(pt, ev);
      }
    }

    /**
     * {@inheritDoc}
     */
    public void mouseMoved(MouseEvent ev) {
      if (Workspace.this.vo != null) {
        Point2D pt = new Point2D.Double(ev.getX(), ev.getY());
        inverse(pt);
        Workspace.this.vo.callbackMove(pt, ev);
      }
    }
  }

  /**
   * Modelo de zoom para a rea de trabalho.
   */
  private final class VSZoomModel extends AbstractZoomModel {
    /**
     * A escala atual.
     */
    private double zoom;

    /**
     * Cria um modelo de zoom.
     */
    public VSZoomModel() {
      super(0.50, 4.00, 0.01, 0.10);
      this.zoom = 1.0;
    }

    /**
     * {@inheritDoc}
     */
    public double getValue() {
      return this.zoom;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void changeValue(double value) {
      Workspace.this.matrix.setTransform(value, 0.0, 0.0, value, 0.0, 0.0);
      this.zoom = value;
      updateSize();
      repaint();
    }
  }
}
