package csbase.algorithms.flows.filters;

import csbase.algorithms.flows.CompassDirection;
import csbase.algorithms.flows.Workspace;
import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.graph.GraphElement;
import csbase.algorithms.flows.graph.GraphNode;
import csbase.algorithms.flows.messages.ChangeCursorMessage;
import csbase.algorithms.flows.messages.PickNodeMessage;

import java.awt.Cursor;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;

public final class ResizeNodeFilter extends WorkspaceFilter {
  private static final int START = 0;
  private static final int WAITING_PRESS_TO_RESIZE_NODE = 1;
  private static final int WAITING_DRAG_TO_RESIZE_NODE = 2;
  private CompassDirection compassDirection;
  private GraphNode currentNode;
  private int currentState;

  public ResizeNodeFilter(Workspace workspace) {
    super(workspace);
    this.currentState = START;
  }

  @Override
  public void callbackButton(Point2D pt, MouseEvent ev) {
    boolean eventWasConsumed = false;
    switch (this.currentState) {
      case WAITING_PRESS_TO_RESIZE_NODE:
        if (wasPressed(ev)) {
          this.currentState = WAITING_DRAG_TO_RESIZE_NODE;
          eventWasConsumed = true;
        }
        break;
      case WAITING_DRAG_TO_RESIZE_NODE:
        if (wasReleased(ev)) {
          unmarkResizeNode();
        }
        break;
    }
    if (!eventWasConsumed) {
      super.callbackButton(pt, ev);
    }
  }

  @Override
  public void callbackDrag(Point2D pt, MouseEvent ev) {
    if (this.currentState == WAITING_DRAG_TO_RESIZE_NODE) {
      this.currentNode.resize(pt, this.compassDirection);
      repaint();
    }
    else {
      super.callbackDrag(pt, ev);
    }
  }

  @Override
  public void callbackMove(Point2D pt, MouseEvent ev) {
    switch (this.currentState) {
      case START:
        pickNode(pt);
        if (this.currentNode != null) {
          this.compassDirection = this.currentNode.getCompassDirection(pt);
          if (this.compassDirection != null) {
            changeCursor(getCursor());
            this.currentState = WAITING_PRESS_TO_RESIZE_NODE;
          }
          else {
            this.compassDirection = null;
            this.currentNode = null;
          }
        }
        break;
      case WAITING_PRESS_TO_RESIZE_NODE:
        boolean unmark = true;
        pickNode(pt);
        if (this.currentNode != null) {
          CompassDirection newCompassDirection =
            this.currentNode.getCompassDirection(pt);
          if (newCompassDirection != null) {
            if (!this.compassDirection.equals(newCompassDirection)) {
              this.compassDirection = newCompassDirection;
              changeCursor(getCursor());
            }
            unmark = false;
          }
        }
        if (unmark) {
          unmarkResizeNode();
        }
        break;
    }
    super.callbackMove(pt, ev);
  }

  @Override
  public void wasElementRemoved(Graph graph, GraphElement element) {
    reset();
  }

  @Override
  public void wasReseted(Graph graph) {
    reset();
  }

  private void changeCursor(Cursor cursor) {
    if (getVS() != null) {
      new ChangeCursorMessage(cursor).sendVS(getVS());
    }
  }

  private void pickNode(Point2D pt) {
    PickNodeMessage message = new PickNodeMessage(pt);
    message.sendVO(this);
    this.currentNode = message.getNode();
  }

  private void reset() {
    unmarkResizeNode();
    if (getVS() != null) {
      changeCursor(getDefaultCursor());
    }
  }

  private void unmarkResizeNode() {
    changeCursor(getDefaultCursor());
    this.compassDirection = null;
    this.currentNode = null;
    this.currentState = START;
  }

  private Cursor getCursor() {
    return Cursor.getPredefinedCursor(getCursorType());
  }

  private int getCursorType() {
    if (CompassDirection.NORTH.equals(this.compassDirection)) {
      return Cursor.N_RESIZE_CURSOR;
    }
    if (CompassDirection.SOUTH.equals(this.compassDirection)) {
      return Cursor.S_RESIZE_CURSOR;
    }
    if (CompassDirection.EAST.equals(this.compassDirection)) {
      return Cursor.E_RESIZE_CURSOR;
    }
    if (CompassDirection.WEST.equals(this.compassDirection)) {
      return Cursor.W_RESIZE_CURSOR;
    }
    if (CompassDirection.NORTH_EAST.equals(this.compassDirection)) {
      return Cursor.NE_RESIZE_CURSOR;
    }
    if (CompassDirection.NORTH_WEST.equals(this.compassDirection)) {
      return Cursor.NW_RESIZE_CURSOR;
    }
    if (CompassDirection.SOUTH_EAST.equals(this.compassDirection)) {
      return Cursor.SE_RESIZE_CURSOR;
    }
    if (CompassDirection.SOUTH_WEST.equals(this.compassDirection)) {
      return Cursor.SW_RESIZE_CURSOR;
    }
    return Cursor.DEFAULT_CURSOR;
  }

  private Cursor getDefaultCursor() {
    return Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);
  }

  private boolean wasPressed(MouseEvent ev) {
    return (ev.getButton() == MouseEvent.BUTTON1) &&
    (ev.getID() == MouseEvent.MOUSE_PRESSED);
  }

  private boolean wasReleased(MouseEvent ev) {
    return (ev.getButton() == MouseEvent.BUTTON1) &&
    (ev.getID() == MouseEvent.MOUSE_RELEASED);
  }
}
