package csbase.algorithms.flows.filters;

import csbase.algorithms.flows.RubberBand;
import csbase.algorithms.flows.Workspace;
import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.graph.GraphElement;
import csbase.algorithms.flows.graph.GraphFileDescriptor;
import csbase.algorithms.flows.graph.GraphLink;
import csbase.algorithms.flows.graph.GraphNode;
import csbase.algorithms.flows.messages.PickElementMessage;
import csbase.algorithms.flows.messages.PickGraphMessage;
import csbase.algorithms.flows.messages.PickLinkMessage;
import csbase.algorithms.flows.messages.PickNodeMessage;
import csbase.algorithms.flows.messages.SelectElementMessage;
import csbase.algorithms.flows.messages.SelectElementsMessage;

import tecgraf.javautils.vix.TypeMessage;

import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

public final class SelectElementFilter extends WorkspaceFilter {
  private static final int START = 0;
  private static final int WAITING_DRAG_TO_MOVE_ELEMENT = 1;
  private static final int MOVING_ELEMENT = 2;
  private static final int WAITING_DRAG_TO_RUBBER_BAND = 3;
  private static final int DRAGGING_RUBBER_BAND = 4;
  private static final int WAITING_SELECT_ONE_OF_GROUP = 5;
  private static final int WAITING_DRAG_TO_MOVE_MANY_ELEMENTS = 6;
  private static final int MOVING_MANY_ELEMENTS = 7;
  private static final int WAITING_FINISH_LINK = 8;
  private GraphElement currentElement;
  private GraphLink currentLink;
  private int currentState;
  private Point2D selectedPoint;
  private RubberBand rubberBand;
  private List<GraphElement> elementCollection;

  public SelectElementFilter(Workspace workspace) {
    super(workspace);
    this.currentState = START;
  }

  @Override
  public void callbackButton(Point2D pt, MouseEvent ev) {
    switch (this.currentState) {
    case WAITING_DRAG_TO_MOVE_ELEMENT: {
      if (wasPressed(ev)) {
        unselectCurrentElement(START);
      }
    }
    case START: {
      if (wasPressed(ev)) {
        PickNodeMessage pickNodeMessage = new PickNodeMessage(pt);
        pickNodeMessage.sendVO(this);
        GraphNode node = pickNodeMessage.getNode();
        if (node != null) {
          this.currentLink = node.createLink(pt);
          if (this.currentLink != null) {
            repaint();
            this.currentState = WAITING_FINISH_LINK;
            break;
          }
        }
        PickElementMessage pickElementMessage = new PickElementMessage(pt);
        pickElementMessage.sendVO(this);
        GraphElement element = pickElementMessage.getElement();
        if (element == null) {
          this.currentElement = null;
          this.currentState = WAITING_DRAG_TO_RUBBER_BAND;
          this.rubberBand = new RubberBand(pt);
          break;
        }
        this.currentElement = element;
        if (this.currentElement != null) {
          this.selectedPoint = (Point2D) pt.clone();
          this.currentElement.setSelected(true);
          repaint();
          this.currentState = WAITING_DRAG_TO_MOVE_ELEMENT;
          break;
        }
      }
      break;
    }
    case MOVING_ELEMENT: {
      if (wasReleased(ev)) {
        this.currentElement.drop(pt);
        unselectCurrentElement(START);
      }
      break;
    }
    case WAITING_DRAG_TO_RUBBER_BAND: {
      this.rubberBand = null;
      this.currentState = START;
      break;
    }
    case DRAGGING_RUBBER_BAND: {
      PickGraphMessage message = new PickGraphMessage();
      message.sendVO(this);
      Rectangle2D bounds = this.rubberBand.getBounds();
      this.elementCollection = new LinkedList<GraphElement>(message.getGraph()
        .getElementCollection(bounds));
      for (GraphElement element : this.elementCollection) {
        element.setSelected(true);
      }
      this.rubberBand = null;
      repaint();
      if (this.elementCollection.size() > 0) {
        this.currentState = WAITING_SELECT_ONE_OF_GROUP;
      }
      else {
        reset();
      }
      break;
    }
    case WAITING_SELECT_ONE_OF_GROUP: {
      if (wasPressed(ev)) {
        PickElementMessage pickElementMessage = new PickElementMessage(pt);
        pickElementMessage.sendVO(this);
        GraphElement element = pickElementMessage.getElement();
        if (element != null && this.elementCollection.contains(element)) {
          this.selectedPoint = (Point2D) pt.clone();
          this.currentState = WAITING_DRAG_TO_MOVE_MANY_ELEMENTS;
        }
        else {
          unselectElements(START);
          this.currentElement = element;
          if (this.currentElement != null) {
            this.selectedPoint = (Point2D) pt.clone();
            this.currentElement.setSelected(true);
            repaint();
            this.currentState = WAITING_DRAG_TO_MOVE_ELEMENT;
          }
          else {
            this.currentState = WAITING_DRAG_TO_RUBBER_BAND;
            this.rubberBand = new RubberBand(pt);
          }
        }
      }
      break;
    }
    case WAITING_DRAG_TO_MOVE_MANY_ELEMENTS: {
      this.currentState = WAITING_SELECT_ONE_OF_GROUP;
      break;
    }
    case MOVING_MANY_ELEMENTS: {
      Iterator<GraphElement> iterator = this.elementCollection.iterator();
      while (iterator.hasNext()) {
        GraphElement element = iterator.next();
        element.drop();
      }
      this.currentState = WAITING_SELECT_ONE_OF_GROUP;
      break;
    }
    case WAITING_FINISH_LINK: {
      PickNodeMessage message = new PickNodeMessage(pt);
      message.sendVO(this);
      GraphNode node = message.getNode();
      if (node != null) {
        GraphFileDescriptor inputFile = node.getInputFileDescriptor(pt);
        if (inputFile != null) {
          this.currentLink.increase(pt);
          if (this.currentLink.finish(inputFile)) {
            this.currentElement = this.currentLink;
            this.currentLink = null;
            this.selectedPoint = (Point2D) pt.clone();
            this.currentElement.setSelected(true);
            repaint();
            this.currentState = WAITING_DRAG_TO_MOVE_ELEMENT;
            break;
          }
        }
      }
      this.currentLink.deattach();
      this.currentLink = null;
      reset();
      repaint();
      this.currentState = START;
      break;
    }
    }
    super.callbackButton(pt, ev);
  }

  @Override
  public void callbackDrag(Point2D point, MouseEvent ev) {
    switch (this.currentState) {
    case WAITING_DRAG_TO_MOVE_ELEMENT:
    case MOVING_ELEMENT:
      this.currentElement.drag(this.selectedPoint, point);
      this.selectedPoint = new Point2D.Double(Math.max(0, point.getX()), Math
        .max(0, point.getY()));
      repaint();
      this.currentState = MOVING_ELEMENT;
      break;
    case DRAGGING_RUBBER_BAND:
    case WAITING_DRAG_TO_RUBBER_BAND:
      this.rubberBand.setPoint(point);
      repaint();
      this.currentState = DRAGGING_RUBBER_BAND;
      break;
    case MOVING_MANY_ELEMENTS:
    case WAITING_DRAG_TO_MOVE_MANY_ELEMENTS:
      double startX = this.selectedPoint.getX();
      double endX = point.getX();
      double startY = this.selectedPoint.getY();
      double endY = point.getY();
      double tx = endX - startX;
      double ty = endY - startY;
      Iterator<GraphElement> iterator = this.elementCollection.iterator();
      while (iterator.hasNext()) {
        GraphElement element = iterator.next();
        element.drag(tx, ty);
      }
      this.selectedPoint = new Point2D.Double(Math.max(0, point.getX()), Math
        .max(0, point.getY()));
      repaint();
      this.currentState = MOVING_MANY_ELEMENTS;
      break;
    case WAITING_FINISH_LINK:
      this.currentLink.pretentToIncrease(point);
      repaint();
      break;
    }
  }

  @Override
  public void callbackRepaint(Graphics2D g) {
    if (this.rubberBand != null) {
      this.rubberBand.paint(g);
    }
    super.callbackRepaint(g);
  }

  @Override
  public boolean msgHandlerVO(TypeMessage message) {
    if (message instanceof SelectElementMessage) {
      handleSelectElementMessage((SelectElementMessage) message);
      return true;
    }
    if (message instanceof SelectElementsMessage) {
      handleSelectElementsMessage((SelectElementsMessage) message);
      return true;
    }
    return super.msgHandlerVO(message);
  }

  @Override
  public void wasElementRemoved(Graph graph, GraphElement element) {
    reset();
  }

  @Override
  public void wasReseted(Graph graph) {
    reset();
  }

  private void handleSelectElementMessage(SelectElementMessage message) {
    reset();
    this.currentElement = message.getElement();
    this.currentElement.setSelected(true);
    this.selectedPoint = (Point2D) message.getPoint().clone();
    repaint();
    this.currentState = WAITING_DRAG_TO_MOVE_ELEMENT;
  }

  private void handleSelectElementsMessage(SelectElementsMessage message) {
    reset();
    List<GraphElement> elements = new LinkedList<GraphElement>(message
      .getElements());
    if (!elements.isEmpty()) {
      for (GraphElement element : elements) {
        element.setSelected(true);
      }
    }
    this.elementCollection = elements;
    repaint();
    this.currentState = WAITING_SELECT_ONE_OF_GROUP;
  }

  private void reset() {
    this.currentLink = null;
    unselectCurrentElement(START);
    unselectElements(START);
    unmarkRubberBand();
  }

  private void unmarkRubberBand() {
    this.rubberBand = null;
    this.currentState = START;
  }

  private void unselectCurrentElement(int nextState) {
    if (this.currentElement != null) {
      this.currentElement.setSelected(false);
      this.currentElement = null;
    }
    this.selectedPoint = null;
    repaint();
    this.currentState = nextState;
  }

  private void unselectElements(int nextState) {
    if (this.elementCollection != null) {
      Iterator<GraphElement> elementIterator = this.elementCollection
        .iterator();
      while (elementIterator.hasNext()) {
        GraphElement element = elementIterator.next();
        element.setSelected(false);
      }
      this.elementCollection = null;
    }
    this.selectedPoint = null;
    repaint();
    this.currentState = nextState;
  }

  private boolean wasPressed(MouseEvent ev) {
    return (ev.getButton() == MouseEvent.BUTTON1)
      && (ev.getID() == MouseEvent.MOUSE_PRESSED);
  }

  private boolean wasReleased(MouseEvent ev) {
    return (ev.getButton() == MouseEvent.BUTTON1)
      && (ev.getID() == MouseEvent.MOUSE_RELEASED);
  }
}
