package csbase.algorithms.flows.graph;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Window;
import java.awt.geom.Dimension2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.rmi.RemoteException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import tecgraf.javautils.LNG;
import tecgraf.javautils.vix.Group;
import tecgraf.javautils.vix.TypeMessage;
import tecgraf.javautils.vix.TypeVO;
import csbase.algorithms.flows.messages.PickGraphMessage;
import csbase.algorithms.flows.messages.RemoveElementsMessage;
import csbase.algorithms.flows.messages.ResetMessage;
import csbase.client.algorithms.AlgorithmConfiguratorFactory;
import csbase.client.algorithms.AlgorithmConfiguratorView;
import csbase.client.algorithms.parameters.validators.ValidationResult;
import csbase.exception.ParseException;
import csbase.exception.algorithms.ParameterNotFoundException;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.flows.Flow;
import csbase.logic.algorithms.flows.FlowLink;
import csbase.logic.algorithms.flows.FlowNode;
import csbase.logic.algorithms.flows.LinkParameter;
import csbase.logic.algorithms.flows.NodeParameter;
import csbase.logic.algorithms.flows.Point;
import csbase.logic.algorithms.parameters.FileParameter;
import csbase.remote.ClientRemoteLocator;

/**
 * <p>
 * O grafo  a representao grfica de um fluxo de algoritmo. Os ns do grafo
 * so a representao grfica das verses dos algoritmos. As arestras so as
 * ligaes entre os algoritmos. A ligao  unidirecional e representa o
 * encameamento de um arquivo de sada com um arquivo de entrada.
 * </p>
 * 
 * @author lmoreira
 */
public final class Graph extends Group implements GraphElementListener {
  /**
   * O ltimo identificador utilizado.
   */
  private int currentId;
  /**
   * Lista de observadores do grafo.
   */
  private final List<GraphListener> graphListenerList;
  /**
   * Indica o modo de operao da validao do parmetros do n, se  completa
   * ou no. Na validao completa, os parmetros obrigatrios precisam
   * estar preenchidos.
   */
  private boolean isFullValidation;
  /**
   * Janela onde o grafo ser desenhado.
   */
  private final Window window;
  /**
   * Nome do fluxo que foi utilizado para gerar o grafo.
   */
  private String flowName;

  /**
   * Constri um grafo.
   * 
   * @param window A janela onde o grafo ser desenhado.
   */
  public Graph(Window window) {
    if (window == null) {
      throw new IllegalArgumentException("O parmetro window est nulo.");
    }
    this.graphListenerList = new ArrayList<GraphListener>();
    this.window = window;
  }

  /**
   * Constri um grafo a partir de um fluxo.
   * 
   * @param window A janela onde o grafo ser desenhado.
   * @param flow O fluxo .
   * @throws ParseException
   * @throws RemoteException
   */
  public Graph(Window window, Flow flow) throws ParseException, RemoteException {
    this(window);
    flowName = flow.getName();
    createGraphNodes(flow);
    createGraphLinks(flow);
  }

  /**
   * Adiciona um observador de grafo.
   * 
   * @param listener O observador.
   */
  public void addGraphListener(GraphListener listener) {
    this.graphListenerList.add(listener);
  }

  /**
   * Adiciona um conjunto de observadores de grafo ao mesmo tempo.
   * 
   * @param listenerList O conjunto de observadores de grafo.
   */
  public void addGraphListeners(List<GraphListener> listenerList) {
    this.graphListenerList.addAll(listenerList);
  }

  /**
   * Remove todos os observadores de grafo.
   */
  public void clearGraphListeners() {
    this.graphListenerList.clear();
  }

  /**
   * Cria uma nova conexo no grafo entre um arquivo de sada de um n e um de
   * entrada de outro n, com o identificador especificado.
   * 
   * @param id Identificador da conexo.
   * @param inputFileDescriptor Arquivo de entrada da conexo.
   * @param outputFileDescriptor Arquivo de sada da conexo.
   * @param pointList Lista de pontos que a conexo percorre.
   */
  public void createGraphLink(int id, GraphFileDescriptor inputFileDescriptor,
    GraphFileDescriptor outputFileDescriptor, List<Point2D> pointList) {
    this.currentId = Math.max(id, this.currentId);
    GraphLink link = new GraphLink(this, id);
    link.attach();
    if (outputFileDescriptor != null) {
      link.increase(new Point2D.Double());
      link.setStart(outputFileDescriptor);
    }
    Iterator<Point2D> pointIterator = pointList.iterator();
    while (pointIterator.hasNext()) {
      Point2D point = pointIterator.next();
      link.increase(point);
    }
    if (inputFileDescriptor != null) {
      link.increase(new Point2D.Double());
      link.setEnd(inputFileDescriptor);
    }
  }

  /**
   * Cria uma nova conexo no grafo entre um arquivo de sada de um n e um de
   * entrada de outro n.
   * 
   * @param inputFileDescriptor Arquivo de entrada da conexo.
   * @param outputFileDescriptor Arquivo de sada da conexo.
   * @param pointList Lista de pontos que a conexo percorre.
   */
  public void createGraphLink(GraphFileDescriptor inputFileDescriptor,
    GraphFileDescriptor outputFileDescriptor, List<Point2D> pointList) {
    this.currentId++;
    createGraphLink(this.currentId, inputFileDescriptor, outputFileDescriptor,
      pointList);
  }

  /**
   * Cria um novo n no grafo a partir do configurador do algoritmo que ele
   * representa.
   * 
   * @param algorithmConfiguratorView A viso do configurador do algoritmo
   *        representado pelo n.
   * @param point Ponto onde o n deve ser colocado no grafo.
   * @param bypassed Indica se o n deve ser desviado.
   * @return O n criado.
   */
  public GraphNode createGraphNode(
    AlgorithmConfiguratorView algorithmConfiguratorView, Point2D point,
    boolean bypassed) {
    this.currentId++;
    GraphNode node =
      new GraphNode(this, this.currentId, algorithmConfiguratorView, point,
      bypassed);
    node.attach();
    return node;
  }

  /**
   * Cria um novo n no grafo a partir do configurador do algoritmo que ele
   * representa.
   * 
   * @param algorithmConfiguratorView A viso do configurador do algoritmo
   *        representado pelo n.
   * @param point Ponto onde o n deve ser colocado no grafo.
   * @param size Tamanho que o n deve ter.
   * @param bypassed Indica se o n deve ser desviado.
   * @return O n criado.
   */
  public GraphNode createGraphNode(
    AlgorithmConfiguratorView algorithmConfiguratorView, Point2D point,
    Dimension2D size, boolean bypassed) {
    this.currentId++;
    GraphNode node =
      new GraphNode(this, this.currentId, algorithmConfiguratorView, point,
      size, bypassed);
    node.attach();
    return node;
  }

  /**
   * Cria um novo n no grafo a partir do configurador do algoritmo que ele
   * representa, com o id especificado.
   * 
   * @param id Identificador do n.
   * @param configuratorView A viso do configurador do algoritmo
   *        representado pelo n.
   * @param point Ponto onde o n deve ser colocado no grafo.
   * @param size Tamanho que o n deve ter.
   * @param bypassed Indica se o n deve ser desviado.
   */
  public void createGraphNode(int id,
    AlgorithmConfiguratorView configuratorView, Point2D point,
    Dimension2D size, boolean bypassed) {
    this.currentId = Math.max(id, this.currentId);
    GraphNode node =
      new GraphNode(this, id, configuratorView, point, size, bypassed);
    node.attach();
  }

  /**
   * Cria um novo n no grafo a partir de informaes do algoritmo que ele
   * representa, com o id especificado. Usado nos casos que no  possvel obter
   * o configurador do algoritmo.
   * 
   * @param id Identificador do n.
   * @param algorithmInfo Informaes sobre o algoritmo.
   * @param versionId Informaes sobre a verso do algoritmo.
   * @param point Ponto onde o n deve ser colocado no grafo.
   * @param size Tamanho que o n deve ter.
   * @param bypassed Indica se o n deve ser desviado.
   */
  public void createGraphNode(int id, AlgorithmInfo algorithmInfo,
    AlgorithmVersionId versionId, Point2D point, Dimension2D size,
    boolean bypassed) {
    this.currentId = Math.max(id, this.currentId);
    GraphNode node =
      new GraphNode(this, id, algorithmInfo, versionId, point, size, bypassed);
    node.attach();
  }

  /**
   * Cria um novo n no grafo a partir de informaes do algoritmo que ele
   * representa, com o id especificado. Usado nos casos que no  possvel obter
   * o configurador do algoritmo.
   * 
   * @param id Identificador do n.
   * @param algorithmName Nome do algoritmo.
   * @param versionId Informaes sobre a verso do algoritmo.
   * @param point Ponto onde o n deve ser colocado no grafo.
   * @param size Tamanho que o n deve ter.
   * @param bypassed Indica se o n deve ser desviado.
   */
  public void createGraphNode(int id, String algorithmName,
    AlgorithmVersionId versionId, Point2D point, Dimension2D size,
    boolean bypassed) {
    this.currentId = Math.max(id, this.currentId);
    GraphNode node =
      new GraphNode(this, id, algorithmName, versionId, point, size, bypassed);
    node.attach();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Rectangle2D getBounds2D() {
    if (this.getVOs().isEmpty()) {
      return null;
    }
    TypeVO typeVO = getVOs().get(0);
    Rectangle2D bounds = (Rectangle2D) typeVO.getBounds2D().clone();
    for (int i = 1; i < getVOs().size(); i++) {
      typeVO = getVOs().get(i);
      bounds = bounds.createUnion(typeVO.getBounds2D());
    }
    bounds = Grid.includeMargin(bounds);
    return bounds;
  }

  /**
   * Retorna o conjunto de elementos do grafo.
   * 
   * @return o conjunto de elementos do grafo.
   */
  public Collection<GraphElement> getElementCollection() {
    return Collections.unmodifiableCollection(getVOs());
  }

  /**
   * Obtm a coleo dos elementos do grafo (<code>GraphElement</code>) que
   * contenham o ponto fornecido.
   * 
   * @param pt O ponto.
   * 
   * @return A coleo de elementos do grafo. Caso no haja elementos que
   *         contenham o ponto fornecido, retornar uma coleo vazia.
   */
  public Collection<GraphElement> getElementCollection(Point2D pt) {
    Collection<GraphElement> elementCollection = new LinkedList<GraphElement>();
    Iterator<GraphElement> elementIterator = getVOs().iterator();
    while (elementIterator.hasNext()) {
      GraphElement element = elementIterator.next();
      if (element.contains(pt)) {
        elementCollection.add(element);
      }
    }
    return elementCollection;
  }

  /**
   * Obtm a coleo dos elementos do grafo (<code>GraphElement</code>) que
   * esto contidos na regio limitada pelo retngulo fornecido.
   * 
   * @param bounds O retngulo.
   * 
   * @return A coleo de elementos do grafo. Caso no haja elementos que
   *         estejam contidos no retngulo fornecido, retornar uma coleo
   *         vazia.
   */
  public Collection<GraphElement> getElementCollection(Rectangle2D bounds) {
    Collection<GraphElement> elementCollection = new LinkedList<GraphElement>();
    Iterator<GraphElement> elementIterator = getVOs().iterator();
    while (elementIterator.hasNext()) {
      GraphElement element = elementIterator.next();
      if (bounds.contains(element.getBounds2D())) {
        elementCollection.add(element);
      }
    }
    return elementCollection;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Graphics2D getGraphics2D() {
    if (getVS() == null) {
      return null;
    }
    return getVS().getGraphics2D();
  }

  /**
   * Retorna uma lista de observadores do grafo.
   * 
   * @return A lista de observadores do grafo.
   */
  public List<GraphListener> getGraphListenerList() {
    return Collections.unmodifiableList(this.graphListenerList);
  }

  /**
   * Retorna a conexo com o identificador especificado ou nulo, caso no exista
   * conexo com esse identificador.
   * 
   * @param id O identificador da conexo.
   * @return A conexo com o identificador especificado ou nulo, caso no exista
   *         conexo com esse identificador.
   */
  public GraphLink getLink(int id) {
    for (GraphLink link : getLinkCollection()) {
      if (link.getId() == id) {
        return link;
      }
    }
    return null;
  }

  /**
   * Obtm a coleo dos conexes do grafo (<code>GraphLink</code>).
   * 
   * @return A coleo de conexes do grafo. Caso no haja conexes, retornar
   *         uma coleo vazia.
   */
  public Collection<GraphLink> getLinkCollection() {
    Collection<GraphLink> linkCollection = new LinkedList<GraphLink>();
    Iterator<GraphElement> elementIterator = getVOs().iterator();
    while (elementIterator.hasNext()) {
      GraphElement element = elementIterator.next();
      if (element instanceof GraphLink) {
        GraphLink graphLink = (GraphLink) element;
        linkCollection.add(graphLink);
      }
    }
    return linkCollection;
  }

  /**
   * Obtm o nome do fluxo representado por este grafo.
   * 
   * @return o nome do fluxo representado por este grafo. Pode ser {@code null}.
   */
  public String getName() {
    return flowName;
  }

  /**
   * Retorna o n com o identificador especificado ou nulo, caso no exista
   * n com esse identificador.
   * 
   * @param id O identificador do n.
   * @return O n com o identificador especificado ou nulo, caso no exista
   *         n com esse identificador.
   */
  public GraphNode getNode(int id) {
    Collection<GraphNode> nodeCollection = getNodeCollection();
    Iterator<GraphNode> nodeIterator = nodeCollection.iterator();
    while (nodeIterator.hasNext()) {
      GraphNode node = nodeIterator.next();
      if (node.getId() == id) {
        return node;
      }
    }
    return null;
  }

  /**
   * Obtm um determinado n do grafo, a partir do nome do algoritmo que ele
   * representa.
   * 
   * @param nodeName nome do algoritmo do n
   * @param nodeVersionId o identificador da verso do algoritmo do n
   * @return o n procurado
   */
  public GraphNode getNode(String nodeName, AlgorithmVersionId nodeVersionId) {
    Collection<GraphNode> nodeCollection = getNodeCollection();
    Iterator<GraphNode> nodeIterator = nodeCollection.iterator();
    while (nodeIterator.hasNext()) {
      GraphNode node = nodeIterator.next();
      if (node.getAlgorithmName().equals(nodeName)
        && node.getAlgorithmVersionId().equals(nodeVersionId)) {
        return node;
      }
    }
    return null;
  }

  /**
   * Obtm a coleo dos ns do grafo (<code>GraphNode</code>).
   * 
   * @return A coleo de ns do grafo. Caso no haja ns, retornar uma coleo
   *         vazia.
   */
  public Collection<GraphNode> getNodeCollection() {
    Collection<GraphNode> nodeCollection = new LinkedList<GraphNode>();
    Iterator<GraphElement> elementIterator = getVOs().iterator();
    while (elementIterator.hasNext()) {
      Object obj = elementIterator.next();
      if (obj instanceof GraphNode) {
        GraphNode node = (GraphNode) obj;
        nodeCollection.add(node);
      }
    }
    return nodeCollection;
  }

  /**
   * Retorna a janela onde o grafo  desenhado.
   * 
   * @return A janela onde o grafo  desenhado.
   */
  public Window getParentWindow() {
    return this.window;
  }

  /**
   * Retorna o conjunto de itens do grafo que esto selecionados.
   * 
   * @return O conjunto de itens selecionados.
   */
  public Set<GraphElement> getSelectedElements() {
    Set<GraphElement> selectedElements = new HashSet<GraphElement>();
    for (GraphElement element : getVOs()) {
      if (element.isSelected()) {
        selectedElements.add(element);
      }
    }
    return Collections.unmodifiableSet(selectedElements);
  }

  /**
   * {@inheritDoc}
   */
  @SuppressWarnings("unchecked")
  @Override
  public List<GraphElement> getVOs() {
    return super.getVOs();
  }

  /**
   * Verifica se o grafo possui elementos.
   * 
   * @return <code>true</code> se o grafo possui elementos ou <code>false</code>
   *         caso contrrio.
   */
  public boolean hasElements() {
    return !getVOs().isEmpty();
  }

  /**
   * Verifica se o grafo possui elementos selecionados.
   * 
   * @return <code>true</code> se o grafo possui elementos selecionados ou
   *         <code>false</code> caso contrrio.
   */
  public boolean hasSelectedElements() {
    Iterator<GraphElement> elementIterator = getVOs().iterator();
    while (elementIterator.hasNext()) {
      GraphElement element = elementIterator.next();
      if (element.isSelected()) {
        return true;
      }
    }
    return false;
  }

  /**
   * <p>
   * Verifica se o grafo representa um fluxo de algoritmos que pode ser
   * executado.
   * </p>
   * <p>
   * Condies para um grafo ser executvel:
   * <ul>
   * <li>Ter ns;</li>
   * <li>Ter apenas um n de entrada (N que no possue arestas de chegada);</li>
   * <li>Todas as arestas devem estar bem formadas (Todas as arestas comeam em
   * um arquivo de sada e terminam em um arquivo de entrada);</li>
   * </ul>
   * </p>
   * 
   * @return <code>true</code> se  executvel ou <code>false</code> caso
   *         contrrio.
   */
  public boolean isExecutable() {
    if (getVOs().isEmpty()) {
      return false;
    }
    for (GraphLink link : getLinkCollection()) {
      if (!link.isWellFormed()) {
        return false;
      }
    }
    for (GraphNode node : getNodeCollection()) {
      if (!node.isWellFormed()) {
        return false;
      }
    }
    return true;
  }

  /**
   * <p>
   * Verifica o modo de operao da validao do parmetros do n, se  completa
   * ou no.
   * </p>
   * 
   * @return <code>true</code> se  completa ou <code>false</code> caso
   *         contrrio.
   */
  public boolean isFullValidation() {
    return this.isFullValidation;
  }

  /**
   * <p>
   * Verifica se o grafo est pronto para ser executado.
   * </p>
   * <p>
   * Condies para um grafo estar pronto para ser executado:
   * <ul>
   * <li>Ser executvel;</li>
   * <li>Todos os ns tm estar prontos para serem executados (parametrizados)</li>
   * </ul>
   * </p>
   * 
   * @return <code>true</code> se est pronto para ser executado ou
   *         <code>false</code> caso contrrio.
   */
  public boolean isReadyToExecute() {
    if (!isExecutable()) {
      return false;
    }
    Iterator<?> iterator = getVOs().iterator();
    while (iterator.hasNext()) {
      GraphElement element = (GraphElement) iterator.next();
      if (element instanceof GraphNode) {
        GraphNode node = (GraphNode) element;
        if (!node.isReadyToExecute()) {
          return false;
        }
      }
    }
    return true;
  }

  /**
   * Faz a juno deste grafo com o outro especificado.
   * 
   * @param graph O grafo que deve ser combinado com esse.
   * @return A coleo de elementos dos dois grafos combinados.
   */
  public Collection<GraphElement> merge(Graph graph) {
    if (graph == null) {
      throw new IllegalArgumentException("O parmetro graph est nulo.");
    }
    List<GraphElement> elements = new LinkedList<GraphElement>();
    Map<GraphFileDescriptor, GraphFileDescriptor> graphFileDescriptors =
      new HashMap<GraphFileDescriptor, GraphFileDescriptor>();
    for (GraphNode node : graph.getNodeCollection()) {
      this.currentId++;
      AlgorithmConfiguratorView algorithmConfiguratorView =
        node.getAlgorithmConfiguratorView();
      Point2D.Double nodePoint = new Point2D.Double(node.getX(), node.getY());
      Dimension nodeSize = new Dimension();
      nodeSize.setSize(node.getWidth(), node.getHeight());
      boolean bypassed = node.isBypassed();
      if (node.isWellFormed()) {
        createGraphNode(this.currentId, algorithmConfiguratorView, nodePoint,
          nodeSize, bypassed);
      }
      else if (node.getAlgorithm() != null) {
        createGraphNode(this.currentId, node.getAlgorithm(), node
          .getAlgorithmVersionId(), nodePoint, nodeSize, bypassed);
      }
      else {
        createGraphNode(this.currentId, node.getAlgorithmName(), node
          .getAlgorithmVersionId(), nodePoint, nodeSize, bypassed);
      }
      GraphNode newNode = getNode(this.currentId);
      for (GraphFileDescriptor fileDescriptor : node
        .getInputFileDescriptorCollection()) {
        GraphFileDescriptor newFileDescriptor =
          newNode.getInputFileDescriptor(fileDescriptor.getParameterName());
        if (newFileDescriptor == null) {
          newFileDescriptor =
            new GraphFileDescriptor(fileDescriptor.getParameterName(),
            fileDescriptor.getParameterLabel(), false, newNode);
          newNode.addInputFileDescriptor(newFileDescriptor);
        }
        graphFileDescriptors.put(fileDescriptor, newFileDescriptor);
      }
      for (GraphFileDescriptor fileDescriptor : node
        .getOutputFileDescriptorCollection()) {
        GraphFileDescriptor newFileDescriptor =
          newNode.getOutputFileDescriptor(fileDescriptor.getParameterName());
        if (newFileDescriptor == null) {
          newFileDescriptor =
            new GraphFileDescriptor(fileDescriptor.getParameterName(),
            fileDescriptor.getParameterLabel(), true, newNode);
          newNode.addOutputFileDescriptor(newFileDescriptor);
        }
        graphFileDescriptors.put(fileDescriptor, newFileDescriptor);
      }
      try {
        Map<String, String> parameterValuesByName =
          new HashMap<String, String>();
        for (String parameterName : node.getParameterNames()) {
          String parameterLabel = node.getParameterLabel(parameterName);
          String parameterType = node.getParameterType(parameterName);
          String parameterValue = node.getParameterValue(parameterName);
          newNode.addParameter(parameterName, parameterLabel, parameterType);
          parameterValuesByName.put(parameterName, parameterValue);
        }
        newNode.setParameterValuesByName(Collections
          .unmodifiableMap(parameterValuesByName));
      }
      catch (ParameterNotFoundException e1) {
        IllegalStateException e2 =
          new IllegalStateException(e1.getLocalizedMessage());
        e2.initCause(e1);
        throw e2;
      }
      catch (ParseException e1) {
        IllegalStateException e2 =
          new IllegalStateException(e1.getLocalizedMessage());
        e2.initCause(e1);
        throw e2;
      }
      elements.add(newNode);
    }
    for (GraphLink link : graph.getLinkCollection()) {
      this.currentId++;
      GraphFileDescriptor inputFileDescriptor =
        graphFileDescriptors.get(link.getInputFileDescriptor());
      GraphFileDescriptor outputFileDescriptor =
        graphFileDescriptors.get(link.getOutputFileDescriptor());
      List<Point2D> pointList = link.getPointList();
      List<Point2D> points = new ArrayList<Point2D>(pointList);
      if (inputFileDescriptor != null) {
        points.remove(points.size() - 1);
      }
      if (outputFileDescriptor != null) {
        points.remove(0);
      }
      createGraphLink(this.currentId, inputFileDescriptor,
        outputFileDescriptor, points);
      GraphLink newLink = getLink(this.currentId);
      elements.add(newLink);
    }
    return elements;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean msgHandlerVO(TypeMessage msg) {
    if (msg instanceof ResetMessage) {
      handleResetMessage();
      return true;
    }
    if (msg instanceof RemoveElementsMessage) {
      return handleRemoveElementsMessage();
    }
    if (msg instanceof PickGraphMessage) {
      handlePickGraphMessage((PickGraphMessage) msg);
      return true;
    }
    return super.msgHandlerVO(msg);
  }

  /**
   * Notifica os observadores sobre mudanas no workspace.
   */
  public void notifyChangedWorkspace() {
    for (int i = 0; i < this.graphListenerList.size(); i++) {
      GraphListener graphListener = this.graphListenerList.get(i);
      graphListener.wasChangedWorkspace(this);
    }
  }

  /**
   * Remove todos os observadores do grafo cadastrados.
   */
  public void removeAllListeners() {
    this.graphListenerList.clear();
    for (Object obj : getElementCollection()) {
      GraphElement element = (GraphElement) obj;
      element.removeAllListeners();
    }
  }

  /**
   * Remove um observador de grafo.
   * 
   * @param listener O observador.
   */
  public void removeGraphListener(GraphListener listener) {
    this.graphListenerList.remove(listener);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void repaint() {
    if (getVS() != null) {
      super.repaint();
    }
  }

  /**
   * Atribui o modo de operao da validao do parmetros do n, se  completa
   * ou no.
   * 
   * @param isFullValidation verdadeito se a validao  completa ou falso, caso
   *        contrrio.
   */
  public void setFullValidation(boolean isFullValidation) {
    this.isFullValidation = isFullValidation;
  }

  /**
   * Gera um fluxo que corresponde a este grafo.
   * 
   * @return O fluxo gerado.
   */
  public Flow toFlow() {
    Set<FlowNode> flowNodes = new HashSet<FlowNode>();
    for (GraphNode graphNode : getNodeCollection()) {
      Set<NodeParameter> flowNodeParameters = new HashSet<NodeParameter>();
      for (String parameterName : graphNode.getParameterNames()) {
        String parameterLabel;
        try {
          parameterLabel = graphNode.getParameterLabel(parameterName);
          String parameterType = graphNode.getParameterType(parameterName);
          String parameterValue = graphNode.getParameterValue(parameterName);
          flowNodeParameters.add(new NodeParameter(parameterName,
            parameterLabel, parameterType, parameterValue));
        }
        catch (ParameterNotFoundException e) {
          throw new IllegalStateException(e);
        }

      }
      FlowNode flowNode;
      if (graphNode.getAlgorithmConfiguratorView() == null) {
        flowNode =
          new FlowNode(graphNode.getId(), graphNode.getAlgorithmName(),
          graphNode.getAlgorithmVersionId(), flowNodeParameters, (int) Math
          .round(graphNode.getX()), (int) Math.round(graphNode.getY()),
          (int) Math.round(graphNode.getWidth()), (int) Math.round(graphNode
          .getHeight()), graphNode.isBypassed(), graphNode.hasExitCode());
      }
      else {
        flowNode =
          new FlowNode(graphNode.getId(), graphNode
          .getAlgorithmConfiguratorView().getConfigurator(),
          flowNodeParameters, (int) Math.round(graphNode.getX()), (int) Math
          .round(graphNode.getY()), (int) Math.round(graphNode.getWidth()),
          (int) Math.round(graphNode.getHeight()), graphNode.isBypassed());
      }
      flowNode.setStandardOutputFile(graphNode.getStandardOutputFile());
      flowNode.setExitCodeLogFile(graphNode.getExitCodeLogFile());
      flowNode.setHasExitCode(graphNode.hasExitCode());
      flowNodes.add(flowNode);
    }
    Set<FlowLink> flowLinks = new HashSet<FlowLink>();
    for (GraphLink graphLink : getLinkCollection()) {
      List<Point> points = new LinkedList<Point>();
      for (Point2D point2D : graphLink.getPointList()) {
        points.add(new Point((int) Math.round(point2D.getX()), (int) Math
          .round(point2D.getY())));
      }
      LinkParameter inputLinkParameter = null;
      GraphFileDescriptor inputFileDescriptor =
        graphLink.getInputFileDescriptor();
      if (inputFileDescriptor != null) {
        GraphNode inputGraphNode = inputFileDescriptor.getNode();
        if (inputGraphNode != null) {
          inputLinkParameter =
            new LinkParameter(inputGraphNode.getId(), inputFileDescriptor
            .getParameterName(), inputFileDescriptor.getParameterLabel());
          points.remove(points.size() - 1);
        }
      }
      LinkParameter outputLinkParameter = null;
      GraphFileDescriptor outputFileDescriptor =
        graphLink.getOutputFileDescriptor();
      if (outputFileDescriptor != null) {
        GraphNode outputGraphNode = outputFileDescriptor.getNode();
        if (outputGraphNode != null) {
          outputLinkParameter =
            new LinkParameter(outputGraphNode.getId(), outputFileDescriptor
            .getParameterName(), outputFileDescriptor.getParameterLabel());
          points.remove(0);
        }
      }
      FlowLink flowLink =
        new FlowLink(graphLink.getId(), outputLinkParameter,
        inputLinkParameter, points);
      flowLinks.add(flowLink);
    }
    return new Flow(flowName, flowNodes, flowLinks);
  }

  /**
   * Valida a configurao do grafo, procurando erros e inconsistncias.
   * 
   * @param complain Indica se deve exibir dilogos quando houver um problema.
   * @return O resultado da validao.
   */
  public ValidationResult validate(boolean complain) {
    Collection<GraphNode> nodeCollection = getNodeCollection();
    if (nodeCollection.isEmpty()) {
      return new ValidationResult(LNG.get(Graph.class.getName()
        + ".error_empty_graph"));
    }
    Iterator<GraphNode> nodeIterator = nodeCollection.iterator();
    ValidationResult validation = new ValidationResult();
    while (nodeIterator.hasNext()) {
      GraphNode node = nodeIterator.next();
      ValidationResult nodeValidation =
        node.validate(complain, isFullValidation());
      if (!nodeValidation.isWellSucceded()) {
        complain = false;
        validation = new ElementValidationResult<GraphNode>(nodeValidation
          .getErrorMessage(),
          node);
      }
    }
    return validation;
  }

  /**
   * {@inheritDoc}
   */
  public void wasAnchored(GraphLink link, GraphFileDescriptor fileDescriptor) {
    Iterator<GraphListener> graphListenerIterator =
      this.graphListenerList.iterator();
    while (graphListenerIterator.hasNext()) {
      GraphListener graphListener =
        graphListenerIterator.next();
      graphListener.wasLinkAnchored(this, link, fileDescriptor);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void wasDragged(GraphElement element, double tx, double ty) {
    Iterator<GraphListener> graphListenerIterator =
      this.graphListenerList.iterator();
    while (graphListenerIterator.hasNext()) {
      GraphListener graphListener =
        graphListenerIterator.next();
      graphListener.wasElementDragged(this, element, tx, ty);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void wasDragged(GraphElement element, Point2D startPoint,
    Point2D endPoint) {
    Iterator<GraphListener> graphListenerIterator =
      this.graphListenerList.iterator();
    while (graphListenerIterator.hasNext()) {
      GraphListener graphListener =
        graphListenerIterator.next();
      graphListener.wasElementDragged(this, element, startPoint, endPoint);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void wasDropped(GraphElement element, Point2D point) {
    Iterator<GraphListener> graphListenerIterator =
      this.graphListenerList.iterator();
    while (graphListenerIterator.hasNext()) {
      GraphListener graphListener =
        graphListenerIterator.next();
      graphListener.wasElementDropped(this, element, point);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void wasIncreased(GraphLink link) {
    Iterator<GraphListener> graphListenerIterator =
      this.graphListenerList.iterator();
    while (graphListenerIterator.hasNext()) {
      GraphListener graphListener =
        graphListenerIterator.next();
      graphListener.wasLinkIncreased(this, link);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void wasLinkStatusChanged(GraphLink link) {
    Iterator<GraphListener> graphListenerIterator =
      this.graphListenerList.iterator();
    while (graphListenerIterator.hasNext()) {
      GraphListener graphListener =
        graphListenerIterator.next();
      graphListener.wasLinkStatusChanged(this, link);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void wasParameterSetEnabled(GraphNode node, String parameterName,
    boolean isEnabled) {
    Iterator<GraphListener> graphListenerIterator =
      this.graphListenerList.iterator();
    while (graphListenerIterator.hasNext()) {
      GraphListener graphListener =
        graphListenerIterator.next();
      graphListener
        .wasParameterSetEnabled(this, node, parameterName, isEnabled);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void wasParameterSetVisible(GraphNode node, String parameterName,
    boolean isVisible) {
    for (GraphListener listener : graphListenerList) {
      listener.wasParameterSetVisible(this, node, parameterName, isVisible);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void wasParametrized(GraphElement element) {
    Iterator<GraphListener> graphListenerIterator =
      this.graphListenerList.iterator();
    while (graphListenerIterator.hasNext()) {
      GraphListener graphListener =
        graphListenerIterator.next();
      graphListener.wasElementParametrized(this, element);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void wasRenamed(GraphNode node) {
  }

  /**
   * {@inheritDoc}
   */
  public void wasResized(GraphNode node) {
    Iterator<GraphListener> graphListenerIterator =
      this.graphListenerList.iterator();
    while (graphListenerIterator.hasNext()) {
      GraphListener graphListener =
        graphListenerIterator.next();
      graphListener.wasNodeResized(this, node);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void wasSelected(GraphElement element) {
    if (element.isSelected()) {
      bringToFront(element);
    }
    Iterator<GraphListener> graphListenerIterator =
      this.graphListenerList.iterator();
    while (graphListenerIterator.hasNext()) {
      GraphListener graphListener =
        graphListenerIterator.next();
      graphListener.wasElementSelected(this, element);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void wasUnanchored(GraphLink link, GraphFileDescriptor fileDescriptor) {
    Iterator<GraphListener> graphListenerIterator =
      this.graphListenerList.iterator();
    while (graphListenerIterator.hasNext()) {
      GraphListener graphListener =
        graphListenerIterator.next();
      graphListener.wasLinkUnanchored(this, link, fileDescriptor);
    }
  }

  /**
   * Adiciona o elemento especificado ao grafo.
   * 
   * @param element O elemento a ser adicionado.
   * @return verdadeiro se o elemento foi adicionado ou falso, caso o elemento
   *         j esteja no grafo.
   */
  boolean addElement(GraphElement element) {
    if (getVOs().contains(element)) {
      return false;
    }
    insertElement(element);
    element.addListener(this);
    for (int i = 0; i < this.graphListenerList.size(); i++) {
      GraphListener graphListener = this.graphListenerList.get(i);
      graphListener.wasElementCreated(this, element);
    }
    return true;
  }

  /**
   * <p>
   * Traz para frente.
   * </p>
   * <p>
   * Faz com que o elemento do grafo fornecido fique na frente de todos os
   * outros.
   * </p>
   * 
   * @param element O elemento.
   */
  void bringToFront(GraphElement element) {
    getVOs().remove(element);
    insertElement(element);
  }

  /**
   * Cria uma aresta.
   * 
   * @param outputFileType O arquivo de sada que origina esta conexo.
   * 
   * @return A aresta.
   */
  GraphLink createLink(GraphFileDescriptor outputFileType) {
    this.currentId++;
    GraphLink link = new GraphLink(this, this.currentId);
    link.setSelected(true);
    link.attach();
    link.increase(new Point2D.Double());
    link.start(outputFileType);
    return link;
  }

  /**
   * Remove um elemento do grafo.
   * 
   * @param element elemento do grafo.
   */
  void removeElement(GraphElement element) {
    getVOs().remove(element);
    for (int i = 0; i < this.graphListenerList.size(); i++) {
      GraphListener graphListener = this.graphListenerList.get(i);
      graphListener.wasElementRemoved(this, element);
    }
  }

  /**
   * Cria uma conexo no grafo com as mesmas caractersticas da conexo de fluxo
   * especificada.
   * 
   * @param flowLink A conexo de fluxo.
   */
  private void createGraphLink(FlowLink flowLink) {
    GraphFileDescriptor inputFileDescriptor = null;
    LinkParameter inputLinkParameter = flowLink.getInput();
    if (inputLinkParameter != null) {
      GraphNode inputGraphNode = getNode(inputLinkParameter.getNodeId());
      inputFileDescriptor =
        inputGraphNode.getInputFileDescriptor(inputLinkParameter.getName());
      if (inputFileDescriptor == null) {
        inputFileDescriptor =
          new GraphFileDescriptor(inputLinkParameter.getName(),
          inputLinkParameter.getLabel(), false, inputGraphNode);
        inputGraphNode.addInputFileDescriptor(inputFileDescriptor);
      }
      /*
       * Seta o valor do parmetro como nulo, para evitar que se tenha o link e
       * um arquivo escolhido manualmente no configurador ao mesmo tempo,
       * conflitando.
       */
      FileParameter fileParameter = inputFileDescriptor.getFileParameter();
      if (fileParameter != null) {
        fileParameter.setValue(null);
      }
    }
    GraphFileDescriptor outputFileDescriptor = null;
    LinkParameter outputLinkParameter = flowLink.getOutput();
    if (outputLinkParameter != null) {
      GraphNode outputGraphNode = getNode(outputLinkParameter.getNodeId());
      outputFileDescriptor =
        outputGraphNode.getOutputFileDescriptor(outputLinkParameter.getName());
      if (outputFileDescriptor == null) {
        outputFileDescriptor =
          new GraphFileDescriptor(outputLinkParameter.getName(),
          outputLinkParameter.getLabel(), true, outputGraphNode);
        outputGraphNode.addOutputFileDescriptor(outputFileDescriptor);
      }
      /*
       * Seta o valor do parmetro como nulo, para evitar que se tenha o link e
       * um arquivo escolhido manualmente no configurador ao mesmo tempo,
       * conflitando.
       */
      FileParameter fileParameter = outputFileDescriptor.getFileParameter();
      if (fileParameter != null) {
        fileParameter.setValue(null);
      }
    }
    List<Point2D> points = new LinkedList<Point2D>();
    for (Point point : flowLink.getPoints()) {
      points.add(new Point2D.Double(point.getX(), point.getY()));
    }
    createGraphLink(flowLink.getId(), inputFileDescriptor,
      outputFileDescriptor, points);
  }

  /**
   * Cria as conexes do grafo com as mesmas caractersticas das conexes do
   * fluxo especificado.
   * 
   * @param flow O fluxo.
   */
  private void createGraphLinks(Flow flow) {
    for (FlowLink flowLink : flow.getLinks()) {
      createGraphLink(flowLink);
    }
  }

  /**
   * Cria um n no grafo que tenha as mesmas caractersticas do n especificado
   * de fluxo.
   * 
   * @param flowNode O n do fluxo.
   * @throws ParseException
   * @throws RemoteException
   */
  private void createGraphNode(FlowNode flowNode) throws ParseException,
    RemoteException {
    Point2D nodePoint = new Point2D.Double(flowNode.getX(), flowNode.getY());
    Dimension2D nodeDimension =
      new Dimension(flowNode.getWidth(), flowNode.getHeight());
    AlgorithmInfo nodeAlgorithm =
      ClientRemoteLocator.algorithmService.getInfo(flowNode.getAlgorithmName());
    boolean bypassed = flowNode.isBypassed();
    if (nodeAlgorithm == null) {
      createGraphNode(flowNode.getId(), flowNode.getAlgorithmName(), flowNode
        .getAlgorithmVersionId(), nodePoint, nodeDimension, bypassed);
    }
    else {
      AlgorithmVersionInfo nodeAlgorithmVersion =
        nodeAlgorithm.getVersionInfo(flowNode.getAlgorithmVersionId());
      if (nodeAlgorithmVersion == null) {
        createGraphNode(flowNode.getId(), nodeAlgorithm, flowNode
          .getAlgorithmVersionId(), nodePoint, nodeDimension, bypassed);
      }
      else {
        AlgorithmConfiguratorView nodeConfiguratorView =
          AlgorithmConfiguratorFactory.getInstance().createConfigurationView(
          this.window, nodeAlgorithmVersion);
        if (nodeConfiguratorView == null) {
          return;
        }
        createGraphNode(flowNode.getId(), nodeConfiguratorView, nodePoint,
          nodeDimension, bypassed);
      }
    }
    GraphNode newNode = getNode(flowNode.getId());
    // Atribui o arquivo de sada.
    newNode.setStandardOutputFile(flowNode.getStandardOutputFile());
    newNode.setHasExitCode(flowNode.hasExitCode());
    newNode.setExitCodeLogFile(flowNode.getExitCodeLogFile());

    Map<String, String> parameterValues = new HashMap<String, String>();
    for (NodeParameter nodeParameter : flowNode.getParameters()) {
      String parameterName = nodeParameter.getName();
      AlgorithmConfiguratorView configuratorView =
        newNode.getAlgorithmConfiguratorView();
      if (configuratorView == null) {
        String parameterLabel = nodeParameter.getLabel();
        String parameterType = nodeParameter.getType();
        newNode.addParameter(parameterName, parameterLabel, parameterType);
      }
      String parameterValue = nodeParameter.getValue();
      parameterValues.put(parameterName, parameterValue);
    }
    newNode.setParameterValuesByName(Collections
      .unmodifiableMap(parameterValues));
  }

  /**
   * Cria os ns do grafo com as mesmas caractersticas dos ns do
   * fluxo especificado.
   * 
   * @param flow O fluxo.
   * @throws ParseException
   * @throws RemoteException
   */
  private void createGraphNodes(Flow flow) throws ParseException,
    RemoteException {
    for (FlowNode flowNode : flow.getNodes()) {
      createGraphNode(flowNode);
    }
  }

  /**
   * Trata mensagens pegar grafo.
   * 
   * @param message A mensagem.
   */
  private void handlePickGraphMessage(PickGraphMessage message) {
    message.setGraph(this);
  }

  /**
   * Trata mensagens remover elementos selecionados.
   * 
   * @return <code>true</code> se conseguiu remover pelo menos um elemento ou
   *         <code>false</code> caso contrrio.
   */
  private boolean handleRemoveElementsMessage() {
    List<GraphElement> elementToRemoveList = new LinkedList<GraphElement>();
    Iterator<GraphElement> elementIterator = getVOs().iterator();
    while (elementIterator.hasNext()) {
      GraphElement element = elementIterator.next();
      if (element.isSelected()) {
        elementToRemoveList.add(element);
      }
    }
    if (elementToRemoveList.isEmpty()) {
      return false;
    }
    Iterator<GraphElement> elementToRemoveIterator =
      elementToRemoveList.iterator();
    while (elementToRemoveIterator.hasNext()) {
      GraphElement elementToRemove =
        elementToRemoveIterator.next();
      elementToRemove.deattach();
    }
    repaint();
    return true;
  }

  /**
   * Trata mensagens reiniciar.
   */
  private void handleResetMessage() {
    int ix = 0;
    while (ix < getVOs().size()) {
      GraphElement element = getVOs().get(ix);
      element.reset();
      ix++;
    }
    Iterator<GraphListener> graphListenerIterator =
      this.graphListenerList.iterator();
    while (graphListenerIterator.hasNext()) {
      GraphListener graphListener =
        graphListenerIterator.next();
      graphListener.wasReseted(this);
    }
    repaint();
  }

  /**
   * Insere o elemento especificado no grafo.
   * 
   * @param element O elemento a ser inserido.
   */
  private void insertElement(GraphElement element) {
    if (element instanceof GraphLink) {
      getVOs().add(0, element);
    }
    else if (element instanceof GraphNode) {
      if (getVOs().isEmpty()) {
        getVOs().add(0, element);
      }
      else {
        for (int i = 0; i < getVOs().size(); i++) {
          Object vo = getVOs().get(i);
          if (vo instanceof GraphNode) {
            getVOs().add(i, element);
            break;
          }
        }
      }
    }
    else {
      String errorMessage =
        MessageFormat.format("O elemento {0}  de um tipo desconhecido ({1}).",
        element, element.getClass().getName());
      throw new IllegalArgumentException(errorMessage);
    }
  }
}
