package csbase.algorithms.flows.graph;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Stroke;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.text.MessageFormat;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import tecgraf.javautils.LNG;
import tecgraf.javautils.vix.TypeVO;
import tecgraf.javautils.vix.VO;
import csbase.logic.algorithms.FileParameterValue;
import csbase.logic.algorithms.parameters.FileParameter;
import csbase.logic.algorithms.parameters.FileParameterListener;
import csbase.logic.algorithms.parameters.FileParameterPipeAcceptance;
import csbase.logic.algorithms.parameters.InputFileParameter;
import csbase.logic.algorithms.parameters.OutputFileParameter;
import csbase.logic.algorithms.parameters.SimpleParameter;
import csbase.logic.algorithms.parameters.SimpleParameterListener;

/**
 * <p>
 * Viso para descritores de parmetros do tipo arquivo.
 * </p>
 * <p>
 * Representa os arquivos do configurador de algoritmo no n que representa o
 * configurador.
 * </p>
 * 
 * @author lmoreira
 */
public final class GraphFileDescriptor extends VO {
  private static final Color BORDER_COLOR = Color.BLACK;
  private static final Color DISABLED_COLOR = Color.LIGHT_GRAY;
  private static final Color ERROR_COLOR = Color.RED;
  private static final Color EMPTY_BACKGROUND_COLOR = Color.WHITE;
  private static final int HEIGHT = 6;
  private static final Color HIGHLIGHTED_COLOR = GraphElement.HIGHT_LIGHT_COLOR;
  private static final int HORIZONTAL_MARGIN = 2;
  private static final Stroke DEFAULT_STROKE = new BasicStroke(0f);
  private static final Stroke THICKER_STROKE = new BasicStroke(1f);
  private static final int VERTICAL_MARGIN = 2;
  private static final int WIDTH = 12;
  private Color backgroundColor;
  private Color borderColor;
  private Color defaultColor;
  private Stroke stroke;
  private FileParameter fileParameter;
  private boolean isInput;
  private boolean isOutput;
  private Set<GraphLink> linkFromSet;
  private GraphLink linkTo;
  private String parameterLabel;
  private String parameterName;
  private GeneralPath path;
  private boolean isVisible;

  public GraphFileDescriptor(String parameterName, String parameterLabel,
    boolean isOutput, GraphNode node) {
    if (parameterName == null) {
      throw new IllegalArgumentException("O parmetro parameterName est nulo.");
    }
    if (parameterLabel == null) {
      throw new IllegalArgumentException(
        "O parmetro parameterLabel est nulo.");
    }
    if (node == null) {
      throw new IllegalArgumentException("O parmetro node est nulo.");
    }
    this.isOutput = isOutput;
    this.isInput = !isOutput;
    this.parameterName = parameterName;
    this.parameterLabel = parameterLabel;
    this.linkFromSet = new HashSet<GraphLink>();
    createDefaultColor();
    updateColor();
    changeVS(null, node);
    createShape(new Point());
    node.addListener(new NodeListener());
    updateVisibility();
  }

  public GraphFileDescriptor(FileParameter fileParameter, GraphNode node) {
    if (fileParameter == null) {
      throw new IllegalArgumentException("O parmetro fileParameter est nulo.");
    }
    if (node == null) {
      throw new IllegalArgumentException("O parmetro node est nulo.");
    }
    this.fileParameter = fileParameter;
    if (this.fileParameter instanceof OutputFileParameter) {
      this.isOutput = true;
    }
    else if (this.fileParameter instanceof InputFileParameter) {
      this.isInput = true;
    }
    else {
      throw new IllegalArgumentException(MessageFormat.format(
        "O parmetro {0}  de uma classe inesperada ({1}).",
        this.fileParameter, this.fileParameter.getClass().getName()));
    }
    this.parameterName = this.fileParameter.getName();
    this.parameterLabel = this.fileParameter.getLabel();
    this.fileParameter.addFileParameterListener(new FileParameterListener() {
      public void hasLinkWasChanged(FileParameter parameter) {
      }

      public void typeWasChanged(FileParameter parameter) {
        createDefaultColor();
        updateColor();
        GraphFileDescriptor.this.vs.repaint();
      }
    });

    this.fileParameter
      .addSimpleParameterListener(new SimpleParameterListener<FileParameterValue>() {

        @Override
        public void capabilityWasChanged(
          SimpleParameter<FileParameterValue> parameter) {
        }

        @Override
        public void defaultValueWasChanged(
          SimpleParameter<FileParameterValue> parameter) {
        }

        @Override
        public void labelWasChanged(
          SimpleParameter<FileParameterValue> parameter) {
        }

        @Override
        public void valueWasChanged(
          SimpleParameter<FileParameterValue> parameter) {
          updateVisibility();
        }

        @Override
        public void visibilityWasChanged(
          SimpleParameter<FileParameterValue> parameter) {
        }
      });
    this.linkFromSet = new HashSet<GraphLink>();
    createDefaultColor();
    updateColor();
    changeVS(null, node);
    createShape(new Point());
    node.addListener(new NodeListener());
    updateVisibility();
  }

  @Override
  public void callbackRepaint(Graphics2D g) {
    if (isVisible()) {
      g.setPaint(this.backgroundColor);
      g.fill(this.path);
      g.setPaint(this.borderColor);
      g.setStroke(stroke);
      g.draw(this.path);
    }
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (!obj.getClass().equals(getClass())) {
      return false;
    }
    GraphFileDescriptor graphFileType = (GraphFileDescriptor) obj;
    if (!getParameterName().equals(graphFileType.getParameterName())) {
      return false;
    }
    if (!this.vs.equals(graphFileType.vs)) {
      return false;
    }
    return super.equals(obj);
  }

  public Rectangle2D getBounds2D() {
    Rectangle2D bounds = this.path.getBounds2D();
    bounds.setFrameFromDiagonal(bounds.getMinX() - HORIZONTAL_MARGIN, bounds
      .getMinY()
      - VERTICAL_MARGIN, bounds.getMaxX() + HORIZONTAL_MARGIN, bounds.getMaxY()
      + VERTICAL_MARGIN);
    return bounds;
  }

  public FileParameter getFileParameter() {
    return this.fileParameter;
  }

  public Collection<GraphLink> getLinkFromCollection() {
    return Collections.unmodifiableCollection(this.linkFromSet);
  }

  public GraphLink getLinkTo() {
    return this.linkTo;
  }

  public GraphNode getNode() {
    return (GraphNode) this.vs;
  }

  public String getParameterLabel() {
    return this.parameterLabel;
  }

  public String getParameterName() {
    return this.parameterName;
  }

  @Override
  public int hashCode() {
    return this.vs.hashCode();
  }

  public boolean isInput() {
    return this.isInput;
  }

  public boolean isOutput() {
    return this.isOutput;
  }

  public boolean validate() {
    if (!isWellFormed()) {
      return false;
    }
    if (isVisible && fileParameter.isEnabled()) {
      if (!fileParameter.isOptional()
        && fileParameter.usesPipe() == FileParameterPipeAcceptance.ALWAYS) {
        if (isInput) {
          if (getLinkTo() == null) {
            return false;
          }
        }
        else if (isOutput) {
          if (getLinkFromCollection().isEmpty()) {
            return false;
          }
        }
      }
    }
    return true;
  }

  public boolean isWellFormed() {
    if (fileParameter == null) {
      return false;
    }
    if (!isVisible) {
      if (getLinkTo() != null) {
        return false;
      }
      if (!getLinkFromCollection().isEmpty()) {
        return false;
      }
    }
    return true;
  }

  public boolean isVisible() {
    return isVisible || !isWellFormed();
  }

  @Override
  public String toString() {
    String fileType = null;
    if (getFileParameter() != null) {
      fileType = getFileParameter().getFileType();
    }
    if (fileType == null) {
      fileType =
        LNG.get(GraphFileDescriptor.class.getName() + ".unknownFileType");
    }
    return getParameterLabel() + " (" + fileType + ")";
  }

  @Override
  public TypeVO pick(Point2D pt) {
    if (!isVisible()) {
      return null;
    }
    return super.pick(pt);
  }

  /**
   * Adiciona uma aresta que se origina desta viso de descritor.
   * 
   * @param link A aresta.
   */
  void addLinkFrom(GraphLink link) {
    this.linkFromSet.add(link);
    if (getFileParameter() != null) {
      getFileParameter().setHasLink(true);
    }
  }

  /**
   * Verifica se esta viso de descritor contm o ponto dado.
   * 
   * @param point O ponto.
   * 
   * @return <code>true</code> se contm ou <code>false</code> caso contrrio.
   */
  boolean contains(Point2D point) {
    return getBounds2D().contains(point);
  }

  /**
   * Obtm a dica desta viso de descritor.
   * 
   * @return A dica.
   */
  String getHint() {
    return toString();
  }

  /**
   * Obtm o ponto que podemos ligar uma aresta a esta viso.
   * 
   * @return O ponto.
   */
  Point2D getLinkablePoint() {
    Rectangle2D bounds = this.path.getBounds2D();
    Point2D point = new Point();
    double y;
    if (isOutput()) {
      y = bounds.getMaxY();
    }
    else {
      y = bounds.getMinY();
    }
    point.setLocation(bounds.getCenterX(), y);
    return point;
  }

  /**
   * Remove a aresta das coleo de arestas que se originam desta viso.
   * 
   * @param link A aresta.
   */
  void removeLinkFrom(GraphLink link) {
    this.linkFromSet.remove(link);
    if (this.linkFromSet.isEmpty()) {
      if (getFileParameter() != null) {
        getFileParameter().setHasLink(false);
      }
    }
  }

  /**
   * Mofidica a aresta que se destina a essa viso.
   * 
   * @param link A aresta.
   */
  void setLinkTo(GraphLink link) {
    this.linkTo = link;
    if (this.linkTo != null) {
      if (getFileParameter() != null) {
        getFileParameter().setHasLink(true);
      }
    }
    else {
      if (getFileParameter() != null) {
        getFileParameter().setHasLink(false);
      }
    }
  }

  /**
   * Modifica a localizao desta viso.
   * 
   * @param pt Nova localizao.
   */
  void setLocation(Point2D pt) {
    createShape(pt);
    updateLinks();
  }

  /**
   * Desliga o destaque.
   */
  void turnOffHightLight() {
    updateColor();
  }

  /**
   * Liga o destaque.
   */
  void turnOnHightLight() {
    this.borderColor = this.backgroundColor = HIGHLIGHTED_COLOR;
  }

  /**
   * Cria a cor-padro.
   */
  private void createDefaultColor() {
    if (!isWellFormed()) {
      this.defaultColor = Color.RED;
    }
    else {
      FileTypeColorManager fileTypeManager = FileTypeColorManager.getInstance();
      if (fileTypeManager == null) {
        this.defaultColor = Color.BLACK;
      }
      else {
        this.defaultColor =
          fileTypeManager.getColor(this.fileParameter.getFileType());
      }
    }
  }

  /**
   * Cria a forma desta viso.
   * 
   * @param point O ponto.
   */
  private void createShape(Point2D point) {
    float centerX = (float) point.getX();
    float lowerY = (float) point.getY();
    if (isOutput()) {
      lowerY += HEIGHT;
    }
    float x = centerX - WIDTH / 2;
    float y = lowerY - HEIGHT;
    this.path = new GeneralPath();
    this.path.moveTo(x + WIDTH / 2, y + HEIGHT);
    this.path.lineTo(x + WIDTH, y);
    this.path.lineTo(x, y);
    this.path.closePath();
  }

  private void updateColor() {
    this.stroke = DEFAULT_STROKE;
    if (!isWellFormed()) {
      this.backgroundColor = ERROR_COLOR;
      this.borderColor = ERROR_COLOR;
    }
    else if (getFileParameter().isEnabled()) {
      if (fileParameter.usesPipe() == FileParameterPipeAcceptance.ALWAYS) {
        this.borderColor = this.defaultColor;
        this.backgroundColor = EMPTY_BACKGROUND_COLOR;
        this.stroke = THICKER_STROKE;
      }
      else {
        this.backgroundColor = this.defaultColor;
        this.borderColor = BORDER_COLOR;
      }
    }
    else {
      this.backgroundColor = DISABLED_COLOR;
      this.borderColor = DISABLED_COLOR;
    }
  }

  private void updateVisibility() {
    if (fileParameter != null) {
      if (fileParameter.getValue() != null) {
        isVisible = false;
      }
      else {
        isVisible = getFileParameter().isVisible();
      }
    }
    else {
      isVisible = true;
    }
    updateColor();
  }

  /**
   * Atualiza as arestas que se originam desta viso ou que se destina a essa
   * viso.
   */
  private void updateLinks() {
    if (this.linkTo != null) {
      this.linkTo.updateEndPoint();
    }
    else {
      Iterator<GraphLink> linkFromIterator = this.linkFromSet.iterator();
      while (linkFromIterator.hasNext()) {
        GraphLink link = linkFromIterator.next();
        link.updateStartPoint();
      }
    }
  }

  private final class NodeListener implements GraphElementListener {
    public void wasAnchored(GraphLink link, GraphFileDescriptor fd) {
      updateColor();
    }

    public void wasDragged(GraphElement element, Point2D startPoint,
      Point2D endPoint) {
    }

    public void wasDragged(GraphElement element, double tx, double ty) {
    }

    public void wasDropped(GraphElement element, Point2D point) {
    }

    public void wasIncreased(GraphLink link) {
    }

    public void wasParameterSetVisible(GraphNode node, String parameterName,
      boolean isVisible) {
      updateVisibility();
      vs.repaint();
    }

    public void wasParameterSetEnabled(GraphNode n, String parameterName,
      boolean isEnabled) {
      updateColor();
    }

    public void wasParametrized(GraphElement element) {
    }

    public void wasLinkStatusChanged(GraphLink link) {
    }

    public void wasRenamed(GraphNode node) {
    }

    public void wasResized(GraphNode n) {
    }

    public void wasSelected(GraphElement element) {
    }

    public void wasUnanchored(GraphLink link, GraphFileDescriptor fd) {
      updateColor();
    }
  }
}
