package csbase.algorithms.flows.graph;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Stroke;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import tecgraf.javautils.LNG;
import tecgraf.javautils.vix.TypeMessage;
import csbase.algorithms.flows.messages.ErrorMessage;
import csbase.algorithms.flows.messages.PickLinkMessage;
import csbase.algorithms.flows.messages.PickNodeMessage;
import csbase.logic.algorithms.parameters.FileParameter;
import csbase.logic.algorithms.parameters.FileParameterListener;
import csbase.logic.algorithms.parameters.SimpleParameter;
import csbase.logic.algorithms.parameters.SimpleParameterListener;

/**
 * <p>
 * A aresta que representa a ligao de 2 parmetros de arquivo de 2 ns que
 * representam configuradores de algoritmo.
 * </p>
 * 
 * @author lmoreira
 */
public final class GraphLink extends GraphElement implements
  FileParameterListener, SimpleParameterListener {
  private static final Color DEFAULT_COLOR = Color.BLACK;
  private static final Color ERROR_COLOR = Color.RED;
  private static final Color REMOVE_LINE_COLOR = new Color(255, 255, 100);
  private static final Stroke STROKE =
    new BasicStroke(0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_ROUND);
  private Color color;
  private GeneralPath generalPath;
  private GraphFileDescriptor inputFileDescriptor;
  private boolean isHighlighted;
  private GraphFileDescriptor outputFileDescriptor;
  private List<GraphPoint> pointList;
  private GraphPoint temporaryPoint;

  GraphLink(Graph graph, int id) {
    super(graph, id);
    this.pointList = new ArrayList<GraphPoint>();
    createShape();
    updateColor();
  }

  public boolean breakPoint(Point2D point) {
    GraphPoint graphPoint = new GraphPoint(point);
    for (int i = 1; i < this.pointList.size(); i++) {
      GraphPoint priorPoint = this.pointList.get(i - 1);
      GraphPoint nextPoint = this.pointList.get(i);
      if (contains(graphPoint, priorPoint, nextPoint)) {
        this.pointList.add(i, new GraphPoint(point));
        return true;
      }
    }
    return false;
  }

  @Override
  public void callbackRepaint(Graphics2D g) {
    super.callbackRepaint(g);
    if (this.generalPath != null) {
      g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
        RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
      g.setPaint(this.color);
      g.setStroke(STROKE);
      g.draw(this.generalPath);
      if (isSelected()) {
        for (int i = 0; i < this.pointList.size(); i++) {
          GraphPoint point = this.pointList.get(i);
          point.paint(g);
        }
        if (this.temporaryPoint != null) {
          this.temporaryPoint.paint(g);
        }
      }
    }
  }

  @Override
  public void deattach() {
    if (this.inputFileDescriptor != null) {
      this.inputFileDescriptor.setLinkTo(null);
      setInputFileDescriptor(null);
    }
    if (this.outputFileDescriptor != null) {
      this.outputFileDescriptor.removeLinkFrom(this);
      setOutputFileDescriptor(null);
    }
    super.deattach();
  }

  public boolean finish(GraphFileDescriptor inputFileDescriptor) {
    if (inputFileDescriptor == null) {
      throw new IllegalArgumentException(
        "O parmetro inputFileDescriptor est nulo.");
    }
    if (inputFileDescriptor.getFileParameter() == null) {
      return false;
    }
    if (this.outputFileDescriptor != null) {
      if (this.outputFileDescriptor.getFileParameter() == null) {
        return false;
      }
      String outputFileType =
        this.outputFileDescriptor.getFileParameter().getFileType();
      String inputFileType =
        inputFileDescriptor.getFileParameter().getFileType();
      if ((inputFileType == null && outputFileType != null)
        || (inputFileType != null && !inputFileType.equals(outputFileType))) {
        if (outputFileType == null) {
          outputFileType =
            LNG.get(GraphLink.class.getName() + ".unknownFileType");
        }
        if (inputFileType == null) {
          inputFileType =
            LNG.get(GraphLink.class.getName() + ".unknownFileType");
        }
        new ErrorMessage(GraphLink.class.getName()
          + ".error_different_file_types", outputFileType, inputFileType)
          .sendVS(getVS());
        return false;
      }
    }
    GraphNode node = inputFileDescriptor.getNode();
    if (inputFileDescriptor.getLinkTo() != null) {
      removeLastPoint();
      new ErrorMessage(GraphLink.class.getName() + ".error_only_one_link")
        .sendVS(getVS());
      return false;
    }
    if (node.canReach(getStartNode())) {
      removeLastPoint();
      new ErrorMessage(GraphLink.class.getName() + ".error_create_cicle")
        .sendVS(getVS());
      return false;
    }
    setEnd(inputFileDescriptor);
    return true;
  }

  @Override
  public Rectangle2D getBounds2D() {
    if (this.generalPath == null) {
      return new Rectangle2D.Double();
    }
    return this.generalPath.getBounds2D();
  }

  @Override
  public String getHint(Point2D point) {
    if (!contains(point)) {
      return null;
    }
    return toString();
  }

  public GraphFileDescriptor getInputFileDescriptor() {
    return this.inputFileDescriptor;
  }

  public List<Point2D> getPointList() {
    List<Point2D> newPointList = new ArrayList<Point2D>(this.pointList.size());
    for (int i = 0; i < this.pointList.size(); i++) {
      GraphPoint graphPoint = this.pointList.get(i);
      newPointList.add(graphPoint.getLocation());
    }
    return Collections.unmodifiableList(newPointList);
  }

  public List<Point2D> getIntermediatePointList() {
    List<Point2D> newPointList = new ArrayList<Point2D>(this.pointList.size());
    for (int i = 1; i < this.pointList.size() - 1; i++) {
      GraphPoint graphPoint = this.pointList.get(i);
      newPointList.add(graphPoint.getLocation());
    }
    return Collections.unmodifiableList(newPointList);
  }

  public GraphFileDescriptor getOutputFileDescriptor() {
    return this.outputFileDescriptor;
  }

  public void hasLinkWasChanged(FileParameter parameter) {
    updateColor();
    repaint();
  }

  public void increase(Point2D pt) {
    pt = Grid.adjustPoint(pt);
    GraphPoint point = new GraphPoint(pt);
    this.pointList.add(point);
    if (this.pointList.size() != 1) {
      GraphPoint previousPoint = this.pointList.get(this.pointList.size() - 2);
      if (canBeRemoved(previousPoint)) {
        this.pointList.remove(previousPoint);
      }
    }
    createShape();
    Iterator listenerIterator = getListenerList().iterator();
    while (listenerIterator.hasNext()) {
      GraphElementListener listener =
        (GraphElementListener) listenerIterator.next();
      listener.wasIncreased(this);
    }
  }

  @Override
  public boolean msgHandlerVO(TypeMessage msg) {
    if (msg instanceof PickLinkMessage) {
      return handlePickLinkMessage((PickLinkMessage) msg);
    }
    return super.msgHandlerVO(msg);
  }

  public void pretentToIncrease(Point2D pt) {
    this.temporaryPoint = new GraphPoint(pt);
    createShape();
  }

  public void removeLastPoint() {
    this.pointList.remove(this.pointList.size() - 1);
  }

  public void setEnd(GraphFileDescriptor inputFileDescriptor) {
    if (inputFileDescriptor == null) {
      throw new IllegalArgumentException(
        "O parmetro inputFileDescriptor est nulo.");
    }
    inputFileDescriptor.setLinkTo(this);
    setInputFileDescriptor(inputFileDescriptor);
    createShape();
  }

  public void setStart(GraphFileDescriptor outputFileDescriptor) {
    if (outputFileDescriptor == null) {
      throw new IllegalArgumentException(
        "O parmetro outputFileDescriptor est nulo.");
    }
    outputFileDescriptor.addLinkFrom(this);
    setOutputFileDescriptor(outputFileDescriptor);
    createShape();
  }

  public boolean start(GraphFileDescriptor outputFileDescriptor) {
    if (outputFileDescriptor.getFileParameter() == null) {
      return false;
    }
    GraphNode node = outputFileDescriptor.getNode();
    if (this.inputFileDescriptor != null) {
      String inputFileType =
        this.inputFileDescriptor.getFileParameter().getFileType();
      String outputFileType =
        outputFileDescriptor.getFileParameter().getFileType();
      if ((inputFileType == null && outputFileType != null)
        || (inputFileType != null && !inputFileType.equals(outputFileType))) {
        if (outputFileType == null) {
          outputFileType =
            LNG.get(GraphLink.class.getName() + ".unknownFileType");
          ;
        }
        if (inputFileType == null) {
          inputFileType =
            LNG.get(GraphLink.class.getName() + ".unknownFileType");
          ;
        }
        new ErrorMessage(GraphLink.class.getName()
          + ".error_different_file_types", outputFileType, inputFileType)
          .sendVS(getVS());
        return false;
      }
    }
    if (getEndNode() != null && getEndNode().canReach(node)) {
      removeLastPoint();
      new ErrorMessage(GraphLink.class.getName() + ".error_create_cicle")
        .sendVS(getVS());
      return false;
    }
    setStart(outputFileDescriptor);
    return true;
  }

  @Override
  public String toString() {
    String start =
      (this.outputFileDescriptor == null ? "X" : this.outputFileDescriptor
        .toString());
    String end =
      (this.inputFileDescriptor == null ? "X" : this.inputFileDescriptor
        .toString());
    return start + " -> " + end;
  }

  @Override
  public boolean turnOnHighlight(Point2D pt) {
    if (!contains(pt)) {
      return false;
    }
    this.isHighlighted = true;
    updateColor();
    return true;
  }

  @Override
  public void turnOffHighlight() {
    this.isHighlighted = false;
    updateColor();
  }

  public void typeWasChanged(FileParameter parameter) {
    updateColor();
    repaint();
    Iterator listenerIterator = getListenerList().iterator();
    while (listenerIterator.hasNext()) {
      GraphElementListener listener =
        (GraphElementListener) listenerIterator.next();
      listener.wasLinkStatusChanged(this);
    }
  }

  public void capabilityWasChanged(SimpleParameter parameter) {
    updateColor();
  }

  public void defaultValueWasChanged(SimpleParameter parameter) {
  }

  public void labelWasChanged(SimpleParameter parameter) {
  }

  public void valueWasChanged(SimpleParameter parameter) {
  }

  public void visibilityWasChanged(SimpleParameter parameter) {
    updateColor();
  }

  @Override
  protected boolean contains(Point2D pt) {
    GraphPoint point = new GraphPoint(pt);
    for (int i = 1; i < this.pointList.size(); i++) {
      GraphPoint startPoint = this.pointList.get(i - 1);
      GraphPoint endPoint = this.pointList.get(i);
      if (contains(point, startPoint, endPoint)) {
        return true;
      }
    }
    return false;
  }

  @Override
  protected boolean doDrag(Point2D startPoint, Point2D endPoint) {
    endPoint = adjustPoint(endPoint);
    int indexOf = this.pointList.indexOf(new GraphPoint(startPoint));
    if (indexOf < 0) {
      return false;
    }
    this.pointList.set(indexOf, new GraphPoint(endPoint));
    if (indexOf == 0) {
      moveFirstPoint(endPoint);
    }
    else if (indexOf == this.pointList.size() - 1) {
      moveLastPoint(endPoint);
    }
    createShape();
    updateColor();
    if (canBeRemoved(new GraphPoint(Grid.adjustPoint(endPoint)))) {
      this.color = REMOVE_LINE_COLOR;
      repaint();
    }
    return true;
  }

  @Override
  protected void doDrag(double tx, double ty) {
    for (int i = 1; i < this.pointList.size() - 1; i++) {
      GraphPoint graphPoint = this.pointList.get(i);
      Point2D point = graphPoint.getLocation();
      double x = point.getX() + tx;
      double y = point.getY() + ty;
      Point2D newPoint = new Point2D.Double(x, y);
      newPoint = adjustPoint(newPoint);
      graphPoint.setLocation(newPoint.getX(), newPoint.getY());
    }
  }

  @Override
  protected void doDrop(Point2D pt) {
    GraphPoint graphPoint = new GraphPoint(pt);
    if (canBeRemoved(graphPoint)) {
      this.pointList.remove(graphPoint);
    }
    else if (this.pointList.indexOf(graphPoint) == 0) {
      PickNodeMessage message = new PickNodeMessage(pt);
      message.sendVO(getGraph());
      GraphNode node = message.getNode();
      if (node != null) {
        node.startLink(this, pt);
      }
      else {
        deattach();
      }
    }
    else if (this.pointList.indexOf(graphPoint) == this.pointList.size() - 1) {
      PickNodeMessage message = new PickNodeMessage(pt);
      message.sendVO(getGraph());
      GraphNode node = message.getNode();
      if (node != null) {
        node.finishLink(this, pt);
      }
      else {
        deattach();
      }
    }
    for (int i = 1; i < this.pointList.size() - 1; i++) {
      GraphPoint currentGraphPoint = this.pointList.get(i);
      this.pointList.set(i, new GraphPoint(Grid.adjustPoint(currentGraphPoint
        .getLocation())));
    }
    createShape();
  }

  @Override
  protected void reset() {
    super.reset();
    if (this.temporaryPoint != null) {
      deattach();
    }
  }

  public GraphNode getEndNode() {
    if (this.inputFileDescriptor != null) {
      return this.inputFileDescriptor.getNode();
    }
    return null;
  }

  public GraphNode getStartNode() {
    if (this.outputFileDescriptor != null) {
      return this.outputFileDescriptor.getNode();
    }
    return null;
  }

  public boolean isWellFormed() {
    if (this.outputFileDescriptor == null) {
      return false;
    }
    if (!this.outputFileDescriptor.isWellFormed()) {
      return false;
    }
    if (this.inputFileDescriptor == null) {
      return false;
    }
    if (!this.inputFileDescriptor.isWellFormed()) {
      return false;
    }
    String outputFileType =
      this.outputFileDescriptor.getFileParameter().getFileType();
    String inputFileType =
      this.inputFileDescriptor.getFileParameter().getFileType();
    if (outputFileType == null) {
      if (inputFileType != null) {
        return false;
      }
      return true;
    }
    return outputFileType.equals(inputFileType);
  }

  void updateEndPoint() {
    this.pointList.set(this.pointList.size() - 1, new GraphPoint(
      this.inputFileDescriptor.getLinkablePoint()));
    this.temporaryPoint = null;
    bringToFront();
    createShape();
  }

  void updateStartPoint() {
    this.pointList.set(0, new GraphPoint(this.outputFileDescriptor
      .getLinkablePoint()));
    this.temporaryPoint = null;
    bringToFront();
    createShape();
  }

  private Point2D adjustPoint(Point2D point) {
    double x = Math.max(point.getX(), 0);
    double y = Math.max(point.getY(), 0);
    Point2D newEndPoint = new Point2D.Double(x, y);
    return newEndPoint;
  }

  private boolean canBeRemoved(GraphPoint point) {
    int indexOf = this.pointList.indexOf(point);
    if (indexOf < 0) {
      return false;
    }
    if (indexOf == 0) {
      return false;
    }
    if (indexOf == this.pointList.size() - 1) {
      return false;
    }
    GraphPoint priorPoint = this.pointList.get(indexOf - 1);
    GraphPoint nextPoint = this.pointList.get(indexOf + 1);
    return contains(point, priorPoint, nextPoint);
  }

  private boolean contains(GraphPoint point, GraphPoint startPoint,
    GraphPoint endPoint) {
    if (point.equals(startPoint)) {
      return true;
    }
    if (point.equals(endPoint)) {
      return true;
    }
    Line2D line =
      new Line2D.Double(startPoint.getLocation(), endPoint.getLocation());
    return line.intersects(point.getBounds());
  }

  private void createShape() {
    if (!this.pointList.isEmpty()) {
      GraphPoint graphPoint = this.pointList.get(0);
      Point2D point = graphPoint.getLocation();
      this.generalPath = new GeneralPath();
      this.generalPath.moveTo((float) point.getX(), (float) point.getY());
      for (int i = 1; i < this.pointList.size(); i++) {
        graphPoint = this.pointList.get(i);
        point = graphPoint.getLocation();
        this.generalPath.lineTo((float) point.getX(), (float) point.getY());
      }
      if (this.temporaryPoint != null) {
        point = this.temporaryPoint.getLocation();
        this.generalPath.lineTo((float) point.getX(), (float) point.getY());
      }
    }
  }

  private boolean handlePickLinkMessage(PickLinkMessage message) {
    if (contains(message.getPoint())) {
      message.setLink(this);
      return true;
    }
    return false;
  }

  private void moveFirstPoint(Point2D pt) {
    if (this.outputFileDescriptor != null) {
      if (!this.outputFileDescriptor.getBounds2D().contains(pt)) {
        this.outputFileDescriptor.removeLinkFrom(this);
        setOutputFileDescriptor(null);
      }
    }
  }

  private void moveLastPoint(Point2D pt) {
    if (this.inputFileDescriptor != null) {
      if (!this.inputFileDescriptor.getBounds2D().contains(pt)) {
        this.inputFileDescriptor.setLinkTo(null);
        setInputFileDescriptor(null);
      }
    }
  }

  private void notifyAnchored(GraphFileDescriptor descriptor) {
    Iterator listenerIterator = getListenerList().iterator();
    while (listenerIterator.hasNext()) {
      GraphElementListener listener =
        (GraphElementListener) listenerIterator.next();
      listener.wasAnchored(this, descriptor);
    }
  }

  private void notifyUnanchored(GraphFileDescriptor descriptor) {
    Iterator listenerIterator = getListenerList().iterator();
    while (listenerIterator.hasNext()) {
      GraphElementListener listener =
        (GraphElementListener) listenerIterator.next();
      listener.wasUnanchored(this, descriptor);
    }
  }

  private void setInputFileDescriptor(GraphFileDescriptor inputFileType) {
    if (inputFileType == null) {
      if (this.inputFileDescriptor != null) {
        GraphFileDescriptor oldDescriptor = this.inputFileDescriptor;
        FileParameter fileParameter =
          this.inputFileDescriptor.getFileParameter();
        if (fileParameter != null) {
          fileParameter.removeFileParameterListener(this);
          fileParameter.removeSimpleParameterListener(this);
        }
        this.inputFileDescriptor = null;
        updateColor();
        notifyUnanchored(oldDescriptor);
      }
    }
    else {
      this.inputFileDescriptor = inputFileType;
      updateEndPoint();
      updateColor();
      notifyAnchored(this.inputFileDescriptor);
      FileParameter fileParameter = this.inputFileDescriptor.getFileParameter();
      if (fileParameter != null) {
        fileParameter.addFileParameterListener(this);
        fileParameter.addSimpleParameterListener(this);
      }
    }
  }

  private void setOutputFileDescriptor(GraphFileDescriptor outputFileType) {
    if (outputFileType == null) {
      if (this.outputFileDescriptor != null) {
        GraphFileDescriptor oldDescriptor = this.outputFileDescriptor;
        FileParameter fileParameter =
          this.outputFileDescriptor.getFileParameter();
        if (fileParameter != null) {
          fileParameter.removeFileParameterListener(this);
          fileParameter.removeSimpleParameterListener(this);
        }
        this.outputFileDescriptor = null;
        updateColor();
        notifyUnanchored(oldDescriptor);
      }
    }
    else {
      this.outputFileDescriptor = outputFileType;
      updateStartPoint();
      updateColor();
      FileParameter fileParameter =
        this.outputFileDescriptor.getFileParameter();
      notifyAnchored(this.outputFileDescriptor);
      if (fileParameter != null) {
        fileParameter.addFileParameterListener(this);
        fileParameter.addSimpleParameterListener(this);
      }
    }
  }

  private void updateColor() {
    if (!isWellFormed()) {
      this.color = ERROR_COLOR;
    }
    else if (this.isHighlighted) {
      this.color = HIGHT_LIGHT_COLOR;
    }
    else {
      this.color = DEFAULT_COLOR;
    }
  }

}
