package csbase.algorithms.flows.graph;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.Window;
import java.awt.geom.Dimension2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.Writer;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import tecgraf.javautils.LNG;
import tecgraf.javautils.vix.TypeMessage;
import csbase.algorithms.flows.CompassDirection;
import csbase.algorithms.flows.ImageVO;
import csbase.algorithms.flows.graph.GraphNodeDecoration.DecorationType;
import csbase.algorithms.flows.graph.utils.GraphNodeUpdateOperation;
import csbase.algorithms.flows.messages.ErrorMessage;
import csbase.algorithms.flows.messages.PickNodeMessage;
import csbase.algorithms.flows.messages.ShowParameterValuesMessage;
import csbase.client.algorithms.AlgorithmConfiguratorView;
import csbase.client.algorithms.AlgorithmConfiguratorViewListener;
import csbase.client.algorithms.parameters.validators.ValidationMode;
import csbase.client.algorithms.parameters.validators.ValidationResult;
import csbase.client.applicationmanager.ApplicationException;
import csbase.client.applicationmanager.ApplicationManager;
import csbase.client.applications.ApplicationImages;
import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.RemoteTask;
import csbase.client.project.tasks.GetChildFromNameTask;
import csbase.client.util.StandardErrorDialogs;
import csbase.exception.ParseException;
import csbase.exception.algorithms.ParameterNotFoundException;
import csbase.logic.ClientProjectFile;
import csbase.logic.ClientProjectFileInputStream;
import csbase.logic.CommonClientProject;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.AlgorithmConfiguratorListener;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.FileParameterValue;
import csbase.logic.algorithms.parameters.FileParameter;
import csbase.logic.algorithms.parameters.FileParameterMode;
import csbase.logic.algorithms.parameters.FileParameterPipeAcceptance;
import csbase.logic.algorithms.parameters.InputFileParameter;
import csbase.logic.algorithms.parameters.OutputFileParameter;

/**
 * <p>
 * Viso do N de Algoritmo
 * </p>
 * 
 * <p>
 * Um n pode estar no estado mal-formado. Isto ocorre quando o algoritmo ou a
 * verso do algoritmo que ele representa foi removida do sistema.
 * </p>
 * 
 * @author lmoreira
 */
public final class GraphNode extends GraphElement {
  /**
   * Largura/altura extra a ser considerada no clculo da borda do n.
   */
  private static final double BORDER_MARGIN = 5.0;
  /**
   * Trao padro da borda.
   */
  private static final Stroke DEFAULT_STROKE = new BasicStroke(0f);

  /**
   * Trao da borda quando n est desviado
   */
  private static final Stroke BYPASSED_STROKE =
    new BasicStroke(0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL, 0,
      new float[] { 9 }, 0);

  /**
   * Cor-padro da face (texto e borda).
   */
  private static final Color DEFAULT_FOREGROUNG_COLOR = Color.BLACK;
  /**
   * Cor para indicar que existe um erro no n.
   */
  private static final Color ERROR_COLOR = new Color(255, 182, 182);
  /**
   * Cor do texto quando o n foi desviado
   */
  private static final Color BYPASSED_TEXT_COLOR = new Color(150, 150, 150);
  /**
   * Espaamento para o cone.
   */
  private static final double ICON_PADDING = 10.0;
  /**
   * Altura mnima.
   */
  private static final double MIN_HEIGHT = 50.0;
  /**
   * Largura mnima.
   */
  private static final double MIN_WIDTH = 50.0;
  /**
   * ndice do canto nordeste.
   */
  private static final int NORTH_EAST_INDEX = 1;
  /**
   * ndice do canto noroeste.
   */
  private static final int NORTH_WEST_INDEX = 0;
  /**
   * Cor para sinalizar que o n no est pronto para executar.
   */
  private static final Color NOT_READY_BACKGROUND_COLOR =
    new Color(255, 255, 210);
  /**
   * Cor para sinalizar que o n est pronto para executar.
   */
  private static final Color READY_BACKGROUND_COLOR = new Color(210, 255, 210);
  /**
   * ndice para o canto sudeste.
   */
  private static final int SOUTH_EAST_INDEX = 2;
  /**
   * ndice para o canto sudoeste.
   */
  private static final int SOUTH_WEST_INDEX = 3;
  /**
   * Espaamento direito do texto.
   */
  private static final double TEXT_RIGHT_PADDING = 1.0;
  /**
   * Espaamento esquerdo para o cone.
   */
  private static final double ICON_LEFT_PADDING = 1.0;

  /**
   * O algoritmo deste n.
   */
  private AlgorithmInfo algorithm;
  /**
   * A viso do nome do algoritmo deste n.
   */
  private GraphString algorithmNameView;
  /**
   * A viso do identificador da verso do algoritmo deste n.
   */
  private GraphString algorithmVersionIdView;
  /**
   * O identificador do algoritmo.
   */
  private AlgorithmVersionId algorithmVersionId;
  /**
   * A cor atual da borda.
   */
  private Color borderColor;
  /**
   * A cor de fundo atual.
   */
  private Color backgroundColor;

  /**
   * A viso do configurador de algoritmos.
   */
  private AlgorithmConfiguratorView configuratorView;
  /**
   * As vises dos cantos do n.
   */
  private GraphPoint[] cornerViews;
  /**
   * As dimenses deste n.
   */
  private Dimension dimension;
  /**
   * O cone principal deste n.
   */
  private ImageVO icon;

  /**
   * As vises dos arquivos de entrada do algoritmo.
   */
  private List<GraphFileDescriptor> inputFileTypeViews;
  /**
   * As vises dos arquivos de sada do algoritmo.
   */
  private List<GraphFileDescriptor> outputFileTypeViews;
  /**
   * O arquivo de sada (stdout) de um algoritmo sendo executado.
   */
  private FileParameterValue standardOutputFile;

  /**
   * Indica se o n prov um cdigo de sada durante sua execuo.
   */
  private boolean hasExitCode;
  /**
   * O arquivo que armazena o cdigo de sada de execuo do algoritmo.
   */
  private FileParameterValue exitCodeLogFile;

  /**
   * Os rtulos dos parmetros do n indexados pelo nome dos mesmos.
   */
  private Map<String, String> parameterLabelsByName;
  /**
   * O nome dos parmetros.
   */
  private Set<String> parameterNames;
  /**
   * Os tipos dos parmetros do n indexados pelo nome dos mesmos.
   */
  private Map<String, String> parameterTypesByName;
  /**
   * Os valores dos parmetros do n indexados pelo nome dos mesmos.
   */
  private Map<String, String> parameterValuesByName;
  /**
   * O ponto superior-esquerdo do n.
   */
  private Point northWestPoint;
  /**
   * A forma do n.
   */
  private Shape shape;

  /**
   * Sinaliza se o usurio indicou que o n deve ser desviado durante a
   * execuo. O desvio propriamente dito s pode ser feito se um conjunto de
   * condies forem atendidas @see {@link #validateBypass()}.
   */
  private boolean bypassed;

  /**
   * Sinaliza se o n pode ser desviado no fluxo. Caso positivo, o algoritmo
   * continua a aparecer no editor de fluxos, mas no  executado. Todas as
   * entradas do n so desviadas para o n seguinte no fluxo durante a
   * execuo. Caso negativo, mesmo que o usurio indique que queira fazer o
   * desvio, este no  feito.
   */
  private boolean canBypass;

  /**
   * A decorao (cone de status) utilizada neste n.
   */
  private GraphNodeDecoration decoration;

  /**
   * Cria uma viso de n para um algoritmo que est instalado no sistema
   * (bem-formado).
   * 
   * @param graph A viso do fluxo (No aceita {@code null}).
   * @param id O identificador do n.
   * @param configuratorView A viso do configurador de algoritmos (No aceita
   *        {@code null}).
   * @param point O ponto superior-esquerdo (No aceita {@code null}).
   * @param bypassed Sinal que indica se o n deve ser desviado.
   */
  GraphNode(Graph graph, int id, AlgorithmConfiguratorView configuratorView,
    Point2D point, boolean bypassed) {
    super(graph, id);
    this.bypassed = bypassed;
    this.canBypass = true;
    this.borderColor = DEFAULT_FOREGROUNG_COLOR;
    this.backgroundColor = NOT_READY_BACKGROUND_COLOR;
    this.configuratorView = configuratorView;
    AlgorithmConfigurator configurator =
      this.configuratorView.getConfigurator();
    this.configuratorView.setValidationMode(ValidationMode.DEFAULT);
    AlgorithmVersionInfo algorithmVersion = configurator.getAlgorithmVersion();
    this.algorithm = algorithmVersion.getInfo();
    this.algorithmNameView = new GraphString(this, this.algorithm.toString());
    this.algorithmVersionIdView =
      new GraphString(this, algorithmVersion.getId().toString());
    this.algorithmVersionId = algorithmVersion.getId();
    this.parameterLabelsByName = new HashMap<String, String>();
    this.parameterValuesByName = new HashMap<String, String>();
    this.parameterTypesByName = new HashMap<String, String>();
    this.parameterNames = new HashSet<String>();
    this.hasExitCode = configurator.hasExitCode();

    Map<String, String> parameterValues = new HashMap<String, String>();
    for (String parameterName : configurator.getParameterNames()) {
      String parameterLabel;
      String parameterValue;
      String parameterType;
      try {
        parameterLabel = configurator.getParameterLabel(parameterName);
        parameterType = configurator.getParameterType(parameterName);
        addParameter(parameterName, parameterLabel, parameterType);
        parameterValue = configurator.getParameterValue(parameterName);
        parameterValues.put(parameterName, parameterValue);
      }
      catch (ParameterNotFoundException e) {
        IllegalStateException illegalStateException =
          new IllegalStateException();
        illegalStateException.initCause(e);
        throw illegalStateException;
      }
    }
    try {
      setParameterValuesByName(parameterValues);
    }
    catch (ParseException e) {
      IllegalStateException illegalStateException = new IllegalStateException();
      illegalStateException.initCause(e);
      throw illegalStateException;
    }
    if (GraphNodeUpdateOperation.hasNewerVersion(this)) {
      this.icon = new ImageVO(ApplicationImages.ICON_WARNING_32.getImage());
      this.icon.setTopPadding(ICON_PADDING);
      this.icon.setBottomPadding(ICON_PADDING);
      this.icon.setRightPadding(ICON_PADDING);
      this.icon.setLeftPadding(ICON_LEFT_PADDING);
      this.algorithmNameView.setRightPadding(TEXT_RIGHT_PADDING);
      this.algorithmVersionIdView.setRightPadding(TEXT_RIGHT_PADDING);
    }

    this.decoration = null;

    computeMeasures(Grid.adjustPoint(point));
    setUpVix();
    adjustGrid();
    setListeners();
    validateBypass();
    validate(false, true);
  }

  /**
   * Obtm o sinal que indica se o cdigo de sada  retornado durante a
   * execuo do algoritmo.
   * 
   * @return Verdadeiro se o n prov um cdigo de sada, ou falso, caso
   *         contrrio.
   */
  public boolean hasExitCode() {
    return hasExitCode;
  }

  /**
   * Atribui o sinal que indica que o n prov cdigo de sada durante sua
   * execuo.
   * 
   * @param hasExitCode Sinal que indica se o n prov cdigo de sada.
   */
  public void setHasExitCode(boolean hasExitCode) {
    this.hasExitCode = hasExitCode;
  }

  /**
   * Cria uma viso de n para um algoritmo que est instalado no sistema
   * (bem-formado).
   * 
   * @param graph A viso do fluxo (No aceita {@code null}).
   * @param id O identificador do n.
   * @param configuratorView A viso do configurador de algoritmos (No aceita
   *        {@code null}).
   * @param point O ponto superior-esquerdo (No aceita {@code null}).
   * @param size A dimenso do n (No aceita {@code null}).<br>
   *        Se a altura ou a largura for menor do que a altura ou a largura
   *        mnima, ela ser desprezada e o n ter altura ou largura mnima.
   * @param bypassed Sinal que indica se o n deve ser desviado.
   */
  GraphNode(Graph graph, int id, AlgorithmConfiguratorView configuratorView,
    Point2D point, Dimension2D size, boolean bypassed) {
    this(graph, id, configuratorView, point, bypassed);
    setBounds2D(point.getX(), point.getY(), size.getWidth(), size.getHeight());
  }

  /**
   * Cria uma viso de n para um algoritmo cuja verso no est instalada no
   * sistema (mal-formado).
   * 
   * @param graph A viso do fluxo (No aceita {@code null}).
   * @param id O identificador do n.
   * @param algorithmInfo O algoritmo (No aceita {@code null}).
   * @param versionId O identificador da verso do algoritmo (No aceita {@code
   *        null}).
   * @param point O ponto superior-esquerdo (No aceita {@code null}).
   * @param size A dimenso do n (No aceita {@code null}).<br>
   *        Se a altura ou a largura for menor do que a altura ou a largura
   *        mnima, ela ser desprezada e o n ter altura ou largura mnima.
   * @param bypassed Sinal que indica se o n deve ser desviado.
   */
  GraphNode(Graph graph, int id, AlgorithmInfo algorithmInfo,
    AlgorithmVersionId versionId, Point2D point, Dimension2D size,
    boolean bypassed) {
    this(graph, id, algorithmInfo.getName(), versionId, point, size, bypassed);
    this.algorithm = algorithmInfo;
  }

  /**
   * Cria uma viso de n para um algoritmo que no est instalado no sistema
   * (mal-formado).
   * 
   * @param graph A viso do fluxo (No aceita {@code null}).
   * @param id O identificador do n.
   * @param algorithmName O nome do algoritmo (No aceita {@code null}).
   * @param versionId O identificador da verso do algoritmo (No aceita {@code
   *        null}).
   * @param point O ponto superior-esquerdo (No aceita {@code null}).
   * @param size A dimenso do n (No aceita {@code null}).<br>
   *        Se a altura ou a largura for menor do que a altura ou a largura
   *        mnima, ela ser desprezada e o n ter altura ou largura mnima.
   * @param bypassed Sinal que indica se o n deve ser desviado.
   */
  GraphNode(Graph graph, int id, String algorithmName,
    AlgorithmVersionId versionId, Point2D point, Dimension2D size,
    boolean bypassed) {
    super(graph, id);
    this.bypassed = bypassed;
    this.canBypass = true;
    this.inputFileTypeViews = new LinkedList<GraphFileDescriptor>();
    this.outputFileTypeViews = new LinkedList<GraphFileDescriptor>();
    this.borderColor = DEFAULT_FOREGROUNG_COLOR;
    this.backgroundColor = ERROR_COLOR;
    this.algorithmNameView = new GraphString(this, algorithmName);
    this.algorithmVersionIdView = new GraphString(this, versionId.toString());
    this.algorithmVersionId = versionId;
    this.parameterNames = new HashSet<String>();
    this.parameterLabelsByName = new HashMap<String, String>();
    this.parameterValuesByName = new HashMap<String, String>();
    this.parameterTypesByName = new HashMap<String, String>();
    this.decoration = null;
    this.hasExitCode = false;
    setUpVix();
    setBounds2D(point.getX(), point.getY(), size.getWidth(), size.getHeight());
  }

  /**
   * Atribui o valor do arquivo que deve receber a sada padro da execuo do
   * algoritmo representado por este n.
   * 
   * @param standardOutputFile arquivo que recebe a sada padro da execuo do
   *        algoritmo
   */
  public void setStandardOutputFile(FileParameterValue standardOutputFile) {
    this.standardOutputFile = standardOutputFile;
    // Garante consistncia.
    if (null != configuratorView && null != configuratorView.getConfigurator()) {
      configuratorView.getConfigurator().setStandardOutputFile(
        standardOutputFile);
    }
  }

  /**
   * Atribui o valor do arquivo que deve armazenar o cdigo de sada da execuo
   * do algortimo representado por este n.
   * 
   * @param exitCodeLogFile arquivo que armazena o cdido de sada de execuo
   *        do algoritmo
   */
  public void setExitCodeLogFile(FileParameterValue exitCodeLogFile) {
    this.exitCodeLogFile = exitCodeLogFile;
    // Garante consistncia.
    if (null != configuratorView && null != configuratorView.getConfigurator()) {
      configuratorView.getConfigurator().setExitCodeLogFile(exitCodeLogFile);
    }
  }

  /**
   * Obtm a sada padro que ser utilizada durante a execuo do algortmo
   * representado por este n.
   * 
   * @return a sada padro que ser utilizada durante a execuo do algortmo.
   */
  public FileParameterValue getStandardOutputFile() {
    return this.standardOutputFile;
  }

  /**
   * Obtm o arquivo que armazena o cdigo de sada da execuo do algoritmo
   * representado por este n.
   * 
   * @return o arquivo que armazena o cdigo de sada da execuo do algoritmo.
   */
  public FileParameterValue getExitCodeLogFile() {
    return this.exitCodeLogFile;
  }

  /**
   * Adiciona uma viso de arquivo de entrada.
   * 
   * @param graphFileDescriptor A viso de arquivo de entrada (No aceita
   *        {@code null}).
   * 
   * @return {@code true} se foi possvel adicionar a viso ou {@code false} se
   *         j existia uma viso igual a esta.
   */
  public boolean addInputFileDescriptor(GraphFileDescriptor graphFileDescriptor) {
    if (graphFileDescriptor == null) {
      throw new IllegalArgumentException(
        "O parmetro graphFileDescriptor est nulo.");
    }
    if (this.inputFileTypeViews.contains(graphFileDescriptor)) {
      return false;
    }
    this.inputFileTypeViews.add(graphFileDescriptor);
    changeVO(null, graphFileDescriptor);
    graphFileDescriptor.changeVS(null, this);
    updateInputFileTypes();
    return true;
  }

  /**
   * Adiciona uma viso de arquivo de sada.
   * 
   * @param graphFileDescriptor A viso de arquivo de sada (No aceita {@code
   *        null}).
   * 
   * @return {@code true} se foi possvel adicionar a viso ou {@code false} se
   *         j existia uma viso igual a esta.
   */
  public boolean addOutputFileDescriptor(GraphFileDescriptor graphFileDescriptor) {
    if (graphFileDescriptor == null) {
      throw new IllegalArgumentException(
        "O parmetro graphFileDescriptor est nulo.");
    }
    if (this.outputFileTypeViews.contains(graphFileDescriptor)) {
      return false;
    }
    this.outputFileTypeViews.add(graphFileDescriptor);
    changeVO(null, graphFileDescriptor);
    graphFileDescriptor.changeVS(null, this);
    updateOutputFileTypes();
    return true;
  }

  /**
   * <p>
   * Cria um parmetro na viso.
   * </p>
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * 
   * @param parameterLabel O rtulo do parmetro (No aceita {@code null}).
   * @param parameterType O tipo do parmetro (No aceita {@code null}).
   * 
   * @return {@code true} em caso de sucesso ou {@code false} se j existe um
   *         parmetro com o mesmo nome cadastrado
   */
  public boolean addParameter(String parameterName, String parameterLabel,
    String parameterType) {
    if (parameterName == null) {
      throw new IllegalArgumentException("O parmetro parameterName est nulo.");
    }
    if (parameterLabel == null) {
      throw new IllegalArgumentException(
        "O parmetro parameterLabel est nulo.");
    }
    if (parameterType == null) {
      throw new IllegalArgumentException("O parmetro parameterType est nulo.");
    }
    if (this.parameterNames.contains(parameterName)) {
      return false;
    }
    this.parameterNames.add(parameterName);
    this.parameterLabelsByName.put(parameterName, parameterLabel);
    this.parameterTypesByName.put(parameterName, parameterType);
    return true;
  }

  /**
   * Exibe a viso de configurador de algoritmos para que o usurio possa
   * parametrizar o n.
   */
  public void askForParameterValues() {
    validate(false, true);
    if (getGraph().isFullValidation()) {
      this.configuratorView.setValidationMode(ValidationMode.DEFAULT);
    }
    else {
      this.configuratorView.setValidationMode(ValidationMode.ALLOW_EMPY_VALUES);
    }
    this.configuratorView.setEnabled(true);
    this.configuratorView.launch();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void callbackRepaint(Graphics2D g) {
    Stroke stroke;
    if (isBypassed() && canBeBypassed()) {
      stroke = BYPASSED_STROKE;
    }
    else {
      g.setPaint(this.backgroundColor);
      g.fill(this.shape);
      stroke = DEFAULT_STROKE;
    }

    g.setPaint(this.borderColor);
    g.setStroke(stroke);
    g.draw(this.shape);
    if (isSelected()) {
      for (int i = 0; i < this.cornerViews.length; i++) {
        this.cornerViews[i].paint(g);
      }
    }
    super.callbackRepaint(g);
  }

  /**
   * Comear uma conexo para o arquivo de sada que possui o ponto informado.
   * 
   * @param pt O ponto (No aceita {@code null}).
   * 
   * @return A conexo ou {@code null} seno existir um arquivo de sada no
   *         ponto informado.
   */
  public GraphLink createLink(Point2D pt) {
    for (GraphFileDescriptor outputFileType : this.outputFileTypeViews) {
      if (!outputFileType.isWellFormed()) {
        return null;
      }
      if (outputFileType.contains(pt)) {
        GraphLink link = getGraph().createLink(outputFileType);
        return link;
      }
    }
    return null;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void deattach() {
    for (GraphLink link : getLinkCollection()) {
      link.deattach();
    }
    super.deattach();
  }

  /**
   * Termina de criar a conexo no arquivo de entrada especificado pelo ponto
   * informado.
   * 
   * @param link A conexo (No aceita {@code null}).
   * @param pt O ponto informado (No aceita {@code null}).
   * 
   * @return {@code true} em caso de sucesso ou {@code false} se no houver um
   *         arquivo de entrada no ponto informado.
   */
  public boolean finishLink(GraphLink link, Point2D pt) {
    GraphFileDescriptor inputFileType = getInputFileDescriptor(pt);
    if (inputFileType == null) {
      return false;
    }
    return link.finish(inputFileType);
  }

  /**
   * Obtm o algoritmo do n.
   * 
   * @return O algoritmo do n ou {@code null} se ele no foi informado.
   */
  public AlgorithmInfo getAlgorithm() {
    return this.algorithm;
  }

  /**
   * Obtm a viso de configurador algoritmo do n.
   * 
   * @return A viso de configurador algoritmo do n ou {@code null} se ela no
   *         foi informada.
   */
  public AlgorithmConfiguratorView getAlgorithmConfiguratorView() {
    return this.configuratorView;
  }

  /**
   * Obtm o nome do algoritmo do n.
   * 
   * @return O nome algoritmo do n.
   */
  public String getAlgorithmName() {
    return this.algorithmNameView.getText();
  }

  /**
   * Obtm o identificador da verso do algoritmo.
   * 
   * @return O identificador da verso do algoritmo.
   */
  public AlgorithmVersionId getAlgorithmVersionId() {
    return this.algorithmVersionId;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Rectangle2D getBounds2D() {
    Rectangle2D inputBounds = getBoundsFileTypes(this.inputFileTypeViews);
    Rectangle2D outputBounds = getBoundsFileTypes(this.outputFileTypeViews);
    Rectangle2D bounds = this.shape.getBounds();
    bounds = bounds.createUnion(inputBounds);
    bounds = bounds.createUnion(outputBounds);
    return bounds;
  }

  /**
   * <p>
   * Obtm o ponto cardeal do ponto em relao ao centro do n.
   * </p>
   * 
   * <p>
   * Se o ponto estiver na regio considerada como borda do n (a borda
   * propriamente dita + uma margem interna {@value #BORDER_MARGIN}), este
   * mtodo informa qual  o ponto cardeal relacionado com o ponto.
   * </p>
   * 
   * <p>
   * Exemplos, ou seja, se o ponto estiver prximo do canto superior-esquerdo
   * ser {@link CompassDirection#NORTH_WEST}, se ele estiver na borda inferior
   * e longe dos cantos ser {@link CompassDirection#SOUTH}.
   * </p>
   * 
   * @param pt O ponto (No aceita {@code null}).
   * 
   * @return O ponto cardeal ou {@code null} para um ponto que no esteja na
   *         borda do n.
   */
  public CompassDirection getCompassDirection(Point2D pt) {
    double westX = this.cornerViews[NORTH_WEST_INDEX].getLocation().getX();
    double eastX = this.cornerViews[NORTH_EAST_INDEX].getLocation().getX();
    double northY = this.cornerViews[NORTH_WEST_INDEX].getLocation().getY();
    double southY = this.cornerViews[SOUTH_WEST_INDEX].getLocation().getY();
    if (isContainedInRectangle(westX, northY, westX + BORDER_MARGIN, northY
      + BORDER_MARGIN, pt)) {
      return CompassDirection.NORTH_WEST;
    }
    if (isContainedInRectangle(eastX - BORDER_MARGIN, northY, eastX, northY
      + BORDER_MARGIN, pt)) {
      return CompassDirection.NORTH_EAST;
    }
    if (isContainedInRectangle(westX, southY - BORDER_MARGIN, westX
      + BORDER_MARGIN, southY, pt)) {
      return CompassDirection.SOUTH_WEST;
    }
    if (isContainedInRectangle(eastX - BORDER_MARGIN, southY - BORDER_MARGIN,
      eastX, southY, pt)) {
      return CompassDirection.SOUTH_EAST;
    }
    if (this.cornerViews[SOUTH_EAST_INDEX].getBounds().contains(pt)) {
      return CompassDirection.SOUTH_EAST;
    }
    if (isContainedInRectangle(westX, northY, eastX, northY + BORDER_MARGIN, pt)) {
      return CompassDirection.NORTH;
    }
    if (isContainedInRectangle(westX, southY - BORDER_MARGIN, eastX, southY, pt)) {
      return CompassDirection.SOUTH;
    }
    if (isContainedInRectangle(eastX - BORDER_MARGIN, northY, eastX, southY, pt)) {
      return CompassDirection.EAST;
    }
    if (isContainedInRectangle(westX, northY, westX + BORDER_MARGIN, southY, pt)) {
      return CompassDirection.WEST;
    }

    return null;
  }

  /**
   * Obtm a altura do n.
   * 
   * @return A altura do n.
   */
  public double getHeight() {
    return this.shape.getBounds().getHeight();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String getHint(Point2D pt) {
    GraphFileDescriptor fileType = pickFileType(pt);
    if (fileType != null) {
      return fileType.getHint();
    }
    if (contains(pt)) {
      if (this.icon != null) {
        if (this.icon.getBounds2D().contains(pt)) {
          return LNG.get("algorithms.warning.new_version");
        }
      }
      if (this.decoration != null
        && this.decoration.getType() != DecorationType.BLANK) {
        if (this.decoration.getImage().getBounds2D().contains(pt)) {
          return decoration.getDescription();
        }
      }

      return this.algorithmNameView + " " + this.algorithmVersionIdView;
    }
    return null;
  }

  /**
   * Obtm a viso de arquivo de entrada que esteja localizada no ponto
   * informado.
   * 
   * @param pt O ponto (No aceita {@code null}).
   * 
   * @return A viso ou {@code null} se no houver uma viso de arquivo de
   *         entrada no ponto informado.
   */
  public GraphFileDescriptor getInputFileDescriptor(Point2D pt) {
    for (GraphFileDescriptor inputFileType : this.inputFileTypeViews) {
      if (inputFileType.contains(pt)) {
        return inputFileType;
      }
    }
    return null;
  }

  /**
   * Obtm a viso de arquivo de entrada que cujo nome do parmetro  igual ao
   * nome informado.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * 
   * @return A viso ou {@code null} se no houver uma viso de arquivo de
   *         entrada com o nome informado.
   */
  public GraphFileDescriptor getInputFileDescriptor(String parameterName) {
    for (GraphFileDescriptor inputFileType : this.inputFileTypeViews) {
      if (inputFileType.getParameterName().equals(parameterName)) {
        return inputFileType;
      }
    }
    return null;
  }

  /**
   * <p>
   * Obtm as vises de arquivo de entrada deste n.
   * </p>
   * 
   * <p>
   * A coleo retornada  imutvel (veja
   * {@link Collections#unmodifiableCollection(Collection)}).
   * </p>
   * 
   * @return As vises de arquivo de entrada ou uma coleo vazia caso no haja
   *         vises.
   */
  public Collection<GraphFileDescriptor> getInputFileDescriptorCollection() {
    return Collections.unmodifiableCollection(this.inputFileTypeViews);
  }

  /**
   * Indica se este n  capaz de criar um arquivo de log.
   * 
   * @return .
   */
  public boolean canCreateLogFile() {
    if (configuratorView == null) {
      return false;
    }
    AlgorithmConfigurator configurator = configuratorView.getConfigurator();
    FileParameterValue logFile = configurator.getLogFile();
    if (logFile == null) {
      return false;
    }
    return true;
  }

  /**
   * Obtm o arquivo da sada padro do algoritmo.
   * 
   * @return O arquivo da sada padro ou {@code null} se ele no existir.
   */
  public ClientProjectFile getStandardOutputClientProjectFile() {
    if (configuratorView == null) {
      return null;
    }

    AlgorithmConfigurator configurator = configuratorView.getConfigurator();
    FileParameterValue stdout = configurator.getStandardOutputFile();
    if (stdout == null) {
      return null;
    }
    CommonClientProject project = DesktopFrame.getInstance().getProject();
    ClientProjectFile projectDirectory = (ClientProjectFile) project.getTree();
    String[] path = stdout.getPathAsArray();
    ClientProjectFile previousFile = projectDirectory;
    ClientProjectFile currentFile = null;
    for (String fileName : path) {
      currentFile = GetChildFromNameTask.runTask(previousFile, fileName);
      if (currentFile == null) {
        // Fora um refresh na rvore, pois o arquivo pode existir no servidor,
        // mas ainda no est no cliente.
        if (refreshProjectTree()) {
          return null;
        }
        currentFile = GetChildFromNameTask.runTask(previousFile, fileName);
        if (currentFile == null) {
          return null;
        }
        return null;
      }
      previousFile = currentFile;
    }
    return currentFile;
  }

  /**
   * Atualiza a rvore de projetos.
   * 
   * @return {@code true} em caso de sucesso ou {@code false} em caso de falha.
   */
  private boolean refreshProjectTree() {
    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      protected void performTask() throws Exception {
        CommonClientProject project = DesktopFrame.getInstance().getProject();
        project.refreshTree();
      }
    };
    Graph graph = getGraph();
    Window window = graph.getParentWindow();
    String title = LNG.get(GraphNode.class.getName() + ".refresh_tree.title");
    String message = LNG.get(GraphNode.class.getName() + ".refresh_tree.msg");
    return task.execute(window, title, message);
  }

  /**
   * Obtm a viso de arquivo de sada que esteja localizada no ponto informado.
   * 
   * @param pt O ponto (No aceita {@code null}).
   * 
   * @return A viso ou {@code null} se no houver uma viso de arquivo de sada
   *         no ponto informado.
   */
  public GraphFileDescriptor getOutputFileDescriptor(Point2D pt) {
    for (GraphFileDescriptor outputFileType : this.outputFileTypeViews) {
      if (outputFileType.contains(pt)) {
        return outputFileType;
      }
    }
    return null;
  }

  /**
   * Obtm a viso de arquivo de sada que cujo nome do parmetro  igual ao
   * nome informado.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * 
   * @return A viso ou {@code null} se no houver uma viso de arquivo de sada
   *         com o nome informado.
   */
  public GraphFileDescriptor getOutputFileDescriptor(String parameterName) {
    for (GraphFileDescriptor outputFileType : this.outputFileTypeViews) {
      if (outputFileType.getParameterName().equals(parameterName)) {
        return outputFileType;
      }
    }
    return null;
  }

  /**
   * <p>
   * Obtm as vises de arquivo de sada deste n.
   * </p>
   * 
   * <p>
   * A coleo retornada  imutvel (veja
   * {@link Collections#unmodifiableCollection(Collection)}).
   * </p>
   * 
   * @return As vises de arquivo de sada ou uma coleo vazia caso no haja
   *         vises.
   */
  public Collection<GraphFileDescriptor> getOutputFileDescriptorCollection() {
    return Collections.unmodifiableCollection(this.outputFileTypeViews);
  }

  /**
   * Obtm o rtulo do parmetro cujo nome  igual ao nome indicado.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * 
   * @return O rtulo.
   * 
   * @throws ParameterNotFoundException Se no existir um parmetro com o nome
   *         indicado.
   */
  public String getParameterLabel(String parameterName)
    throws ParameterNotFoundException {
    String label = this.parameterLabelsByName.get(parameterName);
    if (label == null) {
      throw new ParameterNotFoundException(parameterName);
    }
    return label;
  }

  /**
   * <p>
   * Obtm o conjunto dos nomes dos parmetros deste n.
   * </p>
   * 
   * <p>
   * O conjunto retornado  imutvel (veja
   * {@link Collections#unmodifiableSet(Set)}).
   * </p>
   * 
   * @return O conjunto dos nomes ou um conjunto vazio caso no haja parmetros
   *         neste n.
   */
  public Set<String> getParameterNames() {
    return Collections.unmodifiableSet(this.parameterNames);
  }

  /**
   * Obtm o tipo do parmetro cujo nome  igual ao nome indicado.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * 
   * @return O tipo.
   * 
   * @throws ParameterNotFoundException Se no existir um parmetro com o nome
   *         indicado.
   */
  public String getParameterType(String parameterName)
    throws ParameterNotFoundException {
    String type = this.parameterTypesByName.get(parameterName);
    if (type == null) {
      throw new ParameterNotFoundException(parameterName);
    }
    return type;
  }

  /**
   * Obtm o valor do parmetro cujo nome  igual ao nome indicado.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * 
   * @return O valor ({@code null}  um valor vlido).
   * 
   * @throws ParameterNotFoundException Se no existir um parmetro com o nome
   *         indicado.
   */
  public String getParameterValue(String parameterName)
    throws ParameterNotFoundException {
    if (isWellFormed()) {
      return this.configuratorView.getConfigurator().getParameterValue(
        parameterName);
    }
    if (!this.parameterNames.contains(parameterName)) {
      throw new ParameterNotFoundException(parameterName);
    }
    return this.parameterValuesByName.get(parameterName);
  }

  /**
   * Obtm a largura do n.
   * 
   * @return A largura do n.
   */
  public double getWidth() {
    return this.shape.getBounds().getWidth();
  }

  /**
   * Obtm a abscissa esquerda do n.
   * 
   * @return A abscissa esquerda do n.
   */
  public double getX() {
    return this.shape.getBounds().getX();
  }

  /**
   * Obtm a ordenada superior do n.
   * 
   * @return A ordenada superior do n.
   */
  public double getY() {
    return this.shape.getBounds().getY();
  }

  /**
   * Indica se saem conexes deste n.
   * 
   * @return {@code true} se saem ou {@code false} caso contrrio.
   */
  public boolean hasLinksFrom() {
    return !getLinkFromCollection().isEmpty();
  }

  /**
   * Indica se chegam conexes neste n.
   * 
   * @return {@code true} se chegam ou {@code false} caso contrrio.
   */
  public boolean hasLinksTo() {
    return !getLinkToCollection().isEmpty();
  }

  /**
   * <p>
   * Indica se este n est bem-formado.
   * </p>
   * 
   * <p>
   * Um n bem-formado possui um configurador de algoritmos associado a ele.
   * </p>
   * 
   * @return {@code true} se est bem-formado ou {@code false} caso contrrio.
   */
  public boolean isWellFormed() {
    return (getAlgorithmConfiguratorView() != null);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean msgHandlerVO(TypeMessage message) {
    if (message instanceof PickNodeMessage) {
      return handlePickNodeMessage((PickNodeMessage) message);
    }
    if (message instanceof ShowParameterValuesMessage) {
      return handleShowParameterValuesMessage();
    }
    return super.msgHandlerVO(message);
  }

  /**
   * L o arquivo de log e o escrever no writer informado.
   * 
   * @param writer O escritor (No aceita {@code null}).
   * 
   * @throws IOException Se houver um erro ao ler o arquivo.
   */
  public void readStandardOutputFile(Writer writer) throws IOException {
    if (writer == null) {
      throw new IllegalArgumentException("writer == null");
    }
    PrintWriter printWriter = new PrintWriter(writer);
    ClientProjectFile logFile = getStandardOutputClientProjectFile();
    if (logFile != null) {
      InputStream inputStream = null;
      BufferedReader bufferedReader = null;
      try {
        inputStream = new ClientProjectFileInputStream(logFile, 0);
        bufferedReader = new BufferedReader(new InputStreamReader(inputStream));
        printWriter.printf("\n-- %s %s --\n", getAlgorithmName(),
          getAlgorithmVersionId());
        String line = bufferedReader.readLine();
        while (line != null) {
          printWriter.println(line);
          line = bufferedReader.readLine();
        }
      }
      finally {
        if (bufferedReader != null) {
          bufferedReader.close();
          bufferedReader = null;
        }
        else if (inputStream != null) {
          inputStream.close();
          inputStream = null;
        }
      }
    }
  }

  /**
   * Redimensiona o n.
   * 
   * @param pt O ponto final (No aceita {@code null}).
   * @param compassDirection O sentido do redirecionamento (No aceita {@code
   *        null}).
   */
  public void resize(Point2D pt, CompassDirection compassDirection) {
    if (CompassDirection.NORTH.equals(compassDirection)) {
      resizeNorth(pt);
    }
    else if (CompassDirection.SOUTH.equals(compassDirection)) {
      resizeSouth(pt);
    }
    else if (CompassDirection.EAST.equals(compassDirection)) {
      resizeEast(pt);
    }
    else if (CompassDirection.WEST.equals(compassDirection)) {
      resizeWest(pt);
    }
    else if (CompassDirection.NORTH_EAST.equals(compassDirection)) {
      resizeNorthEast(pt);
    }
    else if (CompassDirection.NORTH_WEST.equals(compassDirection)) {
      resizeNorthWest(pt);
    }
    else if (CompassDirection.SOUTH_EAST.equals(compassDirection)) {
      resizeSouthEast(pt);
    }
    else if (CompassDirection.SOUTH_WEST.equals(compassDirection)) {
      resizeSouthWest(pt);
    }
    for (GraphElementListener listener : getListenerList()) {
      listener.wasResized(this);
    }
  }

  /**
   * Atribui os valores dos parmetros a este n.
   * 
   * @param parameterValuesByName Uma tabela que associa os valores dos
   *        parmetros aos seus nomes (No aceita {@code null}).</br> Os
   *        parmetros que no esto representados nesta tabela tero valor
   *        {@code null} e valores {@code null} so vlidos.
   * 
   * @throws ParseException Se o valor passado no puder ser convertido para o
   *         valor armazenado no parmetro. Exemplo: quando tentamos atribuir o
   *         texto <i>abcd</i> em um parmetro que aceita apenas valores
   *         inteiros.
   */
  public void setParameterValuesByName(Map<String, String> parameterValuesByName)
    throws ParseException {
    if (isWellFormed()) {
      AlgorithmConfigurator configurator =
        this.configuratorView.getConfigurator();
      configurator.setParameterValuesByName(parameterValuesByName);
    }
    else {
      for (String parameterName : parameterValuesByName.keySet()) {
        String parameterValue = parameterValuesByName.get(parameterName);
        if (this.parameterNames.contains(parameterName)) {
          this.parameterValuesByName.put(parameterName, parameterValue);
        }
      }
    }
  }

  /**
   * Retorna uma mensagem de erro informando porque o nome est mal-formado (o
   * algoritmo no existe ou a verso no existe).
   * 
   * @return A mensagem de erro.
   */
  protected String getErrorMessage() {
    String errorMessage;
    if (this.algorithm == null) {
      errorMessage =
        MessageFormat.format(LNG.get(GraphNode.class.getName()
          + ".error_algorithm_not_exists"), this.algorithmNameView.getText());
    }
    else {
      errorMessage =
        MessageFormat.format(LNG.get(GraphNode.class.getName()
          + ".error_version_not_exists"), this.algorithmNameView.getText(),
          this.algorithmVersionIdView);
    }
    return errorMessage;
  }

  /**
   * Exibe uma mensagem de erro informando porque o nome est mal-formado (o
   * algoritmo no existe ou a verso no existe).
   */
  public void showError() {
    new ErrorMessage(getErrorMessage()).sendVS(this);
  }

  /**
   * Exibe o log de um n.
   * 
   * @return {@code true} se puder exibir ou {@code false} caso no haja um log.
   */
  public boolean showLogFile() {
    ClientProjectFile logFile = getStandardOutputClientProjectFile();
    if (logFile == null) {
      return false;
    }
    try {
      ApplicationManager applicationManager = ApplicationManager.getInstance();
      applicationManager.runApplication(logFile);
    }
    catch (ApplicationException e) {
      Window window = getGraph().getParentWindow();
      StandardErrorDialogs.showErrorDialog(window, e.getMessage(), e);
      return false;
    }
    return true;
  }

  /**
   * Exibe os valores dos parmetros do n.
   */
  public void showParameterValues() {
    if (this.configuratorView != null) {
      this.configuratorView.setEnabled(false);
      this.configuratorView.launch();
    }
    else {
      new NodeParameterViewer(this).execute();
    }
  }

  /**
   * Preende o nicio da conexo a um arquivo de sada deste n.
   * 
   * @param link A conexo (No aceita {@code null}).
   * 
   * @param pt O ponto (No aceita {@code null}).
   * 
   * @return {@code true} em caso de sucesso ou {@code false} se no h um
   *         arquivo de entrada no ponto informado.
   */
  public boolean startLink(GraphLink link, Point2D pt) {
    for (GraphFileDescriptor outputFileType : this.outputFileTypeViews) {
      if (outputFileType.contains(pt)) {
        return link.start(outputFileType);
      }
    }
    return false;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String toString() {
    return this.algorithmNameView.getText();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void turnOffHighlight() {
    for (GraphFileDescriptor inputFileType : this.inputFileTypeViews) {
      inputFileType.turnOffHightLight();
    }
    for (GraphFileDescriptor outputFileType : this.outputFileTypeViews) {
      outputFileType.turnOffHightLight();
    }
    this.borderColor = DEFAULT_FOREGROUNG_COLOR;
    Color textColor;
    if (isBypassed() && canBeBypassed()) {
      textColor = BYPASSED_TEXT_COLOR;
    }
    else {
      textColor = DEFAULT_FOREGROUNG_COLOR;
    }
    this.algorithmNameView.setColor(textColor);
    this.algorithmVersionIdView.setColor(textColor);
  }

  /**
   * Destaca o n.
   */
  public void turnOnHighlight() {
    for (GraphFileDescriptor inputFileType : this.inputFileTypeViews) {
      inputFileType.turnOnHightLight();
    }
    for (GraphFileDescriptor outputFileType : this.outputFileTypeViews) {
      outputFileType.turnOnHightLight();
    }
    this.borderColor = HIGHT_LIGHT_COLOR;
    this.algorithmNameView.setColor(HIGHT_LIGHT_COLOR);
    this.algorithmVersionIdView.setColor(HIGHT_LIGHT_COLOR);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean turnOnHighlight(Point2D pt) {
    if (!contains(pt)) {
      return false;
    }
    for (GraphFileDescriptor inputFileType : this.inputFileTypeViews) {
      if (inputFileType.contains(pt)) {
        inputFileType.turnOnHightLight();
        return true;
      }
    }
    for (GraphFileDescriptor outputFileType : this.outputFileTypeViews) {
      if (outputFileType.contains(pt)) {
        outputFileType.turnOnHightLight();
        return true;
      }
    }
    this.borderColor = HIGHT_LIGHT_COLOR;
    this.algorithmNameView.setColor(HIGHT_LIGHT_COLOR);
    this.algorithmVersionIdView.setColor(HIGHT_LIGHT_COLOR);
    return true;
  }

  /**
   * <p>
   * Valida os valores dos parmetros e conexes deste n.
   * </p>
   * 
   * <p>
   * Ele pode exibir mensagens de com o motivo do erro de validao. Ele
   * atualiza a cor do n para estar de acordo com o estado (pronto para
   * executar ou no-pronto para executar).
   * </p>
   * 
   * <p>
   * O n est invlido se  mal-formado ou se o configurador de algoritmos for
   * invlido ou se pelo menos um dos arquivos de entrada ou de sada forem
   * invlidos.
   * </p>
   * 
   * <p>
   * O configurador  invlido se os valores dos parmetros no forem adequados.
   * </p>
   * 
   * <p>
   * Os arquivos so invlidos se esto associados a conexes mal-formadas.
   * </p>
   * 
   * @param complain Indica se deve existir mensagens para o usurio.
   * 
   * @param isFullValidation Indica se ir utilizar validao completa.</br> Na
   *        validao completa, os parmetros obrigatrios no precisam estar
   *        preenchidos.
   * 
   * @return O resultado da validao.
   */
  public ValidationResult validate(boolean complain, boolean isFullValidation) {
    if (!isWellFormed()) {
      if (complain) {
        showError();
      }
      String errorMessage = getErrorMessage();
      return new ElementValidationResult<GraphNode>(errorMessage, this);
    }
    try {
      this.backgroundColor = NOT_READY_BACKGROUND_COLOR;
      ValidationResult validation = validateInputFiles();
      if (!validation.isWellSucceded()) {
        return validation;
      }
      validation = validateOutputFiles();
      if (!validation.isWellSucceded()) {
        return validation;
      }
      validation = validateAlgorithmConfigurator(complain, isFullValidation);
      if (!validation.isWellSucceded()) {
        return validation;
      }
      if (isBypassed() && canBeBypassed()) {
        this.algorithmNameView.setColor(BYPASSED_TEXT_COLOR);
        this.algorithmVersionIdView.setColor(BYPASSED_TEXT_COLOR);
      }
      else {
        this.algorithmNameView.setColor(DEFAULT_FOREGROUNG_COLOR);
        this.algorithmVersionIdView.setColor(DEFAULT_FOREGROUNG_COLOR);
      }
      return new ValidationResult();
    }
    finally {
      repaint();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected boolean contains(Point2D pt) {
    return getBounds2D().contains(pt);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void doDrag(double tx, double ty) {
    double x = this.northWestPoint.getX() + tx;
    double y = this.northWestPoint.getY() + ty;
    this.northWestPoint.setLocation(x, y);
    if (this.icon != null) {
      updateIconLocation();
    }
    if (this.decoration != null
      && this.decoration.getType() != DecorationType.BLANK) {
      updateDecorationLocation();
    }
    createShape();
    updateFileTypes();
    updateTextLocation();
    adjustGrid();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected boolean doDrag(Point2D startPoint, Point2D endPoint) {
    double tx = endPoint.getX() - startPoint.getX();
    double ty = endPoint.getY() - startPoint.getY();
    Point2D newPoint =
      new Point2D.Double(this.northWestPoint.getX() + tx, this.northWestPoint
        .getY()
        + ty);
    this.northWestPoint.setLocation(newPoint);
    if (this.icon != null) {
      updateIconLocation();
    }
    if (this.decoration != null
      && this.decoration.getType() != DecorationType.BLANK) {
      updateDecorationLocation();
    }
    createShape();
    updateFileTypes();
    updateTextLocation();
    adjustGrid();
    return true;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void doDrop(Point2D pt) {
    doDrag(getFirstPoint(), Grid.adjustPoint(getFirstPoint()));
  }

  /**
   * Indica se  possvel chegar ao n informado navegando pelas conexes que se
   * originam deste n.
   * 
   * @param node O n (No aceita {@code null}).
   * 
   * @return {@code true} se  possvel chegar at o n ou {@code false} caso
   *         contrrio.
   */
  boolean canReach(GraphNode node) {
    if (equals(node)) {
      return true;
    }
    for (GraphLink link : getLinkFromCollection()) {
      GraphNode endNode = link.getEndNode();
      if (endNode != null) {
        if (endNode.canReach(node)) {
          return true;
        }
      }
    }
    return false;
  }

  /**
   * <p>
   * Obtm todas conexes que esto ligadas a este n.
   * </p>
   * 
   * <p>
   * A coleo retorna  imutvel (veja
   * {@link Collections#unmodifiableCollection(Collection)}).
   * </p>
   * 
   * @return A coleo das conexes ou uma coleo vazia caso no haja conexes
   *         ligadas a este n.
   */
  public Collection<GraphLink> getLinkCollection() {
    Set<GraphLink> linkSet = new HashSet<GraphLink>();
    linkSet.addAll(getLinkFromCollection());
    linkSet.addAll(getLinkToCollection());
    return linkSet;
  }

  /**
   * <p>
   * Obtm todas conexes que se originam deste n.
   * </p>
   * 
   * <p>
   * A coleo retorna  imutvel (veja
   * {@link Collections#unmodifiableCollection(Collection)}).
   * </p>
   * 
   * @return A coleo das conexes ou uma coleo vazia caso no haja conexes
   *         ligadas a este n.
   */
  public Collection<GraphLink> getLinkFromCollection() {
    Collection<GraphLink> linkFromCollection = new LinkedList<GraphLink>();
    for (GraphFileDescriptor outputFileType : this.outputFileTypeViews) {
      linkFromCollection.addAll(outputFileType.getLinkFromCollection());
    }
    return linkFromCollection;
  }

  /**
   * <p>
   * Obtm todas conexes que se destinam a este n.
   * </p>
   * 
   * <p>
   * A coleo retorna  imutvel (veja
   * {@link Collections#unmodifiableCollection(Collection)}).
   * </p>
   * 
   * @return A coleo das conexes ou uma coleo vazia caso no haja conexes
   *         ligadas a este n.
   */
  public Collection<GraphLink> getLinkToCollection() {
    Collection<GraphLink> linkToCollection = new LinkedList<GraphLink>();
    for (GraphFileDescriptor inputFileType : this.inputFileTypeViews) {
      GraphLink link = inputFileType.getLinkTo();
      if (link != null) {
        linkToCollection.add(link);
      }
    }
    return linkToCollection;
  }

  /**
   * Verifica se este n est pronto para executar.
   * 
   * @return {@code true} se estiver pront ou {@code false} caso contrrio.
   */
  boolean isReadyToExecute() {
    return this.backgroundColor.equals(READY_BACKGROUND_COLOR);
  }

  /**
   * Ajusta a forma do n para que ele se alinhe a grade.
   */
  private void adjustGrid() {
    Double tX = null;
    Double tY = null;
    if (getBounds2D().getMinX() < 0) {
      tX = -1 * getBounds2D().getMinX();
    }
    if (getBounds2D().getMinY() < 0) {
      tY = -1 * getBounds2D().getMinY();
    }
    if (tX == null && tY == null) {
      return;
    }
    if (tX == null) {
      tX = 0.0;
    }
    if (tY == null) {
      tY = 0.0;
    }
    doDrag(tX, tY);
  }

  /**
   * Calcula as medidas de todos os componentes visuais que esto no n (cantos,
   * os textos e a forma dele).
   * 
   * @param centerPoint O ponto central (No aceita {@code null}).
   */
  private void computeMeasures(Point2D centerPoint) {
    double width =
      Math.max(MIN_WIDTH, this.algorithmNameView.getBounds2D().getWidth());
    double height = this.algorithmNameView.getBounds2D().getHeight();
    width =
      Math.max(width, this.algorithmVersionIdView.getBounds2D().getWidth());
    if (this.icon != null) {
      double iconWidth = this.icon.getWidth();
      width += iconWidth;
    }
    if (this.decoration != null
      && this.decoration.getType() != DecorationType.BLANK) {
      double decorationWidth = this.decoration.getImage().getWidth();
      width += decorationWidth;
    }

    height += this.algorithmVersionIdView.getBounds2D().getHeight();
    height = Math.max(MIN_HEIGHT, height);
    if (this.icon != null) {
      double iconHeight = this.icon.getHeight();
      height = Math.max(height, iconHeight);
    }
    if (this.decoration != null
      && this.decoration.getType() != DecorationType.BLANK) {
      double decorationHeight = this.decoration.getImage().getHeight();
      height = Math.max(height, decorationHeight);
    }
    double centerX = centerPoint.getX();
    double centerY = centerPoint.getY();
    double x0 = centerX - width / 2;
    double y0 = centerY - height / 2;
    setBounds2D(x0, y0, width, height);
  }

  /**
   * Cria as vises dos cantos.
   */
  private void createCornerPoints() {
    Rectangle2D boundBox = this.shape.getBounds2D();
    this.cornerViews = new GraphPoint[4];
    this.cornerViews[NORTH_WEST_INDEX] =
      new GraphPoint(boundBox.getX(), boundBox.getY());
    this.cornerViews[NORTH_EAST_INDEX] =
      new GraphPoint(boundBox.getX() + boundBox.getWidth(), boundBox.getY());
    this.cornerViews[SOUTH_EAST_INDEX] =
      new GraphPoint(boundBox.getX() + boundBox.getWidth(), boundBox.getY()
        + boundBox.getHeight());
    this.cornerViews[SOUTH_WEST_INDEX] =
      new GraphPoint(boundBox.getX(), boundBox.getY() + boundBox.getHeight());
  }

  /**
   * <p>
   * Cria as vises dos arquivos de entrada.
   * </p>
   * 
   * <p>
   * S deve ser invocado em ns bem-formados.
   * </p>
   */
  private void createInputFileTypes() {
    this.inputFileTypeViews = new ArrayList<GraphFileDescriptor>();
    for (InputFileParameter inputFileParameter : this.configuratorView
      .getConfigurator().getInputFileParameters()) {
      if (inputFileParameter.getMode().equals(FileParameterMode.REGULAR_FILE)
        && (inputFileParameter.usesPipe().equals(
          FileParameterPipeAcceptance.TRUE) || inputFileParameter.usesPipe()
          .equals(FileParameterPipeAcceptance.ALWAYS))) {
        this.inputFileTypeViews.add(new GraphFileDescriptor(inputFileParameter,
          this));
      }
    }
  }

  /**
   * <p>
   * Cria as vises dos arquivos de sada.
   * </p>
   * 
   * <p>
   * S deve ser invocado em ns bem-formados.
   * </p>
   */
  private void createOutputFileTypes() {
    this.outputFileTypeViews = new ArrayList<GraphFileDescriptor>();
    for (OutputFileParameter outputFileParameter : this.configuratorView
      .getConfigurator().getOutputFileParameters()) {
      if (!outputFileParameter.isLogFile()
        && outputFileParameter.getMode().equals(FileParameterMode.REGULAR_FILE)
        && (outputFileParameter.usesPipe().equals(
          FileParameterPipeAcceptance.TRUE) || outputFileParameter.usesPipe()
          .equals(FileParameterPipeAcceptance.ALWAYS))) {
        this.outputFileTypeViews.add(new GraphFileDescriptor(
          outputFileParameter, this));
      }
    }
  }

  /**
   * Cria a forma do n.
   */
  private void createShape() {
    this.shape =
      new Rectangle2D.Double(this.northWestPoint.getX(), this.northWestPoint
        .getY(), this.dimension.getWidth(), this.dimension.getHeight());
    createCornerPoints();
  }

  /**
   * Cria uma caixa envolvente que engloba todos as vises de arquivo.
   * 
   * @param fileTypeCollection A coleo de vises de arquivo (No aceita
   *        {@code null}).
   * 
   * @return A caixa envolvente.
   */
  private Rectangle2D getBoundsFileTypes(
    Collection<GraphFileDescriptor> fileTypeCollection) {
    Rectangle2D bounds = this.shape.getBounds2D();
    for (GraphFileDescriptor fileType : fileTypeCollection) {
      bounds = bounds.createUnion(fileType.getBounds2D());
    }
    return bounds;
  }

  /**
   * Obtm o ponto central do n.
   * 
   * @return O ponto central.
   */
  private Point2D getCenterPoint() {
    Point2D centerPoint = new Point();
    centerPoint.setLocation(getBounds2D().getCenterX(), getBounds2D()
      .getCenterY());
    return centerPoint;
  }

  /**
   * Obtm o ponto superior-esquerdo deste n.
   * 
   * @return O ponto superior-esquerdo.
   */
  private Point2D getFirstPoint() {
    Point2D firstPoint = new Point();
    firstPoint.setLocation(this.shape.getBounds2D().getMinX(), this.shape
      .getBounds2D().getMinY());
    return firstPoint;
  }

  /**
   * Trata mensagens do tipo Pegar N.
   * 
   * @param message A mensagem (No aceita {@code null}).
   * 
   * @return {@code true} se a mensagem era para pegar este n ou {@code false}
   *         se a mensagem no era para pegar este n.
   */
  private boolean handlePickNodeMessage(PickNodeMessage message) {
    if (!contains(message.getPoint())) {
      return false;
    }
    message.setNode(this);
    return true;
  }

  /**
   * Trata a mensagem exibir valores dos parmetros se estiver selecionado.
   * 
   * @return {@code true} se ele tratou a mensagem ou {@code false} se ele no
   *         tratou a mensagem.
   */
  private boolean handleShowParameterValuesMessage() {
    if (isSelected()) {
      showParameterValues();
      return true;
    }
    return false;
  }

  /**
   * Pega uma viso de arquivo.
   * 
   * @param pt O ponto (No aceita {@code null}).
   * 
   * @return A viso para o arquivo ou {@code null} caso no haja uma viso no
   *         ponto informado.
   */
  private GraphFileDescriptor pickFileType(Point2D pt) {
    GraphFileDescriptor fileType = pickFileType(pt, this.outputFileTypeViews);
    if (fileType != null) {
      return fileType;
    }
    return pickFileType(pt, this.inputFileTypeViews);
  }

  /**
   * Pega uma viso de arquivo entre as vises informadas.
   * 
   * @param pt O ponto (No aceita {@code null}).
   * @param fileTypeCollection A coleo de viso de arquivos (No aceita
   *        {@code null}).
   * 
   * @return A viso para o arquivo ou {@code null} caso no haja uma viso no
   *         ponto informado.
   */
  private GraphFileDescriptor pickFileType(Point2D pt,
    Collection<GraphFileDescriptor> fileTypeCollection) {
    for (GraphFileDescriptor fileType : fileTypeCollection) {
      if (fileType.pick(pt) != null) {
        return fileType;
      }
    }
    return null;
  }

  /**
   * Redimensiona o n alterando a posio da borda leste.
   * 
   * @param pt O ponto (No aceita {@code null}).
   */
  private void resizeEast(Point2D pt) {
    Rectangle2D bounds = this.shape.getBounds2D();
    double newX = pt.getX();
    double eastX = bounds.getMaxX();
    double x = bounds.getMinX();
    double y = bounds.getMinY();
    double height = bounds.getHeight();
    double width = bounds.getWidth();
    double newWidth = width + newX - eastX;
    setBounds2D(x, y, newWidth, height);
  }

  /**
   * Redimensiona o n alterando a posio da borda norte.
   * 
   * @param pt O ponto (No aceita {@code null}).
   */
  private void resizeNorth(Point2D pt) {
    Rectangle2D bounds = this.shape.getBounds2D();
    double x = bounds.getMinX();
    double northY = bounds.getMinY();
    double newY = pt.getY();
    double width = bounds.getWidth();
    double height = bounds.getHeight();
    double newHeight = height + northY - newY;
    if (newHeight < MIN_HEIGHT) {
      newY = bounds.getMaxY() - MIN_HEIGHT;
      newHeight = MIN_HEIGHT;
    }
    setBounds2D(x, newY, width, newHeight);
  }

  /**
   * Redimensiona o n alterando a posio do canto nordeste.
   * 
   * @param pt O ponto (No aceita {@code null}).
   */
  private void resizeNorthEast(Point2D pt) {
    resizeNorth(pt);
    resizeEast(pt);
  }

  /**
   * Redimensiona o n alterando a posio do canto noroeste.
   * 
   * @param pt O ponto (No aceita {@code null}).
   */
  private void resizeNorthWest(Point2D pt) {
    resizeNorth(pt);
    resizeWest(pt);
  }

  /**
   * Redimensiona o n alterando a posio da borda sul.
   * 
   * @param pt O ponto (No aceita {@code null}).
   */
  private void resizeSouth(Point2D pt) {
    Rectangle2D bounds = this.shape.getBounds2D();
    double x = bounds.getMinX();
    double y = bounds.getMinY();
    double newY = pt.getY();
    double southY = bounds.getMaxY();
    double width = bounds.getWidth();
    double height = bounds.getHeight();
    double newHeight = height + newY - southY;
    setBounds2D(x, y, width, newHeight);
  }

  /**
   * Redimensiona o n alterando a posio do canto sudeste.
   * 
   * @param pt O ponto (No aceita {@code null}).
   */
  private void resizeSouthEast(Point2D pt) {
    resizeSouth(pt);
    resizeEast(pt);
  }

  /**
   * Redimensiona o n alterando a posio do canto sudoeste.
   * 
   * @param pt O ponto (No aceita {@code null}).
   */
  private void resizeSouthWest(Point2D pt) {
    resizeSouth(pt);
    resizeWest(pt);
  }

  /**
   * Redimensiona o n alterando a posio da borda oeste.
   * 
   * @param pt O ponto (No aceita {@code null}).
   */
  private void resizeWest(Point2D pt) {
    Rectangle2D bounds = this.shape.getBounds2D();
    double newX = pt.getX();
    double westX = bounds.getMinX();
    double y = bounds.getMinY();
    double height = bounds.getHeight();
    double width = bounds.getWidth();
    double newWidth = width + westX - newX;
    if (newWidth < MIN_WIDTH) {
      newX = bounds.getMaxX() - MIN_WIDTH;
      newWidth = MIN_WIDTH;
    }
    setBounds2D(newX, y, newWidth, height);
  }

  /**
   * Modifica a posio e as dimenses do n.
   * 
   * @param x A abscissa esquerda.
   * @param y A ordenada direita.
   * @param width A largura.
   * @param height A altura.
   */
  private void setBounds2D(double x, double y, double width, double height) {
    double minWidth = MIN_WIDTH;
    double minHeight = MIN_HEIGHT;
    Double iconWidth = null;
    Double decorationWidth = null;

    if (this.icon != null) {
      iconWidth = this.icon.getBounds2D().getWidth();
      minWidth += iconWidth;
    }
    if (this.decoration != null
      && this.decoration.getType() != DecorationType.BLANK) {
      decorationWidth = this.decoration.getImage().getBounds2D().getWidth();
      minWidth += decorationWidth;
    }

    if (width < minWidth) {
      width = minWidth;
    }
    if (height < minHeight) {
      height = minHeight;
    }
    Rectangle2D newBounds =
      Grid.adjustBounds(new Rectangle2D.Double(x, y, width, height));
    x = newBounds.getMinX();
    y = newBounds.getMinY();
    width = newBounds.getWidth();
    height = newBounds.getHeight();
    this.northWestPoint = new Point();
    this.northWestPoint.setLocation(x, y);
    this.dimension = new Dimension();
    this.dimension.setSize(width, height);
    if (this.icon != null) {
      updateIconLocation();
    }
    if (this.decoration != null
      && this.decoration.getType() != DecorationType.BLANK) {
      updateDecorationLocation();
    }

    createShape();
    updateFileTypes();
    if (iconWidth != null) {
      width -= iconWidth;
    }
    this.algorithmNameView.setMaxWidth(width);
    this.algorithmVersionIdView.setMaxWidth(width);
    updateTextLocation();
  }

  /**
   * Configura os observadores.
   */
  private void setListeners() {
    getGraph().addGraphListener(new GraphListener() {
      public void wasChangedWorkspace(Graph graph) {
      }

      public void wasElementCreated(Graph graph, GraphElement element) {
        validateBypass();
        validate(false, true);
      }

      public void wasElementDragged(Graph graph, GraphElement element,
        double tx, double ty) {
      }

      public void wasElementDragged(Graph graph, GraphElement element,
        Point2D startPoint, Point2D endPoint) {
      }

      public void wasElementDropped(Graph graph, GraphElement element,
        Point2D pt) {
      }

      public void wasElementParametrized(Graph graph, GraphElement element) {
      }

      public void wasElementRemoved(Graph graph, GraphElement element) {
        validateBypass();
        validate(false, true);
      }

      public void wasElementSelected(Graph graph, GraphElement element) {
      }

      public void wasLinkAnchored(Graph graph, GraphLink link,
        GraphFileDescriptor fileDescriptor) {
        validateBypass();
        validate(false, true);
      }

      public void wasLinkIncreased(Graph graph, GraphLink link) {
      }

      public void wasLinkStatusChanged(Graph graph, GraphLink link) {
        validateBypass();
        validate(false, true);
      }

      public void wasLinkUnanchored(Graph graph, GraphLink link,
        GraphFileDescriptor fileDescriptor) {
        validateBypass();
        validate(false, true);
      }

      public void wasNodeResized(Graph graph, GraphNode node) {
      }

      public void wasParameterSetEnabled(Graph graph, GraphNode node,
        String parameterName, boolean isEnabled) {
        validateBypass();
        validate(false, true);
      }

      public void wasReseted(Graph graph) {
      }

      public void wasParameterSetVisible(Graph graph, GraphNode node,
        String parameterName, boolean isVisible) {
        validateBypass();
        validate(false, true);
      }
    });
    this.configuratorView
      .addAlgorithmConfiguratorViewListener(new AlgorithmConfiguratorViewListener() {

        public void wasConfirmed(AlgorithmConfiguratorView view) {
          for (GraphElementListener listener : getListenerList()) {
            listener.wasParametrized(GraphNode.this);
          }
        }

        public void wasCancelled(AlgorithmConfiguratorView view) {
        }
      });
    AlgorithmConfigurator configurator =
      this.configuratorView.getConfigurator();
    configurator
      .addAlgorithmConfiguratorListener(new AlgorithmConfiguratorListener() {
        public void parameterLabelWasChanged(
          AlgorithmConfigurator configurator, String name, String label) {
        }

        public void parameterWasSetEnabled(AlgorithmConfigurator configurator,
          String parameterName, boolean parameterIsEnabled) {
          validateBypass();
          validate(false, true);
          for (GraphElementListener listener : getListenerList()) {
            listener.wasParameterSetEnabled(GraphNode.this, parameterName,
              parameterIsEnabled);
          }
        }

        public void parameterWasSetVisible(AlgorithmConfigurator configurator,
          String parameterName, boolean parameterIsVisible) {
          validateBypass();
          validate(false, true);
          for (GraphElementListener listener : getListenerList()) {
            listener.wasParameterSetVisible(GraphNode.this, parameterName,
              parameterIsVisible);
          }
        }

        public <V> void parameterValueWasChanged(
          AlgorithmConfigurator configurator, String parameterName,
          V parameterValue) {
          validateBypass();
          validate(false, true);
        }

        public void wasSetEnabled(AlgorithmConfigurator configurator) {
        }
      });
  }

  /**
   * Configura a cadeida de objetos <i>vix</i>.
   */
  private void setUpVix() {
    changeVO(null, this.algorithmNameView);
    this.algorithmNameView.changeVS(null, this);
    changeVO(null, this.algorithmVersionIdView);
    this.algorithmVersionIdView.changeVS(null, this);
    for (GraphFileDescriptor inputFileType : this.inputFileTypeViews) {
      changeVO(null, inputFileType);
      inputFileType.changeVS(null, this);
    }
    for (GraphFileDescriptor outputFileType : this.outputFileTypeViews) {
      changeVO(null, outputFileType);
      outputFileType.changeVS(null, this);
    }
    if (this.icon != null) {
      changeVO(null, this.icon);
      this.icon.changeVS(null, this);
    }

    if (this.decoration != null) {
      changeVO(null, this.decoration.getImage());
      this.decoration.getImage().changeVS(null, this);
    }
  }

  /**
   * Verifica se o ponto informado est contido no retngulo formado pelos
   * pontos ({@code x1}, {@code y1}) e ({@code x2}, {@code y2}).
   * 
   * @param x1 A abscissa esquerda.
   * @param y1 A ordenada superior.
   * @param x2 A abscissa direita.
   * @param y2 A ordenada inferior.
   * 
   * @param pt O ponto procurado (No aceita {@code null}).
   * 
   * @return {@code true} se estiver contido no retngulo ou {@code false} caso
   *         contrrio.
   */
  private boolean isContainedInRectangle(double x1, double y1, double x2,
    double y2, Point2D pt) {
    Rectangle2D bounds = new Rectangle2D.Double();
    bounds.setFrameFromDiagonal(x1, y1, x2, y2);
    return bounds.contains(pt);
  }

  /**
   * Atualiza a posio das vises de arquivo.
   */
  private void updateFileTypes() {
    updateInputFileTypes();
    updateOutputFileTypes();
  }

  /**
   * Atualiza a posio do cone principal.
   */
  private void updateIconLocation() {
    double x = this.northWestPoint.getX();
    double y = this.northWestPoint.getY();
    double height = this.dimension.getHeight();
    this.icon.setX(x);
    this.icon.setCenterY(y + (height / 2));
  }

  /**
   * Atualiza a posio do cone de decorao.
   */
  private void updateDecorationLocation() {
    double nodeX = this.northWestPoint.getX();
    double nodeY = this.northWestPoint.getY();
    double nodeHeight = this.dimension.getHeight();
    double nodeWidth = this.dimension.getWidth();
    double padding = 5;

    double decorationX =
      nodeX + nodeWidth - decoration.getImage().getWidth() - padding;
    this.decoration.getImage().setX(decorationX);
    double decorationY =
      nodeY + nodeHeight - decoration.getImage().getHeight() - padding;
    this.decoration.getImage().setY(decorationY);
  }

  /**
   * Atualiza a posio das vises de arquivo de entrada.
   */
  private void updateInputFileTypes() {
    if (this.inputFileTypeViews == null) {
      createInputFileTypes();
    }
    Rectangle2D bounds = this.shape.getBounds2D();
    double leftX = bounds.getMinX();
    double rightX = bounds.getMaxX();
    double upperY = bounds.getMinY();
    double inputInterval =
      (rightX - leftX) / (this.inputFileTypeViews.size() + 1.0);
    double x = leftX;
    for (GraphFileDescriptor inputFileType : this.inputFileTypeViews) {
      x += inputInterval;
      Point2D pt = new Point();
      pt.setLocation(x, upperY);
      inputFileType.setLocation(pt);
    }
  }

  /**
   * Atualiza a posio das vises de arquivo de sada.
   */
  private void updateOutputFileTypes() {
    if (this.outputFileTypeViews == null) {
      createOutputFileTypes();
    }
    Rectangle2D bounds = this.shape.getBounds2D();
    double leftX = bounds.getMinX();
    double rightX = bounds.getMaxX();
    double lowerY = bounds.getMaxY();
    double outputInterval =
      (rightX - leftX) / (this.outputFileTypeViews.size() + 1.0);
    double x = leftX;
    for (GraphFileDescriptor outputFileType : this.outputFileTypeViews) {
      x += outputInterval;
      Point2D pt = new Point();
      pt.setLocation(x, lowerY);
      outputFileType.setLocation(pt);
    }
  }

  /**
   * Atualiza a posio dos textos.
   */
  private void updateTextLocation() {
    double availableX = getCenterPoint().getX();
    if (this.icon != null) {
      availableX += this.icon.getWidth() / 2;
      if (this.decoration != null
        && this.decoration.getType() != DecorationType.BLANK) {
        availableX -= this.decoration.getImage().getWidth() / 2;
      }
    }

    this.algorithmNameView.setLocation(availableX, this.northWestPoint.getY()
      + this.dimension.getHeight() * 1 / 4);
    this.algorithmVersionIdView.setLocation(getCenterPoint().getX(),
      this.northWestPoint.getY() + this.dimension.getHeight() * 3 / 4);
  }

  /**
   * <p>
   * Valida os valores dos parmetros deste n.
   * </p>
   * 
   * <p>
   * Ele pode exibir mensagens de com o motivo do erro de validao. Ele
   * atualiza a cor do n para estar de acordo com o estado (pronto para
   * executar ou no-pronto para executar).
   * </p>
   * 
   * <p>
   * Este n precisa estar bem-formado.
   * </p>
   * 
   * <p>
   * O configurador  invlido se os valores dos parmetros no forem adequados.
   * </p>
   * 
   * @param complain Indica se deve existir mensagens para o usurio.
   * 
   * @param isFullValidation Indica se ir utilizar validao completa.</br> Na
   *        validao completa, os parmetros obrigatrios no precisam estar
   *        preenchidos.
   * 
   * @return O resultado da validao.
   */
  private ValidationResult validateAlgorithmConfigurator(boolean complain,
    boolean isFullValidation) {
    ValidationMode validationMode = this.configuratorView.getValidationMode();
    try {
      if (isFullValidation) {
        this.configuratorView.setValidationMode(ValidationMode.DEFAULT);
      }
      else {
        this.configuratorView
          .setValidationMode(ValidationMode.ALLOW_EMPY_VALUES);
      }
      ValidationResult validation = this.configuratorView.validate(false);
      if (validation.isWellSucceded()) {
        this.backgroundColor = READY_BACKGROUND_COLOR;
      }
      else {
        if (complain) {
          this.backgroundColor = ERROR_COLOR;
          repaint();
          this.configuratorView.validate(true);
        }
        this.backgroundColor = NOT_READY_BACKGROUND_COLOR;
      }
      return validation;
    }
    finally {
      this.configuratorView.setValidationMode(validationMode);
    }
  }

  /**
   * <p>
   * Valida os arquivos de entrada deste n.
   * </p>
   * 
   * <p>
   * Um arquivo de entrada  considerado invlido se existir uma conexo
   * mal-formada partindo dele.
   * </p>
   * 
   * @return O resultado da validao.
   */
  private ValidationResult validateInputFiles() {
    for (GraphFileDescriptor descriptor : this.inputFileTypeViews) {
      if (!descriptor.validate()) {
        return new ValidationResult(LNG.get(GraphNode.class.getName()
          + ".error_invalid_input_file_descriptor", new Object[] { descriptor
          .toString() }));
      }
      if (descriptor.getFileParameter().isEnabled()) {
        GraphLink link = descriptor.getLinkTo();
        if (link != null) {
          if (!link.isWellFormed()) {
            return new ValidationResult(LNG.get(GraphNode.class.getName()
              + ".error_invalid_link", new Object[] { link.toString() }));
          }
        }
      }
    }
    return new ValidationResult();
  }

  /**
   * <p>
   * Valida os arquivos de sada deste n.
   * </p>
   * 
   * <p>
   * O arquivo de sada  considerado invlido se existir pelo menos uma conexo
   * mal-formada chegando nele.
   * </p>
   * 
   * @return O resultado da validao.
   */
  private ValidationResult validateOutputFiles() {
    for (GraphFileDescriptor descriptor : this.outputFileTypeViews) {
      if (!descriptor.validate()) {
        return new ValidationResult(LNG.get(GraphNode.class.getName()
          + ".error_invalid_output_file_descriptor", new Object[] { descriptor
          .toString() }));
      }
      if (descriptor.getFileParameter().isEnabled()) {
        for (GraphLink link : descriptor.getLinkFromCollection()) {
          if (!link.isWellFormed()) {
            return new ValidationResult(LNG.get(GraphNode.class.getName()
              + ".error_invalid_link", new Object[] { link.toString() }));
          }
        }
      }
    }
    return new ValidationResult();
  }

  /**
   * Indica que esse n deve ser desviado ou no no fluxo. Caso positivo, no
   * entanto, ainda vai ser verificado se o desvio  possvel. Caso negativo, o
   * n deixa de ser desviado.
   * 
   * @param bypassed indica se o n deve ser desviado ou no.
   * @return verdadeiro se foi possvel fazer/remover o desvio do algoritmo ou
   *         falso caso contrrio. No h porque impedir a remoo do desvio, no
   *         momento, portanto se o parmetro bypassed for falso, o mtodo deve
   *         sempre retornar verdadeiro.
   */
  public boolean setBypassed(boolean bypassed) {
    validateBypass();
    if (!bypassed || canBypass) {
      this.bypassed = bypassed;
      validate(false, true);
      return true;
    }
    return false;
  }

  /**
   * Retorna verdadeiro se o n estiver marcado para ser desviado ou falso, caso
   * contrrio. O n pode estar marcado para ser desviado, mas no estar sendo
   * realmente desviado na execuo no momento, caso no esteja cumprindo as
   * condies necessrias.
   * 
   * @return verdadeiro se o n est marcado para ser desviado ou falso, caso
   *         contrrio.
   */
  public boolean isBypassed() {
    return bypassed;
  }

  /**
   * Retorna verdadeiro se o n estiver cumprindo as condies necessrias para
   * que seja desviado ou falso, caso contrrio. O fato de retornar verdadeiro
   * no necessariamente indica que o n est sendo desviado no momento. O n s
   * ser desviado caso esteja marcado para o desvio (atributo bypassed
   * verdadeiro) e esteja cumprindo as condies (atributo canBypass
   * verdadeiro).
   * 
   * @return verdadeiro se o n estiver em condio de ser desviado ou falso
   *         caso contrrio.
   */
  public boolean canBeBypassed() {
    return canBypass;
  }

  /**
   * Valida o n em relao s condies necessrias para que ele possa ser
   * desviado. Se o n cumprir todas as condies, o desvio pode ser feito (a
   * varivel canBypass recebe verdadeiro).
   * 
   * Para que um n possa ser desviado as entradas do seu algoritmo tm que ser
   * compatveis com suas sadas. @see
   * {@link #isEquivalent(Collection, Collection)}
   * 
   * O fato de cada n do fluxo poder ser desviado individualmente no indica
   * que o fluxo vai poder ser executado com os desvios. Existem algumas
   * condies de desvios que no permitem a execuo do fluxo. @see
   * FlowAlgorithmConfigurator
   */
  protected void validateBypass() {
    Collection<GraphLink> linkFromCollection = this.getLinkFromCollection();
    Collection<GraphLink> linkToCollection = this.getLinkToCollection();
    if (inputFileTypeViews.size() == outputFileTypeViews.size()) {
      if (linkFromCollection.size() > 0 && linkToCollection.size() > 0) {
        if (isEquivalent(linkFromCollection, linkToCollection)) {
          canBypass = true;
          return;
        }
      }
    }
    canBypass = false;
  }

  /**
   * Determina se as ligaes de entrada e sada de um n so equivalentes. O
   * nmero de entradas precisa ser igual ao de sadas e o nmero de entradas de
   * um determinado tipo tambm precisa ser o mesmo das sadas deste tipo.
   * 
   * @param linkFromCollection ligaes de entrada do n
   * @param linkToCollection ligaes de sada do n
   * @return verdadeiro se as colees de ligaes de entrada e sada so
   *         verdadeiras ou falso caso contrrio.
   */
  protected boolean isEquivalent(Collection<GraphLink> linkFromCollection,
    Collection<GraphLink> linkToCollection) {
    Map<String, Integer> linkToTypeCount = getLinkToTypeCount(linkToCollection);
    Map<String, Integer> linkFromTypeCount =
      getLinkFromTypeCount(linkFromCollection);
    if (linkToTypeCount.keySet().size() == linkFromTypeCount.keySet().size()) {
      for (String type : linkToTypeCount.keySet()) {
        if (!(linkToTypeCount.get(type).equals(linkFromTypeCount.get(type)))) {
          return false;
        }
      }
      return true;
    }
    return false;
  }

  /**
   * Agrupa as ligaes de sada de um n pelos seus tipos e retorna o nmero de
   * ligaes de cada tipo.
   * 
   * Se mais de uma ligao estiver conectada a uma mesma sada ("split"), as
   * ligaes contaro como uma nica, ou seja, s ser contada uma ligao por
   * sada.
   * 
   * As ligaes sem tipo especificado sero agrupadas sob a String
   * "NOT_SPECIFIED"
   * 
   * @param linkFromCollection ligaes de sada do n
   * @return mapa com o nmero de ligaes de cada tipo
   */
  protected Map<String, Integer> getLinkFromTypeCount(
    Collection<GraphLink> linkFromCollection) {
    Map<String, Integer> types = new HashMap<String, Integer>();
    List<GraphFileDescriptor> outputs = new ArrayList<GraphFileDescriptor>();
    for (GraphLink linkFrom : linkFromCollection) {
      GraphFileDescriptor inputFileDescriptor =
        linkFrom.getInputFileDescriptor();
      if (inputFileDescriptor != null) {
        String outputFileType = "NOT_SPECIFIED";
        FileParameter fileParameter = inputFileDescriptor.getFileParameter();
        if (fileParameter != null && fileParameter.getFileType() != null) {
          outputFileType = fileParameter.getFileType();
        }

        GraphFileDescriptor outputFileDescriptor =
          linkFrom.getOutputFileDescriptor();
        if (outputFileDescriptor != null) {
          //S conta um link por output
          if (!outputs.contains(outputFileDescriptor)) {
            outputs.add(outputFileDescriptor);
            if (types.get(outputFileType) == null) {
              types.put(outputFileType, 1);
            }
            else {
              Integer typeCount = types.get(outputFileType);
              types.put(outputFileType, typeCount++);
            }
          }
        }
      }
    }
    return types;
  }

  /**
   * Agrupa as ligaes de entrada de um n pelos seus tipos e retorna o nmero
   * de ligaes de cada tipo.
   * 
   * As ligaes sem tipo especificado sero agrupadas sob a String
   * "NOT_SPECIFIED"
   * 
   * @param linkToCollection ligaes de entrada do n
   * @return mapa com o nmero de ligaes de cada tipo
   */
  protected Map<String, Integer> getLinkToTypeCount(
    Collection<GraphLink> linkToCollection) {
    Map<String, Integer> types = new HashMap<String, Integer>();
    for (GraphLink linkTo : linkToCollection) {
      GraphFileDescriptor outputFileDescriptor =
        linkTo.getOutputFileDescriptor();
      if (outputFileDescriptor != null) {
        FileParameter fileParameter = outputFileDescriptor.getFileParameter();
        String inpuFileType = "NOT_SPECIFIED";
        if (fileParameter != null && fileParameter.getFileType() != null) {
          inpuFileType = fileParameter.getFileType();
        }
        if (types.get(inpuFileType) == null) {
          types.put(inpuFileType, 1);
        }
        else {
          Integer typeCount = types.get(inpuFileType);
          types.put(inpuFileType, typeCount++);
        }
      }
    }
    return types;
  }

  /**
   * Adiciona a decorao especificada a este n.
   * 
   * @param type o tipo de decorao a ser adicionada.
   */
  public void addDecoration(DecorationType type) {
    GraphNodeDecoration newDecoration =
      GraphNodeDecoration.createDecoration(type);
    replaceDecoration(newDecoration);
  }

  /**
   * Remove a decorao utilizada atualmente por este n.
   */
  public void removeDecoration() {
    replaceDecoration(new GraphNodeDecoration());
  }

  /**
   * Substitui o cone da decorao atual por outro, da nova decorao escolhida
   * no <i>vix</i>.
   * 
   * @param newDecoration cone da nova decorao.
   */
  private void replaceDecoration(GraphNodeDecoration newDecoration) {
    if (decoration != null) {
      ImageVO oldDecoration = this.decoration.getImage();
      oldDecoration.changeVS(this, null);
      changeVO(oldDecoration, null);
    }
    this.decoration = newDecoration;
    decoration.getImage().changeVS(null, this);
    changeVO(null, this.decoration.getImage());
    updateDecorationLocation();
    repaint();
  }

}
