package csbase.algorithms.flows.graph.utils;

import java.awt.geom.Point2D;
import java.text.MessageFormat;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.graph.GraphFileDescriptor;
import csbase.algorithms.flows.graph.GraphLink;
import csbase.algorithms.flows.graph.GraphNode;
import csbase.algorithms.flows.graph.ParametersTransferable;
import csbase.client.algorithms.AlgorithmConfiguratorView;
import csbase.client.util.StandardErrorDialogs;
import csbase.exception.ParseException;
import csbase.exception.algorithms.ParameterNotFoundException;
import csbase.logic.algorithms.AlgorithmConfigurator;

/**
 * Classe com mtodos utilitrios para auxiliar as aes de copiar/colar
 * parmetros e ligaes entre os ns dos grafos.
 * 
 * @author isabella
 */
public class CopyAndPasteOperation {

  /**
   * N de origem (o valor dos parmetros sero copiados dele). Pode ser nulo;
   * nesse caso s teremos o valor dos parmetros em
   * {@link CopyAndPasteOperation#parameters}.
   */
  private GraphNode fromNode;

  /**
   * N de destino (o valor dos parmetros sero colados nele).
   */
  private GraphNode toNode;

  /**
   * Os parmetros a serem copiados.
   */
  private ParametersTransferable parameters;

  /**
   * O estado da operao.
   */
  private OperationStatus status;

  /**
   * Um 'backup' dos valores dos parmetros do n de destino.
   */
  private Map<Object, Object> memento;

  /**
   * Construtor.
   * 
   * @param fromNode N de origem. (No aceita <code>null</code>)
   * @param toNode N de destino. (No aceita <code>null</code>)
   */
  public CopyAndPasteOperation(GraphNode fromNode, GraphNode toNode) {
    this(new ParametersTransferable(fromNode), fromNode, toNode);
  }

  /**
   * Construtor
   * 
   * @param parameters Os parmetros a serem copiados. (No aceita
   *        <code>null</code>)
   * @param toNode N de destino. (No aceita <code>null</code>)
   */
  public CopyAndPasteOperation(ParametersTransferable parameters,
    GraphNode toNode) {
    this(parameters, null, toNode);
  }

  /**
   * Construtor privado.
   * 
   * O parmetro parameters e o parmetro fromNode no podem ser nulos
   * simultaneamente.
   * 
   * @param parameters Os parmetros a serem copiados.
   * @param fromNode N de origem.
   * @param toNode N de destino. (No aceita <code>null</code>)
   */
  protected CopyAndPasteOperation(ParametersTransferable parameters,
    GraphNode fromNode, GraphNode toNode) {
    if (toNode == null) {
      throw new IllegalArgumentException("O parmetro toNode est nulo!");
    }
    if (fromNode == null && parameters == null) {
      throw new IllegalArgumentException(
        "Os parmetros fromNode e parameters esto nulos!");
    }
    this.fromNode = fromNode;
    this.parameters = parameters;
    this.toNode = toNode;
    this.status = new OperationStatus();
    this.memento = getMemento();
  }

  /**
   * "Cola" o valor da lista de parmetros em um determinado n. Caso todos os
   * parmetros existam com as mesmas propriedades no n, a operao pode ser
   * feita automaticamente. Caso alguns parmetros no existam, ou tenham
   * propriedades diferentes no n de destino, o usurio ter que confirmar a
   * operao.
   * 
   * @return Verdadeiro se os parmetros podem ser "colados" automaticamente ou
   *         o usurio confirmar a operao, ou Falso caso o usurio cancele a
   *         operao.
   */
  public boolean pasteParameters() {
    Map<String, String> parameterValuesByName = getCompatibleParameterValues();

    boolean confirmPaste = true;
    try {
      // Tenta efetivamente copiar os parmetros para o destino.
      toNode.setParameterValuesByName(parameterValuesByName);
    }
    catch (ParseException e) {
      rollbackPaste();
      confirmPaste = false;
    }
    return confirmPaste;
  }

  /**
   * Retorna o estado da operao, inclusive indicando se esta pode prosseguir.
   * Se necessrio, pede a confirmao do usurio.
   * 
   * @return o estado da operao.
   */
  public OperationStatus getConfirmation() {
    makeOperationStatus();
    boolean result = showConfirmationDialog();
    status.setConfirmed(result);

    return status;
  }

  /**
   * Obtm uma cpia de segurana dos valores dos parmetros de um n.
   * 
   * @return O mapa com o nome do parmetros do n e seus respectivos valores.
   */
  protected Map<Object, Object> getMemento() {
    AlgorithmConfiguratorView algorithmConfiguratorView =
      toNode.getAlgorithmConfiguratorView();
    AlgorithmConfigurator configurator =
      algorithmConfiguratorView.getConfigurator();
    return configurator.exportValues();
  }

  /**
   * Retorna os valores dos parmetros "compatveis" (que podem ser copiados e
   * colados) entre os ns origem e destino da operao.
   * 
   * @return Um mapa com o nome e o valor dos parmetros "compatveis".
   */
  protected Map<String, String> getCompatibleParameterValues() {
    Set<String> parameterNames = parameters.getParameterNames();
    Map<String, String> parameterValuesByName = new HashMap<String, String>();
    for (String parameterName : parameterNames) {
      // Confere se parmetro existe no destino
      if (parameterExists(parameterName)) {
        // Confere se o tipo do parmetro  o mesmo no destino
        String parameterType = parameters.getParameterType(parameterName);
        if (hasSameParameterType(parameterName, parameterType)) {
          // No  necessrio conferir se o label  o mesmo, 
          // o campo pode ser considerado o mesmo
          String parameterValue = parameters.getParameterValue(parameterName);
          parameterValuesByName.put(parameterName, parameterValue);
        }
      }
    }
    return parameterValuesByName;
  }

  /**
   * Compara uma determinada lista de parmetros com os parmetros do n
   * especificado para determinar se so equivalentes (mesma quantidade de
   * parmetros, mesmas propriedades, etc.). Essa comparao retorna num Mapa,
   * que contm o nome e o valor dos parmetros que so equivalentes. No caso de
   * parmetros que apresentam diferenas entre a lista e o n, so adicionados
   * um registro na lista de avisos ou na lista de erros (dependendo do tipo da
   * diferena identificada), que so passadas ao mtodo. Caso os parmetros
   * sejam equivalentes, essas listas no sofrem modificaes.
   * 
   */
  protected void makeOperationStatus() {
    OperationStatus status = new OperationStatus();

    if (!toNode.isWellFormed()) {
      String errorMessage =
        MessageFormat.format(LNG.get(CopyAndPasteOperation.class.getName()
          + ".error_wrong_labels"), toNode.getAlgorithmName());
      status.addError(errorMessage);
    }

    Set<String> parameterNames = parameters.getParameterNames();

    for (String parameterName : parameterNames) {
      // Confere se parmetro existe no destino
      if (parameterExists(parameterName)) {
        // Confere se o tipo do parmetro  o mesmo no destino
        String parameterType = parameters.getParameterType(parameterName);
        if (hasSameParameterType(parameterName, parameterType)) {
          // Confere se o label do parmetro  o mesmo no destino
          String parameterLabel = parameters.getParameterLabel(parameterName);
          if (!hasSameParameterLabel(parameterName, parameterLabel)) {
            String newParameterLabel = "";
            try {
              newParameterLabel = toNode.getParameterLabel(parameterName);
            }
            // O parmetro existe no n aqui, a exception no deve acontecer.
            catch (ParameterNotFoundException e) {
            }
            // S um warning  necessrio, o campo pode ser considerado o mesmo
            String warningMessage =
              MessageFormat.format(LNG.get(CopyAndPasteOperation.class
                .getName()
                + ".error_wrong_labels"), parameterName, parameterLabel,
                newParameterLabel);
            status.addWarning(warningMessage);
          }
        }
        else {
          String errorMessage =
            MessageFormat.format(LNG.get(CopyAndPasteOperation.class.getName()
              + ".error_wrong_types"), parameterName);
          status.addError(errorMessage);
        }
      }
      else {
        String warningMessage =
          MessageFormat.format(LNG.get(CopyAndPasteOperation.class.getName()
            + ".warning_no_parameter"), parameterName);
        status.addWarning(warningMessage);
      }
    }

    if (fromNode != null) {
      if (!canCopyLinks()) {
        String errorMessage =
          LNG.get(CopyAndPasteOperation.class.getName() + ".error_links");
        status.addError(errorMessage);
      }
    }

    this.status = status;
  }

  /**
   * Compara o label de um parmetro especfico do n, com um determinado label.
   * Caso sejam diferentes,  adicionado um registro na lista de avisos.
   * 
   * @param parameterName Nome do parmetro que ser comparado
   * @param parameterLabel Label que ser comparado ao do parmetro do n
   * @return Verdadeiro se os labels so iguais ou Falso caso contrrio
   */
  protected boolean hasSameParameterLabel(String parameterName,
    String parameterLabel) {
    String nodeParameterLabel;
    try {
      nodeParameterLabel = toNode.getParameterLabel(parameterName);
    }
    catch (ParameterNotFoundException e) {
      return false;
    }
    if (!nodeParameterLabel.equals(parameterLabel)) {
      return false;
    }
    return true;
  }

  /**
   * Compara o tipo de um parmetro especfico do n, com um determinado tipo.
   * Caso sejam diferentes,  adicionado um registro na lista de erros.
   * 
   * @param parameterName Nome do parmetro que ser comparado
   * @param parameterType Tipo que ser comparado ao do parmetro do n
   *        diferentes
   * @return Verdadeiro se os tipos so iguais ou Falso caso contrrio
   */
  protected boolean hasSameParameterType(String parameterName,
    String parameterType) {
    String nodeParameterType;
    try {
      nodeParameterType = toNode.getParameterType(parameterName);
    }
    catch (ParameterNotFoundException e) {
      return false;
    }
    if (!nodeParameterType.equals(parameterType)) {
      return false;
    }
    return true;
  }

  /**
   * Determina se um determinado parmetro existe no n. Caso negativo, 
   * adicionado um registro na lista de avisos.
   * 
   * @param parameterName Nome do parmetro que ser testado
   * @return Verdadeiro se o parmetro existir no n ou Falso caso contrrio
   */
  protected boolean parameterExists(String parameterName) {
    if (!toNode.getParameterNames().contains(parameterName)) {
      return false;
    }
    return true;
  }

  /**
   * Mostra uma janela com avisos e erros que so apresentados ao usurio e pede
   * uma confirmao para continuar a operao que estava sendo feita.
   * 
   * @return verdadeiro se o usurio confirmar ou falso, caso contrrio.
   */
  protected boolean showConfirmationDialog() {
    List<String> errors = status.getErrors();
    List<String> warnings = status.getWarnings();
    if (!warnings.isEmpty() || !errors.isEmpty()) {
      String title =
        (LNG.get(CopyAndPasteOperation.class.getName() + ".title"));
      StringBuilder content = new StringBuilder();
      content.append("<html>");
      content.append("<b>");
      content
        .append(LNG.get(CopyAndPasteOperation.class.getName() + ".header"));
      content.append("</b>");
      if (!errors.isEmpty()) {
        content.append("<br><br>");
        content.append("<i>");
        content.append(LNG.get(CopyAndPasteOperation.class.getName()
          + ".errors"));
        content.append("</i>");

        content.append("<ul>");
        for (String error : errors) {
          content.append("<li>" + error + "</li>");
        }
        content.append("</ul>");
      }
      if (!warnings.isEmpty()) {
        if (errors.isEmpty()) {
          content.append("<br><br>");
        }
        content.append("<i>");
        content.append(LNG.get(CopyAndPasteOperation.class.getName()
          + ".warnings"));
        content.append("</i>");

        content.append("<ul>");
        for (String warning : warnings) {
          content.append("<li>" + warning + "</li>");
        }
        content.append("</ul>");
      }
      content.append("</html>");
      int result =
        StandardDialogs.showYesNoDialog(toNode.getGraph().getParentWindow(),
          title, content);
      if (result == StandardDialogs.NO_OPTION) {
        return false;
      }
    }
    return true;
  }

  /**
   * Restaura o estado dos parmetros de um n, aps o cancelamento de uma
   * operao de "colar".
   */
  protected void rollbackPaste() {
    AlgorithmConfigurator configurator =
      toNode.getAlgorithmConfiguratorView().getConfigurator();
    try {
      configurator.importValues(memento);
    }
    catch (ParseException e) {
      StandardErrorDialogs.showErrorDialog(toNode.getGraph().getParentWindow(),
        e);
    }
  }

  /**
   * Copia todas as ligaes de um n e "cola" no outro. Os links originais so
   * removidos do grafo se a cpia for bem sucedida.
   * 
   * @return verdadeiro se todas as ligaes forem copiadas com sucesso ou
   *         falso, caso pelo menos uma no consiga ser copiada.
   */
  public boolean copyLinks() {
    boolean result = copyFromLinks();
    return result && copyToLinks();
  }

  /**
   * Copia todas as ligaes que <b>saem</b> de um n e "cola" no outro. Os
   * links originais so removidos do grafo se a cpia for bem sucedida.
   * 
   * @return verdadeiro se todas as ligaes forem copiadas com sucesso ou
   *         falso, caso pelo menos uma no consiga ser copiada.
   */
  protected boolean copyFromLinks() {
    if (fromNode != null) {
      boolean result = true;
      Collection<GraphLink> fromLinks = fromNode.getLinkFromCollection();
      for (GraphLink fromLink : fromLinks) {
        GraphNode endNode = fromLink.getEndNode();
        if (canCopyLink(fromLink, toNode, endNode)) {
          result = result && copyLink(fromLink, toNode, endNode);
        }
      }
      return result;
    }
    return false;
  }

  /**
   * Copia todas as ligaes que <b>chegam</b> em um n e "cola" no outro. Os
   * links originais so removidos do grafo se a cpia for bem sucedida.
   * 
   * @return verdadeiro se todas as ligaes forem copiadas com sucesso ou
   *         falso, caso pelo menos uma no consiga ser copiada.
   */
  protected boolean copyToLinks() {
    if (fromNode != null) {
      boolean result = true;
      Collection<GraphLink> toLinks = fromNode.getLinkToCollection();
      for (GraphLink toLink : toLinks) {
        GraphNode startNode = toLink.getStartNode();
        if (canCopyLink(toLink, startNode, toNode)) {
          result = result && copyLink(toLink, startNode, toNode);
        }
      }
      return result;
    }
    return false;
  }

  /**
   * Copia o link especificado e o liga nos ns. O link original  removido do
   * grafo se a cpia for bem sucedida.
   * 
   * @param link Ligao a ser copiada
   * @param startNode N que representa a sada da ligao
   * @param endNode N que representa a entrada da ligao
   * @return verdadeiro se a ligao for copiada com sucesso ou falso, caso
   *         contrrio.
   */
  protected boolean copyLink(GraphLink link, GraphNode startNode,
    GraphNode endNode) {
    try {
      String outputParamName =
        link.getOutputFileDescriptor().getParameterName();
      String inputParamName = link.getInputFileDescriptor().getParameterName();
      GraphFileDescriptor inputFileDescriptor =
        endNode.getInputFileDescriptor(inputParamName);
      GraphFileDescriptor outputFileDescriptor =
        startNode.getOutputFileDescriptor(outputParamName);
      if (inputFileDescriptor != null && outputFileDescriptor != null) {
        List<Point2D> pointList = link.getIntermediatePointList();
        link.deattach();
        Graph graph = startNode.getGraph();
        graph.createGraphLink(inputFileDescriptor, outputFileDescriptor,
          pointList);
        return true;
      }
      return false;
    }
    catch (Exception e) {
      return false;
    }
  }

  /**
   * Indica se todos as ligaes poderiam ser copiadas automaticamente de um n
   * para outro.
   * 
   * @return verdadeiro se os links puderem ser copiados automaticamente
   *         ou falso, caso contrrio.
   */
  public boolean canCopyLinks() {
    boolean result = canCopyFromLinks();
    return result && canCopyToLinks();
  }

  /**
   * Indica se todos as ligaes que <b>chegam</b> em um n poderiam ser
   * copiadas automaticamente para outro.
   * 
   * @return verdadeiro se os links puderem ser copiados automaticamente
   *         ou falso, caso contrrio.
   */
  protected boolean canCopyToLinks() {
    if (fromNode != null) {
      boolean result = true;
      Collection<GraphLink> toLinks = fromNode.getLinkToCollection();
      for (GraphLink toLink : toLinks) {
        GraphNode startNode = toLink.getStartNode();
        result = result && canCopyLink(toLink, startNode, toNode);
      }
      return result;
    }
    return false;
  }

  /**
   * Indica se todas as ligaes que <b>saem</b> de um n poderiam ser copiadas
   * automaticamente para outro.
   * 
   * @return verdadeiro se os links puderem ser copiados automaticamente
   *         ou falso, caso contrrio.
   */
  protected boolean canCopyFromLinks() {
    if (fromNode != null) {
      boolean result = true;
      Collection<GraphLink> fromLinks = fromNode.getLinkFromCollection();
      for (GraphLink fromLink : fromLinks) {
        GraphNode endNode = fromLink.getEndNode();
        result = result && canCopyLink(fromLink, toNode, endNode);
      }
      return result;
    }
    return false;
  }

  /**
   * Indica se uma ligao de um n poderia ser copiada automaticamente para
   * outro.
   * 
   * @param link Ligao a ser copiada.
   * @param startNode N origem.
   * @param endNode N destino.
   * @return verdadeiro se a ligao puder ser copiada automaticamente ou falso,
   *         caso contrrio.
   */
  protected boolean canCopyLink(GraphLink link, GraphNode startNode,
    GraphNode endNode) {
    String outputParamName = link.getOutputFileDescriptor().getParameterName();
    String inputParamName = link.getInputFileDescriptor().getParameterName();
    GraphFileDescriptor inputFileDescriptor =
      endNode.getInputFileDescriptor(inputParamName);
    GraphFileDescriptor outputFileDescriptor =
      startNode.getOutputFileDescriptor(outputParamName);
    if (inputFileDescriptor != null && outputFileDescriptor != null) {
      return true;
    }
    return false;
  }

}
