package csbase.algorithms.flows.graph.utils;

import java.awt.Dimension;
import java.awt.Window;
import java.awt.geom.Dimension2D;
import java.awt.geom.Point2D;

import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.graph.GraphNode;
import csbase.client.algorithms.AlgorithmConfiguratorFactory;
import csbase.client.algorithms.AlgorithmConfiguratorView;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;

/**
 * Operao de atualizao de verso do algoritmo de um n de um fluxo.
 * 
 * @author isabella
 */
public class GraphNodeUpdateOperation {

  /**
   * O n a ser atualizado.
   */
  private GraphNode node;

  /**
   * Construtor
   * 
   * @param node O n a ser atualizado.
   */
  public GraphNodeUpdateOperation(GraphNode node) {
    if (node == null) {
      throw new IllegalArgumentException("O parmetro node est nulo!");
    }
    this.node = node;
  }

  /**
   * Determina se existe uma verso mais nova para o algoritmo que um
   * determinado n do grafo representa.
   * 
   * @param node N que dever ser testado
   * @return Verdadeiro se o n representa um algoritmo desatualizado
   */
  public static boolean hasNewerVersion(GraphNode node) {
    AlgorithmVersionId nodeVersionId = node.getAlgorithmVersionId();
    AlgorithmInfo algorithm = node.getAlgorithm();
    if (algorithm != null) {
      AlgorithmVersionId latestVersionId = algorithm.getLastVersion().getId();
      if (!nodeVersionId.equals(latestVersionId)) {
        return true;
      }
    }
    return false;
  }

  /**
   * Determina se existe uma verso mais nova para o algoritmo que o n
   * representa.
   * 
   * @return Verdadeiro se o n representa um algoritmo desatualizado.
   */
  public boolean hasNewerVersion() {
    return hasNewerVersion(node);
  }

  /**
   * Atualiza o n que representa um algoritmo desatualizado (que
   * tem pelo menos uma verso mais nova disponvel). A janela de configurao
   * de parmetros pode ser mostrada aps a atualizao.
   * 
   * @param askForParameters Se verdadeiro, a janela de configurao de
   *        parmetros ser aberta aps a atualizao no caso de avisos ou
   *        erros.
   * @return verdadeiro se a atualizao do n foi feita com sucesso
   *         ou falso, caso contrrio.
   */
  public boolean update(boolean askForParameters) {
    return updateNode(askForParameters);
  }

  /**
   * Atualiza o n que representa um algoritmo desatualizado (que
   * tem pelo menos uma verso mais nova disponvel). A janela de configurao
   * de parmetros do n  mostrada aps a atualizao.
   * 
   * @return verdadeiro se a atualizao do n foi feita com sucesso
   *         ou falso, caso contrrio.
   */
  public boolean update() {
    return update(true);
  }

  /**
   * Atualiza um determinado n que representa um algoritmo desatualizado (que
   * tem pelo menos uma verso mais nova disponvel). A janela de configurao
   * de parmetros do n  mostrada aps a atualizao no caso de avisos ou
   * erros se askForParameters for verdadeiro.
   * 
   * @param askForParameters Se verdadeiro, a janela de configurao de
   *        parmetros ser aberta aps a atualizao no caso de avisos ou erros
   * @return verdadeiro se a atualizao do n foi feita com sucesso
   *         ou falso, caso contrrio.
   */
  protected boolean updateNode(boolean askForParameters) {
    boolean complain = false;
    boolean result = true;
    if (hasNewerVersion()) {
      AlgorithmInfo algorithmInfo = node.getAlgorithm();
      AlgorithmVersionInfo algorithmVersionInfo =
        algorithmInfo.getLastVersion();
      Graph graph = node.getGraph();
      Window window = graph.getParentWindow();
      AlgorithmConfiguratorView algorithmConfiguratorView =
        AlgorithmConfiguratorFactory.getInstance().createConfigurationView(
          window, algorithmVersionInfo);
      if (algorithmConfiguratorView == null) {
        return true;
      }
      Point2D nodeLocation =
        new Point2D.Double(Math.max(0, node.getX()), Math.max(0, node.getY()));
      Dimension2D dimension = new Dimension();
      dimension.setSize(node.getWidth(), node.getHeight());
      GraphNode newNode =
        graph.createGraphNode(algorithmConfiguratorView, nodeLocation,
          dimension, node.isBypassed());
      CopyAndPasteOperation operation =
        new CopyAndPasteOperation(node, newNode);
      OperationStatus operationStatus = operation.getConfirmation();
      if (operationStatus.isConfirmed()) {
        //Se tudo correr bem, copia as ligaes do n e remove o n antigo
        result = result && operation.pasteParameters();
        result = result && operation.copyLinks();
        if (!result) {
          complain = true;
        }
        else {
          node.deattach();
          if (askForParameters
            && !(operationStatus.getWarnings().isEmpty() && operationStatus
              .getErrors().isEmpty())) {
            newNode.askForParameterValues();
          }
        }
      }
      else {
        result = false;
        complain = false;
      }

      if (!result) {
        //rollback
        newNode.deattach();
      }
    }
    return result || !complain;
  }
}
