package csbase.algorithms.flows.zoom;

import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;

import csbase.algorithms.flows.actions.Action;
import csbase.client.applications.ApplicationImages;

/**
 * <p>
 * Controle de zoom.
 * </p>
 * 
 * <p>
 * Painel com os widgets que controlam o zoom.
 * </p>
 * 
 * @author lmoreira
 */
public final class ZoomControl extends JToolBar {
  /** Modelo de zoom. */
  private ZoomModel model;

  /** Ao de zoom in. */
  private ZoomInAction zoomInAction;

  /** Ao de zoom out. */
  private ZoomOutAction zoomOutAction;

  /**
   * Cria um controle de zoom para o modelo de zoom dado.
   * 
   * @param model O modelo de zoom.
   */
  public ZoomControl(ZoomModel model) {
    setLayout(new FlowLayout(FlowLayout.RIGHT));
    setFloatable(false);
    setModel(model);
    buildZoomInAction();
    buildZoomOutAction();
    buildSlider();
    buildSpinner();
  }

  /**
   * Obtm a ao de zoom in.
   * 
   * @return A ao de zoom in.
   */
  public Action getZoomInAction() {
    return zoomInAction;
  }

  /**
   * Obtm a ao de zoom out.
   * 
   * @return A ao de zoom out.
   */
  public Action getZoomOutAction() {
    return zoomOutAction;
  }

  /**
   * Constri o slider.
   */
  private void buildSlider() {
    ZoomSliderModel sliderModel = new ZoomSliderModel(model);
    JSlider slider = new JSlider(sliderModel);
    slider.setMajorTickSpacing(sliderModel.getMajorTicks());
    slider.setMinorTickSpacing(sliderModel.getMinorTicks());
    slider.setOrientation(SwingConstants.HORIZONTAL);
    slider.setPaintTicks(true);
    add(slider);
  }

  /**
   * Constri o spinner.
   */
  private void buildSpinner() {
    JPanel panel = new JPanel();
    panel.add(new JLabel("%"));
    panel.add(new JSpinner(new ZoomSpinnerModel(model)));
    add(panel);
  }

  /**
   * Constri a ao zoom in e os widgets que a utilizam.
   */
  private void buildZoomInAction() {
    zoomInAction = new ZoomInAction();
    JButton zoomInButton = new JButton(zoomInAction);
    zoomInButton.setText("");
    zoomInButton.setToolTipText(zoomInAction.getToolTipText());
    add(zoomInButton);
  }

  /**
   * Constri a ao zoom out e os widgets que a utilizam.
   */
  private void buildZoomOutAction() {
    zoomOutAction = new ZoomOutAction();
    JButton zoomOutButton = new JButton(zoomOutAction);
    zoomOutButton.setText("");
    zoomOutButton.setToolTipText(zoomOutAction.getToolTipText());
    add(zoomOutButton);
  }

  /**
   * Atribui o modelo de zoom a este controle de zoom.
   * 
   * @param zoomModel O modelo de zoom.
   */
  private void setModel(ZoomModel zoomModel) {
    model = zoomModel;
    model.addListener(new ZoomControlListener());
  }

  /**
   * Zoom in.
   */
  private void zoomIn() {
    model.setValue(model.getNextBlockValue());
  }

  /**
   * Zoom out.
   */
  private void zoomOut() {
    model.setValue(model.getPreviousBlockValue());
  }

  /**
   * O listener de zoom que comunica ao controle de zoom sobre mudanas de zoom.
   */
  private final class ZoomControlListener implements ZoomListener {
    public void wasChanged(ZoomModel zoomModel) {
      zoomInAction.setEnabled(zoomModel.hasNextBlockValue());
      zoomOutAction.setEnabled(zoomModel.hasPreviousBlockValue());
    }
  }

  /**
   * Ao zoom in.
   */
  private final class ZoomInAction extends Action {
    /**
     * Cria a ao.
     */
    ZoomInAction() {
      super("csbase.algorithms.flows.zoom.ZoomControl.zoomIn",
        ApplicationImages.ICON_ZOOMIN_16, KeyStroke.getKeyStroke(
          KeyEvent.VK_ADD, 0));
    }

    public void actionPerformed(ActionEvent e) {
      zoomIn();
    }
  }

  /**
   * Ao zoom out.
   */
  private final class ZoomOutAction extends Action {
    /**
     * Cria a ao.
     */
    ZoomOutAction() {
      super("csbase.algorithms.flows.zoom.ZoomControl.zoomOut",
        ApplicationImages.ICON_ZOOMOUT_16, KeyStroke.getKeyStroke(
          KeyEvent.VK_SUBTRACT, 0));
    }

    public void actionPerformed(ActionEvent e) {
      zoomOut();
    }
  }
}
