/*
 * Detalhes da ltima alterao:
 * 
 * $Author: karla $
 * $Date: 2010-05-19 15:33:42 -0300 (Wed, 19 May 2010) $
 * $Revision: 105780 $
 */
package csbase.client;

import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.rmi.RemoteException;
import java.text.MessageFormat;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.swing.JComponent;
import javax.swing.JFrame;

import tecgraf.javautils.FileUtils;
import tecgraf.javautils.FormatUtils;
import tecgraf.javautils.LNG;
import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.configurationmanager.ConfigurationManagerException;
import tecgraf.javautils.configurationmanager.MissingPropertyException;
import tecgraf.javautils.gui.GUIUtils;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.algorithms.parameters.DefaultParameterViewFactory;
import csbase.client.algorithms.parameters.ParameterViewFactory;
import csbase.client.desktop.LocalTask;
import csbase.client.desktop.RemoteTask;
import csbase.client.externalresources.ExternalResources;
import csbase.client.kernel.ClientException;
import csbase.client.login.AbstractLoginUI;
import csbase.client.login.InitialContext;
import csbase.client.login.LoginInterface;
import csbase.client.login.LoginUI;
import csbase.client.login.PreLogin;
import csbase.client.login.UserPasswordLogin;
import csbase.client.openbus.OpenBusLogin;
import csbase.client.remote.ClientRemoteMonitor;
import csbase.client.util.StandardErrorDialogs;
import csbase.exception.CSBaseException;
import csbase.exception.ConfigurationException;
import csbase.logic.Permission;
import csbase.logic.ServerURI;
import csbase.logic.User;
import csbase.remote.ClientRemoteLocator;

/**
 * Representa um cliente.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class Client {

  /**
   * O nome do parmetro que define a URL do servidor. Essa URL  do tipo
   * nome:porta.
   */
  public static final String SERVER_URL_PARAMETER = "server_url";

  /**
   * O nome do parmetro que define o endereo da mquina onde o cliente est
   * rodando.
   */
  public static final String CLIENT_ADDRESS_PARAMETER = "client_address";

  /** 
   * O endereo padro para a mquina onde o cliente est rodando. 
   */
  private static final String DEFAULT_HOST_ADDRESS = "127.0.0.1";

  /**
   * O nome do parmetro que define o token de indentificao de um usurio que
   * realizou pr-login antes de carregar o cliente do sistema.
   */
  protected static final String PRE_LOGIN_TOKEN_PARAMETER = "token";

  /**
   * O nome do parmetro que define o locale que o usurio deseja carregar o
   * sistema.
   */
  protected static final String LOCALE_PARAMETER = "locale";

  /**
   * O nome do parmetro que representa o identificador da credencial de um
   * usurio do OpenBus.
   * 
   * @deprecated
   */
  @Deprecated
  public static final String CREDENTIAL_PARAMETER = "credential";

  /**
   * O nome do parmetro que representa o identificador da credencial de um
   * usurio do OpenBus.
   */
  public static final String CREDENTIAL_IDENTIFIER_PARAMETER =
    "credential_identifier";
  /**
   * O nome do parmetro que representa o dono da credencial de um usurio do
   * OpenBus.
   */
  public static final String CREDENTIAL_OWNER_PARAMETER = "credential_owner";
  /**
   * O nome do parmetro que representa o delegado da credencial de um usurio
   * do OpenBus.
   */
  public static final String CREDENTIAL_DELEGATE_PARAMETER =
    "credential_delegate";

  /**
   * O nome do parmetro que representa o IOR de um Servio de Dados do OpenBus.
   */
  public static final String DATA_SERVICE_IOR_PARAMETER = "data_service_ior";

  /**
   * O nome da propriedade que indica qual a classe do localizador de servios
   * remotos.
   */
  private static final String REMOTE_LOCATOR_CLASS_PROPERTY =
    "remote.locator.class";

  /**
   * O nome da propriedade que indica qual a classe do monitorador do servidor.
   */
  private static final String REMOTE_MONITOR_CLASS_PROPERTY =
    "remote.monitor.class";

  /**
   * O nome da propriedade que define os arquivos de idiomas que pertencem ao
   * CSBase.
   */
  private static final String BASIC_LANGUAGE_FILE_PROPERTY =
    "basic.language.file";

  /**
   * O nome da propriedade que define os arquivos de idiomas adicionais (das
   * instncias).
   */
  private static final String ADDITIONAL_LANGUAGE_FILE_PROPERTY =
    "additional.language.file";

  /**
   * O nome da propriedade que define os arquivos de idiomas externos (das
   * instncias).
   */
  private static final String EXTERNAL_LANGUAGE_FILE_BASENAME_PROPERTY =
    "external.language.file.basename";

  /**
   * O nome da propriedade que definm os arquivos de idiomas para a traduo de
   * componentes do Swing.
   */
  private static final String SWING_LANGUAGE_FILE_PROPERTY =
    "swing.language.file";

  /**
   * O nome da propriedade que define a locale default na falta de uma ser
   * informada.
   */
  protected static final String DEFAULT_LOCALE_PROPERTY = "default.locale";

  /** O nome da propriedade que defina uma permisso para execuo do sistema. */
  private static final String EXECUTION_PERMISSION_PROPERTY =
    "execution.permission";

  /** O nome da propriedade que indica qual a classe do dilogo de login. */
  private static final String LOGIN_CLASS_PROPERTY = "login.class";

  /** O nome da propriedade que indica qual a classe da interface de usurio. */
  private static final String UI_CLASS_PROPERTY = "ui.class";

  /**
   * O nome da propriedade que indica qual a classe do contexto inicial de
   * execuo do cliente.
   */
  private static final String INITIAL_CONTEXT_CLASS_PROPERTY =
    "initial.context.class";

  /**
   * O nome da propriedade que indica o nome do servidor deve ser exibido nas
   * janelas primrias.
   */
  private static final String SHOW_SERVER_PROPERTY = "show.server.name";

  /** A configurao de um cliente. */
  private Configuration configuration;

  /** A instncia nica do cliente. */
  private static Client instance;

  /**
   * A fbrica responsvel por criar a viso dos parmetros dos algoritmos.
   */
  private ParameterViewFactory parameterViewFactory;

  /** O nome do sistema. */
  private String systemName;

  /**
   * O nome do servidor, sem a porta, ao qual o cliente est conectado.
   */
  private String serverName;

  /**
   * O ip do servidor ao qual o cliente est conectado.
   */
  private String serverIp;

  /** 
   * Um mapa com os parmetros de inicializao do cliente. 
   */
  private Map<String, String> parameterMap;

  /** Indica se as janelas primrias devem ter o nome do servidor */
  private boolean showServerName;

  /**
   * A interface de usurio.
   */
  private ClientUI clientUI;

  /**
   * Cria um cliente, definindo a sua instncia nica.
   * 
   * @param systemName .
   * @param parameterMap A URL para conexo com o servidor.
   * 
   * @throws IllegalStateException Caso j exista um cliente criado.
   * @throws ConfigurationException Caso no seja possvel obter a configurao
   *         da classe.
   */
  protected Client(String systemName, Map<String, String> parameterMap) {
    if (instance != null) {
      throw new IllegalStateException("J existe um cliente criado.");
    }
    this.systemName = systemName;
    this.parameterMap = parameterMap;
    String server = this.parameterMap.get(SERVER_URL_PARAMETER);
    ServerURI uri = ServerURI.parse(server);
    this.serverName = uri.getHost();
    try {
      this.configuration =
        ConfigurationManager.getInstance().getConfiguration(this.getClass());
    }
    catch (ConfigurationManagerException e) {
      throw new ConfigurationException(e);
    }
    this.showServerName =
      configuration.getOptionalBooleanProperty(SHOW_SERVER_PROPERTY, false);
    this.parameterViewFactory = new DefaultParameterViewFactory();
    instance = this;
  }

  /**
   * Retorna o nome da instncia do Client.
   * 
   * @return Retorna o nome da instncia do Client.
   */
  public String getSystemName() {
    return this.systemName;
  }

  /**
   * Indica se o nome do servidor deve ser exibido no ttulo das janelas
   * primrias.
   * 
   * @return true se o nome do servidor deve ser exibido no ttulo das janelas
   *         primrias
   */
  public boolean showServerName() {
    return showServerName;
  }

  /**
   * Executa o cliente.
   * 
   * @return true, caso o cliente tenha sido executado com sucesso, ou false,
   *         caso contrrio.
   */
  public boolean execute() {
    Locale defaultLocale = null;
    defaultLocale =
      FormatUtils.parseLocale(this.parameterMap.get(LOCALE_PARAMETER));
    System.out.println("Locale padro: "
      + (defaultLocale == null ? Locale.getDefault() : defaultLocale));

    createClientRemoteMonitor();

    LoginInterface login;

    if (parameterMap.get(PRE_LOGIN_TOKEN_PARAMETER) != null) {
      login =
        new PreLogin(parameterMap.get(PRE_LOGIN_TOKEN_PARAMETER),
          createInitialContext(defaultLocale));
    }
    else if (parameterMap.get(CREDENTIAL_IDENTIFIER_PARAMETER) != null) {
      login = new OpenBusLogin(this.createInitialContext(defaultLocale));
    }
    else {
      login = new UserPasswordLogin(createLoginDialog());
    }

    /* efetua o login */
    final InitialContext initialContext =
      login.login(defaultLocale, systemName);

    if (initialContext == null)
      return false;

    /*
     * a ordem de carga dos bundles determina como ser montada a hierarquia de
     * bundles atravs das chamadas aos mtodos LNG.load() e LNG.loadURL()
     */
    loadLanguageFiles(initialContext.getLocale());
    loadExternalLanguageFiles(initialContext.getLocale());

    if (!this.doesLoggedUserHavePermission()) {
      StandardDialogs.showErrorDialog(null, this.systemName, LNG
        .get("csbase.client.Client.user.without.permission.message"));
      return false;
    }

    postInitialization(initialContext);

    /* incio da carga do cliente */
    return executeClientUI(login, initialContext);
  }

  /**
   * Executa o cliente solicitando login e senha ao usurio.
   * 
   * @param defaultLocale O locale default configurado na instalao do
   *        servidor.
   * @return true, caso a execuo do cliente ocorra sem problemas, ou false,
   *         caso contrrio.
   */
  protected InitialContext executeDefaultLogin(Locale defaultLocale) {
    return this.createLoginDialog().execute(defaultLocale);
  }

  /**
   * Obtem a classe de Interface de usurio definida pela propriedade respectiva
   * 
   * @param uiClassProperty .
   * 
   * @return A classe de interface de usurio.
   * 
   * @throws ConfigurationException Caso a propriedade informando a interface de
   *         usurio no seja encontrada ou a classe informada no exista.
   */
  protected Class<ClientUI> getUIClass(String uiClassProperty) {
    Class<ClientUI> uiClass;
    try {
      uiClass = this.configuration.getMandatoryClassProperty(uiClassProperty);
      return uiClass;
    }
    catch (MissingPropertyException e) {
      throw new ConfigurationException(e);
    }
    catch (ClassNotFoundException e) {
      throw new ConfigurationException(e);
    }
  }

  /**
   * Carrega o cliente aps o login do usurio no sistema.
   * 
   * @param initialContext Informaes de login.
   * @param login Tipo do login executado
   * 
   * @return true, caso a carga do cliente ocorra sem problemas, ou false, caso
   *         contrrio.
   */
  protected boolean executeClientUI(final LoginInterface login,
    final InitialContext initialContext) {
    try {
      this.clientUI = this.createUI(getUIClass(UI_CLASS_PROPERTY));
    }
    catch (ClientException e) {
      e.printStackTrace();
      StandardDialogs.showErrorDialog(null, systemName, LNG
        .get("csbase.client.Client.create.client.ui.error.message"));
      login.notifyException(e);
      return false;
    }

    login.notifyPreInitialization();

    RemoteTask<Void> remoteTask = new RemoteTask<Void>() {
      @Override
      protected void handleError(Exception e) {
        if (e instanceof RemoteException)
          super.handleError(e);
        else {
          e.printStackTrace();
          String message;
          if (e.getMessage() == null)
            message =
              LNG.get("csbase.client.Client.start.client.ui.error.message");
          else
            message = e.getMessage();
          StandardDialogs.showErrorDialog(null, systemName, message);
        }
        login.notifyException(e);
      }

      @Override
      protected void performTask() throws RemoteException, ClientException {
        clientUI.preInitialization(initialContext);
        clientUI.showUI();
      }
    };

    // task exibida somente no pre-login
    if (login instanceof UserPasswordLogin)
      remoteTask.setProgressDialogEnabled(false);
    else
      remoteTask.setProgressDialogDelay(0);

    boolean wasTaskExecuted =
      remoteTask.execute(null, this.systemName, MessageFormat.format(LNG
        .get("csbase.client.Client.starting.client.ui.message"),
        this.systemName), 0, LocalTask.CLOSE_BUTTON);

    login.notifyInitialized();

    if (wasTaskExecuted)
      clientUI.postInitialization();

    return wasTaskExecuted;
  }

  /**
   * Verifica se o usurio logado tem permisso para executar o cliente.
   * 
   * @return true, se o usurio pode executar o cliente, ou false, caso
   *         contrrio.
   * 
   * @throws ConfigurationException Caso a classe da permisso no seja
   *         encontrada.
   */
  protected boolean doesLoggedUserHavePermission() {
    Class<? extends Permission> executionPermissionClass;
    try {
      executionPermissionClass =
        this.configuration
          .getOptionalClassProperty(EXECUTION_PERMISSION_PROPERTY);
    }
    catch (ClassNotFoundException e) {
      throw new ConfigurationException(e);
    }
    User loggedUser = User.getLoggedUser();
    if (!loggedUser.isAdmin()) {
      if (executionPermissionClass != null) {
        if (loggedUser.getPermission(executionPermissionClass) == null) {
          return false;
        }
      }
    }
    return true;
  }

  /**
   * Carrega os arquivos de idioma.
   * 
   * @param locale O locale do usurio logado.
   * 
   * @throws ConfigurationException Caso no tenham sido definidos arquivos de
   *         idiomas ou no tenha sido definida alguma propriedade obrigatria.
   */
  public void loadLanguageFiles(Locale locale) {
    List<String> basicLanguageFileList;
    try {
      basicLanguageFileList =
        this.configuration
          .getMandatoryListProperty(BASIC_LANGUAGE_FILE_PROPERTY);
    }
    catch (MissingPropertyException e) {
      throw new ConfigurationException(e);
    }
    List<String> additionalLanguageFileList =
      this.configuration
        .getOptionalListProperty(ADDITIONAL_LANGUAGE_FILE_PROPERTY);
    List<String> allLanguageFileList = new LinkedList<String>();
    allLanguageFileList.addAll(basicLanguageFileList);
    allLanguageFileList.addAll(additionalLanguageFileList);
    Iterator<String> allLanguageFileIterator = allLanguageFileList.iterator();
    while (allLanguageFileIterator.hasNext()) {
      String idiomFileBaseName = allLanguageFileIterator.next();
      LNG.load(idiomFileBaseName, locale);
    }
    JComponent.setDefaultLocale(locale);
    if (!locale.getLanguage().equals(new Locale("en", "", "").getLanguage())) {
      try {
        GUIUtils.translateSwingComponents(locale, this.configuration
          .getMandatoryListProperty(SWING_LANGUAGE_FILE_PROPERTY));
      }
      catch (MissingPropertyException e) {
        throw new ConfigurationException(e);
      }
    }
  }

  /**
   * Carrega os arquivos de idioma que so externos. Os arquivos externos
   * definidos sero inseridos no fim da hierarquia de bundles.
   * 
   * @param locale O locale do usurio logado.
   * 
   * @return true, se todos os bundles externos foram carregados com sucesso
   */
  protected boolean loadExternalLanguageFiles(Locale locale) {

    // prefixo que indica a URL-Base para obteno dos bundles
    String urlPrefix = getCodeBaseStr();

    // carrega todos os arquivos de idiomas externos definidos
    List<String> externalLanguageFileList =
      getConfiguration().getOptionalListProperty(
        EXTERNAL_LANGUAGE_FILE_BASENAME_PROPERTY);

    boolean success = true;
    for (String idiomFileBaseName : externalLanguageFileList) {
      try {
        LNG.loadURL(urlPrefix, idiomFileBaseName, locale);
      }
      catch (IOException e) {
        String name =
          urlPrefix + idiomFileBaseName + "_" + locale + ".properties";
        StandardErrorDialogs.showErrorDialog(null, getSystemName(),
          "Falha na carga de bundle de idiomas externo: " + name, e);
        success = false;
      }
    }
    return success;
  }

  /**
   * Retorna a URL que representa o codebase da aplicao ou, se no for
   * possvel obter os recursos externos, retorna o path absoluto corrente.
   * 
   * @return prefixo de uma url, incluindo protocolo (http://, file:// etc)
   * 
   */
  public static String getCodeBaseStr() {
    if (ExternalResources.getInstance().isEnabled()) {
      try {
        return ExternalResources.getInstance().getCodeBase().toExternalForm();
      }
      catch (CSBaseException e) {
        return FileUtils.prepareURL("");
      }
    }
    return FileUtils.prepareURL("");
  }

  /**
   * Cria o monitorador do servidor.
   */
  protected void createClientRemoteMonitor() {
    Class monitorClass = getClientRemoteMonitorClass();
    Constructor constructor;

    // Tipos dos argumentos do construtor do Monitor
    Class[] argTypes = { String.class, String.class, Class.class };
    try {
      constructor = monitorClass.getConstructor(argTypes);
    }
    catch (NoSuchMethodException e) {
      throw new ConfigurationException(e,
        "No foi possvel obter o construtor da classe de monitorao do "
          + "servidor.");
    }

    // Argumentos para o construtor do Monitor
    Object[] args =
      { this.systemName, this.parameterMap.get(SERVER_URL_PARAMETER),
          this.getClientRemoteLocatorClass() };

    try {
      constructor.newInstance(args);
    }
    catch (InstantiationException e) {
      throw new ConfigurationException(e,
        "No foi possvel criar o monitorador do servidor.");
    }
    catch (IllegalAccessException e) {
      throw new ConfigurationException(e,
        "No foi possvel criar o monitorador do servidor.");
    }
    catch (InvocationTargetException e) {
      Throwable cause = e.getCause();
      if (cause instanceof RuntimeException) {
        throw (RuntimeException) cause;
      }
      throw new ConfigurationException(e,
        "No foi possvel criar o monitorador do servidor.");
    }
    ClientRemoteMonitor.getInstance().setSystemName(this.systemName);
  }

  /**
   * A classe do localizador de servios remotos.
   * 
   * @return O localizador de servios remotos.
   * 
   * @throws ConfigurationException Caso a classe informada no exista.
   */
  protected Class getClientRemoteLocatorClass() {
    try {
      return this.configuration.getOptionalClassProperty(
        REMOTE_LOCATOR_CLASS_PROPERTY, ClientRemoteLocator.class);
    }
    catch (ClassNotFoundException e) {
      throw new ConfigurationException(e);
    }
  }

  /**
   * A classe de monitorao do servidor.
   * 
   * @return O monitorador do servidor.
   * 
   * @throws ConfigurationException Caso a classe informada no exista.
   */
  protected Class getClientRemoteMonitorClass() {
    try {
      return this.configuration.getOptionalClassProperty(
        REMOTE_MONITOR_CLASS_PROPERTY, ClientRemoteMonitor.class);
    }
    catch (ClassNotFoundException e) {
      throw new ConfigurationException(e);
    }
  }

  /**
   * <p>
   * Esse mtodo  chamado aps o usurio ter feito o login.
   * </p>
   * <p>
   * OBS: Nesse momento, as configuraes de idioma e os campos do locator para
   * acesso aos servios j esto definidos.
   * </p>
   * 
   * @param initialContext O contexto inicial de execuo do cliente.
   */
  protected void postInitialization(InitialContext initialContext) {
    /*
     * Este mtodo no deve ser abstrato, pois nem todas as classes que herdam
     * de Client tem ps-inicializao.
     */
  }

  /**
   * Cria a interface de usurio.
   * 
   * @param uiClass Classe da interface de usurio
   * 
   * @return A interface de usurio.
   * 
   * @throws ClientException Caso ocorra algum problema ao iniciar a interface
   *         de usurio.
   * @throws ConfigurationException Caso ocorra algum problema durante a
   *         construo da interface de usurio
   */
  protected ClientUI createUI(Class<ClientUI> uiClass) throws ClientException {
    Constructor<ClientUI> constructor;
    try {
      constructor = uiClass.getConstructor();
    }
    catch (NoSuchMethodException e) {
      throw new ConfigurationException(e,
        "No foi possvel obter o construtor da classe de interface do usurio.");
    }
    try {
      return constructor.newInstance();
    }
    catch (InstantiationException e) {
      throw new ConfigurationException(e,
        "No foi possvel criar a interface do usurio.");
    }
    catch (IllegalAccessException e) {
      throw new ConfigurationException(e,
        "No foi possvel criar a interface do usurio.");
    }
    catch (InvocationTargetException e) {
      Throwable cause = e.getCause();
      if (cause instanceof ClientException) {
        throw (ClientException) cause;
      }
      if (cause instanceof RuntimeException) {
        throw (RuntimeException) cause;
      }
      throw new ConfigurationException(e,
        "No foi possvel criar a interface do usurio.");
    }
  }

  /**
   * Cria o dilogo de login.
   * 
   * @return O dilogo de login.
   * 
   * @throws ConfigurationException Caso ocorra algum erro na criao do dilogo
   *         de login ou a classe informada como classe de login no exista.
   */
  protected AbstractLoginUI createLoginDialog() {
    Class<? extends AbstractLoginUI> loginClass;
    try {
      loginClass =
        this.configuration.getOptionalClassProperty(LOGIN_CLASS_PROPERTY,
          LoginUI.class);
    }
    catch (ClassNotFoundException e) {
      throw new ConfigurationException(e);
    }
    Constructor<? extends AbstractLoginUI> constructor;
    try {
      constructor = loginClass.getConstructor();
    }
    catch (NoSuchMethodException e) {
      throw new ConfigurationException(e,
        "No foi possvel obter o construtor da classe de login.");
    }
    try {
      return constructor.newInstance();
    }
    catch (InstantiationException e) {
      throw new ConfigurationException(e,
        "No foi possvel criar o dilogo de login.");
    }
    catch (IllegalAccessException e) {
      throw new ConfigurationException(e,
        "No foi possvel criar o dilogo de login.");
    }
    catch (InvocationTargetException e) {
      throw new ConfigurationException(e,
        "No foi possvel criar o dilogo de login.");
    }
  }

  /**
   * Obtm a instncia nica do cliente.
   * 
   * @return A instncia nica do cliente.
   * 
   * @throws IllegalArgumentException Caso a instncia ainda no tenha sido
   *         criada.
   */
  public static Client getInstance() {
    if (instance == null) {
      throw new IllegalArgumentException(
        "A instncia do cliente ainda no foi criada.");
    }
    return instance;
  }

  /**
   * Obtm um parmetro do cliente.
   * 
   * @param parameterName O nome do parmetro que se deseja.
   * 
   * @return O parmetro, ou null, caso no exista.
   */
  public final String getParameter(String parameterName) {
    return this.parameterMap.get(parameterName);
  }

  /**
   * Obtm o nome do servidor, sem a porta, ao qual o cliente est conectado. O
   * endereo do servidor com a porta tambm  um parmetro do cliente que pode
   * ser obtido atravs do mtodo
   * <code>getParameter(Client.SERVER_URL_PARAMETER)</code>.
   * 
   * @return O nome do servidor
   */
  public final String getServerName() {
    return serverName;
  }

  /**
   * Obtm o ip do servidor.
   * 
   * @return o ip do servidor.
   * 
   * @throws ConfigurationException se no foi possvel obter o ip, se um ou se
   *         um scope_id foi especificado para um endereo IPv6 global ou se
   *         existe um {@link SecurityManager gerente de segurana} e seu mtodo
   *         checkConnect no permite a operao.
   */
  public synchronized String getServerAddress() throws ConfigurationException {

    if (null == serverIp) {
      try {
        InetAddress address = InetAddress.getByName(getServerName());
        byte[] ip = address.getAddress();
        serverIp = "";
        for (int i = 0; i < ip.length; i++) {
          if (i > 0) {
            serverIp += ".";
          }
          serverIp += ip[i] & 0xFF;
        }
      }
      catch (UnknownHostException e) {
        throw new ConfigurationException(e,
          "No foi possvel obter o ip do servidor.");
      }
      catch (SecurityException e) {
        throw new ConfigurationException(e,
          "No foi possvel obter o ip do servidor.");
      }
    }
    return serverIp;
  }

  /**
   * Cria o contexto inicial de execuo do cliente.
   * 
   * @param locale O idioma selecionado pelo usurio.
   * 
   * @return O contexto inicial de execuo do cliente.
   * 
   * @throws ConfigurationException Caso ocorra algum erro na criao do
   *         contexto inicial.
   */
  protected InitialContext createInitialContext(Locale locale) {
    Class<InitialContext> initialContextClass;
    try {
      initialContextClass =
        this.configuration.getOptionalClassProperty(
          INITIAL_CONTEXT_CLASS_PROPERTY, InitialContext.class);
    }
    catch (ClassNotFoundException e) {
      throw new ConfigurationException(e);
    }
    Constructor<InitialContext> constructor;
    try {
      constructor = initialContextClass.getConstructor(Locale.class);
    }
    catch (NoSuchMethodException e) {
      throw new ConfigurationException(e,
        "No foi possvel obter o construtor da classe contexto inicial.");
    }
    try {
      return constructor.newInstance(locale);
    }
    catch (InstantiationException e) {
      throw new ConfigurationException(e,
        "No foi possvel criar o contexto inicial.");
    }
    catch (IllegalAccessException e) {
      throw new ConfigurationException(e,
        "No foi possvel criar o contexto inicial.");
    }
    catch (InvocationTargetException e) {
      throw new ConfigurationException(e,
        "No foi possvel criar o contexto inicial.");
    }
  }

  /**
   * Obtm o endereo da mquina onde o cliente est rodando.
   * 
   * @return O endereo da mquina onde o cliente est rodando.
   */
  public final String getHostAddress() {
    String hostAddress = this.getParameter(CLIENT_ADDRESS_PARAMETER);
    if (hostAddress != null) {
      return hostAddress;
    }
    try {
      return InetAddress.getLocalHost().getHostAddress();
    }
    catch (UnknownHostException e) {
      return DEFAULT_HOST_ADDRESS;
    }
  }

  /**
   * Retorna o locale default configurado como parmetro no arquivo JNLP do
   * cliente.
   * 
   * @return O locale configurado no JNLP do cliente ou null se o parmetro no
   *         foi encontrado.
   */
  public final Locale getDefaultLocale() {
    if (this.getParameter(LOCALE_PARAMETER) != null) {
      return new Locale(this.getParameter(LOCALE_PARAMETER));
    }
    return null;
  }

  /**
   * Obtm a configurao da classe.
   * 
   * @return A configurao do usurio
   */
  protected Configuration getConfiguration() {
    return configuration;
  }

  /**
   * Obtm o componente grfico concreto que representa a interface de usurio.
   * 
   * @return O componente grfico concreto.
   */
  public final JFrame getView() {
    if (this.clientUI == null)
      return null;
    return this.clientUI.getView();
  }

  /**
   * Obtm a fbrica de viso de parmetros dos algoritmos.
   * 
   * @return A fbrica.
   */
  public ParameterViewFactory getParameterViewFactory() {
    return parameterViewFactory;
  }

  /**
   * Atribui a fbrica de viso de parmetros dos algoritmos.
   * 
   * @param parameterViewFactory A fbrica.
   */
  protected void setParameterViewFactory(
    ParameterViewFactory parameterViewFactory) {
    this.parameterViewFactory = parameterViewFactory;
  }
}
