package csbase.client.algorithms.commands.view;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import tecgraf.javautils.FormatUtils;
import tecgraf.javautils.LNG;
import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.configurationmanager.ConfigurationManagerException;
import tecgraf.javautils.gui.GBC;
import tecgraf.javautils.gui.GUIUtils;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.algorithms.commands.cache.CommandsCache;
import csbase.client.algorithms.commands.cache.events.AbstractCommandUpdatedEventListener;
import csbase.client.algorithms.commands.cache.events.CommandUpdatedEvent.Type;
import csbase.client.applications.ApplicationImages;
import csbase.client.desktop.DesktopComponentFrame;
import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.RemoteTask;
import csbase.client.kernel.ClientException;
import csbase.client.project.tasks.GetFileTask;
import csbase.client.util.StatusBar;
import csbase.client.util.event.EventListener;
import csbase.client.util.gui.log.LogPanel;
import csbase.client.util.gui.log.LogPanel.FileEvent;
import csbase.client.util.gui.log.LogPanel.ThrowableEvent;
import csbase.client.util.gui.log.LogPanelReloader.StatusChangedEvent;
import csbase.exception.OperationFailureException;
import csbase.exception.ParseException;
import csbase.logic.ClientProjectFile;
import csbase.logic.ClientProjectFileInputStream;
import csbase.logic.CommandFinalizationType;
import csbase.logic.CommandInfo;
import csbase.logic.CommandStatus;
import csbase.logic.CommonClientProject;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.FileParameterValue;

/**
 * Dilogo abstrato de visualizao de {@link CommandInfo}.
 * 
 * @author Tecgraf / PUC-Rio
 */
abstract class AbstractAlgorithmCommandView extends DesktopComponentFrame {

  /**
   * Indica se erros devem ser mostrados no console.
   */
  private static boolean logEnabled;

  /**
   * Dimenso mnima preferida. S deve ser menor que isso se no couber na
   * tela.
   * 
   * Altura mnima
   */
  private static final long MINIMUN_PREFERRED_HEIGHT = 250;
  /**
   * Largura mnina
   */
  private static final long MINIMUN_PREFERRED_WIDTH = 800;

  /**
   * Aba que apresenta a configurao do comando.
   */
  private ConfigurationReportTab configurationViewTab;
  /**
   * Aba selecionada no momento.
   */
  private Tab selected;
  /**
   * Painel que contm as abas.
   */
  private JTabbedPane tabbedPane;
  /**
   * Barra de status.
   */
  private StatusBar statusBar;

  /**
   * Mapeia o tipo de aba para o componente visual que a implementa.
   */
  private Map<TabType, Component> _tabComponents =
    new HashMap<TabType, Component>();

  /**
   * Recebe notificaes sobre atualizaces feitas no comando.
   */
  private AbstractCommandUpdatedEventListener commandListener;

  /**
   * Comando representado nessa viso.
   */
  private CommandInfo command;

  /**
   * O configurador a ter seus parmetros apresentados. Deve-se ignorar o
   * configurador do comando pois pode ser diferente deste. Isso ocorrerira no
   * caso de estar representando um n de um comando de fluxo (No aceita
   * {@code null}).
   */
  private AlgorithmConfigurator configurator;

  static {
    try {
      Configuration configuration =
        ConfigurationManager.getInstance().getConfiguration(
        AbstractAlgorithmCommandView.class);

      logEnabled =
        configuration.getOptionalBooleanProperty("log.enable", false);
    }
    catch (ConfigurationManagerException e1) {
      e1.printStackTrace();
      logEnabled = false;
    }
  }

  /**
   * @param index identificador da view. Usado para garantir a unicidade.
   * @param owner A janela pai deste dilogo. No pode ser nula e tem que ser
   *        uma instncia de {@link JFrame} ou {@link JDialog}.
   * @param command O comando (No aceita {@code null}).
   * @param configurator O configurador a ter seus parmetros apresentados.
   *        Deve-se ignorar o configurador do comando pois pode ser diferente
   *        deste. Isso ocorrerira no caso de estar representando um n de um
   *        comando de fluxo (No aceita {@code null}).
   * 
   * @throws ClientException Erro ao obter informaes sobre o comando.
   */
  public AbstractAlgorithmCommandView(final Object index,
    final DesktopComponentFrame owner, final CommandInfo command,
    final AlgorithmConfigurator configurator) throws ClientException {

    super(index, owner, createTitle(command));

    this.command = command;
    this.configurator = configurator;
    this.commandListener = createCommandListener(command);
    CommandsCache.getInstance().addEventListener(commandListener);

    this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
    this.addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosed(WindowEvent e) {
        /**
         * Desseleciona a tab que estiver selecionada, para que esta se
         * finalize.
         */
        if (null != selected) {
          selected.setSelected(false);
        }
        stop();
        removeDesktopComponentFrame(getIndex());
      }
    });

    initialize(configurator);
    initializeGui(configurator);
  }

  /**
   * Mtodo que possibilita classes filhas a inicializarem atributos antes da
   * inicializao dos componentes grficos da classe.
   * 
   * @param cnf O configurador a ter seus parmetros apresentados.
   *        Deve-se ignorar o configurador do comando pois pode ser diferente
   *        deste. Isso ocorrerira no caso de estar representando um n de um
   *        comando de fluxo (No aceita {@code null}).
   * 
   * @throws ClientException Erro ao obter informaes sobre o comando.
   */
  protected void initialize(AlgorithmConfigurator cnf)
    throws ClientException {

  }

  /**
   * Mostra essa janela dando preferncia para abrir com a tab escolhida.
   * 
   * @param preferredTab Qual a tab gostaria que estivesse selecionada quando
   *        esta janela for criada. No  garantido que esta janela abra com a
   *        tab escolhida selecionada, pois aquela tab pode nem existir caso
   *        seja a tab de log.
   */
  public void show(final TabType preferredTab) {
    if (null != preferredTab) {
      Component component = _tabComponents.get(preferredTab);
      if (null != component) {
        tabbedPane.setSelectedComponent(component);
      }
    }
    setVisible(true);
  }

  /** Remove o observador do comando. */
  protected void stop() {
    CommandsCache.getInstance().removeEventListener(commandListener);
  }

  /**
   * @see java.awt.Window#pack()
   */
  @Override
  public void pack() {
    super.pack();
    updateSize();
    center(owner);
    updateLocation();
  }

  /**
   * Retorna comando representado por esta viso.
   * 
   * @return O comando representado pela viso.
   */
  protected CommandInfo getCommand() {
    return command;
  }

  /**
   * Determina o comando a ser visualizado por esta viso.
   * 
   * @param command O comando a ser visualizado.
   */
  protected void setCommand(CommandInfo command) {
    this.command = command;
    if (command.getStatus() == CommandStatus.FINISHED) {
      stop();
    }
  }

  /**
   * Retorna um listener para o comando a ser visualizado, para que a viso
   * possa ser atualizada.
   * 
   * @param command O comando a ser visualizado.
   * @return O listener para o comando.
   */
  private AbstractCommandUpdatedEventListener createCommandListener(
    CommandInfo command) {
    return new AbstractCommandUpdatedEventListener(command.getProjectId(),
      command.getId()) {
      @Override
      public void eventFired(Type type, CommandInfo cmd) {
        // Atualiza a instncia com o valor recebido pelo evento.
        setCommand(cmd);
        updateStatusBar();
      }

      @Override
      protected void eventInterrupted(Exception exception, String description) {
        statusBar.setError(description);
      }
    };
  }

  /**
   * Cria a tab de relatrio do configurador.
   * 
   * @param configurator O configurador a ter seus parmetros apresentados.
   *        Deve-se ignorar o configurador do comando pois pode ser diferente
   *        deste. Isso ocorrerira no caso de estar representando um n de um
   *        comando de fluxo (No aceita {@code null}).
   * 
   * @return a tab de relatrio do configurador.
   * 
   * @throws ClientException Erro ao obter informaes sobre o comando.
   */
  protected abstract ConfigurationReportTab createConfigurationReportTab(
    final AlgorithmConfigurator configurator) throws ClientException;

  /**
   * Retorna a aba que apresenta a configurao do comando.
   * 
   * @return A aba de configurao do comando.
   */
  protected Tab getConfigurationReportTab() {
    return configurationViewTab;
  }

  /**
   * Cria os componentes e inicializa os listeners desta janela.
   * 
   * @param configurator O configurador a ter seus parmetros apresentados.
   *        Deve-se ignorar o configurador do comando pois pode ser diferente
   *        deste. Isso ocorrerira no caso de estar representando um n de um
   *        comando de fluxo (No aceita {@code null}).
   * 
   * @throws ClientException Erro ao obter informaes sobre o comando.
   */
  private void initializeGui(final AlgorithmConfigurator configurator)
    throws ClientException {

    String title = createTitle(command);
    this.setTitle(title);

    this.setLayout(new BorderLayout());

    final JPanel buttonPanel = createButtonPanel();
    statusBar = new StatusBar();
    statusBar.showStatusBar();

    JPanel southPane = new JPanel(new BorderLayout());
    southPane.add(buttonPanel, BorderLayout.NORTH);
    southPane.add(statusBar, BorderLayout.SOUTH);
    add(southPane, BorderLayout.SOUTH);

    /** Deve-se criar a tab configurationViewTab antes de adicionar o evento. */
    this.tabbedPane = createTabbedPane(configurator);
    add(this.tabbedPane, BorderLayout.CENTER);

    /** Atualiza a barra de estado com o estado inicial do comando. */
    updateStatusBar();

    this.pack();
  }

  /**
   * Cria um ttulo que contm um texto fixo mais a descrio do comando e a
   * hora em que ele foi submetido.
   * 
   * @param command O comando (No aceita {@code null}).
   * 
   * @return o ttulo criado.
   */
  private static String createTitle(CommandInfo command) {
    String title = LNG.get("AbstractAlgorithmCommandView.title");
    String description = command.getDescription();
    if (description != null && description.length() != 0) {
      title += " : " + description;
    }
    Date startDate = command.getSubmittedDate();
    if (startDate != null) {
      title += " : " + FormatUtils.format(startDate);
    }
    return title;
  }

  /**
   * Cria o painel de tabs contento a tab de relatrio do configurador e a de
   * log.
   * 
   * @param configurator O configurador a ter seus parmetros apresentados.
   *        Deve-se ignorar o configurador do comando pois pode ser diferente
   *        deste. Isso ocorrerira no caso de estar representando um n de um
   *        comando de fluxo (No aceita {@code null}).
   * 
   * @return o painel de tabs.
   * 
   * @throws ClientException Erro ao obter informaes sobre o comando.
   */
  private JTabbedPane createTabbedPane(final AlgorithmConfigurator configurator)
    throws ClientException {

    final JTabbedPane tabbedPane = new JTabbedPane();
    tabbedPane.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        JPanel panel = (JPanel) tabbedPane.getSelectedComponent();
        Tab newSelected = (Tab) panel.getComponent(0);

        if (newSelected.equals(selected)) {
          return;
        }

        if (null != selected) {
          selected.setSelected(false);
        }

        selected = newSelected;
        newSelected.setSelected(true);
      }
    });

    createTabs(tabbedPane, configurator);

    return tabbedPane;
  }

  /**
   * Cria no painel de tabs, as tabs de relatrio do configurador e a de log.
   * 
   * @param tabbedPane Painel que receber as tabs.
   * 
   * @param configurator O configurador a ter seus parmetros apresentados.
   *        Deve-se ignorar o configurador do comando pois pode ser diferente
   *        deste. Isso ocorrerira no caso de estar representando um n de um
   *        comando de fluxo (No aceita {@code null}).
   * 
   * @throws ClientException Erro ao obter informaes sobre o comando.
   */
  private void createTabs(final JTabbedPane tabbedPane,
    final AlgorithmConfigurator configurator) throws ClientException {

    configurationViewTab = createConfigurationReportTab(configurator);
    addTab(tabbedPane, configurationViewTab, TabType.CONFIGURATION_REPORT);

    /**
     * Utiliza o tamanho da tab de configurao como o tamanho para as tabs de
     * log. Caso no fosse definido um tamanho para as tabs de log, estas teriam
     * o tamanho definido pela pgina do arquivo de log.
     */
    Dimension tabsDimension = new Dimension(600, 300);
    Tab logTab = createLogTab(configurator, tabsDimension);
    if (null != logTab) {
      addTab(tabbedPane, logTab, TabType.LOG);
    }

    if (hasAdditionalLogsTab()) {
      if (getAdditionalLogs() != null && getAdditionalLogs().size() > 0) {
        Tab additionalLogsTab = createAdditionalLogsTab(tabsDimension);
        if (null != additionalLogsTab) {
          addTab(tabbedPane, additionalLogsTab, TabType.ADDITIONAL_LOGS);
        }
      }
    }

  }

  /**
   * Retorna a lista de arquivos de logs adicionais do comando.
   * 
   * @return A lista de logs adicionais.
   */
  private Set<ClientProjectFile> getAdditionalLogs() {
    FindAdditionalLogsTask task = new FindAdditionalLogsTask(getCommand());
    if (task.execute(SwingUtilities.getWindowAncestor(this), getTitle(), LNG
      .get("AbstractAlgorithmCommandView.loading.additional_logs"))) {
      Set<ClientProjectFile> additionalLogFiles = task.getResult();
      return additionalLogFiles;
    }
    else {
      StandardDialogs.showErrorDialog(this.getParent(), "Erro",
        "Erro ao obter arquivos de log adicionais do comando "
        + getCommand().getId());
      return null;
    }
  }

  /**
   * Adiciona uma nova aba a esta viso.
   * 
   * @param tabbedPane O painel que armazena as abas.
   * @param tab Aba a ser adicionada.
   * @param preferredTab Aba que deve ser mostrada preferencialmente, ao abrir a
   *        viso.
   */
  private void addTab(final JTabbedPane tabbedPane, final Tab tab,
    final TabType preferredTab) {
    JPanel tabPanel = new JPanel(new GridBagLayout());
    tabPanel.add(tab, new GBC(0, 0).both());
    tabbedPane.addTab(tab.getTitle(), tabPanel);

    _tabComponents.put(preferredTab, tabPanel);
  }

  /**
   * Cria a aba que apresenta o log de execuo do comando.
   * 
   * @param configurator Configurador do comando.
   * @param preferredSize Tamanho preferencial da aba.
   * @return A aba de log.
   */
  protected Tab createLogTab(final AlgorithmConfigurator configurator,
    final Dimension preferredSize) {

    FileParameterValue log = configurator.getStandardOutputFile();
    if (null == log) {
      return null;
    }
    else {
      final LogTab logTab = new LogTab(log);
      logTab.setPreferredSize(preferredSize);
      return logTab;
    }
  }

  /**
   * Cria uma aba adicional para logs configurvel por sistema.
   * 
   * @param preferredSize O tamanho preferencial da aba ou nulo
   * @return a aba adicional de logs
   */
  protected Tab createAdditionalLogsTab(final Dimension preferredSize) {
    final AdditionalLogsTab logTab = new AdditionalLogsTab(getCommand());
    logTab.setPreferredSize(preferredSize);
    return logTab;
  }

  /**
   * Cria um painel com botes para a viso.
   * 
   * @return O painel com botes.
   */
  private JPanel createButtonPanel() {
    final JButton closeButton =
      new JButton(LNG.get("AbstractAlgorithmCommandView.button.close.label"));
    closeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        close();
      }
    });
    final JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
    buttonPanel.add(closeButton);

    return buttonPanel;
  }

  /**
   * Atualiza a barra de status com o estado do comando.
   */
  protected void updateStatusBar() {
    switch (command.getStatus()) {
      case SCHEDULED:
        statusBar.setStatus(LNG
          .get("AbstractAlgorithmCommandView.status.scheduled"));
        break;
      case FINISHED:
        statusBar
          .setStatus(getFinalizationTypeDescription(configurator));
        break;
      case SYSTEM_FAILURE:
        statusBar.setStatus(LNG
          .get("AbstractAlgorithmCommandView.status.system_failure"));
        break;
      case EXECUTING:
      case INIT:
      case UPLOADING:
      case DOWNLOADING:
        if (command.isValid()) {
          if (command.isQueued()) {
            statusBar.setStatus(LNG
              .get("AbstractAlgorithmCommandView.status.queued"));
          }
          else {
            statusBar.setStatus(LNG
              .get("AbstractAlgorithmCommandView.status.running"));
          }
        }
        else {
          statusBar.setStatus(LNG
            .get("AbstractAlgorithmCommandView.status.disconnected"));
        }
        break;
      default:
        statusBar.setStatus(LNG
          .get("AbstractAlgorithmCommandView.status.unknown"));
        break;
    }
  }

  /**
   * Retorna a descrio do tipo de finalizao, levando em considerao a
   * o tipo de finalizao indicado no configurador do algoritmo. Deve-se
   * ignorar o configurador do comando em alguns casos, pois pode ser diferente
   * deste. Isso ocorreria no caso do algoritmo ser um n de um
   * fluxo (No aceita {@code null}).
   * 
   * @param configurator O configurador do algoritmo.
   * @return A descrio do tipo de finalizao de comando.
   */
  private String getFinalizationTypeDescription(
    AlgorithmConfigurator configurator) {
    CommandFinalizationType finalizationType = command.getFinalizationType();
    switch (finalizationType) {
      case EXECUTION_ERROR:
      case NO_EXIT_CODE:
      case SUCCESS:
        return getFinalizationTypeDescriptionFromConfigurator(configurator,
          finalizationType);
      default:
        return getDeafultFinalizationTypeDescription(finalizationType);
    }
  }

  /**
   * Retorna o descrio padro do tipo de finalizao de comando.
   * 
   * @param finalizationStatus O tipo de finalizao de comando.
   * @return A descrio padro do tipo de finalizao de comando.
   */
  private String getDeafultFinalizationTypeDescription(
    CommandFinalizationType finalizationStatus) {
    switch (command.getFinalizationType()) {
      case END:
        return LNG.get("AbstractAlgorithmCommandView.status.finished");
      case EXECUTION_ERROR:
        return (LNG
          .get("AbstractAlgorithmCommandView.status.finished.error"));
      case NO_EXIT_CODE:
        return (LNG.get("AbstractAlgorithmCommandView.status.finished.no_code"));
      case SUCCESS:
        return (LNG.get("AbstractAlgorithmCommandView.status.finished.success"));
      case FAILED:
        return (LNG.get("AbstractAlgorithmCommandView.status.finished.failed"));
      case KILLED:
        return (LNG.get("AbstractAlgorithmCommandView.status.finished.killed"));
      case LOST:
        return (LNG.get("AbstractAlgorithmCommandView.status.finished.lost"));
      default:
        return (LNG.get("AbstractAlgorithmCommandView.status.finished"));
    }
  }

  /**
   * Retorna a descrio do tipo de finalizao indicado no configurador do
   * algoritmo. Caso no seja possvel determinar o tipo de finalizao, retorna
   * a descrio do tipo de finalizao apontada no comando (pode ser diferente
   * no caso de um algoritmo que  n de um fluxo).
   * 
   * @param configurator O configurador do algoritmo.
   * @return A descrio do tipo de finalizao de comando.
   * @param commandFinalizationType O tipo de finalizao do comando.
   */
  private String getFinalizationTypeDescriptionFromConfigurator(
    AlgorithmConfigurator configurator,
    CommandFinalizationType commandFinalizationType) {
    if (configurator.hasExitCode()) {
      Integer exitCode = getExitCode(configurator);
      if (exitCode != null) {
        /*
         * Mesmo o comando tendo status de erro/sem cdigo de sada, temos
         * que testar o cdigo de sada deste configurador especfico. No
         * caso de fluxo, esse algortimo pode conseguir executar com
         * sucesso, enquanto outro algortimo termina sua execuo com o erro
         * que levou ao status EXECUTION_ERROR do comando.
         */
        if (exitCode != 0) {
          return (LNG.get(
            "AbstractAlgorithmCommandView.status.finished.error.code",
            new Object[] { exitCode }));
        }
        else {
          return (LNG
            .get("AbstractAlgorithmCommandView.status.finished.success"));
        }
      }
      else {
        return getDeafultFinalizationTypeDescription(commandFinalizationType);
      }
    }
    else {
      return (LNG.get("AbstractAlgorithmCommandView.status.finished"));
    }
  }

  /**
   * L o arquivo que guarda o cdigo de sada do algoritmo e retorna o cdigo.
   * 
   * @param configurator Configurador do algoritmo.
   * @return cdigo de sada do algoritmo.
   */
  private Integer getExitCode(final AlgorithmConfigurator configurator) {
    final Object projectId = command.getProjectId();
    RemoteTask<Integer> exitCodeTask = new RemoteTask<Integer>() {
      @Override
      protected void performTask() throws OperationFailureException,
        IOException, ParseException, NumberFormatException {
        ClientProjectFileInputStream inputStream = null;
        BufferedReader bufferedReader = null;
        try {
          if (configurator.hasExitCode()
            && configurator.getExitCodeLogFile() != null) {
            ClientProjectFile clientProjectFile =
              getClientProjectFile(projectId, configurator.getExitCodeLogFile());
            if (clientProjectFile != null) {
              inputStream =
                new ClientProjectFileInputStream(clientProjectFile, 0);
              bufferedReader =
                new BufferedReader(new InputStreamReader(inputStream));
              String line = bufferedReader.readLine();
              if (line != null && !line.isEmpty()) {
                int exitCode = Integer.parseInt(line);
                setResult(exitCode);
              }
              else {
                setResult(null);
              }
            }
          }
        }
        finally {
          try {
            if (inputStream != null) {
              inputStream.close();
            }
            if (bufferedReader != null) {
              bufferedReader.close();
            }
          }
          catch (IOException e) {
          }
        }
      }
    };
    boolean successful =
      exitCodeTask.execute(this.getOwner(),
      "AbstractAlgorithmCommandView.loading.exit_code.title",
      "AbstractAlgorithmCommandView.loading.exit_code.message");

    if (successful) {
      return exitCodeTask.getResult();
    }
    else {
      return null;
    }

  }

  /**
   * Ajusta o tamanho do dilogo para que ele no passe dos limites do monitor.
   */
  private void updateSize() {

    Dimension screenSize = GUIUtils.getScreenDimension();
    Dimension currentSize = getSize();

    double currentWidth = currentSize.getWidth();
    double newWidth = Math.max(currentWidth, MINIMUN_PREFERRED_WIDTH);
    double usuableWidth = screenSize.getWidth();
    newWidth = Math.min(newWidth, usuableWidth);

    double currentHeight = currentSize.getHeight();
    double newHeight = Math.max(currentHeight, MINIMUN_PREFERRED_HEIGHT);
    double usuableHeight = screenSize.getHeight();
    newHeight = Math.min(newHeight, usuableHeight);

    Dimension newSize = new Dimension();
    newSize.setSize(newWidth, newHeight);
    setSize(newSize);
  }

  /**
   * Ajusta a localizao do dilogo para que ele no passe dos limites do
   * monitor.
   */
  private void updateLocation() {
    Dimension screenSize = GUIUtils.getScreenDimension();
    Dimension currentSize = getSize();
    double usuableWidth = screenSize.getWidth();
    double currentWidth = currentSize.getWidth();
    double usuableHeight = screenSize.getHeight();
    double currentHeight = currentSize.getHeight();
    Point currentLocation = getLocation();
    double currentLeftmostX = currentLocation.getX();
    double availableWidth = usuableWidth - currentLeftmostX;
    double currentUpperY = currentLocation.getY();
    double availableHeight = usuableHeight - currentUpperY;
    double newLeftmostX;
    if (availableWidth < currentWidth) {
      newLeftmostX = currentLeftmostX - (currentWidth - availableWidth);
      newLeftmostX = Math.max(newLeftmostX, 0);
    }
    else {
      newLeftmostX = currentLeftmostX;
    }
    double newUpperY;
    if (availableHeight < currentHeight) {
      newUpperY = currentUpperY - (currentHeight - availableHeight);
      newUpperY = Math.max(newUpperY, 0);
    }
    else {
      newUpperY = currentUpperY;
    }
    Point newLocation = new Point();
    newLocation.setLocation(newLeftmostX, newUpperY);
    setLocation(newLocation);
  }

  /**
   * Retorna o {@link ClientProjectFile} que corresponde ao
   * {@link FileParameterValue} especificado.
   * 
   * @param projectId Identificador do projeto ao qual o log pertence.
   * @param fileParameter O {@link FileParameterValue}
   * @return {@link ClientProjectFile} referente ao {@link FileParameterValue}
   *         especificado
   */
  private ClientProjectFile getClientProjectFile(final Object projectId,
    final FileParameterValue fileParameter) {
    CommonClientProject project = DesktopFrame.getInstance().getProject();

    if (null == project) {
      log2Console(
        true,
        "[ERROR] No foi posivel recuperar o arquivo %s pois no h projeto aberto.",
        fileParameter.getPath());
      return null;
    }

    if (!project.getId().equals(projectId)) {
      log2Console(
        true,
        "[ERROR] No foi posivel recuperar o arquivo %s pois ele pertence ao projeto %s e no ao projeto %s aberto.",
        fileParameter.getPath(), projectId, project.getId());
      return null;
    }

    final ClientProjectFile file =
      GetFileTask.runTask(project, fileParameter.getPathAsArray());

    log2Console(null == file,
      "[ERROR] No foi possvel recuperar o arquivo %s.", fileParameter
      .getPath());

    return file;
  }

  /**
   * Retorna o configurador do comando apresentado por essa viso.
   * 
   * @return configurator O configurador do comando apresentado por essa viso.
   */
  public AlgorithmConfigurator getConfigurator() {
    return configurator;
  }

  /**
   * <p>
   * Escreve no stdout uma dada mensagem caso a condio dada seja verdadeira e
   * o log desta classe estiver habilitado.<br>
   * Essa mensagem ser pr-fixada pela data e pelo nome desta classe -
   * "[&lt;data&gt;] [AbstractAlgorithmCommandView] <mensagem>".<br>
   * Para habilitar o log basta a propriedade log.enable ter o valor
   * <code>true</code>
   * </p>
   * <p>
   * Este mtodo foi criado temporariamente para auxiliar na busca de
   * informaes com intuito de resolver o issue [CSBASE-1042]
   * </p>
   * 
   * @param condition condio para a mensagem ser enviada para o stdout.
   * @param format template da mensagem a ser escrita no stdout.
   * @param args argumentos da mensagem a ser escrita no stdout.
   */
  private static void log2Console(boolean condition, String format,
    Object... args) {
    if (condition && logEnabled) {
      String errorMessage = String.format(format, args);
      System.out.printf("[%s] [AbstractAlgorithmCommandView] %s\n",
        new SimpleDateFormat().format(new Date()), errorMessage);
    }
  }

  /**
   * Aba que apresenta os parmetros do configurador do comando executado.
   * 
   * @author Tecgraf
   */
  protected abstract class ConfigurationReportTab extends Tab {

    /**
     * {@inheritDoc}
     */
    @Override
    protected void setSelected(boolean selected) {
      super.setSelected(selected);
      if (selected) {
        updateStatusBar();
      }
    }
  }

  /**
   * Aba que apresenta o log de execuo do comando
   */
  private class LogTab extends Tab {

    /**
     * Ttulo da aba
     */
    private final String title;

    /**
     * Viso atual da aba
     */
    private AbstractView view;

    /**
     * Construtor
     * 
     * @param log Parmetro que guarda o arquivo de log
     */
    LogTab(final FileParameterValue log) {

      setLayout(new GridBagLayout());
      title = createTitle(log);

      ClientProjectFile file =
        getClientProjectFile(command.getProjectId(), log);
      if (null == file) {
        showOpenLogView(log);
      }
      else {
        showLogPanelView(log, file);
      }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getTitle() {
      return title;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void setSelected(boolean selected) {
      view.setSelected(selected);
    }

    /**
     * Cria uma viso para a tab com um painel de log para o arquivo passado
     * como parmetro.
     * 
     * @param log
     * 
     * @param file
     */
    public void showLogPanelView(final FileParameterValue log,
      final ClientProjectFile file) {
      LogPanelView logPanelView = new LogPanelView(this, log, file);
      showView(logPanelView, new GBC(0, 0).both());
    }

    /**
     * Cria uma viso para a tab com um boto centralizado que permite que o
     * usurio force a abertura do painal de log do arquivo.
     * 
     * @param log
     */
    private void showOpenLogView(final FileParameterValue log) {
      OpenLogView openLogView = new OpenLogView(this, log);
      showView(openLogView, null);
    }

    /**
     * Mostra uma determinada viso na aba.
     * 
     * @param view Viso a ser mostrada.
     * @param obj Objeto que contm configurao de layout para a viso.
     */
    private void showView(AbstractView view, Object obj) {
      setVisible(false);

      removeAll();
      add(view, obj);
      this.view = view;
      if (null != selected && selected.equals(this)) {
        this.view.setSelected(true);
      }

      setVisible(true);
    }

    /**
     * Monta o ttulo da aba, indicando o caminho do arquivo de log da aba.
     * 
     * @param log Arquivo de log da aba.
     * @return ttulo da aba
     */
    private String createTitle(final FileParameterValue log) {
      String[] logPath = log.getPathAsArray();
      String logLabel = logPath[logPath.length - 1];
      return String.format(LNG
        .get("AbstractAlgorithmCommandView.tab.log.title.template"), logLabel);
    }
  }

  /**
   * Indica se essa viso possui uma aba para logs adicionais (configurvel por
   * sistema).
   * 
   * @return verdadeiro se a viso possui uma aba para logs adicionais ou falso,
   *         caso contrrio.
   */
  private boolean hasAdditionalLogsTab() {
    ConfigurationManager cnfManager = ConfigurationManager.getInstance();
    final boolean defaultValue = false;
    if (cnfManager == null)
      return defaultValue;
    try {
      Class<AdditionalLogsTab> propClass = AdditionalLogsTab.class;
      Configuration cnf = cnfManager.getConfiguration(propClass);
      String propName = "enabled";
      Boolean isEnabled =
        cnf.getOptionalBooleanProperty(propName, defaultValue);
      return isEnabled;
    }
    catch (Exception e) {
      return defaultValue;
    }
  }

  /**
   * Viso de uma aba.
   */
  abstract class AbstractView extends JPanel {
    /**
     * @param selected
     * @see Tab#setSelected(boolean)
     */
    public abstract void setSelected(boolean selected);
  }

  /**
   * Viso que apresenta um painel de visualizao do log de execuo do
   * comando.
   */
  private class LogPanelView extends AbstractView {

    /**
     * Painel de visualizao do log.
     */
    private final LogPanel logPanel;
    /**
     * Indica se o painel no foi selecionado ainda.
     */
    private final AtomicBoolean wasNeverSelected;
    /**
     * Indica se o comando j entrou em execuo.
     */
    private final AtomicBoolean wasRunningStatus;
    /**
     * Indica se o evento de atualizao de status deve ser ignorado.
     */
    private final AtomicBoolean ignoreReloaderStatusChangedEvent;

    /**
     * Construtor
     * 
     * @param logTab Painel de visualizao do log.
     * @param log Parmetro que determina o arquivo de log.
     * @param file Arquivo com o conted do log.
     */
    LogPanelView(final LogTab logTab, final FileParameterValue log,
      final ClientProjectFile file) {

      final int pageSizeKb = LogPanel.DEFAULT_PAGE_SIZE_KB;

      logPanel =
        LogPanel.createLogPanelWithToolBar(LogPanel.RELOAD | LogPanel.PAGING,
          pageSizeKb);

      logPanel.addThrowableEventListener(new EventListener<ThrowableEvent>() {
        @Override
        public void eventFired(ThrowableEvent event) {
          statusBar.setError(event.getMessage());
        }
      });

      logPanel.addFileEventListener(new EventListener<FileEvent>() {
        public void eventFired(FileEvent event) {
          if (FileEvent.Type.NOT_FOUND == event.getType()) {
            logPanel.getReloader().stop();
            logPanel.closeFile();
            logTab.showOpenLogView(log);
          }
          updateStatusBar();
        }
      });

      wasNeverSelected = new AtomicBoolean(true);

      ignoreReloaderStatusChangedEvent = new AtomicBoolean(false);

      /** Armazena o estado do boto do reload. */
      wasRunningStatus = new AtomicBoolean(logPanel.getReloader().isRunning());

      logPanel.getReloader().addStatusChangedEventListener(
        new EventListener<StatusChangedEvent>() {
        public void eventFired(StatusChangedEvent event) {
          if (!ignoreReloaderStatusChangedEvent.compareAndSet(true, false)) {
            wasRunningStatus.set(event.isRunning());
          }
        }
      });

      CommandsCache.getInstance().addEventListener(
        new AbstractCommandUpdatedEventListener(getCommand().getProjectId(),
        getCommand().getId()) {
        @Override
        protected void eventFired(Type type, CommandInfo command) {
          /*
           * Caso o commando tenha terminado sua execuo (independente do
           * porqu),  preciso parar de recarregar o arquivo de log
           */
          switch (type) {
            case end:
            case success:
            case error:
            case failed:
            case killed:
            case lost:
            case no_code:
            case removed: {
              logPanel.getReloader().stop();
            }
          }
        }
      });

      setLayout(new GridBagLayout());
      add(logPanel, new GBC(0, 0).both());
      logPanel.openFile(file);
    }

    /**
     * Gerencia a recarga do log. Se esta tab no estiver selecionada, no faz
     * sentido a recarga ficar ligada.<br>
     * 
     * @see csbase.client.algorithms.commands.view.AbstractAlgorithmCommandView.AbstractView#setSelected(boolean)
     */
    @Override
    public void setSelected(boolean selected) {
      /*
       * Se a tab de log est sendo selecionada pela primeira vez e o comando
       * ainda no terminou, a recarga de log deve estar ligada. Se este no for
       * o caso, a recarga deve ficar desligada se esta tab estiver sendo
       * desselecionada ou deve ficar no ltimo estado em que o usurio a
       * deixou. <br>
       * 
       * O ignoreReloaderStatusChangedEvent permite que ao sair desta aba, o
       * reloader seja desligado, mas que o estado escolhido pelo usurio
       * atravs do boto de reload ou pelo fato do comando estar em execuo,
       * seja armazenado no wasRunningStatus. Assim, os eventos gerados pela
       * ativao ou desativao do reload a partir da seleo de abas so
       * ignorados e os demais - usurio escolhendo o estado atravs do boto de
       * recarga - so pegos e o estado da recarga  salvo. Porm, como a lgica
       * da 1a seleo  diferente, este evento no deve ser ignorado.
       */

      boolean ignoreReloader = !wasNeverSelected.get();

      if (wasNeverSelected.compareAndSet(true, !selected)
        && !command.getStatus().equals(CommandStatus.FINISHED)) {
        // O if-elseif desta forma pois fica mais legvel assim.
      }
      else if (!wasRunningStatus.get()) {
        // A recarga estava desligada, ento deve continuar assim.
        return;
      }

      ignoreReloaderStatusChangedEvent.set(ignoreReloader);

      if (selected) {
        logPanel.getReloader().start();
      }
      else {
        logPanel.getReloader().stop();
      }
    }
  }

  /**
   * Viso que permite a abertura de um arquivo de log.
   */
  private class OpenLogView extends AbstractView {

    /**
     * Construtor
     * 
     * @param logTab A aba de log.
     * @param log O parmetro que armazena o arquivo de log.
     */
    OpenLogView(final LogTab logTab, final FileParameterValue log) {

      final Object projectId = command.getProjectId();
      /**
       * Caso o arquivo no tenha sido encontrado, talvez por ainda no ter sido
       * criado, devemos por um boto para que o usurio consiga forar o
       * sistema a abri-lo de novo.
       */
      class OpenAction extends AbstractAction {
        OpenAction() {
          super(LNG
            .get("AbstractAlgorithmCommandView.tab.log.button.reload.label"),
            ApplicationImages.ICON_REFRESH_24);
        }

        public void actionPerformed(ActionEvent e) {
          ClientProjectFile file = getClientProjectFile(projectId, log);
          if (null == file) {
            String warningMsg =
              LNG
                .get("AbstractAlgorithmCommandView.tab.log.file.notFound.dialog");
            String[] buttons = new String[] { LogPanel.getString("button.ok") };
            JOptionPane.showOptionDialog(AbstractAlgorithmCommandView.this,
              warningMsg, AbstractAlgorithmCommandView.this.getTitle(),
              JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE, null,
              buttons, buttons[0]);
          }
          else {
            logTab.showLogPanelView(log, file);
            updateStatusBar();
          }
        }
      }
      final JButton openLogBtn = new JButton(new OpenAction());
      openLogBtn.setToolTipText(LNG
        .get("AbstractAlgorithmCommandView.tab.log.button.reload.tooltip"));

      setLayout(new BorderLayout());
      add(openLogBtn, BorderLayout.CENTER);
    }

    /**
     * @see csbase.client.algorithms.commands.view.AbstractAlgorithmCommandView.AbstractView#setSelected(boolean)
     */
    @Override
    public void setSelected(boolean selected) {
      if (selected) {
        statusBar.setWarning(LNG
          .get("AbstractAlgorithmCommandView.tab.log.file.notFound.statusbar"));
      }
      else {
        updateStatusBar();
      }
    }
  }
}
