/*
 * $Id: FindAdditionalLogsTask.java 100749 2010-01-14 19:07:56Z clinio $
 */

package csbase.client.algorithms.commands.view;

import java.io.IOException;
import java.rmi.RemoteException;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.RemoteTask;
import csbase.logic.ClientProjectFile;
import csbase.logic.CommandInfo;
import csbase.logic.CommonClientProject;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.FileParameterValue;

/**
 * Task que busca os arquivos de log adicionais do comando.
 */
class FindAdditionalLogsTask extends RemoteTask<Set<ClientProjectFile>> {

  /**
   * Informaes sobre o comando cujo log deve ser apresentado.
   */
  protected CommandInfo commandInfo;

  /**
   * Filtra um array de arquivos e retorna um subconjunto com os arquivos de log
   * adicionais. A filtragem  feita de acordo com o padro especificado para o
   * sistema e ignorando os arquivos de log padro.
   * 
   * @param files O array de arquivos a ser filtrado.
   * @return O conjunto de arquivos de log adicionais.
   * @throws RemoteException em caso de falha na chamada remota.
   */
  private Set<ClientProjectFile> findAdditionalLogFiles(
    final ClientProjectFile[] files) throws RemoteException {
    final Set<ClientProjectFile> additionalLogFiles =
      new HashSet<ClientProjectFile>();
    for (final ClientProjectFile file : files) {
      if (!file.isDirectory() && !isStandardLogFile(file)) {
        if (matchesPatterns(file)) {
          additionalLogFiles.add(file);
        }
      }
    }
    return additionalLogFiles;
  }

  /**
   * Retorna o caminho para o diretrio onde os logs adicionais esto
   * armazenados.
   * 
   * @return o caminho para o diretrio onde os logs adicionais esto
   *         armazenados
   * @throws RemoteException em caso de falha na chamada remota.
   */
  private ClientProjectFile getAdditionalLogsDir() throws RemoteException {
    ClientProjectFile logsDir = null;
    final AlgorithmConfigurator configurator = commandInfo.getConfigurator();
    if (configurator != null) {
      final FileParameterValue additionalLogsDirectory =
        configurator.getAdditionalLogsDirectory();
      if (additionalLogsDirectory != null) {
        final String[] logsDirPath = additionalLogsDirectory.getPathAsArray();
        final DesktopFrame desktop = DesktopFrame.getInstance();
        final CommonClientProject project = desktop.getProject();
        logsDir = project.getFile(logsDirPath);
      }
    }
    return logsDir;
  }

  /**
   * Retorna o padro de nome de arquivos de log adicionais especificado para o
   * sistema.
   * 
   * @return O padro de nome de arquivos de log adicionais.
   */
  private List<String> getLogFilePattern() {
    final ConfigurationManager cnfManager = ConfigurationManager.getInstance();
    if (cnfManager == null) {
      return null;
    }
    try {
      final Class<AdditionalLogsTab> propClass = AdditionalLogsTab.class;
      final Configuration cnf = cnfManager.getConfiguration(propClass);
      final String propName = "log.file.pattern";
      final List<String> pattern = cnf.getOptionalListProperty(propName);
      return pattern;
    }
    catch (final Exception e) {
      return null;
    }
  }

  /**
   * Testa se um determinado arquivo pertence a um determinado conjunto de
   * arquivos padro.
   * 
   * @param file O arquivo a ser testado.
   * @param standardFiles Conjunto de arquivos padro.
   * @return Verdadeiro se o arquivo pertence ao conjunto de arquivos padro ou
   *         falso, caso contrrio.
   */
  private boolean isStandardFile(final ClientProjectFile file,
    final Set<FileParameterValue> standardFiles) {
    for (final FileParameterValue standardFile : standardFiles) {
      return file.getStringPath().equals(standardFile.getPath());
    }
    return false;
  }

  /**
   * Testa se um determinado arquivo  um log padro (arquivo padro de sada do
   * algortimo ou de armazenamento do cdigo de sada).
   * 
   * @param file O arquivo a ser testado.
   * @return Verdadeiro se o arquivo  um log padro ou falso, caso contrrio.
   * @throws RemoteException em caso de falha na chamada remota.
   */
  private boolean isStandardLogFile(final ClientProjectFile file)
    throws RemoteException {

    final AlgorithmConfigurator configurator = commandInfo.getConfigurator();
    // Testa se  arquivo padro de sada
    final Set<FileParameterValue> standardOutputs =
      configurator.getStandardOutputFiles();
    if (isStandardFile(file, standardOutputs)) {
      return true;
    }

    // Testa se  arquivo de armazenamento de cdigo de sada
    final Set<FileParameterValue> exitCodeFile =
      configurator.getExitCodeLogFiles();
    if (isStandardFile(file, exitCodeFile)) {
      return true;
    }
    return false;
  }

  /**
   * Testa se um determinado arquivo satisfaz algum dos padres de nomenclatura
   * para arquivos de log adicionais definidos para este sistema. Caso positivo,
   * o arquivo pode ser considerado um log adicional.
   * 
   * @param file O arquivo a ser testado.
   * @return Verdadeiro se o arquivo pode ser considerado um log adicional de
   *         acordo com o padro de nomenclatura definido ou falso, caso
   *         contrrio.
   */
  private boolean matchesPatterns(final ClientProjectFile file) {
    final List<String> logFilePatterns = getLogFilePattern();
    if (logFilePatterns != null) {
      for (final String pattern : logFilePatterns) {
        if (file.getName().matches(pattern)) {
          return true;
        }
      }
    }
    return false;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void performTask() throws IOException {
    final ClientProjectFile additionalLogsDir = getAdditionalLogsDir();
    if (additionalLogsDir != null) {
      final ClientProjectFile[] children = additionalLogsDir.getChildren();
      final Set<ClientProjectFile> additionalLogFiles =
        findAdditionalLogFiles(children);
      setResult(additionalLogFiles);
    }
  }

  /**
   * Construtor
   * 
   * @param commandInfo Informaes sobre o comando cujo log deve ser
   *        apresentado.
   */
  FindAdditionalLogsTask(final CommandInfo commandInfo) {
    this.commandInfo = commandInfo;
  }
}
