package csbase.client.algorithms.commands.view;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StringWriter;

import javax.swing.JDialog;
import javax.swing.JFrame;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.GBC;
import csbase.algorithms.flows.Workspace;
import csbase.algorithms.flows.actions.Action;
import csbase.algorithms.flows.configurator.FlowAlgorithmConfiguratorView;
import csbase.algorithms.flows.filters.AddNodePopupActionFilter;
import csbase.algorithms.flows.filters.PopupFilter;
import csbase.algorithms.flows.filters.WorkspaceFilter;
import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.graph.GraphNode;
import csbase.algorithms.flows.graph.GraphNodeDecoration.DecorationType;
import csbase.algorithms.flows.graph.actions.GraphElementAction;
import csbase.algorithms.flows.messages.PickNodeMessage;
import csbase.client.algorithms.AlgorithmConfiguratorFactory;
import csbase.client.algorithms.commands.cache.CommandsCache;
import csbase.client.algorithms.commands.cache.events.AbstractCommandUpdatedEventListener;
import csbase.client.algorithms.commands.cache.events.CommandUpdatedEvent.Type;
import csbase.client.applications.ApplicationImages;
import csbase.client.desktop.DesktopComponentFrame;
import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.RemoteTask;
import csbase.client.kernel.ClientException;
import csbase.client.project.tasks.GetFileTask;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.ClientProjectFile;
import csbase.logic.ClientProjectFileInputStream;
import csbase.logic.CommandInfo;
import csbase.logic.CommandStatus;
import csbase.logic.CommonClientProject;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.FileParameterValue;
import csbase.logic.algorithms.flows.Flow;
import csbase.logic.algorithms.flows.FlowNode;
import csbase.logic.algorithms.flows.configurator.FlowAlgorithmConfigurator;

/**
 * Dilogo de visualizao de comandos de fluxo.
 * 
 * @author Tecgraf / PUC-Rio
 */
final class FlowAlgorithmCommandView extends AbstractAlgorithmCommandView {

  /**
   * Painel de fluxo
   */
  private FlowAlgorithmConfiguratorView configuratorView;

  /**
   * Tab preferencial
   */
  private TabType preferredTab;

  /**
   * @param index identificador da view. Usado para garantir a unicidade.
   * @param owner A janela pai deste dilogo. No pode ser nula e tem que ser
   *        uma instncia de {@link JFrame} ou {@link JDialog}.
   * @param command O comando (No aceita {@code null}).
   * @param configurator O configurador a ter seus parmetros apresentados.
   *        Deve-se ignorar o configurador do comando pois pode ser diferente
   *        deste. Isso ocorrerira no caso de estar representando um n de um
   *        comando de fluxo (No aceita {@code null}).
   * 
   * @throws ClientException Erro ao obter informaes sobre o comando.
   */
  public FlowAlgorithmCommandView(final Object index,
    final DesktopComponentFrame owner, final CommandInfo command,
    final AlgorithmConfigurator configurator) throws ClientException {
    super(index, owner, command, configurator);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void initialize(AlgorithmConfigurator configurator)
    throws ClientException {

    AlgorithmConfiguratorFactory configuratorFactory =
      AlgorithmConfiguratorFactory.getInstance();
    configuratorView =
      (FlowAlgorithmConfiguratorView) configuratorFactory.createReportView(
      owner, configurator);
  }

  /**
   * Mostra essa janela dando preferncia para abrir com a tab escolhida.
   * 
   * @param tab Qual a tab gostaria que estivesse selecionada quando esta janela
   *        for criada. No  garantido que esta janela abra com a tab escolhida
   *        selecionada, pois aquela tab pode nem existir caso seja a tab de
   *        log.
   */
  @Override
  public void show(final TabType tab) {
    this.preferredTab = tab;
    super.show(tab);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected ConfigurationReportTab createConfigurationReportTab(
    final AlgorithmConfigurator configurator) throws ClientException {
    return new FlowConfiguratorReportTab(configurator);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected Tab createLogTab(final AlgorithmConfigurator configurator,
    final Dimension preferredSize) {
    return new FlowConsolidatedLogTab(getCommand());
  }

  /**
   * @author Tecgraf/PUC-Rio
   */
  private final class FlowConfiguratorReportTab extends ConfigurationReportTab {

    /**
     * Recebe notificaes sobre atualizaces feitas no comando.
     */
    private AbstractCommandUpdatedEventListener commandListener;

    /**
     * Construtor
     * 
     * @param configurator configurador
     * @throws ClientException se houver problemas
     */
    FlowConfiguratorReportTab(final AlgorithmConfigurator configurator)
      throws ClientException {

      addCommandListener();
      createFilters(configuratorView.getWorkspace());

      setLayout(new GridLayout());
      add(configuratorView.getMainComponent(), new GBC(0, 0).both());
      addCommandStatusDecoration(configuratorView.getGraph());
    }

    /**
     * Adiciona um listener para o comando a ser visualizado, caso seja
     * necessrio.
     */
    private void addCommandListener() {
      CommandInfo command = getCommand();
      if (command != null && command.getStatus() != CommandStatus.FINISHED) {
        this.commandListener = createCommandListener(command);
        CommandsCache.getInstance().addEventListener(commandListener);
      }
    }

    /**
     * Cria um listener para o comando a ser visualizado, para que a viso possa
     * ser atualizada.
     * 
     * @param command O comando a ser visualizado.
     * @return um listener para o comando.
     */
    private AbstractCommandUpdatedEventListener createCommandListener(
      CommandInfo command) {
      return new AbstractCommandUpdatedEventListener(command.getProjectId(),
        command.getId()) {
        @Override
        public void eventFired(Type type, CommandInfo cmd) {
          try {
            addCommandStatusDecoration(configuratorView.getGraph());
            if (cmd.getStatus() == CommandStatus.FINISHED) {
              if (commandListener != null) {
                CommandsCache.getInstance()
                  .removeEventListener(commandListener);
              }
            }
          }
          catch (ClientException e) {
            StandardErrorDialogs.showErrorDialog(FlowAlgorithmCommandView.this,
              e);
          }
        }
      };
    }

    /**
     * Adiciona as decoraes que representam o estado do comando s caixas do
     * fluxo
     * 
     * @param graph O grafo que contm as caixas do fluxo.
     * 
     * @throws ClientException em caso de falha.
     */
    private void addCommandStatusDecoration(final Graph graph)
      throws ClientException {

      RemoteTask<Void> exitCodeTask = new RemoteTask<Void>() {
        @Override
        protected void performTask() throws Exception {
          FlowAlgorithmConfigurator configurator =
            (FlowAlgorithmConfigurator) getConfigurator();
          Flow flow = configurator.getFlow();
          ClientProjectFileInputStream inputStream = null;
          BufferedReader bufferedReader = null;
          for (FlowNode node : flow.getNodes()) {
            DecorationType decoration = DecorationType.BLANK;
            if (node.hasExitCode() && node.getExitCodeLogFile() != null) {
              decoration = DecorationType.EXECUTION_UNKNOWN;
              try {
                ClientProjectFile clientProjectFile =
                  getClientProjectFile(node.getExitCodeLogFile());
                if (clientProjectFile != null) {
                  inputStream =
                    new ClientProjectFileInputStream(clientProjectFile, 0);
                  bufferedReader =
                    new BufferedReader(new InputStreamReader(inputStream));
                  String line = bufferedReader.readLine();
                  if (line != null && !line.isEmpty()) {
                    int exitCode = Integer.parseInt(line);
                    if (exitCode != 0) {
                      decoration = DecorationType.EXECUTION_ERROR;
                    }
                    else {
                      decoration = DecorationType.EXECUTION_SUCCESS;
                    }
                  }
                }
              }
              finally {
                if (bufferedReader != null) {
                  bufferedReader.close();
                }
                if (inputStream != null) {
                  inputStream.close();
                }
              }
            }
            GraphNode graphNode = graph.getNode(node.getId());
            graphNode.addDecoration(decoration);
          }
        }
      };

      if (getCommand() != null) {
        if (getCommand().getStatus() == CommandStatus.FINISHED) {
          boolean successful =
            exitCodeTask
              .execute(
                owner,
                LNG
                  .get("csbase.client.algorithms.view.flow.FlowAlgorithmReportPanel.reading_exit_code_title"),
                LNG
                  .get("csbase.client.algorithms.view.flow.FlowAlgorithmReportPanel.reading_exit_code"));

          if (!successful) {
            throw new ClientException(exitCodeTask.getError());
          }
        }
        else {
          DecorationType decoration;
          switch (getCommand().getStatus()) {
            case SYSTEM_FAILURE:
              decoration = DecorationType.SYSTEM_FAILURE;
              break;
            case SCHEDULED:
              decoration = DecorationType.SCHEDULED;
              break;
            case INIT:
            case UPLOADING:
            case EXECUTING:
            case DOWNLOADING:
              decoration = DecorationType.EXECUTION;
              break;
            default:
              decoration = DecorationType.BLANK;
          }
          
          for (GraphNode graphNode : graph.getNodeCollection()) {
            if (!graphNode.isBypassed()) {              
              graphNode.addDecoration(decoration);
            }
          }
        }
      }
      else {
        throw new ClientException("No foi possvel encontrar comando!");
      }
    }

    /**
     * Retorna o {@link ClientProjectFile} que corresponde ao
     * {@link FileParameterValue} que armazena o arquivo de cdido de sada do
     * fluxo.
     * 
     * @param exitCodeLog Parmetro que armazena o arquivo de cdigo de sada do
     *        fluxo.
     * @return O {@link ClientProjectFile} correspondente.
     */
    private ClientProjectFile getClientProjectFile(
      final FileParameterValue exitCodeLog) {
      CommonClientProject project = DesktopFrame.getInstance().getProject();
      final ClientProjectFile file =
        GetFileTask.runTask(project, exitCodeLog.getPathAsArray());
      return file;
    }

    /**
     * @see Tab#getTitle()
     */
    @Override
    public String getTitle() {
      return LNG.get("FlowAlgorithmCommandView.tab.params.title");
    }

    /**
     * Cria os filtros.
     * 
     * @param workspace workspace
     */
    private void createFilters(final Workspace workspace) {

      // Cria o menu popup.
      new AddNodePopupActionFilter(workspace) {
        @Override
        protected Action createAction(GraphNode graphNode, Point2D point) {
          return new ShowAlgorithmCommandViewPopupAction(graphNode);
        }
      }.attach();
      new PopupFilter(workspace).attach();
      // Cria a ao que ser chamada quando houver um duplo clique em algum n.
      new ShowAlgorithmCommandViewDoubleClickAction(workspace).attach();
    }

    /**
     * Ao de que exibe a viso do comando de um n de fluxo de algoritmos.
     * Nenhum log ser mostrado nesta viso pois os arquivos de log foram
     * persistidos como sendo do fluxo e no dos ns.
     */
    private class ShowAlgorithmCommandViewDoubleClickAction extends
      WorkspaceFilter {

      /**
       * @param workspace A rea de trabalho (No aceita {@code null}).
       */
      public ShowAlgorithmCommandViewDoubleClickAction(final Workspace workspace) {
        super(workspace);
      }

      /**
       * @see tecgraf.javautils.vix.Filter#callbackButton(java.awt.geom.Point2D,
       *      java.awt.event.MouseEvent)
       */
      @Override
      public void callbackButton(Point2D pt, MouseEvent ev) {
        if (ev.getButton() == MouseEvent.BUTTON1
          && ev.getID() == MouseEvent.MOUSE_CLICKED && ev.getClickCount() == 2) {
          PickNodeMessage pickNodeMessage = new PickNodeMessage(pt);
          pickNodeMessage.sendVO(this);
          GraphNode node = pickNodeMessage.getNode();
          if (node != null) {
            try {
              AlgorithmCommandViewFactory
                .showView(FlowAlgorithmCommandView.this, getCommand(), node,
                  preferredTab);
            }
            catch (Exception ex) {
              StandardErrorDialogs.showErrorDialog(
                FlowAlgorithmCommandView.this, ex);
            }
          }
        }
        super.callbackButton(pt, ev);
      }
    }

    /**
     * Ao de menu popup que exibe a viso do comando de um n de fluxo de
     * algoritmos.
     */
    private final class ShowAlgorithmCommandViewPopupAction extends
      GraphElementAction {

      /**
       * Cria a ao.
       * 
       * @param node O n (No aceita {@code null}).
       */
      public ShowAlgorithmCommandViewPopupAction(final GraphNode node) {
        super(node,
          "FlowAlgorithmCommandView.tab.params.action.show.node.params",
          ApplicationImages.ICON_INFORMATION_16);
      }

      /**
       * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
       */
      public void actionPerformed(ActionEvent e) {
        GraphNode node = (GraphNode) getElement();
        if (node != null) {
          try {
            AlgorithmCommandViewFactory.showView(FlowAlgorithmCommandView.this,
              getCommand(), node, preferredTab);
          }
          catch (Exception ex) {
            StandardErrorDialogs.showErrorDialog(FlowAlgorithmCommandView.this,
              ex);
          }
        }
      }
    }
  }

  /**
   * A aba de log consolidado do fluxo.
   */
  class FlowConsolidatedLogTab extends AbstractConsolidatedLogTab {

    /**
     * Construtor.
     * 
     * @param command Informaes sobre o comando executado (fluxo).
     */
    FlowConsolidatedLogTab(CommandInfo command) {
      super(command);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void loadLog() {
      textArea.setText("");
      RemoteTask<String> task = new RemoteTask<String>() {

        @Override
        protected void performTask() throws IOException {
          StringWriter writer = new StringWriter();
          for (GraphNode node : configuratorView.getGraph().getNodeCollection()) {
            node.readStandardOutputFile(writer);
          }
          setResult(writer.getBuffer().toString());
        }

      };
      if (task.execute(getOwner(), getTitle(), LNG
        .get("FlowAlgorithmCommandView.loading.log"))) {
        textArea.setText(task.getResult());
      }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected String getTitle() {
      return LNG.get("FlowAlgorithmCommandView.tab.log.title");
    }
  }

}
