/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.client.algorithms.parameters.validators;

import csbase.client.desktop.DesktopFrame;
import csbase.client.project.tasks.GetChildFromNameTask;
import csbase.client.util.ClientUtilities;
import csbase.logic.ClientProjectFile;
import csbase.logic.algorithms.FileParameterValue;
import csbase.logic.algorithms.parameters.FileParameter;
import csbase.logic.algorithms.parameters.SimpleParameter;

/**
 * Validador do {@link FileParameter Parmetro do Tipo Arquivo}.
 * 
 * @author lmoreira
 */
public abstract class FileParameterValidator extends
  SimpleParameterValidator<FileParameterValue> {

  /**
   * Indica se o arquivo precisa existir.
   */
  private boolean mustExists;

  /**
   * Cria um validador de arquivo.
   * 
   * @param optional Indica se o valor  opcional {@code true} ou obrigatrio
   *        {@code false}.
   * @param mustExists Indica se o arquivo precisa existir.
   */
  protected FileParameterValidator(boolean optional, boolean mustExists) {
    super(optional);
    this.mustExists = mustExists;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public final ValidationResult validateValue(SimpleParameter<?> parameter,
    FileParameterValue value) {
    if (!parameter.isVisible()) {
      return new ValidationResult();
    }
    if (!parameter.isEnabled()) {
      return new ValidationResult();
    }
    FileParameter fileParameter = (FileParameter) parameter;
    if (fileParameter.hasLink()) {
      return new ValidationResult();
    }
    ValidationResult result = super.validateValue(parameter, value);
    if (!result.isWellSucceded()) {
      return result;
    }
    if (value != null) {
      return validateFile((FileParameter) parameter, value);
    }
    return new ValidationResult();
  }

  /**
   * Valida um arquivo selecionado pelo usurio.
   * 
   * @param parameter O parmetro (No aceita {@code null}).
   * @param file O arquivo (No aceita {@code null}).
   * 
   * @return .
   */
  private ValidationResult validateFile(FileParameter parameter,
    FileParameterValue file) {
    String[] path = file.getPathAsArray();
    for (String component : path) {
      if (!ClientUtilities.isValidFileName(component)) {
        return new ValidationResult(String.format(getMessage(
          FileParameterValidator.class, "invalid_file_name"), component));
      }
    }
    ClientProjectFile clientProjectFile =
      (ClientProjectFile) DesktopFrame.getInstance().getProject().getTree();
    for (int i = 0; i < path.length - 1; i++) {
      String fileName = path[i];
      if (!fileName.equals(".")) {
        clientProjectFile =
          GetChildFromNameTask.runTask(clientProjectFile, fileName);
        if (clientProjectFile == null) {
          String errorMessage =
            String.format(getMessage(FileParameterValidator.class,
              "file_not_found"), fileName, parameter.getLabel());
          return new ValidationResult(errorMessage);
        }
        if (!clientProjectFile.isDirectory()) {
          String errorMessage =
            String.format(getMessage(FileParameterValidator.class,
              "not_directory"), file.getPath());
          return new ValidationResult(errorMessage);
        }
      }
    }
    String fileName = path[path.length - 1];
    if (!fileName.equals(".")) {
      clientProjectFile =
        GetChildFromNameTask.runTask(clientProjectFile, fileName);
    }
    switch (parameter.getMode()) {
      case DIRECTORY:
        if (clientProjectFile == null) {
          String errorMessage =
            String.format(getMessage(FileParameterValidator.class,
              "file_not_found"), fileName, parameter.getLabel());
          return new ValidationResult(errorMessage);
        }
        if (!clientProjectFile.isDirectory()) {
          String errorMessage =
            String.format(getMessage(FileParameterValidator.class,
              "not_directory"), file.getPath());
          return new ValidationResult(errorMessage);
        }
        return new ValidationResult();
      case REGULAR_FILE:
        if (clientProjectFile == null) {
          if (mustExists) {
            String errorMessage =
              String.format(getMessage(FileParameterValidator.class,
                "file_not_found"), fileName, parameter.getLabel());
            return new ValidationResult(errorMessage);
          }
          return new ValidationResult();
        }
        if (clientProjectFile.isDirectory()) {
          String errorMessage =
            String.format(getMessage(FileParameterValidator.class, "not_file"),
              file.getPath());
          return new ValidationResult(errorMessage);
        }
        return new ValidationResult();
      case DIRECTORY_AND_REGULAR_FILE:
        if (clientProjectFile == null) {
          if (mustExists) {
            String errorMessage =
              String.format(getMessage(FileParameterValidator.class,
                "file_not_found"), fileName, parameter.getLabel());
            return new ValidationResult(errorMessage);
          }
        }
        return new ValidationResult();
      default:
        throw new IllegalStateException(String.format(
          "Modo desconhecido (%0$s).", parameter.getMode()));
    }
  }
}