/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.client.algorithms.parameters.validators;

import java.util.List;

import csbase.client.desktop.DesktopFrame;
import csbase.client.project.tasks.GetChildFromNameTask;
import csbase.client.util.ClientUtilities;
import csbase.logic.ClientProjectFile;
import csbase.logic.algorithms.FileParameterValue;
import csbase.logic.algorithms.parameters.InputFileListParameter;
import csbase.logic.algorithms.parameters.SimpleParameter;

/**
 * Validador do {@link InputFileListParameter Parmetro do Tipo Lista de Arquivo
 * de Entrada}.
 * 
 * @author lmoreira
 */
public final class InputFileListParameterValidator extends
  SimpleParameterValidator<List<FileParameterValue>> {

  /**
   * Cria um validador de arquivo.
   * 
   * @param optional Indica se o valor  opcional {@code true} ou obrigatrio
   *        {@code false}.
   */
  public InputFileListParameterValidator(boolean optional) {
    super(optional);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public final ValidationResult validateValue(SimpleParameter<?> parameter,
    List<FileParameterValue> value) {
    if (!parameter.isVisible() || !parameter.isEnabled()) {
      return new ValidationResult();
    }
    ValidationResult result = super.validateValue(parameter, value);
    if (!result.isWellSucceded()) {
      return result;
    }
    if (value != null) {
      for (FileParameterValue file : value) {
        result = validateFile((InputFileListParameter) parameter, file);
        if (!result.isWellSucceded()) {
          return result;
        }
      }
    }
    return new ValidationResult();
  }

  /**
   * Valida um arquivo selecionado pelo usurio.
   * 
   * @param parameter O parmetro (No aceita {@code null}).
   * @param file O arquivo (No aceita {@code null}).
   * 
   * @return .
   */
  private ValidationResult validateFile(InputFileListParameter parameter,
    FileParameterValue file) {
    String[] path = file.getPathAsArray();
    for (String component : path) {
      if (!ClientUtilities.isValidFileName(component)) {
        return new ValidationResult(String.format(getMessage(
          InputFileListParameterValidator.class, "invalid_file_name"),
          component));
      }
    }
    ClientProjectFile clientProjectFile =
      (ClientProjectFile) DesktopFrame.getInstance().getProject().getTree();
    for (int i = 0; i < path.length - 1; i++) {
      String fileName = path[i];
      if (!fileName.equals(".")) {
        clientProjectFile =
          GetChildFromNameTask.runTask(clientProjectFile, fileName);
        if (clientProjectFile == null) {
          String errorMessage =
            String.format(getMessage(InputFileListParameterValidator.class,
              "file_not_found"), fileName, parameter.getLabel());
          return new ValidationResult(errorMessage);
        }
        if (!clientProjectFile.isDirectory()) {
          String errorMessage =
            String.format(getMessage(InputFileListParameterValidator.class,
              "not_directory"), file.getPath());
          return new ValidationResult(errorMessage);
        }
      }
    }
    String fileName = path[path.length - 1];
    if (!fileName.equals(".")) {
      clientProjectFile =
        GetChildFromNameTask.runTask(clientProjectFile, fileName);
    }
    switch (parameter.getMode()) {
      case DIRECTORY:
        if (clientProjectFile == null) {
          String errorMessage =
            String.format(getMessage(InputFileListParameterValidator.class,
              "file_not_found"), fileName, parameter.getLabel());
          return new ValidationResult(errorMessage);
        }
        if (!clientProjectFile.isDirectory()) {
          String errorMessage =
            String.format(getMessage(InputFileListParameterValidator.class,
              "not_directory"), file.getPath());
          return new ValidationResult(errorMessage);
        }
        return new ValidationResult();
      case REGULAR_FILE:
        if (clientProjectFile == null) {
          String errorMessage =
            String.format(getMessage(InputFileListParameterValidator.class,
              "file_not_found"), fileName, parameter.getLabel());
          return new ValidationResult(errorMessage);
        }
        if (!clientProjectFile.isRegularFile()) {
          String errorMessage =
            String.format(getMessage(InputFileListParameterValidator.class,
              "not_file"), file.getPath());
          return new ValidationResult(errorMessage);
        }
        return new ValidationResult();
      case DIRECTORY_AND_REGULAR_FILE:
        if (clientProjectFile == null) {
          String errorMessage =
            String.format(getMessage(InputFileListParameterValidator.class,
              "file_not_found"), fileName, parameter.getLabel());
          return new ValidationResult(errorMessage);
        }
        return new ValidationResult();
      default:
        throw new IllegalStateException(String.format(
          "Modo desconhecido (%0$s).", parameter.getMode()));
    }
  }
}
