/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.client.algorithms.parameters.validators;

import java.text.MessageFormat;

import csbase.logic.algorithms.parameters.SimpleParameter;

/**
 * Validador do {@link csbase.logic.algorithms.parameters.IntegerParameter
 * Parmetro do Tipo Inteiro}.
 * 
 * @author lmoreira
 */
public final class IntegerParameterValidator extends
  SimpleParameterValidator<Integer> {
  /**
   * O valor mximo considerado vlido.
   */
  private Integer maximum;
  /**
   * O valor mnimo considerado vlido.
   */
  private Integer minimum;

  /**
   * Cria um validador.
   * 
   * @param isOptional Indica se  o valor  opcional/obrigatrio.
   * @param maximum O mximo (Aceita {@code null}; se o mnimo existir ele tem
   *        que ser maior ou igual a ele).
   * @param minimum O mnimo (Aceita {@code null}; se o mximo existir ele tem
   *        que ser menor ou igual a ele).
   */
  public IntegerParameterValidator(boolean isOptional, Integer maximum,
    Integer minimum) {
    super(isOptional);
    setMaximum(maximum);
    setMinimum(minimum);
  }

  /**
   * Obtm o valor mximo.
   * 
   * @return O valor mximo ou {@code null} se ele no existir.
   */
  public Integer getMaximum() {
    return this.maximum;
  }

  /**
   * Obtm o valor mnimo.
   * 
   * @return O valor mnimo ou {@code null} se ele no existir.
   */
  public Integer getMinimum() {
    return this.minimum;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ValidationResult validateValue(SimpleParameter<?> parameter,
    Integer value) {
    ValidationResult validationResult = super.validateValue(parameter, value);
    if (!validationResult.isWellSucceded()) {
      return validationResult;
    }
    if (parameter.isEnabled() && parameter.isEnabled()) {
      if (value != null) {
        if (this.minimum != null) {
          if (value.compareTo(this.minimum) < 0) {
            return new ValidationResult(MessageFormat.format(getMessage(
              getClass(), "bellow_minimum"), parameter.getLabel(), value,
              this.minimum));
          }
        }
        if (this.maximum != null) {
          if (value.compareTo(this.maximum) > 0) {
            return new ValidationResult(MessageFormat.format(getMessage(
              getClass(), "above_maximum"), parameter.getLabel(), value,
              this.maximum));
          }
        }
      }
    }
    return new ValidationResult();
  }

  /**
   * Atribui o mximo a este parmetro.
   * 
   * @param maximum O mximo (Aceita {@code null}; se o mnimo existir ele tem
   *        que ser maior ou igual a ele).
   */
  public void setMaximum(Integer maximum) {
    if (this.minimum != null) {
      if (maximum != null && maximum.compareTo(this.minimum) < 0) {
        throw new IllegalArgumentException(
          "O valor mximo no pode ser menor do que o valor mnimo.");
      }
    }
    this.maximum = maximum;
  }

  /**
   * Atribui o mnimo a este parmetro.
   * 
   * @param minimum O mnimo (Aceita {@code null}; se o mximo existir ele tem
   *        que ser menor ou igual a ele).
   */
  public void setMinimum(Integer minimum) {
    if (this.maximum != null) {
      if (minimum != null && minimum.compareTo(this.maximum) > 0) {
        throw new IllegalArgumentException(
          "O valor mnimo no pode ser maior do que o valor mximo.");
      }
    }
    this.minimum = minimum;
  }
}
