package csbase.client.algorithms.view.simple;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Window;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import tecgraf.javautils.gui.GBC;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.algorithms.parameters.ParameterGroupView;
import csbase.client.algorithms.parameters.ParameterGroupViewListener;
import csbase.client.algorithms.parameters.ParameterView;
import csbase.client.algorithms.parameters.ParameterViewListener;
import csbase.client.algorithms.parameters.SimpleParameterView;
import csbase.client.algorithms.parameters.ParameterView.Mode;
import csbase.client.algorithms.parameters.validators.ValidationMode;
import csbase.client.algorithms.parameters.validators.ValidationResult;
import csbase.client.algorithms.view.AbstractAlgorithmConfiguratorPanel;
import csbase.client.kernel.ClientException;
import csbase.exception.algorithms.FormulaEvaluationException;
import csbase.logic.algorithms.parameters.ParameterGroup;
import csbase.logic.algorithms.parameters.SimpleAlgorithmConfigurator;
import csbase.logic.algorithms.parameters.SimpleParameter;
import csbase.logic.algorithms.parameters.ValidationExpression;

/**
 * Painel representando a viso dos parmetros de um configurador simples.
 * 
 * @author Tecgraf / PUC-Rio
 */
public class SimpleAlgorithmConfiguratorPanel extends
  AbstractAlgorithmConfiguratorPanel<SimpleAlgorithmConfigurator> {

  /**
   * Lista de vises de grupo
   */
  private List<ParameterGroupView> groupViews;

  /**
   * @param owner A janela pai da que detm este painel (No aceita {@code null}
   *        ).
   * @param configurator
   * @param mode Modo de visualizao. Pode ser
   *        {@link ParameterView.Mode#CONFIGURATION} ou
   *        {@link ParameterView.Mode#REPORT}.
   * 
   * @throws ClientException Erro ao obter informaes sobre o comando.
   */
  public SimpleAlgorithmConfiguratorPanel(Window owner,
    SimpleAlgorithmConfigurator configurator, ParameterView.Mode mode)
    throws ClientException {
    super(owner, configurator, mode);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected AbstractParametersPanel createParametersConfigurationPanel() {
    return new SimpleParametersConfigurationPanel();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected AbstractParametersPanel createParametersReportPanel() {
    return new SimpleParametersReportPanel();
  }

  /**
   * Adio de listaner em todos os grupos do painel.
   * 
   * @param listener o listener
   */
  public void addParameterGroupViewListener(ParameterGroupViewListener listener) {
    for (ParameterGroupView aGroupView : groupViews) {
      aGroupView.addParameterGroupViewListener(listener);
    }
  }

  /**
   * Painel interno que detm a viso dos parmetros.
   */
  private abstract class AbstractSimpleParametersPanel extends
    AbstractParametersPanel {

    /**
     * Construtor.
     * 
     * @param mode Modo de visualizao. Pode ser
     *        {@link ParameterView.Mode#CONFIGURATION} ou
     *        {@link ParameterView.Mode#REPORT}.
     */
    protected AbstractSimpleParametersPanel(final Mode mode) {
      super();
      executeBeforeInitialization();
      initialize(mode);
      executeAfterInitialization();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void setValidationMode(ValidationMode validationMode) {
      for (SimpleParameterView<?> parameterView : getParameterViews()) {
        parameterView.getParameterValidator().setMode(validationMode);
      }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public ValidationResult validate(boolean complain) {
      for (ParameterGroupView groupView : groupViews) {
        ValidationResult validation = groupView.validate(complain);
        if (!validation.isWellSucceded()) {
          return validation;
        }
      }
      SimpleAlgorithmConfigurator configurator = getConfigurator();
      for (ValidationExpression expression : configurator.getExpressions()) {
        try {
          if (!expression.evaluate()) {
            List<SimpleParameterView<?>> parameterViews =
              new LinkedList<SimpleParameterView<?>>();
            for (SimpleParameter<?> parameter : expression.getParameters()) {
              SimpleParameterView<?> parameterView =
                getParameterView(parameter.getName());
              parameterViews.add(parameterView);
            }
            for (SimpleParameterView<?> parameterView : parameterViews) {
              parameterView.highlightError();
            }
            JOptionPane.showMessageDialog(getWindow(), expression
              .getErrorMessage(), "Erro no configurador de parmetros",
              JOptionPane.ERROR_MESSAGE);
            for (SimpleParameterView<?> parameterView : parameterViews) {
              parameterView.clearError();
            }
            return new ValidationResult(expression.getErrorMessage());
          }
        }
        catch (FormulaEvaluationException e) {
          StandardDialogs.showErrorDialog(getWindow(),
            "Erro de validao dos paremtros", e.getLocalizedMessage());
          e.printStackTrace();
        }
      }

      return new ValidationResult();
    }

    /**
     * Mtodo a ser executado logo antes da inicializao do painel.
     */
    protected void executeBeforeInitialization() {
    }

    /**
     * Mtodo a ser executado logo depois da inicializao do painel.
     */
    protected void executeAfterInitialization() {
    }

    /**
     * Inicializa o painel.
     * 
     * @param mode Modo de visualizao. Pode ser
     *        {@link ParameterView.Mode#CONFIGURATION} ou
     *        {@link ParameterView.Mode#REPORT}.
     */
    private void initialize(final Mode mode) {
      groupViews = createGroupsView(mode);
      setLayout(new GridBagLayout());
      populateComponent(mode);
    }

    /**
     * Liga os componentes ao componente principal.
     * 
     * @param mode Modo de visualizao. Pode ser
     *        {@link ParameterView.Mode#CONFIGURATION} ou
     *        {@link ParameterView.Mode#REPORT}.
     */
    private void populateComponent(final Mode mode) {
      removeAll();

      JPanel groupViewsPanel = new JPanel(new GridBagLayout());
      GridBagConstraints constraints = new GridBagConstraints();
      constraints.anchor = GridBagConstraints.NORTHWEST;
      constraints.insets = new Insets(2, 2, 2, 2);
      constraints.gridheight = 1;
      constraints.gridwidth = 1;
      constraints.gridx = 1;
      constraints.gridy = 1;
      constraints.weightx = 1.0;

      for (ParameterGroupView groupView : groupViews) {
        JComponent aGroupViewPanel = groupView.getPanel();
        if (aGroupViewPanel != null) {
          groupView.populatePanel();
          if (groupView.fillVerticalSpace()) {
            constraints.fill = GridBagConstraints.BOTH;
            constraints.weighty = 1.0;
          }
          else {
            constraints.fill = GridBagConstraints.HORIZONTAL;
            constraints.weighty = 0;
          }
          groupViewsPanel.add(aGroupViewPanel, constraints);
          constraints.gridy++;
        }
      }

      JScrollPane groupViewsScrollPane = new JScrollPane(groupViewsPanel);
      groupViewsScrollPane.setBorder(BorderFactory.createEtchedBorder());
      add(groupViewsScrollPane, new GBC(0, 0).both());

      revalidate();
    }

    /**
     * Cria a viso dos grupos de parmetro dessa algoritmo.
     * 
     * @param mode
     * @return A lista de vises criadas.
     */
    private List<ParameterGroupView> createGroupsView(
      final ParameterView.Mode mode) {

      List<ParameterGroupView> groupViews =
        new LinkedList<ParameterGroupView>();
      for (ParameterGroup group : getConfigurator().getGroups()) {
        ParameterGroupView parameterGroupView =
          new ParameterGroupView(SimpleAlgorithmConfiguratorPanel.this, group,
            mode);
        groupViews.add(parameterGroupView);
        parameterGroupView
          .addParameterViewListener(new ParameterViewListener() {
            public void visibilityWasChanged(ParameterView<?> view) {
              populateComponent(mode);
            }
          });
        parameterGroupView
          .addParameterGroupViewListener(new ParameterGroupViewListener() {
            public void childParameterVisibilityChanged(
              ParameterGroupView groupView) {
              populateComponent(mode);
            }
          });
      }

      return groupViews;
    }

    /**
     * <p>
     * Obtm as vises de parmetros.
     * </p>
     * 
     * <p>
     * A lista retornada  imutvel (veja
     * {@link Collections#unmodifiableList(List)}).
     * </p>
     * 
     * @return A lista de vises de parmetros (Se no houver vises de
     *         parmetros a lista estar vazia).
     */
    private List<SimpleParameterView<?>> getParameterViews() {
      List<SimpleParameterView<?>> parameterViews =
        new LinkedList<SimpleParameterView<?>>();
      for (ParameterGroupView groupView : groupViews) {
        parameterViews.addAll(groupView.getParameterViews());
      }
      return Collections.unmodifiableList(parameterViews);
    }

    /**
     * Obtm uma viso de parmetro especfica.
     * 
     * @param parameterName O nome do parmetro (No aceita {@code null}).
     * 
     * @return A viso de parmetro ou {@code null} se ela no existir.
     */
    private SimpleParameterView<?> getParameterView(String parameterName) {
      for (ParameterGroupView groupView : groupViews) {
        SimpleParameterView<?> parameterView =
          groupView.getParameterView(parameterName);
        if (parameterView != null) {
          return parameterView;
        }
      }
      return null;
    }
  }

  /**
   * Painel representando a viso dos parmetros de um configurador simples em
   * modo de configurao.
   */
  private final class SimpleParametersConfigurationPanel extends
    AbstractSimpleParametersPanel {

    /**
     * Construtor
     */
    protected SimpleParametersConfigurationPanel() {
      super(Mode.CONFIGURATION);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void executeBeforeInitialization() {
      getConfigurator().resetValues();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void executeAfterInitialization() {
      getConfigurator().updateTriggers();
    }
  }

  /**
   * Painel representando a viso dos parmetros de um configurador simples em
   * modo de relatrio.
   */
  private final class SimpleParametersReportPanel extends
    AbstractSimpleParametersPanel {

    /**
     * Construtor
     */
    protected SimpleParametersReportPanel() {
      super(Mode.REPORT);
    }
  }
}
