/*
 * $Id:$
 */

package csbase.client.algorithms.view.xapp;

import java.awt.Color;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.Window;
import java.net.InetAddress;
import java.net.UnknownHostException;

import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import tecgraf.javautils.gui.GUIUtils;
import csbase.client.Client;
import csbase.client.algorithms.parameters.OutputFileParameterView;
import csbase.client.algorithms.parameters.ParameterView;
import csbase.client.algorithms.parameters.TextParameterView;
import csbase.client.algorithms.parameters.ParameterView.Mode;
import csbase.client.algorithms.parameters.validators.ValidationMode;
import csbase.client.algorithms.parameters.validators.ValidationResult;
import csbase.client.algorithms.view.AbstractAlgorithmConfiguratorPanel;
import csbase.client.kernel.ClientException;
import csbase.logic.algorithms.parameters.OutputFileParameter;
import csbase.logic.algorithms.parameters.TextParameter;
import csbase.logic.algorithms.xapps.XAppAlgorithmConfigurator;

/**
 * Painel representando a viso dos parmetros de um configurador xapp.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class XAppAlgorithmConfiguratorPanel extends
  AbstractAlgorithmConfiguratorPanel<XAppAlgorithmConfigurator> {

  /**
   * Painel interno que detm a viso dos parmetros.
   */
  private abstract class AbstractXappParametersPanel extends
    AbstractParametersPanel {

    /**
     * O parmetro display para redirecionamento.
     */
    private TextParameterView displayParameter;

    /**
     * O parmetro log.
     */
    private OutputFileParameterView logFileParameter;

    /**
     * Cria as vises de parmetros.
     * 
     * @param mode Modo de visualizao. Pode ser
     *        {@link ParameterView.Mode#CONFIGURATION} ou
     *        {@link ParameterView.Mode#REPORT}.
     */
    private void createParameterViews(final Mode mode) {
      final XAppAlgorithmConfigurator configurator = getConfigurator();
      final String dsp = getDefaultDisplay();
      configurator.setDefaultDisplay(dsp);

      final TextParameter dspParameter = configurator.getDisplayParameter();
      this.displayParameter = new TextParameterView(dspParameter, mode);
      final OutputFileParameter logParameter =
        configurator.getLogFileParameter();
      this.logFileParameter = new OutputFileParameterView(logParameter, mode);
    }

    /**
     * Mtodo a ser executado logo aps da inicializao do painel.
     */
    protected void executeAfterInitialization() {
    }

    /**
     * Mtodo a ser executado logo antes da inicializao do painel.
     */
    protected void executeBeforeInitialization() {
    }

    /**
     * Monta o display default do cliente.
     * 
     * @return uma string com esse valor
     */
    private String getDefaultDisplay() {
      try {
        String addr = Client.getInstance().getParameter("client_address");
        if (addr == null) {
          addr = InetAddress.getLocalHost().getHostName();
        }
        return addr + ":0.0";
      }
      catch (final UnknownHostException e) {
        return "";
      }
    }

    /**
     * Inicializa o painel.
     * 
     * @param mode Modo de visualizao. Pode ser
     *        {@link ParameterView.Mode#CONFIGURATION} ou
     *        {@link ParameterView.Mode#REPORT}.
     */
    private void initialize(final Mode mode) {
      setLayout(new GridLayout(1,1));
      createParameterViews(mode);
      populateMainPanel();
    }

    /**
     * Preenche o painel principal.
     */
    private void populateMainPanel() {
      final JPanel parametersPanel = new JPanel(new GridBagLayout());
      final JComponent[][] components =
        new JComponent[][] { this.displayParameter.getComponents(),
            this.logFileParameter.getComponents() };
      GUIUtils.mountBasicGridPanel(parametersPanel, components);
      final JScrollPane scroll = new JScrollPane(parametersPanel);
      add(scroll);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void setValidationMode(final ValidationMode validationMode) {
      displayParameter.getParameterValidator().setMode(validationMode);
      logFileParameter.getParameterValidator().setMode(validationMode);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public final ValidationResult validate(final boolean complain) {
      final ValidationResult validation =
        this.displayParameter.validate(complain);
      if (!validation.isWellSucceded()) {
        return validation;
      }
      // Testar ainda se segue o formato pro DISPLAY
      final String displayValue =
        this.displayParameter.getParameter().getValue();
      if (displayValue == null
        || !displayValue.matches("[^:]*:[0-9]+[.][0-9]+")) {
        final String errorMessage = "Display de sada do programa invlido.";
        if (complain) {
          this.displayParameter.getLabel().setForeground(Color.RED);
          Toolkit.getDefaultToolkit().beep();
          JOptionPane.showMessageDialog(getWindow(), errorMessage,
            "Erro no configurador de parmetros", JOptionPane.ERROR_MESSAGE);
          this.displayParameter.getLabel().setForeground(Color.BLACK);
        }
        return new ValidationResult(errorMessage);
      }
      final ValidationResult logFileValidation =
        this.logFileParameter.validate(complain);
      if (!logFileValidation.isWellSucceded()) {
        return logFileValidation;
      }
      return new ValidationResult();
    }

    /**
     * Construtor.
     * 
     * @param mode Modo de visualizao. Pode ser
     *        {@link ParameterView.Mode#CONFIGURATION} ou
     *        {@link ParameterView.Mode#REPORT}.
     */
    protected AbstractXappParametersPanel(final Mode mode) {
      super();
      executeBeforeInitialization();
      initialize(mode);
      executeAfterInitialization();
    }
  }

  /**
   * Painel em modo {@link ParameterView.Mode#CONFIGURATION}.
   */
  private final class XappParametersConfigurationPanel extends
    AbstractXappParametersPanel {

    /**
     * {@inheritDoc}
     */
    @Override
    protected void executeBeforeInitialization() {
      getConfigurator().resetValues();
    }

    /**
     * Construtor.
     */
    protected XappParametersConfigurationPanel() {
      super(Mode.CONFIGURATION);
    }
  }

  /**
   * Painel em modo {@link ParameterView.Mode#REPORT}.
   */
  private final class XappParametersReportPanel extends
    AbstractXappParametersPanel {

    /**
     * Construtor.
     */
    protected XappParametersReportPanel() {
      super(Mode.REPORT);
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected AbstractParametersPanel createParametersConfigurationPanel() {
    return new XappParametersConfigurationPanel();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected AbstractParametersPanel createParametersReportPanel() {
    return new XappParametersReportPanel();
  }

  /**
   * @param owner janela me
   * @param configurator configurador
   * @param mode Modo de visualizao. Pode ser
   *        {@link ParameterView.Mode#CONFIGURATION} ou
   *        {@link ParameterView.Mode#REPORT}.
   * 
   * @throws ClientException em caso de erro.
   */
  public XAppAlgorithmConfiguratorPanel(final Window owner,
    final XAppAlgorithmConfigurator configurator, final ParameterView.Mode mode)
    throws ClientException {
    super(owner, configurator, mode);
  }
}
