/*
 * $Id: ApplicationType.java 106082 2010-05-26 02:36:42Z juliana $
 */
package csbase.client.applicationmanager;

import java.util.List;
import java.util.Observable;

import javax.swing.ImageIcon;

import csbase.logic.applicationservice.ApplicationRegistry;

/**
 * Implementa uma aplicao genrica.
 * 
 * @author Tecgraf/PUC-Rio
 */
public abstract class ApplicationType extends Observable {

  /**
   * Identificador do tipo da aplicao
   */
  private ApplicationRegistry applicationRegistry = null;

  /**
   * Mtodo protegido para lanamento de aplicao.
   * 
   * @see #startApplication()
   * @throws ApplicationException em caso de erro.
   */
  final protected void launchApplication() throws ApplicationException {
    startApplication();
  }

  /**
   * Obtm o registry da aplicao.
   * 
   * @return o registry.
   */
  final protected ApplicationRegistry getApplicationRegistry() {
    return this.applicationRegistry;
  }

  /**
   * Obtm o id do tipo da aplicao.
   * 
   * @return o id.
   */
  public final String getId() {
    return this.applicationRegistry.getId();
  }

  /**
   * Obtm o id da instncia da aplicao.
   * 
   * @return o id
   */
  final public int getInstanceId() {
    final ApplicationManager appManager = ApplicationManager.getInstance();
    final int id = appManager.getInstanceId(this);
    return id;
  }

  /**
   * Obtm o cone da aplicao.
   * 
   * @return a imagem.
   */
  public final ImageIcon getImageIcon() {
    final byte[] imgDef = applicationRegistry.getImageDefinition();
    if (imgDef == null) {
      return null;
    }
    return new ImageIcon(imgDef);
  }

  /**
   * Obtm o nome da aplicao.
   * 
   * @return o nome.
   */
  public final String getName() {
    final ApplicationManager appManager = ApplicationManager.getInstance();
    return appManager.getApplicationName(applicationRegistry);
  }

  /**
   * Obtm a verso da aplicao com nmero (<code>double</code>) para
   * comparao.
   * 
   * @return texto indicativo.
   */
  public final int getVersionNumber() {
    return this.applicationRegistry.getVersionNumber();
  }

  /**
   * Obtm a verso da aplicao.
   * 
   * @return texto indicativo.
   */
  public final String getVersion() {
    return this.applicationRegistry.getVersion();
  }

  /**
   * Obtm o e-mail do autor da aplicao.
   * 
   * @return o nome.
   */
  public final String getAuthorMail() {
    return this.applicationRegistry.getAuthorMail();
  }

  /**
   * Obtm o nome do autor da aplicao.
   * 
   * @return o nome.
   */
  public final String getAuthorName() {
    return this.applicationRegistry.getAuthorName();
  }

  /**
   * Obtm a descrio da aplicao.
   * 
   * @return a descrio.
   */
  public final String getDescription() {
    final ApplicationManager applicationManager =
      ApplicationManager.getInstance();

    return applicationManager.getApplicationDescription(applicationRegistry);
  }

  /**
   * Construtor protegido para atualizao do campo de identificador de
   * instncia.
   * 
   * @param id o identificador da aplicao.
   */
  protected ApplicationType(final String id) {
    final ApplicationManager applicationManager =
      ApplicationManager.getInstance();
    this.applicationRegistry = applicationManager.getApplicationRegistry(id);
  }

  /**
   * Consulta o valor inteiro de uma propriedade
   * 
   * @param propName o nome
   * @param defaultValue o valor default
   * @return a propriedade
   */
  public final int getIntSpecificProperty(final String propName,
    final int defaultValue) {
    if (applicationRegistry.isPropertyNull(propName)) {
      return defaultValue;
    }
    return applicationRegistry.getIntSpecificProperty(propName);
  }

  /**
   * Consulta o valor long de uma propriedade
   * 
   * @param propName o nome
   * @param defaultValue o valor default
   * @return a propriedade
   */
  public final long getLongSpecificProperty(final String propName,
    final long defaultValue) {
    if (applicationRegistry.isPropertyNull(propName)) {
      return defaultValue;
    }
    return applicationRegistry.getLongSpecificProperty(propName);
  }

  /**
   * Consulta o valor long de uma propriedade
   * 
   * @param propName o nome
   * @param defaultValue o valor default
   * @return a propriedade
   */
  public final int getLongSpecificProperty(final String propName,
    final int defaultValue) {
    if (applicationRegistry.isPropertyNull(propName)) {
      return defaultValue;
    }
    return applicationRegistry.getIntSpecificProperty(propName);
  }

  /**
   * Consulta o valor inteiro de uma propriedade
   * 
   * @param propName o nome
   * @return a propriedade
   */
  public final String getStringSpecificProperty(final String propName) {
    return applicationRegistry.getStringSpecificProperty(propName);
  }

  /**
   * Consulta o valor lista de strings de uma propriedade
   * 
   * @param propName o nome
   * @return a lista
   */
  public final List<String> getStringListSpecificProperty(final String propName) {
    return applicationRegistry.getStringListSpecificProperty(propName);
  }

  /**
   * Consulta o valor booleano de uma propriedade
   * 
   * @param propName o nome
   * @param defaultValue o valor default
   * @return a propriedade
   */
  public final boolean getBooleanSpecificProperty(final String propName,
    final boolean defaultValue) {
    if (applicationRegistry.isPropertyNull(propName)) {
      return defaultValue;
    }
    return applicationRegistry.getBooleanSpecificProperty(propName);
  }

  /**
   * Mtodo para enviar atributos para uma aplicao; deve ser definido na
   * aplicao de acordo com a necessidade da mesma.
   * 
   * @param name nome do atributo.
   * @param value seu valor.
   */
  public void sendMessage(String name, Object value) {
  }

  /**
   * Mtodo que sinaliza a aplicao que ela j foi iniciada.
   * 
   * @throws Exception em caso de falha.
   */
  public void postInitialization() throws Exception {
  }

  /**
   * Mtodo para encerramento da execuo, que pode ser redefinido para
   * comportamentos prprios da aplicaes.
   */
  public abstract void killApplication();

  /**
   * Mtodo abstrato para abertura do dilogo das aplicaes
   * 
   * @throws ApplicationException Em caso de erro ao iniciar a aplicao.
   */
  public abstract void startApplication() throws ApplicationException;

  /**
   * Mtodo abstrato para fechamento do dilogo das aplicaes
   */
  public abstract void finishApplication();

  /**
   * Mtodo abstrato para fechamento da aplicao.
   * 
   * @return um flag indicativo de sucesso no fechamento.
   */
  public abstract boolean closeApplication();

}
