/*
 * $Id$
 */

package csbase.client.applications;

import java.awt.Window;
import java.awt.event.ActionEvent;
import java.text.MessageFormat;

import javax.swing.Action;
import javax.swing.ImageIcon;

import csbase.client.util.StandardErrorDialogs;

/**
 * <p>
 * Implementao abstrata de {@link ApplicationAction} que visa facilitar a
 * criao de aes dentro de uma aplicao.
 * </p>
 * <p>
 * Esta classe ir buscar na fraseologia da ao pelo nome, mnemnico e dica da
 * ao. As chaves que referenciam estes valores so, respectivamente, "{@code
 * getClass().getSimpleName()}.{@value #TAG_NAME}", "{@code
 * getClass().getSimpleName()}.{@value #TAG_MNEMONIC}", "{@code
 * getClass().getSimpleName()}.{@value #TAG_TOOLTIP}".
 * </p>
 * <p>
 * Alm disso, essa classe tambm disponibiliza, para seus descendentes, os
 * mtodos {@link #getApplication()} e {@link #getString(String, Object...)}.
 * Este ltimo possibilita o acesso a fraseologia da aplicao.
 * </p>
 * 
 * @param <T> tipo da aplicao.
 * 
 * @author Tecgraf / PUC-Rio
 */
public abstract class AbstractSimpleApplicationAction<T extends Application>
  extends ApplicationAction<T> {
  /**
   * Tag usado para buscar o nome da ao.
   */
  final static protected String TAG_NAME = "name";

  /**
   * Tag usado para buscar o mnemnico da ao.
   */
  final static protected String TAG_MNEMONIC = "mnemonic";

  /**
   * Tag usado para buscar o tooltip da ao.
   */
  final static protected String TAG_TOOLTIP = "tooltip";

  /**
   * Construtor.
   * 
   * @param application aplicao na qual esta ao est sendo executada.
   * @param icon a imagem da ao.
   */
  protected AbstractSimpleApplicationAction(final T application,
    final ImageIcon icon) {
    this(application);
    putValue(Action.SMALL_ICON, icon);
  }

  /**
   * Construtor.
   * 
   * @param application aplicao na qual esta ao est sendo executada.
   */
  protected AbstractSimpleApplicationAction(final T application) {
    super(application);

    final String name = getName();
    if (name != null) {
      putValue(Action.NAME, name);
    }

    final Integer mnemonic = getMnemonic();
    if (mnemonic != null) {
      putValue(Action.MNEMONIC_KEY, mnemonic);
    }

    final String tooltip = getToolTip();
    if (tooltip != null) {
      putValue(Action.SHORT_DESCRIPTION, tooltip);
    }
  }

  /**
   * Repassa a chamada ao mtodo {@link #handleActionPerformed(ActionEvent)} e
   * trata quaisquer excees que ele venha a lanar.
   * 
   * @param ae evento que gerou a ao.
   */
  @Override
  public final void actionPerformed(ActionEvent ae) {
    try {
      handleActionPerformed(ae);
    }
    catch (Exception e) {
      final Window window = getApplication().getApplicationFrame();
      final String title = getApplication().getApplicationFrame().getTitle();

      StandardErrorDialogs.showErrorDialog(window, title, e);
    }
  }

  /**
   * Mtodo que implementa a lgica da ao.
   * 
   * @param ae evento que gerou a ao.
   * 
   * @throws Exception se houver falha na ao.
   */
  protected abstract void handleActionPerformed(ActionEvent ae)
    throws Exception;

  /**
   * Retorna um texto de internacionalizao.
   * 
   * @param key_sufix o sufixo da chave. A chave ser: \"{@code
   *        getClass().getSimpleName()}.{@code key_sufix}\"
   * 
   * @param args argumentos do texto. O texto ser formatado utilizando-se da
   *        classe {@link MessageFormat}.
   * 
   * @return o texto internacionalizado.
   */
  protected final String getString(String key_sufix, Object... args) {
    final String className = getClass().getSimpleName();
    final String formattedKey = String.format("%s.%s", className, key_sufix);
    final T app = getApplication();
    if (app.hasString(formattedKey)) {
      return app.getString(formattedKey, args);
    }
    return null;
  }

  /**
   * Obtm o nome da ao.<br>
   * O nome ser procurado na fraseologia da aplicao a partir da chave \"
   * {@code getClass().getSimpleName()}.{@value #TAG_NAME}\".
   * 
   * @return o nome da ao ou null caso no seja encontrado na fraseologia da
   *         aplicao.
   */
  private final String getName() {
    return getString(TAG_NAME);
  }

  /**
   * Obtm a dica da ao.<br>
   * O nome ser procurado na fraseologia da aplicao a partir da chave \"
   * {@code getClass().getSimpleName()}.{@value #TAG_TOOLTIP}\".
   * 
   * @return a dica da ao ou null caso no seja encontrado na fraseologia da
   *         aplicao.
   */
  private final String getToolTip() {
    return getString(TAG_TOOLTIP);
  }

  /**
   * Obtm o mnemonico da ao.<br>
   * O nome ser procurado na fraseologia da aplicao a partir da chave \"
   * {@code getClass().getSimpleName()}.{@value #TAG_MNEMONIC}\".
   * 
   * @return o mnemonico da ao ou null caso no seja encontrado na fraseologia
   *         da aplicao.
   */
  private final Integer getMnemonic() {
    String mn = getString(TAG_MNEMONIC);
    if (mn == null) {
      return null;
    }
    if (mn.trim().isEmpty())
      return null;
    return Integer.valueOf(mn.charAt(0));
  }
}
