package csbase.client.applications.algorithmsmanager;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.Vector;

import javax.swing.ButtonGroup;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;

import tecgraf.javautils.LNG;
import csbase.client.applications.ApplicationExitAction;
import csbase.client.applications.ApplicationFrame;
import csbase.client.applications.ApplicationImages;
import csbase.client.applications.ApplicationProject;
import csbase.client.applications.algorithmsmanager.actions.AlgorithmManagementAction;
import csbase.client.applications.algorithmsmanager.actions.AlgorithmsManagerAction;
import csbase.client.applications.algorithmsmanager.actions.CategoryManagementAction;
import csbase.client.applications.algorithmsmanager.actions.ReloadAlgorithmsAction;
import csbase.client.applications.algorithmsmanager.dialogs.AlgorithmsPropertiesDialog;
import csbase.client.applications.algorithmsmanager.versiontree.actions.ShowHistory;
import csbase.client.desktop.DesktopFrame;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.logic.AlgoEvent;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.Category;
import csbase.logic.algorithms.CategorySet;
import csbase.logic.algorithms.HistoryProvider;

/**
 * Classe que representa a aplicao para Gerncia de Algoritmos. Essa aplicao
 * visa inicialmente prover funcionalidades administrativas relacionadas a
 * instalao de algoritmos no sistema. Por exemplo, essa aplicao deve prover
 * uma funcionalidade que permite associar categorias a algoritmos e vice-versa.
 * 
 */
public class AlgorithmsManager extends ApplicationProject {
  /** Valor default para o tamanho da janela principal da aplicao */
  private static final Dimension DEFAULT_MAIN_FRAME_SIZE = new Dimension(1024,
    768);

  /** Cache do conjunto de categorias disponveis no servidor */
  private static CategorySet categorySet;

  /** Algoritmos carregados do servidor */
  private static SortedSet<AlgorithmInfo> allAlgorithmInfo;

  /** Mapeamento dos algoritmos e as categorias associadas */
  private static HashMap<AlgorithmInfo, SortedSet<Category>> algorithmToCategoryMap;

  /** Listeners interessados em mudanas na aplicao */
  private List<AlgorithmsManagerListener> algorithmsManagerListenerList;

  /** Painel principal */
  private JPanel mainPanel;

  /** Ao de gerncia de algoritmos */
  private CategoryManagementAction categoryManagementAction;

  /** Ao de gerncia de algoritmos */
  private AlgorithmManagementAction algorithmManagementAction;

  /** Ao de recarregar os algoritmos a partir do servidor */
  private ReloadAlgorithmsAction reloadAlgorithmsAction;

  /** Boto toggle que ativa o gerenciamento de categorias */
  private JToggleButton categoryToggleButton;

  /** Boto toggle que ativa o gerenciamento de algoritmos */
  private JToggleButton algorithmToggleButton;

  static {
    categorySet = new CategorySet();
    allAlgorithmInfo = new TreeSet<AlgorithmInfo>();
    algorithmToCategoryMap = new HashMap<AlgorithmInfo, SortedSet<Category>>();
  }

  /**
   * Constri a aplicao para gerncia de algoritmos.
   * 
   * @param id Identificador da aplicao
   */
  public AlgorithmsManager(String id) {
    super(id);
    loadCategoriesAndAlgorithmsData();
    algorithmsManagerListenerList = new Vector<AlgorithmsManagerListener>();
    addCategoryEventsObserver();
    addCategorySetEventsObserver();
    addAlgorithmEventsObserver();
    buildFrame();
    initApplicationAction();
  }

  /**
   * Inicializa a ao principal da aplicao.
   */
  private void initApplicationAction() {
    if (categoryToggleButton.isSelected()) {
      getCategoryManagementAction().initAction();
    }
    else {
      getAlgorithmManagementAction().initAction();
    }
  }

  /**
   * Constri a janela principal da aplicao.
   * 
   */
  private void buildFrame() {
    ApplicationFrame mainFrame = getApplicationFrame();
    mainFrame.setJMenuBar(buildMenuBar());
    mainFrame.getContentPane().add(buildToolBar(), BorderLayout.NORTH);
    mainFrame.getContentPane().add(getMainPanel());
    mainFrame.showStatusBar();
    mainFrame.setSize(DEFAULT_MAIN_FRAME_SIZE);
    mainFrame.setLocationRelativeTo(null);
  }

  /**
   * Adiciona um observador para eventos de categorias de algoritmos que ocorram
   * no servidor.
   */
  private void addCategoryEventsObserver() {
    Category.addObserver(new Observer() {
      @Override
      public void update(Observable o, Object arg) {
        AlgoEvent event = (AlgoEvent) arg;
        AlgorithmsManager.this.handleCategoryEvent(event);
      }
    });
  }

  /**
   * Adiciona um observador para eventos de categorias de algoritmos que ocorram
   * no servidor.
   */
  private void addCategorySetEventsObserver() {
    CategorySet.addObserver(new Observer() {
      @Override
      public void update(Observable o, Object arg) {
        AlgoEvent event = (AlgoEvent) arg;
        AlgorithmsManager.this.handleCategoryEvent(event);
      }
    });
  }

  /**
   * Adiciona um observador para eventos de categorias de algoritmos que ocorram
   * no servidor.
   */
  private void addAlgorithmEventsObserver() {
    AlgorithmInfo.addObserver(new Observer() {
      @Override
      public void update(Observable o, Object arg) {
        AlgoEvent event = (AlgoEvent) arg;
        AlgorithmsManager.this.handleAlgorithmEvent(event);
      }
    });
  }

  /**
   * Obtm o painel principal da aplicao. Se ainda no existir, cria pela
   * primeira vez.
   * 
   * @return o painel principal
   */
  private JPanel getMainPanel() {
    if (mainPanel == null) {
      mainPanel = new JPanel();
      mainPanel.setLayout(new BorderLayout());
    }
    return mainPanel;
  }

  /**
   * Obtm o tamanho do painel principal da aplicao.
   * 
   * @return o tamanho do painel principal da aplicao
   */
  public Dimension getMainPanelSize() {
    return getMainPanel().getSize();
  }

  /**
   * Constri a barra de menu.
   * 
   * @return a barra de menu
   */
  private JMenuBar buildMenuBar() {
    JMenuBar menuBar = new JMenuBar();
    menuBar.add(buildAdminMenu());
    menuBar.add(buildRepositoryMenu());
    return menuBar;
  }

  /**
   * Constri o menu de administrao.
   * 
   * @return o menu criado
   */
  private JMenu buildAdminMenu() {
    JMenu adminMenu = new JMenu(getString("AlgorithmsManager.menu.admin"));
    adminMenu.add(new JMenuItem(getAlgorithmManagementAction()));
    adminMenu.add(new JMenuItem(getCategoryManagementAction()));
    adminMenu.addSeparator();
    adminMenu.add(new JMenuItem(getReloadAlgorithmsAction()));
    adminMenu.addSeparator();
    adminMenu.add(new ApplicationExitAction(this));
    return adminMenu;
  }

  /**
   * Constri o menu de administrao.
   * 
   * @return o menu criado
   */
  private JMenu buildRepositoryMenu() {
    JMenu repositoryMenu = new JMenu(
      getString("AlgorithmsManager.menu.repository"));
    repositoryMenu.add(new JMenuItem(new ShowHistory(getApplicationFrame(),
      new RepositoryHistory())));
    JMenuItem algoPropsMenuItem = repositoryMenu
      .add(getString("AlgorithmsManager.menu.repository.properties"));
    algoPropsMenuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        AlgorithmsPropertiesDialog algoPropDialog = AlgorithmsPropertiesDialog
          .getFrame(AlgorithmsManager.this);
        algoPropDialog.setVisible(true);
      }
    });
    repositoryMenu.add(algoPropsMenuItem);
    return repositoryMenu;
  }

  /**
   * Constri a barra de ferramentas.
   * 
   * @return o componente criado
   */
  private JComponent buildToolBar() {
    JToolBar toolBar = new JToolBar();
    ButtonGroup manageGroup = new ButtonGroup();
    categoryToggleButton = getToggleButton(manageGroup,
      getCategoryManagementAction(), false);
    algorithmToggleButton = getToggleButton(manageGroup,
      getAlgorithmManagementAction(), true);
    toolBar.add(algorithmToggleButton);
    toolBar.add(categoryToggleButton);
    return toolBar;
  }

  /**
   * Obtm a ao de gerenciamento de categorias.
   * 
   * @return a ao
   */
  public CategoryManagementAction getCategoryManagementAction() {
    if (categoryManagementAction == null) {
      categoryManagementAction = new CategoryManagementAction(this,
        ApplicationImages.ICON_ALGORITHM_CATEGORY_16);
      categoryManagementAction.setEnabled(true);
    }
    return categoryManagementAction;
  }

  /**
   * Obtm a ao de gerenciamento de categorias.
   * 
   * @return a ao
   */
  public AlgorithmManagementAction getAlgorithmManagementAction() {
    if (algorithmManagementAction == null) {
      algorithmManagementAction = new AlgorithmManagementAction(this,
        ApplicationImages.ICON_ALGORITHM_MNG_16);
      algorithmManagementAction.setEnabled(true);
    }
    return algorithmManagementAction;
  }

  /**
   * Obtm a ao de recarregar os algoritmos instalados no servidor.
   * 
   * @return a ao
   */
  public ReloadAlgorithmsAction getReloadAlgorithmsAction() {
    if (reloadAlgorithmsAction == null) {
      reloadAlgorithmsAction = new ReloadAlgorithmsAction(this);
      reloadAlgorithmsAction.setEnabled(true);
    }
    return reloadAlgorithmsAction;
  }

  /**
   * Obtm o boto com estado (toggle) associado a uma determinada ao. Somente
   * um boto de gerenciamento pode estar selecionado de cada vez, alternando
   * ento seus estados que indicam a seleo.
   * 
   * @param manageGroup grupo dos botes de gerenciamento
   * 
   * @param action ao associada ao boto
   * @param setSelected se true, o boto vai ser selecionado
   * @return o boto com o estado de selecionado ou no
   */
  private JToggleButton getToggleButton(ButtonGroup manageGroup,
    AlgorithmsManagerAction action, boolean setSelected) {
    JToggleButton button = new JToggleButton(action);
    button.setText(null);
    button.setSelected(setSelected);
    manageGroup.add(button);
    return button;
  }

  /**
   * Modifica o painel principal da janela.
   * 
   * @param panel painel a ser exibido na janela principal da aplicao
   */
  public void changeMainPanel(JPanel panel) {
    mainPanel.removeAll();
    if (panel != null) {
      mainPanel.add(panel);
    }
    mainPanel.revalidate();
    getApplicationFrame().repaint();
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.Application#killApplication()
   */
  @Override
  public void killApplication() {
    final JFrame mainFrame = getApplicationFrame();
    mainFrame.dispose();
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.applications.Application#userCanKillApplication()
   */
  @Override
  protected boolean userCanKillApplication() {
    return true;
  }

  /**
   * Recarrega os algoritmos instalados no servidor.
   */
  public void reloadAlgorithmsFromServer() {
    boolean reloaded = AlgorithmManagementProxy
      .reloadAlgorithms(getApplicationFrame());
    loadCategoriesAndAlgorithmsData();
    rebuildManagementActions();
    initApplicationAction();
  }

  /**
   * Atualiza os algoritmos da aplicao a partir do servidor.
   */
  public void updateApplicationAlgorithms() {
    loadAlgorithmsSet();
    mapAlgorithmsToCategories();
  }

  /**
   * Reconstri as aes de gerenciamento de algoritmos e de categorias.
   * 
   */
  private void rebuildManagementActions() {
    algorithmsManagerListenerList.clear();
    getAlgorithmManagementAction().updateToReload();
    getCategoryManagementAction().updateToReload();
  }

  /**
   * Carrega o conjunto de dados de algoritmos da rvore, a partir de uma lista
   * de algoritmos.
   * 
   */
  private static void loadAlgorithmsSet() {
    List<AlgorithmInfo> allAlgorithms = Arrays.asList(AlgorithmManagementProxy
      .retrieveAllAlgorithmInfos(DesktopFrame.getInstance().getDesktopFrame()));

    allAlgorithmInfo.clear();
    for (AlgorithmInfo algorithmInfo : allAlgorithms) {
      allAlgorithmInfo.add(algorithmInfo);
    }
  }

  /**
   * Carrega os dados de categorias a partir dos dados disponveis no servio de
   * algoritmos, e refaz o mapeamento de algoritmos e categorias.
   */
  public static void updateApplicationCategories() {
    categorySet = AlgorithmManagementProxy.retrieveAllCategories(DesktopFrame
      .getInstance().getDesktopFrame());
    mapAlgorithmsToCategories();
  }

  /**
   * Mapeia os algoritmos s categorias associadas.
   */
  private static void mapAlgorithmsToCategories() {
    algorithmToCategoryMap.clear();

    for (Category category : categorySet.getAllCategories()) {
      Iterator<AlgorithmInfo> algorithmIterator = allAlgorithmInfo.iterator();
      while (algorithmIterator.hasNext()) {
        AlgorithmInfo algo = algorithmIterator.next();
        if (category.containsAlgorithm(algo)) {
          if (!algorithmToCategoryMap.containsKey(algo)) {
            algorithmToCategoryMap.put(algo, new TreeSet<Category>());
          }
          algorithmToCategoryMap.get(algo).add(category);
        }
      }
    }
  }

  /**
   * Mapeia os algoritmos da categoria especificada. Isso geralmente deve ser
   * feito quando uma nova categoria  adicionada no sistema.
   * 
   * @param category categoria cujos algoritmos devem ser mapeados
   */
  private void mapCategoryAlgorithms(Category category) {
    Set<AlgorithmInfo> algorithms = category.getAlgorithms();
    for (AlgorithmInfo algorithmInfo : algorithms) {
      if (!algorithmToCategoryMap.containsKey(algorithmInfo)) {
        algorithmToCategoryMap.put(algorithmInfo, new TreeSet<Category>());
      }
      algorithmToCategoryMap.get(algorithmInfo).add(category);
    }
    mapSubCategoriesAlgorithms(category.getCategories());
  }

  /**
   * Desmapeia os algoritmos da categoria especificada. Isso geralmente deve ser
   * feito quando uma nova categoria  removida do sistema, ou tem a lista dos
   * seus algoritmos alterada.
   * 
   * @param categoryId identificador da categoria cujos algoritmos devem ser
   *        desmapeados
   */
  private void unmapCategoryAlgorithms(String categoryId) {
    Category category = categorySet.getCategory(categoryId);
    Set<AlgorithmInfo> algorithms = category.getAlgorithms();
    for (AlgorithmInfo algorithmInfo : algorithms) {
      if (algorithmToCategoryMap.containsKey(algorithmInfo)) {
        SortedSet<Category> categoryset = algorithmToCategoryMap
          .get(algorithmInfo);
        categoryset.remove(category);
      }
    }
    unmapSubCategoriesAlgorithms(category.getCategories());
  }

  /**
   * Mapeia os algoritmos das sub-categorias especificadas.
   * 
   * @param subCategories sub-categorias cujos algoritmos devem ser mapeados
   */
  private void mapSubCategoriesAlgorithms(SortedSet<Category> subCategories) {
    for (Category subCat : subCategories) {
      mapCategoryAlgorithms(subCat);
    }
  }

  /**
   * Desmapeia os algoritmos das sub-categorias especificadas.
   * 
   * @param subCategories sub-categorias cujos algoritmos devem ser desmapeados
   */
  private void unmapSubCategoriesAlgorithms(SortedSet<Category> subCategories) {
    for (Category subCat : subCategories) {
      unmapCategoryAlgorithms(subCat.getId());
    }
  }

  /**
   * Mapeia as categorias em que o algoritmo especificado faz parte. Isso
   * geralmente deve ser feito quando um novo algoritmo  adicionada no sistema,
   * ou quando so modificadas as categorias em que o algoritmo faz parte.
   * 
   * @param algoInfo algoritmo cujas categorias devem ser mapeadas
   */
  private void mapAlgorithmCategories(AlgorithmInfo algoInfo) {
    if (!algorithmToCategoryMap.containsKey(algoInfo)) {
      algorithmToCategoryMap.put(algoInfo, new TreeSet<Category>());
    }
  }

  /**
   * Desmapeia as categorias em que o algoritmo especificado faz parte. Isso
   * geralmente deve ser feito quando um algoritmo  removido do sistema, ou
   * quando so modificadas as categorias em que o algoritmo faz parte.
   * 
   * @param algoInfo algoritmo cujas categorias devem ser desmapeadas
   */
  private void unmapAlgorithmCategories(AlgorithmInfo algoInfo) {
    if (algorithmToCategoryMap.containsKey(algoInfo)) {
      SortedSet<Category> categories = algorithmToCategoryMap.get(algoInfo);
      for (Category category : categories) {
        category.removeAlgorithm(algoInfo);
      }
      algorithmToCategoryMap.remove(algoInfo);
    }
  }

  /**
   * Obtm o conjunto de categorias carregadas do servidor.
   * 
   * @return o conjunto de categorias de algoritmos
   */
  public SortedSet<Category> getCategories() {
    return categorySet.getCategories();
  }

  /**
   * Obtm as categorias associadas a um determinado algoritmo.
   * 
   * @param algorithmInfo algoritmo procurado
   * 
   * @return as categorias associadas ao algoritmo, ou null, caso no tenha
   *         nenhuma categoria associada
   */
  public static SortedSet<Category> getAlgorithmCategories(
    AlgorithmInfo algorithmInfo) {
    return algorithmToCategoryMap.get(algorithmInfo);
  }

  /**
   * Carrega os dados de categorias e de algoritmos.
   * 
   */
  public static void loadCategoriesAndAlgorithmsData() {
    loadAlgorithmsSet();
    updateApplicationCategories();
  }

  /**
   * Adiciona um listener para mudanas ocorridas na aplicao de gerncia de
   * algoritmos.
   * 
   * @param listener listener a ser adicionado
   */
  public void addAlgorithmsManagerListener(AlgorithmsManagerListener listener) {
    if (!algorithmsManagerListenerList.contains(listener)) {
      algorithmsManagerListenerList.add(listener);
    }
  }

  /**
   * Remove um listener da aplicao.
   * 
   * @param listener listener
   */
  public void removeAlgorithmsManagerListener(AlgorithmsManagerListener listener) {
    algorithmsManagerListenerList.remove(listener);
  }

  /**
   * Notifica os listeners da aplicao que uma categoria foi criada.
   * 
   * @param category categoria criada ou a categoria raiz que recebeu a nova
   *        categoria
   * 
   */
  private void notifyCategoryCreated(Category category) {
    for (AlgorithmsManagerListener listener : algorithmsManagerListenerList) {
      listener.categoryCreated(category);
    }
  }

  /**
   * Notifica os listeners da aplicao que uma categoria foi removida.
   * 
   * @param category categoria removida
   */
  private void notifyCategoryRemoved(Category category) {
    for (AlgorithmsManagerListener listener : algorithmsManagerListenerList) {
      listener.categoryRemoved(category);
    }
  }

  /**
   * Notifica os listeners da aplicao que um conjunto de categorias foi
   * alterada.
   * 
   * @param modifiedCategorySet conjunto com as categorias modificadas
   */
  private void notifyCategoryUpdated(CategorySet modifiedCategorySet) {
    for (AlgorithmsManagerListener listener : algorithmsManagerListenerList) {
      listener.categoryUpdated(modifiedCategorySet);
    }
  }

  /**
   * Notifica os listeners da aplicao que um algoritmo foi criado.
   * 
   * @param algoInfo algoritmo criado
   * 
   */
  private void notifyAlgorithmCreated(AlgorithmInfo algoInfo) {
    for (AlgorithmsManagerListener listener : algorithmsManagerListenerList) {
      listener.algorithmCreated(algoInfo);
    }
  }

  /**
   * Notifica os listeners da aplicao que um algoritmo foi removido.
   * 
   * @param algoInfo algoritmo removido
   * 
   */
  private void notifyAlgorithmRemoved(AlgorithmInfo algoInfo) {
    for (AlgorithmsManagerListener listener : algorithmsManagerListenerList) {
      listener.algorithmRemoved(algoInfo);
    }
  }

  /**
   * Notifica os listeners da aplicao que um algoritmo foi alterado.
   * 
   * @param algoInfo algoritmo alterado
   * 
   */
  private void notifyAlgorithmUpdated(AlgorithmInfo algoInfo) {
    for (AlgorithmsManagerListener listener : algorithmsManagerListenerList) {
      listener.algorithmUpdated(algoInfo);
    }
  }

  /**
   * Atualiza a representao da rvore, sempre que for recebida uma notificao
   * de alterao no servidor.
   * 
   * @param event
   * 
   * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
   */
  public void handleCategoryEvent(AlgoEvent event) {
    Category category = null;
    if (event.item instanceof Category) {
      category = (Category) event.item;
    }

    switch (event.type) {
      case AlgoEvent.CREATE:
        createCategory(category);
        notifyCategoryCreated(category);
        break;

      case AlgoEvent.DELETE:
        removeCategory(category.getId());
        notifyCategoryRemoved(category);
        break;

      case AlgoEvent.MODIFY:
        if (event.item instanceof CategorySet) {
          // Atualiza no cache as categorias que foram modificadas
          CategorySet modifiedCategorySet = (CategorySet) event.item;
          updateCategories(modifiedCategorySet);
          notifyCategoryUpdated(modifiedCategorySet);
          break;
        }

      default:
        String msg = getString("AlgorithmsManager.msg.error.unknown_event",
          new Object[] { new Integer(event.type) });
        System.err.println(msg);
        break;
    }
  }

  /**
   * Atualiza os algoritmos da aplicao, sempre que for recebida uma
   * notificao de alterao no servidor.
   * 
   * @param event
   * 
   * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
   */
  private void handleAlgorithmEvent(AlgoEvent event) {
    AlgorithmInfo algoInfo = null;
    if (event.item instanceof AlgorithmInfo) {
      algoInfo = (AlgorithmInfo) event.item;
    }

    switch (event.type) {
      case AlgoEvent.CREATE:
        createAlgorithm(algoInfo);
        notifyAlgorithmCreated(algoInfo);
        break;

      case AlgoEvent.DELETE:
        removeAlgorithm(algoInfo);
        notifyAlgorithmRemoved(algoInfo);
        break;

      case AlgoEvent.MODIFY:
        updateAlgorithm(algoInfo);
        notifyAlgorithmUpdated(algoInfo);
        break;

      default:
        String msg = getString("AlgorithmsManager.msg.error.unknown_event",
          new Object[] { new Integer(event.type) });
        System.err.println(msg);
        break;
    }
  }

  /**
   * Remove do cache o algoritmo especificado.
   * 
   * @param algoInfo algoritmo removido
   */
  private void removeAlgorithm(AlgorithmInfo algoInfo) {
    allAlgorithmInfo.remove(algoInfo);
    unmapAlgorithmCategories(algoInfo);
  }

  /**
   * Cria ou substitui no cache o algoritmo especificado.
   * 
   * @param algoInfo algoritmo criado
   */
  private void createAlgorithm(AlgorithmInfo algoInfo) {
    allAlgorithmInfo.add(algoInfo);
    mapAlgorithmCategories(algoInfo);
  }

  /**
   * Atualiza no cache o algoritmo especificada.
   * 
   * @param algoInfo algoritmo alterado
   */
  private void updateAlgorithm(AlgorithmInfo algoInfo) {
    removeAlgorithm(algoInfo);
    createAlgorithm(algoInfo);
  }

  /**
   * Atualiza no cache as categorias modificadas.
   * 
   * @param modifiedCategorySet conjunto com as categorias modificadas
   */
  private void updateCategories(CategorySet modifiedCategorySet) {
    for (Category category : modifiedCategorySet.getCategories()) {
      updateCategory(category);
    }
  }

  /**
   * Remove do cache a categoria especificada.
   * 
   * @param categoryId identificador da categoria a ser removida
   */
  private void removeCategory(String categoryId) {
    Category category = categorySet.getCategory(categoryId);
    if (category == null) {
      return;
    }
    unmapCategoryAlgorithms(category.getId());
    Category parentCategory = null;
    parentCategory = category.getParentCategory();
    if (parentCategory == null) {
      categorySet.removeCategory(categoryId);
    }
    else {
      // Substituir na categoria pai
      parentCategory.removeCategory(category);
    }
  }

  /**
   * Cria ou substitui no cache a categoria especificada.
   * 
   * @param category categoria a ser includa ou substituda
   */
  private void createCategory(Category category) {
    Category parentCategory = null;
    parentCategory = category.getParentCategory();
    if (parentCategory == null) {
      removeCategory(category.getId());
      categorySet.addCategory(category);
      mapCategoryAlgorithms(category);
    }
    else {
      // TODO ANDREA TESTAR
      // Substituir o ramo completo acima da categoria sendo criada, pois o
      // pai de cada uma mudou no servidor
      Category rootCreatedCategory = getRootCategory(parentCategory);
      Category oldCategory = categorySet.getRootCategory(rootCreatedCategory
        .getId());
      removeCategory(oldCategory.getId());
      categorySet.addCategory(rootCreatedCategory);
      mapCategoryAlgorithms(rootCreatedCategory);
    }
  }

  /**
   * Cria ou substitui no cache a categoria especificada.
   * 
   * @param category categoria a ser includa ou substituda
   */
  private void updateCategory(Category category) {
    createCategory(category);
  }

  /**
   * Obtm a categoria correspondente  categoria raiz (de mais alto nvel) na
   * hierarquia da categoria especificada.
   * 
   * @param category categoria procurada
   * 
   * @return a categoria raiz (de mais alto nvel) na hierarquia da categoria
   *         especificada
   */
  private Category getRootCategory(Category category) {
    Category parent = category.getParentCategory();
    if (parent == null) {
      return category;
    }
    return getRootCategory(parent);
  }

  /**
   * Obter todas as categorias disponveis no servidor, inclusive as
   * subcategorias.
   * 
   * @return as categorias disponveis no servidor, inclusive as subcategorias
   */
  public static List<Category> getAllCategories() {
    return categorySet.getAllCategories();
  }

  /**
   * Obter todas os algoritmos disponveis no servidor.
   * 
   * @return as algoritmos disponveis no servidor
   */
  public SortedSet<AlgorithmInfo> getAllAgorithms() {
    return allAlgorithmInfo;
  }

  /**
   * Essa classe implementa o histrico do repositrio de algoritmos.
   * 
   */
  private class RepositoryHistory implements HistoryProvider {

    @Override
    public List getHistory() {
      String repositoryName = LNG.get("algomanager.tree.root");
      String[] spath = new String[] { repositoryName };
      return AlgorithmManagementProxy.retrieveHistory(spath);
    }

    @Override
    public String getName() {
      String repositoryName = LNG.get("algomanager.tree.root");
      return repositoryName;
    }

  }

  /**
   * Verifica se j existe em memria um algoritmo cadastrado com esse nome.
   * 
   * @param algoName nome do algoritmo
   * 
   * @return retorna true se j existe em memria um algoritmo cadastrado com
   *         esse nome, caso contrrio, retorna false
   */
  public boolean containsAlgorithm(String algoName) {
    SortedSet<AlgorithmInfo> allAgorithms = getAllAgorithms();
    for (AlgorithmInfo algo : allAgorithms) {
      if (algo.getName().trim().toLowerCase().equals(
        algoName.trim().toLowerCase())) {
        return true;
      }
    }
    return false;
  }

  /**
   * Verifica se j existe em memria uma categoria (no nvel raiz) cadastrada
   * com esse nome.
   * 
   * @param categoryName nome da categoria
   * 
   * @return retorna true se j existe em memria uma categoria cadastrada com
   *         esse nome, caso contrrio, retorna false
   */
  public boolean containsCategory(String categoryName) {
    SortedSet<Category> categories = getCategories();
    for (Category category : categories) {
      if (category.getName().trim().toLowerCase().equals(
        categoryName.trim().toLowerCase())) {
        return true;
      }
    }
    return false;
  }
}
