/*
 * $Id$
 */

package csbase.client.applications.algorithmsmanager.versiontree.actions;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.rmi.RemoteException;
import java.security.AccessControlException;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.Client;
import csbase.client.applications.algorithmsmanager.versiontree.VersionTree;
import csbase.client.applications.algorithmsmanager.versiontree.niotransfer.FileClient;
import csbase.client.desktop.LocalTask;
import csbase.client.desktop.RemoteTask;
import csbase.client.desktop.Task;
import csbase.client.externalresources.ExternalResources;
import csbase.client.externalresources.LocalFile;
import csbase.client.externalresources.StandaloneLocalFile;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.client.util.SingletonFileChooser;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.FileInfo;
import csbase.logic.Utilities;
import csbase.remote.niotransfer.RequestTransferInfo;
import csbase.util.FileSystemUtils;

/**
 * @author Tecgraf / PUC-Rio
 * 
 *         Ao abstrata de exportao de arquivo.
 */
public abstract class AbstractExportFileAction extends
  AbstractVersionTreeNodeAction {

  // Arquivo a ser exportado.
  private final FileInfo source;

  /**
   * Construtor.
   * 
   * @param tree rvore que detm o n fonte da ao.
   * @param name Nome da ao.
   * @param file Arquivo a ser exportado.
   */
  public AbstractExportFileAction(VersionTree tree, String name, FileInfo file) {
    super(tree, name);

    this.source = file;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionPerformed(ActionEvent e) {
    if (FileSystemUtils.canRead()) {
      fileSystemExport();
    }
    else {
      /**
       * Exibe uma pgina para upload de um arquivo de documentao, a partir do
       * disco rgido do usurio para o servidor. Essa funcionalidade s est
       * disponvel via applet.
       */
      appletExport(source); // #TODO A remover quando todos sistemas CSBase
                            // forem assinados
    }
  }

  protected abstract void appletExport(FileInfo source);

  protected abstract RequestTransferInfo prepareDownload(FileInfo source)
    throws RemoteException;

  /**
   * Obtm o arquivo a ser exportado na rvore de algoritmos e o arquivo de
   * destino no sistema de arquivos do usurio. Cria ento uma
   * {@link csbase.client.desktop.LocalTask} para exportar o arquivo.
   * 
   * @param e informaes sobre o evento (no usado nesta implementao).
   */
  private void fileSystemExport() {
    final LocalFile targetFile = getTargetFile();
    if (targetFile == null) {
      return;
    }
    if (targetFile.exists()) {
      if (!confirmOverwrite(targetFile)) {
        return;
      }
    }
    final RequestTransferInfo requestInfo = getRequestInfo();
    if (requestInfo == null) {
      return;
    }
    Task<Void> task = new LocalTask<Void>() {
      @Override
      protected void performTask() throws Exception {
        String serverName = Client.getInstance().getServerName();
        new FileClient(serverName, requestInfo).receive(targetFile);
      }

      @Override
      protected void handleError(Exception error) {
        if (error instanceof IOException) {
          error.printStackTrace();
          StandardDialogs.showErrorDialog(getWindow(), getName(), LNG
            .get("algomanager.error.download_fatal"));
          return;
        }
        super.handleError(error);
      }
    };
    String waitMsg = LNG.get("algomanager.msg.download_wait");
    task.execute(getWindow(), getName(), waitMsg, false, true);
  }

  /**
   * Exibe um dilogo para confirmar com o usurio se este deseja sobrescrever o
   * arquivo existente.
   * 
   * @param targetFile arquivo de destino.
   * 
   * @return <code>true</code> se o usurio confirmar, <code>false</code> caso
   *         contrrio.
   */
  private boolean confirmOverwrite(LocalFile targetFile) {
    String msg;
    try {
      msg = String.format(LNG.get("algomanager.msg.confirm.file_exists"),
        targetFile.getName());
      int answer = StandardDialogs.showYesNoDialog(getWindow(), getName(), msg);
      return (answer == JOptionPane.YES_OPTION);
    }
    catch (IOException ex) {
      msg = LNG.get("algomanager.error.download_io");
      StandardErrorDialogs.showErrorDialog(getWindow(), getName(), msg, ex);
      return false;
    }
  }

  /**
   * Retorna o arquivo de destino local para a importao (se existir, ser
   * sobrescrito).
   * 
   * @return arquivo de destino local.
   */
  private LocalFile getTargetFile() {
    /**
     * Acesso ao sistema de arquivos permitido: obtm arquivos com o
     * JFileChooser.
     */
    try {
      JFileChooser chooser = SingletonFileChooser.getInstance();
      chooser
        .setFileSelectionMode(source.isDirectory() ? JFileChooser.DIRECTORIES_ONLY
          : JFileChooser.FILES_ONLY);
      chooser.setMultiSelectionEnabled(false);

      File target = new File(chooser.getCurrentDirectory() + File.separator
        + source.getName());
      chooser.setSelectedFile(target);

      int result = chooser.showSaveDialog(getWindow());
      if (result == JFileChooser.APPROVE_OPTION) {
        return new StandaloneLocalFile(chooser.getSelectedFile());
      }
      return null;
    }
    catch (AccessControlException ex) {
      // #TODO JWS
      return null;
    }
  }

  /**
   * Retorna um {@link RequestTransferInfo}, isto , um objeto contendo as
   * informaes necessrias para solicitar a transferncia.
   * 
   * @return objeto contendo as informaes necessrias para a transferncia.
   */
  private RequestTransferInfo getRequestInfo() {
    Task<RequestTransferInfo> task = new RemoteTask<RequestTransferInfo>() {
      @Override
      protected void performTask() throws Exception {
        setResult(prepareDownload(source));
      }
    };

    String waitMsg = LNG.get("algomanager.msg.upload_wait");
    if (!task.execute(getWindow(), getName(), waitMsg, false, true)) {
      return null;
    }
    return task.getResult();
  }

  /**
   * Carrega o arquivo atualmente selecionado em um navegador, permitindo sua
   * visualizao e salvamento no disco rgido. Obs: Funcionalidade disponvel
   * somente via applet.
   * 
   * @param algoId .
   * @param filePath .
   * @param dialogTitle .
   */
  protected void export(Object algoId, String filePath, String dialogTitle) {
    if (ExternalResources.getInstance().isEnabled()) {
      String[] filePathArray = Utilities.splitProjectPath(filePath);
      String urlStr = AlgorithmManagementProxy.retrieveDownloadURL(algoId,
        filePathArray, getWindow());
      if (urlStr == null) {
        return;
      }
      try {
        URL url = new URL(urlStr);
        ExternalResources.getInstance().showDocument(url);
      }
      catch (Exception ex) {
        StandardErrorDialogs.showErrorDialog(getWindow(), dialogTitle, ex);
      }
    }
    else {
      StandardDialogs.showInfoDialog(getWindow(), dialogTitle, LNG
        .get("algomanager.error.download_not_available"));
    }
  }
}
