package csbase.client.applications.algorithmsmanager.versiontree.niotransfer;

import java.io.IOException;
import java.nio.channels.SocketChannel;

import csbase.client.externalresources.LocalFile;
import csbase.remote.niotransfer.RequestTransferInfo;
import csbase.remote.niotransfer.SimpleFileTransferProtocol;


/**
 * Classe que define o papel do cliente na execuo do protocolo de transmisso
 * de arquivos.
 *
 * @author Tecgraf
 */
class ClientSimpleFileTransferProtocol extends
  SimpleFileTransferProtocol {
  
  /**
   * Cria o objeto que implementa o protocolo no cliente, indicando a capacidade
   * do buffer a ser utilizado e o canal de comunicao com o servidor.
   * 
   * @param capacity capacidade do buffer a ser utilizado.
   * @param channel canal de comunicao com o servidor.
   */
  ClientSimpleFileTransferProtocol(
    int capacity, SocketChannel channel) {
    super(capacity, channel);
  }

  /**
   * Envia um arquivo para o servidor.
   * 
   * @param sourceFile arquivo de origem no sistema de arquivos do cliente.
   * @param requestInfo informaes necessrias para solicitar a transferncia.
   *        
   * @throws IOException
   */
  public void writeFile(LocalFile sourceFile, RequestTransferInfo requestInfo) 
  throws IOException {
    super.operation = Operation.IMPORT;
    super.transferKey = requestInfo.getKey();
    super.transferKeySize = transferKey.length();
    super.fileSize = sourceFile.getLength();
    io.setSource(sourceFile.getInputStream());
    processState();
  }
  
  /**
   * Recebe um arquivo do servidor.
   *
   * @param requestInfo informaes necessrias para solicitar a transferncia.
   * @param targetFile arquivo de destino para a exportao.
   *        
   * @throws IOException 
   */
  public void readFile(RequestTransferInfo requestInfo, LocalFile targetFile) 
  throws IOException {
    super.operation = Operation.EXPORT;
    super.transferKey = requestInfo.getKey();
    super.transferKeySize = transferKey.length();
    io.setTarget(targetFile.getOutputStream(false));
    processState();
  }
  
  /**
   * {@inheritDoc}
   */
  @Override
  protected boolean contentMissing() throws IOException {
    switch (operation) {
      case IMPORT:
        return (io.availableForReading() > 0);
      case EXPORT:
        return (totalBytesReceived < fileSize);
      default:
        throw new IllegalStateException("operao "+operation+" desconhecida");
    }    
  }

  /**
   * Envia o tamanho da chave representando a transferncia para o servidor.
   *
   * @return <code>true</code> se o procedimento foi bem-sucedido e o protocolo
   *         puder seguir para a prxima etapa.
   *         
   * @throws IOException
   */
  @Override
  protected boolean processKeySize() throws IOException {
    return io.writeInt(transferKeySize);
  }

  /**
   * Envia a chave representando a transferncia para o servidor.
   *
   * @return <code>true</code> se o procedimento foi bem-sucedido e o protocolo
   *         puder seguir para a prxima etapa.
   *         
   * @throws IOException
   */
  @Override
  protected boolean processKey() throws IOException {
    return io.writeString(transferKey);
  }

  /**
   * Envia ou recebe o tamanho do arquivo a ser transmitido, dependendo da
   * operao escolhida.
   *
   * @return <code>true</code> se o procedimento foi bem-sucedido e o protocolo
   *         puder seguir para a prxima etapa.
   *         
   * @throws IOException
   */
  @Override
  protected boolean processFileSize() throws IOException {
    switch (operation) {
      case IMPORT:
        return io.writeLong(fileSize);
      case EXPORT:
        fileSize = io.readLong();
        return (fileSize != -1);
      default:
        return false;
    }
  }

  /**
   * Envia ou recebe o tamanho de um <b>pacote</b> a ser transmitido, dependendo
   * da operao escolhida.
   *
   * @return <code>true</code> se o procedimento foi bem-sucedido e o protocolo
   *         puder seguir para a prxima etapa.
   *         
   * @throws IOException
   * 
   * @see SimpleFileTransferProtocol para uma definio do termo <b>pacote</p>.
   */
  @Override
  protected boolean processPackSize() throws IOException {
    switch (operation) {
      case IMPORT:
        packSize = io.availableForReading();
        packSize = 
          calcPackSizeThatFitsInBuffer(packSize, io.getContentBufferSize());    
        return io.writeInt(packSize);
      case EXPORT:
        packSize = io.readInt();
        if (packSize == -1) {
          // No conseguiu ler tudo ainda. No muda o estado, aguardando a 
          // prxima notificao.
          return false;
        }
        if (packSize > fileSize) {
          throw new IOException("packSize > fileSize");
        }
        return true;
      default:
        return false;
    }
  }

  /**
   * Envia ou recebe um <b>pacote</b> de dados provenientes do arquivo.
   *
   * @return <code>true</code> se o procedimento foi bem-sucedido e o protocolo
   *         puder seguir para a prxima etapa.
   *         
   * @throws IOException
   * 
   * @see SimpleFileTransferProtocol para uma definio do termo <b>pacote</p>.
   */  
  @Override
  protected boolean processPack() throws IOException {
    switch (operation) {
      case IMPORT:
        io.writePack(packSize);
        return true;
      case EXPORT:
        long bytesReceived = io.readPack(fileSize, packSize);
        if (bytesReceived == -1) {
          // No conseguiu ler nem um pacote ainda. No muda o estado, aguardando a 
          // prxima notificao.
          return false;
        }
        totalBytesReceived += bytesReceived;
        return true;
      default:
        return false;
    }
  }
  
  /**
   * Envia ou recebe uma confirmao de recebimento do arquivo como um todo
   * (atestando o sucesso da transferncia).
   *
   * @return <code>true</code> se o procedimento foi bem-sucedido e o protocolo
   *         puder seguir para a prxima etapa.
   *         
   * @throws IOException
   */    
  @Override
  protected boolean processAck() throws IOException {
    switch (operation) {
      case IMPORT:
        int result = io.readInt();
        if (result == -1) {
          return false;
        }
        else if (result != ACK_VALUE) {
          throw new IOException("result != ACK_VALUE");
        }
        return true;
      case EXPORT:
        io.writeInt(ACK_VALUE);
        return true;
      default:
        return false;
    }
  }
  
  /**
   * {@inheritDoc}
   */
  @Override
  protected void closeResources() throws IOException {
    io.closeResources();
  }

  /**
   * {@inheritDoc}. O cliente no precisa nunca interromper o processamento do
   * protocolo, pois seu I/O  bloqueante e por isso bloquear naturalmente se
   * o protocolo estiver aguardando dados para continuar.
   */
  @Override
  protected boolean hasToWait() {
    return false;
  }

}
