/*
 * DiskInfo.java
 *
 * @version $Revision: 75300 $
 */
package csbase.client.applications.diskinfo;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DateFormat;
import java.util.*;

import javax.swing.*;
import javax.swing.Timer;

import tecgraf.javautils.LNG;
import csbase.client.applications.*;
import csbase.exception.BugException;

/**
 * A classe <code>DiskInfo</code> modela a aplicao de monitorao de uso de
 * disco.
 *
 * @author Cristina Ururahy
 */
public class DiskInfo extends Application {

  /** Painel que contm o grfico */
  private ApplicationFrame mainFrame = null;

  /** Lista de painis de monitorao */
  private final List<DiskInfoPanel> monitoringPanels;

  /** Painel de usurios */
  private DiskUserPanel uPanel;

  /** Painel de espao total */
  private DiskSpacePanel pPanel;

  /** Painel de projetos */
  private DiskProjectPanel prjPanel;

  /** Toggle de animao */
  private final JCheckBox animationToggle = new JCheckBox();

  /** Temporizador de atualizao */
  private Timer updateTimer = null;

  /**
   * Montagem do menu.
   *
   * @return a barra de menu.
   */
  private JMenuBar buildMenuBar() {
    final JMenuBar menuBar = new JMenuBar();
    menuBar.add(buildOptionsMenu());
    menuBar.add(new JPanel());
    menuBar.add(buildHelpMenu());
    return menuBar;
  }

  /**
   * Montagem do menu de edio
   *
   * @return um menu de edio.
   */
  private JMenu buildOptionsMenu() {
    final JMenu optMenu = new JMenu(getString("menu.options"));
    final JMenuItem updItem = optMenu.add(getString("menu.options.update"));
    updItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        refreshPies();
      }
    });
    updItem.setIcon(ApplicationImages.ICON_REFRESH_16);
    optMenu.addSeparator();
    optMenu.add(new ApplicationExitAction(this));
    return optMenu;
  }

  /**
   * Montagem do menu de ajuda
   *
   * @return um menu de about.
   */
  private JMenu buildHelpMenu() {
    final JMenu aboutMenu = new JMenu(getString("menu.help"));
    aboutMenu.add(new ApplicationAboutAction(this));
    return aboutMenu;
  }

  /**
   * Cria a janela.
   *
   * @return referncia para a janela criada
   */
  private ApplicationFrame createMainFrame() {
    animationToggle.setText(getString("toggle.animation"));
    ApplicationFrame frame = getApplicationFrame();
    Container cp = frame.getContentPane();
    JPanel panel = new JPanel();
    panel.setLayout(new BorderLayout());

    /* Se h mais de um tipo de monitorao configurado, usa diversos painis */
    if (monitoringPanels.size() > 1) {
      JTabbedPane tabbedPane = new JTabbedPane();
      for (DiskInfoPanel p : monitoringPanels) {
        tabbedPane.add(p);
      }
      panel.add(tabbedPane, BorderLayout.CENTER);
    }
    else {
      panel.add(monitoringPanels.get(0), BorderLayout.CENTER);
    }

    JToolBar optionPanel = new JToolBar(JToolBar.HORIZONTAL);
    JButton refButton = new JButton();
    refButton.setToolTipText(getString("tooltip.refresh"));
    refButton.setText(null);
    refButton.setIcon(ApplicationImages.ICON_REFRESH_16);
    refButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
        refreshPies();
      }
    });
    optionPanel.add(refButton);
    optionPanel.add(new JPanel());
    optionPanel.add(animationToggle);
    panel.add(optionPanel, BorderLayout.NORTH);

    cp.add(panel);
    return frame;
  }

  /**
   * Exibe a janela da aplicao.
   */
  public void start() {
    mainFrame.pack();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void killApplication() {
    updateTimer.stop();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean userCanKillApplication() {
    return (true);
  }

  /**
   * Atualizaco dos paineis.
   */
  private void rotatePies() {
    if (!animationToggle.isSelected()) {
      return;
    }
    for (DiskInfoPanel p : monitoringPanels) {
      p.rotatePie();
    }
  }

  /**
   * Atualizaco dos paineis.
   */
  private void refreshPies() {
    for (DiskInfoPanel p : monitoringPanels) {
      p.refreshPie(this);
    }
    DateFormat df =
      DateFormat.getDateTimeInstance(DateFormat.LONG, DateFormat.LONG,
        LNG.getLocale());
    getApplicationFrame().getStatusBar().setStatus(df.format(new Date()));
  }

  /**
   * Criaco da tarefa de atualizaco dos grficos.
   */
  private void createUpdateTimer() {
    int delay = 1000; // milisegundos
    ActionListener updateTask = new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        rotatePies();
      }
    };
    updateTimer = new Timer(delay, updateTask);
    updateTimer.start();
  }

  /**
   * Construtor da aplicao DiskInfo.
   *
   * @param id identificador da aplicaco.
   * @throws BugException se nenhum tipo de monitorao foi configurado pelas
   *         propriedades da aplicao
   */
  public DiskInfo(final String id) throws BugException {
    super(id);
    monitoringPanels = new ArrayList<DiskInfoPanel>();

    /* Painel de monitorao de espao por usurio */
    if (getBooleanSpecificProperty("hasUserPanel", true)) {
      uPanel = new DiskUserPanel(this);
      monitoringPanels.add(uPanel);
    }
    else {
      uPanel = null;
    }

    /* Painel de monitorao de espao total */
    if (getBooleanSpecificProperty("hasSpacePanel", false)) {
      pPanel = new DiskSpacePanel(this);
      monitoringPanels.add(pPanel);
    }
    else {
      pPanel = null;
    }

    /* Painel de monitorao da rea de projetos */
    if (getBooleanSpecificProperty("hasProjectPanel", false)) {
      prjPanel = new DiskProjectPanel(this);
      monitoringPanels.add(prjPanel);
    }
    else {
      prjPanel = null;
    }
    if (monitoringPanels.size() == 0) {
      throw new BugException("Nenhum tipo de monitorao foi configurado");
    }

    mainFrame = createMainFrame();
    mainFrame.setJMenuBar(buildMenuBar());
    start();
    refreshPies();
    createUpdateTimer();
  }
}
