package csbase.client.applications.imageviewer;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.AffineTransform;
import java.awt.image.ImageObserver;
import java.nio.ByteBuffer;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JToolBar;

import tecgraf.javautils.gui.GUIUtils;
import csbase.client.applications.ApplicationAboutAction;
import csbase.client.applications.ApplicationExitAction;
import csbase.client.applications.ApplicationFrame;
import csbase.client.applications.ApplicationImages;
import csbase.client.applications.ApplicationProject;
import csbase.client.desktop.DesktopTask;
import csbase.client.desktop.RemoteTask;
import csbase.logic.ClientProjectFile;

/**
 * A classe <code>ImageViewer</code> implementa a aplicao de exibio de
 * imagens em um projeto.
 * @author Ana Lodi
 */
public class ImageViewer extends ApplicationProject {

  /** Tipos de arquivos tratados */
  final private String[] FTYPES = {
     "GIF", "JPG"
  };

  /** Painel onde ser exibida a imagem. */
  private ImgPanel imagePanel = null;

  private JScrollPane imageScrolledPanel = null;

  private Image img;

  /** Rtulo do path corrente. */
  final private JTextField pathText = new JTextField();

  /** Largura da figura */
  private int width = 0;

  /** Altura da figura */
  private int height = 0;

  /** Indica se a imagem est pronta */
  private boolean ready = false;

  /** Observador utilizado na formao da imagem */
  private ImgObserver observer = null;

  /**
   * Cria a frame da aplicao
   */
  private void createMainFrame() {
    final ApplicationFrame mainFrame = getApplicationFrame();
    mainFrame.setJMenuBar(createMenuBar());
    mainFrame.setSize(new Dimension(800, 600));
    Container cp = mainFrame.getContentPane();
    cp.add(createToolBar(), BorderLayout.NORTH);
    cp.add(createViewer(), BorderLayout.CENTER);
    cp.add(createPathPanel(), BorderLayout.SOUTH);
  }

  /**
   * Abertura de arquivo.
   */
  private synchronized void actionOpen() {
    try {
      final ClientProjectFile newFile = browseFileOpen(FTYPES, FTYPES[0]);
      openFile(newFile);
    }
    catch(Exception e) {
      showException(getString("error.visualization"), e);
    }
  }

  /**
   * Abre o arquivo dado.
   * @param newFile arquivo a ser aberto 
   */
  private void openFile(ClientProjectFile newFile) {
    if (newFile == null) {
      return;
    }
    pathText.setText(newFile.getName());
    img = createImageFromFile(newFile);
    observer = new ImgObserver();

    // Se a largura ou altura no estiverem especificados, 
    //notifica o observador (observer).
    width = img.getWidth(observer);
    height = img.getHeight(observer);

    // Parece no ser necesssrio 
    Toolkit.getDefaultToolkit().prepareImage(img, width, height, observer);
    ready = ((width >= 0) && (height >= 0));

    // Chamando repaint(), caso o observador no tenha sido notificado.
    imagePanel.repaint();
  }

  /**
   * Cria uma Imagem a partir de um ClientProjectFile.
   * @param file arquivo com a imagem a ser aberta.
   * @return a imagem gerada
   */
  private Image createImageFromFile(final ClientProjectFile file) {
    if (file == null) {
      showError(null, getString("error.open_null"));
      return null;
    }
    if (file.isDirectory()) {
      showError(null, getString("error.open_directory"));
      return null;
    }
    final RemoteTask<byte[]> task = new RemoteTask<byte[]>() {
       @Override
      public void performTask() throws Exception {
         if (file.openFileChannel(true)) {
           try {
             int size = (int)file.size();
             ByteBuffer buffer = ByteBuffer.allocate(size);
             file.read(buffer, 0);
             buffer.flip();
             setResult(buffer.array());
           }
           finally {
             file.close();
           }
         }
       }
    };
    Image img = null;

    final String msg = getString("message.wait_open");
    if (!task.execute(getApplicationFrame(), getName(), msg)) {
       return null;
    }
    try {
      byte[] bArray = task.getResult();
      img = Toolkit.getDefaultToolkit().createImage(bArray);
    }
    catch (Throwable t) {
      showException(getString("error.visualization"), new Exception(t));
    }
    return img;
  }

  /**
   * Cria o painel de visualizaao de imagens.
   * @return painel de visualizaao de imagens
   */
  private JComponent createViewer() {
    imagePanel = new ImgPanel();
    imageScrolledPanel = new JScrollPane(imagePanel);
    imageScrolledPanel.setVerticalScrollBarPolicy(
      JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
    imageScrolledPanel.setHorizontalScrollBarPolicy(
      JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    return imageScrolledPanel;
  }

  /**
   * Cria o painel de visualizao do path
   * @return o painel de visualizao do path
   */
  private JPanel createPathPanel() {
    pathText.setEditable(false);
    final JComponent[][] components = {
        { new JLabel(getString("label.current_file")), pathText },
    };
    final JPanel up = GUIUtils.createBasicGridPanel(components);
    return up;
  }

  /**
   * Montagem da toolbar.
   *
   * @return o painel.
   */
  private JToolBar createToolBar() {
    final JToolBar toolBar = new JToolBar(getName());
    final JButton openButton = new JButton(ApplicationImages.ICON_OPEN_16);
    openButton.setToolTipText(getString("tooltip.file_open"));
    openButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          actionOpen();
        }
    });
    toolBar.add(openButton);
    return toolBar;
  }

  /**
   * Montagem do menu de arquivos.
   * @return o menu.
   */
  private JMenu createAboutMenu() {
    final JMenu aboutMenu = new JMenu(getString("menu.help"));
    aboutMenu.add(new ApplicationAboutAction(this));
    return aboutMenu;
  }

  /**
   * Montagem do menu de arquivos.
   * @return o menu.
   */
  private JMenu createFileMenu() {
    final JMenu fileMenu = new JMenu(getString("menu.file"));
    final JMenuItem openItem = fileMenu.add(getString("menu.file.open"));
    openItem.addActionListener( new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          actionOpen();
        }
    });
    openItem.setIcon(ApplicationImages.ICON_OPEN_16);
    fileMenu.addSeparator();
    fileMenu.add(new ApplicationExitAction(this));
    return fileMenu;
  }

  /**
   * Montagem do menu.
   * @return .
   */
  private JMenuBar createMenuBar() {
    final JMenuBar menuBar = new JMenuBar();
    menuBar.add(createFileMenu());
    menuBar.add(new JPanel());
    menuBar.add(createAboutMenu());
    return menuBar;
  }

  /**
   * Trmino ASAP da aplicao.
   */
  @Override
  public void killApplication() {
  }

  /**
   * Mtodo que permite sempre ao usurio matar a aplicao.
   * @return o flag indicativo.
   */
  @Override
  public boolean userCanKillApplication() {
    return true;
  }

  /**
   * Recebe o arquivo para ser aberto diretamente
   * @param name .
   * @param value .
   */
  @Override
  public void sendMessage(String name, Object value) {
    if (value == null) return;
    if (name.equals(PROJECT_FILE_MESSAGE)) {
       ClientProjectFile file = (ClientProjectFile)value;
       openFile(file);
    }
  }

  /**
   * Classe que implementa o observador utilizado na formao da imagem
   */
  class ImgObserver implements ImageObserver {
    public boolean imageUpdate(Image i, int f, int x, int y, int w, int h) {
      synchronized (ImageViewer.this) {
        if (((f & ERROR) == ERROR) || ((f & ABORT) == ABORT)) {
          img = null;
          return false;
        }
        if ((f & WIDTH) == WIDTH) {
          width = w;
        }
        if ((f & HEIGHT) == HEIGHT) {
          height = h;
        }
        if ((f & ALLBITS) == ALLBITS) {
          ready = true;
          imagePanel.setPreferredSize(new Dimension(width, height));
          imagePanel.revalidate();
          imagePanel.repaint();
          return false;
        }
        return true;
      }
    }
  }

  /**
   * Painel de Exibio de imagens
   */
  class ImgPanel extends JPanel {
    public void paintComponent(Graphics g) {
      int dx;
      int dy;
      super.paintComponent(g);
      if (!ready) {
        return;
      }
      Graphics2D g2d = (Graphics2D)g;

      //Centralizando a imagem
      dx = (imagePanel.getWidth()-width)/2;
      dy = (imagePanel.getHeight()-height)/2;
      if (dx < 0) {
        dx = 0;
      }
      if (dy < 0) {
        dy = 0;
      }
      g2d.drawImage(
        img, AffineTransform.getTranslateInstance(dx, dy), observer);
    }
  }

  /**
   * Constri e exibe o dilogo de execuo de algoritmos
   * @param id .
   */
  public ImageViewer(final String id) {
    super(id);
    createMainFrame();
  }
}
