/*
 * $Id$
 */

package csbase.client.applications.projectsmanager.actions;

import java.util.ArrayList;
import java.util.List;

import javax.swing.ImageIcon;

import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.applications.ApplicationFrame;
import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.ProjectsManagerUI;
import csbase.client.applications.projectsmanager.actions.core.ProjectsManagerAction;
import csbase.client.applications.projectsmanager.dialogs.StatusDialog;
import csbase.client.applications.projectsmanager.models.ProjectSpaceAllocation;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;
import csbase.client.applications.projectsmanager.proxy.AllocateAreaTask;

/**
 * Ao que trata alterao na alocao de projetos. Projetos com espao alocado
 * que sero desalocados e projeto esperando alocao que sero alocados.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class DeallocateAreaAction extends ProjectsManagerAction {

  /**
   * Projetos que tero sua alocao alterada.
   */
  private List<ProjectsManagerData> projectsManagerData;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param projectsManagerData Projetos que tero sua alocao alterada.
   */
  public DeallocateAreaAction(final ProjectsManager projectsManager,
    final List<ProjectsManagerData> projectsManagerData) {
    super(projectsManager);
    this.projectsManagerData = projectsManagerData;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionDone() throws Exception {
    if (projectsManagerData == null || projectsManagerData.size() <= 0) {
      return;
    }

    final ProjectsManager projectsManager = getProjectsManager();

    final List<ProjectsManagerData> projects = validateProjects();
    if (projects == null) {
      return;
    }

    final ApplicationFrame appFrame = projectsManager.getApplicationFrame();
    final boolean ret = runTask(projects);
    if (ret) {
      projectsManager.removeLocalProjects(projects);
      projectsManager.refreshProjectsTable();
    }
    else {
      final String errMsg = getString("deallocation.error.message");
      final String errTitle = getString("deallocation.error.title");
      StandardDialogs.showErrorDialog(appFrame, errTitle, errMsg);
    }
  }

  /**
   * Roda e trata a execuo da task.
   * 
   * @param projects Lista de projetos vlidos para a operao.
   * @return Boolean indicando se a operao foi bem sucedida.
   * @throws Exception
   */
  private boolean runTask(List<ProjectsManagerData> projects) throws Exception {

    final ProjectsManager projectsManager = getProjectsManager();
    final ApplicationFrame appFrame = projectsManager.getApplicationFrame();
    final String[] projectNames = new String[projects.size()];
    final String[] ownerIds = new String[projects.size()];

    int i = 0;
    for (ProjectsManagerData pmd : projects) {
      projectNames[i] = pmd.getProjectName();
      ownerIds[i] = pmd.getOwnerId();
      i++;
    }

    final AllocateAreaTask aat =
      new AllocateAreaTask(projectsManager, projectNames, ownerIds, false);
    final String taskTitle = getString("title");
    final String taskMsg = getString("message");
    aat.execute(appFrame, taskTitle, taskMsg);

    if (aat.wasCancelled()) {
      final String err = getString("cancelled.message");
      aat.showError(err);
      return false;
    }

    if (aat.getStatus() != true) {
      final Exception exception = aat.getError();
      throw exception;
    }

    return aat.getResult();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ImageIcon getStandardImageIcon() {
    return ProjectsManagerUI.DEALLOCATED;
  }

  /**
   * Atualiza lista de projetos sobre os quais a ao ser realizada.
   * 
   * @param projectsManagerData Lista de projetos
   */
  public void setProjectsManagerData(
    final List<ProjectsManagerData> projectsManagerData) {
    this.projectsManagerData = projectsManagerData;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected List<ProjectsManagerData> validateProjects() {

    final List<ProjectsManagerData> prjList =
      new ArrayList<ProjectsManagerData>();
    prjList.addAll(projectsManagerData);

    final List<ProjectsManagerData> failedProjects =
      new ArrayList<ProjectsManagerData>();

    int i = 0;
    while (i < prjList.size()) {
      final ProjectsManagerData pmd = prjList.get(i);
      if (pmd.getProjectSpaceAllocation() != ProjectSpaceAllocation.WAITING_DEALLOCATION) {
        failedProjects.add(pmd);
        prjList.remove(i);
      }
      else {
        i++;
      }
    }

    final ProjectsManager projectsManager = getProjectsManager();
    if (prjList.size() == 0) {
      final String deniedMessage =
        getString("project.selection.denied.message")
          + getString("project.requirements.message");
      StatusDialog.showNoneOkDialog(projectsManager, deniedMessage,
        failedProjects, getString("project.selection.failure.message"));
      return null;
    }
    else if (failedProjects.size() > 0) {
      final String someDeniedMessage =
        getString("project.selection.some.denied.message")
          + getString("project.requirements.message");
      final int res =
        StatusDialog.showSomeOkDialog(projectsManager, someDeniedMessage,
          prjList, getString("project.selection.some.failure.message"));
      if (res == 0) {
        return null;
      }
    }

    return prjList;
  }

}