package csbase.client.applications.projectsmanager.actions;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.swing.ImageIcon;

import csbase.client.applications.ApplicationImages;
import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.actions.core.ProjectsManagerAction;
import csbase.client.applications.projectsmanager.dialogs.ProjectHistoryDialog;
import csbase.client.applications.projectsmanager.dialogs.StatusDialog;
import csbase.client.applications.projectsmanager.models.ProjectSpaceAllocation;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;
import csbase.logic.User;
import csbase.logic.ProjectPermissions.SharingType;

/**
 * Ao que abre dilogo de incluso de comentrio no histrico de projetos.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class OpenProjectHistoryDialogAction extends ProjectsManagerAction {

  /**
   * Lista de projetos selecionados.
   */
  private List<ProjectsManagerData> projectsManagerData;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param projectsManagerData Lista de projetos selecionados.
   */
  public OpenProjectHistoryDialogAction(final ProjectsManager projectsManager,
    final List<ProjectsManagerData> projectsManagerData) {
    super(projectsManager);
    this.projectsManagerData = projectsManagerData;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionDone() throws Exception {
    if (projectsManagerData == null || projectsManagerData.size() == 0) {
      return;
    }

    final ProjectsManager projectsManager = getProjectsManager();
    final List<ProjectsManagerData> projects = validateProjects();
    if (projects == null) {
      return;
    }

    final ProjectHistoryDialog phd =
      new ProjectHistoryDialog(projectsManager, projects);
    phd.pack();
    phd.center();
    phd.setVisible(true);
  }

  /**
   * Atualiza lista de projetos sobre os quais a ao ser realizada.
   * 
   * @param projectsManagerData lista de projetos.
   */
  public void setProjectsManagerData(
    final List<ProjectsManagerData> projectsManagerData) {
    this.projectsManagerData = projectsManagerData;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ImageIcon getStandardImageIcon() {
    return ApplicationImages.ICON_HISTORY_16;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected List<ProjectsManagerData> validateProjects() {

    final List<ProjectsManagerData> prjList =
      new ArrayList<ProjectsManagerData>();
    prjList.addAll(projectsManagerData);

    final List<ProjectsManagerData> failedProjects =
      new ArrayList<ProjectsManagerData>();

    final User loggedUser = User.getLoggedUser();
    final Object loggedUserId = loggedUser.getId();

    int i = 0;
    while (i < prjList.size()) {
      final ProjectsManagerData pmd = prjList.get(i);
      final ProjectSpaceAllocation spaceAllocation =
        pmd.getProjectSpaceAllocation();

      final SharingType sharingType = pmd.getSharingType();
      final Set<Object> usersRW = pmd.getUsersRW();

      final boolean isAdmin = loggedUser.isAdmin();
      final boolean isOwner = pmd.getOwnerId().equals(loggedUserId);
      final boolean hasWriteAccess =
        sharingType == SharingType.ALL_RW
          || (sharingType == SharingType.PARTIAL && usersRW
            .contains(loggedUserId));

      /*
       * Para que um usurio possa escrever algo no histrico do projeto, 
       * preciso que: - O projeto possa ser aberto (no esteja aguardando
       * alocao / desalocao). - O usurio seja o administrador, ou o dono do
       * projeto, ou que o projeto seja pblico ou compartilhado e o usurio
       * tenha acesso de escrita.
       */
      if (spaceAllocation.isOpenable()
        && (isAdmin || isOwner || hasWriteAccess)) {
        i++;
      }
      else {
        failedProjects.add(pmd);
        prjList.remove(i);
      }
    }

    final ProjectsManager projectsManager = getProjectsManager();
    if (prjList.size() == 0) {
      final String deniedMessage =
        getString("project.selection.denied.message")
          + getString("project.requirements.message");
      StatusDialog.showNoneOkDialog(projectsManager, deniedMessage,
        failedProjects, getString("project.selection.failure.message"));
      return null;
    }
    else if (failedProjects.size() > 0) {
      final String someDeniedMessage =
        getString("project.selection.some.denied.message")
          + getString("project.requirements.message");
      final int res =
        StatusDialog.showSomeOkDialog(projectsManager, someDeniedMessage,
          prjList, getString("project.selection.some.failure.message"));
      if (res == 0) {
        return null;
      }
    }

    return prjList;
  }
}
