package csbase.client.applications.projectsmanager.actions;

import java.util.List;

import javax.swing.ImageIcon;

import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.applications.ApplicationImages;
import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.ProjectsManagerUI;
import csbase.client.applications.projectsmanager.actions.core.ProjectsManagerAction;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;
import csbase.client.applications.projectsmanager.proxy.MultipleProjectSpaceTask;
import csbase.client.applications.projectsmanager.proxy.ReloadSelectedProjectsTask;

/**
 * Ao responsvel por recarregar todas as informaes dos projetos
 * selecionados.
 * 
 * @author jnlopes
 * 
 */
public class RefreshProjectsAction extends ProjectsManagerAction {

  /**
   * Array de projetos.
   */
  private List<ProjectsManagerData> projectsManagerData;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param projectsManagerData Array de projetos.
   */
  public RefreshProjectsAction(final ProjectsManager projectsManager,
    final List<ProjectsManagerData> projectsManagerData) {
    super(projectsManager);
    this.projectsManagerData = projectsManagerData;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionDone() throws Exception {

    if (projectsManagerData == null || projectsManagerData.size() <= 0) {
      return;
    }

    final String[] projectNames = new String[projectsManagerData.size()];
    final String[] ownerId = new String[projectsManagerData.size()];

    int count = 0;
    for (ProjectsManagerData pmd : this.projectsManagerData) {
      projectNames[count] = pmd.getProjectName();
      ownerId[count] = pmd.getOwnerId();

      count++;
    }

    final Double[] occupiedSpace =
      runMultipleProjectSpaceTask(projectNames, ownerId);

    projectsManagerData = runUpdateProjectDataTask();

    count = 0;
    final ProjectsManager projectsManager = getProjectsManager();
    for (ProjectsManagerData pmd : this.projectsManagerData) {
      if (occupiedSpace[count] != pmd.getOccupiedSpace()) {
        pmd.setOccupiedSpace(occupiedSpace[count]);
        projectsManager.updateProjectOccupiedSpace(pmd);
      }
      count++;
    }

    projectsManager.refreshProjectsTable();
  }

  /**
   * Executa task que obtm dados atualizados dos projetos.
   * 
   * @return Projetos atualizados.
   */
  private List<ProjectsManagerData> runUpdateProjectDataTask() {

    final ProjectsManager projectsManager = this.getProjectsManager();

    final ReloadSelectedProjectsTask rspt =
      new ReloadSelectedProjectsTask(projectsManager, projectsManagerData);

    rspt.execute(projectsManager.getApplicationFrame(), getString("title"),
      getString("message"));
    if (rspt.wasCancelled()) {
      final String err = getString("cancelled.message");
      rspt.showError(err);
      return null;
    }

    if (rspt.getStatus() != true) {
      final Exception exception = rspt.getError();
      ProjectsManagerUI.handleException(this.getProjectsManager(), exception);
      return null;
    }

    final List<ProjectsManagerData> projectList = rspt.getResult();

    if (projectList == null) {
      final String errTitle = getString("modification.date.title");
      final String errMsg = getString("modification.date.message");
      StandardDialogs.showErrorDialog(getProjectsManager()
        .getApplicationFrame(), errTitle, errMsg);
      return null;
    }

    return projectList;
  }

  /**
   * Executa task que obtm espao ocupado por projetos.
   * 
   * @param projectNames Nomes dos projetos.
   * @param ownerId Ids dos donos dos projetos
   * @return Espaos ocupados pelos projetos
   */
  private Double[] runMultipleProjectSpaceTask(final String[] projectNames,
    final String[] ownerId) {

    final ProjectsManager projectsManager = this.getProjectsManager();

    final MultipleProjectSpaceTask spst =
      new MultipleProjectSpaceTask(projectsManager, projectNames, ownerId);

    spst.execute(projectsManager.getApplicationFrame(), getString("title"),
      getString("message"));
    if (spst.wasCancelled()) {
      final String err = getString("cancelled.message");
      spst.showError(err);
      return null;
    }

    if (spst.getStatus() != true) {
      final Exception exception = spst.getError();
      ProjectsManagerUI.handleException(this.getProjectsManager(), exception);
      return null;
    }

    final Double[] occupiedSpace = spst.getResult();

    if (occupiedSpace == null) {
      final String errTitle = getString("occupied.space.error.title");
      final String errMsg = getString("occupied.space.error.message");
      StandardDialogs.showErrorDialog(getProjectsManager()
        .getApplicationFrame(), errTitle, errMsg);
      return null;
    }

    return occupiedSpace;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ImageIcon getStandardImageIcon() {
    return ApplicationImages.ICON_REFRESH_ONE_16;
  }

  /**
   * Atualiza lista de projetos sobre os quais a ao ser realizada.
   * 
   * @param projectsManagerData Lista de projetos
   */
  public void setProjectsManagerData(
    final List<ProjectsManagerData> projectsManagerData) {
    this.projectsManagerData = projectsManagerData;
  }

}
