package csbase.client.applications.projectsmanager.actions;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.swing.ImageIcon;

import csbase.client.applications.ApplicationFrame;
import csbase.client.applications.ApplicationImages;
import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.actions.core.ProjectsManagerAction;
import csbase.client.applications.projectsmanager.dialogs.StatusDialog;
import csbase.client.applications.projectsmanager.models.ProjectSpaceAllocation;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;
import csbase.client.applications.projectsmanager.proxy.core.ProjectsManagerTask;
import csbase.client.applications.projectsmanager.usersearch.UserDialog;
import csbase.logic.CommonClientProject;
import csbase.logic.User;
import csbase.logic.ProjectPermissions.SharingType;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.ProjectServiceInterface;

/**
 * Define usurios que compartilham projetos, e seus modos de acesso.
 * 
 * @author Tecgraf
 */
public class SetProjectSharedAction extends ProjectsManagerAction {

  /**
   * Lista de projetos selecionados.
   */
  private List<ProjectsManagerData> projectsManagerData;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param projectsManagerData a lista de projetos.
   */
  public SetProjectSharedAction(final ProjectsManager projectsManager,
    final List<ProjectsManagerData> projectsManagerData) {
    super(projectsManager);
    this.projectsManagerData = projectsManagerData;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionDone() throws Exception {
    if (projectsManagerData == null || projectsManagerData.size() == 0) {
      return;
    }

    final List<ProjectsManagerData> projects = validateProjects();
    if (projects == null) {
      return;
    }

    final ProjectsManager projectsManager = getProjectsManager();
    final ApplicationFrame appFrame = projectsManager.getApplicationFrame();

    final UserDialog ud;
    if (projects.size() == 1) {
      final ProjectsManagerData pmd = projects.get(0);
      ud =
        new UserDialog(appFrame, projectsManager.getUsers(), pmd.getUsersRO(),
          pmd.getUsersRW(), true, true);
    }
    else {
      ud = new UserDialog(appFrame, projectsManager.getUsers(), true, true);
    }

    final String title = getString("selection.title");
    ud.setTitle(title);
    ud.execute();

    final Set<Object> usersRO = ud.getUsersRO();
    final Set<Object> usersRW = ud.getUsersRW();

    if (usersRO.size() == 0 && usersRW.size() == 0) {
      return;
    }

    final boolean ret = runTask(projects, usersRO, usersRW);
    if (ret) {
      projectsManager.refreshProjectsTable();
      projectsManager.refreshInfoPanel();
    }
  }

  /**
   * Atualiza lista de projetos sobre os quais a ao ser realizada.
   * 
   * @param projectsManagerData Lista de projetos
   */
  public void setProjectsManagerData(
    final List<ProjectsManagerData> projectsManagerData) {
    this.projectsManagerData = projectsManagerData;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ImageIcon getStandardImageIcon() {
    return ApplicationImages.ICON_SHARED_16;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected List<ProjectsManagerData> validateProjects() {

    final List<ProjectsManagerData> prjList =
      new ArrayList<ProjectsManagerData>();
    prjList.addAll(projectsManagerData);

    final List<ProjectsManagerData> failedProjects =
      new ArrayList<ProjectsManagerData>();

    final User loggedUser = User.getLoggedUser();
    final Object loggedUserId = loggedUser.getId();

    int i = 0;
    while (i < prjList.size()) {
      final ProjectsManagerData pmd = prjList.get(i);
      final ProjectSpaceAllocation spaceAllocation =
        pmd.getProjectSpaceAllocation();

      final boolean isAdmin = loggedUser.isAdmin();
      final boolean isOwner = pmd.getOwnerId().equals(loggedUserId);

      /*
       * Para que um usurio possa tornar um projeto pblico  preciso que: - O
       * projeto possa ser aberto (no esteja aguardando alocao /
       * desalocao). - O usurio seja o administrador ou o dono do projeto.
       */
      if (spaceAllocation.isOpenable() && (isAdmin || isOwner)) {
        i++;
      }
      else {
        failedProjects.add(pmd);
        prjList.remove(i);
      }
    }

    final ProjectsManager projectsManager = getProjectsManager();
    if (prjList.size() == 0) {
      final String deniedMessage =
        getString("project.selection.denied.message")
          + getString("project.requirements.message");
      StatusDialog.showNoneOkDialog(projectsManager, deniedMessage,
        failedProjects, getString("project.selection.failure.message"));
      return null;
    }
    else if (failedProjects.size() > 0) {
      final String someDeniedMessage =
        getString("project.selection.some.denied.message")
          + getString("project.requirements.message");
      final int res =
        StatusDialog.showSomeOkDialog(projectsManager, someDeniedMessage,
          prjList, getString("project.selection.some.failure.message"));
      if (res == 0) {
        return null;
      }
    }
    return prjList;
  }

  /**
   * Roda e trata a execuo da task.
   * 
   * @param projects Lista de projetos vlidos para a operao.
   * @param idsRO Conjunto de ids dos usurios com permisso de leitura a serem
   *        adicionados aos projetos.
   * @param idsRW Conjunto de ids dos usurios com permisso de leitura e
   *        escrita a serem adicionados aos projetos.
   * @return Indica se a operao foi realizada com sucesso.
   * @throws Exception
   */
  private boolean runTask(final List<ProjectsManagerData> projects,
    final Set<Object> idsRO, final Set<Object> idsRW) throws Exception {
    final ProjectsManager projectsManager = getProjectsManager();
    final DefineUsersTask dutRO =
      new DefineUsersTask(projectsManager, projects, idsRO, true);

    final ApplicationFrame frame = projectsManager.getApplicationFrame();
    dutRO.execute(frame, getString("title"), getString("message"));

    if (dutRO.wasCancelled()) {
      final String err = getString("cancelled.message");
      dutRO.showError(err);
      return false;
    }

    if (dutRO.getStatus() != true) {
      final Exception exception = dutRO.getError();
      throw exception;
    }

    final DefineUsersTask dutRW =
      new DefineUsersTask(projectsManager, projects, idsRW, false);
    dutRW.execute(frame, getString("title"), getString("message"));

    if (dutRW.wasCancelled()) {
      final String err = getString("cancelled.message");
      dutRW.showError(err);
      return false;
    }

    if (dutRW.getStatus() != true) {
      final Exception exception = dutRW.getError();
      throw exception;
    }

    for (ProjectsManagerData pmd : projects) {
      pmd.setSharingType(SharingType.PARTIAL);
      pmd.setUsers(idsRO, idsRW);
    }

    return true;
  }

}

/**
 * Task que adiciona usurios a um conjunto de projetos (transformando o escopo
 * do projeto em "compartilhado" se este ainda no for).
 * 
 * @author Tecgraf
 */
class DefineUsersTask extends ProjectsManagerTask<Void> {

  /**
   * Lista de projetos aos quais os usurios sero adicionados.
   */
  final private List<ProjectsManagerData> projects;

  /**
   * Lista de usurios que sero adicionados aos projetos.
   */
  final private Set<Object> userIds;

  /**
   * Modo de acesso dos usurios sobre os projetos.
   */
  final private boolean readOnly;

  /**
   * Construtor.
   * 
   * @param projectsManager A aplicao.
   * @param projects Lista de projetos aos quais os usurios sero adicionados.
   * @param userIds Lista de usurios que sero adicionados aos projetos.
   * @param readOnly Modo de acesso dos usurios sobre os projetos.
   */
  public DefineUsersTask(final ProjectsManager projectsManager,
    final List<ProjectsManagerData> projects, final Set<Object> userIds,
    final boolean readOnly) {
    super(projectsManager);
    this.projects = projects;
    this.userIds = userIds;
    this.readOnly = readOnly;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void performTask() throws Exception {

    if (userIds == null) {
      return;
    }

    final ProjectServiceInterface psi = ClientRemoteLocator.projectService;

    for (ProjectsManagerData pmd : projects) {
      final CommonClientProject ccp =
        psi.openProject(pmd.getOwnerId(), pmd.getProjectName(), false);

      ccp.setSharingType(SharingType.PARTIAL);

      if (readOnly) {
        ccp.updateUsersRO(userIds);
      }
      else {
        ccp.updateUsersRW(userIds);
      }
      ccp.close(true);
    }
  }

}
