package csbase.client.applications.projectsmanager.actions;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.swing.ImageIcon;

import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.applications.ApplicationFrame;
import csbase.client.applications.ApplicationImages;
import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.actions.core.ProjectsManagerAction;
import csbase.client.applications.projectsmanager.dialogs.StatusDialog;
import csbase.client.applications.projectsmanager.models.ProjectSpaceAllocation;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;
import csbase.client.applications.projectsmanager.proxy.UserRemovalFromProjectTask;
import csbase.logic.User;
import csbase.logic.ProjectPermissions.SharingType;

/**
 * Ao de remover um usurio de um projeto (do qual ele no  dono).
 * 
 * @author jnlopes
 */
public class UserRemovalFromProjectAction extends ProjectsManagerAction {

  /**
   * Projetos do qual o usurio ser removido.
   */
  private List<ProjectsManagerData> projectsManagerData;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param projectsManagerData os dados.
   */
  public UserRemovalFromProjectAction(final ProjectsManager projectsManager,
    final List<ProjectsManagerData> projectsManagerData) {
    super(projectsManager);
    this.projectsManagerData = projectsManagerData;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionDone() throws Exception {

    if (projectsManagerData == null || projectsManagerData.size() <= 0) {
      return;
    }

    final ProjectsManager projectsManager = getProjectsManager();

    final List<ProjectsManagerData> projects = validateProjects();
    if (projects == null) {
      return;
    }

    final boolean ret = runTask(projects);
    if (ret) {
      projectsManager.refreshProjectsTable();
    }
    else {
      StandardDialogs.showWarningDialog(projectsManager.getApplicationFrame(),
        getString("shared.user.removal.error.title"),
        getString("shared.user.removal.error.message"));
    }
  }

  /**
   * Roda e trata a execuo da task.
   * 
   * @param projects Lista de projetos vlidos para a operao.
   * @return Boolean indicando se a operao foi bem sucedida.
   * @throws Exception
   */
  private boolean runTask(List<ProjectsManagerData> projects) throws Exception {

    final List<String> projectNames = new ArrayList<String>(projects.size());
    final List<String> ownerIds = new ArrayList<String>(projects.size());

    for (ProjectsManagerData pmd : projects) {
      projectNames.add(pmd.getProjectName());
      ownerIds.add(pmd.getOwnerId());
    }

    final ProjectsManager projectsManager = getProjectsManager();
    final ApplicationFrame frame = projectsManager.getApplicationFrame();

    final UserRemovalFromProjectTask urfp =
      new UserRemovalFromProjectTask(getProjectsManager(), projectNames,
        ownerIds);
    urfp.execute(frame, getString("title"), getString("message"));

    if (urfp.wasCancelled()) {
      final String err = getString("cancelled.message");
      urfp.showError(err);
      return false;
    }

    if (urfp.getStatus() != true) {
      final Exception exception = urfp.getError();
      throw exception;
    }
    return urfp.getResult();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ImageIcon getStandardImageIcon() {
    return ApplicationImages.ICON_CANCEL_16;
  }

  /**
   * Atualiza lista de projetos sobre os quais a ao ser realizada.
   * 
   * @param projectsManagerData Lista de projetos
   */
  public void setProjectsManagerData(
    final List<ProjectsManagerData> projectsManagerData) {
    this.projectsManagerData = projectsManagerData;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected List<ProjectsManagerData> validateProjects() {

    final List<ProjectsManagerData> prjList =
      new ArrayList<ProjectsManagerData>();
    prjList.addAll(projectsManagerData);

    final List<ProjectsManagerData> failedProjects =
      new ArrayList<ProjectsManagerData>();

    final User loggedUser = User.getLoggedUser();
    final Object loggedUserId = loggedUser.getId();

    int i = 0;
    while (i < prjList.size()) {
      final ProjectsManagerData pmd = prjList.get(i);
      final ProjectSpaceAllocation spaceAllocation =
        pmd.getProjectSpaceAllocation();

      final SharingType sharingType = pmd.getSharingType();
      final Set<Object> usersRW = pmd.getUsersRW();

      final boolean isAdmin = loggedUser.isAdmin();
      final boolean isOwner = pmd.getOwnerId().equals(loggedUserId);
      final boolean hasWriteAccess =
        sharingType == SharingType.PARTIAL && usersRW.contains(loggedUserId);

      /*
       * Para que um usurio possa se remover de um projeto,  preciso que: - O
       * projeto possa ser aberto (no esteja aguardando alocao /
       * desalocao). - O usurio no seja o administrador, ou o dono do
       * projeto. - O projeto esteja no modo compartilhado e ele tenha permisso
       * de escrita.
       */
      if (spaceAllocation.isOpenable() && !isAdmin && !isOwner
        && hasWriteAccess) {
        i++;
      }
      else {
        failedProjects.add(pmd);
        prjList.remove(i);
      }
    }

    final ProjectsManager projectsManager = getProjectsManager();
    if (prjList.size() == 0) {
      final String deniedMessage =
        getString("project.selection.denied.message")
          + getString("project.requirements.message");
      StatusDialog.showNoneOkDialog(projectsManager, deniedMessage,
        failedProjects, getString("project.selection.failure.message"));
      return null;
    }
    else if (failedProjects.size() > 0) {
      final String someDeniedMessage =
        getString("project.selection.some.denied.message")
          + getString("project.requirements.message");
      final int res =
        StatusDialog.showSomeOkDialog(projectsManager, someDeniedMessage,
          prjList, getString("project.selection.some.failure.message"));
      if (res == 0) {
        return null;
      }
    }

    return prjList;
  }
}
