package csbase.client.applications.projectsmanager.proxy;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;

import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.ProjectsManagerUI;
import csbase.client.applications.projectsmanager.models.ProjectHistoryBlockType;
import csbase.client.applications.projectsmanager.proxy.core.ProjectsManagerTask;
import csbase.logic.CommonClientProject;
import csbase.logic.ProjectAdminInfo;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.ProjectServiceInterface;

/**
 * Task que trata alterao na alocao de projetos. Projetos com espao alocado
 * que sero desalocados e projeto esperando alocao que sero alocados.
 * 
 * @author jnlopes
 * 
 */
public class AllocateAreaTask extends ProjectsManagerTask<Boolean> {

  /**
   * Nome do projeto.
   */
  final private String[] projectNames;

  /**
   * Id do dono do projeto.
   */
  final private String[] ownerIds;

  /**
   * Boolean que indica se rea deve ser definida como alocada ou desalocada.
   */
  final private boolean allocate;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param projectNames Nomes dos projetos
   * @param ownerIds ids dos donos de projetos.
   * @param allocate boolean que indice se a rea deve ser definida como alocada
   *        ou desalocada.
   */
  public AllocateAreaTask(final ProjectsManager projectsManager,
    final String[] projectNames, final String[] ownerIds, final boolean allocate) {
    super(projectsManager);

    this.projectNames = projectNames;
    this.ownerIds = ownerIds;
    this.allocate = allocate;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void performTask() throws RemoteException {

    final ProjectAdminInfo[] prjAdmInfo = ProjectAdminInfo.getAll();

    if (prjAdmInfo == null) {
      setResult(false);
      return;
    }

    final List<ProjectAdminInfo> selectedProjects =
      new ArrayList<ProjectAdminInfo>();

    if (allocate) {

      for (ProjectAdminInfo p : prjAdmInfo) {
        for (int i = 0; i < projectNames.length; i++) {
          if (p.isLocked() && p.getProjectName().equals(projectNames[i])
            && p.getUserId().equals(this.ownerIds[i])) {

            p.setAreaLockedSize(p.getAreaRequestSize());
            selectedProjects.add(p);
          }
        }
      }

      if (selectedProjects.size() == 0) {
        setResult(true);
        return;
      }

      final ProjectAdminInfo[] sp =
        selectedProjects.toArray(new ProjectAdminInfo[selectedProjects.size()]);
      ProjectAdminInfo.allocateAreaForProjects(sp);

      saveInfoToProjectHistory();
    }
    else {
      for (ProjectAdminInfo p : prjAdmInfo) {
        for (int i = 0; i < projectNames.length; i++) {
          if (p.isWaitingAreaFree()
            && p.getProjectName().equals(projectNames[i])
            && p.getUserId().equals(this.ownerIds[i])) {

            selectedProjects.add(p);
          }
        }
      }

      final ProjectAdminInfo[] sp =
        selectedProjects.toArray(new ProjectAdminInfo[selectedProjects.size()]);
      ProjectAdminInfo.freeAreaForProjects(sp);
    }

    setResult(true);
  }

  /**
   * Obtm os objetos CommonClientProject (que so criados no momento em que as
   * reas so liberadas) e adiciona a informao de que o projeto teve sua rea
   * liberada.
   * 
   * @throws RemoteException
   */
  private void saveInfoToProjectHistory() throws RemoteException {

    final ProjectServiceInterface psi = ClientRemoteLocator.projectService;

    for (int i = 0; i < projectNames.length; i++) {
      final CommonClientProject ccp =
        psi.openProject(ownerIds[i], projectNames[i], false);
      ProjectsManagerUI.addAutomaticHistoryBlock(ccp,
        ProjectHistoryBlockType.AREA_ALLOCATED);
    }
  }

}
