package csbase.client.applications.projectsmanager.proxy;

import java.util.List;

import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.models.ProjectSpaceAllocation;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;
import csbase.client.applications.projectsmanager.proxy.core.ProjectsManagerTask;
import csbase.exception.ServiceFailureException;
import csbase.logic.CommonClientProject;
import csbase.logic.ProjectAdminInfo;
import csbase.logic.User;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.ProjectServiceInterface;

/**
 * Task que recarrega todas as informaes dos projetos selecionados, com
 * exceo do espao ocupado.
 * 
 * @author jnlopes
 * 
 */
public class ReloadSelectedProjectsTask extends
  ProjectsManagerTask<List<ProjectsManagerData>> {

  /**
   * Array de projetos a serem atualizados.
   */
  final private List<ProjectsManagerData> projectsManagerData;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param projectsManagerData Lista de projetos a serem atualizados.
   */
  public ReloadSelectedProjectsTask(final ProjectsManager projectsManager,
    final List<ProjectsManagerData> projectsManagerData) {
    super(projectsManager);
    this.projectsManagerData = projectsManagerData;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void performTask() throws Exception {
    getProjectsData();
    setResult(projectsManagerData);
  }

  /**
   * Obtm dados dos projetos.
   * 
   * @throws Exception
   */
  private void getProjectsData() throws Exception {

    final ProjectServiceInterface psi = ClientRemoteLocator.projectService;

    for (int i = 0; i < projectsManagerData.size(); i++) {
      final ProjectsManagerData pmd = projectsManagerData.get(i);

      try {
        final CommonClientProject ccp =
          psi.openProject(pmd.getOwnerId(), pmd.getProjectName(), false);

        final long allocatedSpace = ccp.getLockingAreaSize();

        if (allocatedSpace == 0) {
          pmd.setProjectSpaceAllocation(ProjectSpaceAllocation.NO_ALLOCATION);
        }
        else if (allocatedSpace > 0) {
          pmd.setProjectSpaceAllocation(ProjectSpaceAllocation.ALLOCATED);
        }

        final String userName = User.getName(ccp.getUserId());
        pmd.setOwnerName(userName);
        pmd.setProjectDescription(ccp.getDescription());
        pmd.setCreationDate(ccp.getCreationDate());
        pmd.setLocationInServer(ccp.getLocationInServer());
        pmd.setModificationDate(ccp.getLastModificationDate());
        pmd.setSharingType(ccp.getSharingType());
        pmd.setUsers(ccp.getUsersRO(), ccp.getUsersRW());
        ccp.close(false);
      }
      catch (ServiceFailureException sfe) {
        final ProjectAdminInfo[] allInfo = ProjectAdminInfo.getAll();
        for (ProjectAdminInfo p : allInfo) {
          if (p.getUserId().equals(pmd.getOwnerId())
            && p.getProjectName().equals(pmd.getProjectName())) {
            pmd.clearData();
            if (p.isWaitingAreaFree()) {
              pmd
                .setProjectSpaceAllocation(ProjectSpaceAllocation.WAITING_DEALLOCATION);
            }
            else {
              pmd
                .setProjectSpaceAllocation(ProjectSpaceAllocation.WAITING_ALLOCATION);
            }
            break;
          }
        }
        continue;
      }
    }
  }

}
