package csbase.client.applications.projectsmanager.proxy;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;

import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.models.ProjectSpaceAllocation;
import csbase.client.applications.projectsmanager.proxy.core.ProjectsManagerTask;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.ProjectServiceInterface;

/**
 * Task para modificao do status de alocao de projetos.
 * 
 * @author jnlopes
 */
public class StatusModificationTask extends ProjectsManagerTask<String[]> {

  /**
   * Nomes dos projetos.
   */
  final private String[] projectNames;

  /**
   * Lista dos ids dos donos dos projetos.
   */
  final private String[] ownerIds;

  /**
   * Status de alocao do projeto.
   */
  final private ProjectSpaceAllocation targetAllocationStatus;

  /**
   * Espao (em GB) que deve ser reservado ao projeto.
   */
  private int reservedSpace;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param projectNames Nomes dos projetos.
   * @param ownerIds ids dos donos dos projetos
   * @param targetAllocationStatus Status de alocao do projeto.
   * @param reservedSpace Espao (em GB) que foi alocado ao projeto.
   */
  public StatusModificationTask(final ProjectsManager projectsManager,
    final String[] projectNames, final String[] ownerIds,
    final ProjectSpaceAllocation targetAllocationStatus, final int reservedSpace) {
    super(projectsManager);
    this.projectNames = projectNames;
    this.ownerIds = ownerIds;
    this.targetAllocationStatus = targetAllocationStatus;
    this.reservedSpace = reservedSpace;
  }

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param projectNames Nomes dos projetos.
   * @param ownerIds ids dos donos dos projetos
   */
  public StatusModificationTask(final ProjectsManager projectsManager,
    final String[] projectNames, final String[] ownerIds) {
    super(projectsManager);
    this.projectNames = projectNames;
    this.ownerIds = ownerIds;
    this.targetAllocationStatus = ProjectSpaceAllocation.NO_ALLOCATION;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void performTask() throws Exception {

    final List<String> list = new ArrayList<String>();
    final ProjectServiceInterface psi = ClientRemoteLocator.projectService;

    for (int i = 0; i < projectNames.length; i++) {
      try {
        if (targetAllocationStatus == ProjectSpaceAllocation.NO_ALLOCATION) {
          psi.setAllocatedProjectAsUnallocated(ownerIds[i], projectNames[i]);
        }
        else {
          if (targetAllocationStatus == ProjectSpaceAllocation.ALLOCATED) {
            psi.setUnallocatedProjectAsAllocated(ownerIds[i], projectNames[i],
              reservedSpace);
          }
          else {
            psi.setUnallocatedProjectAsLocked(ownerIds[i], projectNames[i],
              reservedSpace);
          }
        }
      }
      catch (RemoteException re) {
        list.add(projectNames[i]);
      }
    }

    if (list.size() > 0) {
      setResult(list.toArray(new String[list.size()]));
    }
    else {
      setResult(null);
    }
  }

}
