/*
 * $Id: DesktopFrame.java 105668 2010-05-17 20:38:53Z jnlopes $
 */
package csbase.client.desktop;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Image;
import java.awt.SystemTray;
import java.awt.TrayIcon;
import java.awt.Window;
import java.awt.TrayIcon.MessageType;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;
import java.rmi.RemoteException;
import java.util.Locale;
import java.util.Map;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JSplitPane;
import javax.swing.KeyStroke;

import tecgraf.javautils.LNG;
import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.configurationmanager.ConfigurationManagerException;
import tecgraf.javautils.gui.GUIUtils;
import csbase.client.ClientServerManager;
import csbase.client.ClientUI;
import csbase.client.applicationmanager.ApplicationException;
import csbase.client.applicationmanager.ApplicationManager;
import csbase.client.applications.executor.ExecutorFrame;
import csbase.client.desktop.dircontents.DirectoryContentsPanel;
import csbase.client.ias.AdminFrame;
import csbase.client.kernel.ClientException;
import csbase.client.login.InitialContext;
import csbase.client.openbus.OpenBusAccessPoint;
import csbase.client.project.ProjectPreferences;
import csbase.client.project.ProjectTree;
import csbase.client.project.ProjectTreeAdapter;
import csbase.client.project.action.ShowTreeFilterAction;
import csbase.client.project.dialogs.ProjectBuildingDialog;
import csbase.client.project.tasks.OpenProjectOnTreeRemoteTask;
import csbase.client.remote.AdministrationObserver;
import csbase.client.remote.AlgorithmManagementObserver;
import csbase.client.remote.ClientRemoteMonitor;
import csbase.client.remote.ClientRemoteMonitorListener;
import csbase.client.remote.CommandObserver;
import csbase.client.remote.ProjectAdminObserver;
import csbase.client.remote.SchedulerObserver;
import csbase.client.remote.manager.trustedserver.TrustedServerManager;
import csbase.client.remote.srvproxies.EventLogProxy;
import csbase.client.remote.srvproxies.NotificationProxy;
import csbase.client.util.StandardErrorDialogs;
import csbase.exception.ConfigurationException;
import csbase.logic.CommonClientProject;
import csbase.logic.MonitoredServerListener;
import csbase.logic.Session;
import csbase.logic.SharedObject;
import csbase.logic.User;
import csbase.remote.ClientRemoteLocator;

/**
 * A classe <code>DesktopFrame</code> implementa a interface principal do
 * sistema no lado cliente.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class DesktopFrame implements ClientUI {

  /**
   * Nome da propriedade que indica se o desktop deve comear maximizado.
   */
  private static final String MAXIMIZE_PROP = "desktop.maximize";

  /**
   * Nome da categoria para o SharedObject que armazena as preferncias do
   * usurio.
   */
  private static final String PREFERENCE_CATEGORY = "UserPreferences";

  /**
   * Nome do SharedObject que armazena as preferncias do desktop.
   */
  private static final String DESKTOP_PREFERENCE = "DesktopPreferences";

  /**
   * Nome do SharedObject que armazena as preferncias do projeto.
   */
  private static final String PROJECT_PREFERENCE = "ProjectPreferences";

  /**
   * Atributo da sesso que representa o nome de um projeto a ser aberto ao
   * iniciar o desktop.
   */
  private static final String PROJECT_NAME_ATTRIBUTE = "project_name";

  /**
   * Atributo da sesso que representa o nome de um algoritmo a ser executado ao
   * iniciar o desktop.
   */
  private static final String ALGORITHM_NAME_ATTRIBUTE = "algorithm_name";

  /**
   * Atributo da sesso que representa os parmetros de um algoritmo a ser
   * executado ao iniciar o desktop.
   */
  private static final String ALGORITHM_PARAMETERS_ATTRIBUTE =
    "algorithm_parameters";

  /**
   * Fila de log de aes do cliente
   */
  private static String[] desktopQueue = new String[] { "Desktop" };

  /**
   * Indicao default se a rvore de projetos deve ser atualizada
   * automaticamente.
   */
  private static boolean AUTO_UPDATE = true;

  /**
   * Indicao default se a rvore de projetos deve exibir um filtro.
   */
  private static boolean TREE_FILTER = false;

  /**
   * Componente responsvel pela rea de aplicaes.
   */
  private static DesktopFrame instance = null;

  /**
   * Componente que indica como se a atualizao da rvore de projetos deve ser
   * automtica.
   */
  private static JCheckBoxMenuItem projectAutoUpdateItem;

  /**
   * Componente que indica a rvore deve exibir ou no um filtro dos seus ns.
   */
  private static JCheckBoxMenuItem showTreeFilterItem;

  /**
   * A janela de administrao do sistema.
   */
  protected AdminFrame adminFrame;

  /**
   * Painel que exibe o contedo do diretrio selecionado na rvore
   */
  protected DirectoryContentsPanel dirContentsPanel;

  /**
   * Painel de notificaes.  protected porque sua inicializao pode ser
   * sobrescrita pelas subclasses.
   */
  protected NotificationPanel notifPanel;

  /**
   * Painel da rvore de projetos.
   */
  protected JComponent projectTreePanel;

  /**
   * JSplitPane que separa rvore de projetos e a tabela com os arquivos.
   */
  protected JSplitPane splitPaneBetweenTreeAndTable;

  /**
   * JSplitPane que separa rvore de projetos e os frames das aplicaes.
   */
  protected JSplitPane splitPaneBetweenTreeAndApp;

  /**
   * JSplitPane que separa os frames das aplicaes e o painel de notificaes.
   */
  protected JSplitPane splitPaneBetweenAppAndNotif;

  /**
   * A rvore de diretrios do projeto aberto
   */
  protected ProjectTree projectTree;

  /**
   * Vetor de janelas
   */
  private Vector<DesktopWindowInterface> dependentWindows = null;

  /**
   * A janela onde essa interface  apresentada.
   */
  private DesktopComponentFrame mainFrame = null;

  /**
   * Este Listener recebe as notificaes quando a conexo com o servidor
   * perdida ou restabelecida. Exibe mensagens nessas situaes.
   */
  protected MonitoredServerListener connMonitorListener;

  /**
   * Imagem de erro
   */
  private Image errorImage = null;

  /**
   * Gerente de configuraes. Usado para obteno de propriedades desta classe.
   */
  private Configuration configuration;

  /**
   * Preferncias do desktop do usurio logado.
   */
  private DesktopPreferences desktopPreferences;

  /**
   * Preferncias do projeto do usurio logado.
   */
  private ProjectPreferences projectPreferences;

  /**
   * Painel de filtro para a rvore de projetos.
   */
  protected TreeFilterPanel treeFilterPanel;

  /**
   * Indica se arquivos ocultos (que comeam com ".") devem ser exibidos.
   */
  private boolean showHiddenFiles;

  /**
   * Encerra o cliente fechando todas as janelas dependentes do tipo
   * DesktopComponentFrame.
   */
  private void shutdownDependents() {
    int base = 0;
    DesktopWindowInterface lastWindow = null;
    while (dependentWindows.size() > 0) {
      DesktopWindowInterface window = dependentWindows.get(base);
      if (window == lastWindow) {
        base++;
        continue;
      }
      window.close();
      lastWindow = window;
    }
  }

  /**
   * Mtodo para ajustar o ttulo das janelas dependentes do tipo
   * DesktopComponentFrame.
   * 
   * 
   * @param title ttulo
   */
  public String adjustTitle(String title) {
    return title;
  }

  /**
   * Mtodo para atribuir o ttulo do desktop.
   * 
   * @param title ttulo
   */
  protected final void setTitle(String title) {
    this.mainFrame.setTitle(title);
  }

  /**
   * Mtodo para consultar o ttulo do desktop.
   * 
   * @return ttulo
   */
  public String getTitle() {
    return this.mainFrame.getTitle();
  }

  /**
   * Mtodo para atribuir a imagem do desktop frame.
   * 
   * @param image a imagem da janela do desktop.
   */
  public void setFrameImage(Image image) {
    this.mainFrame.setIconImage(image);
  }

  /**
   * Obtm imagem de erro.
   * 
   * @return a imagem
   */
  public Image getDesktopErrorImage() {
    return this.errorImage;
  }

  /**
   * Configura a imagem de erro.
   * 
   * @param image a imagem
   */
  public void setDesktopErrorImage(Image image) {
    this.errorImage = image;
  }

  /**
   * Mtodo para consultar a imagem do desktop frame.
   * 
   * @return a imagem.
   */
  public Image getFrameImage() {
    return this.mainFrame.getIconImage();
  }

  /**
   * Mtodo para atribuir o menu bar da janela.
   * 
   * @param menuBar o menu
   */
  public void setMenuBar(JMenuBar menuBar) {
    this.mainFrame.setJMenuBar(menuBar);
  }

  /**
   * Mtodo para consultar o menu bar da janela.
   * 
   * @return menubar o menu
   */
  public JMenuBar getMenuBar() {
    final JMenuBar menuBar = this.mainFrame.getJMenuBar();
    return menuBar;
  }

  /**
   * Encerra a aplicao cliente principal.
   */
  public void shutdownDesktop() {
    final ClientRemoteMonitor clientRemoteMonitor =
      ClientRemoteMonitor.getInstance();

    clientRemoteMonitor.deleteListener(connMonitorListener);

    /* Termina todas as aplicaes iniciadas dentro do Desktop */
    EventLogProxy.addClientInformation(desktopQueue, "logout");

    final ApplicationManager appManager = ApplicationManager.getInstance();
    appManager.finishAllApplications();

    shutdownDependents();
    stopServicesObservers();
    // Guardas as configuraes do desktop do usurio
    if (clientRemoteMonitor.isAlive()) {
      saveDesktopPreferences();
      ClientServerManager.getInstance().shutdown();
    }
    mainFrame.setVisible(false);
    mainFrame.close();

    OpenBusAccessPoint oap = OpenBusAccessPoint.getInstance();
    if (oap.isInited()) {
      if (oap.isConnected()) {
        oap.disconnect();
      }
      oap.shutdown();
    }

    synchronized (instance) {
      instance = null;
    }
    try {
      System.exit(0);
    }
    catch (Throwable t) {
    } // Applet
  }

  /**
   * Volta s configuraes originais do desktop.
   */
  protected void restoreOriginalPreferences() {
    splitPaneBetweenTreeAndTable.resetToPreferredSizes();
    splitPaneBetweenTreeAndApp.resetToPreferredSizes();
    splitPaneBetweenAppAndNotif.resetToPreferredSizes();
    final Boolean mx =
      configuration.getOptionalBooleanProperty(MAXIMIZE_PROP, false);
    final boolean maximize = mx.booleanValue();
    if (maximize) {
      maximizeDesktop();
    }
    else {
      setDefaultDesktopSize();
    }
  }

  /**
   * Ajusta o desktop de acordo com as preferncias.
   */
  protected void setPreferences() {
    DesktopPreferences pref = getDesktopPreferences();
    if (pref != null) {
      splitPaneBetweenTreeAndTable.setDividerLocation(pref
        .getSPaneBetweenTreeAndTableLocation());
      splitPaneBetweenTreeAndApp.setDividerLocation(pref
        .getSPaneBetweenTreeAndAppLocation());
      splitPaneBetweenAppAndNotif.setDividerLocation(pref
        .getSPaneBetweenAppAndNotifLocation());
      if (pref.isMaximized()) {
        maximizeDesktop();
      }
      else {
        setDesktopSize(pref.getDesktopDimension());
      }
      showHiddenFiles = pref.shouldShowHiddenFiles();
    }
    else {
      // Est usando o sistema pela primeira vez
      final Boolean mx =
        configuration.getOptionalBooleanProperty(MAXIMIZE_PROP, false);
      final boolean maximize = mx.booleanValue();
      if (maximize) {
        maximizeDesktop();
      }
      else {
        setDefaultDesktopSize();
      }
      showHiddenFiles = false;
    }
  }

  /**
   * Salva as configuraes do desktop do usurio
   */
  private void saveDesktopPreferences() {
    boolean maximized =
      (mainFrame.getExtendedState() == JFrame.MAXIMIZED_BOTH) ? true : false;
    desktopPreferences =
      new DesktopPreferences(splitPaneBetweenTreeAndTable.getDividerLocation(),
        splitPaneBetweenTreeAndApp.getDividerLocation(),
        splitPaneBetweenAppAndNotif.getDividerLocation(), mainFrame.getSize(),
        maximized, showHiddenFiles);

    final SharedObject shared =
      new SharedObject(PREFERENCE_CATEGORY, User.getLoggedUser().getId(),
        DESKTOP_PREFERENCE, false, desktopPreferences.getContent());

    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      protected void performTask() throws Exception {
        ClientRemoteLocator.sharedObjectService.saveSharedObject(shared);
      }
    };
    task.execute(mainFrame, getTitle(), LNG
      .get("DesktopFrame.save.restore.preferences"));
  }

  /**
   * Obtm as preferncias do usurio para configurao do desktop.
   * 
   * @return as preferncias do usurio
   */
  protected DesktopPreferences getDesktopPreferences() {
    if (desktopPreferences == null) {
      RemoteTask<SharedObject> task = new RemoteTask<SharedObject>() {
        @Override
        protected void performTask() throws Exception {
          setResult(ClientRemoteLocator.sharedObjectService.getSharedObject(
            PREFERENCE_CATEGORY, User.getLoggedUser().getId(),
            DESKTOP_PREFERENCE));
        }
      };
      boolean success =
        task.execute(mainFrame, getTitle(), LNG
          .get("DesktopFrame.get.restore.preferences"));
      if (success) {
        SharedObject shared = task.getResult();
        if (shared != null) {
          final Object contents = shared.getContents();
          desktopPreferences =
            new DesktopPreferences((Map<String, Object>) contents);
        }
      }
    }
    return desktopPreferences;
  }

  /**
   * Salva a opo de atualizao automtica da rvore de projetos.
   */
  private void saveProjectPreferences() {
    boolean autoUpdate = AUTO_UPDATE;
    boolean treeFilter = TREE_FILTER;
    if (projectAutoUpdateItem != null)
      autoUpdate = projectAutoUpdateItem.isSelected();
    if (showTreeFilterItem != null)
      treeFilter = showTreeFilterItem.isSelected();
    projectPreferences = new ProjectPreferences(autoUpdate, treeFilter);
    final SharedObject shared =
      new SharedObject(PREFERENCE_CATEGORY, User.getLoggedUser().getId(),
        PROJECT_PREFERENCE, false, projectPreferences.getContent());
    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      protected void performTask() throws Exception {
        ClientRemoteLocator.sharedObjectService.saveSharedObject(shared);
      }
    };
    task.execute(mainFrame, getTitle(), LNG
      .get("DesktopFrame.save.project.preferences"));
  }

  /**
   * Obtm as preferncias do usurio para atualizao automtica da rvore de
   * projetos.
   * 
   * @return as preferncias do usurio
   */
  protected ProjectPreferences getProjectPreferences() {
    if (projectPreferences == null) {
      RemoteTask<SharedObject> task = new RemoteTask<SharedObject>() {
        @Override
        protected void performTask() throws Exception {
          setResult(ClientRemoteLocator.sharedObjectService.getSharedObject(
            PREFERENCE_CATEGORY, User.getLoggedUser().getId(),
            PROJECT_PREFERENCE));
        }
      };
      boolean success =
        task.execute(mainFrame, getTitle(), LNG
          .get("DesktopFrame.get.project.preferences"));
      if (success) {
        SharedObject shared = task.getResult();
        if (shared != null) {
          final Object contents = shared.getContents();
          projectPreferences =
            new ProjectPreferences((Map<String, Object>) contents);
        }
      }
    }
    // Tentou carregar o sharedObject e no conseguiu
    if (projectPreferences == null) {
      projectPreferences = new ProjectPreferences(AUTO_UPDATE, TREE_FILTER);
    }
    return projectPreferences;
  }

  /**
   * Indica se a rvore de projetos deve ser autualizada automaticamente.
   * 
   * @return true se a rvore deve ser atualizada.
   */
  public boolean mustUpdateProjectTree() {
    return getProjectPreferences().getAutoUpdate();
  }

  /**
   * Registra uma janela cliente na janela principal. Ao fechar a janela
   * pricipal, todas as janelas registradas tambm so fechadas.
   * 
   * @param win Uma nova janela registrada
   */
  public void addWindow(DesktopWindowInterface win) {
    dependentWindows.add(win);
  }

  /**
   * Descadastra uma janela da cliente janela principal.
   * 
   * @param win A janela que deve ser removida da janela principal.
   */
  public void removeWindow(DesktopWindowInterface win) {
    dependentWindows.remove(win);
  }

  /**
   * Consulta  janela Java (JFrame)
   * 
   * @return .
   */
  public DesktopComponentFrame getDesktopFrame() {
    return mainFrame;
  }

  /**
   * Verifica se o usuario logado possui projetos
   * 
   * @return <code>true</code> se o usurio possui projetos
   */
  public boolean userHasProject() {
    RemoteTask<Boolean> task = new RemoteTask<Boolean>() {
      @Override
      protected void performTask() throws Exception {
        setResult(CommonClientProject.userHasHisOwnProjects(User
          .getLoggedUser().getId()));
      }
    };

    if (!task.execute(getDesktopFrame(), getTitle(), LNG
      .get("DesktopFrame.info.checkIfUserHasProject"))) {
      // o erro j foi exibido
      return false;
    }
    return task.getResult();
  }

  /**
   * Verifica se o usuario logado participa de projetos de outros usurios
   * 
   * @return <code>true</code> se o usurio participa de projetos de outros
   *         usurios
   */
  public boolean userHasOthersProject() {
    try {
      final User loggedUser = User.getLoggedUser();
      return CommonClientProject.userParticipatesOnSharedProjects(loggedUser
        .getId());
    }
    catch (Exception e) {
      StandardErrorDialogs.showErrorDialog(getDesktopFrame(), getTitle(), e);
    }
    return false;
  }

  /**
   * Pendura um listener na rvore de projetos, que ir verificar, a cada vez
   * que um projeto for removido, se o usurio ainda possui projetos. Se o
   * projeto removido for o ltimo do usurio, o item especificado ser
   * removido.
   * 
   * @param item componente da interface, geralmente um menu ou um boto.
   */
  protected void enableIfProject(final JComponent item) {
    projectTree.addProjectTreeListener(new ProjectTreeAdapter() {
      @Override
      public void projectRemoved() {
        item.setEnabled(userHasProject());
      }
    });
  }

  /**
   * Pendura um listener para na rvore de projetos que ser notificado a cada
   * vez que um projeto for aberto ou fechado. Se for um evento de abertura de
   * projeto, o item especificado ser habilitado.
   * 
   * @param item item componente da interface, geralmente um menu ou um boto.
   */
  protected void enableIfProjectOpened(final JComponent item) {
    projectTree.addProjectTreeListener(new ProjectTreeAdapter() {
      @Override
      public void projectChanged(CommonClientProject project) {
        item.setEnabled(project != null);
      }
    });
  }

  /**
   * Habilita o gerencimento do projeto se o usurio logado for dono do mesmo ou
   * se for o administrador.
   * 
   * @param item .
   */
  protected void enableIfAdminProject(final JMenuItem item) {
    projectTree.addProjectTreeListener(new ProjectTreeAdapter() {
      @Override
      public void projectChanged(CommonClientProject project) {
        item.setEnabled((project != null)
          && (project.getUserId().equals(User.getLoggedUser().getId()) || User
            .getLoggedUser().isAdmin()));
      }
    });
  }

  /**
   * Consulta ao desktop criado na aplicao cliente.
   * 
   * @return .
   */
  public static DesktopFrame getInstance() {
    return instance;
  }

  /**
   * Constri e apresenta os componentes da interface principal do cliente.
   */
  public void showDesktop() {
    setPreferences();

    // Exibio
    mainFrame.setVisible(true);

    EventLogProxy.addClientInformation(desktopQueue, "login");
  }

  /**
   * Seta o tamanho da janela. Este mtodo deve ser sobrescrito em projetos com
   * requisitos especficos quanto ao tamanho da janela.
   */
  private void setDefaultDesktopSize() {
    final int defaultWidth = 800;
    final int defaultHeight = 600;
    final Integer widthStr =
      configuration.getOptionalIntegerProperty("desktop.width", defaultWidth);
    final Integer heightStr =
      configuration.getOptionalIntegerProperty("desktop.height", defaultHeight);
    final int w = widthStr.intValue();
    final int h = heightStr.intValue();
    setDesktopSize(new Dimension(w, h));
  }

  /**
   * Seta o tamanho da janela. Este mtodo deve ser sobrescrito em projetos com
   * requisitos especficos quanto ao tamanho da janela.
   * 
   * @param dimension Dimenso da janela
   */
  private void setDesktopSize(Dimension dimension) {
    mainFrame.pack();
    mainFrame.setExtendedState(Frame.NORMAL);
    mainFrame.setSize(dimension);
    mainFrame.validate();
    // Centralizao
    GUIUtils.centerOnScreen(mainFrame);
  }

  /**
   * Maximiza o desktop.
   */
  private void maximizeDesktop() {
    mainFrame.pack();
    mainFrame.setExtendedState(Frame.MAXIMIZED_BOTH);
    // usamos validate() para garantir que mudanas no tamanho (p.ex.
    // maximize) sejam consideradas antes do frame ser exibido
    mainFrame.validate();
  }

  /**
   * Obtm o projeto que est aberto na janela principal da ISI.
   * 
   * @return O projeto corrente selecionado na janela principal da ISI.
   */
  public CommonClientProject getProject() {
    return this.projectTree.getProject();
  }

  /**
   * Obtm a rvore do projeto aberto.
   * 
   * @return A rvore do projeto aberto.
   */
  public ProjectTree getTree() {
    return projectTree;
  }

  /**
   * Inicializa os observadores do usurio, dos projetos do usurio e dos
   * algoritmos
   * 
   * @throws ClientException Caso ocorra falha na inicializao dos observadores
   */
  protected void startServicesObservers() throws ClientException {
    try {
      SchedulerObserver.start();
      CommandObserver.start();
      AdministrationObserver.start();
      ProjectAdminObserver.start();
      AlgorithmManagementObserver.start();
      if (TrustedServerManager.getInstance().isEnabled()) {
        TrustedServerManager.getInstance().start();
      }
    }
    catch (Exception e) {
      throw new ClientException(e);
    }
  }

  /**
   * Remove os observadores do usurio, dos projetos do usurio e dos algoritmos
   */
  protected void stopServicesObservers() {
    try {
      SchedulerObserver.stop();
      CommandObserver.stop();
      AdministrationObserver.stop();
      ProjectAdminObserver.stop();
      AlgorithmManagementObserver.stop();
      if (TrustedServerManager.getInstance().isEnabled()) {
        TrustedServerManager.getInstance().stop();
      }
    }
    catch (Exception e) {
      System.out.println("Falha ao remover os observadores: " + e.getMessage());
    }
  }

  /**
   * Constri a interface principal do cliente.
   * 
   * @throws ClientException Caso ocorra falha na construo do cliente
   * @throws ConfigurationException Caso ocorra falha na obteno do gerente de
   *         configurao.
   */
  public DesktopFrame() throws ClientException {
    if (instance != null) {
      throw new ClientException("Double Desktop!");
    }

    final Locale locale = LNG.getLocale();
    final RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      protected void performTask() throws RemoteException {
        ApplicationManager.setInstance(locale);
      }
    };
    final DesktopComponentFrame frm = this.getDesktopFrame();
    final boolean executed = task.execute(frm, "ApplicationService", "");
    if (!executed) {
      final Exception excpt = task.getError();
      throw new ClientException("ApplicationManager error!", excpt);
    }

    try {
      final ConfigurationManager manager = ConfigurationManager.getInstance();
      this.configuration = manager.getConfiguration(this.getClass());
    }
    catch (ConfigurationManagerException e) {
      throw new ConfigurationException(e);
    }
    dependentWindows = new Vector<DesktopWindowInterface>();
    mainFrame = new DesktopComponentFrame();
    mainFrame.setPreferredSize(new Dimension(800, 600));
    mainFrame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    instance = this;
    startServicesObservers();

    // Coloca o cone do desktop
    String iPath = this.configuration.getOptionalProperty("iconPath");
    if (iPath != null) {
      URL res = getClass().getResource(iPath);
      Image image = new ImageIcon(res).getImage();
      setFrameImage(image);
    }
  }

  /**
   * {@inheritDoc} Cadastra o listener que observa a conexo do cliente com o
   * servidor no <code>ClientRemoteMonitor</code>.
   */
  public void preInitialization(InitialContext initialContext) {
    connMonitorListener = new ClientRemoteMonitorListener(mainFrame);
    ClientServerManager.getInstance().addCommonListener(connMonitorListener);
  }

  /**
   * {@inheritDoc}
   */
  public void showUI() {
    this.showDesktop();
  }

  /**
   * {@inheritDoc}
   */
  public void postInitialization() {
    if (this.loadProject()) {
      this.executeAlgorithm();
    }
  }

  /**
   * {@inheritDoc}
   */
  public JFrame getView() {
    return this.getDesktopFrame();
  }

  /**
   * Executa um algoritmo que est definido na sesso do usurio.
   */
  private void executeAlgorithm() {
    final ClientRemoteMonitor clientRemoteMonitor =
      ClientRemoteMonitor.getInstance();
    final Session session = clientRemoteMonitor.getSession();
    final String algorithmName =
      (String) session.getAttribute(ALGORITHM_NAME_ATTRIBUTE);
    if (algorithmName == null) {
      return;
    }

    Map<String, String> algorithmParameters =
      (Map<String, String>) session
        .getAttribute(ALGORITHM_PARAMETERS_ATTRIBUTE);
    try {
      final ApplicationManager applicationManager =
        ApplicationManager.getInstance();
      final ExecutorFrame executorFrame =
        (ExecutorFrame) applicationManager.runApplication("executor");
      executorFrame.showConfigurator(algorithmName, algorithmParameters);
    }
    catch (ApplicationException e) {
      StandardErrorDialogs.showErrorDialog(this.getDesktopFrame(), e);
    }
  }

  /**
   * Carrega um projeto que est definido na sesso do usurio.
   * 
   * @return true, caso o projeto tenha sido carregado, ou false, caso
   *         contrrio.
   */
  private boolean loadProject() {
    String projectName =
      (String) ClientRemoteMonitor.getInstance().getSession().getAttribute(
        PROJECT_NAME_ATTRIBUTE);
    if (projectName == null) {
      return false;
    }
    this.openProject(projectName);
    return true;
  }

  /**
   * Abre o projeto.
   * 
   * @param projectName nome do projeto
   * @return o projeto aberto
   */
  public CommonClientProject openProject(String projectName) {
    final OpenProjectOnTreeRemoteTask openProjectTask =
      new OpenProjectOnTreeRemoteTask(projectName,
        User.getLoggedUser().getId(), this.projectTree);
    if (openProjectTask.execute(getDesktopFrame(), LNG
      .get("PRJ_WAITING_OPEN_PROJECT"), LNG.get("PRJ_WAITING_OPEN_PROJECT"))) {
      final CommonClientProject result = openProjectTask.getResult();
      return result;
    }
    return null;
  }

  /**
   * Constri o item de menu para restaurar as configuraes do desktop.
   * 
   * @return o item de menu para restaurar as configuraes do desktop
   */
  protected JMenuItem createRestoreDesktopItem() {
    final String restoreText = LNG.get("DesktopFrame.menu.restore.preferences");
    final JMenuItem restoreItem = new JMenuItem(restoreText);
    restoreItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        restoreOriginalPreferences();
      }
    });
    return restoreItem;
  }

  /**
   * Constri o item de menu para configurar a atualizao automtica do
   * projeto.
   * 
   * @return .
   */
  protected JMenuItem createProjectAutoUpdateItem() {
    projectAutoUpdateItem =
      new JCheckBoxMenuItem(LNG.get("DesktopFrame.menu.project.preferences"),
        getProjectPreferences().getAutoUpdate());
    projectAutoUpdateItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        saveProjectPreferences();
        if (projectAutoUpdateItem.isSelected() && getProject() != null) {
          RemoteTask<Void> task = new RemoteTask<Void>() {
            @Override
            protected void performTask() throws Exception {
              DesktopFrame.this.getProject().refreshTree();
            }
          };
          task.execute(mainFrame, getTitle(), LNG
            .get("DesktopFrame.refresh.project.tree"));
        }
      }
    });
    projectAutoUpdateItem.setToolTipText(LNG
      .get("DesktopFrame.menu.project.preferences.tooltip"));
    return projectAutoUpdateItem;
  }

  /**
   * Constri o item de menu para configurar a exibio de um filtro dos ns da
   * rvore de projetos
   * 
   * @return o tem de menu que permite habilitar ou desabilitar a exibio do
   *         filtro
   */
  protected JMenuItem createShowTreeFilterItem() {
    /*
     * precisamos do lock esttico sobre a classe porque se neste ponto
     * showTreeFilter = null o synchronized() abaixo lanaria NPE
     */
    synchronized (DesktopFrame.class) {
      showTreeFilterItem =
        new JCheckBoxMenuItem(new ShowTreeFilterAction(treeFilterPanel));
      String showTreeAccelerator =
        LNG.get("DesktopFrame.menu.project.filter.accelerator");
      if (null != showTreeAccelerator) {
        showTreeFilterItem.setAccelerator(KeyStroke
          .getKeyStroke(showTreeAccelerator));
      }
      boolean isSelected = getProjectPreferences().getTreeFilter();
      showTreeFilterItem.setSelected(isSelected);
      treeFilterPanel.setVisible(isSelected);
    }
    treeFilterPanel.addListener(new TreeFilterPanelListener() {
      public void visibilityChanged(boolean panelVisible) {
        saveProjectPreferences();
        synchronized (showTreeFilterItem) {
          showTreeFilterItem.setSelected(panelVisible);
        }
      }
    });
    return showTreeFilterItem;
  }

  /**
   * Cria o painel de notificaes.
   * 
   * @return o painel de notificaes
   */
  protected JComponent createNotificationPanel() {
    try {
      NotificationProxy.resetObservers();
    }
    catch (Exception e) {
      StandardErrorDialogs.showErrorDialog(getDesktopFrame(), getTitle(), e);
    }
    this.notifPanel = new BasicNotificationPanel();
    notifPanel.setUser(User.getLoggedUser());
    notifPanel.addNotificationHandler(CommandNotificationHandler.getInstance());
    notifPanel
      .addNotificationHandler(CommandPersistenceCommandNotificationHandler
        .getInstance());
    return notifPanel.getPanel();
  }

  /**
   * Cria o painel de detalhes dos arquivos de um diretrio.
   * 
   * @return o painel de detalhes
   */
  protected JComponent createDirectoryContentsPanel() {
    dirContentsPanel =
      new DirectoryContentsPanel(getDesktopFrame(), getTitle(), projectTree);
    return dirContentsPanel;
  }

  /**
   * Cria o painel de filtro para a rvore de projetos.
   * 
   * @return painel de filtro para a rvore de projetos.
   */
  protected JComponent createTreeFilterPanel() {
    treeFilterPanel =
      TreeFilterPanel.getInstance(projectTree, dirContentsPanel);
    treeFilterPanel.setVisible(false);
    return treeFilterPanel;
  }

  /**
   * Obtm a configurao da classe.
   * 
   * @return a configurao da classe
   */
  protected final Configuration getConfiguration() {
    return this.configuration;
  }

  /**
   * Define se os arquivos ocultos devem ser exibidos.
   * 
   * @param showHiddenFiles - true se os arquivos devem ser exibidos
   */
  public void setShowHiddenFiles(boolean showHiddenFiles) {
    this.showHiddenFiles = showHiddenFiles;
  }

  /**
   * @return true se os arquivos ocultos devem ser exibidos
   */
  public boolean shouldShowHiddenFiles() {
    return showHiddenFiles;
  }

  /**
   * Mtodo que retorna o tray icon do sistema (se quiser). Por default,
   * retorna-se <code>null</code> (sem tray). Ao redefinir o metodo no sistema,
   * deve-se verificar so o S.O. possui tal funcionalidade (mtodo
   * {@link SystemTray#isSupported()})
   * 
   * @return o tray icon (ou null) se o desktop no tiver tray.
   */
  protected TrayIcon getTrayIcon() {
    return null;
  }

  /**
   * @param message mensagem
   */
  final public void displayTrayInfoMessage(final String message) {
    displayTrayMessage(message, MessageType.INFO);
  }

  /**
   * @param message mensagem
   */
  final public void displayTrayWarningMessage(final String message) {
    displayTrayMessage(message, MessageType.WARNING);
  }

  /**
   * @param message mensagem
   */
  final public void displayTrayErrorMessage(final String message) {
    displayTrayMessage(message, MessageType.ERROR);
  }

  /**
   * Exibe uma mensagem de tray
   * 
   * @param message mensagem
   * @param type tipo
   */
  private void displayTrayMessage(final String message, MessageType type) {
    final TrayIcon trayIcon = getTrayIcon();
    if (trayIcon == null) {
      return;
    }
    final String systemName = getTitle();
    trayIcon.displayMessage(systemName, message, type);
  }

  /**
   * Abre o dilogo de criao de projeto.
   * 
   * @param window Janela pai do dilogo.
   * @return O projeto criado.
   */
  public CommonClientProject openProjectCreationDialog(final Window window) {
    return ProjectBuildingDialog.createProject(window, null);
  }

  /**
   * Abre o dilogo de update de projeto.
   * 
   * @param window Janela pai do dilogo.
   * @param project O projeto a ser alterado.
   * @return O projeto criado.
   */
  public CommonClientProject openProjectUpdateDialog(final Window window,
    final CommonClientProject project) {
    return ProjectBuildingDialog.updateProject(window, project, null);
  }

}
