/*
 * $Id$
 */
package csbase.client.desktop;

import java.awt.Dimension;
import java.util.HashMap;
import java.util.Map;

/**
 * Classe que representa as preferncias da configurao do desktop do usurio.
 * Possui as posies dos JSplitPanes e o tamanho da janela principal. Para
 * persistir os dados desta classe em um <code>SharedObject</code>, usar o
 * mtodo <code>getContent</code>.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class DesktopPreferences {

  /**
   * Nome da preferncia:
   */
  private final static String SPANE_TREE = "SPANE_TREE";

  /**
   * Nome da preferncia:
   */
  private final static String SPANE_APP = "SPANE_APP";

  /**
   * Nome da preferncia:
   */
  private final static String SPANE_NOTIF = "SPANE_NOTIF";

  /**
   * Nome da preferncia:
   */
  private final static String DIMENSION = "DIMENSION";

  /**
   * Nome da preferncia:
   */
  private final static String MAXIMIZED = "MAXIMIZED";

  /**
   * Nome da preferncia "showHiddenFiles".
   * 
   * @see #showHiddenFiles
   */
  private final static String SHOW_HIDDEN_FILES = "SHOW_HIDDEN_FILES";

  /**
   * Atributo da preferncia:
   * 
   * @see #SPANE_TREE
   */
  private int sPaneBetweenTreeAndTableLocation;

  /**
   * Atributo da preferncia:
   * 
   * @see #SPANE_APP
   */
  private int sPaneBetweenTreeAndAppLocation;

  /**
   * Atributo da preferncia:
   * 
   * @see #SPANE_NOTIF
   */
  private int sPaneBetweenAppAndNotifLocation;

  /**
   * Atributo da preferncia:
   * 
   * @see #DIMENSION
   */
  private Dimension desktopDimension;

  /**
   * Atributo da preferncia:
   * 
   * @see #MAXIMIZED
   */
  private boolean maximized;

  /**
   * Indica se os arquivos com nomes comeando com "." devem ser exibidos.
   * 
   * @see #SHOW_HIDDEN_FILES
   */
  private boolean showHiddenFiles = false;

  /**
   * Contri o <code>DesktopPreferences</code> a partir do mapa de valores
   * lido do <code>SharedObject</code>.
   * 
   * @param preferences mapa de valores
   */
  DesktopPreferences(Map<String, Object> preferences) {
    this.sPaneBetweenTreeAndTableLocation =
      (Integer) preferences.get(SPANE_TREE);
    this.sPaneBetweenTreeAndAppLocation = (Integer) preferences.get(SPANE_APP);
    this.sPaneBetweenAppAndNotifLocation =
      (Integer) preferences.get(SPANE_NOTIF);
    this.desktopDimension = (Dimension) preferences.get(DIMENSION);
    this.maximized =
      (Boolean) getPrefOrDefault(preferences, MAXIMIZED, Boolean.FALSE);
    this.showHiddenFiles =
      (Boolean) getPrefOrDefault(preferences, SHOW_HIDDEN_FILES, Boolean.FALSE);
  }

  /**
   * Retorna o valor de uma das preferncias de um mapa, ou um valor default
   * caso ela no exista.
   * 
   * @param preferences - preferncias
   * @param prefName - nome da preferncia (chave do mapa)
   * @param defaultValue - valor default usado caso a preferncia solicitada no
   *        exista no mapa
   * @return valor da preferncia no mapa ou <b>defaultValue</b> caso ela no
   *         exista
   */
  private Object getPrefOrDefault(Map<String, Object> preferences,
    String prefName, Object defaultValue) {
    Object pref = preferences.get(prefName);
    return pref == null ? defaultValue : pref;
  }

  /**
   * Contri o <code>DesktopPreferences</code>.
   * 
   * @param treeAndTableSplitPaneLocation a posio do JSplitPane entre o painel
   *        da rvore de projetos e o da tabela com arquivos e diretrios
   * @param treeAndAppSplitPaneLocation a posio do JSplitPane entre o painel o
   *        painel da rvore de projetos e o de aplicaes
   * @param appAndNotifSplitPaneLocation a posio do JSplitPane entre o painel
   *        de aplicaes e o de notificaes
   * @param dimension dimenso do desktop
   * @param maximized Inidica se o desktop est maximizado
   * @param showHiddenFiles indica se arquivos ocultos devem ser exibidos
   */
  DesktopPreferences(int treeAndTableSplitPaneLocation,
    int treeAndAppSplitPaneLocation, int appAndNotifSplitPaneLocation,
    Dimension dimension, boolean maximized, boolean showHiddenFiles) {
    this.sPaneBetweenTreeAndTableLocation = treeAndTableSplitPaneLocation;
    this.sPaneBetweenTreeAndAppLocation = treeAndAppSplitPaneLocation;
    this.sPaneBetweenAppAndNotifLocation = appAndNotifSplitPaneLocation;
    this.desktopDimension = dimension;
    this.maximized = maximized;
    this.showHiddenFiles = showHiddenFiles;
  }

  /**
   * Obtm um mapa com os valores das preferncias para persistir no
   * <code>SharedObject</code>.
   * 
   * @return um mapa com os valores das preferncias
   */
  Map<String, Object> getContent() {
    Map<String, Object> map = new HashMap<String, Object>();
    map.put(SPANE_TREE, sPaneBetweenTreeAndTableLocation);
    map.put(SPANE_APP, sPaneBetweenTreeAndAppLocation);
    map.put(SPANE_NOTIF, sPaneBetweenAppAndNotifLocation);
    map.put(DIMENSION, desktopDimension);
    map.put(MAXIMIZED, maximized);
    map.put(SHOW_HIDDEN_FILES, showHiddenFiles);
    return map;
  }

  /**
   * Obtm a dimenso do desktop.
   * 
   * @return a dimenso do desktop
   */
  public Dimension getDesktopDimension() {
    return desktopDimension;
  }

  /**
   * Obtm a posio do JSplitPane entre o painel de aplicaes e o de
   * notificaes.
   * 
   * @return a posio do JSplitPane entre o painel de aplicaes e o de
   *         notificaes.
   */
  public int getSPaneBetweenAppAndNotifLocation() {
    return sPaneBetweenAppAndNotifLocation;
  }

  /**
   * Obtm a posio do JSplitPane entre o painel o painel da rvore de projetos
   * e o de aplicaes.
   * 
   * @return a posio do JSplitPane entre o painel o painel da rvore de
   *         projetos e o de aplicaes.
   */
  public int getSPaneBetweenTreeAndAppLocation() {
    return sPaneBetweenTreeAndAppLocation;
  }

  /**
   * Obtm a posio do JSplitPane entre o painel da rvore de projetos e o da
   * tabela com arquivos e diretrios.
   * 
   * @return a posio do JSplitPane entre o painel da rvore de projetos e o da
   *         tabela com arquivos e diretrios.
   */
  public int getSPaneBetweenTreeAndTableLocation() {
    return sPaneBetweenTreeAndTableLocation;
  }

  /**
   * Indica se o desktop est maximizado.
   * 
   * @return true se o desktop estiver maximizado, ou false caso contrrio
   */
  public boolean isMaximized() {
    return maximized;
  }

  /**
   * Indica se os arquivos cujos nomes comeam com "." devem ser exibidos.
   * 
   * @return true se os arquivos cujos nomes comeam com "." devem ser exibidos
   */
  public boolean shouldShowHiddenFiles() {
    return showHiddenFiles;
  }
}
