/*
 * $Id: NotificationPanel.java 94308 2009-07-29 19:30:43Z pasti $
 */

package csbase.client.desktop;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

import tecgraf.javautils.LNG;
import csbase.client.applications.ApplicationImages;
import csbase.client.remote.srvproxies.NotificationProxy;
import csbase.client.util.ClientUtilities;
import csbase.client.util.DateTranslator;
import csbase.client.util.NotificationTranslator;
import csbase.logic.Notification;
import csbase.logic.NotificationData;
import csbase.logic.NotificationHandler;
import csbase.logic.User;

/**
 * Classe base para um painel simplificado de cliente de notificaes vindas de
 * um servidor de sistema integrador. Este painel implementa um polling no
 * servio definido pela interface <code>NotificationServiceInterface</code>.
 * 
 * @author Tecgraf/PUC-Rio
 */
public abstract class NotificationPanel implements Observer {

  /**
   * Dilogo de composio de notificaes
   */
  protected NotificationCompositionFrame composeDialog = null;

  /**
   * rea de notificaes
   */
  protected JTextArea notificationArea = null;

  /**
   * Referncia ao painel principal
   */
  protected JPanel mainPanel = null;

  /**
   * Id do usurio.
   */
  protected Object userId = null;

  /**
   * Lista de handlers que sero chamados quando ocorre uma notificao.
   */
  protected LinkedList<NotificationHandler> notificationHandlers = null;

  /**
   * Adio de linha de notificao.
   * 
   * @param line a linha a ser colocada.
   */
  private void addNotificationLine(final String line) {
    notificationArea.append(line);
    notificationArea.setCaretPosition(notificationArea.getText().length());
  }

  /**
   * Adio de vrias linhas (lista) de notificao.
   * 
   * @param lineList a linha a ser colocada.
   */
  final protected void writeNotificationsLines(final List<String> lineList) {
    for (Iterator<String> lineIterator = lineList.iterator(); lineIterator
      .hasNext();) {
      this.addNotificationLine(lineIterator.next());
    }
  }

  /**
   * Abertura do dilogo de composio de notificaes.
   */
  final protected void openComposeDialog() {
    if (this.composeDialog == null) {
      this.composeDialog = new NotificationCompositionFrame();
    }
    this.composeDialog.start();
  }

  /**
   * Construo do painel de notificao.
   * 
   * @param composeIcon cone a ser atribudo ao boto de compor mensagem
   * @param clearIcon cone a ser atribudo ao boto de limpar mensagem
   * @return o <code>JPanel</code>
   */
  final protected JPanel createButtonsPanel(final ImageIcon composeIcon,
    final ImageIcon clearIcon) {
    // Boto de envio de mensagem
    final JButton notButton = ClientUtilities.createImageButton(composeIcon);
    notButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        openComposeDialog();
      }
    });
    notButton.setToolTipText(LNG.get("notification.send.button.tooltip"));
    // Boto de apagamento da rea de mensagens
    final JButton clrButton = ClientUtilities.createImageButton(clearIcon);
    clrButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        clearPanel();
      }
    });
    clrButton.setToolTipText(LNG.get("notification.clear.button.tooltip"));

    final JPanel xPanel = new JPanel();
    final GridBagLayout gridBag = new GridBagLayout();
    xPanel.setLayout(gridBag);
    GridBagConstraints gridBagConstraints = new GridBagConstraints();
    gridBagConstraints.fill = GridBagConstraints.NONE;
    gridBagConstraints.gridx = 0;
    gridBagConstraints.gridy = 0;
    gridBagConstraints.insets = new Insets(3, 2, 0, 2);
    gridBag.setConstraints(notButton, gridBagConstraints);
    xPanel.add(notButton);
    gridBagConstraints.gridy = 1;
    gridBagConstraints.insets = new Insets(3, 2, 0, 2);
    gridBag.setConstraints(clrButton, gridBagConstraints);
    xPanel.add(clrButton);
    return xPanel;
  }

  /**
   * Limpeza da rea de notificaes
   */
  final protected void clearPanel() {
    notificationArea.setText("");
  }

  /**
   * Construo do painel de notificao.
   * 
   * @param composeIcon cone do boto para criar uma mensagem
   * @param clearIcon cone do boto para apagar a rea de notificaes
   * @return o <code>JPanel</code>
   */
  final protected JPanel createMainPanel(final ImageIcon composeIcon,
    final ImageIcon clearIcon) {
    notificationArea = new JTextArea();
    notificationArea.setEditable(false);
    notificationArea.setFont(new Font(Font.MONOSPACED, Font.PLAIN, 12));
    clearPanel();
    final JScrollPane scrolledPane = new JScrollPane(notificationArea);
    JPanel xPanel = new JPanel();
    xPanel.setLayout(new BorderLayout());
    xPanel.add(scrolledPane, BorderLayout.CENTER);
    xPanel.add(createButtonsPanel(composeIcon, clearIcon), BorderLayout.WEST);
    return xPanel;
  }

  /**
   * Consulta ao painel.
   * 
   * @return o <code>JPanel</code>
   */
  final public JPanel getPanel() {
    return mainPanel;
  }

  /**
   * Recebimento de mensagens de observao.
   * 
   * @param observable a fonte.
   * @param obj a informao.
   */
  final public void update(final Observable observable, final Object obj) {
    if (obj == null) {
      return;
    }
    
    if (obj instanceof ArrayList<?>) {
      @SuppressWarnings("unchecked")
      final ArrayList<Notification> notifs = (ArrayList<Notification>) obj;
      sendMessages(notifs);
    }
  }

  /**
   * Adiciona um handler para ser chamado quando a notificao chegar.
   * 
   * @param handler o mtodo que ser chamado.
   */
  public void addNotificationHandler(NotificationHandler handler) {
    synchronized (notificationHandlers) {
      this.notificationHandlers.add(handler);
    }
  }

  /**
   * Chama os handlers que tiverem interesse nessa notificao.
   * 
   * @param data os dados da notificao.
   */
  private void callNotificationHandlers(final NotificationData data) {
    synchronized (notificationHandlers) {
      for (NotificationHandler handler : notificationHandlers) {
        handler.gotNotification(data);
      }
    }
  }

  /**
   * Envio de mensagens.
   * 
   * @param notifications o vetor de notificaes.
   */
  private void sendMessages(final ArrayList<Notification> notifications) {
    if (notifications == null) {
      return;
    }

    /*
     * Indica que pelo menos uma mensagem foi inserida no painel de notificao.
     * Neste caso o sistema deve emitir um som chamando a ateno do usurio.
     */
    boolean msgDisplayed = false;

    final NotificationTranslator translator = getTranslator();

    if (translator == null) {
      SwingUtilities.invokeLater(new Runnable() {
        public void run() {
          final Date now = new Date();
          final String msg = LNG.get("notification.translator.null.error");
          addNotificationLine(makeNotificationLine(now, "!!!", msg, false));
        }
      });
      msgDisplayed = true;
    }
    else {
      for (Notification notification : notifications) {
        final NotificationData data = notification.getData();
        Object result = null;
        try {
          // Usando reflexividade para chamar o mtodo certo
          final Method method =
            translator.getClass().getMethod("translate",
              new Class[] { data.getClass() });
          result = method.invoke(translator, new Object[] { data });
        }
        catch (Exception e) {
          result = LNG.get("notification.translation.error");
        }
        final Date date = notification.getDate();
        final String sender = notification.getSenderName();
        final String msg = result.toString();
        final boolean vol = notification.isVolatile();

        if (notification.display()) {
          SwingUtilities.invokeLater(new Runnable() {
            public void run() {
              String line = makeNotificationLine(date, sender, msg, vol);
              addNotificationLine(line);
              if (data.mustPopUp())
                showNotificationPopUp(sender, date, msg);
            }
          });
          msgDisplayed = true;
        }

        //Essa chamada tem que ficar fora da thread do Swing
        callNotificationHandlers(data);
      }
    }

    if (msgDisplayed) {
      Toolkit.getDefaultToolkit().beep();
    }
  }

  /**
   * Exibe dilogo modal com a notificao recebida.
   * 
   * @param senderLogin o remetente.
   * @param date a data da mensagem.
   * @param text o texo.
   */
  private void showNotificationPopUp(final String senderLogin, final Date date,
    final String text) {

    NotificationReplyFrame replyFrame =
      new NotificationReplyFrame(senderLogin, date, text);
    replyFrame.start();
  }

  /**
   * Monta uma linha de notificao.
   * 
   * @param date a data.
   * @param senderName um texto do remetente.
   * @param line texto da mensagem.
   * @param volatileFlag flag indicativo de mensagem voltil.
   * @return retorna a linha de notificao montada
   */
  private String makeNotificationLine(Date date, String senderName,
    String line, boolean volatileFlag) {
    String prefix = volatileFlag ? "[-] " : "[+] ";
    String formDate = (DateTranslator.getInstance()).translate(date);
    return (prefix + formDate + " - " + "[" + senderName + "]: "
      + line.replace('\n', ' ') + "\n");
  }

  /**
   * Altera o usurio corrente
   * 
   * @param user o objeto que representa o usurio.
   */
  final public void setUser(final User user) {
    Object uid = null;
    if (user != null) {
      uid = user.getId();
    }
    else {
      if (composeDialog != null) {
        composeDialog.close();
      }
    }
    if (userId != null) {
      NotificationProxy.deleteUserObserver(this, userId);
    }
    if (uid != null) {
      NotificationProxy.addUserObserver(this, uid);
    }
    userId = uid;
  }

  /**
   * Mtodo de busca de um tradutor de notificaes (objetos)
   * 
   * @return um tradutor.
   */
  public abstract NotificationTranslator getTranslator();

  /**
   * Construtor de escolha da cones de mensagens.
   * 
   * @param composeIcon cone a ser atribudo ao boto de compor mensagem
   * @param clearIcon cone a ser atribudo ao boto de limpar mensagem
   */
  public NotificationPanel(final ImageIcon composeIcon,
    final ImageIcon clearIcon) {
    this.notificationHandlers = new LinkedList<NotificationHandler>();
    this.mainPanel = createMainPanel(composeIcon, clearIcon);
  }

  /**
   * Construtor padro (sem escolha da cones).
   */
  public NotificationPanel() {
    this(ApplicationImages.ICON_COMPOSEMSG_32,
      ApplicationImages.ICON_CLEARMSG_32);
  }
}
