/**
 * $Id: RemoteTask.java 83898 2008-10-06 23:10:41Z ernani $
 */
package csbase.client.desktop;

import java.rmi.RemoteException;

import tecgraf.javautils.LNG;
import csbase.client.ClientServerManager;
import csbase.client.util.StandardErrorDialogs;
import csbase.exception.BugException;
import csbase.exception.CSBaseRuntimeException;
import csbase.exception.PermissionException;
import csbase.exception.ServiceFailureException;
import csbase.exception.UnavailableServiceException;
import csbase.logic.ServerURI;

/**
 * A classe <code>RemoteTask</code> modela uma tarefa executada no cliente que
 * envolve chamadas a mtodos remotos. As excees que representam falhas na
 * execuo de chamadas remotas so tratadas, exibindo-se mensagens de erro para
 * o usurio. Tambm  tratada a interrupo da tarefa, resultante de um
 * cancelamento comandado pelo usurio.
 * 
 * @param <R> A classe do resultado da tarefa.
 */
public abstract class RemoteTask<R> extends Task<R> {

  /** conexo com o servidor */
  protected ServerURI serverURI;

  /**
   * Construtor.
   */
  protected RemoteTask() {
    /** nada a fazer */
  }

  /**
   * Construtor.
   * 
   * @param serverURI - conexo com o servidor.
   */
  protected RemoteTask(ServerURI serverURI) {
    this.serverURI = serverURI;
  }

  /**
   * Tratamento de erros especfico para cada tipo de tarefa. Esse tratamento 
   * chamado na thread do Swing. Pode ser redefinido por uma especializao
   * dessa classe, porm com cautela, para que a padronizao do tratamento de
   * erros no seja quebrada!
   * 
   * @param error exceo que representa erro durante execuo da tarefa
   */
  @Override
  protected void handleError(Exception error) {
    /* erros do servidor */
    if (error instanceof CSBaseRuntimeException) {
      handleServerError((CSBaseRuntimeException) error);
    }
    /* falha no RMI */
    else if (error instanceof RemoteException) {
      StandardErrorDialogs.showErrorDialog(parentWindow, taskTitle, LNG
        .get("desktop.msg.commfailure"), error, getAdditionalInfo());
      if (this.serverURI != null)
        ClientServerManager.getInstance().update(this.serverURI);
      else
        ClientServerManager.getInstance().update();
    }
    else {
      StandardErrorDialogs.showErrorDialog(parentWindow, taskTitle, LNG
        .get("desktop.msg.clientbug"), error, getAdditionalInfo());
    }
  }

  /**
   * Tratamento para erros repassados pelo proxy do servio remoto.
   * 
   * @param cserror exceo que representa esse erro
   */
  protected void handleServerError(CSBaseRuntimeException cserror) {
    String msg = cserror.getMessage();
    if (msg == null) {
      if (cserror instanceof UnavailableServiceException) {
        msg = LNG.get("desktop.msg.unavailable");
      }
      else if (cserror instanceof ServiceFailureException) {
        msg = LNG.get("desktop.msg.servicefailure");
      }
      else if (cserror instanceof PermissionException) {
        msg = LNG.get("desktop.msg.permissionfailure");
      }
      else if (cserror instanceof BugException) {
        msg = LNG.get("desktop.msg.serverbug");
      }
      else {
        msg = LNG.get("desktop.msg.unknownerror");
      }
    }
    StandardErrorDialogs.showErrorDialog(parentWindow, taskTitle, msg, cserror,
      getAdditionalInfo());
  }
}
