/*
 * $Id$
 */
package csbase.client.facilities.algorithms.actions;

import java.awt.Window;
import java.rmi.RemoteException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import tecgraf.javautils.LNG;
import csbase.client.applicationmanager.ApplicationException;
import csbase.client.applicationmanager.ApplicationManager;
import csbase.client.applications.executor.ExecutorFrame;
import csbase.client.desktop.DesktopFrame;
import csbase.client.facilities.algorithms.parameters.NodeParametersValues;
import csbase.client.util.StandardErrorDialogs;
import csbase.client.util.sga.CommandRequestedListener;
import csbase.exception.OperationFailureException;
import csbase.logic.CommandInfo;
import csbase.logic.CommandSubmission;
import csbase.logic.Priority;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.RemoteObserver;

/**
 * Ao de executar um algoritmo programaticamente.
 * 
 * @author pizzol
 * 
 */
public class ExecuteAlgorithmAction {
  /** Nome do algoritmo */
  private final String algorithmName;

  /** Identificador da verso do algoritmo */
  private final AlgorithmVersionId algoVersionId;

  /** Janela pai que originou a ao */
  protected Window owner;

  /** Identificador do comando que foi executado */
  private String commandId;

  /** Indica se um e-mail de aviso ser enviado ao trmino do comando */
  private boolean mailAtEnd;

  /**
   * Listeners interessados na solicitao de um comando
   */
  private final List<CommandRequestedListener> listeners;

  /**
   * Constri a ao de executar um algoritmo.
   * 
   * @param algorithmName nome do algoritmo
   * @param algoVersionId identificador da verso do algoritmo
   * @param owner janela pai que originou a ao
   */
  public ExecuteAlgorithmAction(String algorithmName,
    AlgorithmVersionId algoVersionId, Window owner) {
    listeners = new LinkedList<CommandRequestedListener>();
    this.algorithmName = algorithmName;
    this.algoVersionId = algoVersionId;
    this.owner = owner;
    this.mailAtEnd = false;
  }

  /**
   * Executa um algoritmo, dado o nome e identificador da verso, configurando-o
   * com os valores de parmetros especificados.
   * 
   * @param paramValues valores dos parmetros do algoritmo a serem preenchidos
   * @param description descrio do comando
   * @param hasUserInteraction se true, indica que vai ter interao do usurio
   *        na parametrizao do algoritmo e tambm na submisso do comando para
   *        execuo, caso contrrio, o comando ser submetido
   *        programaticamente, sem a interferncia do usurio
   * @param sgaServerName nome do servidor do SGA a ser usado para execuo. Se
   *        for null, indica que a seleo do servidor ser automtica.
   * 
   * @return retorna true, se o comando foi executado com sucesso, caso
   *         contrrio, retorna false
   * @throws RemoteException caso tenha havido falha na chamada remota
   * @throws OperationFailureException caso o escalonador de comandos no esteja
   *         inicializado
   */
  public boolean executeCommand(NodeParametersValues paramValues,
    String description, boolean hasUserInteraction, String sgaServerName)
    throws RemoteException, OperationFailureException {

    ConfigAlgorithmAction configAction =
      new ConfigAlgorithmAction(algorithmName, algoVersionId, owner);
    if (!configAction.config(paramValues)) {
      return false;
    }

    return executeCommand(configAction.getConfigurator(), description,
      hasUserInteraction, sgaServerName);
  }

  /**
   * Executa um algoritmo a partir do seu configurador devidamente preenchido.
   * 
   * @param configurator configurador do algoritmo, que j possui os parmetros
   *        inicializados
   * @param description descrio do comando
   * @param hasUserInteraction se true, indica que vai ter interao do usurio
   *        na parametrizao do algoritmo e tambm na submisso do comando para
   *        execuo, caso contrrio, o comando ser submetido
   *        programaticamente, sem a interferncia do usurio
   * @param sgaServerName nome do servidor do SGA a ser usado para execuo. Se
   *        for null, indica que a seleo do servidor ser automtica.
   * @param observers Observadores remotos do comando executado
   * 
   * @return retorna true, se o comando foi executado com sucesso, caso
   *         contrrio, retorna false
   * @throws RemoteException caso tenha havido falha na chamada remota
   * @throws OperationFailureException caso o escalonador de comandos no esteja
   *         inicializado
   */
  public boolean executeCommand(AlgorithmConfigurator configurator,
    String description, boolean hasUserInteraction, String sgaServerName,
    RemoteObserver... observers) throws RemoteException,
    OperationFailureException {
    if (!hasUserInteraction) {
      return executeRemoteCommand(makeCommandSubmission(configurator,
        description, sgaServerName), observers);
    }
    else {
      return executeInterativeCommand(configurator, description);
    }
  }

  /**
   * Executa um algoritmo a partir do seu configurador devidamente preenchido.
   * 
   * @param configurator configurador do algoritmo, que j possui os parmetros
   *        inicializados
   * @param description descrio do comando
   * @param hasUserInteraction se true, indica que vai ter interao do usurio
   *        na parametrizao do algoritmo e tambm na submisso do comando para
   *        execuo, caso contrrio, o comando ser submetido
   *        programaticamente, sem a interferncia do usurio
   * @param sgaServerName nome do servidor do SGA a ser usado para execuo. Se
   *        for null, indica que a seleo do servidor ser automtica.
   * @return retorna true, se o comando foi executado com sucesso, caso
   *         contrrio, retorna false
   * @throws RemoteException caso tenha havido falha na chamada remota
   * @throws OperationFailureException caso o escalonador de comandos no esteja
   *         inicializado
   */
  public boolean executeCommand(AlgorithmConfigurator configurator,
    String description, boolean hasUserInteraction, String sgaServerName)
    throws RemoteException, OperationFailureException {
    return executeCommand(configurator, description, hasUserInteraction,
      sgaServerName, (RemoteObserver[]) null);
  }

  /**
   * Dispara a execuo remota do comando. Deve-se ter o cuidado de chamar esse
   * mtodo em uma <code>RemoteTask</code>.
   * 
   * @param submission informaes do comando a ser submetido para execuo
   * @param observers Observadores remotos do comando
   * @return true
   * 
   * @throws RemoteException caso tenha havido falha na chamada remota
   * @throws OperationFailureException caso o escalonador de comandos no esteja
   *         inicializado
   * 
   */
  private boolean executeRemoteCommand(final CommandSubmission submission,
    RemoteObserver... observers) throws RemoteException,
    OperationFailureException {
    if (ClientRemoteLocator.schedulerService != null) {
      Set<CommandInfo> commands =
        ClientRemoteLocator.schedulerService.submitCommand(submission,
          observers);
      commandId = commands.iterator().next().getId();
    }
    else {
      throw new OperationFailureException(LNG
        .get("ExecuteAlgorithmAction.exception.no_scheduler"));
    }
    return true;
  }

  /**
   * Executa o comando de maneira interativa com o usurio, abrindo a aplicao
   * Executor de Algoritmos somente com o algoritmo de interesse carregado.
   * Dessa maneira, o usurio pode interagir tanto na parametrizao do
   * algoritmo, quanto na submisso do comando para execuo.
   * 
   * @param configurator configurador do algoritmo, que j possui os parmetros
   *        inicializados
   * @param description descrio do comando
   * @return retorna true, se o comando foi executado com sucesso, caso
   *         contrrio, retorna false
   */
  protected boolean executeInterativeCommand(
    AlgorithmConfigurator configurator, String description) {
    try {
      ExecutorFrame executorFrame = new ExecutorFrame("executor");
      executorFrame.enableLoadAlgorithms(false);
      executorFrame.showCommandRequestedInfo(false);
      ApplicationManager applicationManager = ApplicationManager.getInstance();
      applicationManager.runApplication(executorFrame);

      executorFrame.addCommandRequestedListener(new CommandRequestedListener() {
        public void commandsWereRequested(Set<CommandInfo> submittedCommands) {
          if (!submittedCommands.isEmpty()) {
            Iterator<CommandInfo> iterator = submittedCommands.iterator();
            setCommandId(iterator.next().getId());
            notifyCommandRequestedListeners(submittedCommands);
          }
        }
      });
      executorFrame.setSingleAlgorithm(configurator.getAlgorithmName(),
        new AlgorithmVersionId[] { configurator.getAlgorithmVersionId() });
      executorFrame.setCommandDescription(description);
      executorFrame.showConfigurator(configurator.getAlgorithmName(),
        configurator.getParameterValuesByName());
      return true;
    }
    catch (ApplicationException e) {
      StandardErrorDialogs.showExceptionDialog(owner, owner.getName(), e);
      return false;
    }
  }

  /**
   * Notifica os listeners de um comando solicitado.
   * 
   * @param submittedCommands os comandos solicitados
   */
  protected void notifyCommandRequestedListeners(
    Set<CommandInfo> submittedCommands) {
    for (CommandRequestedListener listener : listeners) {
      listener.commandsWereRequested(submittedCommands);
    }
  }

  /**
   * Adiciona um listener para um comando solicitado.
   * 
   * @param listener listener de um comando solicitado
   */
  public void addCommandRequestedListener(CommandRequestedListener listener) {
    if (listener == null) {
      throw new IllegalArgumentException("O parmetro listener est nulo.");
    }
    listeners.add(listener);
  }

  /**
   * Monta a configurao da submisso do comando, a partir de um configurador
   * de algoritmos que j deve estar com seus parmetros inicializados.
   * 
   * @param configurator Configurador do algoritmo.
   * @param description Descrio do comando.
   * @param sgaServerName SGA em que o algoritmo vai ser executado.
   * @param mailAtEnd Indica se deve ser enviado um e-mail ao final da execuo
   *        do comando.
   * @return A configurao da submisso do comando.
   */
  protected CommandSubmission makeCommandSubmission(
    AlgorithmConfigurator configurator, String description, String sgaServerName) {
    String projectId = DesktopFrame.getInstance().getProject().getId();
    CommandSubmission submission =
      new CommandSubmission(configurator, projectId);
    submission.configureSimpleExecution(sgaServerName);
    submission.setMailAtEnd(mailAtEnd);
    submission.setDescription(description);
    submission.setPriority(Priority.ROOT);
    return submission;
  }

  /**
   * Obtm o nome do algoritmo.
   * 
   * @return o nome do algoritmo
   */
  public String getAlgorithmName() {
    return algorithmName;
  }

  /**
   * Obtm o identificador da verso do algoritmo.
   * 
   * @return o identificador da verso do algoritmo
   */
  public AlgorithmVersionId getAlgoVersionId() {
    return algoVersionId;
  }

  /**
   * Obtm o identificador do comando.
   * 
   * @return o identicador do comando
   */
  public Object getCommandId() {
    return commandId;
  }

  /**
   * Estabelece o identificador do comando. Um comando pode ser submetido
   * programaticamente para execuo sem a interferncia do usurio, ou tambm
   * pelo dilogo de execuo de comandos.
   * 
   * @param commandId identificador do comando
   */
  protected void setCommandId(String commandId) {
    this.commandId = commandId;
  }

  /**
   * Define se o usurio deve recever um email quando o comando terminar.
   * 
   * @param mailAtEnd true se o usurio deve receber o email, false caso
   *        contrrio
   */
  public void setMailAtEnd(boolean mailAtEnd) {
    this.mailAtEnd = mailAtEnd;
  }
}
