/*
 * $Id$
 */
package csbase.client.facilities.algorithms.actions;

import java.awt.Window;
import java.rmi.RemoteException;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import tecgraf.javautils.LNG;
import csbase.algorithms.flows.FlowApplication;
import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.graph.GraphNode;
import csbase.client.applicationmanager.ApplicationException;
import csbase.client.applicationmanager.ApplicationManager;
import csbase.client.applications.ApplicationFrame;
import csbase.client.desktop.RemoteTask;
import csbase.client.facilities.algorithms.parameters.NodeParametersValues;
import csbase.client.util.StandardErrorDialogs;
import csbase.client.util.sga.CommandRequestedListener;
import csbase.exception.OperationFailureException;
import csbase.exception.ParseException;
import csbase.logic.CommandInfo;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.flows.Flow;
import csbase.logic.algorithms.flows.configurator.FlowAlgorithmConfigurator;
import csbase.logic.algorithms.flows.configurator.Node;

/**
 * Ao de executar um fluxo de algoritmos programaticamente.
 * 
 * @author pizzol
 * 
 */
public class ExecuteFlowAction extends ExecuteAlgorithmAction {
  /** Ao de configurador do fluxo de algoritmos */
  private ConfigAlgorithmAction configAction;

  /**
   * Constri uma ao para executar um fluxo de algoritmos programaticamente.
   * 
   * @param flowName nome do fluxo
   * @param flowVersionId identificador do fluxo
   * @param owner janela que originou a ao
   */
  public ExecuteFlowAction(String flowName, AlgorithmVersionId flowVersionId,
    Window owner) {
    super(flowName, flowVersionId, owner);
  }

  /**
   * Obtm o configurador do fluxo de algoritmos.
   * 
   * @return o configurador do fluxo de algoritmos, ou null, caso no seja um
   *         configurador de fluxo
   */
  public FlowAlgorithmConfigurator getConfigurator() {
    if (configAction == null) {
      configAction =
        new ConfigAlgorithmAction(getAlgorithmName(), getAlgoVersionId(), owner);
    }

    AlgorithmConfigurator algorithmConfigurator =
      configAction.getConfigurator();
    if (algorithmConfigurator instanceof FlowAlgorithmConfigurator) {
      return (FlowAlgorithmConfigurator) algorithmConfigurator;
    }
    return null;
  }

  /**
   * Executa um comando a partir de um configurador de um fluxo de algoritmos
   * que esteja ou no instalado no servidor.
   * 
   * @param flowConfigurator configurador do fluxo de algoritmos
   * @param paramValuesList lista dos valores de parmetros dos algoritmos a
   *        serem inicializados no fluxo, ou null, caso j estejam preenchidos
   * @param description descrio do comando
   * @param hasUserInteraction se true, indica que vai ter interao do usurio
   *        na parametrizao do algoritmo e tambm na submisso do comando para
   *        execuo, caso contrrio, o comando ser submetido
   *        programaticamente, sem a interferncia do usurio
   * @param openFlowApplication se true, a aplicao a ser aberta para
   *        edio/execuo do fluxo  a de Construtor de Fluxos, caso
   *        contrrio, abre-se o Executor de Algoritmos
   * @param sgaServerName nome do servidor do SGA a ser usado para execuo. Se
   *        for null, indica que a seleo do servidor ser automtica.
   * 
   * @return retorna true, se o comando foi executado com sucesso, caso
   *         contrrio, retorna false
   * 
   * @throws RemoteException caso tenha havido falha na chamada remota
   * @throws OperationFailureException caso o escalonador de comandos no esteja
   *         inicializado
   */
  public boolean executeCommand(FlowAlgorithmConfigurator flowConfigurator,
    List<NodeParametersValues> paramValuesList, String description,
    boolean hasUserInteraction, boolean openFlowApplication,
    String sgaServerName) throws RemoteException, OperationFailureException {

    if (!configureFlowConfiguratorParametersValues(flowConfigurator,
      paramValuesList)) {
      return false;
    }

    if (hasUserInteraction && openFlowApplication) {
      return executeFlowApplication(flowConfigurator, paramValuesList,
        description);
    }
    return executeCommand(flowConfigurator, description, hasUserInteraction,
      sgaServerName);
  }

  /**
   * Abre a aplicao do Construtor de Fluxo para a execuo de um fluxo de
   * algoritmos que esteja ou no instalado no servidor, dependendo de como o
   * configurador do algoritmo foi criado.
   * 
   * O usurio pode interagir tanto na parametrizao do algoritmo, quanto na
   * submisso do comando para execuo.
   * 
   * @param flowConfigurator configurador do fluxo de algoritmos
   * @param paramValuesList lista dos valores de parmetros dos algoritmos a
   *        serem inicializados no fluxo
   * @param commandDescription descrio do comando
   * 
   * @return retorna true se a aplicao foi aberta com sucesso, caso contrrio,
   *         retorna false
   */
  private boolean executeFlowApplication(
    FlowAlgorithmConfigurator flowConfigurator,
    List<NodeParametersValues> paramValuesList, String commandDescription) {
    try {
      ApplicationManager applicationManager = ApplicationManager.getInstance();
      FlowApplication flowApp =
        applicationManager.runApplication(FlowApplication.class);
      Graph graph =
        createGraph(flowConfigurator, flowApp.getApplicationFrame());
      if (!configureGraphNodesValues(graph, paramValuesList)) {
        applicationManager.killApplication(flowApp);
        return false;
      }

      flowApp.executeGraph(graph, commandDescription);
      flowApp.hideAlgorithmTreePanel();

      flowApp.addCommandRequestedListener(new CommandRequestedListener() {
        public void commandsWereRequested(Set<CommandInfo> submittedCommands) {
          if (!submittedCommands.isEmpty()) {
            Iterator<CommandInfo> iterator = submittedCommands.iterator();
            setCommandId(iterator.next().getId());
            notifyCommandRequestedListeners(submittedCommands);
          }
        }
      });
      return true;
    }
    catch (ApplicationException e) {
      StandardErrorDialogs.showExceptionDialog(owner, owner.getName(), e);
      return false;
    }
  }

  /**
   * Configura os valores dos parmetros do grafo que representa o fluxo. S 
   * vlido quando a execuo do fluxo  interativa e utilizando a aplicao do
   * construtor de fluxo.
   * 
   * @param paramValuesList lista com os valores dos parmetros a serem
   *        inicializados nos ns do grafo
   * 
   * @return retorna true, se a configurao de todos os ns do grafo foi feita
   *         com sucesso, caso contrrio, retorna false
   */
  private boolean configureGraphNodesValues(Graph graph,
    List<NodeParametersValues> paramValuesList) {
    if (paramValuesList == null)
      return true;
    for (NodeParametersValues nodeParametersValues : paramValuesList) {
      GraphNode graphNode =
        graph.getNode(nodeParametersValues.getAlgorithmName(),
          nodeParametersValues.getAlgorithmVersionId());
      if (graphNode == null) {
        return false;
      }

      try {
        graphNode.setParameterValuesByName(nodeParametersValues
          .getAllParametersValues());
      }
      catch (ParseException pe) {
        StandardErrorDialogs.showErrorDialog(owner, LNG
          .get("ExecuteFlowAction.msg.error.invalid.params"), pe);
        return false;
      }

    }
    return true;
  }

  /**
   * Cria remotamente o grafo que representa o fluxo.
   * 
   * @param flowConfigurator configurador do fluxo
   * @param appFrame janela pai
   * @return o grafo criado
   */
  private Graph createGraph(final FlowAlgorithmConfigurator flowConfigurator,
    final ApplicationFrame appFrame) {
    RemoteTask<Graph> task = new RemoteTask<Graph>() {

      @Override
      protected void performTask() throws Exception {
        Flow flow = flowConfigurator.getFlow();
        Graph graph = new Graph(appFrame, flow);
        setResult(graph);
      }
    };
    if (!task.execute(appFrame, "Criando o grafo do fluxo",
      "Carregando informaes do fluxo.")) {
      return null;
    }
    return task.getResult();
  }

  /**
   * Configura os valores dos parmetros dos algoritmos do fluxo.
   * 
   * @param paramValuesList lista com os valores dos parmetros dos algoritmos a
   *        serem inicializados no fluxo. Se true, os parmetros j esto
   *        preenchidos.
   * @return retorna true, se a configurao de todos os algoritmos foi feita
   *         com sucesso, caso contrrio, retorna false
   */
  private boolean configureFlowConfiguratorParametersValues(
    FlowAlgorithmConfigurator flowConfigurator,
    List<NodeParametersValues> paramValuesList) {
    if (flowConfigurator == null) {
      return false;
    }
    if (paramValuesList == null) {
      return true;
    }
    Set<Node> nodes = flowConfigurator.getNodes();

    for (NodeParametersValues nodeParametersValues : paramValuesList) {
      if (!configureFlowConfiguratorNode(nodes, nodeParametersValues)) {
        return false;
      }
    }
    return true;
  }

  /**
   * Configura um n do configurador de fluxo de algoritmos.
   * 
   * @param nodes conjunto de ns do configurador de fluxo de algoritmos
   * @param nodeParametersValues valores dos parmetros do n do algoritmo a ser
   *        configurado
   * @return retorna true, se a configurao do n foi feita com sucesso, ou
   *         retorna false, caso o n no tenha sido encontrado ou a
   *         configurao do n falhou
   */
  private boolean configureFlowConfiguratorNode(Set<Node> nodes,
    NodeParametersValues nodeParametersValues) {
    String nodeName = nodeParametersValues.getAlgorithmName();
    AlgorithmVersionId nodeVersionId =
      nodeParametersValues.getAlgorithmVersionId();

    Node node = getNode(nodes, nodeName, nodeVersionId);
    if (node == null) {
      return false;
    }

    ConfigAlgorithmAction nodeConfigAction =
      new ConfigAlgorithmAction(node.getConfigurator(), owner);
    if (!nodeConfigAction.config(nodeParametersValues)) {
      return false;
    }
    return true;
  }

  /**
   * Obtm um determinado n no conjunto de ns do fluxo de algoritmos.
   * 
   * @param nodes conjunto de ns do fluxo de algoritmos
   * @param nodeName nome do n procurado
   * @param nodeVersionId identificador da verso do n procurado
   * @return retorn o n procurado ou null, caso o n no tenha sido encontrado
   */
  private Node getNode(Set<Node> nodes, String nodeName,
    AlgorithmVersionId nodeVersionId) {
    Node node = null;
    for (Iterator<Node> iterator = nodes.iterator(); iterator.hasNext();) {
      node = iterator.next();
      if (node.getConfigurator().getAlgorithmName().equals(nodeName)
        && node.getConfigurator().getAlgorithmVersionId().equals(nodeVersionId)) {
        break;
      }
    }
    return node;
  }

}
