package csbase.client.facilities.commandtable;

import java.awt.Component;
import java.awt.Dimension;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.SortOrder;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import tecgraf.javautils.FormatUtils;
import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.table.DefaultModifiableObjectTableProvider;
import tecgraf.javautils.gui.table.ObjectTableProvider;
import csbase.client.algorithms.commands.cache.CommandsCache;
import csbase.client.algorithms.commands.cache.CommandsFilter;
import csbase.client.applications.ApplicationImages;
import csbase.client.desktop.RemoteTask;
import csbase.client.util.DateTableCellRenderer;
import csbase.client.util.table.ProgressCellRenderer;
import csbase.logic.CommandInfo;
import csbase.logic.CommandStatus;
import csbase.logic.User;

/**
 * A classe <code>CommandMonitoringTable</code>  um componente de tabela,
 * baseado em uma <code>JTable</code>, que apresenta uma lista de informaes
 * sobre Algoritmos (comandos) em execuo.<br>
 */
public class CommandMonitoringTable extends AbstractCommandMonitoringTable {

  /**
   * Filtro que ser usado na cache de comandos para obter todos os comandos que
   * no estejam nem agendados e nem terminados.
   */
  private static final CommandsFilter commandsFilter =
    new CommandsFilter(false, CommandStatus.SCHEDULED, CommandStatus.FINISHED,
      CommandStatus.SYSTEM_FAILURE);

  /** Provedor de dados para a tabela */
  private CommandMonitoringTableProvider myProvider;

  // --------------------------------------------------------------------------
  // --------------------------------------------------------------------------
  // --------------------------------------------------------------------------

  /**
   * Construtor que constri por default uma tabela de seleo nica.
   */
  public CommandMonitoringTable() {
    this(ListSelectionModel.SINGLE_SELECTION, false);
  }

  /**
   * Construtor que permite escolher o modo de seleo da tabela.
   * 
   * @param selectionMode Indica o modo de seleo da tabela.
   * @see javax.swing.JTable#setSelectionMode(int)
   */
  public CommandMonitoringTable(int selectionMode) {
    this(selectionMode, false);
  }

  /**
   * Construtor que permite escolher o modo de seleo da tabela.
   * 
   * @param selectionMode Indica o modo de seleo da tabela.
   * @param showProgress - TRUE indica para mostrarmos a coluna 'progresso' na
   *        tabela de algoritmos em execuo, FALSE para omitir tal coluna.
   *        Valor obtido da propriedade de mesmo nome.
   * @see javax.swing.JTable#setSelectionMode(int)
   */
  public CommandMonitoringTable(int selectionMode, boolean showProgress) {
    super(selectionMode, commandsFilter, showProgress);

    setupDescriptionColumn();
    setupStatusColumn();
    if (showProgressColumn)
      setupProgressColumn();
    setupStartTimeColumn();
    setupRAMMemoryColumn();
    setupSwapMemoryColumn();
    setupCPUColumn();
    setupWallTimeColumn();
    setPreferredScrollableViewportSize(new Dimension(800, 100));
    start();
  }

  /**
   * Retorna um objeto contendo informaes sobre o comando correspondente 
   * linha especificada.
   * 
   * @param row ndice de linha na tabela.
   * @return objeto contendo informaes sobre um comando.
   */
  @Override
  public CommandInfo getCommandInfo(int row) {
    return (getMonitoringSet(row)).getCommandInfo();
  }

  /**
   * Retorna uma instncia nica (singleton) do provedor de dados para a tabela.
   */
  @Override
  protected ObjectTableProvider getObjectTableProvider() {
    if (myProvider == null)
      myProvider = new CommandMonitoringTableProvider();
    return myProvider;
  }

  /**
   * Prepara a coluna "descrio", que precisa de um renderizador especial.
   */
  private void setupDescriptionColumn() {
    TableColumn column =
      this.getColumnModel().getColumn(
        myProvider.getColumnNameIndex(LNG
          .get("CommandMonitoringTable.description")));
    column.setCellRenderer(myProvider.getDescriptionRenderer());
  }

  /**
   * Prepara a coluna "status", que precisa de um renderizador especial.
   */
  private void setupStatusColumn() {
    TableColumn column =
      this.getColumnModel()
        .getColumn(
          myProvider.getColumnNameIndex(LNG
            .get("CommandMonitoringTable.status")));
    column.setCellRenderer(myProvider.getCommandStatusRenderer());
  }

  /**
   * Prepara a coluna 'Progresso', que precisa de um renderizador especial.
   */
  private void setupProgressColumn() {
    TableColumn column =
      this.getColumnModel().getColumn(
        myProvider.getColumnNameIndex(LNG
          .get("CommandMonitoringTable.progress")));
    column.setCellRenderer(new ProgressCellRenderer());
  }

  /**
   * Prepara a coluna "Incio", que precisa de um renderizador especial.
   */
  private void setupStartTimeColumn() {
    TableColumn column =
      this.getColumnModel().getColumn(
        myProvider.getColumnNameIndex(LNG
          .get("CommandMonitoringTable.submission.date")));
    column.setCellRenderer(new DateTableCellRenderer());
    // Inicialmente, ordena a tabela pela data de submisso.
    sort(column.getModelIndex(), SortOrder.ASCENDING);
  }

  /**
   * Prepara a coluna "Memria RAM", que precisa de um renderizador especial.
   */
  private void setupRAMMemoryColumn() {
    TableColumn column =
      this.getColumnModel().getColumn(
        myProvider.getColumnNameIndex(LNG
          .get("CommandMonitoringTable.ram.usage")));
    column.setCellRenderer(myProvider.getPercentRenderer());
  }

  /**
   * Prepara a coluna "Memria Swap", que precisa de um renderizador especial.
   */
  private void setupSwapMemoryColumn() {
    TableColumn column =
      this.getColumnModel().getColumn(
        myProvider.getColumnNameIndex(LNG
          .get("CommandMonitoringTable.swap.usage")));
    column.setCellRenderer(myProvider.getPercentRenderer());
  }

  /**
   * Prepara a coluna "CPU", que precisa de um renderizador especial.
   */
  private void setupCPUColumn() {
    TableColumn column =
      this.getColumnModel().getColumn(
        myProvider.getColumnNameIndex(LNG
          .get("CommandMonitoringTable.cpu.usage")));
    column.setCellRenderer(myProvider.getPercentRenderer());
  }

  /**
   * Prepara a coluna "Tempo de parede", que precisa de um renderizador
   * especial.
   */
  private void setupWallTimeColumn() {
    TableColumn column =
      this.getColumnModel().getColumn(
        myProvider.getColumnNameIndex(LNG
          .get("CommandMonitoringTable.time.elapsed")));
    column.setCellRenderer(myProvider.getWallTimeRenderer());
  }

  // --------------------------------------------------------------------------
  // --------------------------------------------------------------------------
  // --------------------------------------------------------------------------

  /**
   * Provedor de dados para esta tabela.
   */
  public class CommandMonitoringTableProvider extends
    DefaultModifiableObjectTableProvider {

    /**
     * Nomes das colunas dessa tabela.
     */
    private String[] COLUMN_NAMES = null;

    /**
     * Tipos das colunas dessa tabela.
     */
    private Class<?>[] COLUMN_CLASSES = null;

    /**
     * Pontilhado que aparece quando no h informao para exibir ou houve
     * falha na comunicao com o servidor.
     */
    private static final String DOTTED = "-----";

    /**
     * Construtor.
     */
    public CommandMonitoringTableProvider() {
      init();
    }

    /**
     * @author ururahy
     */
    private final class PercentageTableCellRenderer extends
      DefaultTableCellRenderer {
      /**
       * {@inheritDoc}
       */
      @Override
      public Component getTableCellRendererComponent(JTable table,
        Object value, boolean isSelected, boolean hasFocus, int row, int column) {
        super.getTableCellRendererComponent(table, value, isSelected, hasFocus,
          row, column);
        if (value == null) {
          setText(DOTTED);
          return this;
        }
        String text = ((Double) value).intValue() + " %";
        setText(text);
        return this;
      }
    }

    /**
     * {@inheritDoc}
     */
    public String[] getColumnNames() {
      return COLUMN_NAMES;
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    public Class[] getColumnClasses() {
      return COLUMN_CLASSES;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Object[] getCellValues(Object o) {
      CommandInfo info = ((CommandMonitoringSet) o).getCommandInfo();
      List<Object> values = new ArrayList<Object>();
      values.add(CommandStatusHelper.createCellData(info));

      if (showProgressColumn)
        values.add(info.getProgressData());

      if (User.getLoggedUser().isAdmin())
        values.add(getUserName(info));

      values.add(info.getSGAName());
      values.add(info.getExecutionNodeDescription());
      values.add(info.getDescription());
      values.add(info.getTip());
      values.add(info.getSubmittedDate());
      values.add(info.getCpuPerc());
      values.add(info.getRAMMemoryPerc());
      values.add(info.getSwapMemoryPerc());
      values.add(info.getWallTimeSec());

      return values.toArray();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isCellEditable(int rowIndex, int columnIndex) {
      return isColumnEditable(columnIndex);
    }

    /**
     * {@inheritDoc}
     */
    public void setValueAt(Object row, Object newValue, int colIndex) {
      if (isColumnEditable(colIndex)) {
        CommandInfo commandInfo = ((CommandMonitoringSet) row).getCommandInfo();
        setDescription(commandInfo, newValue);
      }
    }

    /**
     * @param colIndex
     * @return .
     */
    private boolean isColumnEditable(int colIndex) {
      // A nica coluna editvel  a de descrio.
      return colIndex == getColumnNameIndex(LNG
        .get("CommandMonitoringTable.description"));
    }

    /**
     * Atualiza a descrio do comando.
     * 
     * @param commandInfo As informaes do comando (No aceita {@code null}).
     * @param newValue
     */
    private void setDescription(final CommandInfo commandInfo,
      final Object newValue) {

      String description = (String) newValue;
      if (description.length() == 0) {
        description = null;
      }
      String previousDescription = commandInfo.getDescription();
      commandInfo.setDescription(description);

      RemoteTask<Void> task = new RemoteTask<Void>() {
        @Override
        protected void performTask() throws RemoteException {
          CommandsCache.getInstance().updateDescription(commandInfo);
        }
      };
      String taskTitle = LNG.get("CommandMonitoringTable.title");
      String taskMessage = LNG.get("CommandMonitoringTable.message");
      boolean success = task.execute(null, taskTitle, taskMessage);
      if (!success) {
        commandInfo.setDescription(previousDescription);
      }
    }

    /**
     * Retorna o ndice da coluna com o cabealho especificado.
     * 
     * @param headerName cabealho
     * @return ndice da coluna no modelo. -1 se no pde obter o ndice.
     */
    int getColumnNameIndex(String headerName) {
      String[] colNames = getColumnNames();
      for (int i = 0; i < colNames.length; i++) {
        if (headerName.equals(colNames[i])) {
          return i;
        }
      }
      return -1;
    }

    /**
     * Retorna o login do usurio que disparou o comando.
     * 
     * @param cmd comando.
     * @return login do usurio.
     */
    private String getUserName(CommandInfo cmd) {
      try {
        User user = User.getUser(cmd.getUserId());
        if (user == null) {
          throw new Exception();
        }
        return user.getName();
      }
      catch (Exception e) {
        return LNG.get("CommandMonitoringTable.error");
      }
    }

    /**
     * Retorna um renderizador para colunas que apresentem um valor percentual.
     * 
     * @return renderizador para colunas que apresentem um valor percentual.
     */
    public TableCellRenderer getPercentRenderer() {
      return new PercentageTableCellRenderer();
    }

    /**
     * Retorna um renderizador para colunas que apresentem o estado de um
     * comando.
     * 
     * @return renderizador para colunas que apresentem o estado de um comando.
     */
    public TableCellRenderer getCommandStatusRenderer() {
      return new CommandStatusCellRenderer();
    }

    /**
     * Retorna um renderizador para colunas que apresentem um valor percentual.
     * 
     * @return renderizador para colunas que apresentem um valor percentual.
     */
    public TableCellRenderer getWallTimeRenderer() {
      return new DefaultTableCellRenderer() {
        @Override
        public Component getTableCellRendererComponent(JTable table,
          Object value, boolean isSelected, boolean hasFocus, int row,
          int column) {
          super.getTableCellRendererComponent(table, value, isSelected,
            hasFocus, row, column);
          if (value == null) {
            setText(DOTTED);
            return this;
          }
          int wtime = (Integer) value;
          String text = FormatUtils.formatInterval(wtime);
          setText(text);
          return this;
        }
      };
    }

    /**
     * Retorna um renderizador para a coluna de descrio.
     * 
     * @return renderizador para a coluna de descrio.
     */
    public TableCellRenderer getDescriptionRenderer() {
      DefaultTableCellRenderer cellRenderer = new DefaultTableCellRenderer();
      cellRenderer.setIcon(ApplicationImages.ICON_EDITABLE_TABLE_CELL_14);
      return cellRenderer;
    }

    /**
     * Inicializa variveis da tabela.
     */
    private void init() {
      // nomes das colunas
      List<String> names = new ArrayList<String>();
      names.add(LNG.get("CommandMonitoringTable.status"));
      if (showProgressColumn)
        names.add(LNG.get("CommandMonitoringTable.progress"));
      if (User.getLoggedUser().isAdmin())
        names.add(LNG.get("CommandMonitoringTable.user"));
      names.add(LNG.get("CommandMonitoringTable.server"));
      names.add(LNG.get("CommandMonitoringTable.node"));
      names.add(LNG.get("CommandMonitoringTable.description"));
      names.add(LNG.get("CommandMonitoringTable.command"));
      names.add(LNG.get("CommandMonitoringTable.submission.date"));
      names.add(LNG.get("CommandMonitoringTable.cpu.usage"));
      names.add(LNG.get("CommandMonitoringTable.ram.usage"));
      names.add(LNG.get("CommandMonitoringTable.swap.usage"));
      names.add(LNG.get("CommandMonitoringTable.time.elapsed"));
      COLUMN_NAMES = names.toArray(new String[names.size()]);

      // classes das colunas
      List<Class<?>> classes = new ArrayList<Class<?>>();
      classes.add(CommandStatusCellData.class); // status
      if (showProgressColumn)
        classes.add(Number.class); // progresso
      if (User.getLoggedUser().isAdmin())
        classes.add(String.class); // user
      classes.add(String.class); // server
      classes.add(String.class); // node
      classes.add(String.class); // description
      classes.add(String.class); // command
      classes.add(Date.class); // submission-date
      classes.add(Double.class); // cpu-usage
      classes.add(Double.class); // ram-usage
      classes.add(Double.class); // swap-usage
      classes.add(Integer.class); // elapsed time
      COLUMN_CLASSES = classes.toArray(new Class[classes.size()]);
    }
  }
}
