package csbase.client.facilities.commandtable;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SortOrder;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.table.DefaultModifiableObjectTableProvider;
import tecgraf.javautils.gui.table.ObjectTableProvider;
import csbase.client.algorithms.commands.cache.CommandsCache;
import csbase.client.algorithms.commands.cache.CommandsFilter;
import csbase.client.applications.ApplicationImages;
import csbase.client.desktop.RemoteTask;
import csbase.client.util.DateTableCellRenderer;
import csbase.logic.CommandInfo;
import csbase.logic.CommandStatus;
import csbase.logic.Priority;
import csbase.logic.User;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.SchedulerServiceInterface;

/**
 * A classe <code>CommandSubmittedTable</code>  um componente de tabela,
 * baseado em uma <code>JTable</code>, que apresenta uma lista de informaes
 * sobre Algoritmos (comandos) submetidos, aguardando execuo.<br>
 */
public class CommandSubmittedTable extends AbstractCommandMonitoringTable {

  /**
   * Tipos das colunas na viso do administrador.
   */
  private static final Class<?>[] ADMIN_COLUMN_CLASSES =
    new Class[] { String.class, String.class, String.class, String.class,
        Date.class, Priority.class, Integer.class };

  /**
   * Tipos das colunas na viso de usurios no administradores.
   */
  private static final Class<?>[] COLUMN_CLASSES =
    new Class[] { String.class, String.class, String.class, Date.class,
        Priority.class, Integer.class };

  /**
   * Filtro a ser usado na cache de comandos para obter apenas os comandos
   * agendados.
   */
  private static final CommandsFilter commandsFilter =
    new CommandsFilter(CommandStatus.SCHEDULED);

  /** Provedor de dados para a tabela */
  private MyModifiableObjectTableProvider myProvider;

  /**
   * Construtor que constri por default uma tabela de seleo nica.
   */
  public CommandSubmittedTable() {
    this(ListSelectionModel.SINGLE_SELECTION);
  }

  /**
   * Construtor que permite escolher o modo de seleo da tabela.
   * 
   * @param selectionMode Indica o modo de seleo da tabela.
   * 
   * @see javax.swing.JTable#setSelectionMode(int)
   */
  public CommandSubmittedTable(int selectionMode) {
    super(selectionMode, commandsFilter);
    setupDescriptionColumn();
    setupSubmittedTimeColumn();
    setupPriorityColumn();
    setupPositionColumn();
    setPreferredScrollableViewportSize(new Dimension(800, 100));
    // Obtm a posio da coluna 'position' na tabela. 
    int pos =
      myProvider.getColumnNameIndex(LNG.get("CommandSubmittedTable.position"));
    // Ordena a tabela pela coluna 'position'.
    this.sort(pos, SortOrder.ASCENDING);
    start();
  }

  /**
   * Prepara a coluna "descrio", que precisa de um renderizador especial.
   */
  private void setupDescriptionColumn() {
    TableColumn column =
      this.getColumnModel().getColumn(
        myProvider.getColumnNameIndex(LNG
          .get("CommandSubmittedTable.description")));
    column.setCellRenderer(myProvider.getDescriptionRenderer());
  }

  /**
   * Prepara a coluna "Data de incio", adicionando um renderizador e um editor
   * (usando <i>comboBox</i>).
   */
  public void setupPriorityColumn() {
    String[] values = new String[Priority.values().length];
    for (int i = 0; i < Priority.values().length; i++) {
      values[i] =
        LNG
          .get("CommandSubmittedTable.priority." + Priority.values()[i].name());
    }
    TableColumn comboBoxColumn =
      this.getColumnModel().getColumn(
        myProvider
          .getColumnNameIndex(LNG.get("CommandSubmittedTable.priority")));
    JComboBox comboBox = new JComboBox(values);
    comboBoxColumn.setCellEditor(new DefaultCellEditor(comboBox));
    comboBoxColumn.setCellRenderer(myProvider.getPriorityRenderer());
  }

  /**
   * Prepara a coluna "Posio", adicionando um renderizador e um editor (s
   * aceita dados numricos).
   */
  public void setupPositionColumn() {
    TableColumn textFieldColumn =
      this.getColumnModel().getColumn(
        myProvider
          .getColumnNameIndex(LNG.get("CommandSubmittedTable.position")));
    JTextField textField = new JTextField();
    textField.addKeyListener(new KeyAdapter() {
      @Override
      public void keyTyped(KeyEvent ev) {
        if ((!Character.isDigit(ev.getKeyChar()))
          && (!(ev.getKeyChar() == KeyEvent.VK_BACK_SPACE)))
          ev.consume();
      }
    });
    textField.setColumns(5);
    textFieldColumn.setCellEditor(new DefaultCellEditor(textField));
    textFieldColumn.setCellRenderer(myProvider.getPositionRenderer());
  }

  /**
   * Retorna uma instncia nica (singleton) do provedor de dados para a tabela.
   */
  @Override
  protected ObjectTableProvider getObjectTableProvider() {
    if (myProvider == null) {
      myProvider = new MyModifiableObjectTableProvider();
    }
    return myProvider;
  }

  /**
   * Prepara a coluna "Data de incio", que precisa de um renderizador especial.
   */
  private void setupSubmittedTimeColumn() {
    TableColumn column =
      this.getColumnModel().getColumn(
        myProvider.getColumnNameIndex(LNG
          .get("CommandSubmittedTable.submission.date")));
    column.setCellRenderer(new DateTableCellRenderer());
  }

  /**
   * Provedor de dados para esta tabela.
   */
  private class MyModifiableObjectTableProvider extends
    DefaultModifiableObjectTableProvider {

    /**
     * {@inheritDoc}
     */
    public String[] getColumnNames() {
      List<String> columnNames = new ArrayList<String>();
      if (User.getLoggedUser().isAdmin()) {
        columnNames.add(LNG.get("CommandSubmittedTable.user"));
      }
      columnNames.add(LNG.get("CommandSubmittedTable.server"));
      columnNames.add(LNG.get("CommandSubmittedTable.description"));
      columnNames.add(LNG.get("CommandSubmittedTable.command"));
      columnNames.add(LNG.get("CommandSubmittedTable.submission.date"));
      columnNames.add(LNG.get("CommandSubmittedTable.priority"));
      columnNames.add(LNG.get("CommandSubmittedTable.position"));
      return columnNames.toArray(new String[columnNames.size()]);
    }

    /**
     * {@inheritDoc}
     */
    public Class<?>[] getColumnClasses() {
      if (User.getLoggedUser().isAdmin()) {
        return ADMIN_COLUMN_CLASSES;
      }
      return COLUMN_CLASSES;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Object[] getCellValues(Object o) {
      CommandInfo info = ((CommandMonitoringSet) o).getCommandInfo();
      List<Object> cellValues = new ArrayList<Object>();
      if (User.getLoggedUser().isAdmin()) {
        cellValues.add(getUserName(info));
      }
      cellValues.add(info.getSGAName());
      cellValues.add(info.getDescription());
      cellValues.add(info.getTip());
      cellValues.add(info.getSubmittedDate());
      cellValues.add(info.getPriority());
      cellValues.add(info.getGlobalPosition());
      return cellValues.toArray();
    }

    /**
     * Indica se determinada clula da tabela  editvel. Somente  permitida a
     * edio para administradores, nas colunas "prioridade" e "posio".
     * 
     * @param rowIndex linha
     * @param colIndex coluna
     * 
     * @return <em>true</em> se foi possvel editar a linha.
     */
    public boolean isCellEditable(int rowIndex, int colIndex) {
      return isColumnEditable(colIndex);
    }

    /**
     * Edita o contedo de determinada coluna da linha especificada. Somente 
     * permitida a edio para administradores, nas colunas "prioridade" e
     * "posio".
     * 
     * @param row objeto representando uma linha da tabela.
     * @param newValue novo valor para a clula.
     * @param colIndex ndice da coluna da clula.
     */
    public void setValueAt(Object row, Object newValue, int colIndex) {
      if (!isColumnEditable(colIndex)) {
        return;
      }
      CommandInfo info = ((CommandMonitoringSet) row).getCommandInfo();
      if (null == info) {
        return;
      }
      if (colIndex == getColumnNameIndex(LNG
        .get("CommandSubmittedTable.priority"))) {
        setPriority(info, newValue);
      }
      else if (colIndex == getColumnNameIndex(LNG
        .get("CommandSubmittedTable.position"))) {
        setPosition(info, newValue);
      }
      else if (colIndex == getColumnNameIndex(LNG
        .get("CommandSubmittedTable.description"))) {
        setDescription(info, newValue);
      }
    }

    /**
     * Retorna o ndice da coluna com o cabealho especificado.
     * 
     * @param headerName cabealho
     * 
     * @return ndice da coluna no modelo.
     */
    int getColumnNameIndex(String headerName) {
      String[] colNames = getColumnNames();
      for (int i = 0; i < colNames.length; i++) {
        if (headerName.equals(colNames[i])) {
          return i;
        }
      }
      return -1;
    }

    private boolean isColumnEditable(int colIndex) {
      if (User.getLoggedUser().isAdmin()) {
        return (colIndex == getColumnNameIndex(LNG
          .get("CommandSubmittedTable.priority"))
          || colIndex == getColumnNameIndex(LNG
            .get("CommandSubmittedTable.position")) || colIndex == getColumnNameIndex(LNG
          .get("CommandSubmittedTable.description")));
      }
      else {
        return colIndex == getColumnNameIndex(LNG
          .get("CommandSubmittedTable.description"));
      }
    }

    /**
     * Retorna o login do usurio que submeteu o comando.
     * 
     * @param cmd comando.
     * 
     * @return login do usurio.
     */
    private String getUserName(CommandInfo cmd) {
      try {
        User user = User.getUser(cmd.getUserId());
        if (user == null) {
          throw new Exception();
        }
        return user.getName();
      }
      catch (Exception e) {
        return LNG.get("CommandSubmittedTable.error");
      }
    }

    /**
     * Atualiza a descrio do comando.
     * 
     * @param commandInfo As informaes do comando (No aceita {@code null}).
     */
    private void setDescription(final CommandInfo commandInfo,
      final Object newValue) {

      String description = (String) newValue;
      if (description.length() == 0) {
        description = null;
      }
      String previousDescription = commandInfo.getDescription();
      commandInfo.setDescription(description);

      RemoteTask<Void> task = new RemoteTask<Void>() {
        @Override
        protected void performTask() throws RemoteException {
          CommandsCache.getInstance().updateDescription(commandInfo);
        }
      };
      String taskTitle = LNG.get("CommandSubmittedTable.title");
      String taskMessage = LNG.get("CommandSubmittedTable.message");
      boolean success = task.execute(null, taskTitle, taskMessage);
      if (!success) {
        commandInfo.setDescription(previousDescription);
      }
    }

    /**
     * Modifica a prioridade de um comando, contanto que a prioridade
     * corresponda a um valor vlido.
     * 
     * @param cmd comando.
     * 
     * @param priority novo valor de prioridade (entrado pelo usurio).
     */
    private void setPriority(final CommandInfo cmd, Object priority) {
      for (final Priority p : Priority.values()) {
        if (priority.toString().equals(
          LNG.get("CommandSubmittedTable.priority." + p.name()))) {
          final SchedulerServiceInterface schedulerService =
            ClientRemoteLocator.schedulerService;
          RemoteTask<Boolean> task = new RemoteTask<Boolean>() {
            @Override
            public void performTask() throws Exception {
              setResult(schedulerService.setPriority(cmd.getId(), p));
            }
          };
          task.execute(null, LNG.get("CommandSubmittedTable.title"), LNG
            .get("CommandSubmittedTable.message"));
        }
      }
    }

    /**
     * Retorna um renderizador para a coluna "Prioridade".
     * 
     * @return renderizador para a coluna "Prioridade".
     */
    public TableCellRenderer getPriorityRenderer() {
      return new DefaultTableCellRenderer() {
        @Override
        public Component getTableCellRendererComponent(JTable table,
          Object value, boolean isSelected, boolean hasFocus, int row,
          int column) {
          Component comp =
            super.getTableCellRendererComponent(table, value, isSelected,
              hasFocus, row, column);
          if (value instanceof Priority) {
            Priority priority = (Priority) value;
            String priorityName = priority.name();
            String text =
              LNG.get("CommandSubmittedTable.priority." + priorityName);

            DefaultTableCellRenderer renderer = (DefaultTableCellRenderer) comp;
            renderer.setText(text);
            if (User.getLoggedUser().isAdmin()) {
              renderer.setIcon(ApplicationImages.ICON_EDITABLE_TABLE_CELL_14);
            }
          }
          return this;
        }
      };
    }

    /**
     * Modifica a posio do comando na fila de submisso.
     * 
     * @param cmd comando.
     * 
     * @param position nova posio.
     */
    private void setPosition(final CommandInfo cmd, final Object position) {
      final SchedulerServiceInterface schedulerService =
        ClientRemoteLocator.schedulerService;
      RemoteTask<Boolean> task = new RemoteTask<Boolean>() {
        @Override
        public void performTask() throws Exception {
          String positionAsString = (String) position;
          // Para evitar que NumberFormattedException seja lanada quando 
          // a posio de um comando  definida para "",  necessrio verificar 
          // se a posio  uma string vazia. Se a posio for vazia, a
          // posio  setada para zero, fazendo com que a posio do comando 
          // volte a sua posio original na fila de submisso. 
          if (positionAsString.isEmpty()) {
            positionAsString = "0";
          }
          setResult(schedulerService.setPosition(cmd.getId(), Integer
            .parseInt(positionAsString) - 1));
        }
      };
      task.execute(null, LNG.get("CommandSubmittedTable.title"), LNG
        .get("CommandSubmittedTable.message"));
    }

    /**
     * Retorna um renderizador para a coluna "Posio".
     * 
     * @return renderizador para a coluna "Posio".
     */
    public TableCellRenderer getPositionRenderer() {
      DefaultTableCellRenderer cellRenderer = new DefaultTableCellRenderer();
      if (User.getLoggedUser().isAdmin()) {
        cellRenderer.setIcon(ApplicationImages.ICON_EDITABLE_TABLE_CELL_14);
      }
      return cellRenderer;
    }

    /**
     * Retorna um renderizador para a coluna de descrio.
     * 
     * @return renderizador para a coluna de descrio.
     */
    public TableCellRenderer getDescriptionRenderer() {
      DefaultTableCellRenderer cellRenderer = new DefaultTableCellRenderer();
      cellRenderer.setIcon(ApplicationImages.ICON_EDITABLE_TABLE_CELL_14);
      return cellRenderer;
    }
  }
}
