package csbase.client.facilities.sgatable;

import java.awt.Component;
import java.awt.Dimension;
import java.rmi.RemoteException;
import java.util.Comparator;
import java.util.Vector;

import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.table.DefaultObjectTableProvider;
import tecgraf.javautils.gui.table.ObjectTableProvider;
import csbase.client.facilities.monitoringtable.MonitoringTable;
import csbase.logic.MonitoringSet;
import csbase.logic.SGAInfo;
import csbase.logic.SGASet;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.SGAServiceInterface;

/**
 * A classe <code>SGAClusterTable</code>  um componente de tabela, baseado em
 * uma <code>JTable</code>, que apresenta uma lista de informaes sobre os ns
 * (SGAs) de um cluster.
 * 
 * @author Leonardo Abreu de Barros
 */
public class SGAClusterTable extends MonitoringTable {
  /**
   * Pontilhado que aparece quando no h informao para exibir ou houve falha
   * na comunicao com o servidor.
   */
  private final String DOTTED = "-----";

  /**
   * Indica se conseguiu coletar as informaes no servidor.
   */
  private boolean gotRows;

  /** Nome do cluster */
  private String clusterName;

  /** Provedor de dados para a tabela */
  private MyObjectTableProvider myProvider;

  /**
   * Construtor que constri por default uma tabela de seleo nica.
   * 
   * @param cluster informaes sobre o cluster.
   */
  public SGAClusterTable(SGASet cluster) {
    this(cluster, ListSelectionModel.SINGLE_SELECTION);
  }

  /**
   * Construtor que permite escolher o modo de seleo da tabela.
   * 
   * @param cluster informaes sobre o cluster.
   * @param selectionMode Indica o modo de seleo da tabela.
   * 
   * @see javax.swing.JTable#setSelectionMode(int)
   */
  public SGAClusterTable(SGASet cluster, int selectionMode) {
    super(selectionMode);
    this.clusterName = cluster.getName();
    setupStateColumn();
    setupCPUColumn();
    setupRAMMemoryColumn();
    setupSwapMemoryColumn();
    setPreferredScrollableViewportSize(new Dimension(800, 100));
  }

  /**
   * Prepara a coluna "Estado", que precisa de renderizadores e comparadores
   * especiais.
   */
  private void setupStateColumn() {
    int columnIndex = myProvider.getColumnNameIndex(LNG.get("MON_STATE"));
    TableColumn column = this.getColumnModel().getColumn(columnIndex);
    column.setCellRenderer(myProvider.getStateRenderer());
    this.setComparator(columnIndex, myProvider.getStateComparator());
  }

  /**
   * Prepara a coluna "CPU", que precisa de renderizadores e comparadores
   * especiais.
   */
  private void setupCPUColumn() {
    int columnIndex = myProvider.getColumnNameIndex(LNG.get("MON_CPU_USAGE"));
    TableColumn column = this.getColumnModel().getColumn(columnIndex);
    column.setCellRenderer(myProvider.getCPURenderer());
    this.setComparator(columnIndex, myProvider.getCPUComparator());
  }

  /**
   * Prepara a coluna "Memria RAM", que precisa de renderizadores e
   * comparadores especiais.
   */
  private void setupRAMMemoryColumn() {
    int columnIndex =
      myProvider.getColumnNameIndex(LNG.get("MON_RAM_MEMORY_USAGE"));
    TableColumn column = this.getColumnModel().getColumn(columnIndex);
    column.setCellRenderer(myProvider.getRAMMemoryRenderer());
    this.setComparator(columnIndex, myProvider.getRAMMemoryComparator());
  }

  /**
   * Prepara a coluna "Memria Swap", que precisa de renderizadores e
   * comparadores especiais.
   */
  private void setupSwapMemoryColumn() {
    int columnIndex =
      myProvider.getColumnNameIndex(LNG.get("MON_SWAP_MEMORY_USAGE"));
    TableColumn column = this.getColumnModel().getColumn(columnIndex);
    column.setCellRenderer(myProvider.getSwapMemoryRenderer());
    this.setComparator(columnIndex, myProvider.getSwapMemoryComparator());
  }

  /**
   * Retorna uma instncia nica (singleton) do provedor de dados para a tabela.
   */
  @Override
  protected ObjectTableProvider getObjectTableProvider() {
    if (myProvider == null) {
      myProvider = new MyObjectTableProvider();
    }
    return myProvider;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Vector<MonitoringSet> getNewRows() throws RemoteException {
    gotRows = false;
    Vector<MonitoringSet> newRows = new Vector<MonitoringSet>();
    SGAServiceInterface sgaService = ClientRemoteLocator.sgaService;
    SGASet cluster = sgaService.getSGASet(clusterName);
    if (cluster != null) {
      SGAInfo[] sgaNodes = cluster.getAllInfo();
      for (int i = 0; i < sgaNodes.length; i++) {
        newRows.addElement(sgaNodes[i]);
      }
    }
    gotRows = true;
    return newRows;
  }

  /**
   * Provedor de dados para esta tabela.
   */
  private class MyObjectTableProvider extends DefaultObjectTableProvider {

    /**
     * {@inheritDoc}
     */
    public String[] getColumnNames() {
      return new String[] { LNG.get("MON_SERVER"), LNG.get("MON_STATE"),
          LNG.get("MON_CPU_USAGE"), LNG.get("MON_RAM_MEMORY_USAGE"),
          LNG.get("MON_SWAP_MEMORY_USAGE"), LNG.get("MON_PLATFORM") };
    }

    /**
     * {@inheritDoc}
     */
    public Class<?>[] getColumnClasses() {
      return new Class<?>[] { String.class, String.class, String.class,
          String.class, String.class, String.class };
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Object[] getCellValues(Object row) {
      SGAInfo node = (SGAInfo) row;
      return new Object[] { getName(node), node, node, node, node,
          getPlatform(node) };
    }

    /**
     * Retorna o ndice da coluna com o cabealho especificado.
     * 
     * @param headerName cabealho
     * 
     * @return ndice da coluna no modelo.
     */
    int getColumnNameIndex(String headerName) {
      String[] colNames = getColumnNames();
      for (int i = 0; i < colNames.length; i++) {
        if (headerName.equals(colNames[i])) {
          return i;
        }
      }
      return -1;
    }

    /**
     * Retorna o nome de um n.
     * 
     * @param node objeto com informaes sobre o n.
     * 
     * @return o nome do n.
     */
    private String getName(SGAInfo node) {
      return node.getHostName();
    }

    /**
     * Retorna um renderizador para a coluna "Estado".
     * 
     * @return renderizador para a coluna "Estado".
     */
    public TableCellRenderer getStateRenderer() {
      return new DefaultTableCellRenderer() {
        @Override
        public Component getTableCellRendererComponent(JTable table,
          Object value, boolean isSelected, boolean hasFocus, int row,
          int column) {
          Component comp =
            super.getTableCellRendererComponent(table, value, isSelected,
              hasFocus, row, column);
          if (value instanceof SGAInfo) {
            SGAInfo node = (SGAInfo) value;
            String text;
            if (node.getAlive()) {
              text = LNG.get("MON_AVAILABLE");
            }
            else {
              text = LNG.get("MON_INACCESSIBLE");
            }
            ((DefaultTableCellRenderer) comp).setText(text);
          }
          return comp;
        }
      };
    }

    /**
     * Retorna um comparador para a coluna "Estado".
     * 
     * @return comparador para a coluna "Estado".
     */
    public Comparator<SGAInfo> getStateComparator() {
      return new Comparator<SGAInfo>() {
        public int compare(SGAInfo node1, SGAInfo node2) {
          if (node1.getAlive()) {
            return (node2.getAlive()) ? 0 : (-1);
          }
          return (node2.getAlive()) ? 1 : 0;
        }
      };
    }

    /**
     * Retorna um renderizador para a coluna "CPU".
     * 
     * @return renderizador para a coluna "CPU".
     */
    public TableCellRenderer getCPURenderer() {
      return new DefaultTableCellRenderer() {
        @Override
        public Component getTableCellRendererComponent(JTable table,
          Object value, boolean isSelected, boolean hasFocus, int row,
          int column) {
          Component comp =
            super.getTableCellRendererComponent(table, value, isSelected,
              hasFocus, row, column);
          if (!gotRows) {
            ((DefaultTableCellRenderer) comp).setText(DOTTED);
            return comp;
          }
          if (value instanceof SGAInfo) {
            SGAInfo node = (SGAInfo) value;
            String text;
            if (node.getAlive()) {
              int load = (int) node.getCPULoad1();
              if (load < 0) {
                text = DOTTED;
              }
              else {
                text = "" + load + " %";
              }
            }
            else {
              text = DOTTED;
            }
            ((DefaultTableCellRenderer) comp).setText(text);
          }
          return comp;
        }
      };
    }

    /**
     * Retorna um comparador para a coluna "CPU".
     * 
     * @return comparador para a coluna "CPU".
     */
    public Comparator<SGAInfo> getCPUComparator() {
      return new Comparator<SGAInfo>() {
        public int compare(SGAInfo node1, SGAInfo node2) {
          if (node1.getAlive()) {
            if (node2.getAlive()) {
              int cpu1 = (int) node1.getCPULoad1();
              int cpu2 = (int) node2.getCPULoad1();
              if (cpu1 < cpu2) {
                return -1;
              }
              return (cpu1 == cpu2) ? 0 : 1;
            }
            return -1;
          }
          return (node2.getAlive()) ? 1 : 0;
        }
      };
    }

    /**
     * Retorna um renderizador para a coluna "Memria RAM".
     * 
     * @return renderizador para a coluna "Memria RAM".
     */
    public TableCellRenderer getRAMMemoryRenderer() {
      return new DefaultTableCellRenderer() {
        @Override
        public Component getTableCellRendererComponent(JTable table,
          Object value, boolean isSelected, boolean hasFocus, int row,
          int column) {
          Component comp =
            super.getTableCellRendererComponent(table, value, isSelected,
              hasFocus, row, column);
          if (!gotRows) {
            ((DefaultTableCellRenderer) comp).setText(DOTTED);
            return comp;
          }
          if (value instanceof SGAInfo) {
            SGAInfo node = (SGAInfo) value;
            String text;
            if (node.getAlive()) {
              int freemem = (int) node.getRAMFreeMemory();
              int mem = 100 - freemem;
              if (freemem < 0) {
                text = DOTTED;
              }
              else {
                text = "" + mem + " %";
              }
            }
            else {
              text = DOTTED;
            }
            ((DefaultTableCellRenderer) comp).setText(text);
          }
          return comp;
        }
      };
    }

    /**
     * Retorna um comparador para a coluna "Memria RAM".
     * 
     * @return comparador para a coluna "Memria RAM".
     */
    public Comparator<SGAInfo> getRAMMemoryComparator() {
      return new Comparator<SGAInfo>() {
        public int compare(SGAInfo node1, SGAInfo node2) {
          if (node1.getAlive()) {
            if (node2.getAlive()) {
              int free1 = (int) node1.getRAMFreeMemory();
              int free2 = (int) node2.getRAMFreeMemory();
              if (free1 > free2) {
                return -1;
              }
              return (free1 == free2) ? 0 : 1;
            }
            return -1;
          }
          return (node2.getAlive()) ? 1 : 0;
        }
      };
    }

    /**
     * Retorna um renderizador para a coluna "Memria Swap".
     * 
     * @return renderizador para a coluna "Memria Swap".
     */
    public TableCellRenderer getSwapMemoryRenderer() {
      return new DefaultTableCellRenderer() {
        @Override
        public Component getTableCellRendererComponent(JTable table,
          Object value, boolean isSelected, boolean hasFocus, int row,
          int column) {
          Component comp =
            super.getTableCellRendererComponent(table, value, isSelected,
              hasFocus, row, column);
          if (!gotRows) {
            ((DefaultTableCellRenderer) comp).setText(DOTTED);
            return comp;
          }
          if (value instanceof SGAInfo) {
            SGAInfo node = (SGAInfo) value;
            String text;
            if (node.getAlive()) {
              int freemem = (int) node.getSwapFreeMemory();
              int mem = 100 - freemem;
              if (freemem < 0) {
                text = DOTTED;
              }
              else {
                text = "" + mem + " %";
              }
            }
            else {
              text = DOTTED;
            }
            ((DefaultTableCellRenderer) comp).setText(text);
          }
          return comp;
        }
      };
    }

    /**
     * Retorna um comparador para a coluna "Memria Swap".
     * 
     * @return comparador para a coluna "Memria Swap".
     */
    public Comparator<SGAInfo> getSwapMemoryComparator() {
      return new Comparator<SGAInfo>() {
        public int compare(SGAInfo node1, SGAInfo node2) {
          if (node1.getAlive()) {
            if (node2.getAlive()) {
              int free1 = (int) node1.getSwapFreeMemory();
              int free2 = (int) node2.getSwapFreeMemory();
              if (free1 > free2) {
                return -1;
              }
              return (free1 == free2) ? 0 : 1;
            }
            return -1;
          }
          return (node2.getAlive()) ? 1 : 0;
        }
      };
    }

    /**
     * Retorna a plataforma no qual o n est executando.
     * 
     * @param node n (SGA).
     * 
     * @return plataforma no qual o n est executando.
     */
    private String getPlatform(SGAInfo node) {
      return node.getPlatformId();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void invalidateRows() {
    // TODO Auto-generated method stub

  }
}
