package csbase.client.facilities.sgatable;

import java.awt.Component;
import java.awt.Dimension;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Vector;

import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.table.DefaultObjectTableProvider;
import tecgraf.javautils.gui.table.ObjectTableProvider;
import csbase.client.facilities.monitoringtable.MonitoringTable;
import csbase.logic.MonitoringSet;
import csbase.logic.SGAInfo;
import csbase.logic.SGASet;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.SGAServiceInterface;

/**
 * A classe <code>SGATable</code>  um componente de tabela, baseado em uma
 * <code>JTable</code>, que apresenta uma lista de informaes sobre Servidores
 * de Gerncia de Algoritmos (SGAs) ativos.
 * 
 * @author Leonardo Abreu de Barros
 * @version $Revision: 104691 $
 * 
 */
public class SGATable extends MonitoringTable {
  /**
   * Pontilhado que aparece quando no h informao para exibir ou houve falha
   * na comunicao com o servidor.
   */
  private final String DOTTED = "-----";

  /**
   * Indica se conseguiu coletar as informaes no servidor.
   */
  private boolean gotRows;

  /** Provedor de dados para a tabela */
  private MyObjectTableProvider myProvider;

  /**
   * Construtor que constri por default uma tabela de seleo nica.
   */
  public SGATable() {
    this(ListSelectionModel.SINGLE_SELECTION);
  }

  /**
   * Construtor que permite escolher o modo de seleo da tabela.
   * 
   * @param selectionMode Indica o modo de seleo da tabela.
   * 
   * @see javax.swing.JTable#setSelectionMode(int)
   */
  public SGATable(int selectionMode) {
    super(selectionMode);
    setupStateColumn();
    setupCPUColumn();
    setupRAMMemoryColumn();
    setupSwapMemoryColumn();
    setPreferredScrollableViewportSize(new Dimension(800, 100));
  }

  /**
   * Prepara a coluna "Estado", que precisa de renderizadores e comparadores
   * especiais.
   */
  private void setupStateColumn() {
    int columnIndex = myProvider.getColumnNameIndex(LNG.get("MON_STATE"));
    TableColumn column = this.getColumnModel().getColumn(columnIndex);
    column.setCellRenderer(myProvider.getStateRenderer());
    this.setComparator(columnIndex, myProvider.getStateComparator());
  }

  /**
   * Prepara a coluna "CPU", que precisa de renderizadores e comparadores
   * especiais.
   */
  private void setupCPUColumn() {
    int columnIndex = myProvider.getColumnNameIndex(LNG.get("MON_CPU_USAGE"));
    TableColumn column = this.getColumnModel().getColumn(columnIndex);
    column.setCellRenderer(myProvider.getCPURenderer());
    this.setComparator(columnIndex, myProvider.getCPUComparator());
  }

  /**
   * Prepara a coluna "Memria RAM", que precisa de renderizadores e
   * comparadores especiais.
   */
  private void setupRAMMemoryColumn() {
    int columnIndex =
      myProvider.getColumnNameIndex(LNG.get("MON_RAM_MEMORY_USAGE"));
    TableColumn column = this.getColumnModel().getColumn(columnIndex);
    column.setCellRenderer(myProvider.getRAMMemoryRenderer());
    this.setComparator(columnIndex, myProvider.getRAMMemoryComparator());
  }

  /**
   * Prepara a coluna "Memria Swap", que precisa de renderizadores e
   * comparadores especiais.
   */
  private void setupSwapMemoryColumn() {
    int columnIndex =
      myProvider.getColumnNameIndex(LNG.get("MON_SWAP_MEMORY_USAGE"));
    TableColumn column = this.getColumnModel().getColumn(columnIndex);
    column.setCellRenderer(myProvider.getSwapMemoryRenderer());
    this.setComparator(columnIndex, myProvider.getSwapMemoryComparator());
  }

  /**
   * Retorna uma instncia nica (singleton) do provedor de dados para a tabela.
   * 
   * @return uma instncia nica (singleton) do provedor de dados
   */
  @Override
  protected ObjectTableProvider getObjectTableProvider() {
    if (myProvider == null) {
      myProvider = new MyObjectTableProvider();
    }
    return myProvider;
  }

  /**
   * Mtodo para obter novos dados da tabela. (definido como abstrato em
   * MonitoringTable)
   * 
   * @return coleo de objetos, cada qual representando uma linha da tabela.
   * 
   * @throws RemoteException
   */
  @Override
  public Vector<MonitoringSet> getNewRows() throws RemoteException {
    gotRows = false;
    SGAServiceInterface sgaService = ClientRemoteLocator.sgaService;
    if (sgaService == null) {
      return null;
    }
    final Vector<String> sgaNames = sgaService.getAllSGANames();
    Vector<MonitoringSet> newRows = new Vector<MonitoringSet>();
    for (int i = 0; i < sgaNames.size(); i++) {
      String name = sgaNames.get(i);
      SGASet sga = sgaService.getSGASet(name);
      newRows.add(sga);
    }
    gotRows = true;
    return newRows;
  }

  /**
   * Indica se algum SGA est ativo.
   * 
   * @return <code>true</code> se o SGA estiver ativo, <code>false</code> caso
   *         contrrio.
   */
  public boolean isAnySgaAlive() {
    for (int i = 0; i < model.getRowCount(); i++) {
      SGASet sga = ((SGASet) model.getRow(i));
      if (sga.getAlive()) {
        return true;
      }
    }
    return false;
  }

  /**
   * Retorna um objeto contendo informaes sobre o SGA correspondente  linha
   * especificada.
   * 
   * @param row ndice de linha na tabela.
   * 
   * @return objeto contendo informaes sobre um SGA.
   */
  public SGASet getSGASet(int row) {
    return ((SGASet) getMonitoringSet(row));
  }

  /**
   * Provedor de dados para esta tabela.
   */
  private class MyObjectTableProvider extends DefaultObjectTableProvider {

    /**
     * (non-Javadoc)
     * 
     * @see tecgraf.javautils.gui.table.ObjectTableProvider#getColumnNames()
     */
    public String[] getColumnNames() {
      return new String[] { LNG.get("MON_SERVER"), LNG.get("MON_STATE"),
          LNG.get("MON_CPU_USAGE"), LNG.get("MON_RAM_MEMORY_USAGE"),
          LNG.get("MON_SWAP_MEMORY_USAGE"), LNG.get("MON_PLATFORM") };
    }

    /**
     * (non-Javadoc)
     * 
     * @see tecgraf.javautils.gui.table.ObjectTableProvider#getColumnClasses()
     */
    public Class<?>[] getColumnClasses() {
      return new Class[] { String.class, String.class, String.class,
          String.class, String.class, String.class, String.class };
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Object[] getCellValues(Object row) {
      SGASet sga = (SGASet) row;
      return new Object[] { getName(sga), sga, sga, sga, sga,
          sga.getPlatformId() };
    }

    /**
     * Retorna o ndice da coluna com o cabealho especificado.
     * 
     * @param headerName cabealho
     * 
     * @return ndice da coluna no modelo.
     */
    int getColumnNameIndex(String headerName) {
      String[] colNames = getColumnNames();
      for (int i = 0; i < colNames.length; i++) {
        if (headerName.equals(colNames[i])) {
          return i;
        }
      }
      return -1;
    }

    /**
     * Retorna o nome de um SGA. Se for um cluster, retorna tambm o nmero de
     * ns.
     * 
     * @param sga objeto com informaes sobre o SGA.
     * 
     * @return o nome do SGA/cluster.
     */
    private String getName(SGASet sga) {
      if (sga.isCluster()) {
        int num = sga.getNumNodes();
        return (sga.getName() + " (" + num + " " + LNG.get("MON_NODES_CLUSTER") + ")");
      }
      return sga.getName();
    }

    /**
     * Retorna um renderizador para a coluna "Estado".
     * 
     * @return renderizador para a coluna "Estado".
     */
    public TableCellRenderer getStateRenderer() {
      return new DefaultTableCellRenderer() {
        @Override
        public Component getTableCellRendererComponent(JTable table,
          Object value, boolean isSelected, boolean hasFocus, int row,
          int column) {
          Component comp =
            super.getTableCellRendererComponent(table, value, isSelected,
              hasFocus, row, column);
          if (value instanceof SGASet) {
            SGASet sga = (SGASet) value;
            ((DefaultTableCellRenderer) comp).setText(LNG.get("MON_"
              + sga.getStatus()));
          }
          return comp;
        }
      };
    }

    /**
     * Retorna um comparador para a coluna "Estado".
     * 
     * @return comparador para a coluna "Estado".
     */
    public Comparator<SGASet> getStateComparator() {
      return new Comparator<SGASet>() {
        public int compare(SGASet sga1, SGASet sga2) {
          int status1 = SGASet.Status.getIndex(sga1.getStatus());
          int status2 = SGASet.Status.getIndex(sga2.getStatus());
          if (status1 < status2) {
            return -1;
          }
          return (status1 == status2) ? 0 : 1;
        }
      };
    }

    /**
     * Retorna um renderizador para a coluna "CPU".
     * 
     * @return renderizador para a coluna "CPU".
     */
    public TableCellRenderer getCPURenderer() {
      return new DefaultTableCellRenderer() {
        @Override
        public Component getTableCellRendererComponent(JTable table,
          Object value, boolean isSelected, boolean hasFocus, int row,
          int column) {
          Component comp =
            super.getTableCellRendererComponent(table, value, isSelected,
              hasFocus, row, column);
          if (!gotRows) {
            ((DefaultTableCellRenderer) comp).setText(DOTTED);
            return comp;
          }
          if (value instanceof SGASet) {
            SGASet sga = (SGASet) value;
            String text;
            if (sga.getAlive()) {
              if (sga.isCluster()) {
                SGAInfo[] info = sga.getAllInfo();
                ArrayList<Double> values = new ArrayList<Double>();
                for (int i = 0; i < info.length; i++) {
                  double val = info[i].getCPULoad1();
                  if (val >= 0) {
                    values.add(new Double(val));
                  }
                }
                if (values.size() == 0) {
                  text = DOTTED;
                }
                else {
                  double average = getAverage(values);
                  double stDev = getStandardDev(values, average);
                  String mean = "x\u0304 \u0304";
                  char sigma = '\u03c3';
                  text =
                    (" " + mean + " = " + (int) average + " %   ( " + sigma
                      + " = " + (int) stDev + " )");
                }
              }
              else {
                int load = (int) sga.getCPULoad1();
                if (load < 0) {
                  text = DOTTED;
                }
                else {
                  text = "" + load + " %";
                }
              }
            }
            else {
              text = DOTTED;
            }
            ((DefaultTableCellRenderer) comp).setText(text);
          }
          return comp;
        }
      };
    }

    /**
     * Retorna um comparador para a coluna "CPU".
     * 
     * @return comparador para a coluna "CPU".
     */
    public Comparator<SGASet> getCPUComparator() {
      return new Comparator<SGASet>() {
        public int compare(SGASet sga1, SGASet sga2) {
          if (sga1.getAlive()) {
            if (sga2.getAlive()) {
              int cpu1 = getCPUAverage(sga1.getAllInfo());
              int cpu2 = getCPUAverage(sga2.getAllInfo());
              if (cpu1 < cpu2) {
                return -1;
              }
              return (cpu1 == cpu2) ? 0 : 1;
            }
            return -1;
          }
          return (sga2.getAlive()) ? 1 : 0;
        }
      };
    }

    /**
     * Retorna a mdia de uso de CPU para um grupo de SGAs.
     * 
     * @param sgaInfos array de dados de SGAs.
     * 
     * @return mdia de uso de CPU para o grupo de SGAs especificado.
     */
    private int getCPUAverage(SGAInfo[] sgaInfos) {
      if (sgaInfos.length == 1) {
        int load = (int) sgaInfos[0].getCPULoad1();
        return load;
      }
      ArrayList<Double> values = new ArrayList<Double>();
      for (int i = 0; i < sgaInfos.length; i++) {
        double val = sgaInfos[i].getCPULoad1();
        if (val >= 0) {
          values.add(new Double(val));
        }
      }
      return (int) getAverage(values);
    }

    /**
     * Retorna um renderizador para a coluna "Memria RAM".
     * 
     * @return renderizador para a coluna "Memria RAM".
     */
    public TableCellRenderer getRAMMemoryRenderer() {
      return new DefaultTableCellRenderer() {
        @Override
        public Component getTableCellRendererComponent(JTable table,
          Object value, boolean isSelected, boolean hasFocus, int row,
          int column) {
          Component comp =
            super.getTableCellRendererComponent(table, value, isSelected,
              hasFocus, row, column);
          if (!gotRows) {
            ((DefaultTableCellRenderer) comp).setText(DOTTED);
            return comp;
          }
          if (value instanceof SGASet) {
            SGASet sga = (SGASet) value;
            String text;
            if (sga.getAlive()) {
              if (sga.isCluster()) {
                SGAInfo[] info = sga.getAllInfo();
                ArrayList<Double> values = new ArrayList<Double>();
                for (int i = 0; i < info.length; i++) {
                  double val = info[i].getRAMFreeMemory();
                  if (val >= 0) {
                    values.add(new Double(100.0 - val));
                  }
                }
                if (values.size() == 0) {
                  text = DOTTED;
                }
                else {
                  double average = getAverage(values);
                  double stDev = getStandardDev(values, average);
                  String mean = "x\u0304 \u0304";
                  char sigma = '\u03c3';
                  text =
                    (" " + mean + " = " + (int) average + " %   ( " + sigma
                      + " = " + (int) stDev + " )");
                }
              }
              else {
                int freemem = (int) sga.getRAMFreeMemory();
                if (freemem < 0) {
                  text = DOTTED;
                }
                else {
                  int mem = 100 - freemem;
                  text = "" + mem + " %";
                }
              }
            }
            else {
              text = DOTTED;
            }
            ((DefaultTableCellRenderer) comp).setText(text);
          }
          return comp;
        }
      };
    }

    /**
     * Retorna um comparador para a coluna "Memria RAM".
     * 
     * @return comparador para a coluna "Memria RAM".
     */
    public Comparator<SGASet> getRAMMemoryComparator() {
      return new Comparator<SGASet>() {
        public int compare(SGASet sga1, SGASet sga2) {
          if (sga1.getAlive()) {
            if (sga2.getAlive()) {
              int free1 = getRAMFreeMemAverage(sga1.getAllInfo());
              int free2 = getRAMFreeMemAverage(sga2.getAllInfo());
              if (free1 > free2) {
                return -1;
              }
              return (free1 == free2) ? 0 : 1;
            }
            return -1;
          }
          return (sga2.getAlive()) ? 1 : 0;
        }
      };
    }

    /**
     * Retorna a mdia de Memria RAM para um grupo de SGAs.
     * 
     * @param sgaInfos array de dados de SGAs.
     * 
     * @return mdia de Memria RAM para o grupo de SGAs especificado.
     */
    private int getRAMFreeMemAverage(SGAInfo[] sgaInfos) {
      if (sgaInfos.length == 1) {
        return (int) sgaInfos[0].getRAMFreeMemory();
      }
      ArrayList<Double> values = new ArrayList<Double>();
      for (int i = 0; i < sgaInfos.length; i++) {
        double val = sgaInfos[i].getRAMFreeMemory();
        if (val >= 0) {
          values.add(new Double(val));
        }
      }
      return (int) getAverage(values);
    }

    /**
     * Retorna um renderizador para a coluna "Memria Swap".
     * 
     * @return renderizador para a coluna "Memria Swap".
     */
    public TableCellRenderer getSwapMemoryRenderer() {
      return new DefaultTableCellRenderer() {
        @Override
        public Component getTableCellRendererComponent(JTable table,
          Object value, boolean isSelected, boolean hasFocus, int row,
          int column) {
          Component comp =
            super.getTableCellRendererComponent(table, value, isSelected,
              hasFocus, row, column);
          if (!gotRows) {
            ((DefaultTableCellRenderer) comp).setText(DOTTED);
            return comp;
          }
          if (value instanceof SGASet) {
            SGASet sga = (SGASet) value;
            String text;
            if (sga.getAlive()) {
              if (sga.isCluster()) {
                SGAInfo[] info = sga.getAllInfo();
                ArrayList<Double> values = new ArrayList<Double>();
                for (int i = 0; i < info.length; i++) {
                  double val = info[i].getSwapFreeMemory();
                  if (val >= 0) {
                    values.add(new Double(100.0 - val));
                  }
                }
                if (values.size() == 0) {
                  text = DOTTED;
                }
                else {
                  double average = getAverage(values);
                  double stDev = getStandardDev(values, average);
                  String mean = "x\u0304 \u0304";
                  char sigma = '\u03c3';
                  text =
                    (" " + mean + " = " + (int) average + " %   ( " + sigma
                      + " = " + (int) stDev + " )");
                }
              }
              else {
                int freemem = (int) sga.getSwapFreeMemory();
                if (freemem < 0) {
                  text = DOTTED;
                }
                else {
                  int mem = 100 - freemem;
                  text = "" + mem + " %";
                }
              }
            }
            else {
              text = DOTTED;
            }
            ((DefaultTableCellRenderer) comp).setText(text);
          }
          return comp;
        }
      };
    }

    /**
     * Retorna um comparador para a coluna "Memria Swap".
     * 
     * @return comparador para a coluna "Memria Swap".
     */
    public Comparator<SGASet> getSwapMemoryComparator() {
      return new Comparator<SGASet>() {
        public int compare(SGASet sga1, SGASet sga2) {
          if (sga1.getAlive()) {
            if (sga2.getAlive()) {
              int free1 = getSwapFreeMemAverage(sga1.getAllInfo());
              int free2 = getSwapFreeMemAverage(sga2.getAllInfo());
              if (free1 > free2) {
                return -1;
              }
              return (free1 == free2) ? 0 : 1;
            }
            return -1;
          }
          return (sga2.getAlive()) ? 1 : 0;
        }
      };
    }

    /**
     * Retorna a mdia de Memria Swap para um grupo de SGAs.
     * 
     * @param sgaInfos array de dados de SGAs.
     * 
     * @return mdia de Memria Swap para o grupo de SGAs especificado.
     */
    private int getSwapFreeMemAverage(SGAInfo[] sgaInfos) {
      if (sgaInfos.length == 1) {
        return (int) sgaInfos[0].getSwapFreeMemory();
      }
      ArrayList<Double> values = new ArrayList<Double>();
      for (int i = 0; i < sgaInfos.length; i++) {
        double val = sgaInfos[i].getSwapFreeMemory();
        if (val >= 0) {
          values.add(new Double(val));
        }
      }
      return (int) getAverage(values);
    }

    /**
     * Calcula a mdia de um conjunto de valores.
     * 
     * @param values array com os valores
     * 
     * @return a mdia dos valores
     */
    private double getAverage(List<Double> values) {
      int size = values.size();
      if (size == 0)
        return -1;
      double sum = 0;
      for (Double value : values) {
        sum += value.doubleValue();
      }
      return sum / size;
    }

    /**
     * Calcula o desvio padro de um conjunto de valores.
     * 
     * @param values array com os valores
     * @param average mdia dos valores
     * 
     * @return o desvio padro
     */
    private double getStandardDev(List<Double> values, double average) {
      int size = values.size();
      if (size == 0) {
        return -1;
      }
      double sum = 0;
      for (Double value : values) {
        sum += Math.pow(value.doubleValue() - average, 2);
      }
      return Math.sqrt(sum / size);
    }

  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void invalidateRows() {
    // TODO Auto-generated method stub
  }
}
