/**
 * $Id: ImportUsersDialog.java 104691 2010-04-26 21:43:07Z costa $
 */

package csbase.client.ias;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringReader;
import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import tecgraf.javautils.LNG;
import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.configurationmanager.ConfigurationManagerException;
import tecgraf.javautils.gui.GBC;
import tecgraf.javautils.gui.StandardDialogs;
import tecgraf.javautils.gui.table.DefaultObjectTableProvider;
import tecgraf.javautils.gui.table.ObjectTableModel;
import tecgraf.javautils.gui.table.ObjectTableProvider;
import tecgraf.javautils.gui.table.SortableTable;
import csbase.client.applications.ApplicationImages;
import csbase.client.desktop.DesktopComponentDialog;
import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.RemoteTask;
import csbase.client.kernel.ClientException;
import csbase.client.project.ClientProjectFileType;
import csbase.client.project.ProjectFileChooser;
import csbase.client.project.ProjectFileChooserOpen;
import csbase.client.project.ProjectTreePath;
import csbase.client.remote.srvproxies.UserGroupProxy;
import csbase.client.util.ClientUtilities;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.ClientProjectFile;
import csbase.logic.ProjectFileType;
import csbase.logic.User;
import csbase.logic.UserGroup;
import csbase.logic.UserInfo;
import csbase.util.parser.exception.MalformedInputException;
import csbase.util.parser.tsv.TsvParser;

/**
 * Tela para importao de usurios a partir de arquivos CSV.
 */
public final class ImportUsersDialog {
  /**
   * Classes das colunas da tabela de importao.
   */
  private static final Class<?>[] COL_CLASSES =
    new Class[] { String.class, String.class, String.class, String.class };
  /**
   * Ttulos das colunas da tabela de importao.
   */
  private static final String[] COL_NAMES =
    new String[] { LNG.get("IAS_USER"), LNG.get("IAS_USER_NAME"),
        LNG.get("IAS_USER_EMAIL"), LNG.get("IAS_USERGROUP") };
  private Window owner;
  private String windowLabel;
  private DesktopComponentDialog window;
  private JButton roleButton;
  private JButton permissionButton;
  private JButton actionButton;
  private JTextField filePath;
  private Container contentPane;
  private ClientProjectFile currentFile;
  private SortableTable sortableTable;
  private ObjectTableModel<String[]> tableModel;
  private UserRoleDefinitionDialog userRoleDefinitionDialog;
  private UserPermissionDefinitionDialog userPermissionDefinitionDialog;
  private UserInfo defaultUserInfo;
  private static final String FILE_TYPE = "CSV";
  private static final int FILE_PATH_COLUMNS = 30;
  private static final int COLUMN_COUNT = 4;
  private static final int LOGIN_COLUMN = 0;
  private static final int NAME_COLUMN = 1;
  private static final int E_MAIL_COLUMN = 2;
  private static final int USERGROUP_COLUMN = 3;

  /** Listas dos novos usurios que foram importados. */
  private List<String> insertedUserList = new ArrayList<String>();

  /** Lista dos usurios que foram alterados com a importao. */
  private List<String> editedUserList = new ArrayList<String>();

  /**
   * Resultados da operao de importar os usurios.
   */
  private enum ImportResults {
    /**
     * Todos foram importados.
     */
    ALL_IMPORTED,
    /**
     * Apenas alguns foram importados.
     */
    SOME_IMPORTED,
    /**
     * Nenhum foi importado.
     */
    NONE_IMPORTED
  }

  /**
   * Construtor.
   * 
   * @param owner - janela-me desta
   */
  public ImportUsersDialog(Window owner) {
    this.owner = owner;
    this.windowLabel = LNG.get("IAS_IMPORT_USERS_TITLE");
    this.window = new DesktopComponentDialog((JFrame) owner, this.windowLabel);
    this.window.center(owner);
    this.contentPane = this.window.getContentPane();
    this.defaultUserInfo = new UserInfo();
    this.userRoleDefinitionDialog =
      new UserRoleDefinitionDialog(this.window, this.defaultUserInfo, true);
    this.userPermissionDefinitionDialog =
      new UserPermissionDefinitionDialog(this.window, this.defaultUserInfo,
        true);
  }

  /**
   * Exibe o dilogo.
   */
  public void showDialog() {
    createFileChooserPanel();
    createUsersPanel();
    createButtonPanel();
    this.window.pack();
    this.window.center(this.owner);
    this.window.setVisible(true);
  }

  private void createUsersPanel() {
    JPanel usersPanel = new JPanel();
    usersPanel.setLayout(new BorderLayout());
    createTable();
    usersPanel.add(new JScrollPane(this.sortableTable), BorderLayout.CENTER);
    this.contentPane.add(usersPanel, BorderLayout.CENTER);
  }

  private void createTable() {
    ObjectTableProvider provider = new DefaultObjectTableProvider() {
      /**
       * {@inheritDoc}
       */
      @Override
      public Object[] getCellValues(Object item) {
        Object[] line = (Object[]) item;
        return (line == null) ? null : new String[] {
            (String) line[LOGIN_COLUMN], (String) line[NAME_COLUMN],
            (String) line[E_MAIL_COLUMN], (String) line[USERGROUP_COLUMN] };
      }

      /**
       * {@inheritDoc}
       */
      public String[] getColumnNames() {
        return COL_NAMES;
      }

      /**
       * {@inheritDoc}
       */
      public Class<?>[] getColumnClasses() {
        return COL_CLASSES;
      }
    };
    this.tableModel =
      new ObjectTableModel<String[]>(new ArrayList<String[]>(), provider);
    this.sortableTable = new SortableTable(tableModel);
    this.sortableTable.addMouseListener(new MouseAdapter() {
      @Override
      public void mousePressed(MouseEvent e) {
        updateActionButton();
      }
    });
  }

  private void createFileChooserPanel() {
    JPanel fileChooserPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
    JLabel theLabel = new JLabel(LNG.get("IAS_USERS_FILE") + ':');
    this.filePath = new JTextField(FILE_PATH_COLUMNS);
    this.filePath.setEditable(false);
    JButton findButton = createFindButton();
    fileChooserPanel.add(theLabel);
    fileChooserPanel.add(filePath);
    fileChooserPanel.add(findButton);
    this.contentPane.add(fileChooserPanel, BorderLayout.NORTH);
  }

  private JButton createFindButton() {
    final JButton findButton =
      ClientUtilities.createImageButton(ApplicationImages.ICON_OPEN_16);
    findButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
        if (useProject()) {
          showProjectFileChooser();
        }
        else {
          showFileChooser();
        }
      }
    });
    return findButton;
  }

  /**
   * Indica se a aplicao utiliza o conceito de projeto.
   * 
   * @return true, se aplicao utiliza projeto, false caso contrrio.
   */
  private boolean useProject() {
    try {
      Configuration configuration =
        ConfigurationManager.getInstance().getConfiguration(
          UserManagerPanel.class);
      return configuration.getOptionalBooleanProperty(
        "application.use.project", true);
    }
    catch (ConfigurationManagerException cme) {
      return true;
    }
  }

  /**
   * Exibe a arvore de projeto para escolha do arquivo que ser importado.
   */
  private void showProjectFileChooser() {
    ProjectFileChooserOpen chooser;
    try {
      // Se no existe arquivo CSV no sistema, usa o TEXT.
      String fileTypeExt = FILE_TYPE;
      ClientProjectFileType projFileType =
        ClientProjectFileType.getFileType(FILE_TYPE);
      if (projFileType.getCode().equals(ProjectFileType.UNKNOWN)) {
        fileTypeExt = "TEXT";
      }
      chooser =
        new ProjectFileChooserOpen(window, DesktopFrame.getInstance()
          .getProject(), false, ProjectFileChooser.FILE_ONLY, fileTypeExt);
    }
    catch (ClientException e) {
      StandardErrorDialogs.showErrorDialog(window, e);
      return;
    }
    ProjectTreePath treePath = chooser.getSelectedPath();
    if (treePath != null) {
      updateChosenFile(treePath.getFile());
    }
  }

  /**
   * Exibe o sistema de arquivo local para escolha do arquivo que ser
   * importado.
   */
  private void showFileChooser() {
    File file = null;
    try {
      JFileChooser fc = new JFileChooser();
      fc.setAcceptAllFileFilterUsed(true);
      fc.addChoosableFileFilter(new Filter("txt", LNG
        .get("ias.file_type.txt.description")));
      fc.addChoosableFileFilter(new Filter("csv", LNG
        .get("ias.file_type.csv.description")));
      int returnVal = fc.showOpenDialog(window);
      if (returnVal == JFileChooser.APPROVE_OPTION) {
        file = fc.getSelectedFile();
      }
    }
    catch (Exception e) {
      StandardErrorDialogs.showErrorDialog(window, windowLabel, LNG
        .get("IAS_IMPORT_USERS_LOAD_DATA_FAILED"));
      return;
    }
    if (file != null) {
      updateChosenFile(file);
    }
  }

  /**
   * Atualiza a tabela com os dados do arquivo escolhido.
   * 
   * @param clientProjectFile - arquivo CSV com os usurios a serem importados
   */
  private void updateChosenFile(ClientProjectFile clientProjectFile) {
    if (clientProjectFile.equals(this.currentFile)) {
      return;
    }
    this.currentFile = clientProjectFile;
    StringBuffer buffer = this.fillBuffer(clientProjectFile);
    this.tableModel.clear();
    try {
      this.loadDataFromFile(new StringReader(buffer.toString()));
    }
    catch (Exception e) {
      StandardErrorDialogs.showErrorDialog(window, windowLabel, LNG
        .get("IAS_IMPORT_USERS_LOAD_DATA_FAILED"));
      e.printStackTrace();
      return;
    }
    this.filePath.setText(this.currentFile.getStringPath());
    updateActionButton();
  }

  /**
   * Atualiza a tabela com os dados do arquivo escolhido.
   * 
   * @param file - arquivo com os usurios a serem importados.
   */
  private void updateChosenFile(File file) {
    this.currentFile = null;
    this.tableModel.clear();
    BufferedReader reader = null;
    try {
      FileInputStream fileStream = new FileInputStream(file);
      reader = new BufferedReader(new InputStreamReader(fileStream));
      this.loadDataFromFile(reader);
    }
    catch (Exception e) {
      StandardErrorDialogs.showErrorDialog(window, windowLabel, LNG
        .get("IAS_IMPORT_USERS_LOAD_DATA_FAILED"));
      e.printStackTrace();
      return;
    }
    this.filePath.setText(file.getName());
    updateActionButton();
  }

  private void updateActionButton() {
    boolean canEnableActionButton = false;
    if (this.sortableTable.getSelectedRow() != -1) {
      canEnableActionButton = true;
    }
    this.actionButton.setEnabled(canEnableActionButton);
  }

  private StringBuffer fillBuffer(final ClientProjectFile projectFile) {
    RemoteTask<StringBuffer> createBufferTask = new RemoteTask<StringBuffer>() {
      @Override
      public void performTask() throws Exception {
        if (projectFile.openFileChannel(true)) {
          try {
            ByteBuffer byteBuffer =
              ByteBuffer.allocate((int) projectFile.size());
            projectFile.read(byteBuffer, 0);
            String byteData = new String(byteBuffer.array());
            StringBuffer buffer = new StringBuffer();
            buffer.append(byteData);
            setResult(buffer);
          }
          finally {
            projectFile.close();
          }
        }
      }
    };

    if (createBufferTask.execute(window, windowLabel, LNG
      .get("IAS_IMPORT_IMPORTING"))) {
      return createBufferTask.getResult();
    }
    return new StringBuffer();
  }

  /**
   * L os dados do arquivo e preenche a tabela.
   * 
   * @param reader - fonte para leitura dos dados
   * @throws IOException se houver algum erro de I/O
   * @throws MalformedInputException se os dados estiverem mal-formados
   */
  private void loadDataFromFile(Reader reader) throws IOException,
    MalformedInputException {
    TsvParser tsvParser = new TsvParser();
    String[][] fields = tsvParser.parse(reader);
    int linesRead = 0;
    for (int i = 0; i < fields.length; i++) {
      if (fields[i].length != COLUMN_COUNT) {
        continue;
      }
      this.tableModel.add(new String[] { fields[i][0], fields[i][1],
          fields[i][2], fields[i][3] });
      linesRead++;
    }
    if (linesRead < fields.length) {
      /*
       * nem todas as linhas foram aproveitadas
       */
      StandardDialogs.showWarningDialog(window, LNG
        .get("IAS_IMPORT_USERS_TITLE"), LNG
        .get("IAS_IMPORT_USERS_LOAD_DATA_PARTIAL"));
    }
  }

  private void createButtonPanel() {
    createRoleButton();
    createPermissionButton();
    createActionButton();
    JButton closeButton = createCloseButton();
    ClientUtilities.adjustEqualSizes(roleButton, permissionButton,
      actionButton, closeButton);
    JPanel buttonsPanel = new JPanel();
    buttonsPanel.add(roleButton);
    buttonsPanel.add(permissionButton);
    buttonsPanel.add(actionButton);
    buttonsPanel.add(closeButton);
    this.contentPane.add(buttonsPanel, BorderLayout.SOUTH);
  }

  private void createPermissionButton() {
    this.permissionButton = new JButton(LNG.get("IAS_USER_PERMISSIONS_BUTTON"));
    permissionButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        userPermissionDefinitionDialog.showDialog();
      }
    });
  }

  private void createRoleButton() {
    this.roleButton = new JButton(LNG.get("IAS_USER_ROLES_BUTTON"));
    this.roleButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        userRoleDefinitionDialog.showDialog();
      }
    });
  }

  private JButton createCloseButton() {
    JButton closeButton = new JButton(LNG.get("IAS_CLOSE"));
    closeButton.setEnabled(true);
    closeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        window.close();
      }
    });
    return closeButton;
  }

  /**
   * Cria o boto que efetivamente importa os usurios lidos do arquivo.
   */
  private void createActionButton() {
    actionButton = new JButton(LNG.get("IAS_IMPORT_USERS_ACTION_BUTTON"));
    actionButton.setEnabled(false);
    actionButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        RemoteTask<ImportResults> remoteTask = new RemoteTask<ImportResults>() {
          @Override
          protected void performTask() throws Exception {
            setResult(importUsers());
          }
        };
        remoteTask.execute(window, windowLabel, LNG
          .get("IAS_WAITING_USER_IMPORTING"));
        ImportResults result = remoteTask.getResult();
        if (result == ImportResults.NONE_IMPORTED) {
          /*
           * nenhum usurio foi importado
           */
          StandardDialogs.showErrorDialog(window, LNG
            .get("IAS_IMPORT_USERS_TITLE"), LNG.get("IAS_IMPORT_ALL_FAILED"));
        }
        else {
          if (result == ImportResults.SOME_IMPORTED) {
            /*
             * avisamos que alguns usurios no foram importados
             */
            StandardDialogs
              .showWarningDialog(window, LNG.get("IAS_IMPORT_USERS_TITLE"), LNG
                .get("IAS_IMPORT_SOME_FAILED"));
          }
          ImportedUserDialog importedUserDialog =
            new ImportedUserDialog(window, insertedUserList, editedUserList);
          importedUserDialog.showDialog();
        }
        insertedUserList.clear();
        editedUserList.clear();
      }
    });
  }

  /**
   * Importa os usurios selecioandos da lista de disponveis. Os usurios cujos
   * grupos no existam no sero criados.
   * 
   * @return 0 se todos os usurios foram importados com sucesso, 1 se apenas
   *         alguns foram importados, 2 se nenhum foi importado
   * @throws Exception
   */
  private ImportResults importUsers() throws Exception {
    int[] selectedRows = this.sortableTable.getSelectedRows();
    List<UserInfo> usersToImport = new ArrayList<UserInfo>();
    for (int i = 0; i < selectedRows.length; i++) {
      String login =
        (String) this.sortableTable.getValueAt(selectedRows[i],
          ImportUsersDialog.LOGIN_COLUMN);
      String name =
        (String) this.sortableTable.getValueAt(selectedRows[i],
          ImportUsersDialog.NAME_COLUMN);
      String eMail =
        (String) this.sortableTable.getValueAt(selectedRows[i],
          ImportUsersDialog.E_MAIL_COLUMN);
      String userGroupName =
        (String) this.sortableTable.getValueAt(selectedRows[i],
          ImportUsersDialog.USERGROUP_COLUMN);
      UserGroup userGroup =
        UserGroupProxy.getUserGroupByName(this.window, this.windowLabel, LNG
          .get("IAS_WAITING_USERGROUP"), userGroupName);
      if (userGroup != null) {
        UserInfo userInfo =
          new UserInfo(login, name, eMail, (Object[]) this.defaultUserInfo
            .getAttribute(User.ROLE_IDS), (Object[]) this.defaultUserInfo
            .getAttribute(User.PERMISSION_IDS));
        userInfo.setAttribute(User.PASSWORD, login);
        userInfo.setAttribute(UserGroup.USERGROUP_ID, userGroup.getId());
        usersToImport.add(userInfo);
      }
    }
    ImportResults result = ImportResults.ALL_IMPORTED;
    if (usersToImport.isEmpty()) {
      /*
       * todos os grupos eram invlidos. nenhum usurio vai ser importado.
       */
      return ImportResults.NONE_IMPORTED;
    }
    else if (usersToImport.size() < selectedRows.length) {
      /*
       * alguns grupos eram invlidos, portanto alguns usurios no sero
       * importados
       */
      result = ImportResults.SOME_IMPORTED;
    }
    for (UserInfo ui : usersToImport) {
      importUser(ui);
    }
    return result;
  }

  /**
   * Importa um usurio.
   * 
   * @param newUserInfo usurio que ser importado.
   * 
   * @throws Exception
   */
  private void importUser(UserInfo newUserInfo) throws Exception {
    User user = User.getUser(newUserInfo.getLogin());

    // Insere um novo usurio.
    if (user == null) {
      User.createUser(newUserInfo);
      this.insertedUserList.add(newUserInfo.getLogin());
    }

    // Atualiza um usurio j existente.
    else {
      this.updateUser(user, newUserInfo);
      this.editedUserList.add(user.getUserInfo().getLogin());
    }
  }

  /**
   * Atualiza as informaes de um usurio.
   * 
   * @param currentUser usurio que ser atualizado.
   * @param newUserInfo novas informaes do usurio.
   * 
   * @throws Exception
   */
  private void updateUser(User currentUser, UserInfo newUserInfo)
    throws Exception {
    List<Object> permissionList = new ArrayList<Object>();
    List<Object> roleList = new ArrayList<Object>();

    // Obtm permisses.
    permissionList =
      merge((Object[]) currentUser.getUserInfo().getAttribute(
        User.PERMISSION_IDS), (Object[]) newUserInfo
        .getAttribute(User.PERMISSION_IDS));

    // Obtm perfis.
    roleList =
      merge((Object[]) currentUser.getUserInfo().getAttribute(User.ROLE_IDS),
        (Object[]) newUserInfo.getAttribute(User.ROLE_IDS));

    // Atualiza info.
    UserInfo userInfo = currentUser.getUserInfo();
    userInfo.setAttribute(User.LOGIN, newUserInfo.getAttribute(User.LOGIN));
    userInfo.setAttribute(User.NAME, newUserInfo.getAttribute(User.NAME));
    userInfo.setAttribute(User.MAIL, newUserInfo.getAttribute(User.MAIL));
    userInfo.setAttribute(User.PERMISSION_IDS, permissionList.toArray());
    userInfo.setAttribute(User.ROLE_IDS, roleList.toArray());
    userInfo.setAttribute(UserGroup.USERGROUP_ID, newUserInfo
      .getAttribute(UserGroup.USERGROUP_ID));

    // Atualiza usurio.
    User.modifyUser(currentUser.getId(), userInfo);
  }

  /**
   * Faz um merge de dois arrays.
   * 
   * @param array1
   * @param array2
   * 
   * @return lista com o resultado do merge.
   */
  private List<Object> merge(Object[] array1, Object[] array2) {
    List<Object> returnList = new ArrayList<Object>();
    for (int i = 0; i < array1.length; i++) {
      returnList.add(array1[i]);
    }
    for (int i = 0; i < array2.length; i++) {
      if (!returnList.contains(array2[i])) {
        returnList.add(array2[i]);
      }
    }
    return returnList;
  }

  /**
   * Classe responsvel por exibir duas listas com o login de usurios.
   * 
   * @author ntl9
   */
  private class ImportedUserDialog {
    /** Dialogo. */
    private DesktopComponentDialog dialog;
    private Window owner;

    /** Tamanho da lista. */
    private final int LIST_HEIGHT = 300;
    private final int LIST_WIDTH = 250;

    /**
     * Constri um dilogo modal, com duas lista de usurios.
     * 
     * @param owner janela pai do dialogo.
     * @param insertedList lista dos usurios inseridos na importao.
     * @param editedList lista dos usurios alterados na importao.
     */
    public ImportedUserDialog(Window owner, List<String> insertedList,
      List<String> editedList) {
      if (insertedList != null && editedList != null) {
        this.owner = owner;
        createDialog(insertedList, editedList);
      }
    }

    private void createDialog(List<String> insertedList, List<String> editedList) {
      // Cria e cofigura o deskComponentDialog
      dialog = DesktopComponentDialog.createDialog(this.owner);
      dialog.setTitle(LNG.get("IAS_IMPORT_RESULT"));
      dialog.setDefaultCloseOperation(DesktopComponentDialog.DISPOSE_ON_CLOSE);

      // Cria as lista que sero exibidas.
      JList insertedJList =
        new JList(insertedList.toArray(new String[insertedList.size()]));
      insertedJList.setPreferredSize(new Dimension(this.LIST_WIDTH,
        this.LIST_HEIGHT));
      insertedJList.setVisibleRowCount(10);
      JList editedJList =
        new JList(editedList.toArray(new String[editedList.size()]));
      editedJList.setPreferredSize(new Dimension(this.LIST_WIDTH,
        this.LIST_HEIGHT));
      editedJList.setVisibleRowCount(10);

      JPanel panel = new JPanel(new GridBagLayout());
      // label "inseridos"
      GBC gbc = new GBC().west().insets(11, 11, 0, 11);
      panel.add(new JLabel(LNG.get("IAS_INSERTED")), gbc);

      // lista de usurios inseridos
      gbc = new GBC(0, 1).both().insets(5, 11, 0, 11);
      panel.add(new JScrollPane(insertedJList), gbc);

      // label de "atualizados"
      gbc = new GBC(1, 0).west().insets(11, 0, 0, 11);
      panel.add(new JLabel(LNG.get("IAS_EDITED")), gbc);

      // lista de usurios atualizados
      gbc = new GBC(1, 1).both().insets(5, 0, 0, 11);
      panel.add(new JScrollPane(editedJList), gbc);

      // boto para fechar a janela
      gbc = new GBC(0, 2).width(2).center().insets(17, 0, 11, 0);
      panel.add(createCloseButton(), gbc);
      dialog.getContentPane().add(panel);
    }

    /**
     * Cria o boto responsvel por fechar o dilogo.
     * 
     * @return .
     */
    private JButton createCloseButton() {
      JButton closeButton = new JButton(LNG.get("IAS_CLOSE"));

      // Evendo de fechar a janela.
      closeButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent evento) {
          dialog.close();
        }
      });
      return closeButton;
    }

    /**
     * Exibe o dilogo.
     */
    public void showDialog() {
      dialog.pack();
      dialog.center(this.owner);
      dialog.setVisible(true);
    }
  }
}
