package csbase.client.ias;

import java.awt.Color;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.text.MessageFormat;
import java.util.Collections;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.GUIUtils;
import csbase.client.remote.ClientRemoteMonitor;
import csbase.client.remote.srvproxies.UserGroupProxy;
import csbase.client.remote.srvproxies.UserProxy;
import csbase.client.util.ClientUtilities;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.User;
import csbase.logic.UserGroup;

/**
 * Dilogo para incluso de dados de um ou mais usurios.
 * 
 * <p>NOTA 1: Caso o mtodo de autenticao do servidor no seja validao 
 * local, o dilogo exibe um <i>checkbox</i> permitindo uma autenticao local 
 * "forada". Isto  necessrio devido a uma circunstncia atual na Petrobras
 * de descentralizao na administrao de servidores LDAP. Como existem muitas
 * configuraes diferentes, o sistema tem dificuldade em conectar-se a alguns
 * desses servidores, o que impossibilita usurios cadastrados nesses servidores
 * de acessarem o sistema. Para contornar esse problema, foi criada essa 
 * possibilidade de login local forado para o administrador poder atribuir a
 * usurios enquadrados nessa situao, enquanto uma soluo mais geral no for
 * encontrada.</p> 
 * 
 * <p>NOTA 2: a sigla "DTO" usada na documentao refere-se ao padro <i>Data
 * Transfer Object</i>, utilizado por um objeto que centraliza as informaes
 * do usurio (ver {@link UserInfoDialog#userInfo}).</p>
 * 
 * @author Leonardo Barros
 */
public class IncludeUserInfoDialog extends UserInfoDialog {
  /** Combo de grupo de usurios ao qual o usurio pertence */
  private JComboBox userGroupComboBox;
  /** Checkbox indicando login local forado. */
  private JCheckBox forceLocalLoginCheckBox;
  /** Boto de incluso de usurio */
  private JButton includeUserButton;
  /** Boto de perfis */
  private JButton roleButton;
  /** Boto de permisses */
  private JButton permissionButton;

  /**
   * Cria o dilogo de incluso de usurios.
   * 
   * @param ownerWindow janela que criou este dilogo.
   */
  public IncludeUserInfoDialog(Window ownerWindow) {
    super(
      ownerWindow, 
      ClientUtilities.addSystemNameToTitle(
        LNG.get("ias.user.inclusion_title")));
  }

  /**
   * {@inheritDoc}
   * <br>Acrescenta ao DTO as selees de grupo de usurio e atributo "login
   * local forado".
   * <br>OBS: Os dilogos de informaes complementares (perfis, permisses e
   * senhas) se encarregam de atualizar o DTO, portanto no  necessrio 
   * atualizar essas informaes aqui.
   */
  protected void updateUserInfo() {
    super.updateUserInfo();
    UserGroupName userGroupName = (UserGroupName) userGroupComboBox.
        getSelectedItem();
    UserGroup userGroup = userGroupName.getUserGroup();
    userInfo.setAttribute(UserGroup.USERGROUP_ID, userGroup.getId());
    if (!ClientRemoteMonitor.getInstance().isLocalLogin()) {
      Boolean forceLocalLogin =
          new Boolean(forceLocalLoginCheckBox.isSelected());
      userInfo.setAttribute(User.FORCE_LOCAL_LOGIN, forceLocalLogin);
    }
  }

  /**
   * {@inheritDoc}
   * <br>Acrescenta ao painel o boto "Incluir".
   * 
   * @see IncludeUserInfoDialog#makeIncludeUserButton()
   */
  protected JPanel makeLowerPanel() {
    JPanel buttonPanel = super.makeLowerPanel();
    includeUserButton = makeIncludeUserButton();
    includeUserButton.setEnabled(false);
    buttonPanel.add(includeUserButton, 0);
    return buttonPanel;
  }
  
  /**
   * Cria o boto de incluso de usurio, atribuindo-lhe como ao o mtodo
   * {@link IncludeUserInfoDialog#createUser()}.
   *
   * @return o boto de incluso de usurio.
   */
  private JButton makeIncludeUserButton() {
    JButton button = new JButton(LNG.get("IAS_USER_INCLUDE"));
    button.addActionListener(
        new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        createUser();
      }
    });
    return button;
  }

  /**
   * Executa a ao de criar o usurio, usando os dados preenchidos nos campos
   * do dilogo. Se a criao for bem-sucedida, limpa os campos, possibilitando
   * mltiplas incluses em srie.
   * O mtodo assume que qualquer erro ocorrido na criao do usurio dever
   * ser reportado no ponto em que ocorrer, portanto no se preocupa com a 
   * exibio de mensagem de erro, apenas de sucesso.
   *
   * @return <code>true</code> se o usurio tiver sido criado com sucesso,
   *         <code>false</code> se este no tiver sido criado, seja por falta
   *         de informaes, seja por erro na criao.
   */
  protected boolean createUser() {
    updateUserInfo();
    // Caso o servidor aceite troca de senhas, pede uma senha para o login.
    // Caso contrrio, cadastra como senha local o prprio login do usurio, a
    // qual s ser usada caso o administrador force o login local para esse
    // usurio.
    if (ClientRemoteMonitor.getInstance().canChangePasswords() ||
        (forceLocalLoginCheckBox != null &&
         forceLocalLoginCheckBox.isSelected())) {
      IncludePasswordDialog dialog =
          new IncludePasswordDialog(userInfoDialog);
      String password = dialog.getNewPassword();
      if (password == null) {
        return false;
      }
      userInfo.setAttribute(User.PASSWORD, password);
    }
    else {
      userInfo.setAttribute(User.PASSWORD, userInfo.getLogin());
    }
    User user =
      UserProxy.createUser(
        userInfoDialog, dialogTitle, LNG.get("IAS_WAITING_USER_CREATION"), 
        userInfo);
    if (user == null) {
      return false;
    }
    JOptionPane.showMessageDialog(
        userInfoDialog,
        MessageFormat.format(
        LNG.get("IAS_USER_INCLUDED_WITH_SUCCESS"),
        new Object[] {user.getLogin()}), dialogTitle,
        JOptionPane.INFORMATION_MESSAGE);
    clearFields();
    return true;
  }

  /**
   * Limpa todos os campos de dados do usurio (menos o combo de UserGroup e o 
   * checkbox de login local forado - estes permanecem com a ltima seleo 
   * para facilitar a incluso sucessiva de vrios usurios com essas mesmas
   * caractersticas.
   * <br>Desabilita tambm o boto de incluir.
   */
  protected void clearFields() {
    // #TODO Passar a limpeza dos campos superiores para a classe pai. 
    login.setText("");
    name.setText("");
    email.setText("");
    login.requestFocus();
    clearRoleSelection();
    clearPermissionSelection();
    includeUserButton.setEnabled(false);
  }

  /**
   * Limpa a seleo de permisses.
   */
  private void clearPermissionSelection() {
    userInfo.setAttribute(User.PERMISSION_IDS, new Object[]{});
  }

  /**
   * Limpa a seleo de perfis.
   */
  private void clearRoleSelection() {
    userInfo.setAttribute(User.ROLE_IDS, new Object[]{});
  }

  /**
   * {@inheritDoc}
   * <br>Adiciona listeners de mudana nos dados aos campos do dilogo.
   */
  protected void makeDialog() {
    try {
      super.makeDialog();
      // #TODO passar a adio de listeners para os campos superiores para
      // a superclasse.
      addUserInfoChangeListeners();
    }
    catch (IllegalStateException ex) {
      StandardErrorDialogs.showErrorDialog(ownerWindow, dialogTitle, ex.getMessage());
    }
  }

  /**
   * Adiciona listeners de mudana nos dados a todos os campos do dilogo.
   */
  private void addUserInfoChangeListeners() {
    // #TODO Passar os change listeners dos campos para o pai.
    UserInfoChangeListener changeListener = new UserInfoChangeListener();
    login.addKeyListener(changeListener);
    name.addKeyListener(changeListener);
    email.addKeyListener(changeListener);
    if (userGroupComboBox != null) {
      userGroupComboBox.addItemListener(changeListener);
    }
    // #TODO Teste: simplesmente "forceLocalLoginCheckBox != null"
    if (!ClientRemoteMonitor.getInstance().isLocalLogin()) {
      forceLocalLoginCheckBox.addItemListener(changeListener);
    }
  }

  /**
   * Constri o combo de grupos de usurios disponveis, ordenados 
   * alfabeticamente.
   *
   * @return o combo de grupos de usurios disponveis.
   */
  private JComboBox makeUserGroupComboBox() {
    JComboBox combo = new JComboBox();
    Vector userGroups =
      UserGroupProxy.getAllUserGroups(
        userInfoDialog, dialogTitle, LNG.get("IAS_WAITING_ALL_USERGROUPS"));
    if (userGroups == null) {
      throw new IllegalStateException(LNG.get("ias.usergroup.retrieval_error"));
    }
    Collections.sort(userGroups, UserGroup.getNameComparator());
    int userGroupSize = userGroups.size();
    for (int i = 0; i < userGroupSize; i++) {
      combo.addItem(new UserGroupName( (UserGroup) userGroups.get(i)));
    }
    if (userGroupSize > 0) {
      combo.setSelectedIndex(0);
    }
    return combo;
  }

  /**
   * {@inheritDoc}
   */
  protected JPanel makeUpperPanel() {
    // Atualmente  necessrio sobrescrever por completo o mtodo makeMainPanel
    // da superclasse, pois no  possvel aproveitar o uso do mtodo utilitrio
    // de criao de painis se todos os parmetros no forem passados juntos.
    // Outro problema desse mtodo  usar como parmetro um array e no uma
    // java.util.List, obrigando a implementao a seguir. Uma reengenharia no
    // mtodo de construo de painis facilitaria substancialmente a presente
    // implementao.
    JLabel loginL = new JLabel(LNG.get("IAS_USER"));
    login = new JTextField();
    JLabel nameL = new JLabel(LNG.get("IAS_USER_NAME"));
    name = new JTextField(20);
    JLabel emailL = new JLabel(LNG.get("IAS_USER_EMAIL"));
    email = new JTextField();
    JLabel userGroupL = new JLabel(LNG.get("IAS_USER_USERGROUP"));
    userGroupComboBox = makeUserGroupComboBox();
    int rowCount = 4;
    JLabel forceLocalLoginL = null;
    if (!ClientRemoteMonitor.getInstance().isLocalLogin()) {
      forceLocalLoginL = new JLabel(LNG.get("IAS_USER_AUTHENTICATION"));
      forceLocalLoginCheckBox = new JCheckBox(LNG.get(
          "IAS_USER_FORCE_LOCAL_LOGIN"));
      rowCount++;
    }
    JComponent[][] rows = new JComponent[rowCount][];
    rows[0] = new JComponent[] {
        loginL, login};
    rows[1] = new JComponent[] {
        nameL, name};
    rows[2] = new JComponent[] {
        emailL, email};
    rows[3] = new JComponent[] {
        userGroupL, userGroupComboBox};
    if (!ClientRemoteMonitor.getInstance().isLocalLogin()) {
      rows[4] = new JComponent[] {
          forceLocalLoginL, forceLocalLoginCheckBox};
    };
    return GUIUtils.createBasicGridPanel(rows);
  }

  /**
   * {@inheritDoc}
   * <br>Se o usurio confirmar a incluso, verifica se os dados requeridos
   * esto preenchidos (ver {@link UserInfoDialog#requiredFieldsAreFilled()}. 
   * Se no estiverem, pede o preenchimento e cancela o fechamento.
   */
  protected boolean confirmClose() {
    // #TODO Usar constantes para todos os tags de propriedades.
    Object[] options = {
        LNG.get("IAS_INCLUDE"), LNG.get("IAS_NOT_INCLUDE"), 
        LNG.get("IAS_CANCEL")
    };
    int selected =
        JOptionPane.showOptionDialog(
        userInfoDialog, LNG.get("IAS_USER_ADD_CONFIRMATION"),
        LNG.get("IAS_USER_ADD_NOT_DONE_TITLE"),
        JOptionPane.DEFAULT_OPTION, JOptionPane.INFORMATION_MESSAGE, null,
        options, options[0]);
    if (options[selected].equals(LNG.get("IAS_INCLUDE"))) {
      if (requiredFieldsAreFilled()) {
        return createUser();
      }
      StandardErrorDialogs.showErrorDialog(
        userInfoDialog, dialogTitle, LNG.get("IAS_FILL_ALL_FIELDS"));
      return false;
    }
    if (options[selected].equals(LNG.get("IAS_NOT_INCLUDE"))) {
      return true;
    }
    return false;
  }

  /**
   * {@inheritDoc}
   * <br>Acrescenta os botes de seleo de permisses e seleo de perfis ao
   * painel.</br>
   */
  protected JPanel makeMiddlePanel() {
    JPanel panel = new JPanel();
    roleButton = makeRoleButton();
    panel.add(roleButton);
    permissionButton = makePermissionButton();
    panel.add(permissionButton);
    return panel;
  }

  /**
   * Cria o boto de seleo de permisses, atribuindo-lhe como ao a criao
   * do dilogo de seleo de permisses, alm habilitao do boto de incluso
   * aps a seleo, caso os requisitos estejam preenchidos.
   * 
   * @return boto de seleo de permisses.
   */
  private JButton makePermissionButton() {
    JButton button = 
      new JButton(LNG.get("IAS_USER_PERMISSIONS_BUTTON"));
    button.addActionListener(
        new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        new UserPermissionDefinitionDialog(
          userInfoDialog, userInfo, null, true).showDialog();
        enableIncludeUserButtonIfAllowed();
      }
    });
    return button;
  }

  /**
   * Cria o boto de seleo de perfis, atribuindo-lhe como ao a criao
   * do dilogo de seleo de perfis, alm da habilitao do boto de incluso
   * aps a seleo, caso os requisitos estejam preenchidos.
   * 
   * @return boto de seleo de perfis.
   */
  private JButton makeRoleButton() {
    JButton button = new JButton(LNG.get("IAS_USER_ROLES_BUTTON"));
    button.addActionListener(
        new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        new UserRoleDefinitionDialog(userInfoDialog, userInfo, null, true).
          showDialog();
        enableIncludeUserButtonIfAllowed();
      }
    });
    return button;
  }
  
  /**
   * {@inheritDoc}
   * 
   * @return true se e somente se todos os campos requeridos estiverem 
   *         preenchidos, false caso contrrio.
   */
  protected boolean unsavedDataInput() {
    return requiredFieldsAreFilled();
  }

  /**
   * Habilita o boto de incluir somente se as condies para incluso forem
   * atendidas (campos requeridos preenchidos). Caso contrrio, desabilita.
   */
  private void enableIncludeUserButtonIfAllowed() {
    includeUserButton.setEnabled(requiredFieldsAreFilled());
  }
  
  /**
   * Classe que implementa o listener para modificaes nos campos de dados do
   * usurio.
   */
  private class UserInfoChangeListener
      extends KeyAdapter
      implements ItemListener {
    // #TODO Passar esta classe para a superclasse ou como uma classe 
    // standalone.
    /**
     * {@inheritDoc}
     */
    public void itemStateChanged(ItemEvent ie) {
      enableIncludeUserButtonIfAllowed();
    }

    /**
     * {@inheritDoc}
     */
    public void keyReleased(KeyEvent kev) {
      enableIncludeUserButtonIfAllowed();
    }
  }

}
