package csbase.client.ias;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.GUIUtils;
import csbase.client.desktop.DesktopComponentDialog;
import csbase.logic.User;
import csbase.logic.UserInfo;

/**
 * Classe abstrata que representa o que h de comum aos dilogos que exibem 
 * dados de um usurio.
 *
 * @author Marcus Leal
 */
public abstract class UserInfoDialog {
  /** Instncia do dilogo */
  protected DesktopComponentDialog userInfoDialog;
  /** Janela que criou o dilogo */
  protected Window ownerWindow;
  /** Campo nome do usurio */
  protected JTextField name;
  /** Campo login do usurio */
  protected JTextField login;
  /** Campo e-mail do usurio */
  protected JTextField email;
  /** Ttulo para o dilogo e seus subdilogos */
  protected String dialogTitle;
  /**
   * Objeto que centraliza as informaes do usurio (padro <i>Data Transfer 
   * Object</i> - DTO), compartilhadas entre esse dilogo e seus dilogos de
   * informaes complementares (dilogos de perfis, permisses e senha)
   */
  protected UserInfo userInfo;

  /**
   * Construtor que inicializa atributos comuns (referncia para a janela-pai,
   * ttulo do dilogo e DTO de informaos do usurio).
   *  
   * @param ownerWindow janela que criou este dilogo.
   * @param dialogTitle ttulo deste dilogo, usado tambm em alguns dos seus 
   *        subdilogos.
   */
  protected UserInfoDialog(Window ownerWindow, String dialogTitle) {
    this.ownerWindow = ownerWindow;
    this.dialogTitle = dialogTitle;
    // UserInfo tem uma inconsistncia potencial: embora o construtor abaixo
    // substitua nulo por arrays vazios de Object nos casos dos parmetros de
    // perfis e permisses, o mtodo setAttribute permite atribuir diretamente
    // nulo a estes atributos. Para evitar dvidas entre teste de nulo ou vazio,
    // esta classe estar sempre atribuindo um array vazio para designar 
    // ausncia de parmetros.
    userInfo = new UserInfo(null, null, null, new Object[]{}, new Object[]{});
  }

  /**
   * Exibe/esconde o dilogo. Se o dilogo ainda no tiver sido construdo, 
   * realiza essa tarefa antes de invocar as operaes de visibilidade.
   * 
   * @param b indica se o dilogo deve estar visvel (<code>true</code>) ou
   *        invisvel (<code>false</code>).
   */
  public void setVisible(boolean b) {
    if (userInfoDialog == null) {
      makeDialog();
    }
    userInfoDialog.setVisible(b);
  }
  
  /**
   * Cria um dilogo bsico de dados de um usurio, dividido em 3 reas: 
   * <ol>
   * <li>um painel superior com os campos de texto login, nome e e-mail;
   * </li>
   * <li>um painel central vazio, preparado para botes que acionam dilogos de 
   * informaes complementares, como perfis, permisses e senha;</li>
   * <li>e finalmente um painel inferior para botes de ao, contendo um boto
   * "Fechar".</li>
   * </ol>
   * Tanto o boto "X" no canto superior esquerdo da janela quanto o boto
   * "Fechar" executam o mesmo procedimento {@link UserInfoDialog#close()} ao 
   * fechar.
   * 
   * @see UserInfoDialog#close() 
   */
  protected void makeDialog() {
    userInfoDialog = 
      new DesktopComponentDialog((JFrame)ownerWindow, dialogTitle);
    JPanel userInfoPanel = new JPanel(new BorderLayout());
    userInfoPanel.setBorder(
      BorderFactory.createTitledBorder(LNG.get("IAS_USER_REGISTRATION")));
    userInfoPanel.add(makeUpperPanel(), BorderLayout.NORTH);
    userInfoPanel.add(makeMiddlePanel(), BorderLayout.SOUTH);
    Container contentPane = userInfoDialog.getContentPane();
    contentPane.setLayout(new BorderLayout());    
    contentPane.add(userInfoPanel, BorderLayout.CENTER);
    contentPane.add(makeLowerPanel(), BorderLayout.SOUTH);
    userInfoDialog.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        close();
      }
    });
    userInfoDialog.pack();
    userInfoDialog.center(ownerWindow);
  }

  /**
   * Constri o painel superior contendo as principais informaes do usurio
   * (login, nome e e-mail).
   *
   * @return o painel superior do dilogo.
   */
  protected JPanel makeUpperPanel() {
    JLabel loginL = new JLabel(LNG.get("IAS_USER"));
    login = new JTextField();
    JLabel nameL = new JLabel(LNG.get("IAS_USER_NAME"));
    name = new JTextField(20);
    JLabel emailL = new JLabel(LNG.get("IAS_USER_EMAIL"));
    email = new JTextField();
    JComponent[][] rows = new JComponent[][] {
        { loginL, login },
        { nameL, name },
        { emailL, email }
    };
    return GUIUtils.createBasicGridPanel(rows);
  }

  /**
   * Constri o painel central, reservado para a entrada de informaes
   * complementares do usurio. Essa implementao retorna um painel vazio.
   *
   * @return painel central do dilogo.
   */
  protected JPanel makeMiddlePanel() {
    return new JPanel();
  }
  
  /**
   * Cria o boto de fechamento da janela, atribuindo-lhe como ao o mtodo
   * {@link UserInfoDialog#close()}. 
   *
   * @return o boto de fechamento.
   * 
   * @see UserInfoDialog#close()
   */
  protected JButton makeCloseButton() {
    JButton button = new JButton(LNG.get("IAS_CLOSE"));
    button.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent ev) {
          close();
        }
      });
    return button;
  }

  /**
   * Mtodo para fechamento do dilogo. Verifica se houve entrada de dados no-
   * salvos, e em caso positivo, confirma com o usurio se este deseja sair
   * ignorando os dados entrados, cancelar ou gravar suas alteraes antes de 
   * fechar. Termina por fechar o dilogo, a no ser que o usurio tenha 
   * cancelado a operao.
   * 
   * @see UserInfoDialog#unsavedDataInput()
   * @see UserInfoDialog#confirmClose()
   */
  private void close() {
    if (unsavedDataInput()) {
      if (!confirmClose()) {
        return;
      }
    }
    userInfoDialog.close();
  }

  /**
   * Indica se os campos requeridos esto preenchidos.
   *
   * @return <code>true</code> se os campos requeridos estiverem preenchidos ou
   *         <code>false</code> se estiverem vazios.
   */
  protected boolean requiredFieldsAreFilled() {
    return (!login.getText().trim().equals("")) &&
      (!name.getText().trim().equals(""));
  }

  /**
   * Atualiza o contedo do DTO de usurio com as informaes contidas nos 
   * campos do dilogo (login, nome e e-mail).
   */
  protected void updateUserInfo() {
    String nameText = name.getText().trim();
    String loginText = login.getText().trim();
    String emailText = email.getText().trim();
    userInfo.setAttribute(User.NAME, nameText);
    userInfo.setAttribute(User.LOGIN, loginText);
    userInfo.setAttribute(User.MAIL, emailText);
  }

  /**
   * Constri o painel inferior, contendo os botes de ao (o painel bsico
   * s contm o boto fechar).
   *
   * @return o painel inferior do dilogo.
   */
  protected JPanel makeLowerPanel() {
    JPanel buttonPanel = new JPanel();
    buttonPanel.add(makeCloseButton());
    return buttonPanel;
  }
  

  /**
   * Verifica se houve entrada de dados no-salvos na interface.
   *
   * @return <code>true</code> se algum dado foi editado ou <code>false</code>
   *         caso contrrio
   */
  protected abstract boolean unsavedDataInput();

  /**
   * Avisa o usurio de que h dados entrados no-salvos, e confirma se este
   * deseja realmente sair ignorando os dados entrados, cancelar ou gravar suas
   * alteraes antes de fechar.
   *
   * @return <code>true</code> caso o dilogo deva ser realmente fechado, ou
   *         <code>false</code> caso o usurio queira cancelar o fechamento
   *         para revisar suas alteraes.
   */
  protected abstract boolean confirmClose();

}
