/*
 * Detalhes da ltima alterao:
 * 
 * $Author: clinio $ $Date: 2008-09-17 10:05:53 -0300 (Wed, 17 Sep 2008) $
 * $Revision: 100778 $
 */
package csbase.client.login;

import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Locale;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;

import tecgraf.javautils.gui.GUIUtils;
import csbase.client.remote.ClientRemoteMonitor;
import csbase.exception.CSBaseException;
import csbase.exception.ConfigurationException;
import csbase.logic.Version;

/**
 * Representa um dilogo modal de login.  usado pelo cliente para verificar se
 * um usurio pode ou no acessar o sistema.
 * 
 * @author Tecgraf/PUC-Rio
 */
public abstract class AbstractLoginUI {

  /**
   * Dilogo propriamente dito.
   */
  final private JFrame frame = new JFrame();

  /**
   * Indica se o dilogo foi cancelado ou no.
   */
  private boolean wasCancelled;

  /**
   * Define o lock utilizado para o fechamento do dilogo.
   */
  final private Lock continueLock = new ReentrantLock();

  /**
   * Define a condio para finalizao do dilogo.
   */
  final private Condition continueCondition = continueLock.newCondition();

  /**
   * 
   */
  final private JButton loginButton = new JButton("");

  /**
   * 
   */
  final private JButton cancelButton = new JButton("");

  /**
   * Campo texto onde o usurio digita o seu login.
   */
  final private JTextField loginTextField = new JTextField();

  /**
   * Campo texto onde o usurio digita sua senha.
   */
  final private JPasswordField passwordTextField = new JPasswordField();

  /**
   * A combo com os idiomas suportados pelo sistema.
   */
  final private JComboBox languageComboBox = new JComboBox();

  /**
   * Consulta o valor de frame
   * 
   * @return o valor
   */
  protected final JFrame getFrame() {
    return frame;
  }

  /**
   * Consulta o valor de loginButton
   * 
   * @return o valor
   */
  protected final JButton getLoginButton() {
    return loginButton;
  }

  /**
   * Consulta o valor de cancelButton
   * 
   * @return o valor
   */
  protected final JButton getCancelButton() {
    return cancelButton;
  }

  /**
   * Consulta o valor de loginTextField
   * 
   * @return o valor
   */
  protected final JTextField getLoginTextField() {
    return loginTextField;
  }

  /**
   * Consulta o valor de passwordTextField
   * 
   * @return o valor
   */
  protected final JPasswordField getPasswordTextField() {
    return passwordTextField;
  }

  /**
   * Consulta o valor de languageComboBox
   * 
   * @return o valor
   */
  protected final JComboBox getLanguageComboBox() {
    return languageComboBox;
  }

  /**
   * Cancela o dilogo.
   */
  private void cancel() {
    this.wasCancelled = true;
    this.close();
  }

  /**
   * Fecha o dilogo de login.
   */
  private void close() {
    this.frame.dispose();
    unlock();
  }

  /**
   * Desabilita componentes do dilogo
   */
  private void disableDialog() {
    frame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    loginButton.setEnabled(false);
    cancelButton.setEnabled(false);
    adjustDialogStatus(false);
  }

  /**
   * Efetua o login do usurio no servidor.
   * 
   * @param login O login do usurio.
   * @param password A senha do usurio.
   * @param locale O locale escolhido pelo usurio.
   * 
   * @return true, caso o login tenha sido realizado com sucesso, ou false, caso
   *         contrrio.
   * 
   * @throws CSBaseException Caso ocorra algum erro durante o procedimento.
   */
  private boolean doLogin(final String login, final String password,
    final Locale locale) throws CSBaseException {
    ClientRemoteMonitor.getInstance().start(login, password, locale);
    return ClientRemoteMonitor.getInstance().isAlive();
  }

  /**
   * Habilita os componentes do dilogo
   */
  private void enableDialog() {
    frame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    adjustDialogStatus(true);
    loginButton.setEnabled(true);
    cancelButton.setEnabled(true);
  }

  /**
   * <p>
   * Executa o dilogo.
   * </p>
   * 
   * <p>
   * OBS: No pode ser chamado na thread do Swing.
   * </p>
   * 
   * @param defaultLocale Um {@code Locale} para ser utilizado como default na
   *        combo de idiomas. Se nenhum for especificado, ser utilizado o valor
   *        retornado por {@link Locale#getDefault()}.
   * 
   * @return Os dados informados pelo usurio.
   * 
   * @throws IllegalStateException Caso a execuo do dilogo seja solicitada na
   *         thread do Swing.
   */
  public final InitialContext execute(final Locale defaultLocale) {
    if (SwingUtilities.isEventDispatchThread()) {
      throw new IllegalStateException(
        "O dilogo de login no pode ser executado na Thread do Swing.");
    }

    languageComboBox.setModel(new LocaleComboBoxModel());

    final Locale locale =
      defaultLocale == null ? Locale.getDefault() : defaultLocale;
    loginExecuted(locale);

    mountDialog();

    this.frame.pack();
    GUIUtils.centerOnScreen(this.frame);
    this.frame.toFront();

    this.continueLock.lock();
    try {
      this.frame.setVisible(true);
      ClientRemoteMonitor.getInstance().getServerLookupThread().start();
      this.continueCondition.awaitUninterruptibly();
    }
    finally {
      this.continueLock.unlock();
    }
    return this.getLoginData();
  }

  /**
   * Obtm os dados informados pelo usurio.
   * 
   * @return Os dados informados pelo usurio, ou null, caso o usurio tenha
   *         cancelado o dilogo.
   */
  public InitialContext getLoginData() {
    if (this.wasCancelled) {
      return null;
    }
    final LocaleComboBoxItem item =
      (LocaleComboBoxItem) this.languageComboBox.getSelectedItem();
    final InitialContext loginData = new InitialContext(item.getLocale());
    return loginData;
  }

  /**
   * Efetua o login.
   */
  private void login() {
    final String login = this.loginTextField.getText();
    final String password = new String(this.passwordTextField.getPassword());
    final LocaleComboBoxItem selectedLocale =
      (LocaleComboBoxItem) this.languageComboBox.getSelectedItem();

    showProgress();
    disableDialog();

    final Thread t = new Thread() {
      private void restoreDialog(final String message, final boolean enableLogin) {
        SwingUtilities.invokeLater(new Runnable() {
          public void run() {
            passwordTextField.setText("");
            enableDialog();
            showInvalidLoginMessage(message);
            hideProgress();
            loginButton.setEnabled(enableLogin);
          }
        });
      }

      @Override
      public void run() {
        try {
          if (!AbstractLoginUI.this.doLogin(login, password, selectedLocale
            .getLocale())) {
            restoreDialog("Login ou senha invlidos.", true);
          }
          else if (!ClientRemoteMonitor.getInstance().isSameVersion()) {
            restoreDialog("<html>As verses do cliente e do servidor so "
              + "diferentes.<br>Por favor, atualize o cliente.<html>", false);
            System.err.println("Verso cliente: "+
            		Version.getInstance().getName());
            System.err.println("Verso servidor: "+
            		ClientRemoteMonitor.getInstance().getServer()
            		.getVersionName());
          }
          else {
            wasCancelled = false;
            unlock();
          }
        }
        catch (final Exception e) {
          e.printStackTrace();
          restoreDialog("<html>No foi possvel estabelecer conexo. "
            + "<br>Provavelmente o servidor est fora do ar.<html>", true);
        }
      }
    };

    t.start();
  }

  /**
   * Cria o painel de botes.
   */
  private void setButtonsListeners() {
    this.loginButton.addActionListener(new ActionListener() {
      public void actionPerformed(final ActionEvent ev) {
        login();
      }
    });

    this.cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(final ActionEvent ev) {
        cancel();
      }
    });
  }

  /**
   * Libera o lock que mantm o dilogo aberto.
   */
  private void unlock() {
    this.continueLock.lock();
    try {
      this.continueCondition.signalAll();
    }
    finally {
      this.continueLock.unlock();
    }
  }

  /**
   * Cria o dilogo de login.
   * 
   * @throws ConfigurationException Caso no seja possvel obter a configurao
   *         da classe.
   */
  public AbstractLoginUI() {
    frame.setResizable(false);
    setButtonsListeners();

    frame.getRootPane().setDefaultButton(loginButton);
    frame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
    frame.addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosing(final WindowEvent e) {
        cancel();
      }
    });
  }
  
  /**
   * Sinalizao de execuo do login.
   * @param locale o locale associado.
   */
  protected void loginExecuted(final Locale locale) {
    for (int i = 0; i < languageComboBox.getItemCount(); i++) {
      final LocaleComboBoxItem comboItem =
        (LocaleComboBoxItem) languageComboBox.getItemAt(i);
      final Locale itemLocale = comboItem.getLocale();
      if (itemLocale.equals(locale)) {
        final LocaleComboBoxModel model = (LocaleComboBoxModel)languageComboBox.getModel();
        model.setSelectedItem(comboItem);
        break;
      }
    }
  }

  /**
   * Ajuste do dilogo em modo habilitado conforme indicativo
   * 
   * @param isEnabled indicativo de habilitado.
   */
  abstract protected void adjustDialogStatus(final boolean isEnabled);

  /**
   * Exibe barra de progresso
   */
  abstract protected void showProgress();

  /**
   * Mostra no status bar o texto de falha de login.
   * 
   * @param text texto a ser mostrado na barra de status
   */
  abstract protected void showInvalidLoginMessage(final String text);

  /**
   * Oculta barra de progresso
   */
  abstract protected void hideProgress();

  /**
   * Cria o painel com o contedo do dilogo. O contedo do dilogo  o seu
   * painel principal e a barra de status.
   */
  abstract protected void mountDialog();

}
