/*
 * $Id: LoginUI.java 87797 2009-01-29 17:05:01Z clinio $
 */

package csbase.client.login;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Toolkit;
import java.net.URL;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JProgressBar;
import javax.swing.JTextField;
import javax.swing.UIManager;

import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.gui.GBC;
import tecgraf.javautils.gui.GUIUtils;
import csbase.exception.ConfigurationException;

/**
 * Representa um dilogo modal de login.  usado pelo cliente para verificar se
 * um usurio pode ou no acessar o sistema.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class LoginUI extends AbstractLoginUI {

  /**
   * O nome da propriedade que define o caminho para a imagem do dilogo de
   * login.
   */
  private static final String IMAGE_PATH_PROPERTY = "image.path";

  /**
   * O nome da propriedade que define o caminho para o cone do dilogo de
   * login.
   */
  private static final String ICON_PATH_PROPERTY = "icon.path";

  /**
   * Diretrio de imagens do CSBase.
   */
  private static final String ICON_DIRECTORY =
    "/csbase/client/resources/applicationimages/";

  /**
   * A configurao do dilogo.
   */
  final private Configuration configuration;

  /**
   * Barra de progresso
   */
  final private JProgressBar progressBar = createProgressBar(Color.black);

  /**
   * Painel de status
   */
  private JPanel statusPanel;

  /**
   * Label com a mensagem
   */
  private JLabel messageLabel;

  /**
   * Painel principal do dilogo
   */
  private JPanel dataPanel;

  /**
   * Label com icone do sistema
   */
  private JLabel image;

  /**
   * Painel de input
   */
  private JPanel inputPanel;

  /**
   * componentes extras para inserir no painel de input
   */
  private JComponent[][] extraInputComponents;

  /**
   * Cria o dilogo de login.
   * 
   * @throws ConfigurationException Caso no seja possvel obter a configurao
   *         da classe.
   */
  public LoginUI() {
    super();
    try {
      configuration = ConfigurationManager.getInstance().getConfiguration(this.getClass());
    }
    catch (Exception e) {
     throw new IllegalStateException(e);
    }
  }

  /**
   * Cria o cone do dilogo de login.
   * 
   * @return A imagem a ser exibida como cone do dilogo, ou null, caso nenhuma
   *         tenha sido informada.
   */
  private Image createIconImage() {
    String iconPath =
      this.configuration.getOptionalProperty(ICON_PATH_PROPERTY);
    if (iconPath == null) {
      return null;
    }
    final URL iconPathUrl = this.getClass().getResource(iconPath);
    return Toolkit.getDefaultToolkit().getImage(iconPathUrl);
  }

  /**
   * Adiciona o <code>messageLabel</code> ao painel de status.
   */
  private void addMessageLabel() {
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.weightx = 1;
    gbc.insets = new Insets(3, 3, 2, 2);
    gbc.anchor = GridBagConstraints.WEST;
    statusPanel.add(messageLabel, gbc);
  }

  /**
   * Cria a barra de status do dilogo.
   * 
   * @return A barra de status, ou null, caso no tenha sido definida mensagem
   *         para a barra de status.
   */
  private JLabel createStatusBar() {
    final String property = "status.message";
    final String statusMessage = configuration.getOptionalProperty(property);
    if (statusMessage == null) {
      return null;
    }
    final JLabel statusBar = new JLabel(statusMessage);
    final ImageIcon INFO_ICON =
      new ImageIcon(LoginUI.class.getResource(ICON_DIRECTORY
        + "Information16.png"));
    statusBar.setVerticalTextPosition(JLabel.TOP);
    statusBar.setIcon(INFO_ICON);
    return statusBar;
  }

  /**
   * Cria o painel principal do dilogo. O painel principal do dilogo 
   * composto pelo painel de dados e pelo painel de botes.
   * 
   * @return O painel principal do dilogo.
   */
  private JPanel createMainPanel() {
    JPanel mainPanel = new JPanel();
    mainPanel.setLayout(new GridBagLayout());
    dataPanel = createDataPanel();
    mainPanel.add(dataPanel, new GBC(1,1));
    mainPanel.add(createButtonPanel(), new GBC(1,2));
    return mainPanel;
  }

  /**
   * Cria o painel de botes.
   * 
   * @return O painel de botes.
   */
  private JPanel createButtonPanel() {
    final JButton loginButton = getLoginButton();
    loginButton.setText("Entrar");

    final JButton cancelButton = getCancelButton();
    cancelButton.setText("Cancelar");

    final JComponent[] cmps = new JComponent[] { loginButton, cancelButton };
    GUIUtils.matchPreferredSizes(cmps);

    final JPanel buttonPanel = new JPanel(new FlowLayout());
    buttonPanel.add(loginButton);
    buttonPanel.add(cancelButton);
    return buttonPanel;
  }

  /**
   * Adiciona novos componentes no painel de entrada de dados. Os novos so
   * adicionados abaixo dos j existentes.
   * 
   * @param extraComponents
   */
  public void addExtraInputComponents(final JComponent[][] extraComponents) {
    this.extraInputComponents = extraComponents;
  }

  /**
   * Cria o painel de dados.
   * 
   * @return O painel de dados.
   */
  protected JPanel createDataPanel() {
    final JTextField loginTextField = getLoginTextField();
    final JLabel userLabel = new JLabel("Usurio:");

    final JPasswordField passwordTextField = getPasswordTextField();
    final JLabel passwordLabel = new JLabel("Senha:");

    final JComboBox languageComboBox = getLanguageComboBox();
    final JLabel localeLabel = new JLabel("Idioma:");

    final JPanel panel = new JPanel(new GridBagLayout());

    image = this.createImageLabel();
    if (image != null) {
      panel.add(image, new GBC(0,0).center().insets(5, 10, 5, 5));
    }

    JComponent[][] inputComponents =
      new JComponent[][] { { userLabel, loginTextField },
          { passwordLabel, passwordTextField },
          { localeLabel, languageComboBox } };

    if (extraInputComponents != null) {
      int c = inputComponents[0].length + extraInputComponents[0].length;
      int r = inputComponents.length + extraInputComponents.length;
      JComponent[][] jc = new JComponent[r][c];
      System.arraycopy(inputComponents, 0, jc, 0, inputComponents.length);
      System.arraycopy(extraInputComponents, 0, jc, inputComponents.length,
        extraInputComponents.length);
      inputPanel = GUIUtils.createBasicGridPanel(jc);
    }
    else {
      inputPanel = GUIUtils.createBasicGridPanel(inputComponents);
    }
    
    panel.add(inputPanel, new GBC(1,0).center());
    return panel;
  }

  /**
   * Cria a imagem do dilogo.
   * 
   * @return A imagem do dilogo, ou null, caso no tenha sido informada a
   *         imagem do dilogo.
   */
  private JLabel createImageLabel() {
    String imagePath =
      this.configuration.getOptionalProperty(IMAGE_PATH_PROPERTY);
    if (imagePath == null) {
      return null;
    }
    URL imagePathUrl = this.getClass().getResource(imagePath);
    return new JLabel(new ImageIcon(imagePathUrl));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  final protected void showProgress() {
    final String property = "progress.message";
    final String progressMessage = configuration.getOptionalProperty(property, "Aguarde...");
    
    progressBar.setString(progressMessage);
    if (messageLabel != null) {
      statusPanel.remove(messageLabel);
    }
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.weightx = 1;
    gbc.insets = new Insets(2, 2, 2, 2);
    gbc.fill = GridBagConstraints.BOTH;
    statusPanel.add(progressBar, gbc);
    statusPanel.validate();
    statusPanel.repaint();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  final protected void hideProgress() {
    statusPanel.remove(progressBar);
    addMessageLabel();

    final JFrame frame = getFrame();
    frame.pack();

    statusPanel.validate();
    statusPanel.repaint();
  }

  /**
   * Cria uma barra de progresso
   * 
   * @param c Cor do texto
   * @return Uma instncia da barra de progresso
   */
  private static JProgressBar createProgressBar(Color c) {
    Object selectionBackground =
      UIManager.get("ProgressBar.selectionBackground");
    Object selectionForeground =
      UIManager.get("ProgressBar.selectionForeground");
    UIManager.put("ProgressBar.selectionBackground",
      new javax.swing.plaf.ColorUIResource(c));
    UIManager.put("ProgressBar.selectionForeground",
      new javax.swing.plaf.ColorUIResource(c));
    final JProgressBar pb = new JProgressBar();
    UIManager.put("ProgressBar.selectionBackground", selectionBackground);
    UIManager.put("ProgressBar.selectionForeground", selectionForeground);
    pb.setIndeterminate(true);
    pb.setStringPainted(true);
    return pb;
  }

  /**
   * Habilita/Desabilita todos os components de um container
   * 
   * @param container O container que contm os components
   * @param b true habilita e false desabilita
   */
  private void setEnableContainer(Container container, boolean b) {
    for (Component c : container.getComponents()) {
      if (c instanceof Container)
        setEnableContainer((Container) c, b);
      c.setEnabled(b);
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void adjustDialogStatus(final boolean status) {
    if (status) {
      setEnableContainer(dataPanel, true);
    }
    else {
      setEnableContainer(dataPanel, false);
      image.setEnabled(true);
    }
  }

  /**
   * Mostra no status bar o texto de falha de login.
   * 
   * @param text texto a ser mostrado na barra de status
   */
  @Override
  protected void showInvalidLoginMessage(final String text) {
    ImageIcon ERROR_ICON =
      new ImageIcon(LoginUI.class.getResource(ICON_DIRECTORY + "Error16.png"));
    messageLabel.setVerticalTextPosition(JLabel.TOP);
    messageLabel.setIcon(ERROR_ICON);
    messageLabel.setText(text);
  }


  /**
   * {@inheritDoc}
   */
  @Override
  protected void mountDialog() {
    final JFrame frame = getFrame();
    final String defaultTitle = "CSBase";
    final String propName = "title";
    final String title = configuration.getOptionalProperty(propName, defaultTitle);
    frame.setTitle(title);
    
    final Image icon = createIconImage();
    if (icon != null) {
      frame.setIconImage(icon);
    }

    final JTextField loginText = getLoginTextField();
    final JPasswordField pwdText = getPasswordTextField();
    final int N = 15;
    pwdText.setColumns(N);
    loginText.setColumns(N);
    
    final JPanel panel = new JPanel(new BorderLayout());
    final JPanel mainPanel = createMainPanel();
    panel.add(mainPanel, BorderLayout.CENTER);

    statusPanel = new JPanel(new GridBagLayout());
    statusPanel.setBorder(BorderFactory.createLoweredBevelBorder());
    panel.add(statusPanel, BorderLayout.SOUTH);
    messageLabel = createStatusBar();
    if (messageLabel != null) {
      addMessageLabel();
    }

    frame.setContentPane(panel);
  }
}
