/*
 * $Id$
 */
package csbase.client.openbus;

import java.lang.reflect.InvocationTargetException;
import java.rmi.RemoteException;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.Properties;
import java.util.logging.Level;

import openbusidl.acs.Credential;
import openbusidl.rs.IRegistryService;
import openbusidl.ss.ISession;
import openbusidl.ss.ISessionHelper;
import openbusidl.ss.ISessionService;
import openbusidl.ss.SessionEventSink;
import openbusidl.ss.SessionEventSinkHelper;
import openbusidl.ss.SessionEventSinkPOA;

import org.omg.CORBA.StringHolder;
import org.omg.CORBA.UserException;

import scs.core.ComponentId;
import scs.core.IComponent;
import scs.core.IComponentHelper;
import scs.core.IComponentHolder;
import scs.core.StartupFailed;
import scs.core.servant.ComponentBuilder;
import scs.core.servant.ComponentContext;
import scs.core.servant.ExtendedFacetDescription;
import tecgraf.openbus.Openbus;
import tecgraf.openbus.data_service.DataDescriptionFactory;
import tecgraf.openbus.data_service.DataDescriptionHelper;
import tecgraf.openbus.data_service.project.ProjectItemDescriptionFactory;
import tecgraf.openbus.exception.ACSLoginFailureException;
import tecgraf.openbus.exception.CORBAException;
import tecgraf.openbus.exception.InvalidCredentialException;
import tecgraf.openbus.exception.OpenBusException;
import tecgraf.openbus.exception.RSUnavailableException;
import tecgraf.openbus.exception.SSUnavailableException;
import tecgraf.openbus.exception.ServiceUnavailableException;
import tecgraf.openbus.project.ProjectItemDescriptionHelper;
import tecgraf.openbus.util.Log;
import csbase.logic.BusInfo;
import csbase.remote.ClientRemoteLocator;

/**
 * Responsvel por realizar a conexo com o OpenBus.
 * 
 * @author Tecgraf/PUC-Rio
 */
public final class OpenBusAccessPoint {
  /**
   * A instncia nica.
   */
  private static OpenBusAccessPoint instance;
  /**
   * Indica se a conexo foi feita por credencial ou no.
   */
  private boolean connectedByCredential;
  /**
   * Indica se o acesso ao OpenBus j foi iniciado.
   */
  private boolean inited;
  /**
   * <i>Thread</i> responsvel pela execuo do ORB.
   */
  private ORBThread orbThread;
  /**
   * O componente membro de uma sesso do OpenBus (opcional).
   */
  private IComponent sessionMember;
  /**
   * O identificador do membro da sesso do usurio.
   */
  private String sessionMemberIdentifier;
  /**
   * A sesso do usurio no OpenBus.
   */
  private ISession session;
  /**
   * O canal de eventos da sesso.
   */
  private SessionEventSink sessionEventSink;
  /**
   * O servant do tratador de eventos de sesso.
   */
  private SessionEventSinkPOA sessionMemberEventSinkServant;

  /**
   * Construtor.
   */
  private OpenBusAccessPoint() {
    PrivilegedAction<Object> action = new PrivilegedAction<Object>() {
      @Override
      public Object run() {
        System.setSecurityManager(null);
        return null;
      }
    };
    AccessController.doPrivileged(action);
  }

  /**
   * Obtm a instncia nica.
   * 
   * @return A instncia nica.
   */
  public static OpenBusAccessPoint getInstance() {
    if (instance == null) {
      instance = new OpenBusAccessPoint();
    }
    return instance;
  }

  /**
   * Inicializa o acesso ao barramento.
   * 
   * @param sessionEventSinkClass A classe do tratador de eventos da sesso
   *        (opcional).
   * 
   * @throws StartupFailed Caso no seja possvel iniciar o tratador de eventos
   *         de sesso.
   * @throws CORBAException Caso ocorra algum erro de CORBA.
   * @throws OpenBusException Caso ocorra algum erro na inicializao.
   * @throws RemoteException Caso haja falha na comunicao com o servidor
   *         CSBase.
   */
  public void init(Class<? extends SessionEventSinkPOA> sessionEventSinkClass)
    throws StartupFailed, CORBAException, OpenBusException, RemoteException {
    BusInfo busInfo = ClientRemoteLocator.openBusService.getBusInfo();

    Log.setLogsLevel(Level.FINEST);

    Properties orbProperties = new Properties();
    orbProperties.setProperty("org.omg.CORBA.ORBClass", "org.jacorb.orb.ORB");
    orbProperties.setProperty("org.omg.CORBA.ORBSingletonClass",
      "org.jacorb.orb.ORBSingleton");
    Openbus bus = Openbus.getInstance();
    try {
      bus.resetAndInitialize(null, orbProperties, busInfo.getHost(), busInfo
        .getPort());
    }
    catch (UserException e) {
      throw new CORBAException(e);
    }
    this.inited = true;

    org.omg.CORBA_2_3.ORB orb = (org.omg.CORBA_2_3.ORB) bus.getORB();
    orb.register_value_factory(DataDescriptionHelper.id(),
      new DataDescriptionFactory());
    orb.register_value_factory(ProjectItemDescriptionHelper.id(),
      new ProjectItemDescriptionFactory());

    org.omg.CORBA.Object obj;
    if (sessionEventSinkClass != null) {
      ComponentBuilder builder =
        new ComponentBuilder(bus.getRootPOA(), bus.getORB());
      ExtendedFacetDescription[] descriptions = new ExtendedFacetDescription[1];
      descriptions[0] =
        new ExtendedFacetDescription("SessionEventSink", SessionEventSinkHelper
          .id(), sessionEventSinkClass.getCanonicalName());
      ComponentContext context;
      try {
        context =
          builder.newComponent(descriptions, null, new ComponentId(
            "SessionEventSink", (byte) 1, (byte) 0, (byte) 0, "Java"));
      }
      catch (UserException e) {
        throw new CORBAException(e);
      }
      catch (InstantiationException e) {
        throw new OpenBusException(e);
      }
      catch (IllegalAccessException e) {
        throw new OpenBusException(e);
      }
      catch (InvocationTargetException e) {
        throw new OpenBusException(e);
      }
      catch (IllegalArgumentException e) {
        throw new OpenBusException(e);
      }
      catch (ClassNotFoundException e) {
        throw new OpenBusException(e);
      }
      catch (NoSuchMethodException e) {
        throw new OpenBusException(e);
      }

      obj = context.getIComponent();
      this.sessionMember = IComponentHelper.narrow(obj);

      this.sessionMemberEventSinkServant =
        (SessionEventSinkPOA) context.getFacets().get("SessionEventSink");
    }

    this.orbThread = new ORBThread();
  }

  /**
   * Verifica se o acesso ao OpenBus j foi iniciado.
   * 
   * @return {@code true} caso o acesso ao OpenBus j tenha sido iniciado ou,
   *         {@code false}, caso contrrio.
   */
  public boolean isInited() {
    return this.inited;
  }

  /**
   * Conecta ao barramento a partir de um par usurio/senha.
   * 
   * @param user O usurio.
   * @param password A senha.
   * 
   * @throws ServiceUnavailableException Caso algum dos servios bsicos esteja
   *         indisponvel.
   * @throws ACSLoginFailureException
   * @throws InvalidCredentialException Caso a credencial esteja invlida.
   */
  public void connect(String user, String password)
    throws ServiceUnavailableException, ACSLoginFailureException,
    InvalidCredentialException {
    this.connectedByCredential = false;
    Openbus bus = Openbus.getInstance();
    IRegistryService rs = bus.connect(user, password);
    this.connect(rs);
  }

  /**
   * Conecta ao barramento a partir de uma credencial.
   * 
   * @param credential A credencial.
   * 
   * @throws ServiceUnavailableException Caso algum dos servios bsicos esteja
   *         indisponvel.
   * @throws InvalidCredentialException Caso a credencial esteja invlida.
   */
  public void connect(Credential credential)
    throws ServiceUnavailableException, InvalidCredentialException {
    Openbus bus = Openbus.getInstance();
    if (bus.isConnected()) {
      return;
    }
    this.connectedByCredential = true;
    IRegistryService rs = bus.connect(credential);
    this.connect(rs);
  }

  /**
   * Conecta ao barramento.
   * 
   * @param rs O Servio de Registro.
   * 
   * @throws ServiceUnavailableException Caso algum dos servios bsicos esteja
   *         indisponvel.
   */
  private void connect(IRegistryService rs) throws ServiceUnavailableException {
    Openbus bus = Openbus.getInstance();
    if (rs == null) {
      throw new RSUnavailableException();
    }
    ISessionService ss = bus.getSessionService();
    if (ss == null) {
      throw new SSUnavailableException();
    }

    if (this.sessionMember != null) {
      org.omg.CORBA.Object obj;

      IComponent sessionComponent = ss.getSession();
      if (sessionComponent == null) {
        IComponentHolder sessionComponentHolder = new IComponentHolder();
        StringHolder memberIdentifierHolder = new StringHolder();
        if (ss.createSession(this.sessionMember, sessionComponentHolder,
          memberIdentifierHolder)) {
          sessionComponent = sessionComponentHolder.value;
          obj = sessionComponent.getFacet(ISessionHelper.id());
          this.session = ISessionHelper.narrow(obj);
          this.sessionMemberIdentifier = memberIdentifierHolder.value;
        }
      }
      else {
        obj = sessionComponent.getFacet(ISessionHelper.id());
        this.session = ISessionHelper.narrow(obj);
        this.sessionMemberIdentifier = session.addMember(this.sessionMember);
      }

      obj = sessionComponent.getFacet(SessionEventSinkHelper.id());
      this.sessionEventSink = SessionEventSinkHelper.narrow(obj);
    }

    orbThread.start();
  }

  /**
   * Verifica se h conexo com o barramento.
   * 
   * @return {@code true}, caso haja conexo com o barramento, ou {@code false},
   *         caso contrrio.
   */
  public boolean isConnected() {
    return Openbus.getInstance().isConnected();
  }

  /**
   * Desfaz a conexo com o barramento.
   */
  public void disconnect() {
    Openbus bus = Openbus.getInstance();
    try {
      if (this.sessionMember != null) {
        this.session.removeMember(this.sessionMemberIdentifier);
        this.sessionEventSink.disconnect();
      }
    }
    finally {
      if (!this.connectedByCredential) {
        bus.disconnect();
      }
    }
  }

  /**
   * Finaliza o acesso ao barramento.
   */
  public void shutdown() {
    Openbus bus = Openbus.getInstance();
    if (bus.isConnected()) {
      bus.disconnect();
    }
    this.inited = false;
    this.orbThread = null;
  }

  /**
   * Obtm o canal de eventos da sesso.
   * 
   * @return O canal de eventos da sesso.
   */
  public SessionEventSink getSessionEventSink() {
    return this.sessionEventSink;
  }

  /**
   * Obtm o identificador do membro da sesso.
   * 
   * @return O identificador do membro da sesso.
   */
  public String getSessionMemberIdentifier() {
    return this.sessionMemberIdentifier;
  }

  /**
   * Obtm o servant do tratador de eventos de sesso.
   * 
   * @return O servant do tratador de eventos de sesso.
   */
  public SessionEventSinkPOA getSessionMemberEventSinkServant() {
    return this.sessionMemberEventSinkServant;
  }

  /**
   * Responsvel por executar e finalizar o ORB.
   * 
   * @author Tecgraf/PUC-Rio
   */
  private static class ORBThread extends Thread {
    /**
     * {@inheritDoc}
     */
    @Override
    public void run() {
      Openbus.getInstance().run();
    }
  }
}
