/**
 * $Id$
 */
package csbase.client.openbus;

import java.rmi.RemoteException;
import java.util.Locale;
import java.util.logging.Level;

import openbusidl.ss.SessionEventSinkPOA;
import scs.core.StartupFailed;
import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.configurationmanager.ConfigurationManagerException;
import tecgraf.javautils.gui.StandardDialogs;
import tecgraf.openbus.exception.OpenBusException;
import tecgraf.openbus.util.Log;
import csbase.client.Client;
import csbase.client.login.InitialContext;
import csbase.client.login.LoginInterface;
import csbase.client.remote.ClientRemoteMonitor;
import csbase.exception.CSBaseException;
import csbase.exception.ConfigurationException;
import csbase.logic.openbus.Credential;

/**
 * Executa o procedimento de login atravs de uma credencial do OpenBus.
 * 
 * @author Tecvraf/PUC-Rio
 */
public final class OpenBusLogin implements LoginInterface {
  /** A configurao desta classe. */
  private Configuration configuration;
  /**
   * O contexto inicial de execuo do cliente.
   */
  private InitialContext context;
  /**
   * A credencial do usurio no OpenBus.
   */
  private Credential credential;

  /**
   * Cria o objeto responsvel por logar o usurio atravs de uma credencial do
   * OpenBus.
   * 
   * @param context O contexto inicial de execuo do cliente.
   */
  public OpenBusLogin(InitialContext context) {
    try {
      this.configuration =
        ConfigurationManager.getInstance().getConfiguration(this.getClass());
    }
    catch (ConfigurationManagerException e) {
      throw new ConfigurationException(e);
    }

    this.context = context;
    String credentialIdentifier =
      Client.getInstance().getParameter(Client.CREDENTIAL_IDENTIFIER_PARAMETER);
    String credentialOwner =
      Client.getInstance().getParameter(Client.CREDENTIAL_OWNER_PARAMETER);
    String credentialDelegate =
      Client.getInstance().getParameter(Client.CREDENTIAL_DELEGATE_PARAMETER);
    if (credentialDelegate == null) {
      credentialDelegate = "";
    }
    this.credential =
      new Credential(credentialIdentifier, credentialOwner, credentialDelegate);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public InitialContext login(Locale defaultLocale, String systemName) {
    ClientRemoteMonitor.getInstance().getServerLookupThread().start();

    try {
      ClientRemoteMonitor.getInstance().start(this.credential, defaultLocale);
      if (!ClientRemoteMonitor.getInstance().isAlive()) {
        StandardDialogs.showErrorDialog(null, systemName,
          "Login ou senha invlidos.");
        return null;
      }
    }
    catch (CSBaseException e) {
      StandardDialogs
        .showErrorDialog(
          null,
          systemName,
          "No foi possvel acessar o servidor. Provavelmente o servidor est fora do ar.");
      e.printStackTrace();
      return null;
    }

    Log.setLogsLevel(Level.FINEST);

    openbusidl.acs.Credential openbusCredential =
      new openbusidl.acs.Credential(credential.getIdentifier(), credential
        .getOwner(), credential.getDelegate());
    OpenBusAccessPoint oap = OpenBusAccessPoint.getInstance();

    Class<SessionEventSinkPOA> sessionEventSinkClass = null;
    try {
      sessionEventSinkClass =
        this.configuration
          .<SessionEventSinkPOA> getOptionalClassProperty("session.event.sink.class");
    }
    catch (ClassNotFoundException e) {
      StandardDialogs
        .showErrorDialog(null, systemName,
          "A classe do receptor de eventos de sesso do OpenBus no foi encontrada.");
      e.printStackTrace();
      return null;
    }

    try {
      oap.init(sessionEventSinkClass);
      oap.connect(openbusCredential);
    }
    catch (StartupFailed e) {
      e.printStackTrace();
      StandardDialogs
        .showErrorDialog(
          null,
          systemName,
          "No foi possvel acessar o servidor. Provavelmente o servidor est fora do ar.");
      return null;
    }
    catch (OpenBusException e) {
      e.printStackTrace();
      StandardDialogs
        .showErrorDialog(
          null,
          systemName,
          "No foi possvel acessar o servidor. Provavelmente o servidor est fora do ar.");
      return null;
    }
    catch (RemoteException e) {
      e.printStackTrace();
      StandardDialogs
        .showErrorDialog(
          null,
          systemName,
          "No foi possvel acessar o servidor. Provavelmente o servidor est fora do ar.");
      return null;
    }

    return this.context;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void notifyException(Exception e) {
    // Nada a ser feito.
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void notifyInitialized() {
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void notifyPreInitialization() {
    // Nada a ser feito.
  }
}
