/*
 * $Id: CommonProjectSelectDialog.java 95877 2009-09-04 21:57:56Z jnlopes $
 */
package csbase.client.project;

import java.awt.Dimension;
import java.awt.GridBagLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.ButtonGroup;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JRootPane;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.RowFilter;
import javax.swing.SortOrder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.text.Document;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.GBC;
import tecgraf.javautils.gui.GUIUtils;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.desktop.DesktopComponentDialog;
import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.RemoteTask;
import csbase.client.project.ProjectTableModel.RowData;
import csbase.client.project.tasks.OpenProjectOnTreeRemoteTask;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.CommonClientProject;
import csbase.logic.ProjectAdminInfo;
import csbase.logic.User;
import csbase.logic.UserOutline;
import csbase.logic.UserProjectInfo;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.ProjectServiceInterface;

/**
 * A classe <code>ProjectOpenDialog</code> mostra um dilogo modal para abertura
 * de um projeto do usurio logado, um projeto de outros usurios (habilitado
 * apenas para o admin), ou um projeto compartilhado por outro usurio.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class ProjectOpenDialog {

  /**
   * A janela que deu origem a esse dialogo
   */
  protected Window owner;

  /**
   * A janela de seleo de projeto.
   */
  protected DesktopComponentDialog dialog;

  /**
   * O boto default da janela
   */
  protected JButton defaultButton;

  /**
   * O projeto selecionado
   */
  protected CommonClientProject project;

  /**
   * RadioButton para listar todos os projetos do prprio usurio.
   */
  protected JRadioButton choiceOpenUser;

  /**
   * RadioButton para listar todos os projetos de todos os usurios.
   */
  protected JRadioButton choiceOpenShared;

  /**
   * RadioButton para listar todos os projetos compartilhados de outros
   * usurios.
   */
  protected JRadioButton choiceOpenAny;

  /**
   * Texto usado como filtro para a tabela.
   */
  private JTextField filterText;
  /**
   * Boto para limpar o campo de filtro.
   */
  private JButton filterButton;

  /**
   * Tabela de usurios compartilhados.
   */
  private ProjectTableSortableTable projectsTable;

  /**
   * O elemento <code>RowData</code> que corresponde ao projeto selecionado.
   * 
   * @see ProjectTableModel.RowData
   */
  private RowData selectedProject;

  /**
   * Lista de projetos compartilhados com o usurio logado.
   */
  private List<RowData> sharedProjectsList;

  /**
   * Lista de projetos cujo usurio logado seja o dono.
   */
  private List<RowData> currentUserProjectsList;

  /**
   * Lista de todos os projetos (compartilhados ou no) de todos os usurios.
   */
  private List<RowData> allUsersProjectsList;

  /**
   * Ao de abertura do projeto selecionado.
   */
  private AbstractAction openAction;

  /**
   * Ao de cancelamento da tela.
   */
  private AbstractAction cancelAction;

  /**
   * Mostra um dilogo com a lista de nomes dos projetos de usurio e aguarda
   * que um deles seja selecionado. O dilogo  modal. No caso da confimao da
   * seleo, o projeto  aberto.
   * 
   * @param owner A janela a partir da qual esse dilogo foi chamado.
   * 
   * @return O projeto selecionado.
   */
  public static CommonClientProject openProject(Window owner) {
    try {
      return new ProjectOpenDialog(owner).project;
    }
    catch (Exception e) {
      StandardErrorDialogs.showErrorDialog(owner, LNG
        .get("ProjectOpenDialog.title.open"), LNG
        .get("CommonProjectSelectDialog.error.no.project"));
    }
    return null;
  }

  /**
   * Constri uma viso de dilogo para seleo de projetos de um um usurio.
   * 
   * @param owner A janela a partir da qual esse dilogo foi chamado.
   */
  protected ProjectOpenDialog(Window owner) {
    this.owner = owner;
    dialog = makeDialog();
    dialog.setVisible(true);
  }

  /**
   * Cria um dilogo que aguarda a abertura de um projeto. Solicita ao servidor
   * que o projeto selecionado seja aberto.
   * 
   * @return Dilogo de abertura de um projeto.
   */
  private DesktopComponentDialog makeDialog() {

    initializeActions();

    /*
     * Cria o painel principal da janela.
     */
    JPanel mainPanel = new JPanel(new GridBagLayout());

    makeOpenOptionsPanel(mainPanel);
    createProjectPanel(mainPanel);

    GBC gbc = new GBC(0, 5).northwest().bottom(5).horizontal();
    mainPanel.add(makeButtonPanel(), gbc);

    /*
     * Cria um dilogo modal.
     */
    final DesktopComponentDialog d;
    if (owner instanceof JFrame) {
      d = new DesktopComponentDialog((JFrame) owner, getTitle());
    }
    else if (owner instanceof JDialog) {
      d = new DesktopComponentDialog((JDialog) owner, getTitle());
    }
    else {
      return null;
    }
    d.addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosing(WindowEvent e) {
        d.close();
      }
    });
    d.getContentPane().add(mainPanel);
    d.setPreferredSize(new Dimension(450, 400));
    d.pack();
    d.center(owner);
    JRootPane rootPane = d.getRootPane();
    rootPane.setDefaultButton(defaultButton);
    InputMap iMap =
      rootPane.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
    ActionMap aMap = rootPane.getActionMap();

    KeyStroke escKeystroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
    String actionMapKey = escKeystroke.toString();
    iMap.put(escKeystroke, actionMapKey);
    aMap.put(actionMapKey, cancelAction);
    return d;
  }

  /**
   * Inicializao das aes de abertura do projeto selecionado e de
   * cancelamento da tela.
   */
  private void initializeActions() {
    /*
     * Ao de abertura do projeto selecionado.
     */
    openAction = new AbstractAction(LNG.get("ProjectOpenDialog.open")) {

      @Override
      public void actionPerformed(ActionEvent e) {
        handleOpenAction();
      }
    };

    /*
     * Ao de cancelamento da tela.
     */
    cancelAction =
      new AbstractAction(LNG.get("CommonProjectSelectDialog.cancel")) {
        @Override
        public void actionPerformed(ActionEvent e) {
          dialog.close();
        }
      };
  }

  /**
   * @param mainPanel Painel principal da janela.
   */
  protected void makeOpenOptionsPanel(JPanel mainPanel) {
    GBC gbc;
    /*
     * Projetos do Usurio.
     */
    choiceOpenUser = new JRadioButton(LNG.get("ProjectOpenDialog.option.user"));
    choiceOpenUser.setEnabled(userHasHisOwnProjects());

    gbc = new GBC(0, 0).left(11).west().top(10).none();
    mainPanel.add(choiceOpenUser, gbc);

    /*
     * Projetos Compartilhados por Outros Usurios.
     */
    choiceOpenShared =
      new JRadioButton(LNG.get("ProjectOpenDialog.option.others"));
    choiceOpenShared.setEnabled(userHasSharedProjects());

    gbc = new GBC(0, 1).left(11).west().none();
    mainPanel.add(choiceOpenShared, gbc);

    /*
     * Projetos de Todos os Usurios.
     */
    if (User.getLoggedUser().isAdmin()) {
      choiceOpenAny = new JRadioButton(LNG.get("ProjectOpenDialog.option.any"));

      gbc = new GBC(0, 2).left(11).west().none();
      mainPanel.add(choiceOpenAny, gbc);
    }

    configureRadioButtons();
  }

  /**
   * @return <code>true</code> se o usurio possui seus prprios projetos
   */
  private boolean userHasHisOwnProjects() {
    /*
     * podemos usar o getter porque fazemos cache da lista (ela s  recuperada
     * do servidor uma nica vez)
     */
    List<RowData> ownProjects = getCurrentUserProjectsList();
    return ownProjects != null && !ownProjects.isEmpty();
  }

  /**
   * @return <code>true</code> se existem projetos compartilhados com o usurio
   *         corrente
   */
  private boolean userHasSharedProjects() {
    /*
     * podemos usar o getter porque fazemos cache da lista (ela s  recuperada
     * do servidor uma nica vez)
     */
    List<RowData> sharedProjects = getSharedProjects();
    return sharedProjects != null && !sharedProjects.isEmpty();
  }

  /**
   * Cria e configura os radio-buttons.
   */
  private void configureRadioButtons() {
    /*
     * definimos o estado inicial dos radio-buttons. Como ainda no adicionamos
     * os listeners, a tabela no vai ser atualizada
     */
    if (userHasHisOwnProjects()) {
      choiceOpenUser.setSelected(true);
    }
    else if (userHasSharedProjects()) {
      choiceOpenShared.setSelected(true);
    }
    else if (choiceOpenAny != null) {
      choiceOpenAny.setSelected(true);
    }

    ButtonGroup mainRadiosGroup = new ButtonGroup();
    /*
     * projetos do prrpio usurio
     */
    choiceOpenUser.addItemListener(new ItemListener() {
      @Override
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED) {
          /*
           * Preencher projectList com todos os projetos do prprio usurio.
           */
          final List<RowData> projectList = getCurrentUserProjectsList();
          updateProjectTable(projectList, false);
        }
      }
    });
    mainRadiosGroup.add(choiceOpenUser);
    /*
     * projetos compartilhados com o usurio
     */
    choiceOpenShared.addItemListener(new ItemListener() {

      @Override
      public void itemStateChanged(ItemEvent e) {
        /*
         * Preencher projectList com todos os projetos de outros usurios que
         * esto compartilhados.
         */
        if (e.getStateChange() == ItemEvent.SELECTED) {
          updateProjectTable(getSharedProjects(), true);
        }
      }
    });
    mainRadiosGroup.add(choiceOpenShared);
    /*
     * projetos de todos os usurios (apenas para o admin)
     */
    if (choiceOpenAny != null) {
      choiceOpenAny.addItemListener(new ItemListener() {
        @Override
        public void itemStateChanged(ItemEvent e) {
          /*
           * Preencher projectList com todos os projetos de todos os usurios.
           */
          if (e.getStateChange() == ItemEvent.SELECTED) {
            final List<RowData> projectList = getAllUsersProjectsList();
            updateProjectTable(projectList, true);
          }
        }
      });
      mainRadiosGroup.add(choiceOpenAny);
    }
  }

  /**
   * @param mainPanel Painel principal da janela.
   */
  private void createProjectPanel(JPanel mainPanel) {
    /*
     * inicializamos a tabela de acordo com a opo selecionada
     */
    if (choiceOpenUser.isSelected()) {
      projectsTable =
        new ProjectTableSortableTable(getCurrentUserProjectsList());
    }
    else if (choiceOpenShared.isSelected()) {
      projectsTable = new ProjectTableSortableTable(getSharedProjects());
    }
    else if (choiceOpenAny.isSelected()) {
      projectsTable = new ProjectTableSortableTable(getAllUsersProjectsList());
    }
    else {
      throw new RuntimeException("opo desconhecida");
    }

    projectsTable.setUserColumnVisible(!choiceOpenUser.isSelected());

    final ListSelectionModel listSelectionModel =
      projectsTable.getSelectionModel();

    listSelectionModel.addListSelectionListener(new ListSelectionListener() {
      @Override
      public void valueChanged(ListSelectionEvent e) {
        if (e.getValueIsAdjusting()) {
          return;
        }
        selectedProject = projectsTable.getSelectedRowData();
      }
    });

    MouseListener mouseListener = new MouseAdapter() {
      @Override
      public void mouseClicked(MouseEvent e) {
        if (e.getClickCount() == 2) {
          handleOpenAction();
        }
      }

    };
    projectsTable.addMouseListener(mouseListener);
    /*
     * remapeamos a ao de ENTER da tabela para abrir o projeto selecionado
     */
    InputMap im =
      projectsTable.getInputMap(JTable.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
    ActionMap actionMap = projectsTable.getActionMap();
    im.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0),
      "openProjectActionName");
    actionMap.put("openProjectActionName", openAction);

    projectsTable.sort(0, SortOrder.ASCENDING);
    projectsTable.setSelectedFirstRow();

    final JPanel filterPanel = new JPanel(new GridBagLayout());

    final JScrollPane scrollPane = new JScrollPane(projectsTable);

    GBC gbc = new GBC(0, 3).insets(10, 15, 10, 15).both();

    mainPanel.add(scrollPane, gbc);

    gbc = new GBC(0, 0).west().insets(10, 15, 10, 0);
    final JLabel filterLabel =
      new JLabel(LNG.get("ProjectSelectonPanel.filter.label"));
    filterPanel.add(filterLabel, gbc);

    filterText = new JTextField();
    gbc = new GBC(1, 0).insets(10).horizontal().filly();
    filterPanel.add(filterText, gbc);

    filterButton = new JButton(LNG.get("ProjectSelectonPanel.filter.clear"));
    gbc = new GBC(2, 0).east().insets(10, 0, 10, 15);
    filterPanel.add(filterButton, gbc);

    gbc = new GBC(0, 4).horizontal().northwest().top(5);
    mainPanel.add(filterPanel, gbc);

    setupFilterControls();
  }

  /**
   * Configura os controles (textfield + boto) para filtragem da tabela com os
   * usurios.
   */
  private void setupFilterControls() {
    /*
     * Implementamos um DocumentListener para ativar o filtro quando o contedo
     * do campo for alterado de qualquer forma
     */
    final Document document = filterText.getDocument();

    document.addDocumentListener(new DocumentListener() {
      @Override
      public void changedUpdate(DocumentEvent e) {
        filterTableContent();
      }

      @Override
      public void insertUpdate(DocumentEvent e) {
        filterTableContent();
      }

      @Override
      public void removeUpdate(DocumentEvent e) {
        filterTableContent();
      }
    });

    /*
     * Queremos que o contedo do filtro seja todo selecionado quando o campo
     * ganhar o foco
     */
    filterText.addFocusListener(new FocusListener() {
      @Override
      public void focusGained(FocusEvent e) {
        filterText.selectAll();
      }

      @Override
      public void focusLost(FocusEvent e) {
        filterText.select(0, 0);
      }
    });

    /*
     * ao do boto "limpar"
     */
    filterButton.addActionListener(new AbstractAction() {
      @Override
      public void actionPerformed(ActionEvent e) {
        filterText.setText("");
        filterTableContent();
      }
    });
  }

  /**
   * Filtra o contedo da tabela a partir do contedo do campo "filtro". O texto
   * do campo  usado como <code>".*texto.*"</code>.
   */
  private void filterTableContent() {
    final String text = filterText.getText();
    if (!text.isEmpty()) {
      /*
       * o "(?i)" define que a regex  case-insensitive
       */
      projectsTable.setRowFilter(RowFilter.regexFilter("(?i).*" + text + ".*",
        0, 1));
    }
    else {
      projectsTable.setRowFilter(null);
    }
    projectsTable.setSelectedFirstRow();
  }

  /**
   * Atualiza a tabela que faz a listagem dos projetos disponveis para serem
   * abertos.
   * 
   * @param projectList - Lista de projetos.
   * @param showUser - Boolean que define se a coluna "Usurio" ser exibida.
   */
  public void updateProjectTable(List<RowData> projectList, boolean showUser) {
    projectsTable.setNewValues(projectList);
    projectsTable.setUserColumnVisible(showUser);
    projectsTable.sort(0, SortOrder.ASCENDING);
    projectsTable.setSelectedFirstRow();
  }

  /**
   * @return Lista de todos os projetos (compartilhados ou no) de todos os
   *         usurios.
   */
  private List<RowData> getAllUsersProjectsList() {
    if (allUsersProjectsList != null) {
      return allUsersProjectsList;
    }

    RemoteTask<List<RowData>> task = new RemoteTask<List<RowData>>() {

      @Override
      protected void performTask() throws Exception {
        final List<RowData> projectsList = new ArrayList<RowData>();
        List<UserOutline> allUsers = User.getAllOutlines();
        for (int i = 0; i < allUsers.size(); i++) {
          UserOutline outline = allUsers.get(i);
          List<RowData> projectListTemp = getAllProjectsByUser(outline);
          if ((projectListTemp != null) && (!projectListTemp.isEmpty())) {
            projectsList.addAll(projectListTemp);
          }
        }
        setResult(projectsList);
      }
    };

    if (!task.execute(owner, getTitle(), LNG
      .get("OpenAnyProjectDialog.getUsers"))) {
      return null;
    }

    allUsersProjectsList = task.getResult();
    return allUsersProjectsList;
  }

  /**
   * @return Lista de projetos cujo usurio logado seja o dono.
   */
  private List<RowData> getCurrentUserProjectsList() {
    if (currentUserProjectsList != null) {
      return currentUserProjectsList;
    }
    RemoteTask<List<RowData>> task = new RemoteTask<List<RowData>>() {
      @Override
      protected void performTask() throws Exception {
        setResult(getAllProjectsByUser(User.getLoggedUser().getOutline()));
      }
    };
    if (!task.execute(owner, getTitle(), LNG
      .get("OpenAnyProjectDialog.getUsers"))) {
      return null;
    }
    currentUserProjectsList = task.getResult();
    return currentUserProjectsList;
  }

  /**
   * @param outLine - Usurios cujos projetos sero listados.
   * @return - Lista de projetos por usurios, estejam os projetos
   *         compartilhados ou no.
   * @throws RemoteException
   */
  private List<RowData> getAllProjectsByUser(UserOutline outLine)
    throws RemoteException {

    List<RowData> projectList = new ArrayList<RowData>();
    final UserOutline user = outLine;
    final String[] projNames =
      CommonClientProject.getAllProjectNames(user.getId());

    if (projNames != null) {
      for (int i = 0; i < projNames.length; i++) {
        RowData project = new RowData(user, projNames[i]);
        projectList.add(project);
      }
    }
    return projectList;

  }

  /**
   * @return Lista de projetos compartilhados com o usurio logado.
   */
  private List<RowData> getSharedProjects() {
    if (sharedProjectsList != null) {
      return sharedProjectsList;
    }

    final Object currentUserId = User.getLoggedUser().getId();

    RemoteTask<List<RowData>> task = new RemoteTask<List<RowData>>() {

      @Override
      protected void performTask() throws Exception {
        final List<RowData> projectList = new ArrayList<RowData>();
        List<UserProjectInfo> allUserProjects =
          CommonClientProject.getAllUserProjects(currentUserId);

        for (UserProjectInfo userProjectInfo : allUserProjects) {
          Object userId = userProjectInfo.getUserId();
          User user = User.getUser(userId);
          if (user == null) {
            // TODO devemos fazer mais do que isso?
            System.err.println(String.format(
              "Usurio '%s' no est cadastrado mas possui projetos", userId
                .toString()));
            continue;
          }
          String projName = userProjectInfo.getProjectName();
          RowData project = new RowData(user.getOutline(), projName);
          projectList.add(project);
        }
        setResult(projectList);
      }
    };

    final String msg = LNG.get("CommonProjectSelectDialog.info.get.projects");
    if (!task.execute(owner, getTitle(), msg)) {
      sharedProjectsList = null;
    }
    else {
      sharedProjectsList = task.getResult();
    }
    return sharedProjectsList;
  }

  /**
   * Cria um painel com os botes que vo ser usados no dilogo requisitado.
   * Esse mtodo deve ser sobrescrito dependendo acao da selecao.
   * 
   * @return O painel recm-criado.
   */
  protected JPanel makeButtonPanel() {
    JPanel panel = new JPanel();
    defaultButton = (JButton) panel.add(new JButton(openAction));

    JButton cancelButton = (JButton) panel.add(new JButton(cancelAction));

    GUIUtils
      .matchPreferredSizes(new JComponent[] { defaultButton, cancelButton });
    return panel;
  }

  /**
   * Executa a ao de abrir o projeto selecionado e instalar um observador para
   * ele.
   */
  protected void handleOpenAction() {

    if (selectedProject == null) {
      StandardDialogs.showErrorDialog(owner, getTitle(), LNG
        .get("ProjectOpenDialog.error.no.selection"));
      return;
    }

    if (!checkOpenableProject(selectedProject)) {
      return;
    }

    /*
     * Se o resultado for null  porque a operao no pode ser efetuada.
     */
    project = openProject();
    if (project != null) {
      dialog.close();
    }
  }

  /**
   * Verifica se projeto pode ser aberto (no est aguardando alocao de rea
   * solicitada).
   * 
   * @param selectedProject Nome do projeto a ser verificado.
   * @return Boolean indicando se projeto pode ser aberto (no est aguardando
   *         alocao de rea solicitada).
   */
  private boolean checkOpenableProject(final RowData selectedProject) {

    final RemoteTask<Boolean> task = new RemoteTask<Boolean>() {

      @Override
      protected void performTask() throws Exception {
        final Object userId = selectedProject.user.getId();
        ProjectServiceInterface psi = ClientRemoteLocator.projectService;
        final ProjectAdminInfo[] projectAdminInfos =
          psi.getLockedProjectAdminInfo();

        for (ProjectAdminInfo projectAdminInfo : projectAdminInfos) {
          if (selectedProject.projectName.equals(projectAdminInfo
            .getProjectName())
            && userId.equals(projectAdminInfo.getUserId())) {
            setResult(!projectAdminInfo.isLocked());
            return;
          }
        }
        setResult(true);
      }
    };

    final DesktopFrame desktopFrame = DesktopFrame.getInstance();
    final JFrame frame = desktopFrame.getView();

    task.execute(frame, LNG
      .get("CommonProjectSelectDialog.checking.project.blocked.title"), LNG
      .get("CommonProjectSelectDialog.checking.project.blocked.msg"));

    if (task.wasCancelled()) {
      StandardDialogs.showErrorDialog(desktopFrame.getView(), LNG
        .get("CommonProjectSelectDialog.cancelled.task.title"), LNG
        .get("CommonProjectSelectDialog.cancelled.task.msg"));
      return false;
    }

    if (task.getStatus() != true) {
      StandardDialogs.showErrorDialog(desktopFrame.getView(), LNG
        .get("CommonProjectSelectDialog.locked.project.failure.title"), LNG
        .get("CommonProjectSelectDialog.locked.project.failure.msg"));
      return false;
    }

    final boolean openable = task.getResult();
    if (!openable) {
      StandardDialogs.showErrorDialog(desktopFrame.getView(), LNG
        .get("CommonProjectSelectDialog.locked.project.title"), LNG
        .get("CommonProjectSelectDialog.locked.project.msg"));
    }
    return openable;
  }

  /**
   * Informa o ttulo que vai ser utilizado na janela de dilogo. Esse mtodo
   * depende da ao que originou o dilogo.
   * 
   * @return O ttulo da janela.
   */
  protected String getTitle() {
    return LNG.get("ProjectOpenDialog.title.open");
  }

  /**
   * Obtm o projeto selecionado.
   * 
   * @return Projeto selecionado.
   */
  private CommonClientProject openProject() {
    UserOutline user = selectedProject.user;
    String projectName = selectedProject.projectName;

    OpenProjectOnTreeRemoteTask openProjectTask =
      new OpenProjectOnTreeRemoteTask(projectName, user.getId(), DesktopFrame
        .getInstance().getTree());
    if (openProjectTask
      .execute(dialog, LNG.get("ProjectOpenDialog.title.open"), String.format(
        LNG.get("ProjectOpenDialog.opening.project"), projectName, user
          .getName()))) {
      return openProjectTask.getResult();
    }
    return null;
  }
}
