package csbase.client.project;

import java.util.ArrayList;
import java.util.List;

import javax.swing.table.DefaultTableModel;

import csbase.logic.UserOutline;

/**
 * Modelo para a tabela de abertura de projetos.
 * 
 * @author Tecgraf
 */
class ProjectTableModel extends DefaultTableModel {

  /**
   * Ttulo da coluna com os nomes dos arquivos PVTSim.
   */
  private static final String USER_COL_NAME = "Usurio";
  /**
   * Ttulo da coluna com os nomes dos fluidos PVTSim.
   */
  private static final String PROJECT_COL_NAME = "Projeto";

  /**
   * Nmero de colunas.
   */
  private int num_cols = 2;

  /**
   * Classe para representar o par (usurio, nome do projeto) associado a cada
   * linha da tabela.
   */
  static class RowData {

    /**
     * Informaes do usurio.
     */
    UserOutline user;

    /**
     * Nome do projeto.
     */
    String projectName;

    /**
     * Construtor.
     * 
     * @param userName Informaes do usurio
     * @param projectName Nome do projeto
     */
    RowData(UserOutline userName, String projectName) {
      this.user = userName;
      this.projectName = projectName;
    }
  }

  /**
   * Lista de pares (arq, fluido). Corresponde s linhas da tabela.
   */
  List<RowData> rows;

  /**
   * Construtor.
   * 
   * @param projects Lista com os pares (usurio, nome do projeto).
   */
  public ProjectTableModel(List<RowData> projects) {
    rows = new ArrayList<RowData>();

    for (RowData project : projects) {
      rows.add(new RowData(project.user, project.projectName));
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String getColumnName(int column) {
    if (getColumnCount() == 1) {
      if (column == 0) {
        return PROJECT_COL_NAME;
      }
    }
    if (column == 0) {
      return USER_COL_NAME;
    }
    return PROJECT_COL_NAME;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public int getRowCount() {
    return rows == null ? 0 : rows.size();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Object getValueAt(int row, int column) {
    RowData rowData = rows.get(row);
    if (getColumnCount() == 1) {
      if (column == 0) {
        return rowData.projectName;
      }
    }
    if (column == 0) {
      return rowData.user.getName();
    }
    return rowData.projectName;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean isCellEditable(int row, int column) {
    return false;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public int getColumnCount() {
    return num_cols;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Class<?> getColumnClass(int columnIndex) {

    if (getColumnCount() == 1) {
      if (columnIndex == 0) {
        return String.class;
      }
    }
    if (columnIndex == 0) {
      return UserOutline.class;
    }
    return String.class;

  }

  /**
   * Troca as linhas atuais da tabela por novas linhas.
   * 
   * @param projects Lista com os pares (usurio, nome do projeto).
   */
  public void replaceRows(List<RowData> projects) {
    getDataVector().removeAllElements();

    rows = new ArrayList<RowData>();
    for (RowData project : projects) {
      rows.add(project);
    }
    fireTableDataChanged();
  }

  /**
   * Obtm o par (usurio, nome do projeto) associado a uma linha especfica.
   * 
   * @param rowIndex ndice da linha.
   * @return Par (usurio, nome do projeto) associado  linha.
   */
  public RowData getRow(int rowIndex) {
    if (rowIndex < 0) {
      return null;
    }
    return rows.get(rowIndex);
  }

  /**
   * Habilita/Desabilita a visualizao da coluna "Usurios".
   * 
   * @param visible <code>True</code> para habilitar a visualizao,
   *        <code>False</code> para desabilitar.
   */
  public void setUserColumnVisible(boolean visible) {
    if (!visible) {
      if (num_cols == 2) {
        num_cols--;
        fireTableStructureChanged();
      }
    }
    else {
      if (num_cols == 1) {
        num_cols++;
        fireTableStructureChanged();
      }
    }
  }
}
