/*
 * $Id: ProjectTreeNode.java 105935 2010-05-24 11:12:27Z costa $
 */

package csbase.client.project;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.DesktopPreferences;
import csbase.client.project.tasks.GetChildrenTask;
import csbase.logic.ClientProjectFile;
import csbase.logic.CommonClientProject;
import csbase.logic.NoHiddenFileFilter;
import csbase.logic.ProjectFileFilter;

/**
 * Define um n de uma rvore de projetos.
 * 
 * @author Rodrigo Carneiro Henrique (rodrigoh)
 */
public final class ProjectTreeNode implements TreeNode {
  /**
   * N-pai.
   */
  private ProjectTreeNode parent;
  /**
   * Arquivo associado ao n.
   */
  private ClientProjectFile file;
  /**
   * Lista de ns filhos.
   */
  private List<ProjectTreeNode> childList;
  /**
   * Filtro para exibio dos ns.
   */
  private ProjectFileFilter filter;
  /**
   * Comparador.
   */
  private Comparator<ProjectTreeNode> comparator;

  /**
   * Construtor.
   * 
   * @param project projeto ao qual o n pertence
   * @param filter filtro para exibio dos ns
   * @param comparator comparador
   */
  ProjectTreeNode(CommonClientProject project, ProjectFileFilter filter,
    Comparator<ProjectTreeNode> comparator) {
    this(null, (ClientProjectFile) project.getTree(), filter, comparator);
    loadChildren();
  }

  /**
   * Construtor.
   * 
   * @param parent n-pai
   * @param file arquivo associado ao n
   * @param filter filtro para exibio dos ns
   * @param comparator comparador
   */
  ProjectTreeNode(ProjectTreeNode parent, ClientProjectFile file,
    ProjectFileFilter filter, Comparator<ProjectTreeNode> comparator) {
    this.parent = parent;
    this.file = file;
    this.filter = filter;
    this.comparator = comparator;
    this.childList = new LinkedList<ProjectTreeNode>();
  }

  /**
   * Carrega os filhos do n corrente, recuperando os dados do servidor.
   */
  void loadChildren() {
    loadChildren(true);
  }

  /**
   * Carrega os ns-filhos do n corrente, opcionalmente obtendo os dados do
   * servidor.
   * 
   * @param bringFromServer <code>true</code> para desconsiderar o cache local e
   *        obter os dados do servidor
   */
  void loadChildren(boolean bringFromServer) {
    if (!file.isDirectory())
      return;
    this.childList.clear();

    final ClientProjectFile[] children;
    if (bringFromServer) {
      children = GetChildrenTask.runTask(file);
    }
    else {
      // No queremos buscar no servidor os ns da rvore que ainda no foram
      // carregados, mas apenas filtrar os que j esto no cliente.
      children = file.getLocalChildren();
    }
    if (children != null) {
      addChildren(children);
    }
  }

  /**
   * @param children
   */
  private void addChildren(final ClientProjectFile[] children) {
    NoHiddenFileFilter noDotFileFilter = NoHiddenFileFilter.getInstance();
    boolean showHiddenFiles =
      DesktopFrame.getInstance().shouldShowHiddenFiles();
    for (int i = 0; i < children.length; i++) {
      /*
       * se o nome do arquivo comea com ".", ele  ignorado
       */
      if (!showHiddenFiles && !noDotFileFilter.accept(children[i])) {
        continue;
      }
      if (filter == null || filter.accept(children[i])) {
        ProjectTreeNode treeNode =
          new ProjectTreeNode(this, children[i], this.filter, this.comparator);
        this.childList.add(treeNode);
      }
    }
    Collections.sort(this.childList, this.comparator);
  }

  /**
   * Carrega toda a subrvore do servidor. O cache local  descartado, e a
   * subrvore est inteiramente preenchida.
   */
  void loadSubtree() {
    if (!file.isDirectory()) {
      return;
    }
    childList.clear();
    ClientProjectFile[] children = GetChildrenTask.runTask(file, true, true);
    addChildren(children);
  }

  /**
   * @return {@link ClientProjectFile} associado ao n
   */
  public ClientProjectFile getClientProjectFile() {
    return this.file;
  }

  /**
   * {@inheritDoc}
   */
  public Enumeration<ProjectTreeNode> children() {
    return new ChildEnumeration();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (!getClass().equals(obj.getClass())) {
      return false;
    }
    ProjectTreeNode node = (ProjectTreeNode) obj;
    return this.file.getAbsolutePath().equals(node.file.getAbsolutePath());
  }

  /**
   * {@inheritDoc}
   */
  public boolean getAllowsChildren() {
    return this.file.isDirectory();
  }

  /**
   * Obtm o n-filho do n corrente associado a um {@link ClientProjectFile}
   * especfico.
   * 
   * @param file arquivo associado ao n-filho
   * @return n-filho associado ao arquivo, ou <code>null</code> caso este no
   *         exista
   */
  ProjectTreeNode getChild(ClientProjectFile file) {
    for (ProjectTreeNode node : childList) {
      if (node.file.equals(file)) {
        return node;
      }
    }
    return null;
  }

  /**
   * Obtm o n-filho do n corrente associado a um nome de arquivo.
   * 
   * @param fileName nome do arquivo
   * @return n-filho associado ao arquivo com o nome especifica, ou
   *         <code>null</code> caso este no exista
   */
  ProjectTreeNode getChild(String fileName) {
    for (ProjectTreeNode node : childList) {
      if (node.file.getName().equals(fileName)) {
        return node;
      }
    }
    return null;
  }

  /**
   * Obtm o nome do n corrente.
   * 
   * @return nome do n corrente
   */
  String getName() {
    return file.getName();
  }

  /**
   * {@inheritDoc}
   */
  public TreeNode getChildAt(int childIndex) {
    return this.childList.get(childIndex);
  }

  /**
   * {@inheritDoc}
   */
  public int getChildCount() {
    if (!isLeaf() && childList.isEmpty()) {
      /*
       * esta situao representa um n que sabidamente tem filhos no servidor
       * (isLeaf() == false) mas estes no foram ainda carregados pela sua
       * representao no cliente (childList.isEmpty() == true); precisamos
       * obt-los agora
       */
      //      System.out.println("ProjectTreeNode: recuperando filhos de "
      //        + file.getName());
      loadChildren();
    }
    return childList.size();
  }

  /**
   * {@inheritDoc}
   */
  public TreeNode getParent() {
    return this.parent;
  }

  /**
   * {@inheritDoc}
   */
  public int getIndex(TreeNode node) {
    return this.childList.indexOf(node);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public int hashCode() {
    return this.file.hashCode();
  }

  /**
   * {@inheritDoc}
   */
  public boolean isLeaf() {
    // FIXME no leva em considerao o filtro corrente
    return !file.hasChildren();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String toString() {
    return this.file.toString();
  }

  /**
   * Enumerao de ns.
   * 
   * @author Tecgraf
   */
  private final class ChildEnumeration implements Enumeration<ProjectTreeNode> {
    /**
     * Iterador sobre a enumerao.
     */
    private Iterator<ProjectTreeNode> iterator;

    /**
     * Construtor.
     */
    ChildEnumeration() {
      this.iterator = ProjectTreeNode.this.childList.iterator();
    }

    /**
     * {@inheritDoc}
     */
    public boolean hasMoreElements() {
      return this.iterator.hasNext();
    }

    /**
     * {@inheritDoc}
     */
    public ProjectTreeNode nextElement() {
      return (this.iterator.next());
    }
  }

  /**
   * @return filtro para exibio dos ns
   */
  ProjectFileFilter getFilter() {
    return this.filter;
  }

  /**
   * Obtm os ndices dos ns dentre os filhos do n corrente.
   * 
   * @param treeNodeArray array com ns filhos do n corrente
   * @return array com os ndices dos ns dentre os filhos do n corrente. Ns
   *         que no sejam filhos do n corrente tero ndice -1.
   */
  int[] getIndexes(ProjectTreeNode[] treeNodeArray) {
    int[] indexes = new int[treeNodeArray.length];
    for (int i = 0; i < indexes.length; i++) {
      indexes[i] = this.getIndex(treeNodeArray[i]);
    }
    return indexes;
  }

  /**
   * @return caminho da raiz at o n corrente, sob a forma de um
   *         {@link TreePath}
   */
  TreePath getTreePath() {
    ProjectTreeNode node = this;
    /*
     * obtemos o caminho da raiz at o n corrente
     */
    List<ProjectTreeNode> pathToRoot = new ArrayList<ProjectTreeNode>();
    do {
      /*
       * inserimos na posio 0 porque queremos o caminho da raiz at o n
       * corrente
       */
      pathToRoot.add(0, node);
      node = (ProjectTreeNode) node.getParent();
    } while (node != null);

    return new TreePath(pathToRoot.toArray(new ProjectTreeNode[pathToRoot
      .size()]));
  }

  /**
   * Redefine o arquivo associado ao n, e atualiza a lista de filhos.
   * 
   * @param file - novo arquivo a ser associado ao n
   */
  public void setClientProjectFile(ClientProjectFile file) {
    this.file = file;
    this.loadChildren();
    /*
     * precisamos reapontar os arquivos associados aos ns-filhos para seu novo
     * pai
     */
    for (ProjectTreeNode child : childList) {
      child.file.setParent(file);
    }
  }

  /**
   * Adiciona um novo n  rvore, se esta operao for compatvel com o filtro
   * de visualizao atual. Arquivos com nome comeando com "." so exibidos
   * apenas se o usurio optou por visualizar tais arquivos.
   * 
   * @param file - arquivo associado ao novo n
   * @return true se um novo n foi criado na rvore (i.e. se o arquivo era
   *         compatvel com os filtros)
   * @see DesktopPreferences#shouldShowHiddenFiles()
   */
  public boolean addChild(ClientProjectFile file) {
    boolean showHiddenFiles =
      DesktopFrame.getInstance().shouldShowHiddenFiles();
    NoHiddenFileFilter noDotFileFilter = NoHiddenFileFilter.getInstance();
    /*
     * se o nome do arquivo comea com ponto, ele  ignorado
     */
    if (!showHiddenFiles && !noDotFileFilter.accept(file)) {
      return false;
    }
    if (filter == null || filter.accept(file)) {
      this.childList.add(new ProjectTreeNode(this, file, this.filter,
        this.comparator));
      Collections.sort(this.childList, this.comparator);
      /*
       * adicionamos o filho com sucesso, devemos indicar que o n corrente
       * possui filhos
       */
      this.file.setHasChildren(true);
      return true;
    }
    return false;
  }

  /**
   * Remove o n-filho associado a um determinado arquivo.
   * 
   * @param file arquivo
   * @return <code>true</code> se o n foi removido
   */
  public boolean removeChild(ClientProjectFile file) {
    ProjectTreeNode treeNode = this.getChild(file);
    if (treeNode != null) {
      if (this.childList.remove(treeNode)) {
        Collections.sort(this.childList, this.comparator);
        /*
         * se este era o ltimo filho, informamos que o n no possui mais
         * filhos
         */
        this.file.setHasChildren(!this.childList.isEmpty());
        return true;
      }
    }
    return false;
  }

  /**
   * Redefine o filtro.
   * 
   * @param filter novo filtro
   */
  void setFilter(ProjectFileFilter filter) {
    this.filter = filter;
  }
}
