package csbase.client.remote;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Observable;

import javax.swing.SwingUtilities;

import tecgraf.javautils.LNG;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.AlgoEvent;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.Category;
import csbase.logic.algorithms.CategorySet;
import csbase.remote.AlgorithmServiceInterface;
import csbase.remote.RemoteObserver;

/**
 * Classe responsvel por observar remotamente o servio de algoritmos
 * (algoService). Sempre que ocorrer criao, modificao ou remoo de
 * algoritmos, o singleton desta classe ser avisado e por sua vez ir notificar
 * quaisquer janelas que se registrarem como observadores deste.
 */
public class AlgorithmManagementObserver extends Observable implements
  RemoteObserver {

  /** Instncia nica do observador */
  private static AlgorithmManagementObserver instance = null;

  /**
   * Cria um singleton desta classe.
   * 
   * @throws RemoteException erro de rmi
   */
  public static void start() throws RemoteException {
    if (instance != null) {
      return;
    }
    instance = new AlgorithmManagementObserver();
  }

  /**
   * @throws RemoteException erro de rmi
   */
  public static void stop() throws RemoteException {
    if (instance == null) {
      return;
    }
    instance.uninstall();
  }

  /**
   * Registra o objeto como observador do algoService e ServerMonitor.
   * 
   * @throws RemoteException erro de rmi
   */
  public void install() throws RemoteException {
    UnicastRemoteObject.exportObject(this, 0);
    ClientRemoteMonitor.getInstance().addObserver(
      AlgorithmServiceInterface.SERVICE_NAME, this, null);
  }

  /**
   * Esse mtodo  chamado pelo algoService sempre que ocorre alguma alterao
   * nos algoritmos.
   * 
   * @param remoteEvent Objeto que descreve o evento ocorrido no servidor.
   * 
   * @throws RemoteException erro de rmi
   */
  public void update(final Object remoteEvent) throws RemoteException {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        try {
          AlgoEvent event = (AlgoEvent) remoteEvent;
          if (event.item instanceof AlgorithmInfo) {
            AlgorithmInfo.update(event);
          }
          else if (event.item instanceof Category) {
            Category.update(event);
          }
          else if (event.item instanceof CategorySet) {
            CategorySet.update(event);
          }
          else {
            setChanged();
            notifyObservers(event);
          }
        }
        catch (Exception e) {
          StandardErrorDialogs.showErrorDialog(null, LNG.get("ERRO"), e);
        }
      }
    });
  }

  /**
   * Remove do servidor o registro desse observador.
   * 
   * @throws RemoteException erro de rmi
   */
  public void uninstall() throws RemoteException {
    ClientRemoteMonitor.getInstance().deleteObserver(
      AlgorithmServiceInterface.SERVICE_NAME, this, null);
    instance = null;
    UnicastRemoteObject.unexportObject(this, true);
  }

  /**
   * Obtm a instncia nica (singleton) do observador remoto do servio de
   * algoritmos.
   * 
   * @return a instncia nica do observador
   */
  public static AlgorithmManagementObserver getInstance() {
    return instance;
  }

  /**
   * Construtor que exporta este objeto para o RMI e o registra como observador
   * do algoService e do ServerMonitor.
   * 
   * @throws RemoteException erro de rmi
   */
  private AlgorithmManagementObserver() throws RemoteException {
    super();
    install();
  }
}
