package csbase.client.remote.manager.trustedserver;

import java.rmi.RemoteException;
import java.security.cert.Certificate;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.SortedSet;

import csbase.client.desktop.RemoteTask;
import csbase.logic.User;
import csbase.logic.trustedserver.TrustedServer;
import csbase.logic.trustedserver.TrustedServerAddEvent;
import csbase.logic.trustedserver.TrustedServerEvent;
import csbase.logic.trustedserver.TrustedServerEventType;
import csbase.logic.trustedserver.TrustedServerInfo;
import csbase.logic.trustedserver.TrustedServerModifyEvent;
import csbase.logic.trustedserver.TrustedServerRemoveEvent;
import csbase.remote.ClientRemoteLocator;
import csbase.util.restart.RestartListener;
import csbase.util.restart.RestartManager;

/**
 * <p>
 * Representa um gerenciador de servidor confiveis num cliente.
 * </p>
 * 
 * <p>
 *  responsvel por verificar atualizaes no repositrio de servidores confiveis
 * localizado no servidor central. Fazendo as devidas atualizaes.
 * </p>
 * 
 * <p>
 * Componentes do cliente que precisem estar atualizados com o repositrio de
 * servidores confiveis, devem se cadastrar como ouvintes neste gerenciador para
 * receber as devidas notificaes.
 * </p>
 * 
 * <p>
 * Os mtodos desta classe normalmente faro acessos ao servidor. Portanto,
 * chamadas  mtodos desta classe devem ser realizadas com o uso de {@link
 * RemoteTask}. Os mtodos que no fazem acesso ao servidor informaro
 * explicitamente esse comportamento.
 * </p>
 *
 * @author Tecgraf/PUC-Rio
 */
public final class TrustedServerManager {
  /** A instncia nica do gerenciador de servidores confiveis. */
  private static TrustedServerManager instance;

  /** Representa o estado do gerenciador de servidores confiveis. */
  private TrustedServerManagerState state;

  /** Representa o estado habilitado do gerenciador de servidores confiveis. */
  private static final TrustedServerManagerEnabledState ENABLED_STATE =
    new TrustedServerManagerEnabledState();

  /** Representa o estado desabilitado do gerenciador de servidores confiveis. */
  private static final TrustedServerManagerDisabledState DISABLED_STATE =
    new TrustedServerManagerDisabledState();

  /**
   * Cria um gerenciador de servidores locais.
   */
  private TrustedServerManager() {
    if (this.canEnable()) {
      this.state = ENABLED_STATE;
    }
    else {
      this.state = DISABLED_STATE;
    }
  }

  /**
   * Verifica se o gerenciador de servidores locais pode ser habilitado ou no.
   *
   * @return true, caso o gerenciador possa ser habilitado, ou false, caso
   *         contrrio.
   */
  private boolean canEnable() {
    if (ClientRemoteLocator.trustedServerService != null) {
      if (User.getLoggedUser().isAdmin()) {
        return true;
      }
    }
    return false;
  }

  /**
   * Obtm a instncia nica do gerenciador de servidores locais.
   *
   * @return A instncia nica do gerenciador de servidores locais.
   */
  public static TrustedServerManager getInstance() {
    if (instance == null) {
      instance = new TrustedServerManager();
    }
    return instance;
  }

  /**
   * Inicia o monitoramento ao servio de servidores locais.
   *
   * @throws RemoteException Caso ocorra algum erro na comunicao com o
   *         servio.
   */
  public void start()
  throws RemoteException {
    this.state.start();
  }

  /**
   * Pra o monitoramento ao servio de servidores locais.
   */
  public void stop() {
    this.state.stop();
  }

  /**
   * Obtm um conjunto ordenado (por nome) com todos os servidores locais
   * pertencentes ao repositrio de servidores locais.
   *
   * @return Os servidores locais.
   *
   * @throws RemoteException Caso ocorra algum erro na comunicao com o
   *         servio.
   */
  public SortedSet<TrustedServer> getAllTrustedServers()
  throws RemoteException {
    return this.state.getAllTrustedServers();
  }

  /**
   * Adiciona um servidor local ao repositrio de servidores locais.
   *
   * @param localServer O servidor local a ser adicionado.
   *
   * @return true, caso o servidor local tenha sido adicionado, ou false, caso
   *         j exista um outro servidor local com o mesmo nome.
   *
   * @throws RemoteException Caso ocorra algum erro na comunicao com o
   *         servio.
   */
  public boolean addTrustedServer(TrustedServerInfo localServer)
  throws RemoteException {
    return this.state.addTrustedServer(localServer);
  }
  
  /**
   * Adiciona o certificado a um servidor
   * 
   * @param serverName O nome do servidor a receber o certificado
   * @param cert A Instncia do certificado
   * @return true se o certificado foi adicionado com sucesso, false caso
   *         contrrio
   * @throws RemoteException Caso ocorra algum erro na comunicao com o
   *         servio.
   */
  public boolean addCertificate(String serverName, Certificate cert)
    throws RemoteException {
    return this.state.addCertificate(serverName, cert);
  }
  
  /**
   * Remove um certificado do servidor
   * 
   * @param serverName O nome do servidor
   * @throws RemoteException Caso ocorra algum erro na comunicao com o
   *         servio.
   */
  public void removeCertificate(String serverName) throws RemoteException {
    this.state.removeCertificate(serverName);
  }

  /**
   * Modifica um servidor local no repositrio de servidores locais.
   *
   * @param oldLocalServer O servidor local no seu estado antigo.
   * @param newLocalServer O servidor local em seu estado atual.
   *
   * @return true, caso o servidor local tenha sido modificado, ou false, caso
   *         j exista um outro servidor local com o mesmo nome.
   *
   * @throws RemoteException Caso ocorra algum erro na comunicao com o
   *         servio.
   */
  public boolean modifyTrustedServer(
    TrustedServer oldLocalServer, TrustedServerInfo newLocalServer)
  throws RemoteException {
    return this.state.modifyTrustedServer(oldLocalServer, newLocalServer);
  }

  /**
   * Remove um servidor local.
   *
   * @param localServer O servidor local a ser removido.
   *
   * @throws RemoteException Caso ocorra algum erro na comunicao com o
   *         servio.
   */
  public void removeTrustedServer(TrustedServer localServer)
  throws RemoteException {
    this.state.removeTrustedServer(localServer);
  }

  /**
   * Adiciona um ouvinte  lista de ouvintes.
   *
   * @param listener O ouvinte a ser adicionado.
   */
  public void addTrustedServerManagerListener(
    TrustedServerManagerListener listener) {
    this.state.addTrustedServerManagerListener(listener);
  }

  /**
   * Remove um ouvinte da lista de ouvintes.
   *
   * @param listener O ouvinte a ser removido.
   */
  public void removeTrustedServerManagerListener(
    TrustedServerManagerListener listener) {
    this.state.removeTrustedServerManagerListener(listener);
  }

  /**
   * Verifica se o gerenciador de servidores locais est habilitado ou no.
   *
   * @return true, caso o gerenciador de servidores locais esteja habilitado,
   *         ou false, caso contrrio.
   */
  public boolean isEnabled() {
    return this.state.equals(ENABLED_STATE);
  }

  /**
   * Representa um estado de um gerenciador de servidores confiveis.
   *
   * @author Tecgraf/PUC-Rio
   */
  private interface TrustedServerManagerState {
    void start()
    throws RemoteException, UnsupportedOperationException;

    void stop()
    throws UnsupportedOperationException;

    SortedSet<TrustedServer> getAllTrustedServers()
    throws RemoteException, UnsupportedOperationException;

    boolean addTrustedServer(TrustedServerInfo localServer)
    throws RemoteException, UnsupportedOperationException;

    boolean modifyTrustedServer(
      TrustedServer oldLocalServer, TrustedServerInfo newLocalServer)
    throws RemoteException, UnsupportedOperationException;

    void removeTrustedServer(TrustedServer localServer)
    throws RemoteException, UnsupportedOperationException;

    void addTrustedServerManagerListener(TrustedServerManagerListener listener)
    throws UnsupportedOperationException;

    void removeTrustedServerManagerListener(TrustedServerManagerListener listener)
    throws UnsupportedOperationException;
    
    boolean addCertificate(String serverName, Certificate cert)
    throws RemoteException, UnsupportedOperationException;;
    
    void removeCertificate(String serverName) 
    throws RemoteException, UnsupportedOperationException;
  }

  /**
   * Representa o estado desabilitado do gerenciador de servidores locais.
   *
   * @author Tecgraf/PUC-Rio
   */
  private static final class TrustedServerManagerDisabledState
  implements TrustedServerManagerState {
    public void start()
    throws UnsupportedOperationException {
      throw new UnsupportedOperationException();
    }

    public void stop()
    throws UnsupportedOperationException {
      throw new UnsupportedOperationException();
    }

    public SortedSet<TrustedServer> getAllTrustedServers()
    throws UnsupportedOperationException {
      throw new UnsupportedOperationException();
    }

    public boolean addTrustedServer(TrustedServerInfo localServer)
    throws RemoteException, UnsupportedOperationException {
      throw new UnsupportedOperationException();
    }

    public boolean modifyTrustedServer(
      TrustedServer oldLocalServer, TrustedServerInfo newLocalServer)
    throws RemoteException, UnsupportedOperationException {
      throw new UnsupportedOperationException();
    }

    public void removeTrustedServer(TrustedServer localServer)
    throws RemoteException, UnsupportedOperationException {
      throw new UnsupportedOperationException();
    }

    public void addTrustedServerManagerListener(
      TrustedServerManagerListener listener)
    throws UnsupportedOperationException {
      throw new UnsupportedOperationException();
    }

    public void removeTrustedServerManagerListener(
      TrustedServerManagerListener listener)
    throws UnsupportedOperationException {
      throw new UnsupportedOperationException();
    }

    public boolean addCertificate(String serverName, Certificate cert)
      throws UnsupportedOperationException {
      throw new UnsupportedOperationException();      
    }

    public void removeCertificate(String serverName) throws RemoteException,
      UnsupportedOperationException {
      throw new UnsupportedOperationException();
    }
  }

  /**
   * Representa o estado habilitado do gerenciador de servidores locais.
   *
   * @author Tecgraf/PUC-Rio
   */
  private static final class TrustedServerManagerEnabledState
  implements TrustedServerManagerState, Observer {
    /**
     * Lista com os ouvintes interessados no contedo do repositrio de
     * servidores locais.
     */
    private List listenerList;

    /** O observador do servio de servidores locais. */
    private TrustedServerServiceRemoteObserver remoteObserver;

    TrustedServerManagerEnabledState() {
      this.listenerList = new LinkedList();
    }

    /**
     * Verifica se o observador do servio de servidores locais j foi
     * iniciado.
     *
     * @return true, caso o observador j tenha sido iniciado, ou false, caso
     *         contrrio.
     */
    private boolean wasStarted() {
      return (this.remoteObserver != null);
    }

    public void start()
    throws RemoteException {
      if (this.wasStarted()) {
        throw new IllegalStateException(
          "Tentativa de iniciar o gerenciador de servidores locais que j foi iniciado.");
      }
      this.remoteObserver = new TrustedServerServiceRemoteObserver();
      this.remoteObserver.addObserver(this);
      this.remoteObserver.install();
    }

    public void stop() {
      if (!this.wasStarted()) {
        throw new IllegalStateException(
          "Tentativa de para o gerenciador de servidores locais sem ter sido iniciado.");
      }
      this.remoteObserver.deleteObserver(this);
      this.remoteObserver.uninstall();
      this.remoteObserver = null;
    }

    public SortedSet<TrustedServer> getAllTrustedServers()
    throws RemoteException {
      return ClientRemoteLocator.trustedServerService.getAllTrustedServers();
    }

    public boolean addTrustedServer(TrustedServerInfo localServer)
    throws RemoteException {
      return ClientRemoteLocator.trustedServerService.addTrustedServer(localServer);
    }

    public boolean modifyTrustedServer(
      TrustedServer oldLocalServer, TrustedServerInfo newLocalServer)
    throws RemoteException {
      return ClientRemoteLocator.trustedServerService.modifyTrustedServer(
        oldLocalServer, newLocalServer);
    }

    public void removeTrustedServer(TrustedServer localServer)
    throws RemoteException {
      ClientRemoteLocator.trustedServerService.removeTrustedServer(localServer);
    }

    public void addTrustedServerManagerListener(
      TrustedServerManagerListener listener) {
      if (!this.wasStarted()) {
        throw new IllegalStateException(
          "Tentativa de adicionar ouvinte ao gerenciador de servidores locais que no foi iniciado.");
      }
      this.listenerList.add(listener);
    }

    public void removeTrustedServerManagerListener(
      TrustedServerManagerListener listener) {
      if (!this.wasStarted()) {
        throw new IllegalStateException(
          "Tentativa de remover ouvinte do gerenciador de servidores locais que no foi iniciado.");
      }
      this.listenerList.remove(listener);
    }

    /**
     * Tratador de evento de adio de um servidor local.
     *
     * @param event O evento de adio de um servidor local.
     */
    private void handleEvent(TrustedServerAddEvent event) {
      this.fireWasAddedLocalServer(event.getTrustedServer());
    }

    /**
     * Tratador de evento de modificao de um servidor local.
     *
     * @param event O evento de modificao de um servidor local.
     */
    private void handleEvent(TrustedServerModifyEvent event) {
      this.fireWasModifiedLocalServer(
        event.getOldTrustedServer(), event.getNewTrustedServer());
    }

    /**
     * Tratador de evento de remoo de um servidor local.
     *
     * @param event O evento de remoo de um servidor local.
     */
    private void handleEvent(TrustedServerRemoveEvent event) {
      this.fireWasRemovedLocalServer(event.getTrustedServer());
    }

    /**
     * Informa a todos os ouvintes que um servidor local foi adicionado ao
     * repositrio de servidores locais.
     *
     * @param localServer O servidor local adicionado.
     */
    private void fireWasAddedLocalServer(TrustedServer localServer) {
      Iterator listenerIterator = this.listenerList.iterator();
      while (listenerIterator.hasNext()) {
        TrustedServerManagerListener listener =
          (TrustedServerManagerListener)listenerIterator.next();
        listener.wasAddedTrustedServer(localServer);
      }
    }

    /**
     * Notifica aos ouvintes interessados que um servidor local foi modificado.
     *
     * @param oldLocalServer O servidor local em seu estado antigo (antes da
     *        modificao).
     * @param newLocalServer O servidor local em seu estado atual (j
     *        modificado).
     */
    private void fireWasModifiedLocalServer(
      TrustedServer oldLocalServer, TrustedServer newLocalServer) {
      Iterator listenerIterator = this.listenerList.iterator();
      while (listenerIterator.hasNext()) {
        TrustedServerManagerListener listener =
          (TrustedServerManagerListener)listenerIterator.next();
        listener.wasModifiedTrustedServer(oldLocalServer, newLocalServer);
      }
    }

    /**
     * Notifica aos ouvintes interessados que um servidor local foi removido.
     *
     * @param localServer O servidor local removido.
     */
    private void fireWasRemovedLocalServer(TrustedServer localServer) {
      Iterator listenerIterator = this.listenerList.iterator();
      while (listenerIterator.hasNext()) {
        TrustedServerManagerListener listener =
          (TrustedServerManagerListener)listenerIterator.next();
        listener.wasRemovedTrustedServer(localServer);
      }
    }

    public void update(Observable o, Object arg) {
      TrustedServerEvent localServerEvent = (TrustedServerEvent)arg;
      if (localServerEvent.getType().equals(TrustedServerEventType.ADD)) {
        handleEvent((TrustedServerAddEvent)localServerEvent);
      }
      else if (localServerEvent.getType().equals(TrustedServerEventType.MODIFY)) {
        handleEvent((TrustedServerModifyEvent)localServerEvent);
      }
      else if (localServerEvent.getType().equals(TrustedServerEventType.REMOVE)) {
        handleEvent((TrustedServerRemoveEvent)localServerEvent);
      }
      else {
        throw new IllegalStateException(
          "Recebido um evento de um tipo desconhecido: "+
          localServerEvent.getType());
      }
    }

    public boolean addCertificate(String serverName, Certificate cert)
      throws RemoteException {
      return ClientRemoteLocator.trustedServerService.addCert(serverName, cert);
    }

    public void removeCertificate(String serverName) throws RemoteException {
      ClientRemoteLocator.trustedServerService.removeCert(serverName);
    }
  }

  static {
    RestartManager.getInstance().addListener(
      new RestartListener() {
        public void restart() {
          TrustedServerManager.instance = null;
        }
      });
  }
}
