/*
 * XAppProxy.java
 * @author Andr Luiz Clinio
 * $Revision: 91606 $  - $Date: 2009-05-18 11:18:43 -0300 (Mon, 18 May 2009) $
 */
package csbase.client.remote.srvproxies;

import java.net.InetAddress;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Set;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import tecgraf.javautils.LNG;
import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.RemoteTask;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.CommandInfo;
import csbase.logic.CommandSubmission;
import csbase.logic.CommonClientProject;
import csbase.logic.Priority;
import csbase.logic.SGASet;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.SchedulerServiceInterface;

/**
 * Classe que modela um proxy ISI/SchedulerService.
 *
 * @author Andr Luiz Clinio
 */
public class XAppProxy {

  final private static String PREFIX =
           "csbase.client.remote.srvproxies.XAppProxy";

  /**
   * Formatao de mensagem segundo o padro de tag com nome da classe.
   * @param tag o tag da mensagem (sem o prefixo da classe).
   * @return a string internacionalizada.
   */
  private static String getString(final String tag) {
     final String fullTag = PREFIX+"."+tag;
     return LNG.get(fullTag);
  }

  /**
   * Formatao de mensagem segundo o padro de tag com nome da classe.
   * @param tag o tag da mensagem (sem o prefixo da classe).
   * @param objs os parmetros que sero formatados.
   * @return a string internacionalizada.
   */
  private static String getString(final String tag, final Object[] objs) {
     return MessageFormat.format(XAppProxy.getString(tag), objs);
  }

  /**
   * Mtodo de aviso de erro
   * @param msg mensagem a ser exibida.
   */
  private static void showError(String msg) {
    final DesktopFrame mainFrame = DesktopFrame.getInstance();
    if (mainFrame == null) {
      return;
    }
    final JFrame jFrame = mainFrame.getDesktopFrame();
    if (jFrame == null) {
      return;
    }
    if ((msg == null) || msg.trim().equals("")) {
      msg = XAppProxy.getString("failed.request");
    }
    final String title =  XAppProxy.getString("failure.title");
    StandardErrorDialogs.showErrorDialog(jFrame, title, msg);
  }

  /**
   * Monta o display default do cliente.
   *
   * @return uma string com esse valor
   */
  private static String getDefaultDisplay() {
    try {
      return InetAddress.getLocalHost().getHostName()+":0.0";
    }
    catch (Exception e) {
      final String msg = XAppProxy.getString("display.question");
      return JOptionPane.showInputDialog(msg, "");
    }
  }

  /**
   * Mtodo para execuo do desktop.
   *
   * @param appId identificador do programa.
   *
   * @return flag booleano de sucesso da operao
   */
  public static boolean runDesktopApplication(final Object appId) {
    final String dspName = getDefaultDisplay();
    if (dspName == null) {
      return false;
    }
    final DesktopFrame mainFrame = DesktopFrame.getInstance();
    if (mainFrame == null) {
      showError(XAppProxy.getString("no.server.error"));
      return false;
    }
    final CommonClientProject prj = mainFrame.getProject();
    if (prj == null) {
      showError(XAppProxy.getString("no.project.error"));
      return false;
    }
    final String prjId = prj.getId();
    return runApplication(prjId, appId, dspName, null);
  }

  /**
   * Mtodo para execuo.
   *
   * @param prjId Identificador do projeto.
   * @param appId identificador do programa.
   * @param dspName nome do display X-Windows.
   * @param sgaName nome do SGA.
   *
   * @return flag booleano de sucesso da operao
   *
   */
  public static boolean runApplication(final String prjId, final Object appId,
  final String dspName, final String sgaName) {
    final SchedulerServiceInterface schedulerService = ClientRemoteLocator.schedulerService;
    final DesktopFrame mainFrame = DesktopFrame.getInstance();
    if ((mainFrame == null) || (schedulerService == null)) {
      showError(XAppProxy.getString("no.server.error"));
      return false;
    }
    RemoteTask<Void> task = new RemoteTask<Void>() {
        @Override
        public void performTask() throws Exception {
          HashMap<String, String> parameters = new HashMap<String, String>();
          parameters.put("DISPLAY",dspName);
          String algorithmName = appId.toString();
          AlgorithmVersionId algorithmVersionId = new AlgorithmVersionId(1, 0, 0);
          AlgorithmConfigurator configurator =
            ClientRemoteLocator.algorithmService.createAlgorithmConfigurator(
              algorithmName, algorithmVersionId);
          configurator.setParameterValuesByName(parameters);
          final CommandSubmission submission = new CommandSubmission(
              configurator, prjId);
          if (sgaName == null) {
            submission.configureXAppExecution();
          }
          else {
            submission.configureXAppExecution(sgaName);
          }
          submission.setPriority(Priority.HIGH);
          final String _realSGAName = sgaName;
          Set<CommandInfo> commands = schedulerService.submitCommand(submission);
          if (commands.isEmpty()) {
             String error = XAppProxy.getString("failed.launch");
             final SGASet sga =
               ClientRemoteLocator.sgaService.getSGASet(_realSGAName);
             if (sga == null || !sga.getAlive()) {
                final Object[] objs = new Object[] {_realSGAName};
                error = XAppProxy.getString("failed.sga", objs);
             }
             throw new Exception(error);
          }
        }
      };

    final String title = XAppProxy.getString("lauching.title");
    final String msg = XAppProxy.getString("lauching.message");
    return task.execute(mainFrame.getDesktopFrame(), title, msg);
  }
}
