package csbase.client.util.sga;

import csbase.client.algorithms.AlgorithmConfiguratorView;
import csbase.client.kernel.ClientException;
import csbase.logic.SGASet;

import tecgraf.javautils.LNG;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * Define um painel para seleo de SGAs do tipo cluster.
 * 
 * @author ururahy
 *
 */
public class SGAParallelSelectionPanel extends SGASelectionPanel {
  private JLabel numServerLabel;
  private JTextField numServerTextField;
  private JComboBox sgaServerComboBox;
  private JPanel numCPUPanel;

  /**
   * Define um painel para seleo de SGAs do tipo cluster.
   * 
   * @param configuratorView a viso do configurador de algoritmos, que informa
   *        se o algoritmo  de execuo paralela.
   */
  public SGAParallelSelectionPanel(AlgorithmConfiguratorView configuratorView) {
    super(configuratorView);
    this.numServerLabel = new JLabel(
        LNG.get("SGAParallelSelectionPanel.label.num_server"));
    this.numServerTextField = new MyJTextField();
    this.sgaServerComboBox = makeSGAServerComboBox();
    this.createNumCPUPanel();
  }

  /**
   * 
   * @see csbase.client.util.sga.SGASelectionPanel#getManualServerCollection()
   */
  @Override
  public Collection<SGASet> getManualServerCollection() {
    Collection<SGASet> selectedServerCollection = new LinkedList<SGASet>();
    SGASet selectedServer = getSelectedServer();
    if (selectedServer != null) {
      selectedServerCollection.add(selectedServer);
    }
    return Collections.unmodifiableCollection(selectedServerCollection);
  }

  /**
   * 
   * @see csbase.client.util.sga.SGASelectionPanel#getManualSelectedServer()
   */
  @Override
  public SGASet getManualSelectedServer() {
    return (SGASet)this.sgaServerComboBox.getSelectedItem();
  }

  @Override
  public boolean hasAvailableServers() {
    if (this.sgaServerComboBox.getItemCount() == 0) {
      return false;
    }
    return true;
  }

  /**
   * <p>Verifica se a seleo de servidores est pronta.</p>
   *  <p>A seleo  considerada pronta, se e somente se 1 dos seguintes
   * critrios for atendido:
   *  <ul>
   *    <li>A seleo de servidor estiver no modo automtico.</li>
   *    <li>A seleo de servidor estiver no modo manual e existir ao
   *    menos um servidor selecionado.</li>
   *  </ul>
   *  </p>
   *
   * @return .
   */
  @Override
  public boolean isSelectionReady() {
    if (!this.hasAvailableServers()) {
      return false;
    }
    if (this.numServerTextField.getText().equals("")) {
        return false;
    }
    return true;
  }

  /**
   * 
   * @see csbase.client.util.sga.SGASelectionPanel#populateGUIComponents()
   */
  @Override
  public void populateGUIComponents() {
    this.removeAll();
    this.setLayout(new GridBagLayout());
    GridBagConstraints c = new GridBagConstraints();
    c.anchor = GridBagConstraints.WEST;
    c.gridx = 1;
    c.gridy = 1;
    c.gridwidth = 1;
    c.gridheight = 1;
    c.insets = new Insets(2, 2, 2, 2);
    c.weightx = 1.0;
    c.weighty = 0.0;
    c.fill = GridBagConstraints.HORIZONTAL;
    this.add(this.platformFilterPanel, c);
    c.insets = new Insets(2, 2, 2, 2);
    c.gridx = 1;
    c.gridy++;
    this.add(this.automaticServerRadioButton, c);
    c.gridy++;
    this.add(this.manualServerRadioButton, c);
    c.weightx = 0.0;
    c.gridy++;
    c.insets = new Insets(2, 20, 2, 2);
    this.add(this.sgaServerComboBox, c);
    c.insets = new Insets(2, 2, 2, 2);
    c.gridy++;
    this.add(this.numCPUPanel, c);
    this.revalidate();
    this.updateEnabledServers();
  }

  /**
   * Obtm o nmero de cpus que sero utilizados na execuo paralela.
   * 
   * @return nmero de cpus.
   */
  public Integer getCpuCountForParallelExecution() {
    return Integer.parseInt(this.numServerTextField.getText());
  }
  
  /**
   * Cria um painel para a insero do nmero de CPUs desejada para a
   * execuo de um algoritmo.
   */
  private void createNumCPUPanel() {
    this.numCPUPanel = new JPanel();
    this.numCPUPanel.setLayout(new GridBagLayout());
    GridBagConstraints c = new GridBagConstraints();
    c.anchor = GridBagConstraints.WEST;
    c.insets = new Insets(2, 2, 2, 2);
    c.gridx = 1;
    c.gridy = 1;
    c.gridwidth = 1;
    c.gridheight = 1;
    c.weightx = 0.0;
    c.weighty = 0.0;
    c.fill = GridBagConstraints.NONE;
    this.numCPUPanel.add(this.numServerLabel, c);
    c.weightx = 1.0;
    c.gridx++;
    this.numCPUPanel.add(this.numServerTextField, c);
  }

  @Override
  public void updateEnabledServers() {
    if (this.hasAvailableServers()) {
      if (this.manualServerRadioButton.isSelected()) {
        this.sgaServerComboBox.setEnabled(true);
      }
      else {
        this.sgaServerComboBox.setEnabled(false);
      }
      this.manualServerRadioButton.setEnabled(true);
      this.automaticServerRadioButton.setEnabled(true);
      this.numServerLabel.setEnabled(true);
      this.numServerLabel.setEnabled(true);
    }
    else {
      this.manualServerRadioButton.setEnabled(false);
      this.automaticServerRadioButton.setEnabled(false);
      this.numServerLabel.setEnabled(false);
      this.numServerTextField.setEnabled(false);
      this.sgaServerComboBox.setEnabled(false);
    }
  }

  /**
   * Recarrega os SGAs disponveis (podem ser filtrados por plataforma).
   */
  @Override
  public void reloadSGAs() {
    try {
      sgaServerComboBox.removeAllItems();
      for (SGASet sgaSet : this.loadServers()) {
        sgaServerComboBox.addItem(sgaSet);
      }
      updateEnabledServers();
    } catch (ClientException e) {
      e.printStackTrace();
    }
  }

  /**
   * Cria um componente do tipo JComboBox para a seleo de SGAs.
   *
   * @return O componente JComboBox criado e populado com os SGAs disponveis.
   */
  private JComboBox makeSGAServerComboBox() {
    JComboBox combo = null;
    try {
      combo = new JComboBox(this.loadServers());
      combo.addItemListener(
        new ItemListener() {
          public void itemStateChanged(ItemEvent e) {
            fireChangedServer();
          }
        });
    }
    catch (ClientException e) {
      e.printStackTrace();
    }
    return combo;
  }

  /**
   * A classe MyJTextField cria um componente do tipo JTextField onde os
   * caracteres vlidos para a insero na caixa de texto consiste apenas de
   * dgitos numricos.
   */
  private class MyJTextField extends JTextField {
    public MyJTextField() {
      addKeyListener(new KeyAdapter() {
        @Override
        public void keyTyped(KeyEvent ev) {
          if ((!Character.isDigit(ev.getKeyChar())) && 
              (!(ev.getKeyChar()==KeyEvent.VK_BACK_SPACE)))
            ev.consume();   
        }
        
        @Override
        public void keyReleased(KeyEvent ev) {
          if((Character.isDigit(ev.getKeyChar())) || 
              (ev.getKeyChar() == KeyEvent.VK_BACK_SPACE) || 
              (ev.getKeyChar() == KeyEvent.VK_DELETE))
            fireChangedServer();
        }
      });
      this.setColumns(5);
    }
  }
}
