package csbase.logic;

import java.rmi.NoSuchObjectException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

/**
 * Classe responsvel por observar no cliente quando um lock de arquivo  obtido
 * ou expirado.
 */
public class ClientFileLockListener extends UnicastRemoteObject implements
  FileLockListenerInterface {

  /**
   * Indica se j chegou a notificao de que o lock foi obtido ou o pedido
   * expirou.
   */
  private boolean gotNotification;

  /**
   * Indica se conseguiu o lock.
   */
  private boolean locked;

  /**
   * Constri o listener que  notificado quando o lock  obtido ou expirado

   * @throws RemoteException falha na exportao do objeto
   */
  public ClientFileLockListener()
  throws RemoteException {
    super();
    this.gotNotification = false;
    this.locked = false;
  }

  /**
   * {@inheritDoc} 
   * Remove o objeto exportado e chama o mtodo performFileLocked
   * na thread do Swing com o invokeLater.
   */
  public synchronized final void fileLocked(Object lockId)
  throws RemoteException {
    locked = true;
    unexportObject(this, true);
    gotNotification();
  }

  /**
   * {@inheritDoc}
   * Remove o objeto exportado e chama o mtodo
   * performFileLockExpired na thread do Swing com o invokeLater.
   */
  public synchronized final void fileLockExpired() throws RemoteException {
    locked = false;
    unexportObject(this, true);
    gotNotification();
  }

  /**
   * Espera por uma notificao: de <code>fileLocked</code> ou de
   * <code>fileLockExpired</code>.
   *
   * @return true se obteve o lock do arquivo
   */
  public synchronized boolean waitLock() {
    while (!gotNotification) {
      try {
        wait();
      }
      catch (InterruptedException e) {
      }
    }
    return locked;
  }

  /**
   * Chegou a notificao de que o lock do arquivo foi obtido ou expirado.
   */
  private synchronized void gotNotification() {
    gotNotification = true;
    notify();
  }

  /**
   * Chama o <code>unexportObject</code> do listener se ele ainda no recebeu
   * a notificao de lock obtido ou de timeout.
   */
  public synchronized void unexportListener() {
    if (!gotNotification) {
      try {
        unexportObject(this, true);
      }
      catch (NoSuchObjectException e) {
        // Acontece apenas quando o listener j recebeu a notificao, pois
        // os mtodos fileLocked e fileLockExpired j fazem o unexportObject.
      }
    }
  }

}
