/*
 * $Id: ClientProjectFileInputStream.java 105206 2010-05-06 23:06:04Z costa $
 */

package csbase.logic;

import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.rmi.RemoteException;

/**
 * Implementa um leitor (stream) de um arquivo (client-side).
 * 
 * @author Tecgraf/PUC-Rio
 */
public class ClientProjectFileInputStream extends InputStream {

  /**
   * Arquivo a ser lido
   */
  private final ClientProjectFile file;

  /**
   * Cache contendo bytes que podem ser lidos sem fazer chamadas remotas
   */
  private final ByteBuffer cache;

  /**
   * Marca anteriormente registrada para o stream (relativa ao arquivo).
   */
  private long fileMark = 0;

  /**
   * Posio corrente no arquivo
   */
  private long currentFilePosition = 0;

  /**
   * {@inheritDoc}
   */
  @Override
  public int available() {
    return cache.remaining();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void close() throws IOException {
    file.close();
  }

  /**
   * <p>
   * Mtodo para ler bytes (efetivamente) do arquivo remoto armazenando-os na
   * cache.
   * </p>
   * Se a cache for grande o suficiente, o mtodo ler a partir da posio
   * especificada at o fim do arquivo. Caso contrrio, ler apenas o nmero de
   * bytes que couberem na cache.
   * 
   * @param startReadPosition posio a partir da qual ser feita a leitura no
   *        arquivo. Se a posio for maior ou igual que o tamanho do arquivo, o
   *        mtodo retorna sem lanar exceo.
   * 
   * @throws IOException em caso de erro de I/O
   */
  private void fillCache(final long startReadPosition) throws IOException {
    try {
      final long size = file.size();
      if (startReadPosition >= size) {
        cache.position(cache.limit());
        return;
      }
      int expectedBytes = cache.capacity();
      final long fileRemaining = size - startReadPosition;
      if (fileRemaining < cache.capacity()) {
        expectedBytes = (int) fileRemaining;
      }
      cache.clear();
      int totalBytesRead = 0;
      do {
        final long bytesRead = file.read(cache, startReadPosition);
        totalBytesRead += bytesRead;
      } while (totalBytesRead < expectedBytes);
      cache.flip();
    }
    catch (final Exception e) {
      throw new IOException(e);
    }
  }

  /**
   * Obtm o <code>ClientProjectFile</code>.
   * 
   * @return o <code>ClientProjectFile</code>
   */
  public ClientProjectFile getClientProjectFile() {
    return file;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public synchronized void mark(final int readlimit) {
    fileMark = currentFilePosition;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean markSupported() {
    return true;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public int read() throws IOException {
    if (cache.remaining() == 0) {
      fillCache(currentFilePosition);
      if (cache.remaining() == 0) {
        return -1;
      }
    }
    final int data = cache.get() & 0xFF;
    currentFilePosition++;
    return data;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public int read(final byte[] buffer) throws IOException {
    if (cache.remaining() == 0) {
      fillCache(currentFilePosition);
      if (cache.remaining() == 0) {
        return -1;
      }
    }
    int bufferRemaining = buffer.length;
    int totalBytesRead = 0;
    while (bufferRemaining > cache.remaining()) {
      final int requestedBytes = cache.remaining();
      cache.get(buffer, totalBytesRead, requestedBytes);
      totalBytesRead += requestedBytes;
      currentFilePosition += requestedBytes;
      bufferRemaining -= requestedBytes;
      fillCache(currentFilePosition);
      if (cache.remaining() == 0) {
        return totalBytesRead;
      }
    }
    final int requestedBytes = bufferRemaining;
    cache.get(buffer, totalBytesRead, requestedBytes);
    totalBytesRead += requestedBytes;
    currentFilePosition += requestedBytes;
    return totalBytesRead;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public synchronized void reset() throws IOException {
    if (fileMark >= 0) {
      fillCache(fileMark);
    }
    fileMark = -1;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public long skip(final long nbytes) throws IOException {
    long n = nbytes;
    if (n < 0) {
      return n;
    }
    currentFilePosition += n;
    if (currentFilePosition > file.size()) {
      n -= currentFilePosition - file.size();
      currentFilePosition = file.size();
    }
    if (n > cache.remaining()) {
      fillCache(currentFilePosition);
    }
    else {
      cache.position(cache.position() + (int) n);
    }
    return n;
  }

  /**
   * Cria um <code>InputStream</code> a partir de um arquivo da rea de
   * projetos, abrindo-o para leitura/escrita e lendo um bloco (cache) inicial.
   * 
   * @param file Arquivo da rea de projetos (<code>ClientProjectFile</code>).
   * 
   * @param bSize Tamanho desejado para a cache.
   * 
   * @throws IOException em caso de erro de I/O
   * @throws RemoteException falha de rmi na criao do channel no servidor
   */
  public ClientProjectFileInputStream(final ClientProjectFile file,
    final int bSize) throws IOException, RemoteException {
    final int cacheSize = bSize <= 0 ? 16 * 1024 : bSize;
    this.file = file;
    cache = ByteBuffer.allocate(cacheSize);
    file.openFileChannel(true);
    fillCache(0);
  }
}
