package csbase.logic;

import java.io.Serializable;
import java.rmi.Remote;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * Representa uma sesso de um usurio logado. Esse objeto deve existir durante
 * todo o perodo em que o servidor estiver sendo utilizado.
 */
public final class Session implements Serializable {
  /**
   * O usurio da sesso.
   */
  private User user;
  private Serializable key;
  private Remote spy;
  private Map<String, Serializable> attributes;

  /**
   * Cria uma sesso com a opo de um espio de conexo. Esse espio  um
   * objeto remoto que pode ser criado para monitorar a conexo com o cliente
   * atravs da callback {@link java.rmi.server.Unreferenced#unreferenced()}.
   * 
   * @param user O usurio da sesso.
   * @param key Chave de identificao de uma sesso.
   * @param spy Espio da conexo.
   * @param attributes Os atributos da sesso.
   */
  public Session(User user, Serializable key, Remote spy,
    Map<String, Serializable> attributes) {
    this.user = user;
    this.key = key;
    this.spy = spy;
    if (attributes == null) {
      this.attributes = new HashMap<String, Serializable>();
    }
    else {
      this.attributes = new HashMap<String, Serializable>(attributes);
    }
  }

  /**
   * Getter.
   * 
   * @return O usurio da sesso corrente.
   */
  public User getUser() {
    return this.user;
  }

  /**
   * Getter.
   * 
   * @return Chave da sesso corrente.
   */
  public Object getKey() {
    return this.key;
  }

  /**
   * Obtm um atributo da sesso.
   * 
   * @param attributeName O nome do atributo.
   * 
   * @return O atributo da sesso, ou null, caso no exista.
   */
  public Serializable getAttribute(String attributeName) {
    return this.attributes.get(attributeName);
  }

  public Map<String, Serializable> getAttributes() {
    return Collections.unmodifiableMap(this.attributes);
  }

  public boolean equals(Object object) {
    if (object == null) {
      return false;
    }
    if (!(object.getClass().equals(Session.class))) {
      return false;
    }
    Session session = (Session) object;
    return this.key.equals(session.key);
  }

  public int hashCode() {
    return this.key.hashCode();
  }

  public String toString() {
    return this.key.toString();
  }
}
