package csbase.logic.algorithms.flows;

import java.io.Serializable;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import csbase.logic.algorithms.FileParameterValue;

/**
 * <p>
 * Fluxo de algoritmos
 * </p>
 * 
 * @author lmoreira
 */
public final class Flow implements Serializable, Cloneable {
  /**
   * O nome do fluxo.
   */
  private String name;

  /**
   * As conexes.
   */
  private Set<FlowLink> links;

  /**
   * Os ns.
   */
  private Set<FlowNode> nodes;

  /**
   * Cria um fluxo de algoritmos.
   * 
   * @param name O nome do fluxo (Aceita {@code null}).
   * @param nodes Os ns (No aceita {@code null}).
   * @param links As conexes (No aceita {@code null}).
   */
  public Flow(String name, Set<FlowNode> nodes, Set<FlowLink> links) {
    this.name = name;
    setLinks(links);
    setNodes(nodes);
  }

  /**
   * <p>
   * Atribui o valor do arquivo que deve receber a sada padro da execuo de
   * um algoritmo representado por este configurador.
   * </p>
   * <p>
   * Este valor ser repassado para todos os ns deste fluxo.
   * </p>
   * 
   * @param standardOutputFile
   */
  public void setStandardOutputFile(FileParameterValue standardOutputFile) {

    String stdOutPath = standardOutputFile.getPath();
    String type = standardOutputFile.getType();
    for (FlowNode node : nodes) {
      int nodeId = node.getId();
      String nodeStdOutPath = String.format("%s.%s", stdOutPath, nodeId);
      FileParameterValue nodeStdOut =
        new FileParameterValue(nodeStdOutPath, type);

      node.setStandardOutputFile(nodeStdOut);
    }
  }

  /**
   * <p>
   * Obtm um conjunto imutvel contendo os arquivos de sada padro de cada n
   * que sero utilizadas durante a execuo de um fluxo.
   * </p>
   * 
   * @return um conjunto imutvel contendo os arquivos de sada padro de cada
   *         n que sero utilizadas durante a execuo de um fluxo.
   */
  public Set<FileParameterValue> getStandardOutputFiles() {
    Set<FileParameterValue> stdouts = new HashSet<FileParameterValue>();
    for (FlowNode node : nodes) {
      if (null != node.getStandardOutputFile()) {
        stdouts.add(node.getStandardOutputFile());
      }
    }
    return Collections.unmodifiableSet(stdouts);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Flow clone() {
    try {
      Flow clone = (Flow) super.clone();
      clone.name = this.name;
      clone.links = new HashSet<FlowLink>();
      for (FlowLink flowLink : this.links) {
        clone.links.add(flowLink.clone());
      }
      clone.nodes = new HashSet<FlowNode>();
      for (FlowNode flowNode : this.nodes) {
        clone.nodes.add(flowNode.clone());
      }
      return clone;
    }
    catch (CloneNotSupportedException e) {
      throw new IllegalStateException(e);
    }
  }

  /**
   * <p>
   * Obtm as conexes deste fluxo.
   * </p>
   * 
   * <p>
   * O conjunto retornado  imutvel (veja
   * {@link Collections#unmodifiableSet(Set)}).
   * </p>
   * 
   * @return As conexes, caso no haja conexes, retorna um conjunto vazio.
   */
  public Set<FlowLink> getLinks() {
    return Collections.unmodifiableSet(this.links);
  }

  /**
   * Obtm o nome do fluxo.
   * 
   * @return o nome do fluxo ou {@code null} se ele no tiver nome.
   */
  public String getName() {
    return name;
  }

  /**
   * Obtm um n dado o identificador do n.
   * 
   * @param nodeId O identificador do n.
   * 
   * @return O n ou {@code null} se ele no existir.
   */
  public FlowNode getNode(int nodeId) {
    for (FlowNode node : nodes) {
      if (node.getId() == nodeId) {
        return node;
      }
    }
    return null;
  }

  /**
   * <p>
   * Obtm os ns deste fluxo.
   * </p>
   * 
   * <p>
   * O conjunto retornado  imutvel (veja
   * {@link Collections#unmodifiableSet(Set)}).
   * </p>
   * 
   * @return As conexes, caso no haja ns, retorna um conjunto vazio.
   */
  public Set<FlowNode> getNodes() {
    return Collections.unmodifiableSet(this.nodes);
  }

  /**
   * <p>
   * Atribui conexes a este fluxo.
   * </p>
   * 
   * @param links O conjunto de conexes (No aceita {@code null}).
   */
  private void setLinks(Set<FlowLink> links) {
    if (links == null) {
      throw new IllegalArgumentException("O parmetro links est nulo.");
    }
    this.links = new HashSet<FlowLink>(links);
  }

  /**
   * <p>
   * Atribui um nome a este fluxo.
   * </p>
   * 
   * @param name O nome do fluxo (No aceita {@code null}).
   */
  public void setName(String name) {
    if (name == null) {
      throw new IllegalArgumentException("O parmetro name est nulo.");
    }
    this.name = name;
  }

  /**
   * <p>
   * Atribui ns a este fluxo.
   * </p>
   * 
   * @param nodes O conjunto de ns (No aceita {@code null}).
   */
  private void setNodes(Set<FlowNode> nodes) {
    if (nodes == null) {
      throw new IllegalArgumentException("O parmetro nodes est nulo.");
    }
    this.nodes = new HashSet<FlowNode>(nodes);
  }

  /**
   * <p>
   * Atribui o valor do arquivo que deve receber o cdigo de sada de execuo
   * de um algoritmo representado por este configurador.
   * </p>
   * <p>
   * Este valor ser repassado para todos os ns deste fluxo.
   * </p>
   * 
   * @param exitCodeLogFile arquivo com o cdigo de sada.
   */
  public void setExitCodeLogFile(FileParameterValue exitCodeLogFile) {
    String exitCodeFilePath = exitCodeLogFile.getPath();
    String type = exitCodeLogFile.getType();
    for (FlowNode node : nodes) {
      int nodeId = node.getId();
      String nodeExitCodeFilePath =
        String.format("%s.%s", exitCodeFilePath, nodeId);
      FileParameterValue nodeExitCodeFile =
        new FileParameterValue(nodeExitCodeFilePath, type);
      if (node.hasExitCode()) {
        node.setExitCodeLogFile(nodeExitCodeFile);
      }
    }
  }

  /**
   * <p>
   * Obtm um conjunto imutvel contendo que devem receber o cdigo de sada de
   * execuo de cada n durante a execuo de um fluxo.
   * </p>
   * 
   * @return um conjunto imutvel contendo os arquivos de cdigo de sada dos
   *         ns do fluxo.
   */
  public Set<FileParameterValue> getExitCodeLogFiles() {
    Set<FileParameterValue> exitCodeFiles = new HashSet<FileParameterValue>();
    for (FlowNode node : nodes) {
      if (node.hasExitCode() && node.getExitCodeLogFile() != null) {
        exitCodeFiles.add(node.getExitCodeLogFile());
      }
    }
    return Collections.unmodifiableSet(exitCodeFiles);
  }

  /**
   * Indica se pelo menos um n do fluxo prov cdigo de sada durante sua
   * execuo.
   * 
   * @return Verdadeiro se pelo menos um n prov cdigo de sada, ou falso,
   *         caso contrrio.
   */
  public boolean hasExitCode() {
    for (FlowNode node : nodes) {
      if (node.hasExitCode()) {
        return true;
      }
    }
    return false;
  }
}
