package csbase.logic.algorithms.parameters;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import csbase.exception.ParseException;

/**
 * Enumerao para seleo mltipla.
 * 
 * @author lmoreira
 */
public final class EnumerationListParameter extends
  ListParameter<EnumerationItem> {

  /**
   * Expresso regular utilizada pelo mtodo {@link #setValueAsText(String)}
   * para separar a lista de {@link #items} da lista de objetos selecionados.
   */
  private static final Pattern valueAsTextPattern =
    Pattern.compile("^\\{(.*)\\}:(\\{.*\\})?$");

  /**
   * Expresso regular utilizada pelo mtodo {@link #setItemsAsText(String)}
   * para recriar cada item e adicion-los  lista de {@link #items}
   */
  private static final Pattern itemsPattern =
    Pattern
      .compile("\\{id=(.*?)/label=(.*?)/value=(.*?)/description=(.*?)/visible=(.*?)\\}");

  /**
   * Formato da representao de um {@link EnumerationItem item} como um texto.
   */
  private static final String itemAsTextFormat =
    "{id=%s/label=%s/value=%s/description=%s/visible=%s}";

  /**
   * Os tens desta enumerao indexados pelos seus rtulos.
   */
  private Map<String, EnumerationItem> itemsByLabel;

  /**
   * Os tems preservando a ordem de insero.
   */
  private List<EnumerationItem> items;

  /**
   * Os observadores deste parmetro.
   */
  private transient List<EnumerationListParameterListener> listeners;

  /**
   * Indica se os itens devem ser ordenados.
   */
  private boolean mustSortItems;

  /**
   * TODO Este construtor ser removido numa reestruturao dos testes. Cria uma
   * enumerao para seleo mltipla.
   * 
   * @param name O nome do parmetro (No aceita {@code null}).
   * @param label O rtulo do parmetro (No aceita {@code null}).
   * @param description A descrio do parmetro (No aceita {@code null}).
   * @param isOptional Indica se o parmetro  opcional.
   * @param commandLinePattern O padro para construo da linha de comando. O
   *        padro ser utilizado para escrever o trecho da linha do comando
   *        referente ao parmetro. Esta string ser passada para o mtodo
   *        MessageFormat.format(String,Object...). O primeiro formato ({0}) 
   *        referente ao nome e o segundo formato ({1})  referente ao valor. Se
   *        {@code null} o parmetro no produzir sada na linha de comando.
   */
  public EnumerationListParameter(String name, String label,
    String description, boolean isOptional, String commandLinePattern) {
    this(name, label, description, isOptional, commandLinePattern, false);
  }

  /**
   * Cria uma enumerao para seleo mltipla.
   * 
   * @param name O nome do parmetro (No aceita {@code null}).
   * @param label O rtulo do parmetro (No aceita {@code null}).
   * @param description A descrio do parmetro (No aceita {@code null}).
   * @param isOptional Indica se o parmetro  opcional.
   * @param commandLinePattern O padro para construo da linha de comando. O
   *        padro ser utilizado para escrever o trecho da linha do comando
   *        referente ao parmetro. Esta string ser passada para o mtodo
   *        MessageFormat.format(String,Object...). O primeiro formato ({0}) 
   *        referente ao nome e o segundo formato ({1})  referente ao valor. Se
   *        {@code null} o parmetro no produzir sada na linha de comando.
   * @param mustSortItems indica se a enumerao deve ficar ordenada.
   */
  public EnumerationListParameter(String name, String label,
    String description, boolean isOptional, String commandLinePattern,
    boolean mustSortItems) {
    super(name, label, description, isOptional, commandLinePattern);
    this.itemsByLabel = new HashMap<String, EnumerationItem>();
    this.items = new LinkedList<EnumerationItem>();
    this.listeners = new LinkedList<EnumerationListParameterListener>();
    this.mustSortItems = mustSortItems;
    /* TODO Passar essa informao como atributo para a superclasse. */
    setEnabled(false);
  }

  /**
   * Adiciona um observador de parmetros.
   * 
   * @param listener O observador (No aceita {@code null}).
   */
  public final void addEnumerationListParameterListener(
    EnumerationListParameterListener listener) {
    if (listener == null) {
      throw new IllegalArgumentException("O parmetro listener est nulo.");
    }
    this.listeners.add(listener);
  }

  /**
   * <p>
   * Obtm os tens desta enumerao.
   * </p>
   * 
   * <p>
   * A lista retornada  imutvel (veja
   * {@link Collections#unmodifiableList(List)}).
   * </p>
   * 
   * @return A lista de tens (se no houver tens a lista estar vazio).
   */
  public List<EnumerationItem> getItems() {
    return Collections.unmodifiableList(this.items);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String getType() {
    return "ENUMERATION_LIST";
  }

  /**
   * Modifica os itens da enumerao. Remove os itens anteriores.
   * 
   * @param items Os tens (No aceita {@code null}).
   */
  public void setItems(List<EnumerationItem> items) {
    if (items == null) {
      throw new IllegalArgumentException("O parmetro items est nulo.");
    }
    setValue(null);
    this.items.clear();
    this.itemsByLabel.clear();
    for (EnumerationItem item : items) {
      addItem(item);
    }
    if (mustSortItems) {
      Collections.sort(this.items);
    }
    setEnabled(!items.isEmpty());
    for (EnumerationListParameterListener listener : this.listeners) {
      listener.itemsWereChanged(this);
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String getValueAsText() {

    StringBuilder sb = new StringBuilder();
    sb.append('{');
    for (int inx = 0; inx < items.size(); inx++) {
      EnumerationItem item = items.get(inx);
      if (0 < inx) {
        sb.append(',');
      }
      sb.append(getItemAsText(item));
    }
    sb.append('}');
    sb.append(':');
    String selectedValues = super.getValueAsText();
    if (null != selectedValues) {
      sb.append(selectedValues);
    }

    return sb.toString();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void setValueAsText(String parameterValue) throws ParseException {

    if (parameterValue == null) {
      setValue(null);
    }
    else {
      Matcher match = valueAsTextPattern.matcher(parameterValue);
      if (match.find() && 0 < match.groupCount()) {
        /*
         * A lista de itens indica o universo de itens de enumerao
         * selecionveis.
         */
        String items = match.group(1);
        setItemsAsText(items);
        /*
         * A lista de labels indica os valores selecionados.  o valor da lista.
         * {label1,label2,label3}
         */
        String selecteds = 2 == match.groupCount() ? match.group(2) : null;
        super.setValueAsText(selecteds);
      }
      else {
        throw new ParseException("No  possivel atribuir o valor\n({0}),"
          + "j que ele no representa uma codificao vlida uma lista de "
          + "itens de enumerao.\nParmetro envolvido: {1}.", new Object[] {
            parameterValue, this });
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected String getCommandItemValue(EnumerationItem itemValue) {
    return itemValue.getValue();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected EnumerationItem getItemValueFromText(String itemValue) {
    return this.itemsByLabel.get(itemValue);
  }

  /**
   * Adicionar tem de enumerao.
   * 
   * @param item O tem (No aceita {@code null}).
   * 
   * @return {@code true} se for possvel adicionar ou {@code false} se j
   *         existe um tem com este nome.
   */
  private boolean addItem(EnumerationItem item) {
    if (item == null) {
      throw new IllegalArgumentException("O parmetro item est nulo.");
    }
    if (this.itemsByLabel.containsKey(item.getLabel())) {
      return false;
    }
    this.itemsByLabel.put(item.getLabel(), item);
    this.items.add(item);
    return true;
  }

  /**
   * Cria os atributos transientes.
   * 
   * @param in
   * @throws IOException
   * @throws ClassNotFoundException
   */
  private void readObject(java.io.ObjectInputStream in) throws IOException,
    ClassNotFoundException {
    in.defaultReadObject();
    listeners = new LinkedList<EnumerationListParameterListener>();
  }

  /**
   * Obtm o valor de um {@link EnumerationItem item} sob a forma de String.
   * 
   * @param item O item.
   * 
   * @return O valor de um {@link EnumerationItem item} sob a forma de String.
   */
  private String getItemAsText(EnumerationItem item) {
    String id = (item.getId() == null ? "" : item.getId());
    String label = (item.getLabel() == null ? "" : item.getLabel());
    String value = (item.getValue() == null ? "" : item.getValue());
    String description =
      (item.getDescription() == null ? "" : item.getDescription());
    String visible = Boolean.toString(item.isVisible());

    return String.format(itemAsTextFormat, id, label, value, description,
      visible);
  }

  /**
   * Modifica os itens da enumerao. Remove os itens anteriores.
   * 
   * @param itemsAsText Os itens sob a forma de String (No aceita {@code null}
   *        ).
   * 
   * @throws ParseException Se o valor passado no estiver em um formato aceito
   *         por este parmetro.
   */
  private void setItemsAsText(String itemsAsText) throws ParseException {
    List<EnumerationItem> items = new ArrayList<EnumerationItem>();
    Matcher matcher = itemsPattern.matcher(itemsAsText);
    while (matcher.find()) {
      if (matcher.groupCount() == 5) {
        String id = matcher.group(1);
        String label = matcher.group(2);
        String value = matcher.group(3);
        String description = matcher.group(4);
        boolean visible = Boolean.valueOf(matcher.group(5));

        EnumerationItem item =
          new EnumerationItem(id, label, value, description);
        item.setVisible(visible);
        items.add(item);
      }
      else {
        throw new ParseException("No  possivel atribuir o valor\n({0}),"
          + "j que ele no representa uma codificao vlida para itens de "
          + "enumerao.\nParmetro envolvido: {1}.", new Object[] {
            matcher.group(0), this });
      }
    }
    setItems(items);
  }
}
