package csbase.logic.algorithms.parameters;

import csbase.exception.algorithms.ExpressionFunctionExecutionException;

import java.text.MessageFormat;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * Assinatura de um mtodo.
 *
 * @author lmoreira
 */
public abstract class ExpressionFunction {
  /** Nome do mtodo. */
  private String name;

  /** Array com os tipos dos parmetros. */
  private List<Class> parameterTypes;

/**
   * Cria uma assinatura.
   *
   * @param name O nome do mtodo.
   * @param parameterTypes O array com os tipos dos parmetros.
   */
  public ExpressionFunction(String name, Class... parameterTypes) {
    setName(name);
    setParameterTypes(parameterTypes);
  }

  public final boolean acceptParameters(Object... parameters) {
    if (parameters == null) {
      throw new IllegalArgumentException("O parmetro parameters est nulo.");
    }
    if (parameters.length != this.parameterTypes.size()) {
      return false;
    }
    for (int i = 0; i < this.parameterTypes.size(); i++) {
      Object parameter = parameters[i];
      Class expectedParameterType = this.parameterTypes.get(i);
      if (parameter == null) {
        if (expectedParameterType.isPrimitive()) {
          return false;
        }
      }
      else {
        Class<?extends Object> currentParameterType = parameter.getClass();
        if (currentParameterType.isPrimitive()) {
          currentParameterType = getWrapperType(currentParameterType);
        }
        if (!expectedParameterType.isAssignableFrom(currentParameterType)) {
          return false;
        }
      }
    }
    return true;
  }

  @Override
  public final boolean equals(Object other) {
    if (other == null) {
      return false;
    }
    if (!(other instanceof ExpressionFunction)) {
      return false;
    }
    ExpressionFunction function = (ExpressionFunction)other;
    if (!function.name.equals(this.name)) {
      return false;
    }
    if (function.parameterTypes.size() != this.parameterTypes.size()) {
      return false;
    }
    for (int i = 0; i < this.parameterTypes.size(); i++) {
      if (
        !this.parameterTypes.get(i)
        .isAssignableFrom(function.parameterTypes.get(i))) {
        return false;
      }
    }
    return true;
  }

  public final Object execute(
    SimpleAlgorithmConfigurator configurator, Object... parameters)
  throws ExpressionFunctionExecutionException {
    if (configurator == null) {
      throw new IllegalArgumentException("O parmetro configurator est nulo.");
    }
    if (parameters == null) {
      throw new IllegalArgumentException("O parmetro parameters est nulo.");
    }
    if (!acceptParameters(parameters)) {
      throw new ExpressionFunctionExecutionException(
        "A funo {0} no aceita os parmetros passados.\nParmetros passados: {1}.",
        toString(), Arrays.asList(parameters));
    }
    return doOperation(configurator, parameters);
  }

  /**
   * Obtm o nome do mtodo.
   *
   * @return O nome do mtodo.
   */
  public final String getName() {
    return this.name;
  }

  public final List<Class> getParameterTypes() {
    return Collections.unmodifiableList(this.parameterTypes);
  }

  @Override
  public final int hashCode() {
    return this.name.hashCode();
  }

  @Override
  public final String toString() {
    StringBuffer buffer = new StringBuffer();
    String separator = "";
    buffer.append(this.name);
    buffer.append("(");
    for (Class parameterType : this.parameterTypes) {
      buffer.append(separator);
      buffer.append(getName(parameterType));
      separator = ", ";
    }
    buffer.append(")");
    return buffer.toString();
  }

  protected abstract Object doOperation(
    SimpleAlgorithmConfigurator configurator, Object... parameters)
  throws ExpressionFunctionExecutionException;

  /**
   * Obtm o nome de uma classe.<br>
   * Quando temos array uni ou multidimensionais, este mtodo colocar os "[]"
   * adequados.
   *
   * @param typeClass A classe.
   *
   * @return O nome da classe.
   */
  private static String getName(Class typeClass) {
    if (typeClass.isArray()) {
      return getName(typeClass.getComponentType())+"[]";
    }
    return typeClass.getName();
  }

  private static Class getWrapperType(Class primitiveType) {
    if (boolean.class.equals(primitiveType)) {
      return Boolean.class;
    }
    if (byte.class.equals(primitiveType)) {
      return Byte.class;
    }
    if (char.class.equals(primitiveType)) {
      return Character.class;
    }
    if (double.class.equals(primitiveType)) {
      return Double.class;
    }
    if (float.class.equals(primitiveType)) {
      return Float.class;
    }
    if (int.class.equals(primitiveType)) {
      return Integer.class;
    }
    if (long.class.equals(primitiveType)) {
      return Long.class;
    }
    if (short.class.equals(primitiveType)) {
      return Short.class;
    }
    throw new IllegalArgumentException(
      "A classe informada no  um tipo primitivo.\nClasse: "+primitiveType);
  }

  /**
   * 
  DOCUMENT ME!
   *
   * @param name
   *
   * @throws IllegalArgumentException .
   */
  private void setName(String name) {
    if (name == null) {
      throw new IllegalArgumentException("O parmetro name est nulo.");
    }
    this.name = name;
  }

  /**
   * Ajusta o array com os tipos dos parmetros.
   *
   * @throws IllegalArgumentException .
   */
  private void setParameterTypes(Class... parameterTypes) {
    if (parameterTypes == null) {
      throw new IllegalArgumentException(
        "O parmetro parameterTypes est nulo.");
    }
    this.parameterTypes = new ArrayList<Class>(parameterTypes.length);
    for (Class parameterType : parameterTypes) {
      if (parameterType == null) {
        throw new IllegalArgumentException(
          "Parmetros nulos no so aceitos.\n"+Arrays.asList(parameterTypes));
      }
      if (parameterType.isPrimitive()) {
        parameterType = getWrapperType(parameterType);
      }
      this.parameterTypes.add(parameterType);
    }
  }
}
