/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.logic.algorithms.parameters;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import csbase.logic.algorithms.FileParameterValue;

/**
 * <p>
 * Parmetro do tipo lista de arquivo de entrada.
 * </p>
 * <p>
 * Ele possui 3 modos de funcionamento: aceitar apenas arquivos regulares,
 * aceitar apenas diretrios ou aceitar arquivos regulares e diretrios (veja
 * {@link FileParameterMode}).
 * </p>
 */
public final class InputFileListParameter extends
  SimpleParameter<List<FileParameterValue>> {
  /**
   * Chave utilizada para informar o <b>caminho</b> do arquivo no mapa de
   * parmetros dos mtodos de importao ({@link #importValue(Map)}) e
   * exportao ({@link #exportValue()}).
   */
  private static String PATH = ".path";

  /**
   * Chave utilizada para informar o <b>tipo</b> do arquivo no mapa de
   * parmetros dos mtodos de importao ({@link #importValue(Map)}) e
   * exportao ({@link #exportValue()}).
   */
  private static String TYPE = ".type";

  /**
   * O tipo de arquivo que este parmetro filtra. Se ele se importa com o tipo
   * do arquivo, ele valer {@code null}.
   */
  private String fileType;

  /**
   * Associao com os observador do parmetro do tipo arquivo.
   */
  private transient List<InputFileListParameterListener> listeners;

  /**
   * O modo de funcionamento atual (apenas arquivos, apenas diretrios ou
   * ambos).
   */
  private FileParameterMode mode;

  /**
   * Indica se os caminhos para os arquivos precisam ser ordenados.
   */
  private boolean mustSort;

  /**
   * Cria um parmetro do tipo arquivo.
   * 
   * @param name O nome deste parmetro (No aceita {@code null}).
   * @param label O rtulo deste parmetro (No aceita {@code null}).
   * @param description A descrio deste parmetro (No aceita {@code null}).
   * @param isOptional Indica se o valor do parmetro  opcional.
   * @param commandLinePattern O padro para construo da linha de comando. O
   *        padro ser utilizado para escrever o trecho da linha do comando
   *        referente ao parmetro. Esta string ser passada para o mtodo
   *        MessageFormat.format(String,Object...). O primeiro formato ({0}) 
   *        referente ao nome e o segundo formato ({1})  referente ao valor. Se
   *        {@code null} o parmetro no produzir sada na linha de comando.
   * @param fileType O tipo dos arquivos aceitos neste parmetro (Aceita {@code
   *        null}).
   * @param mode O modo de funcionamento deste parmetro (No aceita {@code
   *        null}).
   * @param mustSort Indica se os caminhos para os arquivos precisam ser
   *        ordenados.
   */
  public InputFileListParameter(String name, String label, String description,
    boolean isOptional, String commandLinePattern, String fileType,
    FileParameterMode mode, boolean mustSort) {
    super(name, label, description, null, isOptional, commandLinePattern);
    listeners = new LinkedList<InputFileListParameterListener>();
    this.fileType = fileType;
    this.mustSort = mustSort;
    setMode(mode);
  }

  /**
   * Adiciona um arquivo a este parmetro.
   * 
   * @param file O arquivo (No aceita {@code null}).
   * 
   * @return {@code true} se o arquivo foi adicionado ou {@code false} quando
   *         ele j estiver no conjunto de arquivos deste parmetro.
   */
  public boolean addFile(FileParameterValue file) {
    if (file == null) {
      throw new IllegalArgumentException("O parmetro file est nulo.");
    }
    List<FileParameterValue> oldFiles = getValue();
    List<FileParameterValue> newFiles = new ArrayList<FileParameterValue>();
    if (oldFiles != null) {
      newFiles.addAll(oldFiles);
    }
    if (newFiles.contains(file)) {
      return false;
    }
    newFiles.add(file);
    if (mustSort) {
      Collections.sort(newFiles);
    }
    setValue(newFiles);
    return true;
  }

  /**
   * Adiciona um observador de parmetros do tipo arquivo a este parmetro.
   * 
   * @param listener O observador (No aceita {@code null}).
   */
  public void addListener(InputFileListParameterListener listener) {
    if (listener == null) {
      throw new IllegalArgumentException("O parmetro listener est nulo.");
    }
    listeners.add(listener);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Map<String, Object> exportValue() {
    Map<String, Object> valuesByName = new HashMap<String, Object>();
    Collection<Map<String, String>> exportableValues =
      new LinkedList<Map<String, String>>();
    List<FileParameterValue> files = getValue();
    if (files != null) {
      for (FileParameterValue file : files) {
        Map<String, String> exportableValue = new HashMap<String, String>();
        exportableValue.put(PATH, file.getPath());
        exportableValue.put(TYPE, file.getType());
        exportableValues.add(Collections.unmodifiableMap(exportableValue));
      }
      valuesByName.put(getName(),
        Collections.unmodifiableCollection(exportableValues));
    }
    return Collections.unmodifiableMap(valuesByName);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String getCommandValue(String projectDirectory, String fileSeparator) {
    List<FileParameterValue> files = getValue();
    if (files == null) {
      return null;
    }
    if (files.isEmpty()) {
      return null;
    }
    String commandValue = "";
    String separator = "";
    for (FileParameterValue file : files) {
      commandValue += separator;
      if (file.getPath().startsWith(fileSeparator)) {
        commandValue += file.getPath();
      }
      else {
        String directory =
          getProjectRepositoryFromExecutionDirectory(fileSeparator);

        commandValue += directory + fileSeparator;
        commandValue += projectDirectory + fileSeparator;
        commandValue += file.getPath().replace("/", fileSeparator);
      }
      separator = ",";
    }
    return commandValue;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Object getExpressionValue() {
    return null;
  }

  /**
   * <p>
   * Obtm a lista de arquivos deste parmetro.
   * </p>
   * 
   * <p>
   * A lista retornada  imutvel (veja
   * {@link Collections#unmodifiableList(List)}).
   * </p>
   * 
   * <p>
   * Se no houver arquivos a lista retornado estar vazia.
   * </p>
   * 
   * <p>
   * Se o arquivo participar de uma conexo, a lista retornada estar vazia.
   * </p>
   * 
   * @return O conjunto de arquivos.
   */
  public List<FileParameterValue> getFiles() {
    List<FileParameterValue> files = getValue();
    if (files == null) {
      files = new ArrayList<FileParameterValue>();
    }
    return Collections.unmodifiableList(files);
  }

  /**
   * Obtm o tipo de arquivo que este parmetro aceita.
   * 
   * @return O tipo de arquivo ou {@code null} se ele no se importar com um
   *         tipo especfico.
   */
  public String getFileType() {
    return fileType;
  }

  /**
   * Obtm o modo de funcionamento (apenas arquivos, apenas diretrios ou ambos)
   * deste parmetro.
   * 
   * @return O modo de funcionamento.
   */
  public FileParameterMode getMode() {
    return mode;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String getType() {
    return "INPUT_FILE";
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String getValueAsText() {
    List<FileParameterValue> files = getFiles();
    if (files.isEmpty()) {
      return null;
    }
    String separator = "";
    String valueAsText = "";
    for (FileParameterValue file : files) {
      valueAsText += separator;
      valueAsText += file.getPath() + ":" + file.getType();
      separator = "|";
    }
    return valueAsText;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void importValue(Map<String, Object> parameterValues) {
    Collection<Map<String, String>> exportableValues =
      (Collection<Map<String, String>>) parameterValues.get(getName());
    if (exportableValues != null) {
      List<FileParameterValue> files = new ArrayList<FileParameterValue>();
      for (Map<String, String> exportableValue : exportableValues) {
        String path = exportableValue.get(PATH);
        String type = exportableValue.get(TYPE);
        FileParameterValue file = new FileParameterValue(path, type);
        if (!files.contains(file)) {
          files.add(file);
        }
      }
      if (files.isEmpty()) {
        setValue(null);
      }
      else {
        setValue(files);
      }
    }
  }

  /**
   * Indica se precisa ordenar a lista de arquivos.
   * 
   * @return true se precisa e false caso contrrio.
   */
  public boolean mustSort() {
    return mustSort;
  }

  /**
   * Move para cima o elemento do ndice
   * 
   * @param index ndice
   */
  public void moveUp(final int index) {
    final List<FileParameterValue> files = getValue();
    if (files == null) {
      return;
    }
    final int size = files.size();
    if (size == 0 || index == 0) {
      return;
    }

    final FileParameterValue element = files.remove(index);
    files.add(index - 1, element);
    fireValueWasChangedEvent();
  }

  /**
   * Move para cima o elemento do ndice
   * 
   * @param index ndice
   */
  public void moveDown(final int index) {
    final List<FileParameterValue> files = getValue();
    if (files == null) {
      return;
    }
    final int size = files.size();
    if (size == 0 || index == size - 1) {
      return;
    }

    final FileParameterValue element = files.remove(index);
    files.add(index + 1, element);
    fireValueWasChangedEvent();
  }

  /**
   * Remove um arquivo deste parmetro.
   * 
   * @param file O arquivo (No aceita {@code null}).
   * 
   * @return {@code true} se ele removeu o arquivo ou {@code false} se o arquivo
   *         no estiver neste parmetro.
   */
  public boolean removeFile(FileParameterValue file) {
    if (file == null) {
      throw new IllegalArgumentException("O parmetro file est nulo.");
    }
    List<FileParameterValue> files = getValue();
    if (files == null) {
      return false;
    }
    if (!files.remove(file)) {
      return false;
    }
    if (files.isEmpty()) {
      setValue(null);
    }
    else {
      fireValueWasChangedEvent();
    }
    return true;
  }

  /**
   * Remove um observador deste parmetro.
   * 
   * @param listener O observador (No aceita {@code null}).
   * 
   * @return {@code true} se ele removeu o observador ou {@code false} se o
   *         observador no estiver neste parmetro.
   */
  public boolean removeListener(InputFileListParameterListener listener) {
    if (listener == null) {
      throw new IllegalArgumentException("O parmetro listener est nulo.");
    }
    return listeners.remove(listener);
  }

  /**
   * Atribui um tipo de arquivo especfico a este parmetro.
   * 
   * @param fileType O tipo de arquivo (Aceita {@code null}).
   */
  public void setFileType(String fileType) {
    this.fileType = fileType;
    fireFileTypeWasChanged();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void setValueAsText(String parameterValue) {
    if ((parameterValue == null) || (parameterValue.length() == 0)) {
      setValue(null);
      return;
    }
    List<FileParameterValue> files = new ArrayList<FileParameterValue>();
    String[] components = parameterValue.split("\\|");
    for (String component : components) {
      FileParameterValue file;
      Pattern withTypePattern = Pattern.compile("^(.+):(.+)$");
      Matcher withTypeMatcher = withTypePattern.matcher(component);
      if (withTypeMatcher.matches()) {
        String path = withTypeMatcher.group(1);
        String type = withTypeMatcher.group(2);
        file = new FileParameterValue(path, type);
      }
      else {
        String path = parameterValue;
        file = new FileParameterValue(path);
      }
      files.add(file);
    }
    setValue(files);
  }

  /**
   * Dispara o evento
   * {@link InputFileListParameterListener#fileTypeWasChanged(InputFileListParameter)}
   * .
   */
  private void fireFileTypeWasChanged() {
    for (InputFileListParameterListener listener : listeners) {
      listener.fileTypeWasChanged(this);
    }
  }

  /**
   * <p>
   * Obtm o caminho do repositrio de projetos relativo ao diretrio de
   * execuo do comando.
   * </p>
   * 
   * <p>
   * Ateno: este mtodo  um workaround, ele ser removido, quando utilizarmos
   * um caminho absoluto para o repositrio de projetos, mas esta informao no
   * est e nem deve estar disponvel no cliente, logo quando for possvel criar
   * o configurador de algoritmos no servidor, este mtodo ir ser removido.
   * </p>
   * 
   * @param fileSeparator O separador de diretrios a ser utilizado (No aceita
   *        {@code null}).
   * 
   * @return O caminho.
   */
  private String getProjectRepositoryFromExecutionDirectory(String fileSeparator) {
    if (fileSeparator == null) {
      throw new IllegalArgumentException("O parmetro fileSeparator est nulo.");
    }
    String directory = "";
    String separator = "";
    for (int i = 0; i < 6; i++) {
      directory += separator;
      directory += "..";
      separator = fileSeparator;
    }
    return directory;
  }

  /**
   * Atribui o modo de funcionamento (apenas arquivo, apenas diretrio ou ambos)
   * deste parmetro.
   * 
   * @param mode O modo de funcionamento (No aceita {@code null}).
   */
  private void setMode(FileParameterMode mode) {
    if (mode == null) {
      throw new IllegalArgumentException("O parmetro mode est nulo.");
    }
    this.mode = mode;
  }

  /**
   * Cria os atributos transientes.
   * 
   * @param in
   * @throws IOException
   * @throws ClassNotFoundException
   */
  private void readObject(java.io.ObjectInputStream in) throws IOException,
    ClassNotFoundException {
    in.defaultReadObject();
    listeners = new LinkedList<InputFileListParameterListener>();
  }
}
