package csbase.logic.algorithms.parameters;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.Vector;

import tecgraf.javautils.FileUtils;
import csbase.exception.ParseException;
import csbase.exception.algorithms.ParameterNotFoundException;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.CommandLineContext;
import csbase.logic.algorithms.ExecutionType;
import csbase.logic.algorithms.FileParameterValue;
import csbase.logic.algorithms.parameters.triggers.Trigger;

/**
 * <p>
 * Configurador de algoritmos simples.
 * </p>
 * 
 * @author lmoreira
 */
public final class SimpleAlgorithmConfigurator extends AlgorithmConfigurator {

  /**
   * O nome do executvel.
   */
  private String commandBinaryName;

  /**
   * O nome do arquivo de entrada padro.
   */
  private String defaultInputFileParameterName;
  /**
   * As expresses de validao.
   */
  private Set<ValidationExpression> expressions;
  /**
   * Os grupos.
   */
  private List<ParameterGroup> groups;
  /**
   * Os parmetros ocultos.
   */
  private Set<HiddenParameter> hiddenParameters;
  /**
   * Indica se o configurador  editvel.
   */
  private boolean isEnabled;
  /**
   * Um flag que indica se as sadas padro e de erro devem ser exibidas.
   */
  private boolean showOutput;
  /**
   * O shell que ir executar o comando.
   */
  private String shell;
  /**
   * Os gatilhos que modificam propriedades deste configurador.
   */
  private Set<Trigger> triggers;

  /**
   * Cria um configurador de algoritmos simples.
   * 
   * @param algorithmVersion A verso do algoritmo (No aceita {@code null}).
   * @param description A descrio (Aceita {@code null}).
   * @param executionType O tipo de execuo (No aceita {@code null}).
   * @param abbreviation A abreviatura (Aceita {@code null}).
   * @param provideId Sinal que indica se  necessrio fornecer o identificador
   *        do comando na linha de comando.
   * @param command O nome do executvel (No aceita {@code null}).
   * @param shell O shell que ir executar o comando (Aceita {@code null}).
   * @param defaultInputFileParameterName O arquivo de entrada padro (Aceita
   *        {@code null}).
   * @param showOutput Um flag que indica se as sadas padro e de erro devem
   *        ser exibidas.
   * @param commandLinePattern pattern de linha de comando.
   */
  public SimpleAlgorithmConfigurator(AlgorithmVersionInfo algorithmVersion,
    String description, ExecutionType executionType, String abbreviation,
    boolean provideId, String command, String shell,
    String defaultInputFileParameterName, boolean showOutput, final String commandLinePattern) {
    super(algorithmVersion, description, executionType, abbreviation, provideId, commandLinePattern);
    this.isEnabled = true;
    this.expressions = new HashSet<ValidationExpression>();
    this.groups = new LinkedList<ParameterGroup>();
    this.hiddenParameters = new HashSet<HiddenParameter>();
    this.triggers = new HashSet<Trigger>();
    this.shell = shell;
    this.defaultInputFileParameterName = defaultInputFileParameterName;
    this.showOutput = showOutput;
    setCommandBinaryName(command);
  }

  /**
   * Adiciona uma expresso de validao a este configurador.
   * 
   * @param expression A expresso (No aceita {@code null}).
   * 
   * @return {@code true} sucesso ou {@code false} se no puder adicionar a
   *         expresso, porque existe uma expresso igual a esta j cadastrada.
   */
  public boolean addExpression(ValidationExpression expression) {
    if (expression == null) {
      throw new IllegalArgumentException("O parmetro expression est nulo.");
    }
    return this.expressions.add(expression);
  }

  /**
   * <p>
   * Adiciona um grupo a este configurador.
   * </p>
   * 
   * <p>
   * Mantm a ordem de insero.
   * </p>
   * 
   * @param group O grupo (No aceita {@code null}).
   * 
   * @return {@code true} sucesso ou {@code false} se no puder adicionar o
   *         grupo, porque existe um grupo igual a este j cadastrado.
   */
  public boolean addGroup(ParameterGroup group) {
    if (group == null) {
      throw new IllegalArgumentException("O parmetro group est nulo.");
    }
    if (this.groups.contains(group)) {
      return false;
    }
    this.groups.add(group);
    Iterator<?> simpleParameterIterator =
      group.getSimpleParameters().iterator();
    while (simpleParameterIterator.hasNext()) {
      SimpleParameter<?> parameter =
        (SimpleParameter<?>) simpleParameterIterator.next();
      parameter.addSimpleParameterListener(new SimpleParameterListener() {
        public void capabilityWasChanged(SimpleParameter parameter) {
          fireParameterWasSetEnabled(parameter.getName(), parameter.isEnabled());
        }

        public void defaultValueWasChanged(SimpleParameter parameter) {
        }

        public void labelWasChanged(SimpleParameter parameter) {
          fireParameterLabelWasChanged(parameter.getName(), parameter
            .getLabel());
        }

        public void valueWasChanged(SimpleParameter parameter) {
          fireParameterValueWasChanged(parameter.getName(), parameter
            .getValue());
        }

        public void visibilityWasChanged(SimpleParameter parameter) {
          fireParameterWasSetVisible(parameter.getName(), parameter.isVisible());
        }
      });
      parameter.addSimpleParameterListener(new SimpleParameterListener() {
        public void capabilityWasChanged(SimpleParameter parameter) {
        }

        public void defaultValueWasChanged(SimpleParameter parameter) {
        }

        public void labelWasChanged(SimpleParameter parameter) {
        }

        public void valueWasChanged(SimpleParameter parameter) {
          updateTriggers();
        }

        public void visibilityWasChanged(SimpleParameter parameter) {
        }
      });
    }
    return true;
  }

  /**
   * Adiciona um parmetro oculto a este configurador.
   * 
   * @param hiddenParameter O parmetro oculto (No aceita {@code null}).
   * 
   * @return {@code true} sucesso ou {@code false} se no puder adicionar o
   *         parmetro oculto, porque existe um parmetro oculto igual a este j
   *         cadastrado.
   */
  public boolean addHiddenParameter(HiddenParameter hiddenParameter) {
    if (hiddenParameter == null) {
      throw new IllegalArgumentException(
        "O parmetro hiddenParameter est nulo.");
    }
    return this.hiddenParameters.add(hiddenParameter);
  }

  /**
   * <p>
   * Adiciona um gatilho a este configurador.
   * </p>
   * 
   * <p>
   * Quando o gatilho  adicionado ele  atualizado, para que haja uma sincronia
   * entre o estado do configurador e o funcionamento do gatilho.
   * </p>
   * 
   * @param trigger O gatilho (No aceita {@code null}).
   * 
   * @return {@code true} sucesso ou {@code false} se no puder adicionar o
   *         gatilho, porque existe um gatilho igual a este j cadastrado.
   */
  public boolean addTrigger(Trigger trigger) {
    if (trigger == null) {
      throw new IllegalArgumentException("O parmetro trigger est nulo.");
    }
    if (!this.triggers.add(trigger)) {
      return false;
    }
    for (Trigger aTrigger : getTriggers()) {
      aTrigger.update(this);
    }
    return true;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Map<Object, Object> exportValues() {
    Map<Object, Object> parameterValues = new HashMap<Object, Object>();
    Iterator<ParameterGroup> groupIterator = getGroups().iterator();
    while (groupIterator.hasNext()) {
      ParameterGroup group = groupIterator.next();
      Map<String, Object> values = group.exportValue();
      for (String parameterKey : values.keySet()) {
        parameterValues.put(parameterKey, values.get(parameterKey));
      }
    }
    return Collections.unmodifiableMap(parameterValues);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Set<String> getBinaryDirectories(String platformId,
    String fileSeparator) {
    String binaryDirectory = getBinaryDirectory(platformId, fileSeparator);
    Set<String> binaryDirectories = new TreeSet<String>();
    binaryDirectories.add(binaryDirectory);
    return binaryDirectories;
  }

  /**
   * Obtm o nome do executvel.
   * 
   * @return O nome do executvel.
   */
  public String getCommandBinaryName() {
    return this.commandBinaryName;
  }

  /**
   * Obtm o nome do arquivo de entrada padro.
   * 
   * @return O nome do arquivo de entrada padro ou {@code null} se no houver
   *         um.
   */
  public String getDefaultInputFileParameterName() {
    return this.defaultInputFileParameterName;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Set<FileParameterValue> getDirectories() {
    Set<FileParameterValue> directories = new HashSet<FileParameterValue>();
    List<FileParameter> fileParameters = new LinkedList<FileParameter>();
    fileParameters.addAll(getInputFileParameters());
    fileParameters.addAll(getOutputFileParameters());
    for (FileParameter parameter : fileParameters) {
      FileParameterValue file = parameter.getValue();
      if (file != null && file.isDirectory()) {
        directories.add(file);
      }
    }
    for (InputFileListParameter inputFileListParameter : getInputFileListParameters()) {
      List<FileParameterValue> files = inputFileListParameter.getFiles();
      for (FileParameterValue file : files) {
        if (file.isDirectory()) {
          directories.add(file);
        }
      }
    }
    return Collections.unmodifiableSet(directories);
  }

  /**
   * <p>
   * Obtm o conjunto de expresses de validao associadas a este configurador.
   * </p>
   * 
   * <p>
   * O conjunto retornado  imutvel (veja
   * {@link Collections#unmodifiableSet(Set)}).
   * </p>
   * 
   * @return O conjunto de expresses ou um conjunto vazio se no houver
   *         expresses.
   */
  public Set<ValidationExpression> getExpressions() {
    return Collections.unmodifiableSet(this.expressions);
  }

  /**
   * <p>
   * Obtm a lista de grupos associadas a este configurador.
   * </p>
   * 
   * <p>
   * A lista retornada  imutvel (veja
   * {@link Collections#unmodifiableList(List)}).
   * </p>
   * 
   * @return A lista de grupos ou um conjunto vazio se no houver grupos.
   */
  public List<ParameterGroup> getGroups() {
    return Collections.unmodifiableList(this.groups);
  }

  /**
   * <p>
   * Obtm o conjunto de parmetros ocultos associados a este configurador.
   * </p>
   * 
   * <p>
   * O conjunto retornado  imutvel (veja
   * {@link Collections#unmodifiableSet(Set)}).
   * </p>
   * 
   * @return O conjunto de parmetros ocultos ou um conjunto vazio se no houver
   *         parmetros ocultos.
   */
  public Set<HiddenParameter> getHiddenParameters() {
    return Collections.unmodifiableSet(this.hiddenParameters);
  }

  public List<InputFileListParameter> getInputFileListParameters() {
    List<InputFileListParameter> inputFileListParameters =
      new LinkedList<InputFileListParameter>();
    Iterator<ParameterGroup> groupIterator = getGroups().iterator();
    while (groupIterator.hasNext()) {
      ParameterGroup group = groupIterator.next();
      inputFileListParameters.addAll(group.getInputFileListParameters());
    }
    return Collections.unmodifiableList(inputFileListParameters);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public List<InputFileParameter> getInputFileParameters() {
    List<InputFileParameter> inputFileParameters =
      new LinkedList<InputFileParameter>();
    Iterator<ParameterGroup> groupIterator = getGroups().iterator();
    while (groupIterator.hasNext()) {
      ParameterGroup group = groupIterator.next();
      inputFileParameters.addAll(group.getInputFileParameters());
    }
    return Collections.unmodifiableList(inputFileParameters);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Set<FileParameterValue> getInputFiles() {
    Set<FileParameterValue> inputFiles = new HashSet<FileParameterValue>();
    List<InputFileParameter> inputFileParameters = getInputFileParameters();
    for (InputFileParameter inputFileParameter : inputFileParameters) {
      FileParameterValue file = inputFileParameter.getValue();
      if (file != null && file.isRegularFile()) {
        inputFiles.add(file);
      }
    }
    for (InputFileListParameter inputFileListParameter : getInputFileListParameters()) {
      List<FileParameterValue> files = inputFileListParameter.getFiles();
      for (FileParameterValue file : files) {
        if (file.isRegularFile()) {
          inputFiles.add(file);
        }
      }
    }
    return Collections.unmodifiableSet(inputFiles);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public List<OutputFileParameter> getOutputFileParameters() {
    List<OutputFileParameter> outputFileParameters =
      new LinkedList<OutputFileParameter>();
    Iterator<ParameterGroup> groupIterator = getGroups().iterator();
    while (groupIterator.hasNext()) {
      ParameterGroup group = groupIterator.next();
      outputFileParameters.addAll(group.getOutputFileParameters());
    }
    return Collections.unmodifiableList(outputFileParameters);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Set<FileParameterValue> getOutputFiles() {
    Set<FileParameterValue> outputFiles = new HashSet<FileParameterValue>();
    List<OutputFileParameter> outputFileParameters = getOutputFileParameters();
    for (OutputFileParameter outputFileParameter : outputFileParameters) {
      if ((outputFileParameter.isVisible() || !outputFileParameter
        .ignoreIfInvisible())
        && (outputFileParameter.isEnabled() || !outputFileParameter
          .ignoreIfDisabled())) {
        FileParameterValue file = outputFileParameter.getValue();
        if (file != null && file.isRegularFile()) {
          outputFiles.add(file);
        }
      }
    }
    return Collections.unmodifiableSet(outputFiles);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String getParameterLabel(String parameterName)
    throws ParameterNotFoundException {
    if (parameterName == null) {
      throw new IllegalArgumentException("O parmetro parameterName est nulo.");
    }
    for (ParameterGroup group : getGroups()) {
      SimpleParameter<?> parameter = group.getSimpleParameter(parameterName);
      if (parameter != null) {
        return parameter.getLabel();
      }
    }
    throw new ParameterNotFoundException(parameterName, toString());
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Set<String> getParameterNames() {
    Set<String> parameterNames = new HashSet<String>();
    for (ParameterGroup group : getGroups()) {
      for (SimpleParameter<?> parameter : group.getSimpleParameters()) {
        parameterNames.add(parameter.getName());
      }
    }
    return Collections.unmodifiableSet(parameterNames);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String getParameterType(String parameterName)
    throws ParameterNotFoundException {
    if (parameterName == null) {
      throw new IllegalArgumentException("O parmetro parameterName est nulo.");
    }
    for (ParameterGroup group : getGroups()) {
      SimpleParameter<?> parameter = group.getSimpleParameter(parameterName);
      if (parameter != null) {
        return parameter.getType();
      }
    }
    throw new ParameterNotFoundException(parameterName, toString());
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String getParameterValue(String parameterName)
    throws ParameterNotFoundException {
    if (parameterName == null) {
      throw new IllegalArgumentException("O parmetro parameterName est nulo.");
    }
    for (ParameterGroup group : getGroups()) {
      SimpleParameter<?> parameter = group.getSimpleParameter(parameterName);
      if (parameter != null) {
        return parameter.getValueAsText();
      }
    }
    throw new ParameterNotFoundException(parameterName, toString());
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Set<String> getPlatforms() {
    Vector<?> platformVector = getAlgorithmVersion().getSupportedPlatforms();
    Set<String> platformSet = new HashSet<String>();
    for (Object platformAsObject : platformVector) {
      String platform = (String) platformAsObject;
      platformSet.add(platform);
    }
    return Collections.unmodifiableSet(platformSet);
  }

  /**
   * Obtm o shell que ir executar o comando deste configurador.
   * 
   * @return O shell ou {@code null} se ele no for informado.
   */
  public String getShell() {
    return this.shell;
  }

  /**
   * Obtm um parmetro simples dado o seu nome.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * 
   * @return O parmetro ou {@code null} se o parmetro no for encontrado.
   */
  public SimpleParameter<?> getSimpleParameter(String parameterName) {
    List<SimpleParameter<?>> parameters = getSimpleParameters();
    for (SimpleParameter<?> parameter : parameters) {
      if (parameter.getName().equals(parameterName)) {
        return parameter;
      }
    }
    return null;
  }

  /**
   * <p>
   * Obtm a lista de todos parmetros simples associados a este configurador de
   * algoritmos.
   * </p>
   * 
   * <p>
   * A lista retornada  imutvel (veja
   * {@link Collections#unmodifiableList(List)}).
   * </p>
   * 
   * @return A lista de parmetros ou uma lista vazia se no houver parmetros.
   */
  public List<SimpleParameter<?>> getSimpleParameters() {
    List<SimpleParameter<?>> parameters = new LinkedList<SimpleParameter<?>>();
    for (ParameterGroup group : getGroups()) {
      for (Parameter<?> parameter : group.getParameters()) {
        if (parameter instanceof SimpleParameter) {
          parameters.add((SimpleParameter<?>) parameter);
        }
      }
    }
    return Collections.unmodifiableList(parameters);
  }

  /**
   * <p>
   * Obtm o conjunto de gatilhos associados a este configurador.
   * </p>
   * 
   * <p>
   * O conjunto retornado  imutvel (veja
   * {@link Collections#unmodifiableSet(Set)}).
   * </p>
   * 
   * @return O conjunto de gatilhos ou um conjunto vazio se no houver gatilhos
   *         associados a este configurador.
   */
  public Set<Trigger> getTriggers() {
    return Collections.unmodifiableSet(this.triggers);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean isEnabled() {
    return isEnabled;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean isSetDefaultValues() {
    for (ParameterGroup group : getGroups()) {
      if (!group.isSetDefaultValue()) {
        return false;
      }
    }
    return true;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void importValues(Map<Object, Object> parameterValues) {
    if (parameterValues == null) {
      throw new IllegalArgumentException(
        "O parmetro parameterValues est nulo.");
    }
    Iterator<ParameterGroup> groupIterator = getGroups().iterator();
    while (groupIterator.hasNext()) {
      ParameterGroup group = groupIterator.next();
      group.importValue(parameterValues);
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String makeCommandLine(CommandLineContext context) {
    StringBuilder commandLineBuilder = new StringBuilder();
    commandLineBuilder.append("(");
    commandLineBuilder.append("cd ");
    String fileSeparator = context.getFileSeparator();
    StringBuilder binaryDirectoryBuilder = new StringBuilder();
    String serverDirectory = context.getServerDirectory();
    binaryDirectoryBuilder.append(serverDirectory);
    if (!serverDirectory.endsWith(fileSeparator)) {
      binaryDirectoryBuilder.append(fileSeparator);
    }
    binaryDirectoryBuilder.append(getBinaryDirectory(context.getPlatformId(),
      fileSeparator));
    commandLineBuilder.append(binaryDirectoryBuilder);
    commandLineBuilder.append("; (");

    if (getShell() != null) {
      commandLineBuilder.append(getShell());
      commandLineBuilder.append(" ");
    }
    commandLineBuilder.append('.');
    commandLineBuilder.append(fileSeparator);
    commandLineBuilder.append(getCommandBinaryName());
    String projectDirectory = context.getProjectDirectory();
    StringBuilder parameterListBuilder = new StringBuilder();
    Iterator<ParameterGroup> groupIterator = getGroups().iterator();
    String parameterSeparator = "";
    while (groupIterator.hasNext()) {
      ParameterGroup group = groupIterator.next();
      String groupCommandLine =
        group.getCommandLine(projectDirectory, fileSeparator);
      if (groupCommandLine != null) {
        parameterListBuilder.append(parameterSeparator);
        parameterListBuilder.append(groupCommandLine);
        parameterSeparator = " ";
      }
    }
    for (HiddenParameter parameter : getHiddenParameters()) {
      parameterListBuilder.append(parameterSeparator);
      parameterListBuilder.append(parameter.getCommandLine());
      parameterSeparator = " ";
    }
    String parameterList = parameterListBuilder.toString();
    if (parameterList.length() != 0) {
      commandLineBuilder.append(' ');
      commandLineBuilder.append(parameterList);
    }
    if (provideId()) {
      commandLineBuilder.append(' ');
      commandLineBuilder.append("cmdId=");
      commandLineBuilder.append(context.getCommandId());
    }
    if (isParallelExecution()) {
      commandLineBuilder.append(' ');
      commandLineBuilder.append("numCPU=");
      commandLineBuilder.append(context.getCPUCountForParallelExecution());
    }
    if (isDistributedExecution()) {
      commandLineBuilder.append(' ');
      commandLineBuilder.append("nexec=");
      commandLineBuilder.append(context
        .getExecutionCountPerSGAForDistributedExecution());
      commandLineBuilder.append(' ');
      commandLineBuilder.append("servidores=");
      String separator = "";
      for (String sgaName : context.getSGANamesForDistributedExecution()) {
        commandLineBuilder.append(separator);
        commandLineBuilder.append(sgaName);
        separator = ",";
      }
    }

    String standardOutputFile = null;
    if (getStandardOutputFile() != null) {
      // FIXME o separador deveria ser um char
      standardOutputFile =
        FileUtils.joinPath(fileSeparator.charAt(0), FileParameter
          .getProjectRepositoryFromExecutionDirectory(fileSeparator),
          projectDirectory, getStandardOutputFile().getPath());
    }
    if (showOutput) {
      if (standardOutputFile != null) {
        commandLineBuilder.append(" 2>&1");
        commandLineBuilder.append(getExitCodeCaptureCommand(context));
        commandLineBuilder.append(") | tee -a ");
        commandLineBuilder.append(standardOutputFile);
      }
      else {
        commandLineBuilder.append(getExitCodeCaptureCommand(context));
        commandLineBuilder.append(")");
      }
    }
    else {
      if (standardOutputFile != null) {
        commandLineBuilder.append(" >> ");
        commandLineBuilder.append(standardOutputFile);
        commandLineBuilder.append(" 2>&1");
      }
      else {
        commandLineBuilder.append(" >> /dev/null 2>&1");
      }
      commandLineBuilder.append(getExitCodeCaptureCommand(context));
      commandLineBuilder.append(")");
    }

    commandLineBuilder.append(")");
    return commandLineBuilder.toString();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void resetValues() {
    Iterator<ParameterGroup> parameterGroupIterator = getGroups().iterator();
    while (parameterGroupIterator.hasNext()) {
      ParameterGroup parameterGroup = parameterGroupIterator.next();
      parameterGroup.resetValue();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean setDefaultInputFile(FileParameterValue inputFile) {
    if (getDefaultInputFileParameterName() == null) {
      return false;
    }
    for (InputFileParameter inputFileParameter : getInputFileParameters()) {
      if (inputFileParameter.getName().equals(
        getDefaultInputFileParameterName())) {
        inputFileParameter.setValue(inputFile);
        return true;
      }
    }
    return false;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean setEnabled(boolean isEnabled) {
    if (this.isEnabled == isEnabled) {
      return false;
    }
    for (SimpleParameter<?> parameter : getSimpleParameters()) {
      parameter.setEnabled(isEnabled);
    }
    if (isEnabled) {
      updateTriggers();
    }
    this.isEnabled = isEnabled;
    fireWasSetEnabled();
    return true;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void setParameterValue(String parameterName, String parameterValue)
    throws ParseException, ParameterNotFoundException {
    if (parameterName == null) {
      throw new IllegalArgumentException("O parmetro parameterName est nulo.");
    }
    for (ParameterGroup group : getGroups()) {
      SimpleParameter<?> parameter = group.getSimpleParameter(parameterName);
      if (parameter != null) {
        parameter.setValueAsText(parameterValue);
        return;
      }
    }
    throw new ParameterNotFoundException(parameterName, toString());
  }

  /**
   * Atualiza os gatilhos cujas condies sejam aceitas.
   */
  public void updateTriggers() {
    for (Trigger trigger : this.triggers) {
      trigger.update(this);
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected String getCurrentParameterFileVersion() {
    return "1.0";
  }

  /**
   * Obtm o diretrio do binrio para um SGA especfico.
   * 
   * @param platformId O identificador da plataforma (No aceita {@code null}).
   * @param fileSeparator O separador de arquivo (No aceita {@code null}).
   * 
   * @return O diretrio do binrio.
   */
  private String getBinaryDirectory(String platformId, String fileSeparator) {
    AlgorithmVersionInfo versionInfo = getAlgorithmVersion();
    String algorithmRepositoryDirectoryName =
      versionInfo.getRepositoryDirName();
    String algorithmDirectoryName = versionInfo.getInfo().getDirectory();
    String versionsDirectoryName = versionInfo.getVersionsDirName();
    String versionDirectoryName = versionInfo.getDirectory();
    String binariesDirectoryName = versionInfo.getBinDirName();
    StringBuilder commandBuilder = new StringBuilder();
    commandBuilder.append(algorithmRepositoryDirectoryName);
    commandBuilder.append(fileSeparator);
    commandBuilder.append(algorithmDirectoryName);
    commandBuilder.append(fileSeparator);
    commandBuilder.append(versionsDirectoryName);
    commandBuilder.append(fileSeparator);
    commandBuilder.append(versionDirectoryName);
    commandBuilder.append(fileSeparator);
    commandBuilder.append(binariesDirectoryName);
    commandBuilder.append(fileSeparator);
    commandBuilder.append(platformId);
    String binaryDirectory = commandBuilder.toString();
    return binaryDirectory;
  }

  /**
   * Atribui o nome do executvel deste configurador.
   * 
   * @param command O nome do executvel (No aceita {@code null}).
   */
  private void setCommandBinaryName(String command) {
    if (command == null) {
      throw new IllegalArgumentException("O parmetro command est nulo.");
    }
    this.commandBinaryName = command;
  }
}
