package csbase.logic.algorithms.xapps;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.Vector;

import tecgraf.javautils.FileUtils;
import csbase.exception.ParseException;
import csbase.exception.algorithms.ParameterNotFoundException;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.CommandLineContext;
import csbase.logic.algorithms.CommandLineParameterFormatter;
import csbase.logic.algorithms.ExecutionType;
import csbase.logic.algorithms.FileParameterValue;
import csbase.logic.algorithms.parameters.FileParameter;
import csbase.logic.algorithms.parameters.FileParameterMode;
import csbase.logic.algorithms.parameters.FileParameterPipeAcceptance;
import csbase.logic.algorithms.parameters.InputFileParameter;
import csbase.logic.algorithms.parameters.OutputFileParameter;
import csbase.logic.algorithms.parameters.SimpleParameter;
import csbase.logic.algorithms.parameters.TextListParameter;
import csbase.logic.algorithms.parameters.TextParameter;

/**
 * Configurador de algoritmos para aplicao X.
 * 
 * @author lmoreira e ururahy
 */
public class XAppAlgorithmConfigurator extends AlgorithmConfigurator {

  /** Nome do script a ser executado no lanamento da aplicao. */
  private final String SCRIPT_NAME = "run.script";

  /** O parmetro display. */
  private TextParameter displayParameter;

  /**
   * Indica se o configurador est habilitado ( {@code true} ) ou desabilitado (
   * {@code false} ).
   */
  private boolean isEnabled;

  /** O parmetro que define os parmetros para a aplicao. */
  private TextListParameter paramsParameter;

  /** O parmetro log. */
  private OutputFileParameter logFileParameter;

  /**
   * O mapa de parmetros.<br>
   * A chave  o nome do parmetro e o valor  o prprio parmetro.
   */
  private Map<String, SimpleParameter<?>> parameters;

  /**
   * Cria o configurador.
   * 
   * @param algorithmVersion A verso do algoritmo (No aceita {@code null}).
   */
  public XAppAlgorithmConfigurator(AlgorithmVersionInfo algorithmVersion, final String commandLinePattern) {
    super(algorithmVersion, null, ExecutionType.XAPPS, "XAPPS", false, commandLinePattern);
    createParameters();
    setEnabled(true);
  }

  @Override
  public final Map<Object, Object> exportValues() {
    Map<Object, Object> parameterValues = new HashMap<Object, Object>();
    parameterValues.put(this.displayParameter.getName(), this.displayParameter
      .getValueAsText());
    parameterValues.put(this.paramsParameter.getName(), this.paramsParameter
      .getValueAsText());
    parameterValues.put(this.logFileParameter.getName(), this.logFileParameter
      .getValueAsText());
    return Collections.unmodifiableMap(parameterValues);
  }

  @Override
  public final Set<String> getBinaryDirectories(String platformId,
    String fileSeparator) {
    AlgorithmVersionInfo versionInfo = getAlgorithmVersion();
    String verDir = versionInfo.getDirectory();
    String algoDir = versionInfo.getInfo().getDirectory();
    String versDir = versionInfo.getVersionsDirName();
    String binDir = versionInfo.getBinDirName();
    StringBuffer binaryDirectoryText = new StringBuffer();
    String repDir = versionInfo.getRepositoryDirName();
    binaryDirectoryText.append(repDir);
    binaryDirectoryText.append(fileSeparator);
    binaryDirectoryText.append(algoDir);
    binaryDirectoryText.append(fileSeparator);
    binaryDirectoryText.append(versDir);
    binaryDirectoryText.append(fileSeparator);
    binaryDirectoryText.append(verDir);
    binaryDirectoryText.append(fileSeparator);
    binaryDirectoryText.append(binDir);
    binaryDirectoryText.append(fileSeparator);
    binaryDirectoryText.append(platformId);
    String command = binaryDirectoryText.toString();
    Set<String> binaryDirectories = new HashSet<String>();
    binaryDirectories.add(command);
    return binaryDirectories;
  }

  @Override
  public final Set<FileParameterValue> getDirectories() {
    return Collections.unmodifiableSet(new HashSet<FileParameterValue>());
  }

  /**
   * Obtm o parmetro "Display".
   * 
   * @return .
   */
  public final TextParameter getDisplayParameter() {
    return this.displayParameter;
  }

  @Override
  public final List<InputFileParameter> getInputFileParameters() {
    return Collections.unmodifiableList(new LinkedList<InputFileParameter>());
  }

  @Override
  public final Set<FileParameterValue> getInputFiles() {
    Set<FileParameterValue> inputFiles = new HashSet<FileParameterValue>();
    List<InputFileParameter> inputFileParameters = getInputFileParameters();
    for (InputFileParameter inputFileParameter : inputFileParameters) {
      FileParameterValue file = inputFileParameter.getValue();
      if ((file != null) && (file.isRegularFile())) {
        inputFiles.add(file);
      }
    }
    return Collections.unmodifiableSet(inputFiles);
  }

  @Override
  public final List<OutputFileParameter> getOutputFileParameters() {
    return Collections.unmodifiableList(Arrays.asList(this.logFileParameter));
  }

  @Override
  public final Set<FileParameterValue> getOutputFiles() {
    Set<FileParameterValue> outputFiles = new HashSet<FileParameterValue>();
    List<OutputFileParameter> outputFileParameters = getOutputFileParameters();
    for (OutputFileParameter outputFileParameter : outputFileParameters) {
      FileParameterValue file = outputFileParameter.getValue();
      if ((file != null) && (file.isRegularFile())) {
        outputFiles.add(file);
      }
    }
    return Collections.unmodifiableSet(outputFiles);
  }

  @Override
  public final String getParameterLabel(String parameterName)
    throws ParameterNotFoundException {
    if (parameterName == null) {
      throw new IllegalArgumentException("O parmetro parameterName est nulo.");
    }
    SimpleParameter<?> parameter = this.parameters.get(parameterName);
    if (parameter == null) {
      throw new ParameterNotFoundException(parameterName);
    }
    return parameter.getLabel();
  }

  @Override
  public final Set<String> getParameterNames() {
    Set<String> names = new TreeSet<String>();
    names.add(this.displayParameter.getName());
    names.add(this.logFileParameter.getName());
    return Collections.unmodifiableSet(names);
  }

  @Override
  public final String getParameterType(String parameterName)
    throws ParameterNotFoundException {
    if (parameterName == null) {
      throw new IllegalArgumentException("O parmetro parameterName est nulo.");
    }
    SimpleParameter<?> parameter = this.parameters.get(parameterName);
    if (parameter == null) {
      throw new ParameterNotFoundException(parameterName);
    }
    return parameter.getType();
  }

  @Override
  public final String getParameterValue(String parameterName)
    throws ParameterNotFoundException {
    if (parameterName == null) {
      throw new IllegalArgumentException("O parmetro parameterName est nulo.");
    }
    SimpleParameter<?> parameter = this.parameters.get(parameterName);
    if (parameter == null) {
      throw new ParameterNotFoundException(parameterName);
    }
    return parameter.getValueAsText();
  }

  /**
   * Obtm o parmetro "Parmetros".
   * 
   * @return .
   */
  public final TextListParameter getParamsParameter() {
    return this.paramsParameter;
  }

  @Override
  public final Set<String> getPlatforms() {
    Vector<?> platformVector = getAlgorithmVersion().getSupportedPlatforms();
    Set<String> platformSet = new HashSet<String>();
    for (Object platformAsObject : platformVector) {
      String platform = (String) platformAsObject;
      platformSet.add(platform);
    }
    return Collections.unmodifiableSet(platformSet);
  }

  @Override
  public final void importValues(Map<Object, Object> parameterValues)
    throws ParseException {
    this.displayParameter.setValueAsText((String) parameterValues
      .get(this.displayParameter.getName()));
    this.paramsParameter.setValueAsText((String) parameterValues
      .get(this.paramsParameter.getName()));
    this.logFileParameter.setValueAsText((String) parameterValues
      .get(this.logFileParameter.getName()));
  }

  @Override
  public final boolean isEnabled() {
    return isEnabled;
  }

  @Override
  public final boolean isSetDefaultValues() {
    for (SimpleParameter<?> parameter : this.parameters.values()) {
      Object currentValue = parameter.getValue();
      Object defaultValue = parameter.getDefaultValue();
      if (currentValue == null) {
        if (defaultValue != null) {
          return false;
        }
      }
      else if (!currentValue.equals(defaultValue)) {
        return false;
      }
    }
    return true;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  // XXX - Ver mountCommand em ./server/services/xappservice/XAppService.java
  public final String makeCommandLine(CommandLineContext context) {
    String fileSeparator = context.getFileSeparator();
    String platformId = context.getPlatformId();
    String serverDirectory = context.getServerDirectory();
    String projectDirectory = context.getProjectDirectory();
    String startDir;
    if (projectDirectory != null) {
      startDir =
        serverDirectory + fileSeparator + projectDirectory + fileSeparator;
    }
    else {
      startDir = new String("/tmp");
    }
    // XXX - Verificar se precisa mesmo e calcular!
    // Object userId = userId = new String("null");
    // Object prjId = new String("null");
    StringBuffer commStringBuffer = new StringBuffer();
    commStringBuffer.append("(");
    // Executa o comando a partir do diretrio do binrio do algoritmo.
    commStringBuffer.append("cd ");
    StringBuilder binaryDirectoryBuilder = new StringBuilder();
    binaryDirectoryBuilder.append(serverDirectory);
    if (!serverDirectory.endsWith(fileSeparator)) {
      binaryDirectoryBuilder.append(fileSeparator);
    }
    binaryDirectoryBuilder.append(getBinaryDirectoryPath(fileSeparator,
      platformId));
    commStringBuffer.append(binaryDirectoryBuilder);
    commStringBuffer.append("; ");

    String displayLine =
      displayParameter.getCommandLine(projectDirectory, fileSeparator);
    if (!displayLine.equals("")) {
      commStringBuffer.append("export ");
      commStringBuffer.append(displayLine);
      commStringBuffer.append("; ");
    }
    commStringBuffer.append("." + fileSeparator + SCRIPT_NAME + " ");
    commStringBuffer.append("--csbase-platform-id" + " " + platformId + " ");
    // commStringBuffer.append("--csbase-user-id"+" "+userId+" ");
    // commStringBuffer.append("--csbase-project-id"+" "+prjId+" ");
    commStringBuffer.append("--csbase-start-directory" + " " + startDir + " ");
    String params = getParametersCommand();
    if ((params != null) && !params.equals("")) {
      commStringBuffer.append(params);
    }
    String standardOutputFile = null;
    if (getStandardOutputFile() != null) {
      // FIXME o separador deveria ser um char
      standardOutputFile =
        FileUtils.joinPath(fileSeparator.charAt(0), FileParameter
          .getProjectRepositoryFromExecutionDirectory(fileSeparator),
          projectDirectory, getStandardOutputFile().getPath());
    }
    if (standardOutputFile != null) {
      commStringBuffer.append(" >> " + standardOutputFile + " 2>&1");
    }
    else {
      commStringBuffer.append(" >> /dev/null 2>&1");
    }
    commStringBuffer.append(')');
    return commStringBuffer.toString();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public final void resetValues() {
    for (SimpleParameter<?> parameter : this.parameters.values()) {
      parameter.resetValue();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public final boolean setDefaultInputFile(FileParameterValue inputFile) {
    return false;
  }

  /**
   * Modifica o display.
   * 
   * @param display O display (No aceita {@code null}).
   */
  public final void setDefaultDisplay(String display) {
    this.displayParameter.setDefaultValue(display);
    this.displayParameter.setValue(display);
  }

  @Override
  public final boolean setEnabled(boolean isEnabled) {
    if (isEnabled() == isEnabled) {
      return false;
    }
    for (SimpleParameter<?> parameter : parameters.values()) {
      parameter.setEnabled(isEnabled);
    }
    this.isEnabled = isEnabled;
    fireWasSetEnabled();
    return true;
  }

  @Override
  public final void setParameterValue(String parameterName,
    String parameterValue) throws ParseException, ParameterNotFoundException {
    if (parameterName == null) {
      throw new IllegalArgumentException("O parmetro parameterName est nulo.");
    }
    SimpleParameter<?> parameter = this.parameters.get(parameterName);
    if (parameter == null) {
      throw new ParameterNotFoundException(parameterName);
    }
    parameter.setValueAsText(parameterValue);
  }

  @Override
  public final void setParameterValuesByName(
    Map<String, String> parameterValuesByName) throws ParseException {
    if (parameterValuesByName == null) {
      throw new IllegalArgumentException(
        "O parmetro parameterValuesByName est nulo.");
    }
    this.displayParameter.setValueAsText(parameterValuesByName
      .get(this.displayParameter.getName()));
    this.paramsParameter.setValueAsText(parameterValuesByName
      .get(this.paramsParameter.getName()));
    this.logFileParameter.setValueAsText(parameterValuesByName
      .get(this.logFileParameter.getName()));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected final String getCurrentParameterFileVersion() {
    return "1.1";
  }

  /**
   * Cria os parmetros.
   */
  private void createParameters() {
    final String commandLinePattern = getCommandLinePattern();
    this.displayParameter =
      new TextParameter("DISPLAY", "Display", "Display", null, true,
        "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO", null);
    this.paramsParameter =
      new TextListParameter("PARAMS", "Parmetros",
        "Parmetros para a aplicao X", true, commandLinePattern, null);
    this.logFileParameter =
      new OutputFileParameter("LOG_FILE", "Arquivo de Log", "Arquivo de Log",
        true, commandLinePattern, "TEXT", FileParameterMode.REGULAR_FILE,
        FileParameterPipeAcceptance.FALSE, false);
    this.parameters = new HashMap<String, SimpleParameter<?>>();
    this.parameters.put(this.displayParameter.getName(), this.displayParameter);
    this.parameters.put(this.paramsParameter.getName(), this.paramsParameter);
    this.parameters.put(this.logFileParameter.getName(), this.logFileParameter);
  }

  /**
   * Obtm o caminho do diretrio do binrio que implementa o algoritmo deste
   * configurador.
   * 
   * @param fileSeparator O separador de arquivos.
   * @param platformId
   * 
   * @return O caminho do diretrio do binrio.
   */
  private String getBinaryDirectoryPath(String fileSeparator, String platformId) {
    AlgorithmVersionInfo versionInfo = getAlgorithmVersion();
    String verDir = versionInfo.getDirectory();
    String algoDir = versionInfo.getInfo().getDirectory();
    String versDir = versionInfo.getVersionsDirName();
    String binDir = versionInfo.getBinDirName();
    StringBuffer commandText = new StringBuffer();
    String repDir = versionInfo.getRepositoryDirName();
    commandText.append(repDir);
    commandText.append(fileSeparator);
    commandText.append(algoDir);
    commandText.append(fileSeparator);
    commandText.append(versDir);
    commandText.append(fileSeparator);
    commandText.append(verDir);
    commandText.append(fileSeparator);
    commandText.append(binDir);
    commandText.append(fileSeparator);
    commandText.append(platformId);
    return commandText.toString();
  }

  /**
   * Obtm o trecho da linha de comando criado pelos parmetros.
   * 
   * @return O trecho da linha de comando ou {@code null} se no houver
   *         parmetros definidos.
   */
  private String getParametersCommand() {
    List<String> items = this.paramsParameter.getValue();
    if (items == null) {
      return null;
    }
    StringBuffer commandText = new StringBuffer();
    Iterator<String> itemIterator = items.iterator();
    int i = 0;
    while (itemIterator.hasNext()) {
      String itemValue = itemIterator.next();
      if (itemValue != null && itemValue.length() > 0) {
        commandText.append("--param");
        commandText.append(i);
        commandText.append(' ');
        commandText.append(itemValue);
        commandText.append(' ');
      }
      i++;
    }
    return commandText.toString().trim();
  }
}
