package csbase.logic.filters;

import java.io.File;
import java.io.FileFilter;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

/**
 * Classe que permite realizar a busca de arquivos em vrios nveis da
 * hierarquia de diretrios. Para isto, realiza buscas sucessivas nos diretrios
 * e armazena o resultado final em uma lista.
 * 
 * @see LocalFileFinder#find(File)
 * 
 * @author Tecgraf
 */
public class LocalFileFinder implements Serializable {

  /**
   * Seleciona o tipo de busca que ser aplicado aos arquivos
   */
  private SearchType type;

  /**
   * Enumerao com os possveis tipos de busca (para fins de otimizao).
   */
  enum SearchType {

    /**
     * Nome do arquivo satisfaz a expresso regular
     */
    REGEX,

    /**
     * Nome do arquivo  igual ao texto pesquisado
     */
    EQUALS,

    /**
     * Nome do arquivo comea com o texto pesquisado
     */
    STARTS_WITH,

    /**
     * Nome do arquivo termina com o texto pesquisado
     */
    ENDS_WITH,

    /**
     * Busca todos os arquivos
     */
    ALL,
  }

  /**
   * Filtro utilizado nas buscas para descer atravs dos vrios nveis da
   * hierarquia a partir da raiz.
   */
  private LocalFileFilter filter;

  /**
   * Texto sendo pesquisado pelo usurio.
   */
  private String text;

  /**
   * Padro utilizado para representar o texto j compilado, este atributo faz
   * sentido somente se temos uma busca do tipo expresso regular.
   * 
   * @see SearchType#REGEX
   */
  private Pattern pattern;

  /**
   * Indica que NO devem ser consideradas diferenas de maisculas/minsculas
   */
  private final boolean isCaseInsensitive;

  /**
   * Indica que os arquivos de sistema devem ser ignorados.
   */
  private final boolean ignoreFileSystem;

  /**
   * Construtor.
   * 
   * @param inputText - texto informado pelo usurio que est sendo pesquisado.
   * @param isCaseInsensitive - Ignorar minsculas e maisculas.
   * @param isRegex - Indica que o texto  uma expresso regular.
   * @param ignoreFileSystem - Indica que os arquivos de sistema devem ser
   *        ignorados
   */
  public LocalFileFinder(final String inputText,
    final boolean isCaseInsensitive, final boolean isRegex,
    final boolean ignoreFileSystem) {

    this.text = inputText;
    this.isCaseInsensitive = isCaseInsensitive;
    this.ignoreFileSystem = ignoreFileSystem;

    if (isRegex) {
      // text  uma expresso regular 
      this.type = SearchType.REGEX;
    }
    else if (text.equals("*")) {
      // retornar todos os arquivos
      this.type = SearchType.ALL;
    }
    else if (text.lastIndexOf('*') == 0) {
      // *text -> qualquer arquivo que termina com o texto informado
      this.type = SearchType.ENDS_WITH;
      this.text = text.substring(1);
    }
    else if (text.lastIndexOf('*') == text.length()) {
      // text* -> qualquer arquivo que comea com o texto informado
      this.type = SearchType.STARTS_WITH;
      this.text = text.substring(0, text.length() - 1);
    }
    else if (text.contains("*")) {
      // mais de 1 ocorrncia de '*', usamos expresso regular para estes casos
      this.type = SearchType.REGEX;
      this.text = text.replace("*", ".*");
    }
    else {
      // nenhum '*', nome do arquivo deve ser idntico ao texto informado
      this.type = SearchType.EQUALS;
    }

    // se o tipo for uma expresso regular, criar/compilar o padro
    if (this.type == SearchType.REGEX) {
      int flags = 0;
      if (isCaseInsensitive) {
        flags = Pattern.CASE_INSENSITIVE;
      }
      try {
        this.pattern = Pattern.compile(this.text, flags);
      }
      catch (PatternSyntaxException e) {
        this.pattern = null;
      }
    }

    // nos demais casos, 'ignorar maisculas/minsculas'
    else if (isCaseInsensitive) {
      this.text = text.toUpperCase();
    }

  }

  /**
   * Pesquisa os arquivos aceitos pelo filtro (em vrios nveis da hierarquia) e
   * armazena a lista de resultado vlida no prprio filtro.
   * 
   * @param dir - diretrio a partir de onde a busca de arquivos ser feita
   */
  private void listFiles(File dir) {
    File[] dirList = dir.listFiles(filter);
    for (File d : dirList) {
      listFiles(d);
    }
  }

  /**
   * Pesquisa os arquivos que satisfazem os critrios da busca e retorna a lista
   * de arquivos obtida.
   * 
   * @param startDir - diretrio origem a partir de onde a busca ser feita
   * 
   * @return lista de arquivos vlidos
   */
  public List<File> find(File startDir) {
    this.filter = new LocalFileFilter(type);
    listFiles(startDir);
    return filter.getFileList();
  }

  /**
   * Filtro utilizado para descer em vrios nveis da hierarquia de diretrios e
   * acumular o resultado vlido.
   */
  private class LocalFileFilter implements FileFilter, Serializable {

    /**
     * Lista de arquivos aceitos pelo filtro
     */
    private List<File> fileList;

    /**
     * Tipo de busca sendo realizada
     */
    private SearchType type;

    /**
     * Construtor.
     * 
     * @param type - tipo de busca sendo realizada
     */
    LocalFileFilter(SearchType type) {
      this.type = type;
      this.fileList = new ArrayList<File>();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean accept(File file) {
      String fileName = file.getName();

      if (ignoreFileSystem && fileName.charAt(0) == '.') {
        return false;
      }

      if (isCaseInsensitive && type != SearchType.REGEX) {
        fileName = fileName.toUpperCase();
      }

      boolean isDirectory = file.isDirectory();
      if (!isDirectory) {

        boolean isValidFile;
        switch (type) {
          case STARTS_WITH:
            isValidFile = fileName.startsWith(text);
            break;

          case ENDS_WITH:
            isValidFile = fileName.endsWith(text);
            break;

          case REGEX:
            if (pattern != null) {
              isValidFile = pattern.matcher(fileName).matches();
            }
            else {
              isValidFile = false;
            }
            break;

          case ALL:
            isValidFile = true;
            break;

          case EQUALS:
            isValidFile = fileName.equals(text);
            break;

          default:
            isValidFile = false;
        }

        if (isValidFile)
          fileList.add(file);
      }

      return isDirectory;
    }

    /**
     * Obtm a lista de arquivos aceitos pelo filtro.
     * 
     * @return lista de arquivos
     */
    public List<File> getFileList() {
      return fileList;
    }

  }

}
