-- $Id: shellExtension.lua 105148 2010-05-06 14:10:25Z ururahy $

-------------------------------------------------------------------------------
-- Biblioteca de extensoes do Shell para executar algoritmos legados no CSBase.
--
-- ATENCAO: Todos os scripts .lua dos algoritmos dos sistemas (websintesi,
-- infogrid, csgrid, marlim) usam este arquivo. Alteracoes devem ser
-- comunicadas aos responsaveis dos projetos para que testes sejam feitos.
--
-- Autores: Leandro Pinto (lmoreira)
--          Taciana Melcop (taciana)
--          Andr Costa (costa)
--          Cristina Ururahy
-------------------------------------------------------------------------------

-- variveis globais
FILESEPARATOR_WIN = "\\"
FILESEPARATOR_UNIX = "/"
LINEBREAK_WIN = string.char(13,10) -- CR LF
SHELL_BASH = "bash"
SHELL_SH = "sh"
SHELL_KSH = "ksh"

fileSeparator = fileSeparator or FILESEPARATOR_UNIX

-- ------------------------------------------------------------
-- Obtem uma tabela contendo todos os parmetros da linha de comando.
--
-- Parametros:
-- command - A linha de comando.
-- name - O nome do parmetro.
--
-- Valores de retorno:
-- O valor do parametro. Se nao for encontrado, retornara nil.
-- Uma explicao textual sobre o erro ou nil (sucesso).

local _parameterCache = {}

function getParameterValue (command, name)
  local value = nil
  local errorMessage = nil

  -- ainda no existia entrada na cache para 'command'
  if not _parameterCache[command] then
    _parameterCache[command] = {}
    for _, param in pairs(command) do
      string.gsub(param, "(.-)=(.*)", function (name, val)
        _parameterCache[command][name] = val
      end)
    end
  end

  -- se j processamos este nome antes, retorna direto da cache
  if not _parameterCache[command][name] then
    errorMessage = "parametro "..name.." nao foi encontrado"
  end

  return _parameterCache[command][name], errorMessage
end

-- ============================================================
-- FUNES PARA MANIPULAO DE TABELAS

-- retorna uma nova tabela que  uma "shallow copy" de outra (i.e.
-- campos que sejam tabelas sero copiados por referncia)
local function _tableCopy (from)
  local to = {}
  for k, v in pairs(from) do
    to[k] = v
  end
  return to
end

-- ============================================================
-- FUNES PARA MANIPULAO DE STRINGS

local _CMD_DISCARD = " > /dev/null 2>&1"
local _CMD_ERR_DISCARD = " 2> /dev/null"

-- na ausncia do suporte  API POSIX, chdir() deve ser "simulado" a
-- cada chamada; para isto, implementamos o conceito de diretrio
-- corrente atravs de _CURRENT_DIR
local _CURRENT_DIR = "."..fileSeparator

-- iterador: retorna todas as linhas de um texto, uma de cada vez
function strLineIterator (text)
  -- garante que 'text' termine com \n, a menos que a string seja vazia.
  if string.match(text,"%S") and string.sub(text, -1, -1) ~= "\n" then
    text = text.."\n"
  end
  return coroutine.wrap(function ()
    for line in string.gmatch(text, "(.-)\n") do
      coroutine.yield(line)
    end
  end)
end

-- ------------------------------------------------------------
-- retorna uma cpia de 'str' aonde as ocorrncias de patt
-- foram trocadas por repl

function strReplaceAll (str, patt, repl)
  str, nchanges = string.gsub(str, patt, repl)
  while nchanges and nchanges > 0 do
    str, nchanges = string.gsub(str, patt, repl)
  end
  return str
end

-- ------------------------------------------------------------
-- retorna uma cpia de 'str' aonde as ocorrncias do char 'dupl' foram
-- removidas

function strRemoveDuplicates (str, dupl)
  return string.gsub(str, dupl.."+", dupl)
end

-- ------------------------------------------------------------
-- retorna a primeira linha de um texto

function strGetFirstLine (text)
  local i = string.find(text, "\n")
  return i and string.sub(text, 1, i-1) or text
end

-- ------------------------------------------------------------
-- retorna uma string que pode ser usada para comparaes independente
-- de maisculas/minsculas. p.ex.: abcdef --> [aA][bB][cC][dD][eE][fF]

function strCaseEscape (str)
  -- usamos um par extra de parnteses para forar apenas um valor de
  -- retorno
  return (string.gsub(str, "%a", function (c)
    return "["..string.lower(c)..string.upper(c).."]"
  end))
end

-- ------------------------------------------------------------
-- faz o escape de alguns dos caracteres especiais para regexps (p.ex.
-- "arq-saida.dat" --> "arq%-saida%.dat")

function regexpEscape (regexp)
  return string.gsub(regexp, "([-.%%*])", "%%%1")
end

-- ------------------------------------------------------------

function quoteEscape (str)
  return string.gsub(str, "\\", "\\\\")
end

-- ------------------------------------------------------------
-- substitui 'mark' em um path pelo separador de diretrios em uso
-- FIXME a rigor deveramos usar _regexpEscape(mark)...

local function _fixFileSep (path, mark, newSep)
  return string.gsub(path, mark, newSep or fileSeparator)
end

-- ============================================================
-- FUNES PARA MANIPULAO DE ARQUIVOS E DIRETRIOS

-- ------------------------------------------------------------
-- Separa um path na forma /aaa/bbb/ccc/file em dir + arquivo (no caso,
-- dir = /aaa/bbb/ccc/ e arquivo = file)
--
-- Parametros:
--   path : O caminho completo (incluindo o nome do arquivo)
--
-- OBS.: o ltimo componente do path  _sempre_ separado, independente
-- do path terminar com '/'. P.ex.: /aaa/bbb/ccc/ --> /aaa/bbb/ , ccc/
--
-- Valores de retorno:
--   diretorio
--   arquivo
--   mensagem de erro ou nil (sucesso)

function splitPath (path)
  local dir = ""
  local file = path
  local patt
  patt = _fixFileSep("^(.*/)(.+)$", "/")
  string.gsub(path, patt, function (p, f)
    dir, file = p, f
  end)
  return dir, file
end

-- ------------------------------------------------------------
-- Obtem o caminho para o projeto do arquivo fornecido.
--
-- Parametros:
-- caminho O caminho para o arquivo, no formato:
-- .../project/username/projname/...
--
-- Valores de retorno:
-- O caminho para o projeto (tudo at ...projname/) ou nil (erro)
-- restante do path ou msg de erro

function getProjectPath (path)
  local projectPath
  local remainder

  local patt = _fixFileSep("(.-project/.-/.-/.-)(.*)", "/")
  local _, nsubs = string.gsub(path, patt, function (projPath, rest)
    projectPath, remainder = projPath, rest
  end)
  if nsubs == 0 then
    return nil, "path de projeto invalido"
  end

  return projectPath, remainder
end

-- ------------------------------------------------------------
-- Transforma o caminho real para o arquivo em um caminho relativo para a
-- origem da arvore de projetos.
--
-- Parametros:
-- caminho O caminho para o arquivo.
--
-- Valores de retorno:
-- O caminho alterado ou nil se houver erro.
-- Uma mensagem de erro ou nil (sucesso).

function toRelativePath (path)
  local errorMessage
  local projectPath
  local relativePath
  projectPath, relativePath = getProjectPath(path)
  if not projectPath then
    -- neste caso, relativePath contm a mensagem de erro
    return nil, relativePath
  end
  return relativePath
end

-- ------------------------------------------------------------
-- Transforma uma parametro do tipo array de strings (String concatenada
-- com ',' as separando) para um array
--
-- Parametros:
-- valor O valor do parametro.
--
-- Valores de retorno:
-- A tabela com as strings.
--
-- OBS.: o primeiro ndice da tabela de resultados  UM

function toStringArray (valueList)
  local tableValue = {}

  for val in string.gmatch(valueList, "(.-),") do
    table.insert(tableValue, val)
  end

  -- temos que cobrir o caso em que a string no termina com ","
  string.gsub(valueList, "([^,]*[^,])$", function (last)
    table.insert(tableValue, last)
  end)

  return tableValue
end

-- ------------------------------------------------------------
-- Transforma uma parametro do tipo array de strings (String concatenada
-- com ',' as separando) para uma string com campos separados por \n
-- Assumimos que a string **no** termina com ","
--
-- Parametros:
-- str O valor do parametro.
-- newSeparator (opcional) outro valor para o "\n"
-- oldSeparator (opcional) outro valor para a ","
--
-- Valores de retorno:
-- uma string com campos separados por \n,
-- o nmero de campos que h na string.

function splitWithLineFeed (str, newSeparator, oldSeparator)
  oldSeparator = oldSeparator or ","
  newSeparator = newSeparator or "\n"
  local fields, num_fields = string.gsub(str,"(.-)"..oldSeparator,
    "%1"..newSeparator)
  num_fields = num_fields + 1
  return fields, num_fields
end

-- ------------------------------------------------------------
-- retorna 'path' prefixado por _CURRENT_DIR; as excees so para os
-- casos em que 'path'  absoluto ou comea com '~' -- neste caso, o
-- prprio path  retornado

local function _currDir (path)
  -- se path  absoluto, no altera
  if string.find(path, string.format("^[~%q].*", fileSeparator)) then
    return path
  else
    return shrinkDir(_CURRENT_DIR..fileSeparator..path)
  end
end

-- ------------------------------------------------------------
-- retorna true se 'file' existe
-- RETORNA: true = sucesso, false = erro

function fileExists (file)
  return (os.execute("test -f ".._currDir(file)) == 0)
end

-- ------------------------------------------------------------
-- retorna true se 'dir'  um diretrio

function isDir (dir)
  return (os.execute("test -d ".._currDir(dir)) == 0)
end

-- ------------------------------------------------------------
-- retorna true se 'dir' est vazio

function isEmptyDir (dir)
  local isEmpty  = os.execute("test `ls -a1 ".._currDir(dir).."| head -3 | wc -l` -lt 3")
  return tonumber(isEmpty) == 0
end

-- ------------------------------------------------------------
-- prefixa um comando com o chdir necessrio

local function _chdir (cmd)
  return "cd ".._CURRENT_DIR.."; "..cmd
end

-- ------------------------------------------------------------
-- executa um comando, antes indo para _CURRENT_DIR
-- RETORNA: true = sucesso, false = erro

local function _execute (cmd)
  return (os.execute(_chdir(cmd)) == 0)
end

-- ------------------------------------------------------------
-- cria um diretrio
-- RETORNA: true = sucesso, false = erro

function mkdir (dir)
  return _execute("mkdir "..dir.._CMD_DISCARD)
end

-- ------------------------------------------------------------
-- remove um diretrio
-- RETORNA: true = sucesso, false = erro

function rmdir (dir)
  if isDir(dir) then
    return _execute("rm -rf "..dir)
  end
  return nil
end

-- ------------------------------------------------------------
-- Em alguns casos a string que representa o diretrio corrente
-- comeou a ficar grande demais!
-- Troca os casos xx/yy/../ para xx/ e os casos // e /./ para /

function shrinkDir (dir)
  dir = strReplaceAll (dir, "([^/%.]+/%.%./)", "")
  dir = strReplaceAll (dir, "(/%.?/)", "/")
  return dir
end

-- ------------------------------------------------------------
-- altera o diretrio corrente. se o novo diretrio foi especificado com
-- path absoluto, este passa a ser o diretrio corrente; seno, o novo
-- diretrio  anexado ao fim do diretrio corrente

function chdir (dir)
  if string.find(dir, string.format("^[~%q].*", fileSeparator)) then
    _CURRENT_DIR = dir
  else
    _CURRENT_DIR = shrinkDir (_CURRENT_DIR..fileSeparator..dir)
  end
end

-- ------------------------------------------------------------
-- retorna o valor associado a uma varivel de ambiente 
-- No pode usar getenv de Lua pois no pega $$

function getenv (var)
  local result = io.popen("echo $"..var)
  local env = result:read()
  result:close()
  return env
end

-- ------------------------------------------------------------
-- copia arquivo 'from' para 'to'
-- RETORNA: true = sucesso, false = erro

function fileCopy (from, to)
  return _execute("cp "..from.." "..to)
end

-- ------------------------------------------------------------
-- move arquivo 'from' para 'to'
-- RETORNA: true = sucesso, false = erro

function fileMove (from, to)
  -- OBS.: no usamos os.rename() porque queremos tirar proveito dos
  -- wildcards (*, ? etc.) do shell
  return _execute("mv -f "..from.." "..to)
end

-- ------------------------------------------------------------
-- acrescenta texto ao final de um arquivo

function fileAppend (fileName, text)
  local fHandle, err = fileOpen(fileName, "a+")
  assert(fHandle, err)
  fHandle:write(text)
  fHandle:close()
end

-- ------------------------------------------------------------
-- remove arquivo
-- RETORNA: true = sucesso, false = erro

function fileRemove (name)
--  return (os.remove(_currDir(name)) ~= nil)
  return _execute("rm -f "..name)
end

-- ------------------------------------------------------------
-- abre um arquivo no modo especificado.
-- RETORNA: filehandle (sucesso) ou nil, erro (erro)

function fileOpen (name, mode)
  return io.open(_currDir(name), mode)
end

-- ------------------------------------------------------------
-- retorna:
--     o primeiro componente de um path
--     o resto do path (excluindo o separador)
-- Exemplos:
-- /aaa/bbb --> /aaa
-- aaa      --> aaa
-- aaa/bbb  --> aaa
-- /        --> /

local function _getFirstDirPart (path)
  -- no estamos interessados em lidar com repeties de '/'
  path = strRemoveDuplicates(path, fileSeparator)
  local repl = _fixFileSep("^(/?[^/].-)/.*", "/")
  p = string.gsub(path, repl, "%1", 1)
  local len = string.len(p)
  -- len+2 para no considerarmos o separador
  return p, string.sub(path, len+2)
end

-- ------------------------------------------------------------
-- retorna um path absoluto associado a um diretrio
-- ASSERTIVA: 'relPath'  um path terminado em '/'

function dirGetAbsolutePath (relPath)
  local absPathHandle = io.popen('bash -c "cd '..relPath..'; pwd"')
  local absPath = string.gsub(absPathHandle:read("*a"), "%s+$", "")
  absPathHandle:close()
  return absPath
end

function dirGetAbsolutePathShell (shell, relPath)
  local absPathHandle = io.popen('ksh -c "cd '..relPath..'; pwd"')
  local absPath = string.gsub(absPathHandle:read("*a"), "%s+$", "")
  absPathHandle:close()
  return absPath
end


-- ------------------------------------------------------------
-- retorna um path absoluto associado a um arquivo
-- ASSERTIVA: 'relPath'  um path para um ARQUIVO

function fileGetAbsolutePath (relPath)
  local dir, file = splitPath(relPath)
  return dirGetAbsolutePath(dir)..fileSeparator..file
end

-- ------------------------------------------------------------
-- substitui "/cygdrive/?/..." por "?:/...". retorna o novo path convertido
-- para o formato do Windows

function cygwin2win (cygPath)
  local winPath = string.gsub(cygPath, "^/cygdrive/(%a)", "%1:")
  return _fixFileSep(winPath, FILESEPARATOR_UNIX,
    quoteEscape(FILESEPARATOR_WIN))
end

-- ------------------------------------------------------------
-- substitui "/cygdrive/?/..." por "?:/...". retorna o novo path convertido
-- para o formato do UNIX

function cygwin2unix (cygPath)
  return string.gsub(cygPath, "^/cygdrive/(%a)", "%1:")
end

-- ------------------------------------------------------------
-- retorna apenas um arquivo/diretrio no dir corrente, um nome por
-- linha. Retorna uma string vazia quando nenhum arquivo casar com o
-- padro
-- ATENO: no lista arquivos cujos nomes comeam com "." 

function listOneFile (pattern)
  pattern = pattern or ""
  local dir, patt = splitPath(pattern)
  -- Retira a ltima barra do diretrio
  dir = string.gsub(dir, "(.+)\/$","%1")
  local removeDot = ""
  if dir == "" then
    dir = "."
    removeDot = "| cut -c3- "
  end
  local _, dirname = splitPath(dir)
  local findStr = " ( /usr/bin/find %s \\\! -name '%s' -prune -a -name  '%s' -print %s | head -1 ) "
  local findCmd = string.format(findStr, dir, dirname, patt, removeDot).._CMD_ERR_DISCARD
  local result = io.popen(_chdir(findCmd))
  local files = result:read("*a")
  result:close()
  return string.gsub(files, "(%w-)\n.*","%1") -- Pega apenas o primeiro
end

-- ------------------------------------------------------------
-- retorna lista dos arquivos/diretrios no dir corrente, um nome por
-- linha. Retorna uma string vazia quando nenhum arquivo casar com o
-- padro
-- ATENO: no lista arquivos cujos nomes comeam com "." 

function listFiles (pattern)
  pattern = pattern or ""
  local dir, patt = splitPath(pattern)
  -- Retira a ltima barra do diretrio
  dir = string.gsub(dir, "(.+)\/$","%1")
  local removeDot = ""
  if dir == "" then
    dir = "."
    removeDot = "| cut -c3- "
  end
  local _, dirname = splitPath(dir)
  local findStr = " ( /usr/bin/find %s \\\! -name '%s' -prune -a -name  '%s' -print %s ) "
  local findCmd = string.format(findStr, dir, dirname, patt, removeDot).._CMD_ERR_DISCARD
  local result = io.popen(_chdir(findCmd))
  local files = result:read("*a")
  result:close()
  return files
end

-- ------------------------------------------------------------
-- retorna lista dos diretrios no dir corrente, um nome por
-- linha. Exclui os diretrios que casam com o parmetro "excludePattern".
-- Este parmetro  case insensitive.
-- ATENO: no lista diretrios cujos nomes comeam com "." 

function listDirs (excludePattern)
  excludePattern = excludePattern or ""
  findStr = "/usr/bin/find . -maxdepth 1 -type d ! -name  \".*\" -prune | grep -iEv '%s' "
  local findCmd = string.format(findStr, excludePattern).._CMD_ERR_DISCARD
  local result = io.popen(_chdir(findCmd))
  local dirs = result:read("*a")
  result:close()
  return dirs
end

-- ------------------------------------------------------------
-- cria arquivo com tamanho 0 (ATENO: destri arquivos j existentes)

function emptyFile (name)
  local fileHandle = io.open(_currDir(name), "w")
  fileHandle:close()
end

-- ------------------------------------------------------------
-- Altera o ttulo da janela do shell que est executando o comando

function setWinTitle(title)
  if title and tostring(title) then
    print("\027];"..title.."\007")
  end
end

-- ------------------------------------------------------------
-- grava fileOut como cpia de fileIn aonde todas as ocorrncias de
-- "search" foram trocadas por "replace". "search" e "replace" sao fornecidos
-- aos pares na tabela

function fileReplaceTexts (fileIn, fileOut, tableArg)
  local fileOutHandle = io.open(_currDir(fileOut), "w")
  assert(fileOutHandle, "impossivel criar arquivo de saida "..fileOut)
  for line in io.lines(_currDir(fileIn)) do
    local newLine, count
    for i=1, #tableArg, 2 do
      newLine, count = string.gsub(line, tableArg[i], tableArg[i+1])
      if count > 0 then break end
    end
    fileOutHandle:write((newLine or line).."\n")
  end
  fileOutHandle:close()
end

-- ------------------------------------------------------------
-- grava fileOut como cpia de fileIn aonde todas as ocorrncias de
-- "search" foram trocadas por "replace". 

function fileReplaceText(fileIn, fileOut, search, replace)
  local searchReplace = {}
  table.insert(searchReplace, search)
  table.insert(searchReplace, replace)
  fileReplaceTexts(fileIn, fileOut, searchReplace)
end

-- ------------------------------------------------------------
-- executa comando 'cmd', opcionalmente direcionando sada para um
-- logFile
-- RETORNA: true = sucesso, false = erro

local _availableShells = { [SHELL_BASH] = true, [SHELL_SH] = true, 
  [SHELL_KSH] = true, }

function execCmd (shell, cmd, logFile)
  assert(_availableShells[shell], "shell "..shell.." nao suportado")

  local redirect = ""
  if logFile then
    redirect = " 2>&1 | tee "..logFile
  end
  -- no precisamos diferenciar bash de sh no que diz respeito 
  -- execuo de comandos via '-c'
  return _execute(shell..' -c "'..cmd..redirect..'"')
end

-- ------------------------------------------------------------
-- anloga a execCmd(), mas esta recebe tambm um nmero varivel de
-- argumentos que sero passados via stdin para o comando

function execCmdWithParams (shell, cmd, logFile, ...)
  -- arg  a tabela com os argumentos a serem passados para o comando
  -- via stdin
  assert(_availableShells[shell], "shell "..shell.." nao suportado")

  local redirect = ""
  if logFile then
    redirect = " 2>&1 | tee "..logFile
  end
  local stdinArgs = table.concat(arg, "\\n")

  if shell == SHELL_BASH then
    cmd = string.format("%s -c \"echo -e '%s' | %s %s\"", shell, stdinArgs, cmd, redirect)
  elseif shell == SHELL_SH or sheell == SHELL_KSH then
    cmd = string.format("%s -c \"echo '%s' | %s %s\"", shell, stdinArgs, cmd, redirect)
  end
  return _execute(cmd)
end

-- ------------------------------------------------------------
-- anloga a execCmd(), mas esta recebe uma varivel de ambiente e um nmero
-- varivel de argumentos que sero passados via stdin para o comando

function execCmdWithEnvVarParams (shell, envVar, cmd, logFile, ...)
  -- arg  a tabela com os argumentos a serem passados para o comando
  -- via stdin
  assert(_availableShells[shell], "shell "..shell.." nao suportado")

  local redirect = ""
  if logFile then
    redirect = " 2>&1 | tee "..logFile
  end
  local stdinArgs = table.concat(arg, "\\n")

  if shell == SHELL_BASH then
    cmd = string.format("%s -c \"export %s; echo -e '%s' | %s %s\"", shell, envVar, stdinArgs, cmd, redirect)
  elseif shell == SHELL_SH or sheell == SHELL_KSH then
    cmd = string.format("%s -c \"export %s; echo '%s' | %s %s\"", shell, envVar, stdinArgs, cmd, redirect)
  end
  return _execute(cmd)
end

-- executa uma linha de comando com os parametros dados
function execCmdLine (shell, cmd, logFile, ...)
  assert(_availableShells[shell], "shell "..shell.." nao suportado")
  local stdinArgs = table.concat(arg, " ")

  local redirect = ""
  if logFile then
    redirect = " 2>&1 | tee "..logFile
  end
  return _execute(shell..' -c "'..cmd.." "..stdinArgs.." "..redirect..'"')
end

-- ------------------------------------------------------------
-- para uma tabela com paths, retorna:
--     * prefixo comum a todos (ou "" caso no exista prefixo comum)
--     * nova tabela com paths correspondentes ao original, sem o prefixo
--       comum
-- Ex.: para paths = { "/aaa/bbb/ccc", "/aaa/bbb/ddd", "/aaa/bbb/eee/fff" }
--      o resultado seria:
--      "/aaa/bbb", { "ccc", "ddd", "eee/fff" }

function getCommonPath (paths)
  if (#paths==1) then
    local commonPath, rest = splitPath(paths[1])
    local restTable = {}
    table.insert(restTable, rest)
    return commonPath, restTable
  end
  local commonPath = ""
  -- newPaths  uma cpia _por valor_ de paths
  local newPaths = _tableCopy(paths)
  local referencePath = paths[1]
  repeat
    -- usamos sempre uma nova tabela; desperdia memria, mas simplifica
    -- processamento (isto garante que a cada iterao newPaths seja
    -- sempre vlida)
    local tempTable = {}
    -- pega o 1o componente do path de referncia
    local dir
    dir, referencePath = _getFirstDirPart(referencePath)
    local allMatch = true
    for i, path in ipairs(newPaths) do
      -- trataremos o primeiro path novamente, mas isso simplifica o loop
      local d, rest = _getFirstDirPart(path)
      if d ~= dir then
        -- o path em questo no compartilha o 1o componente com os
        -- demais; interrompemos o loop interno e sinalizamos para o
        -- loop externo
        allMatch = false
        break
      end
      -- ok, adiciona o path atual  tabela temporria
      tempTable[i] = rest
    end
    if not allMatch then
      -- algum dos paths no bateu
      break
    end
    -- todos os paths compartilhavam o 1o componente; "oficializa"
    -- tempTable como a nova newPaths
    if commonPath == "" then
        commonPath = dir
    else
        commonPath = commonPath..fileSeparator..dir
    end
    newPaths = tempTable
  until referencePath == ""
  -- commonPath pode estar comeando com '//'
  commonPath = strRemoveDuplicates(commonPath, fileSeparator)
--table.setn(newPaths, #paths)
  return commonPath, newPaths
end

-- ------------------------------------------------------------
-- dados um diretrio de referncia e um path para um arquivo em que
-- ambos compartilham um prefixo comum, retorna menor caminho relativo
-- para se chegar a 'baseDir' a partir do 'path'

function findRelativePath (baseDir, path)
    local commonPath, paths = getCommonPath { baseDir, path }
    local repl = _fixFileSep("[^/]+", "/")
    local relPath = string.gsub(paths[1], repl, "..") .. fileSeparator ..
        paths[2]
    relPath = strRemoveDuplicates(relPath, fileSeparator)
    -- o path resultante no pode ser absoluto; retiramos o '/' no
    -- comeo, caso este exista
    return string.gsub(relPath, "^/?(.*)$", "%1")
end

-- ------------------------------------------------------------
-- cria um arquivo temporrio atravs de mktemp; o path  opcional
-- retorna o nome do arquivo

function mktemp (path)
    if path then
        path = path.."/XXXXXX"
    else
        path = _currDir("XXXXXX")
    end
    local fileHandle = io.popen("mktemp -q "..path)
    -- temos que usar splitPath porque queremos retornar apenas o nome do
    -- arquivo
    local _, temp = splitPath(fileHandle:read("*a"))
    fileHandle:close()
    -- precisamos remover o \n no final do nome
    return (string.gsub(temp, "%s", ""))
end

-- ------------------------------------------------------------
-- captura de um path associado a um script Lua de um algoritmo o nome
-- do mesmo, a sua verso e a plataforma aonde ele ser executado. A
-- string deve obedecer o seguinte formato:
-- [...]/[NOME]/versions/[VERSO]/bin/[PLATAFORMA]/[NOME DO SCRIPT]
--
-- RETORNA: nome, verso e plataforma
--
-- FIXME assume que o path possui separadores no formato UNIX

function getAlgorithmInfo (path)
  local name, version, platform
  string.gsub(path, ".*/([^/]+)/versions/([^/]+)/bin/([^/]+)/?.*$",
    function (n, v, p)
      name, version, platform = n, v, p
    end)
  return name, version, platform
end

-- ---------------------------------------------------------------
-- Monta o caminho a partir do nome, versao e plataforma

function getAlgorithmPath(algo, version, platform)
  return "/"..algo.."/versions/"..version.."/bin/"..platform.."/"
end

-- ---------------------------------------------------------------
-- simula o 'grep': retorna as linhas do arquivo-texto que batem
-- com um determinado padro. Um terceiro parmetro (opcional) define se
-- apenas a primeira ocorrncia deve ser retornada.
-- Quando apenas um resultado foi encontrado, a linha e retornada. Para
-- multiplas linhas, os resultados so retornados como itens de uma tabela,
-- indexados de acordo com a ordem em que foram encontrados (i.e. result[1]
-- = 1a linha, result[2] = 2a linha etc.)
-- Se nenhuma ocorrencia for encontrada, retorna nil

function grep (arq, pattern, firstOnly)
  local result = {}
  for line in io.lines(_currDir(arq)) do
    if string.find(line, pattern) then
      -- removemos espaos no incio e no fim da linha
      local str = string.gsub(line, "%s*(.-)%s*$", "%1")
      table.insert(result, str)
      if firstOnly then return result[1] end
    end
  end
  if #result == 0 then return nil end
  return result 
end

-- ---------------------------------------------------------------
-- Escreve uma mensagem num arquivo dado

function writeMsg (fileName, message)
  local file = fileOpen(fileName, "a+")
  if file~= nil then
    file:write(message.."\n")
    file:close()
  end
end

-- ------------------------------------------------------------
-- Espera um certo numero de segundos
-- RECEBE: O numero de segundos que ira esperar

function sleep (sec)
  os.execute("sleep "..sec)
end

-- ------------------------------------------------------------------
-- Cria uma frmula matemtica a partir do valor codificado pelo
-- parmetro do tipo frmula.
--
-- Parmetros:
-- value - O valor codificado.
--
-- Valor de retorno:
-- A frmula.
-- ------------------------------------------------------------------
function getFormula(value)
  local replacements = {}

  replacements = {
    Space = ' ',
    Underline = '_',
    PlusSign = '+',
    MinusSign = '-',
    Slash = '/',
    Asterisk = '*',
    Percent = '%',
    Dot = '.',
    OpenRoundBracket = '(',
    CloseRoundBracket = ')',
    OpenSquareBracket = '[',
    CloseSquareBracket = ']',
    OpenCurlyBracket = '{',
    CloseCurlyBracket = '}',
    SemiColon = ';',
    LessThan = '<',
    LargeThan = '>',
    EqualsSign = '=',
    Caret = '^',
  }
  
  value = string.gsub(value, "__([%w]*)__", 
    function (key)
      local value = replacements[key]

      if value == nil then
        return "__" .. key .. "__"
      end

      return value
    end)
  return value
end

