package csbase.remote;

import java.io.File;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.Channel;

import csbase.exception.project.FileLockedException;
import csbase.logic.ProjectFileChannelLoadListener;

/**
 * Interface para leitura e escrita em arquivos remotos.
 */
public interface ProjectFileChannel {
  /**
   * Abre este arquivo. Se o arquivo for link ou estiver publicado, abre apenas
   * para leitura. Caso contrrio, abre de acordo com o parmetro dado.
   * 
   * @param readOnly True se o arquivo for aberto apenas para leitura, ou false,
   *        caso seja aberto para leitura e escrita.
   * 
   * @return Verdadeiro em caso de sucesso e falso caso a operao falhe.
   */
  boolean open(boolean readOnly);

  /**
   * Indica se este arquivo est aberto para leitura e escrita.
   * 
   * @return Verdadeiro caso este arquivo esteja aberto, falso caso contrrio.
   */
  public boolean isOpen();

  /**
   * Informa a posio corrente deste arquivo. Reflete o nmero de bytes a
   * partir do incio do arquivo at a posio corrente.
   * 
   * @return Posio corrente do arquivo.
   * 
   * @throws IOException se houver falha de I/O
   */
  long position() throws IOException;

  /**
   * Altera a posio corrente deste arquivo. Reflete o nmero de bytes a partir
   * do incio do arquivo at a posio corrente.
   * 
   * @param newPosition Nova posio corrente do arquivo.
   * 
   * @throws IOException se houver falha de I/O
   */
  void position(long newPosition) throws IOException;

  /**
   * L uma seqncia de bytes deste arquivo a partir da posio fornecida. Veja
   * a interface ReadableByteChannel para uma descrio completa da semntica
   * deste mtodo.
   * 
   * @param dst O buffer no qual os bytes sero escritos.
   * @param position A posio do arquivo a partir da qual os bytes sero lidos.
   * 
   * @return O nmero de bytes lidos ou -1 no caso de final de arquivo.
   * 
   * @throws IOException se houver falha de I/O
   */
  public int read(ByteBuffer dst, long position) throws IOException;

  /**
   * Escreve uma seqncia de bytes neste arquivo a partir da posio fornecida.
   * 
   * @param src O buffer no qual esto os bytes a serem escritos.
   * @param position A posio do arquivo a partir da qual os bytes sero
   *        escritos
   * 
   * @return O nmero de bytes escritos.
   * 
   * @throws IOException se houver falha de I/O
   * @throws FileLockedException caso o arquivo esteja bloqueado para escrita
   *         por outra sesso de usurio
   */
  public int write(ByteBuffer src, long position) throws IOException,
    FileLockedException;

  /**
   * Faz upload de um arquivo.
   * 
   * @param file o arquivo
   * @param chunkSize o tamnaho do chnk de envio de bytes.
   * @param listener o listener
   * @throws IOException se houver falha de I/O
   * @throws FileLockedException caso o arquivo esteja bloqueado para escrita
   *         por outra sesso de usurio
   */
  public void upload(final File file, final long chunkSize,
    final ProjectFileChannelLoadListener listener) throws IOException,
    FileLockedException;

  /**
   * Faz download de um arquivo.
   * 
   * @param file o arquivo
   * @param chunkSize o tamnaho do chnk de envio de bytes.
   * @param listener o listener
   * @throws IOException se houver falha de I/O
   * @throws FileLockedException caso o arquivo esteja bloqueado para escrita
   *         por outra sesso de usurio
   */
  public void download(final File file, final long chunkSize,
    final ProjectFileChannelLoadListener listener) throws IOException,
    FileLockedException;
  
  /**
   * Fecha este arquivo. Aps chamado este mtodo, o objeto que representa o
   * arquivo no poder mais ser utilizado para leitura e/ou escrita.
   * 
   * @throws IOException se houver falha de I/O
   * 
   * @see Channel#close
   */
  public void close() throws IOException;

  /**
   * Fecha este arquivo, sem enviar mais nada pelo canal. Aps chamado este 
   * mtodo, o objeto que representa o arquivo no poder mais ser utilizado 
   * para leitura e/ou escrita.
   * 
   * @throws IOException se houver falha de I/O
   */
  public void closeOnError() throws IOException;  
  
  /**
   * Consulta o tamanho do arquivo.
   * 
   * @return o tamanho.
   * 
   * @throws IOException se houver falha de I/O
   */
  public long size() throws IOException;

  /**
   * Ajusta do tamanho do arquivo.
   * 
   * @param newSize o novo tamanho.
   * 
   * @throws IOException se houver falha de I/O
   */
  public void truncate(long newSize) throws IOException;
}
