/*
 * $Id: ProjectFileChannelFromServer.java 92186 2009-06-05 19:34:07Z ururahy $
 */
package csbase.remote;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.SocketChannel;

import csbase.exception.PermissionException;
import csbase.exception.project.FileLockedException;
import csbase.logic.ProjectFileChannelLoadListener;

/**
 * Modela um canal de comunicao cliente/servidor via socket, para manipulao
 * de arquivos do projeto. Usa um protocolo padro para abrir, ler, escrever e
 * fechar um arquivo no servidor.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class ProjectFileChannelFromServer extends ProjectFileChannelProtocol
  implements ProjectFileChannel, Serializable {

  /**
   * Servidor
   */
  final private String server;

  /**
   * bytes
   */
  final private byte[] pathBytes;

  /**
   * Indicativo de arquivo normal.
   */
  final private boolean isRegularFile;

  /**
   * Porta
   */
  final private int port;

  /**
   * Tamanho do arquivo.
   */
  private long fileSize;

  /**
   * Indicatico R/O
   */
  final private boolean isReadOnly;

  /**
   * Hash da sesso.
   */
  final private byte[] sessionHashBytes;

  /**
   * 
   */
  final private static int LONG_SIZE = Long.SIZE / 8;

  /**
   * Nome do arquivo
   */
  private String fileName;

  /**
   * {@inheritDoc}
   */
  public synchronized boolean open(boolean readOnly) {
    if (this.isReadOnly && (readOnly == false)) {
      throw new PermissionException();
    }
    try {
      socketChannel = SocketChannel.open(new InetSocketAddress(server, port));
      socketChannel.configureBlocking(true);
      socket = socketChannel.socket();
      buffer = ByteBuffer.allocate(1024);
      params[0] = pathBytes.length;
      /*
       * se o canal deve ser aberto para escrita, precisamos passar alm do path
       * para o arquivo o hash da sesso que ser usado para validar a operao
       * de escrita. Neste caso, os parmetros so: 1: tamanho do path 2:
       * tamanho do hash 3: path 4: hash
       */
      if (isRegularFile) {
        if (readOnly) {
          opcode = ProjectFileChannelProtocol.OPEN_READ_ONLY;
        }
        else {
          opcode = ProjectFileChannelProtocol.OPEN_READ_WRITE;
          params[1] = sessionHashBytes.length;
        }
      }
      else {
        opcode = ProjectFileChannelProtocol.OPEN_READ_ONLY;
      }
      writeOperation();
      buffer.clear();
      buffer.put(pathBytes);
      if (opcode == ProjectFileChannelProtocol.OPEN_READ_WRITE) {
        buffer.put(sessionHashBytes);
      }
      writeBuffer();
      fileSize = size();
      return true;
    }
    catch (Exception e) {
      log("ProjectFileChannelFromServer:open: " + e);
    }
    return false;
  }

  /**
   * {@inheritDoc}
   */
  public synchronized boolean isOpen() {
    return ((socketChannel != null) ? socketChannel.isOpen() : false);
  }

  /**
   * {@inheritDoc}
   */
  public synchronized long size() throws IOException {
    opcode = ProjectFileChannelProtocol.GET_SIZE;
    writeOperation();
    readBuffer(LONG_SIZE);
    return buffer.getLong();
  }

  /**
   * {@inheritDoc}
   */
  public synchronized void truncate(long newSize) throws IOException {
    opcode = ProjectFileChannelProtocol.TRUNCATE;
    params[0] = newSize;
    writeOperation();
  }

  /**
   * {@inheritDoc}
   */
  public synchronized long position() throws IOException {
    opcode = ProjectFileChannelProtocol.GET_POSITION;
    writeOperation();
    readBuffer(LONG_SIZE);
    return buffer.getLong();
  }

  /**
   * {@inheritDoc}
   */
  public synchronized void position(long newPosition) throws IOException {
    opcode = ProjectFileChannelProtocol.SET_POSITION;
    params[0] = newPosition;
    writeOperation();
  }

  /**
   * {@inheritDoc}
   */
  public synchronized int read(ByteBuffer dst, long position)
    throws IOException {
    int count;
    int received = 0;
    long missing = fileSize - position;
    dst.clear();
    opcode = ProjectFileChannelProtocol.READ;
    params[0] = position;
    if (missing < dst.remaining()) {
      params[1] = missing;
    }
    else {
      params[1] = dst.limit() - dst.position();
    }
    writeOperation();
    while ((received < params[1]) && (dst.hasRemaining())) {
      count = socketChannel.read(dst);
      if (count == -1) {
        throw new IOException("ProjectServiceFromServer: read: count=-1");
      }
      received += count;
    }
    return received;
  }

  /**
   * {@inheritDoc}
   */
  public synchronized int write(ByteBuffer src, long position)
    throws IOException, FileLockedException {
    opcode = ProjectFileChannelProtocol.WRITE;
    params[0] = position;
    params[1] = src.limit() - src.position();
    writeOperation();
    readBufferAndCheckFileLocking();
    return socketChannel.write(src);
  }

  /**
   * {@inheritDoc}
   */
  public synchronized void upload(final File file, final long chunkSize,
    final ProjectFileChannelLoadListener listener) throws IOException,
    FileLockedException {
    final long size = file.length();
    opcode = ProjectFileChannelProtocol.UPLOAD;
    params[0] = size;
    writeOperation();

    readBufferAndCheckFileLocking();

    final FileInputStream inStream = new FileInputStream(file);
    final FileChannel fileChannel = inStream.getChannel();
    long sent = 0;
    long remaining = size;
    while (sent < size) {
      long transfered = fileChannel.transferTo(sent, chunkSize, socketChannel);
      sent += transfered;
      remaining -= transfered;
      listener.transferedBytes(sent, size, transfered);
    }
  }

  /**
   * {@inheritDoc}
   */
  public synchronized void download(final File file, final long chunkSize,
    final ProjectFileChannelLoadListener listener) throws IOException,
    FileLockedException {
    opcode = ProjectFileChannelProtocol.DOWNLOAD;
    writeOperation();
    
    /* O servidor responde com um cdigo numrico 
     * informando o tamanho da transferncia */
    readBuffer(LONG_SIZE);
    final long size = buffer.getLong();

    final FileOutputStream outStream = new FileOutputStream(file);
    final FileChannel fileChannel = outStream.getChannel();
    long received = 0;
    long remaining = size;
    while (received < size) {
      final long blockSize = chunkSize > remaining ? remaining : chunkSize;
      long transfered = fileChannel.transferFrom(socketChannel, received, blockSize);
      received += transfered;
      remaining -= transfered;
      listener.transferedBytes(received, size, transfered);
    }
  }

  /**
   * Faz leitura do buffer (aps envio de operao) e testa se a exceo de
   * locking de arquivos deve ser lanada.
   * 
   * @throws IOException se houver falha de I/O
   */
  private void readBufferAndCheckFileLocking() throws IOException {
    /* O servidor responde com um cdigo numrico informando se a operao 
     * possvel (p.ex. se o arquivo estiver bloqueado)
     */
    readBuffer(LONG_SIZE);
    long returnCode = buffer.getLong();

    // DEBUG
    // System.out.println("CHNL LOCK CLI: code = " + returnCode);
    if (returnCode == -1L) {
      throw new FileLockedException(fileName, !isRegularFile);
    }
  }

  /**
   * {@inheritDoc}
   */
  public synchronized void close() throws IOException {
    opcode = ProjectFileChannelProtocol.CLOSE;
    writeOperation();
    socketChannel.close();
    socket.close();
  }

  /**
   * {@inheritDoc}
   */
  public synchronized void closeOnError() throws IOException {
    socketChannel.close();
    socket.close();
  }  
  
  /**
   * {@inheritDoc}
   */
  @Override
  protected void log(String text) {
    System.out.println(text);
  }

  /**
   * Construtor
   * 
   * @param pathBytes path
   * @param isRegularFile indicativo de arquivo normal
   * @param server servidor
   * @param port porta
   * @param isReadOnly indicativo de RO
   * @param sessionHash sesso.
   * @param fileName nome do arquivo
   */
  public ProjectFileChannelFromServer(byte[] pathBytes, boolean isRegularFile,
    String server, int port, boolean isReadOnly, byte[] sessionHash,
    String fileName) {
    this.port = port;
    this.pathBytes = pathBytes;
    this.isRegularFile = isRegularFile;
    this.server = server;
    this.isReadOnly = isReadOnly;
    this.sessionHashBytes = sessionHash;
    this.fileName = fileName;
  }
}
