/**
 * $Id: ProjectFileChannelProtocol.java 83942 2008-10-07 21:20:53Z clinio $
 */

package csbase.remote;

import java.io.IOException;
import java.net.Socket;
import java.nio.ByteBuffer;
import java.nio.InvalidMarkException;
import java.nio.channels.SocketChannel;

import csbase.exception.BugException;

/**
 * Classe que define o protocolo do canal de comunicao do servio de projetos
 * (arquivos) com o servidor.
 * 
 * @author Tecgraf/PUC-Rio
 */
public abstract class ProjectFileChannelProtocol {
  protected static final byte OPEN_READ_WRITE = 0;
  protected static final byte OPEN_READ_ONLY = 1;
  protected static final byte READ = 2;
  protected static final byte WRITE = 3;
  protected static final byte GET_POSITION = 4;
  protected static final byte SET_POSITION = 5;
  protected static final byte GET_SIZE = 6;
  protected static final byte CLOSE = 7;
  protected static final byte TRUNCATE = 8;
  protected static final byte UPLOAD = 9;
  protected static final byte DOWNLOAD = 10;
  protected static final byte[] CODE_PARAMS = new byte[Byte.MAX_VALUE];
  protected static final byte MAX_PARAMS = 2;

  static {
    CODE_PARAMS[OPEN_READ_WRITE] = 2;
    CODE_PARAMS[OPEN_READ_ONLY] = 1;
    CODE_PARAMS[READ] = 2;
    CODE_PARAMS[WRITE] = 2;
    CODE_PARAMS[GET_POSITION] = 0;
    CODE_PARAMS[SET_POSITION] = 1;
    CODE_PARAMS[GET_SIZE] = 0;
    CODE_PARAMS[CLOSE] = 0;
    CODE_PARAMS[TRUNCATE] = 1;
    CODE_PARAMS[UPLOAD] = 1;
    CODE_PARAMS[DOWNLOAD] = 0;
  }

  /**
   * Socket
   */
  protected Socket socket;

  /**
   * Socket channel
   */
  protected SocketChannel socketChannel;

  /**
   * Buffer
   */
  protected ByteBuffer buffer;

  /**
   * Opcode
   */
  protected byte opcode;

  /**
   * Parmetros
   */
  protected long[] params;

  /**
   * Identificador
   */
  protected String id;

  /**
   * Assegura que o buffer tenha ao menos <code>count</code> bytes para serem
   * lidos. Se j no estiverem no buffer, ou s se s houver uma parte, l do
   * socket o que falta.
   * 
   * @param count bytes desejados.
   * 
   * @return nmero de bytes presentes no buffer, quase sempre maior ou igual ao
   *         nmero desejado de bytes. S ser menor (-1) em caso do socket ter
   *         sido fechado sem conter os bytes desejados.
   * 
   * @throws IOException se houver falha de I/O
   */
  protected int readBuffer(int count) throws IOException {
    try {
      int remaining = buffer.remaining();
      int missing = count - remaining;
      if (missing <= 0) {
        return remaining;
      }
      if (!buffer.hasRemaining()) {
        buffer.position(0);
        buffer.mark();
      }
      else {
        if (missing >= (buffer.capacity() - buffer.limit())) {
          buffer.compact();
        }
        else {
          buffer.mark();
          buffer.position(buffer.limit());
        }
      }
      buffer.limit(buffer.capacity());
      int received = 0;
      while (received < missing) {
        int length = socketChannel.read(buffer);
        if (length == 0) {
          // Pela interpretao atual da API, esse resultado no deveria nunca
          // ocorrer; como o socket  bloqueante, ele s deveria desbloquear
          // aps ler ao menos 1 byte. Este teste foi includo para tentar
          // localizar um bug que ocorre quando uma das threads 
          // SocketFileChannel no fecha, deixando a CPU em busy wait (99.9% 
          // de uso).
          String msg = "### BUG ### O read bloqueante saiu com 0 bytes lidos.";
          log(msg);
          throw new BugException(msg);
        }
        if (length == -1) {
          log("SocketFileChannel:readBuffer: erro no protocolo: "
            + "fim alcanado em " + received + " ao invs de " + count);
          return -1;
        }
        received += length;
      }
      buffer.limit(buffer.position());
      buffer.reset();
      return received + remaining;
    }
    catch (IOException e) {
      log("ProjectFileChannelProtocol:readBuffer: erro de I/O: " + e);
      throw e;
    }
    catch (InvalidMarkException e) {
      log("ProjectFileChannelProtocol:readBuffer: erro ao resetar posio no stream"
        + e);
      throw e;
    }
  }

  /**
   * Faz a escrita no buffer.
   * 
   * @throws IOException se houver falha de I/O
   */
  protected void writeBuffer() throws IOException {
    try {
      int count = 0;
      int length = buffer.position();
      buffer.flip();
      while ((count += socketChannel.write(buffer)) < length) {
        // Pela interpretao atual da API, teoricamente este mtodo deveria
        // bloquear at transmitir TODOS os bytes presentes no buffer. Este
        // log foi inserido para termos certeza de que isso ocorre na prtica.
        String msg =
          "### BUG ### Esperada escrita de " + length + " bytes." + "Apenas "
            + count + " foram escritos.";
        log(msg);
        throw new BugException(msg);
      }
    }
    catch (IOException e) {
      log("ProjectFileChannelProtocol:writeBuffer: erro de I/O: " + e);
      throw e;
    }
  }

  /**
   * L do buffer a prxima operao a ser executada. A operao  composta de
   * um opcode (obrigatrio) e um ou dois parmetros (opcionais). O resultado da
   * leitura  armazenado nas variveis internas <code>opcode</code> e
   * <code>params</code>.
   * 
   * @throws Exception se houver falha.
   */
  protected void readOperation() throws Exception {
    int received = readBuffer(1); // opcode
    if (received < 1) {
      log("ProjectFileChannelProtocol:readOperation: erro no protocolo: "
        + "opcode no recebido");
      opcode = CLOSE;
      return;
    }
    opcode = buffer.get();
    int num_params = CODE_PARAMS[opcode];
    if (num_params == 0) {
      return;
    }
    int params_size = num_params * 8; // num_params parmetros do tipo long
    int missing = (params_size + 1) - received;
    if (missing > 0) {
      received = readBuffer(params_size);
      if (received < params_size) {
        log("ProjectFileChannelProtocol:readOperation: erro no protocolo: "
          + "opcode " + opcode + ": parmetros incompletos: " + received
          + " bytes recebidos. Esperados " + params_size);
        opcode = CLOSE;
        return;
      }
    }
    for (int i = 0; i < num_params; i++) {
      params[i] = buffer.getLong();
    }
  }

  /**
   * Escreve no buffer o opcode da operao que vai ser executada, assim como
   * seus parmetros (opcionais).
   * 
   * @see #readOperation()
   * 
   * @throws IOException se houver falha de I/O
   */
  protected void writeOperation() throws IOException {
    buffer.clear();
    buffer.put(opcode);
    int num_params = CODE_PARAMS[opcode];
    for (int i = 0; i < num_params; i++) {
      buffer.putLong(params[i]);
    }
    writeBuffer();
  }

  /**
   * Registra uma mensagem no log.
   * 
   * @param text - mensagem
   */
  protected abstract void log(String text);

  /**
   * Construtor
   */
  protected ProjectFileChannelProtocol() {
    params = new long[MAX_PARAMS];
    id = "";
  }

  /**
   * Construtor
   * 
   * @param socket o socket
   */
  protected ProjectFileChannelProtocol(Socket socket) {
    this();
    this.socket = socket;
    this.socketChannel = socket.getChannel();
    this.buffer = ByteBuffer.allocate(1024);
    this.buffer.position(0);
    this.buffer.limit(0);
  }
}
