package csbase.remote;

import java.rmi.RemoteException;
import java.security.cert.Certificate;
import java.util.SortedSet;

import csbase.logic.trustedserver.TrustedServer;
import csbase.logic.trustedserver.TrustedServerInfo;

/**
 * Interface que define o comportamento do servio de servidores confiveis.
 *
 * @author Tecgraf/PUC-Rio
 */
public interface TrustedServerServiceInterface extends ServiceInterface, RemoteObservable {
  /** Nome do servio */
  public static final String SERVICE_NAME = "TrustedServerService";

  /**
   * Obtm um conjunto ordenado (por nome) com todos os servidores confiveis
   * pertencentes ao repositrio de servidores confiveis.
   *
   * @return Os servidores confiveis.
   *
   * @throws RemoteException Caso ocorra algum erro na comunicao com o
   *         servio.
   */
  public SortedSet<TrustedServer> getAllTrustedServers()
  throws RemoteException;

  /**
   * Adiciona um servidor confivel ao repositrio de servidores confiveis.
   *
   * @param trustedServerInfo As informaes para criar o servidor confivel a ser
   *        adicionado.
   *
   * @return true, caso o servidor confivel tenha sido adicionado, ou false, caso
   *         j exista um outro servidor confivel com o mesmo nome.
   *
   * @throws RemoteException Caso ocorra algum erro na comunicao com o
   *         servio.
   */
  public boolean addTrustedServer(TrustedServerInfo trustedServerInfo)
  throws RemoteException;

  /**
   * Remove um servidor confivel.
   *
   * @param trustedServer O servidor confivel a ser removido.
   *
   * @throws RemoteException Caso ocorra algum erro na comunicao com o
   *         servio.
   */
  public void removeTrustedServer(TrustedServer trustedServer)
  throws RemoteException;

  /**
   * Modifica um servidor confivel no repositrio de servidores confiveis.
   *
   * @param trustedServer O servidor confivel no seu estado antigo.
   * @param trustedServerInfo As informaes que sero usadas para alterar o
   *        servidor confivel.
   *
   * @return true, caso o servidor confivel tenha sido modificado, ou false, caso
   *         j exista um outro servidor confivel com o mesmo nome.
   *
   * @throws RemoteException Caso ocorra algum erro na comunicao com o
   *         servio..
   */
  public boolean modifyTrustedServer(
    TrustedServer trustedServer, TrustedServerInfo trustedServerInfo)
  throws RemoteException;
  
  /**
   * Adiciona um certificado ao servidor
   * 
   * @param serverName O nome do servidor
   * @param c A instncia do certificado
   * @return true se o certificado foi adicionado com sucesso, false caso
   *         contrrio
   * @throws RemoteException Caso ocorra algum erro na comunicao com o
   *         servio.
   */
  public boolean addCert(String serverName, Certificate c)
    throws RemoteException;

  /**
   * Remove o certificado do servidor
   * 
   * @param serverName O nome do servidor
   * @throws RemoteException Caso ocorra algum erro na comunicao com o
   *         servio.
   */
  public void removeCert(String serverName) throws RemoteException;
      
}
