/*
 * $Id: ServerSideProperties.java 96766 2009-09-28 17:49:23Z ururahy $
 */

package csbase.server;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Properties;

/**
 * Conjunto de propriedades de um servidor CSBASE.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class ServerSideProperties {

  /**
   * Armazenador de propriedades.
   */
  final private Properties properties = new Properties();

  /**
   * Indicativo de origem de carga de propriedades.
   */
  final private String name;

  /**
   * Flag de carregamento de propriedade.
   */
  private boolean isLoaded = false;

  /**
   * Strings convencionanda spara mencionar que a propriedade no  usada no
   * sistema; embora tenha que ser explicitamente ajustada.
   */
  static final private String[] NOT_USED = new String[] { "NULL" };

  /**
   * Strings convencionanda spara mencionar que a propriedade no  usada no
   * sistema; embora tenha que ser explicitamente ajustada.
   */
  static final private String[] BOOL_VALUES = new String[] { "true", "false" };

  /**
   * Verifica se uma propriedade foi definida com valor no-nulo ou sinalizada
   * como no usada no sistema.
   * 
   * @param value o valor da propriedade
   * @return indicativo
   * @see ServerSideProperties#NOT_USED
   */
  final static public boolean isPropertyValueNull(final String value) {
    for (final String str : NOT_USED) {
      if (value.equals(str.trim())) {
        return true;
      }
    }
    return false;
  }

  /**
   * Obtm um conjunto de propriedades com base em um arquivo definido pela
   * chave.
   * 
   * @param key a identificao da propriedade
   * @return o o conjunto da propriedade (que pode ser vazia, caso o arquivo no
   *         exista ou no esteja especificado).
   */
  protected final Properties getExternalPropertyFile(final String key) {
    final String fileName = getInternalProperty(key);
    if (isPropertyValueNull(fileName)) {
      final Properties empty = new Properties();
      return empty;
    }
    try {
      final Properties props = new Properties();
      final FileInputStream in = new FileInputStream(fileName);
      props.load(in);
      in.close();
      return props;
    }
    catch (IOException e) {
      throw new IllegalStateException(e);
    }
  }

  /**
   * Obtm uma propriedade do tipo boolean.
   * 
   * @param key a identificao da propriedade
   * @return o valor da propriedade.
   */
  protected final boolean getBooleanProperty(final String key) {
    final String value = getInternalProperty(key);
    if (!isPropertyBooleanValue(value)) {
      final String fmt = "[%s] - Propriedade (boolean) mal formatada: %s = %s";
      final String msg = String.format(fmt, name, key, value);
      throw new IllegalArgumentException(msg);
    }
    return (Boolean.valueOf(value)).booleanValue();
  }

  /**
   * Obtm uma propriedade do tipo double.
   * 
   * @param key a identificao da propriedade
   * @return o valor da propriedade.
   */
  protected final double getDoubleProperty(final String key) {
    final String value = getInternalProperty(key);
    try {
      return Double.parseDouble(value);
    }
    catch (final Exception e) {
      final String fmt = "[%s] - Propriedade (double) mal formatada: %s = %s";
      final String msg = String.format(fmt, name, key, value);
      throw new IllegalArgumentException(msg);
    }
  }

  /**
   * Consulta efetivamente a chave, levantando exceo se no houver.
   * 
   * @param key chave
   * @return valor
   */
  private String getInternalProperty(final String key) {
    final String value = properties.getProperty(key);
    if (value == null) {
      final String fmt = "[%s] - Propriedade indefinida detectada: %s";
      final String msg = String.format(fmt, name, key);
      throw new IllegalStateException(msg);
    }
    return value.trim();
  }

  /**
   * Obtm uma propriedade do tipo int.
   * 
   * @param key a identificao da propriedade
   * @return o valor da propriedade.
   */
  protected final int getIntProperty(final String key) {
    final String value = getInternalProperty(key);
    try {
      return Integer.parseInt(value);
    }
    catch (final Exception e) {
      final String fmt = "[%s] - Propriedade (int) mal formatada: %s = %s";
      final String msg = String.format(fmt, name, key, value);
      throw new IllegalArgumentException(msg);
    }
  }

  /**
   * Obtm uma lista de valores de propriedades.
   * <ul>
   * <li>propriedade.1=ABCD</li>
   * <li>propriedade.2=EFGH</li>
   * </ul>
   * <ul>
   * <li>propriedade.nome.1=ABCD</li>
   * <li>propriedade.nome.2=EFGH</li>
   * </ul>
   * 
   * @param keyPrefix O prefixo-nome da propriedade (nos exemplos acima, seriam
   *        "propriedade" e "propriedade.nome" respectivamente).
   * 
   * @return a lista com todas as propriedades (caso no existam propriedades, a
   *         lista estar vazia).
   */
  final protected List<String> getStringListProperty(final String keyPrefix) {
    final ArrayList<String> list = new ArrayList<String>();
    for (int i = 1; true; i++) {
      final String key = keyPrefix + "." + i;
      final boolean continueSearch = hasProperty(key);
      if (!continueSearch) {
        break;
      }
      final String value = getInternalProperty(key);
      list.add(value);
    }
    return Collections.unmodifiableList(list);
  }

  /**
   * Obtm uma propriedade
   * 
   * @param key a identificao da propriedade
   * @return o valor da propriedade como uma string (com <b>trim</b>).
   * @throws IllegalStateException se a propriedade no estiver setada.
   */
  protected final String getStringProperty(final String key) {
    return getInternalProperty(key);
  }

  /**
   * Consulta se uma propriedade existe.
   * 
   * @param key a identificao da propriedade
   * @return indicativo
   */
  protected final boolean hasProperty(final String key) {
    final String property = properties.getProperty(key);
    if (property == null) {
      return false;
    }
    return true;
  }

  /**
   * Verifica se uma propriedade foi definida com valor booleano vlido
   * 
   * @param value o valor da propriedade
   * @return indicativo
   * @see ServerSideProperties#BOOL_VALUES
   */
  final private boolean isPropertyBooleanValue(final String value) {
    for (final String str : BOOL_VALUES) {
      if (value.equals(str.trim())) {
        return true;
      }
    }
    return false;
  }

  /**
   * Verifica se uma propriedade foi definida com valor no-nulo ou sinalizada
   * como no usada no sistema.
   * 
   * @param key a identificao da propriedade
   * @return indicativo
   */
  protected final boolean isPropertyNull(final String key) {
    final String value = getInternalProperty(key);
    return isPropertyValueNull(value);
  }

  /**
   * Carregamento das propriedades do servio.
   * 
   * @param inStream stream de entrada
   * @throws ServerException se houver falha de carga de propriedades no
   *         servidor
   */
  public void load(final InputStream inStream) throws ServerException {
    if (inStream == null) {
      final String msg = "Carga de propriedade com stream invlido (nulo).";
      throw new ServerException(msg);
    }

    if (this.isLoaded) {
      final String msg = "Dupla carga de arquivo de propriedade:" + name;
      throw new ServerException(msg);
    }
    
    try {
      properties.load(inStream);
      inStream.close();
      this.isLoaded = true;
    }
    catch (final Exception e) {
      final String msg = "Falha carga de propriedade:" + name;
      throw new ServerException(msg, e);
    }
  }

  /**
   * Ajuste de propriedade
   * @param key chave
   * @param value valor
   */
  void overrideProperty(final String key, final String value) {
    properties.setProperty(key, value);
  }

  /**
   * Construtor
   * 
   * @param name nome.
   */
  public ServerSideProperties(final String name) {
    this.name = name;
  }

  /**
   * Construtor
   */
  ServerSideProperties() {
    this.name = "empty";
  }

}
