/*
 * $Id: Service.java 102155 2010-02-24 21:50:09Z pizzol $
 */
package csbase.server;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.Properties;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;
import java.util.logging.ConsoleHandler;
import java.util.logging.FileHandler;
import java.util.logging.Logger;

import tecgraf.javautils.FileUtils;
import csbase.logic.Permission;
import csbase.logic.SecureKey;
import csbase.logic.User;
import csbase.remote.RemoteObjectObservable;
import csbase.remote.ServiceInterface;
import csbase.server.services.loginservice.LoginService;

/**
 * Classe que implementa o framework para definio de um servio. Nem todos os
 * servios so servidores RMI: os que precisam de validao de usurio so
 * exportados por meio de proxies. Cada servio tem recursos de log e um
 * conjunto de propriedades especficos. Os nomes do arquivo de log e do arquivo
 * de propriedades so construdos a partir do nome do servio.
 * 
 * @author Tecgraf/PUC-Rio
 */
public abstract class Service extends RemoteObjectObservable implements
  ServiceInterface {

  /**
   * Constante que define o nome padro do arquivo de idiomas.
   */
  private final static String DEFAULT_LAGUANGE_FILE_NAME = "idiom";

  /**
   * Propriedade que define arquivos de idiomas adicionais
   */
  private final static String ADDITIONAL_LANGUAGE_FILE_PROPERTY = "additional.language.file";

  /**
   * Flag indicativo de servio executando.
   */
  private boolean isRunning = false;

  /**
   * Nome do arquivo de log a ser usado pelo servio.
   */
  private String serviceLogFileName = null;

  /**
   * Handler do log a ser usado pelo servio.
   */
  final private Logger serviceLogger;

  /**
   * Propriedades do servio carregadas pelo arquivo de propriedades.
   */
  private final ServerSideProperties serviceProperties;

  /**
   * Nome do servio
   */
  private String serviceName = null;

  /**
   * Instncias dos servios
   */
  private static Hashtable<String, Service> services = new Hashtable<String, Service>();

  /**
   * Bundles de internacionalizao (indexados por objetos locale)
   */
  private final Hashtable<Locale, ServiceResourceBundle> bundles = new Hashtable<Locale, ServiceResourceBundle>();

  /**
   * Dados locais da thread, utilizado exclusivamente para armazenar a sesso do
   * usurio que realizou a chamada remota.
   */
  private static ThreadLocal<Object> threadUserKey = new ThreadLocal<Object>();

  /**
   * Dados locais da thread, utilizado para armazenar a string que identifica o
   * sistema onde o usurio da sesso atual originou a chamada remota. O default
   *  ser nulo, e indica que o sistema  o prprio levantado pelo servidor.
   */
  private static ThreadLocal<String> threadSystemKey = new ThreadLocal<String>();

  /**
   * Classe de bundle dos servios. Existe apenas para possibilitar ajustar o
   * parent.
   */
  private class ServiceResourceBundle extends PropertyResourceBundle {

    /**
     * Construtor.
     * 
     * @param stream Arquivo de propriedades a ser lido
     * @throws IOException em caso de falha de I/O.
     */
    ServiceResourceBundle(InputStream stream) throws IOException {
      super(stream);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void setParent(ResourceBundle parent) {
      super.setParent(parent);
    }
  }

  /**
   * Guarda nos dados locais do thread corrente o identificador da sesso do
   * usurio. Este mtodo deve ser chamado pelos objetos remotos exportados pelo
   * servidor antes de repassar a chamada ao respectivo servio.
   * 
   * @param key a chave a ser guardada.
   */
  public static void setKey(final Object key) {
    threadUserKey.set(key);
  }

  /**
   * Guarda o usurio nos dados locais do thread corrente. Este mtodo  chamado
   * quando a chamada de um mtodo do servio no  feito pelo proxy de um
   * cliente, mas quando  feito, por exemplo, por outros servios, ou pelo WIO.
   * 
   * @param userId o identificador do usurio.
   */
  public static void setUserId(final Object userId) {
    threadUserKey.set(userId);
  }

  /**
   * Indica se a thread que origino a chamada deste mtodo veio internamente do
   * servidor.
   * 
   * @return true quando a chamda a este mtodo veio de uma thread interna do
   *         servidor. False quando a chamada  feita pelo cliente ou por outro
   *         servidor, ou seja, atravs dos proxies para os servios.
   */
  public static boolean isInternalServerRequest() {
    final Object localObject = threadUserKey.get();
    if (localObject instanceof SecureKey) {
      return false;
    }
    return true;
  }

  /**
   * Este mtodo retorna o usurio que realizou a chamada a um mtodo de um
   * servio. Para esse mecanismo funcionar, quando este mtodo  chamdo durante
   * a execuo de uma chamada remota,  preciso que cada objeto remoto
   * exportado pelo servidor chame o setKey antes de repassar a chamada para o
   * respectivo servio. Quando este mtodo  chamado a partir de outro servio,
   * para funcionar, o mtodo setUser deve ser chamado antes de fazer chamadas
   * para o servio.
   * 
   * @return o usurio registrado.
   */
  public static final User getUser() {
    final Object localObject = threadUserKey.get();
    if (localObject instanceof SecureKey) {
      return LoginService.getInstance().getUserByKey(localObject);
    }
    User loggedUser = null;
    try {
      loggedUser = User.getUser(localObject);
    }
    catch (Exception e) {
      Server.getInstance().logSevereMessage(
        "No foi possvel obter o usurio que solicitou a "
          + "chamada ao servio");
    }
    return loggedUser;
  }

  /**
   * Esse mtodo s pode ser chamado durante a execuo de uma chamada remota.
   * Nessa situao, este mtodo retorna o usurio que realizou a chamada. Para
   * esse mecanismo funcionar,  preciso que cada objeto remoto exportado pelo
   * servidor chame o setKey antes de repassar a chamada para o respectivo
   * servio.
   * 
   * @return a chave solicitada.
   */
  public static final Object getKey() {
    return threadUserKey.get();
  }

  /**
   * Carregamento de bundle de internacionalizao.
   * 
   * @param locale locale usado na traduo
   * @return um flag indicativo para o correto carregamento.
   * @see #bundles
   */
  private boolean loadLanguageBundle(final Locale locale) {
    List<String> languageFileList = new ArrayList<String>();
    languageFileList.add(DEFAULT_LAGUANGE_FILE_NAME);
    languageFileList
      .addAll(getStringListProperty(ADDITIONAL_LANGUAGE_FILE_PROPERTY));
    ResourceBundle parent = null;
    for (String filePath : languageFileList) {
      try {
        String idiomFileName = filePath + "_" + locale + ".properties";
        final InputStream in = getClass().getResourceAsStream(idiomFileName);
        final ServiceResourceBundle bundle = new ServiceResourceBundle(in);
        bundles.put(locale, bundle);
        logInfoMessage("Arquivo de bundle " + idiomFileName + " carregado.");
        bundle.setParent(parent);
        parent = bundle;
        in.close();
      }
      catch (Exception e) {
        final String msg = e.getMessage();
        logSevereMessage("Falha na leitura de: " + filePath + " --> " + msg);
        logSevereMessage("Falha no bundle de: " + locale);
        return false;
      }
    }
    return true;
  }

  /**
   * Retorno do locale padro de internacionalizao.
   * 
   * @return o locale.
   */
  final public Locale getDefaultLocale() {
    final Server server = Server.getInstance();
    return server.getDefaultLocale();
  }

  /**
   * Consulta do locale da thread.
   * 
   * @return o locale ou <code>null</code>.
   */
  final protected Locale getThreadLocale() {
    final Object key = getKey();
    if (key == null) {
      return null;
    }
    final LoginService lgService = LoginService.getInstance();
    final Locale locale = lgService.getUserSessionLocale(key);
    return locale;
  }

  /**
   * Obtm uma determinada permisso para o usurio que realizou a chamada ao
   * mtodo no servio.
   * 
   * @param <T> Tipo de permisso.
   * 
   * @param permissionClass classe que representa a permisso.
   * @return uma instncia da permisso.
   */
  final protected <T extends Permission> T getUserPermission(
    Class<T> permissionClass) {
    return getUser().getPermission(permissionClass);
  }

  /**
   * Traduz uma determinada chave na sua string formatada com base no locale.
   * 
   * @param key chave do bundle.
   * @param locale o locale desejado para intercionalizao.
   * @param objects array de objetos de formatao.
   * @return string com o texto respectivo a chave.
   */

  final public String getFormattedString(final String key,
    final Object[] objects, final Locale locale) {
    final String txt = getString(key, locale);
    final String msg = MessageFormat.format(txt, objects);
    return msg;
  }

  /**
   * Traduz uma determinada chave na sua string formatada inferindo o locale do
   * usurio (thread) que originou a chamada (ser usado um locale padro se
   * houver falha nesta identificao).
   * 
   * @param key chave do bundle.
   * @param objects array de objetos de formatao.
   * @return string com o texto respectivo a chave.
   */
  final public String getFormattedString(final String key,
    final Object[] objects) {
    final String txt = getString(key);
    final String msg = MessageFormat.format(txt, objects);
    return msg;
  }

  /**
   * Traduz uma determinada chave na sua string inferindo o locale do usurio
   * (thread) que originou a chamada (ser usado um locale padro se houver
   * falha nesta identificao).
   * 
   * @param key chave do bundle.
   * @return String com o texto respectivo a chave.
   */

  final public String getString(final String key) {
    Locale locale = getThreadLocale();
    if (locale == null) {
      locale = getDefaultLocale();
    }
    return getString(key, locale);
  }

  /**
   * Traduz uma determinada chave na sua string correspondente de acordo com o
   * locale desejado.
   * 
   * @param key chave do bundle.
   * @param locale o locale desejado para intercionalizao.
   * @return String com o texto respectivo a chave.
   */
  final public String getString(final String key, final Locale locale) {
    if (key == null) {
      logSevereMessage("Chave nula em consulta  internacionalizao");
      return "<<<null-key>>>";
    }

    if (locale == null) {
      logSevereMessage("Locale nulo em consulta  internacionalizao");
      return "<<<null-locale>>>";
    }

    if (!bundles.containsKey(locale)) {
      final boolean loaded = loadLanguageBundle(locale);
      if (!loaded) {
        return "<<<null-bundle:" + locale + ">>>";
      }
    }

    try {
      final ResourceBundle bnd = bundles.get(locale);
      return bnd.getString(key);
    }
    catch (MissingResourceException mre) {
      logSevereMessage("Texto no definido: " + key + ":" + locale);
      return "<<<" + key + ":" + locale + ">>>";
    }
    catch (Exception e) {
      final String txt = "Falha de internacionalizao";
      final String msg = e.getMessage();
      logSevereMessage(txt + " [" + key + ":" + locale + "] - " + msg);
      return "<<<" + key + ":" + locale + ">>>";
    }
  }

  /**
   * Carregamento das propriedades do servio.
   * 
   * @param fileName o path para o arquivo de propriedades a ser carregado.
   * @return as propriedades do servio
   * @throws ServerException em caso de falha na carga do servio.
   */
  private ServerSideProperties loadProperties(final String fileName)
    throws ServerException {
    try {
      final File file = new File(fileName);
      final FileInputStream stream = new FileInputStream(file);
      final ServerSideProperties prop = new ServerSideProperties(fileName);
      prop.load(stream);
      return prop;
    }
    catch (FileNotFoundException e) {
      final String msg = "No existe arquivo de propriedades:" + fileName;
      throw new ServerException(msg);
    }

  }

  /**
   * Inicializao do log de execuo do servio.
   * 
   * @return o logger do servio (criado no mtodo)
   * 
   * @throws ServerException se houver falha de inicializao.
   */
  private Logger initServiceLog() throws ServerException {
    try {
      final Logger logger = Logger.getLogger(serviceLogFileName);
      logger.setUseParentHandlers(false);
      final FileHandler serviceLogFileHandler = new FileHandler(
        serviceLogFileName);
      logger.addHandler(serviceLogFileHandler);
      /*
       * adiciona um handler para o console. O nvel de log default para ele 
       * definido no Logging.properties, e por default  SEVERE. Para
       * desabilitar o log para o console, basta mudar o nvel default para OFF.
       */
      logger.addHandler(new ConsoleHandler());
      return logger;
    }
    catch (Exception e) {
      throw new ServerException(e);
    }
  }

  /**
   * Retorna o nome prefixado da chave. Fora o uso de prefixo, mesmo que a
   * propriedade <code>"Server.service.prefixed.properties"</code> no esteja
   * ajustada para <code>true</code>.
   * 
   * @param key a chave
   * @return a chave com prefixo
   */
  final private String getPrefixedPropertyKey(final String key) {
    return getName() + "." + key;
  }

  /**
   * Faz checagem de propriedades corretamente carregadas.
   */
  private void checkPropertiesLoaded() {
    if (serviceProperties == null) {
      final String msg = "Falha interna de implementao do servio!\n\n"
        + "Propriedades do servio ainda no devidamente carregadas";
      throw new IllegalStateException(msg);
    }
  }

  /**
   * Verifica a existncia de uma propriedade
   * 
   * @param key a identificao da propriedade
   * 
   * @return indicativo
   */
  final public boolean isPropertyNull(final String key) {
    checkPropertiesLoaded();
    final String prefixedKey = getPrefixedPropertyKey(key);

    // Verificando se a propriedade foi redefinida pelo servidor (Server).
    final Server server = Server.getInstance();
    if (server.overridesServiceProperty(prefixedKey)) {
      // Se no existir no servidor, ser levantada exceo!
      final String value = server.getStringServiceProperty(prefixedKey);
      return ServerSideProperties.isPropertyValueNull(value);
    }

    // Se no existir no arquivo do servio, ser levantada exceo!
    return serviceProperties.isPropertyNull(prefixedKey);
  }

  /**
   * Obtm um conjunto de propriedades com base em um arquivo definido pela
   * chave.
   * 
   * @param key a identificao da propriedade
   * @return o o conjunto da propriedade (que pode ser vazia, caso o arquivo no
   *         exista ou no esteja especificado).
   */
  protected final Properties getExternalPropertyFile(final String key) {
    final String filePath = getStringProperty(key);
    if (ServerSideProperties.isPropertyValueNull(filePath)) {
      final Properties empty = new Properties();
      return empty;
    }

    try {
      final Properties props = new Properties();
      final FileInputStream in = new FileInputStream(filePath);
      props.load(in);
      in.close();
      return props;
    }
    catch (Exception e) {
      throw new IllegalStateException(e);
    }
  }

  /**
   * Recupera uma propriedade da aplicao (lista)
   * 
   * @param key a chave (prefixo)
   * @return os valores em forma de lista de strings.
   * @throws IllegalStateException se a propriedade no estiver setada.
   */
  public final List<String> getStringListProperty(final String key) {
    checkPropertiesLoaded();
    final Server server = Server.getInstance();
    final String prefixedKey = getPrefixedPropertyKey(key);
    final boolean overriden = server.overridesServiceProperty(prefixedKey
      .concat(".1"));
    if (overriden) {
      return server.getStringListServiceProperty(prefixedKey);
    }
    return serviceProperties.getStringListProperty(prefixedKey);
  }

  /**
   * Obtm uma propriedade do servio.
   * 
   * @param key a identificao da propriedade.
   * @return o valor da propriedade como uma string ou null (caso a propriedade
   *         no seja encontrada).
   */
  public final String getStringProperty(final String key) {
    checkPropertiesLoaded();
    final Server server = Server.getInstance();
    final String prefixedKey = getPrefixedPropertyKey(key);
    final boolean overriden = server.overridesServiceProperty(prefixedKey);
    if (overriden) {
      return server.getStringServiceProperty(prefixedKey);
    }
    return serviceProperties.getStringProperty(prefixedKey);
  }

  /**
   * Recupera uma propriedade da aplicao (double)
   * 
   * @param key a chave
   * @return o valor
   * @throws IllegalStateException se a propriedade no estiver setada.
   */
  public final double getDoubleProperty(final String key) {
    final Server server = Server.getInstance();
    final String prefixedKey = getPrefixedPropertyKey(key);
    final boolean overriden = server.overridesServiceProperty(prefixedKey);
    if (overriden) {
      return server.getDoubleServiceProperty(prefixedKey);
    }
    return serviceProperties.getDoubleProperty(prefixedKey);
  }

  /**
   * Recupera uma propriedade da aplicao (int)
   * 
   * @param key a chave
   * @return o valor
   * @throws IllegalStateException se a propriedade no estiver setada.
   */
  final public int getIntProperty(final String key) {
    final Server server = Server.getInstance();
    final String prefixedKey = getPrefixedPropertyKey(key);
    final boolean overriden = server.overridesServiceProperty(prefixedKey);
    if (overriden) {
      return server.getIntServiceProperty(prefixedKey);
    }
    return serviceProperties.getIntProperty(prefixedKey);
  }

  /**
   * Recupera uma propriedade da aplicao (long)
   * 
   * @param key a chave
   * @return o valor
   * @throws IllegalStateException se a propriedade no estiver setada.
   */
  final public boolean getBooleanProperty(final String key) {
    final Server server = Server.getInstance();
    final String prefixedKey = getPrefixedPropertyKey(key);
    final boolean overriden = server.overridesServiceProperty(prefixedKey);
    if (overriden) {
      return server.getBooleanServiceProperty(prefixedKey);
    }
    return serviceProperties.getBooleanProperty(prefixedKey);
  }

  /**
   * Obtm o formato de caminho adequado para o sistema operacional em uso, a
   * partir de 3 tipos possveis de formatos de propriedades indicando caminhos
   * para arquivos:
   * <ol>
   * <li><b>Formato Unix relativo<b>: os separadores "/" sero substitudos por
   * <code>File.separator</code> (recomendado)</li>
   * <li><b>Formato Unix absoluto<b>: o caminho ser retornado sem alteraes e
   * s funcionar em sistemas <i>UNIX-LIKE</i></li>
   * <li><b>Outros formatos<b>: o caminho ser retornado sem alteraes e s 
   * garantido de funcionar no sistema para o qual foi criado</li>
   * </ol>
   * 
   * @param propertyPath caminho para um arquivo.
   * 
   * @return caminho no formato do S.O. corrente (usando
   *         <code>File.separator</code>) para caminhos Unix relativos; retorna
   *         o caminho original caso este esteja em outros formatos.
   */
  final protected String getOSPropertyPath(final String propertyPath) {
    final String UNIX_SEPARATOR = "/";
    if (propertyPath.indexOf(UNIX_SEPARATOR) < 1) {
      return propertyPath;
    }
    String[] splittedPath = FileUtils.splitPath(propertyPath, UNIX_SEPARATOR);
    String osPath = FileUtils.joinPath(splittedPath);
    if (propertyPath.lastIndexOf(UNIX_SEPARATOR) == propertyPath.length() - 1) {
      return osPath + File.separator;
    }
    return osPath;
  }

  /**
   * Mtodo de consulta do nome do servio
   * 
   * @return o nome.
   */
  public final String getName() {
    return serviceName;
  }

  /**
   * Mtodo para obteno de todos os servios.
   * 
   * @return Hashtable com as referncias para todos os servios.
   */
  public static Hashtable<String, Service> getServices() {
    return services;
  }

  /**
   * Faz checagem de inicializao do log quando usa-se uma string para seu uso.
   * 
   * @param msg a string a ser loggada.
   * @throws RuntimeException se o log no estiver ainda inicializado.
   */
  private void checkLogInited(final String msg) {
    if (serviceLogger == null) {
      final String fmt = "Erro interno no servio %s por";
      final String err1 = String.format(fmt, getName());
      final String err2 = "uso de log antes de sua incializao!";
      final String txt = "Mensagem original: [" + msg + "]";
      final String message = err1 + " " + err2 + " - " + txt;
      throw new RuntimeException(message);
    }
  }

  /**
   * Gerao de log de informao.
   * 
   * @param msg a mensagem a ser gerada.
   * 
   * @see #serviceLogger
   */
  public final void logInfoMessage(final String msg) {
    checkLogInited(msg);

    final String message = ServerLogFormatter.getQualifiedMessage(msg);
    final ServerLogRecord slr = new ServerLogRecord(ServerLevel.INFO, message,
      getUser());
    serviceLogger.log(slr);
  }

  /**
   * Gerao de log de mensagem de alerta.
   * 
   * @param msg a mensagem a ser gerada.
   * 
   * @see #serviceLogger
   */
  public final void logWarningMessage(final String msg) {
    checkLogInited(msg);

    final String message = ServerLogFormatter.getQualifiedMessage(msg);
    ServerLogRecord slr = new ServerLogRecord(ServerLevel.WARNING, message,
      getUser());
    serviceLogger.log(slr);
  }

  /**
   * Gerao de log de mensagem de alerta.
   * 
   * @param pattern Um padro para formatar a mensagem.
   * @param arguments Os argumentos que sero formatados no padro.
   * 
   * @see MessageFormat
   * @see #serviceLogger
   */
  public void logWarningMessage(final String pattern, final Object[] arguments) {
    final String msg = MessageFormat.format(pattern, arguments);
    logWarningMessage(msg);
  }

  /**
   * Gerao de log de mensagem de depurao.
   * 
   * @param msg a mensagem a ser gerada.
   * 
   * @see #serviceLogger
   */
  public void logDebugMessage(String msg) {
    checkLogInited(msg);

    ServerLogRecord slr = new ServerLogRecord(ServerLevel.DEBUG,
      ServerLogFormatter.getQualifiedMessage(msg), getUser());
    serviceLogger.log(slr);
  }

  /**
   * Gerao de log de mensagem de depurao.
   * 
   * @param format um texto contendo parmetros a serem preenchidos pelos
   *        argumentos para formar a mensagem. Os parmetros devem ser
   *        fornecidos segundo a especificao de
   *        {@link String#format(String, Object[])}.
   * @param args argumentos que iro completar o formato para compor a mensagem.
   * 
   * @see String#format(String, Object[])
   * @see #serviceLogger
   */
  public void logDebugMessage(String format, Object... args) {
    String msg = String.format(format, args);
    logDebugMessage(msg);
  }

  /**
   * Gerao de log de mensagem com severidade.
   * 
   * @param msg a mensagem a ser gerada.
   * 
   * @see #serviceLogger
   */
  public final void logSevereMessage(final String msg) {
    checkLogInited(msg);

    ServerLogRecord slr = new ServerLogRecord(ServerLevel.SEVERE,
      ServerLogFormatter.getQualifiedMessage(msg), getUser());
    serviceLogger.log(slr);
  }

  /**
   * Gerao de log de mensagem com severidade.
   * 
   * @param msg a mensagem a ser gerada.
   * @param t um objeto <code>Throwable</code> de erro.
   * 
   * @see #serviceLogger
   */
  public final void logSevereMessage(final String msg, final Throwable t) {
    checkLogInited(msg);

    ServerLogRecord slr = new ServerLogRecord(ServerLevel.SEVERE,
      ServerLogFormatter.getQualifiedMessage(msg), getUser());
    slr.setThrown(t);
    serviceLogger.log(slr);
  }

  /**
   * Gerao de log de mensagem com severidade.
   * 
   * @param pattern Um padro para formatar a mensagem.
   * @param arguments Os argumentos que sero formatados no padro.
   * @param t um objeto <code>Throwable</code> de erro.
   * 
   * @see MessageFormat
   * @see #serviceLogger
   */
  final public void logSevereMessage(final String pattern,
    final Object[] arguments, final Throwable t) {
    final String msg = MessageFormat.format(pattern, arguments);
    logSevereMessage(msg, t);
  }

  /**
   * Mtodo para o SSI notificar que o servio est morrendo
   * 
   * @see #isRunning
   */
  public final void setAsDead() {
    isRunning = false;
  }

  /**
   * Mtodo para o SSI notificar que o servio est rodando
   * 
   * @see #isRunning
   */
  public final void setAsStarted() {
    isRunning = true;
  }

  /**
   * Verifica se o servio pode executar algum pedido.
   * 
   * @return um flag booleano indicativo.
   */
  public final boolean canWork() {
    try {
      if (!isAlive()) {
        return false;
      }
      return true;
    }
    catch (Exception e) {
      return false;
    }
  }

  /**
   * Verifica se o servio est ativo.
   * 
   * @return um flag booleano indicativo.
   */
  public final boolean isAlive() {
    return isRunning;
  }

  /**
   * Obtm a instncia do servio.
   * 
   * @param serviceName o nome do servio desejado.
   * 
   * @return a instncia.
   */
  public static Service getInstance(final String serviceName) {
    return services.get(serviceName);
  }

  /**
   * Construtor protegido da classe <code>Servico</code> que se encarrega de
   * inicializar todas as funcionalidades de um servio (log, propriedades etc).
   * 
   * @param srvName nome no servio
   * 
   * @throws ServerException se houver erro na instanciao.
   */
  protected Service(final String srvName) throws ServerException {
    isRunning = false;
    serviceName = srvName;

    final Server server = Server.getInstance();
    final String sep = File.separator;
    final String logFileName = srvName + ".log";
    final String logDirName = server.getLogDevDirectoryName();
    serviceLogFileName = logDirName + sep + logFileName;

    /* Inicialicao do log */
    serviceLogger = initServiceLog();

    /* Carga de propriedades do servio */
    final String proDirName = Server.getPropertiesRootDirectoryName();
    final String proFileName = srvName + ".properties";
    final String servicePropertiesFileName = proDirName + sep + proFileName;
    serviceProperties = loadProperties(servicePropertiesFileName);

    /* Adio automtica do servio ao Service Manager */
    final ServiceManager serviceManager = ServiceManager.getInstance();
    serviceManager.addService(this);

    /* Atribui o valor da instncia do servico */
    services.put(serviceName, this);
  }

  /**
   * Guarda nos dados locais da thread corrente, a string que identifica o
   * sistema onde o usurio da sesso atual originou a chamada remota. Este
   * mtodo  chamado quando a chamada de um mtodo do servio no  feito pelo
   * proxy de um cliente, mas quando  feito, por exemplo, por outros servios.
   * 
   * Um exemplo de atribuio a essa varivel, por exemplo, temos quando h uma
   * execuo de um comando do sistema atual, feita via o servio OpenDreams.
   * Nesse caso, o servio OpenDreams deve garantir que a atribuio do sistema
   * em que a chamada remota foi originada seja feita na thread do servio, para
   * futura checagem de permisso para realizar essa operao.
   * 
   * 
   * @param systemId o identificador (string) do sistema
   */
  public static void setSystemId(final String systemId) {
    threadSystemKey.set(systemId);
  }

  /**
   * Obtm a string que identifica o sistema onde o usurio da sesso atual
   * originou a chamada remota. Se a varivel de sistema no foi inicializada,
   * ento o valor retornado  null.
   * 
   * Para esse mecanismo funcionar, quando este mtodo  chamado a partir de
   * outro servio, o mtodo setSystemId deve ter sido chamado antes de fazer
   * chamadas para o servio.
   * 
   * @return o identificado (string) do sistema que originou a chamada remota,
   *         ou retorna null caso a varivel que indica o sistema no tenha sido
   *         atualizada
   */
  public static final String getSystemId() {
    return threadSystemKey.get();
  }

  /**
   * Mtodo abstrato de inicializao do servio.
   * 
   * @throws ServerException em caso de falha
   */
  public abstract void initService() throws ServerException;

  /**
   * Mtodo abstrato de trmino do servio
   * 
   * @throws ServerException em caso de falha.
   */
  public abstract void shutdownService() throws ServerException;
}
