/*
 * $Id: ServiceManager.java 100631 2010-01-12 12:45:33Z clinio $
 */

package csbase.server;

import java.lang.reflect.Constructor;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import csbase.logic.User;
import csbase.remote.HttpServiceInterface;
import csbase.remote.RemoteObserver;
import csbase.remote.ServiceInterface;
import csbase.server.services.httpservice.HttpService;
import csbase.server.services.loginservice.LoginService;
import csbase.server.services.loginservice.LogoutEvent;

/**
 * Classe que implementa o gerente de servios do SSI, que ser representado por
 * um singleton.
 * 
 * @see Service
 * @author Tecgraf/PUC-Rio
 */
public class ServiceManager implements RemoteObserver {
  /**
   * Sufixo dos nomes das classes Proxies dos servios.
   */
  private static final String PROXY_SUFFIX = "Proxy";

  /**
   * Instncia do singleton para o gerente de servios.
   */
  private static ServiceManager instance;

  /**
   * Mapa de servios para execuo no servidor.
   */
  private final Map<String, Service> servicesMap;

  /**
   * Mapa com os construtores para os servios que possuem proxies. <br>
   * <code>SERVICE_NAME</code> -> Construtor do proxy
   */
  private final Map<String, Constructor<? extends ServiceProxy>> proxiesConstructorsMap;

  /**
   * Guarda os proxies dos usurios logados. <br>
   * chave da sesso -> <code>Map</code> de proxies
   */
  private final Map<Object, Map<String, ServiceProxy>> usersProxiesMap;

  /**
   * Mtodo para carregar o gerente de servios; deve ser o primeiro mtodo a
   * ser chamado pois cria o nico objeto da classe.
   */
  protected static void init() {
    if (instance != null) {
      return;
    }
    instance = new ServiceManager();
  }

  /**
   * Mtodo para inicializar os servios cadastrados no gerente de servios.
   * 
   * @throws ServerException caso ocorra falha na inicializao do servio
   */
  protected void initAllServices() throws ServerException {
    final Server server = Server.getInstance();
    server.logDebugMessage("Inicializando servios...");

    // Iniciando servios
    for (Service service : servicesMap.values()) {
      service.initService();
      service.setAsStarted();
    }

    // Instalando service manager
    try {
      install();
    }
    catch (RemoteException re) {
      throw new ServerException(re);
    }

    server.logInfoMessage("Servios inicializados.");
  }

  /**
   * Mtodo para terminar todos os servios cadastrados no gerente de servios.
   * 
   * @throws ServerException caso ocorra alguma falha no trmino de servios
   */
  public void shutdownAllServices() throws ServerException {
    final Server server = Server.getInstance();
    server.logInfoMessage("Finalizando servios...");

    // Desinstalando service manager.
    try {
      uninstall();
    }
    catch (RemoteException re) {
      throw new ServerException(re);
    }

    // Montagem da lista de servios na ordem reversa
    final ArrayList<Service> invertedOrderServices = new ArrayList<Service>();
    final Collection<Service> services = servicesMap.values();
    for (final Service service : services) {
      // Inserindo no comeo da lista para ter a lista em ordem reversa.
      invertedOrderServices.add(0, service);
      server.logInfoMessage("Preparando trmino de "+ service.getName() + "...");
    }

    // Shutdown de servios na ordem reversa.
    for (final Service service : invertedOrderServices) {
      if (service.isAlive()) {
        service.setAsDead();
        service.shutdownService();
        server.logInfoMessage(service.getName() + " terminado!");
      }
    }

    server.logInfoMessage("Servios finalizados!");
  }

  /**
   * Mtodo que retorna a instncia do gerente de servios (singleton).
   * 
   * @return a instncia de <code>ServiceManager</code>.
   */
  public static ServiceManager getInstance() {
    return instance;
  }

  /**
   * Construtor privado que faz as inicializaes do singleton.
   */
  private ServiceManager() {
    servicesMap = new LinkedHashMap<String, Service>();
    proxiesConstructorsMap =
      new HashMap<String, Constructor<? extends ServiceProxy>>();

    final HashMap<Object, Map<String, ServiceProxy>> hashMap =
      new HashMap<Object, Map<String, ServiceProxy>>();
    usersProxiesMap = Collections.synchronizedMap(hashMap);
  }

  /**
   * Passa a observar eventos de {@link LoginService}.
   * 
   * @throws RemoteException caso ocorra algum erro de rmi no cadastro do
   *         observador
   */
  public void install() throws RemoteException {
    final LoginService loginService = LoginService.getInstance();
    loginService.addObserver(this, null);
  }

  /**
   * Invalida os proxies de um usurio que fez logout.
   * 
   * @param ev evento.
   */
  public void update(Object ev) {
    if (!(ev instanceof LogoutEvent)) {
      return;
    }
    final Object sessionKey = ((LogoutEvent) ev).getSessionKey();
    shutdownProxies(sessionKey);
  }

  /**
   * Deixa de observar eventos de {@link LoginService}.
   * 
   * @throws RemoteException casos ocorra alguma falha de rmi na remoo do
   *         observador
   */
  public void uninstall() throws RemoteException {
    final LoginService loginService = LoginService.getInstance();
    if (loginService != null) {
      loginService.deleteObserver(this, null);
    }
  }

  /**
   * Mtodo para adicionar (cadastrar) um novo servio ao SSI.
   * 
   * @param service novo servio para o cadatro.
   * 
   * @throws IllegalArgumentException caso o <code>service</code> seja null
   */
  public void addService(final Service service) {
    if (service == null) {
      throw new IllegalArgumentException("service == <<null>>");
    }
    final String serviceName = service.getName();
    servicesMap.put(serviceName, service);

    /*
     * Guarda os construtores dos proxies dos servios que so exportveis para
     * posterior uso na criao de proxies.
     */
    final String proxyName = service.getClass().getName() + PROXY_SUFFIX;
    try {

      /* Somente esta linha  no checada por causa do uso de Class.forName() */
      @SuppressWarnings("unchecked")
      Class<? extends ServiceProxy> proxyClass =
        (Class<? extends ServiceProxy>) Class.forName(proxyName);

      final Class<?>[] args = new Class<?>[] { Object.class };
      final Constructor<? extends ServiceProxy> constructor =
        proxyClass.getConstructor(args);
      proxiesConstructorsMap.put(serviceName, constructor);
    }
    catch (ClassNotFoundException e) {
      Server.getInstance().logInfoMessage(
        "Servio " + serviceName + " no possui proxy");
    }
    catch (NoSuchMethodException e) {
      Server.getInstance().logWarningMessage(
        "Proxy para " + serviceName + " no possui o construtor esperado.");
    }
    Server.getInstance().logInfoMessage(
      "Servio " + serviceName + " adicionado ao servidor.");
  }

  /**
   * Cria os proxies de todos os servios disponveis para uma sesso de um
   * usurio logado. O mapa de proxies tem como chave os nomes dos servios
   * conforme as constantes <code>XxxxServiceInterface.SERVICE_NAME</code>.
   * 
   * @param sessionKey Chave da sesso de um usurio.
   * @param servicesNames Nomes dos servios que sero recuperados.
   * 
   * @return Mapa com os proxies dos servios disponveis. Se nenhum servio
   *         estiver disponvel  retornado um mapa com tamanho 0. Se a sesso
   *         for invlida  retornado <code>null</code>.
   * 
   * @throws IllegalArgumentException caso o <code>sessionKey</code> ou o
   *         <code>servicesNames</code> seja null
   */
  public Map<String, ServiceInterface> getRemoteServices(Object sessionKey,
    Set<String> servicesNames) {
    if (sessionKey == null || servicesNames == null) {
      throw new IllegalArgumentException(
        "(sessionKey || servicesNames) == <<null>>");
    }
    if (!LoginService.getInstance().isValidSession(sessionKey)) {
      return null;
    }
    Map<String, ServiceInterface> remoteServices =
      new HashMap<String, ServiceInterface>();
    for (final String serviceName : servicesNames) {
      final ServiceInterface srv = getRemoteService(sessionKey, serviceName);
      if (srv != null) {
        remoteServices.put(serviceName, srv);
      }
    }
    return Collections.unmodifiableMap(remoteServices);
  }

  /**
   * Cria o proxy de um determinado servio para uma sesso de um usurio
   * logado. Se j existe um proxy do servio especificado para essa chave,
   * ento essa referncia  retornada.
   * 
   * @param sessionKey Chave da sesso de um usurio.
   * @param serviceName Nome do Servio. Conforme constante
   *        <code>XxxxServiceInterface.SERVICE_NAME</code>.
   * 
   * @return Proxy para o servio especificado. <code>null</code> caso o servio
   *         solicitado no esteja disponvel.
   * 
   * @throws IllegalArgumentException caso o <code>sessionKey</code> ou o
   *         <code>serviceName</code> seja null
   */
  public ServiceInterface getRemoteService(Object sessionKey, String serviceName) {
    if (sessionKey == null || serviceName == null) {
      throw new IllegalArgumentException(
        "(sessionKey || serviceName) == <<null>>");
    }

    if (!LoginService.getInstance().isValidSession(sessionKey)) {
      return null;
    }
    
    /* O servio de HTTP  um caso especial, pois ainda no foi migrado para o
     * padro de proxies para servios exportados. Por isso  retornado uma
     * referncia para a implementao do mesmo. Em breve, esse servio ter um
     * proxy.
     */
    if (serviceName.equals(HttpServiceInterface.SERVICE_NAME)) {
      return HttpService.getInstance();
    }
    synchronized (usersProxiesMap) {
      Map<String, ServiceProxy> proxies = usersProxiesMap.get(sessionKey);
      if (proxies == null) {
        proxies = new HashMap<String, ServiceProxy>();
      }
      ServiceProxy proxy = proxies.get(serviceName);
      if (proxy != null) {
        return proxy;
      }
      proxy = newProxy(sessionKey, serviceName);
      if (proxy == null) {
        return null;
      }
      proxies.put(serviceName, proxy);
      usersProxiesMap.put(sessionKey, proxies);
      return proxy;
    }
  }

  /**
   * Cria um proxy para o servio indicado, no mesmo pacote e com o mesmo nome
   * da classe que implementa o servio, mais um sufixo conforme a constante
   * {@link #PROXY_SUFFIX}.
   * 
   * @param sessionKey Chave da sesso do usurio.
   * @param serviceName Nome do servio.
   * 
   * @return O proxy para o servio ou <code>null</code> caso no exista proxy
   *         para esse servio.
   */
  private ServiceProxy newProxy(Object sessionKey, String serviceName) {
    try {
      final Constructor<? extends ServiceProxy> constructor =
        proxiesConstructorsMap.get(serviceName);
      if (constructor == null) {
        return null;
      }
      final Object[] args = new Object[] { sessionKey };
      final ServiceProxy proxy = constructor.newInstance(args);
      return proxy;
    }
    catch (Exception e) {
      final LoginService loginService = LoginService.getInstance();
      final User key = loginService.getUserByKey(sessionKey);
      final Server server = Server.getInstance();
      server.logSevereMessage("Falha ao criar proxy de " + serviceName
        + " para " + key, e);
      return null;
    }
  }

  /**
   * Finaliza os <i>proxies</i> para os servios do usurio especificado.
   * 
   * @param sessionKey Chave para a sesso do usurio.
   */
  private void shutdownProxies(Object sessionKey) {
    final Map<String, ServiceProxy> proxies = usersProxiesMap.remove(sessionKey);
    if (proxies == null) {
      return;
    }
    final Set<Entry<String, ServiceProxy>> entries = proxies.entrySet();
    for (Entry<String, ServiceProxy> entry : entries) {
      final ServiceProxy proxy = entry.getValue();
      try {
        proxy.shutdownProxy();
      }
      catch (Exception ex) {
        final String serviceName = entry.getKey();
        final String fmt = "Falha ao remover proxy de %s para %s";
        final String message = String.format(fmt, serviceName, sessionKey);
        final Server server = Server.getInstance();
        server.logSevereMessage(message);
      }
    }
  }
}
