/*
 * $Id: CSKeyStore.java 88775 2009-02-28 01:29:37Z clinio $
 */

package csbase.server.keystore;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.GeneralSecurityException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.Signature;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.text.MessageFormat;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

import csbase.exception.OperationFailureException;
import csbase.server.Server;

/**
 * Representa um repositrio de chaves.
 * 
 * @author Tecgraf/PUC-Rio
 */
public final class CSKeyStore {
  /** O algoritmo utilizado para gerar ou verificar uma assinatura digital. */
  private static final String SIGNATURE_ALGORITHM = "SHA1withDSA";

  /** O tipo do repositrio de chaves. */
  private static final String KEYSTORE_TYPE = "JKS";

  /** O repositrio de chaves propriamente dito. */
  private KeyStore keyStore;

  /** A instncia nica do repositrio de chaves. */
  private static CSKeyStore instance;

  /** A senha do repositrio de chaves. */
  private char[] password;

  /**
   * A data da ltima atualizao do repositrio em relao ao arquivo do
   * repositrio de chaves.
   */
  private long lastSynchronizationDate;

  /** O arquivo do repositrio de chaves. */
  private File keyStoreFile;

  /** Representa um controle de acesso ao repositrio de chaves. */
  private ReadWriteLock keyStoreLock;

  /**
   * Cria um repositrio de chaves.
   * 
   * @param pathName A localizao do arquivo que contm o repositrio de
   *        chaves.
   * @param password A senha do repositrio de chaves.
   * 
   * @throws IllegalArgumentException Caso a localizao do arquivo seja nula.
   */
  private CSKeyStore(String pathName, char[] password) {
    if (pathName == null) {
      throw new IllegalArgumentException(
        "A localizao do arquivo do repositrio de chaves no pode ser nula.");
    }
    this.password = password;
    this.keyStoreFile = new File(pathName);
    this.resetKeyStoreData();
    this.keyStoreLock = new ReentrantReadWriteLock();
  }

  /**
   * <p>
   * Sincroniza o repositrio de chaves/certificados em relao ao arquivo do
   * repositrio.
   * </p>
   * <p>
   * Atualiza o estado do repositrio de chaves.
   * </p>
   */
  private synchronized void synchronize() {
    this.keyStoreLock.writeLock().lock();
    try {
      this.loadKeyStore();
    }
    finally {
      this.keyStoreLock.writeLock().unlock();
    }
  }

  /**
   * Carrega o repositrio de chaves a partir do seu arquivo.
   */
  private void loadKeyStore() {
    if (this.lastSynchronizationDate == this.keyStoreFile.lastModified()) {
      return;
    }
    this.resetKeyStoreData();
    InputStream fileInputStream;
    try {
      fileInputStream = new FileInputStream(this.keyStoreFile);
    }
    catch (FileNotFoundException e) {
      Server.getInstance().logSevereMessage(
        MessageFormat.format(
          "O arquivo do repositrio de chaves no existe. Caminho: {0}.",
          this.keyStoreFile.getName()), e);
      return;
    }
    KeyStore keyStoreTemp;
    try {
      try {
        keyStoreTemp = KeyStore.getInstance(KEYSTORE_TYPE);
      }
      catch (KeyStoreException e) {
        Server.getInstance().logSevereMessage(
          MessageFormat.format(
            "O tipo de repositrio de chaves/certificados {0} no existe.",
            KEYSTORE_TYPE), e);
        return;
      }
      try {
        keyStoreTemp.load(fileInputStream, this.password);
      }
      catch (IOException e) {
        Server.getInstance().logSevereMessage(
          "Falha ao carregar o arquivo do repositrio de chaves/certificados.",
          e);
        return;
      }
      catch (GeneralSecurityException e) {
        Server.getInstance().logSevereMessage(
          "Falha ao carregar o arquivo do repositrio de chaves/certificados.",
          e);
        return;
      }
    }
    finally {
      try {
        fileInputStream.close();
      }
      catch (IOException e) {
        e.printStackTrace();
        Server.getInstance()
          .logSevereMessage(
            "Falha ao fechar o arquivo do repositrio de chaves/certificados.",
            e);
      }
    }
    this.keyStore = keyStoreTemp;
    this.lastSynchronizationDate = this.keyStoreFile.lastModified();
  }

  /**
   * Restaura os dados sobre o repositrio de chaves/certificados aos seus
   * valores iniciais.
   */
  private void resetKeyStoreData() {
    this.keyStore = null;
    this.lastSynchronizationDate = -1;
  }

  /**
   * Assina um dado em nome de uma entidade.
   * 
   * @param alias O nome da entidade.
   * @param privateKeyPassword A senha da chave privada da entidade.
   * @param data O dado que ser assinado.
   * 
   * @return o dado assinado ou null, caso a entidade no esteja cadastrada no
   *         repositrio de chaves.
   * 
   * @throws OperationFailureException Caso ocorra alguma falha na operao.
   * @throws IllegalArgumentException Caso algum dos parmetros recebidos seja
   *         nulo.
   */
  public byte[] sign(String alias, String privateKeyPassword, String data)
    throws OperationFailureException {
    if (alias == null) {
      throw new IllegalArgumentException(
        "O nome da entidade no pode ser nulo.");
    }
    if (privateKeyPassword == null) {
      throw new IllegalArgumentException(
        "A senha da chave privada da entidade no pode ser nula.");
    }
    if (data == null) {
      throw new IllegalArgumentException(
        "O dado a ser assinado no pode ser nulo.");
    }
    PrivateKey privateKey = this.getKey(alias, privateKeyPassword);
    if (privateKey == null) {
      return null;
    }
    try {
      Signature signature = Signature.getInstance(SIGNATURE_ALGORITHM);
      signature.initSign(privateKey);
      signature.update(data.getBytes());
      return signature.sign();
    }
    catch (GeneralSecurityException e) {
      throw new OperationFailureException(e);
    }
  }

  /**
   * Obtm uma chave de uma determinada entidade.
   * 
   * @param alias O nome da entidade.
   * @param privateKeyPassword A senha para acesso  chave privada.
   * 
   * @return A chave ou null, caso no exista.
   * 
   * @throws OperationFailureException Caso ocorra alguma falha na operao.
   */
  private PrivateKey getKey(String alias, String privateKeyPassword)
    throws OperationFailureException {
    this.synchronize();
    this.keyStoreLock.readLock().lock();
    try {
      if (this.keyStore == null) {
        return null;
      }
      return (PrivateKey) this.keyStore.getKey(alias, privateKeyPassword
        .toCharArray());
    }
    catch (GeneralSecurityException e) {
      throw new OperationFailureException(e);
    }
    finally {
      this.keyStoreLock.readLock().unlock();
    }
  }

  /**
   * Verifica se a assinatura de um determinado dado foi feita pela entidade.
   * 
   * @param alias O nome da entidade.
   * @param data O dado que foi assinado.
   * @param signedData A assinatura do dado.
   * 
   * @return true, caso a assinatura seja vlida, ou false, caso contrrio.
   * 
   * @throws OperationFailureException Caso ocorra alguma falha na operao.
   */
  public boolean verify(String alias, String data, byte[] signedData)
    throws OperationFailureException {
    Certificate certificate = this.getCertificate(alias);
    if (certificate == null) {
      return false;
    }
    try {
      Signature signature = Signature.getInstance(SIGNATURE_ALGORITHM);
      signature.initVerify(certificate);
      signature.update(data.getBytes());
      return signature.verify(signedData);
    }
    catch (GeneralSecurityException e) {
      throw new OperationFailureException(e);
    }
  }

  /**
   * Verifica se uma determinada entidade existe no repositrio.
   * 
   * @param alias O nome da entidade.
   * 
   * @return true, caso a entidade exista, ou false,caso contrrio.
   * 
   * @throws OperationFailureException Caso ocorra alguma falha na operao.
   */
  public boolean containsAlias(String alias) throws OperationFailureException {
    this.synchronize();
    this.keyStoreLock.readLock().lock();
    try {
      if (this.keyStore == null) {
        return false;
      }
      return this.keyStore.containsAlias(alias);
    }
    catch (KeyStoreException e) {
      throw new OperationFailureException(e);
    }
    finally {
      this.keyStoreLock.readLock().unlock();
    }
  }

  /**
   * Obtm um certificado de uma determinada entidade.
   * 
   * @param alias O nome da entidade.
   * 
   * @return O certificado ou null, caso no exista.
   * 
   * @throws OperationFailureException Caso ocorra alguma falha na operao.
   */
  public Certificate getCertificate(String alias)
    throws OperationFailureException {
    this.synchronize();
    this.keyStoreLock.readLock().lock();
    try {
      if (this.keyStore == null) {
        return null;
      }
      return this.keyStore.getCertificate(alias);
    }
    catch (KeyStoreException e) {
      throw new OperationFailureException(e);
    }
    finally {
      this.keyStoreLock.readLock().unlock();
    }
  }

  /**
   * Cria a instncia nica do repositrio de chaves.
   * 
   * @param pathName A localizao do arquivo que contm o repositrio de
   *        chaves.
   * 
   * @throws IllegalStateException Caso j exista uma instncia criada.
   * 
   * @see #getInstance()
   */
  public static void createInstance(String pathName) {
    if (instance != null) {
      throw new IllegalStateException(
        "J existe uma instncia de repositrio de chaves/certificados criada.");
    }
    instance = new CSKeyStore(pathName, null);
  }

  /**
   * Cria a instncia nica do repositrio de chaves.
   * 
   * @param pathName A localizao do arquivo que contm o repositrio de
   *        chaves.
   * @param password A senha do repositrio de chaves.
   * 
   * @throws IllegalArgumentException Caso a senha seja nula.
   * @throws IllegalStateException Caso j exista uma instncia criada.
   * 
   * @see #getInstance()
   */
  public static void createInstance(String pathName, String password) {
    if (password == null) {
      throw new IllegalArgumentException(
        "No  permitido criar uma instncia com senha nula.");
    }
    if (instance != null) {
      throw new IllegalStateException(
        "J existe uma instncia de repositrio de chaves/certificados criada.");
    }
    instance = new CSKeyStore(pathName, password.toCharArray());
  }

  /**
   * Obtm a instncia nica do repositrio de chaves.
   * 
   * @return A instncia nica ou null, caso esta ainda no tenha sido criada.
   * 
   * @see #createInstance(String)
   * @see #createInstance(String, String)
   */
  public static CSKeyStore getInstance() {
    return instance;
  }

  /**
   * Adiciona um certificado. Se a entidade j possuir um certificado o mesmo
   * ser sobreescrito.
   * 
   * @param alias O nome da entidade
   * @param cert A instncia do certificado
   * @return indicativo de sucesso.
   */
  public boolean addCertificate(String alias, Certificate cert) {
    if (alias == null || cert == null)
      throw new IllegalArgumentException("alias == null || cert == null");
    try {
      this.keyStoreLock.writeLock().lock();
      this.keyStore.setCertificateEntry(alias, cert);
      this.writeKeyStore();
      return this.keyStore.containsAlias(alias);
    }
    catch (KeyStoreException e) {
      Server.getInstance().logSevereMessage(
        "No foi possvel adicionar o certificado para a entidade " + alias, e);
    }
    finally {
      this.keyStoreLock.writeLock().unlock();
    }
    return false;
  }

  /**
   * Salva o keystore em disco
   * 
   */
  private void writeKeyStore() {
    try {
      this.keyStoreLock.writeLock().lock();
      OutputStream out = new FileOutputStream(this.keyStoreFile);
      this.keyStore.store(out, this.password);
      out.close();
    }
    catch (FileNotFoundException e) {
      Server.getInstance().logSevereMessage(
        MessageFormat.format(
          "O arquivo do repositrio de chaves no existe. Caminho: {0}.",
          this.keyStoreFile.getName()), e);
    }
    catch (KeyStoreException e) {
      Server.getInstance().logSevereMessage("KeyStore no inicializado.", e);
    }
    catch (NoSuchAlgorithmException e) {
      Server.getInstance().logSevereMessage(
        "Algoritmo de verificao desconhecido", e);
    }
    catch (CertificateException e) {
      Server.getInstance().logSevereMessage("Erro salvando certificados", e);
    }
    catch (IOException e) {
      Server.getInstance().logSevereMessage("Erro lendo " + this.keyStoreFile,
        e);
    }
    finally {
      this.keyStoreLock.writeLock().unlock();
    }
  }

  /**
   * Remove o certificado de uma entidade
   * 
   * @param alias O nome da entidade
   */
  public void removeCertificate(String alias) {
    if (alias == null)
      throw new IllegalArgumentException("alias == null");

    try {
      this.keyStoreLock.writeLock().lock();
      if (this.keyStore.containsAlias(alias)) {
        this.keyStore.deleteEntry(alias);
        this.writeKeyStore();
      }
    }
    catch (KeyStoreException e) {
      Server.getInstance().logSevereMessage(
        "No foi possvel remover o certificado para a entidade " + alias, e);
    }
    finally {
      this.keyStoreLock.writeLock().unlock();
    }
  }
}
