/*
 * $Id: Algorithm.java 104105 2010-04-12 16:08:59Z pasti $
 */

package csbase.server.services.algorithmservice;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import tecgraf.javautils.FileUtils;
import csbase.exception.OperationFailureException;
import csbase.logic.FileInfo;
import csbase.logic.IPathFactory;
import csbase.logic.Utilities;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmOutline;
import csbase.logic.algorithms.AlgorithmProperty;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.AlgorithmProperty.PropertyType;
import csbase.server.FileSystem;
import csbase.server.ServerException;
import csbase.server.ServerUtilities;
import csbase.server.services.languageservice.LanguageService;
import csbase.util.FileSystemUtils;

/**
 * Classe que representa um algoritmo no servidor.
 * 
 * @author Tecgraf/PUC-Rio
 */
class Algorithm {
  /**
   * Nome do arquivo que contm o identificador do algoritmo.
   */
  private static final String NAME_FILE_NAME = "name.txt";

  /**
   * Nome da propriedade que determina a ordem das propriedades dos algoritmos e
   * verses.
   */
  private static final String SORT_ORDER_PROPERTY = "ordem";

  /**
   * Informaes do algoritmo que podem ser serializadas para o cliente.
   */
  private final ModifiableAlgorithmInfo info;

  /**
   * Lista de propriedades dos algoritmos. A lista mantm a ordem de
   * apresentao de tais propriedades. A propriedades mapeiam a chave no label
   * (String de apresentao)
   */
  private static List<AlgorithmProperty> algorithmProperties;

  /**
   * Lista de propriedades das verses. A lista mantm a ordem de apresentao
   * de tais propriedades. A propriedades mapeiam a chave no label (String de
   * apresentao)
   */
  private static List<AlgorithmProperty> versionProperties;

  /**
   * Obtm as informaes de um algoritmo.
   * 
   * @return um objeto do tipo <code>AlgorithmInfo</code> (serializvel e
   *         imutvel) com as informaes do algoritmo.
   */
  public AlgorithmInfo getInfo() {
    return info.toAlgorithmInfo();
  }

  /**
   * Obtm o resumo de um algoritmo.
   * 
   * @return um objeto do tipo <code>AlgorithmOutline</code> com o resumo
   */
  public AlgorithmOutline getOutline() {
    return info.getOutline();
  }

  /**
   * Obtm as informaes de um algoritmo j presente (completo) no repositrio
   * e constri sua representao no SSI.
   * 
   * @param dir diretrio do algoritmo no repositrio
   * @return a representao do algoritmo
   * @throws ServerException
   */
  public static Algorithm includeAlgorithm(final String dir)
    throws ServerException {
    /* Obtem o nome e outros dados do algoritmo */
    final String algoDirPath =
      AlgorithmInfo.REPOSITORY_NAME + File.separator + dir;
    final String name = readName(algoDirPath);
    final Hashtable<String, String> propertyValues =
      loadAlgorithmPropertyValues(algoDirPath);

    /* Cria a representao do algoritmo */
    final Algorithm algo = new Algorithm(name, dir, propertyValues);

    /* Obtm as verses do algoritmo */
    algo.readVersions();
    return algo;
  }

  /**
   * Obtm o nome de um algoritmo ou verso a partir do arquivo correspondente.
   * 
   * @param dirPath path para o diretrio do algoritmo ou verso no repositrio
   * @return o nome do algoritmo ou verso.
   * @throws ServerException em caso de erro.
   */
  private static String readName(final String dirPath) throws ServerException {
    BufferedReader br = null;
    try {
      br =
        new BufferedReader(new FileReader(
          (dirPath + File.separator + NAME_FILE_NAME)));
      return br.readLine();
    }
    catch (final IOException e) {
      throw new ServerException("Falha na obteno de nome de "
        + "algoritmo/verso no arquivo "
        + (dirPath + File.separator + NAME_FILE_NAME), e);
    }
    finally {
      if (br != null) {
        try {
          br.close();
        }
        catch (final IOException ieo) {
          ieo.printStackTrace();
        }
      }
    }
  }

  /**
   * Altera o arquivo com o nome de um algoritmo.
   * 
   * @param dirPath path para o diretrio do algoritmo no repositrio
   * @param name o nome do algoritmo
   * 
   * @throws ServerException em caso de erro.
   */
  private static void writeName(final String dirPath, final String name)
    throws ServerException {
    final String aNameFile = dirPath + File.separator + NAME_FILE_NAME;
    writeSingleLineFile(aNameFile, name);
  }

  /**
   * Obtm as informaes das verses do algoritmo disponveis no repositrio.
   * 
   * @throws ServerException em caso de erro.
   */
  private void readVersions() throws ServerException {
    final String versionsDirectoryPath =
      AlgorithmInfo.REPOSITORY_NAME + File.separator + this.info.getDirectory()
        + File.separator + AlgorithmInfo.VERSIONS_DIR;
    final File versionsDirectoryFile = new File(versionsDirectoryPath);
    if (!versionsDirectoryFile.exists()) {
      final String message =
        MessageFormat.format("O diretrio {0} no existe.",
          new Object[] { versionsDirectoryFile.getAbsolutePath() });
      throw new ServerException(message);
    }
    if (!versionsDirectoryFile.isDirectory()) {
      final String message =
        MessageFormat.format("O arquivo {0} deveria ser um diretrio.",
          new Object[] { versionsDirectoryFile.getAbsolutePath() });
      throw new ServerException(message);
    }
    final File[] versionDirs = versionsDirectoryFile.listFiles();
    if (versionDirs.length <= 0) {
      final String message =
        MessageFormat.format(
          "O algoritmo {0} no possui verses.\nDiretrio de verses: {1}.",
          new Object[] { this.info.getName(),
              versionsDirectoryFile.getAbsoluteFile() });
      getService().logWarningMessage(message);
      return;
    }
    for (final File versionDir : versionDirs) {
      try {
        readVersion(versionsDirectoryPath, versionDir);
      }
      catch (final ServerException exception) {
        final String message =
          MessageFormat
            .format(
              "Erro ao tentar ler informaes sobre o algoritmo {0} diretrio de verso {1}.",
              new Object[] { this.info, versionDir.getAbsolutePath() });
        getService().logSevereMessage(message, exception);
      }
    }
  }

  /**
   * L dados de verso.
   * 
   * @param versionsDirectoryPath path de verso.
   * @param versionDirectoryFile arquivo de verso.
   * @throws ServerException em caso de erro.
   */
  void readVersion(final String versionsDirectoryPath,
    final File versionDirectoryFile) throws ServerException {
    if (!versionDirectoryFile.isDirectory()) {
      final String message =
        MessageFormat.format("O arquivo {0} deveria ser um diretrio.",
          new Object[] { versionDirectoryFile.getAbsolutePath() });
      throw new ServerException(message);
    }
    final String versionDirectoryName = versionDirectoryFile.getName().trim();
    if (!versionDirectoryName
      .equalsIgnoreCase(FileSystemUtils.VERSION_CONTROL_DIR)) {
      final String versionDirectoryPath =
        versionsDirectoryPath + File.separator + versionDirectoryName;
      final Hashtable<String, String> versionPropertyValues =
        loadVersionPropertyValues(versionDirectoryPath);
      final Map<String, List<FileInfo>> supportedPlatforms =
        getSupportedPlatforms(versionDirectoryPath);
      final AlgorithmVersionInfo versionInfo =
        new AlgorithmVersionInfo(getInfo(), versionDirectoryName,
          supportedPlatforms, versionPropertyValues);
      versionInfo.setDocumentation(getDocumentation(versionDirectoryPath));
      versionInfo.setConfigurators(getConfigurators(versionDirectoryPath));
      this.info.includeVersion(versionInfo.getId(), versionInfo);
    }
  }

  /**
   * Consulta a lista de configuradores.
   * 
   * @param versionDirectoryPath
   * @return a lista
   * @throws ServerException em caso de erro
   */
  private List<FileInfo> getConfigurators(final String versionDirectoryPath)
    throws ServerException {
    final File directory =
      new File(versionDirectoryPath + File.separator
        + AlgorithmVersionInfo.CONFIGURATOR_DIR);
    if (!directory.exists()) {
      final String message =
        MessageFormat.format("O diretrio {0} no existe.",
          new Object[] { directory.getAbsolutePath() });
      throw new ServerException(message);
    }
    if (!directory.isDirectory()) {
      final String message =
        MessageFormat.format("O arquivo {0} deveria ser um diretrio.",
          new Object[] { directory.getAbsolutePath() });
      throw new ServerException(message);
    }

    return FileInfo.createFilesInfo(directory);
  }

  /**
   * Consulta a documentao.
   * 
   * @param versionDirectoryPath path
   * @return lista de informaes
   * @throws ServerException em caso de erro
   */
  private List<FileInfo> getDocumentation(final String versionDirectoryPath)
    throws ServerException {
    final File directory =
      new File(versionDirectoryPath + File.separator
        + AlgorithmVersionInfo.DOCUMENTATION_DIR);
    if (!directory.exists()) {
      final String message =
        MessageFormat.format("O diretrio {0} no existe.",
          new Object[] { directory.getAbsolutePath() });
      throw new ServerException(message);
    }
    if (!directory.isDirectory()) {
      final String message =
        MessageFormat.format("O arquivo {0} deveria ser um diretrio.",
          new Object[] { directory.getAbsolutePath() });
      throw new ServerException(message);
    }

    return FileInfo.createFilesInfo(directory);
  }

  /**
   * Descobre quais as plataformas suportadas por uma verso de algoritmo. Os
   * nomes dos subdiretrios de "BIN_DIR" so os identificadores das plataformas
   * suportadas.
   * 
   * @param vDirPath path para o diretrio da verso do algoritmo
   * @return uma Hashtable contendo as identificaes (Strings) das plataformas
   *         suportadas (chave) e os executaveis associados as plataformas
   *         (valor da hash).
   * @throws ServerException Em caso de erro.
   */
  private Map<String, List<FileInfo>> getSupportedPlatforms(
    final String vDirPath) throws ServerException {

    final Map<String, List<FileInfo>> supportedPlatforms =
      new HashMap<String, List<FileInfo>>();
    final String path =
      vDirPath + File.separator + AlgorithmVersionInfo.BIN_DIR;
    final File binDir = new File(path);
    if (!binDir.exists()) {
      final String message =
        MessageFormat.format(
          "O diretrio - {0} - que armazena as plataformas no existe.",
          new Object[] { binDir.getAbsolutePath() });

      getService().logWarningMessage(message);
      return supportedPlatforms;
    }

    if (!binDir.isDirectory()) {
      throw new ServerException(MessageFormat.format(
        "O arquivo {0} deveria ser um diretrio.", new Object[] { binDir
          .getAbsolutePath() }));
    }
    final File[] platformsDir = binDir.listFiles();
    if (platformsDir.length == 0) {
      final String message =
        MessageFormat.format(
          "O diretrio - {0} - que armazena as plataformas est vazio.",
          new Object[] { binDir.getAbsolutePath() });
      getService().logWarningMessage(message);
      return supportedPlatforms;
    }

    for (final File platform : platformsDir) {
      if (platform.isDirectory() && FileInfo.isNameValid(platform.getName())) {

        final List<FileInfo> executablesInfo =
          FileInfo.createFilesInfo(platform);
        supportedPlatforms.put(platform.getName(), executablesInfo);
      }
    }

    return supportedPlatforms;
  }

  /**
   * Cria no repositrio a estrutura para um novo algoritmo.
   * 
   * @param name nome do novo algoritmo
   * @param propertyValues valores.
   * @return representao do algoritmo
   * 
   * @throws ServerException
   */
  public static Algorithm createAlgorithm(final String name,
    final Hashtable<String, String> propertyValues) throws ServerException {
    final String algoDirName = defineAlgoDirName(name);
    final String algoDirPath =
      AlgorithmInfo.REPOSITORY_NAME + File.separator + algoDirName;
    try {
      createAlgoRootDir(algoDirPath);

      /* cria a estrutura de subdiretrios do algoritmo */
      createAlgoSubDirectories(algoDirPath);

      /* cria os arquivos com o nome do algoritmo e data atualizao */
      writeName(algoDirPath, name);
      final String pathToPropFile =
        algoDirPath + File.separator + AlgorithmInfo.PROPERTY_VALUES_FILE;
      savePropertyValues(pathToPropFile, propertyValues);
      return new Algorithm(name, algoDirName, propertyValues);
    }
    catch (final ServerException se) {
      try {
        ServerUtilities.removeFile(new File(algoDirPath), true);
      }
      catch (final Exception e) {
        getService().logSevereMessage(
          "Falha na remoo de arquivo/diretrio \n" + "\tExceo: " + e);
      }
      throw se;
    }
  }

  /**
   * Define o nome do diretrio "raiz" de um algoritmo, utilizando como "base" o
   * nome do algoritmo. O nome do diretrio no dever conter caracteres
   * especiais e deve ser nico no repositrio.
   * 
   * @param name nome do algoritmo
   * @return uma String com o nome do diretrio do algoritmo
   */
  private static String defineAlgoDirName(final String name) {
    /* Transforma o nome do algoritmo num nome de diretrio vlido */
    String algoDirName = FileUtils.fixDirectoryName(name);

    /* Garante a unicidade do nome no repositrio */
    while (getService().algorithmIdRegistered(algoDirName)) {
      algoDirName = algoDirName + "_";
    }
    return algoDirName;
  }

  /**
   * Cria o diretrio "raiz" de um algoritmo.
   * 
   * @param algoDirPath path para o diretrio "raiz" do algoritmo
   * @throws ServerException em caso de erro.
   */
  private static void createAlgoRootDir(final String algoDirPath)
    throws ServerException {
    final File algoDir = new File(algoDirPath);
    if (algoDir.exists()) {
      throw new ServerException("Diretrio " + algoDirPath + " j existe!");
    }
    if (!algoDir.mkdir()) {
      throw new ServerException("Falha na criao do diretrio " + algoDirPath);
    }
  }

  /**
   * Cria a estrutura de subdiretrios para um algoritmo.
   * 
   * @param algoDirPath path para o diretrio "raiz" do algoritmo
   * @throws ServerException
   */
  private static void createAlgoSubDirectories(final String algoDirPath)
    throws ServerException {
    final String verDirPath =
      algoDirPath + File.separator + AlgorithmInfo.VERSIONS_DIR;
    final File verDir = new File(verDirPath);
    if (!verDir.mkdir()) {
      throw new ServerException("Falha na criao do diretrio de verses "
        + verDirPath);
    }
  }

  /**
   * Remove o algoritmo do repositrio.
   * 
   * @throws ServerException em caso de erro.
   */
  public void remove() throws ServerException {
    /* remover os diretorios */
    final String algoDirPath =
      AlgorithmInfo.REPOSITORY_NAME + File.separator + info.getDirectory();
    final File algoFile = new File(algoDirPath);
    try {
      ServerUtilities.removeFile(algoFile, true);
    }
    catch (final Exception e) {
      throw new ServerException(e.getMessage(), e);
    }
  }

  /**
   * Recarrega as informaes de um algoritmo.
   * 
   * @throws ServerException
   */
  public void reload() throws ServerException {
    /* Reobtem as verses do algoritmo */
    readVersions();
  }

  /**
   * Remove uma verso de algoritmo do repositrio.
   * 
   * @param versionId verso a remover
   * @throws ServerException em caso de erro.
   */
  public void removeVersion(final Object versionId) throws ServerException {
    final AlgorithmVersionInfo versionInfo = info.getVersionInfo(versionId);

    /* remover os diretorios */
    final String versionDirPath = versionInfo.getDirPath();
    final File versionFile = new File(versionDirPath);
    try {
      ServerUtilities.removeFile(versionFile, true);
    }
    catch (final Exception e) {
      throw new ServerException(e.getMessage(), e);
    }
    info.deleteVersion(versionId);
  }

  /**
   * Cria no repositrio a estrutura para uma nova verso de algoritmo.
   * 
   * @param major numero da verso
   * @param minor numero da reviso
   * @param patch numero da correo
   * @param propertyValues Mapa com as propriedades dos algoritmos.
   * @return identificador da verso criada
   * @throws ServerException
   */
  public Object createVersion(final int major, final int minor,
    final int patch, final Map<String, String> propertyValues)
    throws ServerException {
    final String algoDir = info.getDirectory();
    final String algoDirPath =
      AlgorithmInfo.REPOSITORY_NAME + File.separator + algoDir;
    final String versionDir =
      AlgorithmVersionInfo.getDirectoryFor(major, minor, patch);
    final String versionDirPath =
      algoDirPath + File.separator + AlgorithmInfo.VERSIONS_DIR
        + File.separator + versionDir;
    createVersionRootDir(versionDirPath);
    try {
      /* cria a estrutura de subdiretrios do algoritmo */
      createVersionSubDirectories(versionDirPath);
    }
    catch (final ServerException se) {
      try {
        ServerUtilities.removeFile(new File(versionDirPath), true);
      }
      catch (final Exception e) {
        getService().logSevereMessage("Falha na remoo de arquivo/diretrio.",
          e);
      }
      throw se;
    }

    /* cria as informaes da verso */
    final Map<String, List<FileInfo>> supportedPlatforms =
      new Hashtable<String, List<FileInfo>>();
    final String pathToPropFile =
      versionDirPath + File.separator
        + AlgorithmVersionInfo.PROPERTY_VALUES_FILE;
    savePropertyValues(pathToPropFile, propertyValues);
    final AlgorithmVersionInfo versionInfo =
      new AlgorithmVersionInfo(getInfo(), new AlgorithmVersionId(major, minor,
        patch), supportedPlatforms, propertyValues);
    final Object versionId = versionInfo.getId();
    info.includeVersion(versionId, versionInfo);
    return versionId;
  }

  /**
   * Cria o diretrio "raiz" de uma verso.
   * 
   * @param versionDirPath path para o diretrio "raiz" da verso
   * @throws ServerException
   */
  private void createVersionRootDir(final String versionDirPath)
    throws ServerException {
    final File versionDir = new File(versionDirPath);
    if (versionDir.exists()) {
      throw new ServerException("Diretrio " + versionDirPath + " j existe!");
    }
    if (!versionDir.mkdir()) {
      throw new ServerException("Falha na criao do diretrio "
        + versionDirPath);
    }
  }

  /**
   * Cria a estrutura de subdiretrios para uma verso de algoritmo
   * 
   * @param versionDirPath path para o diretrio "raiz" da verso
   * @throws ServerException
   */
  private void createVersionSubDirectories(final String versionDirPath)
    throws ServerException {
    final String confDirPath =
      versionDirPath + File.separator + AlgorithmVersionInfo.CONFIGURATOR_DIR;
    final File confDir = new File(confDirPath);
    if (!confDir.mkdir()) {
      throw new ServerException("Falha na criao do diretrio do "
        + "configurador " + confDirPath);
    }
    final String binDirPath =
      versionDirPath + File.separator + AlgorithmVersionInfo.BIN_DIR;
    final File binDir = new File(binDirPath);
    if (!binDir.mkdir()) {
      throw new ServerException("Falha na criao do diretorio de "
        + "executveis: " + binDirPath);
    }
    final String docDirPath =
      versionDirPath + File.separator + AlgorithmVersionInfo.DOCUMENTATION_DIR;
    final File docDir = new File(docDirPath);
    if (!docDir.mkdir()) {
      throw new ServerException(
        "Falha na criao do diretorio de arquivos de documentao: "
          + docDirPath);
    }
  }

  /**
   * Atualiza os arquivos de configurao de uma verso de um algoritmo.
   * 
   * @param versionId identificador da verso
   * @throws OperationFailureException
   */
  protected void updateConfigurations(final Object versionId)
    throws OperationFailureException {

    final AlgorithmVersionInfo versionInfo = info.getVersionInfo(versionId);
    final File configDir = new File(versionInfo.getConfiguratorDirPath());
    if (!configDir.exists()) {
      String msg =
        LanguageService.getInstance().get(
          "server.algoservice.error.config_version_not_found");
      final Object[] args = new Object[] { info.getId(), versionId };
      msg = MessageFormat.format(msg, args);
      throw new OperationFailureException(msg);
    }

    final List<FileInfo> configurators = FileInfo.createFilesInfo(configDir);
    versionInfo.setConfigurators(configurators);
  }

  /**
   * Atualiza os arquivos de documentao de uma verso de um algoritmo.
   * 
   * @param versionId identificador da verso
   * @throws OperationFailureException
   */
  protected void updateDocumentations(final Object versionId)
    throws OperationFailureException {

    final AlgorithmVersionInfo versionInfo = info.getVersionInfo(versionId);
    final File docDir = new File(versionInfo.getDocDirPath());
    if (!docDir.exists()) {
      String msg =
        LanguageService.getInstance().get(
          "server.algoservice.error.doc_version_not_found");
      final Object[] args = new Object[] { info.getId(), versionId };
      msg = MessageFormat.format(msg, args);
      throw new OperationFailureException(msg);
    }

    final List<FileInfo> documentation = FileInfo.createFilesInfo(docDir);
    versionInfo.setDocumentation(documentation);
  }

  /**
   * Atualiza os executveis de uma plataforma de uma verso de algoritmo.
   * 
   * @param versionId identificador da verso
   * @param platform plataforma cujos executveis deseja-se atualizar.
   * @throws OperationFailureException
   */
  protected void updateExecutables(final Object versionId, final String platform)
    throws OperationFailureException {
    final AlgorithmVersionInfo versionInfo = info.getVersionInfo(versionId);
    final String platDirPath = versionInfo.getPlatformPath(platform);
    final File platDir = new File(platDirPath);
    if (!platDir.exists()) {
      String msg =
        LanguageService.getInstance().get(
          "server.algoservice.error.plat_version_not_found");
      final Object[] args = new Object[] { info.getId(), versionId, platform };
      msg = MessageFormat.format(msg, args);
      throw new OperationFailureException(msg);
    }

    final List<FileInfo> executables = FileInfo.createFilesInfo(platDir);
    for (FileInfo file : executables) {
      if (!file.isDirectory()) {
        try {
          String path = new File(platDir, file.getPath()).getAbsolutePath();
          FileSystem.enableExecutionPermission(path);
        }
        catch (final ServerException e) {
          throw new OperationFailureException(e);
        }
      }
    }
    versionInfo.setPlatformExecutables(platform, executables);
  }

  /**
   * Remove arquivos de uma verso de algoritmo.
   * 
   * @param versionId identificador da verso
   * @param filePaths caminhos para os arquivos a serem removidos.
   * @throws Exception em caso de erro.
   */
  void removeFile(final Object versionId, final String[][] filePaths)
    throws Exception {
    for (final String[] filePath2 : filePaths) {
      final String filePath = Utilities.makeRelativePath(filePath2);
      final File serverFile = new File(filePath);
      try {
        // Remove o arquivo do servidor
        if (!serverFile.delete()) {
          final String msg =
            LanguageService.getInstance().get(
              "server.algoservice.error.copy_server");
          throw new Exception(msg);
        }
      }
      catch (final Exception e) {
        String msg =
          LanguageService.getInstance().get(
            "server.algoservice.error.remove_doc");
        final Object[] args =
          new Object[] { serverFile.getName(), info.getId(), versionId };
        msg = MessageFormat.format(msg, args);
        throw new Exception(msg + e.getMessage(), e);
      }
    }

    // Recarrega o algoritmo
    reload();
  }

  /**
   * Apaga arquivo.
   * 
   * @param file arquivo
   * @return indicativo
   */
  private boolean deleteFile(final File file) {
    if (file.isDirectory()) {
      for (final File child : file.listFiles()) {
        if (!deleteFile(child)) {
          return false;
        }
      }
    }
    return file.delete();
  }

  /**
   * Remove arquivos executveis de uma verso de algoritmo.
   * 
   * @param version Verso que detm os arquivos.
   * @param platform Plataforma que detm os arquivos.
   * @param files Representao dos arquivos a serem removidos.
   * @throws Exception caso no tenha sido possvel remover o arquivo.
   */
  public void removeExecutableFiles(final AlgorithmVersionInfo version,
    final String platform, final FileInfo[] files) throws Exception {
    for (final FileInfo file : files) {
      final String path = version.getExecFilePath(platform, file.getPath());
      final File serverFile = new File(path);
      try {
        // Remove o arquivo do servidor
        if (!deleteFile(serverFile)) {
          final String msg =
            LanguageService.getInstance().get(
              "server.algoservice.error.copy_server");
          throw new Exception(msg);
        }
      }
      catch (final Exception e) {
        String msg =
          LanguageService.getInstance().get(
            "server.algoservice.error.remove_exe");
        final Object[] args =
          new Object[] { serverFile.getName(), info.getId(), version.getId(),
              platform };
        msg = MessageFormat.format(msg, args);
        throw new Exception(msg + e.getMessage(), e);
      }
    }

    // Recarrega o algoritmo
    reload();
  }

  /**
   * Remove arquivos de configurao de uma verso de algoritmo.
   * 
   * @param version Verso que detm os arquivos.
   * @param files Representao dos arquivos a serem removidos.
   * @throws Exception caso no tenha sido possvel remover o arquivo.
   */
  public void removeConfigurationFiles(final AlgorithmVersionInfo version,
    final FileInfo[] files) throws Exception {
    for (final FileInfo file : files) {
      final String path =
        version.getConfiguratorDirPath() + File.separator + file.getPath();
      final File serverFile = new File(path);
      try {
        // Remove o arquivo do servidor
        if (!deleteFile(serverFile)) {
          final String msg =
            LanguageService.getInstance().get(
              "server.algoservice.error.copy_server");
          throw new Exception(msg);
        }
      }
      catch (final Exception e) {
        String msg =
          LanguageService.getInstance().get(
            "server.algoservice.error.remove_config");
        final Object[] args = new Object[] { info.getId(), version.getId() };
        msg = MessageFormat.format(msg, args);
        throw new Exception(msg + e.getMessage(), e);
      }
    }

    // Recarrega o algoritmo
    reload();
  }

  /**
   * Remove arquivos de documentao de uma verso de algoritmo.
   * 
   * @param version Verso que detm os arquivos.
   * @param files Representao dos arquivos a serem removidos.
   * @throws Exception caso no tenha sido possvel remover o arquivo.
   */
  public void removeDocumentationFiles(final AlgorithmVersionInfo version,
    final FileInfo[] files) throws Exception {
    for (final FileInfo file : files) {
      final String path =
        version.getDocDirPath() + File.separator + file.getPath();
      final File serverFile = new File(path);
      try {
        // Remove o arquivo do servidor
        if (!deleteFile(serverFile)) {
          final String msg =
            LanguageService.getInstance().get(
              "server.algoservice.error.copy_server");
          throw new Exception(msg);
        }
      }
      catch (final Exception e) {
        String msg =
          LanguageService.getInstance().get(
            "server.algoservice.error.remove_doc");
        final Object[] args =
          new Object[] { serverFile.getName(), info.getId(), version.getId() };
        msg = MessageFormat.format(msg, args);
        throw new Exception(msg + e.getMessage(), e);
      }
    }

    // Recarrega o algoritmo
    reload();
  }

  /**
   * <p>
   * Faz a cpia do arquivo dado para o caminho especificado pelo toPath. Em
   * seguida, faz a cpia do mesmo arquivo para o diretrio do cliente. Se
   * {@code setExecutable} for verdadeiro a cpia criada/sobrescrita ter
   * permisso de execuo.
   * </p>
   * <p>
   * Ao final da execuo os algoritmos sero recarregados.
   * </p>
   * 
   * @param fromPath caminho para o arquivo a ser copiado.
   * @param toPath caminho de destino.
   * @param setExecutable indica se deve habilitar a permisso de execuo.
   * @return true se for criado um arquivo novo para a cpia, false se j
   *         existisse um arquivo com o mesmo nome no diretrio (este ento 
   *         sobrescrito).
   * @throws Exception
   */
  boolean copyFile(final String fromPath, final String toPath,
    final boolean setExecutable) throws Exception {
    try {
      return copyFileWithoutReloading(fromPath, toPath, setExecutable);
    }
    finally {
      reload();
    }
  }

  /**
   * Copia arquivos de uma pasta para a outra com a possibilidade de dar
   * permisso de execuo s cpias. Para definir o caminho do arquivo de
   * destino, ser construido um FileInfo usando o targetDir como path se este
   * for diferente de <tt>null</tt> e mantendo o nome do arquivo original. Este
   * novo FileInfo ser passado como parmetro para o
   * {@link IPathFactory#getPath(FileInfo) targetPathFactory.getPath(FileInfo)}
   * que retornar o caminho fsico do novo arquivo. Ao final da execuo os
   * algoritmos sero recarregados.
   * 
   * @param files Arquivos a serem transferidos.
   * @param sourcePathFactory Fbrica para criar o caminho dos arquivos a serem
   *        transferidos.
   * @param targetDir diretrio de destino (pode ser <tt>null</tt>).
   * @param targetPathFactory Fbrica para criar o caminho dos arquivos de
   *        destino. Este caminho  relativo ao valor do parmetro targetDir.
   * @param setExecutables <tt>true</tt> se as cpias devero ter permisso de
   *        execuo ou no.
   * @param records Uma lista para gravar as informaoes dos arquivos copiados
   *        (pode ser <tt>null</tt>).
   * @throws Exception
   */
  public void copyFiles(final List<FileInfo> files,
    final IPathFactory sourcePathFactory, final FileInfo targetDir,
    final IPathFactory targetPathFactory, final boolean setExecutables,
    final List<ICopyRecord> records) throws Exception {

    try {
      copyFilesWithoutReloading(files, sourcePathFactory, targetDir,
        targetPathFactory, setExecutables, records);
    }
    finally {
      reload();
    }
  }

  /**
   * Copia arquivos de uma pasta para a outra com a possibilidade de dar
   * permisso de execuo s cpias. Para definir o caminho do arquivo de
   * destino, ser construido um FileInfo usando o targetDir como path se este
   * for diferente de <tt>null</tt> e mantendo o nome do arquivo original. Este
   * novo FileInfo ser passado como parmetro para o
   * {@link IPathFactory#getPath(FileInfo) targetPathFactory.getPath(FileInfo)}
   * que retornar o caminho fsico do novo arquivo. Ao final da execuo os
   * algoritmos <b>NO</b> sero recarregados.
   * 
   * @param files Arquivos a serem transferidos.
   * @param sourcePathFactory Fbrica para criar o caminho dos arquivos a serem
   *        transferidos.
   * @param targetDir diretrio de destino.
   * @param targetPathFactory Fbrica para criar o caminho dos arquivos de
   *        destino. Este caminho  relativo ao valor do parmetro targetDir.
   * @param setExecutables <tt>true</tt> se as cpias devero ter permisso de
   *        execuo ou no.
   * @param records Uma lista com informaoes dos arquivos copiados.
   * @throws Exception
   */
  void copyFilesWithoutReloading(final List<FileInfo> files,
    final IPathFactory sourcePathFactory, final FileInfo targetDir,
    final IPathFactory targetPathFactory, final boolean setExecutables,
    final List<ICopyRecord> records) throws Exception {

    for (final FileInfo file : files) {
      FileInfo targetFileInfo;
      if (null != targetDir) {
        final String[] newDirPath =
          Utilities.splitProjectPath(targetDir.getPath());
        targetFileInfo =
          new FileInfo(newDirPath, file.getName(), file.isDirectory());
      }
      else {
        targetFileInfo = new FileInfo(file.getName(), file.isDirectory());
      }
      final String targetPath = targetPathFactory.getPath(targetFileInfo);

      if (file.isDirectory()) {
        final File targetFile = new File(targetPath);
        if (!targetFile.exists()) {
          targetFile.mkdir();
          if (null != records) {
            records.add(new CopyDirectoryRecord(targetFileInfo));
          }
        }
        copyFilesWithoutReloading(file.getChildren(), sourcePathFactory,
          targetFileInfo, targetPathFactory, setExecutables, records);
      }
      else {
        final String sourcePath = sourcePathFactory.getPath(file);
        final boolean created =
          copyFileWithoutReloading(sourcePath, targetPath, setExecutables);
        if (null != records) {
          records.add(new CopyFileRecord(file, targetFileInfo, created));
        }
      }
    }
  }

  /**
   * Faz a cpia do arquivo dado para o caminho especificado pelo toPath. Em
   * seguida, faz a cpia do mesmo arquivo para o diretrio do cliente. Se
   * {@code setExecutable} for verdadeiro a cpia criada/sobrescrita ter
   * permisso de execuo. Ao final da execuo os algoritmos <b>NO</b> sero
   * recarregados.
   * 
   * @param fromPath caminho para o arquivo a ser copiado.
   * @param toPath caminho de destino.
   * @param setExecutable indica se deve habilitar a permisso de execuo.
   * 
   * @return true se for criado um arquivo novo para a cpia, false se j
   *         existisse um arquivo com o mesmo nome no diretrio (este ento 
   *         sobrescrito).
   * @throws Exception
   */
  boolean copyFileWithoutReloading(final String fromPath, final String toPath,
    final boolean setExecutable) throws Exception {
    if (fromPath.equals(toPath)) {
      // Ignoro copia de arquivos iguais
      return false;
    }
    // O padro  rescrever os arquivos.
    boolean copyCreated = false;
    final File from = new File(fromPath);
    final File serverTo = new File(toPath);
    if (!serverTo.exists()) {
      serverTo.createNewFile(); // gera erro quando for um diretrio.
      copyCreated = true;
    }
    if (FileSystem.copyFile(from, serverTo)) {
      if (setExecutable) {
        FileSystem.enableExecutionPermission(toPath);
      }
    }
    else {
      final String msg =
        LanguageService.getInstance().get(
          "server.algoservice.error.copy_server");
      throw new Exception(msg);
    }
    return copyCreated;
  }

  /**
   * Inclui uma plataforma para uma verso de algoritmo.
   * 
   * @param versionId identificador da verso
   * @param platform plataforma a incluir
   * @throws ServerException
   */
  public void includePlatform(final Object versionId, final String platform)
    throws ServerException {
    final AlgorithmVersionInfo versionInfo = info.getVersionInfo(versionId);
    final String platDirPath = versionInfo.getPlatformPath(platform);
    final File platDir = new File(platDirPath);
    if (!platDir.mkdirs()) {
      throw new ServerException("Falha na criao do diretorio de plataforma: "
        + platDirPath);
    }
    versionInfo.setPlatform(platform);
  }

  /**
   * Remove uma plataforma de uma verso de algoritmo.
   * 
   * @param versionId identificador da verso
   * @param platform plataforma a remover
   * @throws ServerException
   */
  public void removePlatform(final Object versionId, final String platform)
    throws ServerException {
    final AlgorithmVersionInfo versionInfo = info.getVersionInfo(versionId);
    final String platDirPath = versionInfo.getPlatformPath(platform);
    final File platDir = new File(platDirPath);
    try {
      ServerUtilities.removeFile(platDir, true);
      final Map<String, List<FileInfo>> supportedPlatforms =
        versionInfo.getPlatforms();
      supportedPlatforms.remove(platform);
    }
    catch (final Exception e) {
      throw new ServerException(e.getMessage(), e);
    }
  }

  /**
   * Escreve o contedo de um arquivo texto.
   * 
   * @param fileName path do arquivo
   * @param content contedo a ser escrito
   * @throws ServerException
   */
  private static void writeSingleLineFile(final String fileName,
    final String content) throws ServerException {
    try {
      final FileWriter fw = new FileWriter(fileName);
      fw.write(content);
      fw.close();
    }
    catch (final IOException e) {
      throw new ServerException("Falha na escrita do arquivo " + fileName, e);
    }
  }

  /**
   * Obtm referncia para o servio de gerncia de algoritmos.
   * 
   * @return referncia para o servio de gerncia de algoritmos.
   */
  private static AlgorithmService getService() {
    return AlgorithmService.getInstance();
  }

  /**
   * Constri a representao de um algoritmo no SSI. Deve ser utilizado apenas
   * para <b>operacoes de inclusao do servico</b>!
   * 
   * @param name nome do algoritmo
   * @param dir diretrio do algoritmo no repositrio
   * @param attributes Atributos do algoritmo lidos do arquivo de propriedades
   */
  private Algorithm(final String name, final String dir,
    final Hashtable<String, String> attributes) {
    /* O identificador do algoritmo  o nome de seu diretrio */
    final String id = dir;
    this.info = new ModifiableAlgorithmInfo(id, name, dir, attributes);
  }

  /**
   * Retorna a lista de <code>Property</code> dos algoritmos. Cada
   * <code>Property</code> mapeia a chave da propriedade ao Label (String de
   * apresentao).
   * 
   * @return Lista com as propriedades dos algoritmos.
   */
  public static List<AlgorithmProperty> getAlgorithmProperties() {
    if (algorithmProperties == null) {
      final AlgorithmService algoService = AlgorithmService.getInstance();
      final String algPropertiesPath =
        algoService.getAlgorithmPropertyNamesPath();
      algorithmProperties = getProperties(algPropertiesPath);
    }
    return algorithmProperties;
  }

  /**
   * Retorna a lista de <code>Property</code> das verses dos algoritmos. Cada
   * <code>Property</code> mapeia a chave da propriedade ao Label (String de
   * apresentao).
   * 
   * @return Lista com as propriedades das verses dos algoritmos.
   */
  public static List<AlgorithmProperty> getVersionProperties() {
    if (versionProperties == null) {
      final AlgorithmService algoService = AlgorithmService.getInstance();
      final String versionPropertiesPath =
        algoService.getVersionPropertyNamesPath();
      versionProperties = getProperties(versionPropertiesPath);
    }
    return versionProperties;
  }

  /**
   * Mtodo para leitura dos nomes das propriedades dos algoritmos e das
   * verses. Tais arquivos de propriedades possuem uma propriedade que
   * determina a ordem de leitura das chaves de propridades (ordem =
   * propriedade1, propriedade2, propriedadeX) As demais propriedades neste
   * arquivo mapeiam a chave no label de exibio. propriedade1 = Meu ttulo 
   * propriedade2 = Tenho o nome propriedadeX = Eu sou ... A propriedade ordem 
   * importante para que as propriedades sejam exibidas sempre na mesma ordem.
   * Caso ela no exista, no garantimos a ordem de exibico das propriedades.
   * 
   * @param propertyPath Nome do arquivo de propriedades
   * @return Um lista de <code>Property<code> que possui as chaves das
   * propriedades ordenadas
   */
  private static List<AlgorithmProperty> getProperties(final String propertyPath) {
    final Properties properties = readPropertiesFile(propertyPath);
    final List<String> keys = new ArrayList<String>();
    final String sortedProperties = properties.getProperty(SORT_ORDER_PROPERTY);
    final List<AlgorithmProperty> propertyLabels =
      new ArrayList<AlgorithmProperty>();
    // Existe a propriedade ordem
    if (sortedProperties != null) {
      final String[] sortedPropertiesArray = sortedProperties.split(",");
      for (final String element : sortedPropertiesArray) {
        keys.add(element.trim());
      }
      String propertyKey = null;
      String propertyLabel = null;
      PropertyType propertyType;
      for (int i = 0; i < keys.size(); i++) {
        propertyKey = keys.get(i);
        propertyLabel = properties.getProperty(propertyKey);
        final String type =
          properties.getProperty(propertyKey + AlgorithmProperty.TYPE_SUFFIX);

        /* Trata da possibilidade da ordem conter chaves inexistentes */
        if (propertyLabel != null) {
          if (type == null) {
            propertyLabels
              .add(new AlgorithmProperty(propertyKey, propertyLabel));
          }
          else {
            propertyType = PropertyType.valueOf(type.toUpperCase());
            propertyLabels.add(new AlgorithmProperty(propertyKey,
              propertyLabel, propertyType));
          }
        }
      }
    }
    // A propriedade ordem no existe, consideramos ordem de leitura
    else {
      for (final Enumeration<?> e = properties.propertyNames(); e
        .hasMoreElements();) {
        final String propertyKey = (String) e.nextElement();
        if (propertyKey.endsWith(AlgorithmProperty.TYPE_SUFFIX)) {
          continue;
        }
        final String propertyLabel = properties.getProperty(propertyKey);
        final String type =
          properties.getProperty(propertyKey + AlgorithmProperty.TYPE_SUFFIX);
        /* Trata da possibilidade da ordem conter chaves inexistentes */
        if (propertyLabel != null) {
          if (type == null) {
            propertyLabels
              .add(new AlgorithmProperty(propertyKey, propertyLabel));
          }
          else {
            final PropertyType propertyType =
              PropertyType.valueOf(type.toUpperCase());
            propertyLabels.add(new AlgorithmProperty(propertyKey,
              propertyLabel, propertyType));
          }
        }
      }
    }
    return propertyLabels;
  }

  /**
   * Recupera os atributos estendidos de um algoritmo do local onde eles foram
   * persistidos.
   * 
   * @param algoDirPath caminho para o diretorio do algoritmo
   * @return chaves e valores dos atributos estendidos.
   */
  private static Hashtable<String, String> loadAlgorithmPropertyValues(
    final String algoDirPath) {
    final String pathToPropFile =
      algoDirPath + File.separator + AlgorithmInfo.PROPERTY_VALUES_FILE;
    final List<AlgorithmProperty> algProperties = getAlgorithmProperties();
    return loadPropertiesValues(pathToPropFile, algProperties);
  }

  /**
   * Obtm os valores das propriedades no caminho especificado. As chaves das
   * propriedades lidas esto no atributo <code>properties</code>.
   * 
   * @param propPath Caminho do arquivo de propriedades.
   * @param propertiesList Lista com as propriedades que devem ser lidas do
   *        arquivo dado
   * @return uma hashtable com os valores das propriedades associados s chaves
   */
  private static Hashtable<String, String> loadPropertiesValues(
    final String propPath, final List<AlgorithmProperty> propertiesList) {
    final Hashtable<String, String> result = new Hashtable<String, String>();
    final Properties properties = readPropertiesFile(propPath);
    for (final AlgorithmProperty property : propertiesList) {
      final String propKey = property.getKey();
      final PropertyType type = property.getType();
      if (propKey != null) {
        final String propValue = properties.getProperty(propKey);
        if (propValue != null && propValue.length() > 0) {
          if (type == PropertyType.DOUBLE) {
            // Tenta converter. Se falhar ignora propriedade e loga.
            try {
              new Double(propValue);
            }
            catch (final NumberFormatException e) {
              getService().logSevereMessage(
                "Falha na leitura do valor " + propValue + " da varivel "
                  + propKey + " em: " + propPath);
              continue;
            }
          }
          result.put(propKey, propValue);
        }
      }
    }
    readHideAlgorithmProperty(properties, result);
    return result;
  }

  /**
   * Procura pela propriedade ocultar, usada pelo ExecutorFrame, para que as
   * aplicaes X no apaream no menu de algoritmos.
   * 
   * @param properties propriedades lidas de um arquivo de propriedades do
   *        algoritmo
   * @param algoPropertiesTable hashing com os valores das propriedades
   *        associados s chaves
   */
  private static void readHideAlgorithmProperty(final Properties properties,
    final Hashtable<String, String> algoPropertiesTable) {
    final String propValue =
      properties.getProperty(AlgorithmInfo.HIDE_ALGORITHM_PROPERTY);
    if (propValue != null
      && propValue.trim().equals(AlgorithmInfo.HIDE_ALGORITHM_VALUE)) {
      algoPropertiesTable.put(AlgorithmInfo.HIDE_ALGORITHM_PROPERTY,
        AlgorithmInfo.HIDE_ALGORITHM_VALUE);
    }
  }

  /**
   * Recupera as propriedades da versao do algoritmo do local onde elas foram
   * persistidas
   * 
   * @param versionDirectoryPath caminho para o diretorio da versao
   * @return tabela com os nomes e valores dos atributos estendidos
   */
  private Hashtable<String, String> loadVersionPropertyValues(
    final String versionDirectoryPath) {
    final String pathToPropFile =
      versionDirectoryPath + File.separator
        + AlgorithmVersionInfo.PROPERTY_VALUES_FILE;
    final List<AlgorithmProperty> vp = getVersionProperties();
    return loadPropertiesValues(pathToPropFile, vp);
  }

  /**
   * L as propriedades de um arquivo e carrega em um objeto-coleo de
   * propriedades. Nao deve ser utilizado diretamente. As propriedaes do
   * algoritmo e da versao podem ser obtidos atraves dos metodos
   * loadAlgorithmPropertyValues e loadVersionPropertyValues
   * 
   * @param path caminho para o arquivo que contm as propriedades.
   * @return o objeto properties.
   */
  private static Properties readPropertiesFile(final String path) {
    InputStream inputStream = null;
    final Properties properties = new Properties();
    try {
      inputStream = new FileInputStream(path);
      properties.load(inputStream);
    }
    catch (final FileNotFoundException e) {
      getService().logDebugMessage(
        "Arquivo de propriedades " + path + " no foi encontrado.");
    }
    catch (final IOException e) {
      getService().logSevereMessage(
        "Erro ao ler arquivo de propriedades " + path, e);
    }
    finally {
      if (inputStream != null) {
        try {
          inputStream.close();
        }
        catch (final IOException ioe) {
          getService().logSevereMessage(
            "Erro ao fechar arquivo de propriedades " + path, ioe);
        }
      }
    }
    return properties;
  }

  /**
   * Define novos atributos estendidos para o algoritmo e persiste a mudanca
   * 
   * @param newValues
   */
  public void setAlgorithmFieldValuesList(
    final Hashtable<String, String> newValues) {
    final String algoDirPath =
      AlgorithmInfo.REPOSITORY_NAME + File.separator + info.getDirectory();

    /* Periste Mudancas */
    final String pathToPropFile =
      algoDirPath + File.separator + AlgorithmInfo.PROPERTY_VALUES_FILE;
    savePropertyValues(pathToPropFile, newValues);

    /* Altera dados em memoria */
    this.info.setPropertyValues(newValues);
  }

  /**
   * Define novos atributos estendidos para o algoritmo e persiste a mudanca
   * 
   * @param id .
   * @param newValues
   */
  public void setVersionFieldValuesList(final AlgorithmVersionId id,
    final Hashtable<String, String> newValues) {
    final AlgorithmVersionInfo version = this.getInfo().getVersionInfo(id);
    final String versionDirPath = version.getDirPath();

    /* Persiste mudancas */
    final String pathToPropFile =
      versionDirPath + File.separator
        + AlgorithmVersionInfo.PROPERTY_VALUES_FILE;
    savePropertyValues(pathToPropFile, newValues);

    /* Altera dados em memoria */
    version.setPropertyValues(newValues);
  }

  /**
   * Persiste os atributos estendidos do algoritmo
   * 
   * @param pathToPropFile caminho para o arquivo de propriedades
   * @param propertyValues valores dos atributos estendidos
   */
  private static void savePropertyValues(final String pathToPropFile,
    final Map<String, String> propertyValues) {
    final Properties properties = new Properties();
    for (final String attributeKey : propertyValues.keySet()) {
      final String attributeValue = propertyValues.get(attributeKey);
      properties.put(attributeKey, attributeValue);
    }
    try {
      writePropertiesFile(pathToPropFile, properties);
    }
    catch (final ServerException e) {
      getService().logInfoMessage(e.getMessage());
    }
  }

  /**
   * Armazena um arquivo de propriedades num dado local. Nao deve ser utilizado
   * diretamente! As propriedades estendidas do algoritmo podem ser salvas
   * utilizando os metodos saveAlgorithmPropertyValues e
   * saveVersionPropertyValues
   * 
   * @param pathToPropFile nome e caminho completo do arquivo de propriedades
   * @param properties propriedades a serem salvas
   * @throws ServerException em caso de erro.
   */
  private static void writePropertiesFile(final String pathToPropFile,
    final Properties properties) throws ServerException {
    FileOutputStream out = null;
    try {
      out = new FileOutputStream(pathToPropFile);
    }
    catch (final FileNotFoundException e) {
      final String message =
        "Erro ao escrever arquivo de propriedades " + pathToPropFile;
      throw new ServerException(message, e);
    }
    finally {
      try {
        properties.store(out, "Arquivo de propriedades do algoritmo");
        if (out != null) {
          out.close();
        }
      }
      catch (final IOException e) {
        final String message =
          "Erro ao escrever arquivo de propriedades " + pathToPropFile;
        throw new ServerException(message, e);
      }
    }
  }

  /**
   * Interface para a funcionalidade de copiar um arquivo/diretrio.
   */
  public static interface ICopyRecord {
    /**
     * Verifica se  ou no um diretrio.
     * 
     * @return retorna true se for um diretrio, caso contrrio, retorna false
     */
    boolean isDirectory();

    /**
     * Verifica se o arquivo/diretrio foi criado.
     * 
     * @return retorna true o arquivo/diretrio foi criado, caso contrrio,
     *         retorna false
     */
    boolean wasCreated();
  }

  /**
   * Mantm o diretrio a ser copiado.
   */
  public static class CopyDirectoryRecord implements ICopyRecord {

    /**
     * O path.
     */
    private final FileInfo path;

    /**
     * Construtor
     * 
     * @param path o path
     */
    private CopyDirectoryRecord(final FileInfo path) {
      this.path = path;
    }

    /**
     * Obtm o diretrio.
     * 
     * @return o caminho do diretrio
     */
    public FileInfo getDirectory() {
      return path;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isDirectory() {
      return true;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean wasCreated() {
      return true;
    }
  }

  /**
   * Encapsula o resultado da cpia de um arquivo.
   */
  public static class CopyFileRecord implements ICopyRecord {
    /**
     * Representao do arquivo fonte.
     */
    private final FileInfo sourceFile;
    /**
     * Representao do arquivo de destino.
     */
    private final FileInfo targetFile;
    /**
     * Indica se o arquivo foi criado ou sobrescrito.
     */
    private final boolean created;

    /**
     * Construtor
     * 
     * @param sourceFile arquivo origem
     * @param targetFile arquivo destino
     * @param created indicativo se arquivo foi criado ou sobrescrito.
     */
    private CopyFileRecord(final FileInfo sourceFile,
      final FileInfo targetFile, final boolean created) {
      this.sourceFile = sourceFile;
      this.targetFile = targetFile;
      this.created = created;
    }

    /**
     * Obtm uma representao do arquivo fonte.
     * 
     * @return uma representao do arquivo fonte.
     */
    public FileInfo getSourceFile() {
      return sourceFile;
    }

    /**
     * Obtm uma representao do arquivo de destino.
     * 
     * @return uma representao do arquivo de destino.
     */
    public FileInfo getTargetFile() {
      return targetFile;
    }

    /**
     * {@inheritDoc}
     */
    public boolean isDirectory() {
      return false;
    }

    /**
     * Indica se o arquivo foi criado ou sobrescrito.
     * 
     * @return <tt>true</tt> se o arquivo foi criado ou <tt>false</tt> caso
     *         tenha sido sobrescrito.
     */
    public boolean wasCreated() {
      return created;
    }
  }
}
