package csbase.server.services.algorithmservice;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInput;
import java.io.ObjectInputStream;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;
import java.io.StringReader;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.rmi.RemoteException;
import java.security.DigestException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Map.Entry;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import tecgraf.javautils.FileUtils;
import csbase.exception.BugException;
import csbase.exception.ConfigurationException;
import csbase.exception.HttpServiceException;
import csbase.exception.OperationFailureException;
import csbase.exception.ParseException;
import csbase.exception.PermissionException;
import csbase.exception.ServiceFailureException;
import csbase.exception.algorithms.AlgorithmNotFoundException;
import csbase.exception.algorithms.CategoriesFileNotSavedException;
import csbase.logic.AdminPermission;
import csbase.logic.AlgoEvent;
import csbase.logic.AlgorithmAdminPermission;
import csbase.logic.AlgorithmsReloadNotificationData;
import csbase.logic.FileInfo;
import csbase.logic.IPathFactory;
import csbase.logic.Permission;
import csbase.logic.User;
import csbase.logic.Utilities;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmOutline;
import csbase.logic.algorithms.AlgorithmProperty;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.Category;
import csbase.logic.algorithms.CategorySet;
import csbase.logic.algorithms.ConfigurationPathFactory;
import csbase.logic.algorithms.DocumentationPathFactory;
import csbase.logic.algorithms.ExecutablePathFactory;
import csbase.logic.algorithms.HistoryRecord;
import csbase.logic.algorithms.flows.Flow;
import csbase.logic.algorithms.flows.FlowAlgorithmParser;
import csbase.logic.algorithms.flows.configurator.FlowAlgorithmConfigurator;
import csbase.logic.algorithms.parameters.SimpleAlgorithmConfigurator;
import csbase.logic.algorithms.parsers.SimpleAlgorithmParser;
import csbase.logic.algorithms.xml.category.XmlCategoriesReader;
import csbase.logic.algorithms.xml.category.XmlCategoriesWriter;
import csbase.remote.AlgorithmServiceInterface;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.TransactionCallbackInterface;
import csbase.remote.niotransfer.RequestTransferInfo;
import csbase.server.FileSystem;
import csbase.server.Server;
import csbase.server.ServerException;
import csbase.server.Service;
import csbase.server.TransactionManager;
import csbase.server.services.algorithmservice.Algorithm.CopyDirectoryRecord;
import csbase.server.services.algorithmservice.Algorithm.CopyFileRecord;
import csbase.server.services.algorithmservice.Algorithm.ICopyRecord;
import csbase.server.services.algorithmservice.niotransfer.FileServer;
import csbase.server.services.algorithmservice.niotransfer.FileTransferrer;
import csbase.server.services.httpservice.HttpService;
import csbase.server.services.httpservice.UploadHandler;
import csbase.server.services.notificationservice.NotificationService;
import csbase.util.FileSystemUtils;
import csbase.util.Unzip;

/**
 * A classe <code>AlgoService</code> implementa o servio de gerncia de
 * algoritmos instalados no SSI. Observa o HttpService para saber quando o
 * upload dos executaveis terminou.
 */
public class AlgorithmService extends Service implements
  AlgorithmServiceInterface, FileTransferrer {

  /**
   * Nome da classe decara! configuradores lgicos escritos em Java.
   */
  private static final String CONFIGURATOR_CLASS_NAME = "Logic";

  /**
   * Chave para identificao do histrico de um diretrio dentro da tabela de
   * histrico.
   */
  private static final String DIR_HISTORY_KEY = "_DIR_HISTORY_KEY";

  /** Mensagem de erro para algoritmo no encontrado */
  private static final String ERROR_ALGO_NOT_FOUND = "server.algoservice.error.algo_not_found";

  /** Mensagem de erro para arquivo de configurao no encontrado */
  private static final String ERROR_CONF_VERSION_NOT_FOUND = "server.algoservice.error.conf_version_not_found";

  /** Mensagem de erro para problema durante a cpia de arquivos */
  private static final String ERROR_COPY = "server.algoservice.error.copy";

  // #TODO A remover quando todos sistemas CSBase forem assinados {
  /** Mensagem de erro para arquivo de documentao no encontrado */
  private static final String ERROR_DOC_VERSION_NOT_FOUND = "server.algoservice.error.doc_version_not_found";

  /** Mensagem de erro para plataforma no encontrada */
  private static final String ERROR_PLAT_VERSION_NOT_FOUND = "server.algoservice.error.plat_version_not_found";
  // #TODO } A remover quando todos sistemas CSBase forem assinados

  /** Mensagem de erro para verso no encontrada */
  private static final String ERROR_VERSION_NOT_FOUND = "server.algoservice.error.version_not_found";

  /** Registro de histrico para arquivo copiado */
  private static final String HISTORY_COPY_FILE = "server.algoservice.history.copy_file";

  /** Registro de histrico para diretrio copiado */
  private static final String HISTORY_COPY_DIRECTORY = "server.algoservice.history.copy_directory";

  /** Registro de histrico para algoritmo criado */
  private static final String HISTORY_CREATE_ALGORITHM = "server.algoservice.history.create_algorithm";

  /** Registro de histrico para verso criada */
  private static final String HISTORY_CREATE_VERSION = "server.algoservice.history.create_version";

  /** Registro de histrico para verso duplicada */
  private static final String HISTORY_DUPLICATE_VERSION = "server.algoservice.history.duplicate_version";

  /** Registro de histrico para plataforma includa */
  private static final String HISTORY_INCLUDE_PLATFORM = "server.algoservice.history.include_platform";

  /** Registro de histrico para algoritmo removido */
  private static final String HISTORY_REMOVE_ALGORITHM = "server.algoservice.history.remove_algorithm";

  /** Registro de histrico para arquivo removido */
  private static final String HISTORY_REMOVE_FILE = "server.algoservice.history.remove_file";

  /** Registro de histrico para plataforma removida */
  private static final String HISTORY_REMOVE_PLATFORM = "server.algoservice.history.remove_platform";
  /** Registro de histrico para verso removida */
  private static final String HISTORY_REMOVE_VERSION = "server.algoservice.history.remove_version";
  /** Nome do arquivo serializado que contm a tabela de histrico */
  private static final String HISTORY_TABLE_FILE_NAME = ".algorithms.csbase_history";

  /** Registro de histrico para arquivo de configurao carregado */
  private static final String HISTORY_UPDATE_CONFIGURATOR = "server.algoservice.history.update_configurator";

  /** Registro de histrico para arquivo de documentao carregado */
  private static final String HISTORY_UPDATE_DOC = "server.algoservice.history.update_doc";

  /** Registro de histrico para executvel carregado */
  private static final String HISTORY_UPDATE_EXECUTABLE = "server.algoservice.history.update_executable";

  /** Extenso para arquivos ZIP */
  private static final String ZIP_EXTENSION = "zip";

  /** Chave para um identificador de algoritmo. */
  private static final String ALGO_ID_KEY = "algoId";

  /** Chave para um identificador de usurio. */
  private static final String USER_ID_KEY = "userId";

  /** Chave para um tipo de arquivo da rvore de algoritmos. */
  private static final String ALGO_FILE_TYPE_KEY = "type";

  /** Chave para um identificador de verso. */
  private static final String VERSION_ID_KEY = "versionId";

  /** Chave para o nome de uma plataforma. */
  private static final String PLATFORM_NAME_KEY = "platformName";

  /**
   * Propriedade que define o intervalo de recarga dos algoritmos em minutos.
   * Caso essa propriedade no seja definida, o servio no faz a recarga.
   */
  private static final String PROP_RELOAD_ALGS_INTERVAL_IN_MIN = "reloadAlgsIntervalInMinutes";

  /** Chave para um nome de arquivo. */
  private static final String FILE_NAME_KEY = "fileName";

  /** Chave para o caminho absoluto no sistema de arquivos. */
  private static final String FILE_PATH_KEY = "filePath";
  
  /**
   * Chave para o indicador que indica se o arquivo deve ser expandido ao final
   * do upload, caso este seja um arquivo zip.
   */
  private static final String EXPAND_IF_ZIP = "expandIfZip";

  /** Tipos de arquivos na rvore de algoritmos */
  private enum AlgoFileType {
    /** Arquivo executvel (ou binrio) */
    EXECUTABLE,
    /** Arquivo de configurao */
    CONFIGURATION,
    /** Arquivo de documentao */
    DOCUMENTATION,
    /** Pacote de verso */
    VERSION_PACK
  }

  /** Flag usado para desativar a thread de reinicio do servio */
  private boolean shutdown;

  /** Hashtable que contm os algoritmos cadastrados. */
  private Hashtable<String, Algorithm> registeredAlgorithms;

  /** Hashtable que contm os parmetros cadastrados. */
  private Properties parameters;

  /** Hashtable que contm as configuraes dos parmetros cadastrados. */
  private Properties parametersConfiguration;

  /**
   * Transao que garante a execuo da gerncia do repositrio em modo
   * exclusivo.
   */
  private TransactionManager transaction;

  /**
   * Thread que recarrega os algoritmos.
   */
  private ReloadAlgsThread reloadAlgsThread;

  /**
   * Servidor utilizado por este servio para transferncia de arquivos entre
   * cliente e servidor.
   */
  private final FileServer fileServer;

  /**
   * As categorias.
   */
  private CategorySet categorySet;

  private boolean isFirstCategoryLoad;

  /**
   * Cria o servio de algoritmos. Dispara uma thread para aguardar solicitaes
   * de transferncia de arquivos de/para a rvore de algoritmos.
   * 
   * @throws ServerException
   */
  protected AlgorithmService() throws ServerException {
    super(SERVICE_NAME);
    categorySet = new CategorySet();
    // XXX - Temporrio. Aguardando a soluo do Bug #5847.
    ClientRemoteLocator.algorithmService = this;
    fileServer = new FileServer(this);
    new Thread() {
      @Override
      public void run() {
        fileServer.dispatch();
      }
      // FIXME no deveramos iniciar a thread aqui -- a no ser que a classe
      // fosse final
      // http://findbugs.sourceforge.net/bugDescriptions.html#SC_START_IN_CTOR
    }.start();
  }

  /**
   * Constri a instncia do servio de gerncia de algoritmos
   * 
   * @throws ServerException caso ocorra algum problema durante a instanciao.
   */
  public static void createService() throws ServerException {
    new AlgorithmService();
  }

  /**
   * Obtm uma referncia para a instncia nica (singleton) do servio de
   * algoritmos.
   * 
   * @return referncia para a instncia nica do servio de algoritmos.
   */
  public static AlgorithmService getInstance() {
    return (AlgorithmService) getInstance(SERVICE_NAME);
  }

  /**
   * Verifica se existe um algoritmo com o identificador especificado.
   * 
   * @param id identificador do algoritmo
   * 
   * @return <code>true</code> se existe o algoritmo, <code>false</code> caso
   *         contrrio
   */
  public boolean algorithmIdRegistered(Object id) {
    return (registeredAlgorithms.get(id) != null);
  }

  /**
   * {@inheritDoc}
   */
  public AlgorithmInfo changeAlgorithmProperties(String name,
    Hashtable<String, String> newValues) throws AlgorithmNotFoundException {
    checkPermission(name);
    Algorithm algorithm = getAlgorithm(name);
    algorithm.setAlgorithmFieldValuesList(newValues);
    AlgorithmInfo algoInfo = algorithm.getInfo();
    logInfoMessage("Algoritmo " + name + " modificado em "
      + algoInfo.getDirectory() + " (apenas propriedades)");
    fireModifyEvent(algoInfo);
    return algoInfo;
  }

  /**
   * {@inheritDoc}
   */
  public AlgorithmInfo changeVersionProperties(String name,
    AlgorithmVersionId versionId, Hashtable<String, String> newValues)
    throws AlgorithmNotFoundException {
    checkPermission(name);
    Algorithm algorithm = getAlgorithm(name);
    algorithm.setVersionFieldValuesList(versionId, newValues);
    AlgorithmInfo algoInfo = getInfo(name);
    logInfoMessage("Verso " + versionId + " do algoritmo " + name
      + " modificada (apenas propriedades)");
    fireModifyEvent(algoInfo);
    return algoInfo;
  }

  /**
   * {@inheritDoc}
   */
  public void copyFile(Object algoIdFrom, Object versionIdFrom,
    String[][] fromFileTokens, Object algoIdTo, Object versionIdTo,
    String[][] toFileTokens) {
    copyFile(algoIdFrom, versionIdFrom, fromFileTokens, algoIdTo, versionIdTo,
      toFileTokens, false);
  }

  /**
   * {@inheritDoc}
   */
  public void copyFile(Object algoIdFrom, Object versionIdFrom,
    String[][] fromFileTokens, Object algoIdTo, Object versionIdTo,
    String[][] toFileTokens, boolean setExecutables) {
    if (fromFileTokens == null || fromFileTokens.length == 0) {
      throw new IllegalArgumentException("fromFileTokens");
    }
    if (toFileTokens == null || toFileTokens.length == 0) {
      throw new IllegalArgumentException("toFileTokens");
    }
    checkPermission(algoIdTo);
    Algorithm algoTo = registeredAlgorithms.get(algoIdTo);
    if (algoTo == null) {
      throw new ServiceFailureException(getAlgoNotFoundMessage(algoIdTo));
    }
    Algorithm algoFrom = registeredAlgorithms.get(algoIdFrom);
    if (algoFrom == null) {
      throw new ServiceFailureException(getAlgoNotFoundMessage(algoIdFrom));
    }
    boolean fileCreated = false;
    for (int i = 0; i < fromFileTokens.length; i++) {
      String fromFilePath = FileUtils.joinPath(fromFileTokens[i]);
      String toFilePath = FileUtils.joinPath(toFileTokens[i]);
      try {
        if (algoTo.copyFile(fromFilePath, toFilePath, setExecutables)) {
          fileCreated = true;
        }
      }
      catch (Exception e) {
        String fileName = fromFileTokens[i][fromFileTokens[i].length - 1];
        String msg = getString(ERROR_COPY);
        Object[] args = new Object[] { fileName, versionIdTo, algoIdTo };
        msg = MessageFormat.format(msg, args);
        throw new ServiceFailureException(msg, e);
      }
    }

    // ATENO: Esta implementao assume que cpias simultneas de mltiplos
    // arquivos sempre ocorrero de um nico diretrio de origem para um nico
    // diretrio de destino - de acordo com a interface atual de
    // administrao. Caso esta premissa venha a mudar, o cdigo a seguir
    // ter de ser revisto.
    // #TODO Essa implementao amarra a estrutura de diretrios - rever
    // Caminho de destino
    int toPathLen = toFileTokens[0].length - 2; // Exclui dir "versions" e o
    // arquivo
    String[] toPath = new String[toPathLen];
    toPath[0] = getRootDirName();
    toPath[1] = algoTo.getInfo().getName();
    toPath[2] = versionIdTo.toString();
    toPath[3] = toFileTokens[0][4]; // Diretrio "versions"  ignorado
    if (toPathLen == 5) { // Remoo de executvel
      toPath[4] = toFileTokens[0][5]; // Caminho da plataforma
    }

    // Caminho de origem
    int fromPathLen = fromFileTokens[0].length - 3;
    String[] fromPath = new String[fromPathLen];
    fromPath[0] = algoFrom.getInfo().getName();
    fromPath[1] = versionIdFrom.toString();
    fromPath[2] = fromFileTokens[0][4]; // Diretrio "versions"  ignorado
    if (fromPathLen == 4) { // Remoo de executvel
      fromPath[3] = fromFileTokens[0][5]; // Caminho da plataforma
    }
    String joinedFromPath = FileUtils.joinPath(fromPath);
    String[] toNames = new String[toFileTokens.length];
    for (int i = 0; i < toNames.length; i++) {
      toNames[i] = toFileTokens[i][toFileTokens[i].length - 1];
      String description = MessageFormat.format(getString(HISTORY_COPY_FILE),
        new Object[] { toNames[i], joinedFromPath });
      appendHistory(toPath, description);
    }

    // Se algum arquivo de destino tiver sido criado, notifica mudana no
    // algoritmo
    if (fileCreated) {
      fireModifyEvent(algoTo.getInfo());
    }
  }

  /**
   * <p>
   * Copia arquivos de uma pasta para a outra com a possibilidade de dar
   * permisso de execuo s cpias.
   * </p>
   * <p>
   * Para definir o caminho do arquivo de destino, ser construido um FileInfo
   * usando o targetDir como path se este for diferente de <tt>null</tt> e
   * mantendo o nome do arquivo original. Este novo FileInfo ser passado como
   * parmetro para o {@link IPathFactory#getPath(FileInfo)
   * targetPathFactory.getPath(FileInfo)} que retornar o caminho fsico do novo
   * arquivo.
   * </p>
   * <p>
   * Ao final da execuo os algoritmos sero recarregados.
   * </p>
   * 
   * @param sourceVersion Informaes da verso de origem.
   * @param files Arquivos a serem transferidos.
   * @param sourcePathFactory Fbrica para criar o caminho dos arquivos a serem
   *        transferidos.
   * @param targetVersion Informaes da verso de destino.
   * @param targetDir diretrio de destino.
   * @param targetPathFactory Fbrica para criar o caminho dos arquivos de
   *        destino. Este caminho  relativo ao valor do parmetro targetDir.
   * @param setExecutables <tt>true</tt> se as cpias devero ter permisso de
   *        execuo ou no.
   */
  @Override
  public void copyFiles(final AlgorithmVersionInfo sourceVersion,
    final List<FileInfo> files, final IPathFactory sourcePathFactory,
    final AlgorithmVersionInfo targetVersion, final FileInfo targetDir,
    final IPathFactory targetPathFactory, final boolean setExecutables) {

    if (null == sourceVersion)
      throw new IllegalArgumentException("sourceVersion");
    if (files == null || files.size() == 0)
      throw new IllegalArgumentException("files");
    if (null == sourcePathFactory)
      throw new IllegalArgumentException("sourcePathFactory");
    if (null == targetVersion)
      throw new IllegalArgumentException("targetVersion");
    if (null == targetPathFactory)
      throw new IllegalArgumentException("targetPathFactory");

    String sourceAlgorithmId = sourceVersion.getInfo().getId();
    Algorithm sourceAlgorithm = registeredAlgorithms.get(sourceAlgorithmId);
    if (null == sourceAlgorithm) {
      throw new ServiceFailureException(
        getAlgoNotFoundMessage(sourceAlgorithmId));
    }

    checkPermission(targetVersion.getInfo().getId());
    String targetAlgorithmId = targetVersion.getInfo().getId();
    Algorithm targetAlgorithm = registeredAlgorithms.get(targetAlgorithmId);
    if (null == targetAlgorithm) {
      throw new ServiceFailureException(
        getAlgoNotFoundMessage(targetAlgorithmId));
    }

    List<ICopyRecord> records = new ArrayList<ICopyRecord>();
    try {
      targetAlgorithm.copyFiles(files, sourcePathFactory, targetDir,
        targetPathFactory, setExecutables, records);
    }
    catch (Exception e) {
      StringBuilder fileName = new StringBuilder();
      for (int inx = 0; inx < files.size(); inx++) {
        if (inx > 0)
          fileName.append(", ");
        fileName.append(files.get(inx).getPath());
      }

      String msg = getString(ERROR_COPY);
      Object[] args = new Object[] { fileName, targetVersion.getId(),
          targetAlgorithmId };
      msg = MessageFormat.format(msg, args);
      throw new ServiceFailureException(msg, e);
    }

    /*
     * Cria entradas no histrico a reapeito dos arquivos que foram copiados.
     */
    for (ICopyRecord record : records) {
      if (record.isDirectory()) {
        CopyDirectoryRecord directoryRecord = (CopyDirectoryRecord) record;
        String path = getHistoryPath(sourceVersion, targetPathFactory,
          directoryRecord.getDirectory());

        String description = MessageFormat.format(
          getString(HISTORY_COPY_DIRECTORY), new Object[] { directoryRecord
            .getDirectory().getPath() });
        appendHistory(Utilities.splitProjectPath(path), description);
      }
      else {
        CopyFileRecord fileRecord = (CopyFileRecord) record;
        String sourcePath = getHistoryPath(sourceVersion, sourcePathFactory,
          fileRecord.getSourceFile());
        String targetPath = getHistoryPath(targetVersion, targetPathFactory,
          fileRecord.getTargetFile());

        String description = MessageFormat.format(getString(HISTORY_COPY_FILE),
          new Object[] { fileRecord.getTargetFile().getPath(), sourcePath });

        appendHistory(Utilities.splitProjectPath(targetPath), description);
      }
    }

    /*
     * Se algum arquivo de destino tiver sido criado, notifica mudana no
     * algoritmo.
     */
    for (ICopyRecord record : records) {
      if (record.wasCreated()) {
        fireModifyEvent(targetAlgorithm.getInfo());
        break;
      }
    }
  }

  /**
   * Cria no repositrio a estrutura para um novo algoritmo.
   * 
   * @param name nome do novo algoritmo
   * @param atributeValues .
   * 
   * @return informaes do novo algoritmo
   */
  public AlgorithmInfo createAlgorithm(String name,
    Hashtable<String, String> atributeValues) {
    checkPermission(name);
    Algorithm algo;
    try {
      algo = Algorithm.createAlgorithm(name, atributeValues);
    }
    catch (Exception e) {
      logSevereMessage("Falha na criao de algoritmo [" + name + "]", e);
      return null;
    }
    AlgorithmInfo algoInfo = algo.getInfo();
    registeredAlgorithms.put(algoInfo.getId(), algo);
    logInfoMessage("Algoritmo " + name + " criado em "
      + algoInfo.getDirectory());
    String[] path = new String[] { getRootDirName() };
    String description = MessageFormat.format(
      getString(HISTORY_CREATE_ALGORITHM), new Object[] { name });
    appendHistory(path, description);
    AlgoEvent action = new AlgoEvent(AlgoEvent.CREATE, algoInfo);
    setChanged();
    notifyObservers(action);
    return algoInfo;
  }

  /**
   * {@inheritDoc}
   */
  public AlgorithmConfigurator createAlgorithmConfigurator(
    String algorithmName, AlgorithmVersionId algorithmVersionId)
    throws AlgorithmNotFoundException {
    if (algorithmName == null) {
      throw new IllegalArgumentException("O parmetro algorithmName est nulo.");
    }
    if (algorithmVersionId == null) {
      throw new IllegalArgumentException(
        "O parmetro algorithmVersionId est nulo.");
    }
    Algorithm algorithm = getAlgorithm(algorithmName);
    AlgorithmInfo algorithmInfo = algorithm.getInfo();
    AlgorithmVersionInfo version = algorithmInfo
      .getVersionInfo(algorithmVersionId);
    if (version == null) {
      throw new AlgorithmNotFoundException(String.format(
        getString("AlgoService.error.version.not.found"), algorithmVersionId,
        algorithmName));
    }

    try {
      SimpleAlgorithmConfigurator simpleConfigurator = createSimpleAlgorithmConfigurator(version);
      if (simpleConfigurator != null) {
        return simpleConfigurator;
      }
      FlowAlgorithmConfigurator flowConfigurator = createFlowAlgorithmConfigurator(version);
      if (flowConfigurator != null) {
        return flowConfigurator;
      }
      AlgorithmConfigurator configurator = createCustomAlgorithmConfigurator(version);
      if (configurator != null) {
        return configurator;
      }
    }
    catch (ParseException e) {
      throw new ServiceFailureException(e.getLocalizedMessage(), e);
    }
    catch (OperationFailureException e) {
      throw new ServiceFailureException(e.getMessage(), e);
    }
    throw new ServiceFailureException(getString("error_no_component"), version
      .getInfo(), version, AlgorithmVersionInfo.CONFIG_FILE,
      AlgorithmVersionInfo.FLOW_CONFIG_FILE);
  }

  /**
   * {@inheritDoc}
   */
  public AlgorithmInfo createVersion(Object algoId, int major, int minor,
    int patch, Hashtable<String, String> properties) {
    checkPermission(algoId);
    Algorithm algo = registeredAlgorithms.get(algoId);
    if (algo == null) {
      logWarningMessage("Tentativa de criao de verso para algoritmo inexistente: "
        + algoId);
      return null;
    }
    Object versionId;
    try {
      versionId = algo.createVersion(major, minor, patch, properties);
    }
    catch (ServerException se) {
      logSevereMessage("Falha na criao de verso para algoritmo "
        + algo.getInfo().getName(), se);
      return null;
    }
    String[] path = new String[] { getRootDirName(), algo.getInfo().getName() };
    String description = MessageFormat.format(
      getString(HISTORY_CREATE_VERSION), new Object[] { versionId });
    appendHistory(path, description);
    logInfoMessage("Verso " + versionId + " criada para algoritmo "
      + algo.getInfo().getName());
    fireModifyEvent(algo.getInfo());
    return algo.getInfo();
  }

  /**
   * {@inheritDoc}
   */
  public void duplicateVersion(Object algorithmId, Object sourceVersionId,
    int majorTo, int minorTo, int patchTo) {
    checkPermission(algorithmId);
    // Verifica a existncia do algoritmo
    Algorithm algorithm = registeredAlgorithms.get(algorithmId);
    if (algorithm == null) {
      throw new ServiceFailureException(getAlgoNotFoundMessage(algorithmId));
    }

    // Verifica a existncia da verso de origem
    AlgorithmInfo algorithmInfo = algorithm.getInfo();
    AlgorithmVersionInfo sourceVersion = algorithmInfo
      .getVersionInfo(sourceVersionId);
    if (sourceVersion == null) {
      throw new ServiceFailureException(getVersionNotFoundMessage(algorithmId,
        sourceVersion));
    }
    Map<String, String> properties = sourceVersion.getPropertyValues();
    try {
      // Pega referncia para a verso de destino
      Object versionIdTo = algorithm.createVersion(majorTo, minorTo, patchTo,
        properties);
      AlgorithmVersionInfo targetVersion = algorithmInfo
        .getVersionInfo(versionIdTo);

      // Copia os arquivos de documentao
      List<FileInfo> docs = sourceVersion.getDocumentation();
      if (docs != null) {
        IPathFactory sourcePathFactory = new DocumentationPathFactory(
          sourceVersion);
        IPathFactory targetPathFactory = new DocumentationPathFactory(
          targetVersion);
        FileInfo targetDir = null; // Grava tudo a partir da raiz do diretrio
        // de documentao.
        List<ICopyRecord> records = null; // No tenho interesse em saber o
        // resultado de cada cpia.
        algorithm.copyFilesWithoutReloading(docs, sourcePathFactory, targetDir,
          targetPathFactory, false, records);
      }

      // Copia os arquivos de configurao
      List<FileInfo> configs = sourceVersion.getConfigurators();
      if (configs != null) {
        IPathFactory sourcePathFactory = new ConfigurationPathFactory(
          sourceVersion);
        IPathFactory targetPathFactory = new ConfigurationPathFactory(
          targetVersion);
        FileInfo targetDir = null; // Grava tudo a partir da raiz do diretrio
        // dos configuradores.
        List<ICopyRecord> records = null; // No tenho interesse em saber o
        // resultado de cada cpia.
        algorithm.copyFilesWithoutReloading(configs, sourcePathFactory,
          targetDir, targetPathFactory, false, records);
      }

      // Copia os executveis
      Map<String, List<FileInfo>> platforms = sourceVersion.getPlatforms();
      if (platforms != null) {
        // Para cada plataforma, insere a plataforma e os executaveis
        for (String platform : platforms.keySet()) {
          algorithm.includePlatform(targetVersion.getId(), platform);

          List<FileInfo> execs = platforms.get(platform);
          IPathFactory sourcePathFactory = new ExecutablePathFactory(
            sourceVersion, platform);
          IPathFactory targetPathFactory = new ExecutablePathFactory(
            targetVersion, platform);
          FileInfo targetDir = null; // Grava tudo a partir da raiz do diretrio
          // dos configuradores.
          List<ICopyRecord> records = null; // No tenho interesse em saber o
          // resultado de cada cpia.
          algorithm.copyFilesWithoutReloading(execs, sourcePathFactory,
            targetDir, targetPathFactory, true, records);
        }
      }
      algorithm.reload();

      String[] path = new String[] { getRootDirName(),
          algorithm.getInfo().getName() };
      String description = MessageFormat.format(
        getString(HISTORY_DUPLICATE_VERSION), new Object[] { sourceVersion,
            versionIdTo });
      appendHistory(path, description);

      // Notifica mudana no algoritmo
      fireModifyEvent(algorithmInfo);
    }
    catch (Exception e) {
      logSevereMessage("duplicateVersion: " + e.getMessage(), e);
      throw new ServiceFailureException(
        "Falha na criao de verso para algoritmo "
          + algorithm.getInfo().getName(), e);
    }
  }

  /**
   * {@inheritDoc}
   */
  public List<AlgorithmProperty> getAlgorithmProperties() {
    return Algorithm.getAlgorithmProperties();
  }

  /**
   * {@inheritDoc}
   */
  public CategorySet getAllCategories() {
    return categorySet;
  }

  /**
   * {@inheritDoc}
   */
  public Object[] getAllIds() {
    int size = registeredAlgorithms.size();
    Object[] send = new Object[size];
    int index = 0;
    for (String id : registeredAlgorithms.keySet()) {
      send[index] = id;
      index++;
    }
    return send;
  }

  /**
   * {@inheritDoc}
   */
  public AlgorithmInfo[] getAllInfo() {
    Enumeration<Algorithm> elements = registeredAlgorithms.elements();
    int size = registeredAlgorithms.size();
    AlgorithmInfo[] send = new AlgorithmInfo[size];
    int index = 0;
    while (elements.hasMoreElements()) {
      Algorithm algo = elements.nextElement();
      send[index] = algo.getInfo();
      index++;
    }
    return send;
  }

  /**
   * {@inheritDoc}
   */
  public AlgorithmOutline[] getAllOutlines() {
    if (registeredAlgorithms == null) {
      return null;
    }
    Enumeration<Algorithm> elements = registeredAlgorithms.elements();
    int size = registeredAlgorithms.size();
    AlgorithmOutline[] send = new AlgorithmOutline[size];
    int index = 0;
    while (elements.hasMoreElements()) {
      Algorithm algo = elements.nextElement();
      send[index] = algo.getOutline();
      index++;
    }
    return send;
  }

  /**
   * {@inheritDoc}
   */
  public AlgorithmInfo getInfo(Object id) {
    if (id == null) {
      return null;
    }
    Algorithm algo = registeredAlgorithms.get(id);
    if (algo == null) {
      return null;
    }
    return algo.getInfo();
  }

  /**
   * {@inheritDoc}
   */
  public AlgorithmInfo getInfo(String name) {
    Collection<Algorithm> algorithmCollection = registeredAlgorithms.values();
    Iterator<Algorithm> algorithmIterator = algorithmCollection.iterator();
    while (algorithmIterator.hasNext()) {
      Algorithm algo = algorithmIterator.next();
      AlgorithmInfo info = algo.getInfo();
      if (info.getName().equals(name)) {
        return info;
      }
    }
    return null;
  }

  /**
   * {@inheritDoc}
   */
  public List<AlgorithmProperty> getVersionProperties() {
    return Algorithm.getVersionProperties();
  }

  /**
   * {@inheritDoc}
   */
  public AlgorithmInfo includePlatform(Object algoId, Object versionId,
    String platform) {
    checkPermission(algoId);
    Algorithm algo = registeredAlgorithms.get(algoId);
    if (algo == null) {
      logWarningMessage("Tentativa de alterao de verso (incluso plat) de "
        + "algoritmo inexistente: " + algoId);
      return null;
    }
    try {
      algo.includePlatform(versionId, platform);
    }
    catch (ServerException se) {
      logSevereMessage("Falha na criao da plataforma " + platform
        + " para verso " + versionId + " do algoritmo "
        + algo.getInfo().getName(), se);
      return null;
    }
    String[] path = new String[] { getRootDirName(), algo.getInfo().getName(),
        versionId.toString(), AlgorithmVersionInfo.BIN_DIR };
    String description = MessageFormat.format(
      getString(HISTORY_INCLUDE_PLATFORM), new Object[] { platform });
    appendHistory(path, description);
    logInfoMessage("Plataforma " + platform + " inserida para verso "
      + versionId + " do algoritmo " + algo.getInfo().getName());
    fireModifyEvent(algo.getInfo());
    return algo.getInfo();
  }

  /**
   * Inicializao do servio.
   * 
   * @throws ServerException se ocorrer um erro durante a inicializao.
   */
  @Override
  public void initService() throws ServerException {
    transaction = new TransactionManager();
    int reloadAlgsIntervalInMinutes = getIntProperty(PROP_RELOAD_ALGS_INTERVAL_IN_MIN);
    logInfoMessage("Intervalo de recarga de algoritmos (em min): "
      + reloadAlgsIntervalInMinutes);
    if (reloadAlgsIntervalInMinutes != 0) {
      reloadAlgsThread = new ReloadAlgsThread(reloadAlgsIntervalInMinutes);
      logDebugMessage("Thread de recarga de algoritmos criada.");
    }
    loadParameters();
    loadAlgorithms();
    isFirstCategoryLoad = true;
    loadCategories();
    initHistoryTable();
    logInfoMessage("Servio de algoritmos iniciado.");
  }

  /**
   * Verifica se h transao ativa de gerncia do repositrio.
   * 
   * @return true caso haja transao ativa, false caso contrrio.
   */
  public boolean isLocked() {
    return transaction.isLocked();
  }

  /**
   * Inicia uma transao de gerncia do repositrio de algoritmos (equivale 
   * obteno de um "lock").
   * 
   * @param cb Callback que  utilizada pelo servio para saber se quem efetuou
   *        o lock ainda est respondendo.
   * 
   * @return true caso o lock tenha sido feito com sucesso, false caso
   *         contrrio.
   */
  public boolean lock(TransactionCallbackInterface cb) {
    checkPermission();
    return transaction.lock(cb);
  }

  /**
   * {@inheritDoc}
   */
  public boolean reloadAlgorithms() {
    return reloadAlgorithms(true);
  }

  /**
   * Recarrega os algoritmos.
   * 
   * @param notifyAllUsers true indica que todos os usurios devem ser
   *        notificados, false apenas o usurio que solicitou a recarga
   * @return true caso a recarga tenha sido efetuada com sucesso
   */
  public boolean reloadAlgorithms(boolean notifyAllUsers) {
    checkPermission();
    TransactionCallbackInterface cb = new TransactionCallbackInterface() {
      public boolean isAlive() {
        return true;
      }
    };
    final String uName = getUser().getName();
    logInfoMessage("Pedido de recarga de algoritmos vinda de " + uName);
    try {
      if (!lock(cb)) {
        logSevereMessage("Recarga falhou por bloqueio do servio!");
        return false;
      }
      loadParameters();
      loadAlgorithms();
      loadCategories();
      unlock(cb);
      logInfoMessage("Recarga de algoritmos efetuada!");
      final NotificationService notService = NotificationService.getInstance();
      final AlgorithmsReloadNotificationData data = new AlgorithmsReloadNotificationData();
      if (notifyAllUsers) {
        notService.notifyAllUsersFromService(this, data);
      }
      else {
        notService.notifyUserFromService(this, getUser().getId(), data);
      }
      return true;
    }
    catch (ServerException se) {
      logSevereMessage("Falha na recarga! Exceo: " + se, se);
      return false;
    }
  }

  /**
   * {@inheritDoc}
   */
  public boolean removeAlgorithm(Object id) {
    checkPermission(id);
    Algorithm algo = registeredAlgorithms.get(id);
    if (algo == null) {
      logWarningMessage("Tentativa de remoo de algoritmo inexistente: " + id);
      return false;
    }
    try {
      algo.remove();
    }
    catch (ServerException se) {
      logSevereMessage("Falha na remoo de algoritmo [" + id + "]", se);
      return false;
    }
    registeredAlgorithms.remove(id);
    AlgorithmInfo info = algo.getInfo();
    String[] path = new String[] { getRootDirName() };
    String description = MessageFormat.format(
      getString(HISTORY_REMOVE_ALGORITHM), new Object[] { id });
    appendHistory(path, description);
    logInfoMessage("Algoritmo " + id + " removido");
    AlgoEvent action = new AlgoEvent(AlgoEvent.DELETE, info);
    setChanged();
    notifyObservers(action);
    return true;
  }

  /**
   * {@inheritDoc}
   * 
   * @throws IllegalArgumentException
   */
  public void removeFile(Object algoId, Object versionId, String[][] filePaths) {
    if (filePaths == null) {
      throw new IllegalArgumentException("filePaths == null");
    }
    if (filePaths.length == 0) {
      throw new IllegalArgumentException("filePaths.length == 0");
    }
    checkPermission(algoId);
    Algorithm algo = registeredAlgorithms.get(algoId);
    if (algo == null) {
      throw new ServiceFailureException(getAlgoNotFoundMessage(algoId));
    }
    try {
      algo.removeFile(versionId, filePaths);
      // ATENO: Esta implementao assume que remoes mltiplas sempre
      // ocorrero no mesmo diretrio - de acordo com a interface atual de
      // administrao. Caso esta premissa venha a mudar, o cdigo a seguir
      // ter de ser revisto.
      int pathLen = filePaths[0].length - 2; // Exclui dir "versions" e o
      // arquivo
      String[] path = new String[pathLen];
      path[0] = getRootDirName();
      path[1] = algo.getInfo().getName();
      path[2] = versionId.toString();
      path[3] = filePaths[0][4]; // Diretrio "versions"  ignorado
      if (pathLen == 5) { // Remoo de executvel
        path[4] = filePaths[0][5]; // Caminho da plataforma
      }
      String[] names = new String[filePaths.length];
      for (int i = 0; i < names.length; i++) {
        names[i] = filePaths[i][filePaths[i].length - 1];
        String description = MessageFormat.format(
          getString(HISTORY_REMOVE_FILE), new Object[] { names[i] });
        appendHistory(path, description);
      }
      fireModifyEvent(algo.getInfo());
    }
    catch (Exception e) {
      throw new ServiceFailureException(e.getMessage(), e);
    }
  }

  /**
   * Remove arquivos executveis de uma verso de algoritmo.
   * 
   * @param version Verso que detm os arquivos.
   * @param platform Plataforma dos executveis
   * @param files Representao dos arquivos a serem removidos.
   * 
   * @throws ServiceFailureException caso no tenha sido possvel remover o
   *         arquivo.
   */
  public void removeExecutableFiles(AlgorithmVersionInfo version,
    String platform, FileInfo[] files) {

    if (null == version)
      throw new IllegalArgumentException("version == null");
    if (null == platform)
      throw new IllegalArgumentException("platform == null");
    if (null == files)
      throw new IllegalArgumentException("files == null");
    if (0 == files.length)
      throw new IllegalArgumentException("files.length == 0");

    String algorithmId = version.getInfo().getId();
    checkPermission(version.getInfo().getId());
    Algorithm algorithm = registeredAlgorithms.get(algorithmId);
    if (algorithm == null) {
      throw new ServiceFailureException(getAlgoNotFoundMessage(algorithmId));
    }

    try {
      algorithm.removeExecutableFiles(version, platform, files);

      String[] basePath = new String[5];
      basePath[0] = getRootDirName();
      basePath[1] = algorithm.getInfo().getName();
      basePath[2] = version.getId().toString();
      basePath[3] = version.getBinDirName();
      basePath[4] = platform;
      for (FileInfo file : files) {
        String description = MessageFormat.format(
          getString(HISTORY_REMOVE_FILE), new Object[] { file.getPath() });

        appendHistory(basePath, description);
      }

      fireModifyEvent(algorithm.getInfo());
    }
    catch (Exception e) {
      throw new ServiceFailureException(e.getMessage(), e);
    }
  }

  /**
   * Remove arquivos de configurao de uma verso de algoritmo.
   * 
   * @param version Verso que detm os arquivos.
   * @param files Representao dos arquivos a serem removidos.
   * 
   * @throws ServiceFailureException caso no tenha sido possvel remover o
   *         arquivo.
   */
  public void removeConfigurationFiles(AlgorithmVersionInfo version,
    FileInfo[] files) {

    if (null == version)
      throw new IllegalArgumentException("version == null");
    if (null == files)
      throw new IllegalArgumentException("files == null");
    if (0 == files.length)
      throw new IllegalArgumentException("files.length == 0");

    String algorithmId = version.getInfo().getId();
    checkPermission(version.getInfo().getId());
    Algorithm algorithm = registeredAlgorithms.get(algorithmId);
    if (algorithm == null) {
      throw new ServiceFailureException(getAlgoNotFoundMessage(algorithmId));
    }

    try {
      algorithm.removeConfigurationFiles(version, files);

      String[] basePath = new String[4];
      basePath[0] = getRootDirName();
      basePath[1] = algorithm.getInfo().getName();
      basePath[2] = version.getId().toString();
      basePath[3] = version.getConfiguratorDirName();
      for (FileInfo file : files) {
        String description = MessageFormat.format(
          getString(HISTORY_REMOVE_FILE), new Object[] { file.getPath() });

        appendHistory(basePath, description);
      }

      fireModifyEvent(algorithm.getInfo());
    }
    catch (Exception e) {
      throw new ServiceFailureException(e.getMessage(), e);
    }
  }

  /**
   * Remove arquivos de documentao de uma verso de algoritmo.
   * 
   * @param version Verso que detm os arquivos.
   * @param files Representao dos arquivos a serem removidos.
   * 
   * @throws ServiceFailureException caso no tenha sido possvel remover o
   *         arquivo.
   */
  public void removeDocumentationFiles(AlgorithmVersionInfo version,
    FileInfo[] files) {

    if (null == version)
      throw new IllegalArgumentException("version == null");
    if (null == files)
      throw new IllegalArgumentException("files == null");
    if (0 == files.length)
      throw new IllegalArgumentException("files.length == 0");

    String algorithmId = version.getInfo().getId();
    checkPermission(version.getInfo().getId());
    Algorithm algorithm = registeredAlgorithms.get(algorithmId);
    if (algorithm == null) {
      throw new ServiceFailureException(getAlgoNotFoundMessage(algorithmId));
    }

    try {
      algorithm.removeDocumentationFiles(version, files);

      String[] basePath = new String[4];
      basePath[0] = getRootDirName();
      basePath[1] = algorithm.getInfo().getName();
      basePath[2] = version.getId().toString();
      basePath[3] = version.getDocumentationDirName();
      for (FileInfo file : files) {
        String description = MessageFormat.format(
          getString(HISTORY_REMOVE_FILE), new Object[] { file.getPath() });

        appendHistory(basePath, description);
      }

      fireModifyEvent(algorithm.getInfo());
    }
    catch (Exception e) {
      throw new ServiceFailureException(e.getMessage(), e);
    }
  }

  /**
   * {@inheritDoc}
   */
  public AlgorithmInfo removePlatform(Object algoId, Object versionId,
    String platform) {
    checkPermission(algoId);
    Algorithm algo = registeredAlgorithms.get(algoId);
    if (algo == null) {
      logWarningMessage("Tentativa de alterao de verso (remoo plat) de "
        + "algoritmo inexistente: " + algoId);
      return null;
    }
    try {
      algo.removePlatform(versionId, platform);
    }
    catch (ServerException se) {
      logSevereMessage("Falha na remoo da plataforma " + platform
        + " da verso " + versionId + " do algoritmo "
        + algo.getInfo().getName(), se);
      return null;
    }
    String[] path = new String[] { getRootDirName(), algo.getInfo().getName(),
        versionId.toString(), AlgorithmVersionInfo.BIN_DIR };
    String description = MessageFormat.format(
      getString(HISTORY_REMOVE_PLATFORM), new Object[] { platform });
    appendHistory(path, description);
    logInfoMessage("Plataforma " + platform + " removida da verso "
      + versionId + " do algoritmo " + algo.getInfo().getName());
    fireModifyEvent(algo.getInfo());
    return algo.getInfo();
  }

  /**
   * {@inheritDoc}
   */
  public AlgorithmInfo removeVersion(Object algoId, Object versionId) {
    checkPermission(algoId);
    Algorithm algo = registeredAlgorithms.get(algoId);
    if (algo == null) {
      logWarningMessage("Tentativa de remoo de verso de "
        + "algoritmo inexistente: " + algoId);
      return null;
    }
    try {
      algo.removeVersion(versionId);
    }
    catch (ServerException se) {
      logSevereMessage("Falha na remoo da verso " + versionId
        + " do algoritmo " + algo.getInfo().getName(), se);
      return null;
    }
    String[] path = new String[] { getRootDirName(), algo.getInfo().getName() };
    String description = MessageFormat.format(
      getString(HISTORY_REMOVE_VERSION), new Object[] { versionId });
    appendHistory(path, description);
    logInfoMessage("Verso " + versionId + " removida do algoritmo "
      + algo.getInfo().getName());
    fireModifyEvent(algo.getInfo());
    return algo.getInfo();
  }

  // #TODO A remover quando todos sistemas CSBase forem assinados {
  /**
   * {@inheritDoc}
   */
  public String retrieveConfigUploadURL(Object algoId, Object versionId) {
    checkPermission(algoId);
    Algorithm algo = registeredAlgorithms.get(algoId);
    AlgorithmInfo info = algo.getInfo();
    AlgorithmVersionInfo versionInfo = info.getVersionInfo(versionId);
    String algoPath = versionInfo.getConfiguratorDirPath();
    try {
      File file = new File(algoPath);
      if (!file.exists()) {
        String msg = getString(ERROR_CONF_VERSION_NOT_FOUND);
        Object[] args = new Object[] { algoId, versionId };
        msg = MessageFormat.format(msg, args);
        throw new ServiceFailureException(msg);
      }
      String canonicalPath = file.getCanonicalPath();
      AlgorithmObserver obs = new AlgorithmObserver(algoId, versionId,
        AlgorithmObserver.UPLOAD_CONFIG, Service.getUser().getId());
      String uploadPresentation = getStringProperty("uploadConfigPresentation");
      String uploadResult = getStringProperty("uploadResult");
      HttpService httpService = getHttpService();
      return httpService.getUploadURL(canonicalPath, uploadPresentation,
        uploadResult, obs);
    }
    catch (IOException ex) {
      throw new ServiceFailureException(ex.getMessage(), ex);
    }
    catch (OperationFailureException e) {
      throw new ServiceFailureException(e.getMessage(), e);
    }
  }

  /**
   * {@inheritDoc}
   */
  public String retrieveDocUploadURL(Object algoId, Object versionId) {
    checkPermission(algoId);
    Algorithm algo = registeredAlgorithms.get(algoId);
    if (algo == null) {
      throw new ServiceFailureException(getAlgoNotFoundMessage(algoId));
    }
    AlgorithmInfo info = algo.getInfo();
    AlgorithmVersionInfo versionInfo = info.getVersionInfo(versionId);
    String docPath = versionInfo.getDocDirPath();
    try {
      File file = new File(docPath);
      if (!file.exists()) {
        String msg = getString(ERROR_DOC_VERSION_NOT_FOUND);
        Object[] args = new Object[] { algoId, versionId };
        msg = MessageFormat.format(msg, args);
        throw new ServiceFailureException(msg);
      }
      String canonicalPath = file.getCanonicalPath();
      AlgorithmObserver obs = new AlgorithmObserver(algoId, versionId,
        AlgorithmObserver.UPLOAD_HTML, Service.getUser().getId());
      String uploadPresentation = getStringProperty("uploadHtmlPresentation");
      String uploadResult = getStringProperty("uploadResult");
      HttpService httpService = getHttpService();
      return httpService.getUploadURL(canonicalPath, uploadPresentation,
        uploadResult, obs);
    }
    catch (IOException ex) {
      throw new ServiceFailureException(ex.getMessage(), ex);
    }
    catch (OperationFailureException e) {
      throw new ServiceFailureException(e.getMessage(), e);
    }
  }

  // #TODO } A remover quando todos sistemas CSBase forem assinados

  /**
   * {@inheritDoc}
   */
  public String retrieveDownloadURL(Object algoId, String[] filePath) {
    Algorithm algo = registeredAlgorithms.get(algoId);
    if (algo == null) {
      throw new ServiceFailureException(getAlgoNotFoundMessage(algoId));
    }
    String filePathText = Utilities.makeRelativePath(filePath);
    File file = new File(filePathText);
    String absFilePath = file.getAbsolutePath();
    try {
      HttpService httpService = getHttpService();
      return httpService.getFileDownloadURL(absFilePath);
    }
    catch (IOException e) {
      throw new ServiceFailureException(e.getMessage(), e);
    }
    catch (OperationFailureException e) {
      throw new ServiceFailureException(e.getMessage(), e);
    }
  }

  // #TODO A remover quando todos sistemas CSBase forem assinados {
  /**
   * {@inheritDoc}
   */
  public String retrieveExecUploadURL(Object algoId, Object versionId,
    String platform) {
    checkPermission(algoId);
    Algorithm algo = registeredAlgorithms.get(algoId);
    AlgorithmInfo info = algo.getInfo();
    AlgorithmVersionInfo versionInfo = info.getVersionInfo(versionId);
    String platPath = versionInfo.getPlatformPath(platform);
    try {
      File file = new File(platPath);
      if (!file.exists()) {
        String msg = getString(ERROR_PLAT_VERSION_NOT_FOUND);
        Object[] args = new Object[] { algoId, versionId, platform };
        msg = MessageFormat.format(msg, args);
        throw new ServiceFailureException(msg);
      }
      String canonicalPath = file.getCanonicalPath();
      AlgorithmObserver obs = new AlgorithmObserver(algoId, versionId,
        platform, Service.getUser().getId());
      String uploadPresentation = getStringProperty("uploadExecPresentation");
      String uploadResult = getStringProperty("uploadResult");
      HttpService httpService = getHttpService();
      return httpService.getUploadURL(canonicalPath, uploadPresentation,
        uploadResult, obs);
    }
    catch (IOException ex) {
      throw new ServiceFailureException(ex.getMessage(), ex);
    }
    catch (OperationFailureException e) {
      throw new ServiceFailureException(e.getMessage(), e);
    }
  }

  /**
   * {@inheritDoc}
   */
  public String retrieveVersionUploadURL(final Object algoId) {
    checkPermission(algoId);
    Algorithm algo = registeredAlgorithms.get(algoId);
    AlgorithmInfo info = algo.getInfo();
    File repositoryDirectory = new File(AlgorithmInfo.REPOSITORY_NAME);
    File algorithmDirectory = new File(repositoryDirectory, info.getDirectory());
    File file = new File(algorithmDirectory, AlgorithmInfo.VERSIONS_DIR);
    if (!file.exists()) {
      String userMessage = String.format(
        "O repositrio do algoritmo %s est inconsistente.\n", info.getName());
      String logDetails = String.format(
        "\nDetalhes: o diretrio %s no foi encontrado.\n", file
          .getAbsolutePath());
      logSevereMessage(userMessage + logDetails);
      throw new ServiceFailureException(userMessage);
    }
    final String directoryPath;
    try {
      directoryPath = file.getCanonicalPath();
    }
    catch (IOException e) {
      String userMessage = String.format(
        "Erro ao importar verso do algoritmo %s.\n", info.getName());
      logSevereMessage(userMessage, e);
      throw new ServiceFailureException(userMessage);
    }
    User user = Service.getUser();
    final Object userId = user.getId();
    String uploadPresentation = getStringProperty("uploadVersionPresentation");
    String uploadResult = getStringProperty("uploadResult");
    try {
      return getHttpService().getUploadURL(directoryPath, uploadPresentation,
        uploadResult, new UploadHandler() {

          public void execute(String filePath) throws HttpServiceException {
            try {
              Algorithm algorithm = registeredAlgorithms.get(algoId);
              if (algorithm == null) {
                String detailedMessage = String.format(
                  "O algoritmo %s no foi encontrado ao tentar atualizar a verso"
                    + " com o arquivo %s.\n", algoId, filePath);
                logSevereMessage(detailedMessage);
                throw new HttpServiceException(
                  "O algoritmo da verso que est sendo importada no foi entrada.\n"
                    + "O repositrio de algoritmos pode estar incosistente ou \n"
                    + "Ele foi removido durante esta operao.");
              }
              Unzip unzip = new Unzip(new File(filePath));
              List<ZipEntry> zipEntries = unzip.listZipEntries();
              if (zipEntries.isEmpty()) {
                String userMessage = "O arquivo informado no  um pacote de verso de algoritmo.\n";
                String detailedMessage = String
                  .format(
                    "%sDetalhes: o arquivo zip est vazio.\n"
                      + "Arquivo utilizado: %s.\nIdentificador do algoritmo: %s.\n"
                      + "Identificador do usurio: %s.\n", userMessage,
                    filePath, algoId, userId);
                logSevereMessage(detailedMessage);
                throw new HttpServiceException(userMessage);
              }

              ZipEntry rootZipEntry = null;
              String rootDirectoryName = null;
              for (ZipEntry zipEntry : zipEntries) {
                String[] path = FileUtils.splitPath(zipEntry.getName(), "/");
                if (path.length == 1) {
                  if (rootZipEntry != null) {
                    String userMessage = "O arquivo informado no  um pacote de verso de algoritmo.\n";
                    String detailedMessage = String
                      .format(
                        "%sDetalhes: o arquivo possui 2 ou mais arquivos/diretrios na raz.\n"
                          + "Arquivo utilizado: %s.\nIdentificador do algoritmo: %s.\n"
                          + "Identificador do usurio: %s.\n", userMessage,
                        filePath, algoId, userId);
                    logSevereMessage(detailedMessage);
                    throw new HttpServiceException(userMessage);
                  }
                  rootZipEntry = zipEntry;
                  rootDirectoryName = zipEntry.getName();
                  rootDirectoryName = rootDirectoryName.replace("/", "");
                }
              }

              if (rootZipEntry == null) {
                String userMessage = "O arquivo informado no  um pacote de verso de algoritmo.\n";
                String detailedMessage = String
                  .format(
                    "%sDetalhes: o arquivo no possui o nome do diretrio raz da verso.\n"
                      + "Arquivo utilizado: %s.\nIdentificador do algoritmo: %s.\n"
                      + "Identificador do usurio: %s.\n", userMessage,
                    filePath, algoId, userId);
                logSevereMessage(detailedMessage);
                throw new HttpServiceException(userMessage);
              }

              if (!rootZipEntry.isDirectory()) {
                String userMessage = "O arquivo informado no  um pacote de verso de algoritmo.\n";
                String detailedMessage = String
                  .format(
                    "%sDetalhes: o arquivo no possui um diretrio raz da verso.\n"
                      + "Arquivo utilizado: %s.\nIdentificador do algoritmo: %s.\n"
                      + "Identificador do usurio: %s.\nArquivo na raz: %s.\n",
                    userMessage, filePath, algoId, userId, rootZipEntry);
                logSevereMessage(detailedMessage);
                throw new HttpServiceException(userMessage);
              }

              String regex = "v_([0-9]{3})_([0-9]{3})_([0-9]{3})";
              Pattern versionPattern = Pattern.compile(regex);
              Matcher versionMatcher = versionPattern
                .matcher(rootDirectoryName);
              if (!versionMatcher.matches()) {
                String userMessage = "O arquivo informado no  um pacote de verso de algoritmo.\n";
                String detailedMessage = String
                  .format(
                    "%sDetalhes: o nome do diretrio raz no est no formato vlido.\n"
                      + "Arquivo utilizado: %s.\nIdentificador do algoritmo: %s.\n"
                      + "Identificador do usurio: %s.\nNome do arquivo: %s.\n"
                      + "Expresso regular do padro: %s.\n", userMessage,
                    filePath, algoId, userId, rootDirectoryName, regex);
                logSevereMessage(detailedMessage);
                throw new HttpServiceException(userMessage);
              }

              int major = Integer.parseInt(versionMatcher.group(1));
              int minor = Integer.parseInt(versionMatcher.group(2));
              int patch = Integer.parseInt(versionMatcher.group(3));

              AlgorithmVersionId algorithmVersionId = new AlgorithmVersionId(
                major, minor, patch);
              AlgorithmInfo algorithmInfo = algorithm.getInfo();
              String algorithmName = algorithmInfo.getName();
              if (algorithmInfo.getVersionInfo(algorithmVersionId) != null) {
                String userMessage = String.format(
                  "Esta verso j est instalada no sistema \n"
                    + "(Algoritmo %s %d.%d.%d).\n", algorithmName, major,
                  minor, patch);
                String detailedMessage = String.format(
                  "%sIdentificador do algoritmo: %s\nArquivo utilizado: %s.\n"
                    + "Identificador do usurio: %s.\n", userMessage, filePath,
                  algoId, userId);
                logSevereMessage(detailedMessage);
                throw new HttpServiceException(userMessage);
              }

              File repositoryDirectory = new File(AlgorithmInfo.REPOSITORY_NAME);
              File algorithmDirectory = new File(repositoryDirectory,
                algorithmInfo.getDirectory());
              File versionsDirectory = new File(algorithmDirectory,
                AlgorithmInfo.VERSIONS_DIR);

              unzip.decompress(versionsDirectory);

              if (!new File(filePath).delete()) {
                String detailedMessage = String.format(
                  "Erro ao tentar excluir o arquivo %s.\n"
                    + "Por favor, tente exclu-lo manualmente.\n", filePath);
                logWarningMessage(detailedMessage);
              }

              File versionDirectory = new File(versionsDirectory,
                rootDirectoryName);

              File binaryDirectory = new File(versionDirectory,
                AlgorithmVersionInfo.BIN_DIR);

              if (!binaryDirectory.isDirectory()) {
                String userMessage = "O arquivo informado no  um pacote de verso de algoritmo.\n";
                String detailedMessage = String
                  .format(
                    "%s\nDiretrio de binrios ausente.\n"
                      + "Identificador do algoritmo: %s\nArquivo utilizado: %s.\n"
                      + "Identificador do usurio: %s.\n", userMessage,
                    filePath, algoId, userId);
                logSevereMessage(detailedMessage);
                if (!FileUtils.delete(versionDirectory)) {
                  detailedMessage = String.format(
                    "Erro ao tentar excluir o arquivo %s.\n"
                      + "Por favor, tente exclu-lo manualmente.\n",
                    versionDirectory.getAbsolutePath());
                  logWarningMessage(detailedMessage);
                }
                throw new HttpServiceException(userMessage);
              }

              File[] platformDirectories = binaryDirectory.listFiles();

              for (File platformDirectory : platformDirectories) {
                if (!platformDirectory.isDirectory()) {
                  String userMessage = "O arquivo informado no  um pacote de verso de algoritmo.\n";
                  String detailedMessage = String
                    .format(
                      "%s\nO arquivo %s deveria ser um diretrio.\n"
                        + "Identificador do algoritmo: %s\nArquivo utilizado: %s.\n"
                        + "Identificador do usurio: %s.\n", userMessage,
                      platformDirectory, filePath, algoId, userId);
                  logSevereMessage(detailedMessage);
                  if (!FileUtils.delete(versionDirectory)) {
                    detailedMessage = String.format(
                      "Erro ao tentar excluir o arquivo %s.\n"
                        + "Por favor, tente exclu-lo manualmente.\n",
                      versionDirectory.getAbsolutePath());
                    logWarningMessage(detailedMessage);
                  }
                  throw new HttpServiceException(userMessage);
                }

                File[] binaryFiles = platformDirectory.listFiles();
                for (File binaryFile : binaryFiles) {
                  FileSystem.enableExecutionPermission(binaryFile
                    .getAbsolutePath());
                }
              }

              algorithm.readVersion(versionsDirectory.getAbsolutePath(),
                versionDirectory);

              fireModifyEvent(algorithmInfo);

            }
            catch (IOException e) {
              String userMessage = "Ocorreu um erro de ES ao ler ou extrair o pacote de verso.\n"
                + "Pode ser que o arquivo informado no seja um arquivo zip.\n";
              String detailedMessage = String.format(
                "%sIdentificador do algoritmo: %s\n"
                  + "Arquivo utilizado: %s.\nIdentificador do usurio: %s.\n",
                userMessage, filePath, algoId, userId);
              logSevereMessage(detailedMessage, e);
              throw new HttpServiceException(userMessage);
            }
            catch (ServerException e) {
              String userMessage = "Erro ao ler a verso do algoritmo.";
              String detailedMessage = String.format(
                "%sIdentificador do algoritmo: %s\n"
                  + "Arquivo utilizado: %s.\nIdentificador do usurio: %s.\n",
                userMessage, filePath, algoId, userId);
              logSevereMessage(detailedMessage, e);
              throw new HttpServiceException(userMessage);
            }
          }
        });
    }
    catch (OperationFailureException e) {
      throw new ServiceFailureException(e.getMessage(), e);
    }
  }

  // #TODO } A remover quando todos sistemas CSBase forem assinados

  /**
   * {@inheritDoc}
   */
  public List<HistoryRecord> retrieveHistory(String[] viewPath) {
    if (viewPath == null) {
      throw new IllegalArgumentException("viewPath == null");
    }
    Map historyTable = readHistoryTable();
    if (historyTable == null) {
      return null;
    }
    Map dirTable = getDirTable(viewPath, historyTable);
    List rawList = getDirHistory(dirTable);
    List<HistoryRecord> recordList = getRecordList(viewPath, rawList);
    return recordList;
  }

  /**
   * Trmino do servio de gerncia de algoritmos.
   */
  @Override
  public void shutdownService() {
    if (reloadAlgsThread != null) {
      reloadAlgsThread.interrupt();
    }
    if (!categorySet.isCategorySetSaved()) {
      saveCategoriesBeforeShutdown();
    }
    logInfoMessage("Shutdown...");
  }

  /**
   * Tenta salvar o conjunto de categorias antes do trmino do servio. O
   * resultado dessa operao  registrado no log do servio.
   */
  private void saveCategoriesBeforeShutdown() {
    logInfoMessage("Tentando salvar o arquivo de categorias [categories.xml] com problema na ltima persistncia...");
    try {
      if (writeCategoriesFile(categorySet)) {
        logInfoMessage("Shutdown do servio - o arquivo de categorias [categories.xml] foi salvo com sucesso.");
      }
    }
    catch (CategoriesFileNotSavedException ex) {
      logSevereMessage("Shutdown do servio - ERRO - o arquivo de categorias [categories.xml] no foi salvo.");
    }
  }

  /**
   * Termina a transao de gerncia do repositrio de algoritmos.
   * 
   * @param cb Callback que  utilizada pelo servio para saber se quem efetuou
   *        o lock ainda est respondendo. Deve ser passado o mesmo objeto
   *        inicialmente passado para o mtodo lock.
   */
  public void unlock(TransactionCallbackInterface cb) {
    checkPermission();
    transaction.unlock(cb);
  }

  /**
   * Dispara uma notificao para os observadores da classe de que determinado
   * algoritmo mudou.
   * 
   * @param info informaes sobre o algoritmo que sofreu mudanas.
   */
  protected void fireModifyEvent(AlgorithmInfo info) {
    AlgoEvent event = new AlgoEvent(AlgoEvent.MODIFY, info);
    setChanged();
    notifyObservers(event);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected boolean has2Update(Object arg, Object event) {
    return true;
  }

  // #TODO A remover quando todos sistemas CSBase forem assinados {
  /**
   * Este mtodo  chamado pelo observador quando o upload do configurador
   * termina para notificar os clientes.
   * 
   * @param algoId identificador do algoritmo.
   * @param versionId identificador da verso do algoritmo.
   * @param filePath caminho do arquivo recm-carregado.
   * @param userId identificador do usurio.
   */
  void updateConfigurator(Object algoId, Object versionId, String filePath,
    Object userId) {
    try {
      Algorithm algo = registeredAlgorithms.get(algoId);
      if (algo == null) {
        return;
      }
      AlgorithmInfo info = algo.getInfo();
      AlgorithmVersionInfo vInfo = info.getVersionInfo(versionId);
      FileInfo from = new FileInfo(new File(filePath));
      vInfo.addConfigurator(from);
      String[] path = new String[] { getRootDirName(),
          algo.getInfo().getName(), versionId.toString(),
          AlgorithmVersionInfo.CONFIGURATOR_DIR };
      String description = MessageFormat
        .format(getString(HISTORY_UPDATE_CONFIGURATOR), new Object[] { from
          .getName() });
      appendHistory(userId, path, description);
      fireModifyEvent(info);
    }
    catch (Exception e) {
      logSevereMessage("Falha no updateConfigurator: " + e.getMessage(), e);
    }
  }

  /**
   * Este mtodo  chamado pelo observador quando o upload do arquivo de
   * documentao termina para notificar os clientes.
   * 
   * @param algoId identificador do algoritmo.
   * @param versionId identificador da verso do algoritmo.
   * @param filePath caminho do arquivo recm-carregado.
   * @param userId identificador do usurio que carregou o arquivo.
   */
  void updateDoc(Object algoId, Object versionId, String filePath, Object userId) {
    try {
      Algorithm algo = registeredAlgorithms.get(algoId);
      if (algo == null) {
        return;
      }
      AlgorithmInfo info = algo.getInfo();
      AlgorithmVersionInfo versionInfo = info.getVersionInfo(versionId);
      FileInfo from = new FileInfo(new File(filePath));
      versionInfo.addDocumentation(from);
      String[] path = new String[] { getRootDirName(),
          algo.getInfo().getName(), versionId.toString(),
          AlgorithmVersionInfo.DOCUMENTATION_DIR };
      String description = MessageFormat.format(getString(HISTORY_UPDATE_DOC),
        new Object[] { from.getName() });
      appendHistory(userId, path, description);
      fireModifyEvent(info);
    }
    catch (Exception e) {
      logSevereMessage("Falha no updateDoc: " + e.getMessage(), e);
    }
  }

  /**
   * <p>
   * Este mtodo  chamado pelo observador quando o upload de um executvel
   * termina para notificar os clientes. Caso o arquivo carregado seja um zip,
   * seu contedo  extrado. Como essa operao  possivelmente demorada, ela 
   * executada por uma thread.
   * </p>
   * OBS: durante o upload, eventuais permisses de execuo so perdidas pela
   * api que realiza a carga. Para compensar essa perda, este mtodo concede
   * permisso de execuo para cada arquivo carregado. Isso gera um efeito
   * colateral indesejado: arquivos no-executveis carregados tambm tero
   * permisso de execuo. Devido  dificuldade de identificar arquivos
   * realmente executveis dentre os carregados no servidor, e principalmente
   * pelo fato de uma permisso no-intencional de execuo no afetar em nada o
   * fluxo de funcionamento normal do sistema, optou-se por no tratar essa
   * inconsistncia.
   * 
   * @param algoId Identificador do algoritmo.
   * @param versionId Identificador da verso do algoritmo.
   * @param platform Nome da plataforma.
   * @param filePath caminho do executvel recm-carregado.
   * @param userId identificador do usurio que carregou o executvel.
   * 
   * @see UpdateExecutableThread
   */
  void updateExecutable(Object algoId, Object versionId, String platform,
    String filePath, Object userId) {
    Thread t = new UpdateExecutableThread(algoId, versionId, platform,
      filePath, userId);
    t.start();
  }

  // #TODO } A remover quando todos sistemas CSBase forem assinados

  /**
   * Adiciona uma entrada ao histrico do servio de algoritmos.
   * 
   * @param userId identificador do usurio que efetuou a ao.
   * @param path caminho no qual ocorreu uma ao que deve ser registrada no
   *        histrico.
   * @param description descrio da ao ocorrida.
   */
  private void appendHistory(Object userId, String[] path, String description) {
    User user = null;
    try {
      user = User.getUser(userId);
    }
    catch (Exception e) {
      Server.getInstance().logSevereMessage(
        "No foi possvel obter o usurio que solicitou a "
          + "chamada ao servio");
    }
    if (user == null) {
      return;
    }
    appendHistory(user, path, description);
  }

  /**
   * Adiciona uma entrada ao histrico do servio de algoritmos.
   * 
   * @param path caminho no qual ocorreu uma ao que deve ser registrada no
   *        histrico.
   * @param description descrio da ao ocorrida.
   */
  private void appendHistory(String[] path, String description) {
    User user = Service.getUser();
    if (user == null) {
      return;
    }
    appendHistory(user, path, description);
  }

  /**
   * Adiciona uma entrada ao histrico do servio de algoritmos.
   * 
   * @param user usurio que efetuou a ao.
   * @param path caminho no qual ocorreu uma ao que deve ser registrada no
   *        histrico.
   * @param description descrio da ao ocorrida.
   */
  private void appendHistory(User user, String[] path, String description) {
    List record = new ArrayList();
    record.add(HistoryRecord.DATE_COLUMN_INDEX, new Date());
    record.add(HistoryRecord.PATH_COLUMN_INDEX, path);
    record.add(HistoryRecord.LOGIN_COLUMN_INDEX, user.getLogin());
    record.add(HistoryRecord.NAME_COLUMN_INDEX, user.getName());
    record.add(HistoryRecord.DESCRIPTION_COLUMN_INDEX, description);
    insertRecord(path, record);
  }

  /**
   * Verifica se  o administrador ou, caso contrrio, se o usurio tem
   * permisso de administrao sobre o servio de algoritmos do servidor
   * corrente.
   */
  private void checkPermission() {
    User user = Service.getUser();
    if (user.isAdmin()) {
      return;
    }
    Server server = Server.getInstance();
    String serverName = AdminPermission.LOCAL + server.getServerName();
    Permission p = user.getMatchAttributesPermission(
      AlgorithmAdminPermission.class, serverName);
    if (p == null) {
      throw new PermissionException();
    }
  }

  /**
   * Verifica se  o administrador ou, caso contrrio, se o usurio tem
   * permisso de administrao sobre o servio de algoritmos do servidor
   * corrente.
   * 
   * @param algorithm Identificador do algoritmo.
   * @throws PermissionException
   */
  private void checkPermission(Object algorithm) throws PermissionException {
    User user = Service.getUser();
    if (user.isAdmin()) {
      return;
    }
    String algoAttribute = AlgorithmAdminPermission.ALGORITHM_UPDATE
      + algorithm;
    AlgorithmAdminPermission ap = (AlgorithmAdminPermission) user
      .getMatchAttributesPermission(AlgorithmAdminPermission.class,
        algoAttribute);
    if (ap == null) {
      throw new PermissionException();
    }
    Server server = Server.getInstance();
    String name = AdminPermission.LOCAL + server.getServerName();
    if (ap.getMatchAttribute(name) == null) {
      throw new PermissionException();
    }
  }

  /**
   * Cria o configurador lgico personalizado (em Java).
   * 
   * @param version A verso do algoritmo (No aceita {@code null}).
   * 
   * @return O configurador.
   * 
   * @throws OperationFailureException Se houver um erro.
   */
  private AlgorithmConfigurator createCustomAlgorithmConfigurator(
    AlgorithmVersionInfo version) throws OperationFailureException {
    String configuratorClassName = version.getPackageName() + "."
      + CONFIGURATOR_CLASS_NAME;
    Class<?> configuratorClass;
    try {
      configuratorClass = Class.forName(configuratorClassName);
    }
    catch (ClassNotFoundException e) {
      return null;
    }
    if (!AlgorithmConfigurator.class.isAssignableFrom(configuratorClass)) {
      throw new OperationFailureException(
        getString("error_wrong_component_type"), configuratorClassName,
        AlgorithmConfigurator.class.getName());
    }
    Constructor<?> configuratorConstructor;
    try {
      configuratorConstructor = configuratorClass.getConstructor(new Class[] {
          AlgorithmVersionInfo.class, String.class });
    }
    catch (NoSuchMethodException e) {
      throw new OperationFailureException(
        getString("error_wrong_component_no_constructor"),
        CONFIGURATOR_CLASS_NAME);
    }
    AlgorithmConfigurator configurator;
    try {
      final String cmdLinePattern = getServiceCommandLinePattern();
      final Object[] args = new Object[] { version, cmdLinePattern };
      configurator = (AlgorithmConfigurator) configuratorConstructor
        .newInstance(args);
    }
    catch (InstantiationException e) {
      throw new OperationFailureException(getString("error_configurator"), e);
    }
    catch (IllegalAccessException e) {
      throw new OperationFailureException(MessageFormat.format(
        getString("error_wrong_component_no_constructor"), configuratorClass
          .getName()), e);
    }
    catch (InvocationTargetException e) {
      throw new OperationFailureException(getString("error"), e
        .getTargetException());
    }
    return configurator;
  }

  /**
   * Cria o configurador lgico do fluxo de algoritmos.
   * 
   * @param version A verso do algoritmo (No aceita {@code null}).
   * 
   * @return O configurador.
   * 
   * @throws OperationFailureException Se houver um erro.
   */
  private FlowAlgorithmConfigurator createFlowAlgorithmConfigurator(
    AlgorithmVersionInfo version) throws OperationFailureException {
    if (version == null) {
      throw new IllegalArgumentException("O parmetro version est nulo.");
    }
    String configText = readConfiguratorFile(version,
      AlgorithmVersionInfo.FLOW_CONFIG_FILE);
    if (configText == null) {
      return null;
    }
    Flow flow;
    try {
      flow = new FlowAlgorithmParser().decode(configText);
    }
    catch (ConfigurationException e) {
      throw new OperationFailureException(e.getLocalizedMessage(), e);
    }
    catch (ParseException e) {
      throw new OperationFailureException(e.getLocalizedMessage(), e);
    }
    return new FlowAlgorithmConfigurator(version, flow);
  }

  /**
   * Consulta o pattern default para montagem de linhas de comando.
   * 
   * @return pattern
   */
  private String getServiceCommandLinePattern() {
    return getStringProperty("default.command.line.pattern");
  }

  /**
   * Criar configurador.
   * 
   * @param version A verso do algoritmo.
   * 
   * @return O configurador de algoritmo.
   * 
   * @throws ParseException
   * @throws IllegalArgumentException Se o parmetro {@code versionInfo} for
   *         nulo.
   */
  private SimpleAlgorithmConfigurator createSimpleAlgorithmConfigurator(
    AlgorithmVersionInfo version) throws ParseException {
    if (version == null) {
      throw new IllegalArgumentException("O parmetro version est nulo.");
    }
    String configText = readConfiguratorFile(version,
      AlgorithmVersionInfo.CONFIG_FILE);
    String configPropertiesText = readConfiguratorFile(version,
      AlgorithmVersionInfo.CONFIG_PROPERTIES_FILE);
    String paramPropertiesText = readConfiguratorFile(version,
      AlgorithmVersionInfo.PARAMETERS_PROPERTIES_FILE);

    if (configText != null) {
      StringReader configXmlReader = new StringReader(configText);

      Map<String, String> config = new HashMap<String, String>();
      Map<String, String> params = new HashMap<String, String>();

      Properties configProperties = getProperties(configPropertiesText);
      Properties paramProperties = getProperties(paramPropertiesText);

      /*
       * Carrega as propriedades-padro de todos os algoritmos + propriedades
       * especficas do algoritmo em questo. As propriedades do algoritmo devem
       * ser carregadas por ltimo, para poderem sobrescrever as
       * propriedades-padro.
       */
      addPropertiesToMap(this.parametersConfiguration, config);
      addPropertiesToMap(configProperties, config);
      addPropertiesToMap(this.parameters, params);
      addPropertiesToMap(paramProperties, params);

      final SimpleAlgorithmParser parser = new SimpleAlgorithmParser();
      final String srvCmdPattern = getServiceCommandLinePattern();
      return parser.load(configXmlReader, config, params, version,
        srvCmdPattern);
    }
    return null;
  }

  /**
   * Mtodo utilitrio que adiciona todas as propriedades {@link Properties} a
   * um mapa {@link Map}.
   * 
   * @param <K> Tipo da chave do mapa.
   * @param <V> Tipo do valor do mapa.
   * @param properties Propriedades a serem adicionadas.
   * @param map Mapa que receber as propriedades.
   */
  @SuppressWarnings("unchecked")
  private <K, V> void addPropertiesToMap(final Properties properties,
    final Map<K, V> map) {
    for (Entry<Object, Object> entry : properties.entrySet()) {
      map.put((K) entry.getKey(), (V) entry.getValue());
    }
  }

  /**
   * Retorna as propriedades {@link Properties} a partir do contedo de um
   * arquivo de propriedades.
   * 
   * @param configPropertiesText Contedo do arquivo de propriedades.
   * @return Propriedades contidas no arquivo.
   */
  private Properties getProperties(String configPropertiesText) {
    if (configPropertiesText == null) {
      return new Properties();
    }
    ByteArrayInputStream configPropertiesInputStream = new ByteArrayInputStream(
      configPropertiesText.getBytes());
    Properties properties = new Properties();
    try {
      properties.load(configPropertiesInputStream);
    }
    catch (IOException e) {
      IllegalStateException re = new IllegalStateException(e
        .getLocalizedMessage());
      re.initCause(e);
      throw re;
    }
    return properties;
  }

  /**
   * Obtm mensagem de erro para o caso do algoritmo especificado no ter sido
   * encontrado.
   * 
   * @param algoId identificador do algoritmo no encontrado.
   * 
   * @return mensagem de erro.
   */
  private String getAlgoNotFoundMessage(Object algoId) {
    String msg = getString(ERROR_ALGO_NOT_FOUND);
    Object[] args = new Object[] { algoId };
    return MessageFormat.format(msg, args);
  }

  /**
   * Retorna um algoritmo.
   * 
   * @param name O nome do algoritmo
   * 
   * @return o algoritmo solicitado
   * @throws AlgorithmNotFoundException quando o algoritmo no  encontrado
   */
  private Algorithm getAlgorithm(String name) throws AlgorithmNotFoundException {
    Collection<Algorithm> algorithmCollection = registeredAlgorithms.values();
    Iterator<Algorithm> algorithmIterator = algorithmCollection.iterator();
    while (algorithmIterator.hasNext()) {
      Algorithm algo = algorithmIterator.next();
      AlgorithmInfo info = algo.getInfo();
      if (info.getName().equals(name) || info.getId().equals(name)) {
        return algo;
      }
    }
    throw new AlgorithmNotFoundException(String.format(
      getString("AlgoService.error.algo.not.found"), name));
  }

  /**
   * <p>
   * Obtm o histrico do diretrio e de todos os seus subdiretrios.
   * </p>
   * NOTA DE IMPLEMENTAO: Para evitar problemas de versionamento com objetos
   * serializados, s so serializados objetos bsicos do Java, como Integers,
   * Strings e Collections. O histrico retornado se compe de uma lista em que
   * cada item aponta para outras listas, estas ltimas contendo os dados de
   * entradas de histrico.
   * 
   * @param table tabela que guarda todas as entradas de histrico para o
   *        diretrio e seus subdiretrios.
   * 
   * @return histrico do diretrio e de todos os seus subdiretrios.
   */
  private List getDirHistory(Map table) {
    List allHistory = new ArrayList();
    for (Iterator iter = table.keySet().iterator(); iter.hasNext();) {
      Object key = iter.next();
      if (DIR_HISTORY_KEY.equals(key)) {
        List dirHistory = (List) table.get(DIR_HISTORY_KEY);
        if (dirHistory != null) {
          allHistory.addAll(dirHistory);
        }
      }
      else {
        Map subTable = (Map) table.get(key);
        if (subTable == null) {
          logSevereMessage("Falha na estrutura da rvore de histrico",
            new IllegalStateException("subTable == null"));
          continue;
        }
        List subDirsHistory = getDirHistory(subTable);
        if (subDirsHistory != null) {
          allHistory.addAll(subDirsHistory);
        }
      }
    }
    return allHistory;
  }

  /**
   * Obtm a tabela que guarda todas as entradas de histrico para o diretrio
   * especificado e seus subdiretrios.
   * 
   * @param viewPath caminho do diretrio (na viso do cliente).
   * @param historyTable referncia para a tabela de histrico.
   * 
   * @return tabela que guarda todas as entradas de histrico para o diretrio
   *         especificado e seus subdiretrios. Se no havia nenhuma entrada de
   *         histrico para o diretrio em questo, uma tabela vazia 
   *         retornada.
   */
  private Map getDirTable(String[] viewPath, Map historyTable) {
    Map table = historyTable;

    // #TODO Esse mtodo  praticamente idntico ao insertRecord: refactoring
    //  necessrio.
    // O path[0] equivale ao diretrio "algorithms", que no figura na tabela
    for (int i = 1; i < viewPath.length; i++) {
      String key = viewPath[i];
      Map swap = (Map) table.get(key);
      if (swap == null) {
        swap = new HashMap();
        table.put(key, swap);
      }
      table = swap;
    }
    return table;
  }

  /**
   * Obtm o caminho para o arquivo que armazena a tabela de histrico.
   * 
   * @return caminho para o arquivo que armazena a tabela de histrico.
   */
  private String getHistoryTablePath() {
    return getRootDirName() + File.separator + HISTORY_TABLE_FILE_NAME;
  }

  /**
   * Obtm uma lista de <code>HistoryRecord</code>, a partir de uma lista de
   * listas (<code>List</code> de <code>List</code>). A justificativa para isso
   *  que somente so serializados objetos bsicos do Java, para evitar
   * problemas de compatibilidade com verses futuras.
   * 
   * @param viewPath caminho do diretrio (na viso do cliente).
   * @param rawList lista em estado bruto (lista de listas).
   * 
   * @return lista de <code>HistoryRecord</code>.
   */
  private List<HistoryRecord> getRecordList(String[] viewPath, List rawList) {
    List<HistoryRecord> recordList = new ArrayList<HistoryRecord>();
    for (Iterator iter = rawList.iterator(); iter.hasNext();) {
      List l = (List) iter.next();
      Date date = (Date) l.get(HistoryRecord.DATE_COLUMN_INDEX);
      String[] absLogPath = (String[]) l.get(HistoryRecord.PATH_COLUMN_INDEX);
      String login = (String) l.get(HistoryRecord.LOGIN_COLUMN_INDEX);
      String name = (String) l.get(HistoryRecord.NAME_COLUMN_INDEX);
      String description = (String) l
        .get(HistoryRecord.DESCRIPTION_COLUMN_INDEX);
      String[] relLogPath;
      if (viewPath.length == absLogPath.length) {
        relLogPath = new String[] { "." };
      }
      else {
        int len = absLogPath.length - viewPath.length;
        relLogPath = new String[len];
        System.arraycopy(absLogPath, viewPath.length, relLogPath, 0, len);
      }
      HistoryRecord record = new HistoryRecord(date, relLogPath, login, name,
        description);
      recordList.add(record);
    }
    return recordList;
  }

  /**
   * Obtm o nome do diretrio raiz do repositrio de algoritmos.
   * 
   * @return nome do diretrio raiz.
   */
  private String getRootDirName() {
    return AlgorithmInfo.REPOSITORY_NAME;
  }

  /**
   * Obtm mensagem de erro para o caso da verso especificada no ter sido
   * encontrada.
   * 
   * @param algoId identificador do algoritmo ao qual deveria pertencer a verso
   *        no encontrada.
   * @param versionId identificador da verso no encontrada.
   * 
   * @return mensagem de erro.
   */
  private String getVersionNotFoundMessage(Object algoId, Object versionId) {
    String msg = getString(ERROR_VERSION_NOT_FOUND);
    Object[] args = new Object[] { versionId, algoId };
    return MessageFormat.format(msg, args);
  }

  /**
   * Verifica a existncia da tabela de histrico; caso no encontre, constri e
   * serializa uma tabela vazia.
   */
  private void initHistoryTable() {
    File historyFile = new File(getHistoryTablePath());
    if (!historyFile.exists()) {
      logInfoMessage("Tabela de histrico no encontrada. Criando...");
      writeHistoryTable(new HashMap());
    }
  }

  /**
   * Insere uma nova entrada na tabela de histrico.
   * 
   * @param path caminho no qual ocorreu uma ao que deve ser registrada no
   *        histrico.
   * @param record registro a ser inserido na tabela.
   */
  private void insertRecord(String[] path, List record) {
    Map historyTable = readHistoryTable();
    if (historyTable == null) {
      return;
    }
    Map table = getDirTable(path, historyTable);
    List dirHistory = (List) table.get(DIR_HISTORY_KEY);
    if (dirHistory == null) {
      dirHistory = new ArrayList();
      table.put(DIR_HISTORY_KEY, dirHistory);
    }
    dirHistory.add(record);
    writeHistoryTable(historyTable);
  }

  /**
   * Faz a (re)carga dos algoritmos no servidor.
   * 
   * @throws ServerException caso ocorra algum problema durante a (re)carga.
   */
  private void loadAlgorithms() throws ServerException {
    String searchPath = getRootDirName();
    Server.getInstance().checkDirectory(searchPath);
    searchIds(searchPath);
  }

  /**
   * Carregas as categorias do sistema (novo leitor xml - ainda em teste).
   * 
   * @throws ServerException caso ocorra algum problema durante a carga
   */
  private void loadCategories() throws ServerException {
    categorySet.removeAllCategories();

    if (!readCategoriesFile(categorySet)) {
      return;
    }
    logInfoMessage("As categorias de algoritmos foram carregadas com sucesso.");
    isFirstCategoryLoad = false;

    // System.out.println("AlgorithmService: nmero de categorias (raiz) lidas: "
    // + categorySet.getSize());
  }

  /**
   * L um arquivo do configurador de uma verso de algoritmo.
   * 
   * @param version A verso do algoritmo (No aceita {@code null}).
   * @param configFileName O nome do arquivo (No aceita {@code null}).
   * 
   * @return O contedo do arquivo ou {@code null} se houver algum erro.
   */
  private String readConfiguratorFile(AlgorithmVersionInfo version,
    String configFileName) {
    String configuratorPath = version.getConfiguratorDirPath() + "/"
      + configFileName;
    configuratorPath = configuratorPath.replace('/', File.separatorChar);
    File configuratorFile = new File(configuratorPath);
    if (!configuratorFile.exists()) {
      String algorithmName = version.getInfo().getName();
      AlgorithmVersionId versionId = version.getId();
      String message = MessageFormat
        .format(
          "Foi solicitado o arquivo {0} do algoritmo {1} verso {2}, porm "
            + "o arquivo no existe.", configuratorPath, algorithmName,
          versionId);
      logDebugMessage(message);
      return null;
    }
    BufferedReader bufferedReader = null;
    try {
      bufferedReader = new BufferedReader(new FileReader(configuratorFile));
      StringBuffer buffer = new StringBuffer();
      String line;
      while ((line = bufferedReader.readLine()) != null) {
        buffer.append(line + "\n");
      }
      return buffer.toString();
    }
    catch (IOException e) {
      String algorithmName = version.getInfo().getName();
      AlgorithmVersionId versionId = version.getId();
      String errorMessage = MessageFormat.format(
        "Foi solicitado o '{2}' do algoritmo {0} verso {1}, "
          + "porm ocorreu uma exceo de IO.", algorithmName, versionId,
        configFileName);
      logSevereMessage(errorMessage, e);
      return null;
    }
    finally {
      if (bufferedReader != null) {
        try {
          bufferedReader.close();
        }
        catch (IOException e) {
          String algorithmName = version.getInfo().getName();
          AlgorithmVersionId versionId = version.getId();
          String errorMessage = MessageFormat.format(
            "Foi solicitado o '{2}' do algoritmo {0} verso {1}, "
              + "porm ocorreu uma exceo de IO, "
              + "quando o stream estava sendo fechado.", algorithmName,
            versionId, configFileName);
          logSevereMessage(errorMessage, e);
        }
        bufferedReader = null;
      }
    }
  }

  /**
   * L o arquivo serializado que representa a tabela de histricos e retorna a
   * tabela lida na forma de um objeto.
   * 
   * @return tabela de histrico ou <code>null</code> caso tenha ocorrido algum
   *         problema durante a leitura da tabela.
   */
  private Map readHistoryTable() {
    Map table = null;
    ObjectInput input = null;
    try {
      input = new ObjectInputStream(new BufferedInputStream(
        new FileInputStream(getHistoryTablePath())));
      table = (Map) input.readObject();
    }
    catch (IOException ex) {
      logSevereMessage("Erro ao ler a tabela de histrico.", ex);
    }
    catch (ClassNotFoundException ex) {
      logSevereMessage(
        "No foi encontrada nenhuma classe no arquivo especificado.", ex);
    }
    finally {
      try {
        if (input != null) {
          input.close();
        }
      }
      catch (IOException ex) {
        logSevereMessage(
          "Erro ao fechar stream de leitura da tabela de histrico.", ex);
      }
    }
    return table;
  }

  /**
   * Percorre o repositrio de algoritmos, obtendo as informaes dos algoritmos
   * correntemente cadastrados.
   * 
   * @param path caminho para o repositrio de algoritmos.
   * 
   * @throws ServerException caso ocorra algum problema durante a busca.
   */
  private void searchIds(String path) throws ServerException {
    try {
      File repDir = new File(path);
      Server.getInstance().checkDirectory(path);
      registeredAlgorithms = new Hashtable<String, Algorithm>();
      File[] algoDirs = repDir.listFiles();
      for (int i = 0; i < algoDirs.length; i++) {
        File dir = algoDirs[i];
        if (dir.isDirectory()) {
          String dirName = dir.getName().trim();
          if (dirName.equalsIgnoreCase(FileSystemUtils.VERSION_CONTROL_DIR)) {
            continue;
          }
          logDebugMessage("Identificando algoritmo em :" + dirName);
          try {
            Algorithm algo = Algorithm.includeAlgorithm(dirName);
            String newId = algo.getInfo().getId();
            registeredAlgorithms.put(newId, algo);
            // LOG
            logDebugMessage("Algoritmo registrado: " + newId);
            logDebugMessage("Propriedades:"
              + algo.getInfo().getPropertyValues());
          }
          catch (ServerException se) {
            logSevereMessage("Falha na obteno de um algoritmo no "
              + "repositrio do SSI - Exceo: " + se + "\n", se);
          }
        }
      }
    }
    catch (Exception e) {
      throw new ServerException("Falha na busca de algoritmos no "
        + "repositrio do SSI\n" + "\tLocalizao: " + path + "\n"
        + "\tExceo: " + e + "\n", e);
    }
  }

  /**
   * Grava (serializa) a tabela de histrico em arquivo.
   * 
   * @param table tabela de histrico.
   */
  private void writeHistoryTable(Map table) {
    ObjectOutput output = null;
    try {
      output = new ObjectOutputStream(new BufferedOutputStream(
        new FileOutputStream(getHistoryTablePath())));
      output.writeObject(table);
    }
    catch (IOException ex) {
      logSevereMessage("Erro ao gravar a tabela de histrico.", ex);
    }
    finally {
      try {
        if (output != null) {
          output.close();
        }
      }
      catch (IOException ex) {
        logSevereMessage(
          "Erro ao fechar stream de gravao da tabela de histrico.", ex);
      }
    }
  }

  /**
   * Obtm o {@link HttpService servio de HTTP}.
   * 
   * @return O servio.
   * 
   * @throws OperationFailureException Se o servio no existir.
   */
  private HttpService getHttpService() throws OperationFailureException {
    HttpService service = HttpService.getInstance();
    if (service == null) {
      throw new OperationFailureException(
        "O servio de HTTP no foi encontrado.");
    }
    return service;
  }

  /**
   * {@inheritDoc}
   */
  public RequestTransferInfo prepareUploadVersionPack(Object algoId,
    String fileName) {
    checkPermission(algoId);
    Algorithm algo = registeredAlgorithms.get(algoId);
    AlgorithmInfo algoInfo = algo.getInfo();
    File repositoryDirectory = new File(AlgorithmInfo.REPOSITORY_NAME);
    File algorithmDirectory = new File(repositoryDirectory, algoInfo
      .getDirectory());
    File versionsDir = new File(algorithmDirectory, AlgorithmInfo.VERSIONS_DIR);
    if (!versionsDir.exists()) {
      String userMessage = String.format(
        getString("server.algoservice.error.illegal_algo_dir_state"), algoInfo
          .getName());
      String logDetails = String.format(
        "\nDetalhes: o diretrio %s no foi encontrado.\n", versionsDir
          .getAbsolutePath());
      logSevereMessage(userMessage + logDetails);
      throw new ServiceFailureException(userMessage);
    }
    String targetPath;
    try {
      targetPath = versionsDir.getCanonicalPath() + File.separator + fileName;
    }
    catch (IOException e) {
      String userMessage = String.format(
        getString("server.algoservice.error.upload_version_io"), algoInfo
          .getName());
      logSevereMessage(userMessage, e);
      throw new ServiceFailureException(userMessage);
    }
    RequestTransferInfo info = null;
    Map<String, Object> notifyArgs = new HashMap<String, Object>();
    notifyArgs.put(ALGO_FILE_TYPE_KEY, AlgoFileType.VERSION_PACK);
    notifyArgs.put(USER_ID_KEY, Service.getUser().getId());
    notifyArgs.put(ALGO_ID_KEY, algoId);
    notifyArgs.put(FILE_PATH_KEY, targetPath);
    notifyArgs.put(EXPAND_IF_ZIP, Boolean.TRUE);
    try {
      info = fileServer.prepareUpload(targetPath, notifyArgs);
    }
    catch (DigestException e) {
      String clientMsg = getString("server.algoservice.error.upload_fatal");
      String detailMsg = "Falha na gerao da chave representando a transferncia";
      throw new ServiceFailureException(clientMsg, detailMsg, e);
    }
    catch (IOException e) {
      String clientMsg = getString("server.algoservice.error.upload_fatal");
      String detailMsg = "O arquivo de destino " + targetPath
        + " no pde ser criado no servidor.";
      throw new ServiceFailureException(clientMsg, detailMsg, e);
    }
    return info;
  }

  /**
   * {@inheritDoc}
   */
  public RequestTransferInfo prepareUploadExecFile(Object algoId,
    Object versionId, String platformName, String fileName, boolean expandIfZip) {
    checkPermission(algoId);
    String platformPath = getPlatformPath(algoId, versionId, platformName);
    String targetPath = platformPath + File.separator + fileName;
    RequestTransferInfo info = null;
    Map<String, Object> notifyArgs = new HashMap<String, Object>();
    notifyArgs.put(ALGO_FILE_TYPE_KEY, AlgoFileType.EXECUTABLE);
    notifyArgs.put(USER_ID_KEY, Service.getUser().getId());
    notifyArgs.put(ALGO_ID_KEY, algoId);
    notifyArgs.put(VERSION_ID_KEY, versionId);
    notifyArgs.put(PLATFORM_NAME_KEY, platformName);
    notifyArgs.put(FILE_NAME_KEY, fileName);
    notifyArgs.put(EXPAND_IF_ZIP, expandIfZip);
    try {
      info = fileServer.prepareUpload(targetPath, notifyArgs);
    }
    catch (DigestException e) {
      String clientMsg = getString("server.algoservice.error.upload_fatal");
      String detailMsg = "Falha na gerao da chave representando a transferncia";
      throw new ServiceFailureException(clientMsg, detailMsg, e);
    }
    catch (IOException e) {
      String clientMsg = getString("server.algoservice.error.upload_fatal");
      String detailMsg = "O arquivo de destino " + targetPath
        + " no pde ser criado no servidor.";
      throw new ServiceFailureException(clientMsg, detailMsg, e);
    }
    return info;
  }

  /**
   * {@inheritDoc}
   */
  public RequestTransferInfo prepareUploadConfigFile(Object algoId,
    Object versionId, String fileName, boolean expandIfZip) {
    checkPermission(algoId);
    String configDirPath = getConfiguratorDirPath(algoId, versionId);
    String targetPath = configDirPath + File.separator + fileName;
    RequestTransferInfo info = null;
    Map<String, Object> notifyArgs = new HashMap<String, Object>();
    notifyArgs.put(ALGO_FILE_TYPE_KEY, AlgoFileType.CONFIGURATION);
    notifyArgs.put(USER_ID_KEY, Service.getUser().getId());
    notifyArgs.put(ALGO_ID_KEY, algoId);
    notifyArgs.put(VERSION_ID_KEY, versionId);
    notifyArgs.put(FILE_NAME_KEY, fileName);
    notifyArgs.put(EXPAND_IF_ZIP, expandIfZip);
    try {
      info = fileServer.prepareUpload(targetPath, notifyArgs);
    }
    catch (DigestException e) {
      String clientMsg = getString("server.algoservice.error.upload_fatal");
      String detailMsg = "Falha na gerao da chave representando a transferncia";
      throw new ServiceFailureException(clientMsg, detailMsg, e);
    }
    catch (IOException e) {
      String clientMsg = getString("server.algoservice.error.upload_fatal");
      String detailMsg = "O arquivo de destino " + targetPath
        + " no pde ser criado no servidor.";
      throw new ServiceFailureException(clientMsg, detailMsg, e);
    }
    return info;
  }

  /**
   * {@inheritDoc}
   */
  public RequestTransferInfo prepareUploadDocFile(Object algoId,
    Object versionId, String fileName, boolean expandIfZip) {
    checkPermission(algoId);
    String docDirPath = getDocumentationDirPath(algoId, versionId);
    String targetPath = docDirPath + File.separator + fileName;
    RequestTransferInfo info = null;
    Map<String, Object> notifyArgs = new HashMap<String, Object>();
    notifyArgs.put(ALGO_FILE_TYPE_KEY, AlgoFileType.DOCUMENTATION);
    notifyArgs.put(USER_ID_KEY, Service.getUser().getId());
    notifyArgs.put(ALGO_ID_KEY, algoId);
    notifyArgs.put(VERSION_ID_KEY, versionId);
    notifyArgs.put(FILE_NAME_KEY, fileName);
    notifyArgs.put(EXPAND_IF_ZIP, expandIfZip);
    try {
      info = fileServer.prepareUpload(targetPath, notifyArgs);
    }
    catch (DigestException e) {
      String clientMsg = getString("server.algoservice.error.upload_fatal");
      String detailMsg = "Falha na gerao da chave representando a transferncia";
      throw new ServiceFailureException(clientMsg, detailMsg, e);
    }
    catch (IOException e) {
      String clientMsg = getString("server.algoservice.error.upload_fatal");
      String detailMsg = "O arquivo de destino " + targetPath
        + " no pde ser criado no servidor.";
      throw new ServiceFailureException(clientMsg, detailMsg, e);
    }
    return info;
  }

  /**
   * {@inheritDoc}
   */
  public boolean configFileExists(Object algoId, Object versionId,
    String fileName) {
    checkPermission(algoId);
    String configDirPath = getConfiguratorDirPath(algoId, versionId);
    String targetPath = configDirPath + File.separator + fileName;
    return new File(targetPath).exists();
  }

  /**
   * {@inheritDoc}
   */
  public boolean docFileExists(Object algoId, Object versionId, String fileName) {
    checkPermission(algoId);
    String docDirPath = getDocumentationDirPath(algoId, versionId);
    String targetPath = docDirPath + File.separator + fileName;
    return new File(targetPath).exists();
  }

  /**
   * {@inheritDoc}
   */
  public boolean execFileExists(Object algoId, Object versionId,
    String platformName, String fileName) {
    checkPermission(algoId);
    String platformPath = getPlatformPath(algoId, versionId, platformName);
    String targetPath = platformPath + File.separator + fileName;
    return new File(targetPath).exists();
  }

  /**
   * Carrega os parmetros cadastrados no sistema e suas configuraes.
   * 
   * @throws ServerException
   */
  private void loadParameters() throws ServerException {
    parameters = new Properties();
    parametersConfiguration = new Properties();

    loadParameterPropertiesFromFilePath(parameters,
      "AlgorithmParameters.properties");
    loadParameterPropertiesFromFilePath(parametersConfiguration,
      "AlgorithmParametersConfiguration.properties");

    loadParameterPropertiesFromTag(parameters, "parameters.property.file");
    loadParameterPropertiesFromTag(parametersConfiguration,
      "parameters.configuration.property.file");
  }

  /**
   * Carrega propriedades {@link Properties} a partir de um caminho relativo
   * para um arquivo de propriedades.
   * 
   * @param properties Instncia de Properties que receber as propriedades.
   * @param path Caminho relativo para o arquivo de propriedades.
   */
  private void loadParameterPropertiesFromFilePath(Properties properties,
    String path) {
    InputStream inStream;
    try {
      inStream = getClass().getResourceAsStream(path);
      loadPropertiesFromStream(properties, inStream);
    }
    catch (FileNotFoundException e) {
      logWarningMessage("No foi possvel encontrar arquivo de propriedades: "
        + path + ".");
    }
    catch (IOException e) {
      logWarningMessage("No foi possvel ler o arquivo de propriedades: "
        + path + ".");
    }
  }

  /**
   * Carrega propriedades {@link Properties} a partir de uma propriedade do
   * servio que aponta para um outro arquivo de propriedades.
   * 
   * @param properties Instncia de Properties que receber as propriedades.
   * @param tag Nome da propriedade do servio que aponta pro arquivo de
   *        propriedades a ser carregado.
   * @throws ServerException
   */
  private void loadParameterPropertiesFromTag(Properties properties, String tag)
    throws ServerException {
    if (!isPropertyNull(tag)) {
      String propertiesFile = getStringProperty(tag);
      String path = getOSPropertyPath(propertiesFile);
      InputStream inStream;
      try {
        inStream = new FileInputStream(path);
        loadPropertiesFromStream(properties, inStream);
      }
      catch (FileNotFoundException e) {
        throw new ServerException("No foi possvel encontrar o arquivo: "
          + path + " definido na propriedade " + tag
          + " do servio de algoritmos.", e);
      }
      catch (IOException e) {
        throw new ServerException("Erro ao ler o arquivo: " + path
          + " definido na propriedade " + tag + " do servio de algoritmos.", e);
      }
    }
  }

  /**
   * Mtodo utilitrio que carrega propriedades {@link Properties} a partir de
   * um {@link InputStream} de um arquivo de propriedades.
   * 
   * @param properties Instncia de Properties que receber as propriedades.
   * @param inStream Stream do arquivo de propriedades a ser carregado.
   * @throws IOException Caso no seja possvel ler as propriedades do arquivo.
   */
  private void loadPropertiesFromStream(Properties properties,
    InputStream inStream) throws IOException {
    try {
      if (inStream != null) {
        properties.load(inStream);
      }
    }
    finally {
      if (inStream != null) {
        inStream.close();
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  public RequestTransferInfo prepareDownloadExecFile(Object algoId,
    Object versionId, String platformName, String fileName) {
    checkPermission(algoId);
    String platformPath = getPlatformPath(algoId, versionId, platformName);
    String sourcePath = platformPath + File.separator + fileName;
    RequestTransferInfo info = null;
    try {
      info = fileServer.prepareDownload(sourcePath);
    }
    catch (DigestException e) {
      String clientMsg = getString("server.algoservice.error.download_fatal");
      String detailMsg = "Falha na gerao da chave representando a transferncia";
      throw new ServiceFailureException(clientMsg, detailMsg, e);
    }
    catch (IOException e) {
      String clientMsg = getString("server.algoservice.error.download_fatal");
      throw new ServiceFailureException(clientMsg, e);
    }
    return info;
  }

  /**
   * {@inheritDoc}
   */
  public RequestTransferInfo prepareDownloadConfigFile(Object algoId,
    Object versionId, String fileName) {
    checkPermission(algoId);
    String configDirPath = getConfiguratorDirPath(algoId, versionId);
    String sourcePath = configDirPath + File.separator + fileName;
    RequestTransferInfo info = null;
    try {
      info = fileServer.prepareDownload(sourcePath);
    }
    catch (DigestException e) {
      String clientMsg = getString("server.algoservice.error.download_fatal");
      String detailMsg = "Falha na gerao da chave representando a transferncia";
      throw new ServiceFailureException(clientMsg, detailMsg, e);
    }
    catch (IOException e) {
      String clientMsg = getString("server.algoservice.error.download_fatal");
      throw new ServiceFailureException(clientMsg, e);
    }
    return info;
  }

  /**
   * {@inheritDoc}
   */
  public RequestTransferInfo prepareDownloadDocFile(Object algoId,
    Object versionId, String fileName) {
    checkPermission(algoId);
    String docDirPath = getDocumentationDirPath(algoId, versionId);
    String sourcePath = docDirPath + File.separator + fileName;
    RequestTransferInfo info = null;
    try {
      info = fileServer.prepareDownload(sourcePath);
    }
    catch (DigestException e) {
      String clientMsg = getString("server.algoservice.error.download_fatal");
      String detailMsg = "Falha na gerao da chave representando a transferncia";
      throw new ServiceFailureException(clientMsg, detailMsg, e);
    }
    catch (IOException e) {
      String clientMsg = getString("server.algoservice.error.download_fatal");
      throw new ServiceFailureException(clientMsg, e);
    }
    return info;
  }

  /**
   * Obtm o caminho no sistema de arquivos de uma determinada plataforma
   * (relativo  raiz do sistema).
   * 
   * @param algoId identificador do algoritmo.
   * @param versionId identificador da verso.
   * @param platformName nome da plataforma.
   * 
   * @return caminho da plataforma no sistema de arquivos.
   */
  private String getPlatformPath(Object algoId, Object versionId,
    String platformName) {
    Algorithm algo = registeredAlgorithms.get(algoId);
    AlgorithmVersionInfo versionInfo = algo.getInfo().getVersionInfo(versionId);
    return versionInfo.getPlatformPath(platformName);
  }

  /**
   * Obtm o caminho no sistema de arquivos de uma determinado diretrio de
   * arquivos de configurao (relativo  raiz do sistema).
   * 
   * @param algoId identificador do algoritmo.
   * @param versionId identificador da verso.
   * 
   * @return caminho do diretrio de arquivos de configurao no sistema de
   *         arquivos.
   */
  private String getConfiguratorDirPath(Object algoId, Object versionId) {
    Algorithm algo = registeredAlgorithms.get(algoId);
    AlgorithmVersionInfo versionInfo = algo.getInfo().getVersionInfo(versionId);
    return versionInfo.getConfiguratorDirPath();
  }

  /**
   * Obtm o caminho no sistema de arquivos de uma determinado diretrio de
   * arquivos de documentao (relativo  raiz do sistema).
   * 
   * @param algoId identificador do algoritmo.
   * @param versionId identificador da verso.
   * 
   * @return caminho do diretrio de arquivos de documentao no sistema de
   *         arquivos.
   */
  private String getDocumentationDirPath(Object algoId, Object versionId) {
    Algorithm algo = registeredAlgorithms.get(algoId);
    AlgorithmVersionInfo versionInfo = algo.getInfo().getVersionInfo(versionId);
    return versionInfo.getDocDirPath();
  }

  /**
   * Indica se o arquivo possui extenso ZIP.
   * 
   * @param path caminho para o arquivo.
   * 
   * @return true caso o arquivo possua extenso ZIP.
   */
  private boolean isZip(String path) {
    return (ZIP_EXTENSION.equalsIgnoreCase(FileUtils.getFileExtension(path)));
  }

  /**
   * Extrai o contedo de um arquivo zip na raiz de uma plataforma.
   * 
   * @param algoId identificador do algoritmo.
   * @param versionId identificador da verso.
   * @param platform nome da plataforma.
   * @param zipFileName nome do arquivo zip.
   * @param userId identificador do usurio.
   * 
   * @return {@code true} se a extrao for bem-sucedida.
   */
  private boolean extractZipFile(IPathFactory pathFactory, FileInfo file,
    Object userId) {
    String zipPath = pathFactory.getPath(file);
    File zipFile = new File(zipPath);
    try {
      Unzip unzip = new Unzip(zipFile);
      if (unzip.listZipEntries().isEmpty()) {
        String msg = String.format(
          getString("server.algoservice.error.invalid_zip_file"), zipFile
            .getName());
        logSevereMessage(msg);
        NotificationService.getInstance().notifyTo(SERVICE_NAME, userId, msg,
          false);
        return false;
      }
      unzip.decompress(zipFile.getParentFile(), true);
    }
    catch (IOException e) {
      String msg = String.format(
        getString("server.algoservice.error.unzip_io"), zipFile.getName());
      logSevereMessage(msg, e);
      NotificationService.getInstance().notifyTo(SERVICE_NAME, userId, msg,
        false);
      return false;
    }
    finally {
      FileUtils.delete(zipFile);
    }
    return true;
  }

  /**
   * Callback invocada quando um canal com o cliente est para ser fechado,
   * indicando que uma importao acabou de ser realizada (no  invocado na
   * exportao). Este mtodo permite que o servidor realize algum procedimento
   * antes da <code>Task</code> do cliente que solicitou a importao ser
   * finalizada.
   * 
   * @param args informaes sobre a importao recm-realizada.
   */
  public void channelClosing(Map args) {
    AlgoFileType type = (AlgoFileType) args.get(ALGO_FILE_TYPE_KEY);
    Object userId = args.get(USER_ID_KEY);
    Object algoId = args.get(ALGO_ID_KEY);
    Algorithm algo = registeredAlgorithms.get(algoId);
    if (algo == null) {
      String detailedMessage = String.format(
        "O algoritmo %s no foi encontrado.\n", algoId);
      logSevereMessage(detailedMessage);
      String userMsg = getString("server.algoservice.error.algo_dir_not_found");
      NotificationService.getInstance().notifyTo(SERVICE_NAME, userId, userMsg,
        false);
      return;
    }
    switch (type) {
      case CONFIGURATION:
        configurationUploaded(args);
        break;
      case DOCUMENTATION:
        documentationUploaded(args);
        break;
      case EXECUTABLE:
        executableUploaded(args);
        break;
      case VERSION_PACK:
        versionPackUploaded(args);
        break;
      default:
        throw new BugException("Unknown type " + type);
    }
    AlgorithmInfo algoInfo = algo.getInfo();
    fireModifyEvent(algoInfo);
  }

  /**
   * Callback indicando que um arquivo de configurao acabou de ser importado.
   * 
   * @param args informaes sobre a importao recm-realizada.
   */
  private void configurationUploaded(Map args) {
    Object userId = args.get(USER_ID_KEY);
    final Object algoId = args.get(ALGO_ID_KEY);
    Algorithm algo = registeredAlgorithms.get(algoId);
    AlgorithmInfo algoInfo = algo.getInfo();
    final Object versionId = args.get(VERSION_ID_KEY);
    AlgorithmVersionInfo versionInfo = algoInfo.getVersionInfo(versionId);
    String configName = (String) args.get(FILE_NAME_KEY);

    // Ser sempre um arquivo. Diretrios so encapsulados em arquivos zip.
    FileInfo file = new FileInfo(configName, false);
    if ((Boolean) args.get(EXPAND_IF_ZIP) && isZip(file.getName())) {
      IPathFactory pathFactory = new IPathFactory() {
        @Override
        public String getPath(FileInfo fileInfo) {
          return getConfiguratorDirPath(algoId, versionId) + File.separator
            + fileInfo.getPath();
        }
      };
      if (!extractZipFile(pathFactory, file, userId)) {
        return;
      }
    }

    try {
      algo.updateConfigurations(versionId);
    }
    catch (OperationFailureException e) {
      String clientMsg = getString("server.algoservice.error.refresh");
      logSevereMessage(
        "No foi possvel atualizar os arquivos de configurao da verso "
          + versionId + " do algoritmo " + algoId, e);
      NotificationService.getInstance().notifyTo(SERVICE_NAME, userId,
        clientMsg, false);
      return;
    }

    String[] path = new String[] { getRootDirName(), algo.getInfo().getName(),
        versionId.toString(), AlgorithmVersionInfo.CONFIGURATOR_DIR };
    String description = MessageFormat.format(
      getString(HISTORY_UPDATE_CONFIGURATOR), configName);
    appendHistory(userId, path, description);
  }

  /**
   * Callback indicando que um arquivo de documentao acabou de ser importado.
   * 
   * @param args informaes sobre a importao recm-realizada.
   */
  private void documentationUploaded(Map args) {
    Object userId = args.get(USER_ID_KEY);
    final Object algoId = args.get(ALGO_ID_KEY);
    Algorithm algo = registeredAlgorithms.get(algoId);
    AlgorithmInfo algoInfo = algo.getInfo();
    final Object versionId = args.get(VERSION_ID_KEY);
    AlgorithmVersionInfo versionInfo = algoInfo.getVersionInfo(versionId);
    String docName = (String) args.get(FILE_NAME_KEY);

    // Ser sempre um arquivo. Diretrios so encapsulados em arquivos zip.
    FileInfo file = new FileInfo(docName, false);
    if ((Boolean) args.get(EXPAND_IF_ZIP) && isZip(file.getName())) {
      IPathFactory pathFactory = new IPathFactory() {
        @Override
        public String getPath(FileInfo fileInfo) {
          return getDocumentationDirPath(algoId, versionId) + File.separator
            + fileInfo.getPath();
        }
      };
      if (!extractZipFile(pathFactory, file, userId)) {
        return;
      }
    }

    try {
      algo.updateDocumentations(versionId);
    }
    catch (OperationFailureException e) {
      String clientMsg = getString("server.algoservice.error.refresh");
      logSevereMessage(
        "No foi possvel atualizar os arquivos de documentao da verso "
          + versionId + " do algoritmo " + algoId, e);
      NotificationService.getInstance().notifyTo(SERVICE_NAME, userId,
        clientMsg, false);
      return;
    }

    String[] path = new String[] { getRootDirName(), algo.getInfo().getName(),
        versionId.toString(), AlgorithmVersionInfo.DOCUMENTATION_DIR };
    String description = MessageFormat.format(getString(HISTORY_UPDATE_DOC),
      docName);
    appendHistory(userId, path, description);
  }

  /**
   * Callback indicando que um arquivo executvel acabou de ser importado. Se
   * for um zip, este ser descompactado.
   * 
   * @param args informaes sobre a importao recm-realizada.
   */
  private void executableUploaded(Map args) {
    Object userId = args.get(USER_ID_KEY);
    final Object algoId = args.get(ALGO_ID_KEY);
    Algorithm algo = registeredAlgorithms.get(algoId);
    final Object versionId = args.get(VERSION_ID_KEY);
    final String platform = (String) args.get(PLATFORM_NAME_KEY);
    String exeName = (String) args.get(FILE_NAME_KEY);
    // Ser sempre um arquivo. Diretrios so encapsulados em arquivos zip.
    FileInfo file = new FileInfo(exeName, false);
    if ((args.containsKey(EXPAND_IF_ZIP) && (Boolean) args.get(EXPAND_IF_ZIP) )&& isZip(file.getName())) {
      IPathFactory pathFactory = new IPathFactory() {
        @Override
        public String getPath(FileInfo fileInfo) {
          return getPlatformPath(algoId, versionId, platform) + File.separator
            + fileInfo.getPath();
        }
      };
      if (!extractZipFile(pathFactory, file, userId)) {
        return;
      }
    }

    try {
      algo.updateExecutables(versionId, platform);
    }
    catch (OperationFailureException e) {
      String clientMsg = getString("server.algoservice.error.refresh");
      logSevereMessage("No foi possvel atualizar a plataforma " + platform
        + " da verso " + versionId + " do algoritmo " + algoId, e);
      NotificationService.getInstance().notifyTo(SERVICE_NAME, userId,
        clientMsg, false);
      return;
    }
    String[] path = new String[] { getRootDirName(), algo.getInfo().getName(),
        versionId.toString(), AlgorithmVersionInfo.BIN_DIR, platform };
    String description = MessageFormat.format(
      getString(HISTORY_UPDATE_EXECUTABLE), exeName);
    appendHistory(userId, path, description);
  }

  /**
   * Callback indicando que um pacote de verso acabou de ser importado. Se for
   * um zip no formato apropriado, este ser descompactado.
   * 
   * @param args informaes sobre a importao recm-realizada.
   */
  private void versionPackUploaded(Map args) {
    Object userId = args.get(USER_ID_KEY);
    Object algoId = args.get(ALGO_ID_KEY);
    Algorithm algo = registeredAlgorithms.get(algoId);
    AlgorithmInfo algoInfo = algo.getInfo();
    String filePath = (String) args.get(FILE_PATH_KEY);
    try {
      Unzip unzip = new Unzip(new File(filePath));
      List<ZipEntry> zipEntries = unzip.listZipEntries();
      if (zipEntries.isEmpty()) {
        String userMessage = getString("server.algoservice.error.invalid_version_pack");
        String detailedMessage = String.format(
          "%sDetalhes: o arquivo zip est vazio.\n"
            + "Arquivo utilizado: %s.\nIdentificador do algoritmo: %s.\n"
            + "Identificador do usurio: %s.\n", userMessage, filePath, algoId,
          userId);
        logSevereMessage(detailedMessage);
        NotificationService.getInstance().notifyTo(SERVICE_NAME, userId,
          userMessage, false);
        return;
      }
      ZipEntry rootZipEntry = null;
      String rootDirectoryName = null;
      for (ZipEntry zipEntry : zipEntries) {
        String[] splittedPath = FileUtils.splitPath(zipEntry.getName(), "/");
        if (splittedPath.length == 1) {
          if (rootZipEntry != null) {
            String userMessage = getString("server.algoservice.error.invalid_version_pack");
            String detailedMessage = String.format(
              "%sDetalhes: o arquivo possui 2 ou mais arquivos/diretrios na raz.\n"
                + "Arquivo utilizado: %s.\nIdentificador do algoritmo: %s.\n"
                + "Identificador do usurio: %s.\n", userMessage, filePath,
              algoId, userId);
            logSevereMessage(detailedMessage);
            NotificationService.getInstance().notifyTo(SERVICE_NAME, userId,
              userMessage, false);
            return;
          }
          rootZipEntry = zipEntry;
          rootDirectoryName = zipEntry.getName();
          rootDirectoryName = rootDirectoryName.replace("/", "");
        }
      }
      if (rootZipEntry == null) {
        String userMessage = getString("server.algoservice.error.invalid_version_pack");
        String detailedMessage = String.format(
          "%sDetalhes: o arquivo no possui o nome do diretrio raz da verso.\n"
            + "Arquivo utilizado: %s.\nIdentificador do algoritmo: %s.\n"
            + "Identificador do usurio: %s.\n", userMessage, filePath, algoId,
          userId);
        logSevereMessage(detailedMessage);
        NotificationService.getInstance().notifyTo(SERVICE_NAME, userId,
          userMessage, false);
        return;
      }
      if (!rootZipEntry.isDirectory()) {
        String userMessage = getString("server.algoservice.error.invalid_version_pack");
        String detailedMessage = String.format(
          "%sDetalhes: o arquivo no possui um diretrio raz da verso.\n"
            + "Arquivo utilizado: %s.\nIdentificador do algoritmo: %s.\n"
            + "Identificador do usurio: %s.\nArquivo na raz: %s.\n",
          userMessage, filePath, algoId, userId, rootZipEntry);
        logSevereMessage(detailedMessage);
        NotificationService.getInstance().notifyTo(SERVICE_NAME, userId,
          userMessage, false);
        return;
      }
      String regex = "v_([0-9]{3})_([0-9]{3})_([0-9]{3})";
      Pattern versionPattern = Pattern.compile(regex);
      Matcher versionMatcher = versionPattern.matcher(rootDirectoryName);
      if (!versionMatcher.matches()) {
        String userMessage = getString("server.algoservice.error.invalid_version_pack");
        String detailedMessage = String.format(
          "%sDetalhes: o nome do diretrio raz no est no formato vlido.\n"
            + "Arquivo utilizado: %s.\nIdentificador do algoritmo: %s.\n"
            + "Identificador do usurio: %s.\nNome do arquivo: %s.\n"
            + "Expresso regular do padro: %s.\n", userMessage, filePath,
          algoId, userId, rootDirectoryName, regex);
        logSevereMessage(detailedMessage);
        NotificationService.getInstance().notifyTo(SERVICE_NAME, userId,
          userMessage, false);
        return;
      }
      int major = Integer.parseInt(versionMatcher.group(1));
      int minor = Integer.parseInt(versionMatcher.group(2));
      int patch = Integer.parseInt(versionMatcher.group(3));
      AlgorithmVersionId algorithmVersionId = new AlgorithmVersionId(major,
        minor, patch);
      String algorithmName = algoInfo.getName();
      if (algoInfo.getVersionInfo(algorithmVersionId) != null) {
        String userMessage = String.format(
          getString("server.algoservice.error.version_already_exists"),
          algorithmName, major, minor, patch);
        String detailedMessage = String.format(
          "%sIdentificador do algoritmo: %s\nArquivo utilizado: %s.\n"
            + "Identificador do usurio: %s.\n", userMessage, filePath, algoId,
          userId);
        logSevereMessage(detailedMessage);
        NotificationService.getInstance().notifyTo(SERVICE_NAME, userId,
          userMessage, false);
        return;
      }
      File repositoryDirectory = new File(AlgorithmInfo.REPOSITORY_NAME);
      File algorithmDirectory = new File(repositoryDirectory, algoInfo
        .getDirectory());
      File versionsDirectory = new File(algorithmDirectory,
        AlgorithmInfo.VERSIONS_DIR);
      unzip.decompress(versionsDirectory);
      File versionDirectory = new File(versionsDirectory, rootDirectoryName);
      File binaryDirectory = new File(versionDirectory,
        AlgorithmVersionInfo.BIN_DIR);
      if (!binaryDirectory.isDirectory()) {
        String userMessage = "O arquivo informado no  um pacote de verso de algoritmo.\n";
        String detailedMessage = String.format(
          "%s\nDiretrio de binrios ausente.\n"
            + "Identificador do algoritmo: %s\nArquivo utilizado: %s.\n"
            + "Identificador do usurio: %s.\n", userMessage, filePath, algoId,
          userId);
        logSevereMessage(detailedMessage);
        if (!FileUtils.delete(versionDirectory)) {
          detailedMessage = String.format(
            "Erro ao tentar excluir o arquivo %s.\n"
              + "Por favor, tente exclu-lo manualmente.\n", versionDirectory
              .getAbsolutePath());
          logWarningMessage(detailedMessage);
        }
        NotificationService.getInstance().notifyTo(SERVICE_NAME, userId,
          userMessage, false);
        return;
      }
      File[] platformDirectories = binaryDirectory.listFiles();
      for (File platformDirectory : platformDirectories) {
        if (!platformDirectory.isDirectory()) {
          String userMessage = "O arquivo informado no  um pacote de verso de algoritmo.\n";
          String detailedMessage = String.format(
            "%s\nO arquivo %s deveria ser um diretrio.\n"
              + "Identificador do algoritmo: %s\nArquivo utilizado: %s.\n"
              + "Identificador do usurio: %s.\n", userMessage,
            platformDirectory, filePath, algoId, userId);
          logSevereMessage(detailedMessage);
          if (!FileUtils.delete(versionDirectory)) {
            detailedMessage = String.format(
              "Erro ao tentar excluir o arquivo %s.\n"
                + "Por favor, tente exclu-lo manualmente.\n", versionDirectory
                .getAbsolutePath());
            logWarningMessage(detailedMessage);
          }
          NotificationService.getInstance().notifyTo(SERVICE_NAME, userId,
            userMessage, false);
          return;
        }
        File[] binaryFiles = platformDirectory.listFiles();
        for (File binaryFile : binaryFiles) {
          FileSystem.enableExecutionPermission(binaryFile.getAbsolutePath());
        }
      }
      algo.readVersion(versionsDirectory.getAbsolutePath(), versionDirectory);
      AlgorithmVersionInfo versionInfo = algoInfo
        .getVersionInfo(algorithmVersionId);
      String[] path = new String[] { getRootDirName(), algo.getInfo().getName() };
      String description = String.format(
        getString("server.algoservice.history.import_version"), versionInfo);
      appendHistory(userId, path, description);
    }
    catch (IOException e) {
      String userMessage = getString("server.algoservice.error.upload_version_server_io");
      String detailedMessage = String.format(
        "%sIdentificador do algoritmo: %s\n"
          + "Arquivo utilizado: %s.\nIdentificador do usurio: %s.\n",
        userMessage, filePath, algoId, userId);
      logSevereMessage(detailedMessage, e);
      NotificationService.getInstance().notifyTo(SERVICE_NAME, userId,
        userMessage, false);
    }
    catch (ServerException e) {
      String userMessage = getString("server.algoservice.error.upload_version_server");
      String detailedMessage = String.format(
        "%sIdentificador do algoritmo: %s\n"
          + "Arquivo utilizado: %s.\nIdentificador do usurio: %s.\n",
        userMessage, filePath, algoId, userId);
      logSevereMessage(detailedMessage, e);
      NotificationService.getInstance().notifyTo(SERVICE_NAME, userId,
        userMessage, false);
    }
    finally {
      if (!new File(filePath).delete()) {
        String msg = String.format("Erro ao tentar excluir o arquivo %s.\n"
          + "Por favor, tente exclu-lo manualmente.\n", filePath);
        logWarningMessage(msg);
      }
    }
  }

  /**
   * Obtm o caminho a ser utilizado para criar entradas no histrico.<br>
   * Este caminho estar no formato:
   * algotihms/&lt;major&gt;.&lt;minor&gt;.&lt;patch
   * &gt;/[(bin/&lt;plataforma&gt;)|config|html]/...<br>
   * Ex.: .../aeepec3D/1.0.0/bin/Windows
   * 
   * @param version Informaes da verso do arquivo.
   * @param pathFactory Fbrica para criar caminhos relativos ao sistema de
   *        arquivos. <br>
   *        Esta fbrica deve retornar um caminho no formato:
   *        .../&lt;algoritmo&gt
   *        ;/versions/v_&lt;major&gt;_&lt;minor&gt;_&lt;patch
   *        &gt;/[(bin/&lt;plataforma
   *        &gt;)|config|html]/.../[arquivo|(diretrio/)]<br>
   *        Ex.: .../aeepec3D/versions/v_001_000_000/bin/Windows/aeepec3D.lua
   * @param file Informaes do arquivo.
   * 
   * @return o caminho a ser utilizado para criar entradas no histrico.
   */
  private String getHistoryPath(final AlgorithmVersionInfo version,
    IPathFactory pathFactory, FileInfo file) {

    String fileSystemPath = pathFactory.getPath(file);
    String regex = "^.*/" + version.getInfo().getName()
      + "/[^/]+/[^/]+/(.+)/[^/]+/?$";
    String replacement = getRootDirName() + "/" + version.getInfo().getName()
      + "/" + version.getId() + "/$1";
    return fileSystemPath.replaceAll(regex, replacement);
  }

  // #TODO A remover quando todos sistemas CSBase forem assinados {
  /**
   * Thread responsvel por descompactar os executveis carregados (caso estejam
   * em um arquivo zip) e atualizar os objetos da lgica com os novos
   * executveis.
   */
  private class UpdateExecutableThread extends Thread {
    /**
     * Tamanho do buffer de leitura para arquivos zip
     * 
     * @see #extractZipEntry(ZipEntry, ZipInputStream, String)
     */
    private static final int BUFFER_SIZE = 65536;

    /** Extenso para arquivos ZIP */
    private static final String ZIP_EXTENSION = "zip";

    /** Identificador do algoritmo no qual o arquivo foi carregado */
    private final Object algoId;

    /** Caminho para o arquivo recm-carregado */
    private final String filePath;

    /** Nome da plataforma na qual o arquivo foi carregado */
    private final String platform;

    /** Identificador do usurio que realizou o carregamento do arquivo */
    private final Object userId;

    /** Identificador da verso na qual o arquivo foi carregado */
    private final Object versionId;

    /**
     * Cria a thread.
     * 
     * @param algoId Identificador do algoritmo.
     * @param versionId Identificador da verso do algoritmo.
     * @param platform Nome da plataforma.
     * @param filePath caminho do executvel recm-carregado.
     * @param userId identificador do usurio que carregou o executvel.
     */
    public UpdateExecutableThread(Object algoId, Object versionId,
      String platform, String filePath, Object userId) {
      this.algoId = algoId;
      this.versionId = versionId;
      this.platform = platform;
      this.filePath = filePath;
      this.userId = userId;
    }

    /**
     * Atualiza a representao lgica com o arquivo carregado. Se este for um
     * arquivo zip, seu contedo  extrado e o mesmo  removido, sendo que
     * somente os arquivos extrados sero includos na lgica. Por fim, dispara
     * uma notificao para os listeners avisando que a estrutura lgica mudou.
     */
    @Override
    public void run() {
      logInfoMessage("Thread iniciada: updateExecutable.");
      Algorithm algo = registeredAlgorithms.get(algoId);
      if (algo == null) {
        return;
      }
      Collection<String> fileNames = new HashSet<String>();
      fileNames.add(FileUtils.getFileName(filePath));
      if (isZip(filePath)) {
        fileNames = getZippedFileNames(filePath);
        if (fileNames == null) {
          return;
        }
        removeZip(filePath);
      }
      try {
        algo.updateExecutables(versionId, platform);
      }
      catch (OperationFailureException oe) {
        logAndNotifyError(userId, oe);
      }
      AlgorithmInfo info = algo.getInfo();
      fireModifyEvent(info);
      String[] path = new String[] { AlgorithmInfo.REPOSITORY_NAME,
          info.getName(), versionId.toString(), AlgorithmVersionInfo.BIN_DIR,
          platform };
      String description;
      for (String fileName : fileNames) {
        description = MessageFormat.format(
          getString(HISTORY_UPDATE_EXECUTABLE), new Object[] { fileName });
        appendHistory(userId, path, description);
      }
      logInfoMessage("Terminando Thread updateExecutable.");
    }

    /**
     * Extrai o arquivo especificado (<code>ZipEntry</code>) de uma stream de
     * leitura de um arquivo zip (<code>ZipInputStream</code>), gravando o
     * arquivo extrado no diretrio especificado.
     * 
     * @param entry arquivo compactado em um zip.
     * @param in <code>ZipInputStream</code> onde se encontra o arquivo.
     * @param outputPath caminho absoluto do diretrio de destino para o arquivo
     *        extrado.
     * 
     * @throws IOException se ocorrer algum problema durante a extrao.
     */
    private void extractZipEntry(ZipEntry entry, ZipInputStream in,
      String outputPath) throws IOException {
      BufferedOutputStream out = null;
      try {
        String outputFilePath = outputPath + File.separator + entry.getName();
        out = new BufferedOutputStream(new FileOutputStream(outputFilePath),
          BUFFER_SIZE);
        int bytesRead;
        byte[] buffer = new byte[BUFFER_SIZE];
        while ((bytesRead = in.read(buffer, 0, BUFFER_SIZE)) != -1) {
          out.write(buffer, 0, bytesRead);
        }
        out.flush();
      }
      catch (IOException e) {
        throw e;
      }
      finally {
        if (out != null) {
          try {
            out.close();
          }
          catch (IOException e) {
            logSevereMessage(
              "Falha ao fechar stream de gravao para o arquivo "
                + entry.getName(), e);
          }
        }
      }
    }

    /**
     * Obtm os nomes dos arquivos compactados, extrados do arquivo zip.
     * 
     * @param filePath caminho para o arquivo compactado.
     * 
     * @return nomes dos arquivos compactados ou <code>null</code> caso no
     *         tenha sido possvel extrair os arquivos.
     */
    private Collection<String> getZippedFileNames(String filePath) {
      Collection<String> fileNames = null;
      String outputPath = FileUtils.getFilePath(filePath);
      try {
        fileNames = unzip(filePath, outputPath);
      }
      catch (OperationFailureException oe) {
        String serverMsg = "Falha em pedido de servio para usurio " + userId;
        String zipName = FileUtils.getFileName(filePath);
        String clientMsg = MessageFormat
          .format(getString("server.algoservice.error.unzip"),
            new Object[] { zipName });
        Exception se = new ServiceFailureException(clientMsg, oe);
        logSevereMessage(serverMsg, se);
        NotificationService.getInstance().notifyTo(SERVICE_NAME, userId,
          clientMsg, false);
      }
      return fileNames;
    }

    /**
     * Indica se o arquivo possui extenso ZIP.
     * 
     * @param path caminho para o arquivo.
     * 
     * @return true caso o arquivo possua extenso ZIP.
     */
    private boolean isZip(String path) {
      return (ZIP_EXTENSION.equalsIgnoreCase(FileUtils.getFileExtension(path)));
    }

    /**
     * Mtodo de convenincia para registrar em log uma falha ocorrida, bem como
     * notificar o usurio do problema com uma mensagem amigvel.
     * 
     * @param userId identificador do usurio a ser notificado.
     * @param oe exceo ocorrida.
     */
    private void logAndNotifyError(final Object userId,
      OperationFailureException oe) {
      String serverMsg = "Falha em pedido de servio para usurio " + userId;
      String clientMsg = getString("server.algoservice.error.update_tree");
      Exception se = new ServiceFailureException(clientMsg, oe);
      logSevereMessage(serverMsg, se);
      NotificationService.getInstance().notifyTo(SERVICE_NAME, userId,
        clientMsg, false);
    }

    /**
     * Remove o arquivo zip especificado.
     * 
     * @param filePath caminho absoluto para o arquivo zip.
     */
    private void removeZip(final String filePath) {
      if (!(new File(filePath).delete())) {
        logSevereMessage("No foi possvel remover o arquivo zip " + filePath);
      }
    }

    /**
     * Descompacta um arquivo zip no diretrio especificado, retornando o nome
     * dos arquivos extrados.
     * 
     * @param inputFilePath caminho absoluto para o arquivo zip (incluindo o
     *        nome do arquivo).
     * @param outputPath caminho absoluto para o diretrio de destino dos
     *        arquivos extrados.
     * 
     * @return coleo com os nomes dos arquivos extrados. Se ocorrer algum
     *         problema, retorna uma lista vazia (no-nula).
     * 
     * @throws OperationFailureException
     */
    private Collection<String> unzip(String inputFilePath, String outputPath)
      throws OperationFailureException {
      ZipInputStream in = null;
      Collection<String> extractedFileNames = new HashSet<String>();
      try {
        in = new ZipInputStream(new BufferedInputStream(new FileInputStream(
          inputFilePath)));
        ZipEntry entry;
        while ((entry = in.getNextEntry()) != null) {
          extractedFileNames.add(entry.getName());
          extractZipEntry(entry, in, outputPath);
        }
      }
      catch (IOException e) {
        String serverMsg = "Falha ao abrir arquivo " + inputFilePath;
        throw new OperationFailureException(serverMsg, e);
      }
      finally {
        if (in != null) {
          try {
            in.close();
          }
          catch (IOException e) {
            logSevereMessage("Falha ao fechar arquivo " + inputFilePath, e);
          }
        }
      }
      return extractedFileNames;
    }
  }

  // #TODO } A remover quando todos sistemas CSBase forem assinados

  /**
   * Thread que recarrega os algoritmos no intervalo definido pela propriedade
   * <code>restartServiceIntervalInMinutes</code>. Caso esta propriedade no
   * seja definida a recarga no  feita.
   */
  private class ReloadAlgsThread extends Thread {
    /** Intervalo de renicio do servio de algoritmos. */
    private final int reloadIntervalInMilis;

    /**
     * Construtor.
     * 
     * @param reloadAlgIntervalInMinutes intervalo de recarga dos algoritmos.
     */
    ReloadAlgsThread(int reloadAlgIntervalInMinutes) {
      reloadIntervalInMilis = reloadAlgIntervalInMinutes * 60 * 1000;
      setDaemon(true);
      start();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void run() {
      while (!shutdown) {
        try {
          sleep(reloadIntervalInMilis);
        }
        catch (InterruptedException e) {
        }
        if (shutdown) {
          break;
        }
        try {
          setUserId(User.ADMIN_LOGIN);
          reloadAlgorithms(false);
        }
        catch (PermissionException pe) {
          logSevereMessage(
            "Usurio admin no pode fazer recarga dos algoritmos", pe);
        }
      }
    }
  }

  /**
   * Retorna o path do arquivo de configurao de nomes de verso.
   * 
   * @return o path
   */
  final String getVersionPropertyNamesPath() {
    final String tag = "version.property.names.file";
    final String path = getStringProperty(tag);
    return path;
  }

  /**
   * Retorna o path do arquivo de configurao de nomes em algoritmos. Este
   * arquivo de propriedades que especifica quais sao os atributos estendidos,
   * alem de especifica a ordem que os atributos devem ser apresentados
   * 
   * @return o path
   */
  final public String getAlgorithmPropertyNamesPath() {
    final String tag = "algorithm.property.names.file";
    final String path = getStringProperty(tag);
    return path;
  }

  @Override
  public Category createCategory(String parentCategoryId, String name)
    throws RemoteException, CategoriesFileNotSavedException {

    // Busca a categoria pai, que vai receber uma nova sub-categoria
    Category parentCategory = null;
    if (parentCategoryId != null) {
      parentCategory = categorySet.getCategory(parentCategoryId);
      if (parentCategory == null) {
        logSevereMessage("Falha na criao da categoria [" + name
          + "]: categoria pai inexistente.");
        return null;
      }
    }
    Category category = categorySet.createCategory(parentCategory, name);

    try {
      if (writeCategoriesFile(categorySet)) {
        setCategorySetSavedFlag(true);
      }
    }
    catch (CategoriesFileNotSavedException ex) {
      logSevereMessage("Falha na criao da categoria [" + name
        + "]: arquivo xml no foi salvo.");
      setCategorySetSavedFlag(false);
      throw ex;
    }
    finally {
      AlgoEvent action = new AlgoEvent(AlgoEvent.CREATE, category);
      setChanged();
      notifyObservers(action);
    }
    return category;
  }

  /**
   * Estabelece no conjunto de categorias um flag indicando se o conjunto foi ou
   * no salvo com sucesso. A persistncia  feita no xml correspondente ao
   * conjunto de categorias.
   * 
   * @param state se true, indica que o conjunto de categorias foi salvo, caso
   *        contrrio, ocorreu um erro durante a persistncia e o conjunto no
   *        foi salvo
   */
  private void setCategorySetSavedFlag(boolean state) {
    categorySet.setCategorySetSavedFlag(state);
  }

  /**
   * Escreve o arquivo de categorias no diretrio raiz do repositrio de
   * algoritmos, a partir do conjunto de categorias especificado.
   * 
   * @param categorySet conjunto de categorias de algoritmos a ser gravado
   * @return retorna true, se as categorias foram gravadas com sucesso, caso
   *         contrrio, retorna false
   * 
   * @throws CategoriesFileNotSavedException erro no salvamento do arquivo de
   *         categorias, por falha no diretrio raiz de algoritmos
   */
  private boolean writeCategoriesFile(CategorySet categorySet)
    throws CategoriesFileNotSavedException {
    String algoRootDir = getAndCheckAlgoRootDir();
    if (algoRootDir == null) {
      throw new CategoriesFileNotSavedException(
        getString("AlgoService.error.category.root_dir.invalid"));
    }
    return writeCategoriesFile(algoRootDir, categorySet);
  }

  /**
   * Escreve o arquivo de categorias no diretrio raiz do repositrio de
   * algoritmos, a partir do conjunto de categorias especificado.
   * 
   * @param algoRootDir diretrio raiz do repositrio de algoritmos
   * @param categorySet conjunto de categorias de algoritmos a ser gravado
   * @return retorna true, se as categorias foram gravadas com sucesso, caso
   *         contrrio, retorna false
   * @throws CategoriesFileNotSavedException erro ao gravar o arquivo xml de
   *         categorias
   */
  private boolean writeCategoriesFile(String algoRootDir,
    CategorySet categorySet) throws CategoriesFileNotSavedException {
    File categoriesFile = new File(algoRootDir, "categories.xml");
    FileWriter catFileWriter = null;

    try {
      catFileWriter = new FileWriter(categoriesFile);
      XmlCategoriesWriter xmlWriter = new XmlCategoriesWriter(categorySet);
      xmlWriter.write(catFileWriter);
    }
    catch (IOException e) {
      String message = String.format(
        "Erro ao ler/escrever o arquivo de categorias (%s).", categoriesFile
          .getAbsolutePath());
      logSevereMessage(message, e);
      throw new CategoriesFileNotSavedException(
        getString("AlgoService.error.categories.file.not_saved"));
    }
    finally {
      if (catFileWriter != null) {
        try {
          catFileWriter.close();
        }
        catch (IOException e) {
          String message = String.format(
            "Erro ao tentar fechar o stream de escrita para "
              + "o arquivo de categorias (%s).", categoriesFile
              .getAbsolutePath());
          logSevereMessage(message, e);
          throw new CategoriesFileNotSavedException(
            getString("AlgoService.error.categories.file.not_saved"));
        }
      }
    }
    return true;
  }

  @Override
  public CategorySet bindAlgorithmsToCategories(List<Object> algoIds,
    List<String> categoryIds) throws RemoteException,
    CategoriesFileNotSavedException {

    CategorySet modifiedCategories = getModifiedCategorySet(categoryIds);
    List<AlgorithmInfo> algoInfoList = getAlgorithmInfoList(algoIds);
    categorySet.addAlgorithmsToCategories(modifiedCategories.getCategories(),
      algoInfoList);

    try {
      if (writeCategoriesFile(categorySet)) {
        setCategorySetSavedFlag(true);
      }
    }
    catch (CategoriesFileNotSavedException ex) {
      logSevereMessage("Falha na associao de algoritmos s categorias: arquivo xml no foi salvo.");
      setCategorySetSavedFlag(false);
      throw ex;
    }
    finally {
      AlgoEvent action = new AlgoEvent(AlgoEvent.MODIFY, modifiedCategories);
      setChanged();
      notifyObservers(action);
    }
    return modifiedCategories;
  }

  @Override
  public CategorySet unbindAlgorithmsFromCategories(List<Object> algoIds,
    List<String> categoryIds) throws RemoteException,
    CategoriesFileNotSavedException {

    CategorySet modifiedCategories = getModifiedCategorySet(categoryIds);
    List<AlgorithmInfo> algoInfoList = getAlgorithmInfoList(algoIds);

    categorySet.removeAlgorithmsFromCategories(modifiedCategories
      .getCategories(), algoInfoList);

    try {
      if (writeCategoriesFile(categorySet)) {
        setCategorySetSavedFlag(true);
      }
    }
    catch (CategoriesFileNotSavedException ex) {
      logSevereMessage("Falha na desassociao de algoritmos s categorias: arquivo xml no foi salvo.");
      setCategorySetSavedFlag(false);
      throw ex;
    }
    finally {
      AlgoEvent action = new AlgoEvent(AlgoEvent.MODIFY, modifiedCategories);
      setChanged();
      notifyObservers(action);
    }
    return modifiedCategories;
  }

  /**
   * Obtm um conjunto de categorias, contendo as categorias que foram
   * modificadas, a partir dos ids especificados.
   * 
   * @param categoryIds identificadores das categorias procuradas
   * @return o conjunto com as categorias modificadas, ou um conjunto vazio,
   *         caso nenhum dos ids informados tenham sido encontrados
   */
  private CategorySet getModifiedCategorySet(List<String> categoryIds) {
    CategorySet modifiedCategorySet = new CategorySet();
    for (String categoryId : categoryIds) {
      Category category = categorySet.getCategory(categoryId);
      if (category == null) {
        logSevereMessage("Categoria com id [" + categoryId + "] inexistente.");
        continue;
      }
      modifiedCategorySet.addCategory(category);
    }
    return modifiedCategorySet;
  }

  /**
   * Obtm uma lista com as informaes dos algoritmos, a partir dos seus
   * identificadores.
   * 
   * @param algoIds identificadores dos algoritmos procurados
   * @return uma lista com as informaes dos algoritmos
   */
  private List<AlgorithmInfo> getAlgorithmInfoList(List<Object> algoIds) {
    List<AlgorithmInfo> algoInfoList = new java.util.Vector<AlgorithmInfo>();
    for (Object algoId : algoIds) {
      AlgorithmInfo algoInfo = getAlgorithmInfo(algoId);
      if (algoInfo == null) {
        logSevereMessage("Falha na associao de algoritmos s categorias: algoritmo com id ["
          + algoId + "] inexistente.");
      }
      algoInfoList.add(algoInfo);
    }
    return algoInfoList;
  }

  /**
   * Obtm as informaes de um determinado algoritmo.
   * 
   * @param algoId identificador do algoritmo procurado
   * @return as informaes do algoritmo
   */
  private AlgorithmInfo getAlgorithmInfo(Object algoId) {
    Algorithm algorithm = registeredAlgorithms.get(algoId);
    if (algorithm != null) {
      return algorithm.getInfo();
    }
    return null;
  }

  @Override
  public Category removeCategory(String id) throws RemoteException,
    CategoriesFileNotSavedException {

    Category removedCategory = categorySet.removeCategory(id);
    if (removedCategory == null) {
      logSevereMessage("Falha na remoo da categoria [" + id
        + "]: categoria inexistente.");
      return null;
    }

    try {
      if (writeCategoriesFile(categorySet)) {
        setCategorySetSavedFlag(true);
      }
    }
    catch (CategoriesFileNotSavedException ex) {
      logSevereMessage("Falha na remoo da categoria com id [" + id
        + "]: arquivo xml no foi salvo.");
      setCategorySetSavedFlag(true);
      throw ex;
    }
    finally {
      AlgoEvent action = new AlgoEvent(AlgoEvent.DELETE, removedCategory);
      setChanged();
      notifyObservers(action);
    }
    return removedCategory;
  }

  @Override
  public Category getCategory(String id) throws RemoteException {
    return categorySet.getCategory(id);
  }

  /**
   * Obtm o diretrio raiz do repositrio de algoritmos e verifica se  um
   * diretrio vlido.
   * 
   * @return retorna o nome do diretrio raiz do repositrio de algoritmos, ou
   *         retorna null caso no seja um diretrio vlido.
   */
  private String getAndCheckAlgoRootDir() {
    String algoRootDir = getRootDirName();
    try {
      Server.getInstance().checkDirectory(algoRootDir);
    }
    catch (ServerException e) {
      return null;
    }
    return algoRootDir;
  }

  /**
   * L o arquivo de categorias do diretrio raiz do repositrio de algoritmos,
   * e preenche o conjunto de categorias especificado.
   * 
   * @param categorySet conjunto de categorias de algoritmos a ser preenchido
   *        pelo leitor
   * @return retorna true, se as categorias foram lidas com sucesso, caso
   *         contrrio, retorna false
   * 
   */
  private boolean readCategoriesFile(CategorySet categorySet) {
    String algoRootDir = getAndCheckAlgoRootDir();
    if (algoRootDir == null) {
      String message = String
        .format("Erro ao ler o arquivo de categorias: diretrio raiz de algoritmos invlido. ");
      logSevereMessage(message);
      return false;
    }
    return readCategoriesFile(algoRootDir, categorySet);
  }

  /**
   * L o arquivo de categorias do diretrio raiz do repositrio de algoritmos,
   * e preenche o conjunto de categorias especificado.
   * 
   * @param algoRootDir diretrio raiz do repositrio de algoritmos
   * @param categorySet conjunto de categorias de algoritmos a ser preenchido
   *        pelo leitor
   * @return retorna true, se as categorias foram lidas com sucesso, caso
   *         contrrio, retorna false
   * 
   */
  private boolean readCategoriesFile(String algoRootDir, CategorySet categorySet) {
    File categoriesFile = new File(algoRootDir, "categories.xml");
    if (!categoriesFile.exists()) {
      return false;
    }

    InputStream inputStream = null;
    try {
      inputStream = new FileInputStream(categoriesFile);

      final XmlCategoriesReader categoriesReader = new XmlCategoriesReader(
        categorySet, inputStream);
      categoriesReader.read();
    }
    catch (AlgorithmNotFoundException e) {
      String message = String.format(
        "Erro no parser xml do arquivo de categorias (%s).", categoriesFile
          .getAbsolutePath());
      logSevereMessage(message, e);
    }
    catch (IOException e) {
      String message = String.format(
        "Erro ao ler o arquivo de categorias (%s).", categoriesFile
          .getAbsolutePath());
      logSevereMessage(message, e);
      return false;
    }
    finally {
      if (isFirstCategoryLoad) {
        categorySet.changeCategoryIds();
        logCategorySet(categorySet);
        writeCategoriesFile(categorySet);
      }
      if (inputStream != null) {
        try {
          inputStream.close();
          return true;
        }
        catch (IOException e) {
          String message = String.format(
            "Erro ao tentar fechar o stream de leitura para "
              + "o arquivo de categorias (%s).", categoriesFile
              .getAbsolutePath());
          logSevereMessage(message, e);
        }
        inputStream = null;
      }
    }

    return true;
  }

  /**
   * Gera no log as informaes de todas as categorias (inclusive as
   * sub-categorias) que foram criadas, com o identificador da categoria pai, o
   * identificador da categoria sendo criada e o seu nome.
   * 
   * @param categorySet conjunto de categorias
   */
  private void logCategorySet(CategorySet categorySet) {
    if (categorySet == null) {
      return;
    }

    List<Category> allCategories = categorySet.getAllCategories();
    for (Category category : allCategories) {
      String msg = "Id da categoria pai: " + category.getParentId()
        + " - Criao da categoria com id = " + category.getId() + " : "
        + category.getName();
      // System.out.println(msg);
      logInfoMessage(msg);
    }
    logInfoMessage("Nmero de categorias da raiz: " + categorySet.getSize());
    logInfoMessage("Nmero total de categorias criadas: "
      + allCategories.size());
  }

  @Override
  public CategorySet setAlgorithmsToCategories(List<Object> algoIds,
    List<String> categoryIds) throws RemoteException,
    CategoriesFileNotSavedException {
    CategorySet modifiedCategories = getModifiedCategorySet(categoryIds);
    List<AlgorithmInfo> algoInfoList = getAlgorithmInfoList(algoIds);
    categorySet.removeAllAlgorithmsFromCategories(modifiedCategories
      .getCategories());
    categorySet.addAlgorithmsToCategories(modifiedCategories.getCategories(),
      algoInfoList);

    try {
      if (writeCategoriesFile(categorySet)) {
        setCategorySetSavedFlag(true);
      }
    }
    catch (CategoriesFileNotSavedException ex) {
      logSevereMessage("Falha na associao de algoritmos s categorias: arquivo xml no foi salvo.");
      setCategorySetSavedFlag(false);
      throw ex;
    }
    finally {
      AlgoEvent action = new AlgoEvent(AlgoEvent.MODIFY, modifiedCategories);
      setChanged();
      notifyObservers(action);
    }
    return modifiedCategories;
  }

}