package csbase.server.services.algorithmservice.niotransfer;

import java.io.File;
import java.io.IOException;
import java.nio.channels.SelectionKey;
import java.nio.channels.SocketChannel;

import csbase.remote.niotransfer.SimpleFileTransferProtocol;

/**
 * Classe que define o papel do servidor no tratamento do protocolo de 
 * transmisso de arquivos.
 *
 * @author Tecgraf
 */
class ServerSimpleFileTransferProtocol extends SimpleFileTransferProtocol 
implements Runnable {

  /** Indica se existem bytes pendentes a serem transmitidos ou recebidos. */
  private boolean pendingBytes;

  /** Chave representando a conexo com o cliente. */
  private SelectionKey connectionKey;

  /** Entidade que solicitou a transferncia. */
  private FileTransferrer transferrer;

  /** Referncia para o servidor de transferncia de arquivos. */
  private FileServer fileServer;
  
  /**
   * Cria o objeto que implementa o protocolo no servidor, indicando o canal de 
   * comunicao com o cliente.
   *
   * @param fileServer servidor de transferncia de arquivos.
   * @param capacity capacidade do buffer a ser utilizado.
   * @param key chave representando a ligao entre o canal com o cliente e o
   *        conjunto de interesses.
   * @param clientChannel referncia para o canal com o cliente.
   * @param transferrer entidade que solicitou a transferncia.
   */
  ServerSimpleFileTransferProtocol(
    FileServer fileServer, int capacity, SelectionKey key, 
    SocketChannel clientChannel, FileTransferrer transferrer) {
    super(capacity, clientChannel);
    this.connectionKey = key;
    this.transferrer = transferrer;
    this.fileServer = fileServer;
  }
  
  /**
   * {@inheritDoc}
   */
  @Override
  protected boolean contentMissing() throws IOException {
    switch (operation) {
      case EXPORT:
        return (io.availableForReading() > 0);
      case IMPORT:
        return (totalBytesReceived < fileSize);
      default:
        throw new IllegalStateException("operao "+operation+" desconhecida");
    }
  }
  
  /**
   * Aguarda o recebimento do tamanho da chave para 
   * prosseguir na seqncia do protocolo.
   *
   * @return <code>true</code> se o tamanho da chave tiver sido
   *         recebido com sucesso.
   *         
   * @throws IOException
   */
  @Override
  protected boolean processKeySize() throws IOException {
    transferKeySize = io.readInt();
    if (transferKeySize == -1) {
      fileServer.addChangeRequest(
        new ChangeRequest(connectionKey, SelectionKey.OP_READ));
      pendingBytes = true;
      return false;
    }  
    pendingBytes = false;
    return true;
  }

  /**
   * Aguarda o recebimento do caminho da chave para prosseguir na seqncia do 
   * protocolo. De posse da chave, consegue obter as informaes necessrias
   * para efetuar a transferncia, como a operao a ser realizada 
   * (upload/download) e o caminho de destino/origem no servidor. Se estiver 
   * recebendo, cria um arquivo vazio no caminho indicado.
   * Se estiver enviando, abre um canal de leitura para transmitir o arquivo.
   *
   * @return <code>true</code> se a chave tiver sido recebida com sucesso.
   *         
   * @throws IOException
   */  
  @Override
  protected boolean processKey() throws IOException {
    transferKey = io.readString(transferKeySize);
    if (transferKey == null) {
      // No conseguiu ler tudo ainda. No muda o estado, aguardando a prxima
      // notificao.
      fileServer.addChangeRequest(
        new ChangeRequest(connectionKey, SelectionKey.OP_READ));
      pendingBytes = true;
      return false;
    }
    pendingBytes = false;
    DoTransferInfo info = fileServer.getTransferInfo(transferKey);
    String path = info.getPath();
    operation = info.getOperation();
    File file = new File(path);
    switch (operation) {
      case IMPORT:
        io.createTarget(file);    
        break;
      case EXPORT:
        fileSize = file.length();
        io.createSource(file);
        break;
      default:
        // A operao j foi validada anteriormente.
        break;
    }
    return true;
  }

  /**
   * Aguarda o recebimento do tamanho do arquivo (no caso de estar recebendo)
   * ou envia o tamanho (no caso de estar enviando um arquivo), para 
   * prosseguir na seqncia do protocolo.
   *
   * @return <code>true</code> se o tamanho arquivo tiver sido
   *         recebido com sucesso.
   *         
   * @throws IOException
   */  
  @Override
  protected boolean processFileSize() throws IOException {
    switch (operation) {
      case EXPORT:
        boolean success = io.writeLong(fileSize);
        if (!success) {
          fileServer.addChangeRequest(
            new ChangeRequest(connectionKey, SelectionKey.OP_WRITE));
          pendingBytes = true;
          return false;
        }
        pendingBytes = false;
        return true;
      case IMPORT:
        fileSize = io.readLong();
        if (fileSize == -1) {
          fileServer.addChangeRequest(
            new ChangeRequest(connectionKey, SelectionKey.OP_READ));
          pendingBytes = true;
          return false;
        }  
        pendingBytes = false;
        return true;
      default:
        return false;
    }
  }

  /**
   * Aguarda o recebimento (ou envia, no caso de estar enviando um arquivo) do 
   * tamanho de um <b>pacote</b> para prosseguir na seqncia do protocolo.
   *
   * @return <code>true</code> se o tamanho de um <b>pacote</b> tiver sido
   *         recebido com sucesso.
   *         
   * @throws IOException
   */  
  @Override
  protected boolean processPackSize() throws IOException {
    switch (operation) {
      case EXPORT:
        packSize = io.availableForReading();
        packSize = 
          calcPackSizeThatFitsInBuffer(packSize, io.getContentBufferSize());    
        boolean success = io.writeInt(packSize);
        if (!success) {
          fileServer.addChangeRequest(
            new ChangeRequest(connectionKey, SelectionKey.OP_WRITE));
          pendingBytes = true;
          return false;
        }
        pendingBytes = false;
        return true;
      case IMPORT:
        packSize = io.readInt();
        if (packSize == -1) {
          fileServer.addChangeRequest(
            new ChangeRequest(connectionKey, SelectionKey.OP_READ));
          pendingBytes = true;
          return false;
        }  
        pendingBytes = false;
        if (packSize > fileSize) {
          throw new IOException("packSize > fileSize");
        }
        return true;
      default:
        return false;
    }
  }

  /**
   * Aguarda o recebimento (ou envia, no caso de estar enviando um arquivo) de 
   * um <b>pacote</b> para prosseguir na seqncia do protocolo.
   *
   * @return <code>true</code> se o <b>pacote</b> tiver sido
   *         recebido com sucesso.
   *         
   * @throws IOException
   */  
  @Override
  protected boolean processPack() throws IOException {
    switch (operation) {
      case EXPORT:
        boolean success = io.writePack(packSize);
        if (!success) {
          fileServer.addChangeRequest(
            new ChangeRequest(connectionKey, SelectionKey.OP_WRITE));
          pendingBytes = true;
          return false;
        }
        pendingBytes = false;
        return true;
      case IMPORT:
        long bytesReceived = io.readPack(fileSize, packSize);
        if (bytesReceived == -1) {
          fileServer.addChangeRequest(
            new ChangeRequest(connectionKey, SelectionKey.OP_READ));
          pendingBytes = true;
          return false;
        }  
        pendingBytes = false;
        totalBytesReceived += bytesReceived;
        return true;
      default:
        return false;
    }
  }

  /**
   * Aguarda o recebimento (ou envia, no caso de estar enviando um arquivo) de 
   * um cdigo confirmando o sucesso da transferncia para prosseguir na 
   * seqncia do protocolo.
   *
   * @return <code>true</code> se a confirmao tiver sido recebida com sucesso.
   *         
   * @throws IOException
   */  
  @Override
  protected boolean processAck() throws IOException {
    switch (operation) {
      case EXPORT:
        int result = io.readInt();
        if (result == -1) {
          fileServer.addChangeRequest(
            new ChangeRequest(connectionKey, SelectionKey.OP_READ));
          pendingBytes = true;
          return false;
        }
        else if (result != ACK_VALUE) {
          throw new IOException("result != ACK_VALUE");
        }
        return true;
      case IMPORT:
        fileServer.channelClosing(transferKey);
        boolean success = io.writeInt(ACK_VALUE);
        if (!success) {
          fileServer.addChangeRequest(
            new ChangeRequest(connectionKey, SelectionKey.OP_WRITE));
          pendingBytes = true;
          return false;
        }
        pendingBytes = false;
        return true;
      default:
        return false;
    }
  }
  
  /**
   * {@inheritDoc}
   */
  @Override
  protected void closeResources() throws IOException {
    io.closeResources();
    fileServer.addChangeRequest(new ChangeRequest(connectionKey));
  }
  
  /**
   * Trata a ocorrncia de uma exceo no servidor, fechando a conexo com o 
   * cliente e logando a exceo ocorrida.
   * 
   * @param e exceo.
   */
  public void handleError(Exception e) {
  }

  /**
   * Executa o passo atual do protocolo.
   */
  public void run() {
    try {
      processState();
    }
    catch (Exception e) {
      try {
        io.closeResources();
        fileServer.addChangeRequest(new ChangeRequest(connectionKey));
      } 
      catch (IOException e1) {
        transferrer.logSevereMessage(
          "Erro no cancelamento da transferncia", e1);
      }
      finally {
        transferrer.logSevereMessage("Erro na transferncia", e);
      }
    }
  }

  /**
   * {@inheritDoc}. O servidor, por ser no-bloqueante, precisa de uma 
   * interrupo externa do processamento do protocolo se estiver necessitando
   * receber dados do cliente, caso contrrio pode ficar em um loop 
   * desnecessrio.
   */
  @Override
  protected boolean hasToWait() {
    return pendingBytes;
  }
  
}
