/*
 * $Id: ApplicationService.java 105117 2010-05-05 19:28:16Z clinio $
 */
package csbase.server.services.applicationservice;

import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.rmi.RemoteException;
import java.util.Hashtable;
import java.util.List;
import java.util.Properties;

import csbase.logic.applicationservice.ApplicationCategory;
import csbase.logic.applicationservice.ApplicationRegistry;
import csbase.logic.applicationservice.ApplicationRegistryCreationException;
import csbase.remote.ApplicationServiceInterface;
import csbase.remote.ClientRemoteLocator;
import csbase.server.ServerException;
import csbase.server.Service;
import csbase.util.remote.RemoteInputStream;

/**
 * @author Tecgraf/PUC-Rio
 */
public final class ApplicationService extends Service implements
  ApplicationServiceInterface {

  /**
   * Registro de aplicaes
   */
  final private Hashtable<String, ApplicationRegistry> registries =
    new Hashtable<String, ApplicationRegistry>();

  /**
   * Registro de aplicaes
   */
  final private Hashtable<String, ApplicationCategory> categories =
    new Hashtable<String, ApplicationCategory>();

  /**
   * Cria a imagem da aplicao.
   * 
   * @param id id da aplicao.
   * @param directory o diretrio da aplicao.
   * @param size tamanho.
   * @return a imagem <code>Icon</code>.
   * @throws ServerException em caso de falha na carga.
   */
  private byte[] buildImage(final String id, final File directory,
    final int size) throws ServerException {
    final List<String> extensions = getStringListProperty("image.extension");
    File file = null;
    for (final String ext : extensions) {
      final String fileName = id + "." + size + "." + ext;
      final File f = new File(directory, fileName);
      if (f.exists()) {
        file = f;
        break;
      }
    }

    if (file == null) {
      final String path = directory.getAbsolutePath();
      final String prefix = "No foi localizado cone de apl./cat. em: ";
      final String err = prefix + path + " - tamanho: " + size;
      logWarningMessage(err);
      return null;
    }

    if (!file.canRead()) {
      final String path = directory.getAbsolutePath();
      final String prefix = "Sem permisso de leitura de: ";
      final String err = prefix + path + " - tamanho: " + size;
      throw new ServerException(err);
    }

    final String path = file.getAbsolutePath();
    final long length = file.length();
    if (length >= Integer.MAX_VALUE) {
      final String err = "cone muito grande para apl./cat.: " + path;
      throw new ServerException(err);
    }

    try {
      final int sz = (int) length;
      final byte[] array = new byte[sz];
      final FileInputStream fStream = new FileInputStream(file);
      fStream.read(array);
      fStream.close();
      return array;
    }
    catch (final Exception e) {
      final String err = "Falha carga de cone de aplicao/categoria: " + path;
      throw new ServerException(err, e);
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Hashtable<String, ApplicationCategory> getApplicationCategories()
    throws RemoteException {
    return categories;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Hashtable<String, ApplicationRegistry> getApplicationRegistries() {
    return registries;
  }

  /**
   * Busca um arquivo interno localizado dentro do repositrio de
   * aplicaes.
   * @param appId identificador da aplicao.
   * @param resourcePath path do resource dentro do subdiretrio de sua definio.
   * @return o arquivo.
   */
  private File getApplicationFileResource(final String appId,
    final String[] resourcePath) {
    if (resourcePath == null || resourcePath.length < 1) {
      return null;
    }
    
    final int pathLength = resourcePath.length;
    String path = resourcePath[0];
    for (int i = 1; i < pathLength; i++) {
      path = path + File.separator + resourcePath[i];   
    }
    
    final String key = "repository.directory";
    final List<String> repPaths = getStringListProperty(key);
    if (repPaths.size() == 0) {
      return null;
    }

    for (final String repPath : repPaths) {
      final File repository = new File(repPath);
      final File appsRepository = getApplicationsRepository(repository);
      final File app = new File(appsRepository, appId);
      final File resource = new File(app, path);
      if (resource.exists()) {
        return resource;
      }
    }
    return null;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public RemoteInputStream getApplicationResourceAsStream(final String appId,
    final String[] resourcePath) throws RemoteException {
    final File resource = getApplicationFileResource(appId, resourcePath);
    if (resource == null) {
      return null;
    }
    if (resource.exists()) {
      try {
        return new RemoteInputStream(new FileInputStream(resource));
      }
      catch (final FileNotFoundException e) {
        throw new RemoteException("FileNotFoundException no servidor", e);
      }
    }
    return null;
  }

  /**
   * Obtm o repositrio de aplicaes interno ao repositrio.
   * 
   * @param repository o repositrio
   * @return O repositrio de aplicaes.
   */
  private File getApplicationsRepository(final File repository) {
    final File appsRepository = new File(repository, "applications");
    return appsRepository;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected boolean has2Update(final Object arg, final Object event) {
    return true;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void initService() throws ServerException {
    loadRepositories();
  }

  /**
   * Faz a carga de um diretrio de aplicaes.
   * 
   * @param appDirectory o diretrio que representa a aplicao.
   * @throws ServerException em caso de falha.
   */
  private void loadApplication(final File appDirectory) throws ServerException {
    final String id = appDirectory.getName();
    final ApplicationRegistry existant = registries.get(id);
    if (existant != null) {
      final String err = "Dupla de definio de aplicao id: " + id;
      throw new ServerException(err);
    }

    final ApplicationRegistry reg;
    final String fileName = id + ".properties";
    final File propertyFile = new File(appDirectory, fileName);
    try {
      final Properties properties = new Properties();
      final InputStream in = new FileInputStream(propertyFile);
      properties.load(in);

      final byte[] img16 = buildImage(id, appDirectory, 16);
      final byte[] img32 = buildImage(id, appDirectory, 32);
      reg = new ApplicationRegistry(id, properties, img16, img32);
    }
    catch (final IOException e) {
      final String path = propertyFile.getAbsolutePath();
      final String err = "Falha de definio de aplicao: " + path;
      throw new ServerException(err, e);
    }
    catch (final ApplicationRegistryCreationException are) {
      final String path = propertyFile.getAbsolutePath();
      final String name = are.getPropertyName();
      final String prefix = "Prop. de aplicao [" + name + "] indefinida";
      final String err = prefix + " para [" + id + "] em : " + path;
      throw new ServerException(err, are);
    }

    registries.put(id, reg);
    final String className = reg.getClassName();
    logInfoMessage("Registrada aplicao: " + id + " - " + className);
  }

  /**
   * Faz a carga do repositrio de aplicaes.
   * 
   * @param appDirectory repositrio
   * @throws ServerException em caso de falha.
   */
  private void loadApplicationRepository(final File appDirectory)
    throws ServerException {
    if (!appDirectory.exists()) {
      final String err = "No foi encontrado o diretrio de aplicaes: ";
      logWarningMessage(err + appDirectory.getAbsolutePath());
      return;
    }

    if (!appDirectory.isDirectory()) {
      final String err = "No  diretrio (aplicaes): ";
      throw new ServerException(err + appDirectory.getAbsolutePath());
    }

    final FileFilter filter = new FileFilter() {
      @Override
      public boolean accept(final File file) {
        final boolean isDir = file.isDirectory();
        final String name = file.getName();
        final boolean nameOk = !name.startsWith(".");
        return isDir && nameOk;
      }
    };

    final File[] directories = appDirectory.listFiles(filter);
    for (final File directory : directories) {
      loadApplication(directory);
    }
  }

  /**
   * Faz a carga de ids de aplicao.
   * 
   * @param category categoria
   * @param properties propriedades
   * @throws ServerException em caso de categoria vazia.
   */
  private final void loadApplicationsIdsForCategory(
    final ApplicationCategory category, final Properties properties)
    throws ServerException {
    final String id = category.getId();
    boolean noApplicationInside = true;
    for (int i = 1; true; i++) {
      final String key = id + ".application." + i;
      final String v = properties.getProperty(key);
      if (v == null) {
        break;
      }
      final String value = v.trim();
      category.addApplicationId(value);
      noApplicationInside = false;
    }
    if (noApplicationInside) {
      final String err = "Categoria de aplicaes vazia: " + id;
      throw new ServerException(err);
    }
  }

  /**
   * Faz a carga de um diretrio de aplicaes.
   * 
   * @param catDirectory o diretrio que representa a aplicao.
   * @throws ServerException em caso de falha.
   */
  private void loadCategory(final File catDirectory) throws ServerException {
    final String id = catDirectory.getName();
    final ApplicationCategory existant = categories.get(id);
    if (existant != null) {
      final String err = "Dupla de definio de grupo de applicao: " + id;
      throw new ServerException(err);
    }

    final ApplicationCategory category;
    final String fileName = id + ".properties";
    final File file = new File(catDirectory, fileName);
    try {
      final Properties properties = new Properties();
      final InputStream in = new FileInputStream(file);
      properties.load(in);

      final byte[] img16 = buildImage(id, catDirectory, 16);
      final byte[] img32 = buildImage(id, catDirectory, 32);
      category = new ApplicationCategory(id, properties, img16, img32);
      loadApplicationsIdsForCategory(category, properties);
    }
    catch (final IOException e) {
      final String path = file.getAbsolutePath();
      final String err = "Falha de definio de aplicao: " + path;
      throw new ServerException(err, e);
    }

    categories.put(id, category);
    logInfoMessage("Registrado categoria de aplicao: " + id + ".");
  }

  /**
   * Faz a carga do repositrio de aplicaes.
   * 
   * @param catDirectory repositrio
   * @throws ServerException em caso de falha.
   */
  private void loadCategoryRepository(final File catDirectory)
    throws ServerException {
    if (!catDirectory.exists()) {
      final String err = "No foi encontrado o diretrio de categorias: ";
      logWarningMessage(err + catDirectory.getAbsolutePath());
      return;
    }
    if (!catDirectory.isDirectory()) {
      final String err = "No  diretrio (categorias): ";
      throw new ServerException(err + catDirectory.getAbsolutePath());
    }

    final FileFilter filter = new FileFilter() {
      @Override
      public boolean accept(final File file) {
        final boolean isDir = file.isDirectory();
        final String name = file.getName();
        final boolean nameOk = !name.startsWith(".");
        return isDir && nameOk;
      }
    };

    final File[] directories = catDirectory.listFiles(filter);
    for (final File directory : directories) {
      loadCategory(directory);
    }
  }

  /**
   * Faz a carga dos repositrios de aplicaes.
   * 
   * @throws ServerException em caso de falha.
   */
  private void loadRepositories() throws ServerException {
    final String key = "repository.directory";
    final List<String> repPaths = getStringListProperty(key);
    if (repPaths.size() == 0) {
      logSevereMessage("No application repositories defined!");
    }

    for (final String repPath : repPaths) {
      final File repository = new File(repPath);
      final String path = repository.getAbsolutePath();
      logInfoMessage("Carregando repositrio de aplicaes: " + path);
      loadRepository(repository);
    }

    if (registries.size() == 0) {
      logWarningMessage("No applications defined!");
    }
  }

  /**
   * Faz a carga de um repositrio de aplicaes.
   * 
   * @param repository o diretrio que representa o repositrio.
   * @throws ServerException em caso de falha.
   */
  private void loadRepository(final File repository) throws ServerException {
    if (!repository.exists()) {
      final String err = "No foi encontrado o repositrio de aplicaes: ";
      throw new ServerException(err + repository.getAbsolutePath());
    }

    if (!repository.isDirectory()) {
      final String path = repository.getAbsolutePath();
      final String err = "Repositrio de apps: " + path + " no  diretrio!";
      throw new ServerException(err);
    }

    final File appRepository = getApplicationsRepository(repository);
    loadApplicationRepository(appRepository);

    final File catRepository = new File(repository, "categories");
    loadCategoryRepository(catRepository);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void shutdownService() throws ServerException {
    registries.clear();
    categories.clear();
  }

  /**
   * Constri a instncia do servio de gerncia de algoritmos
   * 
   * @throws ServerException caso ocorra algum problema durante a instanciao.
   */
  public static void createService() throws ServerException {
    new ApplicationService();
  }

  /**
   * Obtm uma referncia para a instncia nica (singleton) do servio de
   * algoritmos.
   * 
   * @return referncia para a instncia nica do servio de algoritmos.
   */
  public static ApplicationService getInstance() {
    final String serviceName = ApplicationServiceInterface.SERVICE_NAME;
    return (ApplicationService) Service.getInstance(serviceName);
  }

  /**
   * Construtor
   * 
   * @throws ServerException em caso de falha.
   */
  protected ApplicationService() throws ServerException {
    super(ApplicationServiceInterface.SERVICE_NAME);
    ClientRemoteLocator.applicationService = this;
  }

  /**
   * Construtor
   * 
   * @param srvName nome do servip
   * @throws ServerException exceo.
   */
  protected ApplicationService(final String srvName) throws ServerException {
    super(srvName);
    ClientRemoteLocator.applicationService = this;
  }
}
