/**
 * $Id: CommandPersistenceCleanupThread.java 105206 2010-05-06 23:06:04Z costa $
 */
package csbase.server.services.commandpersistenceservice;

import java.io.File;
import java.io.FileFilter;
import java.util.Calendar;
import java.util.Date;

import csbase.server.ServerException;
import csbase.server.services.projectservice.ProjectService;
import csbase.util.FileSystemUtils;

/**
 * Thread para efetuar a "limpeza" de arquivos associados  execuo de comandos
 * que tenham sido modificados h mais do que um nmero especfico de dias.
 * <p>
 * Esta thread atua apenas uma vez por dia, em horrio determinado pelo
 * administrador via propriedades.
 * <p>
 * As seguintes propriedades controlam o funcionamento desta thread:
 * <ul>
 * <li> <code>SGAService.cleanup.maxDays</code> nmero mximo de dias para manter
 * os arquivos. Arquivos modificados pela ltima vez h mais tempo sero
 * removidos. Default = 0 = "pra sempre" (arquivos no sero removidos)
 * <li> <code>SGAService.cleanup.hour</code> hora para execuo da limpeza
 * peridica, no formato 24h (0-23h). Default = 0h. A limpeza ser executada na
 * hora "cheia", i.e. 0min.
 * </ul>
 * 
 * @see #EXPIRATION_MAX_DAYS
 * @see #CLEANUP_HOUR_PROPERTY
 * 
 * @author Tecgraf
 */
class CommandPersistenceCleanupThread extends Thread {

  /**
   * Hora para execuo da limpeza (default = 0h), no formato 24h (0-23h).
   */
  private static final String CLEANUP_HOUR_PROPERTY = "cleanup.hour";

  /**
   * Nmero mximo de dias para manter os arquivos. Arquivos modificados pela
   * ltima vez h mais tempo sero removidos.
   * <p>
   * dafault = 0 dias = "pra sempre" (os arquivos no sero removidos)
   */
  private static final String EXPIRATION_MAX_DAYS = "cleanup.maxDays";

  /**
   * Nmero de milisegundos em um dia.
   */
  private static final long MILLISECONDS_PER_DAY = 24 * 60 * 60 * 1000L;

  /**
   * Nome do diretrio de persistncia da execuo dos comandos.
   */
  private final String CMDS_DIR_NAME;

  /**
   * Valor da propriedade {@link #EXPIRATION_MAX_DAYS}.
   */
  private int maxDays = 0;

  /**
   * Valor da propriedade {@link #CLEANUP_HOUR_PROPERTY}.
   */
  private int cleanupHour = 0;

  /**
   * Flag usado para definir quando a thread deve ser interrompida.
   */
  private boolean quit = false;

  /**
   * Filtro para identificao dos subdiretrios dos <code>.cmds</code> que
   * devem ser removidos.
   */
  private FileFilter dirFilter;

  /**
   * Contador do nmero de execues removidas.
   */
  private int dirsRemoved;

  /**
   * Contador do nmero de falhas de remoes.
   */
  private int numFails;

  /**
   * Construtor.
   * 
   * @throws ServerException em caso de erro (propriedades obrigatrias
   *         indefinidas, valores incorretos)
   */
  CommandPersistenceCleanupThread() throws ServerException {
    super(CommandPersistenceCleanupThread.class.getSimpleName());
    CommandPersistenceService cmdPersService =
      CommandPersistenceService.getInstance();
    CMDS_DIR_NAME = cmdPersService.getCommandsDirectoryName();
    maxDays = cmdPersService.getIntProperty(EXPIRATION_MAX_DAYS);
    // DEBUG
    //      maxDays = 1;
    if (maxDays < 0) {
      throw new ServerException(String.format("%s deve ser >= 0 [%d]",
        EXPIRATION_MAX_DAYS, maxDays));
    }
    if (maxDays > 0) {
      cleanupHour = cmdPersService.getIntProperty(CLEANUP_HOUR_PROPERTY);
      // DEBUG
      //        cleanupHour = 0;
      if (cleanupHour < 0 || cleanupHour > 23) {
        throw new ServerException(CLEANUP_HOUR_PROPERTY
          + " deve estar no intervalo [0, 23]");
      }
      logInfoMessage("thread iniciada");
      logInfoMessage(String.format("hora para cleanup: %dh", cleanupHour));
      logInfoMessage("arquivos com mais de " + maxDays
        + " dias sero removidos");
    }
    else {
      logInfoMessage(String.format("thread desativada (%s.%s = 0)",
        CommandPersistenceService.SERVICE_NAME, EXPIRATION_MAX_DAYS));
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void run() {
    if (maxDays == 0) {
      /*
       * no devemos fazer limpeza; a thread nem  iniciada
       */
      return;
    }
    doCleanup();
  }

  /**
   * 
   */
  private synchronized void doCleanup() {
    try {
      while (!quit) {
        wait(getWaitDelta());
        removeExpiredFiles();
      }
    }
    catch (InterruptedException e) {
      logInfoMessage("thread interrompida");
    }
    logInfoMessage("thread encerrada");
  }

  /**
   * Remove arquivos expirados.
   * 
   * @see #maxDays
   */
  private void removeExpiredFiles() {
    dirsRemoved = 0;
    numFails = 0;
    logInfoMessage("iniciando limpeza de arquivos expirados");
    /*
     * convertemos q quantidade mxima de dias em ms
     */
    final long maxMillis = maxDays * MILLISECONDS_PER_DAY;
    /*
     * calculamos a data-limite. Qualquer arq modificado antes desta data deve
     * ser descartado.
     */
    final long expirationDateMillis =
      Calendar.getInstance().getTimeInMillis() - maxMillis;
    /*
     * criamos o filtro para determinar os subdiretrios do .cmds que devem ser
     * removidos
     */
    dirFilter = new FileFilter() {
      @Override
      public boolean accept(File path) {
        return path.isDirectory() && path.lastModified() < expirationDateMillis;
      }
    };
    ProjectService projectService = ProjectService.getInstance();
    for (File userDir : projectService.getUserProjectDirs()) {
      for (File prj : FileSystemUtils.getSubDirs(userDir)) {
        File cmdsDir = new File(prj, CMDS_DIR_NAME);
        if (!cmdsDir.exists()) {
          continue;
        }
        /*
         * neste ponto temos um dir .cmds, precisamos agora verificar os
         * arquivos dentro dele
         */
        cleanupCmdsDir(cmdsDir);
      }
    }
    logInfoMessage(String.format(
      "limpeza terminada: %d execues removidas, %d falhas", dirsRemoved,
      numFails));
  }

  /**
   * Executa limpeza em um diretrio <code>.cmds</code>. Como este diretrio
   * possui apenas subdiretrios no formato
   * 
   * <pre>
   * admi_proj_MAASdEUCUp
   * admi_proj_MAASdEURWc
   * ...
   * </pre>
   * 
   * e no faz sentido removermos apenas parte destes diretrios, vamos
   * trat-los de forma atmica, i.e. consultaremos apenas a data da ltima
   * modificao do diretrio. Esta  atualizada sempre que um arquivo (ou
   * diretrio)  criado ou removido; ela <b>no</b>  atulizada quando um
   * arquivo  atualizado ou quando um arquivo  criado em um subdiretrio, mas
   * estes casos sero desconsiderados.
   * 
   * @param cmdsDir diretrio onde ser feita a limpeza
   */
  private void cleanupCmdsDir(File cmdsDir) {
    File[] dirsToRemove = cmdsDir.listFiles(dirFilter);
    for (File dir : dirsToRemove) {
      if (FileSystemUtils.deleteRecursively(dir)) {
        dirsRemoved++;
        logInfoMessage("diretrio " + dir.getAbsolutePath() + " removido");
      }
      // DEBUG
      //      logInfoMessage("removendo dir " + dir.getAbsolutePath());
      else {
        numFails++;
        logSevereMessage("erro removendo diretrio " + dir.getAbsolutePath());
        // FIXME deveramos interromper?
      }
    }
  }

  /**
   * Registra uma mensagem INFO no log do SGAService.
   * 
   * @param msg mensagem
   */
  private void logInfoMessage(String msg) {
    CommandPersistenceService.getInstance().logInfoMessage(msg);
  }

  /**
   * Registra uma mensagem SEVERE no log do SGAService.
   * 
   * @param msg mensagem
   */
  private void logSevereMessage(String msg) {
    CommandPersistenceService.getInstance().logSevereMessage(msg);
  }

  /**
   * Encerra a thread.
   */
  synchronized void shutdown() {
    quit = true;
    logInfoMessage("shutdown na thread solicitado");
    Thread.currentThread().interrupt();
  }

  /**
   * @return quantidade de milissegundos da hora corrente at o prximo cleanup
   */
  private long getWaitDelta() {
    Calendar now = Calendar.getInstance();
    Calendar cleanupTime = Calendar.getInstance();
    cleanupTime.set(Calendar.HOUR_OF_DAY, cleanupHour);
    cleanupTime.set(Calendar.MINUTE, 0);
    cleanupTime.set(Calendar.SECOND, 0);
    if (now.after(cleanupTime)) {
      int day = cleanupTime.get(Calendar.DAY_OF_MONTH);
      cleanupTime.set(Calendar.DAY_OF_MONTH, day + 1);
    }
    // DEBUG
    //    return 10 * 1000; // 10s
    return cleanupTime.getTimeInMillis() - now.getTimeInMillis();
  }

  /**
   * Mtodo para testes.
   * 
   * @param args
   */
  public static void main(String[] args) {
    File dir = new File("/tmp/sgacleanup");
    Date lastModified = new Date(dir.lastModified());
    System.out.println(lastModified.toString());
  }
}
