/*
 * $Id: CommandPersistenceService.java 105206 2010-05-06 23:06:04Z costa $
 */
package csbase.server.services.commandpersistenceservice;

import java.io.Closeable;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadFactory;

import tecgraf.javautils.FileUtils;
import csbase.exception.OperationFailureException;
import csbase.exception.ParseException;
import csbase.exception.ServiceFailureException;
import csbase.exception.algorithms.AlgorithmNotFoundException;
import csbase.logic.CommandFinalizationType;
import csbase.logic.CommandInfo;
import csbase.logic.CommandStatus;
import csbase.logic.Priority;
import csbase.logic.ProjectFile;
import csbase.logic.ProjectFileInfo;
import csbase.logic.ProjectFileType;
import csbase.logic.User;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.ExecutionType;
import csbase.logic.algorithms.FileParameterValue;
import csbase.logic.algorithms.commands.CommandPersistenceCommandNotificationData;
import csbase.logic.algorithms.commands.CommandPersistenceCommandNotificationData.Type;
import csbase.logic.algorithms.flows.configurator.FlowAlgorithmConfigurator;
import csbase.logic.algorithms.serializer.DefaultAlgorithmConfigurationSerializer;
import csbase.logic.algorithms.serializer.FlowAlgorithmConfigurationSerializer;
import csbase.logic.algorithms.serializer.IAlgorithmConfigurationSerializer;
import csbase.logic.algorithms.serializer.exception.AlgorithmConfigurationSerializerException;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.CommandPersistenceServiceInterface;
import csbase.server.ServerException;
import csbase.server.Service;
import csbase.server.services.notificationservice.NotificationService;
import csbase.server.services.projectservice.ProjectService;
import csbase.server.services.projectservice.ServerFileLockListener;
import csbase.server.services.projectservice.ServerProject;
import csbase.server.services.projectservice.ServerProjectFile;
import csbase.server.services.projectservice.ServerProjectFileException;
import csbase.util.callbak.ICallback;

/**
 * Servio de persistncia de comandos.
 * 
 * @see CommandPersistenceServiceInterface
 * @author TecGraf/PUC-Rio
 */
public final class CommandPersistenceService extends Service implements
  CommandPersistenceServiceInterface {

  /**
   * Fbrica de threads utilizada pelo mtodo
   * {@link #getCommandInfos(Object, long, ICallback)}. Ao ser chamado, este
   * mtodo cria uma thread permitindo retornar imediatamente. Essa thread fica
   * responsvel por carregar os comandos e enviar em blocos ao usurio.
   */
  private static ThreadFactory threadFactory = Executors.defaultThreadFactory();

  /**
   * Nome do arquivo de redirecionamento da sada da execuo do comando.
   */
  private static final String OUT_LOG = "out.log";

  /**
   * Nome do arquivo de captura do cdigo de sada do comando.
   */
  private static final String EXIT_CODE_LOG = "exit_code.log";

  /**
   * Tempo mximo de espera pelo lock exclusivo em milesegundos.
   */
  private static final int LOCK_EXCLUSIVE_TIMEOUT = 4000;

  /**
   * Tempo mximo de espera pelo lock compartilhado em milesegundos.
   */
  private static final int LOCK_SHARED_TIMEOUT = 1000;

  /**
   * Nome da propriedade que indica o nome do diretrio que ser usado para
   * armazenar os dados dos comandos executados.
   */
  private static final String COMMANDS_DIRECTORY_NAME_PROPERTY =
    "commandsDirectoryName";

  /**
   * Propriedade que indica o nome do diretrio que ser usado para armazenar os
   * dados dos comandos executados.
   */
  private String commandsDirectoryName;

  /**
   * O nome do diretrio de logs.
   */
  private static final String LOGS_DIRECTORY_NAME = "logs";

  /**
   * O nome do arquivo de propriedades do comando.
   */
  private static final String COMMAND_PROPERTIES_FILE_NAME = "cmd.properties";

  /**
   * O nome do arquivo contendo a configuraco de um algoritmo instalado na
   * rvore de algoritmos.
   */
  private static final String DEFAULT_CONFIGURATION_FILE_NAME =
    "cmd.parameters";

  /**
   * O tipo do arquivo contendo a configuraco de um algoritmo instalado na
   * rvore de algoritmos.
   */
  private static final String DEFAULT_CONFIGURATION_FILE_TYPE = "PARAMETERS";

  /**
   * O nome do arquivo contendo a configuraco de um fluxo.
   */
  private static final String FLOW_CONFIGURATION_FILE_NAME = "cmd.flx";

  /**
   * O tipo do arquivo contendo a configuraco de um fluxo.
   */
  private static final String FLOW_CONFIGURATION_FILE_TYPE = "FLX";

  /**
   * Thread para limpeza dos arquivos de persistncia das execues, de acordo
   * com um tempo mximo de vida. Para mais detalhes, ver documentao da classe
   * {@link CommandPersistenceCleanupThread}.
   */
  private CommandPersistenceCleanupThread cleanupThread = null;

  /**
   * Constri a instncia do servio de escalonamento.
   * 
   * @throws ServerException se houver erro na inicializao.
   */
  public static void createService() throws ServerException {
    new CommandPersistenceService();
  }

  /**
   * Retorno da instncia do servio.
   * 
   * @return o servio.
   */
  public static CommandPersistenceService getInstance() {
    return (CommandPersistenceService) Service
      .getInstance(CommandPersistenceServiceInterface.SERVICE_NAME);
  }

  /**
   * Fecha um fluxo de dados sem gerar exceo.
   * 
   * @param stream O fluxo de dados. Se for {@code null}, o mtodo no faz nada.
   */
  private void close(final Closeable stream) {
    if (stream != null) {
      try {
        stream.close();
      }
      catch (final IOException e) {
        printStackTrace(e);
      }
    }
  }

  /**
   * Cria os diretrios e arquivos onde so persistidas as informaes dos
   * comandos.
   * 
   * @param project projeto
   * @param command informaes do comando
   * @param configurationFileName nome do arquivo onde so gravadas as
   *        configuraes do comando
   * @param configurationFileType tipo do arquivo de configurao
   * @return o projeto onde foram persistidas as informaes do comando
   * @throws OperationFailureException em caso de falha.
   */
  private ServerProject createProjectFiles(final ServerProject project,
    final CommandInfo command, final String configurationFileName,
    final String configurationFileType) throws OperationFailureException {

    AlgorithmConfigurator configurator;
    try {
      configurator = command.getConfigurator();
    }
    catch (final RemoteException e) {
      throw new IllegalArgumentException(String.format(
        "No foi possvel determinar um nome para o arquivo de metadados do "
          + "configurador do comando %s.\n", command.getId()));
    }

    final String commandId = command.getId();
    try {
      final String commandDirectoryName = getCommandDirectoryName(commandId);
      final List<ProjectFileInfo> projectFileInfos =
        new LinkedList<ProjectFileInfo>();
      if (getCommandRepositoryDirectory(project) == null) {
        // Cria o diretrio de comandos se ele no existir
        projectFileInfos.add(new ProjectFileInfo(
          new String[] { commandsDirectoryName },
          ProjectFileType.DIRECTORY_TYPE));
      }
      projectFileInfos.add(new ProjectFileInfo(new String[] {
          commandsDirectoryName, commandDirectoryName },
        ProjectFileType.DIRECTORY_TYPE));
      projectFileInfos.add(new ProjectFileInfo(new String[] {
          commandsDirectoryName, commandDirectoryName, configurationFileName },
        configurationFileType));
      projectFileInfos.add(new ProjectFileInfo(new String[] {
          commandsDirectoryName, commandDirectoryName,
          CommandPersistenceService.COMMAND_PROPERTIES_FILE_NAME }, "TEXT"));
      projectFileInfos.add(new ProjectFileInfo(new String[] {
          commandsDirectoryName, commandDirectoryName,
          CommandPersistenceService.LOGS_DIRECTORY_NAME },
        ProjectFileType.DIRECTORY_TYPE));

      // Atribui valor ao log de sada padro do algoritmo.
      final String outputFilePath =
        FileUtils.joinPath(File.separatorChar, commandsDirectoryName,
          commandDirectoryName, CommandPersistenceService.LOGS_DIRECTORY_NAME,
          CommandPersistenceService.OUT_LOG);
      final FileParameterValue outputFile =
        new FileParameterValue(outputFilePath, "LOG");
      configurator.setStandardOutputFile(outputFile);
      /*
       * Cria os arquivos de log. Pede-se os arquivos a serem criados ao
       * configurador pois, se ele for de fluxo, ele ter criado arquivos para
       * cada um de seus ns a partir do arquivo atribuido acima.
       */
      for (final FileParameterValue stdout : configurator
        .getStandardOutputFiles()) {
        projectFileInfos.add(new ProjectFileInfo(stdout.getPathAsArray(),
          stdout.getType()));
      }

      if (configurator.hasExitCode()) {
        final String exitCodeLogFilePath =
          FileUtils.joinPath(File.separatorChar, commandsDirectoryName,
            commandDirectoryName,
            CommandPersistenceService.LOGS_DIRECTORY_NAME,
            CommandPersistenceService.EXIT_CODE_LOG);
        final FileParameterValue exitCodeLogFile =
          new FileParameterValue(exitCodeLogFilePath, "LOG");
        configurator.setExitCodeLogFile(exitCodeLogFile);

        for (final FileParameterValue exitLog : configurator
          .getExitCodeLogFiles()) {
          projectFileInfos.add(new ProjectFileInfo(exitLog.getPathAsArray(),
            exitLog.getType()));
        }
      }

      final ServerProjectFile rootFile = (ServerProjectFile) project.getTree();
      final Object userId = command.getUserId();
      rootFile.createFiles(projectFileInfos, userId);
    }
    catch (final ServerProjectFileException e) {
      throw new OperationFailureException(getMessage(
        "error_create_project_files", commandId), e);
    }
    return project;
  }

  /**
   * Obtm o diretrio aonde foram/sero salvos os dados do comando.
   * 
   * @param project O projeto (No aceita {@code null}).
   * @param commandId O identificador do comando (No aceita {@code null}).
   * 
   * @return O diretrio do comando ou {@code null} se o comando no existir.
   */
  private ServerProjectFile getCommandDirectory(final ServerProject project,
    final String commandId) {

    final ServerProjectFile commandRepositoryDirectory =
      getCommandRepositoryDirectory(project);
    if (commandRepositoryDirectory == null) {
      return null;
    }
    final String commandDirectoryName = getCommandDirectoryName(commandId);
    final ProjectFile child =
      commandRepositoryDirectory.getChild(commandDirectoryName);
    return (ServerProjectFile) child;
  }

  /**
   * Obtm o nome do diretrio do comando a partir do identificador do comando.
   * 
   * @param commandId O identificador do comando (No aceita {@code null}).
   * 
   * @return O nome do diretrio.
   */
  private String getCommandDirectoryName(final String commandId) {
    final String commandDirectoryName = FileUtils.fixDirectoryName(commandId);
    return commandDirectoryName;
  }

  /**
   * Obtm o identificador de um comando a partir de seu diretrio.<br>
   * Ex.<br>
   * getCommandId("admi_proj_GAASauRbdV") == "admi@proj.GAASauRbdV"<br>
   * getCommandId("admi_FLOW_GAASauRz_0") == "admi@FLOW.GAASauRz-0"
   * 
   * @param commandDirectoryName Nome do diretrio aonde o comando est sendo
   *        persistido.
   * 
   * @return O identificador do comando.
   */
  private String getCommandId(final String commandDirectoryName) {
    String id =
      commandDirectoryName.replaceAll("^([^_]+)_([^_]+)_(.*)$", "$1@$2.$3");
    return id.replaceAll("_", "-");
  }

  /**
   * {@inheritDoc}
   */
  public CommandInfo getCommandInfo(final Object projectId,
    final String commandId) {
    if (projectId == null) {
      final String err = "O parmetro projectId est nulo.";
      throw new IllegalArgumentException(err);
    }
    if (commandId == null) {
      final String err = "O parmetro commandId est nulo.";
      throw new IllegalArgumentException(err);
    }
    try {
      final ServerProject project = getProjectById(projectId);
      final ServerProjectFile commandDirectory =
        getCommandDirectory(project, commandId);
      return readCommandInfo(commandDirectory);
    }
    catch (final OperationFailureException e) {
      final String message = getMessage("error_read_command_info", commandId);
      logSevereMessage(message, e);
      // Retorna um comando com o status SYSTEM_FAILURE
      return createSystemFailureCommand(projectId, commandId);
    }
  }

  /**
   * {@inheritDoc}
   */
  public Set<CommandInfo> getCommandInfos(final Object projectId) {
    if (projectId == null) {
      final String err = "O parmetro projectId est nulo.";
      throw new IllegalArgumentException(err);
    }
    ServerProject project;
    try {
      project = getProjectById(projectId);
    }
    catch (final OperationFailureException e) {
      final String fmt = "Erro ao tentar ler os comandos do projeto %s.\n";
      logSevereMessage(String.format(fmt, projectId), e);
      throw new ServiceFailureException(e.getMessage());
    }

    final Set<CommandInfo> cmdsInfo = new HashSet<CommandInfo>();
    final List<ServerProjectFile> cmdDirectories =
      getCommandsDirectories(project);
    for (final ServerProjectFile cmdDirectory : cmdDirectories) {
      /** XXX - O que fazer quando der problema em um arquivo. Abortamos tudo? */
      final CommandInfo commandInfo = readCommandInfo(cmdDirectory);
      cmdsInfo.add(commandInfo);
    }

    return Collections.unmodifiableSet(cmdsInfo);
  }

  /**
   * {@inheritDoc}
   */
  public void getCommandInfos(final Object projectId,
    final long desiredResponseTime,
    final ICallback<CommandInfosRetrived> callback) {

    if (projectId == null) {
      final String err = "O parmetro projectId est nulo.";
      throw new IllegalArgumentException(err);
    }
    if (callback == null) {
      final String err = "O parmetro callback est nulo.";
      throw new IllegalArgumentException(err);
    }
    if (1 > desiredResponseTime) {
      final String err = "O parmetro avgResponseTime tem que ser maior que 0.";
      throw new IllegalArgumentException(err);
    }
    ServerProject project;
    try {
      project = getProjectById(projectId);
    }
    catch (final OperationFailureException e) {
      final String fmt = "Erro ao tentar ler os comandos do projeto %s.\n";
      logSevereMessage(String.format(fmt, projectId), e);
      throw new ServiceFailureException(e.getMessage());
    }

    final List<ServerProjectFile> cmdDirectories =
      getCommandsDirectories(project);
    CommandPersistenceService.threadFactory.newThread(new Runnable() {

      @Override
      public void run() {
        int index = 0;
        ServerProjectFile cmdDirectory = null;
        long now = System.currentTimeMillis();
        /*
         * Enquanto existir comandos a serem lidos, l e envia para o cliente.
         */
        while (index < cmdDirectories.size()) {
          final long end = now + desiredResponseTime;
          /*
           * L as propriedades dos comandos at que os comandos terminem ou o
           * tempo de resposta seja alcanado. Caso haja erro ao ler algum
           * comando, este ser ignorado.
           */
          final Set<CommandInfo> cmdsInfo = new HashSet<CommandInfo>();
          for (; index < cmdDirectories.size() && now < end; index++) {
            cmdDirectory = cmdDirectories.get(index);
            final CommandInfo commandInfo = readCommandInfo(cmdDirectory);
            cmdsInfo.add(commandInfo);
            now = System.currentTimeMillis();
          }
          /*
           * Envia os comandos lidos at o momento para o cliente atravs do
           * callback.
           */
          if (0 < cmdsInfo.size()) {
            try {
              long total = cmdDirectories.size();
              long missing = total - index;

              CommandInfosRetrived data =
                new CommandInfosRetrived(cmdsInfo, missing, total);
              callback.call(data);
            }
            catch (final Exception e) {
              final String message =
                "Erro ao enviar as informaes de comandos ao usurio.\n";
              logSevereMessage(message, e);
            }
          }
        }
      }
    }).start();
  }

  /**
   * Obtm o repositrio de comandos de um projeto especfico.
   * 
   * @param project O projeto (No aceita {@code null}).
   * 
   * @return O repositrio ou {@code null} se no existir.
   */
  private ServerProjectFile getCommandRepositoryDirectory(
    final ServerProject project) {
    final ServerProjectFile projectRootFile =
      (ServerProjectFile) project.getTree();
    return (ServerProjectFile) projectRootFile.getChild(commandsDirectoryName);
  }

  /**
   * <p>
   * Obtm os diretrios de comandos do repositrio de comandos do projeto
   * fornecido.
   * </p>
   * 
   * @param project O projeto (No aceita {@code null}).
   * 
   * @return A lista de diretrios (se no houver comandos, a lista estar
   *         vazia).
   */
  private List<ServerProjectFile> getCommandsDirectories(
    final ServerProject project) {
    final ServerProjectFile commandRepositoryDirectory =
      getCommandRepositoryDirectory(project);
    if (commandRepositoryDirectory == null) {
      return Collections.emptyList();
    }
    final ProjectFile[] children = commandRepositoryDirectory.getChildren();
    final List<ServerProjectFile> files =
      new ArrayList<ServerProjectFile>(children.length);
    for (final ProjectFile child : children) {
      files.add((ServerProjectFile) child);
    }
    return files;
  }

  /**
   * Obtm o caminho do diretrio onde devem ser gerados os arquivos de log.
   * Este caminho  dado a partir do diretrio do projeto (no includo).
   * 
   * @param commandId O identificador do comando (No aceita {@code null}).
   * 
   * @return O caminho do diretrio.
   */
  public String[] getLogDirectory(final String commandId) {
    final String commandDirectoryName = getCommandDirectoryName(commandId);
    return new String[] { commandsDirectoryName, commandDirectoryName,
        CommandPersistenceService.LOGS_DIRECTORY_NAME };
  }

  /**
   * Obtm uma mensagem no mecanismo de internacionalizao.
   * 
   * @param keySuffix O sufixo da chave (o nome da classe ser pr-fixado ao
   *        sufixo) da mensagem a ser procurada no servico de lingugem.
   * @param args Possveis argumentos utilizados na formataco da mensagem
   *        utilizando para isso, o mtodo
   *        {@link String#format(String, Object...)}
   * 
   * @return A mensagem formatada.
   */
  private String getMessage(final String keySuffix, final Object... args) {
    final String className = CommandPersistenceService.class.getSimpleName();
    final String key = className + "." + keySuffix;
    final String format = getString(key);
    return String.format(format, args);
  }

  /**
   * Obtm um projeto especfico.
   * 
   * @param projectId O identificador do projeto (No aceita {@code null}).
   * 
   * @return O projeto.
   * 
   * @throws OperationFailureException Se o projeto no existir.
   */
  private ServerProject getProjectById(final Object projectId)
    throws OperationFailureException {
    final ServerProject project =
      ProjectService.getInstance().getProject(projectId);
    if (project == null) {
      final String message = getMessage("error_project_not_found", projectId);
      throw new OperationFailureException(message);
    }
    return project;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected boolean has2Update(final Object arg, final Object event) {
    return true;
  }

  /**
   * Inicializao do servio
   * 
   * @throws ServerException
   */
  @Override
  public void initService() throws ServerException {
    /*
     * criao da thread para limpeza dos arquivos de persistncia dos comandos
     */
    createCleanupThread();
  }

  /**
   * Notifica usurios sobre eventos ocorridos com comandos.
   * 
   * @param allUsers Usurios a serem notificados alm do admin.
   * @param notification Notificao a ser enviada indicando se um comando foi
   *        salvo, removido ou atualizado na persistncia.
   */
  public void notifyUsers(final Collection<Object> allUsers,
    final CommandPersistenceCommandNotificationData notification) {
    final NotificationService ntSrv = NotificationService.getInstance();

    for (final Object projectUserId : allUsers) {
      ntSrv.notifyUserFromService(this, projectUserId, notification, false);
    }
    /*
     * Como o administrador tem acesso a todos os comandos, ele deve receber
     * suas notificaes de trmino. Porm, essa notificao no deve ser
     * exposta ao usurio administrador, ela deve apenas ser tratada pelo seu
     * cliente.
     */
    final Object adminId = User.getAdminId();
    if (!allUsers.contains(adminId)) {
      ntSrv.notifyUserFromService(this, adminId, notification, false);
    }
  }

  /**
   * Imprime na sada de erro as pilhas de execuo da execeo de todas as suas
   * causas.
   * 
   * @param throwable A exceo (No aceita {@code null}).
   */
  private void printStackTrace(final Throwable throwable) {
    Throwable currentThrowable = throwable;
    while (currentThrowable != null
      && currentThrowable != currentThrowable.getCause()) {
      currentThrowable.printStackTrace();
      currentThrowable = currentThrowable.getCause();
    }
  }

  /**
   * {@inheritDoc}
   */
  public AlgorithmConfigurator readAlgorithmConfigurator(
    final Object projectId, final String commandId) {
    if (projectId == null) {
      final String err = "O parmetro projectId est nulo.";
      throw new IllegalArgumentException(err);
    }
    if (commandId == null) {
      final String err = "O parmetro commandId est nulo.";
      throw new IllegalArgumentException(err);
    }
    ServerProject project;
    try {
      project = getProjectById(projectId);
    }
    catch (final OperationFailureException e) {
      final String message = getMessage("error_read_command_info", commandId);
      throw new ServiceFailureException(message, e);
    }
    final ServerProjectFile commandDirectory =
      getCommandDirectory(project, commandId);
    IAlgorithmConfigurationSerializer configuratorSerializer =
      new DefaultAlgorithmConfigurationSerializer();
    ServerProjectFile configurationFile =
      (ServerProjectFile) commandDirectory
        .getChild(CommandPersistenceService.DEFAULT_CONFIGURATION_FILE_NAME);
    if (configurationFile == null) {
      configuratorSerializer = new FlowAlgorithmConfigurationSerializer();
      configurationFile =
        (ServerProjectFile) commandDirectory
          .getChild(CommandPersistenceService.FLOW_CONFIGURATION_FILE_NAME);
    }
    if (configurationFile == null) {
      final String path = commandDirectory.getCanonicalPath();
      final String err = getMessage("error_configuration_file_not_found", path);
      throw new ServiceFailureException(err);
    }
    InputStream inputStream = null;
    Object lockId = null;
    final String path = configurationFile.getCanonicalPath();
    try {
      final ServerFileLockListener lockListener = new ServerFileLockListener();
      lockId =
        configurationFile.acquireSharedLock(lockListener,
          CommandPersistenceService.LOCK_SHARED_TIMEOUT);
      if (lockListener.getLock()) {
        inputStream = configurationFile.openInputStream();
        final AlgorithmConfigurator configurator =
          configuratorSerializer.read(inputStream);
        if (configurator.getAdditionalLogsDirectory() == null) {
          //Seta o diretrio padro para arquivos de log adicionais.
          final String additionalLogsDirPath =
            FileUtils.joinPath(File.separatorChar, commandsDirectoryName,
              getCommandDirectoryName(commandId),
              CommandPersistenceService.LOGS_DIRECTORY_NAME);
          final FileParameterValue additionalLogsDirectory =
            new FileParameterValue(additionalLogsDirPath);
          configurator.setAdditionalLogsDirectory(additionalLogsDirectory);
        }
        return configurator;
      }
      final String key =
        "CommandPersistenceService.error.get.shared.lock.config";
      final String fmt = getString(key);
      final String message = String.format(fmt, commandId);
      throw new ServiceFailureException(message);
    }
    catch (final ServerProjectFileException e) {
      final String message = getMessage("error_read_file", path);
      throw new ServiceFailureException(message, e);
    }
    catch (final AlgorithmConfigurationSerializerException e) {
      final String message = getMessage("error_create_configurator", path);
      throw new ServiceFailureException(message, e);
    }
    catch (final AlgorithmNotFoundException e) {
      throw new ServiceFailureException(e.getMessage(), e);
    }
    finally {
      close(inputStream);
      if (lockId != null) {
        configurationFile.releaseLock(lockId);
      }
    }
  }

  /**
   * L as informaes do comando de um diretrio do comando fornecido. Apenas
   * as propriedades do comando so carregadas.
   * 
   * @param commandDirectory O diretrio do comando (No aceita {@code null}).
   * 
   * @return Um comando com apenas suas propriedades carregadas. Caso no seja
   *         possvel carregar as propriedades, ser retornado um comando com o
   *         status {@link CommandStatus#SYSTEM_FAILURE}. Neste as nicas
   *         propriedades garantidas so o identificador do comando e o
   *         identificador do projeto, todas as demais devem ser
   *         desconsideradas.
   */
  private CommandInfo readCommandInfo(ServerProjectFile commandDirectory) {
    try {
      validateCommandInfoDirectory(commandDirectory);
      return readCommandInfoProperties(commandDirectory);
    }
    catch (OperationFailureException e) {
      // Loga o erro 
      final String message =
        getMessage("error_read_command_dir", commandDirectory.getAbsolutePath());
      logSevereMessage(message, e);

      final Object projectId = commandDirectory.getProjectId();
      // Retorna um comando com o status SYSTEM_FAILURE
      String commandId = getCommandId(commandDirectory.getName());
      return createSystemFailureCommand(projectId, commandId);
    }
  }

  /**
   * Valida o diretrio do comando quanto a existncia do arquivo de propriedade
   * e do arquivo de parmetros e que estes tem contedo.
   * 
   * @param commandDirectory O diretrio do comando (No aceita {@code null}).
   * 
   * @throws OperationFailureException se o diretrio dado no contiver o
   *         arquivo de propriedades ou o arquivo de parmetros do comando, ou
   *         algum deles for vazio.
   */
  private void validateCommandInfoDirectory(ServerProjectFile commandDirectory)
    throws OperationFailureException {
    // Verifica se o arquivo de parmetros existe e no  vazio.
    ProjectFile paramFile =
      commandDirectory.getChild(DEFAULT_CONFIGURATION_FILE_NAME);
    if (paramFile == null) {
      paramFile = commandDirectory.getChild(FLOW_CONFIGURATION_FILE_NAME);
    }
    if (paramFile == null || paramFile.size() == 0) {
      final String path = commandDirectory.getCanonicalPath();
      final String err = getMessage("error_configuration_file_not_found", path);
      throw new OperationFailureException(err);
    }

    // Verifica se o arquivo de propriedades existe e no  vazio.    
    ProjectFile infoFile =
      commandDirectory.getChild(COMMAND_PROPERTIES_FILE_NAME);
    if (infoFile == null || infoFile.size() == 0) {
      final String path = commandDirectory.getCanonicalPath();
      final String err =
        getMessage("error_command_properties_not_found", path,
          COMMAND_PROPERTIES_FILE_NAME);
      throw new OperationFailureException(err);
    }
  }

  /**
   * Cria um comando com status {@link CommandStatus#SYSTEM_FAILURE} para ser
   * retornado sempre que no for possvel recuperar as propriedades de um
   * comando. Neste caso, as nicas propriedades garantidas so o identificador
   * do comando e o identificador do projeto, todas as demais devem ser
   * desconsideradas.
   * 
   * @param projectId Identificador do projeto no qual o comando foi executado.
   * @param commandId Identificador do comando.
   * 
   * @return Um comando com status {@link CommandStatus#SYSTEM_FAILURE}.
   */
  private CommandInfo createSystemFailureCommand(final Object projectId,
    String commandId) {
    return new CommandInfo(commandId, 0, false, projectId, null, null, Priority
      .getDefault(), CommandStatus.SYSTEM_FAILURE,
      CommandFinalizationType.NOT_FINISHED, null);
  }

  /**
   * L as informaes do comando de um diretrio do comando fornecido. Apenas
   * as propriedades do comando so carregadas.
   * 
   * @param commandDirectory O diretrio do comando (No aceita {@code null}).
   * 
   * @return As informaes.
   * 
   * @throws OperationFailureException Se houver um erro ao ler as informaes.
   */
  private CommandInfo readCommandInfoProperties(
    final ServerProjectFile commandDirectory) throws OperationFailureException {
    final ServerProjectFile commandPropertyFile =
      (ServerProjectFile) commandDirectory
        .getChild(CommandPersistenceService.COMMAND_PROPERTIES_FILE_NAME);
    if (commandPropertyFile == null) {
      final String message =
        getMessage("error_command_properties_not_found", commandDirectory
          .getAbsolutePath(),
          CommandPersistenceService.COMMAND_PROPERTIES_FILE_NAME);
      throw new OperationFailureException(message);
    }

    InputStream inputStream = null;
    Object lockId = null;
    final String path = commandPropertyFile.getCanonicalPath();
    try {
      final ServerFileLockListener lockListener = new ServerFileLockListener();
      lockId =
        commandPropertyFile.acquireSharedLock(lockListener,
          CommandPersistenceService.LOCK_SHARED_TIMEOUT);
      if (lockListener.getLock()) {
        final Object projectId = commandDirectory.getProjectId();
        inputStream = commandPropertyFile.openInputStream();
        final CommandInfo command =
          readCommandInfoPropertiesFromStream(projectId, inputStream);
        return command;
      }
      final String fmt =
        getString("CommandPersistenceService.error.get.shared.lock");
      final String message = String.format(fmt, path);
      throw new OperationFailureException(message);
    }
    catch (final IOException e) {
      final String message = getMessage("error_read_file", path);
      throw new OperationFailureException(message, e);
    }
    catch (final ServerProjectFileException e) {
      final String message = getMessage("error_open_file", path);
      throw new OperationFailureException(message, e);
    }
    catch (final ParseException e) {
      final String message = getMessage("error_file_format", path);
      throw new OperationFailureException(message, e);
    }
    finally {
      close(inputStream);
      if (lockId != null) {
        commandPropertyFile.releaseLock(lockId);
      }
    }
  }

  /**
   * Cria um comando a partir da leitura de seu arquivo de propriedades.
   * 
   * @param projectId O identificador do projeto (No aceita {@code null}).
   * @param inputStream O stream de entrada com o contedo do arquivo (No
   *        aceita {@code null}).
   * @return commandInfo O comando lido.
   * 
   * @throws ParseException Em caso de erro no formato do arquivo.
   * @throws IOException Se houver um erro de ES ao ler no stream.
   */
  private CommandInfo readCommandInfoPropertiesFromStream(
    final Object projectId, final InputStream inputStream)
    throws ParseException, IOException {
    final CommandPropertyParser parser = new CommandPropertyParser();
    parser.loadProperties(inputStream);

    final String id = parser.getId();
    final String userId = parser.getUserId();
    final Date submittedDate = parser.getSubmittedDate();
    final int globalPosition = parser.getGlobalPosition();
    final boolean mailAtEnd = parser.getMailAtEnd();
    final Priority priority = parser.getPriority();
    final CommandStatus status = parser.getStatus();
    final CommandFinalizationType finalizationType =
      parser.getFinalizationType(status);
    final String tip = parser.getTip();
    final CommandInfo commandInfo =
      new CommandInfo(id, globalPosition, mailAtEnd, projectId, submittedDate,
        userId, priority, status, finalizationType, tip);

    final Integer wallTimeSec = parser.getWallTimeSec();
    if (wallTimeSec != null) {
      commandInfo.setWallTimeSec(wallTimeSec);
    }
    final String description = parser.getDescription();
    if (description != null) {
      commandInfo.setDescription(description);
    }
    final String sgaName = parser.getSGAName();
    if (sgaName != null) {
      commandInfo.setSGAName(sgaName);
    }
    final String platformFilter = parser.getPlatformFilter();
    if (platformFilter != null) {
      commandInfo.setPlatformFilter(platformFilter);
    }

    final boolean isAutomatic = parser.getIsAutomatic();
    final ExecutionType executionType = parser.getExecutionType();
    commandInfo.setExecutionType(executionType);
    switch (executionType) {
      case DISTRIBUTED:
        if (isAutomatic) {
          final int sgaCount = parser.getSgaCountForDistributedExecution();
          commandInfo.configureDistributedExecution(sgaCount);
        }
        else {
          final int executionCountPerSGA =
            parser.getExecutionCountPerSGAForDistributedExecution();
          final List<String> selectedSGAsNames = parser.getSelectedSGAsNames();
          commandInfo.configureDistributedExecution(selectedSGAsNames,
            executionCountPerSGA);
        }
        break;
      case MULTIPLE:
        if (isAutomatic) {
          final int executionCount =
            parser.getExecutionCountForMultipleExecution();
          commandInfo.configureMultipleExecution(executionCount);
        }
        else {
          final int executionCountPerSGA =
            parser.getExecutionCountPerSGAForMultipleExecution();
          final List<String> selectedSGAsNames = parser.getSelectedSGAsNames();
          commandInfo.configureMultipleExecution(selectedSGAsNames,
            executionCountPerSGA);
        }
        break;
      case PARALLEL:
        final int cpuCount = parser.getCpuCountForParallelExecution();
        if (isAutomatic) {
          commandInfo.configureParallelExecution(cpuCount);
        }
        else {
          final String selectedSgaName = parser.getSelectedSGAName();
          commandInfo.configureParallelExecution(selectedSgaName, cpuCount);
        }
        break;
      case SIMPLE:
        if (isAutomatic) {
          commandInfo.configureSimpleExecution();
        }
        else {
          final String selectedSgaName = parser.getSelectedSGAName();
          commandInfo.configureSimpleExecution(selectedSgaName);
        }
        break;
      case XAPPS:
        if (isAutomatic) {
          commandInfo.configureXAppExecution();
        }
        else {
          final String selectedSgaName = parser.getSelectedSGAName();
          commandInfo.configureXAppExecution(selectedSgaName);
        }
        break;
      default:
        final String fmt =
          "Tipo de execuo invlido no arquivo de persistncia %s.";
        final String err = String.format(fmt, executionType);
        throw new IllegalStateException(err);
    }

    return commandInfo;
  }

  /**
   * {@inheritDoc}
   */
  public boolean removeCommandInfo(final Object projectId,
    final String commandId) {
    try {
      final ServerProject project = getProjectById(projectId);
      final ServerProjectFile cmdDirectory =
        getCommandDirectory(project, commandId);
      if (cmdDirectory == null) {
        return false;
      }
      cmdDirectory.remove();
      notifyUsers(project.getAllUsers(true),
        new CommandPersistenceCommandNotificationData(projectId, commandId,
          Type.REMOVED));

      return true;
    }
    catch (final OperationFailureException e) {
      final String message = getMessage("error_remove_command_info", commandId);
      throw new ServiceFailureException(message, e);
    }
    catch (final ServerProjectFileException e) {
      final String message = getMessage("error_remove_command_info", commandId);
      throw new ServiceFailureException(message, e);
    }
  }

  /**
   * {@inheritDoc}
   */
  public boolean[] removeCommandInfos(final List<Object> projectIds,
    final List<String> commandIds) {
    final boolean[] result = new boolean[commandIds.size()];
    Arrays.fill(result, false);

    for (int inx = 0; inx < commandIds.size(); inx++) {
      result[inx] = removeCommandInfo(projectIds.get(inx), commandIds.get(inx));
    }

    return result;
  }

  /**
   * Salva as propriedades de um comando e os seus parmetros
   * 
   * @param project projeto a partir do qual o comando foi executado
   * @param command informaes do comando
   * @param configurationFileName nome do arquivo de parmetros do comando
   * @param configurationSerializer responsvel por persistir as configuraes
   *        do comando
   * @return stream de sada
   * @throws OperationFailureException em caso de falha.
   */
  private OutputStream saveAlgorithmConfiguration(final ServerProject project,
    final CommandInfo command, final String configurationFileName,
    final IAlgorithmConfigurationSerializer configurationSerializer)
    throws OperationFailureException {
    final String commandId = command.getId();
    final ServerProjectFile cmdDirectory =
      getCommandDirectory(project, commandId);
    if (cmdDirectory == null) {
      final String tag = "error_command_directory_not_found";
      final String message = getMessage(tag, commandId);
      throw new OperationFailureException(message);
    }
    final ServerProjectFile configurationFile =
      (ServerProjectFile) cmdDirectory.getChild(configurationFileName);
    if (configurationFile == null) {
      final String tag = "error_file_not_found";
      final String path = cmdDirectory.getCanonicalPath();
      final String message = getMessage(tag, path, configurationFileName);
      throw new OperationFailureException(message);
    }
    OutputStream output = null;
    Object lockId = null;
    try {
      final ServerFileLockListener lockListener = new ServerFileLockListener();
      lockId =
        configurationFile.acquireExclusiveLock(lockListener,
          CommandPersistenceService.LOCK_EXCLUSIVE_TIMEOUT);
      if (lockListener.getLock()) {
        output = configurationFile.openOutputStream();
        final AlgorithmConfigurator configurator = command.getConfigurator();
        configurationSerializer.write(configurator, output);
      }
      else {
        final String key = "CommandPersistenceService.error.get.exclusive.lock";
        final String fmt = getString(key);
        final String path = configurationFile.getCanonicalPath();
        final String message = String.format(fmt, path);
        throw new OperationFailureException(message);
      }
    }
    catch (final ServerProjectFileException e) {
      final String fmt = getMessage("error.open_configuration_file");
      final String err = String.format(fmt, commandId, configurationFileName);
      throw new OperationFailureException(err);
    }
    catch (final AlgorithmConfigurationSerializerException e) {
      final String fmt = getMessage("error.parse_configuration_file");
      final String err = String.format(fmt, commandId, configurationFileName);
      throw new OperationFailureException(err);
    }
    catch (final RemoteException e) {
      final String fmt = getMessage("error.no_configurator");
      final String err = String.format(fmt, commandId, configurationFileName);
      throw new OperationFailureException(err);
    }
    finally {
      close(output);
      if (lockId != null) {
        configurationFile.releaseLock(lockId);
      }
    }
    return output;
  }

  /**
   * <p>
   * Salva um comando no projeto informado.
   * </p>
   * <p>
   * Caso o comando tenha sido salvo com sucesso, gera uma notificao
   * {@link CommandPersistenceCommandNotificationData} do tipo
   * {@link CommandPersistenceCommandNotificationData.Type#SAVED}.
   * </p>
   * 
   * @param command O comando a ser salvo (No aceita {@code null}).
   * 
   * @throws OperationFailureException Se houver uma falha ao salvar o comando.
   */
  public void saveCommandInfo(final CommandInfo command)
    throws OperationFailureException {
    if (command == null) {
      final String err = "O parmetro command est nulo.";
      throw new IllegalArgumentException(err);
    }

    AlgorithmConfigurator configurator;
    final String cmdId = command.getId();
    try {
      configurator = command.getConfigurator();
    }
    catch (final RemoteException e) {
      final String fmt =
        "No foi possvel obter o configurador do comando %s.\n";
      throw new IllegalArgumentException(String.format(fmt, cmdId));
    }

    String configurationFileName;
    String configurationFileType;
    IAlgorithmConfigurationSerializer configurationSerializer;
    if ((configurator.getAlgorithmName() != null)
      && (configurator.getAlgorithmVersionId() != null)) {
      configurationFileName =
        CommandPersistenceService.DEFAULT_CONFIGURATION_FILE_NAME;
      configurationFileType =
        CommandPersistenceService.DEFAULT_CONFIGURATION_FILE_TYPE;
      configurationSerializer = new DefaultAlgorithmConfigurationSerializer();
    }
    else if (configurator instanceof FlowAlgorithmConfigurator) {
      configurationFileName =
        CommandPersistenceService.FLOW_CONFIGURATION_FILE_NAME;
      configurationFileType =
        CommandPersistenceService.FLOW_CONFIGURATION_FILE_TYPE;
      configurationSerializer = new FlowAlgorithmConfigurationSerializer();
    }
    else {
      final String fmt =
        "No foi possvel determinar o arquivo de metadados do configurador do comando %s.\n";
      throw new IllegalArgumentException(String.format(fmt, cmdId));
    }

    final Object projectId = command.getProjectId();
    final ServerProject project = getProjectById(projectId);
    try {
      createProjectFiles(project, command, configurationFileName,
        configurationFileType);
    }
    catch (OperationFailureException ofe) {
      // Tratamento para que saia no log do command-persistence-service em
      // correlao ao que aparece na inexistncia do arquivo (peridico).
      final String fmt =
        "Erro ao criar arquivos de persistncia do comando [%s].\n";
      logSevereMessage(String.format(fmt, cmdId), ofe);
      throw ofe;
    }

    try {
      saveCommandInfoProperties(project, command);
      saveAlgorithmConfiguration(project, command, configurationFileName,
        configurationSerializer);
    }
    catch (final OperationFailureException ofe) {
      try {
        removeCommandInfo(projectId, cmdId);
      }
      catch (final ServiceFailureException sfe) {
        final String fmt =
          "Erro ao remover arquivos de persistncia do comando %s.\n";
        logSevereMessage(String.format(fmt, cmdId), sfe);
      }
      throw ofe;
    }

    // Notifica a todos os interessados que o comando foi salvo com sucesso.
    final Set<Object> allUsers = project.getAllUsers(true);
    final CommandPersistenceCommandNotificationData notif =
      new CommandPersistenceCommandNotificationData(projectId, cmdId,
        Type.SAVED);
    notifyUsers(allUsers, notif);
  }

  /**
   * Salva o arquivo {@value #COMMAND_PROPERTIES_FILE_NAME} de um comando
   * fornecido.
   * 
   * @param project O projeto (No aceita {@code null}).
   * @param command O comando (No aceita {@code null}).
   * 
   * @throws OperationFailureException Se houver algum erro ao escrever no
   *         arquivo.
   */
  private void saveCommandInfoProperties(final ServerProject project,
    final CommandInfo command) throws OperationFailureException {
    final String commandId = command.getId();
    final ServerProjectFile commandDirectory =
      getCommandDirectory(project, commandId);
    if (commandDirectory == null) {
      final String key = "error_command_directory_not_found";
      final String message = getMessage(key, commandId);
      throw new ServiceFailureException(message);
    }
    final ServerProjectFile commandPropertiesFile =
      (ServerProjectFile) commandDirectory
        .getChild(CommandPersistenceService.COMMAND_PROPERTIES_FILE_NAME);
    if (commandPropertiesFile == null) {
      final String path = commandDirectory.getCanonicalPath();
      final String message =
        getMessage("error_file_not_found", path,
          CommandPersistenceService.COMMAND_PROPERTIES_FILE_NAME);
      throw new OperationFailureException(message);
    }
    Object lockId = null;
    final String path = commandPropertiesFile.getCanonicalPath();
    try {
      final ServerFileLockListener lockListener = new ServerFileLockListener();
      lockId =
        commandPropertiesFile.acquireExclusiveLock(lockListener,
          CommandPersistenceService.LOCK_EXCLUSIVE_TIMEOUT);
      if (lockListener.getLock()) {
        saveCommandInfoPropertiesToStream(commandPropertiesFile, command);
      }
      else {
        final String fmt =
          getString("CommandPersistenceService.error.get.exclusive.lock");
        final String message = String.format(fmt, path);
        throw new OperationFailureException(message);
      }
    }
    catch (final IOException e) {
      final String key = "error_write_command_properties";
      final String message = getMessage(key, commandId, path);
      throw new OperationFailureException(message, e);
    }
    finally {
      if (lockId != null) {
        commandPropertiesFile.releaseLock(lockId);
      }
    }
  }

  /**
   * Persiste as propriedades do comando.
   * 
   * @param commandPropertiesFile arquivo de sada
   * @param command informaes do comando
   * @throws IOException caso ocorra falha na persistncia do comando
   * @throws OperationFailureException em caso de falha na escrita do
   *         server-project-file associado.
   */
  private void saveCommandInfoPropertiesToStream(
    final ServerProjectFile commandPropertiesFile, final CommandInfo command)
    throws IOException, OperationFailureException {
    final CommandPropertyParser parser = new CommandPropertyParser();
    final String commandId = command.getId();
    parser.setId(commandId);
    parser.setUserId(command.getUserId());
    parser.setSubmittedDate(command.getSubmittedDate());
    parser.setWallTimeSec(command.getWallTimeSec());
    parser.setGlobalPosition(command.getGlobalPosition());
    parser.setMailAtEnd(command.isMailAtEnd());
    parser.setPriority(command.getPriority());
    parser.setStatus(command.getStatus());
    parser.setFinalizationType(command.getFinalizationType());
    parser.setDescription(command.getDescription());
    parser.setSGAName(command.getSGAName());
    parser.setPlatformFilter(command.getPlatformFilter());
    parser.setTip(command.getTip());

    final boolean isAutomatic = command.isAutomatic();
    parser.setIsAutomatic(isAutomatic);

    final ExecutionType executionType = command.getExecutionType();
    parser.setExecutionType(executionType);

    switch (executionType) {
      case PARALLEL:
        parser.setCpuCountForParallelExecution(command
          .getCpuCountForParallelExecution());
        if (!isAutomatic) {
          parser.setSelectedSGAsNames(command.getSelectedSGAsNames());
        }
        break;
      case MULTIPLE:
        if (isAutomatic) {
          parser.setExecutionCountForMultipleExecution(command
            .getExecutionCountForMultipleExecution());
        }
        else {
          parser.setExecutionCountPerSGAForMultipleExecution(command
            .getExecutionCountPerSGAForMultipleExecution());
          parser.setSelectedSGAsNames(command.getSelectedSGAsNames());

        }
        break;
      case DISTRIBUTED:
        if (isAutomatic) {
          parser.setSgaCountForDistributedExecution(command
            .getSGACountForDistributedExecution());
        }
        else {
          parser.setExecutionCountPerSGAForDistributedExecution(command
            .getExecutionCountPerSGAForDistributedExecution());
          parser.setSelectedSGAsNames(command.getSelectedSGAsNames());
        }
        break;
      case SIMPLE:
        if (!isAutomatic) {
          parser.setSelectedSGAsNames(command.getSelectedSGAsNames());
        }
        break;
      case XAPPS:
        if (!isAutomatic) {
          parser.setSelectedSGAsNames(command.getSelectedSGAsNames());
        }
        break;
      default:
        throw new IllegalStateException(String.format(
          "Tipo de execuo invlido no arquivo de persistncia %s.",
          executionType));
    }

    OutputStream outputStream = null;
    try {
      outputStream = commandPropertiesFile.openOutputStream();
      parser.saveProperties(outputStream);
    }
    catch (final ServerProjectFileException e) {
      final String message =
        getMessage("error_write_command_properties", commandId,
          commandPropertiesFile.getCanonicalPath());
      throw new OperationFailureException(message, e);
    }
    finally {
      if (outputStream != null) {
        outputStream.close();
      }
    }
  }

  /**
   * Mtodo invocado no trmino do servio.
   */
  @Override
  public void shutdownService() {
    if (cleanupThread != null) {
      cleanupThread.shutdown();
    }
  }

  /**
   * {@inheritDoc}
   */
  public void updateCommandDescription(final Object projectId,
    final String commandId, final String description) {
    if (projectId == null) {
      final String err = "O parmetro projectId est nulo.";
      throw new IllegalArgumentException(err);
    }

    if (commandId == null) {
      final String err = "O parmetro commandId est nulo.";
      throw new IllegalArgumentException(err);
    }

    final CommandInfo command = getCommandInfo(projectId, commandId);
    command.setDescription(description);
    updateCommandInfo(command);
  }

  /**
   * Atualiza as informaes do comando.
   * 
   * @param command comando a ser atualizado.
   */
  public void updateCommandInfo(final CommandInfo command) {
    if (command == null) {
      final String err = "O parmetro command est nulo.";
      throw new IllegalArgumentException(err);
    }

    final String commandId = command.getId();
    final Object projectId = command.getProjectId();
    try {
      final ServerProject project = getProjectById(projectId);
      saveCommandInfoProperties(project, command);
      notifyUsers(project.getAllUsers(true),
        new CommandPersistenceCommandNotificationData(projectId, commandId,
          Type.UPDATED));
    }
    catch (final OperationFailureException e) {
      final String message = getMessage("error_save_command_info", commandId);
      throw new ServiceFailureException(message, e);
    }
  }

  /**
   * Construtor da classe.
   * 
   * @throws ServerException se houver erro na inicializao.
   */
  private CommandPersistenceService() throws ServerException {
    super(CommandPersistenceServiceInterface.SERVICE_NAME);
    // XXX - Temporrio. Aguardando a soluo do Bug #5847.
    ClientRemoteLocator.commandPersistenceService = this;
    commandsDirectoryName =
      getStringProperty(CommandPersistenceService.COMMANDS_DIRECTORY_NAME_PROPERTY);
    /*
     * Remove caractres invlidos do nome do projeto. - Remove os espaos antes
     * e depois do nome. - Caractres invlidos so substituidos por '_'. -
     * Somente ser mantido o '.' se este for a primeira letra do nome.
     */
    commandsDirectoryName = commandsDirectoryName.trim();
    if (commandsDirectoryName.startsWith(".")) {
      final String substring = commandsDirectoryName.substring(1);
      commandsDirectoryName = '.' + FileUtils.fixDirectoryName(substring);
    }
    else {
      commandsDirectoryName = FileUtils.fixDirectoryName(commandsDirectoryName);
    }
  }

  /**
   * Obtm o nome do diretrio de persistncia dos arquivos associados s
   * execues dos comandos (default = <code>.cmds</code>).
   * 
   * @return nome do diretrio de persistncia dos arquivos associados s
   *         execues dos comandos
   */
  public String getCommandsDirectoryName() {
    return commandsDirectoryName;
  }

  /**
   * Cria e inicia a thread para limpeza dos arquivos de persistncia.
   * 
   * @throws ServerException
   */
  private void createCleanupThread() throws ServerException {
    cleanupThread = new CommandPersistenceCleanupThread();
    cleanupThread.start();
  }
}
