package csbase.server.services.csfsservice;

import java.lang.reflect.Field;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.text.MessageFormat;
import java.util.*;

import org.omg.CORBA.ORB;

import tecgraf.javautils.FileUtils;
import csbase.server.ServerException;
import csbase.server.Service;
import csfs.remote.*;

/**
 *
 * Servio para copia de arquivos utilizando o CSFS Daemon.
 *
 * O CSFS Daemon exporta uma interface remota que dah acesso ao sistema de
 * arquivos local da maquina que executa o servidor. Essa interface pode ser
 * usada para copiar arquivos entre diversas maquinas e para permitir o
 * gerenciamento dos mesmos.
 *
 * No caso do CSFSService, temos a copia de arquivos entre o CSFS Daemon que
 * exporta a area de projetos (e o repositorio de algoritmos) e o CSFS Daemon
 * que gerencia a area local nas maquinas de execucao. Esse procedimento de
 * copia eh realizado pela comunicacao direta entre esses dois CSFS Daemons.
 *
 * O CSFS Daemon eh definido em CORBA.
 *
 * @author Marcelo Nery dos Santos
 *
 */
public class CSFSService extends Service {

  /** Uma instancia para o ORB */
  private ORB orb;

  /**
   * O formato do corbaloc utilizado pelo ORB (remoto) do CSFS Daemon. Esse
   * valor eh dependente do ORB (fornecedor/versao) e pode ser configurado pelo
   * arquivo de configuracao do servico.
   */
  private String corbalocPattern;

  /** A porta onde o objeto remoto eh iniciado. */
  private int csfsPort;

  /** Uma referencia para o servidor de arquivos que acessa a area do SSI. */
  private FileServer mainFileServer;

	/** Nome do servidor local. */
  private String localhost;

  /** Nome do servico para o <code>ServiceManager</code> */
  public static final String SERVICE_NAME = "CSFSService";

  /**
   * Retorna a instancia do servio.
   *
   * @return o servico.
   */
  public static CSFSService getInstance() {
    return (CSFSService) getInstance(SERVICE_NAME);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected boolean has2Update(Object arg, Object event) {
    // Esse servico nao possui observadores.
    return false;
  }

  /** 
   * Constroi a instncia do servio. 
   * @throws ServerException
   */
  public static void createService() throws ServerException {
    new CSFSService();
  }

  /**
   * Construtor
   * @throws ServerException
   */
  protected CSFSService() throws ServerException {
    super(SERVICE_NAME);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void initService() throws ServerException {
    corbalocPattern = getStringProperty("CSFS_CORBALOC_PATTERN");
    csfsPort = getIntProperty("CSFS_PORT");
    final int retrySleepTime = getIntProperty("RETRY_SLEEP_TIME");

    /*
     * O sistema foi desenvolvido e testado com o uso do JacORB. As classes
     * contidas no jar do csfs possuem as classes que foram compiladas com o
     * compilador de IDL do mesmo. Assim, configuramos a maquina virtual Java
     * para utilizar o JacORB.
     */
    final Properties props = new Properties();
    props.setProperty("OAPort", ""+csfsPort);
    props.setProperty("org.omg.CORBA.ORBClass",
                      "org.jacorb.orb.ORB");
    props.setProperty("org.omg.CORBA.ORBSingletonClass",
                      "org.jacorb.orb.ORBSingleton");

    try {
      localhost = InetAddress.getLocalHost().getHostAddress();
    } catch (UnknownHostException exc) {
      throw new ServerException("Unable to get localhost IP!", exc);
    }

    try {
      orb = ORB.init((String[]) null, props);
    } catch (Exception exc) {
      logSevereMessage("Falha na inicializao do orb.", exc);
      throw new ServerException("ORB.init() failed!");
    }

    /*
     * Inicia uma thread para tentar recuperar uma referencia para o objeto
     * remoto que exporta a arvore de arquivos do SSI. A principio, esse objeto
     * estah na mesma maquina do SSI. Isso pode ser alterado para fazer com que
     * a area de projetos ou o repositorio de algoritmos sejam procurados em
     * outra maquina. A implementacao atual supoe que o CSFS Daemon foi iniciado
     * externamente na mesma maquina do SSI.
     *
     * Podemos avaliar o caso onde o proprio CSFSService incorpora o CSFS
     * Deamon. Ou seja, podemos fazer uma implementacao que faca com que o
     * initService dispare o servidor principal do CSFS.
     *
     */
    new Thread(new Runnable() {
      public void run() {
        while (mainFileServer == null) {
          try {
            mainFileServer = getFileServer(localhost);
            logInfoMessage("Recuperei o mainFileServer "+localhost);
          } catch (ServerException exc) {
            logDebugMessage("Nao consegui recuperar o mainFileServer (exc="
                + exc.getMessage() + "). Vou tentar novamente em "
                + retrySleepTime + " ms...");
          }
          try {
            Thread.sleep(retrySleepTime);
          } catch (InterruptedException e) {
            logSevereMessage("InterruptedException", e);
          }
        }
      }
    }).start();
    logInfoMessage("Servio CSFS iniciado.");
  }

  /**
   * Metodo invocado no termino do servico. Realiza a shutdown do ORB.
   */

  @Override
  public void shutdownService() throws ServerException {
    orb.shutdown(false);
  }

  /**
   *
   * Verifica a existencia de um caminho em um determinado servidor.
   *
   * @param host
   *          o servidor que deve ser consultado
   * @param path
   *          o caminho absoluto do arquivo (relativo a raiz exportada pelo
   *          servidor)
   * @return a indicacao de existencia
   * @throws ServerException
   */
  public boolean checkExistence(String host, String path)
      throws ServerException {
    boolean response = false;
    FileServer fs = getFileServer(host);
    String[] pathArray = FileUtils.splitPath(path);
    try {
      fs.getRoot().getChild(pathArray);
      response = true;
    } catch (FileNotFoundException exc) {
      // a resposta jah foi inicializada como false
    } catch (csfs.remote.ServerException exc) {
      logSevereMessage("Error in CSFS subsystem!", exc);
      throw new ServerException("Remote File Server Exception. message="
          + exc.message, exc);
    } catch (Exception exc) {
      throw new ServerException(
          "Exception during checkExistence operation."
              + " exc.getClass().getName()="+ exc.getClass().getName()
              + " file="+ FileUtils.joinPath(getName(exc)), exc);
    }
    return response;
  }

  /**
   * Retorna o caminho absoluto para o diretrio raiz do CSFS para o servidor
   * especificado.
   *
   * @param host nome do servidor que exporta um sistema de arquivos via CSFS.
   *
   * @return diretrio raiz do CSFS.
   *
   * @throws ServerException
   */
  public String getRootPath(String host) throws ServerException {
    FileServer fs = getFileServer(host);
    try {
      return fs.getProperty("ROOT_DIR");
    } catch (csfs.remote.ServerException exc) {
      throw new ServerException("Exception during getRootPath operation."
        + " exc.getClass().getName()=" + exc.getClass().getName() + " file="
        + FileUtils.joinPath(getName(exc)), exc);
    }
  }

  /**
   * Recupera uma referencia para um objeto remoto, baseado no servidor
   * especificado.
   *
   * @param host
   *          o servidor que exporta um sistema de arquivos pelo CSFS.
   * @return a referencia para o objeto remoto.
   * @throws ServerException
   */
  private FileServer getFileServer(String host) throws ServerException {

    /*
     * Optamos por ter uma configuracao para todas os servidores do CSFS. Dessa
     * forma, a porta nao precisa ser passada como parametro e a porta padrao eh
     * sempre utilizada na recuperacao do objeto remoto. Esse metodo pode ser
     * modificado para receber tambem o parametro da porta.
     */
    String corbaloc = MessageFormat.format(corbalocPattern,
                                           new Object[] {host, ""+csfsPort });
    FileServer response;
    try {
      response = FileServerHelper.narrow(orb.string_to_object(corbaloc));
      response.getName(); // realizando uma chamada remota (sanity-check).
    } catch (Exception exc) {
      throw new ServerException(
          "Error contacting remote file server! corbaloc="
          + corbaloc + "\n", exc);
    }
    return response;
  }

  /**
   *
   * Obtem um mapeamento entre um arquivo e sua data de modificacao para todos
   * os arquivos de uma arvore de diretorios, dentro do servidor especificado.
   * Esse valor eh obtido sobre o arquivo remoto, o que evita problemas de
   * sincronismo de relogios em uma rede composta por varias maquinas.
   *
   * Esse metodo eh a base para permitir que uma arvore de diretorios seja
   * comparada antes e depois da execucao de um algoritmo, fazendo com que a
   * sincronizacao entre a area de armazenamento no SGA e a area de projetos
   * seja feita de uma forma otimizada. Por outro lado, ele pode ter um custo
   * elevado, dado a operacao recursiva sobre a arvore remota.
   *
   * @param host o servidor que mantem a arvore
   * @param path a raiz que deve ser analisada
   * @return mapeamento entre um arquivo e sua data de modificacao para todos
   * os arquivos de uma arvore de diretorios.
   *
   * @throws ServerException
   */
  public Map<String, Long> getTimestamps(String host, String path)
  throws ServerException {
    Map<String, Long> response = new HashMap<String, Long>();
    try {
      FileServer targetFS = getFileServer(host);
      RemoteFile file = lookup(targetFS, path);
      visitRemoteFilesGettingTimestamps(response, file);
    } catch (FileNotFoundException exc) {
      throw new ServerException(
          "Unable to get timestamps for an unexistent path. path=" + path
              + " host=" + host, exc);
    }
    return response;
  }

  /**
   * Obtem um mapeamento entre um arquivo e sua data de modificacao para todos
   * os arquivos de uma arvore de diretorios, dentro do servidor local.
   *
   * @param path caminho para uma rvore de diretrios.
   *
   * @return mapeamento entre um arquivo e sua data de modificacao para todos
   * os arquivos de uma arvore de diretorios, dentro do servidor local.
   *
   * @throws ServerException
   */
  public Map<String, Long> getTimestamps(String path) throws ServerException {
    return getTimestamps(localhost, path);
  }

  /**
   * Implementacao recursiva que percorre uma arvore de diretorios. Sempre que
   * um arquivo eh localizado, a tabela com a data de modificacao eh acrescida
   * em um elemento. No caso de diretorios, esse metodo eh chamado
   * recursivamente para cada um dos seus filhos. Ao final da execucao, o
   * mapeamento possui a informacao para todos os arquivos.
   *
   * @param info
   *          o mapeamento que agrega a informacao sobre as datas de modificacao
   * @param root
   *          um ponto da arvore, que deve ser percorrido recursivamente
   * @throws ServerException
   */
  private void visitRemoteFilesGettingTimestamps(
    Map<String, Long> info, RemoteFile root) throws ServerException {
    try {
      if (!root.isDirectory()) {
        info.put(FileUtils.joinPath(root.getFullName()), new Long(root
            .lastModified()));
      } else {
        RemoteFile[] children = root.getChildren();
        for (int i = 0; i < children.length; i++) {
          visitRemoteFilesGettingTimestamps(info, children[i]);
        }
      }
    } catch (csfs.remote.ServerException exc) {
      logSevereMessage("Error in CSFS subsystem!", exc);
      throw new ServerException(
          "An error occurred in remote file server. message=" + exc.message,
          exc);
    } catch (Exception exc) {
      throw new ServerException(
          "Exception during visitRemoteFilesGettingTimestamps operation."
              + "exc.getClass().getName()="+ exc.getClass().getName()
              + " file="+ FileUtils.joinPath(getName(exc)), exc);
    }
  }

  /**
   * Copia um arquivo da area de armazenamento principal para uma maquina de
   * execucao.
   *
   * @param host
   *          o nome da maquina de execucao
   * @param sourcePath
   *          o caminho absoluto do arquivo na area de armazenamento (relativo a
   *          raiz exportada pelo CSFS).
   * @param targetPath
   *          o caminho absoluto do arquivo a ser criado no destino (relativo a
   *          raiz exportada pelo CSFS).
   * @throws ServerException
   */
  public void copyTo(String host, String sourcePath, String targetPath)
      throws ServerException {
    FileServer sgaFileServer = getFileServer(host);
    copy(mainFileServer, sourcePath, sgaFileServer, targetPath);
  }

  /**
   * Copia um arquivo de uma maquina de execucao para a area de armazenamento
   * principal.
   *
   * @param host
   *          o nome da maquina de execucao
   * @param sourcePath
   *          o caminho absoluto do arquivo na maquina de execucao (relativo a
   *          raiz exportada pelo CSFS).
   * @param targetPath
   *          o caminho absoluto do arquivo a ser criado na area de
   *          armazenamento principal (relativo a raiz exportada pelo CSFS).
   * @throws ServerException
   */
  public void copyFrom(String host, String sourcePath, String targetPath)
      throws ServerException {
    FileServer sgaFileServer = getFileServer(host);
    copy(sgaFileServer, sourcePath, mainFileServer, targetPath);
  }

  /**
   * Realiza a copia de arquivos (Esse metodo opera recursivamente sobre
   * diretorios).
   *
   * @param sourceFS
   *          o servidor (CSFS) de origem.
   * @param sourcePath
   *          o caminho absoluto no servidor de origem (relativo a raiz
   *          exportada pelo CSFS).
   * @param targetFS
   *          o servidor (CSFS) de destino.
   * @param targetPath
   *          o caminho absoluto no servidor de destino (relativo a raiz
   *          exportada pelo CSFS).
   * @throws ServerException
   */
  private void copy(FileServer sourceFS, String sourcePath,
      FileServer targetFS, String targetPath) throws ServerException {
    try {
      // a origem deve existir.
      RemoteFile sourceFile = lookup(sourceFS, sourcePath);
      // o destino pode existir previamente (ou nao).
      RemoteFile targetFile = lookupOrCreate(targetFS, targetPath, sourceFile
          .isDirectory());
      if (sourceFile.isDirectory()) {
        copyDirectory(sourceFile, targetFile);
      } else {
        copyFile(sourceFile, targetFile);
      }
    } catch (Exception exc) {
      throw new ServerException(
          "Exception during copy operation. exc.getClass().getName()="
              + exc.getClass().getName() + " file="
              + FileUtils.joinPath(getName(exc)), exc);
    }
  }

  /**
   * Realiza a copia recursiva de diretorios. Tanto a origem quanto o destino
   * devem apontar explicitamente para um diretorio.
   *
   * Um tratamento peculiar sobre a preexistencia de arquivos e diretorios na
   * arvore remota eh realizado. De forma sucinta, arquivos preexistentes no
   * destino serao sobrescritos, desde que a origem tambem seja um arquivo. Caso
   * um diretorio esteja sendo copiado sobre um destino preexistente, caso o
   * destino tambem seja um diretorio, ele receberah novos elementos. Caso
   * contrario, ou seja, quando um arquivo iria sobreescrever um diretorio, uma
   * excecao eh lancada.
   *
   * Esses procedimentos foram definidos com base em algumas observacoes sobre a
   * coerencia e previsibilidade de alguns algoritmos. Outros tipos de
   * algoritmos talvez possam necessitar de uma outra semantica sobre as
   * operacoes de copia, e esse procedimento podem ser revisto de acordo com o
   * uso do servico.
   *
   * @param sourceFile
   *          o diretorio de origem
   * @param targetFile
   *          o diretorio de destino
   *
   * @throws NotDirectoryException
   * @throws csfs.remote.ServerException
   * @throws InvalidStateException
   * @throws InvalidPathException
   * @throws FileNotFoundException
   * @throws ServerException
   * @throws FileAlreadyExistsException
   * @throws NotFileException
   */
  private void copyDirectory(RemoteFile sourceFile, RemoteFile targetFile)
      throws NotDirectoryException, csfs.remote.ServerException,
      InvalidStateException, InvalidPathException, FileNotFoundException,
      ServerException, FileAlreadyExistsException, NotFileException {
    RemoteFile[] children = sourceFile.getChildren();
    for (int i = 0; i < children.length; i++) {
      if (children[i].isDirectory()) {
        // caso a origem seja um diretorio, tentaremos criar um diretorio no destino
        RemoteFile targetChildFile;
        try {
          targetChildFile = targetFile.createDirectory(
              new String[] { children[i].getName() });
        } catch (FileAlreadyExistsException e) {
          // Caso um elemento jah exista, precisamos ver se eh um arquivo, para
          // levantar a excecao.
          targetChildFile = targetFile.getChild(
              new String[] { children[i].getName() });
          if (!targetChildFile.isDirectory()) {
            throw new ServerException("There is a file in the way: "
                + FileUtils.joinPath(e.name), e);
          }
        }
        // chamamos a funcao recursiva para mais um nivel na arvore
        copyDirectory(children[i], targetChildFile);
      } else {
        // tentamos realizar a copia do arquivos para o diretorio de destino
        copyFile(children[i], targetFile);
      }
    }
  }

  /**
   * Copia um arquivo. Caso o destino seja um arquivo, ele serah sobrescrito.
   * Caso contrario, um novo arquivo serah criado no diretorio de destino (com o
   * mesmo nome do arquivo de origem). Caso um elemento jah exista, se ele for
   * um arquivo, serah sobrescrito. Se for um diretorio, um conflito entre o
   * nome da origem e o elemento contido no diretorio de destino serah
   * detectado, e uma excecao serah lancada.
   *
   * Para que os binarios possam ser executados no Linux, eles devem possuir a
   * permissao de acesso adequada. Optamos por fazer com que todo arquivo
   * copiado tenha essa permissao, independente do fato do arquivo ser realmente
   * um binario. Como a versao atual de Java ainda nao implementa essa
   * atribuicao, o servidor faz uma chamada externa (via JNI/Runtime.exec())
   * para o "chmod". Caso a plataforma do servidor seja "Win32", definido pela
   * variavel "os.arch" (da classe System), a tentativa de se habilitar a
   * permissao de execucao nao surte nenhum efeito.
   *
   * @param sourceFile
   *          o arquivo de origem
   * @param targetFile
   *          o diretorio de destino
   *
   * @throws csfs.remote.ServerException
   * @throws InvalidStateException
   * @throws NotDirectoryException
   * @throws InvalidPathException
   * @throws FileAlreadyExistsException
   * @throws NotFileException
   * @throws FileNotFoundException
   * @throws ServerException
   */
  private void copyFile(RemoteFile sourceFile, RemoteFile targetFile)
      throws csfs.remote.ServerException, InvalidStateException,
      NotDirectoryException, InvalidPathException, FileAlreadyExistsException,
      NotFileException, FileNotFoundException, ServerException {
    RemoteFile targetCopyFile = targetFile;
    if (targetFile.isDirectory()) {
      try {
        targetCopyFile = targetFile.createFile(new String[] { sourceFile
            .getName() });
      } catch (FileAlreadyExistsException e) {
        // Caso um elemento jah exista, precisamos ver se eh um diretorio, para
        // levantar a excecao.
        targetCopyFile = targetFile.getChild(new String[] { sourceFile
            .getName() });
        if (targetCopyFile.isDirectory()) {
          throw new ServerException("There is a file in the way: "
              + FileUtils.joinPath(e.name), e);
        }
      }
    }
//    sourceFile.copyTo(targetCopyFile, "CORBA-32768");
    sourceFile.copyTo(targetCopyFile, "SCP");
    targetCopyFile.enableExecutionPermission();
  }

  /**
   * Remove uma arvore remota recursivamente no servidor especificado.
   *
   * @param host
   *          o servidor que possui a arvore
   * @param path
   *          o caminho absoluto do arquivo (relativo a raiz exportada pelo
   *          CSFS).
   * @throws ServerException
   */
  public void remove(String host, String path) throws ServerException {
    try {
      FileServer targetFS = getFileServer(host);
      RemoteFile file = lookup(targetFS, path);
      recursiveRemove(file);
    } catch (FileNotFoundException exc) {
      throw new ServerException("A file was not found: "
          + FileUtils.joinPath(exc.name), exc);
    }
  }

  /**
   * Implementacao recursiva do metodo de remocao.
   *
   * @param file
   *   a raiz que deve ser removida.
   * @throws ServerException
   */
  private void recursiveRemove(RemoteFile file) throws ServerException {
    try {
      if (file.isDirectory()) {
        RemoteFile[] children = file.getChildren();
        for (int i = 0; i < children.length; i++) {
          recursiveRemove(children[i]);
        }
      }
      file.remove();
    } catch (csfs.remote.ServerException exc) {
      logSevereMessage("Error in CSFS subsystem!", exc);
      throw new ServerException("Remote File Server Exception. message="
          + exc.message, exc);
    } catch (Exception exc) {
      throw new ServerException(
          "Exception during recursiveRemove operation. exc.getClass().getName()="
              + exc.getClass().getName() + " file="
              + FileUtils.joinPath(getName(exc)), exc);
    }
  }

  /**
   * Cria um diretorio em uma maquina.
   *
   * @param host
   *          o servidor onde o diretorio deve ser criado
   * @param dir
   *          o caminho absoluto do arquivo (relativo a raiz exportada pelo
   *          CSFS).
   * @throws ServerException
   */
  public void createDirectory(String host, String dir) throws ServerException {
    FileServer targetFS = getFileServer(host);
    lookupOrCreate(targetFS, dir, true);
  }

  /**
   * Busca uma referencia para um arquivo remoto.
   *
   * @param fs
   *          o servidor de arquivos que mantem o elemento.
   * @param path
   *          o caminho absoluto do arquivo (relativo a raiz exportada pelo
   *          CSFS).
   * @return a referencia para o arquivo remoto
   * @throws FileNotFoundException
   * @throws ServerException
   */
  private RemoteFile lookup(FileServer fs, String path)
      throws FileNotFoundException, ServerException {
    if (fs == null) {
      logWarningMessage("Trying to lookup a file in a null file system");
      throw new ServerException(
          "It is not possible to lookup a path in a null file server reference!");
    }
    RemoteFile response;
    try {
      response = fs.getRoot().getChild(FileUtils.splitPath(path));
    } catch (FileNotFoundException exc) {
      throw exc;
    } catch (csfs.remote.ServerException exc) {
      logSevereMessage("Error in CSFS subsystem!", exc);
      throw new ServerException("Remote File Server Exception. message="
          + exc.message, exc);
    } catch (Exception exc) {
      throw new ServerException(
          "Exception during lookup operation. exc.getClass().getName()="
              + exc.getClass().getName() + " file="
              + FileUtils.joinPath(getName(exc)), exc);
    }
    return response;
  }

  /**
   * Tenta buscar uma referencia para um elemento remoto. Caso nao encontre, um
   * novo elemento serah criado no servidor de destino e a referencia serah
   * retornada.
   *
   * @param fs  o servidor de destino
   * @param path o caminho absoluto do arquivo (relativo a raiz exportada pelo CSFS).
   * @param directory indica qual o tipo de elemento deve ser buscado/criado.
   * @return o elemento
   * @throws ServerException
   */
  private RemoteFile lookupOrCreate(FileServer fs, String path,
      boolean directory) throws ServerException {
    RemoteFile response = null;
    try {
      response = lookup(fs, path);
      if (response.isDirectory() != directory) {
        throw new ServerException("type mismatch. path=" + path
            + " response.isDirectory()=" + response.isDirectory()
            + " expected=" + directory);
      }
    } catch (FileNotFoundException e) {
      try {
        if (directory) {
          response = fs.getRoot().createDirectory(FileUtils.splitPath(path));
        } else {
          response = fs.getRoot().createFile(FileUtils.splitPath(path));
        }
      // talvez seja possivel um refactoring nesse tratamento de excecoes.
      } catch (csfs.remote.ServerException exc) {
        logSevereMessage("Error in CSFS subsystem!", exc);
        throw new ServerException("Remote File Server Exception. message="
            + exc.message, exc);
      } catch (Exception exc) {
        throw new ServerException(
            "Exception during lookupOrCreate operation. exc.getClass().getName()="
                + exc.getClass().getName() + " file="
                + FileUtils.joinPath(getName(exc)), exc);
      }
    } catch (csfs.remote.ServerException exc) {
      logSevereMessage("Error in CSFS subsystem!", exc);
      throw new ServerException("Remote File Server Exception. message="
          + exc.message, exc);
    } catch (Exception exc) {
      throw new ServerException(
          "Exception during lookupOrCreate operation. exc.getClass().getName()="
              + exc.getClass().getName() + " file="
              + FileUtils.joinPath(getName(exc)), exc);
    }
    return response;
  }


  /**
   * Recupera o nome do arquivo associado a uma excecao (caso disponivel).
   *
   * De acordo com a excecao, a informacao sobre o arquivo pode estar disponivel
   * como um membro da classe. O ideal era ter uma superclasse das excecoes de
   * arquivo, e fazer o catch por ela, em vez de um catch generico por
   * Exception... Um mecanismo de heranca de excecoes em CORBA nao foi
   * localizado, e essa implementacao baseada em "reflection" foi usada como
   * alternativa. Na especificacao da IDL, toda excecao de arquivo possui um
   * String[] name indicando o arquivo onde aconteceu o erro.
   *
   * @param exc
   * @return o nome.
   */
  private String[] getName(Exception exc) {
    String[] name = null;
    try {
      Field f = exc.getClass().getDeclaredField("name");
      name = (String[]) f.get(exc);
    } catch (Exception e) {
      name = new String[] { "unspecified" };
    }
    return name;
  }

}
