/*
 * $Id:$
 */

package csbase.server.services.filetransferservice.sftp;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;

import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;
import com.jcraft.jsch.SftpProgressMonitor;
import com.jcraft.jsch.UserInfo;

import csbase.exception.ServiceFailureException;
import csbase.logic.ProjectFileType;
import csbase.logic.filetransferservice.FileTransferConnection;
import csbase.logic.filetransferservice.FileTransferRequest;
import csbase.logic.filetransferservice.FileTransferRequestStatus;
import csbase.server.services.filetransferservice.FileTransferJob;
import csbase.server.services.filetransferservice.FileTransferService;
import csbase.server.services.projectservice.ServerProject;
import csbase.server.services.projectservice.ServerProjectFile;

/**
 * Job de download no protocolo FTP.
 * 
 * @author Tecgraf/PUC-Rio
 */
class SFTPDownloadJob extends FileTransferJob {

  /**
   * {@inheritDoc}
   */
  @Override
  final protected void transfer() throws Exception {
    final FileTransferRequest request = getRequest();
    final FileTransferConnection connection = request.getConnection();
    final String remoteFilePath = request.getRemoteFilePath();
    final String serverName = connection.getServerName();
    final String userName = connection.getUserName();
    final String password = connection.getPassword();

    JSch jsch = new JSch();
    final int port = 22;
    final Session session;
    try {
      session = jsch.getSession(userName, serverName, port);
    }
    catch (JSchException e) {
      throw new ServiceFailureException(e.getMessage());
    }

    final UserInfo ui = new StubUserInfo(password);
    session.setUserInfo(ui);

    final InputStream inputStream;
    try {
      session.connect();
      final Channel c = session.openChannel("sftp");
      c.connect();
      final ChannelSftp channel = (ChannelSftp) c;
      final String[] splittedPath = remoteFilePath.split(File.separator);
      final String file = splittedPath[splittedPath.length-1];
      final String dir = remoteFilePath.substring(0, remoteFilePath.lastIndexOf(File.separator));

      channel.cd(dir);
      inputStream = channel.get(file, (SftpProgressMonitor)null);
    }
    catch (JSchException e) {
      throw new ServiceFailureException(e.getMessage());
    }

    final Object projectId = connection.getProjectId();
    final ServerProject project = ServerProject.getProject(projectId);
    final String[] localPath = request.getLocalFilePath();
    final Object userId = connection.getUserId();

    if (!project.exists(localPath)) {
      final String unknown = ProjectFileType.UNKNOWN;
      final String fileType = ProjectFileType.getFileType(unknown).getCode();
      project.createFile(localPath, fileType, userId, false);
    }
    
    final String[] fullPath = project.makeFullPath(localPath);
    final ServerProjectFile spf =
      ServerProjectFile.findFile(projectId, fullPath);
    spf.setUnderConstruction(true);

    final FileOutputStream outputStream =
      new FileOutputStream(spf.getRealFile());
    try {
      final int chunkSize = FileTransferService.CHUNK_SIZE;
      byte[] b = new byte[chunkSize];
      int bytesRead = 0;
      while ((bytesRead = inputStream.read(b)) != -1) {
        outputStream.write(b, 0, bytesRead);
        request.addTransferredSize(bytesRead);
        if (request.getStatus() == FileTransferRequestStatus.INTERRUPTED) {
          return;
        }
      }
      outputStream.flush();
    }
    catch (final Exception e) {
      spf.remove();
      throw e;
    }
    finally {
      outputStream.close();
    }

    spf.setUnderConstruction(false);
  }

  /**
   * Construtor
   * 
   * @param request requisio.
   */
  public SFTPDownloadJob(final FileTransferRequest request) {
    super(request);
  }

}
