/* HttpService.java
 * $Author: clinio $
 * $Revision: 91260 $ - $Date: 2009-05-07 14:10:50 -0300 (Thu, 07 May 2009) $
 */
package csbase.server.services.httpservice;

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.*;

import csbase.exception.HttpServiceException;
import csbase.exception.InvalidRequestException;
import csbase.logic.User;
import csbase.remote.HttpServiceInterface;
import csbase.server.ServerException;
import csbase.server.Service;
import csbase.server.services.loginservice.LoginService;
import csbase.server.services.projectservice.*;

/**
 * A classe <code>HttpService</code> implementa o servio de  download e upload
 * de arquivos. Cada requisicao de dowload e' inserida na requestList, para um
 * usuario, projeto e filePath Nulo, alem de a data de criacao do registro. O
 * filePath e' nulo no download pois o arquivo pode ter referencias para
 * outros  arquivos que possuem caminhos distintos, sendo que estes precisam
 * de um mesmo codigo de acesso.
 */
public class HttpService extends Service implements HttpServiceInterface {
  private static final String UPLOAD = "upload";
  private static final String DOWNLOAD = "download";
  private static final float MAX_SAVE_TIME = 7;
  private static final float CLEAN_TIME = 1;

  /** Mensagem para parmetros invlidos/inadequados passsados pelo cliente. */
  private static final String INVALID_REQUEST_MSG = "invalid_request_msg";

  /** Hashtable com as requisicoes dos arquivo */
  private Hashtable<String, RequestInfo> requestList = null;

  /** Servidor http */
  private String httpServiceURL = null;

  /** Gerador de numeros magicos para o download */
  private Random numberGenerator;

  /** Thread para limpeza das requisicoes antigas */
  private Cleaner cleaner;

  /** Flag usado para desativar a thread de limpeza */
  private boolean shutdown;

  /**
   * Representacao de usuario, projeto e arquivo.  whenDone indica a data de
   * criacao de uma instancia de registro.
   */
  private class RequestInfo {
    String projectName;
    Object userId;
    Object ownerId;
    String filePath;
    String fileType;
    int downloadType;
    public long whenDone = 0;
    boolean removeFile = false;
    String uploadPresentationPath;
    String uploadResultPath;
    String text;
    Map parameters;
    String jdbcDriverClassName;
    String jdbcURL;
    String jdbcUser;
    String jdbcPassword;
    Observer observer;
    public UploadHandler fileHandler;
  }

  /**
   * Objeto que permite a observao do servidor da classe HttpService.
   * Quando o upload e' finalizado, os observadores sao notificados.
   * E' interessante quando outros servicos querem saber quando o
   * upload terminou.
   */
  private static Observable observable =
    new Observable() {
      @Override
      public void notifyObservers(Object arg) {
        setChanged();
        super.notifyObservers(arg);
      }
    };

  /**
   * Constri a instncia do servio.
   *
   * @throws ServerException .
   */
  public static void createService()
  throws ServerException {
    new HttpService();
  }

  protected HttpService()
  throws ServerException {
    super(SERVICE_NAME);
    try {
      UnicastRemoteObject.exportObject(this, 0);
    }
    catch (Exception e) {
      throw new ServerException("Falha na construo de: "+SERVICE_NAME, e);
    }
  }

  public String getDownloadURL(
    Object userId, String password, Object ownerId, String projectName,
    String[] filePath)
  throws RemoteException {
    //Usurio que fez a chamada ao mtodo. Usar para a autenticao.
    User user = LoginService.getInstance().checkLogin((String)userId, password);
    if (user == null) {
      throw new RemoteException(
        "Falha na autenticao do usurio. \n"+
        "Verifique se o login e a senha so vlidos");
    }
    return getURL(
      userId, ownerId, projectName, filePath, null, DOWNLOAD, FILE_DOWNLOAD_TYPE);
  }

  public String getUploadURL(
    Object userId, String password, Object ownerId, String projectName,
    String[] filePath, String fileType)
  throws RemoteException {
    //Usurio que fez a chamada ao mtodo. Usar para a autenticao.
    User user = LoginService.getInstance().checkLogin((String)userId, password);
    if (user == null) {
      throw new RemoteException(
        "Falha na autenticao do usurio. \n"+
        "Verifique se o login e a senha so vlidos");
    }
    try {
      ServerProjectFile spf =
        getServerProjectFile(ownerId, projectName, filePath);
      if (spf == null) {
        String pathStr = Arrays.toString(filePath);
        throw new InvalidRequestException(
          getString(INVALID_REQUEST_MSG),
          "Caminho "+pathStr+" no encontrado na rvore do servidor");
      }
      if (!spf.isDirectory()) {
        spf.setUnderConstruction(true);
      }
    }
    catch (ServerProjectFileException e) {
      logSevereMessage(
        "Falha para mudar o estado do arquivo! "+"("+filePath+") "+e);
    }
    catch (InvalidRequestException e) {
      logSevereMessage(
        "Argumentos invlidos do cliente: (ownerId="+ownerId+"), "+
        "(projectName="+projectName+"), (filePath="+filePath+")");
    }
    return getURL(userId, ownerId, projectName, filePath, fileType, UPLOAD);
  }

  /**
   * Obtem a url atraves da qual o arquivo deve ser acessado.
   *
   * @param userId Identificador do usurio no servidor http remoto.
   * @param ownerId identificador do usurio dono do projeto.
   * @param projectName identificador do projeto.
   * @param filePath caminho do arquivo local no projeto.
   * @param fileType .
   * @param type indica se e' upload ou download.
   * @param downloadType .
   *
   * @return String que representa a url do arquivo
   *
   * @throws RemoteException .
   */
  private String getURL(
    Object userId, Object ownerId, String projectName, String[] filePath,
    String fileType, String type, int downloadType)
  throws RemoteException {
    if ((userId == null) || (projectName == null) || (filePath == null)) {
      return null;
    }
    /*codifica o path do usuario*/
    String path = filePathtoString(filePath);
    if (path == null) {
      logSevereMessage("Path do arquivo invalido");
      return null;
    }
    /*Cria um objeto RequestInfo para ser adicionado ao hash*/
    RequestInfo requestInfo = new RequestInfo();
    requestInfo.projectName = projectName;
    requestInfo.ownerId = ownerId;
    requestInfo.userId = userId;
    requestInfo.filePath = path;
    requestInfo.fileType = fileType;
    requestInfo.downloadType = downloadType;
    /*Verifica se existe codigo para o usuario e projeto*/
    String accessCode = null;
    if (type.equals(DOWNLOAD)) {
      accessCode = findCode(userId, projectName, null);
    }
    else {
      accessCode = findCode(userId, projectName, path);
      requestInfo.uploadPresentationPath = getStringProperty("uploadPresentation");
      requestInfo.uploadResultPath = getStringProperty("uploadResult");
    }
    if (accessCode == null) {
      /*Gerando um numero magico*/
      accessCode = ""+Math.abs(numberGenerator.nextLong());
      accessCode = accessCode+Math.abs(numberGenerator.nextLong());
    }

    /*Coloca a hora atual no registro*/
    requestInfo.whenDone = System.currentTimeMillis();

    /*adiciona a lista de requisicoes*/
    requestList.put(accessCode, requestInfo);
    String  url;
    if (type.equals(DOWNLOAD)){
      url = httpServiceURL+"/"+type+"/"+accessCode+"/"+type+"/"+path;
    } else {
      url = httpServiceURL+"/"+type+"/"+accessCode+"/"+type;
    }
    logInfoMessage("URL: " + url + " - filePath: " + requestInfo.filePath +
      " - user: " + userId);
    return url;
  }

  private String encode(String text) {
    try {
      return URLEncoder.encode(text, "UTF-8");
    }
    catch (Exception e) {
      logSevereMessage("No foi possvel codificar o texto: "+text);
      return null;
    }
  }

  private String getURL(
    Object userId, Object ownerId, String projectName, String[] filePath,
    String fileType, String type)
  throws RemoteException {
    return getURL(
      userId, ownerId, projectName, filePath, fileType, type,
      UNKNOWN_DOWNLOAD_TYPE);
  }

  /**
   * Verifica se jah exite codigo para o usuario, projeto e filePath, no caso
   * do upload.
   *
   * @param userId .
   * @param projectName .
   * @param filePath .
   *
   * @return accessCode, caso exista. Null caso contrario.
   */
  private String findCode(Object userId, Object projectName, String filePath) {
    for (Enumeration e = requestList.keys(); e.hasMoreElements();) {
      String accessCode = (String)e.nextElement();
      RequestInfo requestInfo = requestList.get(accessCode);
      if (
        (userId.equals(requestInfo.userId)) &&
        (projectName.equals(requestInfo.projectName)) &&
        ((filePath == null) ||
        ((filePath != null) && (filePath.equals(requestInfo.filePath))))) {
        return accessCode;
      }
    }
    return null;
  }

  public String getFileType(String accessCode, String filePath)
  throws RemoteException {
    if (accessCode == null) {
      return null;
    }
    RequestInfo requestInfo = requestList.get(accessCode);
    if (requestInfo == null) {
      return null;
    }
    String mimeType = "application/octet-stream";
    String[] path;
    if (filePath == null) {
      path = requestInfo.filePath.split("/");
    } else {
      String auxPath = expandPath(filePath);
      if (auxPath == null) return null;
      path = auxPath.split("/");
    }
    //Se no tem projeto associado retorna o tipo a partir do nome
    if (requestInfo.projectName == null) {
      return getMimeTypeFromName(path[path.length-1]);
    }
    try {
      ServerProjectFile spf =
        getServerProjectFile(
          requestInfo.ownerId, requestInfo.projectName, path);
      if (spf == null) {
        String pathStr = Arrays.toString(path);
        throw new InvalidRequestException(
          getString(INVALID_REQUEST_MSG),
          "Caminho "+pathStr+" no encontrado na rvore do servidor");
      }
      String fileType = spf.getType();
      mimeType = ServerProjectFileType.getInstance().getMimeType(fileType);
    }
    catch (Exception e) {
      logSevereMessage(
        "Falha obtencao do tipo do arquivo! "+"("+requestInfo.filePath+") "+e);
    }
    return mimeType;
  }

  public int getDownloadType(String accessCode)
  throws RemoteException {
    if (accessCode == null) {
      return UNKNOWN_DOWNLOAD_TYPE;
    }
    RequestInfo requestInfo = requestList.get(accessCode);
    if (requestInfo == null) {
      return UNKNOWN_DOWNLOAD_TYPE;
    }
    return requestInfo.downloadType;
  }

  /**
   * Obtm o mimeType a partir da terminacao do arquivo.
   *
   * @param fileName .
   *
   * @return .
   */
  private String getMimeTypeFromName(String fileName) {
    int index = fileName.lastIndexOf(".");
    String fileExtention = null;
    if (index != -1) {
      fileExtention = fileName.substring(index+1);
    }
    if (fileExtention == null) {
      return "application/zip";
    }
    fileExtention = fileExtention.toLowerCase();
    if (fileExtention.equals("doc")) {
      return "application/msword";
    }
    if (fileExtention.equals("xls")) {
      return "application/vnd.ms-excel";
    }
    if (fileExtention.equals("pdf")) {
      return "application/pdf";
    }
    if (fileExtention.equals("txt")) {
      return "text/plain";
    }
    if (fileExtention.equals("log")) {
      return "text/plain";
    }
    if (fileExtention.equals("jpg")) {
      return "image/jpeg";
    }
    if (fileExtention.equals("gif")) {
      return "image/gif";
    }
    if (fileExtention.equals("tif")) {
      return "image/tiff";
    }
    if (fileExtention.equals("rob")) {
      return "text/plain";
    }
    if (fileExtention.equals("html")) {
      return "text/html";
    }
    return "application/zip";
  }

  public boolean createFile(String accessCode, String fileName)
  throws RemoteException {
    if (accessCode == null) {
      return false;
    }
    RequestInfo reqInfo = requestList.get(accessCode);
    if (reqInfo == null) {
      return false;
    }
    if (reqInfo.projectName == null) {
      if (createFileOutProject(reqInfo.filePath, fileName)){
        reqInfo.filePath = reqInfo.filePath+File.separator+fileName;
        return true;
      } else {
        return false;
      }
    }
    String[] dirPath = reqInfo.filePath.split("/");
    try {
      ServerProjectFile dir =
        getServerProjectFile(reqInfo.ownerId, reqInfo.projectName, dirPath);
      if (dir == null) {
        return false;
      }
      dir.createFile(fileName, reqInfo.fileType, reqInfo.userId);
      /*Troca o caminho do diretorio, pelo do arquivo para que*/
      /*possa atualizar o estado na finalizacao do upload*/
      String fPath = "";
      String[] filePath = new String[dirPath.length+1];
      for (int i = 0; i < dirPath.length; i++) {
        fPath = fPath+dirPath[i]+"/";
        filePath[i] = dirPath[i];
      }
      fPath = fPath+fileName;
      filePath[filePath.length-1] = fileName;
      reqInfo.filePath = fPath;
      //Obtm o ServerProjectFile do arquivo criado
      ServerProjectFile spf =
        getServerProjectFile(reqInfo.ownerId, reqInfo.projectName, filePath);
      if (spf != null) {
        spf.setUnderConstruction(true);
      }
    }
    catch (Exception e) {
      logSevereMessage(
        "Falha na criacao do arquivo no upload! "+"("+fileName+") "+e);
      return false;
    }
    return true;
  }

  /**
   * Cria um arquivo vazio que no est na rea de projetos. Retorna true
   * se o arquivo j existir, ou se conseguir criar.
   */
  private boolean createFileOutProject(String filePath, String fileName){
    try {
      File file = new File(filePath+File.separator+fileName);
      if (file.exists()) return true;
      return file.createNewFile();
    }catch(Exception e){
      logSevereMessage("createFileOutProject: "+e.getMessage());
      return false;
    }
  }

  public String getFilePath(String accessCode)
  throws RemoteException {
    return getFilePath(accessCode, null);
  }

  private static String unixPath(String path) {
    return path.replaceAll("\\\\", "/");
  }

  public String getFilePath(String accessCode, String filePath)
  throws RemoteException {
    if (accessCode == null) {
      return null;
    }
    RequestInfo reqInfo = requestList.get(accessCode);
    if (reqInfo == null) {
      return null;
    }
    //Se no tem projeto associado retorna o prprio caminho.
    if (reqInfo.projectName == null) {
      return reqInfo.filePath;
    }
    String path[];
    if (filePath == null) {
      path = reqInfo.filePath.split("/");
    } else {
      String auxPath = expandPath(filePath);
      if (auxPath == null) return null;
      path = auxPath.split("/");
    }
    try {
      ServerProjectFile spf =
        getServerProjectFile(reqInfo.ownerId, reqInfo.projectName, path);
      if (spf == null) {
        String pathStr = Arrays.toString(path);
        throw new InvalidRequestException(
          getString(INVALID_REQUEST_MSG),
          "Caminho "+pathStr+" no encontrado na rvore do servidor");
      }
      return unixPath(spf.getCanonicalPath());
    }
    catch (Exception e) {
      logSevereMessage(
        "Falha obtencao do caminho do arquivo! "+"("+reqInfo.filePath+") "+e);
      return null;
    }
  }

  private String expandPath(String path) {
    String newPath = path.replaceAll("//", "/");
    do {
      path = newPath;
      newPath = newPath.replaceFirst("^/", "");
      if (newPath.startsWith("../")) return null;
      newPath = newPath.replaceFirst("[^/]+/\\.\\.", "");
      newPath = newPath.replaceFirst("//", "/");
    } while (!newPath.equals(path));
    if (newPath.equals("..")) return null;
    return newPath;
  }

  public String getPresentationPath(String accessCode)
  throws RemoteException {
    if (accessCode == null) {
      return null;
    }
    RequestInfo reqInfo = requestList.get(accessCode);
    if (reqInfo == null) {
      return null;
    }
    try {
      File presentationFile = new File(reqInfo.uploadPresentationPath);
      return unixPath(presentationFile.getCanonicalPath());
    }
    catch (Exception e) {
      logSevereMessage(
        "Falha na obteno do caminho do arquivo: "+
        reqInfo.uploadPresentationPath);
      return null;
    }
  }

  public String getResultPath(String accessCode)
  throws RemoteException {
    if (accessCode == null) {
      return null;
    }
    RequestInfo reqInfo = requestList.get(accessCode);
    if (reqInfo == null) {
      return null;
    }
    try {
      File resultFile = new File(reqInfo.uploadResultPath);
      return unixPath(resultFile.getCanonicalPath());
    }
    catch (Exception e) {
      logSevereMessage(
        "Falha na obteno do caminho do arquivo: "+reqInfo.uploadResultPath);
      return null;
    }
  }

  public boolean isDirectory(String accessCode)
  throws RemoteException {
    if (accessCode == null) {
      return false;
    }
    RequestInfo reqInfo = requestList.get(accessCode);
    if (reqInfo == null) {
      return false;
    }
    try {
      if (reqInfo.projectName == null) {
        File f = new File(reqInfo.filePath);
        return f.isDirectory();
      }
      String[] path = reqInfo.filePath.split("/");
      ServerProjectFile spf =
        getServerProjectFile(reqInfo.ownerId, reqInfo.projectName, path);
      if (spf == null) {
        String pathStr = Arrays.toString(path);
        throw new InvalidRequestException(
          getString(INVALID_REQUEST_MSG),
          "Caminho "+pathStr+" no encontrado na rvore do servidor");
      }
      return spf.isDirectory();
    }
    catch (Exception e) {
      logSevereMessage(
        "Falha no metodo isDirectory do upload! "+"("+reqInfo.filePath+") "+e);
      return false;
    }
  }

  /**
   * Remove o upload da lista de requisicoes e informa que o arquivo nao esta'
   * mais em construcao, se foi um upload de arquivo de projeto.
   * Notifica os observadores que o upload terminou.
   *
   * @param accessCode
   * @param fileName O nome original do arquivo que sofreu upload.
   *
   * @return boolean se a operacao foi realizada com sucesso.
   *
   * @throws HttpServiceException .
   */
  public boolean finishUpload(String accessCode, String fileName)
  throws HttpServiceException {
    if (accessCode == null) {
      return false;
    }
    RequestInfo reqInfo = requestList.get(accessCode);
    if (reqInfo == null) {
      return false;
    }
    if (reqInfo.observer != null) {
      UploadResult result = new UploadResult(accessCode, fileName,
        reqInfo.filePath);
      reqInfo.observer.update(observable, result);
    }
    else if (reqInfo.fileHandler != null) {
      reqInfo.fileHandler.execute(reqInfo.filePath);
    }
    if (reqInfo.projectName == null) {
      requestList.remove(accessCode);
      return true;
    }
    String[] path = reqInfo.filePath.split("/");
    try {
      ServerProjectFile spf =
        getServerProjectFile(reqInfo.ownerId, reqInfo.projectName, path);
      if (spf == null) {
        String pathStr = Arrays.toString(path);
        throw new InvalidRequestException(
          getString(INVALID_REQUEST_MSG),
          "Caminho "+pathStr+" no encontrado na rvore do servidor");
      }
      if (!spf.isDirectory()) {
        spf.setUnderConstruction(false);
      }
      requestList.remove(accessCode);
      return true;
    }
    catch (Exception e) {
      logSevereMessage("Falha na finalizacao do upload! "+"("+path+") "+e);
      return false;
    }
  }

  /**
   * Obtem o objeto no servidor representando o arquivo no caminho especificado.
   *
   * @param userId identificador do usurio.
   * @param projectName nome do projeto.
   * @param filePath caminho para o arquivo.
   *
   * @return arquivo encontrado, ou <code>null</code> caso contrrio.
   *
   * @throws ServerProjectFileException
   */
  private ServerProjectFile getServerProjectFile(
    Object userId, String projectName, String[] filePath)
  throws ServerProjectFileException {
    String projectId = ServerProject.getId(userId, projectName);
    if (projectId == null) {
      throw new InvalidRequestException(
        getString(INVALID_REQUEST_MSG),
        "Projeto no encontrado para os parmetros (userId="+userId+"), "+
        "(projectName="+projectName+")");
    }
    return ServerProjectFile.findFile(projectId, filePath);
  }

  /**
   * Transforma array de String em uma String separada por "/" e
   * codificada (UTF-8).
   *
   * @param filePath .
   *
   * @return .
   */
  private String filePathtoString(String[] filePath) {
    if (filePath == null) {
      return null;
    }
    if (filePath.length == 0) {
      return null;
    }
    String result = "";
    try {
      String pathEncoded;
      for (int i = 0; i < (filePath.length-1); i++) {
        pathEncoded = URLEncoder.encode(filePath[i], "UTF-8");
        result = result + pathEncoded + "/";
      }
      pathEncoded = URLEncoder.encode(filePath[filePath.length - 1], "UTF-8");
      result = result + pathEncoded;
    }
    catch (UnsupportedEncodingException e) {
      logSevereMessage("O caminho do arquivo nao pode ser codificado.");
      return null;
    }
    return result;
  }

  /**
   * Mtodo de busca do servio de projetos
   *
   * @param filePath .
   *
   * @return . protected ProjectService getProjectService() { try {
   *         ServiceManager serviceManager = ServiceManager.getInstance();
   *         ProjectService prjService =
   *         (ProjectService)serviceManager.getServiceByName(
   *         ProjectServiceInterface.SERVICE_NAME); return prjService; } catch
   *         (Exception e) { System.out.println( "Falha na busca do servio de
   *         projetos: " + e.getMessage()); e.printStackTrace(); return null;
   *         }}
   *
   * @throws RemoteException .
   */
  /**
   * Obtm uma URL para exibio de um arquivo do sistema de arquivos (NFS) em
   * um navegador web.
   *
   * @param filePath caminho absoluto do arquivo no sistema de arquivos.
   *
   * @return url para exibio do arquivo em um navegador.
   *
   * @throws RemoteException
   */
  public String getFileDownloadURL(String filePath)
  throws RemoteException {
    return getURL(
      filePath, DOWNLOAD, null, null, FILE_DOWNLOAD_TYPE, null, null, null, null,
      null, null, null);
  }

  /**
   * Obtm a URL para a exibio de um relatrio no formato pdf em um navegador
   * web.
   *
   * @param filePath caminho absoluto do arquivo .jasper que representa o
   *        layout do relatrio
   * @param parameters parmetros passados para o relatrio
   * @param jdbcDriverClassName nome da classe do driver jdbc para criao da
   *        conexo usada no relatrio
   * @param jdbcURL URL de acesso para criao da conexo usada no relatrio
   * @param jdbcUser usurio de acesso para criao da conexo usada no
   *        relatrio
   * @param jdbcPassword senha de acesso para criao da conexo usada no
   *        relatrio
   *
   * @return url para exibio do arquivo em um navegador.
   *
   * @throws RemoteException
   */
  public String getDownloadPDFReportURL(
    String filePath, Map parameters, String jdbcDriverClassName, String jdbcURL,
    String jdbcUser, String jdbcPassword)
  throws RemoteException {
    return getURL(
      filePath, DOWNLOAD, null, null, PDF_REPORT_DOWNLOAD_TYPE, null, parameters,
      jdbcDriverClassName, jdbcURL, jdbcUser, jdbcPassword, null);
  }

  public Map getParameters(String accessCode)
  throws RemoteException {
    if (accessCode == null) {
      return null;
    }
    RequestInfo reqInfo = requestList.get(accessCode);
    if (reqInfo == null) {
      return null;
    }
    return reqInfo.parameters;
  }

  public String getJdbcDriverClassName(String accessCode)
  throws RemoteException {
    if (accessCode == null) {
      return null;
    }
    RequestInfo reqInfo = requestList.get(accessCode);
    if (reqInfo == null) {
      return null;
    }
    return reqInfo.jdbcDriverClassName;
  }

  public String getJdbcURL(String accessCode)
  throws RemoteException {
    if (accessCode == null) {
      return null;
    }
    RequestInfo reqInfo = requestList.get(accessCode);
    if (reqInfo == null) {
      return null;
    }
    return reqInfo.jdbcURL;
  }

  public String getJdbcUser(String accessCode)
  throws RemoteException {
    if (accessCode == null) {
      return null;
    }
    RequestInfo reqInfo = requestList.get(accessCode);
    if (reqInfo == null) {
      return null;
    }
    return reqInfo.jdbcUser;
  }

  public String getJdbcPassword(String accessCode)
  throws RemoteException {
    if (accessCode == null) {
      return null;
    }
    RequestInfo reqInfo = requestList.get(accessCode);
    if (reqInfo == null) {
      return null;
    }
    return reqInfo.jdbcPassword;
  }

  //
  // Mtodos relativos ao download e upload de arquivos do VGE e aplicaes
  // que no dependem de projeto.
  //
  public String getUploadURL(
    String filePath, String presentationPath, String resultPath)
  throws RemoteException {
    return getURL(filePath, UPLOAD, presentationPath, resultPath, null);
  }

  public String getUploadURL(
    String filePath, String presentationPath, String resultPath, UploadHandler fileHandler) {
    if (filePath == null) {
      throw new IllegalArgumentException("O parmetro filePath est nulo.");
    }
    if (presentationPath == null) {
      throw new IllegalArgumentException("O parmetro presentationPath est nulo.");
    }
    if (resultPath == null) {
      throw new IllegalArgumentException("O parmetro resultPath est nulo.");
    }
    if (fileHandler == null) {
      throw new IllegalArgumentException("O parmetro fileHandler est nulo.");
    }
    if (httpServiceURL == null) {
      throw new IllegalStateException("O parmetro httpServiceURL est nulo.");
    }
    /*Verifica se existe codigo para o arquivo*/
    String accessCode = getAccessCode(filePath);
    if (accessCode == null) {
      accessCode = ""+Math.abs(numberGenerator.nextLong());
      accessCode = accessCode+Math.abs(numberGenerator.nextLong());
    }
    /*adiciona a lista de requisicoes*/
    RequestInfo req = new RequestInfo();
    req.ownerId = Service.getUser();
    req.userId = Service.getUser();
    req.downloadType = URL_DOWNLOAD_TYPE;
    req.filePath = filePath;
    req.removeFile = true;
    req.uploadPresentationPath = presentationPath;
    req.uploadResultPath = resultPath;
    req.fileHandler = fileHandler;
    req.whenDone = System.currentTimeMillis();
    requestList.put(accessCode, req);
    String url = httpServiceURL + "/" + UPLOAD + "/" + accessCode + "/"
      + UPLOAD;
    return url;
  }

  public String getUploadURL(
    String filePath, String presentationPath, String resultPath, Observer obs) {
    return getURL(filePath, UPLOAD, presentationPath, resultPath, obs);
  }

  public String getDownloadURL(String filePath)
  throws RemoteException {
    return getURL(filePath, DOWNLOAD, null, null, null);
  }

  public String getDownloadURL(String filePath, Map<String, String> parameters)
  throws RemoteException {
    return getURL(filePath, DOWNLOAD, null, null, null, parameters);
  }

  public String getDownloadTextURL(Object userId, String password, String text)
  throws RemoteException {
    //Usurio que fez a chamada ao mtodo. Usar para a autenticao.
    User user = LoginService.getInstance().checkLogin((String)userId, password);
    if (user == null) {
      throw new RemoteException(
        "Falha na autenticao do usurio. \n"+
        "Verifique se o login e a senha so vlidos");
    }
    return getDownloadTextURL(text);
  }

  public String getDownloadTextURL(String text)
  throws RemoteException {
    //s para no passar nulo e ser nico
    String filePath = "/"+Math.abs(numberGenerator.nextLong())+"/"+"TEXT.txt";
    return getURL(
      filePath, DOWNLOAD, null, null, TEXT_DOWNLOAD_TYPE, text, null, null, null,
      null, null, null);
  }

  public String getDownloadTextURL(Object userId, String password, String text,
      String fileName) throws RemoteException {
    //Usurio que fez a chamada ao mtodo. Usar para a autenticao.
    User user = LoginService.getInstance().checkLogin((String)userId, password);
    if (user == null) {
      throw new RemoteException(
        "Falha na autenticao do usurio. \n"+
        "Verifique se o login e a senha so vlidos");
    }
    return getDownloadTextURL(text, fileName);
  }

  private String getDownloadTextURL(String text, String fileName)
  throws RemoteException {
    //s para no passar nulo e ser nico
    String filePath = "/"+Math.abs(numberGenerator.nextLong())+"/"+fileName;
    return getURL(
      filePath, DOWNLOAD, null, null, TEXT_DOWNLOAD_TYPE, text, null, null, null,
      null, null, null);
  }



  /**
   * Pega a URL que ser chamada pelo browser quando o tipo de dowload for
   * CSV.
   *
   * @param userId identificao do usurio.
   * @param password senha do usurio.
   * @param text texto contendo o contedo do CSV.
   *
   * @return URL de download do arquivo CSV.
   *
   * @throws RemoteException
   */
  public String getDownloadCSVURL(Object userId, String password, String text)
  throws RemoteException {
    return getDownloadCSVURL(userId, password, text, "TEXT");
  }

  /**
   * Pega a URL que ser chamada pelo browser quando o tipo de dowload for
   * CSV. OBS: Esse mtodo no  exportado na interface remota, s pode ser
   * chamada de outro servio.
   *
   * @param text texto contendo o contedo do CSV.
   *
   * @return URL de download do arquivo CSV.
   *
   * @throws RemoteException
   */
  public String getDownloadCSVURL(String text)
  throws RemoteException {
    return getDownloadCSVURL(text,"TEXT");
  }

  /**
   * Pega a URL que ser chamada pelo browser quando o tipo de dowload for
   * CSV. OBS: Esse mtodo no  exportado na interface remota, s pode ser
   * chamada de outro servio.
   *
   * @param text texto contendo o contedo do CSV.
   * @param fileName nome do arquivo gerado para download
   *
   * @return URL de download do arquivo CSV.
   *
   * @throws RemoteException
   */
  public String getDownloadCSVURL(String text, String fileName)
  throws RemoteException {
    String filePath =
      "/"+Math.abs(numberGenerator.nextLong())+"/"+fileName+".csv";
    return getURL(filePath, DOWNLOAD, null, null, CSV_DOWNLOAD_TYPE, text,
      null, null, null,null, null, null);
  }

  public String getText(String accessCode)
  throws RemoteException {
    if (accessCode == null) {
      return null;
    }
    RequestInfo reqInfo = requestList.get(accessCode);
    if (reqInfo == null) {
      return null;
    }
    return reqInfo.text;
  }

  /**
   * Obtem a url atraves da qual o arquivo deve ser acessado.
   *
   * @param filePath caminho do arquivo local no projeto.
   * @param type se e' upload ou download
   * @param presentationPath Path do arquivo html para upload.
   * @param resultPath Path do arquivo html resultado do upload.
   * @param downloadType tipo de download.
   * @param text texto que ser exibido no navegador caso o
   *        <code>downloadType</code> seja
   *        <code>HttpServiceInterface.TEXT_DOWNLOAD_TYPE</code>.
   * @param parameters parmetros que sero armazenados na RequestInfo para uso
   *        geral.
   * @param jdbcDriverClassName nome da classe do driver JDBC para a criao de
   *        uma eventual conexo com um banco de dados a partir do
   *        <code>DownloadServlet</code>.
   * @param jdbcURL URL JDBC para a criao de uma eventual conexo com um
   *        banco de dados a partir do <code>DownloadServlet</code>.
   * @param jdbcUser usurio JDBC para a criao de uma eventual conexo com um
   *        banco de dados a partir do <code>DownloadServlet</code>.
   * @param jdbcPassword senha JDBC para a criao de uma eventual conexo com
   *        um banco de dados a partir do <code>DownloadServlet</code>.
   * @param observer Observador que e' notificado quando o upload termina.
   *
   * @return String que representa a url do arquivo
   *
   * @throws RemoteException .
   */
  private String getURL(
    String filePath, String type, String presentationPath, String resultPath,
    int downloadType, String text, Map parameters, String jdbcDriverClassName,
    String jdbcURL, String jdbcUser, String jdbcPassword, Observer observer) {
    if ((filePath == null) || (type == null)) {
      return null;
    }
    if (httpServiceURL == null) {
      return null;
    }
    /*Verifica se existe codigo para o arquivo*/
    String accessCode = getAccessCode(filePath);
    if (accessCode == null) {
      accessCode = ""+Math.abs(numberGenerator.nextLong());
      accessCode = accessCode+Math.abs(numberGenerator.nextLong());
    }
    /*adiciona a lista de requisicoes*/
    RequestInfo req = new RequestInfo();
    req.ownerId = Service.getUser();
    req.userId = Service.getUser();
    req.downloadType = downloadType;
    req.text = text;
    req.filePath = filePath;
    String fileName = null;
    if (type.equals(UPLOAD)) {
      req.removeFile = true;
      req.uploadPresentationPath = presentationPath;
      req.uploadResultPath = resultPath;
      req.observer = observer;
    }
    else {
      // Obtm o nome do arquivo para incluir na url de download
      String[] path = filePath.split("/");
      if (path == null) {
        fileName = filePath;
      }
      else {
        fileName = path[path.length-1];
      }
      fileName = encode(fileName);

      //parametros da RequestInfo para gerao de relatrios em pdf
      req.parameters = parameters; //parametros passados para o relatrio
      req.jdbcDriverClassName = jdbcDriverClassName;
      req.jdbcURL = jdbcURL;
      req.jdbcUser = jdbcUser;
      req.jdbcPassword = jdbcPassword;
    }
    req.whenDone = System.currentTimeMillis();
    requestList.put(accessCode, req);
    String url;
    if (fileName == null) {
      url = httpServiceURL+"/"+type+"/"+accessCode+"/"+type;
    }
    else {
      url = httpServiceURL+"/"+type+"/"+accessCode+"/"+type+"/"+fileName;
    }
    logInfoMessage("URL: " + url + " - filePath: " + req.filePath +
      " - user: " + Service.getUser().getLogin());
    return url;
  }

  private String getURL(
    String filePath, String type, String presentationPath, String resultPath,
    Observer obs) {
    return getURL(
      filePath, type, presentationPath, resultPath, URL_DOWNLOAD_TYPE, null,
      null, null, null, null, null, obs);
  }

  private String getURL(
    String filePath, String type, String presentationPath, String resultPath,
    Observer obs, Map<String, String> parameters)
  throws RemoteException {
    return getURL(
      filePath, type, presentationPath, resultPath, URL_DOWNLOAD_TYPE, null,
      parameters, null, null, null, null, obs);
  }


  /**
   * Verifica se jah exite codigo para o arquivo.
   *
   * @param filePath .
   *
   * @return codigo de acesso, caso exista. Null caso contrario.
   */
  private String getAccessCode(String filePath) {
    for (Enumeration e = requestList.keys(); e.hasMoreElements();) {
      String code = (String)e.nextElement();
      String fileTemp = requestList.get(code).filePath;
      if (filePath.equals(fileTemp)) {
        return code;
      }
    }
    return null;
  }

  //
  // Fim dos mtodos relativos ao download e upload de arquivos do VGE.
  //
  @Override
  protected boolean has2Update(Object arg, Object event) {
    return true;
  }

  /**
   * Trmino do servio.
   *
   * @throws ServerException .
   */
  @Override
  public void shutdownService()
  throws ServerException {
    shutdown = true;
    if (cleaner != null) {
      cleaner.interrupt();
    }
    requestList = null;
    logInfoMessage("Terminando servio de httpservice...");
  }

  /**
   * Inicializao do servio.
   *
   * @throws ServerException se ocorrer um erro na inicializao
   */
  @Override
  public void initService()
  throws ServerException {
    logDebugMessage("Iniciando servio de httpservice...");
    requestList = new Hashtable<String, RequestInfo>();
    httpServiceURL = getStringProperty("webapp");

    /*Criando o gerador de numeros magico*/
    double seed = Math.random();
    long longSeed = (long)(seed*Long.MAX_VALUE);
    numberGenerator = new Random(longSeed);

    /*Obtem as propriedades para a thread de limpeza*/
    this.shutdown = false;
    float maxSaveTime = (float) getDoubleProperty("maxSaveTime");
    float cleanTime = (float) getDoubleProperty("cleanTime");
    cleaner = new Cleaner(maxSaveTime, cleanTime);
    logInfoMessage("Iniciado o httpservice.");
  }

  public static HttpService getInstance() {
    return (HttpService)getInstance(SERVICE_NAME);
  }

  /**
   * Limpa a fila de requisies, retirando entradas relativas as requisies
   * atendidas a mais de <code>maxSaveTime</code> dias. O thread criado pelo
   * objeto desta classe faz a limpeza na fila a cada <code>cleanTime</code>
   * dias. Tanto <code>maxSaveTime</code> quanto <code>cleanTime</code> so
   * propriedades do servio. O loop pode ser interrompido atravs do campo
   * <code>shutdown</code>.
   */
  private class Cleaner extends Thread {
    private static final long DAY_IN_MS = 1000*60*60*24;
    private final float maxSaveTime;
    private final float cleanTime;

    @Override
    public void run() {
      while (!shutdown) {
        try {
          sleep((long)(cleanTime*DAY_IN_MS));
        }
        catch (InterruptedException e) {}
        if (shutdown) {
          break;
        }
        logInfoMessage("Thread iniciada: cleaner");
        long now = System.currentTimeMillis();
        for (Enumeration e = requestList.keys(); e.hasMoreElements();) {
          String key = (String)e.nextElement();
          RequestInfo reqInfo = requestList.get(key);
          if (
            (reqInfo.whenDone > 0) &&
            (reqInfo.whenDone < (now-(maxSaveTime*DAY_IN_MS)))) {
            requestList.remove(key);
            if (reqInfo.removeFile) {
              (new File(reqInfo.filePath)).delete();
            }
          }
        }
        logInfoMessage("Thread terminada: cleaner");
      }
    }

    public Cleaner(float maxSaveTime, float cleanTime) {
      this.maxSaveTime = maxSaveTime;
      this.cleanTime = cleanTime;
      setDaemon(true);
      this.start();
    }
  }

  /**
   * Pega a URL que ser chamada pelo browser quando o tipo de dowload for
   * CSV.
   *
   * @param userId identificao do usurio.
   * @param password senha do usurio.
   * @param text texto contendo o contedo do CSV.
   * @param fileName nome do arquivo que ser gerado
   *
   * @return URL de download do arquivo CSV.
   *
   * @throws RemoteException
   */
  public String getDownloadCSVURL(Object userId, String password, String text, String fileName)
  throws RemoteException {
    //Usurio que fez a chamada ao mtodo. Usar para a autenticao.
    User user = LoginService.getInstance().checkLogin((String)userId, password);
    if (user == null) {
      throw new RemoteException(
        "Falha na autenticao do usurio. \n"+
        "Verifique se o login e a senha so vlidos");
    }
    return getDownloadCSVURL(text,fileName);
  }




}
