/*
 * $Id: MailService.java 95933 2009-09-08 19:18:03Z mjulia $
 */
package csbase.server.services.mailservice;

import java.util.ArrayList;
import java.util.Date;
import java.util.Properties;
import java.util.regex.Pattern;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import csbase.logic.User;
import csbase.remote.MailServiceInterface;
import csbase.server.Server;
import csbase.server.ServerException;
import csbase.server.Service;
import csbase.server.services.projectservice.ServerProject;
import csbase.server.services.sgaservice.SGAService;

/**
 * A classe <code>MailService</code> implementa o servio de envio e-mails a
 * partir do servidor.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class MailService extends Service implements MailServiceInterface {

  /**
   * Pattern representando um e-mail: <qualquer coisa diferente de
   * espao>@<qualquer coisa diferente de espao>
   */
  private static final Pattern _MAIL_PTTN =
    Pattern.compile("^[^\\s@]+@[^\\s@]+$");

  /** Porta default */
  private static final int SMTP_DEFAULT_PORT = 25;

  /** Email default de suporte */
  private static final String DEFAULT_SUPPORT_EMAIL =
    "csbase@tecgraf.puc-rio.br";

  /**
   * Sesso junto ao servidor de mails que  definido com base na propriedade
   * ajustada pelas propriedades do servio.
   * 
   * @see #createSession()
   */
  private Session session = null;

  /**
   * Consulta ao email do destinatrio
   * 
   * @param userId o identificador do usurio
   * @return o email do usurio
   */
  private String getUserMail(final Object userId) {
    try {
      if (userId == null) {
        logSevereMessage("Detectado id de usurio nulo.");
        return null;
      }
      final User user = User.getUser(userId);
      if (user == null) {
        logSevereMessage("Detectado usurio nulo para: " + userId);
        return null;
      }
      return user.getMail();
    }
    catch (Exception ue) {
      logSevereMessage("Falha na deteco de email do usurio " + userId + "!");
      return null;
    }
  }

  /**
   * Consulta ao email do projeto envolvido
   * 
   * @param projectId o identificador do projeto
   * @return o email do usurio
   */
  private String getProjectMail(final Object projectId) {
    final ServerProject project = ServerProject.getProject(projectId);
    if (projectId == null) {
      logSevereMessage("Detectado id de projeto nulo.");
      return null;
    }
    if (project == null) {
      logSevereMessage("Detectado projeto nulo para: " + projectId);
      return null;
    }
    try {
      final User user = User.getUser(project.getUserId());
      if (user == null) {
        logSevereMessage("Detectado usurio nulo para projeto: " + projectId);
        return null;
      }
      return user.getMail();
    }
    catch (Exception ue) {
      logSevereMessage("Falha na deteco de email de usurio destino!");
    }
    return null;
  }

  /**
   * Criao da sesso no servidor de mail.
   * 
   * @return <code>true</code> se a sesso foi criada com sucesso
   * 
   * @see #session
   * @return indicativo de sucesso.
   */
  private boolean createSession() {
    final Properties props = new Properties();
    final String hostNameProp = "mail.smtp.host";
    final String hostPortProp = "mail.smtp.port";

    final String mailHostName = getStringProperty(hostNameProp);
    final int mailHostPort = getIntProperty(hostPortProp);

    props.put(hostNameProp, mailHostName);
    props.put(hostPortProp, mailHostPort);
    final String fmt = "Servidor de mail - %s:%d";
    final String msg = String.format(fmt, mailHostName, mailHostPort);
    logInfoMessage(msg);

    session = Session.getDefaultInstance(props, null);
    if (session == null) {
      logSevereMessage("No foi possvel instanciar uma sesso de e-mail");
      return false;
    }
    return true;
  }

  /**
   * Mtodo de acesso ao servidor de emails
   * 
   * @param from o remetente
   * @param addressee o destinatrio
   * @param subject o assunto
   * @param content a string enviada
   * @param replyTo endereo para resposta
   * @param copyTo destinatrios para envio de cpias da mensagem (campo "cc:")
   * 
   * @return flag indicativo de sucesso.
   */
  private boolean accessMailServer(final String from, final String addressee,
    final String subject, final String content, final String replyTo,
    final String[] copyTo) {
    if (session == null) {
      return false;
    }

    String sender = createValidSender(from);
    try {
      final Message msg = new MimeMessage(session);
      msg.setFrom(new InternetAddress(sender));

      final InternetAddress[] address = { new InternetAddress(addressee) };
      msg.setRecipients(Message.RecipientType.TO, address);
      msg.setSubject(subject);
      msg.setSentDate(new Date());
      msg.setText(content);

      if (replyTo != null && !replyTo.trim().isEmpty()) {
        final InternetAddress[] replies = { new InternetAddress(replyTo) };
        msg.setReplyTo(replies);
      }

      if (copyTo != null && copyTo.length > 0) {
        final ArrayList<InternetAddress> addresses =
          new ArrayList<InternetAddress>();
        for (int i = 0; i < copyTo.length; i++) {
          if (!copyTo[i].trim().isEmpty()) {
            addresses.add(new InternetAddress(copyTo[i]));
          }
        }
        final InternetAddress[] copies =
          addresses.toArray(new InternetAddress[addresses.size()]);
        msg.setRecipients(Message.RecipientType.CC, copies);
      }
      Transport.send(msg);
    }
    catch (MessagingException mex) {
      final String msg = "Falha no envio de email para [" + addressee + "]! ";
      final String err = "Exceo no servidor: " + mex + "!";
      logSevereMessage(msg + err);
      return false;
    }
    return true;
  }

  /**
   * Verifica se o remetente  vlido, se no cria um utilizando o parmetro
   * passado.<br/> Caso o parmetro passado seja nulo, o remetente default ser
   * utilizado.<br/> Caso o remetente esteja de acordo com a expresso regular <code>'^[^\\s@]+@[^\\s@]+$'</code>
   * , este  retornado, caso contrrio,  criado um e-mail vlido usando o
   * parmetro passado e o remetente default.<br/> Ex.<br/> seed =
   * "infogrid"<br/> remetente default = "default@tecgraf.puc-rio.br"<br/>
   * remetente final = "infogrid@no-reply.tecgraf.puc-rio.br"<br/>
   * 
   * @param seed remetente vlido ou uma semente para ser utilizada na criao
   *        de um vlido.
   * 
   * @return um remetente vlido.
   */
  private String createValidSender(String seed) {
    if (null == seed) {
      seed = makeDefaultSender();
    }
    if (_MAIL_PTTN.matcher(seed).matches()) {
      return seed;
    }

    // Alguns servidores de smtp checam se o e-mail do remetente est bem 
    // formatado. Caso ele no esteja, criaremos um para ele baseado no 
    // remetente passado como parmetro.
    return String.format("%1$s@no-reply.%2$s", seed.split("@")[0],
      makeDefaultSender().split("@")[1]);
  }

  /**
   * Busca da propriedade de e-mail de suporte.
   * 
   * @return o e-mail
   * @see #DEFAULT_SUPPORT_EMAIL
   */
  private String makeSupportTo() {
    return getStringProperty("support");
  }

  /**
   * Busca da propriedade de remetente de e-mail de suporte.
   * 
   * @return o e-mail
   * @see #DEFAULT_SUPPORT_EMAIL
   */
  private String makeDefaultSender() {
    return getStringProperty("from");
  }

  /**
   * Montagem de <i>subject</i> para e-mail.
   * 
   * @param sender remetente
   * 
   * @return o texto.
   */
  private String makeSubject(final String sender) {
    return Server.getInstance().getServerName() + " - " + sender;
  }

  /**
   * Mtodo de envio de email.
   * 
   * @param sender o remetente
   * @param addressee o destinatrio
   * @param content a string enviada
   * 
   * @return flag indicativo de sucesso.
   */
  final public boolean sendMail(final String sender, final String addressee,
    final String content) {
    final String subject = makeSubject(sender);
    return sendMail(sender, addressee, content, subject, null, null);
  }

  /**
   * Mtodo de envio de email com flexibilidade para customizar o assunto, o
   * campo de resposta (reply) e o campo de copia para (cc).
   * 
   * @param sender o remetente
   * @param addressee o destinatrio
   * @param content a string enviada
   * @param subject o assunto do email
   * @param replyTo o endereo que ir receber resposta (reply)
   * @param copyTo os endereos que iro receber uma cpia da mensagem
   * 
   * @return flag indicativo de sucesso.
   */
  final public boolean sendMail(final String sender, final String addressee,
    final String content, final String subject, final String replyTo,
    final String[] copyTo) {
    final String cnt = (content == null ? getString("no.content") : content);
    final String snd = (sender == null ? makeDefaultSender() : sender);
    final String sub = (subject == null ? makeSubject(sender) : subject);
    return accessMailServer(snd, addressee, sub, cnt, replyTo, copyTo);
  }

  /**
   * Manda um email para um projeto
   * 
   * @param sender a mensagem.
   * @param prjId o identificador do projeto destinatrio.
   * @param content a mensagem.
   * 
   * @return um indicativo de sucesso.
   */
  private boolean mailProject(final String sender, final Object prjId,
    final String content) {
    final String addressee = getProjectMail(prjId);
    if (addressee == null) {
      logSevereMessage("Projeto [" + prjId + "] no tem email relacionado!");
      return false;
    }
    return sendMail(sender, addressee, content);
  }

  /**
   * Manda um mail de um usurio para outro usurio.
   * 
   * @param sender o nome remetente.
   * @param userId o identificador do remetente.
   * @param content a mensagem.
   * 
   * @return um indicativo de sucesso.
   */
  final public boolean mailTo(final String sender, final Object userId,
    final String content) {
    return mailUser(sender, userId, content);
  }

  /**
   * Manda um mail para o suporte
   * 
   * @param content a mensagem.
   * 
   * @return um flag indicativo de sucesso..
   */
  final public boolean mailSupport(final String content) {
    return sendMail(makeDefaultSender(), makeSupportTo(), content);
  }

  /**
   * Manda um email para um usurio
   * 
   * @param sender a mensagem.
   * @param userId o identificador do usurio destinatrio.
   * @param content a mensagem.
   * 
   * @return um indicativo de sucesso.
   */
  private boolean mailUser(final String sender, final Object userId,
    final String content) {
    final String addressee = getUserMail(userId);
    if (addressee == null) {
      logSevereMessage("Usurio [" + userId + "] no tem email relacionado!");
      return false;
    }
    return sendMail(sender, addressee, content);
  }

  /**
   * Notifica alguns os usurios.
   * 
   * @param sender o nome remetente.
   * @param usersId um array de usurios
   * @param content a mensagem.
   * 
   * @return um indicativo de sucesso.
   */
  final public boolean mailSomeUsers(final String sender,
    final Object[] usersId, final String content) {
    boolean flag = true;
    if (usersId == null) {
      return true;
    }
    for (int i = 0; i < usersId.length; i++) {
      final Object id = usersId[i];
      flag &= mailUser(sender, id, content);
    }
    return flag;
  }

  /**
   * Notifica um conjunto usurios atravs de um servio
   * 
   * @param service o servio remetente.
   * @param usersId array de identificadores dos destinatrios.
   * @param content a mensagem.
   * 
   * @return flag indicativo de sucesso no envio.
   */
  final public boolean mailSomeUsersFromService(final Service service,
    final Object[] usersId, final String content) {
    if (service == null) {
      logSevereMessage("Servio no identificado tentou enviar email!");
      return false;
    }

    String formatedContent = formatServiceMailContent(service, content);
    return mailSomeUsers(makeDefaultSender(), usersId, formatedContent);
  }

  /**
   * Notifica um usurio atravs de um servio
   * 
   * @param service o servio remetente.
   * @param userId o identificador do usurio destinatrio.
   * @param content a mensagem.
   * 
   * @return flag indicativo de sucesso no envio.
   */
  final public boolean mailUserFromService(final Service service,
    final Object userId, final String content) {
    if (service == null) {
      logSevereMessage("Servio no identificado tentou enviar email!");
      return false;
    }
    String formatedContent = formatServiceMailContent(service, content);
    return mailUser(makeDefaultSender(), userId, formatedContent);
  }
  
  /**
   * Notifica um endereo de email atravs de um servio
   * 
   * @param service o servio remetente.
   * @param address o endereo de email destinatrio.
   * @param content a mensagem.
   * 
   * @return flag indicativo de sucesso no envio.
   */
  final public boolean mailFromService(SGAService service, String address,
    String content) {
    if (service == null) {
      logSevereMessage("Servio no identificado tentou enviar email!");
      return false;
    }
    String formatedContent = formatServiceMailContent(service, content);
    return sendMail(makeDefaultSender(), address, formatedContent);
  }

  /**
   * Notifica um projeto atravs de um servio
   * 
   * @param service o servio remetente.
   * @param projectId o identificador do projeto destinatrio.
   * @param content a mensagem.
   * 
   * @return flag indicativo de sucesso no envio.
   */
  final public boolean mailProjectFromService(final Service service,
    final Object projectId, final String content) {
    if (service == null) {
      logSevereMessage("Servio no identificado tentou enviar email!");
      return false;
    }
    final String srvName = service.getName();
    return mailProject(srvName, projectId, content);
  }

  /**
   * Formata o contedo de um e-mail cujo remetente  um servio. Adiciona um
   * cabealho no e-mail indicando qual servio o gerou e de qual servidor.
   * 
   * @param service remetente
   * @param content contedo ainda no formatado do e-mail
   * @return o contedo do e-mail formatado para ser enviado.
   */
  private String formatServiceMailContent(final Service service,
    final String content) {
    String contentFormat = getString("MailService.services.mail.content");
    String reformatedContent =
      String.format(contentFormat, Server.getInstance().getServerName(),
        service.getName(), content);
    return reformatedContent;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  final protected boolean has2Update(final Object arg, final Object event) {
    return true;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  final public void shutdownService() throws ServerException {
    logInfoMessage("Terminando servio de e-mails...");
  }

  /**
   * {@inheritDoc}
   */
  @Override
  final public void initService() throws ServerException {
    logDebugMessage("Iniciando servio de emails...");
    logInfoMessage("Endereo remetente: " + makeDefaultSender());
    if (!createSession()) {
      throw new ServerException("Falha na construo da sesso de mail!");
    }
    logInfoMessage("Iniciado o servio de emails.");
  }

  /**
   * Busca do objeto servio de e-mail no servidor
   * 
   * @return o objeto <code>MailService</code>
   */
  public static MailService getInstance() {
    return (MailService) getInstance(SERVICE_NAME);
  }

  /**
   * Constri a instncia do servio.
   * 
   * @throws ServerException se houver falha.
   */
  final public static void createService() throws ServerException {
    new MailService();
  }

  /**
   * Construtor padro.
   * 
   * @throws ServerException se houver falha.
   * 
   * @see Service#Service(String)
   */
  protected MailService() throws ServerException {
    super(SERVICE_NAME);
  }

}
