/*
 * $Id: NotificationService.java 102454 2010-03-03 19:11:13Z clinio $
 */

package csbase.server.services.notificationservice;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.ObjectInputStream;
import java.io.InvalidClassException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collection;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingDeque;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

import csbase.logic.Notification;
import csbase.logic.NotificationData;
import csbase.logic.User;
import csbase.logic.UserNotificationData;
import csbase.remote.NotificationServiceInterface;
import csbase.remote.RemoteObserver;
import csbase.server.Server;
import csbase.server.ServerException;
import csbase.server.Service;

/**
 * Fbrica de threads para o executor de updates do servio de notificao
 * 
 * @author Tecgraf/PUC-Rio
 */
class NotificationServiceThreadFactory implements ThreadFactory {
  /**
   * Constri a nova thread
   * 
   * @param r o runnable
   * @return a thread.
   */
  public Thread newThread(Runnable r) {
    final Thread thread = new Thread(r);
    thread.setName("NotificationService::executor::" + thread.hashCode());
    return thread;
  }
}

/**
 * Executor de updates do servio de notificao
 * 
 * @author Tecgraf/PUC-Rio
 */
class NotificationServiceExecutor extends ThreadPoolExecutor {
  /**
   * Indicativo de coleta de estatsticas
   */
  private boolean gatherStats;

  /**
   * Mapa de durao de execuo dos runnables.
   */
  private Map<Runnable, Long> durations;

  /**
   * Tempo mximo de execuo de um runnable (ponderado por seu peso).
   */
  private long weightedMaxRuntime;

  /**
   * Tempo mnimo de execuo de um runnable (ponderado por seu peso).
   */
  private long weightedMinRuntime;

  /**
   * Tempo mximo absoluto de execuo de um runnable.
   */
  private long absoluteMaxRuntime;

  /**
   * Tempo mnimo absoluto de execuo de um runnable.
   */
  private long absoluteMinRuntime;

  /**
   * Nmero de execues
   */
  private AtomicInteger executions;

  /**
   * Nmero de execues com falha.
   */
  private AtomicInteger faults;

  /**
   * Campo que representa o peso da execuo.
   */
  static private ThreadLocal<Integer> executionWeight =
    new ThreadLocal<Integer>();

  /**
   * Ajuste do peso da execuo.
   * 
   * @param weight o peso
   */
  void setExecutionWeight(int weight) {
    executionWeight.set(weight);
  }

  /**
   * Consulta do peso mximo ponderado das execues
   * 
   * @return o peso.
   */
  synchronized long getWeightedMaxRuntime() {
    return weightedMaxRuntime;
  }

  /**
   * Consulta do peso mnimo ponderado das execues
   * 
   * @return o peso.
   */
  synchronized long getWeightedMinRuntime() {
    return weightedMinRuntime;
  }

  /**
   * Consulta do peso mximo absoluto das execues
   * 
   * @return o peso.
   */
  synchronized long getAbsoluteMaxRuntime() {
    return absoluteMaxRuntime;
  }

  /**
   * Consulta do peso mnimo absoluto das execues
   * 
   * @return o peso.
   */
  synchronized long getAbsoluteMinRuntime() {
    return absoluteMinRuntime;
  }

  /**
   * Consulta o nmero de execues da estatstica
   * 
   * @return a quantidade de execues.
   */
  int getExecutions() {
    return executions.intValue();
  }

  /**
   * Consulta o nmero de falhas da estatstica.
   * 
   * @return o nmero de falhas
   */
  int getFaults() {
    return faults.intValue();
  }

  /**
   * Construtor
   * 
   * @see ThreadPoolExecutor#ThreadPoolExecutor(int, int, long, TimeUnit,
   *      BlockingQueue, ThreadFactory)
   * 
   * @param gatherStats indicativo de coleta de estatsca
   * @param corePoolSize tamanho padro do pool de threads
   * @param maximumPoolSize tamanho mximo do pool de threads
   * @param keepAliveTime tempo de vida idle de uma thread
   * @param unit unidade de tempo de {@code keepAliveTime}
   * @param workQueue fila de tarefas.
   * @param threadFactory fbrica de threads.
   */
  NotificationServiceExecutor(boolean gatherStats, int corePoolSize,
    int maximumPoolSize, long keepAliveTime, TimeUnit unit,
    BlockingQueue<Runnable> workQueue, ThreadFactory threadFactory) {
    super(corePoolSize, maximumPoolSize, keepAliveTime, unit, workQueue,
      threadFactory);
    this.gatherStats = gatherStats;
    reset();
  }

  /**
   * Reset dos valores de estatstica
   */
  void reset() {
    durations = new Hashtable<Runnable, Long>();
    weightedMaxRuntime = 0;
    weightedMinRuntime = Long.MAX_VALUE;
    absoluteMaxRuntime = 0;
    absoluteMinRuntime = Long.MAX_VALUE;
    executions = new AtomicInteger(0);
    faults = new AtomicInteger(0);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void beforeExecute(Thread t, Runnable r) {
    if (gatherStats) {
      executionWeight.set(1);
      durations.put(r, System.currentTimeMillis());
    }
    super.beforeExecute(t, r);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void afterExecute(Runnable r, Throwable t) {
    if (gatherStats) {
      Long t0 = durations.remove(r);
      if (t0 == null) {
        faults.incrementAndGet();
      }
      else {
        long dt = System.currentTimeMillis() - t0;
        int weight = executionWeight.get();
        synchronized (this) {
          weightedMaxRuntime = Math.max(weightedMaxRuntime, dt / weight);
          weightedMinRuntime = Math.min(weightedMinRuntime, dt / weight);
          absoluteMaxRuntime = Math.max(absoluteMaxRuntime, dt);
          absoluteMinRuntime = Math.min(absoluteMinRuntime, dt);
        }
      }
      executions.incrementAndGet();
    }
    super.afterExecute(r, t);
  }
}

/**
 * A classe <code>NotificationService</code> implementa o servio de notificao
 * de mensagens.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class NotificationService extends Service implements
  NotificationServiceInterface {

  /**
   * Tabela de mensagens armazenadas.
   */
  private NotificationContainer container = null;

  /**
   * Flag indicativo de sada da thread de limpeza de notificaes velhas do
   * servidor.
   */
  private boolean exitCleanThread = false;

  /**
   * Thread que faz a limpeza das notificaes velhas
   */
  private Thread cleanThread = null;

  /**
   * Flag indicativo de sada da thread de gravao de backup de notificaes do
   * servidor.
   */
  private boolean exitBackupThread = false;

  /**
   * Thread que faz a gravao dos backups das notificaes
   */
  private Thread backupThread = null;

  /**
   * Flag indicativo de sada da thread de gerao de estatsticas de uso das
   * threads de notificao.
   */
  private boolean exitStatsThread = false;

  /**
   * Thread que gera estatsticas de uso das threads de notificao.
   */
  private Thread statsThread = null;

  /**
   * Executor (pool de threads) de notificaes para evitar DoS quando ocorrer
   * avalanche de notificaes.
   */
  private NotificationServiceExecutor executor = null;

  /**
   * Hash de observadores de notificaes de usurio indexada pelo seu id.
   */
  final private Hashtable<Object, ArrayList<RemoteObserver>> usersObservers =
    new Hashtable<Object, ArrayList<RemoteObserver>>();

  /**
   * Busca da instncia de <code>NotificationService</code>.
   * 
   * @return o servio
   * @see NotificationService
   * @see NotificationServiceInterface#SERVICE_NAME
   */
  public static NotificationService getInstance() {
    return (NotificationService) getInstance(SERVICE_NAME);
  }

  /**
   * Registra um observador remoto de notificao.
   * 
   * @param observer O observador remoto que registra este interesse.
   * @param userId identificador do usurio de interesse.
   */
  @Override
  final synchronized public void addObserver(final RemoteObserver observer,
    final Object userId) {
    if ((observer == null) || (userId == null)) {
      return;
    }

    ArrayList<RemoteObserver> uobs = usersObservers.get(userId);
    if (uobs == null) {
      uobs = new ArrayList<RemoteObserver>();
      usersObservers.put(userId, uobs);
    }
    if (!uobs.contains(observer)) {
      uobs.add(observer);
    }
    notifyUserObservers(userId);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  final protected boolean has2Update(final Object arg, final Object event) {
    return true;
  }

  /**
   * Deregistra um observador
   * 
   * @param observer O observador remoto a ser deregistrado.
   * @param userId identificador do usurio de interesse.
   * 
   * @return um indicativo de sucesso.
   */
  @Override
  final synchronized public boolean deleteObserver(
    final RemoteObserver observer, final Object userId) {
    if ((observer == null) || (userId == null)) {
      return false;
    }
    final ArrayList<RemoteObserver> uobs = usersObservers.get(userId);
    if (uobs != null) {
      boolean result = uobs.remove(observer);
      if (uobs.size() == 0) {
        usersObservers.remove(userId);
      }
      return result;
    }
    return false;
  }

  /**
   * Construo e inicializao do executor.
   */
  private void setupExecutor() {
    boolean gatherStats = getBooleanProperty("generateStats");
    int maxThreads = getIntProperty("maxThreads");
    long keepAlive = getIntProperty("keepAliveMinutes");
    keepAlive *= 60 * 1000;
    logInfoMessage("[gatherStats]: " + gatherStats);
    logInfoMessage("[maxThreads]: " + maxThreads);
    logInfoMessage("[keepAlive]: " + keepAlive + "ms");
    executor =
      new NotificationServiceExecutor(gatherStats, maxThreads, maxThreads,
        keepAlive, TimeUnit.SECONDS, new LinkedBlockingDeque<Runnable>(),
        new NotificationServiceThreadFactory());
    executor.allowCoreThreadTimeOut(true);
    if (gatherStats)
      createStatsThread();
  }

  /**
   * Inicializao do servio.
   * 
   * @throws ServerException se ocorrer um erro na inicializao.
   */
  @Override
  final public void initService() throws ServerException {
    setupExecutor();
    readBackup();
    createBackupThread();
    createCleanThread();
    logInfoMessage("Servio de notificao iniciado.");
  }

  /**
   * Notifica todos os usurios. A notificao ser exibida no painel de
   * notificaes.
   * 
   * @param senderName o nome remetente.
   * @param data a mensagem.
   * 
   * @return um indicativo de sucesso.
   */
  final public boolean notifyAllUsers(final String senderName,
    final NotificationData data) {
    if (!canWork()) {
      return false;
    }
    try {
      boolean flag = true;
      final List<User> users = User.getAllUsers();
      for (User user : users) {
        final Object id = user.getId();
        final Object msgId = notifyUser(senderName, id, data);
        if (msgId == null) {
          flag = false;
        }
      }
      return flag;
    }
    catch (Exception e) {
      logSevereMessage("Falha de notificao all-users: " + e.getMessage());
      return false;
    }
  }

  /**
   * Notifica um usurio atravs de um servio. A notificao ser exibida no
   * painel de notificaes.
   * 
   * @param service o servio remetente.
   * @param data a mensagem.
   * 
   * @return um indicativo de sucesso.
   */
  final public boolean notifyAllUsersFromService(final Service service,
    final NotificationData data) {
    if (!canWork()) {
      return false;
    }
    if (service == null) {
      return false;
    }
    final String senderName = translateServiceSenderString(service);
    return notifyAllUsers(senderName, data);
  }

  /**
   * Tradutor de nome de servio
   * @param service o servio
   * @return o texto.
   */
  final private String translateServiceSenderString(final Service service) {
    final Server server = Server.getInstance();
    final String serverName = server.getServerName();
    final String serviceName = service.getName();
    
    
    final String propName = "service.sender.text";
    if (isPropertyNull(propName)) {
      return null;
    }
    
    final String prop = getStringProperty(propName);
    String text = prop.replaceAll("\\$SERVICE_ID", serviceName);
    text = text.replaceAll("\\$SERVER_NAME", serverName);
    return text;
  }
  
  /**
   * Notifica um conjunto de usurios. A notificao ser exibida no painel de
   * notificaes.
   * 
   * @param senderName - nome do servio do qual partiu a notificao
   * @param usersIDs - array de usurios a serem notificados
   * @param data - notificao
   * @return <code>true</code> se todos os usurios foram notificados,
   *         <code>false</code> se um ou mais notificaes falharam
   * 
   * @see #notifySomeUsers(String, Collection, NotificationData)
   */
  protected final boolean notifySomeUsers(final String senderName,
    final Object[] usersIDs, final NotificationData data) {
    if (usersIDs == null) {
      return true;
    }
    return notifySomeUsers(senderName, Arrays.asList(usersIDs), data);
  }

  /**
   * Notifica um conjunto de usurios. A notificao ser exibida no painel de
   * notificaes.
   * 
   * @param senderName - nome do servio do qual partiu a notificao
   * @param usersIDs - conjunto de usurios a serem notificados
   * @param data - notificao
   * @return <code>true</code> se todos os usurios foram notificados,
   *         <code>false</code> se um ou mais notificaes falharam
   */
  final public boolean notifySomeUsers(String senderName,
    Collection<Object> usersIDs, NotificationData data) {
    if (!canWork()) {
      return false;
    }
    if (usersIDs == null) {
      return true;
    }
    boolean status = true;
    for (Object uid : usersIDs) {
      Object msgId = notifyUser(senderName, uid, data);
      if (msgId == null) {
        status = false;
      }
    }
    return status;
  }

  /**
   * Notifica um conjunto usurios atravs de um servio. A notificao ser
   * exibida no painel de notificaes.
   * 
   * @param service o servio remetente.
   * @param usersIDs array de identificadores dos destinatrios.
   * @param data a mensagem.
   * @return flag indicativo de sucesso no envio.
   * 
   * @see #notifySomeUsersFromService(Service, Collection, NotificationData)
   */
  final public boolean notifySomeUsersFromService(final Service service,
    final Object[] usersIDs, final NotificationData data) {
    if (usersIDs == null) {
      return true;
    }
    final String senderName = translateServiceSenderString(service);
    return notifySomeUsers(senderName, Arrays.asList(usersIDs), data);
  }

  /**
   * Notifica um conjunto usurios atravs de um servio. A notificao ser
   * exibida no painel de notificaes.
   * 
   * @param service o servio remetente.
   * @param users cojunto de identificadores dos destinatrios.
   * @param notification a mensagem.
   * @return flag indicativo de sucesso no envio.
   */
  final public boolean notifySomeUsersFromService(final Service service,
    final Collection<Object> users, final NotificationData notification) {
    final String senderName = translateServiceSenderString(service);
    return notifySomeUsers(senderName, users, notification);
  }

  /**
   * Notifica um usurio por um usurio com uma string.
   * 
   * @param senderName o nome remetente.
   * @param userId o identificador do destinatrio.
   * @param content a mensagem.
   * @param mustPopUp se a mensagem  urgente e precisa ser mostrada com dilogo
   *        pop-up.
   * @param volFlag flag de volatilidade da mensagem.
   * 
   * @return flag indicativo de sucesso da operao.
   */
  final public boolean notifyTo(final String senderName, final Object userId,
    final String content, final boolean mustPopUp, final boolean volFlag) {
    if ((senderName == null) || (content == null)) {
      return false;
    }
    final UserNotificationData und =
      new UserNotificationData(content, mustPopUp, volFlag);
    if (userId == null) {
      return notifyAllUsers(senderName, und);
    }
    final Object id = notifyUser(senderName, userId, und);
    if (id == null) {
      return false;
    }
    return true;
  }

  /**
   * Notifica um usurio por um usurio com uma string, onde a mensagem no
   * precisa ser mostrada com pop-up.
   * 
   * @param senderName o nome remetente.
   * @param userId o identificador do destinatrio.
   * @param content a mensagem.
   * @param volFlag flag de volatilidade da mensagem.
   * 
   * @return flag indicativo de sucesso da operao.
   */
  final public boolean notifyTo(final String senderName, final Object userId,
    final String content, final boolean volFlag) {
    return this.notifyTo(senderName, userId, content, false, volFlag);
  }

  /**
   * Notifica um usurio.<br>
   * Notificaes enviadas por este mtodo sero exibidas no painel de
   * notificaes.
   * 
   * @param senderName o nome remetente.
   * @param userId o identificador do destinatrio.
   * @param data a mensagem.
   * 
   * @return identificador da mensagem enviada ou <code>null</code>.
   */
  final public Object notifyUser(final String senderName, final Object userId,
    final NotificationData data) {
    return notifyUser(senderName, userId, data, true);
  }

  /**
   * Notifica um usurio.
   * 
   * @param senderName o nome remetente.
   * @param userId o identificador do destinatrio.
   * @param data a mensagem.
   * @param display <tt>true</tt> se o cliente deve exibir esta notificao ao
   *        painel de notificaes
   * 
   * @return identificador da mensagem enviada ou <code>null</code>.
   */
  final public Object notifyUser(final String senderName, final Object userId,
    final NotificationData data, final boolean display) {
    if (!canWork()) {
      return null;
    }
    final String msgId = generateIdFromUser(userId);
    if (msgId == null) {
      return null;
    }
    try {
      final Notification notif =
        new Notification(msgId, senderName, data, display);
      insertUserNotification(userId, notif);
      notifyUserObservers(userId);
      return msgId;
    }
    catch (Exception e) {
      logSevereMessage("Falha de notificao ao usurio: " + userId
        + " - MSG ID: " + msgId + " (" + senderName + ") -> " + e.getMessage());
      return null;
    }
  }

  /**
   * Notifica um usurio atravs de um servio<br>
   * Notificaes enviadas por este mtodo sero exibidas no painel de
   * notificaes.
   * 
   * @param service o servio remetente.
   * @param userId o identificador do usurio destinatrio.
   * @param data a mensagem.
   * 
   * @return identificador da mensagem enviada ou <code>null</code>.
   */
  final public Object notifyUserFromService(final Service service,
    final Object userId, final NotificationData data) {
    return notifyUserFromService(service, userId, data, true);
  }

  /**
   * Notifica um usurio atravs de um servio.
   * 
   * @param service o servio remetente.
   * @param userId o identificador do usurio destinatrio.
   * @param data a mensagem.
   * @param display <tt>true</tt> se o cliente deve expor essa notificao ao
   *        usurio.
   * 
   * @return identificador da mensagem enviada ou <code>null</code>.
   */
  final public Object notifyUserFromService(final Service service,
    final Object userId, final NotificationData data, final boolean display) {
    if (!canWork()) {
      return null;
    }
    if (service == null) {
      return null;
    }
    final String senderName = translateServiceSenderString(service);
    return notifyUser(senderName, userId, data, display);
  }

  /**
   * Trmino do servio.
   */
  @Override
  final public void shutdownService() {
    logInfoMessage("Finalizando executor de updates...");
    executor.shutdownNow();
    logInfoMessage("Finalizando threads...");
    exitStatsThread = true;
    exitBackupThread = true;
    exitCleanThread = true;
    if (statsThread != null) {
      statsThread.interrupt();
    }
    if (backupThread != null) {
      backupThread.interrupt();
    }
    if (cleanThread != null) {
      cleanThread.interrupt();
    }
    statsThread = null;
    backupThread = null;
    cleanThread = null;
    // a mensagem de log seguinte no faz sentido...
    // melhor remover, ou mudar de lugar
    logInfoMessage("Gravando mensagens no entregues em backup...");
  }

  /**
   * Consulta ao nome do arquivo de backup.
   * 
   * @return o nome do arquivo.
   */
  private String getBackupFileName() {
    final String BACKUP_FILENAME = "backup.dat";
    try {
      final Server server = Server.getInstance();
      final String sep = File.separator;
      final String pName = server.getPersistencyRootDirectoryName();
      final String dName = pName + sep + "notifications";
      server.checkDirectory(dName);
      return dName + sep + BACKUP_FILENAME;
    }
    catch (Throwable t) {
      logSevereMessage("Falha de aquisio de backup:" + t.getMessage());
      return BACKUP_FILENAME;
    }
  }

  /**
   * Consulta ao usurio do destinatrio
   * 
   * @param userId o identificador do usurio
   * 
   * @return o nome do usurio
   */
  private String getUserLogin(final Object userId) {
    try {
      User user = User.getUser(userId);
      if (user == null) {
        logSevereMessage("Detectado usurio nulo para: " + userId);
        return null;
      }
      return user.getLogin();
    }
    catch (Exception ue) {
      logSevereMessage("Falha na deteco de usurio destino " + userId + " - "
        + ue.getMessage());
    }
    return null;
  }

  /**
   * Limpeza de notificaes velhas.
   * 
   * @param days nmero de dias (de antigidade) para limpeza.
   */
  private void cleanNotifications(final long days) {
    if (container == null) {
      return;
    }
    if (container.clean(days)) {
      logDebugMessage("Limpeza de mensagens realiza com sucesso.");
    }
    else {
      logSevereMessage("Falha de limpeza de mensagens");
    }
  }

  /**
   * Gravao de estatsticas de uso das threads de notificao.
   */
  private void writeStats() {
    long wmaxrt = executor.getWeightedMaxRuntime();
    logWarningMessage("Tempo mximo por notificao: " + wmaxrt + "ms");
    long wminrt = executor.getWeightedMinRuntime();
    logWarningMessage("Tempo mnimo por notificao: " + wminrt + "ms");
    long amaxrt = executor.getAbsoluteMaxRuntime();
    logWarningMessage("Tempo mximo por thread de notificaes: " + amaxrt + "ms");
    long aminrt = executor.getAbsoluteMinRuntime();
    logWarningMessage("Tempo mnimo por thread de notificaes: " + aminrt + "ms");
    int execs = executor.getExecutions();
    logWarningMessage("Nmero de notificaes enviadas: " + execs);
    int faults = executor.getFaults();
    logWarningMessage("Nmero de falhas na estatstica acima: " + faults);
    int active = executor.getActiveCount();
    logWarningMessage("Nmero de notificaes sendo enviadas agora: " + active);
    int maxsize = executor.getLargestPoolSize();
    logWarningMessage("Tamanho mximo atingido pelo pool: " + maxsize);
    int size = executor.getPoolSize();
    logWarningMessage("Tamanho atual do pool: " + size);
  }

  /**
   * Criao da thread que gera estatsticas de uso das threads de notificao
   * 
   * @see #initService()
   */
  private void createStatsThread() {
    long statsIntervalMinutes = getIntProperty("statsIntervalMinutes");
    logInfoMessage("[statsIntervalMinutes]: " + statsIntervalMinutes + " min.");
    final long sleepTime = statsIntervalMinutes * 60 * 1000;
    statsThread = new Thread(new Runnable() {
      public void run() {
        while (!exitStatsThread) {
          try {
            writeStats();
            executor.reset();
            Thread.sleep(sleepTime);
          }
          catch (InterruptedException ie) {
            logWarningMessage("Gerao de estatsticas interrompida!");
          }
        }
        logWarningMessage("Finalizando thread de gravao de estatsticas...");
      }
    });
    exitStatsThread = false;
    statsThread.start();
  }

  /**
   * Criao da thread que faz a gravao dos backups
   * 
   * @see #initService()
   */
  private void createBackupThread() {
    /*
     * Busca das propriedades que identificam os tempos para gravao
     * (amostragem) dos dados dos SGAs em disco
     */
    final int TEN_MINUTES = 10;
    long backupIntervalMinutes = getIntProperty("backupIntervalMinutes");
    if (backupIntervalMinutes < TEN_MINUTES) {
      backupIntervalMinutes = TEN_MINUTES;
    }
    logInfoMessage("[backupIntervalMinutes]: " + backupIntervalMinutes
      + " min.");
    final long sleepTime = backupIntervalMinutes * 60 * 1000;
    backupThread = new Thread(new Runnable() {
      public void run() {
        while (!exitBackupThread) {
          try {
            writeBackup();
            Thread.sleep(sleepTime);
          }
          catch (InterruptedException ie) {
            logWarningMessage("Gravao de backups interrompido!");
            // caso o loop seja interrompido,  possvel que tenha
            // deixado o arquivo de backup
            // corrompido. Devemos ento reescrev-lo
            writeBackup();
          }
        }
        logWarningMessage("Finalizando thread de gravao de backups...");
      }
    });
    exitBackupThread = false;
    backupThread.start();
  }

  /**
   * Criao da thread que faz a limpeza de mensagens velhas
   * 
   * @see #initService()
   */
  private void createCleanThread() {
    /*
     * Busca das propriedades que identificam os tempos para gravao
     * (amostragem) dos dados dos SGAs em disco
     */
    final int TEN_DAYS = 10;
    int cleanDays = getIntProperty("cleanIntervalDays");
    if (cleanDays < TEN_DAYS) {
      cleanDays = TEN_DAYS;
    }
    logInfoMessage("[cleanIntervalDays] = " + cleanDays);

    /*
     * Busca das propriedades que identificam o tempo para volatilidade de uma
     * notificao
     */
    final int TEN_SECONDS = 10;
    int volSeconds = getIntProperty("volatileIntervalSeconds");
    if (volSeconds <= 0) {
      volSeconds = TEN_SECONDS;
    }
    logInfoMessage("[volatileIntervalSeconds] = " + volSeconds);

    long daysToMili = (long) 24 * (long) 60 * 60 * 1000;
    long secToMili = 1000;
    final long sleepTime = volSeconds * secToMili;
    final long delta = cleanDays * daysToMili;
    logInfoMessage("Intervalo de limpeza: " + cleanDays + " dias.");
    cleanThread = new Thread(new Runnable() {
      public void run() {
        while (!exitCleanThread) {
          try {
            cleanNotifications(delta);
            Thread.sleep(sleepTime);
          }
          catch (InterruptedException ie) {
            logWarningMessage("Limpeza de mensagens interrompida!");
          }
        }
        logWarningMessage("Finalizando thread de limpeza...");
      }
    });
    exitCleanThread = false;
    cleanThread.start();
  }

  /**
   * Notifica em thread os observadores do usurio.
   * 
   * @param userId identificador do usurio.
   * @param remoteObservers lista de observadores remotos
   * @param notifications lista de notificaes.
   */
  private void notifyRemoteObservers(Object userId,
    ArrayList<RemoteObserver> remoteObservers,
    ArrayList<Notification> notifications) {
    executor.setExecutionWeight(remoteObservers.size());
    ArrayList<RemoteObserver> failedObservers = new ArrayList<RemoteObserver>();
    for (RemoteObserver observer : remoteObservers) {
      try {
        observer.update(notifications);
      }
      catch (Exception e) {
        failedObservers.add(observer);
        final String prefix = "Falha de notificao remota (observao): ";
        final String err = e.getMessage();
        final int nNots = notifications.size();
        logSevereMessage(prefix + userId + " - Num. de Nots: " + nNots + " - "
          + err);
      }
    }

    for (RemoteObserver failedObserver : failedObservers) {
      remoteObservers.remove(failedObserver);
    }
  }

  /**
   * Gerao de um identificador de mensagem para um usurio
   * 
   * @param userId identificador do usurio.
   * 
   * @return identificador da mensagem.
   */
  private String generateIdFromUser(final Object userId) {
    if (userId == null) {
      return null;
    }
    final String id = generateTimeId();
    final String userName = getUserLogin(userId);
    if (userName == null) {
      return null;
    }
    return "MSG-USR-" + userName + "-" + id;
  }

  /**
   * Gerao de um identificador de mensagem
   * 
   * @return identificador da mensagem.
   */
  private String generateTimeId() {
    final String format = "{0,date,yyyyMMdd}{0,time,HHmmss}";
    final MessageFormat formatter = new MessageFormat(format);
    final Object[] args = new Object[1];
    args[0] = Calendar.getInstance().getTime();
    final StringBuffer text = new StringBuffer();
    formatter.format(args, text, null);
    return text.toString();
  }

  /**
   * Insero de uma notificao de usurio
   * 
   * @param usrId o usurio destinatrio
   * @param notif objeto de notificao
   */
  private void insertUserNotification(final Object usrId,
    final Notification notif) {
    if (container != null) {
      container.insertUserNotification(usrId, notif);
    }
  }

  /**
   * Notifica os observadores do usurio.
   * 
   * @param userId identificador
   */
  private void notifyUserObservers(final Object userId) {
    if (container == null) {
      return;
    }
    final Runnable runnable = new Runnable() {
      public void run() {
        final ArrayList<Notification> notifs =
          container.retrieveUserNotifications(userId);
        if (notifs == null || notifs.isEmpty()) {
          return;
        }
        ArrayList<RemoteObserver> uobs;
        synchronized (usersObservers) {
          uobs = usersObservers.get(userId);
          if (uobs == null || uobs.isEmpty()) {
            return;
          }
          uobs = (ArrayList<RemoteObserver>) uobs.clone();
        }
        notifyRemoteObservers(userId, uobs, notifs);
      }
    };
    executor.execute(runnable);
  }

  /**
   * Leitura de um backup de notificaes
   */
  private void readBackup() {
    final String backupFileName = getBackupFileName();
    final File backupFile = new File(backupFileName);
    final String backupFilePath = backupFile.getAbsolutePath();
    logInfoMessage("Iniciando recuperao de backup de: " + backupFilePath);
    
    if (!backupFile.exists()) {
      final String msg = "No foi encontrado arquivo de backup de notificaes: ";
      logWarningMessage(msg + backupFilePath);
      container = new NotificationContainer();
      return;
    }

    try {
      final ObjectInputStream in =
        new ObjectInputStream(new BufferedInputStream(new FileInputStream(
          backupFile)));
      container = (NotificationContainer) in.readObject();
      in.close();
      logInfoMessage("Notificaes recuperadas de: "+ backupFilePath);
    }
    catch (InvalidClassException ce) {
      container = new NotificationContainer();
      final String err = "No foi possvel fazer recuperao de backup. ";
      final String cause = "Verso de container no compatvel com arquivo: ";
      logWarningMessage(err + cause + backupFilePath);
    }
    catch (Exception e) {
      container = new NotificationContainer();
      logSevereMessage("Falha de recuperao de backup: " + backupFilePath, e);
    }
  }

  /**
   * Gravao de um backup de notificaes.
   */
  private void writeBackup() {
    final String bkp = getBackupFileName();
    if (container != null) {
      final File dataFile = new File(bkp);
      if (container.dumpContainer(dataFile)) {
        logDebugMessage("Feito backup de notificaes em: " + bkp);
      }
      else {
        logSevereMessage("Falha de gravao de backup de notificaes.");
      }
    }
  }

  /**
   * Constri a instncia do servio.
   * 
   * @throws ServerException se houver falha no servidor.
   * @see NotificationService
   */
  public static void createService() throws ServerException {
    new NotificationService();
  }

  /**
   * Construtor do servio.
   * 
   * @throws ServerException em caso de falha na construo do servio.
   */
  protected NotificationService() throws ServerException {
    super(SERVICE_NAME);
  }
}
