/*
 * $Id$
 */
package csbase.server.services.openbusservice;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.security.GeneralSecurityException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.security.interfaces.RSAPrivateKey;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;
import java.util.logging.Level;

import openbusidl.acs.Credential;
import openbusidl.rs.IRegistryService;
import openbusidl.rs.ServiceOffer;

import org.omg.CORBA.NO_PERMISSION;
import org.omg.CORBA.ORB;
import org.omg.CORBA.StringHolder;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.UserException;

import tecgraf.openbus.Openbus;
import tecgraf.openbus.data_service.DataDescriptionFactory;
import tecgraf.openbus.data_service.DataDescriptionHelper;
import tecgraf.openbus.data_service.UnstructuredDataFactory;
import tecgraf.openbus.data_service.UnstructuredDataHelper;
import tecgraf.openbus.data_service.project.ProjectItemDescriptionFactory;
import tecgraf.openbus.exception.OpenBusException;
import tecgraf.openbus.lease.LeaseExpiredCallback;
import tecgraf.openbus.project.ProjectItemDescriptionHelper;
import tecgraf.openbus.util.CryptoUtils;
import tecgraf.openbus.util.Log;
import csbase.logic.BusInfo;
import csbase.remote.OpenBusServiceInterface;
import csbase.server.ServerException;
import csbase.server.Service;

/**
 * Representa o servio que trata da conexo do CSBase com o OpenBus.
 * 
 * @author Tecgraf/PUC-Rio
 */
public final class OpenBusService extends Service implements
  OpenBusServiceInterface {
  /**
   * A instncia nica do servio do OpenBus.
   */
  private static OpenBusService instance;
  /**
   * Verifica se o servio do OpenBus deve ser habilitado.
   */
  private boolean enabled;
  /**
   * Os componentes cadastrados no servio.
   */
  private Set<ComponentEntry> components;
  /**
   * <i>Thread</i> responsvel pelo ORB.
   */
  private ORBThread orbThread;
  /**
   * <i>Thread</i> responsvel por obter o Servio de Controle de Acesso.
   */
  private OpenbusConnectionManager acsFetcher;
  /**
   * Informaes sobre a localizao do OpenBus.
   */
  private BusInfo busInfo;
  /**
   * O nome da entidade representada por este servidor no OpenBus.
   */
  private String entityName;
  /**
   * A chave privada deste servidor.
   */
  private RSAPrivateKey privateKey;
  /**
   * O certificado do OpenBus.
   */
  private X509Certificate acsCertificate;
  /**
   * <i>Callback</i> utilizada para tratar a expirao do lease.
   */
  private LeaseExpiredCallbackImpl leaseCallback;

  /**
   * Cria o servio do OpenBus.
   * 
   * @throws ServerException Caso ocorra algum erro na criao do servio.
   */
  private OpenBusService() throws ServerException {
    super(SERVICE_NAME);
    Log.setLogsLevel(Level.parse(getStringProperty("log.level")));

    this.components = new HashSet<ComponentEntry>();

    this.enabled = this.getBooleanProperty("enabled");

    if (enabled) {
      this.entityName = this.getStringProperty("entity.name");
      String accessControlServiceHost = this.getStringProperty("host");
      int accessControlServicePort = this.getIntProperty("port");
      this.busInfo =
        new BusInfo(accessControlServiceHost, accessControlServicePort);

      try {
        this.privateKey =
          CryptoUtils
            .readPrivateKey(this.getStringProperty("private.key.file"));
      }
      catch (GeneralSecurityException e) {
        throw new ServerException(e);
      }
      catch (IOException e) {
        throw new ServerException(e);
      }

      try {
        this.acsCertificate =
          CryptoUtils.readCertificate(this
            .getStringProperty("certificate.file"));
      }
      catch (CertificateException e) {
        throw new ServerException(e);
      }
      catch (FileNotFoundException e) {
        throw new ServerException(e);
      }

      this.leaseCallback = new LeaseExpiredCallbackImpl();

      Properties props = new Properties();
      props.setProperty("org.omg.CORBA.ORBClass", "org.jacorb.orb.ORB");
      props.setProperty("org.omg.CORBA.ORBSingletonClass",
        "org.jacorb.orb.ORBSingleton");

      Openbus bus = Openbus.getInstance();
      try {
        bus.resetAndInitialize(null, props, accessControlServiceHost,
          accessControlServicePort);
      }
      catch (UserException e) {
        throw new ServerException(e);
      }

      org.omg.CORBA_2_3.ORB orb = (org.omg.CORBA_2_3.ORB) bus.getORB();
      orb.register_value_factory(DataDescriptionHelper.id(),
        new DataDescriptionFactory());
      orb.register_value_factory(ProjectItemDescriptionHelper.id(),
        new ProjectItemDescriptionFactory());
      orb.register_value_factory(UnstructuredDataHelper.id(),
        new UnstructuredDataFactory());

      this.acsFetcher =
        new OpenbusConnectionManager(this.entityName, this.privateKey,
          this.acsCertificate, this.leaseCallback);
      this.orbThread = new ORBThread(bus.getORB());
    }
  }

  /**
   * Obtm a instncia nica do servio.
   * 
   * @return A instncia nica do servio.
   */
  public static OpenBusService getInstance() {
    return instance;
  }

  /**
   * Cria a instncia nica do servio.
   * 
   * @throws ServerException Se houver erro na inicializao.
   */
  public static void createService() throws ServerException {
    instance = new OpenBusService();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void initService() {
    if (!this.enabled) {
      return;
    }
    this.orbThread.start();
    this.acsFetcher.start();
  }

  /**
   * Obtm o identificador do usurio que est executando uma requisio atravs
   * do OpenBus.
   * 
   * @return O identificador do usurio ou {@code null}, caso no haja
   *         requisio em execuo.
   */
  public String getRequesterUserId() {
    Openbus bus = Openbus.getInstance();
    Credential credential = bus.getInterceptedCredential();
    if (credential == null || credential.identifier.isEmpty()) {
      return null;
    }
    if (!credential.delegate.equals("")) {
      return credential.delegate;
    }
    return credential.owner;
  }

  /**
   * Obtm o identificador do sistema em que o usurio est executando uma
   * requisio atravs do OpenBus.
   * 
   * @return o identificador do sistema ou {@code null}, caso no haja
   *         requisio em execuo
   */
  public String getRequesterSystemId() {
    Openbus bus = Openbus.getInstance();
    Credential credential = bus.getInterceptedCredential();
    if (credential == null || credential.identifier.isEmpty()) {
      return null;
    }
    return credential.owner;
  }

  /**
   * Inicia o usurio do servio CSBase e tambm o sistema onde esse usurio
   * est executando uma requisio atravs do OpenBus. Para isso, considera-se
   * como base a credencial usada para acesso no OpenBus.
   * 
   * @return retorna o identificador do usurio
   */
  public String initCSBaseAccess() {
    String userId = getRequesterUserId();
    String systemId = getRequesterSystemId();
    Service.setUserId(userId);
    Service.setSystemId(systemId);
    return userId;
  }

  /**
   * Finaliza o acesso do usurio do servio CSBase na sesso em que foi feita
   * uma requisio atravs do OpenBus. O sistema em que esse usurio estava
   * executando a requisio tambm  finalizado na sesso.
   * 
   */
  public void finishCSBaseAccess() {
    Service.setUserId(null);
    Service.setSystemId(null);
  }

  /**
   * Utiliza o Servio de Registro para incluir as ofertas de servio dos
   * componentes cadastrados.
   * 
   * @return {@code true}, se as ofertas foram cadastradadas, ou {@code false},
   *         caso contrrio.
   */
  private boolean registerComponents() {
    Openbus bus = Openbus.getInstance();
    IRegistryService registryService = bus.getRegistryService();
    StringHolder identifier = new StringHolder();
    for (ComponentEntry component : this.components) {
      try {
        if (registryService.register(component.serviceOffer, identifier)) {
          component.registryIdentifier = identifier.value;
        }
      }
      catch (SystemException e) {
        this.logSevereMessage(e.getMessage(), e);
      }
    }
    return true;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void shutdownService() {
    if (!this.enabled) {
      return;
    }
    this.acsFetcher.finish();
    try {
      Openbus bus = Openbus.getInstance();
      this.unregisterComponents();
      bus.removeLeaseExpiredCallback();
      bus.disconnect();
    }
    catch (NO_PERMISSION e) {
    }
    catch (SystemException se) {
      this.logSevereMessage("Erro inesperado ao acessar um componente CORBA.",
        se);
    }
  }

  /**
   * Utiliza o Servio de Registro para retirar as ofertas de servio dos
   * componentes cadastrados.
   * 
   * @return {@code true}, se as ofertas foram descadastradadas, ou {@code
   *         false}, caso contrrio.
   */
  private boolean unregisterComponents() {
    Openbus bus = Openbus.getInstance();
    IRegistryService registryService = bus.getRegistryService();
    if (registryService == null) {
      return false;
    }
    else {
      for (ComponentEntry component : this.components) {
        try {
          registryService.unregister(component.registryIdentifier);
        }
        catch (SystemException e) {
          this.logWarningMessage(e.getMessage());
        }
      }
    }
    return true;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected boolean has2Update(Object arg, Object event) {
    return false;
  }

  /**
   * Verifica se o servio est habilitado.
   * 
   * @return {@code true} caso o servio esteja habilitado, ou {@code false},
   *         caso contrrio.
   */
  public boolean isEnabled() {
    return this.enabled;
  }

  /**
   * Adiciona um componente que ser registrado no OpenBus.
   * 
   * @param serviceOffer A sua oferta de servio.
   * 
   * @return {@code true} caso o componente seja inserido, ou {@code false},
   *         caso contrrio.
   */
  public boolean addComponent(ServiceOffer serviceOffer) {
    if (!this.enabled) {
      return false;
    }
    return this.components.add(new ComponentEntry(serviceOffer));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public BusInfo getBusInfo() {
    return this.busInfo;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public csbase.logic.openbus.Credential getServerCredential() {
    Credential serverCredential = Openbus.getInstance().getCredential();
    if (serverCredential == null || serverCredential.identifier.isEmpty()) {
      return null;
    }
    String serverCredentialDelegate = "";
    if (serverCredential.delegate == null) {
      serverCredentialDelegate = serverCredential.delegate;
    }
    return new csbase.logic.openbus.Credential(serverCredential.identifier,
      serverCredential.owner, serverCredentialDelegate);
  }

  /**
   * Faz a reconexo do servidor com o OpenBus no caso de o lease expirar.
   */
  void leaseExpired() {
    if (this.acsFetcher.isRunning()) {
      this.acsFetcher.finish();
    }
    this.acsFetcher =
      new OpenbusConnectionManager(this.entityName, this.privateKey,
        this.acsCertificate, this.leaseCallback);
    this.acsFetcher.start();

  }

  /**
   * <i>Thread</i> responsvel por conectar o servio ao OpenBus.
   * 
   * @author Tecgraf/PUC-Rio
   */
  private static class OpenbusConnectionManager extends Thread {
    /**
     * Nome deste servidor no OpenBus.
     */
    private String entityName;
    /**
     * A chave privada deste servidor.
     */
    private RSAPrivateKey privateKey;
    /**
     * O certificado digital do Servio de Controle de Acesso do OpenBus.
     */
    private X509Certificate acsCertificate;
    /**
     * <i>Callback</i> executada quando acontece a expirao do <i>lease</i> da
     * credencial.
     */
    private LeaseExpiredCallbackImpl leaseExpiredCallback;

    /**
     * Indica se a <i>thread</i> precisa continuar a sua execuo.
     */
    private boolean mustContinue;

    /**
     * Constri a <i>thread</i> responsvel por conectar o servio ao OpenBus.
     * 
     * @param entityName Nome deste servidor no OpenBus.
     * @param privateKey A chave privada deste servidor.
     * @param acsCertificate O certificado digital do Servio de Controle de
     *        Acesso do OpenBus.
     * @param leaseExpiredCallback <i>Callback</i> executada quando acontece a
     *        expirao do <i>lease</i> da credencial.
     */
    OpenbusConnectionManager(String entityName, RSAPrivateKey privateKey,
      X509Certificate acsCertificate,
      LeaseExpiredCallbackImpl leaseExpiredCallback) {
      this.entityName = entityName;
      this.privateKey = privateKey;
      this.acsCertificate = acsCertificate;
      this.leaseExpiredCallback = leaseExpiredCallback;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void run() {
      OpenBusService service = OpenBusService.getInstance();
      this.mustContinue = true;
      while (this.mustContinue) {
        Openbus bus = Openbus.getInstance();
        try {
          bus.connect(this.entityName, this.privateKey, this.acsCertificate);
          bus.addLeaseExpiredCallback(this.leaseExpiredCallback);
          service.registerComponents();
          this.finish();
        }
        catch (OpenBusException e) {
          service.logSevereMessage(e.getMessage(), e);
          try {
            Thread.sleep(60 * 1000);
          }
          catch (InterruptedException ie) {
            // Nada a ser feito.
          }
        }
        catch (SystemException e) {
          service.logSevereMessage(e.getMessage(), e);
          try {
            Thread.sleep(60 * 1000);
          }
          catch (InterruptedException ie) {
            // Nada a ser feito.
          }
        }
      }
    }

    /**
     * Finaliza a <i>thread</i>.
     */
    void finish() {
      this.mustContinue = false;
    }

    /**
     * Verifica se a <i>thread</i> est em execuo.
     * 
     * @return {@code true}, caso a <i>thread</i> esteja em execuo, ou {@code
     *         false}, caso contrrio.
     */
    boolean isRunning() {
      return this.mustContinue;
    }
  }

  /**
   * Implementa a <i>callback</i> para o <i>lease</i> expirado.
   * 
   * @author Tecgraf/PUC-Rio
   */
  private class LeaseExpiredCallbackImpl implements LeaseExpiredCallback {
    /**
     * Cria a <i>callback</i>.
     */
    LeaseExpiredCallbackImpl() {
      // Criado apenas para que o construtor tenha visibilidade de pacote.
    }

    /**
     * {@inheritDoc}
     */
    public void expired() {
      OpenBusService.getInstance().leaseExpired();
    }
  }

  /**
   * Representa uma entrada de um componente cadastrado.
   * 
   * @author Tecgraf/PUC-Rio
   * 
   */
  private class ComponentEntry {
    /**
     * A oferta de servio do componente.
     */
    ServiceOffer serviceOffer;
    /**
     * O identificador do objeto.
     */
    byte[] objectID;
    /**
     * A identificao do registro do objeto.
     */
    String registryIdentifier;

    /**
     * Cria uma entrada de um componente cadastrado.
     * 
     * @param serviceOffer A oferta de servio.
     */
    ComponentEntry(ServiceOffer serviceOffer) {
      this.serviceOffer = serviceOffer;
    }
  }

  /**
   * Responsvel por executar o ORB.
   * 
   * @author Tecgraf/PUC-Rio
   */
  private static class ORBThread extends Thread {
    /**
     * O ORB.
     */
    private ORB orb;

    /**
     * Cria a <i>thread</i> para execuo do ORB.
     * 
     * @param orb O ORB.
     */
    ORBThread(ORB orb) {
      this.orb = orb;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void run() {
      this.orb.run();
    }
  }
}
