package csbase.server.services.opendreamsservice;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Set;

import tecgraf.openbus.DRMAA.ExitTimeoutException;
import tecgraf.openbus.DRMAA.InvalidJobException;
import tecgraf.openbus.DRMAA.JobInfo;
import tecgraf.openbus.opendreams.JobInfoImpl;
import csbase.logic.CommandEvent;
import csbase.logic.CommandEvent.EventType;

/**
 * Fila de espera pelos jobs em execuo.
 *  
 * @author Tecgraf PUC-Rio
 */
public class JobInfoQueue {

  /**
   * Tempo em milisegundos para reavaliar se a fila j possui informao sobre o
   * trmino do job.
   */
  public static final long WAITING_TIMEOUT = 100;

  /**
   * Informao de trmino para cada job submetido.
   * Diferentes threads podem aguardar a espera por um mesmo comando, porm 
   * apenas uma delas recebe a informao de trmino; as outras threads recebem 
   * uma exceo. Esse mapa relaciona o identificador do comando submetido com
   * a infornao de trmino de job.
   */
  private Map<String, JobInfo> jobsSubmited = new Hashtable<String, JobInfo>();

  /**
   * Adiciona a informao de trmino de job na fila correspondente ao
   * identificador do job.
   * 
   * @param info a informao de trmino do job
   */
  public void put(JobInfo info) {
    jobsSubmited.put(info.jobId, info);
  }
  
  /**
   * Indica que um job foi submetido
   * @param jobId o identificador do job submetido
   */
  public synchronized void jobSubmited(String jobId) {
    JobInfo jobInfo = jobsSubmited.get(jobId);
    if (jobInfo == null) {
      jobInfo = new JobInfoImpl();
      jobInfo.jobId = jobId;
      jobsSubmited.put(jobId, jobInfo);
    }
  }
  
  /**
   * Indica que um job finalizou
   * @param cmdEvent evento com as informaes de trmino do job 
   */
  public synchronized void jobExit(CommandEvent cmdEvent) {
    String jobId = cmdEvent.getCommandId().toString();
    JobInfo jobInfo = jobsSubmited.get(jobId);
    if (jobInfo==null) {
      jobInfo = new JobInfoImpl();
      jobInfo.jobId = jobId;
      jobsSubmited.put(jobId, jobInfo);
    }
   
    List<String[]> resourceUsage = new ArrayList<String[]>();
    if (cmdEvent.getCpuTime() != null) {
      resourceUsage.add(new String[] { "cpu_time",
          cmdEvent.getCpuTime().toString() });
    }
    if (cmdEvent.getElapsedTime() != null) {
      resourceUsage.add(new String[] { "elapsed_time",
          cmdEvent.getElapsedTime().toString() });
    }
    if (cmdEvent.getUserTime() != null) {
      resourceUsage.add(new String[] { "user_time",
          cmdEvent.getUserTime().toString() });
    }
    if (resourceUsage.size() > 0) {
      jobInfo.resourceUsage =
        resourceUsage.toArray(new String[resourceUsage.size()][]);
    }   
    jobInfo.hasExited = true;
    jobInfo.exitStatus = cmdEvent.getType().equals(EventType.SUCCESS) ? 0 : -1;
    jobInfo.wasAborted = cmdEvent.getType().equals(EventType.KILLED);    
    jobInfo.hasSignaled = false;
    jobInfo.terminatingSignal =  "";
    jobInfo.hasCoreDump = false;
    notifyAll();
  }

  /**
   * Aguarda que um determinado job submetido termine sua execuo.
   * 
   * Se mais de uma thread estiver aguardando pelo mesmo job, apenas uma delas
   * recebe a informao de trmino do job. As outras threads recebem e exceo
   * <code>InvalidJobException</code>. Essa mesma exceo  tambm lanada se
   * o identificador do job no for encontrado entre os jobs submetidos nessa
   * sesso.
   * 
   * No caso do timeout ter esgotado sem que o job tenha terminado, a exceo
   * <code>ExitTimeoutException</code>  lanada. Nesse caso, a informao sobre
   * o job permanece disponvel para uma outra requisio.
   * 
   * @param jobId o nome do job monitorado
   * @param timeout o tempo de espera limite em segundos
   * @return a informao sobre o job submetido
   * @throws InvalidJobException se o identificador do job no foi encontrado ou 
   * se outra requisio j recebeu a informao de trmino do job.
   * @throws ExitTimeoutException se o timeout esgotou e o job no terminou.
   */
  public JobInfo waitFor(String jobId, long timeout) throws InvalidJobException, 
  ExitTimeoutException {
    synchronized (this) {
      /* Verifica se existe um job submetido com o identificador especificado */
      JobInfo info = jobsSubmited.get(jobId);
      if (info == null) {
        throw new InvalidJobException(
          "No foi encontrado um job com o nome especificado: "+jobId);
      }
      /* Se o tempo especificado foi TIMEOUT_NO_WAIT retorna o jobInfo */
      /* TODO mjulia Confirmar se o comportamento  esse mesmo */
      if (timeout == Session.TIMEOUT_NO_WAIT) {
        jobsSubmited.remove(jobId);
        return info;
      }
    }
    final long limit = System.currentTimeMillis() * 1000 + timeout;
    long timeToGo = limit - (System.currentTimeMillis() * 1000);
    boolean waitForever = (timeout == Session.TIMEOUT_WAIT_FOREVER);
    while (waitForever || timeToGo > 0) {
      try {
        synchronized(this) {
          wait(WAITING_TIMEOUT);
        }
        if (!waitForever)
          timeToGo = limit - System.currentTimeMillis() * 1000;
        synchronized (this) {
          JobInfo info = jobsSubmited.get(jobId);
          if (info == null) {
            throw new InvalidJobException(
              "Outra requisio j recebeu a informao de trmino desse" +
              "mesmo job: " + jobId);
          }
          if (info.hasExited) {
            jobsSubmited.remove(jobId);
            return info;
          }
        }
      }
      catch (InterruptedException e) {
      }
    }
    /* Quando o tempo esgotou, a informao sobre o job permanece guardada */
    throw new ExitTimeoutException("O timeout esgotou e o job no terminou: " + jobId);
  }

  /**
   * Aguarda que qualquer um dos jobs submetidos termine sua execuo. Quando um
   * deles terminar, retorna a informao sobre esse job.
   * 
   * Apenas uma thread recebe a informao de um determinado job. Se outras threads
   * estiverem aguardando e no houver mais jobs submetidos (no momento em que 
   * a requisio de "wait" for feita), as outras recebem a exceo 
   * <code>InvalidJobException</code>.
   * 
   * No caso do timeout ter esgotado sem que algum job tenha terminado, a exceo
   * <code>ExitTimeoutException</code>  lanada. 
   * 
   * @param timeout o tempo de espera limite em segundos
   * @return a informao sobre o primeiro job que terminou
   * @throws InvalidJobException 
   * @throws ExitTimeoutException se o timeout esgotou e nenhum job terminou.
   */
  public JobInfo waitForAnyJob(long timeout) throws ExitTimeoutException, InvalidJobException {
    final Set<String> jobsAvailable = jobsSubmited.keySet();
    synchronized (this) {
      if (jobsSubmited.size() == 0) {
        throw new InvalidJobException("No h mais jobs submetidos na sesso " +
        		"que ainda no tenham terminado sua execuo");
      }
      /* Se o tempo especificado foi TIMEOUT_NO_WAIT retorna o jobInfo */
      if (timeout == Session.TIMEOUT_NO_WAIT) {
        for (String jobId : jobsAvailable) {
          JobInfo info = jobsSubmited.get(jobId);
          if (info.hasExited) {
            jobsSubmited.remove(jobId);
            return info;
          }
        }
        /* O tempo NO_WAIT esgotou e no h informao sobre trmino de jobs */
        /* TODO mjulia Confirmar se o comportamento  esse mesmo */
        throw new ExitTimeoutException("O timeout esgotou e o job no terminou");
      }
    }    
    final long limit = System.currentTimeMillis() * 1000 + timeout; 
    long timeToGo = limit - (System.currentTimeMillis() * 1000);
    boolean waitForever = (timeout == Session.TIMEOUT_WAIT_FOREVER);
    while (waitForever || timeToGo > 0) {
      try {
        synchronized(this) {
          wait(WAITING_TIMEOUT);
        }
        if (!waitForever)
          timeToGo = limit - System.currentTimeMillis() * 1000;
        synchronized (this) {
          JobInfo info = null;
          for (String jobId : jobsAvailable) {
            info = jobsSubmited.get(jobId);
            if (info.hasExited) {
              jobsSubmited.remove(jobId);
              return info;
            }
          }
          if (info == null) {
            throw new InvalidJobException("No h mais jobs submetidos na " +
            		"sesso que ainda no tenham terminado sua execuo");
          }
        }
      }
      catch (InterruptedException e) {
      }
    }
    throw new ExitTimeoutException("O timeout esgotou e o job no terminou");
  }
}
