package csbase.server.services.opendreamsservice;

import static java.lang.Byte.parseByte;

import java.lang.reflect.InvocationTargetException;

import openbusidl.rs.Property;
import openbusidl.rs.ServiceOffer;

import org.omg.CORBA.UserException;

import scs.core.ComponentId;
import scs.core.IComponent;
import scs.core.IComponentHelper;
import scs.core.servant.ComponentBuilder;
import scs.core.servant.ComponentContext;
import scs.core.servant.ExtendedFacetDescription;
import tecgraf.openbus.Openbus;
import tecgraf.openbus.opendreams.IOpenDreamsHelper;
import tecgraf.openbus.opendreams.JobParameterFactory;
import tecgraf.openbus.opendreams.JobParameterHelper;
import tecgraf.openbus.opendreams.OpenDreamsJobTemplateFactory;
import tecgraf.openbus.opendreams.OpenDreamsJobTemplateHelper;
import csbase.server.ServerException;
import csbase.server.Service;
import csbase.server.services.openbusservice.OpenBusService;

/**
 * O servio <code>OpenDreams</code> publica um componente no OpenBus para
 * execuo remota de algoritmos usando uma interface DRMAA.
 * 
 * @author Tecgraf PUC-Rio
 */
public class OpenDreamsService extends Service {
  /**
   * Indica se o servio deve ser ou no exportado para o OpenBus.
   */
  private boolean exportToOpenBus;

  /** Nome do servio para o <code>ServiceManager</code> */
  public static final String SERVICE_NAME = "OpenDreamsService";
  
  /** Nome default do componente para publicao no <code>OpenBus</code> */
  public static final String COMPONENT_NAME = "OpenDreams";
  
  /** Verso default do componente para publicao no <code>OpenBus</code> */
  public static final String COMPONENT_VERSION = "1.0.0";

  /**
   * Construtor default que cadastra esse servio no <code>ServiceManager</code> .
   * Esse construtor  obrigatrio para que o servio seja iniciado no servidor.
   * 
   * @throws ServerException se ocorrer algum erro na inicializao do servio.
   */
  protected OpenDreamsService() throws ServerException {
    super(SERVICE_NAME);
    this.exportToOpenBus = this.getBooleanProperty("export.to.openbus");
    if (!exportToOpenBus) {
      logInfoMessage("Servio opendreams no foi publicado no OpenBus "
        + "(export.to.openbus=" + exportToOpenBus + ")");
      return;
    }
    OpenBusService openbus = OpenBusService.getInstance();
    if (openbus == null) {
      logWarningMessage("Servio opendreams no foi publicado no OpenBus "
        + "(OpenBusService.getInstance()==null");
      return;
    }
    if (!openbus.isEnabled()) {
      logWarningMessage("Servio opendreams no foi publicado no OpenBus "
        + "(OpenBus est desabilitado");
      return;
    }
    ComponentContext context;
    try {
      context = this.createComponent();
    }
    catch (Exception e) {
      this.logSevereMessage(
        "Erro na criao do componente SCS de servio do OpenDreams", e);
      throw new ServerException(e);
    }
    Openbus bus = Openbus.getInstance();
    org.omg.CORBA.Object obj = context.getIComponent();
    IComponent component = IComponentHelper.narrow(obj);
    openbus.addComponent(new ServiceOffer(new Property[0], component));

    org.omg.CORBA_2_3.ORB orb = (org.omg.CORBA_2_3.ORB) bus.getORB();
    orb.register_value_factory(OpenDreamsJobTemplateHelper.id(),
      new OpenDreamsJobTemplateFactory());
    orb.register_value_factory(JobParameterHelper.id(),
      new JobParameterFactory());
    logInfoMessage("Servio opendreams publicado no OpenBus");
  }

  /**
   * Cria o componente do Servio do OpenDreams que ser exportado no OpenBus.
   * 
   * @return O contexto do componente SCS.
   * 
   * @throws UserException
   * @throws InstantiationException
   * @throws IllegalAccessException
   * @throws ClassNotFoundException
   * @throws InvocationTargetException
   * @throws NoSuchMethodException
   */
  private ComponentContext createComponent() throws UserException,
    InstantiationException, IllegalAccessException, ClassNotFoundException,
    InvocationTargetException, NoSuchMethodException {
    Openbus bus = Openbus.getInstance();
    ComponentBuilder builder =
      new ComponentBuilder(bus.getRootPOA(), bus.getORB());
    ExtendedFacetDescription[] descriptions = new ExtendedFacetDescription[1];
    descriptions[0] =
      new ExtendedFacetDescription("opendreams", IOpenDreamsHelper.id(),
        OpenDreams.class.getCanonicalName());
    String componentName = this.getStringProperty("component.name");
    if (componentName==null || componentName.trim().isEmpty()) {
      componentName = COMPONENT_NAME;
    }
    String componentVersion = this.getStringProperty("component.version");
    if (componentVersion==null || componentVersion.trim().isEmpty()) {
      componentVersion = COMPONENT_VERSION;
    }
    final String[] tmp = componentVersion.split("[\\.]");
    byte major = parseByte(tmp[0]);
    byte minor = tmp.length >= 2 ? parseByte(tmp[1]) : 0;
    byte patch = tmp.length >= 3 ? parseByte(tmp[2]) : 0;
    logInfoMessage("Componente OpenDreams: nome="+componentName + 
      " verso="+componentVersion);
    return builder.newComponent(descriptions, null, new ComponentId(
      componentName, major, minor, patch, "Java"));
  }

  /**
   * Obtm uma instncia do servio.
   * 
   * @return ProjectService
   */
  public static OpenDreamsService getInstance() {
    return (OpenDreamsService) getInstance(SERVICE_NAME);
  }

  /**
   * {@inheritDoc}
   * 
   * Publica um componente <code>OpenDreams</code> como um servio do OpenBus.
   */
  @Override
  public void initService() throws ServerException {
    logInfoMessage("Servio opendreams iniciado.");
  }

  @Override
  public void shutdownService() throws ServerException {
  }

  @Override
  protected boolean has2Update(Object arg, Object event) {
    return false;
  }

  /**
   * Constri a instncia do servio. Esse mtodo  chamado pelo servidor, na
   * criao dos servios.
   * 
   * @throws ServerException se houver falha na criao do servio.
   */
  public static void createService() throws ServerException {
    new OpenDreamsService();
  }

}
