package csbase.server.services.opendreamsservice;

import java.rmi.RemoteException;
import java.util.HashSet;
import java.util.Set;

import tecgraf.javautils.FileUtils;
import tecgraf.openbus.DRMAA.AlreadyActiveSessionException;
import tecgraf.openbus.DRMAA.AuthorizationException;
import tecgraf.openbus.DRMAA.DefaultContactStringException;
import tecgraf.openbus.DRMAA.DeniedByDrmException;
import tecgraf.openbus.DRMAA.DrmCommunicationException;
import tecgraf.openbus.DRMAA.DrmsExitException;
import tecgraf.openbus.DRMAA.DrmsInitException;
import tecgraf.openbus.DRMAA.ExitTimeoutException;
import tecgraf.openbus.DRMAA.InternalException;
import tecgraf.openbus.DRMAA.InvalidArgumentException;
import tecgraf.openbus.DRMAA.InvalidContactStringException;
import tecgraf.openbus.DRMAA.InvalidJobException;
import tecgraf.openbus.DRMAA.InvalidJobTemplateException;
import tecgraf.openbus.DRMAA.JobInfo;
import tecgraf.openbus.DRMAA.JobTemplate;
import tecgraf.openbus.DRMAA.NoActiveSessionException;
import tecgraf.openbus.DRMAA.NoDefaultContactStringSelectedException;
import tecgraf.openbus.DRMAA.OutOfMemoryException;
import tecgraf.openbus.DRMAA.TryLaterException;
import tecgraf.openbus.DRMAA.Version;
import tecgraf.openbus.opendreams.OpenDreamsJobTemplate;
import tecgraf.openbus.opendreams.OpenDreamsJobTemplateImpl;
import csbase.logic.CommandEvent;
import csbase.logic.CommandInfo;
import csbase.logic.CommandSubmission;
import csbase.logic.CommonClientProject;
import csbase.logic.Priority;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.ExecutionType;
import csbase.remote.RemoteObserver;
import csbase.server.Server;
import csbase.server.Service;
import csbase.server.services.commandpersistenceservice.CommandPersistenceService;
import csbase.server.services.openbusservice.OpenBusService;
import csbase.server.services.projectservice.ProjectService;
import csbase.server.services.schedulerservice.SchedulerService;

/**
 * Implementa uma sesso no OpenDreams. Guarda o projeto ao qual essa sesso
 * est associada.
 * 
 * @author Tecgraf PUC-Rio
 */
public class OpenDreamsSession extends Session {

  /**
   * O projeto associado a essa sesso.
   */
  private CommonClientProject project;

  /**
   * Indica se essa sesso j est ativa.
   */
  private boolean isActive = false;
  /**
   * Conjunto de job templates vlidos nessa sesso.
   */
  private Set<JobTemplate> validJobTemplates;
  /**
   * Fila que monitora o trmino da execuo de comandos.
   */
  private JobInfoQueue jobInfoQueue = new JobInfoQueue();
  /**
   * Referncia para o servio.
   */
  private final OpenDreamsService service;
  /**
   * Verso do servidor CSBase.
   */
  private Version drmsVersion;
  /**
   * Nome do servidor CSBase.
   */
  private String drmsInfo;
  /**
   * Nome e verso do servidor CSBase
   */
  private String drmaaImplementation;
  /**
   * Host e porta onde o servidor CSBase est executando
   */
  private String drmsContact;

  /**
   * Construtor.
   */
  public OpenDreamsSession() {
    csbase.logic.Version version = csbase.logic.Version.getInstance();
    this.drmsVersion = new Version(version.getMajorVersion(), version
      .getMinorVersion());
    this.drmsContact = Server.getInstance().getHostName() + ":"
      + Server.getInstance().getPort();
    this.drmsInfo = Server.getInstance().getServerName();
    this.drmaaImplementation = Server.getInstance().getServerName() + " "
      + version.getVersion();
    this.service = OpenDreamsService.getInstance();
  }

  /**
   * {@inheritDoc}
   * 
   * Usa o sistema <b>CSGrid</b> como sistema DRMS. O valor do parmetro
   * <code>contactString</code>, quando no for null ou vazio, deve ser
   * <code>csgrid</code>.
   */
  @Override
  public void init(String contactString) throws DefaultContactStringException,
    NoDefaultContactStringSelectedException, InvalidArgumentException,
    AlreadyActiveSessionException, InternalException, DrmsInitException,
    AuthorizationException, DrmCommunicationException,
    InvalidContactStringException, OutOfMemoryException {
    String userId = null;
    try {
      service
        .logDebugMessage("OpenDreamsSession: init (" + contactString + ")");
      userId = OpenBusService.getInstance().initCSBaseAccess();
      if (contactString != null && !contactString.equals("")) {
        if (!contactString.equals(drmsContact)) {
          throw new InvalidContactStringException(
            "No existe um DRMS com nome " + contactString);
        }
      }
      if (this.isActive) {
        throw new AlreadyActiveSessionException("A sesso j est ativa");
      }
      this.isActive = true;
      this.validJobTemplates = new HashSet<JobTemplate>();
    }
    catch (InvalidContactStringException e) {
      String msg = "Erro na inicializao da sesso do usurio " + userId;
      service.logSevereMessage(msg, e);
      throw e;
    }
    catch (AlreadyActiveSessionException e) {
      String msg = "Erro na inicializao da sesso do usurio " + userId;
      service.logSevereMessage(msg, e);
      throw e;
    }
    catch (Throwable e) {
      String msg = "Erro na inicializao da sesso do usurio " + userId;
      service.logSevereMessage(msg, e);
      throw new InternalException(OpenDreams.formatMessage(e, msg));
    }
    finally {
      OpenBusService.getInstance().finishCSBaseAccess();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void exit() throws OutOfMemoryException, DrmsExitException,
    AuthorizationException, NoActiveSessionException, InternalException,
    DrmCommunicationException {
    String userId = null;
    try {
      service.logDebugMessage("OpenDreamsSession: exit");
      userId = OpenBusService.getInstance().initCSBaseAccess();
      if (!this.isActive) {
        throw new NoActiveSessionException("A sesso j est fechada");
      }
      this.validJobTemplates.clear();
      this.validJobTemplates = null;
    }
    catch (NoActiveSessionException e) {
      String msg = "Erro na finalizao da sesso do usurio " + userId;
      service.logSevereMessage(msg, e);
      throw e;
    }
    catch (Throwable e) {
      String msg = "Erro na finalizao da sesso do usurio " + userId;
      service.logSevereMessage(msg, e);
      throw new InternalException(OpenDreams.formatMessage(e, msg));
    }
    finally {
      OpenBusService.getInstance().finishCSBaseAccess();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public JobTemplate createJobTemplate() throws OutOfMemoryException,
    AuthorizationException, NoActiveSessionException, InternalException,
    DrmCommunicationException {
    String userId = null;
    try {
      service.logDebugMessage("OpenDreamsSession: createJobTemplate");
      userId = OpenBusService.getInstance().initCSBaseAccess();
      if (!this.isActive) {
        throw new NoActiveSessionException("A sesso j est fechada");
      }
      JobTemplate jt = new OpenDreamsJobTemplateImpl();
      this.validJobTemplates.add(jt);
      return jt;
    }
    catch (NoActiveSessionException e) {
      String msg = "Erro na criao de um jobtemplate pelo usurio " + userId;
      service.logSevereMessage(msg, e);
      throw e;
    }
    catch (Throwable e) {
      String msg = "Erro na criao de um jobtemplate pelo usurio " + userId;
      service.logSevereMessage(msg, e);
      throw new InternalException(OpenDreams.formatMessage(e, msg));
    }
    finally {
      OpenBusService.getInstance().finishCSBaseAccess();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void deleteJobTemplate(JobTemplate jt) throws OutOfMemoryException,
    InvalidJobTemplateException, AuthorizationException,
    NoActiveSessionException, InvalidArgumentException, InternalException,
    DrmCommunicationException {
    String userId = null;
    try {
      service.logDebugMessage("OpenDreamsSession: deleteJobTemplate");
      userId = OpenBusService.getInstance().initCSBaseAccess();
      if (!this.isActive) {
        throw new NoActiveSessionException("A sesso j est fechada");
      }
      if (!isValidJobTemplate(jt)) {
        throw new InvalidJobTemplateException("No  um job template vlido");
      }
      this.validJobTemplates.remove(jt);
    }
    catch (NoActiveSessionException e) {
      String msg = "Erro na remoo de um jobtemplate pelo usurio " + userId;
      service.logSevereMessage(msg, e);
      throw e;
    }
    catch (InvalidJobTemplateException e) {
      String msg = "Erro na remoo de um jobtemplate pelo usurio " + userId;
      service.logSevereMessage(msg, e);
      throw e;
    }
    catch (Throwable e) {
      String msg = "Erro na remoo de um jobtemplate pelo usurio " + userId;
      service.logSevereMessage(msg, e);
      throw new InternalException(OpenDreams.formatMessage(e, msg));
    }
    finally {
      OpenBusService.getInstance().finishCSBaseAccess();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String runJob(JobTemplate jt) throws InvalidArgumentException,
    InternalException, AuthorizationException, DrmCommunicationException,
    DeniedByDrmException, InvalidJobTemplateException,
    NoActiveSessionException, TryLaterException, OutOfMemoryException {
    String userId = null;
    try {
      service.logDebugMessage("OpenDreamsSession: runJob");
      userId = OpenBusService.getInstance().initCSBaseAccess();

      if (!this.isActive) {
        throw new NoActiveSessionException("A sesso j est fechada");
      }
      if (!isValidJobTemplate(jt)) {
        throw new InvalidJobTemplateException("No  um job template vlido");
      }
      if (OpenDreamsJobTemplate.class.isInstance(jt)) {
        String result = execAlgorithm(userId, OpenDreamsJobTemplate.class
          .cast(jt));
        return result;
      }
      return null;
    }
    catch (NoActiveSessionException e) {
      String msg = "Erro na submisso do comando pelo usurio " + userId;
      service.logSevereMessage(msg, e);
      throw e;
    }
    catch (InvalidJobTemplateException e) {
      String msg = "Erro na submisso do comando pelo usurio " + userId;
      service.logSevereMessage(msg, e);
      throw e;
    }
    catch (AlgorithmTemplateException e) {
      String msg = "Erro na submisso do comando pelo usurio " + userId;
      service.logSevereMessage(msg, e);
      throw new DeniedByDrmException(OpenDreams.formatMessage(e, msg));
    }
    catch (Throwable e) {
      String msg = "Erro na submisso do comando pelo usurio " + userId;
      service.logSevereMessage(msg, e);
      throw new InternalException(OpenDreams.formatMessage(e, msg));
    }
    finally {
      OpenBusService.getInstance().finishCSBaseAccess();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public JobInfo _wait(String jobName, long timeout)
    throws OutOfMemoryException, AuthorizationException,
    NoActiveSessionException, ExitTimeoutException, InvalidArgumentException,
    InternalException, InvalidJobException, DrmCommunicationException {
    String userId = null;
    try {
      service.logDebugMessage("OpenDreamsSession: _wait (" + jobName + ", "
        + timeout + ")");
      userId = OpenBusService.getInstance().initCSBaseAccess();
      if (jobName.equals(Session.JOB_IDS_SESSION_ANY)) {
        return jobInfoQueue.waitForAnyJob(timeout);
      }
      else {
        return jobInfoQueue.waitFor(jobName, timeout);
      }
    }
    catch (ExitTimeoutException e) {
      service.logSevereMessage(
        "Erro na espara pelo trmino do comando pelo usurio " + userId, e);
      throw e;
    }
    catch (InvalidJobException e) {
      service.logSevereMessage(
        "Erro na espara pelo trmino do comando pelo usurio " + userId, e);
      throw e;
    }
    catch (Throwable e) {
      service.logSevereMessage(
        "Erro na espara pelo trmino do comando pelo usurio " + userId, e);
      throw new InternalException(e.getMessage());
    }
    finally {
      OpenBusService.getInstance().finishCSBaseAccess();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String contact() {
    return this.drmsContact;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Version version() {
    return this.drmsVersion;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String drmaaImplementation() {
    return this.drmaaImplementation;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String drmsInfo() {
    return this.drmsInfo;
  }

  /**
   * Altera o projeto associado a essa sesso. O projeto j deve ter sido
   * aberto.
   * 
   * @param project o projeto associado a essa sesso.
   */
  public void setProject(CommonClientProject project) {
    this.project = project;
  }

  /**
   * Obtm o projeto associado a essa sesso.
   * 
   * @return o projeto associado a essa sesso.
   */
  public CommonClientProject getProject() {
    return this.project;
  }

  /**
   * Verifica se um job template est vlido.
   * 
   * @param jt o job template a ser validado
   * @return verdadeiro, se o job template  vlido e falso, caso contrrio.
   */
  private boolean isValidJobTemplate(JobTemplate jt) {
    return validJobTemplates.contains(jt);
  }

  /**
   * Executa um algoritmo especificado por um job template do opendreams.
   * 
   * @param userId o identificador do usurio que fez a submisso
   * @param jt o job tempate com os argumentos para execuo
   * @return o identificador do comando submetido no opendreams
   * @throws AlgorithmTemplateException se houver um erro na descrio do job
   *         template
   */
  private String execAlgorithm(final String userId,
    final OpenDreamsJobTemplate jt) throws AlgorithmTemplateException {
    /* Faz a validao dos atributos do job template */
    AlgorithmTemplate validator;
    validator = new AlgorithmTemplate(project, jt);
    /* Se o modo de execuo do algoritmo for mltipla, altera para simples */
    final AlgorithmConfigurator configurator = validator
      .getAlgorithmConfigurator();
    if (configurator.getExecutionType().equals(ExecutionType.MULTIPLE)) {
      configurator.setExecutionType(ExecutionType.SIMPLE);
    }
    /* Cria o comando de submisso de job */
    CommandSubmission submission = new CommandSubmission(configurator, project
      .getId());
    submission.setDescription(jt.getJobDescription());
    submission.setPriority(Priority.values()[jt.getJobPriority()]);
    submission.setEmailList(jt.getEmail());
    submission.setMailAtEnd(!jt.getBlockEmail());
    if (submission.getExecutionType().equals(ExecutionType.PARALLEL)) {
      submission.configureParallelExecution(jt.getNumberOfProcesses());
    }
    if (submission.getExecutionType().equals(ExecutionType.DISTRIBUTED)) {
      submission.configureDistributedExecution(jt.getNumberOfProcesses());
    }
    /* Submete o comando para execuo */
    RemoteObserver observer = new RemoteObserver() {
      public void update(Object event) {
        commandFinished((CommandEvent) event, jt, userId);
      }

      @Override
      public void install() throws RemoteException {
      }

      @Override
      public void uninstall() throws RemoteException {
      }
    };
    Set<CommandInfo> commandInfos = SchedulerService.getInstance()
      .submitCommand(submission, observer);
    String cmdId = commandInfos.toArray(new CommandInfo[0])[0].getId();
    jobInfoQueue.jobSubmited(cmdId);
    return cmdId;
  }

  /**
   * Trata o fim da execuo de um comando submetido.
   * 
   * @param event o evento de trmino de execuo
   * @param template o job template correspondente a submisso
   * @param userId o identificador do usurio que fez a submisso
   */
  private void commandFinished(CommandEvent event,
    OpenDreamsJobTemplate template, String userId) {
    try {
      if (template.getOutputPath() != null
        && !template.getOutputPath().isEmpty()) {
        Service.setUserId(userId);
        String jobId = event.getCommandId().toString();
        CommandInfo info = CommandPersistenceService.getInstance()
          .getCommandInfo(project.getId(), jobId);
        // Caminho para o arquivo de sada definido em outputFilePath
        // Esse  o arquivo a ser criado.
        String outputFilePath = FileUtils.joinPath(project.getName(), template
          .getOutputPath());
        // Remove o arquivo definido em outputFilePath se j existir um com o
        // mesmo nome
        if (ProjectService.getInstance().existsFile(project.getId(),
          FileUtils.splitPath(outputFilePath))) {
          ProjectService.getInstance().removeFile(project.getId(),
            FileUtils.splitPath(outputFilePath));
        }
        // Caminho do arquivo de sada default criado pela execuo
        String defaultOutputFilePath = FileUtils.joinPath(project.getName(),
          info.getConfigurator().getStandardOutputFile().getPath());
        // Caminho para o diretrio do arquivo de sada definido em
        // outputFilePath
        String outputDirPath = FileUtils.getFilePath(outputFilePath);
        // Nome do arquivo que deve ser criado no diretrio do projeto
        // Esse nome  o que est especificado em outputFilePath
        String newFileName = FileUtils.getFileName(outputFilePath);
        // Copia o arquivo de sada default para o diretrio do projeto
        ProjectService.getInstance().copyFile(project.getId(),
          FileUtils.splitPath(defaultOutputFilePath),
          FileUtils.splitPath(outputDirPath), newFileName);
      }
    }
    catch (Throwable e) {
      service.logSevereMessage("Erro no tratamento de fim de comando", e);
    }
    jobInfoQueue.jobExit(event);
  }
}
