/*
 * FileConfig.java
 * 
 * $Author: rodrigoh $
 * $Date: 2008-03-26 18:00:05 -0300 (Wed, 26 Mar 2008) $
 * $Revision: 74166 $
 */
package csbase.server.services.projectservice;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Enumeration;
import java.util.Hashtable;

import csbase.logic.ProjectFileType;

/**
 * Modela um arquivo de configurao. A configurao descreve um arquivo na rea
 * de projetos ou na rea global. O arquivo de configurao contm as seguintes
 * informaes.
 * <ul>
 * <li>se o arquivo  um link e, nesse caso, para qual arquivo.</li>
 * <li>se o arquivo est na rea global (de arquivos da base).</li>
 * <li>se o arquivo est na rea de publicados.</li>
 * <li>o tipo do arquivo.</li>
 * <li>o contador de links para esse arquivo.</li>
 * <li>se o arquivo est em construo.</li>
 * <li>o dono do arquivo (no caso dele ter sido publicado).</li>
 * <li>a lista de usurios para os quais esse arquivo foi publicado.</li>
 * </ul>
 */
public class FileConfig {
  /** Extenso dos arquivos de configurao. */
  private static final String CONFIG_EXTENSION = ".csbase";

  /** Extenso dos arquivos de descrio. */
  private static final String DESCRIPTION_EXTENSION = ".csbase_description";

  /** Constante de configurao indicativa de link. */
  protected static final String LINK = "LINK";

  /** Constante de configurao indicativa de arquivo global. */
  protected static final String GLOBAL = "GLOBAL";

  /** Constante de configurao indicativa de arquivo publicado. */
  protected static final String PUBLISHED = "PUBLISHED";

  /** Constante de configurao indicativa de tipo do arquivo. */
  protected static final String TYPE = "TYPE";

  /** Constante de configurao indicativa de contator de referncias. */
  protected static final String COUNT = "COUNT";

  /** Constante de configurao indicativa de arquivo em construo. */
  protected static final String UNDERCONSTRUCTION = "UNDERCONSTRUCTION";

  /**
   * Constante de configurao indicativa de lista de acesso. Atributo antigo,
   * s est sendo mantido para manter compatibilidade com verses em produo.
   */
  protected static final String ACCESS = "ACCESS";

  /**
   * Constante de configurao indicativa de dono do arquivo. Atributo antigo,
   * s est sendo mantido para manter a compatibitidade com verses em
   * produo.
   */
  protected static final String OWNER = "OWNER";

  /** Constante de configurao indicativa do criador do arquivo. */
  protected static final String CREATEDBY = "CREATEDBY";

  /** Constante de configurao indicativa de quem publicou o arquivo. */
  protected static final String PUBLISHEDBY = "PUBLISHEDBY";

  /** Constante de configurao indicativa da estrutura de publicao. */
  protected static final String PUBLISHACCESS = "PUBLISHACCESS";

  /** Constante de configurao indicativa da data de criao do arquivo. */
  protected static final String CREATIONDATE = "CREATIONDATE";
  /** Indica as informaes sobre um arquivo atualizvel. */
  private static final String UPDATABLEFILEINFO = "UPDATABLEFILEINFO";

  /**
   * Prefixo de arquivos de configurao
   */
  private static final String PREFIX = ".";

  /**
   * Representa o arquivo original no sistema de arquivos. Esse arquivo sempre
   * existe, mesmo que seja um link.
   */
  protected File file;

  /**
   * Representa o diretrio no qual est este arquivo. Caso seja o diretrio
   * raiz, seu valor  <code>null</code>.
   */
  protected ServerProjectFile parent;

  /** Indica se o arquivo  um link. */
  protected boolean isLink;

  /** Representa o arquivo real, caso este seja um link. */
  protected File realFile;

  /** Indica se o arquivo  global. Se for, temos um link. */
  protected boolean isGlobal;

  /** Indica se o arquivo  publicado. Se for, temos um link. */
  protected boolean isPublished;

  /** Mantm o estado do arquivo: se est em construo ou no. */
  protected boolean isUnderConstruction;

  /** Indica o tipo do arquivo. */
  protected String type;

  /** Contador de referncias do arquivo. S faz sentido no caso de link. */
  protected int count;

  /** Usurio que criou ou solicitou a criao do arquivo. */
  protected Object createdBy;

  /**
   * Identificador do usurio que publicou o arquivo. S faz sentido para
   * arquivos obtidos por publicao. Indica qual usurio havia publicado quando
   * o arquivo foi obtido.
   */
  protected Object publishedBy;

  /** Identificador do projeto a partir do qual o arquivo foi publicado. */
  protected Object publishedByProject;

  /**
   * Guarda os pares de publicacao: publicado para -> quem publicou. S faz
   * sentido no caso de arquivos publicados.
   */
  protected Hashtable<Object, Object> publishAccess;

  /** Data de criao do arquivo. */
  protected long creationDate;

  /**
   * Informaes referentes a arquivos atualizveis.
   */
  protected UpdatableFileInfo updatableFileInfo;

  /**
   * Carrega as informaes do arquivo de configurao.
   * 
   * @param configFile O arquivo de onde sero extradas as informaes.
   * 
   * @return true, caso o arquivo tenha sido carregado, ou false, caso
   *         contrrio.
   */
  public boolean readConfigurationFile(File configFile) {
    try {
      if (!configFile.isFile()) {
        getService().logWarningMessage(
          "FileConfig:readConfigurationFile: " + configFile.getAbsolutePath()
            + " no  arquivo ou no existe");
        return false;
      }
      ObjectInputStream in =
        new ObjectInputStream(new DataInputStream(new BufferedInputStream(
          new FileInputStream(configFile))));
      boolean flag = true;
      Object owner = null;
      Object[] access = null;
      while (flag) {
        Object name;
        Object value;
        try {
          name = in.readObject();
          if (name == null) {
            flag = false;
            continue;
          }
          value = in.readObject();
        }
        catch (Exception e) {
          getService().logSevereMessage(
            "Erro na leitura de uma propriedade: "
              + " do arquivo de configuracao: " + configFile.getAbsolutePath(),
            e);
          break;
        }
        if (name.equals(LINK)) {
          isLink = true;
          realFile = new File((String) value);
        }
        else if (name.equals(GLOBAL)) {
          isGlobal = ((Boolean) value).booleanValue();
        }
        else if (name.equals(PUBLISHED)) {
          isPublished = ((Boolean) value).booleanValue();
          publishedByProject = getPublishedProject(realFile);
        }
        else if (name.equals(TYPE)) {
          type = (String) value;
        }
        else if (name.equals(COUNT)) {
          count = ((Integer) value).intValue();
        }
        else if (name.equals(UNDERCONSTRUCTION)) {
          isUnderConstruction = ((Boolean) value).booleanValue();
        }

        /* Mantm compatibilidade com projetos antigos */
        else if (name.equals(ACCESS)) {
          access = (Object[]) value;
          if ((access != null) && (owner != null)) {
            makeFileAccess(owner, access);
          }
        }

        /* Mantm compatibilidade com projetos antigos */
        else if (name.equals(OWNER)) {
          owner = value;
          if ((owner != null) && (access != null)) {
            makeFileAccess(owner, access);
          }
        }
        else if (name.equals(CREATEDBY)) {
          createdBy = value;
        }
        else if (name.equals(PUBLISHEDBY)) {
          publishedBy = value;
        }
        else if (name.equals(PUBLISHACCESS)) {
          addPublish((Hashtable<Object, Object>) value);
        }
        else if (name.equals(CREATIONDATE)) {
          creationDate = ((Long) value).longValue();
        }
        else if (name.equals(UPDATABLEFILEINFO)) {
          this.updatableFileInfo = (UpdatableFileInfo) value;
        }
        else {
          getService().logWarningMessage(
            "FileConfig:readConfigurationFile: " + name + " ignorado em "
              + configFile.getAbsolutePath());
        }
      }
      in.close();
      return true;
    }
    catch (Throwable e) {
      getService().logWarningMessage(
        "FileConfig:readConfigurationFile: " + configFile.getAbsolutePath()
          + " corrompido." + " -> " + e.getMessage());
      return false;
    }
  }

  /**
   * Cria a estrutura que representa para quem um arquivo est publicado.
   * 
   * @param owner O identificador do dono do arquivo publicado.
   * @param access Os identificadores dos usurios para os quais o arquivo foi
   *        publicado.
   */
  private void makeFileAccess(Object owner, Object[] access) {
    this.publishedBy = owner;
    if (publishAccess == null) {
      publishAccess = new Hashtable<Object, Object>();
    }
    for (int i = 0; i < access.length; i++) {
      publishAccess.put(access[i], owner);
    }
  }

  /**
   * Adiciona os pares de publicacao: publicado para -> quem publicou.
   * 
   * @param pub Os pares de publicao.
   */
  private void addPublish(Hashtable<Object, Object> pub) {
    /*
     * Verifica se j leu a informao de quem publicou do arquivo de de
     * configurao de link
     */
    if (pub == null) {
      return;
    }
    if (publishAccess == null) {
      publishAccess = pub;
    }
    else {
      for (Enumeration<Object> e = pub.keys(); e.hasMoreElements();) {
        Object publishedTo = e.nextElement();
        publishAccess.put(publishedTo, pub.get(publishedTo));
      }
    }
  }

  /**
   * Obtm o identificador do projeto a partir do qual o arquivo foi publicado.
   * 
   * @param file O arquivo real que foi publicado.
   * 
   * @return O identificador do projeto.
   */
  private Object getPublishedProject(File file) {
    if (file == null) {
      return null;
    }
    String id = null;
    try {
      id = ServerProjectFile.findFileProjectIdFromPublish(file);
    }
    catch (ServerProjectFileException e) {
      return null;
    }
    return id;
  }

  /**
   * Grava as informaes do arquivo de configuraes.
   * 
   * @param configFile O arquivo onde as informaes sero gravadas.
   * 
   * @return true, caso o arquivo tenha sido gravado, ou false, caso contrrio.
   */
  public boolean writeConfigurationFile(File configFile) {
    ObjectOutputStream out = null;
    try {
      out =
        new ObjectOutputStream(new DataOutputStream(new BufferedOutputStream(
          new FileOutputStream(configFile))));
      if (isLink) {
        out.writeObject(LINK);
        out.writeObject(realFile.getAbsolutePath());
        out.writeObject(PUBLISHEDBY);
        out.writeObject(publishedBy);
        out.writeObject(null); // EOF
      }
      else {
        out.writeObject(GLOBAL);
        out.writeObject(new Boolean(isGlobal));
        out.writeObject(PUBLISHED);
        out.writeObject(new Boolean(isPublished));
        out.writeObject(TYPE);
        out.writeObject(new String(type));
        out.writeObject(COUNT);
        out.writeObject(new Integer(count));
        out.writeObject(UNDERCONSTRUCTION);
        out.writeObject(new Boolean(isUnderConstruction));
        out.writeObject(CREATEDBY);
        out.writeObject(createdBy);
        out.writeObject(PUBLISHACCESS);
        out.writeObject(publishAccess);
        out.writeObject(CREATIONDATE);
        out.writeObject(new Long(creationDate));
        out.writeObject(UPDATABLEFILEINFO);
        out.writeObject(this.updatableFileInfo);
        out.writeObject(null); // EOF
      }
      out.flush();
      out.close();
      return true;
    }
    catch (Exception e) {
      getService().logWarningMessage(
        "FileConfig:writeConfigurationFile: " + "erro ao gerar "
          + configFile.getAbsolutePath() + "." + " -> " + e.getMessage());
      try {
        if (out != null) {
          out.flush();
          out.close();
        }
      }
      catch (IOException ioe) {
      }
      return false;
    }
  }

  /**
   * Obtm o identificador do projeto a partir do qual este arquivo foi
   * publicado.
   * 
   * @return O identificador do projeto.
   */
  public Object getSourceProjectId() {
    return this.publishedByProject;
  }

  /**
   * Obtm o mapeamento dos usurios que possuem acesso ao arquivo com o usurio
   * que publicou tal arquivo Se o arquivo no estiver publicado, retorna
   * <code>null</code>.
   * 
   * @return Uma mapeamento com os identificadores dos usurios para os quais o
   *         arquivo foi publicado -> quem publicou ou <code>null</code>.
   */
  public Hashtable<Object, Object> getAccess() {
    return publishAccess;
  }

  /**
   * Obtm a lista de usurios que possuem acesso ao arquivo.
   * 
   * @return Um array com os identificadores dos usurios para os quais o
   *         arquivo foi publicado ou null
   */
  public Object[] getPublishAccess() {
    if (publishAccess == null) {
      return null;
    }
    Object[] access = new Object[publishAccess.size()];
    int i = 0;
    for (Enumeration<Object> e = publishAccess.keys(); e.hasMoreElements();) {
      access[i++] = e.nextElement();
    }
    return access;
  }

  /**
   * Obtm o identificador do usurio que publicou o arquivo principal.
   * 
   * @param publishedTo Identificador do usurio para quem foi publicado.
   * 
   * @return O identificador do usurio que publicou o arquivo.
   */
  public Object whoPublished(Object publishedTo) {
    if (publishedTo == null) {
      return null;
    }
    return publishAccess.get(publishedTo);
  }

  /**
   * Obtm a data de criao do arquivo.
   * 
   * @return data de criao em milesegundos.
   */
  public long getCreationDate() {
    return creationDate;
  }

  /**
   * Obtm o arquivo correspondente a essa configurao.
   * 
   * @return O arquivo descrito pela configurao.
   */
  public File getFile() {
    return file;
  }

  /**
   * Obtm a instncia do servio de projetos.
   * 
   * @return A instncia do servio de projetos.
   */
  ProjectService getService() {
    return ProjectService.getInstance();
  }

  /**
   * Indica se o arquivo  um arquivo de configurao.
   * 
   * @param file o arquivo
   * @return true se  um arquivo de configurao
   */
  public static final boolean isConfigFile(final File file) {
    final String fileName = file.getName();
    return isConfigFileName(fileName);
  }

  /**
   * Indica se o arquivo  um arquivo de descrio.
   * 
   * @param file o arquivo
   * @return true se  um arquivo de descrio
   */
  public static final boolean isDescriptionFile(final File file) {
    final String fileName = file.getName();
    return isDescriptionFileName(fileName);
  }

  /**
   * Indica se o nome do arquivo  um arquivo de configurao.
   * 
   * @param fileName o nome do arquivo
   * @return true se  o nome  de um arquivo de configurao
   */
  public static final boolean isConfigFileName(final String fileName) {
    if (!fileName.startsWith(FileConfig.PREFIX))
      return false;
    if (!fileName.endsWith(FileConfig.CONFIG_EXTENSION))
      return false;
    return true;
  }

  /**
   * Indica se o nome do arquivo  um arquivo de descrio.
   * 
   * @param fileName o nome do arquivo
   * @return true se  o nome  de um arquivo de descrio
   */
  public static final boolean isDescriptionFileName(final String fileName) {
    if (!fileName.startsWith(FileConfig.PREFIX))
      return false;
    if (!fileName.endsWith(FileConfig.DESCRIPTION_EXTENSION))
      return false;
    return true;
  }

  /**
   * Indica se o nome do arquivo  um arquivo de controle (configurao ou
   * descrio).
   * 
   * @param fileName o nome do arquivo
   * @return true se o nome do arquivo  um arquivo de controle (configurao ou
   *         descrio)
   */
  public static final boolean isControlledFileName(final String fileName) {
    return isConfigFileName(fileName) || isDescriptionFileName(fileName);
  }

  /**
   * Obtm um objeto<code>File</code> renomeando um outro <code>File</code>.
   * O arquivo no  renomeado fisicamente. Apenas  criada um objeto
   * <code>File</code> que representa esse arquivo com outro nome.
   * 
   * @param file O objeto <code>File</code> original.
   * @param name O nome do arquivo representado pelo novo objeto
   *        <code>File</code>.
   * 
   * @return Um <code>File</code> que representa o novo nome.
   */
  public static File renamedFile(File file, String name) {
    String fileName = file.getName();
    String filePath = file.getAbsolutePath();
    String dir = filePath.substring(0, filePath.length() - fileName.length());
    return new File(dir + name);
  }

  /**
   * Cria um novo <code>File</code> com nome de configurao.
   * 
   * @see #createConfigFileName(String)
   * @param file o arquivo origem
   * @return um novo <code>File</code>
   */
  public static File renamedFileConfig(File file) {
    final String fileName = file.getName();
    return renamedFile(file, createConfigFileName(fileName));
  }

  /**
   * Cria um novo <code>File</code> com nome de arquivo de descrio.
   * 
   * @see #createDescriptionFileName(String)
   * @param file o arquivo origem
   * @return um novo <code>File</code> com nome de arquivo de descrio
   */
  public static File renamedFileDescription(File file) {
    final String fileName = file.getName();
    return renamedFile(file, createDescriptionFileName(fileName));
  }

  /**
   * Cria o nome de arquivo de configurao.
   * 
   * @param fileName nome do arquivo de origem
   * @return o nome de arquivo de configurao
   */
  public static String createConfigFileName(final String fileName) {
    final String prefix = FileConfig.PREFIX;
    final String ext = FileConfig.CONFIG_EXTENSION;
    return prefix + fileName + ext;
  }

  /**
   * Cria o nome de arquivo de descrio.
   * 
   * @param fileName nome do arquivo de origem
   * @return o nome de arquivo de descrio
   */
  public static String createDescriptionFileName(final String fileName) {
    final String prefix = FileConfig.PREFIX;
    final String ext = FileConfig.DESCRIPTION_EXTENSION;
    return prefix + fileName + ext;
  }

  /**
   * Cria um arquivo de configurao.
   */
  public FileConfig() {
    this.isLink = false;
    this.isGlobal = false;
    this.isPublished = false;
    this.isUnderConstruction = false;
    this.type = ProjectFileType.UNKNOWN;
    this.count = 0;
    this.createdBy = null;
    this.publishedBy = null;
    this.publishedByProject = null;
    this.publishAccess = null;
    this.creationDate = 0;
  }
}
