/*
 * ProjectAdministrator.java
 * 
 * $Id: ProjectAdministrator.java 104847 2010-04-29 16:47:30Z jnlopes $
 */
package csbase.server.services.projectservice;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Calendar;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Map;
import java.util.Vector;

import csbase.logic.ProjectAdminInfo;
import csbase.logic.ProjectAllocationState;
import csbase.logic.User;
import csbase.server.FileSystem;

/**
 * Modela a administrao dos projetos dos usurios. Os projetos podem estar nos
 * seguintes estados: - Desbloqueado - Bloqueado - Removido Os projetos podem
 * ter ou no pedido de reserva de rea de alocao de disco (mount). O
 * administrador pode ou no liberar a rea solicitada e, liberando, deve
 * informar quanto de rea foi alocada. Quando um projeto  removido, no caso
 * dele possuir uma rea alocada (mount), o administrador deve indicar que a
 * rea foi liberada.
 * 
 * @author Tecgraf/PUC-Rio
 */
class ProjectAdministrator {

  /**
   * Nome do arquivo que guarda as informaes de projetos bloqueados.
   */
  private static final String LOCKED_PROJECTS_FILE =
    FileSystem.PROJECT_DIR + File.separator + "locked_projects.csbase";

  /**
   * Caractere que ser utilizado para separar os campos de cada entrada do
   * arquivo locked_projects.csbase (que persiste as informaes dos objetos
   * ProjectAdminInfo).
   */
  private static final String SEPARATOR = "/";

  /**
   * Guarda a instncia do servio de projetos.
   */
  private static ProjectService service;

  /**
   * Guarda as informaes de projetos bloqueados. Mapeia userId (Object) em
   * informao de um projeto (ProjectAdminInfo).
   * 
   * Todos os mtodos que incluem / removem / alteram entradas desta hashtable
   * devem ser synchronized e persistir o resultado, atravs do mtodo {@link ProjectAdministrator#saveLockedProjects()}.
   */
  private Hashtable<Object, Hashtable<String, ProjectAdminInfo>> lockedProjects =
    new Hashtable<Object, Hashtable<String, ProjectAdminInfo>>();

  /**
   * Finaliza o administrador.
   */
  void finish() {
    saveLockedProjects();
  }

  /**
   * Obtm as informaes de todos os projetos dos usurios.
   * 
   * @return um array de <code>ProjectAdminInfo</code>.
   */
  ProjectAdminInfo[] getAllProjectAdminInfo() {
    final Vector<ProjectAdminInfo> projs = new Vector<ProjectAdminInfo>();
    for (Map<String, ProjectAdminInfo> map : lockedProjects.values()) {
      projs.addAll(map.values());
    }

    return (projs.toArray(new ProjectAdminInfo[0]));
  }

  /**
   * Obtm a informao de um projeto de um usurio.
   * 
   * @param userId o identificador do usurio
   * @param projectName o nome do projeto
   * 
   * @return um <code>ProjectAdminInfo</code> do projeto de um usurio.
   */
  ProjectAdminInfo getProjectAdminInfo(Object userId, String projectName) {
    Hashtable<String, ProjectAdminInfo> userProjects =
      lockedProjects.get(userId);
    if (userProjects == null) {
      return null;
    }
    ProjectAdminInfo info = userProjects.get(projectName);
    return info;
  }

  /**
   * Remove a informao de um projeto de um usurio.
   * 
   * @param userId o identificador do usurio
   * @param projectName o nome do projeto
   * 
   * @return um <code>ProjectAdminInfo</code> do projeto de um usurio.
   */
  synchronized ProjectAdminInfo removeProjectAdminInfo(final Object userId,
    final String projectName) {
    final Hashtable<String, ProjectAdminInfo> userProjects =
      lockedProjects.get(userId);
    if (userProjects == null) {
      return null;
    }
    final ProjectAdminInfo info = userProjects.remove(projectName);
    saveLockedProjects();
    return info;
  }

  /**
   * Bloqueia um projeto por reserva de rea.
   * 
   * @param userId o identificador do usurio
   * @param projectName o nome do projeto
   * @param size a rea requisitada
   * 
   * @return A informao de administrao criada para o novo projeto.
   */
  synchronized ProjectAdminInfo lockProject(final Object userId,
    final String projectName, final long size) {
    final ProjectAdminInfo info =
      new ProjectAdminInfo(userId, projectName, service.getRoot(userId,
        projectName).getAbsolutePath(),
        ProjectAllocationState.WAITING_AREA_ALLOCATION);
    info.setAreaRequestSize(size);
    info.setAreaRequestDate(Calendar.getInstance().getTime());
    Hashtable<String, ProjectAdminInfo> userProjects =
      lockedProjects.get(userId);
    if (userProjects == null) {
      userProjects = new Hashtable<String, ProjectAdminInfo>();
      lockedProjects.put(userId, userProjects);
    }
    userProjects.put(projectName, info);
    saveLockedProjects();
    return info;
  }

  /**
   * Desbloqueia um projeto devido a alocao de uma rea.
   * 
   * @param userId o identificador do usurio
   * @param projectName o nome do projeto
   * @param size rea reservada
   * 
   * @return a informao de administrao de projeto.
   * @throws ServerProjectException
   */
  synchronized ProjectAdminInfo unlockProject(final Object userId,
    final String projectName, final long size) throws ServerProjectException {
    final ProjectAdminInfo info = getProjectAdminInfo(userId, projectName);
    if (info == null) {
      throw new ServerProjectException("ProjectAdministrator:unlockProject: "
        + "no existe informao administrativa do projeto " + projectName
        + " do usurio " + userId);
    }
    info.setState(ProjectAllocationState.UNLOCKED_WITH_AREA_ALLOCATED);
    info.setAreaLockedSize(size);
    saveLockedProjects();
    return info;
  }

  /**
   * Retira a informao de administrao de um projeto removido que teve sua
   * rea liberada.
   * 
   * @param userId O identificador do usurio
   * @param projectName O nome do projeto
   * 
   * @return A informao de administrao de projeto.
   * @throws ServerProjectException
   */
  ProjectAdminInfo freeProjectArea(final Object userId, final String projectName)
    throws ServerProjectException {
    final ProjectAdminInfo info = removeProjectAdminInfo(userId, projectName);
    if (info == null) {
      throw new ServerProjectException("ProjectAdministrator:freeProject: "
        + "no existe informao administrativa do projeto " + projectName
        + " do usurio " + userId);
    }
    saveLockedProjects();
    return info;
  }

  /**
   * Informa que um projeto foi removido e aguarda liberao da rea reservada.
   * 
   * @param userId o identificador do usurio
   * @param projectName o nome do projeto
   * 
   * @return O objeto ProjectAdminInfo modificado.
   * @throws ServerProjectException
   */
  synchronized ProjectAdminInfo setAllocatedProjectRemoved(final Object userId,
    final String projectName) throws ServerProjectException {
    final ProjectAdminInfo info = getProjectAdminInfo(userId, projectName);
    if (info == null) {
      throw new ServerProjectException("ProjectAdministrator:removedProject: "
        + "no existe informao administrativa do projeto " + projectName
        + " do usurio " + userId);
    }
    info.setState(ProjectAllocationState.WAITING_AREA_FREE);
    saveLockedProjects();
    return info;
  }

  /**
   * Verifica se um projeto est bloqueado.
   * 
   * @param userId o identificador do usurio
   * @param projectName o nome do projeto
   * 
   * @return verdadeiro, se o projeto est bloqueado ou falso, caso contrrio.
   */
  boolean isLocked(final Object userId, final String projectName) {
    final ProjectAdminInfo info = getProjectAdminInfo(userId, projectName);
    if (info == null) {
      return false;
    }
    return info.isLocked();
  }

  /**
   * Verifica se um projeto est desbloqueado utilizando rea reservada.
   * 
   * @param userId o identificador do usurio
   * @param projectName o nome do projeto
   * 
   * @return verdadeiro, se o projeto est desbloqueado utilizando rea
   *         reservada ou falso, caso contrrio.
   */
  boolean isUnlockedWithAreaAllocated(Object userId, String projectName) {
    ProjectAdminInfo info = getProjectAdminInfo(userId, projectName);
    if (info == null) {
      return false;
    }
    return info.isUnlockedWithAreaAllocated();
  }

  /**
   * Inicializa as informaes sobre projetos bloqueados.
   */
  private synchronized void loadLockedProjects() {
    try {
      BufferedReader in =
        new BufferedReader(new FileReader(LOCKED_PROJECTS_FILE));
      String nextLine = null;
      String[] tokens = null;
      while ((nextLine = in.readLine()) != null) {
        tokens = nextLine.split(SEPARATOR);
        Object userId = tokens[0];
        String projectName = tokens[1];
        int stateInt = Integer.parseInt(tokens[2]);
        final ProjectAllocationState state =
          ProjectAllocationState.getProjectAllocationStateFromCode(stateInt);
        long reqSize = Long.parseLong(tokens[3]);
        long reqDate = Long.parseLong(tokens[4]);
        long lockSize = Long.parseLong(tokens[5]);
        ProjectAdminInfo info =
          new ProjectAdminInfo(userId, projectName, service.getRoot(userId,
            projectName).getAbsolutePath(), state);
        info.setAreaRequestSize(reqSize);
        info.setAreaRequestDate(new Date(reqDate));
        info.setAreaLockedSize(lockSize);
        Hashtable<String, ProjectAdminInfo> userProjects =
          lockedProjects.get(userId);
        if (userProjects == null) {
          userProjects = new Hashtable<String, ProjectAdminInfo>();
          lockedProjects.put(userId, userProjects);
        }
        userProjects.put(projectName, info);
      }
    }
    catch (FileNotFoundException e) {
      service.logWarningMessage("ProjectAdministrator: arquivo "
        + LOCKED_PROJECTS_FILE + " no existe");
    }
    catch (IOException e) {
      service.logSevereMessage("ProjectAdministrator: erro de leitura no "
        + "arquivo " + LOCKED_PROJECTS_FILE, e);
    }
  }

  /**
   * Guarda em arquivo todas as informaes de projetos bloqueados.
   */
  private synchronized void saveLockedProjects() {
    try {
      BufferedWriter out =
        new BufferedWriter(new FileWriter(LOCKED_PROJECTS_FILE));
      for (Enumeration<Object> keys = lockedProjects.keys(); keys
        .hasMoreElements();) {
        Object userId = keys.nextElement();
        Hashtable<String, ProjectAdminInfo> projects =
          lockedProjects.get(userId);
        if (projects == null) {
          continue;
        }
        for (Enumeration<ProjectAdminInfo> elem = projects.elements(); elem
          .hasMoreElements();) {
          ProjectAdminInfo info = elem.nextElement();
          if (info == null) {
            break;
          }
          User user = User.getUser(userId);
          Date date = info.getAreaRequestDate();
          String dateText =
            (date == null) ? "" : String.valueOf(date.getTime());
          out.write(user.getLogin());
          out.write(SEPARATOR);
          out.write(info.getProjectName());
          out.write(SEPARATOR);
          out.write(String.valueOf(info.getState().getCode()));
          out.write(SEPARATOR);
          out.write(String.valueOf(info.getAreaRequestSize()));
          out.write(SEPARATOR);
          out.write(dateText);
          out.write(SEPARATOR);
          out.write(String.valueOf(info.getAreaLockedSize()));
          out.write(SEPARATOR);
          out.newLine();
        }
      }
      out.close();
    }
    catch (FileNotFoundException e) {
      System.out.println(" ARquivo no existe");
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * Construtor.
   * 
   * @param service O servio de projetos.
   */
  ProjectAdministrator(ProjectService service) {
    ProjectAdministrator.service = service;
    loadLockedProjects();
  }
}
