/*
 * $Id$
 */
package csbase.server.services.projectservice;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;

import csbase.exception.ServiceFailureException;
import csbase.server.Service;
import csbase.server.services.loginservice.LoginService;
import csbase.server.services.loginservice.LoginServiceListener;

/**
 * Responsvel por agendar atualizaes de arquivos de projeto.
 * 
 * @author Tecgraf/PUC-Rio
 */
final class ProjectFileUpdateScheduler {
  /**
   * Prefixo utilizado nos nomes das chaves de traduo.
   */
  private static String LNG_PREFIX = "ProjectFileUpdaterService";
  /**
   * Os responsveis pela atualizao propriamente dita (atualizadores).
   */
  private Map<Class<? extends ProjectFileUpdater>, ProjectFileUpdater> updaters;
  /**
   * Responsvel por agendar as tarefas de atualizaco de arquivos.
   */
  private Timer scheduler;
  /**
   * As tarefas de atualizaco de arquivos.
   */
  private Map<UpdatableFileLocation, TimerTask> tasks;
  /**
   * As atualizaes de arquivo que esto agendadas.
   */
  private Map<UpdatableFileLocation, FileUpdateInfo> updates;
  /**
   * Trata notificaes de <i>logout</i> dos usurios para poder finalizar as
   * suas atualizaes.
   */
  private LoginServiceListenerImpl loginServiceListener;

  /**
   * Cria o agendador de atualizaes.
   * 
   * @param updaters A lista de nomes de classes de atualizadores.
   */
  ProjectFileUpdateScheduler(List<String> updaters) {
    this.updaters =
      new HashMap<Class<? extends ProjectFileUpdater>, ProjectFileUpdater>();
    for (String updaterClassName : updaters) {
      try {
        Class<? extends ProjectFileUpdater> updaterClass =
          (Class<? extends ProjectFileUpdater>) Class.forName(updaterClassName);
        try {
          this.updaters.put(updaterClass, updaterClass.newInstance());
        }
        catch (InstantiationException e) {
          ProjectService.getInstance().logSevereMessage(
            "Falha ao instanciar o atualizador de arquivos da classe {0}",
            new Object[] { updaterClassName }, e);
        }
        catch (IllegalAccessException e) {
          ProjectService.getInstance().logSevereMessage(
            "Falha ao instanciar o atualizador de arquivos da classe {0}",
            new Object[] { updaterClassName }, e);
        }
      }
      catch (ClassNotFoundException e) {
        ProjectService.getInstance().logSevereMessage(
          "No foi possvel carregar o atualizador de arquivos da classe {0}",
          new Object[] { updaterClassName }, e);
      }
    }
    this.scheduler = new Timer(true);
    this.tasks = new HashMap<UpdatableFileLocation, TimerTask>();
    this.updates = new HashMap<UpdatableFileLocation, FileUpdateInfo>();
    this.loginServiceListener = new LoginServiceListenerImpl();
  }

  /**
   * Inicializa o agendador de atualizaes.
   */
  public void init() {
    LoginService.getInstance().addListener(this.loginServiceListener);
  }

  /**
   * Finaliza o agendador de atualizaes.
   */
  public void shutdown() {
    LoginService.getInstance().removeListener(this.loginServiceListener);
    this.scheduler.cancel();
  }

  /**
   * Obtm a localizao de um arquivo atualizvel.
   * 
   * @param projectId O identificador do projeto ao qual o arquivo pertence.
   * @param spf O arquivo atualizvel.
   * 
   * @return A localizao do arquivo atualizvel.
   */
  public UpdatableFileLocation getFileLocation(Object projectId,
    ServerProjectFile spf) {
    Object ownerId;
    try {
      ownerId = ServerProject.getOwnerId((String) projectId);
    }
    catch (ServerProjectException e) {
      ProjectService.getInstance().logSevereMessage(
        "Erro ao obter o dono de um projeto a partir do identificador {0}",
        new Object[] { projectId }, e);
      throw new ServiceFailureException(ProjectService.getInstance().getString(
        LNG_PREFIX + "error.project.not.found"));
    }

    return new UpdatableFileLocation(ownerId, spf.getPath());
  }

  /**
   * Obtm informaes sobre a atualizao de um arquivo.
   * 
   * @param fileLocation A localizao do arquivo.
   * 
   * @return Informaes sobre a atualizao do arquivo.
   */
  public FileUpdateInfo getFileUpdateInfo(UpdatableFileLocation fileLocation) {
    return this.updates.get(fileLocation);
  }

  /**
   * Faz o agendamento da atualizao de um arquivo.
   * 
   * @param projectOwnerId O identificador do dono do projeto ao qual o arquivo
   *        pertence.
   * @param spf O arquivo.
   * @param interval O intervalo (em segundos) de atualizaes sucessivas do
   *        arquivo.
   * 
   * @return {@code true} caso o agendamento seja realizado com sucesso, ou
   *         {@code false}, caso contrrio.
   */
  public boolean startUpdate(Object projectOwnerId,
    final ServerProjectFile spf, long interval) {
    final UpdatableFileInfo info = spf.getUpdatableFileInfo();
    Class<? extends ProjectFileUpdater> updaterClass;
    try {
      updaterClass = info.getServiceClass();
    }
    catch (ClassNotFoundException e) {
      ProjectService.getInstance().logSevereMessage(
        "A classe de um atualizador de arquivo no foi encontrada.", e);
      return false;
    }
    final Object key = Service.getKey();

    final UpdatableFileLocation fileLocation =
      new UpdatableFileLocation(projectOwnerId, spf.getPath());
    final ProjectFileUpdater updater = this.updaters.get(updaterClass);
    TimerTask task;
    synchronized (this.tasks) {
      task = this.tasks.get(fileLocation);
      if (task != null) {
        task.cancel();
        this.scheduler.purge();
      }

      task = new TimerTask() {
        @Override
        public void run() {
          Service.setKey(key);
          if (ProjectService.getInstance().existsFile(spf.getProjectId(),
            fileLocation.getPath())) {
            updater.update(fileLocation, info.getUserData());
          }
          else {
            this.cancel();
          }
        }
      };
      this.tasks.put(fileLocation, task);
      this.updates.put(fileLocation, new FileUpdateInfo(Service.getUser()
        .getLogin(), interval));
      this.loginServiceListener.addFileLocation(fileLocation);
    }

    if (interval > 0) {
      this.scheduler.schedule(task, interval * 1000, interval * 1000);
    }
    else {
      this.scheduler.schedule(task, 0);
    }

    return true;
  }

  /**
   * Solicita a atualizao de um arquivo.<br>
   * OBS: este procedimento  assncrono.
   * 
   * @param userLogin O login do usurio que est executando a operao.
   * @param projectOwnerId O identificador do dono do projeto ao qual o arquivo
   *        pertence.
   * @param spf O arquivo.
   * 
   * @return {@code true}, caso a atualizao seja solicitada, ou {@code false},
   *         caso o Servio do OpenBus esteja desabilitado.
   */
  public boolean singleUpdate(String userLogin, Object projectOwnerId,
    ServerProjectFile spf) {
    Service.setUserId(userLogin);
    return this.startUpdate(projectOwnerId, spf, -1);
  }

  /**
   * ra a atualizao de um arquivo.
   * 
   * @param projectOwnerId O identificador do dono do projeto ao qual o arquivo
   *        pertence.
   * @param spf O arquivo.
   * 
   * @return {@code true} caso a atualizao seja terminada, ou {@code false},
   *         caso contrrio.
   */
  public boolean stopUpdate(Object projectOwnerId, ServerProjectFile spf) {
    synchronized (this.tasks) {
      return this.stopUpdate(new UpdatableFileLocation(projectOwnerId, spf
        .getPath()));
    }
  }

  /**
   * Pra a atualizao de um arquivo.
   * 
   * @param path O caminho para o arquivo cuja atualizao foi solicitada.
   * 
   * @return {@code true} caso a atualizao seja terminada, ou {@code false},
   *         caso contrrio.
   */
  boolean stopUpdate(UpdatableFileLocation path) {
    synchronized (this.tasks) {
      this.loginServiceListener.removeFileLocation(path);
      this.updates.remove(path);
      TimerTask task = this.tasks.remove(path);
      if (task == null) {
        return true;
      }
      task.cancel();
      this.scheduler.purge();
    }
    return true;
  }

  /**
   * /** Pra a atualizao de um conjunto de arquivos.
   * 
   * @param fileLocationSet O conjunto de caminhos dos arquivos cuja atualizao
   *        foi solicitada.
   */
  void stopUpdate(Set<UpdatableFileLocation> fileLocationSet) {
    synchronized (this.tasks) {
      for (UpdatableFileLocation filePath : fileLocationSet) {
        this.stopUpdate(filePath);
      }
    }
  }

  /**
   * Tratador de notificaes do Servio de Login.
   * 
   * @author Tecgraf/PUC-Rio
   */
  private class LoginServiceListenerImpl implements LoginServiceListener {
    /**
     * Informa os arquivos utilizados por um determinado usurio em uma sesso.
     */
    private Map<Object, Set<UpdatableFileLocation>> usersToFileLocationSet;
    /**
     * Informa o usurio responsvel pela atualizao de um arquivo.
     */
    private Map<UpdatableFileLocation, Object> fileLocationToUser;

    /**
     * Cria um tratador de notificaes.
     */
    LoginServiceListenerImpl() {
      this.usersToFileLocationSet =
        new HashMap<Object, Set<UpdatableFileLocation>>();
      this.fileLocationToUser = new HashMap<UpdatableFileLocation, Object>();
    }

    /**
     * Adiciona um arquivo  lista do usurio corrente.
     * 
     * @param fileLocation O caminho do arquivo.
     */
    void addFileLocation(UpdatableFileLocation fileLocation) {
      Object sessionKey = Service.getKey();
      if (sessionKey == null) {
        return;
      }
      Set<UpdatableFileLocation> fileLocationSet =
        this.usersToFileLocationSet.get(sessionKey);
      if (fileLocationSet == null) {
        fileLocationSet = new HashSet<UpdatableFileLocation>();
      }
      fileLocationSet.add(fileLocation);
      this.usersToFileLocationSet.put(sessionKey, fileLocationSet);
      this.fileLocationToUser.put(fileLocation, sessionKey);
    }

    /**
     * Remove um arquivo da lista do usurio corrente.
     * 
     * @param fileLocation O caminho do arquivo.
     */
    void removeFileLocation(UpdatableFileLocation fileLocation) {
      Object sessionKey = this.fileLocationToUser.remove(fileLocation);
      if (sessionKey == null) {
        return;
      }
      Set<UpdatableFileLocation> fileLocationSet =
        this.usersToFileLocationSet.get(sessionKey);
      fileLocationSet.remove(fileLocation);
      if (fileLocationSet.size() == 0) {
        this.usersToFileLocationSet.remove(sessionKey);
      }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void userLoggingOut(String login, Object sessionKey,
      String systemName, long time) {
      Set<UpdatableFileLocation> fileLocationSet =
        this.usersToFileLocationSet.get(sessionKey);
      if (fileLocationSet != null) {
        stopUpdate(new HashSet<UpdatableFileLocation>(fileLocationSet));
      }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void systemNameSet(String login, Object sessionKey,
      String systemName, long time) {
      // Nada a ser feito.
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void sessionCreated(String login, Object sessionKey, long time) {
      // Nada a ser feito.
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void connectionLost(String login, Object sessionKey,
      String systemName, long time) {
      // Nada a ser feito.
    }
  }
}
