package csbase.server.services.projectservice;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Hashtable;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import csbase.exception.ServiceFailureException;
import csbase.logic.CloseProjectNotificationData;
import csbase.logic.CommonProject;
import csbase.logic.CommonProjectInfo;
import csbase.logic.NotificationData;
import csbase.logic.OpenProjectNotificationData;
import csbase.logic.ProjectAttribute;
import csbase.logic.ProjectEvent;
import csbase.logic.ProjectFile;
import csbase.logic.ProjectPermissions;
import csbase.logic.User;
import csbase.logic.ProjectPermissions.SharingType;
import csbase.remote.RemoteObjectObservable;
import csbase.server.FileSystem;
import csbase.server.Service;
import csbase.server.services.notificationservice.NotificationService;

/**
 * Modela um projeto no servidor. Essa classe faz os acessos ao sistema de
 * arquivos para fornecer/atualizar as informaes relativas aos projetos.
 */
public class ServerProject extends RemoteObjectObservable implements
  CommonProject {
  /** Extenso dos arquivos de configurao dos projetos. */
  public static final String INFO_EXTENSION = ".csbase_project_info";

  /**
   * Guarda todos os projetos abertos, para no termos mais de um objeto
   * representando o mesmo projeto. Isso facilita a construo mais eficiente de
   * observadores de projetos. Essa tabela faz um mapeamento entre
   * identificadores de projetos e suas respectivas instncias de
   * {@link ServerProject}.
   */
  private static Hashtable<Object, ServerProject> projects =
    new Hashtable<Object, ServerProject>();

  /** Identificador do projeto. */
  private Object id;

  /** Informaes do projeto. */
  private CommonProjectInfo info;

  /** Raiz (diretrio-raiz) do projeto. */
  private ServerProjectFile tree;

  /** Armazena o diretrio raiz do projeto. */
  private File dir;

  /** Indica quantos clientes esto usando este projeto. */
  private int count;

  /**
   * Caminho de onde esse projeto est instalado, a partir da raiz de todos os
   * projetos.
   */
  private String[] path;

  /**
   * Fornece o identificador do projeto.
   * 
   * @return Identificador do projeto.
   */
  public Object getId() {
    return id;
  }

  /**
   * Fornece o caminho onde o projeto est instalado, a partir da raiz de todos
   * os projetos.
   * 
   * @return Caminho a partir da raiz dos projetos at este projeto
   */
  public String[] getPath() {
    if (path != null) {
      return path;
    }
    User user;
    try {
      user = User.getUser(getUserId());
    }
    catch (Exception e) {
      return path;
    }
    path = new String[3];
    path[0] = FileSystem.PROJECT_DIR;
    path[1] = user.getLogin();
    path[2] = getName();
    return path;
  }

  /**
   * Retorna o caminho absoluto para a raiz desse projeto.
   * 
   * @return o caminho absoluto desse projeto
   */
  public String getAbsolutePath() {
    return tree.getAbsolutePath();
  }

  /**
   * Dado um usurio e um nome de projeto, obtm o identificador desse projeto.
   * 
   * @param userId .
   * @param projectName .
   * 
   * @return .
   */
  public static String getId(Object userId, String projectName) {
    try {
      User user = User.getUser(userId);
      String sep = File.separator;
      return FileSystem.PROJECT_DIR + sep + user.getLogin() + sep + projectName;
    }
    catch (Exception ex) {
      return null;
    }
  }

  /**
   * Dado um identificador de um projeto, obtm o nome desse projeto.
   * 
   * @param projectId .
   * 
   * @return .
   * @throws ServerProjectException Se o parmetro {@code projectId} no estiver
   *         no formato esperado.
   */
  public static String getProjectName(String projectId)
    throws ServerProjectException {
    String sep = File.separator;
    Pattern patt = Pattern.compile(".*" + sep + "(.+?)$");
    Matcher matcher = patt.matcher(projectId);
    if (!matcher.matches()) {
      String message =
        String.format(
          "O texto informado (%s) no est no formato esperado para "
            + "um identificador de projeto.", projectId);
      throw new ServerProjectException(message);
    }
    return matcher.group(1);
  }

  /**
   * Dado um identificador de um projeto, obtm o usurio desse projeto.
   * 
   * @param projectId .
   * 
   * @return .
   * 
   * @throws ServerProjectException Se o parmetro {@code projectId} no tiver o
   *         o formato esperado para um identificador de projeto, ou se o
   *         usurio no est cadastrado no sistema.
   */
  public static Object getOwnerId(String projectId)
    throws ServerProjectException {
    String sep = File.separator;
    Pattern patt = Pattern.compile(sep, Pattern.LITERAL);
    String[] elements = patt.split(projectId);
    int ownerIndex = elements.length - 2;
    if (ownerIndex < 0) {
      String message =
        String.format(
          "O texto informado (%s) no est no formato esperado para "
            + "um identificador de projeto.", projectId);
      throw new ServerProjectException(message);
    }
    String userLogin = elements[ownerIndex];
    User user;
    try {
      user = User.getUserByLogin(userLogin);
    }
    catch (Exception e) {
      String message =
        String.format("Erro ao obter o usurio cujo login  %s.\n", userLogin);
      throw new ServerProjectException(message);
    }
    return user.getId();
  }

  /**
   * Obtem o arquivo de configurao do projeto. Esse arquivo est associado ao
   * diretrio raiz do projeto.
   * 
   * @param dir .
   * 
   * @return .
   */
  protected static File getConfigFile(File dir) {
    File configFile =
      new File(dir.getParentFile(), dir.getName() + INFO_EXTENSION);
    return configFile;
  }

  /**
   * Retorna os nomes de todos os projetos de um usurio (apenas aqueles dos
   * quais  dono, no inclui projetos aos quais apenas tem acesso).
   * 
   * @param userId Identificador do usurio.
   * 
   * @return Array de strings contendo os nomes dos projetos dos quais o usurio
   *          dono, no inclui projetos aos quais apenas tem acesso. Caso o
   *         usurio ainda no possua um "home", retorna null.
   * 
   * @throws ServerProjectException
   */
  public static String[] getAllProjectNames(Object userId)
    throws ServerProjectException {
    String dirPath = getId(userId, "");
    if (dirPath == null) {
      throw new ServerProjectException("ServerProject:getAllProjectNames: "
        + "usurio " + userId + " no foi encontrado");
    }
    File dir = new File(dirPath);
    File[] projects = dir.listFiles(new FileFilter() {
      public boolean accept(File file) {
        return file.isDirectory();
      }
    });
    if (projects == null) {
      /*
       * usurio ainda no possui "home"
       */
      return null;
    }
    String[] names = new String[projects.length];
    for (int i = 0; i < names.length; i++) {
      names[i] = projects[i].getName();
    }
    return names;
  }

  /**
   * Verifica se o usurio possui algum projeto.
   * 
   * @param userId identificador do usurio
   * @return <code>true</code> se o usurio possui algum projeto prprio
   * @throws ServerProjectException se o usurio no existe
   */
  public static boolean userHasHisOwnProjects(Object userId)
    throws ServerProjectException {
    String dirPath = getId(userId, "");
    if (dirPath == null) {
      throw new ServerProjectException("ServerProject:getAllProjectNames: "
        + "usurio " + userId + " no foi encontrado");
    }
    File dir = new File(dirPath);
    File[] projects = dir.listFiles(new FileFilter() {
      boolean foundOne = false;

      public boolean accept(File file) {
        if (!foundOne && file.isDirectory()) {
          /*
           * s queremos retornar true para o 1o diretrio encontrado
           */
          foundOne = true;
          return true;
        }
        return false;
      }
    });
    /*
     * o array ser null se houve algum erro ou se dir no  um diretrio, ser
     * vazio se no existe nenhum projeto, ou ter apenas 1 elemento caso haja
     * algum projeto
     */
    return projects != null && projects.length > 0;
  }

  /**
   * Abre um projeto de um usurio. Verifica se o usuario que quer abrir o
   * projeto participa dele.
   * 
   * @param userId Identificador do usuario que quer abrir o projeto
   * @param id Identificador do projeto.
   * @param notify
   * 
   * @return Objeto que representa o projeto.
   * @throws ServerProjectException
   * 
   * @throws ServerProjectException
   */
  public static synchronized ServerProject openProject(Object userId,
    String id, boolean notify) throws ServerProjectException {
    Object ownerId = getOwnerId(id);
    String projectName = getProjectName(id);
    return openProject(userId, ownerId, projectName, notify);
  }

  /**
   * Abre um projeto de um usurio. Verifica se o usuario que quer abrir o
   * projeto participa dele.
   * 
   * @param userId Identificador do usuario que quer abrir o projeto
   * @param ownerId Identificador do usurio dono do projeto.
   * @param projectName Nome do projeto a ser aberto.
   * @param notify indicativo de notificao para os clientes.
   * 
   * @return Objeto que representa o projeto.
   * 
   * @throws ServerProjectException
   */
  public static synchronized ServerProject openProject(Object userId,
    Object ownerId, String projectName, final boolean notify)
    throws ServerProjectException {
    ProjectService prjService = ProjectService.getInstance();
    try {
      User user = User.getUser(userId);
      if (!prjService.checkPermission(user)
        && !prjService.isUserMember(userId, ownerId, projectName)) {
        throw new ServerProjectException("Usuario nao participa do projeto");
      }
    }
    catch (Exception e) {
      throw new ServerProjectException("ServerProject.openProject:"
        + "\n userId: " + userId + "\n ownerId: " + ownerId
        + "\n projectName: " + projectName
        + "Falha na verificacao se o usuario pode acessar o projeto.", e);
    }
    String id = getId(ownerId, projectName);
    ServerProject sp = getProject(id);
    if (sp == null) {
      File dir = new File(id);
      if (!dir.exists() || !dir.isDirectory()) {
        String errMsg =
          "ServerProject:openProject: " + dir.getAbsolutePath()
            + " no existe ou no  um diretrio";
        throw new ServerProjectException(errMsg);
      }
      sp = new ServerProject(id, dir);
      projects.put(id, sp);
      ProjectService.getInstance().logInfoMessage(
        "Projeto " + id + " aberto no servidor. Numero de projetos abertos: "
          + projects.size());
    }
    else {
      // sp.refreshTree();
    }
    sp.count++;
    if (notify) {
      notifyProjectUsers(sp, new OpenProjectNotificationData(sp.getName()));
    }
    return sp;
  }

  /**
   * Envia uma notificao para todos os usurios que tm acesso ao projeto (com
   * exceo do prprio dono).
   * 
   * @param project - projeto
   * @param data - dados da notificao
   */
  private static void notifyProjectUsers(ServerProject project,
    NotificationData data) {
    User loggedUser = Service.getUser();
    if (loggedUser == null || project.getSharingType() != SharingType.PARTIAL) {
      return;
    }
    String loggedUserName = loggedUser.getName();
    Set<Object> userIds = project.getAllUsers(false);
    NotificationService notService = NotificationService.getInstance();
    for (Object uid : userIds) {
      notService.notifyUser(loggedUserName, uid, data);
    }
  }

  /**
   * Recupera um projeto aberto de um usurio. Se o projeto no estiver aberto,
   * retorna null.
   * 
   * @param id Identificador do projeto.
   * 
   * @return Objeto que representa o projeto ou null caso o projeto no esteja
   *         aberto.
   */
  public static ServerProject getProject(Object id) {
    ServerProject sp = projects.get(id);
    return sp;
  }

  /**
   * Cria, abre e retorna um novo projeto.
   * 
   * @param info Informaes do projeto.
   * 
   * @return o projeto criado
   * 
   * @throws ServerProjectException
   */
  public static ServerProject createProject(CommonProjectInfo info)
    throws ServerProjectException {
    ProjectService projectService = ProjectService.getInstance();
    String id = getId(info.userId, info.name);
    File dir = new File(id);
    if (dir.exists()) {
      throw new ServiceFailureException(String.format(projectService
        .getString("ServerProject.error.project.exists"), info.name));
    }
    if (!dir.mkdirs()) {
      throw new ServiceFailureException(String.format(projectService
        .getString("ServerProject.error.create.dir"), info.name));
    }
    /*
     * parmetros adicionais do projeto recm-criado (data de criao e tipo
     * default de compartilhamento)
     */
    final long now = System.currentTimeMillis();
    info.setAttribute(ProjectAttribute.CREATION_DATE.getAttributeKey(), now);
    ProjectPermissions.setSharingType(info, SharingType.PRIVATE);

    ServerProject sp = new ServerProject(id, info, dir);
    File configFile = getConfigFile(dir);
    sp.writeProjectInfo(configFile);
    projects.put(id, sp);
    ProjectService.getInstance().logInfoMessage(
      "Projeto " + id + " criado no servidor. Numero de projetos abertos: "
        + projects.size());
    sp.count++;
    return sp;
  }

  /**
   * Fornece informaes do projeto.
   * 
   * @return CommonProjectInfo info.
   */
  public CommonProjectInfo getInfo() {
    return info;
  }

  /**
   * Fornece o identificador do usurio ao qual o projeto pertence.
   * 
   * @return Identificador do usurio.
   */
  public Object getUserId() {
    return info.userId;
  }

  /**
   * Fornece o nome do projeto.
   * 
   * @return Nome do projeto.
   */
  public String getName() {
    return info.name;
  }

  /**
   * Altera o nome do projeto. Essa modificao s ser persistida quando o
   * mtodo {@link #modify} for chamado.
   * 
   * @param name nome.
   */
  public void setName(String name) {
    // no permite por enquanto.
  }

  /**
   * Fornece a dscrio do projeto.
   * 
   * @return A descrio do projeto.
   */
  public String getDescription() {
    return info.description;
  }

  /**
   * Altera a descrio do projeto. Essa modificao s ser persistida quando o
   * mtodo {@link #modify} for chamado.
   * 
   * @param description descrio.
   */
  public void setDescription(String description) {
    info.description = description;
  }

  /**
   * Fornece os atributos especficos de um projeto. Por exemplo, o sistema de
   * coordenadas do websintesi.
   * 
   * @return Atributos do projeto.
   */
  public Hashtable<String, Object> getAttributes() {
    return info.getAttributes();
  }

  /**
   * Altera os atributos especficos do projeto. Essa modificao s ser
   * persistida quando o mtodo {@link #modify} for chamado.
   * 
   * @param attributes do projeto.
   */
  public void setAttributes(Hashtable<String, Object> attributes) {
    info.setAttributes(attributes);
  }

  /**
   * Retorna a rvore do projeto, na forma de um {@link ProjectFile}.
   * 
   * @return A rvore do projeto.
   */
  public ProjectFile getTree() {
    return tree;
  }

  /**
   * Atualiza a lista de usurios com acesso apenas de leitura ao projeto.
   * 
   * @param usersRO - usurios com acesso apenas de leitura ao projeto
   */
  public void setUsersRO(Set<Object> usersRO) {
    ProjectPermissions.setUsersRO(info, usersRO);
  }

  /**
   * Atualiza a lista de usurios com acesso de leitura e escrita ao projeto.
   * 
   * @param usersRW - usurios com acesso apenas de leitura e escrita ao projeto
   */
  public void setUsersRW(Set<Object> usersRW) {
    ProjectPermissions.setUsersRW(info, usersRW);
  }

  /**
   * Obtm um conjunto com todos os usurios que tm acesso ao projeto, com
   * exceo do prprio dono e do admin.
   * 
   * @return conjunto com todos os usurios que tm acesso ao projeto, com
   *         exceo do prprio dono e do admin
   */
  public Set<Object> getUsersSet() {
    return ProjectPermissions.getAllUsers(info);
  }

  /**
   * Vertifica se o projeto  pblico.
   * 
   * @return true, caso o projeto seja pblico, ou false, caso contrrio.
   */
  public Boolean isPublic() {
    return ProjectPermissions.isPublic(info);
  }

  /**
   * Define o tipo de compartilhamento.
   * 
   * @param type - tipo de compartilhamento
   */
  public void setSharingType(SharingType type) {
    ProjectPermissions.setSharingType(info, type);
  }

  /**
   * Obtm o tipo de compartilhamento.
   * 
   * @return tipo do compartilhamento
   */
  public SharingType getSharingType() {
    return ProjectPermissions.getSharingType(info);
  }

  /**
   * Obtm a lista de usurios com acesso apenas de leitura ao projeto no modo
   * de compartilhamento seletivo.
   * <p>
   * <b>IMPORTANTE:</b> este mtodo no leva em considerao o tipo de
   * compartilhamento do projeto, i.e. pode retornar uma lista no-vazia mesmo
   * que o compartilhamento seja pblico ou privado.
   * 
   * @return lista de usurios com acesso apenas de leitura no modo de
   *         compartilhamento seletivo. Caso no existam usurios nesta
   *         condio, retorna uma lista vazia.
   */
  public Set<Object> getUsersRO() {
    return ProjectPermissions.getUsersRO(info);
  }

  /**
   * Obtm a lista de usurios com acesso de leitura e escrita ao projeto no
   * modo de compartilhamento seletivo.
   * <p>
   * <b>IMPORTANTE:</b> este mtodo no leva em considerao o tipo de
   * compartilhamento do projeto, i.e. pode retornar uma lista no-vazia mesmo
   * que o compartilhamento seja pblico ou privado.
   * 
   * @return lista de usurios com acesso de leitura e escrita no modo de
   *         compartilhamento seletivo. Caso no existam usurios nesta
   *         condio, retorna uma lista vazia.
   */
  public Set<Object> getUsersRW() {
    return ProjectPermissions.getUsersRW(info);
  }

  /**
   * Obtm todos os usurios que tm acesso ao projeto no modo de
   * compartilhamento seletivo (combinao dos conjuntos de acesso RO + acesso
   * RW).
   * <p>
   * <b>IMPORTANTE:</b> este mtodo no leva em considerao o tipo de
   * compartilhamento do projeto, i.e. pode retornar uma lista no-vazia mesmo
   * que o compartilhamento seja pblico ou privado.
   * 
   * @param includeOwner - indica se o prprio dono deve ser includo
   * 
   * @return todos os usurios que tm acesso ao projeto no modo de
   *         compartilhamento seletivo
   * 
   * @see #getUsersRO()
   * @see #getUsersRW()
   */
  public Set<Object> getAllUsers(boolean includeOwner) {
    Set<Object> allUsers = ProjectPermissions.getAllUsers(info);
    if (includeOwner) {
      allUsers.add(getUserId());
    }
    return allUsers;
  }

  /**
   * Verifica se o usurio tem acesso ao projeto. Isto acontece se qualquer uma
   * das opes seguintes  satisfeita:
   * <ul>
   * <li>projeto  pblico (RO ou RW)
   * <li>usurio  o dono do projeto ou o admin
   * <li>compartilhamento  seletivo e o usurio tem acesso RO ou RW ao projeto
   * </ul>
   * 
   * @param userID - identificador do usurio
   * @return <code>true</code> se o usurio tem acesso ao projeto (RO ou RW)
   */
  public boolean userHasAcces(Object userID) {
    return ProjectPermissions.userHasAccess(info, userID);
  }

  /**
   * Verifica se o usurio tem qualquer tipo de acesso RO ao projeto, ou seja,
   * se uma das seguintes condies  satisfeita:
   * <ul>
   * <li>o projeto  pblico apenas para leitura
   * <li>o compartilhamento  seletivo e o usurio possui acesso RO
   * </ul>
   * 
   * @param userID - identificador do usurio
   * @return <code>true</code> se o usurio tem qualquer tipo de acesso RO ao
   *         projeto
   */
  public boolean userHasAccessRO(Object userID) {
    return ProjectPermissions.userHasAccessRO(info, userID);
  }

  /**
   * Verifica se o usurio tem qualquer tipo de acesso RW ao projeto, ou seja,
   * se uma das seguintes condies for satisfeita:
   * <ul>
   * <li>usurio  o admin ou o dono do projeto
   * <li>este  pblico para leitura e escrita
   * <li>o compartilhamento  seletivo e o usurio possui acesso RW
   * </ul>
   * 
   * @param userID - identificador do usurio
   * @return <code>true</code> se o usurio tem acesso RW ao projeto
   */
  public boolean userHasAccessRW(Object userID) {
    return ProjectPermissions.userHasAccessRW(info, userID);
  }

  /**
   * Verifica se o compartilhamento  seletivo e usurio tem acesso RO ao
   * projeto.
   * 
   * @param userID - identificador do usurio
   * @return <code>true</code> se o compartilhamento  seletivo e o usurio tem
   *         acesso RO ao projeto
   */
  public boolean userHasSelectiveAccessRO(Object userID) {
    return ProjectPermissions.userHasSelectiveAccessRO(info, userID);
  }

  /**
   * Verifica se o compartilhamento  seletivo e usurio tem acesso RW ao
   * projeto.
   * 
   * @param userID - identificador do usurio
   * @return <code>true</code> se o compartilhamento  seletivo e o usurio tem
   *         acesso RW ao projeto
   */
  public boolean userHasSelectiveAccessRW(Object userID) {
    return ProjectPermissions.userHasSelectiveAccessRW(info, userID);
  }

  /**
   * Verifica se o usurio  o dono do projeto.
   * 
   * @param userID - identificador do usurio
   * @return <code>true</code> se o usurio  o dono do projeto
   */
  public boolean userIsOwner(Object userID) {
    return info.userId.equals(userID);
  }

  /**
   * Persiste as modificaes feitas nas informaes do projeto. Caso alguma
   * informao no possa ser alterada, seu valor original ser reestabelecido.
   * Notifica os observadores que ocorreu uma mudana nas informaes do
   * projeto.
   * 
   * @throws ServerProjectException
   */
  public void modify() throws ServerProjectException {
    File configFile = getConfigFile(dir);
    if (!configFile.isFile()) {
      return;
    }
    writeProjectInfo(configFile);
    Object[] arg = { info };
    notifyObservers(ProjectEvent.makeEvent(ProjectEvent.INFO_MODIFIED, arg));
  }

  /**
   * Persiste as informaes do projeto.
   * 
   * @param configFile .
   * 
   * @throws ServerProjectException
   */
  private void writeProjectInfo(File configFile) throws ServerProjectException {
    ObjectOutputStream out = null;
    try {
      out =
        new ObjectOutputStream(new DataOutputStream(new BufferedOutputStream(
          new FileOutputStream(configFile))));
      out.writeObject(info);
    }
    catch (IOException e) {
      String errMsg = "Error ao gerar " + configFile.getAbsolutePath();
      throw (new ServerProjectException(errMsg, e));
    }
    finally {
      try {
        if (out != null) {
          out.close();
        }
      }
      catch (IOException e) {
        throw (new ServerProjectException("Erro fechando stream", e));
      }
    }
  }

  /**
   * Sinaliza que a rvore do projeto deve ser atualizada. O evento  enviado
   * apenas com a raiz da rvore, sem os filhos (mas com a informao de se este
   * possui filhos).
   * 
   * @throws ServerProjectException em caso de falha
   * 
   * @see #rebuildTree()
   */
  public void refreshTree() throws ServerProjectException {
    try {
      this.tree.makeSureExists();
    }
    catch (ServerProjectFileException spe) {
      String errMsg =
        "Falha na atualizao da rvore do projeto (" + dir.getAbsolutePath()
          + ").";
      throw (new ServerProjectException(errMsg, spe));
    }
    Object[] arg =
      { ProjectService.buildSingleClientProjectFile(id, null, tree) };
    notifyObservers(ProjectEvent.makeEvent(ProjectEvent.TREE_CHANGED, arg));
  }

  /**
   * Reconstri a rvore do projeto a partir do filesystem. <b>Todos</b> os ns
   * so lidos recursivamente.
   * <p>
   * <b>IMPORTANTE:</b> esta operao  custosa para projetos com muitos
   * arquivos. Apenas os arquivos de controle do CSBase so descartados, todos
   * os demais arquivos e diretrios sero percorridos. O evento enviado em
   * resposta contm a raiz da rvore, que est completamente preenchida.
   * 
   * @throws ServerProjectException
   * 
   * @see #refreshTree()
   */
  public void rebuildTree() throws ServerProjectException {
    try {
      this.tree.makeSureExists();
    }
    catch (ServerProjectFileException spe) {
      String errMsg =
        "Falha na atualizao da rvore do projeto (" + dir.getAbsolutePath()
          + ").";
      throw (new ServerProjectException(errMsg, spe));
    }
    Object[] arg = { ProjectService.buildClientProjectSubtree(id, null, tree) };
    notifyObservers(ProjectEvent.makeEvent(ProjectEvent.TREE_CHANGED, arg));
  }

  /**
   * Remove o projeto. Caso este mtodo retorne com sucesso, o objeto que
   * representa o projeto no poder mais ser utilizado.
   * 
   * @throws ServerProjectException
   */
  public void remove() throws ServerProjectException {
    try {
      tree.remove();
    }
    catch (ServerProjectFileException spe) {
      throw new ServerProjectException("ServerProject:remove:"
        + " erro na remoo do projeto " + id, spe);
    }
    File configFile = getConfigFile(dir);
    if (!configFile.delete()) {
      throw new ServerProjectException("ServerProject:remove:"
        + " erro na remoo do arquivo " + configFile.getAbsolutePath());
    }
    tree = null;
    if (projects.remove(id) == this) {
      Object[] arg = { id };
      notifyObservers(ProjectEvent.makeEvent(ProjectEvent.PROJECT_DELETED, arg));
    }
    else {
      String errMsg =
        "ServerProject:remove: erro na remoo do projeto " + id
          + " da hashtable";
      throw (new ServerProjectException(errMsg));
    }
  }

  /**
   * Notifica os usurios sobre o fechamento do projeto, mas no deve tirar ele
   * de memria. Olhar pendncia 377. Vai ser corrigida com a reimplentao do
   * servio de projetos.
   * 
   * @param notify indicativo de notificao aos clientes.
   */
  public void close(final boolean notify) {
    if (notify) {
      notifyProjectUsers(this, new CloseProjectNotificationData(this.getName()));
    }
    if (--count == 0) {
      //O projeto no deve ser removido de memria
      //projects.remove(id);
    }
  }

  /**
   * Verifica se um arquivo j existe ou no.
   * 
   * @param path O caminho do arquivo, a partir do diretrio raiz do projeto.
   * 
   * @return Verdadeiro, se o arquivo existir. Falso, caso contrrio.
   */
  public boolean exists(String[] path) {
    try {
      String[] fullPath = makeFullPath(path);
      ServerProjectFile.findFile(id, fullPath);
      return true;
    }
    catch (ServiceFailureException e) {
      return false;
    }
  }

  /**
   * Cria um diretrio no projeto. O diretrio pai do arquivo precisa existir.
   * 
   * @param path O caminho onde o novo arquivo vai ser criado, a partir do
   *        diretrio raiz do projeto.
   * 
   * @throws ServerProjectException
   */
  public void createDirectory(String[] path) throws ServerProjectException {
    try {
      String name = path[path.length - 1];
      String[] parentPath = new String[path.length - 1];
      for (int i = 0; i < parentPath.length; i++) {
        parentPath[i] = path[i];
      }
      parentPath = makeFullPath(parentPath);
      ServerProjectFile spf = ServerProjectFile.findFile(id, parentPath);
      spf.createDirectory(name);
    }
    catch (Exception e) {
      String errMsg =
        "ServerProject:createDirectory: " + "error ao criar dir. no projeto "
          + dir.getAbsolutePath();
      throw (new ServerProjectException(errMsg, e));
    }
  }

  /**
   * Cria um arquivo no projeto. O diretrio pai do arquivo precisa existir.
   * 
   * @param path O caminho onde o novo arquivo vai ser criado, a partir do
   *        diretrio raiz do projeto.
   * @param type O tipo do arquivo.
   * @param userId Identificador do usurio que cria o arquivo
   * @param mustCreateParentPath se for true, indica que, no caso do caminho do
   *        arquivo no existir, ele deve ser criado, caso contrrio, esse
   *        caminho no vai ser criado e uma exceo pode ser lanada
   * 
   * @throws ServerProjectException exceo
   */
  public void createFile(String[] path, String type, Object userId,
    boolean mustCreateParentPath) throws ServerProjectException {
    try {
      String name = path[path.length - 1];
      String[] parentPath = new String[path.length - 1];
      for (int i = 0; i < parentPath.length; i++) {
        parentPath[i] = path[i];
      }
      parentPath = makeFullPath(parentPath);
      if (!exists(parentPath) && mustCreateParentPath) {
        createDirectoryPath(parentPath);
      }
      ServerProjectFile spf = ServerProjectFile.findFile(id, parentPath);
      spf.createFile(name, type, userId);
    }
    catch (Exception e) {
      String errMsg =
        "ServerProject:createFile: error ao criar arquivo no " + "projeto "
          + dir.getAbsolutePath();
      throw (new ServerProjectException(errMsg, e));
    }
  }

  /**
   * Cria o caminho de diretrios especificado.
   * 
   * @param parentPath caminho de diretrios a ser criado
   * 
   * @throws ServerProjectException exceo na criao de um diretrio
   */
  private void createDirectoryPath(String[] parentPath)
    throws ServerProjectException {
    ArrayList<String> parentPathList = new ArrayList<String>();
    for (String dirPath : parentPath) {
      parentPathList.add(dirPath);
      String[] dirPathArray = parentPathList.toArray(new String[1]);
      if (exists(dirPathArray)) {
        continue;
      }
      createDirectory(dirPathArray);
    }
  }

  /**
   * Muda o tipo de um arquivo no projeto.
   * 
   * @param path O caminho para o arquivo.
   * @param type O novo tipo do arquivo.
   * 
   * @throws ServerProjectException
   */
  public void changeFileType(String[] path, String type)
    throws ServerProjectException {
    String[] fullPath = makeFullPath(path);
    try {
      ServerProjectFile spf = ServerProjectFile.findFile(id, fullPath);
      spf.changeType(type);
    }
    catch (ServerProjectFileException e) {
      String errMsg =
        "ServerProject:changeFileType: error ao mudar tipo do arquivo "
          + Arrays.toString(fullPath) + " no projeto " + dir.getAbsolutePath();
      throw new ServerProjectException(errMsg, e);
    }
  }

  /**
   * Associa um arquivo ao projeto. Cria arquivo de configurao e atualiza a a
   * rvore de projetos.
   * 
   * @param newFile O arquivo que se deseja associar ao projeto.
   * @param type O tipo do arquivo.
   * 
   * @throws ServerProjectException
   */
  public void associateFile(File newFile, String type)
    throws ServerProjectException {
    try {
      File parent = newFile.getParentFile();
      String[] parentPath = makeFullPath(parent);
      // Garante que o arquivo passado est dentro da hieraquia de diretrios
      // do projeto
      if (exists(parentPath)) {
        ServerProjectFile.setFileType(newFile, type);
        // Para aumentar a probabilidade de refreshTree ser chamado aps a
        // thread de notificao da rvore ter completado seu servio.
        Thread.sleep(1000);
        refreshTree();
      }
    }
    catch (Exception e) {
      String errMsg =
        "ServerProject:createFile2: error ao criar arquivo no projeto "
          + dir.getAbsolutePath();
      throw (new ServerProjectException(errMsg, e));
    }
  }

  /**
   * Cria um link para um arquivo real.
   * 
   * @param linkFile O arquivo de link.
   * @param realFile O arquivo real.
   * @param fileType O tipo do arquivo.
   * 
   * @throws ServerProjectException
   */
  public void createLinkToFile(File linkFile, File realFile, String fileType)
    throws ServerProjectException {
    String name = linkFile.getName();
    File parent = linkFile.getParentFile();
    String[] parentPath = makeFullPath(parent);
    try {
      ServerProjectFile spf = ServerProjectFile.findFile(id, parentPath);
      ServerProjectFile.setFileType(realFile, fileType);
      spf.createLinkToFile(name, realFile);
    }
    catch (Exception e) {
      String errMsg =
        "ServerProject:createLinkToFile: error ao criar " + "link " + name
          + " no projeto " + dir.getAbsolutePath();
      throw (new ServerProjectException(errMsg, e));
    }
  }

  /**
   * DOCUMENT ME!
   * 
   * @param path O caminho do arquivo, a partir do diretrio raiz do projeto.
   * @param desc .
   * 
   * @throws ServerProjectException
   */
  public void appendFileDescription(String[] path, String desc)
    throws ServerProjectException {
    try {
      String[] fullPath = makeFullPath(path);
      ServerProjectFile spf = ServerProjectFile.findFile(id, fullPath);
      spf.appendDescription(desc);
    }
    catch (Exception e) {
      String errMsg =
        "Erro ao " + "anexar descrio em arquivo do projeto "
          + dir.getAbsolutePath();
      throw new ServerProjectException(errMsg, e);
    }
  }

  /**
   * DOCUMENT ME!
   * 
   * @param path O caminho do arquivo, a partir do diretrio raiz do projeto.
   * @param desc .
   * 
   * @throws ServerProjectException
   */
  public void setFileDescription(String[] path, String desc)
    throws ServerProjectException {
    try {
      String[] fullPath = makeFullPath(path);
      ServerProjectFile spf = ServerProjectFile.findFile(id, fullPath);
      spf.setDescription(desc);
    }
    catch (Exception e) {
      String errMsg =
        "ServerProject:setFileDescription: erro ao "
          + "ajustar descrio em arquivo do projeto " + dir.getAbsolutePath();
      throw new ServerProjectException(errMsg, e);
    }
  }

  /**
   * DOCUMENT ME!
   * 
   * @param path O caminho do arquivo, a partir do diretrio raiz do projeto.
   * 
   * @return .
   * 
   * @throws ServerProjectException
   */
  public String getFileDescription(String[] path) throws ServerProjectException {
    try {
      String[] fullPath = makeFullPath(path);
      ServerProjectFile spf = ServerProjectFile.findFile(id, fullPath);
      return spf.getDescription();
    }
    catch (Exception e) {
      String errMsg =
        "ServerProject:getFileDescription: erro ao "
          + "buscar descrio em arquivo do projeto " + dir.getAbsolutePath();
      throw new ServerProjectException(errMsg, e);
    }
  }

  /**
   * Altera o estado do arquivo para arquivo em construo, ou no.
   * 
   * @param path O caminho do arquivo, a partir do diretrio raiz do projeto.
   * @param isUnderConstruction Verdadeiro para arquivo em construo e falso
   *        caso contrrio.
   * 
   * @throws ServerProjectException
   */
  public void setUnderConstruction(String[] path, boolean isUnderConstruction)
    throws ServerProjectException {
    try {
      String[] fullPath = makeFullPath(path);
      ServerProjectFile spf = ServerProjectFile.findFile(id, fullPath);
      spf.setUnderConstruction(isUnderConstruction);
    }
    catch (ServerProjectFileException e) {
      String errMsg =
        "ServerProject:setUnderConstruction: erro ao " + "colocar "
          + isUnderConstruction + " em arquivo " + "do projeto "
          + dir.getAbsolutePath();
      throw (new ServerProjectException(errMsg, e));
    }
  }

  /**
   * Renomeia um arquivo.
   * 
   * @param path O caminho do arquivo, a partir do diretrio raiz do projeto.
   * @param newName o nome novo
   * 
   * @throws ServerProjectException
   */
  public void renameFile(String[] path, String newName)
    throws ServerProjectException {
    try {
      String[] fullPath = makeFullPath(path);
      ServerProjectFile spf = ServerProjectFile.findFile(id, fullPath);
      spf.rename(newName);
    }
    catch (Exception e) {
      String errMsg =
        "ServerProject:renameFile: erro ao " + "remover um arquivo do projeto "
          + dir.getAbsolutePath();
      throw (new ServerProjectException(errMsg, e));
    }
  }

  /**
   * Remove um arquivo.
   * 
   * @param path caminho do arquivo, a partir do diretrio raiz do projeto.
   * 
   * @throws ServerProjectException
   */
  public void removeFile(String[] path) throws ServerProjectException {
    try {
      String[] fullPath = makeFullPath(path);
      ServerProjectFile spf = ServerProjectFile.findFile(id, fullPath);
      spf.remove();
    }
    catch (Exception e) {
      String errMsg =
        "ServerProject:removeFile: erro ao " + "remover um arquivo do projeto "
          + dir.getAbsolutePath();
      throw (new ServerProjectException(errMsg, e));
    }
  }

  /**
   * Constri um caminho completo para um arquivo, incluindo o diretrio raiz,
   * caso ele ainda no esteja no caminho especificado.
   * 
   * @param path caminho para o arquivo
   * 
   * @return um array contendo o caminho completo para o arquivo, incluindo o
   *         diretrio raiz
   */
  public String[] makeFullPath(String[] path) {
    if (path.length > 0 && path[0].equals(dir.getName())) {
      return path;
    }
    String[] fullPath = new String[path.length + 1];
    fullPath[0] = dir.getName();
    for (int i = 0; i < path.length; i++) {
      fullPath[i + 1] = path[i];
    }
    return fullPath;
  }

  /**
   * Constri um caminho completo para o arquivo especificado, incluindo tambm
   * o diretrio raiz.
   * 
   * @param file arquivo
   * 
   * @return um array contendo o caminho completo para o arquivo especificado,
   *         incluindo o diretrio raiz
   */
  private String[] makeFullPath(File file) {
    ArrayList<String> auxPath = new ArrayList<String>();
    File auxFile = file;
    while (!auxFile.getName().equals(dir.getName())) {
      auxPath.add(auxFile.getName());
      auxFile = auxFile.getParentFile();
    }
    auxPath.add(auxFile.getName());
    String[] path = new String[auxPath.size()];
    int j = 0;
    for (int i = auxPath.size() - 1; i >= 0; i--) {
      path[j++] = auxPath.get(i);
    }
    return path;
  }

  /**
   * Chama a notificao da superclasse.
   * 
   * @param event .
   */
  @Override
  public void notifyObservers(Object event) {
    setChanged();
    super.notifyObservers(event);
  }

  /**
   * Retorna true porque s so cadastrados observadores interessados neste
   * projeto.
   * 
   * @param arg .
   * @param event .
   * 
   * @return .
   */
  @Override
  protected boolean has2Update(Object arg, Object event) {
    return true;
  }

  /**
   * Retorna um texto descritivo do projeto.
   * 
   * @return .
   */
  @Override
  public String toString() {
    String text = "";
    text = text + "Id: " + id + "\n";
    text = text + "Userid: " + info.userId + "\n";
    text = text + "Name: " + info.name + "\n";
    text = text + "Descricao: " + info.description + "\n";
    text = text + "Arvore: " + tree + "\n";
    text = text + "Diretorio: " + dir + "\n";
    text = text + "No clientes usando: " + count + "\n";
    return text;
  }

  /**
   * Constri um <code>ServerProject</code>
   * 
   * @param id .
   * @param dir .
   * 
   * @throws ServerProjectException
   */
  private ServerProject(Object id, File dir) throws ServerProjectException {
    super();
    this.id = id;
    this.dir = dir;
    try {
      this.tree = ServerProjectFile.createRoot(dir, id);
    }
    catch (ServerProjectFileException spe) {
      String errMsg =
        "ServerProject:ServerProject: falha na criao de "
          + "ServerProjectFile! (" + dir.getAbsolutePath() + ").";
      throw (new ServerProjectException(errMsg, spe));
    }
    this.count = 0;
    File configFile = getConfigFile(dir);
    if (!configFile.isFile()) {
      return;
    }
    ObjectInputStream in = null;
    try {
      in =
        new ObjectInputStream(new DataInputStream(new BufferedInputStream(
          new FileInputStream(configFile))));
      info = (CommonProjectInfo) in.readObject();
    }
    catch (Exception e) {
      // impede a criao do objeto;
      String errMsg = "Error ao ler " + configFile.getAbsolutePath();
      throw (new ServerProjectException(errMsg, e));
    }
    finally {
      try {
        if (in != null) {
          in.close();
        }
      }
      catch (IOException e) {
        throw (new ServerProjectException("Erro fechando stream", e));
      }
    }
  }

  /**
   * Constri um <code>ServerProject</code>
   * 
   * @param id .
   * @param info .
   * @param dir .
   * 
   * @throws ServerProjectException
   */
  private ServerProject(Object id, CommonProjectInfo info, File dir)
    throws ServerProjectException {
    super();
    this.id = id;
    this.info = info;
    this.dir = dir;
    try {
      this.tree = ServerProjectFile.createRoot(dir, id);
    }
    catch (ServerProjectFileException spe) {
      String errMsg =
        "ServerProject:ServerProject: falha na criao de "
          + "ServerProjectFile! (" + dir.getAbsolutePath() + ").";
      throw (new ServerProjectException(errMsg, spe));
    }
    this.count = 0;
  }
}
