/*
 * $Id: ServerProjectFile.java 105933 2010-05-24 10:05:33Z cassino $
 */
package csbase.server.services.projectservice;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.lang.ref.SoftReference;
import java.rmi.RemoteException;
import java.text.MessageFormat;
import java.util.ArrayDeque;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Comparator;
import java.util.Date;
import java.util.Deque;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.List;
import java.util.Set;
import java.util.Stack;

import tecgraf.javautils.FileUtils;
import csbase.exception.OperationFailureException;
import csbase.exception.ServiceFailureException;
import csbase.logic.ClientProjectFile;
import csbase.logic.ClientProjectFileInfo;
import csbase.logic.FileLockListenerInterface;
import csbase.logic.ProjectEvent;
import csbase.logic.ProjectFile;
import csbase.logic.ProjectFileInfo;
import csbase.logic.ProjectFileType;
import csbase.logic.SecureKey;
import csbase.logic.User;
import csbase.logic.Utilities;
import csbase.remote.ProjectFileChannel;
import csbase.remote.ProjectFileChannelFromServer;
import csbase.server.FileSystem;
import csbase.server.services.publicationservice.PublicationService;

/**
 * Modela um arquivo ou diretrio de um projeto no servidor. Essa classe far os
 * acessos ao sistema de arquivos para manipular as rvores dos projetos.
 */
public class ServerProjectFile extends FileConfig implements ProjectFile {
  /**
   * Representa o arquivo original de configurao no sistema de arquivos, caso
   * exista. No sendo um link, o arquivo pode ainda no ter um arquivo de
   * configurao associado.
   */
  private File configFile;

  /**
   * Representa o arquivo de configurao do arquivo real, caso exista. Esse
   * arquivo s existir caso o original seja um link e o arquivo real tenha um
   * arquivo de configurao.
   */
  private File realConfigFile;

  /**
   * Representa o identificador do projeto ao qual esse arquivo (ou diretrio)
   * pertence.
   */
  private Object projectId;

  /** O estado que indica como o arquivo est aberto. */
  private int state;

  /** Stream para ler objetos do arquivo via rmi */
  private ObjectInputStream objInputStream;

  /** Stream para escrever objetos do arquivo via rmi */
  private ObjectOutputStream objOutputStream;

  /**
   * Cache de objetos do tipo ServerProjectFile que garante a unicidade dos
   * ServerProjectFile criados no lado servidor. As referncias so weak para
   * que no consuma muita mamria (referncias presas no GC).
   */
  final private static HashMap<String, SoftReference<ServerProjectFile>> filesCache =
    new HashMap<String, SoftReference<ServerProjectFile>>();

  /**
   * Comparador para comparaes por nome.
   */
  private final Comparator<ServerProjectFile> nameComparator;

  /**
   * Mtodo que garante a unicidade dos objetos do tipo
   * <code>ServerProjectFile</code> no servidor atravs de uma cache local de
   * controle. Os parmetros so sos mesmos do construtor desta classe, s que
   * constri o objeto de acordo com a real necessidade.
   * 
   * @param file arquivo local
   * @param realFile arquivo local real
   * @param projectId id do projeto
   * @param parent diretrio-pai.
   * @return o arquivo
   */
  final static synchronized ServerProjectFile buildFromCache(final File file,
    final File realFile, final Object projectId, final ServerProjectFile parent) {
    final String key = file.getAbsolutePath();
    if (filesCache.containsKey(key)) {
      final ServerProjectFile spf = filesCache.get(key).get();
      if (spf != null) {
        //        System.out.println("Pegando da cache: " + key);
        return spf;
      }
    }
    final ServerProjectFile srvFile =
      new ServerProjectFile(file, realFile, projectId, parent);
    filesCache.put(key, new SoftReference<ServerProjectFile>(srvFile));
    //    System.out.println("Construindo: " + key);

    return srvFile;
  }

  /**
   * @param serverProjectFile
   */
  final static synchronized void deleteInCache(
    final ServerProjectFile serverProjectFile) {
    final String key = serverProjectFile.getAbsolutePath();
    //      System.out.println("Apagando da cache: " + key);
    filesCache.remove(key);
  }

  /**
   * Limpeza total da cache.
   */
  final static synchronized void clearAllCache() {
    filesCache.clear();
    //    System.out.println("Esvaziando cache: ");
  }

  /**
   * Abre este arquivo para leitura exclusiva que no  feita com o uso de
   * <code>ByteBuffer</code>. A implementao desse mtodo determina o uso de
   * RMI para transmisso dos dados.
   * 
   * @throws ServerProjectFileException
   */
  public void openForReading() throws ServerProjectFileException {
    if (state != CLOSED) {
      throw new ServerProjectFileException("ServerProjectFile:openForReading: "
        + getAbsolutePath() + " j est aberto");
    }
    try {
      objInputStream =
        new ObjectInputStream(new DataInputStream(new BufferedInputStream(
          new FileInputStream(realFile))));
      state = OPENED_FOR_READING;
      getService().logInfoMessage(
        "Arquivo aberto para leitura: " + realFile.getPath());
    }
    catch (FileNotFoundException e) {
      String errMsg =
        "ServerProjectFile:openForReading: " + realFile.getAbsolutePath()
          + " no existe";
      throw (new ServerProjectFileException(errMsg, e));
    }
    catch (IOException e) {
      String errMsg =
        "ServerProjectFile:openForReading: erro de IO em "
          + realFile.getAbsolutePath();
      throw (new ServerProjectFileException(errMsg, e));
    }
  }

  /**
   * Abre este arquivo para escrita exclusiva que no  feita com o uso de
   * <code>ByteBuffer</code>. A implementao desse mtodo determina o uso de
   * RMI para transmisso dos dados.
   * 
   * @throws ServerProjectFileException
   */
  public void openForWriting() throws ServerProjectFileException {
    if (state != CLOSED) {
      throw new ServerProjectFileException("ServerProjectFile:openForWriting: "
        + getAbsolutePath() + " j est aberto");
    }
    try {
      objOutputStream =
        new ObjectOutputStream(new DataOutputStream(new BufferedOutputStream(
          new FileOutputStream(realFile))));
      state = OPENED_FOR_WRITING;
      getService().logInfoMessage(
        "Arquivo aberto para escrita: " + realFile.getPath());
    }
    catch (FileNotFoundException e) {
      String errMsg =
        "ServerProjectFile:openForWriting: " + realFile.getAbsolutePath()
          + " no existe";
      throw (new ServerProjectFileException(errMsg, e));
    }
    catch (IOException e) {
      String errMsg =
        "ServerProjectFile:openForWriting: erro de IO em "
          + realFile.getAbsolutePath();
      throw (new ServerProjectFileException(errMsg, e));
    }
  }

  /**
   * Abre um stream para leitura do arquivo.
   * 
   * @return .
   * @throws ServerProjectFileException Erro ao acessar o arquivo.
   */
  public InputStream openInputStream() throws ServerProjectFileException {
    try {
      return new FileInputStream(realFile);
    }
    catch (FileNotFoundException e) {
      throw new ServerProjectFileException(e);
    }
  }

  /**
   * Abre um stream para escrita do arquivo.
   * 
   * @return .
   * @throws ServerProjectFileException Erro ao acessar o arquivo.
   */
  public OutputStream openOutputStream() throws ServerProjectFileException {
    try {
      return new FileOutputStream(realFile);
    }
    catch (FileNotFoundException e) {
      throw new ServerProjectFileException(e);
    }
  }

  /**
   * L um objeto gravado no arquivo. A implementao desse mtodo utiliza RMI
   * para envio do objeto serializado.
   * 
   * @return O objeto lido do arquivo.
   * @throws ServerProjectFileException
   */
  public Object readObject() throws ServerProjectFileException {
    if (state != OPENED_FOR_READING) {
      throw new ServerProjectFileException("ServerProjectFile:readObject: "
        + getAbsolutePath() + "no est aberto para leitura");
    }
    try {
      return objInputStream.readObject();
    }
    catch (IOException e) {
      String errMsg =
        "ServerProjectFile:readObject: erro de IO em "
          + realFile.getAbsolutePath();
      throw (new ServerProjectFileException(errMsg, e));
    }
    catch (ClassNotFoundException e) {
      String errMsg =
        "ServerProjectFile:readObject: erro de IO em "
          + realFile.getAbsolutePath();
      throw (new ServerProjectFileException(errMsg, e));
    }
  }

  /**
   * Grava um objeto no arquivo. A implementao desse mtodo utiliza RMI para
   * envio do objeto serializado.
   * 
   * @param obj O objeto a ser gravado no arquivo.
   * @throws ServerProjectFileException
   */
  public void writeObject(Object obj) throws ServerProjectFileException {
    if (state != OPENED_FOR_WRITING) {
      throw new ServerProjectFileException("ServerProjectFile:writeObject: "
        + getAbsolutePath() + "no est aberto para escrita");
    }
    try {
      setUnderConstruction(true);
      objOutputStream.writeObject(obj);
    }
    catch (IOException e) {
      String errMsg =
        "ServerProjectFile:writeObject: erro de IO em "
          + realFile.getAbsolutePath();
      throw (new ServerProjectFileException(errMsg, e));
    }
    finally {
      setUnderConstruction(false);
    }
  }

  /**
   * Fecha este arquivo. Aps chamado este mtodo, o objeto que representa o
   * arquivo no poder mais ser utilizado para leitura e/ou escrita.
   * 
   * @throws ServerProjectFileException
   */
  public void close() throws ServerProjectFileException {
    try {
      switch (state) {
        case OPENED_FOR_READING:
          state = CLOSED;
          objInputStream.close();
          objInputStream = null;
          break;
        case OPENED_FOR_WRITING:
          state = CLOSED;
          objOutputStream.flush();
          objOutputStream.close();
          objOutputStream = null;
          break;
      }
      getService().logInfoMessage("Arquivo fechado: " + realFile.getPath());
    }
    catch (IOException e) {
      String errMsg =
        "ServerProjectFile:close: erro de IO em " + realFile.getAbsolutePath();
      throw (new ServerProjectFileException(errMsg, e));
    }
  }

  /**
   * Cria um objeto do tipo ServerProjectFile que representa o arquivo dado. Se
   * o tipo do arquivo for especificado, o arquivo de configurao  criado para
   * guardar esse tipo. O objeto criado  includo como filho do seu pai.
   * 
   * @param name nome do arquivo a ser representado pelo ServerProjectFile.
   * @param type O tipo do arquivo ou <code>null</code>.
   * @param userId identificador do usurio que cria o arquivo.
   * @return O ServerProjectFile criado que representa o arquivo dado.
   * @throws ServerProjectFileException
   */
  private ServerProjectFile _createFile(String name, String type, Object userId)
    throws ServerProjectFileException {
    if (type == null) {
      String errMsg = "ServerProjectFile:createFile: " + "type == null";
      throw (new ServerProjectFileException(errMsg));
    }
    try {
      if (type.equals(ProjectFileType.UNKNOWN)) {
        type = ServerProjectFileType.getInstance().getTypeFromExtension(name);
      }
    }
    catch (Exception e) {
      ProjectService.getInstance().logWarningMessage(
        "Falha na obteno " + "do tipo a partir da extenso: " + e);
    }
    if (isControlledFileName(name)) {
      String errMsg =
        "ServerProjectFile:createFile: " + name + " um nome reservado";
      throw (new ServerProjectFileException(errMsg));
    }
    File newFile = new File(file, name);
    if (newFile.exists()) {
      String errMsg =
        String.format(getString("ServerProjectFile.error.file.exists"), name,
          getName());
      throw new ServiceFailureException(errMsg);
    }
    try {
      newFile.createNewFile();
      ServerProjectFile spf =
        ServerProjectFile.buildFromCache(newFile, newFile, projectId, this);

      spf.configure(type, userId);
      return spf;
    }
    catch (IOException e) {
      String errMsg =
        "ServerProjectFile:create: erro ao criar " + newFile.getAbsolutePath();
      throw (new ServerProjectFileException(errMsg, e));
    }
  }

  /**
   * Atribui as configuraes de criao de um arquivo.
   * 
   * @param type .
   * @param userId .
   * @throws ServerProjectFileException
   */
  private synchronized void configure(String type, Object userId)
    throws ServerProjectFileException {
    try {
      this.type = type;
      this.createdBy = userId;
      long date = Calendar.getInstance().getTimeInMillis();
      this.creationDate = date;
      this.writeConfiguration();
      User user = User.getUser(userId);
      String userName = user.getName();
      String userLogin = user.getLogin();
      if (userName == null) {
        userName = (String) userId;
      }
      String msg = getString("ServerProjectFile.file.created.by");
      msg =
        MessageFormat.format(msg, new Object[] {
            Utilities.getFormattedDate(date), userLogin, userName });
      this.appendDescription(msg);
    }
    catch (Exception e) {
      String errMsg = "ServerProjectFile:configure: ";
      throw (new ServerProjectFileException(errMsg, e));
    }
  }

  /**
   * Cria um objeto do tipo ServerProjectFile que representa a raiz da rvore de
   * projetos.
   * 
   * @param rootDir diretrio-raiz, a ser representado pelo ServerProjectFile.
   * @param projectId O identificador do projeto para o qual est sendo criada a
   *        raiz.
   * @return O ServerProjectFile representando a raiz da rvore de projetos.
   * @throws ServerProjectFileException
   */
  public static ServerProjectFile createRoot(File rootDir, Object projectId)
    throws ServerProjectFileException {
    return ServerProjectFile.buildFromCache(rootDir, rootDir, projectId, null);
  }

  /**
   * Cria um link para um arquivo existente. Esse mtodo deve ser utilizado para
   * criar links de arquivos de projetos para arquivos lidos da base de dados
   * (que ficam na rea global) ou para arquivos publicados (que ficam na rea
   * de dados publicados). Esse mtodo se encarrega de criar o arquivo de
   * configurao associado e de incrementar o contador de referncias do
   * arquivo ao qual o link se referencia.
   * 
   * @param real Arquivo real, j existente.
   * @param link Arquivo link, a ser criado.
   * @throws ServerProjectFileException
   */
  public static void createLink(File real, File link)
    throws ServerProjectFileException {
    createLink(real, link, null);
  }

  /**
   * Cria um link para um arquivo existente. Esse mtodo deve ser utilizado para
   * criar links de arquivos de projetos para arquivos lidos da base de dados
   * (que ficam na rea global) ou para arquivos publicados (que ficam na rea
   * de dados publicados). Esse mtodo se encarrega de criar o arquivo de
   * configurao associado e de incrementar o contador de referncias do
   * arquivo ao qual o link se referencia.
   * 
   * @param real Arquivo real, j existente.
   * @param link Arquivo link, a ser criado.
   * @param userId Identificador do usurio que solicita a criao do link. Esta
   *        informao  adicionada ao arquivo de configurao do link.
   * @throws ServerProjectFileException
   */
  public static void createLink(File real, File link, Object userId)
    throws ServerProjectFileException {
    if (!real.exists()) {
      String errMsg =
        "ServerProjectFile:createLink: no existe o " + "arquivo "
          + real.getAbsolutePath() + ".";
      throw (new ServerProjectFileException(errMsg));
    }
    File realConfigFile = FileConfig.renamedFileConfig(real);
    if (!incCount(realConfigFile)) {
      String errMsg =
        "ServerProjectFile:createLink: Erro de incremento "
          + realConfigFile.getAbsolutePath() + ".";
      throw (new ServerProjectFileException(errMsg));
    }
    if (!FileSystem.createLink(real, link)) {
      String errMsg =
        "ServerProjectFile:createLink: erro ao criar link "
          + link.getAbsolutePath() + " para " + real.getAbsolutePath();
      if (!decCount(realConfigFile, null)) {
        errMsg +=
          ("\nServerProjectFile:createLink: falha ao " + "restaurar " + realConfigFile
            .getAbsolutePath());
      }
      throw (new ServerProjectFileException(errMsg));
    }
    FileConfig fileConfig = new FileConfig();
    if (!fileConfig.readConfigurationFile(realConfigFile)) {
      String errMsg =
        "ServerProjectFile:createLink: erro ao ler o arquivo "
          + realConfigFile.getAbsolutePath();
      throw (new ServerProjectFileException(errMsg));
    }
    File linkConfigFile = renamedFileConfig(link);
    FileConfig config = new FileConfig();
    config.isLink = true;
    config.realFile = real;
    /* Verifica se o arquivo est sendo obtido por publicao */
    /* Neste caso, precisa guardar quem est obtendo o arquivo */
    if ((userId != null) && (fileConfig.isPublished)) {
      Object publishedFrom = fileConfig.whoPublished(userId);
      config.publishedBy = publishedFrom;
    }
    if (!config.writeConfigurationFile(linkConfigFile)) {
      String errMsg =
        "ServerProjectFile:createLink: falha ao criar "
          + linkConfigFile.getAbsolutePath();
      throw (new ServerProjectFileException(errMsg));
    }
  }

  /*
   * ATENO: Este mtodo no deve ser utilizado. Ele  uma implementao rpida
   * para a criao de links para arquivos que no so controlados pelo sistema.
   * O contador de links e incrementado com uma unidade a mais na criao, para
   * que o arquivo no seja apagado quando no houverem mais links para ele.
   */
  public static void createLinkToExternalFile(File linkFile, File realFile,
    String fileType, String projectName, Object userId)
    throws ServerProjectFileException {
    ServerProject project;
    try {
      project = ServerProject.openProject(userId, userId, projectName, false);
      project.createLinkToFile(linkFile, realFile, fileType);
      final File parentFile = realFile.getParentFile();
      final String fileName = realFile.getName();
      File file = new File(parentFile, createConfigFileName(fileName));
      ServerProjectFile.incCount(file);
    }
    catch (ServerProjectException e) {
      throw new ServerProjectFileException(e.getMessage());
    }
  }

  /**
   * Confirma a existncia do arquivo representado por este objeto no sistema de
   * arquivos.
   * 
   * @throws ServerProjectFileException se o arquivo no existe
   */
  synchronized void makeSureExists() throws ServerProjectFileException {
    if (!file.exists()) {
      String errMsg =
        "ServerProjectFile:updateDir: " + file.getAbsolutePath()
          + " no existe no sistema de arquivos";
      throw new ServerProjectFileException(errMsg);
    }
  }

  /**
   * Atualiza o diretrio recursivamente, refletindo mudanas no sistema de
   * arquivos, notificando os observadores em seguida. Este mtodo no deve ser
   * invocado para arquivos.
   * <p>
   * <b>IMPORTANTE:</b> este mtodo pode ser custoso se a subrvore possuir
   * muitos elementos
   * 
   * @throws OperationFailureException caso o diretrio representado por este
   *         objeto no exista no servidor.
   * 
   * @see #refreshDir()
   */
  void rebuildDir() throws OperationFailureException {
    try {
      makeSureExists();
      ClientProjectFile clientDir =
        ProjectService.buildClientProjectSubtree(projectId, null, this);
      // FIXME provavelmente deveramos lanar um dirRebuiltEvent
      fireDirRefreshedEvent(projectId, clientDir);
    }
    catch (ServerProjectFileException ex) {
      throw new OperationFailureException(ex);
    }
  }

  /**
   * Garante que o diretrio existe no servidor, e notifica os observadores. A
   * notificao conter apenas o {@link ClientProjectFile} correspondente, os
   * filhos <b>no</b> so percorridos. Este mtodo deve ser executado apenas
   * para diretrios.
   * 
   * @throws OperationFailureException caso o diretrio no exista no servidor
   * 
   * @see #rebuildDir()
   */
  void refreshDir() throws OperationFailureException {
    try {
      makeSureExists();
      /*
       * primeiro obtemos os ns associados ao path at a raiz e os colocamos em
       * uma pilha
       */
      ServerProjectFile serverFile = this;
      Deque<ServerProjectFile> pathToRoot = new ArrayDeque<ServerProjectFile>();
      while (serverFile != null) {
        pathToRoot.push(serverFile);
        serverFile = serverFile.parent;
      }
      /*
       * agora percorremos o caminho inverso e criamos um ClientProjectFile para
       * cada n, da raiz at o n desejado, para que o CPF enviado no evento de
       * DIR_REFRESHED possua o path at a raiz do projeto
       */
      ClientProjectFile clientDir = null;
      while (!pathToRoot.isEmpty()) {
        clientDir =
          ProjectService.buildSingleClientProjectFile(projectId, clientDir,
            pathToRoot.pop());
      }
      /*
       * neste ponto clientDir  o CPF correspondente ao SPF (ServerProjectFile)
       * corrente e contm as informaes de seus ancestrais (path at a raiz)
       * devidamente preenchidas
       */
      fireDirRefreshedEvent(projectId, clientDir);
    }
    catch (ServerProjectFileException ex) {
      throw new OperationFailureException(ex);
    }
  }

  /**
   * Altera o estado do arquivo para arquivo em construo, ou no.
   * 
   * @param isUnderConstruction Verdadeiro para arquivo em construo e falso
   *        caso contrrio.
   * @throws ServerProjectFileException
   */
  public synchronized void setUnderConstruction(boolean isUnderConstruction)
    throws ServerProjectFileException {
    if (isLink) {
      String errMsg =
        "ServerProjectFile:setUnderConstruction: " + getAbsolutePath()
          + " link";
      throw (new ServerProjectFileException(errMsg));
    }
    if (this.isUnderConstruction != isUnderConstruction) {
      this.isUnderConstruction = isUnderConstruction;
      writeConfiguration();
      boolean[] state = { isGlobal, isPublished, isUnderConstruction };
      fireStateChangedEvent(projectId, file.getName(), getPath(), state,
        whoPublished(), new Long(getModificationDate()), new Long(size()),
        FileLockManager.getInstance().isLocked(this));
    }
  }

  /**
   * Altera o tipo do arquivo.
   * 
   * @param type O tipo do arquivo.
   */
  private synchronized void setType(String type) {
    if (!this.type.equals(type)) {
      this.type = type;
      writeConfiguration();
    }
  }

  /**
   * Altera o tipo do arquivo.
   * 
   * @param type O tipo do arquivo.
   * @throws ServerProjectFileException
   */
  public synchronized void changeType(String type)
    throws ServerProjectFileException {
    setType(type);
    /* Notifica os observadores */
    fireNewFileNameEvent(projectId, getPath(), getName(), type);
  }

  /**
   * Retorna o nome do arquivo.
   * 
   * @return Nome do arquivo.
   */
  public String getName() {
    return file.getName();
  }

  /**
   * Retorna o caminho completo para o arquivo, incluindo o diretrio do
   * projeto.
   * 
   * @return Caminho at o arquivo, incluindo o diretrio do projeto, e o
   *         prprio arquivo
   */
  public String[] getPath() {
    Stack<String> pathStack = new Stack<String>();
    for (ServerProjectFile spf = this; spf != null; spf = spf.parent) {
      pathStack.push(spf.getName());
    }
    String[] pathArray = new String[pathStack.size()];
    for (int i = 0; i < pathArray.length; i++) {
      pathArray[i] = pathStack.pop();
    }
    return pathArray;
  }

  /**
   * Retorna o arquivo real, caso este aqui seja um link.
   * 
   * @return .
   */
  public File getRealFile() {
    return realFile;
  }

  /**
   * Retorna o caminho para o arquivo. Esse caminho  real, ou seja, o caminho
   * absoluto de onde o arquivo se encontra.
   * 
   * @return Caminho absoluto at o arquivo, inclusive.
   */
  public String getAbsolutePath() {
    return file.getAbsolutePath();
  }

  /**
   * Retorna o canocico caminho para o arquivo.
   * 
   * @return Caminho canonico at o arquivo, inclusive.
   */
  public String getCanonicalPath() {
    try {
      return file.getCanonicalPath();
    }
    catch (IOException e) {
      getService().logSevereMessage(
        "ServerProjectFile: Caminho cannico do "
          + "arquivo no pde ser obtido", e);
    }
    return null;
  }

  /**
   * Informa o tipo do arquivo.
   * 
   * @return O tipo do arquivo.
   */
  public String getType() {
    return type;
  }

  /**
   * Retorna o diretrio pai deste arquivo.
   * 
   * @return O diretrio pai deste arquivo ou <code>null</code> caso este
   *         arquivo seja o diretrio raiz.
   */
  public ProjectFile getParent() {
    return parent;
  }

  /**
   * {@inheritDoc}
   */
  public Object getProjectId() {
    return projectId;
  }

  /**
   * Retorna o caminho simplificado para o arquivo (sem o nome do projeto).
   * 
   * @return Caminho simplificado at o arquivo, inclusive.
   */
  public String[] getSimplePath() {
    String[] path = getPath();
    String[] simplePath = new String[path.length - 1];
    for (int i = 1; i < path.length; i++) {
      simplePath[i - 1] = path[i];
    }
    return simplePath;
  }

  /**
   * Retorna o arquivo de configuracao caso esse arquivo nao seja um link.
   * 
   * @return Arquivo de configuracao do projeto.
   */
  private File getConfigFile() {
    return configFile;
  }

  /**
   * Retorna o identificador do usurio que obteve o arquivo publicado.
   * 
   * @return Identificador do usurio que obteve o arquivo publicado.
   */
  public Object whoPublished() {
    return publishedBy;
  }

  /**
   * Retorna o identificador do projeto a partir do qual o arquivo foi
   * publicado.
   * 
   * @return Identificador do projeto a partir do qual o arquivo foi publicado.
   */
  public Object whichProjectPublished() {
    return publishedByProject;
  }

  /**
   * Retorna a lista de arquivos deste diretrio.
   * <p>
   * IMPORTANTE: o array  sempre recriado a partir do
   * {@link ServerProjectFile#filesCache cache} ou do filesystem
   * 
   * @return a lista de arquivos deste diretrio, ou <code>null</code> em caso
   *         de erro
   * 
   * @see #getNewTree()
   */
  public ProjectFile[] getChildren() {
    try {
      return getNewTree();
    }
    catch (Exception e) {
      return null;
    }
  }

  /**
   * Retorna a lista de arquivos deste diretrio do tipo especificado.
   * 
   * @param fileType Tipo dos arquivos que devem ser retornados.
   * @return A lista de arquivos deste diretrio.
   */
  public ProjectFile[] getChildren(String fileType) {
    try {
      return getNewTree(fileType);
    }
    catch (ServerProjectFileException e) {
      return null;
    }
  }

  /**
   * Retorna um determinado arquivo contido neste diretrio.
   * 
   * @param name Nome do arquivo a ser recuperado.
   * @return O arquivo desejado ou null se houver erro, se o arquivo corrente
   *         no for um diretrio ou se o arquivo solicitado no existir
   */
  public synchronized ProjectFile getChild(final String name) {
    if (!file.isDirectory()) {
      String errMsg =
        "ServerProjectFile:getChild: " + file.getAbsolutePath()
          + " no  diretrio";
      ProjectService.getInstance().logWarningMessage(errMsg);
      return null;
    }

    File child = new File(file, name);
    if (!child.exists()) {
      return null;
    }

    try {
      ServerProjectFile spf =
        ServerProjectFile.buildFromCache(child, child, projectId, this);
      return spf;
    }
    catch (Exception e) {
      return null;
    }
  }

  /**
   * Indica se o diretrio possui filhos.
   * 
   * @return <code>true</code> se  um diretrio e se possui filhos
   */
  public synchronized boolean hasChildren() {
    if (!file.isDirectory()) {
      return false;
    }
    /*
     * como no temos um File.isEmpty() ou algo parecido, somos obrigados a
     * pegar todos os filhos de um File apenas para saber se ele tem filhos...
     * =/ Para minimizar os desperdcios, usamos um filtro que retorna um array
     * com apenas um elemento se o diretrio possui um ou mais filhos, e 0 se
     * no possui nenhum. Assim pelo menos evitamos a alocao de um array
     * potencialmente grande...
     */
    String[] childrenList = file.list(new FilenameFilter() {

      boolean foundOne = false;

      @Override
      public boolean accept(File dir, String childName) {
        if (foundOne || FileConfig.isControlledFileName(childName)) {
          return false;
        }
        foundOne = true;
        return true;
      }
    });
    return childrenList != null && childrenList.length > 0;
  }

  /**
   * Cria um diretrio dentro deste diretrio.
   * 
   * @param name Nome do diretrio a ser criado.
   * @throws ServerProjectFileException
   */
  public synchronized void createDirectory(String name)
    throws ServerProjectFileException {
    if (!isDirectory()) {
      String errMsg =
        "ServerProjectFile:createDirectory: " + getAbsolutePath()
          + "no  um diretrio";
      throw (new ServerProjectFileException(errMsg));
    }
    ServerProjectFile spf = _createDirectory(name);
    fireNewFileEvent(projectId, spf);
  }

  /**
   * Cria um subdiretrio deste diretrio.
   * 
   * @param name nome para o subdiretrio.
   * @return referncia para o subdiretrio criado.
   * @throws ServerProjectFileException
   */
  private ServerProjectFile _createDirectory(String name)
    throws ServerProjectFileException {
    if (FileConfig.isControlledFileName(name)) {
      String errMsg =
        "ServerProjectFile:createDirectory: " + name + " um nome reservado";
      throw (new ServerProjectFileException(errMsg));
    }
    File dir = new File(file, name);
    if (dir.exists()) {
      String errMsg =
        String.format(getString("ServerProjectFile.error.file.exists"), name,
          getName());
      throw new ServiceFailureException(errMsg);
    }
    if (!dir.mkdir()) {
      String errMsg =
        "ServerProjectFile:createDirectory: falha ao " + "criar "
          + dir.getAbsolutePath();
      throw (new ServerProjectFileException(errMsg));
    }

    ServerProjectFile spf =
      ServerProjectFile.buildFromCache(dir, dir, projectId, this);
    return spf;
  }

  /**
   * Cria um arquivo dentro deste diretrio.
   * 
   * @param name Nome do arquivo a ser criado.
   * @param type Tipo do arquivo a ser criado.
   * @param userId Identificador do usurio que cria o arquivo.
   * @throws ServerProjectFileException
   */
  public synchronized void createFile(String name, String type, Object userId)
    throws ServerProjectFileException {
    if (userId == null) {
      String errMsg = "ServerProjectFile:createFile: userId == null";
      throw (new ServerProjectFileException(errMsg));
    }
    if (!isDirectory()) {
      String errMsg =
        "ServerProjectFile:createFile: " + getAbsolutePath()
          + " no  um diretrio";
      throw (new ServerProjectFileException(errMsg));
    }
    ServerProjectFile spf = _createFile(name, type, userId);
    fireNewFileEvent(projectId, spf);
  }

  /**
   * Cria um ou mais arquivos dentro deste diretrio.
   * 
   * @param fileInfoList lista de dados dos arquivos a serem criados.
   * @param userId identificador do usurio que solicitou a criao.
   * @throws ServerProjectFileException
   */
  public synchronized void createFiles(List<ProjectFileInfo> fileInfoList,
    Object userId) throws ServerProjectFileException {
    // #TODO Possvel otimizao para os caminhos dos arquivos: repassar somente
    // "a/b/c" elimina a necessidade de se passar "a" ou "a/b", pois os subdirs
    // no encontrados so criados
    if (fileInfoList == null) {
      String errMsg = "ServerProjectFile:createFiles: fileInfoList == null";
      throw (new ServerProjectFileException(errMsg));
    }
    if (fileInfoList.size() == 0) {
      String errMsg = "ServerProjectFile:createFiles: fileInfoList.size() == 0";
      throw (new ServerProjectFileException(errMsg));
    }
    if (userId == null) {
      String errMsg = "ServerProjectFile:createFiles: userId == null";
      throw (new ServerProjectFileException(errMsg));
    }
    if (!isDirectory()) {
      String errMsg =
        "ServerProjectFile:createFiles: " + getAbsolutePath()
          + "no  um diretrio";
      throw (new ServerProjectFileException(errMsg));
    }
    Set<ServerProjectFile> newFiles = new HashSet<ServerProjectFile>();
    for (int i = 0; i < fileInfoList.size(); i++) {
      ServerProjectFile father = this;
      ProjectFileInfo childInfo = fileInfoList.get(i);
      if (childInfo == null) {
        String errMsg = "ServerProjectFile:createFiles: childInfo == null";
        throw (new ServerProjectFileException(errMsg));
      }
      String[] childPath = childInfo.getPath();
      if (childPath == null) {
        String errMsg = "ServerProjectFile:createFiles: childPath == null";
        throw (new ServerProjectFileException(errMsg));
      }
      if (childPath.length == 0) {
        String errMsg = "ServerProjectFile:createFiles: childPath.length == 0";
        throw (new ServerProjectFileException(errMsg));
      }

      boolean hasParentInEvent = false;
      String childType = childInfo.getType();
      for (int j = 0; j < childPath.length; j++) {
        ServerProjectFile child =
          (ServerProjectFile) father.getChild(childPath[j]);
        if (child == null) {
          /*
           * childPath[j]  criado como diretrio se  um subdiretrio
           * intermedirio ou se  folha e seu tipo  DIRECTORY_TYPE
           */
          if (j < (childPath.length - 1)
            || childType.equals(ProjectFileType.DIRECTORY_TYPE)) {
            child = father._createDirectory(childPath[j]);
          }
          else {
            child = father._createFile(childPath[j], childType, userId);
          }

          // Somente os arquivos/diretrios mais externos so notificados.
          // Eventuais arqs/dirs aninhados j esto includos nas
          // referncias <code>children</code> de cada diretrio.
          if (!hasParentInEvent) {
            hasParentInEvent = true;
            newFiles.add(child);
          }
        }
        else {
          if (newFiles.contains(child)) {
            hasParentInEvent = true;
          }
        }
        father = child;
      }
    }
    ServerProjectFile[] filesArray =
      newFiles.toArray(new ServerProjectFile[newFiles.size()]);

    fireNewFilesEvent(projectId, filesArray);
  }

  /**
   * Cria um link dentro deste diretrio.
   * 
   * @param name Nome do link a ser criado.
   * @param realFile Arquivo real, j existente.
   * @throws ServerProjectFileException
   */
  public void createLinkToFile(String name, File realFile)
    throws ServerProjectFileException {
    createLinkToFile(name, realFile, null);
  }

  /**
   * Cria um link dentro deste diretrio.
   * 
   * @param name Nome do link a ser criado.
   * @param realFile Arquivo real, j existente.
   * @param userId Identificador do usurio que solicita a criao do link.
   *        Registra no arquivo de configurao quem obteve o arquivo.
   * @throws ServerProjectFileException
   */
  public synchronized void createLinkToFile(String name, File realFile,
    Object userId) throws ServerProjectFileException {
    if (!isDirectory()) {
      String errMsg =
        "ServerProjectFile:createLinkToFile: " + getAbsolutePath()
          + " no  um diretrio";
      throw (new ServerProjectFileException(errMsg));
    }
    if (FileConfig.isControlledFileName(name)) {
      String errMsg =
        "ServerProjectFile:createLinkToFile: " + "nome de arquivo de link "
          + "invlido: " + name;
      throw (new ServerProjectFileException(errMsg));
    }
    File linkFile = new File(file, name);
    createLink(realFile, linkFile, userId);
    ServerProjectFile spf =
      ServerProjectFile.buildFromCache(linkFile, realFile, projectId, this);
    fireNewFileEvent(projectId, spf);

  }

  /**
   * Tenta obter um lock para o arquivo corrente. Se o arquivo  um diretrio,
   * tenta obter um lock compartilhado; neste caso, se o lock for obtido mas o
   * contador de referncias for > 1 significa que a operao no pode ser
   * realizada, pois outro lock compartilhado existe para o mesmo dir.
   * <p>
   * Se no  um diretrio, tenta obter um lock exclusivo para o mesmo.
   * 
   * @return identificador do lock se o lock foi obtido com sucesso
   * @throws OperationFailureException se o lock no pde ser obtido
   */
  private Object acquireLock() throws OperationFailureException {
    FileLockManager lockManager = FileLockManager.getInstance();
    SecureKey key = null;
    if (!ProjectService.isInternalServerRequest()) {
      key = (SecureKey) ProjectService.getKey();
    }
    Object lockId = null;
    if (isDirectory()) {
      /*
       *  um diretrio, precisamos de um lock compartilhado e temos que ser os
       * nicos a faz-lo
       */
      lockId = lockManager.acquireSharedLock(this, key);
      if (lockId == null) {
        throw new OperationFailureException(String.format(
          getString("ServerProjectFile.error.dir.locked"), getName()));
      }
      if (lockManager.getLockRefCount(this) > 1) {
        /*
         * conseguimos o lock compartilhado mas no fomos os nicos a faz-lo,
         * no podemos prosseguir. Removemos o lock recm-adquirido e abortamos.
         */
        lockManager.releaseLock(this, lockId);
        throw new OperationFailureException(String.format(
          getString("ServerProjectFile.error.dir.locked"), getName()));
      }
    }
    else {
      /*
       *  um arquivo, precisamos de um lock exclusivo
       */
      lockId = lockManager.acquireExclusiveLock(this, key);
      if (lockId == null) {
        throw new OperationFailureException(String.format(
          getString("ServerProjectFile.error.file.locked"), getName()));
      }
    }
    return lockId;
  }

  /**
   * Troca o nome deste arquivo.
   * 
   * @param name Novo nome do arquivo.
   * @throws ServerProjectFileException
   */
  public synchronized void rename(String name)
    throws ServerProjectFileException {
    /*
     * se file-locking est habilitado, s podemos seguir se obtivermos um lock
     * exclusivo sobre o arquivo
     */
    Object lockId = null;
    ServerProjectFile oldFile = this;
    boolean success = false;
    String oldAbsolutePath = getAbsolutePath();
    try {
      lockId = acquireLock();
    }
    catch (OperationFailureException ofe) {
      throw new ServiceFailureException(String.format(
        getString("ServerProjectFile.error.rename"), getName())
        + "\n" + ofe.getMessage());
    }
    try {
      /* No pode renomear o diretrio raiz do Projeto */
      if (this.parent == null) {
        String errMsg =
          "ServerProjectFile:rename:" + getAbsolutePath()
            + "  raiz do projeto ";
        throw (new ServerProjectFileException(errMsg));
      }
      /* Valida o novo nome */
      if (FileConfig.isControlledFileName(name)) {
        String errMsg =
          "ServerProjectFile:rename: " + name + " um nome reservado";
        throw (new ServerProjectFileException(errMsg));
      }
      /* Verifica se j existe um outro arquivo no diretrio com o mesmo nome */
      File newFile = renamedFile(file, name);
      if (newFile.exists()) {
        String errMsg =
          String.format(getString("ServerProjectFile.error.file.exists"), name,
            getName());
        throw new ServiceFailureException(errMsg);
      }

      /* Renomeia o arquivo */
      String[] oldPath = getPath();

      /* Retira o arquivo da cache pois ele vai ser renomeado */
      ServerProjectFile.deleteInCache(this);
      success = rename(newFile);
      if (!success) {
        String errMsg =
          "ServerProjectFile:rename: "
            + " no foi possvel renomear o arquivo " + getAbsolutePath()
            + " para " + newFile.getAbsolutePath();
        throw (new ServerProjectFileException(errMsg));
      }

      /* Notifica os observadores */
      fireNewFileNameEvent(projectId, oldPath, name, type);
    }
    finally {
      if (success) {
        FileLockManager.getInstance().releaseLock(oldAbsolutePath, lockId);
      }
      else {
        FileLockManager.getInstance().releaseLock(oldFile, lockId);
      }
    }
  }

  /**
   * Copia este arquivo para um diretrio, que pode pertencer a outro projeto.
   * Se o diretrio for o mesmo diretrio que este arquivo/diretrio, ele altera
   * o nome da cpia. Ao final, notifica os observadores no novo arquivo.
   * 
   * @param directory Diretrio destino.
   * @throws ServerProjectFileException
   */
  public synchronized void copy(ProjectFile directory)
    throws ServerProjectFileException {
    // Verificando se a cpia est sendo feita no mesmo diretrio.
    // Se isto estiver ocorrendo, o nome do arquivo dever ser modificado.
    ServerProjectFile dir = (ServerProjectFile) directory;
    String copyFileName;
    if (!isDirectory()
      && parent.getAbsolutePath().equals(dir.getAbsolutePath())) {
      copyFileName = getCopyName();
    }
    else {
      copyFileName = file.getName();
    }
    copy(dir, copyFileName);
  }

  /**
   * Copia este arquivo para um diretrio, que pode pertencer a outro projeto,
   * com outro nome. O novo nome no pode corresponder a um arquivo j
   * existente.
   * 
   * @param dir - diretrio de destino
   * @param copyFileName - novo nome para o arquivo. No pode corresponder a um
   *        arquivo que j existe
   * @throws ServerProjectFileException houve erro ao fazer a cpia
   */
  public synchronized void copy(ServerProjectFile dir, String copyFileName)
    throws ServerProjectFileException {
    File newFile = new File(dir.file, copyFileName);
    if (newFile.exists()) {
      String errMsg =
        String.format(getString("ServerProjectFile.error.file.exists"),
          copyFileName, dir.getName());
      throw new ServiceFailureException(errMsg);
    }
    if (!copyFile(newFile)) {
      String errMsg =
        String.format(getString("ServerProjectFile.error.copy"), copyFileName,
          dir.getName());
      throw (new ServerProjectFileException(errMsg));
    }

    /* Obtm o novo filho do diretrio */
    ServerProjectFile spf = (ServerProjectFile) dir.getChild(copyFileName);

    /* Notifica os observadores que um filho foi adicionado */
    Object TargetProjectId = spf.getProjectId();
    fireNewFileEvent(TargetProjectId, spf);
  }

  /**
   * Obtm o nome da cpia. O nome da cpia  formado por
   * [prefixo][sequencial].[extenso]. O prefixo  o nome do arquivo original
   * sem extenso. O sequencial  formado por um nmero entre parnteses. Esse
   * nmero  o nmero seguinte da ltima cpia existente.
   * 
   * @return O nome da cpia.
   * @throws ServerProjectFileException
   */
  private String getCopyName() throws ServerProjectFileException {
    int maxCopyNumber = 0;
    final ServerProjectFile father = (ServerProjectFile) getParent();
    ProjectFile[] brothers = father.getChildren();
    String[] parts = splitFileName();
    String preffix = parts[0];
    String extension = parts[2];

    // Obtendo o maior nmero de cpia.
    for (int i = 0; i < brothers.length; i++) {
      ServerProjectFile brother = (ServerProjectFile) brothers[i];
      String[] brotherParts = brother.splitFileName();
      if (brotherParts[0].equals(preffix) && brotherParts[2].equals(extension)
        && (brotherParts[1].length() != 0)) {
        int copyNumber;
        try {
          copyNumber = Integer.parseInt(brotherParts[1]);
          if (copyNumber > maxCopyNumber) {
            maxCopyNumber = copyNumber;
          }
        }
        catch (NumberFormatException ex) {
          throw new ServerProjectFileException(
            "Erro ao calcular o nome do arquivo.", ex);
        }
      }
    }
    int copyNumber = maxCopyNumber + 1;
    String copyName = preffix + "_" + copyNumber;
    if (extension.length() != 0) {
      copyName += ("." + extension);
    }
    return copyName;
  }

  /**
   * Obtm os nomes e ndices que delimitam o prefixo, o sequencial e a extenso
   * deste arquivo.
   * 
   * @return O array dos textos.
   *         <ul>
   *         <li>ndice 0: Ser utilizado para o prefixo;
   *         <li>ndice 1: Ser utilizado para o sequencial (sem os "()");
   *         <li>ndice 2: Ser utilizado para a extenso.
   *         </ul>
   *         Se no houver uma parte, a posio dela no array ser uma string
   *         vazia.
   */
  private String[] splitFileName() {
    String name = getName();
    int ixLastUnderscore = name.lastIndexOf("_");
    int ixDot = name.lastIndexOf(".");
    int ixEndPreffix = name.length();
    String preffix = "";
    String sequencial = "";
    String extension = "";
    if (ixDot != -1) {
      extension = name.substring(ixDot + 1);
      // removendo a extenso do arquivo.
      name = name.substring(0, ixDot);
      ixEndPreffix = ixDot;
    }
    ixLastUnderscore = name.lastIndexOf("_");
    if (ixLastUnderscore != -1) {
      String sequencialCandidate;
      sequencialCandidate = name.substring(ixLastUnderscore + 1);
      if (sequencialCandidate.matches("\\d+")) {
        sequencial = sequencialCandidate;
        ixEndPreffix = ixLastUnderscore;
      }
    }
    preffix = name.substring(0, ixEndPreffix);
    return new String[] { preffix, sequencial, extension };
  }

  /**
   * Indica se este ServerProjectFile  ancestral de outro.
   * 
   * @param otherFile .
   * @return .
   */
  public boolean isAncestorOf(ServerProjectFile otherFile) {
    return ServerProjectFile.isAncestor(getPath(), otherFile.getPath());
  }

  /**
   * Indica se um path  ancestral do outro.
   * 
   * @param ancestorPath .
   * @param descendantPath .
   * @return .
   */
  public static boolean isAncestor(String[] ancestorPath,
    String[] descendantPath) {
    if (ancestorPath == null || descendantPath == null) {
      return false;
    }
    if (ancestorPath.length >= descendantPath.length) {
      return false;
    }
    for (int i = 0; i < ancestorPath.length; i++) {
      if (!ancestorPath[i].equals(descendantPath[i])) {
        return false;
      }
    }
    return true;
  }

  /**
   * Move este arquivo para outro diretrio. O outro diretrio pode pertencer a
   * outro projeto.
   * 
   * @param directory Diretrio destino.
   * @throws ServerProjectFileException
   */
  public synchronized void move(ProjectFile directory)
    throws ServerProjectFileException {

    /*
     * se file-locking est habilitado, s podemos seguir se obtivermos um lock
     * exclusivo sobre o arquivo
     */
    Object lockId = null;
    try {
      lockId = acquireLock();
    }
    catch (OperationFailureException ofe) {
      throw new ServiceFailureException(String.format(
        getString("ServerProjectFile.error.move"), getName())
        + "\n" + ofe.getMessage());
    }
    boolean success = false;
    File newFile = null;
    String oldAbsolutePath = getAbsolutePath();
    try {
      ServerProjectFile dir = (ServerProjectFile) directory;
      Object SourceProjectId = getProjectId();
      Object TargetProjectId = dir.getProjectId();

      if (SourceProjectId.equals(TargetProjectId) && isDirectory()
        && isAncestorOf(dir)) {
        throw new ServiceFailureException(String.format(
          getString("ServerProjectFile.error.move.into.descendant"), getName(),
          directory.getName(), getName()));
      }
      if (SourceProjectId.equals(TargetProjectId) && isDirectory()
        && this.equals(dir)) {
        throw new ServiceFailureException(String.format(
          getString("ServerProjectFile.error.move.into.equal"), getName()));
      }
      String[] oldPath = getPath();
      newFile = new File(dir.file, file.getName());
      if (newFile.exists()) {
        String errMsg =
          String.format(getString("ServerProjectFile.error.file.exists"),
            getName(), directory.getName());
        throw new ServiceFailureException(errMsg);
      }
      /* Retira o arquivo da cache */
      ServerProjectFile.deleteInCache(this);

      success = rename(newFile);
      if (!success) {
        String errMsg =
          "ServerProjectFile:move: falha ao mover " + getAbsolutePath()
            + " para " + dir.file.getAbsolutePath();
        throw (new ServerProjectFileException(errMsg));
      }
      /* Remove o filho do diretrio onde ele estava */
      parent = dir;

      /* Alterando o projeto, caso esteja sendo movido para um diferente */
      projectId = TargetProjectId;

      /* Notifica os observadores */
      fireNewFileEvent(TargetProjectId, this);
      fireFileDeletedEvent(SourceProjectId, oldPath);
    }
    finally {
      if (success) {
        FileLockManager.getInstance().releaseLock(oldAbsolutePath, lockId);
      }
      else {
        FileLockManager.getInstance().releaseLock(this, lockId);
      }
    }
  }

  /**
   * Remove o arquivo que esse <code>ServerProjectFile</code> representa. Pode
   * ser um diretrio, arquivo ou link. Os arquivos de configurao e descrio
   * so removidos tambm. Aps chamado este mtodo, o objeto que representa o
   * arquivo no poder mais ser utilizado e deve ser coletado.
   * 
   * @throws ServerProjectFileException
   */
  public synchronized void remove() throws ServerProjectFileException {
    if (!removeFile()) {
      String errMsg =
        "ServerProjectFile:remove: falha ao remover " + getAbsolutePath();
      throw (new ServerProjectFileException(errMsg));
    }
    /* Notifica os observadores */
    fireFileDeletedEvent(projectId, getPath());
  }

  /**
   * Retorna as informaes atualizadas desse arquivo.
   * 
   * @return .
   */
  public ClientProjectFileInfo getUpdatedInfo() {
    return new ClientProjectFileInfo(getType(), size(), getModificationDate(),
      isGlobalFile(), isPublishedFile(), isUnderConstruction(), whoPublished(),
      whichProjectPublished(), isLocked());
  }

  /**
   * Indica se este arquivo  um diretrio.
   * 
   * @return Verdadeiro caso seja um diretrio e falso caso contrrio.
   */
  public boolean isDirectory() {
    return file.isDirectory();
  }

  /**
   * Indica se este arquivo  um arquivo global.
   * 
   * @return Verdadeiro caso seja um arquivo global e falso caso contrrio.
   */
  public boolean isGlobalFile() {
    return isGlobal;
  }

  /**
   * Indica se este arquivo  um arquivo publicado.
   * 
   * @return Verdadeiro caso seja um arquivo publicado e falso caso contrrio.
   */
  public boolean isPublishedFile() {
    return isPublished;
  }

  /**
   * Indica se este arquivo  um arquivo em construo.
   * 
   * @return Verdadeiro caso seja um arquivo em construo e falso caso
   *         contrrio.
   */
  public boolean isUnderConstruction() {
    return isUnderConstruction;
  }

  /**
   * {@inheritDoc}
   */
  public boolean isLocked() {
    return FileLockManager.getInstance().isLocked(this);
  }

  /**
   * Indica se este arquivo  um arquivo normal.
   * 
   * @return Verdadeiro caso seja um arquivo normal e falso caso contrrio.
   */
  public boolean isRegularFile() {
    return (!isDirectory() && !isLink);
  }

  /**
   * Obtm a descrio atual deste arquivo.
   * 
   * @return O texto com a descrio do arquivo.
   * @throws ServerProjectFileException
   */
  public synchronized String getDescription() throws ServerProjectFileException {
    return readDescription();
  }

  /**
   * Muda a descrio atual deste arquivo.
   * 
   * @param text novo texto da descrio do arquivo.
   * @throws ServerProjectFileException
   */
  public synchronized void setDescription(String text)
    throws ServerProjectFileException {
    writeDescription(text, false);
  }

  /**
   * Anexa um texto  descrio atual deste arquivo. Permite a gerao de
   * histricos de modificao.
   * 
   * @param text texto a ser anexado.
   * @throws ServerProjectFileException
   */
  public synchronized void appendDescription(String text)
    throws ServerProjectFileException {
    writeDescription(text, true);
  }

  /**
   * Obtm lock de escrita exclusivo para um arquivo. Locks exclusivos so
   * obtidos apenas uma vez.
   * 
   * @return identificador do lock se o lock foi obtido, ou null caso contrrio
   */
  public Object acquireExclusiveLock() {
    SecureKey key = null;
    if (!ProjectService.isInternalServerRequest()) {
      key = (SecureKey) ProjectService.getKey();
    }
    Object lockId =
      FileLockManager.getInstance().acquireExclusiveLock(this, key);
    if (lockId != null) {
      notifyLockStatusChanged();
    }
    return lockId;
  }

  /**
   * Tenta obter um lock de escrita exclusivo para um arquivo. Locks exclusivos
   * so obtidos apenas uma vez. Este mtodo cadastra uma solicitao de lock
   * numa fila priorizada pela ordem de chegada dos pedidos de lock. Quando o
   * lock  obtido, o mtodo FileLockListenerInterface.fileLocked  chamado.
   * Quando o tempo mximo de espera do lock  atingido e o lock no foi obtido
   * o mtodo FileLockListenerInterface.fileLockExpired  chamado.
   * 
   * @param listener observador do lock cadastrado
   * @param timeout tempo mximo de espera pelo lock em milisegundos. A
   *        constante INFINITE_TIMOUT foi criada para timout infinito.
   * @return identificador do pedido de lock
   */
  public Object acquireExclusiveLock(FileLockListenerInterface listener,
    long timeout) {
    SecureKey key = null;
    if (!ProjectService.isInternalServerRequest()) {
      key = (SecureKey) ProjectService.getKey();
    }
    InternalFileLockListener internalListener =
      new InternalFileLockListener(listener);
    return FileLockManager.getInstance().acquireExclusiveLock(this, key,
      internalListener, timeout);
  }

  /**
   * Obtm lock de escrita compartilhado para um arquivo. Locks compartilhados
   * podem ser obtidos mltiplas vezes; cada novo lock para o arquivo incrementa
   * o contador de referncias para o lock. Locks compartilhados s podem ser
   * removidos se o contador de referncias for igual a zero.
   * 
   * @return identificador do lock se o lock foi obtido, ou null caso contrrio
   */
  public Object acquireSharedLock() {
    SecureKey key = null;
    if (!ProjectService.isInternalServerRequest()) {
      key = (SecureKey) ProjectService.getKey();
    }
    Object lockId = FileLockManager.getInstance().acquireSharedLock(this, key);
    if (lockId != null) {
      notifyLockStatusChanged();
    }
    return lockId;
  }

  /**
   * Tenta obter lock de escrita compartilhado para um arquivo. Locks
   * compartilhados podem ser obtidos mltiplas vezes; cada novo lock para o
   * arquivo incrementa o contador de referncias para o lock. Locks
   * compartilhados s podem ser removidos se o contador de referncias for
   * igual a zero. Este mtodo cadastra uma solicitao de lock compartilhado em
   * uma fila priorizada pela ordem de chegada dos pedidos de lock. Quando o
   * lock  obtido, o mtodo FileLockListenerInterface.fileLocked  chamado.
   * Quando o tempo mximo de espera do lock  atingido e o lock no foi obtido
   * o mtodo FileLockListenerInterface.fileLockExpired  chamado.
   * 
   * @param listener observador do lock cadastrado
   * @param timeout tempo mximo de espera pelo lock em milisegundos. A
   *        constante INFINITE_TIMOUT foi criada para timout infinito.
   * @return identificador do pedido de lock
   */
  public Object acquireSharedLock(FileLockListenerInterface listener,
    long timeout) {
    SecureKey key = null;
    if (!ProjectService.isInternalServerRequest()) {
      key = (SecureKey) ProjectService.getKey();
    }
    InternalFileLockListener internalListener =
      new InternalFileLockListener(listener);
    return FileLockManager.getInstance().acquireSharedLock(this, key,
      internalListener, timeout);
  }

  /**
   * Libera lock de um arquivo. Se o lock  um lock exclusivo, este s 
   * efetivamente liberado se o identificador  o mesmo do lock obtido. Se o
   * lock  compartilhado, a contagem de referncias para o lock  decrementada;
   * se chega a zero, o lock  efetivamente removido.
   * 
   * @param lockId identificador do lock
   * @return contagem de referncias para o lock
   */
  public int releaseLock(Object lockId) {
    int lockRefCount = FileLockManager.getInstance().releaseLock(this, lockId);
    if (lockRefCount == 0) {
      notifyLockStatusChanged();
    }
    return lockRefCount;
  }

  /**
   * Fora a remoo do lock do arquivo. S pode ser usado pelo administrador ou
   * pelo usurio dono do projeto, se acontencer alguma situao de ter um lock
   * "preso". Situaes onde as sesso do usurio que foi finalizadas por falha
   * na comunicao, ou usurios que terminam o cliente abruptamente j esto
   * previstas e no devem precisar deste mecanismo de segurana.
   * 
   * @return contagem de referncias para o lock
   */
  public int forceReleaseLock() {
    int lockRefCount = FileLockManager.getInstance().forceReleaseLock(this);
    if (lockRefCount == 0) {
      notifyLockStatusChanged();
    }
    return lockRefCount;
  }

  /**
   * Retorna o identificador do usurio que criou esse arquivo.
   * 
   * @return O identificador do usurio que criou esse arquivo.
   */
  public Object whoCreated() {
    return createdBy;
  }

  /**
   * Retorna o identificador do projeto no qual o arquivo existe.
   * 
   * @param file .
   * @return O identificador do projeto ou null, caso esse arquivo no esteja
   *         no rea de projeto de usurio algum.
   * @throws ServerProjectFileException
   */
  public static String findFileProjectId(File file)
    throws ServerProjectFileException {
    return findFileProjectIdFromDir(file, FileSystem.PROJECT_DIR);
  }

  /**
   * Retorna o identificador do projeto no qual o arquivo est publicado.
   * 
   * @param file .
   * @return O identificador do projeto ou null, caso esse arquivo no esteja
   *         no rea de projeto de usurio algum.
   * @throws ServerProjectFileException
   */
  public static String findFileProjectIdFromPublish(File file)
    throws ServerProjectFileException {
    return findFileProjectIdFromDir(file, FileSystem.PUBLISH_DIR);
  }

  /**
   * Retorna o identificador do projeto no qual o arquivo existe.
   * 
   * @param file .
   * @param dir .
   * @return O identificador do projeto ou null, caso esse arquivo no esteja
   *         no rea de projeto de usurio algum.
   * @throws ServerProjectFileException
   */
  private static String findFileProjectIdFromDir(File file, String dir)
    throws ServerProjectFileException {
    try {
      String relativePath = findFilePath(file, dir);
      if (relativePath == null) {
        return null;
      }
      String[] path = FileUtils.splitPath(relativePath);
      return FileSystem.PROJECT_DIR + File.separator + path[0] + File.separator
        + path[1];
    }
    catch (Exception e) {
      String errMsg =
        "ServerProjectFile:findFileProjectIdFromDir: erro "
          + "ao pegar o projeto do arquivo " + file.getAbsolutePath();
      throw (new ServerProjectFileException(errMsg, e));
    }
  }

  /**
   * Retorna o identificador do usurio que criou o projeto no qual o arquivo
   * est publicado.
   * 
   * @param file .
   * @return O identificador do dono do projeto ou null, caso esse arquivo no
   *         esteja no rea de projeto de usurio algum.
   * @throws ServerProjectFileException
   */
  public static String findPubFileUserId(File file)
    throws ServerProjectFileException {
    try {
      String relativePath = findFilePath(file, FileSystem.PUBLISH_DIR);
      if (relativePath == null) {
        return null;
      }
      String[] path = FileUtils.splitPath(relativePath);
      return path[0];
    }
    catch (Exception e) {
      String errMsg =
        "ServerProjectFile:findFileUserId: erro ao pegar "
          + "o projeto do arquivo " + file.getAbsolutePath();
      throw (new ServerProjectFileException(errMsg, e));
    }
  }

  /**
   * Retorna o caminho relativo do arquivo a partir do seu projeto. Exemplo:
   * /home/.../project/user/proj/file = proj/file
   * 
   * @param file .
   * @return .
   * @throws ServerProjectFileException
   */
  public static String[] findFileRelativePath(File file)
    throws ServerProjectFileException {
    try {
      String path = findFilePath(file, FileSystem.PROJECT_DIR);
      if (path == null) {
        return null;
      }
      String[] totalPath = FileUtils.splitPath(path);
      String[] relPath = new String[totalPath.length - 1];
      for (int i = 0; i < relPath.length; i++) {
        relPath[i] = totalPath[i + 1];
      }
      return relPath;
    }
    catch (Exception e) {
      String errMsg =
        "ServerProjectFile:findFileRelativePath: erro ao pegar "
          + "o caminho relativo do arquivo" + file.getAbsolutePath();
      throw (new ServerProjectFileException(errMsg, e));
    }
  }

  /**
   * Retorna o caminho do arquivo a partir do diretrio de projetos representado
   * pelo PROJECT_DIR ou do diretrio de publicao PUBLISHED_DIR. Excluindo a
   * String o PROJECT_DIR e PUBLISHED_DIR.
   * 
   * @param file .
   * @param dir .
   * @return .
   * @throws ServerProjectFileException
   */
  private static String findFilePath(File file, String dir)
    throws ServerProjectFileException {
    try {
      String path = file.getPath();
      if (dir.charAt(dir.length() - 1) != File.separatorChar) {
        dir += File.separator;
      }
      int index = path.indexOf(dir);
      if (index == -1) {
        return null;
      }
      return path.substring(index + dir.length());
    }
    catch (Throwable e) {
      String errMsg =
        "ServerProjectFile:findFilePath: erro ao pegar o "
          + "caminho do arquivo a partir do diretrio de projetos"
          + file.getAbsolutePath();
      throw (new ServerProjectFileException(errMsg, e));
    }
  }

  /**
   * Publica este arquivo.
   * 
   * @param userIds Lista contendo os identificadores dos usurios que tero
   *        acesso ao arquivo publicado.
   * @param whoPublishedId Identificador do usurio que publicou o arquivo.
   * @return .
   * @throws ServerProjectFileException
   */
  public synchronized boolean publish(Object[] userIds, Object whoPublishedId)
    throws ServerProjectFileException {
    try {
      if (!isRegularFile()) {
        String errMsg =
          "ServerProjectFile:publish: o arquivo " + getAbsolutePath()
            + " no  um arquivo regular";
        throw (new ServerProjectFileException(errMsg));
      }
      publishAccess = new Hashtable<Object, Object>();
      for (int i = 0; i < userIds.length; i++) {
        publishAccess.put(userIds[i], whoPublishedId);
      }
      isPublished = true;
      publishedByProject = projectId;
      if (!writeConfigurationFile(configFile)) {
        String errMsg =
          "ServerProjectFile:publish: falha ao atualizar "
            + configFile.getAbsolutePath();
        publishAccess = null;
        isPublished = false;
        publishedByProject = null;
        throw (new ServerProjectFileException(errMsg));
      }
      File oldFile = file;
      File oldConfigFile = configFile;
      String path = getAbsolutePath();
      String separator = File.separator;
      String from = separator + FileSystem.PROJECT_DIR + separator;
      String to = separator + FileSystem.PUBLISH_DIR + separator;
      from = from.replaceAll("\\\\", "\\\\\\\\");
      to = to.replaceAll("\\\\", "\\\\\\\\");
      String newPath = path.replaceFirst(from, to);
      File newFile = new File(newPath);
      if (newFile.exists()) {
        String errMsg =
          String.format(
            getString("ServerProjectFile.error.public.file.exists"),
            whoPublishedId, getName(), newFile.getParent());
        throw new ServiceFailureException(errMsg);
      }
      if (!rename(newFile)) {
        publishAccess = null;
        isPublished = false;
        publishedByProject = null;
        writeConfigurationFile(configFile);
        String errMsg =
          "ServerProjectFile:publish: " + " Erro ao renomear "
            + oldFile.getAbsolutePath() + " para " + newFile.getAbsolutePath();
        throw (new ServerProjectFileException(errMsg));
      }
      file = oldFile;
      configFile = oldConfigFile;
      realFile = newFile;
      realConfigFile = renamedFileConfig(newFile);
      isLink = true;
      createLink(realFile, file);
      // Cria mensagem no arquivo de descrio do arquivo publicado
      ServerProject sp = ServerProject.getProject(projectId);
      Object userId = sp.getUserId();
      String userName = User.getName(userId);
      if (userName == null) {
        userName = (String) userId;
      }
      String whoPublishedName = User.getName(whoPublishedId);
      String message =
        "Em " + Utilities.getFormattedDate((new Date()).getTime())
          + "\nArquivo do projeto " + sp.getName() + " do usurio " + userName
          + "\nfoi publicado por " + whoPublishedName + ".\n";
      File realDescFile = renamedFileDescription(newFile);
      writeDescription(message, true, realDescFile);
      /*
       * Notifica os observadores do estado do arquivo:
       */
      boolean[] state = { isGlobal, isPublished, isUnderConstruction };
      fireStateChangedEvent(projectId, file.getName(), getPath(), state,
        whoPublishedId, new Long(getModificationDate()), new Long(size()),
        FileLockManager.getInstance().isLocked(this));
      return true;
    }
    catch (Exception e) {
      String errMsg = "Erro ao publicar " + getName() + ".\n " + e.getMessage();
      throw (new ServerProjectFileException(errMsg, e));
    }
  }

  /**
   * Este mtodo no est sendo usado! A implementao deste mtodo est
   * definida em: public synchronized boolean updatePublishAccess ( Object[]
   * usersToRemove, Object[] usersToInsert, Object userId) throws
   * ServerProjectFileException
   * 
   * @param userIds .
   * @param userId .
   * @return .
   * @throws ServerProjectFileException
   */
  public boolean updatePublishAccess(Object[] userIds, Object userId)
    throws ServerProjectFileException {
    return updatePublishAccess(null, userIds, userId);
  }

  /**
   * Atualiza a lista dos usurios para o qual um arquivo est publicado. O
   * arquivo j precisa estar publicado.
   * 
   * @param usersToRemove Identificadores do usurios que no devem mais ter
   *        acesso ao arquivo publicado.
   * @param usersToInsert Identificadores dos usurios que passam a ter acesso
   *        ao arquivo publicado.
   * @param userId Identificador do usurio que publica o arquivo
   * @return .
   * @throws ServerProjectFileException
   */
  public synchronized boolean updatePublishAccess(Object[] usersToRemove,
    Object[] usersToInsert, Object userId) throws ServerProjectFileException {
    if (!isPublished || (publishAccess == null)) {
      String errMsg =
        "ServerProjectFile:updatePublishAccess: o arquivo " + getAbsolutePath()
          + " no est publicado";
      throw (new ServerProjectFileException(errMsg));
    }

    // Isto impede q arquivos publicados sejam re-publicados por quem os obteve
    if (!projectId.equals(publishedByProject)) {
      String errMsg =
        "ServerProjectFile:updatePublishAccess: o arquivo "
          + file.getAbsolutePath() + " obtido por publicao no pode ser"
          + " publicado novamente";
      throw (new ServerProjectFileException(errMsg));
    }
    /* FALTA O LOCK DO ARQUIVO DE CONFIGURAO */
    readConfiguration();
    /* Remove os usurios da publicao */
    for (int i = 0; i < usersToRemove.length; i++) {
      publishAccess.remove(usersToRemove[i]);
    }
    /* Insere os usurios na publicao */
    for (int i = 0; i < usersToInsert.length; i++) {
      publishAccess.put(usersToInsert[i], userId);
    }
    if (!writeConfiguration()) {
      String errMsg =
        "ServerProjectFile:updatePublishAccess: falha ao " + "atualizar "
          + realConfigFile.getAbsolutePath();
      throw (new ServerProjectFileException(errMsg));
    }
    return true;
  }

  /**
   * Despublica este arquivo. Despublicar significa retirar todos os usurios da
   * lista de acesso ao arquivo publicado. Assim, quem tem link para ele
   * continua tendo, quem no tem, no consegue mais. S o dono pode despublicar
   * o arquivo.
   * 
   * @return .
   */
  /*
   * public synchronized boolean unPublish(Object userId) throws
   * ServerProjectFileException { if (!isPublished || (publishAccess == null)) {
   * String errMsg = "ServerProjectFile:unPublish: o arquivo " +
   * getAbsolutePath() + " no est publicado"; throw (new
   * ServerProjectFileException(errMsg)); } //Isto impede q arquivos publicados
   * sejam re-publicados por quem os obteve if
   * (!projectId.equals(publishedByProject)) { String errMsg =
   * "ServerProjectFile:updatePublishAccess: o arquivo " +
   * file.getAbsolutePath() + " obtido por publicao no pode ser" + "
   * publicado novamente"; throw (new ServerProjectFileException(errMsg)); }
   * publishAccess = null; if (!writeConfiguration()) { String errMsg =
   * "ServerProjectFile:unPublish: falha ao " + "atualizar " +
   * realConfigFile.getAbsolutePath(); throw (new
   * ServerProjectFileException(errMsg)); } return true; }
   */
  /**
   * Informa o tamanho corrente deste arquivo. Reflete o nmero corrente de
   * bytes do arquivo.
   * 
   * @return Tamanho corrente do arquivo.
   */
  public long size() {
    return realFile.length();
  }

  /**
   * Informa a data da ltima modificao deste arquivo (formato numrico).
   * 
   * @return data da ltima modificao deste arquivo (formato numrico)
   */
  public long getModificationDate() {
    return realFile.lastModified();
  }

  /**
   * Abre este arquivo para leitura e escrita.
   * 
   * @param isReadOnly - indica se o canal deve ser aberto apenas para leitura
   * @param userKey - chave para a sesso do usurio
   * @return O ProjectFileChannel que deve ser enviado para o cliente e que se
   *         conecta ao arquivo, permitindo sua leitura e escrita.
   * @throws ServerProjectFileException
   */
  public ProjectFileChannel openChannel(boolean isReadOnly, Object userKey)
    throws ServerProjectFileException {
    try {
      String path = realFile.getAbsolutePath();
      byte[] pathBytes = path.getBytes("UTF8");
      SecureKey sk = (SecureKey) userKey;
      byte[] sessionHash = sk.toString().getBytes("UTF8");
      getService().logDebugMessage(
        "Channel do arquivo aberto: " + realFile.getPath());
      return new ProjectFileChannelFromServer(pathBytes, isRegularFile(),
        ProjectService.getHostName(), ProjectService.getFileChannelPort(),
        isReadOnly, sessionHash, getName());
    }
    catch (UnsupportedEncodingException e) {
      String errMsg = "Falha ao abrir " + realFile.getAbsolutePath() + ". ";
      throw (new ServerProjectFileException(errMsg, e));
    }
  }

  /**
   * Retorna um texto descritivo do arquivo.
   * 
   * @return .
   */
  @Override
  public String toString() {
    String text = "";
    text = text + "File: " + file.getAbsolutePath() + "\n";
    if (parent != null) {
      text = text + "Parent: " + parent.getAbsolutePath() + "\n";
    }
    text = text + "isLink: " + isLink + "\n";
    if (isLink) {
      text = text + "realFile: " + file.getAbsolutePath() + "\n";
    }
    text = text + "isGlobal: " + isGlobal + "\n";
    text = text + "isPublished: " + isPublished + "\n";
    text = text + "isUnderConstruction: " + isUnderConstruction + "\n";
    text = text + "Type : " + type + "\n";
    text = text + "Count : " + count + "\n";
    text = text + "CreatedBy:" + createdBy + "\n";
    text = text + "Project :" + projectId + "\n";
    return text;
  }

  /**
   * Obtm a referncia para um arquivo/diretrio no servidor, pertencente ao
   * projeto especificado.
   * 
   * @param projectId identificador do projeto.
   * @param path caminho para o arquivo/diretrio, incluindo o nome do projeto.
   * @return <code>ServerProjectFile</code> representando o arquivo/diretrio
   * @throws ServiceFailureException caso o arquivo no seja encontrado
   */
  public static ServerProjectFile findFile(Object projectId, String[] path)
    throws ServiceFailureException {
    ServerProject sp = ServerProject.getProject(projectId);
    if (sp == null) {
      throw new ServiceFailureException("O projeto " + projectId
        + " no est aberto.");
    }
    /*
     * no devemos prosseguir se o path fornecido no possui o nome do projeto,
     * pois o loop que procura pelos arquivos assume que path[0] == projeto
     */
    if (!sp.getName().equals(path[0])) {
      throw new ServiceFailureException(
        "path para o arquivo deveria conter o projeto");
    }

    ServerProjectFile spf = (ServerProjectFile) sp.getTree();

    for (int i = 1; i < path.length; i++) {
      ServerProjectFile parent = spf;
      spf = (ServerProjectFile) parent.getChild(path[i]);
      // Se no achou, lana ServiceFailureException. Antes, verifica
      // se o arquivo possui arquivos de controle. Se tiver, os remove
      // para que no fique lixo
      if (spf == null) {
        String parentPath = parent.getCanonicalPath();
        String filePath = parentPath + File.separator + path[path.length - 1];
        File file = new File(filePath);
        File configFile = FileConfig.renamedFileConfig(file);
        if (configFile.exists()) {
          configFile.delete();
        }
        File descriptionFile = FileConfig.renamedFileDescription(file);
        if (descriptionFile.exists()) {
          descriptionFile.delete();
        }

        ProjectService service = ProjectService.getInstance();
        // FIXME usar StringBuilder
        String filePathStr = "/";
        for (int j = 1; j < path.length - 1; j++) {
          filePathStr = filePathStr + path[j];
        }
        throw new ServiceFailureException(String.format(service
          .getString("ServerProjectFile.error.file.not.found"),
          path[path.length - 1], filePathStr, sp.getName()));
      }
    }
    return spf;
  }

  /**
   * Obtm o arquivo de configurao associado a um arquivo real.
   * 
   * @param realFile O arquivo real, cuja configurao est sendo requisitada.
   * @return O arquivo de configurao de um arquivo de projeto ou null, caso
   *         esse arquivo no exista.
   */
  public static FileConfig getFileConfig(File realFile) {
    try {
      File configFile = renamedFileConfig(realFile);
      FileConfig config = new FileConfig();
      if (!config.readConfigurationFile(configFile)) {
        return null;
      }
      return config;
    }
    catch (Exception e) {
      return null;
    }
  }

  /**
   * Muda o tipo de um arquivo, alterando ou criando um arquivo de configurao
   * no caso desse no existir.
   * 
   * @param realFile O arquivo real, cuja configurao est sendo requisitada.
   * @param type O tipo do arquivo real.
   * @throws ServerProjectFileException
   */
  public static void setFileType(File realFile, String type)
    throws ServerProjectFileException {
    try {
      File configFile = renamedFileConfig(realFile);
      FileConfig config = new FileConfig();
      if (configFile.exists()) {
        config.readConfigurationFile(configFile);
      }
      config.isGlobal = true;
      config.type = type;
      if (!config.writeConfigurationFile(configFile)) {
        throw (new Exception());
      }
    }
    catch (Exception e) {
      String errMsg =
        "createFileConfigToGlobalFile:" + "Erro na criao da configurao "
          + realFile.getAbsolutePath() + " - " + e.getMessage();
      throw (new ServerProjectFileException(errMsg));
    }
  }

  /**
   * Notifica aos observadores que o estado do arquivo mudou.
   * 
   * @param projectId identificador do projeto
   * @param fileName nome do arquivo
   * @param path caminho do arquivo
   * @param state array de booleano de tamanho 3, o primeiro valor indica se o
   *        arquivo  global, o segundo se  o arquivo est publicado e o
   *        terceiro se o arquivo est em construo
   * @param whoPublished quem publicou o arquivo
   * @param modificationDate a data da ltima modificao
   * @param size o tamanho do arquivo
   * @param locked se o arquivo est bloqueado
   */
  private static void fireStateChangedEvent(Object projectId, String fileName,
    String[] path, boolean[] state, Object whoPublished, Long modificationDate,
    Long size, boolean locked) {
    Object[] arg =
      { path, fileName, state, whoPublished, projectId, modificationDate, size,
          locked };
    ServerProject project = ServerProject.getProject(projectId);
    project.notifyObservers(ProjectEvent.makeEvent(
      ProjectEvent.FILE_STATE_CHANGED, arg));
  }

  /**
   * Notifica aos observadores que um arquivo foi criado.
   * 
   * @param projectId identificador do projeto
   * @param file novo arquivo criado
   */
  private static void fireNewFileEvent(Object projectId, ServerProjectFile file) {

    Object[] args = new Object[2];
    if (null == file.getParent()) {
      args[0] = new String[0];
    }
    else {
      args[0] = file.getParent().getPath();
    }
    args[1] = ProjectService.buildClientProjectSubtree(projectId, null, file);

    ServerProject project = ServerProject.getProject(projectId);
    project
      .notifyObservers(ProjectEvent.makeEvent(ProjectEvent.NEW_FILE, args));
  }

  /**
   * Notifica aos observadores da criao simultnea de um ou mais arquivos.
   * 
   * @param projectId identificador do projeto.
   * @param files referncias para os arquivos recm-criados.
   */
  private static void fireNewFilesEvent(Object projectId,
    ServerProjectFile[] files) {

    String[][] paths = new String[files.length][];
    ClientProjectFile[] clientProjectFiles =
      new ClientProjectFile[files.length];
    for (int fileInx = 0; fileInx < files.length; fileInx++) {
      // O arquivo que foi criado.
      ServerProjectFile file = files[fileInx];
      // Cria um ClientProjectFile representando o caminho at o novo arquivo. 
      if (null == file.getParent()) {
        paths[fileInx] = new String[0];
      }
      else {
        paths[fileInx] = file.getParent().getPath();
      }
      clientProjectFiles[fileInx] =
        ProjectService.buildClientProjectSubtree(projectId, null, file);
    }

    Object[] args = { paths, clientProjectFiles };
    ServerProject project = ServerProject.getProject(projectId);
    project.notifyObservers(ProjectEvent
      .makeEvent(ProjectEvent.NEW_FILES, args));
  }

  /**
   * Notifica aos observadores que um arquivo foi removido.
   * 
   * @param projectId .
   * @param path .
   */
  private static void fireFileDeletedEvent(Object projectId, String[] path) {
    Object[] arg = { path };
    ServerProject project = ServerProject.getProject(projectId);
    project.notifyObservers(ProjectEvent.makeEvent(ProjectEvent.FILE_DELETED,
      arg));
  }

  /**
   * Notifica aos observadores que um arquivo mudou de nome e/ou tipo.
   * 
   * @param projectId .
   * @param path .
   * @param newName .
   * @param newType .
   */
  private static void fireNewFileNameEvent(Object projectId, String[] path,
    String newName, String newType) {
    Object[] arg = { path, newName, newType };
    ServerProject project = ServerProject.getProject(projectId);
    project.notifyObservers(ProjectEvent.makeEvent(ProjectEvent.NEW_FILE_NAME,
      arg));
  }

  /**
   * Notifica aos observadores que um diretrio foi atualizado com os dados do
   * servidor.
   * 
   * @param projectId identificador do projeto.
   * @param dir referncia para o objeto no servidor representando o diretrio.
   */
  private static void fireDirRefreshedEvent(Object projectId,
    ClientProjectFile dir) {
    Object[] arg = { dir.getPath(), dir };
    ServerProject project = ServerProject.getProject(projectId);
    project.notifyObservers(ProjectEvent.makeEvent(ProjectEvent.DIR_REFRESHED,
      arg));
  }

  /**
   * Notifica que o arquivo foi bloqueado ou desbloqueado.
   */
  private void notifyLockStatusChanged() {
    try {
      boolean isLocked = FileLockManager.getInstance().isLocked(this);

      fireStateChangedEvent(getProjectId(), getName(), getPath(),
        new boolean[] { isGlobalFile(), isPublishedFile(),
            isUnderConstruction() }, whoPublished(), getModificationDate(),
        size(), isLocked);
    }
    catch (Exception e) {
      // No interrompe o mecanimo de bloqueio/desbloquio se a notificao
      // falhar.
      ProjectService.getInstance().logWarningMessage(
        "Falha na notificao de bloqueio/desbloqueio de lock do arquivo "
          + file.getAbsolutePath());
    }
  }

  /**
   * L e devolve o contedo do arquivo de descrio associado a um
   * <code>Project File</code>. Se o arquivo no existir, devolve "".
   * 
   * @return .
   * @throws ServerProjectFileException
   */
  private String readDescription() throws ServerProjectFileException {
    File f = renamedFileDescription(realFile);
    if (!f.exists()) {
      String errMsg =
        getName() + " : " + "No existe arquivo de descrio " + f.toString()
          + " para [" + realFile.toString() + "] -" + " ["
          + createDescriptionFileName(realFile.getName()) + "]";
      ProjectService.getInstance().logInfoMessage(errMsg);
      return "";
    }
    try {
      Reader r = new FileReader(f);
      char[] buffer = new char[(int) f.length()];
      r.read(buffer);
      return new String(buffer);
    }
    catch (IOException e) {
      String errMsg =
        "ServerProjectFile:readDescription: falha ao " + "ler "
          + f.getAbsolutePath() + ".";
      throw (new ServerProjectFileException(errMsg, e));
    }
  }

  /**
   * Grava um texto no arquivo de descrio.
   * 
   * @param text O texto a ser gravado.
   * @param append Se  para ser gravado como apndice ou no.
   * @throws ServerProjectFileException
   */
  private void writeDescription(String text, boolean append)
    throws ServerProjectFileException {
    if (isLink) {
      return;
    }
    File f = renamedFileDescription(realFile);
    writeDescription(text, append, f);
  }

  /**
   * Atualiza a descrio. O novo texto pode ser acrescentado  descrio
   * existente ou pode substitui-la.
   * 
   * @param text texto a ser anexado
   * @param append <code>true</code> se o texto deve ser anexado,
   *        <code>false</code> se deve substituir a descrio existente
   * @param descFile arquivo com a descrio
   * @throws ServerProjectFileException
   */
  private static void writeDescription(String text, boolean append,
    File descFile) throws ServerProjectFileException {
    try {
      Writer w = new FileWriter(descFile, append);
      text = "----------------------------------\n" + text + "\n\n";
      w.write(text);
      w.flush();
      w.close();
    }
    catch (IOException e) {
      String errMsg =
        "ServerProjectFile:writeDescription: falha ao " + "escrever em "
          + descFile.getAbsolutePath() + ".";
      throw (new ServerProjectFileException(errMsg, e));
    }
  }

  /**
   * L o arquivo de configurao passado como parmetro, se ele existir.
   * 
   * @param configFile O objeto <code>File</code> que representa o arquivo de
   *        configurao a ser lido.
   * @throws ServerProjectFileException
   */
  private void readConfiguration(File configFile)
    throws ServerProjectFileException {
    if (!configFile.exists()) {
      return;
    }
    if (!configFile.isFile()) {
      String errMsg =
        "ServerProjectFile:readConfiguration: " + configFile.getAbsolutePath()
          + " no  arquivo";
      throw (new ServerProjectFileException(errMsg));
    }
    if (!readConfigurationFile(configFile)) {
      String errMsg =
        "ServerProjectFile:readConfiguration: falha na "
          + "leitura do arquivo de configurao "
          + configFile.getAbsolutePath();
      throw (new ServerProjectFileException(errMsg));
    }
  }

  /**
   * L o arquivo de configurao desse <code>ServerProjectFile</code>. Aps ter
   * sido lido, esse objeto possui as informaes de configurao lidas do
   * arquivo.
   * 
   * @throws ServerProjectFileException
   */
  private void readConfiguration() throws ServerProjectFileException {
    String fileName = file.getName();
    if (fileName.endsWith(FileSystem.LINK_SUFFIX)) {
      fileName =
        fileName.substring(0, fileName.lastIndexOf(FileSystem.LINK_SUFFIX));
    }
    configFile = renamedFileConfig(file);
    readConfiguration(configFile);
    if (!isLink) {
      return;
    }
    realConfigFile = renamedFileConfig(realFile);
    readConfiguration(realConfigFile);
    isLink = true;
  }

  /**
   * Grava o arquivo de configurao desse <code>ServerProjectFile</code>.
   * 
   * @return .
   */
  private boolean writeConfiguration() {
    if (!isLink) {
      return writeConfigurationFile(configFile);
    }
    isLink = false;
    boolean result = writeConfigurationFile(realConfigFile);
    isLink = true;
    return result;
  }

  /**
   * Ordena os filhos desse <code>ServerProjectFile</code> que representa um
   * diretrio.
   * 
   * @param children - lista de filhos.
   */
  private void sortChildren(final ServerProjectFile[] children) {
    Arrays.sort(children, nameComparator);
  }

  /**
   * A partir do sistema de arquivos, reconstri os filhos desse
   * <code>ServerProjectFile</code> que representa um diretrio. So
   * instanciados novos <code>ServerProjectFiles</code> que passam a ser seus
   * filhos.
   * 
   * @return array com os filhos do diretrio
   * 
   * @throws ServerProjectFileException
   * 
   * @see #getNewTree(String)
   */
  private ServerProjectFile[] getNewTree() throws ServerProjectFileException {
    return getNewTree(null);
  }

  /**
   * Gera um array com os filhos diretos deste diretrio, filtrando pelo tipo.
   * 
   * @param type tipo dos arquivos (para filtragem). Pode ser <code>null</code>,
   *        o que significa que todos os arquivos/diretrios sero considerados.
   *        OBS.: diretrios so sempre considerados, independente do filtro.
   * 
   * @return array com os filhos do diretrio, filtrados pelo tipo
   * 
   * @throws ServerProjectFileException
   * 
   * @see #getNewTree()
   */
  private ServerProjectFile[] getNewTree(String type)
    throws ServerProjectFileException {
    if (!file.isDirectory()) {
      String errMsg =
        "ServerProjectFile:buildTree: " + file.getAbsolutePath()
          + " no  diretrio";
      throw (new ServerProjectFileException(errMsg));
    }
    File[] files = getFileListWithoutControlFiles();

    final ServerProjectFile[] children = new ServerProjectFile[files.length];
    int nextChild = 0;
    for (final File child : files) {
      ServerProjectFile spf =
        ServerProjectFile.buildFromCache(child, child, projectId, this);

      /*
       * Tenta inferir o tipo pela extensao, caso o arquivo nao tenha o arquivo
       * de configuracao
       */
      try {
        File cfg = spf.getConfigFile();
        if ((cfg == null) || (!cfg.exists())) {
          ServerProjectFileType spft = ServerProjectFileType.getInstance();
          spf.setType(spft.getTypeFromExtension(spf.getName()));
        }
      }
      catch (Exception e) {
        ProjectService.getInstance().logWarningMessage(
          "Falha na obtencao do tipo a partir da extensao: " + e);
      }
      /*
       * apenas registra como filhos os arquivos com o tipo correto
       */
      if (type == null || spf.isDirectory() || type.equals(spf.getType())) {
        children[nextChild++] = spf;
      }
    }
    sortChildren(children);
    return children;
  }

  /**
   * @return lista de filhos do diretrio corrente, desconsiderando os arquivos
   *         de controle; retorna <code>null</code> se o arquivo no  um
   *         diretrio
   */
  private File[] getFileListWithoutControlFiles() {
    return file.listFiles(new FilenameFilter() {

      @Override
      public boolean accept(File dir, String childName) {
        return !isControlledFileName(childName);
      }
    });
  }

  /**
   * Move o arquivo, o arquivo de configurao e o arquivo de descrio.
   * 
   * @param newFile .
   * @return .
   */
  private boolean rename(File newFile) {
    ProjectService srv = ProjectService.getInstance();
    /* Tenta renomear o arquivo */
    File parentDir = new File(newFile.getParent());
    parentDir.mkdirs();
    if (!file.exists()) {
      srv.logSevereMessage("Impossvel localizar [" + file.getAbsolutePath()
        + "] para fazer o rename!");
      return false;
    }

    // Tenta mover o arquivo.
    if (!move(file, newFile)) {
      if (srv != null) {
        srv.logSevereMessage("Impossvel renomear de ["
          + file.getAbsolutePath() + "] para [" + newFile.getAbsolutePath()
          + "]");
        return false;
      }
    }
    /*
     * Tenta renomear o arquivo de configurao, se existir. Se no conseguir,
     * desfaz o anterior
     */
    File newConfigFile = renamedFileConfig(newFile);
    if (configFile.exists()) {
      if (!move(configFile, newConfigFile)) {
        String errMsg =
          "ServerProjectFile:rename: falha ao mover "
            + configFile.getAbsolutePath() + " para "
            + newConfigFile.getAbsolutePath();
        if (!move(newFile, file)) {
          errMsg =
            errMsg + "\nServerProjectFile:rename: falha ao mover "
              + newFile.getAbsolutePath() + " de volta para "
              + getAbsolutePath();
          configFile.delete();
          configFile = newConfigFile;
          writeConfiguration();
        }
        if (srv != null) {
          srv.logSevereMessage(errMsg);
        }
        return false;
      }
      configFile = newConfigFile;
    }
    /*
     * Tenta renomear o arquivo de descrio, se existir. Se no conseguir,
     * abandona a descrio.
     */
    File descrFile = renamedFileDescription(file);
    File newDescrFile = renamedFileDescription(newFile);
    if (descrFile.exists()) {
      if (!move(descrFile, newDescrFile)) {
        String errMsg =
          "ServerProjectFile:rename: falha ao mover "
            + descrFile.getAbsolutePath() + " de volta para "
            + newDescrFile.getAbsolutePath();
        if (srv != null) {
          srv.logSevereMessage(errMsg);
        }
        descrFile.delete();
        return false;
      }
      descrFile = newDescrFile;
    }
    /* Atualiza as referncias para o arquivo e o arquivo real. */
    file = newFile;
    if (!isLink) {
      realFile = newFile;
    }
    return true;
  }

  /**
   * Tenta mover o arquivo com o mtodo renameTo da classe File. Caso no
   * consiga, usa o mtodo move da classe FileSystem.
   * 
   * @param from .
   * @param to .
   * @return .
   */
  private boolean move(File from, File to) {
    if (!from.renameTo(to)) {
      return FileSystem.move(from, to);
    }
    return true;
  }

  /**
   * Copia o arquivo representado por este <code>ServerProjectFile</code> para
   * um outro arquivo passado como parmetro. Se for um link, o novo arquivo 
   * criado a partir do contedo do link, mas o histrico  mantido. Se for um
   * arquivo comum, seu contedo  copiado juntamente com os arquivos de
   * configurao e descrio, se esses existirem. Se for um diretrio, esse
   * mtodo  chamado recursivamente para cada um de seus filhos. No caso de
   * erro, a operao  desfeita, removendo os arquivos que tiverem sido
   * copiados.
   * 
   * @param newFile .
   * @return Verdadeiro, se a arquivo, link ou diretrio foi copiado com sucesso
   *         ou falso, caso contrrio.
   * @throws ServerProjectFileException em caso de erro
   */
  private boolean copyFile(File newFile) throws ServerProjectFileException {

    if (!checkCopyPermission(newFile)) {
      return false;
    }

    /* Se for link, um novo arquivo  criado com o contedo do link */
    if (isLink) {
      return copyLinkFile(newFile);
    }
    /*
     * Se for um arquivo comum, copia seu contedo e, se houver, copia tambm os
     * arquivos de configurao e de descrio.
     */
    if (!file.isDirectory()) {
      return copyRegularFile(newFile);
    }
    /* Se for diretrio, copia recursivamente seus filhos */
    if (!newFile.mkdirs()) {
      String errMsg =
        "ServerProjectFile:copyFile: erro na criao de diretrio "
          + newFile.getAbsolutePath();
      ProjectService.getInstance().logWarningMessage(errMsg);
      return false;
    }
    ServerProjectFile[] children = (ServerProjectFile[]) getChildren();
    boolean success = true;
    for (int i = 0, n = children.length; i < n; i++) {
      File newChildFile = new File(newFile, children[i].file.getName());
      success = children[i].copyFile(newChildFile);
      if (!success) {
        String errMsg =
          "ServerProjectFile:copyFile: erro na cpia do filho "
            + children[i].getAbsolutePath() + " para "
            + newChildFile.getAbsolutePath();
        ProjectService.getInstance().logWarningMessage(errMsg);
        break;
      }
    }
    return success;
  }

  /**
   * Verifica Se este arquivo pode ser copiado para o arquivo de destino.<br>
   * As condies que no permitiriam esta cpia so:
   * <ul>
   * <li>Este arquivo  o que o diretrio de destino. No se pode copiar um
   * diretrio para dentro de si mesmo, isso geraria um <i>loop</i> infinito.</li>
   * <li>Este arquivo  o mesmo que o de destino. Isso geraria um <i>loop</i>
   * infinito</li>
   * <li>Este arquivo  um diretrio e o destino  um subdiretrio deste. Se
   * isso fosse permitido, iria gerar um loop infinito de cpias.</li>
   * </ul>
   * 
   * @param newFile arquivo de destinho.
   * @return <tt>true</tt> Se este arquivo pode ser copiado para o arquivo de
   *         destino.
   * @throws ServerProjectFileException
   */
  private boolean checkCopyPermission(File newFile)
    throws ServerProjectFileException {

    if (!isDirectory()) {
      return true;
    }

    String targetProjectId = findFileProjectId(newFile);
    if (null == targetProjectId) {
      return false;
    }

    File targetDir = newFile.getParentFile();
    if (null == targetDir) {
      return true;
    }

    if (getProjectId().equals(targetProjectId) && isDirectory()
      && this.file.equals(targetDir)) {
      throw new ServiceFailureException(String.format(
        getString("ServerProjectFile.error.copy.into.equal"), getName()));
    }

    ServerProjectFile targetServerProjectDir =
      ServerProjectFile.buildFromCache(targetDir, targetDir, targetProjectId,
        null);
    if (null != targetServerProjectDir) {

      if (getProjectId().equals(targetProjectId)
        && isAncestorOf(targetServerProjectDir)) {

        throw new ServiceFailureException(String.format(
          getString("ServerProjectFile.error.copy.into.descendant"), getName(),
          targetServerProjectDir.getName()));
      }
    }

    /*
     * Neste ponto uma representao do novo arquivo ser criada pelo mtodo
     * ServerProjectFile#buildFromCache(File, File, Object, ServerProjectFile).
     */
    ServerProjectFile newServerProjectFile =
      ServerProjectFile.buildFromCache(newFile, newFile, targetProjectId,
        targetServerProjectDir);
    if (null == newServerProjectFile || this.equals(newServerProjectFile)) {
      return false;
    }

    return true;
  }

  /**
   * Copia arquivos publicados ou globais, que so links. A cpia  um novo
   * arquivo, com o contedo do link. O arquivo de descrio  mantido e um novo
   * texto  appendado para indicar a cpia.
   * 
   * @param newFile .
   * @return .
   */
  private boolean copyLinkFile(File newFile) {
    try {
      User user = ProjectService.getUser();
      String projId = findFileProjectId(newFile);
      if (user == null || projId == null) {
        String errMsg =
          "ServerProjectFile:copyFile: falha na cpia do link "
            + newFile.getAbsolutePath() + " para " + realFile.getAbsolutePath();
        ProjectService.getInstance().logSevereMessage(errMsg);
        return false;
      }

      ServerProjectFile spf =
        ServerProjectFile.buildFromCache(newFile, newFile, projId, null);

      spf.configure(getType(), user.getId());
      /* Copia o arquivo */
      if (!FileSystem.copyFile(realFile, newFile)) {
        String errMsg =
          "ServerProjectFile:copyFile: erro na cpia fsica de "
            + file.getAbsolutePath() + " para " + newFile.getAbsolutePath();
        ProjectService.getInstance().logWarningMessage(errMsg);
        newFile.delete();
        File configFile = renamedFileConfig(newFile);
        configFile.delete();
        return false;
      }
      /* Copia o arquivo de descrio */
      return copyDescriptionFile(realFile, newFile);
    }
    catch (Exception e) {
      String errMsg =
        "ServerProjectFile:copyFile: falha na cpia do link "
          + newFile.getAbsolutePath() + " para " + realFile.getAbsolutePath();
      ProjectService.getInstance().logSevereMessage(errMsg);
      return false;
    }
  }

  /**
   * Copia um arquivo regular. Faz a cpia do arquivo, do arquivo de
   * configurao e do arquivo do descrio.
   * 
   * @param newFile .
   * @return .
   */
  private boolean copyRegularFile(File newFile) {
    /* Copia o arquivo */
    if (!FileSystem.copyFile(file, newFile)) {
      String errMsg =
        "ServerProjectFile:copyFile: erro na cpia fsica de "
          + file.getAbsolutePath() + " para " + newFile.getAbsolutePath();
      ProjectService.getInstance().logWarningMessage(errMsg);
      return false;
    }
    /* Copia o arquivo de configurao */
    final File parentFile = newFile.getParentFile();
    final String newFileName = newFile.getName();
    File newConfigFile =
      new File(parentFile, createConfigFileName(newFileName));
    if (configFile.exists() && !FileSystem.copyFile(configFile, newConfigFile)) {
      String errMsg =
        "ServerProjectFile:copyFile: erro na cpia de "
          + configFile.getAbsolutePath() + " para "
          + newConfigFile.getAbsolutePath();
      ProjectService.getInstance().logWarningMessage(errMsg);
      newConfigFile.delete();
      newFile.delete();
      return false;
    }
    /* Copia o arquivo de descrio. */
    return copyDescriptionFile(file, newFile);
  }

  /**
   * Copia o arquivo de descrio do arquivo de origem para o destino. O arquivo
   * de descrio da origem ser copiado para o arquivo de descrio do destino.
   * 
   * @param from Arquivo de origem do qual ser obtido o arquivo de descrio.
   * @param to Arquivo de destino.
   * @return .
   */
  private boolean copyDescriptionFile(File from, File to) {
    File descrFile = renamedFileDescription(from);
    File newDescrFile = renamedFileDescription(to);
    if (descrFile.exists() && !FileSystem.copyFile(descrFile, newDescrFile)) {
      String errMsg =
        "ServerProjectFile:copyDescriptionFile: erro na cpia de "
          + descrFile.getAbsolutePath() + " para "
          + newDescrFile.getAbsolutePath();
      ProjectService.getInstance().logSevereMessage(errMsg);
      newDescrFile.delete();
      return false;
    }
    try {
      User user = ProjectService.getInstance().getUser();
      String userName = "";
      if (user != null) {
        userName = user.getName();
      }
      String message =
        "Em " + Utilities.getFormattedDate((new Date()).getTime())
          + "\nArquivo copiado por " + userName + ".\n";
      writeDescription(message, true, newDescrFile);
    }
    catch (Exception e) {
      String errMsg =
        "ServerProjectFile:copyDescriptionFile: erro ao escrever no arquivo "
          + "de configurao: " + newDescrFile.getAbsolutePath();
      ProjectService.getInstance().logSevereMessage(errMsg);
    }
    return true;
  }

  /**
   * Verifica se o arquivo sendo removido  um arquivo de configurao para um
   * link. Em caso positivo, decrementa o contador de referncias do arquivo
   * publicado antes de prosseguir com a remoo.
   * 
   * @param file arquivo sendo removido
   */
  private static void decCountIfLink(File file) {
    String name = file.getName();
    if (isConfigFileName(name)) {
      FileConfig config = new FileConfig();
      config.readConfigurationFile(file);
      if (config.isLink) {
        File realFile = config.realFile;
        File configRealFile = renamedFileConfig(realFile);
        ServerProjectFile.decCount(configRealFile, realFile);
      }
    }
  }

  /**
   * Tenta remover o diretrio, removendo recursivamente todos os arquivos e
   * diretrios dentro deste. Se houver links para arquivos publicados, os
   * contadores destes so decrementados antes da remoo do link.
   * 
   * @param file Diretrio a ser removido (o mtodo no verifica null).
   * @return true caso o diretrio tenha sido removido com sucesso ou no
   *         exista.
   */
  private static boolean pruneDirectory(File file) {
    if (!file.exists()) {
      return true;
    }
    if (file.isDirectory()) {
      File[] fileList = file.listFiles();
      if (fileList.length > 0) {
        for (int i = 0; i < fileList.length; i++) {
          if (fileList[i].isDirectory()) {
            pruneDirectory(fileList[i]);
          }
          else {
            decCountIfLink(fileList[i]);
            fileList[i].delete();
          }
        }
      }
      return file.delete();
    }
    return false;
  }

  /**
   * Remove o arquivo representado por este <code>ServerProjectFile</code>. Se
   * for um link, remove o link e decrementa o contador de referncias do
   * arquivo real. Se for um diretrio, esse mtodo  chamado recursivamente
   * para cada um de seus filhos. Se for um arquivo comum, apaga o arquivo. Em
   * todos os casos, o arquivo  removido fisicamente do sistema de arquivos e,
   * junto com ele, so removidos tambm os arquivos de configurao e de
   * descrio, se existirem. Aps a remoo, esse objeto  removido da lista de
   * filhos do seu pai.
   * 
   * @return flag indicando se o arquivo foi removido com sucesso
   */
  public boolean removeFile() {
    ProjectService srv = ProjectService.getInstance();
    // FIXME precisamos fazer isso se o arquivo for um link?
    /*
     * se file-locking est habilitado, s podemos seguir se obtivermos um lock
     * exclusivo sobre o arquivo
     */
    Object lockId = null;
    try {
      lockId = acquireLock();
    }
    catch (OperationFailureException ofe) {
      throw new ServiceFailureException(String.format(
        getString("ServerProjectFile.error.remove"), getName())
        + "\n" + ofe.getMessage());
    }

    try {
      /* Se for um link, remove o link e decrementa o contador de referncias */
      if (this.isPublishedFile()) {
        if (this.whichProjectPublished() == this.getProjectId()) {
          PublicationService publicationService =
            PublicationService.getInstance();
          if (publicationService == null) {
            String errMsg =
              "ServerProjectFile:remove: Erro obtendo PublicationService";
            srv.logWarningMessage(errMsg);
            return false;
          }
          publicationService.updatePublishAccess(this.projectId,
            this.getPath(), new Object[0], this.whoPublished());
        }
      }
      if (isLink && !decCount(realConfigFile, realFile)) {
        String errMsg =
          "ServerProjectFile:remove: " + getAbsolutePath()
            + " no pode ser removido devido a "
            + "falha ao decrementar o contador de referncias para "
            + realFile.getAbsolutePath();
        srv.logWarningMessage(errMsg);
        return false;
      }

      /* Remove fisicamente o arquivo correspondente a esse objeto */
      if (file.isFile()) {
        if (!file.delete()) {

          String errMsg =
            "ServerProjectFile:remove: " + getAbsolutePath()
              + " no pode ser removido devido a "
              + "falha ao apagar o arquivo " + file.getAbsolutePath();

          srv.logWarningMessage(errMsg);
          return false;
        }
      }

      else if (file.isDirectory()) {

        if (!pruneDirectory(file)) {

          String errMsg =
            "ServerProjectFile:remove: " + getAbsolutePath()
              + " no pode ser removido devido a "
              + "falha ao apagar o arquivo " + file.getAbsolutePath();

          srv.logWarningMessage(errMsg);
          return false;
        }
      }

      /* Remove o arquivo de configurao, se houver. */
      if (configFile != null) {
        configFile.delete();
      }

      /* Remove o arquivo de descrio, se houver */
      File descrFile = renamedFileDescription(file);
      if (descrFile.exists()) {
        descrFile.delete();
      }

      /* Retira o arquivo da cache */
      ServerProjectFile.deleteInCache(this);

      return true;
    }
    catch (Exception e) {
      return false;
    }
    finally {
      if (lockId != null) {
        FileLockManager.getInstance().releaseLock(this, lockId);
      }
    }
  }

  /**
   * Incrementa o contador de referncias a atualiza o arquivo de configurao.
   * 
   * @param configFile O arquivo de configurao a ser atualizado.
   * @return <code>true</code> se a operao foi bem-sucedida
   */
  private static boolean incCount(File configFile) {
    FileConfig config = new FileConfig();
    if (!config.readConfigurationFile(configFile)) {
      return false;
    }
    config.count++;
    if (!config.writeConfigurationFile(configFile)) {
      return false;
    }
    return true;
  }

  /**
   * Decrementa o contador de referncias a atualiza o arquivo de configurao.
   * Se o contador chegar a zero, remove fisicamente o arquivo.
   * 
   * @param configFile O arquivo de configurao a ser atualizado.
   * @param file O arquivo a ser removido no caso do contador chegar a zero.
   * @return <code>true</code> se a operao foi bem-sucedida
   */
  private static boolean decCount(File configFile, File file) {
    FileConfig config = new FileConfig();
    if (!config.readConfigurationFile(configFile)) {
      return false;
    }
    config.count--;
    if (config.count > 0) {
      if (!config.writeConfigurationFile(configFile)) {
        return false;
      }
      return true;
    }
    /*
     * Se o contador for zero, remove o arquivo real, assim como os de
     * configurao e descrio.
     */
    configFile.delete();
    File descrFile = renamedFileDescription(file);
    if (descrFile != null) {
      descrFile.delete();
    }
    file.delete();
    return true;
  }

  /**
   * Verifica se o arquivo foi publicado pelo projeto dado.
   * 
   * @param projId Identificador do projeto
   * @return .
   */
  public boolean wasPublishedBy(Object projId) {
    return (isPublished && projId.equals(publishedByProject));
  }

  /**
   * Testa a igualdade de dois arquivos.
   * 
   * @param o Objeto a ser comparado com o atual
   * @return true se ambos os objetos forem considerados iguais
   */
  @Override
  public boolean equals(Object o) {
    if (!(o instanceof ServerProjectFile)) {
      return false;
    }
    ServerProjectFile other = (ServerProjectFile) o;
    return (this.file.equals(other.file) && this.realFile
      .equals(other.realFile));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public int hashCode() {
    return file.hashCode();
  }

  /**
   * Constri um <code>ServerProjectFile</code>. <b>ATENO:</b> Este mtodo
   * <u>no</u> pode ser chamado diretamente (somente atravs do mtodo esttico
   * <code>newServerProjectFile</code>) pois  fundamental garantir a unicidade
   * destes objetos no servidor.
   * 
   * @param file .
   * @param realFile .
   * @param projectId .
   * @param parent .
   * @see ServerProjectFile#buildFromCache(File, File, Object,
   *      ServerProjectFile)
   */
  private ServerProjectFile(File file, File realFile, Object projectId,
    ServerProjectFile parent) {
    this.file = file;
    this.projectId = projectId;
    this.realFile = realFile;
    this.parent = parent;
    this.state = CLOSED;
    this.objInputStream = null;
    this.objOutputStream = null;

    /*
     * Caso nao consiga ler o arquivo de configuracao faz o log e continua
     * tentando ler o resto
     */
    try {
      readConfiguration();
    }
    catch (ServerProjectFileException e) {
      final ProjectService service = getService();
      service.logSevereMessage("ServerProjectFile:ServerProjectFile", e);
    }
    nameComparator = new Comparator<ServerProjectFile>() {

      @Override
      public int compare(ServerProjectFile o1, ServerProjectFile o2) {
        return o1.getName().compareTo(o2.getName());
      }
    };
  }

  /**
   * Muda a data de alterao deste arquivo
   * 
   * @param date Data em long
   */
  public synchronized void setModificationDate(long date) {
    realFile.setLastModified(date);
  }

  /**
   * Traduz uma determinada chave.
   * 
   * @param key chave a ser traduzida
   * @return chave traduzida
   */
  private String getString(String key) {
    ProjectService projectService = ProjectService.getInstance();
    return projectService.getString(key);
  }

  /**
   * Obtm as informaes sobre um arquivo atualizvel.
   * 
   * @return As informaes sobre um arquivo atualizvel.
   */
  public UpdatableFileInfo getUpdatableFileInfo() {
    return this.updatableFileInfo;
  }

  /**
   * Define as informaes sobre um arquivo atualizvel.
   * 
   * @param updatableFileInfo As informaes sobre um arquivo atualizvel.
   */
  public synchronized void setUpdatableFileInfo(
    UpdatableFileInfo updatableFileInfo) {
    if (updatableFileInfo == null) {
      throw new IllegalArgumentException(
        "As informaes sobre o arquivo no podem ter valor null.");
    }
    if (this.updatableFileInfo != null) {
      if (this.updatableFileInfo.equals(updatableFileInfo)) {
        return;
      }
    }
    this.updatableFileInfo = updatableFileInfo;
    this.writeConfiguration();
  }

  /**
   * Esta classe encapsula um listener remoto de lock de arquivo. O objetivo
   * dela  saber quando um lock de arquivo  obtido para notificar os projetos
   * abertos. No queremos que o FileLockManager acesse classe especficas do
   * servio de projetos para que ele possa ser migrado para outro pacote e seja
   * usado por outros servios.
   */
  class InternalFileLockListener implements FileLockListenerInterface {
    /**
     * Observador do lock cadastrado pela api do servio de projetos.
     */
    FileLockListenerInterface externalListener;

    /**
     * Constri um observador interno de lock. Encapsula o observador externo.
     * 
     * @param externalListener
     */
    InternalFileLockListener(FileLockListenerInterface externalListener) {
      this.externalListener = externalListener;
    }

    /**
     * {@inheritDoc} Este mtodo j chamado em uma <code>Thread</code> separada
     * pelo FileLockManager. Portanto, se a notificao ficar presa em uma
     * chamada rmi, no interrompe a execuo do servio.
     */
    @Override
    public void fileLockExpired() throws RemoteException {
      externalListener.fileLockExpired();
    }

    /**
     * {@inheritDoc} Este mtodo j chamado em uma <code>Thread</code> separada
     * pelo FileLockManager. Portanto, se a notificao ficar presa em uma
     * chamada rmi, no interrompe a execuo do servio.
     */
    @Override
    public void fileLocked(Object lockId) throws RemoteException {
      try {
        externalListener.fileLocked(lockId);
        ServerProjectFile.this.notifyLockStatusChanged();
      }
      catch (RemoteException e) {
        ProjectService.getInstance().logSevereMessage(
          "Falha na notificao de lock obtido. Pedido de lock: "
            + ServerProjectFile.this.getName(), e);
        releaseLock(lockId);
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean isRoot() {
    return parent == null;
  }
}
