/*
 * $Id: ProjectDataService.java 106492 2010-06-03 14:44:46Z mjulia $
 */
package csbase.server.services.projectservice.openbus;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.rmi.RemoteException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.omg.CORBA.Any;
import org.omg.CORBA.ORB;

import scs.core.IComponent;
import scs.core.servant.ComponentContext;
import scs.core.servant.ExtendedFacetDescription;
import tecgraf.ftc.common.exception.FailureException;
import tecgraf.ftc.common.exception.InvalidArraySize;
import tecgraf.ftc.common.exception.MaxClientsReachedException;
import tecgraf.ftc.common.exception.PermissionException;
import tecgraf.ftc.common.logic.RemoteFileChannel;
import tecgraf.ftc.common.logic.RemoteFileChannelImpl;
import tecgraf.ftc.server.FileChannelAccessInfo;
import tecgraf.ftc.server.FileServer;
import tecgraf.ftc.utils.Utils;
import tecgraf.openbus.Openbus;
import tecgraf.openbus.data_service.DataAccessDenied;
import tecgraf.openbus.data_service.DataDescription;
import tecgraf.openbus.data_service.DataKey;
import tecgraf.openbus.data_service.DataNotFound;
import tecgraf.openbus.data_service.DataServiceFinder;
import tecgraf.openbus.data_service.DataServiceUnavailableException;
import tecgraf.openbus.data_service.DataView;
import tecgraf.openbus.data_service.IHierarchicalDataServicePOA;
import tecgraf.openbus.data_service.IHierarchicalNavigationDataService;
import tecgraf.openbus.data_service.IHierarchicalNavigationDataServiceHelper;
import tecgraf.openbus.data_service.InvalidDataKey;
import tecgraf.openbus.data_service.Metadata;
import tecgraf.openbus.data_service.ServiceFailure;
import tecgraf.openbus.data_service.UnknownViewInterface;
import tecgraf.openbus.data_service.UnknownViews;
import tecgraf.openbus.data_service.UnstructuredData;
import tecgraf.openbus.data_service.UnstructuredDataHelper;
import tecgraf.openbus.data_service.UnstructuredDataImpl;
import tecgraf.openbus.data_service.project.ProjectItemDescriptionImpl;
import tecgraf.openbus.project.PATH_SEPARATOR;
import tecgraf.openbus.project.ProjectItemDescription;
import csbase.exception.ServiceFailureException;
import csbase.exception.project.FileLockedException;
import csbase.logic.ClientProjectFile;
import csbase.logic.CommonClientProject;
import csbase.logic.ProjectFile;
import csbase.logic.ProjectFileChannelLoadListener;
import csbase.logic.UserProjectInfo;
import csbase.server.Service;
import csbase.server.services.openbusservice.OpenBusService;
import csbase.server.services.projectservice.ProjectService;
import csbase.server.services.projectservice.UpdatableFileInfo;

/**
 * Representa um Servio de Dados de Projeto.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class ProjectDataService extends IHierarchicalDataServicePOA {
  /**
   * Prefixo utilizado nos nomes das chaves de traduo.
   */
  private static final String LNG_PREFIX = "ProjectDataService.";
  /**
   * Array vazio de descritores de dados. Utilizado principalmente para retorno.
   */
  private static final DataDescription[] EMPTY_DATA_DESCRIPTION_ARRAY =
    new DataDescription[0];
  /**
   * O nome do metadado que representa o caminho absoluto de um dado.
   */
  private static final String ABSOLUTE_PATH_METADATUM_NAME = "ABSOLUTE_PATH";
  /**
   * O contexto do componente ao qual esta faceta pertence.
   */
  private ComponentContext context;
  /**
   * O servidor de arquivos utilizado para prover os dados com viso
   * no-estruturada.
   */
  private FileServer fileServer;
  /**
   * Representa o nome da faceta deste servio.
   */
  private String facetName;
  /**
   * Representa o IOR da faceta deste servio.
   */
  private String facetIOR;

  /**
   * Cria um Servico de Dados de Projeto.
   * 
   * @param context Contexto do componente dono da faceta.
   */
  public ProjectDataService(ComponentContext context) {
    this.context = context;
  }

  /**
   * Define um servidor de arquivos para transferncia de dados com viso
   * no-estruturada.
   * 
   * @param fileServer O servidor de arquivos.
   */
  public void setFileServer(FileServer fileServer) {
    this.fileServer = fileServer;
  }

  /**
   * Define as informaes sobre a faceta deste servio.
   * 
   * @param facetName O nome da faceta.
   * @param facetIOR O IOR da faceta.
   */
  public void setFacetInfo(String facetName, String facetIOR) {
    this.facetName = facetName;
    this.facetIOR = facetIOR;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public DataDescription[] getRoots() throws ServiceFailure {
    try {
      String userId = OpenBusService.getInstance().getRequesterUserId();
      if (userId == null) {
        userId = Service.getUser().getLogin();
      }
      else {
        Service.setUserId(userId);
      }

      Set<UserProjectInfo> projects;
      try {
        projects = ProjectService.getInstance().getAllProjectsInfo(userId);
      }
      catch (ServiceFailureException e) {
        ProjectService prjSrv = ProjectService.getInstance();
        prjSrv.logSevereMessage(
          "Ocorreu um erro ao obter os nomes dos projetos do usurio {0}",
          new Object[] { userId }, e);
        throw new ServiceFailure();
      }
      if (projects == null) {
        return EMPTY_DATA_DESCRIPTION_ARRAY;
      }

      List<DataDescription> roots =
        new ArrayList<DataDescription>(projects.size());
      for (UserProjectInfo projectInfo : projects) {
        String projectName = projectInfo.getProjectName();
        try {
          CommonClientProject project =
            ProjectService.getInstance().openProject(projectInfo.getUserId(),
              projectName, false);
          ClientProjectFile rootFile = (ClientProjectFile) project.getTree();
          try {
            roots.add(this.createDataDescription((String) project.getUserId(),
              rootFile));
          }
          catch (InvalidDataKey e) {
            ProjectService.getInstance().logSevereMessage(
              "Erro ao criar descrio para o arquivo {0}.",
              new Object[] { rootFile.getAbsolutePath() }, e);
            throw new ServiceFailure();
          }
        }
        catch (ServiceFailureException e) {
          ProjectService.getInstance().logSevereMessage(
            "Ocorreu um erro ao abrir o projeto {0} do usurio {1}.",
            new Object[] { projectName, userId }, e);
          throw new ServiceFailure();
        }
      }
      return roots.toArray(EMPTY_DATA_DESCRIPTION_ARRAY);
    }
    catch (RuntimeException e) {
      ProjectService.getInstance().logSevereMessage(e.getMessage(), e);
      throw new ServiceFailure();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public DataDescription[] getChildren(byte[] fKey) throws ServiceFailure,
    InvalidDataKey {
    try {
      String userId = OpenBusService.getInstance().getRequesterUserId();
      if (userId != null) {
        Service.setUserId(userId);
      }

      ProjectDataInfo parentDataInfo = this.getProjectDataInfo(fKey);
      ClientProjectFile parent = parentDataInfo.getProjectFile();
      if (!parent.isDirectory()) {
        throw new ServiceFailure();
      }

      ProjectFile[] children = null;
      try {
        children = parent.getChildren();
      }
      catch (RemoteException e) {
        throw new ServiceFailure();
      }

      if (children == null) {
        return EMPTY_DATA_DESCRIPTION_ARRAY;
      }

      List<DataDescription> nodes =
        new ArrayList<DataDescription>(children.length);
      for (int i = 0; i < children.length; i++) {
        nodes.add(this.createDataDescription(
          parentDataInfo.getProjectOwnerId(), (ClientProjectFile) children[i]));
      }
      return nodes.toArray(EMPTY_DATA_DESCRIPTION_ARRAY);
    }
    catch (RuntimeException e) {
      ProjectService.getInstance().logSevereMessage(e.getMessage(), e);
      throw new ServiceFailure();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public DataDescription getParent(byte[] fKey) throws ServiceFailure,
    InvalidDataKey {
    try {
      String userId = OpenBusService.getInstance().getRequesterUserId();
      if (userId != null) {
        Service.setUserId(userId);
      }

      ProjectDataInfo dataInfo = this.getProjectDataInfo(fKey);
      ClientProjectFile file = dataInfo.getProjectFile();
      if (file == null) {
        throw new InvalidDataKey(fKey);
      }
      ClientProjectFile parent = file.getParent();
      if (parent == null) {
        return null;
      }
      return this.createDataDescription(dataInfo.getProjectOwnerId(), parent);
    }
    catch (RuntimeException e) {
      ProjectService.getInstance().logSevereMessage(e.getMessage(), e);
      throw new ServiceFailure();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public DataDescription getDataDescription(byte[] fKey) throws ServiceFailure,
    InvalidDataKey {
    try {
      String userId = OpenBusService.getInstance().getRequesterUserId();
      if (userId != null) {
        Service.setUserId(userId);
      }

      ProjectDataInfo dataInfo = this.getProjectDataInfo(fKey);
      ClientProjectFile file = dataInfo.getProjectFile();
      if (file == null) {
        throw new InvalidDataKey(fKey);
      }
      return this.createDataDescription(dataInfo.getProjectOwnerId(), file);
    }
    catch (RuntimeException e) {
      ProjectService.getInstance().logSevereMessage(e.getMessage(), e);
      throw new ServiceFailure();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public DataView getDataView(byte[] fKey, String fViewInterface)
    throws ServiceFailure, UnknownViewInterface, InvalidDataKey {
    try {
      String userId = OpenBusService.getInstance().getRequesterUserId();
      if (userId == null) {
        userId = Service.getUser().getLogin();
      }
      else {
        Service.setUserId(userId);
      }

      ProjectDataInfo dataInfo = this.getProjectDataInfo(fKey);
      if (dataInfo == null) {
        throw new InvalidDataKey(fKey);
      }

      ClientProjectFile file = dataInfo.getProjectFile();
      return this.createDataView(userId, dataInfo.getProjectOwnerId(), file,
        fViewInterface);
    }
    catch (RuntimeException e) {
      ProjectService.getInstance().logSevereMessage(e.getMessage(), e);
      throw new ServiceFailure();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public DataView[] getDataViewSeq(byte[][] fKeys, String fViewInterface)
    throws ServiceFailure, UnknownViewInterface, InvalidDataKey {
    try {
      String userId = OpenBusService.getInstance().getRequesterUserId();
      if (userId != null) {
        Service.setUserId(userId);
      }

      DataView[] viewSeq = new DataView[fKeys.length];
      for (int i = 0; i < fKeys.length; i++) {
        viewSeq[i] = this.getDataView(fKeys[i], fViewInterface);
      }
      return viewSeq;
    }
    catch (RuntimeException e) {
      ProjectService.getInstance().logSevereMessage(e.getMessage(), e);
      throw new ServiceFailure();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public byte[] createData(byte[] fParentKey, DataDescription fPrototype)
    throws ServiceFailure, InvalidDataKey {
    try {
      String userId = OpenBusService.getInstance().getRequesterUserId();
      if (userId != null) {
        Service.setUserId(userId);
      }

      ProjectDataInfo parentInfo = this.getProjectDataInfo(fParentKey);
      if (parentInfo == null) {
        throw new InvalidDataKey(fParentKey);
      }

      ClientProjectFile parent = parentInfo.getProjectFile();
      if (!parent.isDirectory()) {
        throw new ServiceFailure();
      }

      ProjectItemDescription prototype = (ProjectItemDescription) fPrototype;
      try {
        if (prototype.fIsContainer) {
          parent.createDirectory(prototype.fName);
        }
        else {
          parent.createFile(prototype.fName, prototype.fType);
        }
      }
      catch (RemoteException e) {
        throw new ServiceFailure();
      }
      catch (ServiceFailureException e) {
        UserProjectInfo parentProjectInfo = getUserProjectInfo(fParentKey);
        ProjectService
          .getInstance()
          .logSevereMessage(
            "Ocorreu um erro ao criar o dado {0} no projeto {1} pertencente ao usurio {2}.",
            new Object[] { prototype.fName, parentProjectInfo.getProjectName(),
                userId }, e);
        throw new ServiceFailure();
      }

      String dataId =
        generateDataId(parentInfo.getProjectOwnerId(), parent, prototype.fName);
      DataKey key = this.generateDataKey(dataId);
      return key.getKey();
    }
    catch (RuntimeException e) {
      ProjectService.getInstance().logSevereMessage(e.getMessage(), e);
      throw new ServiceFailure();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public byte[] copyData(byte[] fParentKey, byte[] fSourceKey)
    throws ServiceFailure, UnknownViews, InvalidDataKey {
    try {
      String userId = OpenBusService.getInstance().getRequesterUserId();
      if (userId != null) {
        Service.setUserId(userId);
      }

      ProjectDataInfo parentDataInfo = this.getProjectDataInfo(fParentKey);
      ClientProjectFile parent = parentDataInfo.getProjectFile();

      ProjectDataInfo sourceDataInfo = this.getProjectDataInfo(fSourceKey);
      ClientProjectFile source = sourceDataInfo.getProjectFile();

      try {
        source.copy(parent);
      }
      catch (RemoteException e) {
        throw new ServiceFailure();
      }

      String dataId =
        generateDataId(parentDataInfo.getProjectOwnerId(), parent, source
          .getName());
      DataKey dataKey = this.generateDataKey(dataId);
      return dataKey.getKey();
    }
    catch (RuntimeException e) {
      ProjectService.getInstance().logSevereMessage(e.getMessage(), e);
      throw new ServiceFailure();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void moveData(byte[] fKey, byte[] fNewParentKey)
    throws ServiceFailure, UnknownViews, InvalidDataKey {
    try {
      String userId = OpenBusService.getInstance().getRequesterUserId();
      if (userId != null) {
        Service.setUserId(userId);
      }

      ProjectDataInfo dataInfo = this.getProjectDataInfo(fKey);
      ClientProjectFile file = dataInfo.getProjectFile();

      ProjectDataInfo newParentDataInfo =
        this.getProjectDataInfo(fNewParentKey);
      ClientProjectFile newParent = newParentDataInfo.getProjectFile();

      try {
        file.move(newParent);
      }
      catch (RemoteException e) {
        throw new ServiceFailure();
      }
    }
    catch (RuntimeException e) {
      ProjectService.getInstance().logSevereMessage(e.getMessage(), e);
      throw new ServiceFailure();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void updateData(byte[] fKey, byte[] fSourceKey) throws ServiceFailure,
    UnknownViews, InvalidDataKey {
    try {
      String userId = OpenBusService.getInstance().getRequesterUserId();
      if (userId != null) {
        Service.setUserId(userId);
      }

      ProjectDataInfo dataInfo = this.getProjectDataInfo(fKey);
      ClientProjectFile file = dataInfo.getProjectFile();

      ProjectDataInfo sourceDataInfo = this.getProjectDataInfo(fSourceKey);
      ClientProjectFile sourceFile = sourceDataInfo.getProjectFile();

      try {
        file.download(new File(sourceFile.getAbsolutePath()), 1024,
          new ProjectFileChannelLoadListener() {
            @Override
            public void transferedBytes(long currentSize, long totalSize,
              long numBytes) {
            }
          });
      }
      catch (FileLockedException e) {
        ProjectService
          .getInstance()
          .logSevereMessage(
            "No foi possvel atualizar o arquivo {0}, pois o mesmo est lockado.",
            new Object[] { file.getAbsolutePath() }, e);
        throw new ServiceFailure();
      }
      catch (IOException e) {
        ProjectService.getInstance().logSevereMessage(
          "Erro ao atualizar o arquivo {0}.",
          new Object[] { file.getAbsolutePath() }, e);
        throw new ServiceFailure();
      }
    }
    catch (RuntimeException e) {
      ProjectService.getInstance().logSevereMessage(e.getMessage(), e);
      throw new ServiceFailure();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void deleteData(byte[] fKey) throws ServiceFailure, InvalidDataKey {
    try {
      String userId = OpenBusService.getInstance().getRequesterUserId();
      if (userId == null) {
        userId = Service.getUser().getLogin();
      }
      else {
        Service.setUserId(userId);
      }

      ProjectDataInfo dataInfo = this.getProjectDataInfo(fKey);
      if (dataInfo == null) {
        throw new InvalidDataKey(fKey);
      }

      UserProjectInfo projectInfo = getUserProjectInfo(fKey);
      ClientProjectFile file = dataInfo.getProjectFile();
      if (file.getParent() == null) {
        ProjectService.getInstance().removeProject(userId,
          projectInfo.getProjectName());
      }
      else {
        try {
          file.remove();
        }
        catch (RemoteException e) {
          ProjectService
            .getInstance()
            .logSevereMessage(
              "Ocorreu um erro ao remover o arquivo {0} no projeto {1} pertencente ao usurio {2}.",
              new Object[] { file.getName(), projectInfo.getProjectName(),
                  userId }, e);
          throw new ServiceFailure();
        }
      }
    }
    catch (RuntimeException e) {
      ProjectService.getInstance().logSevereMessage(e.getMessage(), e);
      throw new ServiceFailure();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public byte[] copyDataFrom(byte[] parentKey, byte[] sourceKey)
    throws ServiceFailure, InvalidDataKey, UnknownViews, DataAccessDenied,
    DataNotFound {
    try {
      String userId = OpenBusService.getInstance().getRequesterUserId();
      if (userId != null) {
        Service.setUserId(userId);
      }

      Openbus bus = Openbus.getInstance();
      bus.setThreadCredential(bus.getInterceptedCredential());
      try {
        ProjectItemDescription parentDescription =
          (ProjectItemDescription) this.getDataDescription(parentKey);
        String parentAbsolutePath = null;
        for (int i = 0; i < parentDescription.fMetadata.length; i++) {
          if (parentDescription.fMetadata[i].fName
            .equals(ABSOLUTE_PATH_METADATUM_NAME)) {
            Any any = parentDescription.fMetadata[i].fValue;
            parentAbsolutePath = any.extract_string();
          }
        }

        DataKey sourceDataKey = new DataKey(sourceKey);
        IComponent dataServiceComponent;
        try {
          dataServiceComponent = DataServiceFinder.find(sourceDataKey);
        }
        catch (DataServiceUnavailableException e) {
          ProjectService.getInstance().logSevereMessage(
            "Servio de Dados de origem fora do ar.");
          throw new ServiceFailure();
        }
        org.omg.CORBA.Object obj =
          dataServiceComponent
            .getFacet(IHierarchicalNavigationDataServiceHelper.id());
        IHierarchicalNavigationDataService dataService =
          IHierarchicalNavigationDataServiceHelper.narrow(obj);
        ProjectItemDescription sourceDescription =
          (ProjectItemDescription) dataService.getDataDescription(sourceKey);

        String path =
          parentDescription.fPath + PATH_SEPARATOR.value
            + sourceDescription.fName;
        String absolutePath =
          parentAbsolutePath + PATH_SEPARATOR.value + sourceDescription.fName;
        List<Metadata> metadata = createMetadataList(absolutePath);
        long date = System.currentTimeMillis();

        ProjectItemDescription prototype =
          new ProjectItemDescriptionImpl(sourceDescription.fName,
            createViewSet(sourceDescription.fIsContainer), metadata, userId,
            sourceDescription.fDescription, path, sourceDescription.fType, 0,
            sourceDescription.fIsContainer, sourceDescription.fCanRead,
            sourceDescription.fCanWrite, date, date);

        byte[] key = this.createData(parentKey, prototype);

        this.updateDataFrom(key, sourceKey);

        return key;
      }
      finally {
        bus.setThreadCredential(null);
      }
    }
    catch (RuntimeException e) {
      ProjectService.getInstance().logSevereMessage(e.getMessage(), e);
      throw new ServiceFailure();
    }
  }

  /**
   * Cria um conjunto com as vises suportadas pelos dados oferecidos por este
   * servio.
   * 
   * @param container Indica se o dado  um continer.
   * 
   * @return O conjunto de vises.
   */
  private static Set<String> createViewSet(boolean container) {
    Set<String> views = new HashSet<String>();
    if (!container) {
      views.add(UnstructuredDataHelper.id());
    }
    return views;
  }

  /**
   * Cria uma lista de metadados para os dados oferecidos por este servio.
   * 
   * @param absolutePath O caminho absoluto do dado.
   * 
   * @return A lista de metadados.
   */
  private static List<Metadata> createMetadataList(String absolutePath) {
    List<Metadata> metadataList = new ArrayList<Metadata>();

    Openbus bus = Openbus.getInstance();
    ORB orb = bus.getORB();
    Any any;

    if (absolutePath != null) {
      any = orb.create_any();
      any.insert_string(absolutePath);
      metadataList.add(new Metadata(ABSOLUTE_PATH_METADATUM_NAME, any));
    }

    return metadataList;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void updateDataFrom(byte[] key, byte[] sourceKey)
    throws ServiceFailure, InvalidDataKey, UnknownViews, DataNotFound,
    DataAccessDenied {
    try {
      String userId = OpenBusService.getInstance().getRequesterUserId();
      if (userId != null) {
        Service.setUserId(userId);
      }

      DataKey sourceDataKey = new DataKey(sourceKey);
      IComponent dataServiceComponent;
      try {
        dataServiceComponent = DataServiceFinder.find(sourceDataKey);
      }
      catch (DataServiceUnavailableException e) {
        ProjectService.getInstance().logSevereMessage(
          "Servio de Dados de origem fora do ar.");
        throw new ServiceFailure();
      }
      org.omg.CORBA.Object obj =
        dataServiceComponent.getFacet(IHierarchicalNavigationDataServiceHelper
          .id());
      IHierarchicalNavigationDataService dataService =
        IHierarchicalNavigationDataServiceHelper.narrow(obj);
      ProjectItemDescription sourceDescription =
        (ProjectItemDescription) dataService.getDataDescription(sourceKey);

      UnstructuredData sourceView;
      try {
        sourceView =
          (UnstructuredData) dataService.getDataView(sourceKey,
            UnstructuredDataHelper.id());
      }
      catch (UnknownViewInterface e1) {
        throw new UnknownViews();
      }

      UnstructuredData view;
      try {
        view =
          (UnstructuredData) this.getDataView(key, UnstructuredDataHelper.id());
      }
      catch (UnknownViewInterface e) {
        throw new UnknownViews();
      }

      DataKey dataKey = new DataKey(view.fKey);
      RemoteFileChannel channel;
      try {
        channel =
          new RemoteFileChannelImpl(dataKey.getDataId().getBytes(
            Utils.CHARSET_ENCODING), view.fWritable, view.fHost, view.fPort,
            view.fAccessKey);
      }
      catch (UnsupportedEncodingException e) {
        ProjectService
          .getInstance()
          .logSevereMessage(
            "Erro ao converter o identificador do dado de destino ({0}) para bytes.",
            new Object[] { dataKey.getDataId() }, e);
        throw new ServiceFailure();
      }

      RemoteFileChannel sourceChannel;
      try {
        sourceChannel =
          new RemoteFileChannelImpl(sourceDataKey.getDataId().getBytes(
            Utils.CHARSET_ENCODING), sourceView.fWritable, sourceView.fHost,
            sourceView.fPort, sourceView.fAccessKey);
      }
      catch (UnsupportedEncodingException e) {
        ProjectService
          .getInstance()
          .logSevereMessage(
            "Erro ao converter o identificador do dado de origem ({0}) para bytes.",
            new Object[] { sourceDataKey.getDataId() }, e);
        throw new ServiceFailure();
      }

      try {
        channel.open(false);
        sourceChannel.open(true);
      }
      catch (PermissionException e) {
        ProjectService.getInstance().logSevereMessage(e.getMessage(), e);
        throw new DataAccessDenied();
      }
      catch (FileNotFoundException e) {
        ProjectService.getInstance().logSevereMessage(e.getMessage(), e);
        throw new ServiceFailure();
      }
      catch (FailureException e) {
        ProjectService.getInstance().logSevereMessage(e.getMessage(), e);
        throw new ServiceFailure();
      }
      catch (MaxClientsReachedException e) {
        ProjectService.getInstance().logSevereMessage(e.getMessage(), e);
        throw new ServiceFailure();
      }

      try {
        channel.transferFrom(sourceChannel, 0, sourceDescription.fSize);
      }
      catch (PermissionException e) {
        ProjectService.getInstance().logSevereMessage(
          "Sem permisso para escrever no arquivo.", e);
        throw new ServiceFailure();
      }
      catch (FailureException e) {
        ProjectService.getInstance().logSevereMessage(
          "Falha na transferncia do arquivo.", e);
        throw new ServiceFailure();
      }
      catch (tecgraf.ftc.common.exception.FileLockedException e) {
        ProjectService.getInstance().logSevereMessage(
          "Tentativa de escrita em arquivo.", e);
        throw new ServiceFailure();
      }
      finally {
        try {
          channel.close();
        }
        catch (FailureException e) {
          e.printStackTrace();
        }
        finally {
          try {
            sourceChannel.close();
          }
          catch (FailureException e) {
          }
        }
      }

      ProjectDataInfo dataInfo = this.getProjectDataInfo(key);
      ClientProjectFile file = dataInfo.getProjectFile();
      if (!ProjectService.getInstance().setUpdatableFileInfo(file,
        new UpdatableFileInfo(OpenBusProjectFileUpdater.class, sourceKey))) {
        ProjectService.getInstance().logWarningMessage(
          "A informao sobre atualizao do arquivo {0} no foi definida",
          new Object[] { file.getAbsolutePath() });
      }
    }
    catch (RuntimeException e) {
      ProjectService.getInstance().logSevereMessage(e.getMessage(), e);
      throw new ServiceFailure();
    }
  }

  /**
   * Obtm informaes sobre o arquivo de projeto representado por uma chave.
   * 
   * @param key A chave.
   * 
   * @return As informaes sobre o arquivo de projeto.
   * 
   * @throws InvalidDataKey Caso a chave no represente um arquivo.
   * @throws ServiceFailure Caso ocorra uma falha no servio.
   */
  private ProjectDataInfo getProjectDataInfo(byte[] key) throws InvalidDataKey,
    ServiceFailure {
    DataKey dataKey = new DataKey(key);
    String dataId = dataKey.getDataId();

    String[] splittedKey = dataId.split(ProjectService.FILE_ID_SEPARATOR);
    if (splittedKey.length < ProjectService.MINIMUM_FILE_ID_SIZE) {
      throw new InvalidDataKey(key);
    }

    Object projectUserId = splittedKey[0];
    String projectName = splittedKey[1];
    CommonClientProject project;
    try {
      project =
        ProjectService.getInstance().openProject(projectUserId, projectName,
          false);
    }
    catch (ServiceFailureException e) {
      ProjectService.getInstance().logSevereMessage(
        MessageFormat.format(ProjectService.getInstance().getString(
          LNG_PREFIX + "error_openning_project"), projectName, projectUserId),
        e);
      throw new ServiceFailure();
    }

    if (splittedKey.length == ProjectService.MINIMUM_FILE_ID_SIZE) {
      ClientProjectFile file = (ClientProjectFile) project.getTree();
      if (file == null) {
        throw new InvalidDataKey(key);
      }
      return new ProjectDataInfo(projectUserId, file);
    }
    else {
      String[] filePath =
        new String[splittedKey.length - ProjectService.MINIMUM_FILE_ID_SIZE];
      for (int i = 0; i < filePath.length; i++) {
        filePath[i] = splittedKey[i + ProjectService.MINIMUM_FILE_ID_SIZE];
      }

      try {
        ClientProjectFile file = project.getFile(filePath);
        if (file == null) {
          throw new InvalidDataKey(key);
        }
        return new ProjectDataInfo(projectUserId, file);
      }
      catch (RemoteException e) {
        throw new ServiceFailure();
      }
    }
  }

  /**
   * Gera um identificador para um arquivo de projeto.
   * 
   * @param file O arquivo de projeto.
   * 
   * @return O identificador.
   */
  private static String generateDataId(Object ownerId, ClientProjectFile file) {
    return generateDataId(ownerId, file.getPath());
  }

  /**
   * Gera um identificador para um arquivo de projeto.
   * 
   * @param pathArray O caminho para o arquivo de projeto.
   * 
   * @return O identificador.
   */
  public static String generateDataId(Object ownerId, String[] pathArray) {
    String dataId = (String) ownerId;
    for (String path : pathArray) {
      dataId += ProjectService.FILE_ID_SEPARATOR + path;
    }
    return dataId;
  }

  /**
   * Gera um identificador para um arquivo de projeto a partir do seu diretrio
   * de origem.
   * 
   * @param parent O diretrio de origem do arquivo.
   * @param childName O nome do arquivo.
   * 
   * @return O identificador.
   */
  private static String generateDataId(Object userId, ClientProjectFile parent,
    String childName) {
    String dataId = generateDataId(userId, parent);
    dataId += ProjectService.FILE_ID_SEPARATOR + childName;
    return dataId;
  }

  /**
   * Gera um identificador unvoco a partir de um identificador de um arquivo de
   * projeto.
   * 
   * @param dataId O identificador do arquivo de projeto.
   * 
   * @return O identificador unvoco.
   * 
   * @throws InvalidDataKey Caso ocorra algum erro ao criar o identificador
   *         unvoco.
   */
  public DataKey generateDataKey(String dataId) throws InvalidDataKey {
    String facetInterfaceName = null;
    if (this.facetName != null) {
      Map<String, ExtendedFacetDescription> facets =
        this.context.getExtendedFacetDescs();
      ExtendedFacetDescription facet = facets.get(this.facetName);
      facetInterfaceName = facet.interface_name;
    }
    return new DataKey(dataId, facetInterfaceName, this.context
      .getComponentId(), this.facetName, this.facetIOR);
  }

  /**
   * Obtm as informaes sobre o usurio e o projeto ao qual um dado pertence.
   * 
   * @param key A chave do dado.
   * 
   * @return As informaes do projeto.
   * 
   * @throws InvalidDataKey Caso a chave recebida seja invlida.
   */
  static final UserProjectInfo getUserProjectInfo(byte[] key)
    throws InvalidDataKey {
    DataKey dataKey = new DataKey(key);
    String dataId = dataKey.getDataId();
    String[] splittedKey = dataId.split(ProjectService.FILE_ID_SEPARATOR);
    if (splittedKey.length < ProjectService.MINIMUM_FILE_ID_SIZE) {
      throw new InvalidDataKey(key);
    }
    return new UserProjectInfo(splittedKey[0], splittedKey[1]);
  }

  /**
   * Cria uma descrio de dado para um arquivo de projeto.
   * 
   * @param ownerId O identificador do dono do projeto.
   * @param file O arquivo de projeto.
   * 
   * @return A descrio do dado.
   * 
   * @throws InvalidDataKey Caso no seja possvel criar a chave unvoca do
   *         dado.
   */
  final ProjectItemDescriptionImpl createDataDescription(Object ownerId,
    ClientProjectFile file) throws InvalidDataKey {
    String dataId = generateDataId(ownerId, file);
    DataKey dataKey = this.generateDataKey(dataId);

    String description = null;
    try {
      description = file.getDescription();
    }
    catch (RemoteException e) {
      ProjectService.getInstance().logWarningMessage(
        "Erro ao obter a descrio do arquivo {0}.",
        new Object[] { file.getAbsolutePath() });
    }
    catch (ServiceFailureException e) {
      ProjectService.getInstance().logWarningMessage(
        MessageFormat.format("Erro ao obter a descrio do arquivo {0}.",
          new Object[] { file.getAbsolutePath() }));
    }

    String[] pathArray = file.getPath();
    String path = PATH_SEPARATOR.value + pathArray[0];
    for (int i = 1; i < pathArray.length; i++) {
      path = path + PATH_SEPARATOR.value + pathArray[i];
    }

    return new ProjectItemDescriptionImpl(dataKey.getKey(), file.getName(),
      createViewSet(file.isDirectory()), createMetadataList(file
        .getAbsolutePath()), (String) file.whoCreated(), description, path,
      file.getType(), file.size(), file.isDirectory(), true, file
        .isRegularFile(), file.getCreationDate(), file.getModificationDate());
  }

  /**
   * Cria uma viso para um arquivo de projeto.
   * 
   * @param userId
   * @param ownerId O identificador do usurio dono do arquivo.
   * @param file O arquivo de projeto.
   * @param viewInterface O nome da interface da viso.
   * 
   * @return A viso.
   * 
   * @throws UnknownViewInterface Caso a viso solicitada no seja oferecida
   *         pelo dado.
   * @throws InvalidDataKey Caso ocorra erro ao criar a chave unvoca do arquivo
   *         de projeto.
   * @throws ServiceFailure
   */
  private DataView createDataView(String userId, Object ownerId,
    ClientProjectFile file, String viewInterface) throws UnknownViewInterface,
    InvalidDataKey, ServiceFailure {
    if (viewInterface.equals(UnstructuredDataHelper.id())) {
      return this.createUnstructuredData(userId, ownerId, file);
    }
    throw new UnknownViewInterface();
  }

  /**
   * Cria uma viso no-estruturada para um arquivo de projeto.
   * 
   * @param requesterId O identificador do usurio que est solicitando a viso
   *        no-estruturada
   * @param ownerId O identificador do usurio dono do projeto.
   * @param file O arquivo de projeto.
   * 
   * @return A viso no-estruturada.
   * 
   * @throws UnknownViewInterface Caso a viso solicitada no seja oferecida
   *         pelo dado.
   * @throws InvalidDataKey Caso ocorra erro ao criar a chave unvoca do arquivo
   *         de projeto.
   * @throws ServiceFailure
   */
  private UnstructuredData createUnstructuredData(String requesterId,
    Object ownerId, ClientProjectFile file) throws UnknownViewInterface,
    InvalidDataKey, ServiceFailure {
    if (file.isDirectory()) {
      throw new UnknownViewInterface();
    }

    String dataIdString = generateDataId(ownerId, file);
    DataKey dataKey = this.generateDataKey(dataIdString);

    byte[] dataId;
    try {
      dataId = dataIdString.getBytes(Utils.CHARSET_ENCODING);
    }
    catch (UnsupportedEncodingException e) {
      ProjectService.getInstance().logSevereMessage(
        "Erro ao converter o identificador do dado em string.", e);
      throw new ServiceFailure();
    }
    FileChannelAccessInfo accessInfo;
    try {
      accessInfo = this.fileServer.createFileChannelInfo(requesterId, dataId);
      return new UnstructuredDataImpl(dataKey.getKey(), accessInfo.getHost(),
        accessInfo.getPort(), accessInfo.getAccessKey(), file.isRegularFile());
    }
    catch (InvalidArraySize e) {
      ProjectService.getInstance().logSevereMessage(
        "Erro ao criar uma descrio do canal do arquivo.", e);
      throw new ServiceFailure();
    }

  }
}
