/**
 * $Id: Command.java 106342 2010-06-01 15:54:24Z ururahy $
 */
package csbase.server.services.sgaservice;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.Serializable;
import java.rmi.RemoteException;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import sgaidl.CompletedCommandInfo;
import sgaidl.ExecutionInfo;
import sgaidl.ProcessInfo;
import sgaidl.ProcessState;
import sgaidl.RunningCommandInfo;
import sgaidl.SGACommand;
import sgaidl.SGAServer;
import tecgraf.javautils.FileUtils;
import csbase.exception.OperationFailureException;
import csbase.exception.ServiceFailureException;
import csbase.logic.CommandFinalizationType;
import csbase.logic.CommandInfo;
import csbase.logic.CommandStatus;
import csbase.logic.SGAFile;
import csbase.logic.SGAInfo;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.CommandLineContext;
import csbase.logic.algorithms.FileParameterValue;
import csbase.server.ServerException;
import csbase.server.services.commandpersistenceservice.CommandPersistenceService;
import csbase.server.services.projectservice.ServerProject;
import csbase.server.services.projectservice.ServerProjectFile;

/**
 * A classe <code>Command</code> representa um comando que est sendo executado
 * no gerenciador de uma mquina hospedeira (ou servidor SGA).
 */
public class Command implements Serializable {
  /**
   * Arquivos gerados pelo comando. Estes arquivos so previamente criados pelo
   * SSI e sinalizados como "em construo". Essa sinalizao  removida no
   * trmino do comando.
   */
  private SGAFile[] commandOutputFiles = null;
  /** Informaes oriundas do SGA a respeito do comand (this) em execuo. */
  private RunningCommandInfo info = null;
  /** Informaes para a execuo do comando. */
  private CommandInfo cmdInfo;
  /** Indica se as informaes do comando esto atualizadas. */
  private boolean newInfo = false;
  /**
   * Indica a data e hora em que as informaes do comando foram atualizadas
   * pela ltima vez.
   */
  private long lastInfoUpdate = 0;

  /** Servio de gerenciamento de SGAs. */
  private transient SGAService service = null;
  /** Projeto aonde est sendo executando o comando. */
  private transient ServerProject project = null;
  /** Caminho do diretrio do projeto deste comando. */
  private String[] projectPath = null;
  /** Caminho do diretrio raz do framework. */
  private String[] rootPath = null;

  /** Indica a data e hora que o comando foi criado */
  private long startTimeMillis = 0;
  /** SGA que est executando o comando. */
  private transient SGA sga = null;
  /** Referncia remota para o comando em execuo. */
  private transient SGACommand remoteReference;
  /** Texto de comando executado no SGA. */
  private String command = null;
  /**
   * Contador de numero de updates (para fazer freqencia de update
   * incremental).
   */
  private long nupdates = 0;
  /** prazo de validade das informaes do comando. */
  private long expirationInfoDelay;

  //CSFS related stuff
  /**
   * Eventualmente, o comando pode ser executado sobre um sandbox no SGA. Isso
   * acontece quando a maquina de execucao nao possui acesso NFS aos dados e
   * binarios, e um sandbox deve ser criado.
   */
  private String[] sandbox = null; // o sandbox eh um diretorio que deve ser 
  // montado usando o FileUtils 
  /**
   * Lista de arquivos com as respectivas datas de modificacao, usado para saber
   * o estado do sandbox antes da execucao do comando
   */
  private Map<String, Long> sandboxSnapshot = null;

  /**
   * SGA
   */
  private SGAInfo node;

  /**
   * Nome do n do SGA que deve executar o comando
   */
  private String hostName;

  /**
   * Limpa as informaes que ficam invlidas quando o comando termina.
   */
  private void eraseFields() {
    this.remoteReference = null;
    this.info = null;
    this.newInfo = false;
  }

  /**
   * Obtm o projeto onde o comando est sendo executado.
   * 
   * @return O projeto.
   */
  public ServerProject getProject() {
    return project;
  }

  /**
   * Obtm o identificador do projeto.
   * 
   * @return .
   */
  public Object getProjectId() {
    return cmdInfo.getProjectId();
  }

  /**
   * Retorna um array com o caminho do projeto, a partir da raiz do sistema
   * (exemplo: {"project","admin","proj1"}).
   * 
   * @return array com o caminho do projeto. Pode retornar <code>null</code>
   *         caso o projeto no seja encontrado.
   */
  String[] getProjectPath() {
    if (this.project == null) {
      return null;
    }
    return this.project.getPath();
  }

  /**
   * Atualiza o estado corrente do Comando.
   * 
   * @throws ServerException se houver alguma exceo for detectada!
   */
  public void update() throws ServerException {
    /*
     * Se a informao ainda  nova, i.e. o cliente no coletou, no faz update.
     * Deve verificar se tem referncia para no logar a exceo vrias vezes.
     */
    if (newInfo || (remoteReference == null)) {
      return;
    }
    try {
      // Busca informaes do comando
      RunningCommandInfo gotInfo = remoteReference.getRunningCommandInfo();
      if (gotInfo != null) {
        setNewInfo(gotInfo);
      }
      ++nupdates;
    }
    catch (Exception e) {
      setRemoteReference(null);
      throw new ServerException("Falha de update do comando:\n" + "\tId: "
        + getId() + "\n", e);
    }
  }

  /**
   * Atribui informaes sobre o comando vindas do SGA.
   * 
   * @param gotInfo informaes sobre o comando.
   */
  private synchronized void setNewInfo(RunningCommandInfo gotInfo) {
    info = gotInfo;
    newInfo = true;
    lastInfoUpdate = System.currentTimeMillis();
  }

  /**
   * Informa quantas vezes o comando foi atualizado Usado para fazer freqencia
   * de update incremental.
   * 
   * @return o nmero de vezes que o comando foi atualizado.
   */
  public long getNUpdates() {
    return nupdates;
  }

  /**
   * Cria os arquivos gerados pelo comando na rea de projetos, com estado "em
   * construo".
   * 
   * @param outputFileCollection os arquivos de sada do comando
   * 
   * @return o conjunto de arquivos criados
   */
  private SGAFile[] createCommandOutputFiles(
    Collection<FileParameterValue> outputFileCollection) {
    if ((outputFileCollection == null) || (outputFileCollection.size() == 0)) {
      return null;
    }
    int numFiles = outputFileCollection.size();
    SGAFile[] commandFiles = new SGAFile[numFiles];
    Iterator<FileParameterValue> outputFileIterator =
      outputFileCollection.iterator();
    for (int i = 0; i < numFiles; i++) {
      FileParameterValue file = outputFileIterator.next();
      String[] fileName = file.getPathAsArray();
      String fileType = file.getType();
      try {
        if (!project.exists(fileName)) {
          project.createFile(fileName, fileType, getUserId(), true);
        }
        project.setUnderConstruction(fileName, true);
        commandFiles[i] = new SGAFile(fileType, fileName);
      }
      catch (Exception e) {
        commandFiles[i] = null;
        service.logSevereMessage("Falha na criao do arquivo "
          + file.getPath(), e);
      }
    }
    return commandFiles;
  }

  /**
   * <p>
   * Atualiza os comandos utilizando o {@link CommandPersistenceService servio
   * de persistncia de comandos}.
   * </p>
   * 
   * <p>
   * O servio de persistncia de comandos pode no estar disponvel em todos os
   * sistemas. Neste caso, este mtodo no faz nada e retorna {@code true}.
   * </p>
   */
  private void updateCommand() {
    try {
      CommandPersistenceService persistenceService =
        CommandPersistenceService.getInstance();
      if (persistenceService == null) {
        return;
      }
      persistenceService.updateCommandInfo(getCommandInfo());
    }
    catch (ServiceFailureException e) {
      service.logSevereMessage("Ocorreu um erro ao atualizar o comando "
        + getId() + " no servio de persistncia", e);
    }
  }

  /**
   * Atualiza o estado dos arquivos gerados pelo comando, removendo a
   * sinalizao "em construo".
   */
  private void updateCommandOutputFiles() {
    if (commandOutputFiles == null) {
      return;
    }
    for (int i = 0; i < commandOutputFiles.length; i++) {
      SGAFile sgaFile = commandOutputFiles[i];
      if (sgaFile != null) {
        try {
          project.setUnderConstruction(sgaFile.getFileName(), false);
        }
        catch (Exception e) {
          service.logSevereMessage("Falha na atualizao do estado do arquivo "
            + sgaFile.getFileNameForSeparator("/"), e);
        }
      }
    }
  }

  /**
   * Trmino do comando.
   * 
   * @param cmdInfo dados sobre o comando completado.
   * 
   * @return <code>true</code> se a finalizao for bem sucedida.
   */
  public boolean finish(CompletedCommandInfo cmdInfo) {
    String msg =
      "Comando terminado.\n" + "Dados da execuo de (" + getId() + ")\n";
    if ((cmdInfo == null) || (cmdInfo.elapsedTimeSec == -1)) {
      msg = msg + "\tTempo decorrido: <No informado>;\n";
    }
    else {
      msg =
        msg + "\tTempo decorrido: " + cmdInfo.elapsedTimeSec + " segundo(s);\n";
    }
    if ((cmdInfo == null) || (cmdInfo.userTimeSec == -1)) {
      msg = msg + "\tTempo de usurio: <No informado>;\n";
    }
    else {
      msg =
        msg + "\tTempo de usurio: " + cmdInfo.userTimeSec + " segundo(s);\n";
    }
    if ((cmdInfo == null) || (cmdInfo.cpuTimeSec == -1)) {
      msg = msg + "\tTempo de CPU: <No informado>.\n";
    }
    else {
      msg = msg + "\tTempo de CPU: " + cmdInfo.cpuTimeSec + " segundo(s);\n";
    }
    service.logInfoMessage(msg);
    remoteReference = null;
    setStatus(CommandStatus.DOWNLOADING);
    boolean success = service.clearExecutionEnvironment(this);
    if (success) {
      if (this.project != null) {
        updateCommandOutputFiles();
      }
    }

    CommandFinalizationType type;
    if (hasExitCode()) {
      try {
        if (hasExecutionError()) {
          type = CommandFinalizationType.EXECUTION_ERROR;
          success = false;
        }
        else {
          type = CommandFinalizationType.SUCCESS;
        }
      }
      catch (ExitCodeNotFoundException e) {
        type = CommandFinalizationType.NO_EXIT_CODE;
        success = false;
      }
    }
    else {
      type = CommandFinalizationType.END;
    }

    setFinished(type);
    // No est claro qual a funo do mtodo a seguir:
    eraseFields();
    return success;
  }

  /**
   * Determina se o comando teve algum erro de execuo (se retornou um cdigo
   * de erro diferente de zero).
   * 
   * @return verdadeiro se houve erro de execuo, ou falso caso contrrio.
   * @throws ExitCodeNotFoundException Caso no seja possvel obter o cdigo de
   *         retorno.
   */
  private boolean hasExecutionError() throws ExitCodeNotFoundException {
    AlgorithmConfigurator configurator = getCommandConfigurator();
    boolean foundAllExitCodes = true;
    if (configurator != null) {
      Set<FileParameterValue> exitCodeLogFiles =
        configurator.getExitCodeLogFiles();
      for (FileParameterValue exitCodeLogFile : exitCodeLogFiles) {
        if (exitCodeLogFile != null) {
          Object projectId = this.getProjectId();
          ServerProject project = ServerProject.getProject(projectId);
          String[] fullPath =
            project.makeFullPath(exitCodeLogFile.getPathAsArray());
          final ServerProjectFile projectFile =
            ServerProjectFile.findFile(projectId, fullPath);
          Integer exitCode = readExitCodeLog(projectFile);
          if (exitCode != null) {
            if (exitCode != 0) {
              return true;
            }
          }
          else {
            foundAllExitCodes = false;
          }
        }
        else {
          foundAllExitCodes = false;
        }
      }
    }
    else {
      foundAllExitCodes = false;
    }
    if (foundAllExitCodes) {
      return false;
    }
    else {
      throw new ExitCodeNotFoundException();
    }
  }

  /**
   * L arquivo que armazena o cdigo de sada do comando e retorna o cdigo ou
   * nulo, caso o comando no retorne nenhum cdigo ou no seja possvel ler o
   * arquivo.
   * 
   * @param projectFile Arquivo que armazena o cdigo de sada.
   * @return O cdigo de sada ou nulo se no houver cdigo ou no for possvel
   *         ler o arquivo.
   */
  private Integer readExitCodeLog(ServerProjectFile projectFile) {
    if (projectFile != null) {
      FileReader fileReader = null;
      BufferedReader buffReader = null;
      try {
        fileReader = new FileReader(projectFile.getRealFile());
        buffReader = new BufferedReader(fileReader);
        String line = buffReader.readLine();
        Integer exitCode = null;
        if (line != null) {
          exitCode = Integer.parseInt(line);
        }
        return exitCode;
      }
      catch (FileNotFoundException e) {
        service.logSevereMessage(
          "Ocorreu um erro na tentativa de abrir o arquivo "
            + projectFile.getPath() + " com o cdigo de sada do comando "
            + this.getId() + ".\n", e);
      }
      catch (IOException e) {
        service.logSevereMessage(
          "Ocorreu um erro na tentativa de ler o contedo do arquivo "
            + projectFile.getPath() + " com o cdigo de sada do comando "
            + this.getId() + ".\n", e);
      }
      catch (NumberFormatException e) {
        service.logSevereMessage(
          "No foi possvel ler o cdigo de sada do comando " + this.getId()
            + ".\n", e);
      }
      finally {
        try {
          if (fileReader != null) {
            fileReader.close();
          }
          if (buffReader != null) {
            buffReader.close();
          }
        }
        catch (IOException e) {
          e.printStackTrace();
        }
      }
    }
    return null;
  }

  /**
   * Indica se o comando prov algum cdigo de retorno.
   * 
   * @return Verdadeiro se o comando prov cdigo de retorno, ou falso caso
   *         contrrio.
   */
  private boolean hasExitCode() {
    boolean hasExitCode = false;
    AlgorithmConfigurator configurator = getCommandConfigurator();
    if (configurator != null) {
      hasExitCode = configurator.hasExitCode();
    }
    return hasExitCode;
  }

  /**
   * Obtm o configurador do comando.
   * 
   * @return O configurador do comando.
   */
  private AlgorithmConfigurator getCommandConfigurator() {
    AlgorithmConfigurator configurator = null;
    try {
      configurator = getCommandInfo().getConfigurator();
    }
    catch (RemoteException e) {
      service.logSevereMessage(
        "Ocorreu um erro na tentativa de obter o configurador do comando "
          + this.getId() + ".\n", e);
    }
    return configurator;
  }

  /**
   * Interrompe o comando. Se houver alguma falha de comunicao, deixa o objeto
   * inalterado.
   * 
   * @throws ServerException
   */
  public synchronized void kill() throws ServerException {
    try {
      remoteReference.kill();
      setFinished(CommandFinalizationType.KILLED);
      service.logWarningMessage("Comando interrompido:\n" + "\tId: " + getId()
        + "\n");
      eraseFields();
    }
    catch (Exception e) {
      remoteReference = null;
      throw new ServerException("Falha na interrupo do comando " + getId(), e);
    }
  }

  /**
   * comando com falha.
   */
  public synchronized void failed() {
    setFinished(CommandFinalizationType.FAILED);
    service.logWarningMessage("Comando com falha:\n" + "\tId: " + getId()
      + "\n");
    eraseFields();
  }

  /**
   * comando perdido.
   */
  public synchronized void lost() {
    setFinished(CommandFinalizationType.LOST);
    service.logWarningMessage("Comando perdido:\n" + "\tId: " + getId() + "\n");
    eraseFields();
  }

  /**
   * Descobre se o comando requer envio de mail ao seu trmino.
   * 
   * @return este flag booleano
   */
  boolean requireMail() {
    return cmdInfo.isMailAtEnd();
  }

  /**
   * Obtm a lista de endereos que devem receber email de aviso de trmino de
   * execuo do comando, alm do prprio dono do projeto.
   * 
   * @return o array com endereos de email
   */
  public String[] getEmailList() {
    return cmdInfo.getEmailList();
  }

  /**
   * Obtm a hora que o comando foi iniciado.
   * 
   * @return hora que o comando foi iniciado
   */
  long getStartTime() {
    return this.startTimeMillis;
  }

  /**
   * Obtm a descricao do comando.
   * 
   * @return a descricao do comando
   */
  String getDescription() {
    return cmdInfo.getDescription();
  }

  /**
   * Obtm a dica para auxiliar a identificao do algoritmo do comando
   * 
   * @return .
   */
  public String getTip() {
    return cmdInfo.getTip();
  }

  /**
   * Obtm o usurio que executou o comando.
   * 
   * @return este identificador
   */
  public Object getUserId() {
    return cmdInfo.getUserId();
  }

  /**
   * Obtm o identificador do comando.
   * 
   * @return este identificador
   */
  public String getId() {
    return cmdInfo.getId();
  }

  /**
   * Obtm informaes do comando.
   * 
   * @return as informaes do comando
   */
  protected CommandInfo getCommandInfo() {
    return this.cmdInfo;
  }

  /**
   * Obtm a String do comando.
   * 
   * @return o comando
   */
  public String getCommand() {
    return this.command;
  }

  /**
   * Obtm o estado de execuo do comando.
   * 
   * @return .
   */
  public CommandStatus getStatus() {
    return cmdInfo.getStatus();
  }

  /**
   * Altera o estado de execuo do comando.
   * 
   * @param cmdStatus o estado corrente.
   * @return <tt>False</tt> indicando que a alterao no foi feita por que o
   *         comando est no estado terminal - {@link CommandStatus#FINISHED} -.
   */
  private boolean setStatus(CommandStatus cmdStatus) {
    if (cmdInfo.setStatus(cmdStatus)) {
      updateCommand();
      // O ideal  migrar para o commandPersistenceService e gerar um
      // arquivo por comando.
      sga.saveCommandTable();
      return true;
    }
    return false;
  }

  /**
   * Altera o estado do comando para {@link CommandStatus#FINISHED terminado} e
   * atribui um valor ao {@link CommandFinalizationType modo em que este comando
   * foi finalizado}.
   * 
   * @param finalizationType o que levou o comando a terminar.
   */
  public void setFinished(CommandFinalizationType finalizationType) {
    cmdInfo.setFinished(finalizationType);
    updateCommand();
    // O ideal  migrar para o commandPersistenceService e gerar um
    // arquivo por comando.
    sga.saveCommandTable();
  }

  /**
   * Obtm a referncia remota para o comando no SGA.
   * 
   * @return a referncia remota
   */
  public SGACommand getRemoteReference() {
    return this.remoteReference;
  }

  /**
   * Atualiza a referncia remota para o comando no SGA. Necessrio quando o SGA
   * saiu do ar e est voltando.
   * 
   * @param remoteReference a referncia remota
   */
  public synchronized void setRemoteReference(SGACommand remoteReference) {
    this.remoteReference = remoteReference;
  }

  /**
   * Informa se h conexo com o comando em execuo.
   * 
   * @return <code>true</code> caso o comando disponha de uma referncia para o
   *         processo em execuo.
   */
  boolean isAlive() {
    return remoteReference != null;
  }

  /**
   * Retorna o caminho do sandbox na maquina de execucao.
   * 
   * @return O caminho eh retornado como um array de string, representando a
   *         sequancia de diretorios dentro da raiz exportada pelo CSFS.
   */
  String[] getSandbox() {
    return sandbox;
  }

  /**
   * Retorna referncia para o SGA que est executando o comando.
   * 
   * @return referncia para o SGA que est executando o comando.
   */
  SGA getSGA() {
    return sga;
  }

  /**
   * Ajusta a referncia para o SGA que est executando o comando.
   * 
   * @param sga referncia para o SGA que est executando o comando.
   */
  void setSGA(SGA sga) {
    this.sga = sga;
  }

  /**
   * Recupera uma lista com os arquivos de saida de um comando.
   * 
   * @return lista com os arquivos de saida de um comando.
   */
  SGAFile[] getCommandOutputFiles() {
    return commandOutputFiles;
  }

  /**
   * Define o estado inicial da area de trabalho na maquina de execucao. Essa
   * informacao eh util para que a copia dos arquivos alterados seja avaliada e
   * realizada apos a execucao de um comando. Verificar a classe
   * <code>CSFSService</code>.
   * 
   * BUG WARNING: O sistema pode ter problemas caso o SSI caia. Quando o comando
   * for concluido, essa mapeamento nao estara disponivel e o sistema nao serah
   * capaz de saber quais arquivos foram modificados. Nesse cenario, eh
   * necesario verificar/implementar um mecanismo de persistencia desse
   * mapeamento, ou forcar a copia de todos os arquivos do sandbox.
   * 
   * @param snapshot
   */
  void setSandboxSnapshot(Map<String, Long> snapshot) {
    this.sandboxSnapshot = snapshot;
  }

  /**
   * Recupera o mapeamento com a informacao sobre o estado inicial dos arquivos
   * (no sandbox).
   * 
   * @return mapeamento com a informacao sobre o estado inicial dos arquivos no
   *         sandbox.
   */
  Map<String, Long> getSandboxSnapshot() {
    return this.sandboxSnapshot;
  }

  /**
   * Define um sandbox temporrio a ser utilizado para a execuo do comando.
   * Substitui os caminhos originalmente preparados para um SGA com acesso 
   * rea de projetos e aos algoritmos via NFS por caminhos dentro do sandbox do
   * CSFS.
   * 
   * @param sandbox array representando o caminho do sandbox temporrio criado
   *        para essa execuo (relativo ao diretrio-raiz do CSFS).
   */
  void setSandbox(String[] sandbox) {
    this.sandbox = sandbox;
  }

  /**
   * Define o caminho do diretrio de projeto onde o comando ser executado.
   * 
   * @param projectPath o caminho do diretrio raz do csfs
   */
  void setExecutionProjectPath(String[] projectPath) {
    this.projectPath = projectPath;
  }

  /**
   * Define o caminho do diretrio raz do csfs.
   * 
   * @param rootPath o caminho do diretrio raz do csfs
   */
  void setRootPath(String[] rootPath) {
    this.rootPath = rootPath;
  }

  /**
   * Executa o comando remoto. Esse metodo deve ser chamado apos a copia dos
   * binarios e dos dados para a maquina de execucao. Ou no caso onde esses
   * arquivos estao acessiveis via NFS.
   */
  protected void executeRemoteCommand() {
    String id = getId();
    setStatus(CommandStatus.UPLOADING);
    if (!service.setupExecutionEnvironment(this)) {
      setFinished(CommandFinalizationType.FAILED);
      sga.failCommand(getId());
      return;
    }
    try {
      AlgorithmConfigurator configurator = cmdInfo.getConfigurator();
      String fileSeparator = node.getFileSeparator();
      // FIXME o separador deveria ser um char
      char separatorChar = fileSeparator.charAt(0);
      /** Define caminhos default */
      if (this.projectPath == null) {
        this.projectPath = getProjectPath();
      }
      String rootDirectory;
      if (this.rootPath == null) {
        rootDirectory = node.getSSIRootDirectory();
        this.rootPath = FileUtils.splitPath(rootDirectory, fileSeparator);
      }
      else {
        rootDirectory = FileUtils.joinPath(separatorChar, this.rootPath);
      }
      if (this.project != null) {
        /** Cria os arquivos de sada no diretrio de projetos. */
        Collection<FileParameterValue> outputFileCollection =
          configurator.getOutputFiles();
        this.commandOutputFiles =
          createCommandOutputFiles(outputFileCollection);
      }
      String outputPath =
        rootDirectory
          + fileSeparator
          + FileUtils.joinPath(separatorChar, this.projectPath)
          + fileSeparator
          + FileUtils.joinPath(separatorChar, CommandPersistenceService
            .getInstance().getLogDirectory(getId())) + fileSeparator;
      // XXX - Como escolher qual o binaryDirectory que deve ser usado?!?!
      // XXX - Quando no  fluxo, teve ter um diretrio s...
      /**
       * Diretorios que contem os binarios necessarios a execucao do comando
       */
      String binaryPath;
      Set<String> binaryDirectorySet =
        configurator.getBinaryDirectories(node.getPlatformId(), fileSeparator);
      if (binaryDirectorySet.size() > 0) {
        String[] binaryDirectories =
          binaryDirectorySet.toArray(new String[binaryDirectorySet.size()]);
        binaryPath = rootDirectory + fileSeparator + binaryDirectories[0];
      }
      else {
        // CORBA nao aceita parametro null p/ String
        binaryPath = "";
      }

      String projectDirectory =
        FileUtils.joinPath(fileSeparator, this.projectPath);
      CommandLineContext context;
      switch (configurator.getExecutionType()) {
        case DISTRIBUTED:
          context =
            CommandLineContext.createDistributedContext(id, node
              .getPlatformId(), fileSeparator, rootDirectory, projectDirectory,
              cmdInfo.getSelectedSGAsNames(), cmdInfo
                .getExecutionCountPerSGAForDistributedExecution());
          break;
        case PARALLEL:
          context =
            CommandLineContext.createParallelContext(id, node.getPlatformId(),
              fileSeparator, rootDirectory, projectDirectory, cmdInfo
                .getCpuCountForParallelExecution());
          break;
        default:
          context =
            CommandLineContext.createSimpleContext(id, node.getPlatformId(),
              fileSeparator, rootDirectory, projectDirectory);
      }
      this.command = configurator.makeCommandLine(context);
      this.service.logInfoMessage("Linha de comando: " + this.command);
      String hostNameParam = hostName;
      if (hostNameParam == null) {
        // CORBA nao aceita parametro null p/ String
        hostNameParam = "";
      }
      SGAServer sgaRemoteReference = this.sga.getRemoteReference();
      this.remoteReference =
        sgaRemoteReference.executeCommand(this.command, id, hostNameParam,
          binaryPath, outputPath);
      if (this.remoteReference == null) {
        setFinished(CommandFinalizationType.FAILED);
        sga.failCommand(id);
        this.service
          .logSevereMessage("FALHA DE COMANDO!\n\tCausa: Falha na aquisio de ref. remota"
            + " a comando!\n\tId: " + id + "\n\n");
      }
      else {
        setStatus(CommandStatus.EXECUTING);
        this.service.logInfoMessage(String.format(
          "Comando posto em execuo:\n\tId: %s\n\t(%s)\n\tComando: %s\n", id,
          this.sga.getName(), this.command));
      }
    }
    catch (Exception e) {
      setFinished(CommandFinalizationType.FAILED);
      sga.failCommand(id);
      this.service.logSevereMessage("FALHA DE COMANDO!\n"
        + "\tCausa: Falha na construo do comando: [" + this.command
        + "]\n\tSGA [" + sga.getName() + "] - id: " + id + ".\n\tErro: " + e
        + ".\n", e);
    }
  }

  /**
   * Refaz a referncia para o servio de gerncia dos SGAs.
   * 
   * @param sgaService servio de gerncia dos SGAs.
   */
  void setService(SGAService sgaService) {
    this.service = sgaService;
  }

  /**
   * Refaz a referncia para o projeto onde o comando foi executado.
   * 
   * @param project projeto onde o comando foi executado.
   */
  void setProject(ServerProject project) {
    this.project = project;
  }

  /**
   * Constri a representao de um comando.
   * 
   * @param hostName Nome do n do SGA que deve executar o comando
   * @param sga representao local do SGA da mquina hospedeira
   * @param cmdInfo informaes sobre o comando a ser executado.
   * @param expirationInfoDelay prazo de validade das informaes do comando.
   * 
   * @throws OperationFailureException Em caso de falha.
   */
  public Command(String hostName, SGA sga, CommandInfo cmdInfo,
    long expirationInfoDelay) throws OperationFailureException {
    this.cmdInfo = cmdInfo;
    // XXX - [CSBASE-567] Rever uso de paths como strings ou arrays.
    // (ver SGAService -- sandbox)
    this.sga = sga;
    this.hostName = hostName;
    this.node =
      hostName == null ? this.sga.getInfo(0) : this.sga.getInfo(hostName);
    this.rootPath =
      FileUtils.splitPath(node.getSSIRootDirectory(), node.getFileSeparator());
    this.expirationInfoDelay = expirationInfoDelay;
    // Adicionado somente para persistncia
    Object projectId = cmdInfo.getProjectId();
    if (projectId != null) {
      this.project = ServerProject.getProject(projectId);
      if (this.project == null) {
        String message =
          String.format("O projeto %s no foi encontrado.", projectId);
        throw new OperationFailureException(message);
      }
    }
    this.service = SGAService.getInstance();
    this.startTimeMillis = Calendar.getInstance().getTimeInMillis();
    setStatus(CommandStatus.INIT);

    service.logInfoMessage("Pedido de comando:\n\tId: " + cmdInfo.getUserId()
      + "\n");
  }

  /**
   * Obtm o node que ir executar o comando.
   * 
   * @return .
   */
  public SGAInfo getNode() {
    return node;
  }

  /**
   * Obtm o nome do projeto no qual o comando foi executado.
   * 
   * @return o nome do projeto no qual o comando foi executado.
   */
  public String getProjectName() {
    return project.getName();
  }

  /**
   * <p>
   * Cria as informaes da monitorao do comando.
   * </p>
   * 
   * <p>
   * ATENO: A sincronizao deste mtodo garante que as informaes so da
   * mesma instncia de RunningCommandInfo.
   * </p>
   * 
   * @return comando com as ltimas informaes obtidas
   */
  public synchronized CommandInfo createCommandInfo() {
    // Indica que a informao j foi lida pelo cliente e ativa o update.
    newInfo = false;
    if ((System.currentTimeMillis() - lastInfoUpdate) > expirationInfoDelay
      || info == null) {
      if (cmdInfo.getStatus() == CommandStatus.EXECUTING && !isAlive()) {
        cmdInfo.invalidateDynamicFields();
      }
      return cmdInfo;
    }
    // As informaes ainda so vlidas, ento confio na cache.
    ProcessInfo[] infos = info.processData;
    if (infos.length == 0) {
      cmdInfo.validateDynamicFields();
      return cmdInfo;
    }

    boolean queued = false;
    /*
     * Se pelo menos um dos processos do comando est em espera na fila do
     * escalonador (escalonador externo, no o Scheduler), o comando fica
     * marcado com enfileirado.
     * 
     * ver CommandInfo#isQueued
     */
    for (ProcessInfo processInfo1 : infos) {
      ProcessState state = processInfo1.state;
      if (state.value() == ProcessState._WAITING) {
        queued = true;
        break;
      }
    }
    cmdInfo.setQueued(queued);

    // Calcula o percentual total de uso de CPU por todos os processos do
    // comando.
    Double cpuPerc = 0.0;
    for (ProcessInfo processInfo : infos) {
      if (processInfo.CPUPerc < 0) {
        cpuPerc = null;
        break;
      }
      cpuPerc += processInfo.CPUPerc;
    }
    cmdInfo.setCpuPerc(cpuPerc);

    // Calcula o total de memria RAM (em MB) utilizada por todos os 
    // processos do comando.    
    Double ramMB = 0.0;
    for (ProcessInfo processInfo : infos) {
      if (processInfo.memoryRamSizeMb < 0) {
        ramMB = null;
        break;
      }
      ramMB += processInfo.memoryRamSizeMb;
    }
    cmdInfo.setRAMMemoryMB(ramMB);
    // Calcula o percentual de memria RAM utilizada por todos os 
    // processos do comando.     
    Double memoryRAMPerc = null;
    if (ramMB != null) {
      // [CSBASE-567]
      // Calculando o total de memria do SGA (considerando caso de ser
      // cluster)
      int sgaRAMMB = 0;
      for (SGAInfo sgaInfo : sga.getAllInfo()) {
        sgaRAMMB += sgaInfo.getRAMMemoryInfoMb();
      }
      memoryRAMPerc = (sgaRAMMB == 0) ? null : 100 * (ramMB / sgaRAMMB);
    }
    cmdInfo.setRAMMemoryPerc(memoryRAMPerc);
    // Calcula o total de memria Swap (em MB) utilizada por todos os 
    // processos do comando.    
    Double swapMB = 0.0;
    for (ProcessInfo processInfo : infos) {
      if (processInfo.memorySwapSizeMb < 0) {
        swapMB = null;
        break;
      }
      swapMB += processInfo.memorySwapSizeMb;
    }
    cmdInfo.setSwapMemoryMB(swapMB);
    // Calcula o percentual de memria swap utilizada por todos os 
    // processos do comando.     
    Double memorySwapPerc = null;
    if (swapMB != null) {
      // [CSBASE-567]
      // Calculando o total de memria do SGA (considerando caso de ser
      // cluster)
      int sgaSwapMB = 0;
      for (SGAInfo sgaInfo : sga.getAllInfo()) {
        sgaSwapMB += sgaInfo.getSwapMemoryInfoMb();
      }
      memorySwapPerc = (sgaSwapMB == 0) ? null : 100 * (swapMB / sgaSwapMB);
    }
    cmdInfo.setSwapMemoryPerc(memorySwapPerc);

    Integer wallTimeSec =
      (infos[0].wallTimeSec < 0) ? null : infos[0].wallTimeSec;
    cmdInfo.setWallTimeSec(wallTimeSec);
    Double userTimeSec = 0.0;
    for (ProcessInfo processInfo : infos) {
      if (processInfo.userTimeSec < 0) {
        userTimeSec = null;
        break;
      }
      userTimeSec += processInfo.userTimeSec;
    }
    cmdInfo.setUserTimeSec(userTimeSec);

    Double systemTimeSec = 0.0;
    for (ProcessInfo processInfo : infos) {
      if (processInfo.systemTimeSec < 0) {
        systemTimeSec = null;
        break;
      }
      systemTimeSec += processInfo.systemTimeSec;
    }
    cmdInfo.setSystemTimeSec(systemTimeSec);

    Double virtualMemorySizeMB = 0.0;
    for (ProcessInfo processInfo : infos) {
      if (processInfo.virtualMemorySizeMB < 0) {
        virtualMemorySizeMB = null;
        break;
      }
      virtualMemorySizeMB += processInfo.virtualMemorySizeMB;
    }
    cmdInfo.setVirtualMemorySizeMB(virtualMemorySizeMB);

    Double bytesInKB = 0.0;
    if (infos[0].bytesInKB < 0)
      bytesInKB = null;
    else
      bytesInKB = infos[0].bytesInKB;
    cmdInfo.setBytesInKB(bytesInKB);

    Double bytesOutKB = 0.0;
    if (infos[0].bytesOutKB < 0)
      bytesOutKB = null;
    else
      bytesOutKB = infos[0].bytesOutKB;
    cmdInfo.setBytesOutKB(bytesOutKB);

    Double diskBytesReadKB = 0.0;
    if (infos[0].diskBytesReadKB < 0)
      diskBytesReadKB = null;
    else
      diskBytesReadKB = infos[0].diskBytesReadKB;
    cmdInfo.setDiskBytesReadKB(diskBytesReadKB);

    Double diskBytesWriteKB = 0.0;
    if (infos[0].diskBytesWriteKB < 0)
      diskBytesWriteKB = null;
    else
      diskBytesWriteKB = infos[0].diskBytesWriteKB;
    cmdInfo.setDiskBytesWriteKB(diskBytesWriteKB);

    Set<String> nodes = new HashSet<String>();
    for (ProcessInfo inf : infos) {
      if (inf.execHost != null) {
        nodes.add(inf.execHost);
      }
    }
    String executionNodeDescription = "";
    if (sga.isCluster()) {
      String sep = "";
      for (String node : nodes) {
        executionNodeDescription += sep + node;
        sep = " | ";
      }
    }
    cmdInfo.setExecutionNodeDescription(executionNodeDescription);

    Map<String, String> specificData = new HashMap<String, String>();
    for (ExecutionInfo execInfo : info.executionData) {
      specificData.put(execInfo.key, execInfo.value);
    }
    cmdInfo.setSpecificData(specificData);
    cmdInfo.validateDynamicFields();
    return cmdInfo;
  }

}
