package csbase.server.services.trustedserverservice;

import java.io.File;
import java.io.IOException;
import java.security.cert.Certificate;
import java.text.MessageFormat;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import csbase.exception.CSBaseException;
import csbase.exception.OperationFailureException;
import csbase.exception.ServiceFailureException;
import csbase.logic.ServerURI;
import csbase.logic.trustedserver.TrustedServer;
import csbase.logic.trustedserver.TrustedServerAddEvent;
import csbase.logic.trustedserver.TrustedServerInfo;
import csbase.logic.trustedserver.TrustedServerModifyEvent;
import csbase.logic.trustedserver.TrustedServerRemoveEvent;
import csbase.remote.TrustedServerServiceInterface;
import csbase.server.FileSystem;
import csbase.server.Server;
import csbase.server.ServerException;
import csbase.server.Service;
import csbase.server.keystore.CSKeyStore;
import csbase.server.services.languageservice.LanguageService;

/**
 * Representa o servio de servidores confiveis.
 *
 * @author Tecgraf/PUC-Rio
 */
public final class TrustedServerService extends Service
implements TrustedServerServiceInterface {
  /**
   * Nome da propriedade que contm o nome do arquivo do repositrio de
   * servidores confiveis.
   */
  private static final String TRUSTED_SERVER_REPOSITORY_NAME_PROPERTY =
    "trustedServerRepositoryName";

  /** O caminho do arquivo do repositrio de servidores confivies. */
  private String trustedServerRepository;

  /** O conjunto ordenado (por nome) de servidores confiveis. */
  private SortedSet<TrustedServer> trustedServerSortedSet;

  /** Um mapa que liga um id ao seu respectivo servidor  */
  private SortedMap<Integer, TrustedServer> trustedServerIdSortedMap;

  /** Um mapa que liga um nome ao seu respectivo servidor */
  private Map<String, TrustedServer> trustedServerNameMap;

  /** A instncia nica do servio de servidores confiveis. */
  private static TrustedServerService instance;
  private static final String TRUSTED_SERVER_ROOT_TAG = "servidores_confiaveis";
  private static final String TRUSTED_SERVER_TAG = "servidor";
  private static final String TRUSTED_SERVER_ID_TAG = "id";
  private static final String TRUSTED_SERVER_NAME_TAG = "nome";
  private static final String TRUSTED_SERVER_SUSPENDED_TAG = "suspenso";
  private static final String TRUSTED_SERVER_LOCAL_TAG = "local";
  private static final String TRUSTED_SERVER_URI_TAG = "uri";

/**
   * <p>
   * Cria o servio de servidores confiveis.
   * </p>
   *
   * @throws ServerException Caso ocorra algum erro durante a criao do
   *         servio.
   */
  private TrustedServerService()
  throws ServerException {
    super(SERVICE_NAME);
    this.createCache();
    this.readProperties();
    if (!this.createTrustedServerRepository()) {
      this.reloadCache(loadTrustedServerRepository(this.trustedServerRepository));
      try {
        this.updateTrustedServers();
      }
      catch (OperationFailureException e) {
        this.logSevereMessage(
          "Erro ao carregar o repositrio de chaves/certificados.", e);
      }
    }
    this.calculateNextId();
  }

  /**
   * Cria as estruturas de dados que sero utilizados como cache de
   * servidores confiveis.
   */
  private void createCache() {
    this.trustedServerSortedSet = new TreeSet<TrustedServer>();
    this.trustedServerIdSortedMap = new TreeMap<Integer, TrustedServer>();
    this.trustedServerNameMap = new HashMap<String, TrustedServer>();
  }

  /**
   * L as propriedades do servio.
   */
  private void readProperties() {
    final Server server = Server.getInstance();
    final String dirPath = server.getPersistencyRootDirectoryName();
    this.trustedServerRepository = dirPath+File.separator+
      this.getStringProperty(TRUSTED_SERVER_REPOSITORY_NAME_PROPERTY);
  }

  /**
   * Gera o prximo identificador vlido para um servidor. Esse
   * identificador  utilizado na criao de um novo servidor.
   *
   * @return o identificador do servidor
   */
  private int calculateNextId() {
    if (this.trustedServerIdSortedMap.isEmpty()) {
      return 1;
    }
    return (this.trustedServerIdSortedMap.lastKey()+1);
  }

  /**
   * <p>Cria o repositrio de servidores confiveis, caso ainda no exista.</p>
   *  <p>A coleo de servidores confiveis  gravada no arquivo e, por isso,
   * deve ser criada antes da chamada deste mtodo.</p>
   *
   * @return .
   *
   * @throws ServerException Caso ocorra algum problema ao criar o repositrio.
   * @throws IllegalStateException Caso a coleo de servidores confiveis ainda
   *         no tenha sido criada.
   */
  private boolean createTrustedServerRepository()
  throws ServerException {
    if (this.trustedServerSortedSet == null) {
      throw new IllegalStateException(
        "Tentativa de criar o arquivo do repositrio de servidores confiveis antes de criar a coleo de servidores confiveis.");
    }
    File localServersFile = new File(this.trustedServerRepository);
    if (localServersFile.exists()) {
      return false;
    }
    try {
      localServersFile.createNewFile();
    }
    catch (IOException e) {
      throw new ServerException(
        "Erro ao criar o arquivo com o repositrio de servidores confiveis. Caminho usado: {0}.",
        new Object[] { localServersFile.getPath() }, e);
    }
    try {
      writeTrustedServerRepository(
        this.trustedServerSortedSet, this.trustedServerRepository);
    }
    catch (CSBaseException e) {
      throw new ServerException(
        "Erro ao gravar servidores confiveis no repositrio.", e);
    }
    return true;
  }

  /**
   * Obtm a instncia nica do servio de servidores confiveis.
   *
   * @return A instncia nica.
   */
  public static TrustedServerService getInstance() {
    return instance;
  }

  /**
   * Cria a instncia do servio de servidores confiveis.
   *
   * @throws ServerException Caso ocorra alguma falha na criao do servio.
   */
  public static void createService()
  throws ServerException {
    instance = new TrustedServerService();
  }

  public void initService() {
  }

  public void shutdownService() {
  }

  protected boolean has2Update(Object arg, Object event) {
    return true;
  }

  /**
   * Atualiza o cache de servidores confiveis. Alteraes no repositrio de
   * chaves/certificados sero refletidas nos servidores confiveis.
   *
   * @throws OperationFailureException
   */
  private void updateTrustedServers()
  throws OperationFailureException {
    CSKeyStore keyStore = CSKeyStore.getInstance();
    if (keyStore == null) {
      return;
    }
    List<TrustedServer> localServerList = new LinkedList<TrustedServer>();
    List<TrustedServer> localServerOldStateList = new LinkedList<TrustedServer>();
    List<TrustedServer> localServerNewStateList = new LinkedList<TrustedServer>();
    Iterator<TrustedServer> localServerIterator =
      this.trustedServerSortedSet.iterator();
    while (localServerIterator.hasNext()) {
      TrustedServer localServer = localServerIterator.next();
      Certificate certificate;
      try {
        certificate = this.getCertificate(localServer.getName());
      }
      catch (CSBaseException e) {
        continue;
      }
      TrustedServer newLocalServer;
      if (certificate == null) {
        newLocalServer = new TrustedServer(localServer);
      }
      else {
        newLocalServer = new TrustedServer(localServer, certificate);
      }
      if (localServer.isCompleted() != newLocalServer.isCompleted()) {
        localServerOldStateList.add(localServer);
        localServerNewStateList.add(newLocalServer);
      }
      localServerList.add(newLocalServer);
    }
    writeTrustedServerRepository(localServerList, this.trustedServerRepository);
    this.reloadCache(localServerList);
    this.fireWasModifiedTrustedServer(
      localServerOldStateList, localServerNewStateList);
  }

  /**
   * Recarrega o cache, limpando-o e adicionando os elementos da coleo
   * recebida.
   *
   * @param trustedServerCollection A coleo com os elementos que faro parte do
   *        cache.
   */
  private void reloadCache(Collection<TrustedServer> trustedServerCollection) {
    this.trustedServerSortedSet.clear();
    this.trustedServerIdSortedMap.clear();
    this.trustedServerNameMap.clear();
    Iterator<TrustedServer> localServerIterator = trustedServerCollection.iterator();
    while (localServerIterator.hasNext()) {
      TrustedServer localServer = localServerIterator.next();
      this.trustedServerSortedSet.add(localServer);
      this.trustedServerIdSortedMap.put(
        (Integer)localServer.getId(), localServer);
      this.trustedServerNameMap.put(localServer.getName(), localServer);
    }
  }

  public SortedSet<TrustedServer> getAllTrustedServers() {
    try {
      this.updateTrustedServers();
    }
    catch (CSBaseException e) {
      throw new ServiceFailureException(
        LanguageService.getInstance()
        .get("TRUSTED_SERVER_SERVICE_LOAD_REPOSITORIES_ERROR"), e);
    }
    return Collections.unmodifiableSortedSet(this.trustedServerSortedSet);
  }

  public boolean addTrustedServer(TrustedServerInfo trustedServerInfo) {
    if (this.hasTrustedServer(trustedServerInfo.getName())) {
      return false;
    }
    TrustedServer newLocalServer;
    try {
      Certificate certificate = this.getCertificate(trustedServerInfo.getName());
      int localServerId = this.calculateNextId();
      if (this.trustedServerIdSortedMap.get(localServerId) != null) {
        throw new IllegalStateException(
          MessageFormat.format(
            "Foi gerado um identificador repetido para o servidor confivel. Identificador: {0}.",
            localServerId));
      }
      if (certificate == null) {
        newLocalServer = new TrustedServer(localServerId, trustedServerInfo);
      }
      else {
        newLocalServer = new TrustedServer(
            localServerId, trustedServerInfo, certificate);
      }
      SortedSet<TrustedServer> localServerSortedSetTemp =
        new TreeSet<TrustedServer>(this.trustedServerSortedSet);
      localServerSortedSetTemp.add(newLocalServer);
      writeTrustedServerRepository(
        localServerSortedSetTemp, this.trustedServerRepository);
    }
    catch (CSBaseException e) {
      throw new ServiceFailureException(
        LanguageService.getInstance()
        .get("TRUSTED_SERVER_SERVICE_ADD_TRUSTED_SERVER_ERROR"), e);
    }
    this.addToCache(newLocalServer);
    this.fireWasAddedTrustedServer(newLocalServer);
    return true;
  }

  /**
   * Obtm o certificado de um servidor confivel.
   *
   * @param trustedServerName O servidor do qual se deseja o certificado.
   *
   * @return O certificado ou null, caso no exista certificado ou se o
   *         repositrio de chaves/certificados ainda no existir.
   *
   * @throws OperationFailureException Caso ocorra alguma falha na operao.
   */
  private Certificate getCertificate(String trustedServerName)
  throws OperationFailureException {
    CSKeyStore keyStore = CSKeyStore.getInstance();
    if (keyStore == null) {
      return null;
    }
    return keyStore.getCertificate(trustedServerName);
  }

  /**
   * Verifica se existe algum servidor confivel no repositrio com o mesmo
   * nome do servidor recebido.
   *
   * @param trustedServer O servidor a ser procurado.
   *
   * @return true, caso exista algum servidor confivel no repositrio com o mesmo
   *         nome, ou false, caso contrrio.
   */
  private boolean hasTrustedServer(TrustedServer trustedServer) {
    TrustedServer matchedLocalServer =
      this.trustedServerNameMap.get(trustedServer.getName());
    if (matchedLocalServer == null) {
      return false;
    }
    if (matchedLocalServer.getId().equals(trustedServer.getId())) {
      return false;
    }
    return true;
  }

  /**
   * Verifica se existe no repositrio algum servidor confivel com o nome
   * dado.
   *
   * @param trustedServerName O nome do servidor 
   *
   * @return true, caso j exista servidor confivel com o nome dado, ou false.
   */
  private boolean hasTrustedServer(String trustedServerName) {
    TrustedServer matchedLocalServer =
      this.trustedServerNameMap.get(trustedServerName);
    if (matchedLocalServer == null) {
      return false;
    }
    return true;
  }

  /**
   * Adiciona o servidor confivel ao cache de servidores confiveis.
   *
   * @param trustedServer O servidor confivel a ser adicionado.
   */
  private void addToCache(TrustedServer trustedServer) {
    this.trustedServerSortedSet.add(trustedServer);
    this.trustedServerIdSortedMap.put((Integer)trustedServer.getId(), trustedServer);
    this.trustedServerNameMap.put(trustedServer.getName(), trustedServer);
  }

  /**
   * Envia aos observadores uma notificao de que um servidor confivel foi
   * adicionado.
   *
   * @param trustedServer O servidor que foi adicionado.
   */
  private void fireWasAddedTrustedServer(TrustedServer trustedServer) {
    this.setChanged();
    this.notifyObservers(new TrustedServerAddEvent(this, trustedServer));
  }

  public void removeTrustedServer(TrustedServer trustedServer) {
    if (this.trustedServerIdSortedMap.get(trustedServer.getId()) == null) {
      throw new IllegalStateException(
        MessageFormat.format(
          "No  possvel remover um servidor confivel que no existe. Identificador {0}.",
          new Object[] { trustedServer.getId() }));
    }
    
    CSKeyStore keyStore = CSKeyStore.getInstance();
    if (keyStore != null) 
      keyStore.removeCertificate(trustedServer.getName());
    
    SortedSet<TrustedServer> localServerSortedSetTemp =
      new TreeSet<TrustedServer>(this.trustedServerSortedSet);
    localServerSortedSetTemp.remove(trustedServer);
    try {
      writeTrustedServerRepository(
        localServerSortedSetTemp, this.trustedServerRepository);
    }
    catch (CSBaseException e) {
      throw new ServiceFailureException(
        LanguageService.getInstance()
        .get("TRUSTED_SERVER_SERVICE_REMOVE_TRUSTED_SERVER_ERROR"), e);
    }
    this.removeFromCache(trustedServer);
    this.fireWasRemovedTrustedServer(trustedServer);
  }

  /**
   * Remove o servidor do cache de servidores confiveis.
   *
   * @param trustedServer O servidor a ser removido.
   */
  private void removeFromCache(TrustedServer trustedServer) {
    this.trustedServerSortedSet.remove(trustedServer);
    this.trustedServerIdSortedMap.remove(trustedServer.getId());
    this.trustedServerNameMap.remove(trustedServer.getName());
  }

  /**
   * Envia aos observadores uma notificao de que um servidor foi
   * removido.
   *
   * @param trustedServer O servidor que foi removido.
   */
  private void fireWasRemovedTrustedServer(TrustedServer trustedServer) {
    this.setChanged();
    this.notifyObservers(new TrustedServerRemoveEvent(this, trustedServer));
  }

  public boolean modifyTrustedServer(
    TrustedServer trustedServer, TrustedServerInfo trustedServerInfo) {
    if (this.trustedServerIdSortedMap.get(trustedServer.getId()) == null) {
      throw new IllegalStateException(
        MessageFormat.format(
          "No  possvel modificar um servidor confivel que no existe. Identificador {0}.",
          new Object[] { trustedServer.getId() }));
    }
    TrustedServer newLocalServer;
    try {
      Certificate certificate = this.getCertificate(trustedServerInfo.getName());
      if (certificate == null) {
        newLocalServer = new TrustedServer(
            (Integer)trustedServer.getId(), trustedServerInfo);
      }
      else {
        newLocalServer = new TrustedServer(
            (Integer)trustedServer.getId(), trustedServerInfo, certificate);
      }
      if (this.hasTrustedServer(newLocalServer)) {
        return false;
      }
      SortedSet<TrustedServer> localServerSortedSetTemp =
        new TreeSet<TrustedServer>(this.trustedServerSortedSet);
      localServerSortedSetTemp.remove(trustedServer);
      localServerSortedSetTemp.add(newLocalServer);
      writeTrustedServerRepository(
        localServerSortedSetTemp, this.trustedServerRepository);
    }
    catch (OperationFailureException e) {
      throw new ServiceFailureException(
        LanguageService.getInstance()
        .get("TRUSTED_SERVER_SERVICE_MODIFY_TRUSTED_SERVER_ERROR"), e);
    }
    this.updateCache(trustedServer, newLocalServer);
    this.fireWasModifiedTrustedServer(trustedServer, newLocalServer);
    return true;
  }
  
  public boolean addCert(String serverName, Certificate c) {
    CSKeyStore keyStore = CSKeyStore.getInstance();
    if (keyStore == null)
      throw new ServiceFailureException(LanguageService.getInstance().get(
        "TRUSTED_SERVER_SERVICE_KEYSTORE_NOT_FOUND"));
    return keyStore.addCertificate(serverName, c);
  }

  public void removeCert(String serverName) {
    CSKeyStore keyStore = CSKeyStore.getInstance();
    if (keyStore == null)
      throw new ServiceFailureException(LanguageService.getInstance().get(
        "TRUSTED_SERVER_SERVICE_KEYSTORE_NOT_FOUND"));
    keyStore.removeCertificate(serverName);
  }

  /**
   * Atualiza um servidor no cache de servidores confiveis.
   *
   * @param oldTrustedServer O servidor em seu estado antigo.
   * @param trustedServer O servidor em seu estado atual.
   */
  private void updateCache(TrustedServer oldTrustedServer, TrustedServer trustedServer) {
    this.trustedServerSortedSet.remove(oldTrustedServer);
    this.trustedServerSortedSet.add(trustedServer);
    this.trustedServerIdSortedMap.put((Integer)trustedServer.getId(), trustedServer);
    this.trustedServerNameMap.put(trustedServer.getName(), trustedServer);
  }

  /**
   * Envia aos observadores uma notificao de que um servidor foi
   * alterado.
   *
   * @param oldTrustedServer O servidor em seu estado antigo.
   * @param trustedServer O servidor em seu estado atual.
   */
  private void fireWasModifiedTrustedServer(
    TrustedServer oldTrustedServer, TrustedServer trustedServer) {
    this.setChanged();
    this.notifyObservers(
      new TrustedServerModifyEvent(this, oldTrustedServer, trustedServer));
  }

  /**
   * <p>
   * Envia aos observadores uma notificao de que um ou mais servidores
   * confiveis foram alterados.
   * </p>
   * <p>
   * OBS: a posio dos servidores nas listas tem que ser a mesma. Um servidor
   * que est na posio 1 da lista de estados antigos, deve estar na posio 1
   * da lista de estados novos.
   * </p>
   * 
   * @param trustedServerOldStateList Uma lista com os servidores locais que
   *        foram alterados, em seu estado antigo.
   * @param trustedServerNewStateList Uma lista com os servidores locais que
   *        foram alterados, em seu estado atual.
   * 
   * @throws IllegalArgumentException Caso o tamanho das duas colees seja
   *         diferent.
   */
  private void fireWasModifiedTrustedServer(
    List<TrustedServer> trustedServerOldStateList,
    List<TrustedServer> trustedServerNewStateList) {
    if (trustedServerOldStateList.size() != trustedServerNewStateList.size()) {
      throw new IllegalArgumentException(
        "A quantidade de servidor locais deve ser igual nas duas listas.");
    }
    Iterator<TrustedServer> localServerOldStateIterator =
      trustedServerOldStateList.iterator();
    Iterator<TrustedServer> localServerNewStateIterator =
      trustedServerNewStateList.iterator();
    while (localServerOldStateIterator.hasNext()) {
      TrustedServer oldLocalServer = localServerOldStateIterator.next();
      TrustedServer newLocalServer = localServerNewStateIterator.next();
      this.setChanged();
      this.notifyObservers(new TrustedServerModifyEvent(this, oldLocalServer,
        newLocalServer));
    }
  }

  /**
   * Carrega os servidores confiveis do repositrio de servidores confiveis.
   *
   * @param filePath O caminho para o arquivo do repositrio de servidores
   *        confiveis.
   *
   * @return Uma coleo com os servidores confiveis carregados.
   *
   * @throws ServerException Caso ocorra algum erro ao carregar os servidores
   *         confiveis.
   */
  private static Collection<TrustedServer> loadTrustedServerRepository(
    String filePath) throws ServerException {
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    factory.setIgnoringElementContentWhitespace(true);
    DocumentBuilder builder;
    try {
      builder = factory.newDocumentBuilder();
    }
    catch (ParserConfigurationException e) {
      throw new ServerException(
        "Erro ao criar leitor de documentos para a leitura do repositrio de servidores confiveis.",
        e);
    }
    Document document;
    try {
      document = builder.parse(new File(filePath));
    }
    catch (SAXException e) {
      throw new ServerException(
        "O arquivo do repositrio de servidores confiveis est invlido. Caminho {0}",
        new Object[] { filePath }, e);
    }
    catch (IOException e) {
      throw new ServerException(
        "O arquivo do repositrio de servidores confiveis est invlido. Caminho {0}",
        new Object[] { filePath }, e);
    }
    Element rootElement = document.getDocumentElement();
    NodeList childNodeList = rootElement.getChildNodes();
    List<TrustedServer> localServerList = new LinkedList<TrustedServer>();
    for (int i = 0; i < childNodeList.getLength(); i++) {
      Node node = childNodeList.item(i);
      if (node.getNodeType() == Node.ELEMENT_NODE) {
        Element localServerElement = (Element) node;
        Integer localServerId =
          new Integer(localServerElement.getAttribute(TRUSTED_SERVER_ID_TAG));
        String localServerName =
          localServerElement.getAttribute(TRUSTED_SERVER_NAME_TAG);
        Boolean isComplete =
          new Boolean(localServerElement
            .getAttribute(TRUSTED_SERVER_SUSPENDED_TAG));
        Boolean isLocal =
          new Boolean(localServerElement.getAttribute(TRUSTED_SERVER_LOCAL_TAG));
        String uriStr = localServerElement.getAttribute(TRUSTED_SERVER_URI_TAG);
        localServerList.add(new TrustedServer(localServerId, localServerName,
          isComplete.booleanValue(), isLocal.booleanValue(), ServerURI.parse(uriStr)));
      }
    }
    return localServerList;
  }

  /**
   * Grava o repositrio de servidores confiveis.
   * 
   * @param trustedServerCollection Uma coleo com os servidores confiveis que
   *        compem o repositrio.
   * @param filePath O caminho para o arquivo onde ser gravado o repositrio.
   * 
   * @throws OperationFailureException Caso ocorra algum problema ao gravar o
   *         repositrio.
   */
  private static void writeTrustedServerRepository(
    Collection<TrustedServer> trustedServerCollection, String filePath)
    throws OperationFailureException {
    final String INDENT_OUTPUT_PROPERTY = "indent";
    final String YES_VALUE = "yes";
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    DocumentBuilder builder;
    try {
      builder = factory.newDocumentBuilder();
    }
    catch (ParserConfigurationException e) {
      throw new OperationFailureException(
        "Erro ao criar gerador de documentos para a gravao do repositrio de servidores locais.",
        e);
    }
    Document document = builder.newDocument();
    Element rootElement = document.createElement(TRUSTED_SERVER_ROOT_TAG);
    Iterator<TrustedServer> trustedServerIterator =
      trustedServerCollection.iterator();
    while (trustedServerIterator.hasNext()) {
      TrustedServer trustedServer = trustedServerIterator.next();
      Element trustedServerElement = document.createElement(TRUSTED_SERVER_TAG);
      Attr idAttribute = document.createAttribute(TRUSTED_SERVER_ID_TAG);
      idAttribute.setValue(trustedServer.getId().toString());
      trustedServerElement.setAttributeNode(idAttribute);
      Attr nameAttribute = document.createAttribute(TRUSTED_SERVER_NAME_TAG);
      nameAttribute.setValue(trustedServer.getName());
      trustedServerElement.setAttributeNode(nameAttribute);
      Attr stateAttribute =
        document.createAttribute(TRUSTED_SERVER_SUSPENDED_TAG);
      stateAttribute.setValue(String.valueOf(trustedServer.isSuspended()));
      trustedServerElement.setAttributeNode(stateAttribute);
      Attr localAttribute = document.createAttribute(TRUSTED_SERVER_LOCAL_TAG);
      localAttribute.setValue(String.valueOf(trustedServer.isLocal()));
      trustedServerElement.setAttributeNode(localAttribute);
      Attr uriAttribute = document.createAttribute(TRUSTED_SERVER_URI_TAG);
      uriAttribute.setValue(trustedServer.getURI().toString());
      trustedServerElement.setAttributeNode(uriAttribute);
      rootElement.insertBefore(trustedServerElement, null);
    }
    document.appendChild(rootElement);
    try {
      TransformerFactory transformerFactory = TransformerFactory.newInstance();
      Transformer transformer = transformerFactory.newTransformer();
      Source input = new DOMSource(document);
      Result output = new StreamResult(new File(filePath));
      transformer.setOutputProperty(INDENT_OUTPUT_PROPERTY, YES_VALUE);
      transformer.transform(input, output);
    }
    catch (TransformerException e) {
      throw new OperationFailureException(
        "Erro ao gerar o repositrio de servidores confiveis.", e);
    }
  }

  /**
   * Obtm um servidor confivel a partir do seu nome.
   *
   * @param trustedServerName O nome do servidor confivel.
   *
   * @return O servidor, ou null, caso no exista.
   *
   * @throws OperationFailureException Caso a operao falhe.
   */
  public TrustedServer getTrustedServer(String trustedServerName)
  throws OperationFailureException {
    this.updateTrustedServers();
    return this.trustedServerNameMap.get(trustedServerName);
  }
}
