/*
 * $Id: WIOServerFile.java 102529 2010-03-04 19:53:36Z rodrigoh $
 */
package csbase.server.services.wioservice;

import java.io.File;
import java.io.RandomAccessFile;
import java.util.Calendar;

import org.omg.PortableServer.POA;

import tecgraf.javautils.FileUtils;
import csbase.server.services.projectservice.ProjectService;
import csbase.server.services.projectservice.ServerProjectFile;
import csbase.server.services.projectservice.UpdatableFileInfo;
import csbase.server.services.wioservice.idl.BytesHolder;
import csbase.server.services.wioservice.idl.SeekType;
import csbase.server.services.wioservice.idl.WIODateTimeInfo;
import csbase.server.services.wioservice.idl.WIOFile;
import csbase.server.services.wioservice.idl.WIOFileHelper;
import csbase.server.services.wioservice.idl.WIOFileMode;
import csbase.server.services.wioservice.idl.WIOFilePOA;
import csbase.server.services.wioservice.idl.WIOProject;
import csbase.server.services.wioservice.idl.WIOProjectHelper;
import csbase.server.services.wioservice.idl.WIOServiceException;

/**
 * A classe <code>WIOServerFile</code> implementa o <i>servant</i> associado 
 * interface idl <code>WIOFile</code>.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class WIOServerFile extends WIOFilePOA {
  /**
   * Arquivo fsico no file system do servidor
   */
  private File javaFile = null;

  /**
   * Projeto relacionado ao arquivo/diretrio
   */
  private WIOServerProject serverProject = null;

  /**
   * Path relacionado ao arquivo/diretrio
   */
  private String serverPath = null;

  /**
   * Modo corrente do arquivo
   */
  private WIOFileMode fileMode = null;

  /**
   * Arquivo de acesso ao file system do servidor
   */
  RandomAccessFile randomFile = null;

  /**
   * Mtodo de ativao do objeto CORBA WIOFile.
   * 
   * @return .
   * @throws Exception em caso de erro.
   */
  protected POA activateCorbaFile() throws Exception {
    final WIOService wioService = WIOService.getInstance();
    return wioService.activateCorbaObject(null, this);
  }

  /**
   * Adiciona um texto  descrio atual do arquivo.
   * 
   * @param description o texto a ser adicionado  descrio do arquivo.
   * @throws WIOServiceException em caso de erro.
   */
  public void appendDescription(final String description)
    throws WIOServiceException {
    if (serverProject == null) {
      throw new WIOServiceException("Projeto nulo em [" + this.toString() + "]");
    }
    try {
      serverProject.appendDescription(serverPath, description);
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Constri a representao de uma data.
   * 
   * @param l a data a ser representada, como um nmero de milisegundos
   * @return a representao da data, como um objeto do tipo
   *         <code>WIODateTimeInfo</code>.
   * 
   */
  private WIODateTimeInfo buildDateTime(final long l) {
    final Calendar cal = Calendar.getInstance();
    cal.setTimeInMillis(l);
    final WIODateTimeInfo dateTime =
      new WIODateTimeInfo(cal.get(Calendar.DAY_OF_MONTH), cal
        .get(Calendar.MONTH) + 1, cal.get(Calendar.YEAR), cal
        .get(Calendar.HOUR_OF_DAY), cal.get(Calendar.MINUTE), cal
        .get(Calendar.SECOND));
    return dateTime;
  }

  /**
   * Verifica se o arquivo pode ser lido.
   * 
   * @return true se o arquivo pode ser lido, false caso contrrio
   * @throws WIOServiceException em caso de erro.
   */
  public boolean canRead() throws WIOServiceException {
    if (javaFile == null) {
      throw new WIOServiceException("Arquivo fisico nulo!");
    }
    return javaFile.canRead();
  }

  /**
   * Verifica se o arquivo pode ser escrito.
   * 
   * @return true se o arquivo pode ser escrito, false caso contrrio
   * @throws WIOServiceException em caso de erro.
   */
  public boolean canWrite() throws WIOServiceException {
    if (javaFile == null) {
      throw new WIOServiceException("Arquivo fisico nulo!");
    }
    return javaFile.canWrite();
  }

  /**
   * Fecha o arquivo fsico.
   * 
   * @throws WIOServiceException em caso de erro.
   */
  public void close() throws WIOServiceException {
    if (fileMode == WIOFileMode.NOT_OPENED) {
      throw new WIOServiceException("Arquivo nao esta aberto.");
    }
    try {
      if (fileMode == WIOFileMode.MODE_WRITE) {
        serverProject.setUnderConstruction(serverPath, false);
      }
      randomFile.close();
      randomFile = null;
      logInfo("Arquivo fechado: " + javaFile.getPath());
      logInfo("Agendando atualizao do arquivo: " + javaFile.getPath());
      final ServerProjectFile spf = this.getServerFile();
      final UpdatableFileInfo ufi = spf.getUpdatableFileInfo();
      if (ufi != null) {
        logInfo("Ativando atualizao pelo usurio "
          + this.serverProject.getLoggedUserId());
        ProjectService.getInstance().singleUpdate(
          this.serverProject.getLoggedUserId(), serverProject.getUser(), spf);
      }
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Obtm a data de criao do arquivo.
   * 
   * @return a data de criao do arquivo, representada por um objeto
   *         <code>WIODateTimeInfo</code>
   * @throws WIOServiceException em caso de erro.
   */
  public WIODateTimeInfo creationTime() throws WIOServiceException {
    try {
      final ServerProjectFile serverFile = getServerFile();
      final long dt = serverFile.getCreationDate();
      return buildDateTime(dt);
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Mtodo de desativao do objeto CORBA WIOFile.
   * 
   * @throws Exception em caso de erro.
   */
  protected void deactivateCorbaFile() throws Exception {
    final WIOService wioService = WIOService.getInstance();
    wioService.deactivateCorbaObject(this);
  }

  /**
   * Desativa o objeto objeto remoto, liberando os recursos associados .
   * 
   * @throws WIOServiceException em caso de erro.
   */
  public void destroy() throws WIOServiceException {
    try {
      deactivateCorbaFile();
      serverProject.decNumFiles();
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Verifica se o arquivo est posicionado no fim.
   * 
   * @return true se arquivo posicionado no fim, false caso contrrio.
   * @throws WIOServiceException em caso de erro.
   */
  public boolean eof() throws WIOServiceException {
    final String myName = "[" + this.toString() + "]";
    if (fileMode == WIOFileMode.NOT_OPENED) {
      final String err = "Falha de eof por arquivo fechado em " + myName;
      logSevere(err);
      throw new WIOServiceException(err);
    }
    if (randomFile == null) {
      final String err = "Falha de eof por random-file nulo em " + myName;
      logSevere(err);
      throw new WIOServiceException(err);
    }
    try {
      return randomFile.getFilePointer() >= randomFile.length();
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Obtm a descrio associada a um arquivo.
   * 
   * @return a descrio do arquivo
   * @throws WIOServiceException em caso de erro.
   */
  public String getDescription() throws WIOServiceException {
    if (serverProject == null) {
      throw new WIOServiceException("Projeto nulo em [" + this.toString() + "]");
    }
    try {
      final String desc = serverProject.getDescription(serverPath);
      return desc.replaceAll("[^\\p{ASCII}]", "?");
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Obtm o i-simo objeto (arquivo ou diretrio) contido neste WIOFile.
   * 
   * @param i o ndice do objeto desejado. Esse ndice deve estar entre 0 e N-1
   *        (nmero mximo dado por <code>getNumFiles</code>).
   * @return o arquivo
   * @throws WIOServiceException em caso de erro.
   */
  public WIOFile getFile(final int i) throws WIOServiceException {
    if (serverPath == null) {
      throw new WIOServiceException("Path nulo detectado!");
    }
    if (serverProject == null) {
      throw new WIOServiceException("Projeto nulo detectado!");
    }
    File[] children = null;
    try {
      final ProjectService prjService = serverProject.getProjectService();
      final String uid = serverProject.getUser();
      final String pid = serverProject.getId();
      children = prjService.getChildrenFiles(uid, pid, serverPath);
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }

    if (children == null) {
      throw new WIOServiceException("Array de filhos nulo detectado!");
    }
    if ((i < 0) || (i >= children.length)) {
      throw new WIOServiceException("Indice invalido para array de filhos!");
    }

    String childPath = null;
    try {
      final ProjectService prjService = serverProject.getProjectService();
      final String uid = serverProject.getUser();
      final String pid = serverProject.getId();
      final File child = children[i];
      childPath = prjService.fileToPath(uid, pid, child);
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
    if (childPath == null) {
      final String msg = "Falha na aquisicao de path arquivo-filho";
      logSevere(msg);
      throw new WIOServiceException(msg);
    }
    WIOFile corbaFile = null;
    try {
      final WIOServerFile sonFile = new WIOServerFile(serverProject, childPath);
      final POA poa = sonFile.activateCorbaFile();
      corbaFile = WIOFileHelper.narrow(poa.servant_to_reference(sonFile));
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }

    if (corbaFile == null) {
      final String msg = "Falha na aquisicao de arquivo-filho";
      throw new WIOServiceException(msg);
    }
    return corbaFile;
  }

  /**
   * Obtm o tipo do arquivo.
   * 
   * @return uma String que identifica o tipo do arquivo.
   * @throws WIOServiceException em caso de erro.
   */
  public String getFileType() throws WIOServiceException {
    if (serverPath == null) {
      throw new WIOServiceException("Path nulo detectado!");
    }
    if (serverProject == null) {
      throw new WIOServiceException("Projeto nulo detectado!");
    }
    try {
      final ProjectService prjService = serverProject.getProjectService();
      final String uid = serverProject.getUser();
      final String pid = serverProject.getId();
      return prjService.getFileType(uid, pid, serverPath);
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Busca do maior valor possvel para uma leitura (em bytes)
   * 
   * @return o tamanho mximo da leitura em bytes.
   */
  private int getMaxRead() {
    try {
      final WIOService wioService = WIOService.getInstance();
      return wioService.getMaxReadKb() * 1024;
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      return 0;
    }
  }

  /**
   * Busca do maior valor possvel para uma escrita (em bytes)
   * 
   * @return o tamanho mximo da escrita em bytes.
   */
  private int getMaxWrite() {
    try {
      final WIOService wioService = WIOService.getInstance();
      return wioService.getMaxWriteKb() * 1024;
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      return 0;
    }
  }

  /**
   * Obtm o modo no qual o arquivo se encontra.
   * 
   * @return um objeto WIOFileMode que representa o modo atual do arquivo
   *         (NOT_OPENED, MODE_READ, MODE_WRITE)
   * @throws WIOServiceException em caso de erro.
   */
  public WIOFileMode getMode() throws WIOServiceException {
    if (javaFile == null) {
      throw new WIOServiceException("Arquivo fisico nulo!");
    }
    return fileMode;
  }

  /**
   * Obtm o nmero de objetos (arquivos ou diretrios) contidos neste
   * <code>WIOFile</code>.
   * 
   * @return o nmero de arquivos/diretriosi contidos, ou 0 (zero) se este
   *         WIOFile representa um arquivo
   * @throws WIOServiceException em caso de erro.
   */
  public int getNumFiles() throws WIOServiceException {
    if (serverPath == null) {
      throw new WIOServiceException("Path nulo detectado!");
    }
    if (serverProject == null) {
      throw new WIOServiceException("Projeto nulo detectado!");
    }
    File[] children = null;
    try {
      final ProjectService prjService = serverProject.getProjectService();
      final String uid = serverProject.getUser();
      final String pid = serverProject.getId();
      children = prjService.getChildrenFiles(uid, pid, serverPath);
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }

    if (children == null) {
      throw new WIOServiceException("Array de filhos nulo detectado!");
    }
    return children.length;
  }

  /**
   * Obtm o diretrio que contm este arquivo.
   * 
   * @return o objeto <code>WIOFile</code> que representa o diretrio "pai"
   * @throws WIOServiceException em caso de erro.
   */
  public WIOFile getParent() throws WIOServiceException {
    final String[] splitPath = FileUtils.splitPath(getPath());
    String parentPath = "";
    final int limit = splitPath.length - 1;
    for (int i = 0; i < limit; i++) {
      parentPath = parentPath + splitPath[i];
      if (i != (limit - 1)) {
        parentPath = parentPath + File.separator;
      }
    }
    WIOFile corbaFile = null;
    try {
      final WIOServerFile parentFile =
        new WIOServerFile(serverProject, parentPath);
      final POA poa = parentFile.activateCorbaFile();
      corbaFile = WIOFileHelper.narrow(poa.servant_to_reference(parentFile));
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }

    if (corbaFile == null) {
      final String err = "Falha de narrow acesso : " + this.toString();
      logSevere(err);
      throw new WIOServiceException(err);
    }
    return corbaFile;
  }

  /**
   * Obtm o caminho (path) deste arquivo.
   * 
   * @return o path como uma string.
   * @throws WIOServiceException em caso de erro.
   */
  public String getPath() throws WIOServiceException {
    if (serverPath == null) {
      throw new WIOServiceException("Path nulo!");
    }
    return serverPath;
  }

  /**
   * Obtm o projeto ao qual pertence o arquivo.
   * 
   * @return um objeto <code>WIOProject</code> que representa o projeto que
   *         contm o arquivo
   * @throws WIOServiceException em caso de erro.
   */
  public WIOProject getProject() throws WIOServiceException {
    WIOProject corbaPrj = null;
    try {
      final POA poa = this.activateCorbaFile();
      final WIOServerProject wsp = serverProject;
      corbaPrj = WIOProjectHelper.narrow(poa.servant_to_reference(wsp));
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }

    if (corbaPrj == null) {
      final String err = "Falha de narrow de projeto : " + this.toString();
      logSevere(err);
      throw new WIOServiceException(err);
    }
    return corbaPrj;
  }

  /**
   * Obtm a representao deste arquivo como um arquivo contido em um projeto
   * do servidor.
   * 
   * @return um objeto <code>ServerProjectFile<code> que representa o arquivo
   * @throws WIOServiceException em caso de erro.
   */
  public ServerProjectFile getServerFile() throws WIOServiceException {
    final String myName = "[" + this.toString() + "]";
    try {
      final ProjectService prjService = serverProject.getProjectService();
      final String uid = serverProject.getUser();
      final String pid = serverProject.getId();
      final String path = this.serverPath;
      final ServerProjectFile serverFile =
        prjService.getServerFile(uid, pid, path);
      if (serverFile == null) {
        String err = "Falha de getServerFile(). Arquivo servidor nao existe: ";
        err = err + myName;
        logSevere(err);
        throw new WIOServiceException(err);
      }
      return serverFile;
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Verifica se  um diretrio.
   * 
   * @return true se  um diretrio, false caso contrrio
   * @throws WIOServiceException em caso de erro.
   */
  public boolean isDirectory() throws WIOServiceException {
    if (javaFile == null) {
      throw new WIOServiceException("Arquivo fisico nulo!");
    }
    return javaFile.isDirectory();
  }

  /**
   * Verifica se este arquivo est publicado.
   * 
   * @return true se este  um arquivo publicado, false caso contrrio
   * @throws WIOServiceException em caso de erro.
   */
  public boolean isPublished() throws WIOServiceException {
    try {
      final ServerProjectFile serverFile = getServerFile();
      return serverFile.isPublishedFile();
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Verifica se este arquivo est em construo.
   * 
   * @return true se o arquivo est em construo, false caso contrrio
   * @throws WIOServiceException em caso de erro.
   */
  public boolean isUnderConstruction() throws WIOServiceException {
    try {
      final ServerProjectFile serverFile = getServerFile();
      return serverFile.isUnderConstruction();
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Obtm a data de ltima modificao do arquivo.
   * 
   * @return a data de modificao do arquivo, representada por um objeto
   *         <code>WIODateTimeInfo</code>
   * @throws WIOServiceException em caso de erro.
   */
  public WIODateTimeInfo lastModificationTime() throws WIOServiceException {
    try {
      final ServerProjectFile serverFile = getServerFile();
      final long dt = serverFile.getModificationDate();
      return buildDateTime(dt);
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Mtodo de logging no servio de WIO.
   * 
   * @param msg a mensagem
   */
  private void logInfo(final String msg) {
    try {
      final WIOService wioService = WIOService.getInstance();
      if (wioService != null) {
        wioService.logInfoMessage(msg);
      }
    }
    catch (final Exception e) {
      System.out.println(msg);
      System.out.println(e.getMessage());
      e.printStackTrace();
    }
  }

  /**
   * Mtodo de logging no servio de WIO.
   * 
   * @param msg a mensagem
   */
  private void logSevere(final String msg) {
    try {
      final WIOService wioService = WIOService.getInstance();
      if (wioService != null) {
        wioService.logSevereMessage(msg);
      }
    }
    catch (final Exception e) {
      System.out.println(msg);
      System.out.println(e.getMessage());
      e.printStackTrace();
    }
  }

  /**
   * Abre o arquivo.
   * 
   * @param mode o modo no qual o arquivo deve ser aberto (leitura ou escrita)
   * @throws WIOServiceException em caso de erro.
   */
  public void open(final WIOFileMode mode) throws WIOServiceException {
    if (fileMode != WIOFileMode.NOT_OPENED) {
      throw new WIOServiceException("Arquivo ja esta aberto.");
    }
    if (mode == WIOFileMode.NOT_OPENED) {
      throw new WIOServiceException("Modo invalido para abertura de arquivo.");
    }
    try {
      if (mode == WIOFileMode.MODE_READ) {
        fileMode = mode;
        randomFile = new RandomAccessFile(javaFile, "r");
        logInfo("Arquivo aberto para leitura: " + javaFile.getPath());
      }
      else if (mode == WIOFileMode.MODE_WRITE) {
        fileMode = mode;
        randomFile = new RandomAccessFile(javaFile, "rws");
        serverProject.setUnderConstruction(serverPath, true);
        logInfo("Arquivo aberto para escrita: " + javaFile.getPath());
      }
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Mtodo de leitura remota.
   * 
   * @param numBytes nmero de bytes a serem lidos
   * @param holder o buffer de leitura (usado na transferncia dos dados via
   *        CORBA)
   * @return o nmero de bytes efetivamente lidos
   * @throws WIOServiceException em caso de erro.
   */
  public int read(final int numBytes, final BytesHolder holder)
    throws WIOServiceException {
    final String myName = "[" + this.toString() + "]";
    int size = numBytes;
    if (fileMode != WIOFileMode.MODE_READ) {
      final String err = "Leitura por arquivo fora do modo READ : " + myName;
      throw new WIOServiceException(err);
    }
    if (size <= 0) {
      final String err = "Tamanho de READ negativo! : " + myName;
      throw new WIOServiceException(err);
    }
    if (randomFile == null) {
      final String err = "Falha em READ por random-file nulo! : " + myName;
      throw new WIOServiceException(err);
    }
    if (size > getMaxRead()) {
      logSevere("Tamanho de READ fora do limite: " + myName + " - "
        + "truncando de " + size + " para " + getMaxRead());
      size = getMaxRead();
    }
    try {
      final long pos = randomFile.getFilePointer();
      final long len = randomFile.length();
      int nbytes = size;
      if ((pos + size) > len) {
        nbytes = (int) (len - pos);
        logInfo("Tamanho de READ maior que resto do arquivo: " + myName + " - "
          + "truncando de " + size + " para " + nbytes + " " + "(cursor=" + pos
          + " tamanho=" + len + ")");
      }
      final byte[] buffer = new byte[nbytes];
      randomFile.readFully(buffer);
      holder.value = buffer;
      return nbytes;
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Redimensiona o arquivo.
   * 
   * @param size o novo tamanho do arquivo (em bytes)
   * @throws WIOServiceException em caso de erro.
   */
  public void resize(final long size) throws WIOServiceException {
    final String myName = "[" + this.toString() + "]";
    if (randomFile == null) {
      final String err = "Falha em READ por random-file nulo! : " + myName;
      throw new WIOServiceException(err);
    }
    try {
      randomFile.setLength(size);
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Estabelece a posio corrente do arquivo. Essa nova posio  determinada
   * pela soma de um deslocamento (offset) a uma posio "base" (inicio do
   * arquivo, posio corrente, fim do arquivo).
   * 
   * @param offset o deslocamento a ser somado  posio "base"
   * @param type a posio "base", representada por um objeto do tipo SeekType
   * @throws WIOServiceException em caso de erro.
   */
  public void seek(final long offset, final SeekType type)
    throws WIOServiceException {
    final String myName = "[" + this.toString() + "]";
    if (randomFile == null) {
      final String err = "Falha de seek por random-file nulo em " + myName;
      logSevere(err);
      throw new WIOServiceException(err);
    }
    try {
      long position = 0;
      if (type == SeekType.SEEK_TYPE_SET) {
        position = offset;
      }
      else if (type == SeekType.SEEK_TYPE_CUR) {
        final long pos = randomFile.getFilePointer();
        position = pos + offset;
      }
      else if (type == SeekType.SEEK_TYPE_END) {
        final long length = randomFile.length();
        position = length + offset;
      }
      randomFile.seek(position);
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Associa uma a descrio ao arquivo.
   * 
   * @param desc a descrio do arquivo
   * @throws WIOServiceException em caso de erro.
   */
  public void setDescription(final String desc) throws WIOServiceException {
    if (serverProject == null) {
      throw new WIOServiceException("Projeto nulo em [" + this.toString() + "]");
    }
    try {
      serverProject.setDescription(serverPath, desc);
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Obtm o tamanho do arquivo.
   * 
   * @return o tamanho do arquivo, em nmero de bytes
   * @throws WIOServiceException em caso de erro.
   */
  public long size() throws WIOServiceException {
    final String myName = "[" + this.toString() + "]";
    if (javaFile == null) {
      String err = "Falha de size(). Arquivo fisico nao existe: ";
      err = err + myName;
      logSevere(err);
      throw new WIOServiceException(err);
    }
    if (isDirectory()) {
      String err = "Falha de size(). Pedido sobre um diretorio: ";
      err = err + myName;
      logSevere(err);
      throw new WIOServiceException(err);
    }
    try {
      return javaFile.length();
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Obtm a posio corrente do arquivo.
   * 
   * @return a posio corrente do arquivo, representada como um deslocamento
   *         (nmero de bytes) em relao ao incio do arquivo
   * @throws WIOServiceException em caso de erro.
   */
  public long tell() throws WIOServiceException {
    final String myName = "[" + this.toString() + "]";
    if (randomFile == null) {
      final String err = "Falha de eof por random-file nulo em " + myName;
      logSevere(err);
      throw new WIOServiceException(err);
    }
    try {
      return randomFile.getFilePointer();
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String toString() {
    String uid = null;
    String pid = null;
    try {
      uid = serverProject.getUser();
      pid = serverProject.getId();
    }
    catch (final Exception e) {
      uid = "<usurio nulo>";
      pid = "<projeto nulo>";
    }
    return "[WIO File]: " + uid + "@" + pid + ":" + serverPath;
  }

  /**
   * Obtm a identificao do usurio que criou o arquivo.
   * 
   * @return a identificao do usurio criador do arquivo
   * @throws WIOServiceException em caso de erro.
   */
  public String whoCreated() throws WIOServiceException {
    try {
      final ServerProjectFile serverFile = getServerFile();
      final java.lang.Object obj = serverFile.whoCreated();
      if (obj == null) {
        return "?";
      }
      return obj.toString();
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Mtodo de escrita remota.
   * 
   * @param content contedo (sequncia de bytes) a ser escrito a partir da
   *        posio corrente.
   * @return o nmero de bytes escritos efetivamente
   * @throws WIOServiceException em caso de erro.
   */
  public int write(final byte[] content) throws WIOServiceException {
    final String myName = "[" + this.toString() + "]";
    if (fileMode != WIOFileMode.MODE_WRITE) {
      final String err = "Leitura por arquivo fora do modo WRITE : " + myName;
      throw new WIOServiceException(err);
    }
    if (randomFile == null) {
      final String err = "Falha em READ por random-file nulo! : " + myName;
      throw new WIOServiceException(err);
    }
    int size = content.length;
    if (size > getMaxWrite()) {
      logSevere("Tamanho de WRITE fora do limite: " + myName + " - "
        + "truncando de " + size + " para " + getMaxWrite());
      size = getMaxWrite();
    }
    try {
      randomFile.write(content, 0, size);
      return size;
    }
    catch (final Exception e) {
      final String err = e.getMessage();
      logSevere(err);
      e.printStackTrace();
      throw new WIOServiceException(WIOService.getExceptionString(e));
    }
  }

  /**
   * Constri o objeto que representa um arquivo remoto,
   * 
   * @param serverProject o projeto que contm o arquivo
   * @param serverPath o path do arquivo
   * @throws Exception em caso de falha.
   */
  protected WIOServerFile(final WIOServerProject serverProject,
    final String serverPath) throws Exception {
    if (serverProject == null) {
      throw new Exception("Projeto/usuario nao identificado! ");
    }
    if (serverPath == null) {
      throw new Exception("Arquivo nao identificado!");
    }
    this.serverProject = serverProject;
    this.serverPath = serverPath;
    this.fileMode = WIOFileMode.NOT_OPENED;
    final ProjectService prjService = serverProject.getProjectService();
    final String uid = serverProject.getUser();
    final String pid = serverProject.getId();
    try {
      javaFile = prjService.getFile(uid, pid, serverPath);
    }
    catch (final Exception e) {
      prjService.logSevereMessage("Falha de acesso pelo WIO - " + "getFile("
        + uid + "," + pid + "," + serverPath + ") -> " + e.getMessage());
      e.printStackTrace();
      throw new Exception(e.getMessage());
    }
    if (javaFile == null) {
      final String prjPath = serverProject.toString();
      throw new Exception("Arquivo fisico no projeto nao existe: " + "["
        + prjPath + "]" + serverPath);
    }
    if (!javaFile.exists()) {
      final String path = javaFile.getAbsolutePath();
      throw new Exception("Arquivo fisico nao existe: " + path);
    }
    serverProject.incNumFiles();
  }
}
