/**
 * $Id: AbstractServlet.java 105726 2010-05-18 18:51:33Z leonardossz $
 */
package csbase.servlet;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Classe bsica para os servlets. Carrega a cada request o arquivo de
 * propriedades <code>WEB-INF/System.properties</code>, e disponibiliza acesso
 * s propriedades atravs de getters.
 * <p>
 * O arquivo <code>System.properties</code> deve ser copiado para o diretrio
 * <code>WEB-INF</code> pelo script de inicializao do sistema.
 */
abstract class AbstractServlet extends HttpServlet {

  /**
   * Diretrio dentro de <code>tomcat/webapps/[sistema]</code> para onde ser
   * copiado o {@link #PROPERTIES_FILE arquivo de propriedades}.
   */
  protected static final String PROPERTIES_DIR = "/WEB-INF";
  /**
   * Arquivo com as propriedades do sistema, mantido pelo administrador. Este
   * arquivo  copiado para dentro do diretrio {@link #PROPERTIES_DIR} pelo
   * script de inicializao do servidor.
   */
  protected static final String PROPERTIES_FILE = "System.properties";
  /**
   * Nome da propriedade que contm o nome do servidor.
   * 
   * @see #PROP_SERVER_HOST_ADDR
   */
  private static final String PROP_SERVER_HOST_NAME = "Server.hostName";
  /**
   * Nome da propriedade que contm a porta RMI do servidor.
   */
  private static final String PROP_SERVER_PORT_RMI = "Server.rmiPort";
  /**
   * Nome da propriedade que contm o endereo IP do servidor.
   * 
   * @see #PROP_SERVER_HOST_NAME
   */
  private static final String PROP_SERVER_HOST_ADDR = "Server.hostAddr";
  /**
   * Porta RMI default do servidor.
   */
  private static final String DEFAULT_RMI_PORT = "1099";
  /**
   * Propriedades definidas no arquivo {@link #PROPERTIES_FILE}.
   */
  private Properties systemProperties;

  @Override
  public void init() throws ServletException {
    try {
      String propsPath = PROPERTIES_DIR + '/' + PROPERTIES_FILE;
      InputStream stream = getServletContext().getResourceAsStream(propsPath);
      if (stream == null) {
        throw new IOException("Arquivo de propriedades " + propsPath
          + " no foi encontrado no servidor");
      }
      systemProperties = new Properties();
      systemProperties.load(stream);
    }
    catch (Exception e) {
      throw new ServletException(e);
    }
  }

  /**
   * Retorna o valor associado  propriedade.
   * 
   * @param key nome da propriedade
   * @return valor associado  propriedade, ou <code>null</code> caso esta no
   *         tenha sido definida.
   */
  protected String getProperty(String key) {
    return systemProperties.getProperty(key);
  }

  /**
   * Retorna o valor associado  propriedade ou um valor default, caso esta no
   * tenha sido definida.
   * 
   * @param key nome da propriedade
   * @param defaultValue valor default
   * 
   * @return valor associado  propriedade ou o valor default caso esta no
   *         tenha sido definida
   */
  protected String getProperty(String key, String defaultValue) {
    return systemProperties.getProperty(key, defaultValue);
  }

  /**
   * Obtm o path RMI para o servidor no formato
   * <code>rmi://IP_SERVIDOR:PORTA_RMI/</code> .
   * 
   * @return path RMI para o servidor
   * 
   * @see #getServerAddress()
   * @see #getServerRMIport()
   */
  protected String getRMIpath() {
    return "rmi://" + getServerAddress() + ':' + getServerRMIport() + '/';
  }

  /**
   * Obtm o nome do servidor. Obtido da propriedade
   * {@value #PROP_SERVER_HOST_NAME}.
   * 
   * @return nome do servidor
   */
  protected String getServerName() {
    return getProperty(PROP_SERVER_HOST_NAME);
  }

  /**
   * Obtm o endereo IP do servidor. Obtido da propriedade
   * {@value #PROP_SERVER_HOST_ADDR}.
   * 
   * @return endereo IP do servidor
   */
  protected String getServerAddress() {
    return getProperty(PROP_SERVER_HOST_ADDR);
  }

  /**
   * Obtm a porta RMI para comunicao com o servidor. Obtido da propriedade
   * {@value #PROP_SERVER_PORT_RMI}. Como esta propriedade pode no ter sido
   * definida, temos um fallback para o default {@value #DEFAULT_RMI_PORT}.
   * 
   * @return porta para comunicao RMI com o servidor
   */
  protected String getServerRMIport() {
    return getProperty(PROP_SERVER_PORT_RMI, DEFAULT_RMI_PORT);
  }

  /**
   * Efetivamente processa o GET.
   * 
   * @param req solicitao feita ao servidor
   * @param resp resposta do servidor
   * @throws ServletException se o GET no foi tratado
   * @throws IOException se houve um erro de I/O durante o processamento do GET
   */
  @Override
  protected abstract void doGet(HttpServletRequest req, HttpServletResponse resp)
    throws IOException, ServletException;
}
