package csbase.client.algorithms.flows.graph.utils;

import static org.junit.Assert.*;

import java.util.HashMap;
import java.util.Map;

import javax.swing.JFrame;

import org.junit.Before;
import org.junit.Test;

import tecgraf.javautils.configurationmanager.ConfigurationManager;
import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.graph.GraphNode;
import csbase.algorithms.flows.graph.utils.CopyAndPasteOperation;
import csbase.algorithms.flows.graph.utils.OperationStatus;
import csbase.client.Client;
import csbase.client.algorithms.parameters.SimpleAlgorithmConfiguratorView;
import csbase.client.algorithms.parameters.ParameterView.Mode;
import csbase.client.kernel.ClientException;
import csbase.exception.algorithms.ParameterNotFoundException;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.SimpleAlgorithmConfiguratorFactory;
import csbase.logic.algorithms.parameters.ParameterGroup;
import csbase.logic.algorithms.parameters.SimpleAlgorithmConfigurator;
import csbase.logic.algorithms.parameters.TextParameter;
import csbase.test.unit.TestConfig;
import csbase.test.unit.mock.MockCSBaseClient;

/**
 * 
 * Testes para a operao de copiar/colar parmetros no editor de fluxos.
 * 
 * @author isabella
 */
public class CopyAndPasteOperationTestTemporaryDisabled {

  /**
   * Cria instncias de configuration manager e do cliente, necessrios para
   * os testes da classe.
   */
  @Before
  public void setup() {
    try {
      ConfigurationManager.getInstance();
    }
    catch (Exception e) {
      ConfigurationManager.createInstance();
    }

    try {
      Client.getInstance();
    }
    catch (Exception e) {
      String server =
        TestConfig.getProperty(TestConfig.SERVER_PROP, "csbase://localhost");
      Map<String, String> params = new HashMap<String, String>();
      params.put(Client.SERVER_URL_PARAMETER, server);
      new MockCSBaseClient("Name", params);
    }

  }

  @Test
  public void testCopyWithNoParametersWhenUserConfirms() throws ClientException {
    SimpleAlgorithmConfigurator fromConfigurator = createSimpleConfigurator(1);
    SimpleAlgorithmConfigurator toConfigurator = createSimpleConfigurator(2);

    assertCopySuccess(fromConfigurator, toConfigurator, true);
  }

  @Test
  public void testCopyWithNoParametersWhenUserCancels() throws ClientException {
    SimpleAlgorithmConfigurator fromConfigurator = createSimpleConfigurator(1);
    SimpleAlgorithmConfigurator toConfigurator = createSimpleConfigurator(2);

    assertCopyFailure(fromConfigurator, toConfigurator, false);
  }

  @Test
  public void testCopyWithSameParametersWhenUserCancels()
    throws ClientException, ParameterNotFoundException {
    SimpleAlgorithmConfigurator fromConfigurator = createSimpleConfigurator(1);
    SimpleAlgorithmConfigurator toConfigurator = createSimpleConfigurator(2);
    ParameterGroup fromGroup = new ParameterGroup("from_group");
    ParameterGroup toGroup = new ParameterGroup("to_group");

    String paramName = "text_param_name";

    createParameter(fromGroup, paramName, "from_text_param_value");
    createParameter(toGroup, paramName, "to_text_param_value");

    fromConfigurator.addGroup(fromGroup);
    toConfigurator.addGroup(toGroup);

    assertCopyFailure(fromConfigurator, toConfigurator, false);

    String parameterValue = toConfigurator.getParameterValue(paramName);
    assertEquals("to_text_param_value", parameterValue);
  }

  @Test
  public void testCopyWithSameParametersWhenUserConfirms()
    throws ClientException, ParameterNotFoundException {
    SimpleAlgorithmConfigurator fromConfigurator = createSimpleConfigurator(1);
    SimpleAlgorithmConfigurator toConfigurator = createSimpleConfigurator(2);
    ParameterGroup fromGroup = new ParameterGroup("from_group");
    ParameterGroup toGroup = new ParameterGroup("to_group");

    String paramName = "text_param_name";

    createParameter(fromGroup, paramName, "from_text_param_value");
    createParameter(toGroup, paramName, "to_text_param_value");
    fromConfigurator.addGroup(fromGroup);
    toConfigurator.addGroup(toGroup);

    assertCopySuccess(fromConfigurator, toConfigurator, true);

    String parameterValue = toConfigurator.getParameterValue(paramName);
    assertEquals("from_text_param_value", parameterValue);
  }

  @Test
  public void testCopyWithDifferentParameterNamesWhenUserCancels()
    throws ClientException, ParameterNotFoundException {
    SimpleAlgorithmConfigurator fromConfigurator = createSimpleConfigurator(1);
    SimpleAlgorithmConfigurator toConfigurator = createSimpleConfigurator(2);
    ParameterGroup fromGroup = new ParameterGroup("from_group");
    ParameterGroup toGroup = new ParameterGroup("to_group");
    createParameter(fromGroup, "from_text_param_name", "from_text_param_value");
    createParameter(toGroup, "to_text_param_name", "to_text_param_value");
    fromConfigurator.addGroup(fromGroup);
    toConfigurator.addGroup(toGroup);

    assertCopyFailure(fromConfigurator, toConfigurator, false);

    String parameterValue =
      toConfigurator.getParameterValue("to_text_param_name");
    assertEquals("to_text_param_value", parameterValue);

  }

  @Test
  public void testCopyWithDifferentParameterNamesWhenUserComfirms()
    throws ClientException, ParameterNotFoundException {
    SimpleAlgorithmConfigurator fromConfigurator = createSimpleConfigurator(1);
    SimpleAlgorithmConfigurator toConfigurator = createSimpleConfigurator(2);
    ParameterGroup fromGroup = new ParameterGroup("from_group");
    ParameterGroup toGroup = new ParameterGroup("to_group");
    createParameter(fromGroup, "from_text_param_name", "from_text_param_value");
    createParameter(toGroup, "to_text_param_name", "to_text_param_value");
    fromConfigurator.addGroup(fromGroup);
    toConfigurator.addGroup(toGroup);

    assertCopySuccess(fromConfigurator, toConfigurator, true);

    String parameterValue =
      toConfigurator.getParameterValue("to_text_param_name");
    assertEquals("to_text_param_value", parameterValue);
  }

  @Test
  public void testCopyWithDifferentParameterLabelWhenUserConfirms()
    throws ClientException, ParameterNotFoundException {
    SimpleAlgorithmConfigurator fromConfigurator = createSimpleConfigurator(1);
    SimpleAlgorithmConfigurator toConfigurator = createSimpleConfigurator(2);
    ParameterGroup fromGroup = new ParameterGroup("from_group");
    ParameterGroup toGroup = new ParameterGroup("to_group");
    createParameter(fromGroup, "text_param_name", "from_text_param_value",
      "from_text_param_label");
    createParameter(toGroup, "text_param_name", "to_text_param_value",
      "to_text_param_label");
    fromConfigurator.addGroup(fromGroup);
    toConfigurator.addGroup(toGroup);

    assertCopySuccess(fromConfigurator, toConfigurator, true);

    String parameterValue = toConfigurator.getParameterValue("text_param_name");
    assertEquals("from_text_param_value", parameterValue);
  }

  @Test
  public void testCopyWithDifferentParameterLabelWhenUserCancels()
    throws ClientException, ParameterNotFoundException {
    SimpleAlgorithmConfigurator fromConfigurator = createSimpleConfigurator(1);
    SimpleAlgorithmConfigurator toConfigurator = createSimpleConfigurator(2);
    ParameterGroup fromGroup = new ParameterGroup("from_group");
    ParameterGroup toGroup = new ParameterGroup("to_group");
    createParameter(fromGroup, "text_param_name", "from_text_param_value",
      "from_text_param_label");
    createParameter(toGroup, "text_param_name", "to_text_param_value",
      "to_text_param_label");
    fromConfigurator.addGroup(fromGroup);
    toConfigurator.addGroup(toGroup);

    assertCopyFailure(fromConfigurator, toConfigurator, false);

    String parameterValue = toConfigurator.getParameterValue("text_param_name");
    assertEquals("to_text_param_value", parameterValue);
  }

  private SimpleAlgorithmConfigurator createSimpleConfigurator(int i) {
    AlgorithmInfo algoInfo =
      SimpleAlgorithmConfiguratorFactory.createAlgorithmInfo(i, 1);
    AlgorithmVersionInfo versionInfo = getVersionInfo(algoInfo, 1);
    SimpleAlgorithmConfigurator configurator =
      SimpleAlgorithmConfiguratorFactory.createConfigurator(versionInfo);
    return configurator;
  }

  private TextParameter createParameter(ParameterGroup group, String name,
    String value) {
    return createParameter(group, name, value, name + "_label");
  }

  private TextParameter createParameter(ParameterGroup group, String name,
    String value, String label) {
    TextParameter textParameter =
      new TextParameter(name, label, "description", value, false,
        "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO", 100);
    group.addParameter(textParameter);
    return textParameter;
  }

  private void assertCopyFailure(SimpleAlgorithmConfigurator fromConfigurator,
    SimpleAlgorithmConfigurator toConfigurator, boolean userConfimartion)
    throws ClientException {
    assertCopyResult(fromConfigurator, toConfigurator, false, userConfimartion);
  }

  private void assertCopySuccess(SimpleAlgorithmConfigurator fromConfigurator,
    SimpleAlgorithmConfigurator toConfigurator, boolean userConfimartion)
    throws ClientException {
    assertCopyResult(fromConfigurator, toConfigurator, true, userConfimartion);
  }

  private void assertCopyResult(SimpleAlgorithmConfigurator fromConfigurator,
    SimpleAlgorithmConfigurator toConfigurator, boolean copyShouldBeMade,
    final boolean userConfirmation) throws ClientException {
    JFrame window = new JFrame("Janela do Grafo");
    SimpleAlgorithmConfiguratorView fromView =
      new SimpleAlgorithmConfiguratorView(window, fromConfigurator,
        Mode.CONFIGURATION);
    SimpleAlgorithmConfiguratorView toView =
      new SimpleAlgorithmConfiguratorView(window, toConfigurator,
        Mode.CONFIGURATION);

    Graph graph = new Graph(window);
    GraphNode fromNode = GraphNodeFactory.createGraphNode(graph, fromView);
    GraphNode toNode = GraphNodeFactory.createGraphNode(graph, toView);

    CopyAndPasteOperation copyAndPasteOperation =
      new CopyAndPasteOperation(fromNode, toNode) {
        /*
         * Reimplementando este mtodo para simular a resposta do usurio sem
         * mostrar a janela de confirmao.
         */
        @Override
        protected boolean showConfirmationDialog() {
          return userConfirmation;
        }
      };
    OperationStatus confirmation = copyAndPasteOperation.getConfirmation();
    boolean confirmed = confirmation.isConfirmed();
    boolean result = true;
    if (confirmed) {
      result = copyAndPasteOperation.pasteParameters();
    }
    boolean copyWasMade = confirmed && result;
    assertEquals(copyShouldBeMade, copyWasMade);

  }

  private AlgorithmVersionInfo getVersionInfo(AlgorithmInfo algo1,
    int versionNumber) {
    return algo1.getVersionInfo(new AlgorithmVersionId(versionNumber, 0, 0));
  }
}
