/*
 * $Author: lmoreira $ $Date: 2005/07/13 20:03:21 $ $Release:$
 */
package csbase.logic.algorithms.parameters;

import junit.framework.Assert;

/**
 * <p>Verificador de Eventos do {@link SimpleParameterListener}:
 * ele proibe ou permite a gerao de eventos e contabiliza os eventos
 * gerados.</p>
 * 
 * <p>Para utilizar o verificador, crie uma instncia, ajuste as permisses, 
 * cadastre a instncia no parmetro que ser verificado, execute o mtodo
 * que no poderia ou deveria gerar os eventos que esto sendo analisados e 
 * em seguida confira se o nmero de eventos gerados  igual ao esperado. O
 * verificador  criado proibindo todos os eventos, para permitir algum evento
 * chame um dos mtodos:
 * <ul> 
 * <li>{@link CheckParameterListener#allowChangeDefaultValue()}</li>
 * <li>{@link CheckParameterListener#allowChangeEnabledStatus()}</li>
 * <li>{@link CheckParameterListener#allowChangeLabel()}</li>
 * <li>{@link CheckParameterListener#allowChangeValue()}</li>
 * <li>{@link CheckParameterListener#allowChangeVisibleStatus()}</li>
 * </ul>
 * </p>
 * 
 * @param <V> O tipo do valor armazenado no parmetro.
 */
public final class CheckParameterListener<V> implements
  SimpleParameterListener<V> {
  /**
   * Indica se pode ocorrer o evento 
   * {@link SimpleParameterListener#defaultValueWasChanged(SimpleParameter)}.
   */
  private boolean canChangeDefaultChangeValue;
  /**
   * Indica se pode ocorrer o evento 
   * {@link SimpleParameterListener#capabilityWasChanged(SimpleParameter)}.
   */
  private boolean canChangeEnabledStatus;
  /**
   * Indica se pode ocorrer o evento 
   * {@link SimpleParameterListener#labelWasChanged(SimpleParameter)}.
   */
  private boolean canChangeLabel;
  /**
   * Indica se pode ocorrer o evento 
   * {@link SimpleParameterListener#valueWasChanged(SimpleParameter)}.
   */
  private boolean canChangeValue;
  /**
   * Indica se pode ocorrer o evento 
   * {@link SimpleParameterListener#visibilityWasChanged(SimpleParameter)}.
   */
  private boolean canChangeVisibleStatus;
  /**
   * Indica quantos eventos ocorreram at o momento.
   */
  private int howManyEventsOccurred;

  /**
   * Cria o observador.
   */
  public CheckParameterListener() {
    howManyEventsOccurred = 0;
    canChangeDefaultChangeValue = false;
    canChangeEnabledStatus = false;
    canChangeLabel = false;
    canChangeValue = false;
    canChangeVisibleStatus = false;
  }

  /**
   * Permite que o evento 
   * {@link SimpleParameterListener#defaultValueWasChanged(SimpleParameter)}
   * ocorra.
   */
  public void allowChangeDefaultValue() {
    canChangeDefaultChangeValue = true;
  }

  /**
   * Permite que o evento 
   * {@link SimpleParameterListener#capabilityWasChanged(SimpleParameter)}
   * ocorra.
   */
  public void allowChangeEnabledStatus() {
    canChangeEnabledStatus = true;
  }

  /**
   * Permite que o evento 
   * {@link SimpleParameterListener#labelWasChanged(SimpleParameter)}
   * ocorra.
   */
  public void allowChangeLabel() {
    canChangeLabel = true;
  }

  /**
   * Permite que o evento 
   * {@link SimpleParameterListener#valueWasChanged(SimpleParameter)}
   * ocorra.
   */
  public void allowChangeValue() {
    canChangeValue = true;
  }

  /**
   * Permite que o evento 
   * {@link SimpleParameterListener#visibilityWasChanged(SimpleParameter)}
   * ocorra.
   */
  public void allowChangeVisibleStatus() {
    canChangeVisibleStatus = true;
  }

  public void capabilityWasChanged(SimpleParameter<V> parameter) {
    howManyEventsOccurred++;
    Assert.assertTrue(canChangeEnabledStatus);
  }

  public void defaultValueWasChanged(SimpleParameter<V> parameter) {
    howManyEventsOccurred++;
    Assert.assertTrue(canChangeDefaultChangeValue);
  }

  /**
   * Obtm a quantidade de eventos gerados at o momento.
   * 
   * @return .
   */
  public int getHowManyEventsOccurred() {
    return howManyEventsOccurred;
  }

  public void labelWasChanged(SimpleParameter<V> parameter) {
    howManyEventsOccurred++;
    Assert.assertTrue(canChangeLabel);
  }

  public void valueWasChanged(SimpleParameter<V> parameter) {
    howManyEventsOccurred++;
    Assert.assertTrue(canChangeValue);
  }

  public void visibilityWasChanged(SimpleParameter<V> parameter) {
    howManyEventsOccurred++;
    Assert.assertTrue(canChangeVisibleStatus);
  }
}
