/*
 * $Author:$
 * $Date:$
 * $Release:$
 */
package csbase.logic.algorithms.parameters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

/**
 * Testes unitrios para a classe {@link Column}.
 *
 * @author lmoreira
 */
public final class ColumnTest extends TestCase {
  /**
   * Testando criar uma coluna opcional.
   */
  public void testColumn1() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean isOptional = true;
    boolean isEditable = true;
    MockColumn column =
      new MockColumn(id, label, null, isOptional, isEditable);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(isOptional, column.isOptional());
    assertEquals(isEditable, column.isEditable());
    assertNull(column.getDefaultValue());
    assertNotNull(column.getRowIndexes());
    assertTrue(column.getRowIndexes().isEmpty());
  }

  /**
   * Testando criar uma coluna obrigatria.
   */
  public void testColumn2() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean isOptional = false;
    boolean isEditable = true;
    MockColumn column =
      new MockColumn(id, label, null, isOptional, isEditable);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(isOptional, column.isOptional());
    assertEquals(isEditable, column.isEditable());
    assertNull(column.getDefaultValue());
    assertNotNull(column.getRowIndexes());
    assertTrue(column.getRowIndexes().isEmpty());
  }

  /**
   * Testando criar uma coluna com valor-padro.
   */
  public void testColumn3() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean isOptional = true;
    boolean isEditable = true;
    Object defaultValue = "Valor-padro";
    MockColumn column =
      new MockColumn(id, label, defaultValue, isOptional, isEditable);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(isOptional, column.isOptional());
    assertEquals(isEditable, column.isEditable());
    assertEquals(defaultValue, column.getDefaultValue());
    assertNotNull(column.getRowIndexes());
    assertTrue(column.getRowIndexes().isEmpty());
  }

  /**
   * Testando criar uma coluna sem rtulo.
   */
  public void testColumn4() {
    String id = "Identificador da coluna";
    boolean isOptional = true;
    boolean isEditable = true;
    try {
      new MockColumn(id, null, null, isOptional, isEditable);
      fail("No deveria ser possvel criar uma coluna com rtulo nulo.");
    }
    catch (IllegalArgumentException e) {}
  }

  /**
   * Testando criar uma coluna no-editvel.
   */
  public void testColumn5() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean isOptional = true;
    boolean isEditable = false;
    MockColumn column =
      new MockColumn(id, label, null, isOptional, isEditable);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(isOptional, column.isOptional());
    assertEquals(isEditable, column.isEditable());
    assertNull(column.getDefaultValue());
    assertNotNull(column.getRowIndexes());
    assertTrue(column.getRowIndexes().isEmpty());
  }
  
  /**
   * Testando criar uma coluna sem identificador.
   */
  public void testColumn6() {
    String label = "Rtulo da coluna";
    boolean isOptional = true;
    boolean isEditable = true;
    try {
      new MockColumn(null, label, null, isOptional, isEditable);
      fail("No deveria ser possvel criar uma coluna com identificador nulo.");
    }
    catch (IllegalArgumentException e) {}
  }
 
  /**
   * Testando o mtodo {@link Column#getDefaultValue(int)} sem valor-padro.
   */
  public void testGetDefaultValue1() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean isOptional = true;
    boolean isEditable = true;
    MockColumn column =
      new MockColumn(id, label, null, isOptional, isEditable);
    assertNull(column.getDefaultValue());
    assertNotNull(column.getRowIndexes());
    assertTrue(column.getRowIndexes().isEmpty());
    assertNull(column.getDefaultValue(0));
  }

  /**
   * Testando o mtodo {@link Column#getDefaultValue(int)} com valor-padro 
   * geral.
   */
  public void testGetDefaultValue2() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean isOptional = true;
    boolean isEditable = true;
    Object defaultValue = "Valor-padro geral";
    MockColumn column =
      new MockColumn(id, label, defaultValue, isOptional, isEditable);
    assertEquals(defaultValue, column.getDefaultValue());
    assertNotNull(column.getRowIndexes());
    assertTrue(column.getRowIndexes().isEmpty());
    assertEquals(defaultValue, column.getDefaultValue(0));
  }

  /**
   * Testando o mtodo {@link Column#getDefaultValue(int)} com valores-padro 
   * especficos.
   */
  public void testGetDefaultValue3() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean isOptional = true;
    boolean isEditable = true;
    Object defaultValue = null;
    Map<Integer, Object> defaultValuesByRowIndex = 
      new HashMap<Integer, Object>();
    defaultValuesByRowIndex.put(1, "Padro 1");
    defaultValuesByRowIndex.put(3, "Padro 3");
    MockColumn column = new MockColumn(id, label, defaultValue, isOptional, isEditable);
    column.addDefaultValue(1, defaultValuesByRowIndex.get(1));
    column.addDefaultValue(3, defaultValuesByRowIndex.get(3));
    assertEquals(defaultValue, column.getDefaultValue());
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertEquals(defaultValuesByRowIndex.get(1), column.getDefaultValue(1));
    assertEquals(defaultValue, column.getDefaultValue(2));
    assertEquals(defaultValuesByRowIndex.get(3), column.getDefaultValue(3));
    assertEquals(defaultValue, column.getDefaultValue(4));
  }

  
  /**
   * Testando o mtodo {@link Column#getDefaultValue(int)} com valores-padro 
   * geral e especficos.
   */
  public void testGetDefaultValue4() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean isOptional = true;
    boolean isEditable = true;
    Object defaultValue = "Valor-padro geral";
    Map<Integer, Object> defaultValuesByRowIndex = 
      new HashMap<Integer, Object>();
    defaultValuesByRowIndex.put(1, "Padro 1");
    defaultValuesByRowIndex.put(3, "Padro 3");
    MockColumn column = new MockColumn(id, label, defaultValue, 
      isOptional, isEditable);
    column.addDefaultValue(1, defaultValuesByRowIndex.get(1));
    column.addDefaultValue(3, defaultValuesByRowIndex.get(3));
    assertEquals(defaultValue, column.getDefaultValue());
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertEquals(defaultValuesByRowIndex.get(1), column.getDefaultValue(1));
    assertEquals(defaultValue, column.getDefaultValue(2));
    assertEquals(defaultValuesByRowIndex.get(3), column.getDefaultValue(3));
    assertEquals(defaultValue, column.getDefaultValue(4));
  }

  /**
   * Testando o mtodo {@link Column#getDefaultValue(int)} com valores-padro 
   * geral e especficos {@code null}.
   */
  public void testGetDefaultValue5() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean isOptional = true;
    boolean isEditable = true;
    Object defaultValue = "Valor-padro geral";
    MockColumn column = new MockColumn(id, label, defaultValue, 
      isOptional, isEditable);
    column.addDefaultValue(1, null);
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertNull(column.getDefaultValue(1));
    assertEquals(defaultValue, column.getDefaultValue(2));
  }
  
  /**
   * Testando o mtodo {@link Column#toString()}.
   */
  public void testToString() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean isOptional = true;
    boolean isEditable = true;
    MockColumn column = new MockColumn(id, label, null, isOptional, isEditable);
    assertEquals(label, column.toString());
  }

  /**
   * Testando o mtodo {@link Column#equals(Object)} com colunas com 
   * identificadores iguais.
   */
  public void testEquals1() {
    String id = "Identificador da coluna";
    String label1 = "Nome da coluna 1";
    boolean isOptional1 = true;
    boolean isEditable1 = true;
    MockColumn column1 = new MockColumn(id, label1, null, isOptional1,
      isEditable1);
    String label2 = "Nome da coluna 2";
    boolean isOptional2 = false;
    boolean isEditable2 = false;
    MockColumn column2 = new MockColumn(id, label2, null, isOptional2,
      isEditable2);
    assertTrue(column1.equals(column2));
  }

  /**
   * Testando o mtodo {@link Column#equals(Object)} com parmetro {@code null}.
   */
  public void testEquals2() {
    String id1 = "Identificador da coluna 1";
    String label1 = "Nome da coluna 1";
    boolean isOptional1 = true;
    boolean isEditable1 = true;
    MockColumn column1 = new MockColumn(id1, label1, null, isOptional1, 
      isEditable1);
    assertFalse(column1.equals(null));
  }

  /**
   * Testando o mtodo {@link Column#equals(Object)} com parmetro de outra
   * classe.
   */
  public void testEquals3() {
    String id1 = "Identificador da coluna 1";
    String label1 = "Nome da coluna 1";
    boolean isOptional1 = true;
    boolean isEditable1 = true;
    MockColumn column1 = new MockColumn(id1, label1, null, isOptional1, 
      isEditable1);
    assertFalse(column1.equals("Objeto de outra classe"));
  }

  /**
   * Testando o mtodo {@link Column#hashCode()}.
   */
  public void testHashCode1() {
    String id = "Identificador da coluna 1";
    String label1 = "Nome da coluna 1";
    boolean isOptional1 = true;
    boolean isEditable1 = true;
    MockColumn column1 = new MockColumn(id, label1, null, isOptional1,
      isEditable1);
    String label2 = "Nome da coluna 2";
    boolean isOptional2 = false;
    boolean isEditable2 = false;
    MockColumn column2 = new MockColumn(id, label2, null, isOptional2,
      isEditable2);
    assertEquals(column1.hashCode(), column2.hashCode());
  }

  /**
   * Testando serializar e desserializar colunas.
   * 
   * @throws IOException Bug.
   * @throws ClassNotFoundException Bug.
   */
  public void testSerialize()
  throws IOException, ClassNotFoundException {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    boolean isOptional = true;
    boolean isEditable = true;
    Object defaultValue = "Valor-padro";
    Map<Integer, Object> defaultValuesByRowIndex = 
      new HashMap<Integer, Object>();
    defaultValuesByRowIndex.put(1, "Padro 1");
    defaultValuesByRowIndex.put(3, "Padro 3");
    MockColumn outputColumn = 
      new MockColumn(id, label, defaultValue, isOptional, isEditable);
    outputColumn.addDefaultValue(1, defaultValuesByRowIndex.get(1));
    outputColumn.addDefaultValue(3, defaultValuesByRowIndex.get(3));
    ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
    ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
    objectOutput.writeObject(outputColumn);
    ByteArrayInputStream inputSource =
      new ByteArrayInputStream(outputSource.toByteArray());
    ObjectInputStream objectInput = new ObjectInputStream(inputSource);
    MockColumn inputColumn = (MockColumn)objectInput.readObject();
    assertEquals(outputColumn.getId(), outputColumn.getId());
    assertEquals(outputColumn.getLabel(), inputColumn.getLabel());
    assertEquals(outputColumn.isOptional(), inputColumn.isOptional());
    assertEquals(outputColumn.isEditable(), inputColumn.isEditable());
    assertEquals(outputColumn.getDefaultValue(), inputColumn.getDefaultValue());
    assertEquals(outputColumn.getRowIndexes(), inputColumn.getRowIndexes());
    for (Integer rowIndex : outputColumn.getRowIndexes()) {
      assertEquals(outputColumn.getDefaultValue(rowIndex), 
        inputColumn.getDefaultValue(rowIndex));
    }
  }
}
