/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.logic.algorithms.parameters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import junit.framework.TestCase;

/**
 * Testes unitrios para a classe {@link FormulaParameter}.
 */
public final class FormulaParameterTest extends TestCase {

  /**
   * Testando a criao de um parmetro Frmula opcional.
   */
  public void testFormulaParameter1() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = true;
    String commandLinePattern = "{0}={1}";
    FormulaParameter parameter = new FormulaParameter(name, label, description,
      isOptional, commandLinePattern);
    assertEquals(name, parameter.getName());
    assertEquals(label, parameter.getLabel());
    assertEquals(description, parameter.getDescription());
    assertEquals(isOptional, parameter.isOptional());
  }

  /**
   * Testando a criao de um parmetro Frmula obrigatrio.
   */
  public void testFormulaParameter2() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "{0}={1}";
    FormulaParameter parameter = new FormulaParameter(name, label, description,
      isOptional, commandLinePattern);
    assertEquals(name, parameter.getName());
    assertEquals(label, parameter.getLabel());
    assertEquals(description, parameter.getDescription());
    assertEquals(isOptional, parameter.isOptional());
  }

  /**
   * Testando a criao de um parmetro Frmula sem nome.
   */
  public void testFormulaParameter3() {
    String name = null;
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "{0}={1}";
    try {
      new FormulaParameter(name, label, description, isOptional,
        commandLinePattern);
      fail("No deveria ser possvel criar um parmetro com nome nulo.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando a criao de um parmetro Frmula sem rtulo.
   */
  public void testFormulaParameter4() {
    String name = "Nome do parmetro";
    String label = null;
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "{0}={1}";
    try {
      new FormulaParameter(name, label, description, isOptional,
        commandLinePattern);
      fail("No deveria ser possvel criar um parmetro com rtulo nulo.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando a criao de um parmetro Frmula sem descrio.
   */
  public void testFormulaParameter5() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = null;
    boolean isOptional = false;
    String commandLinePattern = "{0}={1}";
    try {
      new FormulaParameter(name, label, description, isOptional,
        commandLinePattern);
      fail("No deveria ser possvel criar um parmetro com descrio "
        + "nula.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando o mtodo 
   * {@link FormulaParameter#getCommandLine(String, String)} sem executar
   * escapes.
   */
  public void testGetCommandLine1() {
    String name = "NOME";
    String label = "Rtulo";
    String description = "Descrio";
    boolean isOptional = true;
    String commandLinePattern = "{0}={1}";
    FormulaParameter parameter = new FormulaParameter(name, label, description,
      isOptional, commandLinePattern);
    parameter.setValue("a1");
    String commandLine = "NOME=a1";
    assertEquals(commandLine, parameter.getCommandLine(null, "/"));
  }

  /**
   * Testando o mtodo 
   * {@link FormulaParameter#getCommandLine(String, String)} executando todos
   * os escapes possveis.
   */
  public void testGetCommandLine2() {
    String name = "NOME";
    String label = "Rtulo";
    String description = "Descrio";
    boolean isOptional = true;
    String commandLinePattern = "{0}={1}";
    FormulaParameter parameter = new FormulaParameter(name, label, description,
      isOptional, commandLinePattern);
    parameter.setValue("a1_();[]{}+-*/%<> .=^");
    String commandLine = "NOME=a1" +
        "__Underline__" +
        "__OpenRoundBracket__" +
        "__CloseRoundBracket__" +
        "__SemiColon__" +
        "__OpenSquareBracket__" +
        "__CloseSquareBracket__" +
        "__OpenCurlyBracket__" +
        "__CloseCurlyBracket__" +
        "__PlusSign__" +
        "__MinusSign__" +
        "__Asterisk__" +
        "__Slash__" +
        "__Percent__" +
        "__LessThan__" +
        "__LargeThan__" +
        "__Space__" +
        "__Dot__" +
        "__EqualsSign__" +
        "__Caret__";
    assertEquals(commandLine, parameter.getCommandLine(null, "/"));
  }

  /**
   * Testando o mtodo 
   * {@link FormulaParameter#getCommandLine(String, String)} sem valor.
   */
  public void testGetCommandLine3() {
    String name = "NOME";
    String label = "Rtulo";
    String description = "Descrio";
    boolean isOptional = true;
    String commandLinePattern = "{0}={1}";
    FormulaParameter parameter = new FormulaParameter(name, label, description,
      isOptional, commandLinePattern);
    assertNull(parameter.getCommandLine(null, "/"));
  }
  
  /**
   * Testando a serializao e a desserializao de parmetros do tipo Frmula.
   * 
   * @throws IOException Em caso de erro.
   * @throws ClassNotFoundException Em caso de erro.
   */
  public void testSerialize() throws IOException, ClassNotFoundException {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = true;
    String commandLinePattern = "{0}={1}";
    FormulaParameter outputParameter = new FormulaParameter(name, label,
      description, isOptional, commandLinePattern);
    ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
    ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
    objectOutput.writeObject(outputParameter);
    ByteArrayInputStream inputSource = new ByteArrayInputStream(outputSource
      .toByteArray());
    ObjectInputStream objectInput = new ObjectInputStream(inputSource);
    FormulaParameter inputParameter = (FormulaParameter) objectInput
      .readObject();
    assertEquals(outputParameter, inputParameter);
    assertEquals(outputParameter.getName(), inputParameter.getName());
    assertEquals(outputParameter.getLabel(), inputParameter.getLabel());
    assertEquals(outputParameter.getDescription(), inputParameter
      .getDescription());
    assertEquals(outputParameter.getDefaultValue(), inputParameter
      .getDefaultValue());
    assertEquals(outputParameter.isOptional(), inputParameter.isOptional());
  }
}
