/*
 * $Author:$
 * $Date:$
 * $Release:$
 */
package csbase.logic.algorithms.parameters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import junit.framework.TestCase;

/**
 * Testes unitrios para a classe {@link IntegerColumn}.
 *
 * @author lmoreira
 */
public final class IntegerColumnTest extends TestCase {
  /**
   * Testando criar colunas de inteiros opcionais.
   */
  public void testIntegerColumn1() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    Integer defaultValue = new Integer(123);
    boolean isOptional = true;
    boolean isEditable = true;
    Integer maximum = new Integer(10);
    Integer minimum = new Integer(0);
    IntegerColumn column = new IntegerColumn(id, label, defaultValue,
      isOptional, isEditable, maximum, minimum);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertEquals(isOptional, column.isOptional());
    assertEquals(isEditable, column.isEditable());
    assertEquals(maximum, column.getMaximum());
    assertEquals(minimum, column.getMinimum());
  }

  /**
   * Testando criar colunas de inteiros obrigatrias.
   */
  public void testIntegerColumn2() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    Integer defaultValue = new Integer(123);
    boolean isOptional = false;
    boolean isEditable = true;
    Integer maximum = new Integer(10);
    Integer minimum = new Integer(0);
    IntegerColumn column = new IntegerColumn(id, label, defaultValue,
      isOptional, isEditable, maximum, minimum);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertEquals(isOptional, column.isOptional());
    assertEquals(isEditable, column.isEditable());
    assertEquals(maximum, column.getMaximum());
    assertEquals(minimum, column.getMinimum());
  }

  /**
   * Testando criar colunas de inteiros sem rtulo.
   */
  public void testIntegerColumn3() {
    String id = "Identificador da coluna";
    Integer defaultValue = new Integer(123);
    boolean isOptional = false;
    boolean isEditable = true;
    Integer maximum = new Integer(10);
    Integer minimum = new Integer(0);
    try {
      new IntegerColumn(id, null, defaultValue, isOptional, isEditable,
        maximum, minimum);
      fail("No deveria ser possvel criar uma coluna com rtulo nulo.");
    }
    catch (IllegalArgumentException e) {}
  }

  /**
   * Testando criar colunas de inteiros sem mximo.
   */
  public void testIntegerColumn4() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    Integer defaultValue = new Integer(123);
    boolean isOptional = true;
    boolean isEditable = true;
    Integer minimum = new Integer(0);
    IntegerColumn column = new IntegerColumn(id, label, defaultValue,
      isOptional, isEditable, null, minimum);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertEquals(isOptional, column.isOptional());
    assertEquals(isEditable, column.isEditable());
    assertNull(column.getMaximum());
    assertEquals(minimum, column.getMinimum());
  }

  /**
   * Testando criar colunas de inteiros sem mnimo.
   */
  public void testIntegerColumn5() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    Integer defaultValue = new Integer(123);
    boolean isOptional = true;
    boolean isEditable = true;
    Integer maximum = new Integer(0);
    IntegerColumn column = new IntegerColumn(id, label, defaultValue,
      isOptional, isEditable, maximum, null);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertEquals(isOptional, column.isOptional());
    assertEquals(isEditable, column.isEditable());
    assertEquals(maximum, column.getMaximum());
    assertNull(column.getMinimum());
  }

  /**
   * Testando criar colunas de inteiros com mnimo maior do que o mximo.
   */
  public void testIntegerColumn6() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    Integer defaultValue = new Integer(123);
    boolean isOptional = true;
    boolean isEditable = true;
    Integer maximum = new Integer(0);
    Integer minimum = new Integer(maximum.intValue()+1);
    try {
      new IntegerColumn(id, label, defaultValue, isOptional, isEditable,
        maximum, minimum);
      fail(
        "No deveria ser possvel criar uma coluna com o mnimo "+
        "maior do que o mximo.");
    }
    catch (IllegalArgumentException e) {}
  }

  /**
   * Testando criar colunas de inteiros no-editvel.
   */
  public void testIntegerColumn7() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    Integer defaultValue = new Integer(123);
    boolean isOptional = true;
    boolean isEditable = false;
    Integer maximum = new Integer(10);
    Integer minimum = new Integer(0);
    IntegerColumn column = new IntegerColumn(id, label, defaultValue,
      isOptional, isEditable, maximum, minimum);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertEquals(isOptional, column.isOptional());
    assertEquals(isEditable, column.isEditable());
    assertEquals(maximum, column.getMaximum());
    assertEquals(minimum, column.getMinimum());
  }
  
  /**
   * Testando criar colunas de inteiros sem identificador.
   */
  public void testIntegerColumn8() {
    String label = "Rtulo da coluna";
    Integer defaultValue = new Integer(123);
    boolean isOptional = false;
    boolean isEditable = true;
    Integer maximum = new Integer(10);
    Integer minimum = new Integer(0);
    try {
      new IntegerColumn(null, label, defaultValue, isOptional, isEditable,
        maximum, minimum);
      fail("No deveria ser possvel criar uma coluna com id nulo.");
    }
    catch (IllegalArgumentException e) {}
  }

  /**
   * Testando serializar e desserializar uma coluna de inteiros.
   * 
   * @throws IOException Bug.
   * @throws ClassNotFoundException Bug.
   */
  public void testSerialize()
  throws IOException, ClassNotFoundException {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    Integer defaultValue = new Integer(123);
    boolean isOptional = true;
    boolean isEditable = true;
    Integer maximum = new Integer(1000);
    Integer minimum = new Integer(100);
    IntegerColumn expectedColumn = new IntegerColumn(id, label, defaultValue, 
      isOptional, isEditable, maximum, minimum);
    expectedColumn.addDefaultValue(1, 10);
    expectedColumn.addDefaultValue(3, 30);
    ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
    ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
    objectOutput.writeObject(expectedColumn);
    ByteArrayInputStream inputSource =
      new ByteArrayInputStream(outputSource.toByteArray());
    ObjectInputStream objectInput = new ObjectInputStream(inputSource);
    IntegerColumn loadedColumn = (IntegerColumn)objectInput.readObject();
    assertEquals(defaultValue, loadedColumn.getDefaultValue(0));
    assertEquals(new Integer(10), loadedColumn.getDefaultValue(1));
    assertEquals(defaultValue, loadedColumn.getDefaultValue(2));
    assertEquals(new Integer(30), loadedColumn.getDefaultValue(3));
    assertEquals(defaultValue, loadedColumn.getDefaultValue(4));
    assertEquals(expectedColumn.getMaximum(), loadedColumn.getMaximum());
    assertEquals(expectedColumn.getMinimum(), loadedColumn.getMinimum());
    assertEquals(expectedColumn.getLabel(), loadedColumn.getLabel());
    assertEquals(expectedColumn.isOptional(), loadedColumn.isOptional());
    assertEquals(expectedColumn.isEditable(), loadedColumn.isEditable());
  }
}
