/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.logic.algorithms.parameters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import junit.framework.Assert;
import junit.framework.TestCase;

/**
 * Testes unitrios para a classe {@link SimpleParameterTest}.
 *
 * @author lmoreira
 */
public final class SimpleParameterTest extends TestCase {
  /**
   * Verifica possuem se 2 parmetros possuem estados iguais.
   * 
   * @param <V> O valor do parmetro. 
   * 
   * @param expectedParameter O parmetro esperado (No aceita {@code null}).
   * @param foundParameter O parmetro encontrado (No aceita {@code null}).
   */
	public static <V> void assertEquals(SimpleParameter<V> expectedParameter,
			SimpleParameter<V> foundParameter) {
		assertEquals(expectedParameter.getName(), expectedParameter.getLabel(),
      expectedParameter.getDescription(), expectedParameter.getDefaultValue(),
      expectedParameter.isOptional(),
      expectedParameter.getCommandLinePattern(), expectedParameter.isEnabled(),
      expectedParameter.getSimpleParameterListeners(), foundParameter);
	}

  /**
   * Verifica possuem o parmetro encontrado possui um estado composto
   * por atributos iguais ao outros parmetros fornecidos.
   * 
   * @param <V> O valor do parmetro.
   * 
   * @param name O nome.
   * @param label O rtulo.
   * @param description A descrio.
   * @param defaultValue O valor-padro.
   * @param isOptional Indica se o valor  opcional/obrigatrio.
   * @param commandLinePattern O padro da linha de comando.
   * @param isEnabled Indica se o parmetro precisa estar habilitado/desabilitado.
   * @param listeners O observadores deste parmetro.
   * @param parameter O parmetro encontrado (No aceita {@code null}).
   */
  public static <V> void assertEquals(String name, String label,
    String description, Object defaultValue, boolean isOptional,
    String commandLinePattern, boolean isEnabled,
    List<SimpleParameterListener<V>> listeners, SimpleParameter<V> parameter) {
		Assert.assertEquals(name, parameter.getName());
		Assert.assertEquals(label, parameter.getLabel());
    Assert.assertEquals(commandLinePattern, parameter.getCommandLinePattern());
		Assert.assertEquals(description, parameter.getDescription());
		Assert.assertEquals(defaultValue, parameter.getDefaultValue());
		Assert.assertEquals(isOptional, parameter.isOptional());
		Assert.assertEquals(defaultValue, parameter.getValue());
		Assert.assertEquals(isEnabled, parameter.isEnabled());
		Assert.assertEquals(listeners, parameter.getSimpleParameterListeners());
	}

  /**
   * Cria um parmetro qualquer.
   * 
   * @return .
   */
  public static SimpleParameter<Object> create() {
		return create(null);
	}

  /**
   * Cria um parmetro qualquer, porm com valor-padro igual ao valor 
   * fornecido.
   * 
   * @param value O valor-padro.
   * 
   * @return .
   */
	public static SimpleParameter<Object> create(Object value) {
		return create(value, false);
	}

  /**
   * Cria um parmetro qualquer, porm com valor-padro igual ao valor 
   * e padro para linha de comando fornecidos.
   * 
   * @param name O nome do parmetro.
   * @param value O valor-padro.
   * @param commandLinePattern O padro para linha de comando.
   * 
   * @return .
   */
  public static SimpleParameter<Object> create(String name,
    Object value, String commandLinePattern) {
    return create(name, value, false, commandLinePattern);
  }
  
  /**
   * Cria um parmetro qualquer, porm com validador e com valor-padro iguais 
   * aos valores fornecidos.
   * 
   * @param value O valor-padro.
   * @param isOptional Indica se o valor  opcional/obrigatrio.
   * 
   * @return .
   */
	public static SimpleParameter<Object> create(Object value, boolean isOptional) {
    return create("Nome do parmetro", value, isOptional, "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO");
	}

  /**
   * Cria um parmetro qualquer, porm com validador e com valor-padro iguais 
   * aos valores fornecidos.
   * 
   * @param name O nome do parmetro.
   * @param value O valor-padro.
   * @param isOptional Indica se o valor  opcional/obrigatrio.
   * @param commandLinePattern O padro para linha de comando.
   * 
   * @return .
   */
  public static SimpleParameter<Object> create(String name,
    Object value, boolean isOptional, String commandLinePattern) {
    return new MockSimpleParameter(name, "Rtulo do parmetro",
      "Descrio do parmetro", value, isOptional, commandLinePattern);
  }

  /**
   * Teste para o mtodo 
   * {@link 
   * SimpleParameter#
   * SimpleParameter(String, String, String, Object, boolean, String)}: 
   * criando um parmetro com valor opcional.
   */
	public void testParameter1() {
		boolean isOptional = true;
		String name = "O nome do parmetro";
		String label = "O rtulo do parmetro";
		String description = "A descrio do parmetro";
		String defaultValue = "O valor-padro do parmetro";
    String commandLinePattern = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
		SimpleParameter<Object> parameter = new MockSimpleParameter(name,
				label, description, defaultValue, isOptional, commandLinePattern);
		assertEquals(name, label, description, defaultValue, isOptional,
      commandLinePattern, true, new LinkedList<SimpleParameterListener<Object>>(),
      parameter);
	}

  /**
   * Teste para o mtodo 
   * {@link 
   * SimpleParameter#
   * SimpleParameter(String, String, String, Object, boolean, String)}: 
   * criando um parmetro com valor obrigatrio.
   */
	public void testParameter2() {
		boolean isOptional = false;
		String name = "O nome do parmetro";
		String label = "O rtulo do parmetro";
		String description = "A descrio do parmetro";
		String defaultValue = "O valor-padro do parmetro";
    String commandLinePattern = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
		SimpleParameter<Object> parameter = new MockSimpleParameter(name,
				label, description, defaultValue, isOptional, commandLinePattern);
    assertEquals(name, label, description, defaultValue, isOptional,
      commandLinePattern, true, new LinkedList<SimpleParameterListener<Object>>(),
      parameter);
	}

  /**
   * Teste para o mtodo 
   * {@link 
   * SimpleParameter#
   * SimpleParameter(String, String, String, Object, boolean, String)}: 
   * tentando criar um parmetro com nome igual {@code null}.
   */
	public void testParameter3() {
		try {
			new MockSimpleParameter(null,
					"O rtulo do parmetro", "A descrio do parmetro",
					"O valor-padro do parmetro", false, "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO");
			fail("Esperada IllegalArgumentException");
		} catch (IllegalArgumentException e) {
		}
	}

  /**
   * Teste para o mtodo 
   * {@link 
   * SimpleParameter#
   * SimpleParameter(String, String, String, Object, boolean, String)}: 
   * tentando criar um parmetro com rtulo igual {@code null}.
   */
  public void testParameter4() {
		try {
			new MockSimpleParameter(
					"O nome do parmetro", null, "A descrio do parmetro",
					"O valor-padro do parmetro", false, "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO");
			fail("Esperada IllegalArgumentException");
		} catch (IllegalArgumentException e) {
		}
	}

  /**
   * Teste para o mtodo 
   * {@link 
   * SimpleParameter#
   * SimpleParameter(String, String, String, Object, boolean, String)}: 
   * tentando criar um parmetro com descrio igual {@code null}.
   */
	public void testParameter5() {
		try {
			new MockSimpleParameter(
					"O nome do parmetro", "O rtulo do parmetro", null,
					"O valor-padro do parmetro", false, "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO");
			fail("Esperada IllegalArgumentException");
		} catch (IllegalArgumentException e) {
		}
	}

  /**
   * Teste para o mtodo 
   * {@link 
   * SimpleParameter#
   * SimpleParameter(String, String, String, Object, boolean, String)}: 
   * criando um parmetro com valor-padro igual {@code null}.
   */
	public void testParameter6() {
		String name = "O nome do parmetro";
		String label = "O rtulo do parmetro";
		String description = "A descrio do parmetro";
    String commandLinePattern = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
		boolean isOptional = true;
		SimpleParameter<Object> parameter = new MockSimpleParameter(name,
				label, description, null, isOptional, commandLinePattern);
    assertEquals(name, label, description, null, isOptional, commandLinePattern,
      true, new LinkedList<SimpleParameterListener<Object>>(), parameter);
	}

  /**
   * Teste para o mtodo 
   * {@link 
   * SimpleParameter#
   * SimpleParameter(String, String, String, Object, boolean, String)}: 
   * criando um parmetro com padro para a linha de comando igual {@code null}.
   */
  public void testParameter7() {
    String name = "O nome do parmetro";
    String label = "O rtulo do parmetro";
    String description = "A descrio do parmetro";
    Object defaultValue = "O valor-padro do parmetro";
    String commandLinePattern = null;
    boolean isOptional = true;
    SimpleParameter<Object> parameter = new MockSimpleParameter(name, label,
      description, defaultValue, isOptional, commandLinePattern);
    assertEquals(name, label, description, defaultValue, isOptional,
      commandLinePattern, true, new LinkedList<SimpleParameterListener<Object>>(),
      parameter);
  }
  
  /**
   * Testando o mtodo {@link SimpleParameter#toString()}.
   */
	public void testToString1() {
		SimpleParameter<Object> parameter = create();
		assertEquals(parameter.getName(), parameter.toString());
	}

  /**
   * Testando o mtodo {@link SimpleParameter#equals(Object)}: 2 parmetros com 
   * estados iguais so considerados iguais.
   */
	public void testEquals1() {
		String name1 = "O nome do parmetro";
		String label1 = "O rtulo do parmetro";
		String description1 = "A descrio do parmetro";
		String defaultValue1 = "O valor do parmetro";
		boolean isOptional1 = true;
    String commandLinePattern1 = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
		SimpleParameter<Object> parameter1 = new MockSimpleParameter(name1, label1,
				description1, defaultValue1, isOptional1, commandLinePattern1);
		String label2 = "O rtulo do parmetro 2";
		String description2 = "A descrio do parmetro 2";
		String defaultValue2 = "O valor do parmetro 2";
		boolean isOptional2 = false;
    String commandLinePattern2 = "--$VALOR_DO_PARAMETRO $NOME_DO_PARAMETRO";
		SimpleParameter<Object> parameter2 = new MockSimpleParameter(name1, label2,
				description2, defaultValue2, isOptional2, commandLinePattern2);
		assertTrue(parameter1.equals(parameter2));
	}

  /**
   * Testando o mtodo {@link SimpleParameter#equals(Object)}: 2 parmetros com 
   * nomes iguais so considerados iguais.
   */
  public void testEquals2() {
		String name1 = "O nome do parmetro";
		String label1 = "O rtulo do parmetro";
		String description1 = "A descrio do parmetro";
		String defaultValue1 = "O valor do parmetro";
		boolean isOptional1 = true;
    String commandLinePattern1 = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
		SimpleParameter<Object> parameter1 = new MockSimpleParameter(name1, label1,
				description1, defaultValue1, isOptional1, commandLinePattern1);
		String name2 = "Outro nome de parmetro";
		SimpleParameter<Object> parameter2 = new MockSimpleParameter(name2, label1,
				description1, defaultValue1, isOptional1, commandLinePattern1);
		assertFalse(parameter1.equals(parameter2));
	}

  /**
   * Testando o mtodo {@link SimpleParameter#equals(Object)}: 1 parmetro 
   * diferente de {@code null}.
   */
	public void testEquals3() {
		SimpleParameter<Object> parameter = create();
		assertFalse(parameter.equals(null));
	}

  /**
   * Testando o mtodo {@link SimpleParameter#equals(Object)}: 1 parmetro 
   * diferente de um objeto de outra classe.
   */
	public void testEquals4() {
		SimpleParameter<Object> parameter = create();
		assertFalse(parameter.equals("Objeto de outra classe"));
	}

  /**
   * Testando o mtodo {@link SimpleParameter#hashCode()}: 2 parmetros iguais
   * tm o mesmo cdigo <i>hash</i>.
   */
	public void testHashCode1() {
		String name1 = "O nome do parmetro";
		String label1 = "O rtulo do parmetro";
		String description1 = "A descrio do parmetro";
		String defaultValue1 = "O valor do parmetro";
		boolean isOptional1 = true;
    String commandLinePattern1 = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
		SimpleParameter<Object> parameter1 = new MockSimpleParameter(name1, label1,
				description1, defaultValue1, isOptional1, commandLinePattern1);
		String label2 = "O rtulo do parmetro 2";
		String description2 = "A descrio do parmetro 2";
		String defaultValue2 = "O valor do parmetro 2";
		boolean isOptional2 = false;
    String commandLinePattern2 = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
		SimpleParameter<Object> parameter2 = new MockSimpleParameter(name1, label2,
				description2, defaultValue2, isOptional2, commandLinePattern2);
		assertEquals(parameter1.hashCode(), parameter2.hashCode());
	}

  /**
   * Testando o mtodo 
   * {@link SimpleParameter#addSimpleParameterListener(SimpleParameterListener)}: 
   * adicionando um listener qualquer.
   */
	public void testAddListener1() {
		SimpleParameter<Object> parameter = create();
		assertNotNull(parameter.getSimpleParameterListeners());
		assertTrue(parameter.getSimpleParameterListeners().isEmpty());
		List<SimpleParameterListener<Object>> listeners = 
      new LinkedList<SimpleParameterListener<Object>>();
		SimpleParameterListener<Object> listener = new MockParameterListener();
		listeners.add(listener);
		parameter.addSimpleParameterListener(listener);
		assertEquals(listeners, parameter.getSimpleParameterListeners());
	}

  /**
   * Testando o mtodo 
   * {@link SimpleParameter#addSimpleParameterListener(SimpleParameterListener)}: 
   * tentando adicionar {@code null} como listener.
   */
	public void testAddListener2() {
		SimpleParameter<Object> parameter = create();
		try {
			parameter.addSimpleParameterListener(null);
			fail("Esperada IllegalArgumentException");
		} catch (IllegalArgumentException e) {
		}
	}

  /**
   * Testando o mtodo {@link SimpleParameter#getSimpleParameterListeners()}:
   * a lista de observadores retornada  imutvel.
   */
	public void testGetListeners1() {
		SimpleParameter<Object> parameter = create();
		assertNotNull(parameter.getSimpleParameterListeners());
		assertTrue(parameter.getSimpleParameterListeners().isEmpty());
		List<SimpleParameterListener<Object>> listeners = new LinkedList<SimpleParameterListener<Object>>();
		SimpleParameterListener<Object> listener = new MockParameterListener();
		listeners.add(listener);
		parameter.addSimpleParameterListener(listener);
		assertEquals(listeners, parameter.getSimpleParameterListeners());
		try {
			parameter.getSimpleParameterListeners().clear();
			fail("Esperada UnsupportedOperationException");
		} catch (UnsupportedOperationException e) {
		}
		assertEquals(listeners, parameter.getSimpleParameterListeners());
	}

  /**
   * Testando o mtodo {@link SimpleParameter#setValue(Object)}: modificando
   * o valor de um parmetro. O parmetro possuia um valor inicialmente e recebe
   * um valor novo.
   *
   */
	public void testSetValue1() {
		Object value = "Valor do parmetro";
		SimpleParameter<Object> parameter = create(value);
		assertEquals(value, parameter.getValue());
		CheckParameterListener<Object> listener = new CheckParameterListener<Object>();
		listener.allowChangeValue();
		parameter.addSimpleParameterListener(listener);
		Object otherValue = "Outro valor do parmetro";
		assertTrue(parameter.setValue(otherValue));
		assertEquals(1, listener.getHowManyEventsOccurred());
		assertEquals(otherValue, parameter.getValue());
	}

  /**
   * Testando o mtodo {@link SimpleParameter#setValue(Object)}: modificando
   * o valor de um parmetro. O parmetro possuia um valor inicialmente e recebe
   * {@code null}.
   *
   */
	public void testSetValue2() {
		Object value = "Valor do parmetro";
		SimpleParameter<Object> parameter = create(value);
		assertEquals(value, parameter.getValue());
		CheckParameterListener<Object> listener = new CheckParameterListener<Object>();
		listener.allowChangeValue();
		parameter.addSimpleParameterListener(listener);
		assertTrue(parameter.setValue(null));
		assertEquals(1, listener.getHowManyEventsOccurred());
		assertNull(parameter.getValue());
	}

  /**
   * Testando o mtodo {@link SimpleParameter#setValue(Object)}: tentando 
   * modificar o valor de um parmetro. O parmetro possuia um valor 
   * inicialmente e recebe o mesmo valor.
   */
  public void testSetValue3() {
		Object value = "Valor do parmetro";
		SimpleParameter<Object> parameter = create(value);
		assertEquals(value, parameter.getValue());
		CheckParameterListener<Object> listener = new CheckParameterListener<Object>();
		parameter.addSimpleParameterListener(listener);
		assertFalse(parameter.setValue(value));
		assertEquals(0, listener.getHowManyEventsOccurred());
		assertEquals(value, parameter.getValue());
	}

  /**
   * Testando o mtodo {@link SimpleParameter#setValue(Object)}: 
   * modificando o valor de um parmetro. O parmetro no possuia um valor 
   * inicialmente e recebe um valor qualquer.
   */
  public void testSetValue4() {
		SimpleParameter<Object> parameter = create(null);
		assertNull(parameter.getValue());
		CheckParameterListener<Object> listener = new CheckParameterListener<Object>();
		listener.allowChangeValue();
		parameter.addSimpleParameterListener(listener);
		Object otherValue = "Outro valor do parmetro";
		assertTrue(parameter.setValue(otherValue));
		assertEquals(1, listener.getHowManyEventsOccurred());
		assertEquals(otherValue, parameter.getValue());
	}

  /**
   * Testando o mtodo {@link SimpleParameter#setValue(Object)}: tentando 
   * modificar o valor de um parmetro. O parmetro no possuia um valor 
   * inicialmente e recebe {@code null}.
   */
	public void testSetValue5() {
		SimpleParameter<Object> parameter = create(null);
		assertNull(parameter.getValue());
		CheckParameterListener<Object> listener = new CheckParameterListener<Object>();
		parameter.addSimpleParameterListener(listener);
		assertFalse(parameter.setValue(null));
		assertEquals(0, listener.getHowManyEventsOccurred());
		assertNull(parameter.getValue());
	}

  /**
   * Testando o mtodo {@link SimpleParameter#resetValue()}. 
   */
	public void testResetValue1() {
		Object value = "Valor do parmetro";
		SimpleParameter<Object> parameter = create(value);
		assertEquals(value, parameter.getValue());
		CheckParameterListener<Object> listener = new CheckParameterListener<Object>();
		listener.allowChangeValue();
		parameter.addSimpleParameterListener(listener);
		Object otherValue = "Outro valor do parmetro";
		assertTrue(parameter.setValue(otherValue));
		assertEquals(1, listener.getHowManyEventsOccurred());
		assertEquals(otherValue, parameter.getValue());
		parameter.resetValue();
		assertEquals(2, listener.getHowManyEventsOccurred());
		assertEquals(value, parameter.getValue());
	}

  /**
   * Testando o mtodo 
   * {@link SimpleParameter#getCommandValue(String, String)}: o parmetro
   * possui um valor.
   */
	public void testGetCommandValue1() {
		Object value = "Valor do parmetro";
		SimpleParameter<Object> parameter = create(value);
		assertEquals(value, parameter.getValue());
		assertEquals(value, parameter.getCommandValue("/", null));
	}

  /**
   * Testando o mtodo 
   * {@link SimpleParameter#getCommandValue(String, String)}: o parmetro
   * no possui valor.
   */
	public void testGetCommandValue2() {
		SimpleParameter<Object> parameter = create(null);
		assertNull(parameter.getValue());
    assertEquals(parameter.getValue(), parameter.getCommandValue("/", null));
	}
  
  /**
   * Testando o mtodo
   * {@link SimpleParameter#getCommandLine(String, String)}: 
   * o padro do comando envolve o nome e o valor do comando.
   */
  public void testGetCommandLine1() {
    String name = "nome";
    String value = "valor";
    String commandLinePattern = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
    SimpleParameter<Object> parameter = create(name, value, commandLinePattern);
    String expectedCommandLine = "--nome valor";
    String foundCommandLine = parameter.getCommandLine(null, "/");
    assertEquals(expectedCommandLine, foundCommandLine);
  }

  /**
   * Testando o mtodo
   * {@link SimpleParameter#getCommandLine(String, String)}: 
   * o padro do comando envolve apenas o nome.
   */
  public void testGetCommandLine2() {
    String name = "nome";
    String value = "valor";
    String commandLinePattern = "<<$NOME_DO_PARAMETRO>>";
    SimpleParameter<Object> parameter = create(name, value, commandLinePattern);
    String expectedCommandLine = "<<nome>>";
    String foundCommandLine = parameter.getCommandLine(null, "/");
    assertEquals(expectedCommandLine, foundCommandLine);
  }

  /**
   * Testando o mtodo
   * {@link SimpleParameter#getCommandLine(String, String)}: 
   * o padro do comando envolve apenas o valor.
   */
  public void testGetCommandLine3() {
    String name = "nome";
    String value = "valor";
    String commandLinePattern = "[$VALOR_DO_PARAMETRO]";
    SimpleParameter<Object> parameter = create(name, value, commandLinePattern);
    String expectedCommandLine = "[valor]";
    String foundCommandLine = parameter.getCommandLine(null, "/");
    assertEquals(expectedCommandLine, foundCommandLine);
  }
  
  /**
   * Testando o mtodo
   * {@link SimpleParameter#getCommandLine(String, String)}: 
   * o padro do comando {@code null}.
   */
  public void testGetCommandLine4() {
    String name = "nome";
    String value = "valor";
    String commandLinePattern = null;
    SimpleParameter<Object> parameter = create(name, value, commandLinePattern);
    String expectedCommandLine = null;
    String foundCommandLine = parameter.getCommandLine(null, "/");
    assertEquals(expectedCommandLine, foundCommandLine);
  }
  
  /**
   * Testando o mtodo {@link SimpleParameter#setEnabled(boolean)}:
   * desabilitando um parmetro que est habilitado.
   */
	public void testSetEnabled1() {
		SimpleParameter<Object> parameter = create();
		assertTrue(parameter.isEnabled());
		CheckParameterListener<Object> listener = new CheckParameterListener<Object>();
		listener.allowChangeEnabledStatus();
		parameter.addSimpleParameterListener(listener);
		assertTrue(parameter.setEnabled(false));
		assertEquals(1, listener.getHowManyEventsOccurred());
		assertFalse(parameter.isEnabled());
	}

  /**
   * Testando o mtodo {@link SimpleParameter#setEnabled(boolean)}:
   * habilitando um parmetro que est desabilitado.
   */
	public void testSetEnabled2() {
		SimpleParameter<Object> parameter = create();
		assertTrue(parameter.isEnabled());
		CheckParameterListener<Object> listener = new CheckParameterListener<Object>();
		listener.allowChangeEnabledStatus();
		parameter.addSimpleParameterListener(listener);
		assertTrue(parameter.setEnabled(false));
		assertEquals(1, listener.getHowManyEventsOccurred());
		assertFalse(parameter.isEnabled());
		assertTrue(parameter.setEnabled(true));
		assertEquals(2, listener.getHowManyEventsOccurred());
		assertTrue(parameter.isEnabled());
	}

  /**
   * Testando o mtodo {@link SimpleParameter#setEnabled(boolean)}:
   * tentando habilitar um parmetro que j est habilitado.
   */
	public void testSetEnabled3() {
		SimpleParameter<Object> parameter = create();
		assertTrue(parameter.isEnabled());
		CheckParameterListener<Object> listener = new CheckParameterListener<Object>();
		assertFalse(parameter.setEnabled(true));
		assertEquals(0, listener.getHowManyEventsOccurred());
		assertTrue(parameter.isEnabled());
	}

  /**
   * Testando o mtodo {@link SimpleParameter#setEnabled(boolean)}:
   * tentando desabilitar um parmetro que j est desabilitado.
   */
	public void testSetEnabled4() {
		SimpleParameter<Object> parameter = create();
		assertTrue(parameter.isEnabled());
		assertTrue(parameter.setEnabled(false));
		assertFalse(parameter.isEnabled());
		CheckParameterListener<Object> listener = new CheckParameterListener<Object>();
		assertFalse(parameter.setEnabled(false));
		assertEquals(0, listener.getHowManyEventsOccurred());
		assertFalse(parameter.isEnabled());
	}

  /**
   * Testando os mtodos {@link SimpleParameter#exportValue()} e 
   * {@link SimpleParameter#importValue(Map)}:
   * o parmetro possui valor.
   */
	public void testExportAndImportValue1() {
		Object value = "Valor do parmetro";
		SimpleParameter<Object> parameter = create(value);
		assertEquals(value, parameter.getValue());
		Map<String, Object> exportedValues = parameter.exportValue();
		Object otherValue = "Outro valor do parmetro";
		assertTrue(parameter.setValue(otherValue));
		assertEquals(otherValue, parameter.getValue());
		parameter.importValue(exportedValues);
		assertEquals(value, parameter.getValue());
	}

  /**
   * Testando os mtodos {@link SimpleParameter#exportValue()} e 
   * {@link SimpleParameter#importValue(Map)}:
   * o parmetro no possui valor.
   */
	public void testExportAndImportValue2() {
		SimpleParameter<Object> parameter = create(null);
		assertNull(parameter.getValue());
		Map<String, Object> exportedValues = parameter.exportValue();
		Object otherValue = "Outro valor do parmetro";
		assertTrue(parameter.setValue(otherValue));
		assertEquals(otherValue, parameter.getValue());
		parameter.importValue(exportedValues);
		assertNull(parameter.getValue());
	}

  /**
   * Testando o mtodo {@link SimpleParameter#importValue(Map)}: tentando
   * importar um {@link java.util.Map} {@code null}.
   */
	public void testImportValue1() {
		SimpleParameter<Object> parameter = create();
		try {
			parameter.importValue(null);
			fail("No deveria ser possvel importar um mapa nulo.");
		} catch (IllegalArgumentException e) {
		}
	}

  /**
   * Testando a serializao/desserializao de um parmetro.
   * 
   * @throws java.io.IOException Bug se ocorrer.
   * @throws ClassNotFoundException Bug se ocorrer.
   */
	public void testSerialize1() throws IOException,
			ClassNotFoundException {
		SimpleParameter<Object> outputParameter = create();
    SimpleParameterListener<Object> listener = new MyParameterListener(2);
    outputParameter.addSimpleParameterListener(listener);
		ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
		ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
		objectOutput.writeObject(outputParameter);
		ByteArrayInputStream inputSource = new ByteArrayInputStream(
				outputSource.toByteArray());
		ObjectInputStream objectInput = new ObjectInputStream(inputSource);
		SimpleParameter<?> inputParameter = (SimpleParameter) 
      objectInput.readObject();
		assertEquals(outputParameter, inputParameter);
	}
  
  /**
   * Um listener de parmetro serializvel e com identificador.
   */
  static class MyParameterListener  implements SimpleParameterListener<Object>, 
  Serializable {

    /**
     * O identificador
     */
    private int id;
    
    /**
     * Criando o listener.
     * 
     * @param id O identificador.
     */
    public MyParameterListener(int id) {
      this.id = id;
    }

    @Override
    public int hashCode() {
      return id;
    }

    @Override
    public boolean equals(Object obj) {
      if (this == obj) {
        return true;
      }
      if (obj == null) {
        return false;
      }
      if (getClass() != obj.getClass()) {
        return false;
      }
      final MyParameterListener other = (MyParameterListener) obj;
      if (id != other.id) {
        return false;
      }
      return true;
    }

    public void capabilityWasChanged(SimpleParameter<Object> parameter) {
    }

    public void defaultValueWasChanged(SimpleParameter<Object> parameter) {
    }

    public void labelWasChanged(SimpleParameter<Object> parameter) {
    }
    
    public void valueWasChanged(SimpleParameter<Object> parameter) {
    }

    public void visibilityWasChanged(SimpleParameter<Object> parameter) {
    }
  }
}
