package csbase.logic.algorithms.parameters;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Random;

import junit.framework.TestCase;
import csbase.exception.ParseException;

/**
 * Testes unitrios para a classe {@link TableParameter}.
 * 
 * @author lmoreira
 */
public final class TableParameterTest extends TestCase {

  /**
   * Testando criar um parmetro do tipo tabela obrigatrio.
   */
  public void testTableParameter1() {
    String name = "Nome_do_parametro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
    List<MockColumn> columns = createMockColumns();
    boolean delimitTable = false;
    boolean delimitRows = false;
    int rowCount = 10;
    TableParameter parameter =
      new TableParameter(name, label, description, isOptional,
        commandLinePattern, columns, delimitTable, delimitRows, rowCount, null);
    boolean hasFixedRowCount = true;
    assertEquals(name, label, description, isOptional, commandLinePattern,
      columns, delimitTable, delimitRows, rowCount, hasFixedRowCount, parameter);
  }

  /**
   * Testando criar um parmetro do tipo tabela opcional.
   */
  public void testTableParameter2() {
    String name = "Nome_do_parametro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = true;
    String commandLinePattern = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
    List<MockColumn> columns = createMockColumns();
    boolean delimitTable = false;
    boolean delimitRows = false;
    int rowCount = 10;
    TableParameter parameter =
      new TableParameter(name, label, description, isOptional,
        commandLinePattern, columns, delimitTable, delimitRows, rowCount, null);
    boolean hasFixedRowCount = true;
    assertEquals(name, label, description, isOptional, commandLinePattern,
      columns, delimitTable, delimitRows, rowCount, hasFixedRowCount, parameter);
  }

  /**
   * Testando criar um parmetro do tipo tabela com nome nulo.
   */
  public void testTableParameter3() {
    String name = null;
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = true;
    String commandLinePattern = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
    List<MockColumn> columns = createMockColumns();
    boolean delimitTable = false;
    boolean delimitRows = false;
    int rowCount = 10;
    try {
      new TableParameter(name, label, description, isOptional,
        commandLinePattern, columns, delimitTable, delimitRows, rowCount, null);
      fail("Esperada IllegalArgumentException");
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um parmetro do tipo tabela com rtulo nulo.
   */
  public void testTableParameter4() {
    String name = "Nome_do_parametro";
    String label = null;
    String description = "Descrio do parmetro";
    boolean isOptional = true;
    String commandLinePattern = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
    List<MockColumn> columns = createMockColumns();
    boolean delimitTable = false;
    boolean delimitRows = false;
    int rowCount = 10;
    try {
      new TableParameter(name, label, description, isOptional,
        commandLinePattern, columns, delimitTable, delimitRows, rowCount, null);
      fail("Esperada IllegalArgumentException");
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um parmetro do tipo tabela com descrio nula.
   */
  public void testTableParameter5() {
    String name = "Nome_do_parametro";
    String label = "Rtulo do parmetro";
    String description = null;
    boolean isOptional = true;
    String commandLinePattern = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
    List<MockColumn> columns = createMockColumns();
    boolean delimitTable = false;
    boolean delimitRows = false;
    int rowCount = 10;
    try {
      new TableParameter(name, label, description, isOptional,
        commandLinePattern, columns, delimitTable, delimitRows, rowCount, null);
      fail("Esperada IllegalArgumentException");
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um parmetro do tipo tabela com formato da linha de comando
   * nulo.
   */
  public void testTableParameter6() {
    String name = "Nome_do_parametro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = null;
    List<MockColumn> columns = createMockColumns();
    boolean delimitTable = false;
    boolean delimitRows = false;
    int rowCount = 10;
    TableParameter parameter =
      new TableParameter(name, label, description, isOptional,
        commandLinePattern, columns, delimitTable, delimitRows, rowCount, null);
    boolean hasFixedRowCount = true;
    assertEquals(name, label, description, isOptional, commandLinePattern,
      columns, delimitTable, delimitRows, rowCount, hasFixedRowCount, parameter);
  }

  /**
   * Testando criar um parmetro do tipo tabela opcional com lista de colunas
   * nula.
   */
  public void testTableParameter7() {
    String name = "Nome_do_parametro";
    String label = "Rtulo do parmetro";
    String description = null;
    boolean isOptional = true;
    String commandLinePattern = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
    List<MockColumn> columns = null;
    boolean delimitTable = false;
    boolean delimitRows = false;
    int rowCount = 10;
    try {
      new TableParameter(name, label, description, isOptional,
        commandLinePattern, columns, delimitTable, delimitRows, rowCount, null);
      fail("Esperada IllegalArgumentException");
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um parmetro do tipo tabela opcional com lista de colunas
   * vazia.
   */
  public void testTableParameter8() {
    String name = "Nome_do_parametro";
    String label = "Rtulo do parmetro";
    String description = null;
    boolean isOptional = true;
    String commandLinePattern = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
    List<MockColumn> columns = new LinkedList<MockColumn>();
    boolean delimitTable = false;
    boolean delimitRows = false;
    int rowCount = 10;
    try {
      new TableParameter(name, label, description, isOptional,
        commandLinePattern, columns, delimitTable, delimitRows, rowCount, null);
      fail("Esperada IllegalArgumentException");
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um parmetro do tipo tabela com delimitar tabela verdadeiro.
   */
  public void testTableParameter9() {
    String name = "Nome_do_parametro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
    List<MockColumn> columns = createMockColumns();
    boolean delimitTable = true;
    boolean delimitRows = false;
    int rowCount = 10;
    TableParameter parameter =
      new TableParameter(name, label, description, isOptional,
        commandLinePattern, columns, delimitTable, delimitRows, rowCount, null);
    boolean hasFixedRowCount = true;
    assertEquals(name, label, description, isOptional, commandLinePattern,
      columns, delimitTable, delimitRows, rowCount, hasFixedRowCount, parameter);
  }

  /**
   * Testando criar um parmetro do tipo tabela com delimitar linhas verdadeiro.
   */
  public void testTableParameter10() {
    String name = "Nome_do_parametro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
    List<MockColumn> columns = createMockColumns();
    boolean delimitTable = false;
    boolean delimitRows = true;
    int rowCount = 10;
    TableParameter parameter =
      new TableParameter(name, label, description, isOptional,
        commandLinePattern, columns, delimitTable, delimitRows, rowCount, null);
    boolean hasFixedRowCount = true;
    assertEquals(name, label, description, isOptional, commandLinePattern,
      columns, delimitTable, delimitRows, rowCount, hasFixedRowCount, parameter);
  }

  /**
   * Testando criar um parmetro do tipo tabela com nmero de linhas nulo.
   */
  public void testTableParameter11() {
    String name = "Nome_do_parametro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
    List<MockColumn> columns = createMockColumns();
    boolean delimitTable = false;
    boolean delimitRows = false;
    Integer rowCount = null;
    TableParameter parameter =
      new TableParameter(name, label, description, isOptional,
        commandLinePattern, columns, delimitTable, delimitRows, rowCount, null);
    boolean hasFixedRowCount = false;
    assertEquals(name, label, description, isOptional, commandLinePattern,
      columns, delimitTable, delimitRows, 0, hasFixedRowCount, parameter);
  }

  /**
   * Testando criar um parmetro do tipo tabela com nmero de linhas
   * no-positivo.
   */
  public void testTableParameter12() {
    String name = "Nome_do_parametro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
    List<MockColumn> columns = createMockColumns();
    boolean delimitTable = false;
    boolean delimitRows = false;
    Integer rowCount = 0;
    try {
      new TableParameter(name, label, description, isOptional,
        commandLinePattern, columns, delimitTable, delimitRows, rowCount, null);
      fail();
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando o mtodo
   * {@link TableParameter#addTableParameterListener(TableParameterListener)}.
   */
  public void testAddTableParameterListener1() {
    TableParameter parameter = createMockTableParameter();
    assertEquals(Collections.emptyList(), parameter
      .getTableParameterListeners());
    MockTableParameterListener listener = new MockTableParameterListener();
    parameter.addTableParameterListener(listener);
    assertEquals(Arrays.asList(listener), parameter
      .getTableParameterListeners());
  }

  /**
   * Testando o mtodo
   * {@link TableParameter#addTableParameterListener(TableParameterListener)}
   * com observador nulo.
   */
  public void testAddTableParameteListener2() {
    TableParameter parameter = createMockTableParameter();
    TableParameterListener listener = null;
    try {
      parameter.addTableParameterListener(listener);
      fail();
    }
    catch (IllegalArgumentException e) {
    }
    assertEquals(Collections.emptyList(), parameter
      .getTableParameterListeners());
  }

  /**
   * Testando o mtodo {@link TableParameter#getColumns()}: tentando modificar a
   * lista de colunas retornada diretamente.
   */
  public void testGetColumns1() {
    int expectedColumnCount = 3;
    List<MockColumn> columns = new LinkedList<MockColumn>();
    for (int i = 0; i < expectedColumnCount; i++) {
      columns.add(createMockColumn(Integer.toString(i)));
    }
    TableParameter parameter = createMockTableParameter(columns);
    assertEquals(expectedColumnCount, parameter.getColumnCount());
    try {
      parameter.getColumns().clear();
      fail();
    }
    catch (UnsupportedOperationException e) {
    }
    assertEquals(expectedColumnCount, parameter.getColumnCount());
  }

  /**
   * Testando o mtodo {@link TableParameter#getItemValue(int, int)}.
   */
  public void testGetItemValue1() {
    Random random = new Random();
    int expectedRowCount = 10;
    int expectedColumnCount = 3;
    int[][] defaultValues = new int[expectedRowCount][expectedColumnCount];
    for (int i = 0; i < expectedRowCount; i++) {
      for (int j = 0; j < expectedColumnCount; j++) {
        defaultValues[i][j] = random.nextInt();
      }
    }
    List<MockColumn> columns = new LinkedList<MockColumn>();
    for (int j = 0; j < expectedColumnCount; j++) {
      MockColumn column = createMockColumn(Integer.toString(j), j);
      columns.add(column);
      for (int i = 0; i < expectedRowCount; i++) {
        column.addDefaultValue(i, defaultValues[i][j]);
      }
    }
    TableParameter parameter =
      createMockTableParameter(columns, expectedRowCount);
    for (int i = 0; i < expectedRowCount; i++) {
      for (int j = 0; j < expectedColumnCount; j++) {
        assertEquals(defaultValues[i][j], parameter.getItemValue(i, j));
      }
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#getItemValue(int, int)}: com ndice
   * de linha negativo.
   */
  public void testGetItemValue2() {
    int expectedColumnCount = 3;
    List<MockColumn> columns = new LinkedList<MockColumn>();
    for (int j = 0; j < expectedColumnCount; j++) {
      MockColumn column = createMockColumn(Integer.toString(j), j);
      columns.add(column);
    }
    int expectedRowCount = 5;
    TableParameter parameter =
      createMockTableParameter(columns, expectedRowCount);
    try {
      parameter.getItemValue(-1, 1);
      fail();
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#getItemValue(int, int)}: com ndice
   * de linha maior ou igual ao que o nmero de linhas.
   */
  public void testGetItemValue3() {
    List<MockColumn> columns = new LinkedList<MockColumn>();
    for (int j = 0; j < 3; j++) {
      MockColumn column = createMockColumn(Integer.toString(j), j);
      columns.add(column);
    }
    int expectedRowCount = 5;
    TableParameter parameter =
      createMockTableParameter(columns, expectedRowCount);
    try {
      parameter.getItemValue(expectedRowCount, 1);
      fail();
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#getTableParameterListeners()}:
   * tentando modificar a lista de observadores diretamente.
   */
  public void testGetTableParameterListeners1() {
    TableParameter parameter = createMockTableParameter();
    MockTableParameterListener listener = new MockTableParameterListener();
    parameter.addTableParameterListener(listener);
    try {
      parameter.getTableParameterListeners().clear();
      fail();
    }
    catch (UnsupportedOperationException e) {
    }
    assertEquals(Arrays.asList(listener), parameter
      .getTableParameterListeners());
  }

  /**
   * Testando o mtodo {@link TableParameter#setItemValue(Object,int,int)}:
   * modificando o valor da clula de menor ndices de linha e coluna.
   */
  public void testSetItemValue1() {
    int rowCount = 10;
    List<MockColumn> columns = new LinkedList<MockColumn>();
    int expectedColumnCount = 3;
    for (int j = 0; j < expectedColumnCount; j++) {
      columns.add(createMockColumn(Integer.toString(j), null));
    }
    TableParameter parameter = createMockTableParameter(columns, rowCount);
    for (int i = 0; i < parameter.getRowCount(); i++) {
      for (int j = 0; j < parameter.getColumnCount(); j++) {
        assertNull(parameter.getItemValue(i, j));
      }
    }
    Object newValue = "Novo_Valor";
    CheckParameterListener listener = new CheckParameterListener();
    parameter.addSimpleParameterListener(listener);
    listener.allowChangeValue();
    int rowIndex = 0;
    int columnIndex = 0;
    assertTrue(parameter.setItemValue(newValue, rowIndex, columnIndex));
    assertEquals(1, listener.getHowManyEventsOccurred());
    for (int i = 1; i < parameter.getRowCount(); i++) {
      for (int j = 1; j < parameter.getColumnCount(); j++) {
        assertNull(parameter.getItemValue(i, j));
      }
    }
    assertEquals(newValue, parameter.getItemValue(rowIndex, columnIndex));
  }

  /**
   * Testando o mtodo {@link TableParameter#setItemValue(Object,int,int)}:
   * modificando o valor da clula de maior ndices de linha e coluna.
   */
  public void testSetItemValue2() {
    int rowCount = 10;
    List<MockColumn> columns = new LinkedList<MockColumn>();
    int columnCount = 3;
    for (int j = 0; j < columnCount; j++) {
      columns.add(createMockColumn(Integer.toString(j), null));
    }
    TableParameter parameter = createMockTableParameter(columns, rowCount);
    for (int i = 0; i < parameter.getRowCount(); i++) {
      for (int j = 0; j < parameter.getColumnCount(); j++) {
        assertNull(parameter.getItemValue(i, j));
      }
    }
    Object newValue = "Novo_Valor";
    CheckParameterListener listener = new CheckParameterListener();
    parameter.addSimpleParameterListener(listener);
    listener.allowChangeValue();
    int rowIndex = rowCount - 1;
    int columnIndex = columnCount - 1;
    assertTrue(parameter.setItemValue(newValue, rowIndex, columnIndex));
    assertEquals(1, listener.getHowManyEventsOccurred());
    for (int i = 0; i < parameter.getRowCount() - 1; i++) {
      for (int j = 0; j < parameter.getColumnCount() - 1; j++) {
        assertNull(parameter.getItemValue(i, j));
      }
    }
    assertEquals(newValue, parameter.getItemValue(rowIndex, columnIndex));
  }

  /**
   * Testando o mtodo {@link TableParameter#setItemValue(Object,int,int)}:
   * modificando o valor da clula com {@code null}.
   */
  public void testSetItemValue3() {
    Random random = new Random();
    int rowCount = 10;
    List<MockColumn> columns = new LinkedList<MockColumn>();
    int columnCount = 3;
    int[][] defaultValues = new int[rowCount][columnCount];
    for (int i = 0; i < rowCount; i++) {
      for (int j = 0; j < columnCount; j++) {
        defaultValues[i][j] = random.nextInt();
      }
    }
    for (int j = 0; j < columnCount; j++) {
      MockColumn column = createMockColumn(Integer.toString(j), null);
      columns.add(column);
      for (int i = 0; i < rowCount; i++) {
        column.addDefaultValue(i, defaultValues[i][j]);
      }
    }
    TableParameter parameter = createMockTableParameter(columns, rowCount);
    CheckParameterListener listener = new CheckParameterListener();
    parameter.addSimpleParameterListener(listener);
    listener.allowChangeValue();
    int rowIndex = random.nextInt(rowCount);
    int columnIndex = random.nextInt(columnCount);
    assertTrue(parameter.setItemValue(null, rowIndex, columnIndex));
    assertEquals(1, listener.getHowManyEventsOccurred());
    for (int i = 0; i < rowCount; i++) {
      for (int j = 0; j < columnCount; j++) {
        Object itemValue = parameter.getItemValue(i, j);
        if ((i == rowIndex) && (j == columnIndex)) {
          assertNull(itemValue);
        }
        else {
          assertEquals(defaultValues[i][j], itemValue);
        }
      }
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#setItemValue(Object,int,int)}:
   * modificando o valor da clula que anteriormente estava com {@code null}.
   */
  public void testSetItemValue4() {
    Random random = new Random();
    int rowCount = 10;
    List<MockColumn> columns = new LinkedList<MockColumn>();
    int columnCount = 3;
    for (int j = 0; j < columnCount; j++) {
      MockColumn column = createMockColumn(Integer.toString(j), null);
      columns.add(column);
    }
    TableParameter parameter = createMockTableParameter(columns, rowCount);
    CheckParameterListener listener = new CheckParameterListener();
    parameter.addSimpleParameterListener(listener);
    listener.allowChangeValue();
    int rowIndex = random.nextInt(rowCount);
    int columnIndex = random.nextInt(columnCount);
    Object newValue = "Novo_valor";
    assertTrue(parameter.setItemValue(newValue, rowIndex, columnIndex));
    assertEquals(1, listener.getHowManyEventsOccurred());
    for (int i = 0; i < rowCount; i++) {
      for (int j = 0; j < columnCount; j++) {
        Object itemValue = parameter.getItemValue(i, j);
        if ((i == rowIndex) && (j == columnIndex)) {
          assertEquals(newValue, itemValue);
        }
        else {
          assertNull(itemValue);
        }
      }
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#setItemValue(Object,int,int)}:
   * modificando o valor da clula com o mesmo valor armazenado anteriormente.
   */
  public void testSetItemValue5() {
    Random random = new Random();
    int rowCount = 10;
    List<MockColumn> columns = new LinkedList<MockColumn>();
    int columnCount = 3;
    int[][] defaultValues = new int[rowCount][columnCount];
    for (int i = 0; i < rowCount; i++) {
      for (int j = 0; j < columnCount; j++) {
        defaultValues[i][j] = random.nextInt();
      }
    }
    for (int j = 0; j < columnCount; j++) {
      MockColumn column = createMockColumn(Integer.toString(j), null);
      columns.add(column);
      for (int i = 0; i < rowCount; i++) {
        column.addDefaultValue(i, defaultValues[i][j]);
      }
    }
    TableParameter parameter = createMockTableParameter(columns, rowCount);
    CheckParameterListener listener = new CheckParameterListener();
    parameter.addSimpleParameterListener(listener);
    int rowIndex = random.nextInt(rowCount);
    int columnIndex = random.nextInt(columnCount);
    assertFalse(parameter.setItemValue(defaultValues[rowIndex][columnIndex],
      rowIndex, columnIndex));
    assertEquals(0, listener.getHowManyEventsOccurred());
  }

  /**
   * Testando o mtodo {@link TableParameter#setItemValue(Object,int,int)}:
   * modificando o valor da clula de {@code null} para {@code null}.
   */
  public void testSetItemValue6() {
    Random random = new Random();
    int rowCount = 10;
    List<MockColumn> columns = new LinkedList<MockColumn>();
    int columnCount = 3;
    for (int j = 0; j < columnCount; j++) {
      MockColumn column = createMockColumn(Integer.toString(j), null);
      columns.add(column);
    }
    TableParameter parameter = createMockTableParameter(columns, rowCount);
    CheckParameterListener listener = new CheckParameterListener();
    parameter.addSimpleParameterListener(listener);
    int rowIndex = random.nextInt(rowCount);
    int columnIndex = random.nextInt(columnCount);
    assertFalse(parameter.setItemValue(null, rowIndex, columnIndex));
    assertEquals(0, listener.getHowManyEventsOccurred());
  }

  /**
   * Testando o mtodo {@link TableParameter#setItemValue(Object,int,int)}:
   * utilizando um ndice de linha negativo.
   */
  public void testSetItemValue7() {
    int rowCount = 10;
    List<MockColumn> columns = new LinkedList<MockColumn>();
    int columnCount = 3;
    for (int j = 0; j < columnCount; j++) {
      MockColumn column = createMockColumn(Integer.toString(j), null);
      columns.add(column);
    }
    TableParameter parameter = createMockTableParameter(columns, rowCount);
    CheckParameterListener listener = new CheckParameterListener();
    parameter.addSimpleParameterListener(listener);
    try {
      parameter.setItemValue(null, -1, 0);
      fail();
    }
    catch (IllegalArgumentException e) {
    }
    assertEquals(0, listener.getHowManyEventsOccurred());
  }

  /**
   * Testando o mtodo {@link TableParameter#setItemValue(Object,int,int)}:
   * utilizando um ndice de coluna negativo.
   */
  public void testSetItemValue8() {
    int rowCount = 10;
    List<MockColumn> columns = new LinkedList<MockColumn>();
    int columnCount = 3;
    for (int j = 0; j < columnCount; j++) {
      MockColumn column = createMockColumn(Integer.toString(j), null);
      columns.add(column);
    }
    TableParameter parameter = createMockTableParameter(columns, rowCount);
    CheckParameterListener listener = new CheckParameterListener();
    parameter.addSimpleParameterListener(listener);
    try {
      parameter.setItemValue(null, 0, -1);
      fail();
    }
    catch (IllegalArgumentException e) {
    }
    assertEquals(0, listener.getHowManyEventsOccurred());
  }

  /**
   * Testando o mtodo {@link TableParameter#setItemValue(Object,int,int)}:
   * utilizando um ndice de linha maior do que o mximo permitido.
   */
  public void testSetItemValue9() {
    int rowCount = 10;
    List<MockColumn> columns = new LinkedList<MockColumn>();
    int columnCount = 3;
    for (int j = 0; j < columnCount; j++) {
      MockColumn column = createMockColumn(Integer.toString(j), null);
      columns.add(column);
    }
    TableParameter parameter = createMockTableParameter(columns, rowCount);
    CheckParameterListener listener = new CheckParameterListener();
    parameter.addSimpleParameterListener(listener);
    try {
      parameter.setItemValue(null, rowCount, 0);
      fail();
    }
    catch (IllegalArgumentException e) {
    }
    assertEquals(0, listener.getHowManyEventsOccurred());
  }

  /**
   * Testando o mtodo {@link TableParameter#setItemValue(Object,int,int)}:
   * utilizando um ndice de coluna maior do que o mximo permitido.
   */
  public void testSetItemValue10() {
    int rowCount = 10;
    List<MockColumn> columns = new LinkedList<MockColumn>();
    int columnCount = 3;
    for (int j = 0; j < columnCount; j++) {
      MockColumn column = createMockColumn(Integer.toString(j), null);
      columns.add(column);
    }
    TableParameter parameter = createMockTableParameter(columns, rowCount);
    CheckParameterListener listener = new CheckParameterListener();
    parameter.addSimpleParameterListener(listener);
    try {
      parameter.setItemValue(null, 0, columnCount);
      fail();
    }
    catch (IllegalArgumentException e) {
    }
    assertEquals(0, listener.getHowManyEventsOccurred());
  }

  /**
   * Testando o mtodo {@link TableParameter#createRow()}: criando uma linha com
   * valor-padro por coluna.
   */
  public void testCreateRow1() {
    Integer rowCount = null;
    List<MockColumn> columns = new LinkedList<MockColumn>();
    for (int i = 0; i < 5; i++) {
      MockColumn column = createMockColumn(Integer.toString(i), i);
      columns.add(column);
    }
    TableParameter parameter = createMockTableParameter(columns, rowCount);
    assertEquals(0, parameter.getRowCount());
    CheckTableParameterListener listener = new CheckTableParameterListener();
    parameter.addTableParameterListener(listener);
    listener.allowCreateRow();
    parameter.createRow();
    assertEquals(1, listener.getHowManyEventsOccurred());
    assertEquals(1, parameter.getRowCount());
    for (int i = 0; i < columns.size(); i++) {
      Column<?> column = columns.get(i);
      assertEquals(column.getDefaultValue(), parameter.getItemValue(0, i));
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#createRow()}: criando uma linha sem
   * valor-padro por coluna.
   */
  public void testCreateRow2() {
    Integer rowCount = null;
    List<MockColumn> columns = new LinkedList<MockColumn>();
    for (int i = 0; i < 5; i++) {
      MockColumn column = createMockColumn(Integer.toString(i), null);
      columns.add(column);
    }
    TableParameter parameter = createMockTableParameter(columns, rowCount);
    assertEquals(0, parameter.getRowCount());
    CheckTableParameterListener listener = new CheckTableParameterListener();
    parameter.addTableParameterListener(listener);
    listener.allowCreateRow();
    parameter.createRow();
    assertEquals(1, listener.getHowManyEventsOccurred());
    assertEquals(1, parameter.getRowCount());
    for (int i = 0; i < columns.size(); i++) {
      assertNull(parameter.getItemValue(0, i));
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#createRow()}: criando linhas com
   * valor-padro por coluna e valores-padro por clula.
   */
  public void testCreateRow3() {
    Integer rowCount = null;
    int[][] expectedValues = new int[][] { { 0, 1 }, { 10, 11 }, { 20, 21 } };
    List<MockColumn> columns = new LinkedList<MockColumn>();
    for (int j = 0; j < 2; j++) {
      MockColumn column = createMockColumn(Integer.toString(j), j);
      columns.add(column);
      for (int i = 1; i < 3; i++) {
        column.addDefaultValue(i, expectedValues[i][j]);
      }
    }
    TableParameter parameter = createMockTableParameter(columns, rowCount);
    assertEquals(0, parameter.getRowCount());
    CheckTableParameterListener listener = new CheckTableParameterListener();
    parameter.addTableParameterListener(listener);
    listener.allowCreateRow();
    parameter.createRow();
    assertEquals(1, listener.getHowManyEventsOccurred());
    parameter.createRow();
    assertEquals(2, listener.getHowManyEventsOccurred());
    parameter.createRow();
    assertEquals(3, listener.getHowManyEventsOccurred());
    assertEquals(3, parameter.getRowCount());
    for (int i = 0; i < parameter.getRowCount(); i++) {
      for (int j = 0; j < columns.size(); j++) {
        assertEquals(expectedValues[i][j], parameter.getItemValue(i, j));
      }
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#removeRow(int)}: removendo uma
   * linha qualquer.
   */
  public void testRemoveRow1() {
    Integer rowCount = null;
    int[][] expectedValues =
      new int[][] { { 0, 1 }, { 10, 11 }, { 20, 21 }, { 30, 31 } };
    List<MockColumn> columns = new LinkedList<MockColumn>();
    for (int j = 0; j < 2; j++) {
      MockColumn column = createMockColumn(Integer.toString(j), j);
      columns.add(column);
      for (int i = 0; i < 4; i++) {
        column.addDefaultValue(i, expectedValues[i][j]);
      }
    }
    TableParameter parameter = createMockTableParameter(columns, rowCount);
    for (int i = 0; i < expectedValues.length; i++) {
      parameter.createRow();
    }
    CheckTableParameterListener listener = new CheckTableParameterListener();
    parameter.addTableParameterListener(listener);
    listener.allowRemoveRow();
    int removedRowIndex = 1;
    parameter.removeRow(removedRowIndex);
    assertEquals(1, listener.getHowManyEventsOccurred());
    assertEquals(3, parameter.getRowCount());
    for (int i = 0; i < parameter.getRowCount(); i++) {
      if (i != removedRowIndex) {
        int rowIndex = (i < removedRowIndex ? i : i + 1);
        for (int j = 0; j < columns.size(); j++) {
          assertEquals(expectedValues[rowIndex][j], parameter
            .getItemValue(i, j));
        }
      }
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#removeRow(int)}: removendo a ltima
   * linha.
   */
  public void testRemoveRow2() {
    Integer rowCount = null;
    int[][] expectedValues =
      new int[][] { { 0, 1 }, { 10, 11 }, { 20, 21 }, { 30, 31 } };
    List<MockColumn> columns = new LinkedList<MockColumn>();
    for (int j = 0; j < 2; j++) {
      MockColumn column = createMockColumn(Integer.toString(j), j);
      columns.add(column);
      for (int i = 0; i < 4; i++) {
        column.addDefaultValue(i, expectedValues[i][j]);
      }
    }
    TableParameter parameter = createMockTableParameter(columns, rowCount);
    for (int i = 0; i < expectedValues.length; i++) {
      parameter.createRow();
    }
    CheckTableParameterListener listener = new CheckTableParameterListener();
    parameter.addTableParameterListener(listener);
    listener.allowRemoveRow();
    parameter.removeRow(parameter.getRowCount() - 1);
    assertEquals(1, listener.getHowManyEventsOccurred());
    assertEquals(3, parameter.getRowCount());
    for (int i = 0; i < parameter.getRowCount(); i++) {
      for (int j = 0; j < columns.size(); j++) {
        assertEquals(expectedValues[i][j], parameter.getItemValue(i, j));
      }
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#removeRow(int)}: removendo a
   * primeira linha.
   */
  public void testRemoveRow3() {
    Integer rowCount = null;
    int[][] expectedValues =
      new int[][] { { 0, 1 }, { 10, 11 }, { 20, 21 }, { 30, 31 } };
    List<MockColumn> columns = new LinkedList<MockColumn>();
    for (int j = 0; j < 2; j++) {
      MockColumn column = createMockColumn(Integer.toString(j), j);
      columns.add(column);
      for (int i = 0; i < 4; i++) {
        column.addDefaultValue(i, expectedValues[i][j]);
      }
    }
    TableParameter parameter = createMockTableParameter(columns, rowCount);
    for (int i = 0; i < expectedValues.length; i++) {
      parameter.createRow();
    }
    CheckTableParameterListener listener = new CheckTableParameterListener();
    parameter.addTableParameterListener(listener);
    listener.allowRemoveRow();
    parameter.removeRow(0);
    assertEquals(1, listener.getHowManyEventsOccurred());
    assertEquals(3, parameter.getRowCount());
    for (int i = 0; i < parameter.getRowCount(); i++) {
      for (int j = 0; j < columns.size(); j++) {
        assertEquals(expectedValues[i + 1][j], parameter.getItemValue(i, j));
      }
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#removeRow(int)}: tentando remover
   * com ndice de linha negativo.
   */
  public void testRemoveRow4() {
    Integer rowCount = null;
    int[][] expectedValues =
      new int[][] { { 0, 1 }, { 10, 11 }, { 20, 21 }, { 30, 31 } };
    List<MockColumn> columns = new LinkedList<MockColumn>();
    for (int j = 0; j < 2; j++) {
      MockColumn column = createMockColumn(Integer.toString(j), j);
      columns.add(column);
      for (int i = 0; i < 4; i++) {
        column.addDefaultValue(i, expectedValues[i][j]);
      }
    }
    TableParameter parameter = createMockTableParameter(columns, rowCount);
    for (int i = 0; i < expectedValues.length; i++) {
      parameter.createRow();
    }
    CheckTableParameterListener listener = new CheckTableParameterListener();
    parameter.addTableParameterListener(listener);
    try {
      parameter.removeRow(-1);
      fail();
    }
    catch (IllegalArgumentException e) {
    }
    assertEquals(0, listener.getHowManyEventsOccurred());
    assertEquals(4, parameter.getRowCount());
    for (int i = 0; i < parameter.getRowCount(); i++) {
      for (int j = 0; j < columns.size(); j++) {
        assertEquals(expectedValues[i][j], parameter.getItemValue(i, j));
      }
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#removeRow(int)}: tentando remover
   * com ndice maior do que o ltimo ndice.
   */
  public void testRemoveRow5() {
    Integer rowCount = null;
    int[][] expectedValues =
      new int[][] { { 0, 1 }, { 10, 11 }, { 20, 21 }, { 30, 31 } };
    List<MockColumn> columns = new LinkedList<MockColumn>();
    for (int j = 0; j < 2; j++) {
      MockColumn column = createMockColumn(Integer.toString(j), j);
      columns.add(column);
      for (int i = 0; i < 4; i++) {
        column.addDefaultValue(i, expectedValues[i][j]);
      }
    }
    TableParameter parameter = createMockTableParameter(columns, rowCount);
    for (int i = 0; i < expectedValues.length; i++) {
      parameter.createRow();
    }
    CheckTableParameterListener listener = new CheckTableParameterListener();
    parameter.addTableParameterListener(listener);
    try {
      parameter.removeRow(parameter.getRowCount());
      fail();
    }
    catch (IllegalArgumentException e) {
    }
    assertEquals(0, listener.getHowManyEventsOccurred());
    assertEquals(4, parameter.getRowCount());
    for (int i = 0; i < parameter.getRowCount(); i++) {
      for (int j = 0; j < columns.size(); j++) {
        assertEquals(expectedValues[i][j], parameter.getItemValue(i, j));
      }
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#removeRow(int)}: tentando remover
   * uma linha de uma tabela vazia.
   */
  public void testRemoveRow6() {
    Integer rowCount = null;
    List<MockColumn> columns = createMockColumns();
    TableParameter parameter = createMockTableParameter(columns, rowCount);
    CheckTableParameterListener listener = new CheckTableParameterListener();
    parameter.addTableParameterListener(listener);
    try {
      parameter.removeRow(0);
      fail();
    }
    catch (IllegalArgumentException e) {
    }
    assertEquals(0, listener.getHowManyEventsOccurred());
    assertEquals(0, parameter.getRowCount());
  }

  /**
   * Testando o mtodo {@link TableParameter#getCommandValue(String, String)}:
   * com delimitar tabela verdadeiro e delimitar linhas verdadeiro.
   */
  public void testGetCommandValue1() {
    Column<Integer> column1 =
      createCustomCommandColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomCommandColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    column1.addDefaultValue(0, 1);
    column1.addDefaultValue(1, 3);
    column1.addDefaultValue(2, 5);
    column2.addDefaultValue(0, 2);
    column2.addDefaultValue(1, 4);
    column2.addDefaultValue(2, 6);
    int rowCount = 3;
    TableParameter parameter =
      createMockTableParameter(Arrays
        .<Column<Integer>> asList(column1, column2), rowCount, true, true);
    String commandLineValue = "{{A1,B2},{A3,B4},{A5,B6}}";
    assertEquals(commandLineValue, parameter.getCommandValue(null, "/"));
  }

  /**
   * Testando o mtodo {@link TableParameter#getCommandValue(String, String)}:
   * com delimitar tabela falso e delimitar linhas verdadeiro.
   */
  public void testGetCommandValue2() {
    Column<Integer> column1 =
      createCustomCommandColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomCommandColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    column1.addDefaultValue(0, 1);
    column1.addDefaultValue(1, 3);
    column1.addDefaultValue(2, 5);
    column2.addDefaultValue(0, 2);
    column2.addDefaultValue(1, 4);
    column2.addDefaultValue(2, 6);
    int rowCount = 3;
    TableParameter parameter =
      createMockTableParameter(Arrays
        .<Column<Integer>> asList(column1, column2), rowCount, false, true);
    String commandLineValue = "{A1,B2},{A3,B4},{A5,B6}";
    assertEquals(commandLineValue, parameter.getCommandValue(null, "/"));
  }

  /**
   * Testando o mtodo {@link TableParameter#getCommandValue(String, String)}:
   * com delimitar tabela verdadeiro e delimitar linhas falso.
   */
  public void testGetCommandValue3() {
    Column<Integer> column1 =
      createCustomCommandColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomCommandColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    column1.addDefaultValue(0, 1);
    column1.addDefaultValue(1, 3);
    column1.addDefaultValue(2, 5);
    column2.addDefaultValue(0, 2);
    column2.addDefaultValue(1, 4);
    column2.addDefaultValue(2, 6);
    int rowCount = 3;
    TableParameter parameter =
      createMockTableParameter(Arrays
        .<Column<Integer>> asList(column1, column2), rowCount, true, false);
    String commandLineValue = "{A1,B2,A3,B4,A5,B6}";
    assertEquals(commandLineValue, parameter.getCommandValue(null, "/"));
  }

  /**
   * Testando o mtodo {@link TableParameter#getCommandValue(String, String)}:
   * com delimitar tabela falso e delimitar linhas falso.
   */
  public void testGetCommandValue4() {
    Column<Integer> column1 =
      createCustomCommandColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomCommandColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    column1.addDefaultValue(0, 1);
    column1.addDefaultValue(1, 3);
    column1.addDefaultValue(2, 5);
    column2.addDefaultValue(0, 2);
    column2.addDefaultValue(1, 4);
    column2.addDefaultValue(2, 6);
    int rowCount = 3;
    TableParameter parameter =
      createMockTableParameter(Arrays
        .<Column<Integer>> asList(column1, column2), rowCount, false, false);
    String commandLineValue = "A1,B2,A3,B4,A5,B6";
    assertEquals(commandLineValue, parameter.getCommandValue(null, "/"));
  }

  /**
   * Testando o mtodo {@link TableParameter#getCommandValue(String, String)}:
   * sem linhas.
   */
  public void testGetCommandValue5() {
    Column<Integer> column1 =
      createCustomCommandColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomCommandColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    TableParameter parameter =
      createMockTableParameter(Arrays
        .<Column<Integer>> asList(column1, column2), null, false, false);
    String commandLineValue = null;
    assertEquals(commandLineValue, parameter.getCommandValue(null, "/"));
  }

  /**
   * Testando o mtodo {@link TableParameter#getValueAsText()}.
   */
  public void testGetValueAsText1() {
    Column<Integer> column1 =
      createCustomValueAsTextColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomValueAsTextColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    column1.addDefaultValue(0, 1);
    column1.addDefaultValue(1, 3);
    column1.addDefaultValue(2, 5);
    column2.addDefaultValue(0, 2);
    column2.addDefaultValue(1, 4);
    column2.addDefaultValue(2, 6);
    int rowCount = 3;
    TableParameter parameter =
      createMockTableParameter(Arrays
        .<Column<Integer>> asList(column1, column2), rowCount, false, false);
    String valueAsText = "{{A1,B2},{A3,B4},{A5,B6}}";
    assertEquals(valueAsText, parameter.getValueAsText());
  }

  /**
   * Testando o mtodo {@link TableParameter#getValueAsText()}: com tabela
   * vazia.
   */
  public void testGetValueAsText2() {
    Column<Integer> column1 =
      createCustomValueAsTextColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomValueAsTextColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    column1.addDefaultValue(0, 1);
    column1.addDefaultValue(1, 3);
    column1.addDefaultValue(2, 5);
    column2.addDefaultValue(0, 2);
    column2.addDefaultValue(1, 4);
    column2.addDefaultValue(2, 6);
    TableParameter parameter =
      createMockTableParameter(Arrays
        .<Column<Integer>> asList(column1, column2), null, false, false);
    String valueAsText = null;
    assertEquals(valueAsText, parameter.getValueAsText());
  }

  /**
   * Testando o mtodo {@link TableParameter#getValueAsText()}: com linha vazia.
   */
  public void testGetValueAsText3() {
    Column<Integer> column1 =
      createCustomValueAsTextColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomValueAsTextColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    column1.addDefaultValue(0, 1);
    column1.addDefaultValue(2, 5);
    column2.addDefaultValue(0, 2);
    column2.addDefaultValue(2, 6);
    int rowCount = 3;
    TableParameter parameter =
      createMockTableParameter(Arrays
        .<Column<Integer>> asList(column1, column2), rowCount, false, false);
    String valueAsText = "{{A1,B2},{,},{A5,B6}}";
    assertEquals(valueAsText, parameter.getValueAsText());
  }

  /**
   * Testando o mtodo {@link TableParameter#getValueAsText()}: com a primeira
   * linha vazia.
   */
  public void testGetValueAsText4() {
    Column<Integer> column1 =
      createCustomValueAsTextColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomValueAsTextColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    column1.addDefaultValue(1, 3);
    column1.addDefaultValue(2, 5);
    column2.addDefaultValue(1, 4);
    column2.addDefaultValue(2, 6);
    int rowCount = 3;
    TableParameter parameter =
      createMockTableParameter(Arrays
        .<Column<Integer>> asList(column1, column2), rowCount, false, false);
    String valueAsText = "{{,},{A3,B4},{A5,B6}}";
    assertEquals(valueAsText, parameter.getValueAsText());
  }

  /**
   * Testando o mtodo {@link TableParameter#getValueAsText()}: com a ltima
   * linha vazia.
   */
  public void testGetValueAsText5() {
    Column<Integer> column1 =
      createCustomValueAsTextColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomValueAsTextColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    column1.addDefaultValue(0, 1);
    column1.addDefaultValue(1, 3);
    column2.addDefaultValue(0, 2);
    column2.addDefaultValue(1, 4);
    int rowCount = 3;
    TableParameter parameter =
      createMockTableParameter(Arrays
        .<Column<Integer>> asList(column1, column2), rowCount, false, false);
    String valueAsText = "{{A1,B2},{A3,B4},{,}}";
    assertEquals(valueAsText, parameter.getValueAsText());
  }

  /**
   * Testando o mtodo {@link TableParameter#getValueAsText()}: com a primeira e
   * a ltima clulas vazias.
   */
  public void testGetValueAsText6() {
    Column<Integer> column1 =
      createCustomValueAsTextColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomValueAsTextColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    column1.addDefaultValue(1, 3);
    column1.addDefaultValue(2, 5);
    column2.addDefaultValue(1, 4);
    column2.addDefaultValue(0, 2);
    int rowCount = 3;
    TableParameter parameter =
      createMockTableParameter(Arrays
        .<Column<Integer>> asList(column1, column2), rowCount, false, false);
    String valueAsText = "{{,B2},{A3,B4},{A5,}}";
    assertEquals(valueAsText, parameter.getValueAsText());
  }

  /**
   * Testando o mtodo {@link TableParameter#setValueAsText(String)}.
   * 
   * @throws ParseException
   */
  public void testSetValueAsText1() throws ParseException {
    Column<Integer> column1 =
      createCustomValueAsTextColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomValueAsTextColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    List<Column<Integer>> columns =
      Arrays.<Column<Integer>> asList(column1, column2);
    TableParameter parameter =
      createMockTableParameter(columns, null, false, false);
    String valueAsText = "{{A1,B2},{A3,B4},{A5,B6}}";
    parameter.setValueAsText(valueAsText);
    int rowCount = 3;
    int columnCount = columns.size();
    Integer[][] values = { { 1, 2 }, { 3, 4 }, { 5, 6 } };
    assertEquals(rowCount, parameter.getRowCount());
    assertEquals(columnCount, parameter.getColumnCount());
    for (int i = 0; i < rowCount; i++) {
      for (int j = 0; j < columnCount; j++) {
        assertEquals(values[i][j], parameter.getItemValue(i, j));
      }
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#setValueAsText(String)}: com tabela
   * vazia.
   * 
   * @throws ParseException
   */
  public void testSetValueAsText2() throws ParseException {
    Column<Integer> column1 =
      createCustomValueAsTextColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomValueAsTextColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    List<Column<Integer>> columns =
      Arrays.<Column<Integer>> asList(column1, column2);
    column1.addDefaultValue(0, 1);
    column1.addDefaultValue(1, 3);
    column1.addDefaultValue(2, 5);
    column2.addDefaultValue(0, 2);
    column2.addDefaultValue(1, 4);
    column2.addDefaultValue(2, 6);
    TableParameter parameter =
      createMockTableParameter(columns, 3, false, false);
    String valueAsText = null;
    parameter.setValueAsText(valueAsText);
    int rowCount = 0;
    int columnCount = columns.size();
    assertEquals(rowCount, parameter.getRowCount());
    assertEquals(columnCount, parameter.getColumnCount());
  }

  /**
   * Testando o mtodo {@link TableParameter#setValueAsText(String)}: com linha
   * vazia.
   * 
   * @throws ParseException
   */
  public void testSetValueAsText3() throws ParseException {
    Column<Integer> column1 =
      createCustomValueAsTextColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomValueAsTextColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    List<Column<Integer>> columns =
      Arrays.<Column<Integer>> asList(column1, column2);
    TableParameter parameter =
      createMockTableParameter(columns, null, false, false);
    String valueAsText = "{{A1,B2},{,},{A5,B6}}";
    parameter.setValueAsText(valueAsText);
    int rowCount = 3;
    int columnCount = columns.size();
    Integer[][] values = { { 1, 2 }, { null, null }, { 5, 6 } };
    assertEquals(rowCount, parameter.getRowCount());
    assertEquals(columnCount, parameter.getColumnCount());
    for (int i = 0; i < rowCount; i++) {
      for (int j = 0; j < columnCount; j++) {
        assertEquals(values[i][j], parameter.getItemValue(i, j));
      }
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#setValueAsText(String)}: com a
   * primeira linha vazia.
   * 
   * @throws ParseException
   */
  public void testSetValueAsText4() throws ParseException {
    Column<Integer> column1 =
      createCustomValueAsTextColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomValueAsTextColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    List<Column<Integer>> columns =
      Arrays.<Column<Integer>> asList(column1, column2);
    TableParameter parameter =
      createMockTableParameter(columns, null, false, false);
    String valueAsText = "{{,},{A3,B4},{A5,B6}}";
    parameter.setValueAsText(valueAsText);
    int rowCount = 3;
    int columnCount = columns.size();
    Integer[][] values = { { null, null }, { 3, 4 }, { 5, 6 } };
    assertEquals(rowCount, parameter.getRowCount());
    assertEquals(columnCount, parameter.getColumnCount());
    for (int i = 0; i < rowCount; i++) {
      for (int j = 0; j < columnCount; j++) {
        assertEquals(values[i][j], parameter.getItemValue(i, j));
      }
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#setValueAsText(String)}: com a
   * ltima linha vazia.
   * 
   * @throws ParseException
   */
  public void testSetValueAsText5() throws ParseException {
    Column<Integer> column1 =
      createCustomValueAsTextColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomValueAsTextColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    List<Column<Integer>> columns =
      Arrays.<Column<Integer>> asList(column1, column2);
    TableParameter parameter =
      createMockTableParameter(columns, null, false, false);
    String valueAsText = "{{A1,B2},{A3,B4},{,}}";
    parameter.setValueAsText(valueAsText);
    int rowCount = 3;
    int columnCount = columns.size();
    Integer[][] values = { { 1, 2 }, { 3, 4 }, { null, null } };
    assertEquals(rowCount, parameter.getRowCount());
    assertEquals(columnCount, parameter.getColumnCount());
    for (int i = 0; i < rowCount; i++) {
      for (int j = 0; j < columnCount; j++) {
        assertEquals(values[i][j], parameter.getItemValue(i, j));
      }
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#setValueAsText(String)}: com a
   * primeira e a ltima clulas vazias.
   * 
   * @throws ParseException
   */
  public void testSetValueAsText6() throws ParseException {
    Column<Integer> column1 =
      createCustomValueAsTextColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomValueAsTextColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    List<Column<Integer>> columns =
      Arrays.<Column<Integer>> asList(column1, column2);
    TableParameter parameter =
      createMockTableParameter(columns, null, false, false);
    String valueAsText = "{{,B2},{A3,B4},{A5,}}";
    parameter.setValueAsText(valueAsText);
    int rowCount = 3;
    int columnCount = columns.size();
    Integer[][] values = { { null, 2 }, { 3, 4 }, { 5, null } };
    assertEquals(rowCount, parameter.getRowCount());
    assertEquals(columnCount, parameter.getColumnCount());
    for (int i = 0; i < rowCount; i++) {
      for (int j = 0; j < columnCount; j++) {
        assertEquals(values[i][j], parameter.getItemValue(i, j));
      }
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#setValueAsText(String)}: com um
   * texto que no representa uma tabela.
   */
  public void testSetValueAsText7() {
    Column<Integer> column1 =
      createCustomValueAsTextColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomValueAsTextColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    List<Column<Integer>> columns =
      Arrays.<Column<Integer>> asList(column1, column2);
    TableParameter parameter =
      createMockTableParameter(columns, null, false, false);
    String valueAsText = "no representa uma tabela";
    try {
      parameter.setValueAsText(valueAsText);
      fail();
    }
    catch (ParseException e) {
    }
    int rowCount = 0;
    int columnCount = columns.size();
    assertEquals(rowCount, parameter.getRowCount());
    assertEquals(columnCount, parameter.getColumnCount());
  }

  /**
   * Testando o mtodo {@link TableParameter#setValueAsText(String)}: com um
   * texto que possui uma clula com um valor que no representa um tem.
   */
  public void testSetValueAsText8() {
    Column<Integer> column1 =
      createCustomValueAsTextColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomValueAsTextColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    List<Column<Integer>> columns =
      Arrays.<Column<Integer>> asList(column1, column2);
    TableParameter parameter =
      createMockTableParameter(columns, null, false, false);
    String valueAsText = "{{A1,B2},{tem_invlido,B4},{A5,B6}}";
    try {
      parameter.setValueAsText(valueAsText);
      fail();
    }
    catch (ParseException e) {
    }
    int rowCount = 0;
    int columnCount = columns.size();
    assertEquals(rowCount, parameter.getRowCount());
    assertEquals(columnCount, parameter.getColumnCount());
  }

  /**
   * Testando o mtodo {@link TableParameter#setValueAsText(String)}: com um
   * texto que possui uma clula com um valor que no representa uma linha.
   */
  public void testSetValueAsText9() {
    Column<Integer> column1 =
      createCustomValueAsTextColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomValueAsTextColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    List<Column<Integer>> columns =
      Arrays.<Column<Integer>> asList(column1, column2);
    TableParameter parameter =
      createMockTableParameter(columns, null, false, false);
    String valueAsText = "{{A1,B2},linha_invlida,{A5,B6}}";
    try {
      parameter.setValueAsText(valueAsText);
      fail();
    }
    catch (ParseException e) {
    }
    int rowCount = 0;
    int columnCount = columns.size();
    assertEquals(rowCount, parameter.getRowCount());
    assertEquals(columnCount, parameter.getColumnCount());
  }

  /**
   * Testando o mtodo {@link TableParameter#setValueAsText(String)}: com um
   * texto que possui uma linha que tem uma quantidade de clulas menor do que o
   * nmero de colunas.
   */
  public void testSetValueAsText10() {
    Column<Integer> column1 =
      createCustomValueAsTextColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomValueAsTextColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    List<Column<Integer>> columns =
      Arrays.<Column<Integer>> asList(column1, column2);
    TableParameter parameter =
      createMockTableParameter(columns, null, false, false);
    String valueAsText = "{{A1,B2},{B4},{A5,B6}}";
    try {
      parameter.setValueAsText(valueAsText);
      fail();
    }
    catch (ParseException e) {
    }
    int rowCount = 0;
    int columnCount = columns.size();
    assertEquals(rowCount, parameter.getRowCount());
    assertEquals(columnCount, parameter.getColumnCount());
  }

  /**
   * Testando o mtodo {@link TableParameter#setValueAsText(String)}: com um
   * texto que possui uma linha que tem uma quantidade de clulas maior do que o
   * nmero de colunas.
   */
  public void testSetValueAsText11() {
    Column<Integer> column1 =
      createCustomValueAsTextColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomValueAsTextColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    List<Column<Integer>> columns =
      Arrays.<Column<Integer>> asList(column1, column2);
    TableParameter parameter =
      createMockTableParameter(columns, null, false, false);
    String valueAsText = "{{A1,B2},{A1,B4,},{A5,B6}}";
    try {
      parameter.setValueAsText(valueAsText);
      fail();
    }
    catch (ParseException e) {
    }
    int rowCount = 0;
    int columnCount = columns.size();
    assertEquals(rowCount, parameter.getRowCount());
    assertEquals(columnCount, parameter.getColumnCount());
  }

  /**
   * Testando o mtodo {@link TableParameter#setValueAsText(String)}: com um
   * texto que s possui o abrir chaves da tabela.
   * 
   * @throws ParseException
   */
  public void testSetValueAsText12() throws ParseException {
    Column<Integer> column1 =
      createCustomValueAsTextColumn("id1", null, Arrays.asList(1, 3, 5), Arrays
        .asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomValueAsTextColumn("id2", null, Arrays.asList(2, 4, 6), Arrays
        .asList("B2", "B4", "B6"));
    List<Column<Integer>> columns =
      Arrays.<Column<Integer>> asList(column1, column2);
    column1.addDefaultValue(0, 1);
    column1.addDefaultValue(1, 3);
    column1.addDefaultValue(2, 5);
    column2.addDefaultValue(0, 2);
    column2.addDefaultValue(1, 4);
    column2.addDefaultValue(2, 6);
    int rowCount = 3;
    TableParameter parameter =
      createMockTableParameter(columns, rowCount, false, false);
    String valueAsText = "{}";
    parameter.setValueAsText(valueAsText);
    int columnCount = columns.size();
    assertEquals(0, parameter.getRowCount());
    assertEquals(columnCount, parameter.getColumnCount());
  }

  /**
   * Testando o mtodo {@link TableParameter#exportValue()}.
   */
  public void testExportValue1() {
    Column<Integer> column1 =
      createCustomExportImportColumn("id1", null, Arrays.asList(1, 3, 5),
        Arrays.<Object> asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomExportImportColumn("id2", null, Arrays.asList(2, 4, 6),
        Arrays.<Object> asList("B2", "B4", "B6"));
    column1.addDefaultValue(0, 1);
    column1.addDefaultValue(1, 3);
    column1.addDefaultValue(2, 5);
    column2.addDefaultValue(0, 2);
    column2.addDefaultValue(1, 4);
    column2.addDefaultValue(2, 6);
    int rowCount = 3;
    String name = "tabela";
    TableParameter parameter =
      createMockTableParameter(name, Arrays.<Column<Integer>> asList(column1,
        column2), rowCount, false, false);
    checkExportedValue(parameter.exportValue(), name,
      new Object[] { "A1", "B2" }, new Object[] { "A3", "B4" }, new Object[] {
          "A5", "B6" });
  }

  /**
   * Testando o mtodo {@link TableParameter#exportValue()}: tabela vazia.
   */
  public void testExportValue2() {
    Column<Integer> column1 =
      createCustomExportImportColumn("id1", null, Arrays.asList(1, 3, 5),
        Arrays.<Object> asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomExportImportColumn("id2", null, Arrays.asList(2, 4, 6),
        Arrays.<Object> asList("B2", "B4", "B6"));
    column1.addDefaultValue(0, 1);
    column1.addDefaultValue(1, 3);
    column1.addDefaultValue(2, 5);
    column2.addDefaultValue(0, 2);
    column2.addDefaultValue(1, 4);
    column2.addDefaultValue(2, 6);
    String name = "tabela";
    TableParameter parameter =
      createMockTableParameter(name, Arrays.<Column<Integer>> asList(column1,
        column2), null, false, false);
    checkExportedValue(parameter.exportValue(), name);
  }

  /**
   * Testando o mtodo {@link TableParameter#exportValue()}: com linha vazia.
   */
  public void testExportValue3() {
    Column<Integer> column1 =
      createCustomExportImportColumn("id1", null, Arrays.asList(1, 3, 5),
        Arrays.<Object> asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomExportImportColumn("id2", null, Arrays.asList(2, 4, 6),
        Arrays.<Object> asList("B2", "B4", "B6"));
    column1.addDefaultValue(0, 1);
    column1.addDefaultValue(2, 5);
    column2.addDefaultValue(0, 2);
    column2.addDefaultValue(2, 6);
    int rowCount = 3;
    String name = "tabela";
    TableParameter parameter =
      createMockTableParameter(name, Arrays.<Column<Integer>> asList(column1,
        column2), rowCount, false, false);
    checkExportedValue(parameter.exportValue(), name,
      new Object[] { "A1", "B2" }, new Object[] { null, null }, new Object[] {
          "A5", "B6" });
  }

  /**
   * Testando o mtodo {@link TableParameter#exportValue()}: com a primeira
   * linha vazia.
   */
  public void testExportValue4() {
    Column<Integer> column1 =
      createCustomExportImportColumn("id1", null, Arrays.asList(1, 3, 5),
        Arrays.<Object> asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomExportImportColumn("id2", null, Arrays.asList(2, 4, 6),
        Arrays.<Object> asList("B2", "B4", "B6"));
    column1.addDefaultValue(1, 3);
    column1.addDefaultValue(2, 5);
    column2.addDefaultValue(1, 4);
    column2.addDefaultValue(2, 6);
    int rowCount = 3;
    String name = "tabela";
    TableParameter parameter =
      createMockTableParameter(name, Arrays.<Column<Integer>> asList(column1,
        column2), rowCount, false, false);
    checkExportedValue(parameter.exportValue(), name,
      new Object[] { null, null }, new Object[] { "A3", "B4" }, new Object[] {
          "A5", "B6" });
  }

  /**
   * Testando o mtodo {@link TableParameter#exportValue()}: com a ltima linha
   * vazia.
   */
  public void testExportValue5() {
    Column<Integer> column1 =
      createCustomExportImportColumn("id1", null, Arrays.asList(1, 3, 5),
        Arrays.<Object> asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomExportImportColumn("id2", null, Arrays.asList(2, 4, 6),
        Arrays.<Object> asList("B2", "B4", "B6"));
    column1.addDefaultValue(0, 1);
    column1.addDefaultValue(1, 3);
    column2.addDefaultValue(0, 2);
    column2.addDefaultValue(1, 4);
    int rowCount = 3;
    String name = "tabela";
    TableParameter parameter =
      createMockTableParameter(name, Arrays.<Column<Integer>> asList(column1,
        column2), rowCount, false, false);
    checkExportedValue(parameter.exportValue(), name,
      new Object[] { "A1", "B2" }, new Object[] { "A3", "B4" }, new Object[] {
          null, null });
  }

  /**
   * Testando o mtodo {@link TableParameter#exportValue()}: com a primeira e a
   * ltima clula vazias.
   */
  public void testExportValue6() {
    Column<Integer> column1 =
      createCustomExportImportColumn("id1", null, Arrays.asList(1, 3, 5),
        Arrays.<Object> asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomExportImportColumn("id2", null, Arrays.asList(2, 4, 6),
        Arrays.<Object> asList("B2", "B4", "B6"));
    column1.addDefaultValue(1, 3);
    column1.addDefaultValue(2, 5);
    column2.addDefaultValue(0, 2);
    column2.addDefaultValue(1, 4);
    int rowCount = 3;
    String name = "tabela";
    TableParameter parameter =
      createMockTableParameter(name, Arrays.<Column<Integer>> asList(column1,
        column2), rowCount, false, false);
    checkExportedValue(parameter.exportValue(), name,
      new Object[] { null, "B2" }, new Object[] { "A3", "B4" }, new Object[] {
          "A5", null });
  }

  /**
   * Testando o mtodo {@link TableParameter#importValue(Map)}.
   */
  public void testImportValue1() {
    Column<Integer> column1 =
      createCustomExportImportColumn("id1", null, Arrays.asList(1, 3, 5),
        Arrays.<Object> asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomExportImportColumn("id2", null, Arrays.asList(2, 4, 6),
        Arrays.<Object> asList("B2", "B4", "B6"));
    List<Column<Integer>> columns =
      Arrays.<Column<Integer>> asList(column1, column2);
    String name = "tabela";
    TableParameter parameter =
      createMockTableParameter(name, columns, null, false, false);
    Map<String, Object> importableValue =
      createImportableValue(name, new Object[] { "A1", "B2" }, new Object[] {
          "A3", "B4" }, new Object[] { "A5", "B6" });
    parameter.importValue(importableValue);
    int rowCount = 3;
    int columnCount = columns.size();
    Integer[][] values = { { 1, 2 }, { 3, 4 }, { 5, 6 } };
    assertEquals(rowCount, parameter.getRowCount());
    assertEquals(columnCount, parameter.getColumnCount());
    for (int i = 0; i < rowCount; i++) {
      for (int j = 0; j < columnCount; j++) {
        assertEquals(values[i][j], parameter.getItemValue(i, j));
      }
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#importValue(Map)}: tabela vazia.
   */
  public void testImportValue2() {
    Column<Integer> column1 =
      createCustomExportImportColumn("id1", null, Arrays.asList(1, 3, 5),
        Arrays.<Object> asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomExportImportColumn("id2", null, Arrays.asList(2, 4, 6),
        Arrays.<Object> asList("B2", "B4", "B6"));
    List<Column<Integer>> columns =
      Arrays.<Column<Integer>> asList(column1, column2);
    String name = "tabela";
    TableParameter parameter =
      createMockTableParameter(name, columns, null, false, false);
    Map<String, Object> importableValue = createImportableValue(name);
    parameter.importValue(importableValue);
    int rowCount = 0;
    int columnCount = columns.size();
    assertEquals(rowCount, parameter.getRowCount());
    assertEquals(columnCount, parameter.getColumnCount());
  }

  /**
   * Testando o mtodo {@link TableParameter#importValue(Map)}: com linha vazia.
   */
  public void testImportValue3() {
    Column<Integer> column1 =
      createCustomExportImportColumn("id1", null, Arrays.asList(1, 3, 5),
        Arrays.<Object> asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomExportImportColumn("id2", null, Arrays.asList(2, 4, 6),
        Arrays.<Object> asList("B2", "B4", "B6"));
    List<Column<Integer>> columns =
      Arrays.<Column<Integer>> asList(column1, column2);
    String name = "tabela";
    TableParameter parameter =
      createMockTableParameter(name, columns, null, false, false);
    Map<String, Object> importableValue =
      createImportableValue(name, new Object[] { "A1", "B2" }, new Object[] {
          null, null }, new Object[] { "A5", "B6" });
    parameter.importValue(importableValue);
    int rowCount = 3;
    int columnCount = columns.size();
    Integer[][] values = { { 1, 2 }, { null, null }, { 5, 6 } };
    assertEquals(rowCount, parameter.getRowCount());
    assertEquals(columnCount, parameter.getColumnCount());
    for (int i = 0; i < rowCount; i++) {
      for (int j = 0; j < columnCount; j++) {
        assertEquals(values[i][j], parameter.getItemValue(i, j));
      }
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#importValue(Map)}: com a primeira
   * linha vazia.
   */
  public void testImportValue4() {
    Column<Integer> column1 =
      createCustomExportImportColumn("id1", null, Arrays.asList(1, 3, 5),
        Arrays.<Object> asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomExportImportColumn("id2", null, Arrays.asList(2, 4, 6),
        Arrays.<Object> asList("B2", "B4", "B6"));
    List<Column<Integer>> columns =
      Arrays.<Column<Integer>> asList(column1, column2);
    String name = "tabela";
    TableParameter parameter =
      createMockTableParameter(name, columns, null, false, false);
    Map<String, Object> importableValue =
      createImportableValue(name, new Object[] { null, null }, new Object[] {
          "A3", "B4" }, new Object[] { "A5", "B6" });
    parameter.importValue(importableValue);
    int rowCount = 3;
    int columnCount = columns.size();
    Integer[][] values = { { null, null }, { 3, 4 }, { 5, 6 } };
    assertEquals(rowCount, parameter.getRowCount());
    assertEquals(columnCount, parameter.getColumnCount());
    for (int i = 0; i < rowCount; i++) {
      for (int j = 0; j < columnCount; j++) {
        assertEquals(values[i][j], parameter.getItemValue(i, j));
      }
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#importValue(Map)}: com a ltima
   * linha vazia.
   */
  public void testImportValue5() {
    Column<Integer> column1 =
      createCustomExportImportColumn("id1", null, Arrays.asList(1, 3, 5),
        Arrays.<Object> asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomExportImportColumn("id2", null, Arrays.asList(2, 4, 6),
        Arrays.<Object> asList("B2", "B4", "B6"));
    List<Column<Integer>> columns =
      Arrays.<Column<Integer>> asList(column1, column2);
    String name = "tabela";
    TableParameter parameter =
      createMockTableParameter(name, columns, null, false, false);
    Map<String, Object> importableValue =
      createImportableValue(name, new Object[] { "A1", "B2" }, new Object[] {
          "A3", "B4" }, new Object[] { null, null });
    parameter.importValue(importableValue);
    int rowCount = 3;
    int columnCount = columns.size();
    Integer[][] values = { { 1, 2 }, { 3, 4 }, { null, null } };
    assertEquals(rowCount, parameter.getRowCount());
    assertEquals(columnCount, parameter.getColumnCount());
    for (int i = 0; i < rowCount; i++) {
      for (int j = 0; j < columnCount; j++) {
        assertEquals(values[i][j], parameter.getItemValue(i, j));
      }
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#importValue(Map)}: com a primeira e
   * ltima clulas vazias.
   */
  public void testImportValue6() {
    Column<Integer> column1 =
      createCustomExportImportColumn("id1", null, Arrays.asList(1, 3, 5),
        Arrays.<Object> asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomExportImportColumn("id2", null, Arrays.asList(2, 4, 6),
        Arrays.<Object> asList("B2", "B4", "B6"));
    List<Column<Integer>> columns =
      Arrays.<Column<Integer>> asList(column1, column2);
    String name = "tabela";
    TableParameter parameter =
      createMockTableParameter(name, columns, null, false, false);
    Map<String, Object> importableValue =
      createImportableValue(name, new Object[] { null, "B2" }, new Object[] {
          "A3", "B4" }, new Object[] { "A5", null });
    parameter.importValue(importableValue);
    int rowCount = 3;
    int columnCount = columns.size();
    Integer[][] values = { { null, 2 }, { 3, 4 }, { 5, null } };
    assertEquals(rowCount, parameter.getRowCount());
    assertEquals(columnCount, parameter.getColumnCount());
    for (int i = 0; i < rowCount; i++) {
      for (int j = 0; j < columnCount; j++) {
        assertEquals(values[i][j], parameter.getItemValue(i, j));
      }
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#importValue(Map)}: parmetro nulo.
   */
  public void testImportValue7() {
    TableParameter parameter = createMockTableParameter();
    try {
      parameter.importValue(null);
      fail();
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando o mtodo {@link TableParameter#importValue(Map)}: sem o valor.
   */
  public void testImportValue8() {
    Column<Integer> column1 =
      createCustomExportImportColumn("id1", null, Arrays.asList(1, 3, 5),
        Arrays.<Object> asList("A1", "A3", "A5"));
    Column<Integer> column2 =
      createCustomExportImportColumn("id2", null, Arrays.asList(2, 4, 6),
        Arrays.<Object> asList("B2", "B4", "B6"));
    List<Column<Integer>> columns =
      Arrays.<Column<Integer>> asList(column1, column2);
    column1.addDefaultValue(0, 1);
    column1.addDefaultValue(1, 3);
    column1.addDefaultValue(2, 5);
    column2.addDefaultValue(0, 2);
    column2.addDefaultValue(1, 4);
    column2.addDefaultValue(2, 6);
    String name = "tabela";
    int rowCount = 3;
    int columnCount = columns.size();
    TableParameter parameter =
      createMockTableParameter(name, columns, rowCount, false, false);
    Map<String, Object> importableValue =
      createImportableValue("outra_tabela", new Object[] { "A8", "B7" },
        new Object[] { "A6", "B5" }, new Object[] { "A4", "A3" }, new Object[] {
            "A2", "B1" }, new Object[] { "A0", "B-1" });
    parameter.importValue(importableValue);
    assertEquals(0, parameter.getRowCount());
    assertEquals(columnCount, parameter.getColumnCount());
  }

  /**
   * Verifica o parmetro fornecido possui atributos iguais aos parmetros
   * fornecidos.
   * 
   * @param name O nome (Aceita {@code null}).
   * @param label O rtulo (Aceita {@code null}).
   * @param description A descrio (Aceita {@code null}).
   * @param isOptional Indica se a tabela  opcional (Aceita {@code null}).
   * @param commandLinePattern O padro da linha de comando (Aceita {@code null}
   *        ).
   * @param columns As colunas (Aceita {@code null}).
   * @param delimitTable Indica a tabela deve aparecer delimitada na linha de
   *        comando.
   * @param delimitRows Indica se as linhas da tabela devem aparecer delimitada
   *        na linha de comando.
   * @param rowCount O nmero de linhas da tabela (Aceita {@code null}).
   * @param hasFixedRowCount Indica se o nmero de linhas  fixo.
   * @param parameter O parmetro (No aceita {@code null}).
   */
  private void assertEquals(String name, String label, String description,
    boolean isOptional, String commandLinePattern,
    List<? extends Column> columns, boolean delimitTable, boolean delimitRows,
    int rowCount, boolean hasFixedRowCount, TableParameter parameter) {
    assertEquals(name, parameter.getName());
    assertEquals(label, parameter.getLabel());
    assertEquals(description, parameter.getDescription());
    assertEquals(isOptional, parameter.isOptional());
    assertEquals(commandLinePattern, parameter.getCommandLinePattern());
    assertEquals(columns, parameter.getColumns());
    assertEquals(delimitTable, parameter.delimitTable());
    assertEquals(delimitRows, parameter.delimitRows());
    assertEquals(rowCount, parameter.getRowCount());
    assertEquals(hasFixedRowCount, parameter.hasFixedRowCount());
  }

  /**
   * Verifica se o valor exportado possui os valores esperados.
   * 
   * @param exportedMap O valor exportado.
   * @param name O nome do parmetro (No aceita {@code null}).
   * @param rows As linhas.
   */
  private void checkExportedValue(Map<String, Object> exportedMap, String name,
    Object[]... rows) {
    Object exportedValue = exportedMap.get(name);
    assertNotNull(exportedValue);
    assertTrue(List.class.isAssignableFrom(exportedValue.getClass()));
    List exportedRowList = (List) exportedValue;
    assertEquals(rows.length, exportedRowList.size());
    for (int i = 0; i < rows.length; i++) {
      Object[] row = rows[i];
      Object exportedRowAsObject = exportedRowList.get(i);
      assertTrue(exportedRowAsObject.getClass().isArray());
      Object[] exportedRow = (Object[]) exportedRowAsObject;
      assertTrue(Arrays.deepEquals(row, exportedRow));
    }
    try {
      exportedMap.clear();
      fail();
    }
    catch (UnsupportedOperationException e) {
    }
  }

  /**
   * Cria uma valor importvel.
   * 
   * @param name O nome do parmetro (No aceita {@code null}).
   * @param rows As linhas.
   * 
   * @return .
   */
  private Map<String, Object> createImportableValue(String name,
    Object[]... rows) {
    Map<String, Object> importableValue = new HashMap<String, Object>();
    List<Object[]> rowList = Arrays.asList(rows);
    importableValue.put(name, rowList);
    importableValue = Collections.unmodifiableMap(importableValue);
    checkExportedValue(importableValue, name, rows);
    return importableValue;
  }

  /**
   * Cria uma coluna de valores de linha de comando personalizados.
   * 
   * @param <T> O tipo do dado armazenado na coluna.
   * @param id O identificador da coluna.
   * @param defaultValue O valor-padro da coluna.
   * @param values Os possveis valores da coluna.
   * @param commandValues Os possveis valores da linha de comando da coluna.
   * 
   * @return .
   */
  private <T> Column<T> createCustomCommandColumn(String id, T defaultValue,
    final List<T> values, final List<String> commandValues) {
    return new Column<T>(id, "Rtulo " + id, defaultValue, false, false) {
      @Override
      public String getCommandValue(T itemValue) {
        if (itemValue == null) {
          return null;
        }
        for (int i = 0; i < values.size(); i++) {
          T value = values.get(i);
          if (value.equals(itemValue)) {
            return commandValues.get(i);
          }
        }
        fail();
        throw new IllegalStateException();
      }

      @Override
      public T getItemValueFromText(String itemValue) {
        return null;
      }

      @Override
      public String getItemValueAsText(T itemValue) {
        return null;
      }

      @Override
      public Object getValueToExport(T cellValue) {
        return null;
      }

      @Override
      public T getValueToImport(Object importedValue) {
        return null;
      }
    };
  }

  /**
   * Cria uma coluna de valores como texto personalizados.
   * 
   * @param <T> O tipo do dado armazenado na coluna.
   * @param id O identificador da coluna.
   * @param defaultValue O valor-padro da coluna.
   * @param values Os possveis valores da coluna.
   * @param valuesAsText Os possveis valores como texto da coluna.
   * 
   * @return .
   */
  private <T> Column<T> createCustomValueAsTextColumn(String id,
    T defaultValue, final List<T> values, final List<String> valuesAsText) {
    return new Column<T>(id, "Rtulo " + id, defaultValue, false, false) {
      @Override
      public String getItemValueAsText(T itemValue) {
        if (itemValue == null) {
          return null;
        }
        for (int i = 0; i < values.size(); i++) {
          T value = values.get(i);
          if (value.equals(itemValue)) {
            return valuesAsText.get(i);
          }
        }
        fail();
        throw new IllegalStateException();
      }

      @Override
      public T getItemValueFromText(String itemValue) throws ParseException {
        if (itemValue == null) {
          return null;
        }
        for (int i = 0; i < valuesAsText.size(); i++) {
          String valueAsText = valuesAsText.get(i);
          if (valueAsText.equals(itemValue)) {
            return values.get(i);
          }
        }
        throw new ParseException("tem no encontrado.\n" + "Valor: {0}.\n",
          itemValue);
      }

      @Override
      public String getCommandValue(T itemValue) {
        return null;
      }

      @Override
      public Object getValueToExport(T cellValue) {
        return null;
      }

      @Override
      public T getValueToImport(Object importedValue) {
        return null;
      }
    };
  }

  /**
   * Cria uma coluna de valores como valor para exportao e importao
   * personalizados.
   * 
   * @param <T> O tipo do dado armazenado na coluna.
   * @param id O identificador da coluna.
   * @param defaultValue O valor-padro da coluna.
   * @param values Os possveis valores da coluna.
   * @param exportImportValues Os possveis valores para exportao/importao.
   * 
   * @return .
   */
  private <T> Column<T> createCustomExportImportColumn(String id,
    T defaultValue, final List<T> values, final List<Object> exportImportValues) {
    return new Column<T>(id, "Rtulo " + id, defaultValue, false, false) {
      @Override
      public Object getValueToExport(T itemValue) {
        if (itemValue == null) {
          return null;
        }
        for (int i = 0; i < values.size(); i++) {
          T value = values.get(i);
          if (value.equals(itemValue)) {
            return exportImportValues.get(i);
          }
        }
        fail();
        throw new IllegalStateException();
      }

      @Override
      public T getValueToImport(Object importedValue) {
        if (importedValue == null) {
          return null;
        }
        for (int i = 0; i < exportImportValues.size(); i++) {
          Object exportImportValue = exportImportValues.get(i);
          if (exportImportValue.equals(importedValue)) {
            return values.get(i);
          }
        }
        fail();
        throw new IllegalStateException();
      }

      @Override
      public String getItemValueAsText(T itemValue) {
        return null;
      }

      @Override
      public T getItemValueFromText(String itemValue) {
        return null;
      }

      @Override
      public String getCommandValue(T itemValue) {
        return null;
      }
    };
  }

  /**
   * Cria colunas.
   * 
   * @return .
   */
  private List<MockColumn> createMockColumns() {
    List<MockColumn> columns = new LinkedList<MockColumn>();
    for (int i = 0; i < 10; i++) {
      String id = Integer.toString(i);
      MockColumn mockColumn = createMockColumn(id, i);
      columns.add(mockColumn);
    }
    return columns;
  }

  /**
   * Cria uma coluna com o identificador fornecido.
   * 
   * @param id O identificador (No aceita {@code null}).
   * 
   * @return .
   */
  private MockColumn createMockColumn(String id) {
    MockColumn mockColumn = createMockColumn(id, id);
    return mockColumn;
  }

  /**
   * Cria uma coluna com o identificador e valor-padro fornecidos.
   * 
   * @param id O identificador (No aceita {@code null}).
   * @param defaultValue O valor-padro (Aceita {@code null}).
   * 
   * @return .
   */
  private MockColumn createMockColumn(String id, Object defaultValue) {
    String label = "Rtulo " + id;
    boolean isOptional = true;
    boolean isEditable = true;
    MockColumn mockColumn =
      new MockColumn(id, label, defaultValue, isOptional, isEditable);
    return mockColumn;
  }

  /**
   * Cria um parmetro do tipo tabela qualquer.
   * 
   * @return .
   */
  private TableParameter createMockTableParameter() {
    List<MockColumn> columns = createMockColumns();
    return createMockTableParameter(columns);
  }

  /**
   * Cria um parmetro do tipo tabela com as colunas e o nmero de linhas
   * especificados.
   * 
   * @param columns As colunas (No aceita {@code null}).
   * 
   * @return .
   */
  private TableParameter createMockTableParameter(
    List<? extends Column<?>> columns) {
    int rowCount = 10;
    return createMockTableParameter(columns, rowCount);
  }

  /**
   * Cria um parmetro do tipo tabela com as colunas e o nmero de linhas
   * especificados.
   * 
   * @param columns As colunas (No aceita {@code null}).
   * @param rowCount O nmero de linhas (Aceita {@code null}).
   * 
   * @return .
   */
  private TableParameter createMockTableParameter(
    List<? extends Column<?>> columns, Integer rowCount) {
    boolean delimitTable = false;
    boolean delimitRows = false;
    return createMockTableParameter(columns, rowCount, delimitTable,
      delimitRows);
  }

  /**
   * Cria um parmetro do tipo tabela com as colunas e o nmero de linhas
   * especificados.
   * 
   * @param columns As colunas (No aceita {@code null}).
   * @param rowCount O nmero de linhas (Aceita {@code null}).
   * @param delimitTable Indica se deve delimitar a tabela na linha de comando a
   *        ser gerada.
   * @param delimitRows Indica se deve delimitar as linhas da tabela na linha de
   *        comando a ser gerada.
   * 
   * @return .
   */
  private TableParameter createMockTableParameter(
    List<? extends Column<?>> columns, Integer rowCount, boolean delimitTable,
    boolean delimitRows) {
    String name = "Nome_do_parametro";
    return createMockTableParameter(name, columns, rowCount, delimitTable,
      delimitRows);
  }

  /**
   * Cria um parmetro do tipo tabela com as colunas e o nmero de linhas
   * especificados.
   * 
   * @param name O nome do parmetro (No aceita {@code null}).
   * @param columns As colunas (No aceita {@code null}).
   * @param rowCount O nmero de linhas (Aceita {@code null}).
   * @param delimitTable Indica se deve delimitar a tabela na linha de comando a
   *        ser gerada.
   * @param delimitRows Indica se deve delimitar as linhas da tabela na linha de
   *        comando a ser gerada.
   * 
   * @return .
   */
  private TableParameter createMockTableParameter(String name,
    List<? extends Column<?>> columns, Integer rowCount, boolean delimitTable,
    boolean delimitRows) {
    String commandLinePattern = "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    TableParameter parameter =
      new TableParameter(name, label, description, isOptional,
        commandLinePattern, columns, delimitTable, delimitRows, rowCount, null);
    return parameter;
  }
}
