/*
 * $Author:$
 * $Date:$
 * $Release:$
 */
package csbase.logic.algorithms.parameters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

/**
 * Testes unitrios para a classe {@link TextColumn}.
 *
 * @author lmoreira
 */
public final class TextColumnTest extends TestCase {
  /**
   * Testando criar um coluna de texto opcional.
   */
  public void testTextColumn1() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    String defaultValue = "Valor-padro";
    boolean isOptional = true;
    boolean isEditable = true;
    Integer maxCharacters = new Integer(1234);
    TextColumn column = new TextColumn(id, label, defaultValue, isOptional,
      isEditable, maxCharacters);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertEquals(isOptional, column.isOptional());
    assertEquals(isEditable, column.isEditable());
    assertEquals(maxCharacters, column.getMaxCharacters());
  }

  /**
   * Testando criar uma coluna de texto obrigatria.
   */
  public void testTextColumn2() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    String defaultValue = "Valor-padro";
    boolean isOptional = false;
    boolean isEditable = true;
    Integer maxCharacters = new Integer(1234);
    TextColumn column = new TextColumn(id, label, defaultValue, isOptional,
      isEditable, maxCharacters);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertEquals(isOptional, column.isOptional());
    assertEquals(isEditable, column.isEditable());
    assertEquals(maxCharacters, column.getMaxCharacters());
  }

  /**
   * Testando criar uma coluna de texto sem valor-padro.
   */
  public void testTextColumn3() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    String defaultValue = null;
    boolean isOptional = true;
    boolean isEditable = true;
    Integer maxCharacters = new Integer(1234);
    TextColumn column = new TextColumn(id, label, defaultValue, isOptional,
      isEditable, maxCharacters);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertEquals(isOptional, column.isOptional());
    assertEquals(isEditable, column.isEditable());
    assertEquals(maxCharacters, column.getMaxCharacters());
  }

  /**
   * Testando criar uma coluna de texto sem rtulo.
   */
  public void testTextColumn4() {
    String id = "Identificador da coluna";
    String defaultValue = "Valor-padro";
    boolean isOptional = true;
    boolean isEditable = true;
    Integer maxCharacters = new Integer(1234);
    try {
      new TextColumn(id, null, defaultValue, isOptional, isEditable,
        maxCharacters);
      fail("No deveria ser possvel criar uma coluna com rtulo nulo.");
    }
    catch (IllegalArgumentException e) {}
  }

  /**
   * Testando criar uma coluna de texto sem mximo de caracteres.
   */
  public void testTextColumn5() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    String defaultValue = "Valor-padro";
    boolean isOptional = true;
    boolean isEditable = true;
    TextColumn column = new TextColumn(id, label, defaultValue, isOptional, 
      isEditable, null);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertEquals(isOptional, column.isOptional());
    assertEquals(isEditable, column.isEditable());
    assertNull(column.getMaxCharacters());
  }

  /**
   * Testando criar uma coluna de texto com mximo de caracteres no-positivo.
   */
  public void testTextColumn6() {
    String id = "Identificador da coluna";
    String label = "Nome do parmetro";
    String defaultValue = "Valor-padro";
    boolean isOptional = true;
    boolean isEditable = true;
    Integer maxCharacters = new Integer(0);
    try {
      new TextColumn(id, label, defaultValue, isOptional, isEditable,
        maxCharacters);
      fail(
        "No deveria ser possvel criar uma coluna com quantidade de "+
        "caracters no-positiva.");
    }
    catch (IllegalArgumentException e) {}
  }

  /**
   * Testando criar um coluna de texto no-editvel.
   */
  public void testTextColumn7() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    String defaultValue = "Valor-padro";
    boolean isOptional = true;
    boolean isEditable = false;
    Integer maxCharacters = new Integer(1234);
    TextColumn column = new TextColumn(id, label, defaultValue, isOptional,
      isEditable, maxCharacters);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertEquals(isOptional, column.isOptional());
    assertEquals(isEditable, column.isEditable());
    assertEquals(maxCharacters, column.getMaxCharacters());
  }
  
  /**
   * Testando criar uma coluna de texto sem identificador.
   */
  public void testTextColumn8() {
    String label = "Rtulo da coluna";
    String defaultValue = "Valor-padro";
    boolean isOptional = true;
    boolean isEditable = true;
    Integer maxCharacters = new Integer(1234);
    try {
      new TextColumn(null, label, defaultValue, isOptional, isEditable,
        maxCharacters);
      fail("No deveria ser possvel criar uma coluna com rtulo nulo.");
    }
    catch (IllegalArgumentException e) {}
  }
  
  /**
   * Testando criar um coluna de texto com valores-padro especficos.
   */
  public void testTextColumn9() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    String defaultValue = "Valor-padro";
    boolean isOptional = true;
    boolean isEditable = true;
    Integer maxCharacters = new Integer(1234);
    Map<Integer, String> defaultValuesByRowIndex = 
      new HashMap<Integer, String>();
    defaultValuesByRowIndex.put(1, "Padro 1");
    defaultValuesByRowIndex.put(3, "Padro 3");
    TextColumn column = new TextColumn(id, label, defaultValue, isOptional,
      isEditable, maxCharacters);
    column.addDefaultValue(1, defaultValuesByRowIndex.get(1));
    column.addDefaultValue(3, defaultValuesByRowIndex.get(3));
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertEquals(defaultValuesByRowIndex.get(1), column.getDefaultValue(1));
    assertEquals(defaultValue, column.getDefaultValue(2));
    assertEquals(defaultValuesByRowIndex.get(3), column.getDefaultValue(3));
    assertEquals(defaultValue, column.getDefaultValue(4));
    assertEquals(isOptional, column.isOptional());
    assertEquals(isEditable, column.isEditable());
    assertEquals(maxCharacters, column.getMaxCharacters());
  }
  
  /**
   * Testando criar um coluna de texto com valores-padro especficos sem
   * valor-padro geral.
   */
  public void testTextColumn10() {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    String defaultValue = null;
    boolean isOptional = true;
    boolean isEditable = true;
    Integer maxCharacters = new Integer(1234);
    Map<Integer, String> defaultValuesByRowIndex = 
      new HashMap<Integer, String>();
    defaultValuesByRowIndex.put(1, "Padro 1");
    defaultValuesByRowIndex.put(3, "Padro 3");
    TextColumn column = new TextColumn(id, label, defaultValue, isOptional,
      isEditable, maxCharacters);
    column.addDefaultValue(1, defaultValuesByRowIndex.get(1));
    column.addDefaultValue(3, defaultValuesByRowIndex.get(3));
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertEquals(defaultValuesByRowIndex.get(1), column.getDefaultValue(1));
    assertEquals(defaultValue, column.getDefaultValue(2));
    assertEquals(defaultValuesByRowIndex.get(3), column.getDefaultValue(3));
    assertEquals(defaultValue, column.getDefaultValue(4));
    assertEquals(isOptional, column.isOptional());
    assertEquals(isEditable, column.isEditable());
    assertEquals(maxCharacters, column.getMaxCharacters());
  }
  
  /**
   * Testando serializar e desserializar a coluna.
   * 
   * @throws IOException Bug. 
   * @throws ClassNotFoundException Bug.
   */
  public void testSerialize()
  throws IOException, ClassNotFoundException {
    String id = "Identificador da coluna";
    String label = "Nome da coluna";
    String defaultValue = "Valor-padro";
    boolean isOptional = true;
    boolean isEditable = true;
    Map<Integer, String> defaultValuesByRowIndex = new HashMap<Integer, String>();
    defaultValuesByRowIndex.put(1, "Padro 1");
    defaultValuesByRowIndex.put(2, "Padro 2");
    Integer maxCharacters = new Integer(1);
    TextColumn outputColumn = new TextColumn(id, label, defaultValue,
      isOptional, isEditable, maxCharacters);
    ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
    ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
    objectOutput.writeObject(outputColumn);
    ByteArrayInputStream inputSource =
      new ByteArrayInputStream(outputSource.toByteArray());
    ObjectInputStream objectInput = new ObjectInputStream(inputSource);
    TextColumn inputColumn = (TextColumn)objectInput.readObject();
    assertEquals(outputColumn.getId(), inputColumn.getId());
    assertEquals(outputColumn.getLabel(), inputColumn.getLabel());
    assertEquals(outputColumn.getDefaultValue(0), inputColumn
      .getDefaultValue(0));
    assertEquals(outputColumn.getDefaultValue(1), inputColumn
      .getDefaultValue(1));
    assertEquals(outputColumn.getDefaultValue(2), inputColumn
      .getDefaultValue(2));
    assertEquals(outputColumn.isOptional(), inputColumn.isOptional());
    assertEquals(
      outputColumn.getMaxCharacters(), inputColumn.getMaxCharacters());
  }
}
