package csbase.logic.algorithms.parameters.triggers;

import java.util.Hashtable;
import java.util.List;

import junit.framework.TestCase;
import csbase.logic.FileInfo;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.ExecutionType;
import csbase.logic.algorithms.parameters.MockSimpleParameter;
import csbase.logic.algorithms.parameters.ParameterGroup;
import csbase.logic.algorithms.parameters.SimpleAlgorithmConfigurator;
import csbase.logic.algorithms.parameters.SimpleParameter;
import csbase.logic.algorithms.parameters.conditions.Condition;
import csbase.logic.algorithms.parameters.conditions.SimpleCondition;

/**
 * Testes unitrios para a classe {@link ChangeDefaultValueTrigger}.
 * 
 * @author lmoreira
 */
public final class ChangeDefaultValueTriggerTest extends TestCase {

  /**
   * Pattern padro.
   */
  private static final String CMD_PATTERN = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
  
  /**
   * Testando criar uma instncia de {@link ChangeDefaultValueTrigger}.
   */
  public void testChangeDefaultValueTrigger1() {
    SimpleParameter<?> parameter = createMockParameter();
    Condition condition = createMockCondition();
    Object defaultValue = "Valor-padro do gatilho";
    ChangeDefaultValueTrigger trigger = new ChangeDefaultValueTrigger(
      parameter, condition, defaultValue);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(defaultValue, trigger.getDefaultValue());
  }

  /**
   * Testando criar uma instncia de {@link ChangeDefaultValueTrigger} com
   * parmetro {@code null}.
   */
  public void testChangeDefaultValueTrigger2() {
    Condition condition = createMockCondition();
    Object defaultValue = "Valor-padro do gatilho";
    try {
      new ChangeDefaultValueTrigger(null, condition, defaultValue);
      fail();
    } catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar uma instncia de {@link ChangeDefaultValueTrigger}.
   */
  public void testChangeDefaultValueTrigger3() {
    SimpleParameter<?> parameter = createMockParameter();
    Object defaultValue = "Valor-padro do gatilho";
    try {
      new ChangeDefaultValueTrigger(parameter, null, defaultValue);
      fail();
    } catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar uma instncia de {@link ChangeDefaultValueTrigger} com
   * valor-padro {@code null}.
   */
  public void testChangeDefaultValueTrigger4() {
    SimpleParameter<?> parameter = createMockParameter();
    Condition condition = createMockCondition();
    Object defaultValue = null;
    ChangeDefaultValueTrigger trigger = new ChangeDefaultValueTrigger(
      parameter, condition, defaultValue);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(defaultValue, trigger.getDefaultValue());
  }

  /**
   * Testando o mtodo {@link ChangeDefaultValueTrigger#doAction()} com a
   * condio esperada.
   */
  public void testDoAction1() {
    String parameterName = "Nome do parmetro";
    String parameterDefaultValue = "Valor-padro";
    SimpleParameter<?> parameter = createMockParameter(parameterName,
      parameterDefaultValue);
    Condition condition = new SimpleCondition(parameterName,
      parameterDefaultValue);
    Object newParameterDefaultValue = "Novo valor-padro";
    ChangeDefaultValueTrigger trigger = new ChangeDefaultValueTrigger(
      parameter, condition, newParameterDefaultValue);
    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    ParameterGroup group = createMockParameterGroup();
    group.addParameter(parameter);
    configurator.addGroup(group);
    assertTrue(condition.evaluate(configurator));
    assertTrue(trigger.update(configurator));
    assertEquals(newParameterDefaultValue, parameter.getDefaultValue());
  }

  /**
   * Testando o mtodo {@link ChangeDefaultValueTrigger#doAction()} sem a
   * condio esperada.
   */
  public void testDoAction2() {
    String parameterName = "Nome do parmetro";
    String parameterDefaultValue = "Valor-padro";
    SimpleParameter<?> parameter = createMockParameter(parameterName,
      parameterDefaultValue);
    String anotherParameterDefaultValue = "Outro valor-padro";
    Condition condition = new SimpleCondition(parameterName,
      anotherParameterDefaultValue);
    Object newParameterDefaultValue = "Novo valor-padro";
    ChangeDefaultValueTrigger trigger = new ChangeDefaultValueTrigger(
      parameter, condition, newParameterDefaultValue);
    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    ParameterGroup group = createMockParameterGroup();
    group.addParameter(parameter);
    configurator.addGroup(group);
    assertFalse(condition.evaluate(configurator));
    assertFalse(trigger.update(configurator));
    assertEquals(parameterDefaultValue, parameter.getDefaultValue());
  }

  /**
   * Testando o mtodo {@link ChangeDefaultValueTrigger#equals(Object)} com
   * gatilhos iguais.
   */
  public void testEquals1() {
    SimpleParameter<?> parameter = createMockParameter();
    Condition condition1 = new SimpleCondition("Parmetro 1", "Valor 1");
    Condition condition2 = new SimpleCondition("Parmetro 2", "Valor 2");
    Object defaultValue = "Valor-padro do gatilho";
    ChangeDefaultValueTrigger trigger1 = new ChangeDefaultValueTrigger(
      parameter, condition1, defaultValue);
    ChangeDefaultValueTrigger trigger2 = new ChangeDefaultValueTrigger(
      parameter, condition2, defaultValue);
    assertTrue(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeDefaultValueTrigger#equals(Object)} com
   * parmetros diferentes.
   */
  public void testEquals2() {
    SimpleParameter<?> parameter1 = createMockParameter("Parmetro 1");
    SimpleParameter<?> parameter2 = createMockParameter("Parmetro 2");
    Condition condition = createMockCondition();
    Object defaultValue = "Valor-padro do gatilho";
    ChangeDefaultValueTrigger trigger1 = new ChangeDefaultValueTrigger(
      parameter1, condition, defaultValue);
    ChangeDefaultValueTrigger trigger2 = new ChangeDefaultValueTrigger(
      parameter2, condition, defaultValue);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeDefaultValueTrigger#equals(Object)} com
   * valores-padro diferentes.
   */
  public void testEquals3() {
    SimpleParameter<?> parameter = createMockParameter();
    Condition condition = createMockCondition();
    Object defaultValue1 = "Valor-padro 1";
    Object defaultValue2 = "Valor-padro 2";
    ChangeDefaultValueTrigger trigger1 = new ChangeDefaultValueTrigger(
      parameter, condition, defaultValue1);
    ChangeDefaultValueTrigger trigger2 = new ChangeDefaultValueTrigger(
      parameter, condition, defaultValue2);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeDefaultValueTrigger#equals(Object)} com
   * argumento {@code null}.
   */
  public void testEquals4() {
    SimpleParameter<?> parameter = createMockParameter();
    Condition condition = createMockCondition();
    Object defaultValue = "Valor-padro";
    ChangeDefaultValueTrigger trigger = new ChangeDefaultValueTrigger(
      parameter, condition, defaultValue);
    assertFalse(trigger.equals(null));
  }

  /**
   * Testando o mtodo {@link ChangeDefaultValueTrigger#equals(Object)} com
   * argumento de outra classe.
   */
  public void testEquals5() {
    SimpleParameter<?> parameter = createMockParameter();
    Condition condition = createMockCondition();
    Object defaultValue = "Valor-padro";
    ChangeDefaultValueTrigger trigger = new ChangeDefaultValueTrigger(
      parameter, condition, defaultValue);
    assertFalse(trigger.equals("Outra classe"));
  }

  /**
   * Cria um configurador de algorithms qualquer.
   * 
   * @return .
   */
  private SimpleAlgorithmConfigurator createMockAlgorithmConfigurator() {
    AlgorithmInfo algorithmInfo = new AlgorithmInfo("ID Algoritmo",
      "Algoritmo", "DIR Algoritmo", new Hashtable<String, String>());
    AlgorithmVersionId algorithmVersionId = new AlgorithmVersionId(1, 2, 3);
    AlgorithmVersionInfo algorithmVersionInfo = new AlgorithmVersionInfo(
      algorithmInfo, algorithmVersionId,
      new Hashtable<String, List<FileInfo>>(),
      new Hashtable<String, String>());
    return new SimpleAlgorithmConfigurator(algorithmVersionInfo, null,
      ExecutionType.SIMPLE, null, false, "Comando", null, null, true, CMD_PATTERN);
  }

  /**
   * Cria uma condio qualquer.
   * 
   * @return .
   */
  private Condition createMockCondition() {
    return new SimpleCondition("Nome do parmetro", "Valor qualquer");
  }

  /**
   * Cria um parmetro qualquer.
   * 
   * @return .
   */
  private SimpleParameter<?> createMockParameter() {
    String parameterName = "Nome do parmetro";
    return createMockParameter(parameterName);
  }

  /**
   * Cria um parmetro com o nome especificado.
   * 
   * @param parameterName O nome (No aceita {@code null}).
   * 
   * @return .
   */
  private SimpleParameter<?> createMockParameter(String parameterName) {
    Object defaultValue = "Valor-padro do parmetro";
    return createMockParameter(parameterName, defaultValue);
  }

  /**
   * Cria um parmetro com o nome especificado.
   * 
   * @param parameterName O nome (No aceita {@code null}).
   * @param defaultValue O valor-padro (Aceita {@code null}).
   * 
   * @return .
   */
  private SimpleParameter<?> createMockParameter(String parameterName,
    Object defaultValue) {
    return new MockSimpleParameter(parameterName, "Rtulo do parmetro",
      "Descrio do parmetro", defaultValue, false, "{0}={1}");
  }

  /**
   * Cria um grupo de parmetros qualquer.
   * 
   * @return .
   */
  private ParameterGroup createMockParameterGroup() {
    return new ParameterGroup("Grupo de parmetros");
  }
}
