package csbase.logic.algorithms.parameters.triggers;

import java.util.Hashtable;
import java.util.List;

import junit.framework.TestCase;
import csbase.logic.FileInfo;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.ExecutionType;
import csbase.logic.algorithms.FileParameterValue;
import csbase.logic.algorithms.parameters.FileParameter;
import csbase.logic.algorithms.parameters.FileParameterMode;
import csbase.logic.algorithms.parameters.FileParameterPipeAcceptance;
import csbase.logic.algorithms.parameters.MockFileParameter;
import csbase.logic.algorithms.parameters.ParameterGroup;
import csbase.logic.algorithms.parameters.SimpleAlgorithmConfigurator;
import csbase.logic.algorithms.parameters.conditions.Condition;
import csbase.logic.algorithms.parameters.conditions.SimpleCondition;

/**
 * Testes unitrios para a classe {@link ChangeFileTypeTrigger}.
 * 
 * @author lmoreira
 */
public final class ChangeFileTypeTriggerTest extends TestCase {

  private static final String CMD_PATTERN = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
  
  /**
   * Testando criar objetos da classe {@link ChangeFileTypeTrigger}.
   */
  public void testChangeFileTypeTrigger1() {
    FileParameter fileParameter = createMockFileParameter();
    Condition condition = createMockCondition();
    String fileType = "Texto";
    ChangeFileTypeTrigger trigger =
      new ChangeFileTypeTrigger(fileParameter, condition, fileType);
    assertEquals(fileParameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(fileType, trigger.getFileType());
  }

  /**
   * Testando criar objetos da classe {@link ChangeFileTypeTrigger} com tipo de
   * arquivo {@code null}.
   */
  public void testChangeFileTypeTrigger2() {
    FileParameter fileParameter = createMockFileParameter();
    Condition condition = createMockCondition();
    String fileType = null;
    ChangeFileTypeTrigger trigger =
      new ChangeFileTypeTrigger(fileParameter, condition, fileType);
    assertEquals(fileParameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(fileType, trigger.getFileType());
  }

  /**
   * Testando criar objetos da classe {@link ChangeFileTypeTrigger} com
   * parmetro {@code null}.
   */
  public void testChangeFileTypeTrigger3() {
    Condition condition = createMockCondition();
    String fileType = "Texto";
    try {
      new ChangeFileTypeTrigger(null, condition, fileType);
      fail();
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar objetos da classe {@link ChangeFileTypeTrigger} com condio
   * {@code null}.
   */
  public void testChangeFileTypeTrigger4() {
    FileParameter fileParameter = createMockFileParameter();
    String fileType = "Texto";
    try {
      new ChangeFileTypeTrigger(fileParameter, null, fileType);
      fail();
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando o mtodo {@link ChangeFileTypeTrigger#doAction()} com condio
   * verdadeira.
   */
  public void testDoAction1() {
    String parameterName = "Nome do parmetro";
    String fileType = "Tipo de arquivo";
    FileParameter parameter = createMockFileParameter(parameterName, fileType);
    FileParameterValue fileParameterValue =
      createMockFileParameterValue("Arquivo_do_parametro", fileType);
    parameter.setValue(fileParameterValue);
    Condition condition =
      new SimpleCondition(parameterName, fileParameterValue);
    SimpleAlgorithmConfigurator configurator =
      createMockAlgorithmConfigurator();
    ParameterGroup group = createMockParameterGroup();
    group.addParameter(parameter);
    configurator.addGroup(group);
    assertTrue(condition.evaluate(configurator));
    String newFileType = "Novo Tipo de arquivo";
    ChangeFileTypeTrigger trigger =
      new ChangeFileTypeTrigger(parameter, condition, newFileType);
    assertTrue(trigger.update(configurator));
    assertEquals(newFileType, parameter.getFileType());
  }

  /**
   * Testando o mtodo {@link ChangeFileTypeTrigger#doAction()} com condio
   * falsa.
   */
  public void testDoAction2() {
    String parameterName = "Nome do parmetro";
    String fileType = "Tipo de arquivo";
    FileParameter parameter = createMockFileParameter(parameterName, fileType);
    FileParameterValue fileParameterValue =
      createMockFileParameterValue("Arquivo_do_parametro", fileType);
    parameter.setValue(fileParameterValue);
    Condition condition = new SimpleCondition(parameterName, null);
    SimpleAlgorithmConfigurator configurator =
      createMockAlgorithmConfigurator();
    ParameterGroup group = createMockParameterGroup();
    group.addParameter(parameter);
    configurator.addGroup(group);
    assertFalse(condition.evaluate(configurator));
    String newFileType = "Novo Tipo de arquivo";
    ChangeFileTypeTrigger trigger =
      new ChangeFileTypeTrigger(parameter, condition, newFileType);
    assertFalse(trigger.update(configurator));
    assertEquals(fileType, parameter.getFileType());
  }

  /**
   * Testando o mtodo {@link ChangeFileTypeTrigger#equals(Object)} com
   * parmetros iguais.
   */
  public void testEquals1() {
    String parameterName = "Nome do parmetro";
    String fileType = "Tipo de arquivo";
    FileParameter parameter = createMockFileParameter(parameterName, fileType);
    Condition condition1 =
      new SimpleCondition("Nome do parmetro 1", "Valor 1");
    Condition condition2 =
      new SimpleCondition("Nome do parmetro 2", "Valor 2");
    ChangeFileTypeTrigger trigger1 =
      new ChangeFileTypeTrigger(parameter, condition1, fileType);
    ChangeFileTypeTrigger trigger2 =
      new ChangeFileTypeTrigger(parameter, condition2, fileType);
    assertTrue(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeFileTypeTrigger#equals(Object)} com
   * parmetros diferentes.
   */
  public void testEquals2() {
    String fileType = "Tipo de arquivo";
    String parameterName1 = "Nome do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    FileParameter parameter1 =
      createMockFileParameter(parameterName1, fileType);
    FileParameter parameter2 =
      createMockFileParameter(parameterName2, fileType);
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeFileTypeTrigger trigger1 =
      new ChangeFileTypeTrigger(parameter1, condition, fileType);
    ChangeFileTypeTrigger trigger2 =
      new ChangeFileTypeTrigger(parameter2, condition, fileType);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeFileTypeTrigger#equals(Object)} com tipos de
   * arquivo diferentes.
   */
  public void testEquals3() {
    String parameterName = "Nome do parmetro";
    String fileType = "Tipo de arquivo";
    FileParameter parameter = createMockFileParameter(parameterName, fileType);
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    String fileType1 = "Tipo de arquivo 1";
    String fileType2 = "Tipo de arquivo 2";
    ChangeFileTypeTrigger trigger1 =
      new ChangeFileTypeTrigger(parameter, condition, fileType1);
    ChangeFileTypeTrigger trigger2 =
      new ChangeFileTypeTrigger(parameter, condition, fileType2);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Cria um configurador de algorithms qualquer.
   * 
   * @return .
   */
  private SimpleAlgorithmConfigurator createMockAlgorithmConfigurator() {
    AlgorithmInfo algorithmInfo =
      new AlgorithmInfo("ID Algoritmo", "Algoritmo", "DIR Algoritmo",
        new Hashtable<String, String>());
    AlgorithmVersionId algorithmVersionId = new AlgorithmVersionId(1, 2, 3);
    AlgorithmVersionInfo algorithmVersionInfo =
      new AlgorithmVersionInfo(algorithmInfo, algorithmVersionId,
        new Hashtable<String, List<FileInfo>>(),
        new Hashtable<String, String>());
    return new SimpleAlgorithmConfigurator(algorithmVersionInfo, null,
      ExecutionType.SIMPLE, null, false, "Comando", null, null, true, CMD_PATTERN);
  }

  /**
   * Cria uma condio qualquer.
   * 
   * @return .
   */
  private Condition createMockCondition() {
    return new SimpleCondition("Nome do parmetro", "Valor qualquer");
  }

  /**
   * Cria um parmetro do tipo arquivo qualquer.
   * 
   * @return .
   */
  private FileParameter createMockFileParameter() {
    String parameterName = "Nome do parmetro";
    String fileType = "Tipo de arquivo";
    return createMockFileParameter(parameterName, fileType);
  }

  /**
   * Cria um parmetro do tipo arquivo com nome e tipo especificados.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * @param fileType O tipo de arquivo (Aceita {@code null}).
   * 
   * @return .
   */
  private FileParameter createMockFileParameter(String parameterName,
    String fileType) {
    return new MockFileParameter(parameterName, "Rtulo do parmetro",
      "Descrio do parmetro", false, "{0}={1}", fileType,
      FileParameterMode.REGULAR_FILE, false, FileParameterPipeAcceptance.TRUE,
      true);
  }

  /**
   * <p>
   * Cria um valor para armazenar em parmetros do tipo arquivo.
   * </p>
   * 
   * @param filePath O caminho de um arquivo qualquer (No aceita {@code null}).
   * @param fileType O tipo de um arquivo qualquer (No aceita {@code null}).
   * 
   * @return .
   */
  private FileParameterValue createMockFileParameterValue(String filePath,
    String fileType) {
    FileParameterValue file = new FileParameterValue(filePath, fileType);
    return file;
  }

  /**
   * Cria um grupo de parmetros qualquer.
   * 
   * @return .
   */
  private ParameterGroup createMockParameterGroup() {
    ParameterGroup group = new ParameterGroup("Grupo de parmetros");
    return group;
  }
}
