package csbase.logic.algorithms.parameters.triggers;

import java.util.Hashtable;
import java.util.List;

import junit.framework.TestCase;
import csbase.logic.FileInfo;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.ExecutionType;
import csbase.logic.algorithms.parameters.MockSimpleParameter;
import csbase.logic.algorithms.parameters.ParameterGroup;
import csbase.logic.algorithms.parameters.SimpleAlgorithmConfigurator;
import csbase.logic.algorithms.parameters.SimpleParameter;
import csbase.logic.algorithms.parameters.conditions.Condition;
import csbase.logic.algorithms.parameters.conditions.SimpleCondition;

/**
 * Testes unitrios para a classe {@link ShowParameterTrigger}.
 * 
 * @author lmoreira
 */
public final class ShowParameterTriggerTest extends TestCase {

  /**
   * Pattern padro.
   */
  private static final String CMD_PATTERN = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
  
  /**
   * Testando criar objetos da classe {@link ShowParameterTrigger}.
   */
  public void testShowParameterTrigger1() {
    SimpleParameter<?> parameter = createMockParameter();
    Condition condition = createMockCondition();
    ShowParameterTrigger trigger = new ShowParameterTrigger(parameter,
      condition);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
  }

  /**
   * Testando criar objetos da classe {@link ShowParameterTrigger}.
   */
  public void testShowParameterTrigger2() {
    Condition condition = createMockCondition();
    try {
      new ShowParameterTrigger(null, condition);
      fail();
    } catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar objetos da classe {@link ShowParameterTrigger}.
   */
  public void testShowParameterTrigger3() {
    SimpleParameter<?> parameter = createMockParameter();
    try {
      new ShowParameterTrigger(parameter, null);
      fail();
    } catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando o mtodo {@link ShowParameterTrigger#doAction()} com condio
   * verdadeira.
   */
  public void testDoAction1() {
    String parameterName = "Nome do parmetro";
    Object defaultValue = "Valor-padro";
    SimpleParameter<?> parameter = createMockParameter(parameterName,
      defaultValue);
    parameter.setVisible(false);
    assertEquals(parameterName, parameter.getName());
    assertEquals(defaultValue, parameter.getValue());
    assertFalse(parameter.isVisible());
    Condition condition = new SimpleCondition(parameterName, defaultValue);
    ParameterGroup group = createMockParameterGroup();
    group.addParameter(parameter);
    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    configurator.addGroup(group);
    assertTrue(condition.evaluate(configurator));
    ShowParameterTrigger trigger = new ShowParameterTrigger(parameter,
      condition);
    assertTrue(trigger.update(configurator));
    assertTrue(parameter.isVisible());
  }

  /**
   * Testando o mtodo {@link ShowParameterTrigger#doAction()} com condio
   * falsa.
   */
  public void testDoAction2() {
    String parameterName = "Nome do parmetro";
    Object defaultValue = "Valor-padro";
    SimpleParameter<?> parameter = createMockParameter(parameterName,
      defaultValue);
    assertEquals(parameterName, parameter.getName());
    assertEquals(defaultValue, parameter.getValue());
    assertTrue(parameter.isVisible());
    Condition condition = new SimpleCondition(parameterName, null);
    ParameterGroup group = createMockParameterGroup();
    group.addParameter(parameter);
    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    configurator.addGroup(group);
    assertFalse(condition.evaluate(configurator));
    ShowParameterTrigger trigger = new ShowParameterTrigger(parameter,
      condition);
    assertFalse(trigger.update(configurator));
    assertFalse(parameter.isVisible());
  }

  /**
   * Testando o mtodo {@link ShowParameterTrigger#equals(Object)} com
   * gatilhos iguais.
   */
  public void testEquals1() {
    String parameterName = "Nome do parmetro";
    SimpleParameter<?> parameter = createMockParameter(parameterName);
    Condition condition1 = new SimpleCondition("Nome do parmetro 1", "Valor 1");
    Condition condition2 = new SimpleCondition("Nome do parmetro 2", "Valor 2");
    ShowParameterTrigger trigger1 = new ShowParameterTrigger(parameter,
      condition1);
    ShowParameterTrigger trigger2 = new ShowParameterTrigger(parameter,
      condition2);
    assertTrue(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ShowParameterTrigger#equals(Object)} com
   * parmetros diferentes.
   */
  public void testEquals2() {
    String parameterName1 = "Nome do parmetro 1";
    SimpleParameter<?> parameter1 = createMockParameter(parameterName1);
    String parameterName2 = "Nome do parmetro 2";
    SimpleParameter<?> parameter2 = createMockParameter(parameterName2);
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ShowParameterTrigger trigger1 = new ShowParameterTrigger(parameter1,
      condition);
    ShowParameterTrigger trigger2 = new ShowParameterTrigger(parameter2,
      condition);
    assertFalse(trigger1.equals(trigger2));
  }
  
  /**
   * Testando o mtodo {@link ShowParameterTrigger#equals(Object)} com
   * argumento {@code null}.
   */
  public void testEquals3() {
    SimpleParameter<?> parameter = createMockParameter();
    Condition condition = createMockCondition();
    ShowParameterTrigger trigger = new ShowParameterTrigger(parameter,
      condition);
    assertFalse(trigger.equals(null));
  }

  
  /**
   * Testando o mtodo {@link ShowParameterTrigger#equals(Object)} com
   * argumento de outra classe.
   */
  public void testEquals4() {
    SimpleParameter<?> parameter = createMockParameter();
    Condition condition = createMockCondition();
    ShowParameterTrigger trigger = new ShowParameterTrigger(parameter,
      condition);
    assertFalse(trigger.equals("Outra classe"));
  }
  
  /**
   * Cria um configurador de algorithms qualquer.
   * 
   * @return .
   */
  private SimpleAlgorithmConfigurator createMockAlgorithmConfigurator() {
    AlgorithmInfo algorithmInfo = new AlgorithmInfo("ID Algoritmo",
      "Algoritmo", "DIR Algoritmo", new Hashtable<String, String>());
    AlgorithmVersionId algorithmVersionId = new AlgorithmVersionId(1, 2, 3);
    AlgorithmVersionInfo algorithmVersionInfo = new AlgorithmVersionInfo(
      algorithmInfo, algorithmVersionId,
      new Hashtable<String, List<FileInfo>>(),
      new Hashtable<String, String>());
    return new SimpleAlgorithmConfigurator(algorithmVersionInfo, null,
      ExecutionType.SIMPLE, null, false, "Comando", null, null, true, CMD_PATTERN);
  }

  /**
   * Cria uma condio qualquer.
   * 
   * @return .
   */
  private Condition createMockCondition() {
    return new SimpleCondition("Nome do parmetro", "Valor qualquer");
  }

  /**
   * Cria um parmetro qualquer.
   * 
   * @return .
   */
  private SimpleParameter<?> createMockParameter() {
    String parameterName = "Nome do parmetro";
    return createMockParameter(parameterName);
  }

  /**
   * Cria um parmetro com o nome especificado.
   * 
   * @param parameterName O nome (No aceita {@code null}).
   * 
   * @return .
   */
  private SimpleParameter<?> createMockParameter(String parameterName) {
    Object defaultValue = "Valor-padro do parmetro";
    return createMockParameter(parameterName, defaultValue);
  }

  /**
   * Cria um parmetro com o nome especificado.
   * 
   * @param parameterName O nome (No aceita {@code null}).
   * @param defaultValue O valor-padro (Aceita {@code null}).
   * 
   * @return .
   */
  private SimpleParameter<?> createMockParameter(String parameterName,
    Object defaultValue) {
    return new MockSimpleParameter(parameterName, "Rtulo do parmetro",
      "Descrio do parmetro", defaultValue, false, null);
  }

  /**
   * Cria um grupo de parmetros qualquer.
   * 
   * @return .
   */
  private ParameterGroup createMockParameterGroup() {
    return new ParameterGroup("Grupo de parmetros");
  }
}
