package csbase.logic.algorithms.parsers;

import java.io.StringReader;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Set;
import java.util.SortedSet;

import junit.framework.TestCase;
import csbase.exception.OperationFailureException;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.Category;

/**
 * Teste unitrios para a classe {@link CategoryParser}.
 */
public final class CategoryParserTest extends TestCase {

  /**
   * Testando o mtodo {@link CategoryParser#read(java.io.Reader)}: Lendo um
   * stream que no possui categorias.
   * 
   * @throws OperationFailureException Se houver um erro inesperado.
   */
  public void testRead1() throws OperationFailureException {

    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<categories>";
    xml += "</categories>";

    assertValidXml(xml, new AlgorithmInfo[0]);
  }

  /**
   * Testando o mtodo {@link CategoryParser#read(java.io.Reader)}: Lendo um
   * stream que possui 1 categoria.
   * 
   * @throws OperationFailureException Se houver um erro inesperado.
   */
  public void testRead2() throws OperationFailureException {
    String id = "C.1";
    String name = "Categoria 1";

    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<categories>";
    xml += String.format("<category id = '%s' name = '%s'/>", id, name);
    xml += "</categories>";

    assertValidXml(xml, new AlgorithmInfo[0], new Category(null, id, name));
  }

  /**
   * Testando o mtodo {@link CategoryParser#read(java.io.Reader)}: Lendo um
   * stream que possui mais de 1 categoria.
   * 
   * @throws OperationFailureException Se houver um erro inesperado.
   */
  public void testRead3() throws OperationFailureException {
    String id1 = "C.1";
    String name1 = "Categoria 1";
    String id2 = "C.2";
    String name2 = "Categoria 2";

    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<categories>";
    xml += String.format("<category id = '%s' name = '%s'/>", id1, name1);
    xml += String.format("<category id = '%s' name = '%s'/>", id2, name2);
    xml += "</categories>";

    assertValidXml(xml, new AlgorithmInfo[0], new Category(null, id1, name1),
      new Category(null, id2, name2));
  }

  /**
   * Testando o mtodo {@link CategoryParser#read(java.io.Reader)}: Lendo um
   * stream que possui 1 categoria com sub-categorias.
   * 
   * @throws OperationFailureException Se houver um erro inesperado.
   */
  public void testRead4() throws OperationFailureException {
    String id1 = "C.1";
    String name1 = "Categoria 1";
    String id11 = "C.1.1";
    String name11 = "Categoria 1.1";
    String id12 = "C.1.2";
    String name12 = "Categoria 1.2";

    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<categories>";
    xml += String.format("<category id = '%s' name = '%s'>", id1, name1);
    xml += String.format("<category id = '%s' name = '%s'/>", id11, name11);
    xml += String.format("<category id = '%s' name = '%s'/>", id12, name12);
    xml += "</category>";
    xml += "</categories>";

    Category category1 = new Category(null, id1, name1);
    category1.addCategory(new Category(category1, id11, name11));
    category1.addCategory(new Category(category1, id12, name12));

    assertValidXml(xml, new AlgorithmInfo[0], category1);
  }

  /**
   * Testando o mtodo {@link CategoryParser#read(java.io.Reader)}: Lendo um
   * stream que possui 1 categoria com algoritmos.
   * 
   * @throws OperationFailureException Se houver um erro inesperado.
   */
  public void testRead5() throws OperationFailureException {
    String id1 = "C.1";
    String name1 = "Categoria 1";

    String algorithmId1 = "A.1";
    String algorithmId2 = "A.2";

    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<categories>";
    xml += String.format("<category id = '%s' name = '%s'>", id1, name1);
    xml += String.format("<algorithm id = '%s'/>", algorithmId1);
    xml += String.format("<algorithm id = '%s'/>", algorithmId2);
    xml += "</category>";
    xml += "</categories>";

    Category category1 = new Category(null, id1, name1);
    AlgorithmInfo algorithm1 = createMockAlgorithm(algorithmId1);
    AlgorithmInfo algorithm2 = createMockAlgorithm(algorithmId2);
    category1.addAlgorithm(algorithm1);
    category1.addAlgorithm(algorithm2);
    assertValidXml(xml, new AlgorithmInfo[] { algorithm1, algorithm2 },
      category1);
  }

  /**
   * Testando o mtodo {@link CategoryParser#read(java.io.Reader)}: Lendo um
   * stream que possui 1 categoria sem identificador.
   */
  public void testRead6() {
    String name = "Categoria 1";

    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<categories>";
    xml += String.format("<category name = '%s'/>", name);
    xml += "</categories>";
    assertInvalidXml(xml);
  }

  /**
   * Testando o mtodo {@link CategoryParser#read(java.io.Reader)}: Lendo um
   * stream que possui 1 categoria sem nome.
   */
  public void testRead7() {
    String id = "C.1";

    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<categories>";
    xml += String.format("<category id = '%s'/>", id);
    xml += "</categories>";
    assertInvalidXml(xml);
  }

  /**
   * Testando o mtodo {@link CategoryParser#read(java.io.Reader)}: Lendo um
   * stream que possui 1 categoria com algoritmos sem identificador.
   */
  public void testRead8() {
    String id1 = "C.1";
    String name1 = "Categoria 1";

    String xml = "";
    xml += "<?xml version='1.0' encoding='utf-8'?>\n";
    xml += "<categories>";
    xml += String.format("<category id = '%s' name = '%s'>", id1, name1);
    xml += String.format("<algorithm/>");
    xml += "</category>";
    xml += "</categories>";

    assertInvalidXml(xml);
  }

  /**
   * @param expectedCategory (No aceita {@code null}).
   * @param foundCategory (No aceita {@code null}).
   */
  private void assertCategories(Category expectedCategory,
    Category foundCategory) {
    assertNotNull(foundCategory);
    assertEquals(expectedCategory, foundCategory);
    assertEquals(expectedCategory.getId(), foundCategory.getId());
    assertEquals(expectedCategory.getName(), foundCategory.getName());
    SortedSet<Category> expectedCategories = expectedCategory.getCategories();
    SortedSet<Category> foundCategories = foundCategory.getCategories();
    assertCategories(expectedCategories, foundCategories);
    Set<AlgorithmInfo> expectedAlgorithms = expectedCategory.getAlgorithms();
    Set<AlgorithmInfo> foundAlgorithms = foundCategory.getAlgorithms();
    assertEquals(expectedAlgorithms, foundAlgorithms);
  }

  /**
   * Verifica se as categorias encontradas so iguais as esperadas.
   * 
   * @param expectedCategories As categorias esperadas (No aceita {@code null}
   *        ).
   * @param foundCategories As categorias encontradas (No aceita {@code null}).
   */
  private void assertCategories(Set<Category> expectedCategories,
    Set<Category> foundCategories) {
    assertNotNull(foundCategories);
    assertEquals(expectedCategories, foundCategories);
    for (Category foundCategory : foundCategories) {
      for (Category expectedCategory : expectedCategories) {
        if (expectedCategory.equals(foundCategory)) {
          assertCategories(expectedCategory, foundCategory);
        }
      }
    }
  }

  /**
   * Verifica se o xml informado est invlido.
   * 
   * @param xml O xml (No aceita {@code null}).
   */
  private void assertInvalidXml(String xml) {
    try {
      assertValidXml(xml, new AlgorithmInfo[0]);
      fail();
    }
    catch (OperationFailureException e) {
    }
  }

  /**
   * Verifica se o xml informado possui as categorias esperadas.
   * 
   * @param xml O xml (No aceita {@code null}).
   * @param categories As categorias esperadas (No aceita {@code null}).
   * 
   * @throws OperationFailureException Se ocorrer um erro inesperado.
   */
  private void assertValidXml(String xml, AlgorithmInfo[] algorithms,
    Category... categories) throws OperationFailureException {
    CategoryParser parser = new MockCategoryParser(algorithms);
    Set<Category> foundCategories = parser.read(new StringReader(xml));
    Set<Category> expectedCategories = new HashSet<Category>(Arrays
      .asList(categories));
    assertCategories(expectedCategories, foundCategories);
  }

  /**
   * Cria um algoritmo com o identificador informado.
   * 
   * @param algorithmId O identificador do algoritmo (No aceita {@code null}).
   * 
   * @return O algoritmo.
   */
  private AlgorithmInfo createMockAlgorithm(String algorithmId) {
    AlgorithmInfo algorithm = new AlgorithmInfo(algorithmId, "Nome do "
      + algorithmId, "dir_" + algorithmId, new Hashtable<String, String>());
    return algorithm;
  }
}
