/*
 * $Id: IOHandlerTest.java 100628 2010-01-12 12:10:46Z clinio $
 */
package csbase.remote.niotransfer;

import java.io.File;
import java.io.IOException;

import junit.framework.TestCase;
import util.TestUtils;

/**
 * Teste para a classe {@link IOHandler}.
 * 
 * @author Tecgraf
 */
public class IOHandlerTest extends TestCase {
  /** Diretrio de dados para este teste. */
  private String dataDir;

  /** Instncia da classe sendo testada. */
  private IOHandler io;

  /** Canal de socket "fake". */
  private MockSocketChannel mockChannel;

  /**
   * Cria um channel (mock) e um IOHandler.
   */
  @Override
  public void setUp() {
    dataDir = TestUtils.getDataDir(this.getClass());
    mockChannel = new MockSocketChannel();
    io = new IOHandler(mockChannel, 1000);
  }

  /**
   * Garante que os argumentos para o mtodo sejam inteiros no-negativos.
   * 
   * @throws IOException em caso de erro de I/O
   */
  public void testWritePack1() throws IOException {
    try {
      io.writePack(-1);
      fail("Esperada IllegalStateException");
    }
    catch (IllegalStateException e) {
    }
    assertTrue(io.writePack(0));
  }

  /**
   * Dispara uma exceo em caso do mtodo {@link IOHandler#createSource(File)}
   * no ter sido chamado previamente.
   * 
   * @throws IOException em caso de erro de I/O
   */
  public void testWritePack2() throws IOException {
    try {
      io.writePack(1000);
      fail("Esperada IllegalStateException");
    }
    catch (IllegalStateException e) {
    }
  }

  /**
   * Testa o funcionamento esperado do mtodo.
   * 
   * @throws IOException em caso de erro de I/O
   */
  public void testWritePack3() throws IOException {
    io.createSource(new File(dataDir + "100bytes.txt"));
    assertEquals(0, io.getNioBuffer().position());
    assertEquals(IOHandler.getDefaultBufferCapacity(), io.getNioBuffer()
      .limit());
    mockChannel.setWillWrite(100);
    boolean result = io.writePack(100);
    assertTrue(result);
    assertEquals(0, io.getNioBuffer().position());
    assertEquals(IOHandler.getDefaultBufferCapacity(), io.getNioBuffer()
      .limit());
  }

  /**
   * Testa o funcionamento esperado do mtodo. Este teste considera uma leitura
   * de 50 bytes, seguido de duas chamadas do mtodo que escrevem 10 e 20 bytes,
   * sem portanto esvaziar o buffer.
   * 
   * @throws IOException em caso de erro de I/O
   */
  public void testWritePack4() throws IOException {
    io.createSource(new File(dataDir + "100bytes.txt"));
    // A cada leitura, so lidos no mximo 50 bytes do arquivo
    io.setContentBufferSize(50);
    assertEquals(0, io.getNioBuffer().position());
    assertEquals(IOHandler.getDefaultBufferCapacity(), io.getNioBuffer()
      .limit());
    // Transmite 10 bytes
    mockChannel.setWillWrite(10);
    boolean result = io.writePack(100);
    assertFalse(result);
    assertEquals(90, io.getRemainingPack());// Sobrou do pacote
    assertEquals(40, io.getBytesToSend());// Sobrou no buffer
    assertEquals(10, io.getNioBuffer().position());
    assertEquals(50, io.getNioBuffer().limit());
    assertEquals(10, mockChannel.getBytesWrittenCount());
    // Transmite 20 bytes
    mockChannel.setWillWrite(20);
    result = io.writePack(100);
    assertFalse(result);
    assertEquals(70, io.getRemainingPack());// Sobrou do pacote
    assertEquals(20, io.getBytesToSend());// Sobrou no buffer
    assertEquals(30, io.getNioBuffer().position());
    assertEquals(50, io.getNioBuffer().limit());
    assertEquals(30, mockChannel.getBytesWrittenCount());
  }

  /**
   * Testa o funcionamento esperado do mtodo. A cada chamada, esvazia o
   * contedo do buffer, garantindo portanto que o arquivo seja enviado
   * inteiramente com uma nica chamada.
   * 
   * @throws IOException em caso de erro de I/O
   */
  public void testWritePack5() throws IOException {
    io.createSource(new File(dataDir + "100bytes.txt"));
    // A cada leitura, so lidos no mximo 50 bytes do arquivo
    io.setContentBufferSize(30);
    assertEquals(0, io.getNioBuffer().position());
    assertEquals(IOHandler.getDefaultBufferCapacity(), io.getNioBuffer()
      .limit());
    // Transmite 30 bytes, esvazia o buffer, transmite mais 30, esvazia...
    // repete esses passos 3 vezes, totalizando 90 bytes enviados.
    // Na ltima rodada, transmite os 10 bytes restantes, finalizando a 
    // transmisso do pacote com uma nica chamada.
    mockChannel.setWillWrite(30);
    boolean result = io.writePack(100);
    assertTrue(result);
    assertEquals(0, io.getRemainingPack());// Sobrou do pacote
    assertEquals(0, io.getBytesToSend());// Sobrou no buffer
    assertEquals(0, io.getNioBuffer().position());
    assertEquals(IOHandler.getDefaultBufferCapacity(), io.getNioBuffer()
      .limit());
    assertEquals(100, mockChannel.getBytesWrittenCount());
  }

  /**
   * Testa o funcionamento esperado do mtodo, alternando chamadas que no
   * esvaziam o buffer (e portanto retornam) com chamadas que esvaziam o buffer
   * e fazem internamente nova leitura, buscando transmitir todo o contedo do
   * pacote.
   * 
   * @throws IOException em caso de erro de I/O
   */
  public void testWritePack6() throws IOException {
    io.createSource(new File(dataDir + "100bytes.txt"));
    // A cada leitura, so lidos no mximo 40 bytes do arquivo
    io.setContentBufferSize(40);
    assertEquals(0, io.getNioBuffer().position());
    assertEquals(IOHandler.getDefaultBufferCapacity(), io.getNioBuffer()
      .limit());
    // Transmite 10 bytes, sem esvaziar o buffer
    mockChannel.setWillWrite(10);
    boolean result = io.writePack(100);
    assertFalse(result);
    assertEquals(90, io.getRemainingPack());// Sobrou do pacote
    assertEquals(30, io.getBytesToSend());// Sobrou no buffer
    assertEquals(10, io.getNioBuffer().position());
    assertEquals(40, io.getNioBuffer().limit());
    assertEquals(10, mockChannel.getBytesWrittenCount());
    // Transmite mais 20 bytes, tambm sem esvaziar o buffer
    mockChannel.setWillWrite(20);
    result = io.writePack(100);
    assertFalse(result);
    assertEquals(70, io.getRemainingPack());// Sobrou do pacote
    assertEquals(10, io.getBytesToSend());// Sobrou no buffer
    assertEquals(30, io.getNioBuffer().position());
    assertEquals(40, io.getNioBuffer().limit());
    assertEquals(30, mockChannel.getBytesWrittenCount());
    // Transmite os 10 bytes restantes do buffer, l novamente e manda mais
    // 10 bytes (total: 20 bytes)
    mockChannel.setWillWrite(10);
    result = io.writePack(100);
    assertFalse(result);
    assertEquals(50, io.getRemainingPack());// Sobrou do pacote
    assertEquals(30, io.getBytesToSend());// Sobrou no buffer
    assertEquals(10, io.getNioBuffer().position());
    assertEquals(40, io.getNioBuffer().limit());
    assertEquals(50, mockChannel.getBytesWrittenCount());
    // Transmite 40 bytes, esvazia o buffer e transmite mais 20 (o que faltava
    // para completar o pacote)
    mockChannel.setWillWrite(40);
    result = io.writePack(100);
    assertTrue(result);
    assertEquals(0, io.getRemainingPack());// Sobrou do pacote
    assertEquals(0, io.getBytesToSend());// Sobrou no buffer
    assertEquals(0, io.getNioBuffer().position());
    assertEquals(IOHandler.getDefaultBufferCapacity(), io.getNioBuffer()
      .limit());
    assertEquals(100, mockChannel.getBytesWrittenCount());
  }

  /**
   * Testa a escrita de um inteiro em duas chamadas (2 bytes de cada vez).
   * 
   * @throws IOException em caso de erro de I/O
   */
  public void testWriteInt1() throws IOException {
    mockChannel.setWillWrite(2);
    assertFalse(io.writeInt(32765));
    assertTrue(io.writeInt(32765));
    assertEquals(32765, mockChannel.getInt());
  }

  /**
   * Testa a escrita de um tipo longo em trs chamadas.
   * 
   * @throws IOException em caso de erro de I/O
   */
  public void testWriteLong1() throws IOException {
    long theLong = 98238393938L;
    mockChannel.setWillWrite(3);
    assertFalse(io.writeLong(theLong));
    assertFalse(io.writeLong(theLong));
    mockChannel.setWillWrite(2);
    assertTrue(io.writeLong(theLong));
    assertEquals(theLong, mockChannel.getLong());
  }

}
