/*
 * $Id$
 */
package csbase.util;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 * @author Tecgraf / PUC-Rio
 * 
 * Servio de execues peridicas de uma tarefa.
 */
public class TaskScheduler {

  private final Runnable task;
  private final long period;
  private final TimeUnit unit;

  private final Lock lock;
  private final ScheduledExecutorService executor;
  private ScheduledFuture<?> control;

  /**
   * 
   * @param task a tarefa a ser executada de tempos em tempos.
   * @param period o perodo entre as sucessivas execues.
   * @param unit a unidade de tempo do perodo.
   */
  public TaskScheduler(Runnable task, long period, TimeUnit unit) {
    this.task = task;
    this.period = period;
    this.unit = unit;

    this.lock = new ReentrantLock();
    this.executor = Executors.newScheduledThreadPool(1);
  }

  /**
   * Inicia as execues peridicas da tarefa.
   * 
   * @return <tt>false</tt> caso o processo de execues sucessivas da tarefa
   *         j esteja em execuo
   */
  public boolean start() {
    lock.lock();
    try {
      if (!isRunning()) {
        control = executor.scheduleAtFixedRate(task, 0, period, unit);
        return true;
      }
      else {
        return false;
      }
    }
    finally {
      lock.unlock();
    }
  }

  /**
   * Interrompe as execues peridicas da tarefa.
   * 
   * @return <tt>false</tt> caso a execuo no possa ter sido interrompida.
   *         Tipicamente isso ocorre quando o processo de execues sucessivas
   *         da tarefa j esteja desligado.
   */
  public boolean stop() {
    lock.lock();
    try {
      if (isRunning()) {
        return control.cancel(true);
      }
      else {
        return false;
      }
    }
    finally {
      lock.unlock();
    }
  }

  /**
   * Checa se o processo de execues sucessivas da tarefa est em execuo.
   * 
   * @return <tt>true</tt> se o processo de execues sucessivas da tarefa
   *         est em execuo.
   */
  public boolean isRunning() {
    lock.lock();
    try {
      return !(null == control || control.isDone() || control.isCancelled());
    }
    finally {
      lock.unlock();
    }
  }
}
