package csbase.util.parser.csv;

import csbase.util.parser.FiniteAutomaton;
import csbase.util.parser.Parser;
import csbase.util.parser.State;
import csbase.util.parser.Token;
import csbase.util.parser.actions.AppendAction;
import csbase.util.parser.actions.DiscardAction;
import csbase.util.parser.actions.GenerateTokenAction;
import csbase.util.parser.exception.AutomatonException;
import csbase.util.parser.exception.MalformedInputException;
import csbase.util.parser.iterators.CharSymbolIterator;
import csbase.util.parser.symbols.CharSymbol;
import csbase.util.parser.symbols.Symbol;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.LineNumberReader;
import java.io.Reader;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

public class CsvParser extends Parser {
  public CsvParser() {
    super(new CsvFiniteAutomaton());
  }

  public String[][] parse(File file)
  throws IOException, MalformedInputException {
    FileReader fileReader = new FileReader(file);
    try {
      return this.parse(fileReader);
    }
    finally {
      fileReader.close();
    }
  }

  public String[][] parse(Reader reader)
  throws IOException, MalformedInputException {
    List lineList = this.createCsvRegisterList(reader);
    return toCsvRegisterArray(lineList);
  }

  private static String[][] toCsvRegisterArray(List lineList) {
    String[][] fieldMatrix = new String[lineList.size()][];
    ListIterator lineListIterator = lineList.listIterator();
    for (int lineCounter = 0; lineListIterator.hasNext(); lineCounter++) {
      List fieldList = (List)lineListIterator.next();
      fieldMatrix[lineCounter] = new String[fieldList.size()];
      ListIterator fieldListIterator = fieldList.listIterator();
      for (int fieldCounter = 0; fieldListIterator.hasNext(); fieldCounter++) {
        fieldMatrix[lineCounter][fieldCounter] =
          (String)fieldListIterator.next();
      }
    }
    return fieldMatrix;
  }

  private List createCsvRegisterList(Reader reader)
  throws IOException, MalformedInputException {
    List lineList = new ArrayList();
    LineNumberReader lineNumberReader = new LineNumberReader(reader);
    for (
      String line = lineNumberReader.readLine(); line != null;
      line = lineNumberReader.readLine()) {
      List tokens = null;
      try {
        tokens = this.parse(new CharSymbolIterator(line));
      }
      catch (AutomatonException e) {
        throw new MalformedInputException(lineNumberReader.getLineNumber(), e);
      }
      List stringList = this.createStringList(tokens);
      lineList.add(stringList);
    }
    return lineList;
  }

  private List createStringList(List tokens) {
    List stringList = new ArrayList(tokens.size());
    Iterator tokenIterator = tokens.iterator();
    while (tokenIterator.hasNext()) {
      Token token = (Token)tokenIterator.next();
      List symbolList = token.getSymbolList();
      Iterator symbolListIterator = symbolList.iterator();
      StringBuffer buffer = new StringBuffer();
      while (symbolListIterator.hasNext()) {
        CharSymbol symbol = (CharSymbol)symbolListIterator.next();
        buffer.append((Character)symbol.getObject());
      }
      stringList.add(buffer.toString());
    }
    return stringList;
  }

  private static class CsvFiniteAutomaton extends FiniteAutomaton {
    private static final Symbol COMMA = new CharSymbol(',');
    private static final Symbol SEMICOLON = new CharSymbol(';');    
    private static final Symbol QUOTE = new CharSymbol('\"');
    private static final Symbol[] SPACES =
      {
        new CharSymbol(' '), new CharSymbol('\t'), new CharSymbol('\r'),
        new CharSymbol('\n')
      };

    private CsvFiniteAutomaton() {
      super(new State(true));
      State delimiter = this.getInitialState();
      State inWord = new State(true);
      State inQuoteWord = new State(false);
      State quoteInQuoteWord = new State(true);
      delimiter.setDefaultTransition(AppendAction.getInstance(), inWord);
      delimiter.addTransition(
        COMMA, GenerateTokenAction.getInstance(), delimiter);
      delimiter.addTransition(
    		  SEMICOLON, GenerateTokenAction.getInstance(), delimiter);
      delimiter.addTransition(QUOTE, DiscardAction.getInstance(), inQuoteWord);
      delimiter.addTransition(SPACES, DiscardAction.getInstance(), delimiter);
      inWord.setDefaultTransition(AppendAction.getInstance(), inWord);
      inWord.addTransition(COMMA, GenerateTokenAction.getInstance(), delimiter);
      inWord.addTransition(SEMICOLON, GenerateTokenAction.getInstance(), delimiter);
      inQuoteWord.setDefaultTransition(AppendAction.getInstance(), inQuoteWord);
      inQuoteWord.addTransition(
        QUOTE, DiscardAction.getInstance(), quoteInQuoteWord);
      quoteInQuoteWord.setDefaultTransition(AppendAction.getInstance(), inWord);
      quoteInQuoteWord.addTransition(
        QUOTE, AppendAction.getInstance(), inQuoteWord);
      quoteInQuoteWord.addTransition(
        COMMA, GenerateTokenAction.getInstance(), delimiter);
      quoteInQuoteWord.addTransition(
    		  SEMICOLON, GenerateTokenAction.getInstance(), delimiter);      
    }
  }
}
