package csbase.util.restart;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * Classe responsvel por gerenciar a coleo de objetos que precisam ter seus
 * componentes estticos reiniciados no incio da aplicao.
 *
 * @author Tecgraf/PUC-Rio
 */
public final class RestartManager {
  /** Representa a instncia nica da classe. */
  private static RestartManager instance;

  /** Lista com todos os ouvintes que precisam ser reiniciados. */
  private List restartListenerList;

  private RestartManager() {
    this.restartListenerList = new LinkedList();
  }

  /**
   * Obtm a instncia nica da classe.
   *
   * @return A instncia nica.
   */
  public static RestartManager getInstance() {
    if (instance == null) {
      instance = new RestartManager();
    }
    return instance;
  }

  /**
   * Adiciona um ouvinte que precisa ser reiniciado.
   *
   * @param restartListener O ouvinte a ser adicionado.
   */
  public void addListener(RestartListener restartListener) {
    synchronized (this.restartListenerList) {
      this.restartListenerList.add(restartListener);
    }
  }

  /**
   * Inicia o processo de reincio de todos os ouvintes cadastrados.
   */
  public void restart() {
    synchronized (this.restartListenerList) {
      Iterator iterator = this.restartListenerList.iterator();
      while (iterator.hasNext()) {
        RestartListener restartListener = (RestartListener)iterator.next();
        try {
          restartListener.restart();
        }
        catch (Exception e) {
          System.err.println(
            "Erro ao reiniciar a classe "+restartListener.getClass().getName());
          e.printStackTrace();
        }
      }
    }
  }
}
