package csbase.client.applications;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Locale;

import javax.swing.ImageIcon;

import runner.Runner;
import csbase.client.applicationmanager.ApplicationManager;
import csbase.logic.CommonClientProject;
import csbase.logic.applicationservice.ApplicationRegistry;

/**
 * @author Tecgraf/PUC-Rio
 */
abstract public class Application {

  /**
   * O identificar de mensagens enviadas no mtodo
   * {@link #sendMessage(String, Object)}.
   */
  public static final String PROJECT_FILE_MESSAGE = "PROJECT_FILE";

  /**
   * Frame principal da aplicao (fornecido pelo CSBASE/CSDK)
   */
  final private ApplicationFrame applicationFrame;

  /**
   * Identificador da aplicao
   */
  final private ApplicationRegistry registry;

  /**
   * Monta uma imagem do resource da aplicao.
   * 
   * @param tag nome do arquivo de imagem.
   * 
   * @return uma imagem (cone)
   */
  public final ImageIcon buildApplicationImage(final String tag) {
    final ApplicationRegistry reg = getApplicationRegistry();
    final ImageIcon image = reg.getImageIcon(tag + ".gif");
    if (image == null) {
      return ApplicationImages.ICON_NONE;
    }
    return image;
  }

  /**
   * Consulta ao registro da aplicao.
   * 
   * @return o registro
   */
  final public ApplicationRegistry getApplicationRegistry() {
    return registry;
  }

  /**
   * Retorna o dilogo principal da aplicao.
   * 
   * @return a janela.
   */
  public final ApplicationFrame getApplicationFrame() {
    return this.applicationFrame;
  }

  /**
   * Consulta ao projeto corrente.
   * 
   * @return o projeto corrente (que necessariamente existe).
   */
  public final CommonClientProject getApplicationProject() {
    final Runner runner = Runner.getInstance();
    final CommonClientProject prj = runner.getProject();
    return prj;
  }

  /**
   * Mtodo abstrato para fechamento da aplicao.
   * 
   * @return um flag indicativo de sucesso no fechamento.
   */
  public final boolean closeApplication() {
    if (userCanKillApplication()) {
      killApplication();
      final ApplicationFrame frm = getApplicationFrame();
      frm.setVisible(false);

      final Runner runner = Runner.getInstance();
      runner.removeFromActive(this);
      final String id = getId();
      runner.log(false, "Application " + id + " finished!");

      if (!runner.hasActiveApplications()) {
        runner.log(false, "No more active applications detected!");
        runner.finish(true);
      }
      return true;
    }
    final Runner runner = Runner.getInstance();
    runner.log(false, "Application refused to close itself!");
    return false;
  }

  /**
   * Retorna o locale
   * 
   * @return o locale
   */
  final public Locale getLocale() {
    final Runner runner = Runner.getInstance();
    return runner.getLocale();
  }

  /**
   * Obtm o id da aplicao.
   * 
   * @return o id.
   */
  public final String getId() {
    return registry.getId();
  }

  /**
   * Mtodo para enviar atributos para uma aplicao; deve ser definido na
   * aplicao de acordo com a necessidade da mesma.
   * 
   * @param name nome do atributo.
   * @param value seu valor.
   */
  public void sendMessage(final String name, final Object value) {
  }

  /**
   * Consulta a existncia de uma string de locale
   * 
   * @param key a chave de consulta
   * 
   * @return um indicativo
   */
  public final boolean hasString(final String key) {
    return registry.hasString(key);
  }
  /**
   * Consulta um texto de internacionalizao.
   * 
   * @param key a chave
   * @return o texto
   */
  final public String getString(final String key) {
    return registry.getString(key);
  }

  /**
   * Obtem uma string de internacionalizao.
   * 
   * @param key a chave de busca.
   * @param args os objetos de formatao
   * @return o texto no idioma correto.
   */
  final public String getString(final String key, final Object[] args) {
    return registry.getString(key, args);
  }

  /**
   * Obtm o nome da aplicao.
   * 
   * @return o nome.
   */
  public final String getName() {
    return registry.getApplicationName();
  }

  /**
   * Indica se a aplicao est rodando em ambiente de execuo client-servidor;
   * ou seja, conectada a um servidor CSBASE.
   * 
   * @return indicativo
   */
  final public boolean isConnectedToServer() {
    return false;
  }

  /**
   * Construtor padro.
   * 
   * @param id o identificador da aplicao.
   */
  protected Application(final String id) {
    final Runner runner = Runner.getInstance();
    final ApplicationManager manager = runner.getApplicationManager();
    registry = manager.getApplicationRegistry(id);
    this.applicationFrame = new ApplicationFrame(id);
    applicationFrame.addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosing(final WindowEvent e) {
        closeApplication();
      }
    });
  }

  /**
   * Mtodo de kill pelo usurio  especfico do desktop applications
   * 
   * @return um flag indicativo de possibilidade de fechamento da aplicao.
   */
  protected abstract boolean userCanKillApplication();

  /**
   * Mtodo para encerramento da execuo, que pode ser redefinido para
   * comportamentos prprios das aplicaes.
   */
  protected abstract void killApplication();

  /**
   * Mtodo de ps-inicializao (start-up) da aplicao
   * @throws Exception em caso de falha. 
   */
  public void postInitialization() throws Exception {
  }
}
