package csbase.client.applications;

import java.awt.Window;
import java.io.File;
import java.util.ArrayList;

import javax.swing.JFileChooser;

import runner.Runner;
import runner.filetypes.FileType;
import runner.filetypes.FileTypeFilter;
import runner.filetypes.FileTypes;
import csbase.client.applicationmanager.ApplicationException;
import csbase.logic.ClientProjectFile;
import csbase.logic.CommonClientProject;

/**
 * Classe que representa uma aplicao CSBASE que demanda a existncia de um
 * projeto aberto.
 * 
 * @author Tecgraf/PUC-Rio
 */
public abstract class ApplicationProject extends Application {

  /**
   * Mtodo de busca de arquivos em modo OPEN.
   * 
   * @param project o projeto de onde os arquivos sero seleciondos
   * @param fileTypes os tipos de arquivos selecionveis
   * @param defaultType o tipo default de arquivos (no tratado)
   * @param frame a janela-me do seletor de arquivos.
   * @return o arquivo selecionado ou <code>null</code>
   * @throws ApplicationException  sinalizada apenas por compatibilidade com o
   *         framework do CSBASE.
   */
  public final ClientProjectFile browseFileOpenFromProject(
    final CommonClientProject project, final String[] fileTypes,
    final String defaultType, final Window frame) throws ApplicationException {
    final int type = JFileChooser.OPEN_DIALOG;
    return browse(type, project, fileTypes, defaultType, frame);
  }

  /**
   * Mtodo de busca de arquivos em modo SAVE.
   * 
   * @param project o projeto de onde os arquivos sero seleciondos
   * @param fileTypes os tipos de arquivos selecionveis
   * @param defaultType o tipo default de arquivos (no tratado)
   * @param frame a janela-me do seletor de arquivos.
   * @return o arquivo selecionado ou <code>null</code>
   * @throws ApplicationException  sinalizada apenas por compatibilidade com o
   *         framework do CSBASE.
   */
  public final ClientProjectFile browseFileSaveFromProject(
    final CommonClientProject project, final String[] fileTypes,
    final String defaultType, final Window frame) throws ApplicationException {
    final int saveDialog = JFileChooser.SAVE_DIALOG;
    return browse(saveDialog, project, fileTypes, defaultType, frame);
  }

  /**
   * Mtodo interno de busca de arquivos.
   * 
   * @param mode o modo do dilogo.
   * @param project o projeto de onde os arquivos sero seleciondos
   * @param fileTypes os tipos de arquivos selecionveis
   * @param defaultType o tipo default de arquivos (no tratado)
   * @param frame a janela-me do seletor de arquivos.
   * @return o arquivo selecionado ou <code>null</code>
   */
  private final ClientProjectFile browse(final int mode,
    final CommonClientProject project, final String[] fileTypes,
    final String defaultType, final Window frame) {

    final CommonClientProject runnerProject = getApplicationProject();
    if (!project.equals(runnerProject)) {
      final String msg = "bad project found!";
      throw new IllegalArgumentException(msg);
    }

    final JFileChooser fc = new JFileChooser();
    fc.setLocale(Runner.getInstance().getLocale());
    fc.setDialogType(mode);
    fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
    fc.setAcceptAllFileFilterUsed(false);

    final ArrayList<FileType> types = new ArrayList<FileType>();
    for (final String ft : fileTypes) {
      final FileType tp = FileTypes.getFileType(ft);
      if (tp != null) {
        types.add(tp);
      }
    }
    final FileTypeFilter filter = new FileTypeFilter(types);
    fc.setFileFilter(filter);

    int returnVal = fc.showOpenDialog(frame);
    if (returnVal == JFileChooser.APPROVE_OPTION) {
      final File file = fc.getSelectedFile();
      final ClientProjectFile clientFile = new ClientProjectFile(file);
      return clientFile;
    }
    return null;
  }

  /**
   * Construtor
   * 
   * @param id id da aplicao.
   */
  protected ApplicationProject(final String id) {
    super(id);
  }

}
