package csbase.logic;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.channels.Channel;
import java.rmi.RemoteException;

import runner.Runner;
import csbase.exception.project.FileLockedException;

/**
 * Arquivo na rea de projeto do CSBASE.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class ClientProjectFile implements ProjectFile {

  /**
   * Arquivo interno associado
   */
  final private File file;

  /**
   * Arquivo interno para acesso randmico (I/O).
   */
  private RandomAccessFile randomFile;

  /**
   * Estado do arquivo.
   */
  private int state;

  /**
   * Extrai a extenso do arquivo especificado.
   * 
   * @param path caminho (ou o prprio nome) para o arquivo do qual pretende-se
   *        extrair a extenso.
   * 
   * @return extenso do arquivo ou <code>null</code> se o arquivo no possuir
   *         extenso.
   */
  private final static String getFileExtension(final String path) {
    if (path == null) {
      final String msg = "invalid null path not allowed";
      throw new IllegalArgumentException(msg);
    }
    if (path.trim().equals("")) {
      final String msg = "invalid empty path";
      throw new IllegalArgumentException(msg);
    }
    int periodIndex = path.lastIndexOf(".");
    if (periodIndex == -1 || periodIndex == path.length() - 1) {
      return null;
    }
    return path.substring(periodIndex + 1).toLowerCase();
  }

  /**
   * Consulta o arquivo interno <b>No pode ser usado pela aplicao</b>
   * 
   * @return o arquivo interno (local)
   */
  final File getFile() {
    return file;
  }

  /**
   * Retorna o caminho para o arquivo. Esse caminho no  relativo, ou seja, o
   * caminho absoluto de onde o arquivo se encontra.
   * 
   * @return caminho absoluto at o arquivo, inclusive.
   */
  final public String getAbsolutePath() {
    return file.getAbsolutePath();
  }

  /**
   * Informa o tipo do arquivo.
   * 
   * @return o tipo.
   */
  final public String getType() {
    final String path = getAbsolutePath();
    final String ext = getFileExtension(path);
    if (ext == null) {
      return ProjectFileType.UNKNOWN;
    }
    final Runner runner = Runner.getInstance();
    return runner.getTypeFromExtension(ext);
  }

  /**
   * Indica se este arquivo  um diretrio.
   * 
   * @return verdadeiro caso seja um diretrio e falso caso contrrio.
   */
  final public boolean isDirectory() {
    return file.isDirectory();
  }

  /**
   * Obtm a descrio atual deste arquivo.
   * 
   * @return o texto com a descrio.
   */
  final public String getDescription() {
    return "DESCRIPTION OF " + getAbsolutePath();
  }

  /**
   * Abre este arquivo para leitura exclusiva que no  feita com o uso de
   * <code>ByteBuffer</code>. A implementao desse mtodo determina o uso de
   * RMI para transmisso dos dados.
   * 
   * @return verdadeiro em caso de sucesso e falso caso a operao falhe.
   * 
   * @throws RemoteException falha de rmi (s vlida quando a aplicao roda
   *         efetivamente denro do CSBASE). No ambiente simulado, esta exceo
   *         <b>no</b> ser, de fato, lanada.
   */
  final public boolean openForReading() throws RemoteException {
    if (this.state != CLOSED) {
      return false;
    }
    this.state = OPENED_FOR_READING;
    return true;
  }

  /**
   * Abre este arquivo para escrita exclusiva que no  feita com o uso de
   * <code>ByteBuffer</code>. A implementao desse mtodo determina o uso de
   * RMI para transmisso dos dados.
   * 
   * @return Verdadeiro em caso de sucesso e falso caso a operao falhe.
   * 
   * @throws RemoteException falha de rmi
   */
  final public boolean openForWriting() throws RemoteException {
    if (this.state != CLOSED) {
      return false;
    }
    this.state = OPENED_FOR_WRITING;
    return true;
  }

  /**
   * Abre um canal para acesso ao arquivo.
   * 
   * @param readOnly indicativo de modo apenas-leitura.
   * @return indicativo de sucesso.
   * @throws RemoteException se houver falha.
   */
  public boolean openFileChannel(boolean readOnly) throws RemoteException {
    try {
      final String mode = (readOnly ? "r" : "rws");
      randomFile = new RandomAccessFile(file, mode);
      this.state = OPENED_AS_CHANNEL;
    }
    catch (FileNotFoundException e) {
      return false;
    }
    return true;
  }

  /**
   * Indica se este arquivo est aberto para leitura e escrita.
   * 
   * @return Verdadeiro caso este arquivo esteja aberto, falso caso contrrio.
   */
  final public boolean isOpen() {
    return (state != CLOSED);
  }

  /**
   * Informa a posio corrente deste arquivo. Reflete o nmero de bytes a
   * partir do incio do arquivo at a posio corrente.
   * 
   * @return a posio corrente do arquivo.
   * 
   * @throws IOException se houver falha no acesso ao arquivo.
   */
  final long position() throws IOException {
    return randomFile.getFilePointer();
  }

  /**
   * Altera a posio corrente deste arquivo. Reflete o nmero de bytes a partir
   * do incio do arquivo at a posio corrente.
   * 
   * @param newPosition Nova posio corrente do arquivo.
   * 
   * @throws IOException se houver falha no acesso ao arquivo.
   */
  final void position(final long newPosition) throws IOException {
    randomFile.seek(newPosition);
  }

  /**
   * L uma seqncia de bytes deste arquivo a partir da posio fornecida. Veja
   * a interface ReadableByteChannel para uma descrio completa da semntica
   * deste mtodo.
   * 
   * @param dst O buffer no qual os bytes sero escritos.
   * @param position A posio do arquivo a partir da qual os bytes sero lidos.
   * 
   * @return O nmero de bytes lidos ou -1 no caso de final de arquivo.
   * 
   * @throws Exception se houver falha no acesso ao arquivo.
   */
  final public int read(ByteBuffer dst, long position) throws Exception {
    randomFile.seek(position);
    final int nbytes = dst.capacity();
    final byte[] buffer = new byte[nbytes];
    randomFile.readFully(buffer);
    dst.put(buffer);
    return nbytes;
  }

  /**
   * Escreve uma seqncia de bytes neste arquivo a partir da posio fornecida.
   * 
   * @param src O buffer no qual esto os bytes a serem escritos.
   * @param position A posio do arquivo a partir da qual os bytes sero
   *        escritos
   * 
   * @return O nmero de bytes escritos.
   * 
   * @throws IOException se houver falha no acesso ao arquivo.
   * @throws FileLockedException caso o arquivo esteja bloqueado para escrita
   *         por outra sesso de usurio
   */
  final public int write(ByteBuffer src, long position) throws IOException,
    FileLockedException {
    randomFile.seek(position);
    final byte[] content = src.array();
    final int size = content.length;
    randomFile.write(content, 0, size);
    return size;
  }

  /**
   * Fecha este arquivo. Aps chamado este mtodo, o objeto que representa o
   * arquivo no poder mais ser utilizado para leitura e/ou escrita.
   * 
   * @throws IOException se houver falha no acesso ao arquivo.
   * 
   * @see Channel#close
   */
  final public void close() throws IOException {
    if (state == OPENED_AS_CHANNEL) {
      randomFile.close();
      randomFile = null;
    }
    this.state = CLOSED;
  }

  /**
   * Consulta o tamanho do arquivo.
   * 
   * @return o tamanho.
   * 
   * @throws IOException se houver falha no acesso ao arquivo.
   */
  final public long size() throws IOException {
    return file.length();
  }

  /**
   * Ajusta do tamanho do arquivo.
   * 
   * @param size o novo tamanho.
   * 
   * @throws IOException se houver falha no acesso ao arquivo.
   */
  final public void truncate(final long size) throws IOException {
    randomFile.setLength(size);
  }

  /**
   * L um objeto gravado no arquivo; a implementao desse mtodo utiliza RMI
   * para envio do objeto serializado.
   * 
   * @return o objeto lido do arquivo.
   * 
   * @throws Exception Em caso de erro.
   */
  final public Object readObject() throws Exception {
    if (this.state != OPENED_FOR_READING) {
      final String msg = "Arquivo no est no modo OPENED_FOR_READING";
      throw new Exception(msg);
    }
    final String err = "ClientProjectFile.readObject no implementado";
    throw new Exception(err);
  }

  /**
   * Grava um objeto no arquivo; a implementao desse mtodo utiliza RMI para
   * envio do objeto serializado.
   * 
   * @param object o objeto a ser gravado no arquivo.
   * 
   * @throws Exception Em caso de erro.
   */
  final public void writeObject(final Object object) throws Exception {
    if (this.state != OPENED_FOR_WRITING) {
      final String msg = "Arquivo no est no modo OPENED_FOR_WRITING";
      throw new Exception(msg);
    }
    final String err = "ClientProjectFile.writeObject no implementado";
    throw new Exception(err);
  }

  /**
   * Construtor (<b>NO USE</b>). Este mtodo, apesar de pblico,  para ser
   * usado pelo CSDK somente!
   * 
   * @param file o arquivo local associado.
   */
  public ClientProjectFile(final File file) {
    this.file = file;
    this.randomFile = null;
    this.state = CLOSED;
  }
}
