package csbase.logic.applicationservice;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.PropertyResourceBundle;

import javax.swing.ImageIcon;

import runner.Runner;
import csbase.client.applications.Application;

/**
 * Registro de aplicao.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class ApplicationRegistry {

  /**
   * Bundle interno da aplicao.
   */
  private PropertyResourceBundle bundle = null;

  /**
   * Tabela da propriedades da aplicao preenchida de acordo com a
   * parametrizao do {@link Runner}.
   */
  final private HashMap<String, String> applicationProperties =
    new HashMap<String, String>();

  /**
   * Classe que representa a aplicao.
   */
  private Class<? extends Application> applicationClass;

  /**
   * Identificador da aplicao
   */
  final private String id;

  /**
   * Nome da classe que representa a aplicao.
   */
  private String className;

  /**
   * Indicativo de carga de bundle interno da aplicao.
   */
  private boolean isBundleRequired;

  /**
   * Nome da aplicao.
   */
  private String applicationName;

  /**
   * Carga do bundle interno da aplicao.
   */
  final public void loadInternalBundle() {
    final Runner runner = Runner.getInstance();
    final Locale locale = runner.getLocale();
    final String language = locale.getLanguage();
    final String country = locale.getCountry();
    final String clsName = applicationClass.getSimpleName();
    final String idiomFileName =
      "resources/" + clsName + "_" + language + "_" + country + ".properties";
    final InputStream in = applicationClass.getResourceAsStream(idiomFileName);
    if (in == null) {
      final String err = "internal bundle not found: " + idiomFileName;
      throw new IllegalStateException(err);
    }
    try {
      bundle = new PropertyResourceBundle(in);
    }
    catch (IOException e) {
      final String err = "internal bundle failure error";
      throw new IllegalStateException(err, e);
    }
  }

  /**
   * Busca uma imagem para a aplicao.
   * 
   * @param path caminho denro do diretrio da classe da aplicao.
   * @return a imagem ou null
   */
  final public ImageIcon getImageIcon(final String path) {
    if (applicationClass == null) {
      return null;
    }
    try {
      final String sep = "/";
      final String resPath = "resources" + sep + "images" + sep + path;
      final URL res = applicationClass.getResource(resPath);
      return new ImageIcon(res);
    }
    catch (Exception e) {
      return null;
    }
  }
  
  /**
   * Consulta da existncia de uma string de idioma.
   * 
   * @param key a chave de busca.
   * @return indicativo
   */
  final public boolean hasString(final String key) {
    if (!isBundleRequired()) {
       return false;
    }
    final boolean containsKey = bundle.containsKey(key);
    return containsKey;
  }
  
  /**
   * Consulta um texto de internacionalizao.
   * 
   * @param key a chave
   * @return o texto
   */
  final public String getString(final String key) {
    if (!isBundleRequired()) {
      final String err = "Not configured for bundles (--internal-bundle)!";
      throw new IllegalStateException(err);
    }
    try {
      return bundle.getString(key);
    }
    catch (MissingResourceException e) {
      return (key == null) ? "<<<null>>>" : ("<<<" + key + ">>>");
    }
  }

  /**
   * Obtem uma string de internacionalizao.
   * 
   * @param key a chave de busca.
   * @param args os objetos de formatao
   * @return o texto no idioma correto.
   */
  final public String getString(final String key, final Object[] args) {
    if (args == null) {
      throw new IllegalArgumentException("args == null");
    }
    if (args.length == 0) {
      throw new IllegalArgumentException("args.length == 0");
    }
    final String pattern = getString(key);
    return MessageFormat.format(pattern, args);
  }

  /**
   * Ajusta o valor de applicationClass
   * 
   * @param applicationClass o valor a ser ajustado.
   */
  public final void setApplicationClass(
    Class<? extends Application> applicationClass) {
    this.applicationClass = applicationClass;
  }

  /**
   * Ajusta o valor de className
   * 
   * @param className o valor a ser ajustado.
   */
  public final void setClassName(String className) {
    this.className = className;
  }

  /**
   * Ajusta o valor de isBundleRequired
   * 
   * @param bundleRequired o valor a ser ajustado.
   */
  public final void setBundleLoaded(boolean bundleRequired) {
    this.isBundleRequired = bundleRequired;
  }

  /**
   * Ajusta o valor de applicationName
   * 
   * @param applicationName o valor a ser ajustado.
   */
  public final void setApplicationName(String applicationName) {
    this.applicationName = applicationName;
  }

  /**
   * Ajuste de propriedade da aplicao.
   * 
   * @param key chave
   * @param value valor
   */
  public final void setProperty(final String key, final String value) {
    final Runner runner = Runner.getInstance();
    if (runner.isEmptyString(key)) {
      final String err = "Empty key not allowed for property adjustment.";
      throw new IllegalArgumentException(err);
    }
    if (runner.isEmptyString(value)) {
      final String err = "Empty value not allowed for property adjustment.";
      throw new IllegalArgumentException(err);
    }
    runner.log(false, "Setting application property: " + key + " = " + value);
    applicationProperties.put(key.trim(), value.trim());
  }

  /**
   * Indicativo de propriedade sem valor.
   * 
   * @param propName nome da propriedade.
   * @return indicativo
   */
  final public boolean isPropertyNull(final String propName) {
    final String tag = id + "." + propName;
    if (!applicationProperties.containsKey(tag)) {
      return true;
    }
    final String value = applicationProperties.get(tag);
    if (value.trim().isEmpty()) {
      return true;
    }
    return false;
  }

  /**
   * Obtm uma propriedade especfica.
   * 
   * @param propName nome da propriedade.
   * 
   * @return propriedade especfica.
   * 
   * @throws IllegalStateException se a propriedade no foi definida
   */
  final public String getStringSpecificProperty(final String propName) {
    if (isPropertyNull(propName)) {
      final String err = "Prop. indefinida [" + propName + "] para apl. " + id;
      throw new IllegalStateException(err);
    }

    final String tag = id + "." + propName;
    final String value = applicationProperties.get(tag);
    if (value == null) {
      return null;
    }
    if (value.isEmpty()) {
      return null;
    }
    return value.trim();
  }

  /**
   * Obtm uma propriedade especfica do tipo inteiro.
   * 
   * @param propName nome da propriedade.
   * 
   * @return propriedade especfica do tipo inteiro.
   * 
   * @throws IllegalStateException se a propriedade no foi definida ou se o
   *         valor no pode ser convertido para um nmero inteiro
   */
  public final int getIntSpecificProperty(final String propName) {
    final String str = getStringSpecificProperty(propName);
    try {
      final int value = Integer.parseInt(str);
      return value;
    }
    catch (final Exception e) {
      final String err = "Propriedade " + propName + " inteira mal formatada";
      throw new IllegalStateException(err + " - applicao:" + getId());
    }
  }

  /**
   * Obtm uma propriedade especfica do tipo booleana.
   * 
   * @param propName nome da propriedade.
   * @return propriedade especfica do tipo booleana.
   */
  public final boolean getBooleanSpecificProperty(final String propName) {
    final String str = getStringSpecificProperty(propName);
    final String[] accepts = new String[] { "true", "false" };
    for (final String acc : accepts) {
      if (acc.equals(str.trim())) {
        final boolean value = Boolean.parseBoolean(str);
        return value;
      }
    }
    final String err = "Propriedade " + propName + " booleana mal formatada";
    throw new IllegalArgumentException(err + " - aplicao:" + getId());
  }

  /**
   * Consulta se aplicao no tem nenhuma propriedade ajustada.
   * Este mtodo <b>no</b> deve ser usado pela aplicao, pois
   *  da implementao interna do CSDK.
   * @return indicativo
   */
  final public boolean hasNoProperty() {
    return applicationProperties.size() == 0;
  }

  /**
   * Consulta o valor de id
   * 
   * @return o valor
   */
  public final String getId() {
    return id;
  }

  /**
   * Consulta o valor de applicationClass
   * 
   * @return o valor
   */
  public final Class<? extends Application> getApplicationClass() {
    return applicationClass;
  }

  /**
   * Consulta o valor de className
   * 
   * @return o valor
   */
  public final String getClassName() {
    return className;
  }

  /**
   * Consulta o valor de isBundleRequired
   * 
   * @return o valor
   */
  public final boolean isBundleRequired() {
    return isBundleRequired;
  }

  /**
   * Consulta o valor de applicationName
   * 
   * @return o valor
   */
  public final String getApplicationName() {
    return applicationName;
  }

  /**
   * Construtor
   * 
   * @param id identificador da aplicao.
   */
  public ApplicationRegistry(final String id) {
    this.id = id;
  }

}
