/*
 * $Id: Runner.java 103719 2010-03-30 16:55:49Z clinio $
 */

package runner;

import java.io.IOException;
import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.Locale;
import java.util.Set;

import runner.filetypes.FileType;
import runner.filetypes.FileTypes;
import csbase.client.applicationmanager.ApplicationManager;
import csbase.client.applications.Application;
import csbase.client.applications.ApplicationFrame;
import csbase.logic.CommonClientProject;
import csbase.logic.ProjectFileType;
import csbase.logic.applicationservice.ApplicationRegistry;

/**
 * Classe que representa o executor (<i>runner</i>) de simulaco do CSDK.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class Runner {

  /**
   * Instncia do runner.
   */
  static private Runner instance;

  /**
   * Projeto (simulao) associada a aplicao.
   */
  final private CommonClientProject project = new CommonClientProject();

  /**
   * Lista de aplicaes em execuo.
   */
  final private ArrayList<Application> activeApplications =
    new ArrayList<Application>();

  /**
   * 
   */
  final private ApplicationManager appManager =
    ApplicationManager.getInstance();

  /**
   * Identificador da aplicao
   */
  private String startingId;

  /**
   * Indicativo de modo verbose
   */
  private boolean verbosed = false;

  /**
   * Indica se o modo  verbose.
   * 
   * @return indicativo de verbose
   */
  public final boolean isVerbosed() {
    return verbosed;
  }

  /**
   * Ajuste de indicao de verbose
   * 
   * @param verbosed o indicativo a ser ajustado
   */
  private final void setVerbosed(boolean verbosed) {
    this.verbosed = verbosed;
  }

  /**
   * Ajusta o nome da classe a ser executada.
   * 
   * @param id identificador da aplicao.
   * @param className o nome a ser ajustado
   */
  private final void setClassName(final String id, final String className) {
    final ApplicationRegistry reg = appManager.getApplicationRegistry(id);
    if (reg == null) {
      log(true, "Class name set to not registered application id: [" + id + "]");
      log(true, "Runner aborted!");
      finish(false);
      return;
    }

    try {
      reg.setClassName(className);
      final Class<? extends Application> clazz = loadClassName(className);
      reg.setApplicationClass(clazz);
    }
    catch (ClassNotFoundException e) {
      log(true, "Failed to load class: " + className + "for application id:"
        + id);
      log(true, "Runner aborted!");
      finish(false);
    }
  }

  /**
   * Ajusta o id de uma nova aplicao.
   * 
   * @param id o applicationId a ser ajustado
   */
  private final void newApplicationId(final String id) {
    final ApplicationRegistry reg = appManager.getApplicationRegistry(id);
    if (reg != null) {
      log(true, "Application " + id + " already registered. Discarding...");
    }
    else {
      appManager.buildNewApplicationRegistry(id);
    }
  }

  /**
   * Consulta o id da aplicao
   * 
   * @return o id.
   */
  public final String getStartingApplicationId() {
    return startingId;
  }

  /**
   * Ajusta o id da aplicao
   * 
   * @param applicationId o applicationId a ser ajustado
   */
  private final void setStartingApplicationId(final String applicationId) {
    this.startingId = applicationId;
  }

  /**
   * Indica se o bundle da aplicao  para ser carregado.
   * 
   * @param id id da aplicao.
   * @return indicativo
   */
  public final boolean isBundleLoaded(final String id) {
    final ApplicationRegistry reg = appManager.getApplicationRegistry(id);
    return reg.isBundleRequired();
  }

  /**
   * Ajusta o indicativo de que o bundle da aplicao  para ser carregado.
   * 
   * @param id id da aplicao.
   * @param bundleLoaded o indicativo de ajuste
   */
  private final void setBundleLoaded(final String id, boolean bundleLoaded) {
    final ApplicationRegistry reg = appManager.getApplicationRegistry(id);
    reg.setBundleLoaded(bundleLoaded);
  }

  /**
   * Faz a carga da classe (com base no nome). Esta funo utiliza
   * <b>extraordinariamente</b> a anotao de supresso de avisos (warnings)
   * porque foi necessrio fazer um <i>forName</i>.
   * 
   * (Ver definio de {@link Class#forName(String)})
   * 
   * @param clName o nome da classe
   * @return a classe
   * @throws ClassNotFoundException se a classe da aplicao no for encontrada.
   */
  @SuppressWarnings("unchecked")
  private Class<? extends Application> loadClassName(final String clName)
    throws ClassNotFoundException {
    Class<?> cls = Class.forName(clName);
    return (Class<? extends Application>) cls;
  }

  /**
   * Construtor
   */
  private Runner() {
  }

  /**
   * Lana a aplicao.
   * 
   * @return indicativo de sucesso.
   */
  private boolean loadBundle() {
    try {
      LNG.loadBundle();
      return true;
    }
    catch (IOException e) {
      final String msg = e.getMessage();
      if (msg != null) {
        log(true, "Exception detected: " + msg);
      }
      final Throwable cause = e.getCause();
      if (cause != null) {
        final String err = "Cause of exception inside application class: ";
        log(true, err + cause.getMessage());
        cause.printStackTrace();
      }
      return false;
    }
  }

  /**
   * Lana a aplicao. Extraordinariamente, este mtodo inibe <i>warning</i> de
   * unchecked type, pois precisa fazer um <i>cast</i> explcito para o tipo
   * (classe) que efetivamente define a aplicao.
   * 
   * @param <A> a classe da aplicao.
   * @param id identificador da aplicao.
   * @return indicativo de sucesso.
   */
  @SuppressWarnings("unchecked")
  final public <A extends Application> A runApplication(final String id) {
    final ApplicationRegistry reg = appManager.getApplicationRegistry(id);
    if (reg == null) {
      log(true, "No application " + id + " found!");
      return null;
    }
    try {
      final Class<? extends Application> clazz = reg.getApplicationClass();
      if (clazz == null) {
        throw new Exception("No class found for " + id + "!");
      }

      log(true, "Launching new application " + id + "...");
      final Class<?>[] paramClass = new Class[] { String.class };
      final Constructor<?> constructor = clazz.getConstructor(paramClass);
      final Object[] pObjs = new Object[] { startingId };
      final Application app = (Application) constructor.newInstance(pObjs);
      ApplicationFrame mainFrame = app.getApplicationFrame();
      mainFrame.setVisible(true);
      mainFrame.toFront();
      final A application = (A) app;
      addToActive(application);
      app.postInitialization();
      log(true, "Application " + id + " lanched!");
      return application;
    }
    catch (Exception e) {
      final String msg = e.getMessage();
      if (msg != null) {
        log(true, "Exception detected: " + msg);
      }
      final Throwable cause = e.getCause();
      if (cause != null) {
        log(true, "Cause of exception inside application: "
          + cause.getMessage());
        cause.printStackTrace();
      }
      return null;
    }
  }

  /**
   * Retorna o singleton do <code>Runner</code>.
   * 
   * @return a instncia
   */
  final public static Runner getInstance() {
    if (instance != null) {
      return instance;
    }
    instance = new Runner();
    return instance;
  }

  /**
   * Consulta ao locale.
   * 
   * @return o locale
   */
  final public Locale getLocale() {
    return LNG.getLocale();
  }

  /**
   * Consulta ao projeto corrente.
   * 
   * @return o projeto corrente (que necessariamente existe).
   */
  public final CommonClientProject getProject() {
    return project;
  }

  /**
   * Busca um tipo de arquivo aplicvel a extenso sugerida.
   * 
   * @param extension extenso
   * @return o tipo
   */
  public final String getTypeFromExtension(final String extension) {
    final FileType type = FileTypes.getFileTypeFromExtension(extension);
    if (type == null) {
      return ProjectFileType.UNKNOWN;
    }
    return type.getName();
  }

  /**
   * Chacagem de argumentos da linha de comando.
   * 
   * @param args os argumentos
   */
  private void checkArguments(final String[] args) {
    int i = 0;
    final int N = args.length;
    while (i < N) {
      final String arg = args[i];
      if (arg.equals("--locale")) {
        checkLocaleArgument(args[i + 1], args[i + 2]);
        i = i + 2;
      }
      else if (arg.equals("--file-type")) {
        checkFileTypeArgument(args[i + 1], args[i + 2]);
        i = i + 2;
      }
      else if (arg.equals("--app-property")) {
        setProperty(args[i + 1], args[i + 2], args[i + 3]);
        i = i + 3;
      }
      else if (arg.equals("--app-bundle")) {
        setBundleLoaded(args[i + 1], true);
        i = i + 1;
      }
      else if (arg.equals("--verbose")) {
        setVerbosed(true);
      }
      else if (arg.equals("--start-id")) {
        setStartingApplicationId(args[i + 1]);
        i = i + 1;
      }
      else if (arg.equals("--new-id")) {
        newApplicationId(args[i + 1]);
        i = i + 1;
      }
      else if (arg.equals("--app-name")) {
        setApplicationName(args[i + 1], args[i + 2]);
        i = i + 2;
      }
      else if (arg.equals("--app-class")) {
        setClassName(args[i + 1], args[i + 2]);
        i = i + 2;
      }
      else {
        log(true, "Unrecognized argument found: " + arg + ". Ignoring...");
      }
      i = i + 1;
    }
  }

  /**
   * Ajuste de propriedade da aplicao.
   * 
   * @param id identificador
   * @param key chave
   * @param value valor
   */
  private void setProperty(final String id, final String key, final String value) {
    final ApplicationRegistry reg = appManager.getApplicationRegistry(id);
    if (reg == null) {
      log(true, "Property set to not registered application id: [" + id
        + "]. Ignoring...");
      return;
    }
    reg.setProperty(key, value);
  }

  /**
   * Finaliza o executor.
   * 
   * @param correctly indicativo que o executor terminou corretamente.
   */
  final public void finish(final boolean correctly) {
    final String state = correctly ? "correctly" : "with errors";
    final String text = String.format("Runner terminated %s!", state);
    instance.log(true, text);
    System.exit(correctly ? 0 : -1);
  }

  /**
   * Faz o dump do help.
   */
  private void showHelp() {
    System.out.println(" Runner Parameters:");
    System.out.println("   --verbose");
    System.out.println("   --locale <language> <country>");
    System.out.println("   --file-type <type> <extension>");
    System.out.println("   --new-id <id>");
    System.out.println("   --start-id <id>");
    System.out.println("   --help");
    System.out.println("");
    System.out.println(" Application Parameters:");
    System.out.println("   --app-bundle <id>");
    System.out.println("   --app-name <id> <application name>");
    System.out.println("   --app-class <idr> <class name>");
    System.out.println("   --app-property <id> <key> <value>");
  }

  /**
   * Faz dump de apresentao.
   */
  final public void showHeader() {
    final String string =
      "=========================================================";
    System.out.println(string);
    System.out.println("CSDK/Runner - Executor for CSBASE Applications");
    System.out.println("(C) 2008, Tecgraf/PUC-Rio");
    System.out.println(string);
  }

  /**
   * Faz a checagem de um tipo de arquivo.
   * 
   * @param typeName o nome do tipo
   * @param extension a extenso associada.
   */
  private void checkFileTypeArgument(final String typeName,
    final String extension) {
    if (isEmptyString(typeName) || isEmptyString(extension)) {
      return;
    }

    final FileType type;
    if (FileTypes.hasFileType(typeName)) {
      type = FileTypes.getFileType(typeName);
    }
    else {
      final String msg = "Creating new file type: " + typeName + "...";
      log(false, msg);
      type = FileTypes.createFileType(typeName);
    }
    log(false, "Setting extension \"" + extension + "\" to type: " + typeName
      + "...");
    type.addExtension(extension);

  }

  /**
   * Ajusta o nome do pas para efeito de locale.
   * 
   * @param languageName o nome do idioma a ser usado.
   * @param countryName nome do pas
   */
  private void checkLocaleArgument(final String languageName,
    final String countryName) {
    if (isEmptyString(languageName) || isEmptyString(countryName)) {
      return;
    }
    final String lng = languageName.trim();
    final String cty = countryName.trim();
    log(false, "Setting locale to " + lng + " (" + cty + ")...");
    LNG.setLocale(lng, cty);
  }

  /**
   * Teste se uma string  nula.
   * 
   * @param str a string a ser testada
   * @return um indicativo booleano
   */
  public boolean isEmptyString(final String str) {
    if (str == null) {
      return true;
    }
    final String s = str.trim();
    if (s.equals("")) {
      return true;
    }
    return false;
  }

  /**
   * Faz o log do executor.
   * 
   * @param forced modo forado (independente de verbose).
   * @param text texto.
   */
  public void log(final boolean forced, final String text) {
    if (forced || isVerbosed()) {
      System.out.println("[CSDK] - " + text);
    }
  }

  /**
   * Indica se os argumentos do runner indicam que o usurio pediu o <i>help</i>
   * (ajuda).
   * 
   * @param args argumentos
   * @return indicativo
   */
  private boolean isOnlyHelp(final String[] args) {
    for (String arg : args) {
      if (arg.equals("--help")) {
        return true;
      }
    }
    return false;
  }

  /**
   * Ajusta o nome da aplicao.
   * 
   * @param id identificador da aplicao.
   * @param applicationName o nome
   */
  private final void setApplicationName(final String id,
    final String applicationName) {
    final ApplicationRegistry reg = appManager.getApplicationRegistry(id);
    reg.setApplicationName(applicationName);
  }

  /**
   * Carga de bundles de aplicaes (registry)
   */
  private void loadApplicationsBundles() {
    final Set<String> ids = appManager.getAllAplicationsIds();
    for (String id : ids) {
      final ApplicationRegistry reg = appManager.getApplicationRegistry(id);
      log(false, "Checking " + id + "...");
      if (reg.isBundleRequired()) {
        log(false, "Bundle for " + id + " required. Loading...");
        reg.loadInternalBundle();
      }
      else {
        log(false, "Bundle for " + id + " *not* required!");
      }
    }
  }

  /**
   * @return a instncia
   */
  final public ApplicationManager getApplicationManager() {
    return appManager;
  }

  /**
   * Remoo da lista de aplicaes ativas
   * 
   * @param application aplicao
   */
  public void removeFromActive(final Application application) {
    activeApplications.remove(application);
  }

  /**
   * Adio na lista de aplicaes ativas
   * 
   * @param application aplicao
   */
  public void addToActive(final Application application) {
    activeApplications.add(application);
  }

  /**
   * @return indicativo
   */
  public boolean hasActiveApplications() {
    return activeApplications.size() != 0;
  }

  /**
   * Entry point.
   * 
   * @param args argumentos da linha de comando.
   */
  static public void main(final String[] args) {
    final Runner runner = Runner.getInstance();
    runner.showHeader();

    runner.log(false, "Starting runner...");
    // Teste de exibio de help (apenas).
    if (runner.isOnlyHelp(args)) {
      runner.showHelp();
      runner.finish(true);
    }

    // Verificao de parmetros
    try {
      runner.checkArguments(args);
    }
    catch (RuntimeException e) {
      runner.log(true, "Command line parametrization error: " + e.getMessage());
      runner.log(true, "Check arguments! Execution aborted!");
      if (runner.isVerbosed()) {
        e.printStackTrace();
      }
      runner.finish(false);
    }

    runner.log(false, "Loading runner locale bundle...");
    final boolean localeOk = runner.loadBundle();
    if (!localeOk) {
      runner.log(true, "Locale bundle load failed!");
      runner.finish(false);
    }

    runner.log(false, "Loading applications' bundles (if required)...");
    runner.loadApplicationsBundles();

    final String startId = runner.getStartingApplicationId();
    ApplicationManager appManager = runner.getApplicationManager();
    final ApplicationRegistry reg = appManager.getApplicationRegistry(startId);
    if (reg == null) {
      runner.log(true, "No starting application set! Aborting...");
      runner.finish(false);
      return;
    }

    runner.log(false, "Starting main application...");
    runner.log(false, "  * Application class name: " + reg.getClassName());
    runner.log(false, "  * Application id: " + reg.getId());
    // Aviso de que nenhuma propriedade foi setada.
    if (reg.hasNoProperty()) {
      runner.log(false, "  * No property set for main application.");
    }

    // Incio da execuo
    runner.log(false, "Starting main application...");
    final Application startApp = runner.runApplication(startId);
    if (startApp == null) {
      runner.log(true, "Main application execution failed!");
      runner.finish(false);
    }
    runner.log(true, "Main application started!");
  }

}
