package csfs.client;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import org.omg.CORBA.ORB;

import csfs.remote.FileServer;
import csfs.remote.FileServerHelper;
import csfs.remote.InvalidStateException;
import csfs.remote.NotDirectoryException;
import csfs.remote.RemoteFile;
import csfs.remote.ServerException;
import csfs.remote.WriteChannel;

/**
 * Essa classe possui uma serie de trechos de codigos para testar a
 * escalabilidade e o desempenho do CSFS. Ela poderia ter sido dividida em
 * outras classes, porem, optamos pela facilidade de ter apenas um script para a
 * execucao de testes, que executa o teste de acordo com o codigo contido nesse
 * arquivo. Para se definir um novo teste, usualmente precisamos apenas
 * modificar essa classe localmente, envia-la ao cluster, recompilar o sistema
 * (localmente no cluster) e executar o teste novamente. Existem varios scripts
 * que facilitam a execucao e o empacotamento do resultado (incluindo o codigo
 * nessa classe (que define o teste)).
 */
public class TestScalability {

  /**
   * Diversas threads sao criadas para controlar a copia dos arquivos. A
   * variavel counter eh usada para manter o controle da criacao das threads e
   * verificar o momento onde todas as threads concluiram o seu trabalho.
   */
  private static int counter = 0;

  /**
   * Armazena o momento inicial do teste.
   */
  private long absoluteStartTime = 0;

  /**
   * coolingTime indica o tempo que o sistema deve permanecer em repouso antes
   * que alguma nova macro operacao tenha inicio. Usualmente, durante a execucao
   * de um teste, as maquinas entram em um estado estressante e algumas
   * metricas, como a "carga media", precisam de um tempo de inatividade para
   * que o valor volte a numeros baixos. Para experimentos longos, eh
   * recomendavel um coolingTime condizente, permitindo que os valores baixem
   * (antes do inicio do proximo teste).
   */
  // private static int coolingTime = 5000;
  private static int coolingTime = 120000;

  /**
   * preVerifyCoolingTime eh semelhante ao coolingTime, mas tem um valor
   * diferenciado que precede o momento onde as copias dos arquivos sao
   * verificadas. Usualmente, para cada copia, uma verificacao do arquivo de
   * destino eh realizada (baseada no hash MD5 da origem e da copia).
   */
  // private static int preVerifyCoolingTime = 10000;
  private static int preVerifyCoolingTime = 180000;
  // private static int preVerifyCoolingTime = 300000;

  /**
   * Indica o final dos testes
   */
  private boolean finished = false;

  /**
   * Uma constante para se definir o valor de 1 MegaByte em bytes.
   */
  private long MB = 1024 * 1024;

  /**
   * Um formatados para datas, a ser usado na gerao de logs textuais.
   */
  private SimpleDateFormat dateFormatter = new SimpleDateFormat(
    "dd/MM/yyyy HH:mm:ss");

  /**
   * O metodo go() inicializa algumas variaveis que serao usadas nos testes.
   */
  public synchronized void go(String[] args) {

    try {

      ORB orb = ORB.init(args, null);

      /* Definindo o host a ser testado */
      String targetHost = "n7";
      int targetPort = 10000;

      /* Definindo os hosts que farao o teste */
      String[] sourceHosts = { "n1", "n2", /* "n3", */"n4", "n5", "n6" };
      int[] sourcePorts = { 10000, 10000, /* 10000, */10000, 10000, 10000 };


      /*
       * Inicializa os objetos FileServer de acordo com os arrays definidos
       * anteriormente
       */
      String targetCorbaloc = "corbaloc::1.2@" + targetHost + ":" + targetPort
        + "/standardImplName/FileServerPOA/FileServer";
      org.omg.CORBA.Object targetObj = orb.string_to_object(targetCorbaloc);
      final FileServer targetServer = FileServerHelper.narrow(targetObj);

      FileServer[] sourceServers = new FileServer[sourceHosts.length];

      for (int i = 0; i < sourceServers.length; i++) {
        String sourceCorbaloc = "corbaloc::1.2@" + sourceHosts[i] + ":"
          + sourcePorts[i] + "/standardImplName/FileServerPOA/FileServer";
        org.omg.CORBA.Object sourceObj = orb.string_to_object(sourceCorbaloc);
        FileServer sourceServer = FileServerHelper.narrow(sourceObj);
        sourceServers[i] = sourceServer;
      }

      /* Dado que temos os FileServers instanciados, podemos executar um teste */

      // runFileScalabilityTest(targetServer);
      runCopyScalabilityTest(sourceHosts, targetServer, sourceServers);

      finished = true;

      System.out.println("FINALIZADO!");

    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  /**
   * Esse teste realiza a criacao de dezenas ou milhares de arquivos no servidor
   * de destino. Para evitar que arquivos sejam indevidamente apagados no final
   * do teste, ele verifica se o diretorio de destino estah vazio no inicio do
   * teste.
   */
  private void runFileScalabilityTest(FileServer targetServer)
    throws ServerException {

    RemoteFile[] children = null;
    try {
      children = targetServer.getRoot().getChildren();
    }
    catch (Exception e1) {
      e1.printStackTrace();
    }
    if (children == null || children.length > 0) {
      System.out.println("ERROR: targetServer.getRoot() - Is it empty?");
      System.exit(1);
    }

    try {

      /* Realiza a criacao de diversos arquivos (ver definicao do metodo abaixo) */
      testSingleFileSetScalability(targetServer, 10, 2, true, 0); // 1000 files
      testSingleFileSetScalability(targetServer, 10, 2, true, 64 * 1024); // 1000
                                                                          // files
      testSingleFileSetScalability(targetServer, 10, 2, true, 128 * 1024); // 1000
                                                                            // files
      testSingleFileSetScalability(targetServer, 10, 3, false, 0); // 10000
                                                                    // files
      testSingleFileSetScalability(targetServer, 10, 4, false, 0); // 10000
                                                                    // files

    }
    catch (Exception exc) {
      exc.printStackTrace();
    }
  }

  /**
   * Realiza a criacao de diversos arquivos no sistema de arquivos remoto.
   * 
   * @param targetServer o servidor remoto
   * @param branch o numero de filhos que cada diretorio deve possuir (largura)
   * @param depth o numero de subdiretorios criados (profundidade)
   * @param openChannel indica se um canal para o arquivo deve ser aberto
   * @param fileBuffer o tamanho do buffer para cada canal
   */
  private void testSingleFileSetScalability(FileServer targetServer,
    int branch, int depth, boolean openChannel, int fileBuffer)
    throws ServerException, InterruptedException, Exception,
    InvalidStateException, NotDirectoryException {

    /* obtem a raiz do servidor de arquivos */
    RemoteFile root = targetServer.getRoot();

    System.out.println("\n\n\nBranch=" + branch);
    System.out.println("Depth=" + depth);
    System.out.println("FileBuffer=" + fileBuffer);

    /* apresenta o estado inicial do processo, antes da realizacao do teste */
    System.out.println("\nInitial State!");
    printMemoryStatus(targetServer, 2000);

    /* define um array que irah conter as referencias para os objetos criados */
    ArrayList files = new ArrayList();

    /* cria os diretorio recursivamente */
    createDirectories(root, branch, depth, files);

    /* cria os arquivos nos diretorios folha */
    createFileAtLeaves(root, branch, files);

    System.out.println("\nCreating done!");

    long elementsCreated = files.size();

    System.out.println("elementsCreated=" + elementsCreated);
    printMemoryStatus(targetServer, 2000);

    /* caso se deseje verificar a escalabilidade da abertura dos canais */
    if (openChannel) {
      testOpenChannelScalability(files, targetServer, fileBuffer);
    }

    /*
     * para cada elemento criado, remove o elemento. A ordem invertida permite
     * que os elementos sejam removidos numa ordem bottom-up.
     */
    for (int i = (files.size() - 1); i >= 0; i--) {
      ((RemoteFile) files.get(i)).remove();
    }

    System.out.println("Final State");
    printMemoryStatus(targetServer, 2000);

  }

  /**
   * Cria <num> diretorios recursivamente, decrementando <depth> ateh que este
   * seja igual a zero.
   * 
   * @param file o diretorio que deve possuir os novos diretorios
   * @param num o numero de diretorios a ser criado
   * @param depth a profundidade que a recursao deve continuar acontecendo
   * @param files um agregador para as referencias criadas
   */
  private void createDirectories(RemoteFile file, int num, int depth, List files)
    throws Exception {
    if (depth > 0) {
      for (int i = 0; i < num; i++) {
        RemoteFile child = file.createDirectory(new String[] { "dir_" + i });
        files.add(child);
        createDirectories(child, num, depth - 1, files);
      }
    }
  }

  /**
   * Cria <num> arquivos em um diretorio.
   * 
   * @param dir o diretorio que deve possuir os novos diretorios
   * @param num o numero de diretorios a ser criado
   * @param files um agregador para as referencias criadas
   * @throws Exception
   */
  private void createFileAtLeaves(RemoteFile dir, int num, List files)
    throws Exception {
    RemoteFile[] children = dir.getChildren();
    if (children.length == 0) {
      for (int i = 0; i < num; i++) {
        RemoteFile file = dir.createFile(new String[] { "file_" + i });
        files.add(file);
      }
    }
    else {
      for (int i = 0; i < children.length; i++) {
        createFileAtLeaves(children[i], num, files);
      }
    }
  }

  /**
   * Verifica o efeito da criacao de Channels (CORBA) para os arquivos. Cada
   * Channel possui um descritor de arquivo associado, de acordo com a
   * propriedade "BUFFERED_STREAMS_SIZE", esse descritor pode ser encapsulado em
   * um BufferedOutputStream, ou em um FileOutputStream. A utilizacao do buffer
   * potencialmente acarreta efeitos razoaveis no consumo de memoria pela JVM.
   * Por outro lado, como cada canal mantem um descritor de arquivos aberto, eh
   * provavel que o sistema atinja o seu limite no numero de descritores (antes
   * que a memoria venha a ser um elemento critico). Excetos para os casos onde
   * o buffer por canal eh demasiadamente elevado.
   * 
   * @param files um lista de referencias para os objetos recem criados. Esse
   *        metodo eh dependente da forma com as referencias para os arquivos
   *        sao criadas (ele assume que os arquivos estao no final do array).
   * @param targetServer isso eh um "patch", pois a forma como o tamanho do
   *        buffer eh definida eh geral para todo o sistema de arquivos.
   * @param fileBuffer o tamanho do buffer a ser usado (ou zero caso nao se
   *        queira o buffer)
   */
  private void testOpenChannelScalability(List files, FileServer targetServer,
    int fileBuffer) throws Exception {

    ArrayList channels = new ArrayList();
    targetServer.setProperty("BUFFERED_STREAMS_SIZE", "" + fileBuffer);

    int index = files.size();
    while (index > 0) {
      RemoteFile file = (RemoteFile) files.get(index - 1);
      if (!file.isDirectory()) {
        channels.add(file.getWriteChannel());
      }
      else {
        break;
      }
      index--;
    }
    System.out.println("channels.size()=" + channels.size());
    printMemoryStatus(targetServer, 2000);

    for (int i = 0; i < channels.size(); i++) {
      ((WriteChannel) channels.get(i)).close();
    }

    System.out.println("Channels Closed!");
    /*
     * Essa conta com o log eh apenas para definir um sleep time generico (em
     * funcao do numero de canais)
     */
    printMemoryStatus(targetServer, 1000 + (int) Math
      .log(channels.size() * 100));

  }

  /**
   * Tenta obter o estado de memoria do servidor, dormindo algum tempo entre as
   * chamadas para aumentar as chances que o garbage collector seja invocado.
   * 
   * @param targetServer o servidor de destino
   * @param queryDelay o tempo de "sono" entre as chamadas
   */
  private void printMemoryStatus(FileServer targetServer, int queryDelay)
    throws ServerException, InterruptedException {
    for (int i = 0; i < 5; i++) {
      System.out.println("" + new Date().getTime() + "\tMEMORY STATUS:"
        + targetServer.getMemoryStatus(true));
      Thread.sleep(queryDelay);
    }
  }

  /**
   * Realiza a definicao dos diversos testes de copia. Pode ser usado tanto para
   * testes de desempenho (testando os metodos de copia), quanto para testes de
   * escalabilidade (ao realizar varias transferencias simultaneas para o mesmo
   * servidor alvo).
   * 
   * @param sourceHosts lista com o nome das maquinas de origem
   * @param targetServer o servidor que servirah como destino para os dados
   * @param sourceServers os servidores que servirao como origem de dados
   */
  private void runCopyScalabilityTest(String[] sourceHosts,
    final FileServer targetServer, FileServer[] sourceServers)
    throws ServerException {

    /*
     * Inicia uma thread para recuperar, periodicamente, o estado da memoria do
     * servidor de destino
     */
    startMemoryGrabberThread(targetServer);

    /* ATENCAO: Uma atencao especial deve ser dada as seguintes variaveis */

    /*
     * Esse teste pode envolver a utilizacao de diversos metodos de copia. Uma
     * lista com todos os metodos suportados pode ser deduzida observando o
     * pacote csfs.datatranfer (ou lendo o texto contido na dissertacao
     * ("GridFS: Um Servidor de Arquivos para Ambientes de Computacao em
     * Grade"))
     * 
     * A lista de metodos eh utilizada em um loop, e nada impede que um mesmo
     * metodo seja definido varias vezes. Nesse caso, eh provavel o os
     * parametros da copia sejma variados, como o numero de clientes
     * concorrentes.
     * 
     * Alguns exemplos de metodos: "CORBA-32768", "CORBA-16384", "BLOCKING_NIO",
     * "NON_BLOCKING_NIO", "PureFTPd", "STREAM_BLOCKING_NIO-2"...
     * 
     */
    String[] methods;

    /*
     * parallelism indica o numero de threads que devem ser criadas em cada
     * cliente. Um parallelism de 2 farah com que cada servidor em "sourceHosts"
     * realize duas operaes de cpia simultaneamente (para o servidor de
     * destino). Devido a limitacao de espaco em disco nos servidores, eh comum
     * que para um parallelism mais alto, um arquivo de menor tamanho seja
     * utilizado. Logo, parallelism eh definido como um array e estah
     * relacionado a outros arrays (definidos a seguir).
     */
    int[] parallelism;

    /*
     * basefile define o prefixo "base" do arquivo a ser transferido. Por
     * exemplo, o script de geracao de arquivos de 1 GB gera os arquivos
     * nomeados: 1GB_1, 1GB_2, 1GB_3 e 1GB_4. Caso o teste deva usar arquivos de
     * 1GB, a string "1GB" deve ser dada como parametro. O valor do
     * "parallelism" ira definir o "_x" para cada arquivo a ser transferido
     */
    String[] basefile;

    /*
     * O sistema oferece uma serie de metricas para as transferencias, o valor
     * de "filesize" informa a quantidade de bytes contida em cada arquivo.
     * Eventualmente isso pode ser inferido do proprio arquivo.
     */
    long[] filesize;

    /**
     * numRuns indica quantas vezes o teste para cada metodo deve ser executado,
     * permitindo a obtencao de medias de uma forma simplificada
     */
    int numRuns;

    methods = new String[] { "CORBA-32768" };

    numRuns = 2;
    basefile = new String[] { "1GB" };
    filesize = new long[] { 1024 * MB };

    int[] singleCopyParallelismArray = new int[basefile.length];
    Arrays.fill(singleCopyParallelismArray, 1);

    System.out.println("\n\n\nSINGLE CLIENT! SINGLE THREAD!!!\n\n");

    /*
     * O primeiro teste consiste em usar um unico servidor de origem, com uma
     * unica copia
     */
    for (int i = 0; i < methods.length; i++) {
      runCopyTest(methods[i], numRuns, singleCopyParallelismArray, basefile,
        filesize, new String[] { sourceHosts[0] }, targetServer,
        new FileServer[] { sourceServers[0] });
    }

    System.out.println("\n\n\nMULTI CLIENT! SINGLE THREAD!!!\n\n");

    /* Em sequida, temos diversas fontes, cada uma com uma unica copia */
    for (int i = 0; i < methods.length; i++) {
      runCopyTest(methods[i], numRuns, singleCopyParallelismArray, basefile,
        filesize, sourceHosts, targetServer, sourceServers);
    }

    System.out.println("\n\n\nMULTI CLIENT! MULTI THREAD!!!\n\n");

    /*
     * Normalmente, para copias com paralelismo elevado, eh comum que o tamanho
     * do arquivo seja menor. Logo, os arrays devem ser redefinidos...
     */

    parallelism = new int[] { 8, 8 };
    basefile = new String[] { "128MB", "64MB" };
    filesize = new long[] { 128 * MB, 64 * MB };

    for (int i = 0; i < methods.length; i++) {
      runCopyTest(methods[i], numRuns, parallelism, basefile, filesize,
        sourceHosts, targetServer, sourceServers);
    }

  }

  /**
   * Realiza um teste de copia
   * 
   * @param method o metodo a ser usado
   * @param numRuns o numero de vezes que o teste deve ser repetido (em
   *        sequencia)
   * @param parallelism o numero de copias simultaneas para cada arquivo copiado
   * @param basefile os prefixos base para os arquivos a serem copiados
   * @param filesize o tamanho de cada arquivo
   * @param sourceHosts os nomes dos servidores que servirao como fonte de dados
   * @param targetServer o servidor de destino
   * @param sourceServers os servidores fonte
   */
  private synchronized void runCopyTest(String method, int numRuns,
    int[] parallelism, String[] basefile, long[] filesize,
    String[] sourceHosts, FileServer targetServer, FileServer[] sourceServers)
    throws ServerException {

    /* realiza um "sanity check" */
    if (parallelism.length != basefile.length
      || parallelism.length != filesize.length) {
      System.out
        .println("ERROR: check parallelism, basefile and filesize arrays!!!");
      System.exit(1);
    }

    /*
     * verifica se o alvo estah limpo (evitando erros subsequentes, devido a
     * execucoes anteriores (parciais) do teste
     */
    RemoteFile[] children = null;
    try {
      children = targetServer.getRoot().getChildren();
    }
    catch (Exception e1) {
      e1.printStackTrace();
    }
    if (children == null || children.length > 0) {
      System.out.println("ERROR: targetServer.getRoot() is not empty!");
      System.exit(1);
    }

    /*
     * reinicializa o valor das metricas de transferencia (calculadas no
     * servidor).
     */
    for (int i = 0; i < sourceServers.length; i++) {
      sourceServers[i].resetTransferRateData();
    }

    /*
     * ATENCAO: as informacoes impressas em "System.out" sao utilizadas para
     * geracao automatica de graficos, logo, o formato nao deve ser modificado.
     * Novas linhas de log potencialmente serao ignoradas. Ou seja, nao se deve
     * modificar as linhas, mas novas linhas podem ser adicionadas.
     */
    System.out.println("\n\n\n\nmethod=" + method);
    System.out.println("TIME:" + System.currentTimeMillis());
    System.out.println("numRuns=" + numRuns);
    for (int i = 0; i < parallelism.length; i++) {
      System.out.println("parallelism[" + i + "]=" + parallelism[i]
        + "\tbasefile[" + i + "]=" + basefile[i] + "\tfilesize[" + i + "]="
        + filesize[i]);
    }

    /*
     * como diversas operacoes sao realizadas dentro de um metodo "run()" de uma
     * Thread, definimos uma variavel para armazenar uma eventual excecao
     * lancada por alguma dessas threads
     */
    Exception failure;
    /* bem como uma mensagem correspondente */
    String failureMessage;

    /*
     * a variavel tryLongRun define se o teste deve tentar executar copias em
     * sequencia de um mesmo arquivo, para aumentar o tempo de execucao do
     * experimento e obter dados mais gerais do que se o experimento fosse
     * executado em poucos segundos. Ela soh tem efeito caso exista um unico
     * servidor de origem e uma unica copia (parallelism = 1)
     */
    boolean tryLongRun = true;

    for (int i = 0; i < numRuns; i++) {
      for (int j = 0; j < parallelism.length; j++) {
        failure = null;
        failureMessage = null;
        try {
          ReliableCopyThread[] threads = null;
          long start = 0;
          int threadIndex = 0;
          if ((sourceHosts.length * parallelism[j]) == 1 && tryLongRun) {
            // serao realizadas 8 copias em sequencia
            threads = new ReliableCopyThread[8];
            threadIndex = 0;
            start = System.currentTimeMillis();
            System.out.println("\n\nDATA:\t"
              + dateFormatter.format(new Date(start))
              + "\tIniciando!\thosts:\t" + sourceHosts.length + "\tthreads:\t"
              + parallelism[j] + "\tsequential:\t" + 8 + "\tmetodo:\t" + method
              + "\tFILE:\t" + basefile[j] + "\tTIME:\t" + start + "\n");
            absoluteStartTime = start;
            for (int g = 0; g < threads.length; g++) {
              String filename = basefile[j] + "_" + (g + 1);
              final RemoteFile sourceFile = sourceServers[0].getRoot()
                .getChild(new String[] { filename });
              final RemoteFile targetFile = targetServer.getRoot().createFile(
                new String[] { filename + "." + method + "." + sourceHosts[0]
                  + "." + threadIndex });
              threads[threadIndex] = new ReliableCopyThread(method, this,
                sourceFile, targetFile, 0, method);
              counter = threadIndex;
              if (threadIndex < threads.length - 1) {
                threads[threadIndex].start();
                threadIndex++;
                // antes que a proxima copia seja realizada, a anterior deve ser
                // concluida
                while (counter < threadIndex) {
                  wait();
                }
              }
              else {
                // exceto a ultima (que entra no caso geral, definido abaixo)
                threads[threads.length - 1].start();
              }
            }
          }
          else {
            /*
             * um array com todas as threads que controlarao as copias eh
             * criado. Ver classe ReliableCopyThread
             */
            threads = new ReliableCopyThread[sourceHosts.length
              * parallelism[j]];
            threadIndex = 0;
            for (int l = 0; l < parallelism[j]; l++) {
              for (int k = 0; k < sourceHosts.length; k++) {
                String filename = basefile[j] + "_" + (l + 1);
                // Alguns experimentos indicaram que o inicio de multiplas
                // copias no mesmo instante de tempo produz
                // algumas aberracoes no comportamento do servidor de destino.
                // Como eh de se esperar que os diversos
                // clientes nao estejam totalmente sincronizados nas operacoes
                // de copia, definimos uma formula
                // "magica" para distribuir a solicitacao ao longo de um
                // intervalo de tempo.
                // o valor de "delay" eh completamente ilegivel e foi gerado
                // apos algumas tentativas e erro...
                // Caso deseje alterar, sugiro definir uma formula completamente
                // nova, mas lembre de considerar o
                // tamanho dos arquivos e o numero de copias simultaneas. Para
                // garantir que na maior parte do experimento
                // todas as copias estejam sendo efetivamente realizadas
                // paralelamente.
                int delay = (int) Math
                  .min(
                    (((threadIndex * threadIndex)
                      / (int) (Math.pow(2,
                        (Math.max(sourceHosts.length - 1, 0)))) * (filesize[j] / (2 * 1024 * 1204)))),
                    (filesize[j] / (1024 * 1204)) * sourceHosts.length
                      * parallelism[j] * 4);
                final RemoteFile sourceFile = sourceServers[k].getRoot()
                  .getChild(new String[] { filename });
                final RemoteFile targetFile = targetServer.getRoot()
                  .createFile(
                    new String[] { filename + "." + method + "."
                      + sourceHosts[k] + "." + threadIndex });
                threads[threadIndex] = new ReliableCopyThread(method, this,
                  sourceFile, targetFile, delay, method);
                threadIndex++;
              }
            }
            counter = 0;
            start = System.currentTimeMillis();
            System.out.println("\n\nDATA:\t"
              + dateFormatter.format(new Date(start))
              + "\tIniciando!\thosts:\t" + sourceHosts.length + "\tthreads:\t"
              + parallelism[j] + "\tsequential:\t" + 1 + "\tmetodo:\t" + method
              + "\tFILE:\t" + basefile[j] + "\tTIME:\t" + start + "\n");
            absoluteStartTime = start;
            // Inicia todas as threads, cada uma possui o seu delay interno
            // antes de iniciar a copia.
            for (int l = 0; l < threads.length; l++) {
              threads[l].start();
            }
          }
          // espera a conclusao de todas as threads.
          while (counter < threads.length) {
            wait();
          }
          long end = System.currentTimeMillis();
          Thread.sleep(5000);
          targetServer.getMemoryStatus(true);
          System.out.println("Cooling down!!!");
          Thread.sleep(preVerifyCoolingTime);

          System.out.println("Verifying copies!!!");
          children = targetServer.getRoot().getChildren();
          long totalBytes = 0;
          for (int k = 0; k < children.length; k++) {
            totalBytes = totalBytes + children[k].size();
          }
          long expectedTotalBytes;
          if ((sourceHosts.length * parallelism[j]) == 1 && tryLongRun) {
            expectedTotalBytes = (long) 8 * filesize[j];
          }
          else {
            expectedTotalBytes = (long) parallelism[j] * filesize[j]
              * sourceHosts.length;
          }
          if (totalBytes != expectedTotalBytes) {
            failureMessage = "Method:" + method + " failed! totalBytes="
              + totalBytes + " expectedTotalBytes=" + expectedTotalBytes
              + " parallelism[j]=" + parallelism[j] + " filesize[j]="
              + filesize[j] + " sourceHosts.length=" + sourceHosts.length;
          }
          else {
            // caso o numero de bytes esteja correto, verifica o conteudo dos
            // arquivos.
            GetHashThread sourceHash = null;
            GetHashThread targetHash = null;
            for (int k = 0; k < threads.length && failureMessage == null; k++) {
              sourceHash = new GetHashThread(this, threads[k].getSourceFile());
              targetHash = new GetHashThread(this, threads[k].getTargetFile());
              counter = 0;
              sourceHash.start();
              targetHash.start();
              while (counter < 2) {
                wait();
              }
              if (sourceHash.getException() != null) {
                failureMessage = "Method:" + method
                  + " failed! parallelism[j]=" + parallelism[j]
                  + " filesize[j]=" + filesize[j] + " sourceHosts.length="
                  + sourceHosts.length;
                failure = sourceHash.getException();
              }
              if (targetHash.getException() != null) {
                failureMessage = "Method:" + method
                  + " failed! parallelism[j]=" + parallelism[j]
                  + " filesize[j]=" + filesize[j] + " sourceHosts.length="
                  + sourceHosts.length;
                failure = targetHash.getException();
              }
              if (failureMessage == null) {
                if (!Arrays.equals(sourceHash.getHashResult(), targetHash
                  .getHashResult())) {
                  failureMessage = "Method:" + method
                    + " failed! Hashes differ!!! parallelism[j]="
                    + parallelism[j] + " filesize[j]=" + filesize[j]
                    + " sourceHosts.length=" + sourceHosts.length
                    + " sourceHash.getFile().getName()="
                    + sourceHash.getFile().getName()
                    + " targetHash.getFile().getName()="
                    + targetHash.getFile().getName();
                }
                else {
                  System.out.println("HASH OK!!! k=" + k);
                }
              }
            }
          }

          System.out.println("\nDATA:\t" + dateFormatter.format(new Date(end))
            + "\tFinalizado!\thosts:\t" + sourceHosts.length + "\tthreads:\t"
            + parallelism[j] + "\tmetodo:\t" + method + "\tFILE:\t"
            + basefile[j] + "\tTIME:\t" + end + "\n");
          System.out.println("**********");
          System.out.println("***** metodo:\t" + method);
          System.out.println("***** data (MBytes):\t"
            + ((double) expectedTotalBytes / (1024 * 1024)));
          System.out.println("***** tempo (ms):\t" + (end - start));

          if (failureMessage == null && failure == null) {
            System.out
              .println("***** taxa (MB/s):\t"
                + (((double) expectedTotalBytes / (1024 * 1024)) / (((double) (end - start)) / 1000)));
            System.out.println("***** status: SUCCESS");
            System.out.println("*****");
            System.out.println("***** file:\t" + basefile[j]);
            double sum = 0;
            for (int k = 0; k < sourceServers.length; k++) {
              double rate = sourceServers[k]
                .getAverageTransferRateToHost(targetServer.getName());
              sum += rate;
              System.out.println("+++++ AverageRate: from"
                + sourceServers[k].getName() + " to " + targetServer.getName()
                + " = " + rate);
            }
            System.out.println("+++++ AggregatedRate: to "
              + targetServer.getName() + " = " + sum);
          }

          // remove os arquivos cridados
          children = targetServer.getRoot().getChildren();
          for (int k = 0; k < children.length; k++) {
            children[k].remove();
          }

        }
        catch (Exception e) {
          failure = e;
        }

        if (failureMessage != null) {
          System.out.println("***** status: FAILURE");
          System.out.println("\nDATA:\t" + dateFormatter.format(new Date())
            + " failureMessage=" + failureMessage);
          System.out.println("*****");
          System.out.println("***** file:\t" + basefile[j]);

        }
        if (failure != null) {
          System.out.println("***** status: FAILURE");
          System.err.println("\nDATA:\t" + dateFormatter.format(new Date())
            + " failure.getMessage()=" + failure.getMessage());
          System.out.println("*****");
          System.out.println("***** file:\t" + basefile[j]);
          failure.printStackTrace();
        }

        System.out.println("******** END TIME:\t" + System.currentTimeMillis());
        System.out.println("**********");

        try {
          Thread.sleep(coolingTime);
        }
        catch (InterruptedException e) {
          e.printStackTrace();
        }
      }
    }

    System.out.println("\n\n\n\nmethod(FINISHED)=" + method);
    System.out.println("TIME:" + System.currentTimeMillis());
  }

  /**
   * Chamado pelas threads para indicar o termino do seu metodo run()
   */
  public synchronized void notifyEnd() {
    counter++;
    notify();
  }

  /**
   * Executa o teste
   */
  public static void main(String[] args) {
    TestScalability test = new TestScalability();
    test.go(args);
  }

  /**
   * Realiza a copia do arquivo, armazenando alguma eventual excecao lancada.
   * Essa excecao serah usada posteriormente para avaliar se a copia
   * efetivamente ocorreu.
   */
  class ReliableCopyThread extends Thread {

    /**
     * A classe que gerencia as copias
     */
    private TestScalability parent;

    /**
     * Uma identificacao generica
     */
    private String id;

    /**
     * Uma referencia para o arquivo remoto a ser copiado
     */
    private RemoteFile sourceFile;

    /**
     * Uma referencia para o arquivo remoto de destino
     */
    private RemoteFile targetFile;

    /**
     * placeholder para a excecao
     */
    private Exception exception;

    /**
     * o tempo que essa thread deve dormir antes de iniciar a copia
     */
    private int initialDelay;

    /**
     * o metodo de copia a ser utilizado
     */
    private String method;

    /**
     * Construtor "padrao" (ver comentarios dos campos)
     * 
     * @param id
     * @param parent
     * @param sourceFile
     * @param targetFile
     * @param initialDelay
     * @param method
     */
    public ReliableCopyThread(String id, TestScalability parent,
      RemoteFile sourceFile, RemoteFile targetFile, int initialDelay,
      String method) {
      this.id = id;
      this.parent = parent;
      this.sourceFile = sourceFile;
      this.targetFile = targetFile;
      this.initialDelay = initialDelay;
      this.method = method;
    }

    public void run() {
      try {
        Thread.sleep(initialDelay);
        long start = System.currentTimeMillis();
        System.out.println("DATA:\t" + dateFormatter.format(new Date(start))
          + "\tNAME:" + this.getName() + "\tid=" + id + "\tinitialDelay="
          + initialDelay);
        sourceFile.copyTo(targetFile, method);
        long end = System.currentTimeMillis();
        System.out.println("DATA:\t" + dateFormatter.format(new Date(end))
          + "\tNAME:" + this.getName() + "\tid=" + id + " ABSOLUTE:"
          + (end - absoluteStartTime) + " LOCAL:" + (end - start));
        parent.notifyEnd();
      }
      catch (Exception e) {
        e.printStackTrace();
        this.exception = e;
      }
    }

    public RemoteFile getSourceFile() {
      return sourceFile;
    }

    public RemoteFile getTargetFile() {
      return targetFile;
    }

    public Exception getException() {
      return exception;
    }
  }

  /**
   * Thread para calcular o hash de um arquivo remoto
   */
  class GetHashThread extends Thread {

    /**
     * A classe que gerencia as copias
     */
    private TestScalability parent;

    /** O arquivo que se deseja obter o hash */
    private RemoteFile file;

    /**
     * placeholder para a excecao
     */
    private Exception exception;

    /** o resultado fornecido pelo servidor remoto */
    private byte[] hashResult;

    /**
     * Construtor "padrao" (ver comentarios dos campos)
     * 
     * @param parent
     * @param file
     */
    public GetHashThread(TestScalability parent, RemoteFile file) {
      this.parent = parent;
      this.file = file;
    }

    public void run() {
      try {
        hashResult = file.hash();
        parent.notifyEnd();
      }
      catch (Exception e) {
        this.exception = e;
      }
    }

    public RemoteFile getFile() {
      return file;
    }

    public Exception getException() {
      return exception;
    }

    public byte[] getHashResult() {
      return hashResult;
    }
  }

  /**
   * Metodo auxiliar para criar uma thread que obtem o estado da memoria no
   * servidor.
   * 
   * @param targetServer
   */
  private void startMemoryGrabberThread(final FileServer targetServer) {
    new Thread() {
      public void run() {
        while (!finished) {
          try {
            System.out.println("DATE:" + dateFormatter.format(new Date())
              + targetServer.getMemoryStatus(false));
            sleep(10000);
          }
          catch (Exception e) {
            e.printStackTrace();
          }
        }
      }
    }.start();
  }
}
