package csfs.datatransfer.blockingnio;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.SocketChannel;

import csfs.datatransfer.DataTransferMethod;
import csfs.impl.util.SystemProperties;
import csfs.impl.util.Util;
import csfs.remote.FileServer;
import csfs.remote.InvalidStateException;
import csfs.remote.NotFileException;
import csfs.remote.RemoteFile;
import csfs.remote.ServerException;

public class BlockingNIOMethod implements DataTransferMethod {

  /**
   * Copiar arquivo.
   * 
   * @param localFile File O arquivo local que servir de fonte de dados
   * @param destination RemoteFile O arquivo destino que receber os dados
   * @return boolean uma indicao de sucesso
   * @throws NotFileException Caso o arquivo destino seja um diretrio
   * @throws InvalidStateException Caso o arquivo destino seja invlido
   */
  public boolean copy(File localFile, RemoteFile destination, String method)
    throws NotFileException, InvalidStateException, IOException,
    ServerException {

    boolean response = false;
    /** Verifica se o destino aponta para um arquivo de dados */
    if (destination.isDirectory()) {
      throw new NotFileException(destination.getFullName());
    }

    String buffer = method.substring(method.indexOf('x') + 1);
    /** Obtm o endereo do servidor de cpia NIO no destino */
    String nioAddress = destination.getFileServer()
      .getCopyServerAddress(method);
    int index = nioAddress.lastIndexOf(':');
    String host = nioAddress.substring(0, index);
    int port = Integer.parseInt(nioAddress.substring(index + 1));
    /** Cria um endereo de socket com base nas informaes obtidas */
    InetSocketAddress nio = new InetSocketAddress(host, port);
    /** Recurepa o tamanho do arquivo */
    long size = localFile.length();

    String targetFilename = Util.stringArrayToString(destination.getFullName());

    int tcpBuffer = SystemProperties.getIntProperty(buffer + "_SIZE");

    /** Abre um canal NIO sobre o endereo do servidor de destino */

    SocketChannel socketChannel = null;
    while (socketChannel == null || !socketChannel.isConnected()) {
      socketChannel = SocketChannel.open();
      if (tcpBuffer != 0) {
        System.out.println("TCP_BUFFER=" + tcpBuffer);
        Socket s = socketChannel.socket();
        s.setSendBufferSize(tcpBuffer);
        System.out.println("SendBufferSize=" + s.getSendBufferSize());
        socketChannel.socket().setReceiveBufferSize(tcpBuffer);
        System.out.println("ReceiveBufferSize=" + s.getReceiveBufferSize());
      }
      try {
        socketChannel.socket().setSoTimeout(14400000); // 4 hours!!!
        socketChannel.socket().connect(nio, 14400000); // 4 hours!!!
      }
      catch (IOException e) {
        System.out.println("DATE: " + System.currentTimeMillis()
          + "\tIOException em BlockingNIOMethod! Ignoring!!!");
        try {
          Thread.sleep(10000);
        }
        catch (InterruptedException e1) {
          e1.printStackTrace();
        }
      }
    }

    /** Abre o arquivo local */
    FileInputStream file = new FileInputStream(localFile);
    /** Recupera o canal NIO de leitura do arquivo */
    FileChannel fileChannel = file.getChannel();
    /** Inicializa o protocolo inicial para envio dos dados */
    String initProtocol = "" + size + "\n" + targetFilename + "\n";

    int headerSize = initProtocol.length();
    String headerSizeStr = headerSize + "\n";
    ByteBuffer headerLength = ByteBuffer.allocate(7);
    headerLength.put(headerSizeStr.getBytes());
    headerLength.rewind();
    socketChannel.write(headerLength);

    /** Aloca um buffer para envio de dados ao servidor */
    ByteBuffer out = ByteBuffer.allocate(initProtocol.length());
    /** Escreve as informacoes do protocolo inical no buffer */
    out.put(initProtocol.getBytes());
    /** Rebobina o buffer */
    out.rewind();
    /** Envia os dados do protocolo para o servidor */
    socketChannel.write(out);
    /** Solicita o envio do arquivo atravs de uma funo de alto desempenho */
    long bytesWritten = 0;
    while (bytesWritten < size) {
      bytesWritten += fileChannel.transferTo(bytesWritten, size - bytesWritten,
        socketChannel);
    }
    /** Fecha o canal de leitura do arquivo local */
    fileChannel.close();
    /** Aloca o buffer de recepcao da confirmao do servidor */
    ByteBuffer result = ByteBuffer.allocate(1);
    /** Bloqueia ate o recebimento da confirmao */
    socketChannel.read(result);
    if (result.get(0) == 0) {
      response = true;
    }
    /** Fecha o socket utilizado para o envio */
    socketChannel.close();

    return response;
  }

  public String getName() {
    return "BLOCKING_NIO";
  }

  public String getDetailedInfo() {
    return "BLOCKING_NIO Single Stream";
  }

}
