package csfs.datatransfer.blockingnio;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.Socket;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.SocketChannel;

import csfs.impl.util.Util;

/**
 * A classe <class>NIOReceiverThread</class> implementa uma thread que recebe
 * os dados do arquivo e os escreve no disco local. Inicialmente, o protocolo 
 * tratado para obter informaes a respeito da cpia
 * 
 * @version 1.0 27 Jun 2005
 * @author Marcelo Nery dos Santos
 */

public class BlockingNIOReceiverThread extends Thread {

  /** O socket que foi criado pelo servidor de cpia */
  private Socket socket;

  /**
   * A raiz onde os dados devem ser escritos, caminhos relativos so obtidos
   * pelo socket.
   */
  private String rootDir;

  public static final int HEADER_SIZE = 7;

  /**
   * Contrutor
   * 
   * @param socket Socket O socket que foi criado pelo servidor de cpia
   * @param rootDir String A raiz onde os dados devem ser escritos, caminhos
   *        relativos so obtidos pelo socket.
   */
  public BlockingNIOReceiverThread(Socket socket, String rootDir) {
    this.socket = socket;
    this.rootDir = rootDir;
  }

  public void run() {
    SocketChannel socketChannel = null;
    ByteBuffer response = ByteBuffer.allocate(1);
    try {
      /** Obtem o canal do socket */
      socketChannel = socket.getChannel();
      try {
        /** Aloca o buffer de tamanho fixo para leitura do tamanho do header */
        ByteBuffer headerSizeBuf = ByteBuffer.allocate(HEADER_SIZE);

        /** le o tamanho do header completamente */
        int bytesRead = 0;
        while (bytesRead < HEADER_SIZE) {
          bytesRead += socketChannel.read(headerSizeBuf);
        }
        headerSizeBuf.rewind();

        /** Aloca o buffer de tamanho fixo para leitura do header */
        int headerSize = (int) Util.readLongFromBuffer(headerSizeBuf);

        /** le o header completamente */
        ByteBuffer headerBuf = ByteBuffer.allocate(headerSize);
        bytesRead = 0;
        while (bytesRead < headerSize) {
          bytesRead += socketChannel.read(headerBuf);
        }
        headerBuf.rewind();

        long length = Util.readLongFromBuffer(headerBuf);
        String path = Util.readStringFromBuffer(headerBuf);

        writeDataToDisk(socketChannel, path, length);

        response.put(0, (byte) 0);
      }
      catch (IOException e) {
        response.put(0, (byte) 1);
      }
      finally {
        if (socketChannel != null) {
          socketChannel.write(response);
          socketChannel.close();
        }
      }
    }
    catch (IOException ex) {
      ex.printStackTrace();
    }
  }

  private void writeDataToDisk(SocketChannel socketChannel, String path,
    long length) throws FileNotFoundException, IOException {

    /** Elimina caracteres iniciais desnecessarios */
    while (path.charAt(0) == '/') {
      path = path.substring(1);
    }

    /** Converte o separador de arquivos para o sistema local */
    path = Util.systemFileSeparatorToLocalFileSeparator(path);

    /** Criar o descritor de arquivo para escrever os dados */
    FileOutputStream file = new FileOutputStream(new File(rootDir, path));

    /** Recupera o canal de escrita NIO */
    FileChannel fileChannel = file.getChannel();

    long transferred = 0;
    while (transferred < length) {
      transferred += fileChannel.transferFrom(socketChannel, transferred,
        length - transferred);
    }

    /** Fecha o canal */
    fileChannel.close();
  }
}
