package csfs.datatransfer.corba;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import csfs.datatransfer.DataTransferMethod;
import csfs.impl.util.SystemProperties;
import csfs.remote.ClosedChannelException;
import csfs.remote.InvalidStateException;
import csfs.remote.NotFileException;
import csfs.remote.RemoteFile;
import csfs.remote.ServerException;
import csfs.remote.WriteChannel;

/**
 * A classe <class>CORBAMethod</class>  a implementao da interface
 * DataTransferMethod, para a realizao de transferencias de arquivos via
 * CORBA.
 * 
 * @version 1.0 27 Jun 2005
 * @author Marcelo Nery dos Santos
 */
public class CORBAMethod implements DataTransferMethod {

  /**
   * Copiar arquivo.
   * 
   * @param localFile File O arquivo local que servir de fonte de dados
   * @param destination RemoteFile O arquivo destino que receber os dados
   * @return boolean uma indicao de sucesso
   * @throws NotFileException Caso o arquivo destino seja um diretrio
   * @throws InvalidStateException Caso o arquivo destino seja invlido
   */
  public boolean copy(File localFile, RemoteFile destination, String method)
    throws NotFileException, InvalidStateException, ServerException {
    boolean response = false;
    try {
      /** recupera o canal de escrita do arquivo destino */
      WriteChannel writer = destination.getWriteChannel();

      /**
       * Essa verificao j foi realizada no Servant de RemoteFile, no 
       * necessrio definir o caminho do arquivo remoto para a exceo.
       */
      if (!localFile.isFile()) {
        throw new InvalidStateException(
          "It is not possible to copy a directory", new String[] {});
      }

      int corbaBuffer = Integer.parseInt(method
        .substring(method.indexOf('-') + 1));

      /** define o buffer de envio de dados */
      byte[] buffer = new byte[corbaBuffer];

      long read = 0;
      long size = localFile.length();

      /** Abre o arquivo local para leitura */
      InputStream reader;

      int bufferSize = SystemProperties
        .getIntProperty("BUFFERED_STREAM_SIZE_READER");

      if (bufferSize > 0) {
        reader = new BufferedInputStream(new FileInputStream(localFile),
          bufferSize);
      }
      else {
        reader = new FileInputStream(localFile);
      }

      int bytesRead = 0;
      /** Enquando o arquivo no foi lido completamente */
      while (read < size && bytesRead != -1) {
        int localRead = 0;
        /** Enquanto o buffer nao  preenchido e EOF no  encontrado */
        do {
          bytesRead = reader.read(buffer, localRead, buffer.length - localRead);
          if (bytesRead != -1) {
            localRead += bytesRead;
          }
        } while (bytesRead != -1 && localRead != buffer.length);
        /** atualiza o nmero de bytes lidos do arquivo */
        read += localRead;
        /**
         * escreve o nmero correto de dados no canal, podemos criar um buffer
         * do tamanho exato para evitar o envio desnecessrios de dados para o
         * servidor.
         */
        writer.write(localRead, buffer);
      }
      /** fecha o arquivo local */
      reader.close();
      /** fecha o canal de escrita */
      writer.close();
      /**
       * Se o nmero de bytes enviados corresponde ao tamanho original do
       * arquivo
       */
      if (read == size) {
        response = true;
      }
    }
    catch (ClosedChannelException ex) {
      /** o canal foi aberto nesse metodo, logo ele nao pode estar fechado */
      ex.printStackTrace();
      throw new InvalidStateException(
        "Destination channel was misteriously closed during copy operation:"
          + ex.getMessage(), destination.getFullName());
    }
    catch (IOException ex) {
      /** Caso haja erro na leitura de dados do arquivo local */
      ex.printStackTrace();
      throw new InvalidStateException("Problems reading from local file ("
        + localFile.getPath() + "):" + ex.getMessage(), new String[0]);
    }
    return response;
  }

  public String getName() {
    return "CORBA";
  }

  public String getDetailedInfo() {
    return "CORBA" + SystemProperties.getProperty("CORBA_BUFFER_SIZE");
  }
}
