package csfs.datatransfer.nonblockingnio;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.channels.FileChannel;
import java.nio.channels.SocketChannel;
import java.util.HashMap;
import java.util.Map;

/**
 * Gerenciador de importao. Cada importao ser conduzida por um objeto desta
 * classe.
 * 
 * @author Leonardo Barros
 */
public class ImportHandler {
  /**
   * Coleo de referncias para os gerenciadores ativos (um para cada
   * importao).
   */
  private static Map activeImports = new HashMap();

  /** Arquivo de sada. */
  private FileOutputStream outFile;
  /** Canal de origem. */
  private SocketChannel sourceChannel;
  /** Canal de destino. */
  private FileChannel targetChannel;
  /** Estado atual do objeto (pegando tamanho, pegando arquivo, fechado). */
  private ImportState state;
  /** Caminho do arquivo a ser importado. */
  private String filePath;
  /** Tamanho do arquivo a ser importado. */
  private long fileSize;
  /** Raiz do sistema de arquivos. */
  private String rootDir;

  /**
   * Cria um gerenciador associado ao canal especificado. Este  iniciado no
   * estado <code>GettingSize</code> (pegando o tamanho do arquivo).
   * 
   * @param channel canal do qual ser feita a importao.
   */
  public ImportHandler(SocketChannel channel, String rootDir) {
    sourceChannel = channel;
    this.rootDir = rootDir;
    state = new GettingPathSize(this);
  }

  /**
   * Delega  classe que implementa o estado a operao de leitura do canal.
   * 
   * @throws IOException
   */
  public void read() throws IOException {
    state.read();
  }

  /**
   * Finaliza a importao, fechando todos os recursos abertos e removendo o
   * gerenciador da coleo de gerenciadores ativos.
   * 
   * @throws IOException
   */
  public void close() throws IOException {
    if (outFile != null)
      outFile.close();
    if (targetChannel != null)
      targetChannel.close();
    if (sourceChannel != null)
      sourceChannel.close();
    activeImports.remove(sourceChannel);
  }

  /**
   * Obtm o canal de destino para a importao.
   * 
   * @return canal de destino para a importao.
   */
  public FileChannel getTargetChannel() {
    return targetChannel;
  }

  /**
   * Muda o estado do gerenciador.
   * 
   * @param state novo estado.
   */
  public void changeState(ImportState state) {
    this.state = state;
  }

  /**
   * Obtm o canal de origem da importao.
   * 
   * @return canal de origem da importao.
   */
  public SocketChannel getSourceChannel() {
    return sourceChannel;
  }

  /**
   * Cria um novo gerenciador para tratar eventos de leitura do canal
   * especificado.
   * 
   * @param channel canal ao qual ser associado o novo gerenciador.
   */
  public static void createHandler(SocketChannel channel, String rootDir) {
    activeImports.put(channel, new ImportHandler(channel, rootDir));
  }

  /**
   * Obtm o gerenciador associado ao canal especificado.
   * 
   * @param channel canal com dados a serem lidos.
   * 
   * @return gerenciador que ir tratar da leitura no canal especificado.
   */
  public static ImportHandler getHandler(SocketChannel channel) {
    return (ImportHandler) activeImports.get(channel);
  }

  /**
   * Atribui o caminho do arquivo a ser importado.
   * 
   * @param filePath novo caminho para o arquivo a ser importado.
   * @throws FileNotFoundException
   */
  public void setFilePath(String filePath) throws FileNotFoundException {
    this.filePath = rootDir + File.separator + filePath;
    outFile = new FileOutputStream(this.filePath);
    targetChannel = outFile.getChannel();
  }

  /**
   * Obtm o caminho do arquivo a ser importado.
   * 
   * @return caminho do arquivo a ser importado.
   */
  public String getFilePath() {
    return filePath;
  }

  /**
   * Obtm o tamanho do arquivo a ser importado.
   * 
   * @return tamanho do arquivo a ser importado.
   */
  public long getFileSize() {
    return fileSize;
  }

  /**
   * Atribui o tamanho do arquivo a ser importado.
   * 
   * @param fileSize novo tamanho para o arquivo a ser importado.
   */
  public void setFileSize(long fileSize) {
    this.fileSize = fileSize;
  }

}
