package csfs.datatransfer.nonblockingnio;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.SocketChannel;
import java.nio.charset.Charset;
import java.nio.charset.CharsetEncoder;

/**
 * Cliente para exportao de arquivos, semelhante a um cliente FTP, mas usando
 * a API NIO do Java.
 * 
 * @author Leonardo Barros
 */
public class NonBlockingNIOClient {
  /** Codificao para transmisso de caracteres */
  private static final String ENCODING = "ISO-8859-1";

  /** Host default para o servidor */
  private final static String DEFAULT_SERVER = "localhost";

  /** Porta default em que o servidor aguarda conexes. */
  private final static int DEFAULT_PORT_NUMBER = 7780;

  /**
   * Cria o cliente usando o host e a porta especificados.
   * 
   * @param args argumentos da linha de comando.
   */
  public static void main(String[] args) {
    if (args.length < 1) {
      terminate();
    }
    String server = getServer(args);
    int port = getServerPort(args);
    String filePath = getFilePath(args);
    NonBlockingNIOClient client = new NonBlockingNIOClient();
    boolean success = client.sendFile(server, port, filePath, filePath
      + ".copy");
    System.out.println(success ? "Transferncia completada com sucesso."
      : "Erro na transferncia.");
  }

  /**
   * Obtm o host do servidor (default: <code>DEFAULT_SERVER</code>).
   * 
   * @param args argumentos da linha de comando.
   * 
   * @return host do servidor.
   */
  private static String getServer(String[] args) {
    String server = DEFAULT_SERVER;
    for (int i = 0; i < args.length; i++) {
      if (args[i].equals("-s")) {
        if (args.length < i + 2) {
          terminate();
        }
        server = args[i + 1];
        break;
      }
    }
    System.out.println("Conectando ao servidor: " + server);
    return server;
  }

  /**
   * Obtm a porta na qual o servidor dever escutar (default:
   * <code>DEFAULT_PORT_NUMBER</code>).
   * 
   * @param args argumentos da linha de comando.
   * 
   * @return porta na qual o servidor dever escutar.
   */
  private static int getServerPort(String[] args) {
    int port = DEFAULT_PORT_NUMBER;
    for (int i = 0; i < args.length; i++) {
      if (args[i].equals("-p")) {
        try {
          if (args.length < i + 2) {
            terminate();
          }
          port = Integer.parseInt(args[i + 1]);
        }
        catch (NumberFormatException ex) {
          terminate();
        }
        break;
      }
    }
    System.out.println("Porta: " + port);
    return port;
  }

  /**
   * Obtm caminho para o arquivo a ser exportado.
   * 
   * @param args argumentos da linha de comando.
   * 
   * @return caminho para o arquivo.
   */
  private static String getFilePath(String[] args) {
    String path = null;
    for (int i = 0; i < args.length; i++) {
      if (args[i].equals("-f")) {
        if (args.length < i + 2) {
          terminate();
        }
        path = args[i + 1];
        break;
      }
    }
    if (path == null) {
      terminate();
    }
    System.out.println("Caminho do arquivo: " + path);
    return path;
  }

  /**
   * Exibe mensagem de erro e aborta a execuo.
   */
  private static void terminate() {
    System.out
      .println("Uso: java FTPNIOClient [-s <servidor>] [-p <numero_porta>] "
        + "-f <caminho_arquivo>");
    System.exit(1);
  }

  /**
   * Abre um socket para conexo com o servidor, abre um arquivo e o transfere
   * para o servidor, usando o NIO.
   */
  public boolean sendFile(String server, int port, String sourcePath,
    String targetPath) {
    SocketChannel serverChannel = null;
    while (serverChannel == null || !serverChannel.isConnected()) {
      try {
        serverChannel = SocketChannel.open();
        serverChannel.socket().setSoTimeout(14400000); // 4 hours!!!
        serverChannel.socket().connect(new InetSocketAddress(server, port),
          14400000); // 4 hours!!!
      }
      catch (IOException e) {
        System.out
          .println("DATE: "
            + System.currentTimeMillis()
            + "\tIOException em NonBlockingNIOClient:sendfile():connect()! Ignoring!!!");
        try {
          Thread.sleep(10000);
        }
        catch (InterruptedException e1) {
          e1.printStackTrace();
        }
      }
    }
    FileInputStream in = null;
    FileChannel fileChannel = null;
    try {
      int pathSize = targetPath.length();
      putPathSize(pathSize, serverChannel);
      putPath(targetPath, serverChannel);
      in = new FileInputStream(sourcePath);
      fileChannel = in.getChannel();
      long fileSize = fileChannel.size();
      putFileSize(fileSize, serverChannel);
      putFile(fileChannel, fileSize, serverChannel);
      return waitForAck(serverChannel);
    }
    catch (FileNotFoundException e) {
      System.err
        .println("No foi possvel abrir o arquivo " + sourcePath + ".");
      e.printStackTrace();
      return false;
    }
    catch (IOException e) {
      System.err.println("No foi possvel transferir o arquivo " + sourcePath
        + ".");
      System.out.println("TIME:" + System.currentTimeMillis());
      e.printStackTrace();
      return false;
    }
    finally {
      if (fileChannel != null) {
        try {
          fileChannel.close();
        }
        catch (IOException e) {
        }
      }
      if (serverChannel != null) {
        try {
          serverChannel.close();
        }
        catch (IOException e) {
        }
      }
      if (in != null) {
        try {
          in.close();
        }
        catch (IOException e) {
        }
      }
    }
  }

  /**
   * Aguarda por um byte de "acknowledgment" do servidor indicando que a
   * transferncia foi concluda.
   * 
   * @param channel canal de onde vir o "acknowledgement".
   * 
   * @return true se o byte recebido for do valor esperado.
   * 
   * @throws IOException
   */
  private boolean waitForAck(SocketChannel channel) throws IOException {
    ByteBuffer ack = ByteBuffer.allocate(1);
    channel.read(ack);
    ack.flip();
    return (ack.get() == (byte) 10);
  }

  /**
   * Envia o caminho do arquivo pelo canal especificado.
   * 
   * @param filePath caminho para o arquivo.
   * @param channel canal no qual ser transmitida a informao.
   * 
   * @throws IOException
   * 
   * @throws IOException
   */
  private void putPath(String filePath, SocketChannel channel)
    throws IOException {
    CharBuffer charBuffer = CharBuffer.wrap(filePath.toCharArray());
    Charset charset = Charset.forName(ENCODING);
    CharsetEncoder encoder = charset.newEncoder();
    ByteBuffer byteBuffer = encoder.encode(charBuffer);
    channel.write(byteBuffer);
  }

  /**
   * Envia o arquivo pelo canal especificado.
   * 
   * @param sourceChannel canal de origem (<code>FileChannel</code>) do
   *        arquivo.
   * @param remainingBytes nmero de bytes a serem transmitidos (tamanho).
   * @param targetChannel canal de destino (<code>SocketChannel</code>).
   * 
   * @throws IOException
   */
  private void putFile(FileChannel sourceChannel, long remainingBytes,
    SocketChannel targetChannel) throws IOException {
    while (remainingBytes > 0) {
      long readBytes = sourceChannel.transferTo(0, remainingBytes,
        targetChannel);
      remainingBytes -= readBytes;
    }
  }

  /**
   * Envia o tamanho do arquivo a ser transmitido pelo canal especificado.
   * 
   * @param fileSize tamanho do arquivo (em bytes).
   * @param channel canal no qual ser transmitida a informao.
   * 
   * @throws IOException
   */
  private void putFileSize(long fileSize, SocketChannel channel)
    throws IOException {
    ByteBuffer buffer = ByteBuffer.allocateDirect(8);
    buffer.putLong(fileSize);
    buffer.flip();
    channel.write(buffer);
  }

  /**
   * Envia o tamanho do nome do arquivo a ser transmitido pelo canal
   * especificado.
   * 
   * @param pathSize tamanho do nome do arquivo (em bytes).
   * @param channel canal no qual ser transmitida a informao.
   * 
   * @throws IOException
   */

  private void putPathSize(int pathSize, SocketChannel channel)
    throws IOException {
    ByteBuffer buffer = ByteBuffer.allocateDirect(4);
    buffer.putInt(pathSize);
    buffer.flip();
    channel.write(buffer);
  }

}
