package csfs.datatransfer.nonblockingnio;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.nio.channels.ClosedChannelException;
import java.nio.channels.SelectableChannel;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.util.Iterator;
import java.util.logging.Logger;

/**
 * Servidor para importao de arquivos, semelhante a um servidor FTP, mas
 * usando a API NIO (com sockets no-bloqueantes) do Java.
 * 
 * @author Leonardo Barros
 */
public class NonBlockingNIOServer extends Thread {
  /** Porta default em que o servidor aguarda conexes. */
  private final static int DEFAULT_PORT_NUMBER = 7780;

  /**
   * Multiplexador (<code>Selector</code>) no qual sero registrados
   * sockets. Sempre que esse multiplexador receber um evento do SO, vai
   * verificar se existe algum socket interessado e disparar a callback
   * associada ao mesmo.
   */
  private Selector selector;

  private String rootDir = ".";
  private int port = DEFAULT_PORT_NUMBER;

  /**
   * Cria o servidor.
   * 
   * @param args
   */
  public static void main(String[] args) {
    int port = getServerPort(args);
    new NonBlockingNIOServer(""/* "." */, port).start();
  }

  /**
   * Obtm a porta na qual o servidor dever escutar (default:
   * <code>DEFAULT_PORT_NUMBER</code>).
   * 
   * @param args argumentos da linha de comando.
   * 
   * @return porta na qual o servidor dever escutar.
   */
  private static int getServerPort(String[] args) {
    int port = DEFAULT_PORT_NUMBER;
    if (args.length > 0) {
      try {
        port = Integer.parseInt(args[0]);
      }
      catch (NumberFormatException ex) {
        System.out.println("Uso: java FTPNIOServer <numero_porta>");
        System.exit(1);
      }
    }
    // System.out.println("Escutando na porta "+port);
    return port;
  }

  /**
   * @param port porta na qual o servidor dever aguardar conexes.
   */
  public NonBlockingNIOServer(String rootDir, int port) {
    this.rootDir = rootDir;
    this.port = port;
  }

  /**
   * Cria um novo <code>ServerSocketChannel</code>, no-bloqueante, na porta
   * especificada.
   * 
   * @param port porta na qual o socket vai escutar.
   * 
   * @return <code>ServerSocketChannel</code>
   * 
   * @throws IOException
   */
  private ServerSocketChannel createServerSocketChannel(int port)
    throws IOException {
    ServerSocketChannel serverChannel = ServerSocketChannel.open();
    ServerSocket serverSocket = serverChannel.socket();
    serverSocket.bind(new InetSocketAddress(port));
    Logger.global.info("NonBlockingNIOServer BOUND TO:" + port);
    serverChannel.configureBlocking(false);
    return serverChannel;
  }

  /**
   * Estabelece conexo com um socket cliente. Registra esse socket para eventos
   * de leitura (sempre que o socket cliente estiver disponvel para leitura, o
   * multiplexador ser avisado).
   * 
   * @param key chave contendo o socket do servidor.
   * 
   * @throws IOException
   */
  private void acceptConnection(SelectionKey key) throws IOException {
    ServerSocketChannel serverChannel = (ServerSocketChannel) key.channel();
    SocketChannel clientChannel = serverChannel.accept();
    // System.out.println("Contato estabelecido com: "+clientChannel.socket());
    registerChannel(clientChannel, SelectionKey.OP_READ);
    ImportHandler.createHandler(clientChannel, rootDir);
  }

  /**
   * Registra o canal especificado no multiplexador.
   * 
   * @param channel canal a ser registrado.
   * @param ops eventos (operaes) a observar no canal.
   * 
   * @throws IOException
   */
  private void registerChannel(SelectableChannel channel, int ops)
    throws IOException {
    if (channel == null) {
      return;
    }
    channel.configureBlocking(false);
    channel.register(selector, ops);
  }

  /**
   * Carrega o <code>ImportHandler</code> adequado para tratar o evento de
   * leitura.
   * 
   * @param key chave representando o evento.
   * 
   * @throws IOException
   */
  private void handleRead(SelectionKey key) throws Exception {
    SocketChannel clientChannel = null;
    clientChannel = (SocketChannel) key.channel();
    ImportHandler handler = ImportHandler.getHandler(clientChannel);
    if (handler == null) {
      throw new Exception("Handler no encontrado!!!");
    }
    handler.read();
  }

  /**
   * Cria um multiplexador (<code>Selector</code>) no qual sero registrados
   * sockets. Sempre que esse multiplexador receber um evento do SO, vai
   * verificar se existe algum socket interessado e disparar a callback
   * associada ao mesmo.
   * 
   */
  public void run() {
    try {
      selector = Selector.open();
    }
    catch (IOException e) {
      System.err.println("No foi possvel criar o multiplexador.");
      e.printStackTrace();
    }
    ServerSocketChannel serverChannel = null;
    try {
      serverChannel = createServerSocketChannel(port);
      serverChannel.register(selector, SelectionKey.OP_ACCEPT);
    }
    catch (ClosedChannelException e) {
      System.err.println("No foi possvel registrar o socket do servidor.");
      e.printStackTrace();
    }
    catch (IOException e) {
      System.err.println("No foi possvel criar o socket do servidor.");
      e.printStackTrace();
    }
    try {
      while (true) {
        int selectedKeysCount = 0;
        // Bloqueia, aguardando por eventos
        selectedKeysCount = selector.select();
        if (selectedKeysCount == 0)
          continue;
        Iterator it = selector.selectedKeys().iterator();
        while (it.hasNext()) {
          SelectionKey key = (SelectionKey) it.next();
          if (key.isAcceptable()) {
            acceptConnection(key);
          }
          if (key.isReadable()) {
            handleRead(key);
          }
          it.remove();
        }
      }
    }
    catch (IOException e) {
      System.err.println("No foi possvel tratar um evento do servidor.");
      e.printStackTrace();
    }
    catch (Exception e) {
      System.err.println("No foi possvel tratar um evento do servidor.");
      e.printStackTrace();
    }
  }

}
