package csfs.datatransfer.scp;

import java.io.File;
import java.io.IOException;
import java.util.logging.Logger;

import csfs.datatransfer.DataTransferMethod;
import csfs.impl.util.SystemProperties;
import csfs.impl.util.Util;
import csfs.remote.FileServer;
import csfs.remote.InvalidStateException;
import csfs.remote.NotFileException;
import csfs.remote.RemoteFile;
import csfs.remote.ServerException;

/**
 * Mtodo de transferncia de dados que faz uso do comando "scp" (Unix).
 * 
 * @author lbarros
 */
public class SCPMethod implements DataTransferMethod {

  /** Propriedade que informa o caminho do diretrio raiz. */
  private static final String ROOT_DIR = "CANONICAL_ROOT_DIR";
  /** Propriedade que informa o caminho para o executvel do scp. */
  private static final String SCP_BINARY = "SCP_BINARY";
  /** Propriedade que informa o usurio que deve efetuar a cpia. */
  private static final String USER = "USER";

  /**
   * Copia um arquivo local para um destino remoto, usando scp.
   * 
   * @param localFile referncia para o arquivo local.
   * @param destination referncia para o arquivo remoto.
   * @param method "apelido" do mtodo para o <code>FileServer</code>; usado
   *        somente para recuperar o host de destino.
   * 
   * @return <code>true</code> em caso de cpia bem-sucedida,
   *         <code>false</code> em caso contrrio.
   */
  public boolean copy(File localFile, RemoteFile destination, String method)
    throws NotFileException, InvalidStateException, IOException,
    ServerException {
    FileServer destinationServer = destination.getFileServer();
    String host = destinationServer.getCopyServerAddress(method);
    String targetFilename = Util.stringArrayToString(destination.getFullName());
    String targetPath = destinationServer.getProperty(ROOT_DIR);
    String target = Util.stringArrayToString(new String[] { targetPath,
        targetFilename });
    char remoteFileSeparator = destination.getFileServer().getFileSeparator()
      .charAt(0);
    target = Util.remoteFileSeparatorToSystemFileSeparator(target,
      remoteFileSeparator);
    String scpBinaryPath = SystemProperties.getProperty(SCP_BINARY);
    scpBinaryPath = Util.localFileSeparatorToSystemFileSeparator(scpBinaryPath);
    String source = localFile.getAbsolutePath();
    source = Util.localFileSeparatorToSystemFileSeparator(source);
    String user = destinationServer.getProperty(USER);
    if (user == null) {
      user = "";
    }
    else if (!user.trim().equals("")) {
      user = user.trim() + "@";
    }
    String cmd = scpBinaryPath + " " + source + " " + user + host + ":"
      + target;
    Logger.global.info(">> cmd: " + cmd);
    Process p = Runtime.getRuntime().exec(cmd);
    int result = -1;
    try {
      result = p.waitFor();
    }
    catch (InterruptedException e) {
      // #TODO (Confirmar com Ana) essa espera no deveria ser interrompida
      return false;
    }
    return (result == 0);
  }

  /**
   * Retorna o nome do mtodo.
   * 
   * @return nome do mtodo.
   */
  public String getName() {
    return "SCPMethod";
  }

  /**
   * Retorna informaes mais detalhadas sobre o mtodo.
   * 
   * @return informaes detalhadas sobre o mtodo.
   */
  public String getDetailedInfo() {
    return "SCP Method (scp - Unix built-in)";
  }

}
