package csfs.datatransfer.streamblockingnio;

import java.io.File;
import java.net.InetSocketAddress;
import java.util.HashMap;

import csfs.datatransfer.DataTransferMethod;
import csfs.impl.util.IDGenerator;
import csfs.impl.util.SystemProperties;
import csfs.impl.util.Util;
import csfs.remote.InvalidStateException;
import csfs.remote.NotFileException;
import csfs.remote.RemoteFile;
import csfs.remote.ServerException;

/**
 * A classe <class>StreamBlockingNIOMethod</class>  a implementao da
 * interface DataTransferMethod, para a realizao de transferencias de arquivos
 * via NIO. Ela utiliza o protocolo de multiplas streams baseadas no NIO
 * Bloqueante. Uma implementacao com NIO nao bloqueante (Gather, etc...) seria
 * interessante.
 * 
 * @version 1.0 27 Jun 2005
 * @author Marcelo Nery dos Santos
 */
public class StreamBlockingNIOMethod implements DataTransferMethod {

  private HashMap<Long, Integer> activeCopies;

  public StreamBlockingNIOMethod() {
    activeCopies = new HashMap<Long, Integer>();
  }

  public synchronized void notifyThreadEnd(StreamBlockingNIOSenderThread thread) {
    Long key = new Long(thread.getCopyID());
    if (thread.getResponse()) {
      int activeCount = ((Integer) activeCopies.get(key)).intValue();
      activeCount--;
      if (activeCount == 0) {
        activeCopies.remove(key);
        notifyAll();
      }
      else {
        activeCopies.put(key, new Integer(activeCount));
      }
    }
    else {
      // TODO ralizar um tratamento mais adequado!
      System.out.println("COPY FAILED!!!" + thread.getFilename());
    }
  }

  /**
   * Copiar arquivo.
   * 
   * @param localFile File O arquivo local que servir de fonte de dados
   * @param destination RemoteFile O arquivo destino que receber os dados
   * @return boolean uma indicao de sucesso
   * @throws NotFileException Caso o arquivo destino seja um diretrio
   * @throws InvalidStateException Caso o arquivo destino seja invlido
   */
  public synchronized boolean copy(File localFile, RemoteFile destination,
    String method) throws NotFileException, InvalidStateException,
    ServerException {

    boolean response = false;
    /** Verifica se o destino aponta para um arquivo de dados */
    if (destination.isDirectory()) {
      throw new NotFileException(destination.getFullName());
    }
    int indexMethod = method.indexOf('-');
    int numChannels = Integer.parseInt(method.substring(indexMethod + 1));
    /** Obtm o endereo do servidor de cpia NIO no destino */
    String nioAddress = destination.getFileServer().getCopyServerAddress(
      method.substring(0, indexMethod));
    int index = nioAddress.lastIndexOf(':');
    String host = nioAddress.substring(0, index);
    int port = Integer.parseInt(nioAddress.substring(index + 1));
    /** Cria um endereo de socket com base nas informaes obtidas */
    InetSocketAddress nio = new InetSocketAddress(host, port);
    /** Recurepa o tamanho do arquivo */
    long size = localFile.length();

    long expectedBytesPerChannel = size / numChannels;
    long realBytesPerChannel = expectedBytesPerChannel;

    StreamBlockingNIOSenderThread[] threads = new StreamBlockingNIOSenderThread[numChannels];

    long copyId = IDGenerator.nextId();

    String targetFilename = Util.stringArrayToString(destination.getFullName());

    for (int i = 0; i < threads.length; i++) {
      if (i == threads.length - 1) {
        realBytesPerChannel = size - expectedBytesPerChannel * i;
      }
      threads[i] = new StreamBlockingNIOSenderThread(nio, localFile,
        expectedBytesPerChannel * i, realBytesPerChannel, targetFilename,
        copyId, this);
    }

    Long key = new Long(copyId);
    activeCopies.put(key, new Integer(numChannels));

    for (int i = 0; i < threads.length; i++) {
      threads[i].start();
    }

    while (activeCopies.get(key) != null) {
      try {
        wait();
      }
      catch (InterruptedException exc) {
        exc.printStackTrace();
      }
    }

    response = true;
    for (int i = 0; i < threads.length; i++) {
      response = response && threads[i].getResponse();
    }
    return response;
  }

  // public long probe(FileServer target, boolean accessDisk, int dataSize,
  // String method) throws ServerException {
  //
  // try {
  // String nioAddress = target.getCopyServerAddress("STREAM_BLOCKING_NIO");
  // int index = nioAddress.lastIndexOf(':');
  // String host = nioAddress.substring(0,index);
  // int port = Integer.parseInt(nioAddress.substring(index+1));
  // InetSocketAddress nio = new InetSocketAddress(host,port);
  //
  // if (accessDisk) {
  // File sample = null;
  // try {
  // sample = new File("PROBE_"+dataSize);
  // BufferedOutputStream out = new BufferedOutputStream(new
  // FileOutputStream(sample));
  // for (int i = 0; i < dataSize; i++) {
  // out.write(0);
  // }
  // out.close();
  // } catch (Exception ex) {
  // ex.printStackTrace();
  // }
  // RemoteFile destination = target.getRoot().createFile(new
  // String[]{"An_uncommon_name_GFJFDGHKFGH"});
  // long time = System.currentTimeMillis();
  // copy(sample,destination,method);
  // time = System.currentTimeMillis() - time;
  // sample.delete();
  // destination.remove();
  // return time;
  // } else {
  // SocketChannel socketChannel = SocketChannel.open(nio);
  // /** Inicializa o protocolo inicial para envio dos dados */
  // String initProtocol = "" + 0 + "\n" + dataSize + "\n" + "0" + "\n" +
  // "unspecified" + "\n";
  //
  // ByteBuffer init = ByteBuffer.allocate(initProtocol.length());
  // /** Escreve as informacoes do protocolo inical no buffer */
  // init.put(initProtocol.getBytes());
  // /** Rebobina o buffer */
  // init.rewind();
  // /** Envia os dados do protocolo para o servidor */
  // socketChannel.write(init);
  //
  // /** Aloca um buffer para envio de dados ao servidor */
  // int bufSize = SystemProperties.getIntProperty("CORBA_BUFFER_SIZE");
  // if (dataSize<bufSize) {
  // bufSize = dataSize;
  // }
  // ByteBuffer out = ByteBuffer.allocate(bufSize);
  // for (int i = 0; i < bufSize; i++) {
  // out.put(i,(byte)0);
  // }
  //
  // long sent = 0;
  // long time = System.currentTimeMillis();
  // while (sent<dataSize) {
  // sent = sent + socketChannel.write(out);
  // if (out.position()==out.capacity()) {
  // if (dataSize-sent<bufSize) {
  // out.position(bufSize-((int)(dataSize-sent)));
  // } else {
  // out.rewind();
  // }
  // }
  // }
  // ByteBuffer result = ByteBuffer.allocate(1);
  // socketChannel.read(result);
  // if (result.get(0) != 0) {
  // System.out.println("ERRO NA COPIA SEM USAR O DISCO");
  // // response = true;
  // }
  // socketChannel.close();
  // time = System.currentTimeMillis() - time;
  // return time;
  // }
  // } catch (Exception ex) {
  // ex.printStackTrace();
  // }
  // return 0L;
  // }

  public String getName() {
    return "STREAM_BLOCKING_NIO";
  }

  public String getDetailedInfo() {
    return "MULTI_STREAM_BLOCKING_NIO NUM_CHANNELS="
      + SystemProperties.getProperty("NUM_CHANNELS");
  }
}
