package csfs.datatransfer.streamblockingnio;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;

import java.net.Socket;

import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.SocketChannel;

import java.io.FileNotFoundException;

import csfs.impl.util.*;

public class StreamBlockingNIOReceiverThread extends Thread {

  /** O socket que foi criado pelo servidor de cpia */
  private Socket socket;

  /**
   * A raiz onde os dados devem ser escritos, caminhos relativos so obtidos
   * pelo socket.
   */
  private String rootDir;

  public static final int HEADER_SIZE = 7;

  /**
   * Contrutor
   * 
   * @param socket Socket O socket que foi criado pelo servidor de cpia
   * @param rootDir String A raiz onde os dados devem ser escritos, caminhos
   *        relativos so obtidos pelo socket.
   */
  public StreamBlockingNIOReceiverThread(Socket socket, String rootDir) {
    this.socket = socket;
    this.rootDir = rootDir;
  }

  public void run() {
    SocketChannel socketChannel = null;
    ByteBuffer response = ByteBuffer.allocate(1);
    try {
      /** Obtem o canal do socket */
      socketChannel = socket.getChannel();
      try {
        /** Aloca o buffer de tamanho fixo para leitura do tamanho do header */
        ByteBuffer headerSizeBuf = ByteBuffer.allocate(HEADER_SIZE);

        /** le o tamanho do header completamente */
        int headerSizeAlreadyRead = 0;
        while (headerSizeAlreadyRead < HEADER_SIZE) {
          headerSizeAlreadyRead = headerSizeAlreadyRead
            + socketChannel.read(headerSizeBuf);
        }
        headerSizeBuf.rewind();

        /** Aloca o buffer de tamanho fixo para leitura do header */
        int headerSize = (int) Util.readLongFromBuffer(headerSizeBuf);

        /** le o header completamente */
        ByteBuffer headerBuf = ByteBuffer.allocate(headerSize);
        int headerAlreadyRead = 0;
        while (headerAlreadyRead < headerSize) {
          headerAlreadyRead = headerAlreadyRead + socketChannel.read(headerBuf);
        }
        headerBuf.rewind();

        long offset = Util.readLongFromBuffer(headerBuf);
        long length = Util.readLongFromBuffer(headerBuf);
        long diskOp = Util.readLongFromBuffer(headerBuf);
        String path = Util.readStringFromBuffer(headerBuf);

        if (diskOp == 1) {
          writeDataToDisk(socketChannel, offset, path, length);
        }
        else {
          readDataToNull(socketChannel, length);
        }

        response.put(0, (byte) 0);
      }
      catch (IOException e) {
        response.put(0, (byte) 1);
      }
      finally {
        if (socketChannel != null) {
          socketChannel.write(response);
          socketChannel.close();
        }
      }
    }
    catch (IOException ex) {
      ex.printStackTrace();
    }
  }

  private void writeDataToDisk(SocketChannel socketChannel, long offset,
    String path, long length) throws FileNotFoundException, IOException {

    /** Elimina caracteres iniciais desnecessarios */
    while (path.charAt(0) == '/') {
      path = path.substring(1);
    }

    /** Converte o separador de arquivos para o sistema local */
    path = Util.systemFileSeparatorToLocalFileSeparator(path);

    /** Criar o descritor de arquivo para escrever os dados */
    RandomAccessFile file = new RandomAccessFile(new File(rootDir, path)
      .getAbsolutePath(), "rw");
    // FileOutputStream file = new FileOutputStream(new File(rootDir,
    // path),true);

    /** Recupera o canal de escrita NIO */
    FileChannel fileChannel = file.getChannel();
    /** Posiciona o cursor do arquivo para a posicao adequada */
    fileChannel.position(offset);

    long transferred = 0;
    /** Verifica se ainda existem dados a serem lidos do scoket */
    while (length > transferred) {
      /** Copia os dados chamando uma funo de alto desempenho */
      transferred = transferred
        + fileChannel.transferFrom(socketChannel, offset + transferred, length
          - transferred);
    }

    /** Fecha o canal */
    fileChannel.close();
  }

  private void readDataToNull(SocketChannel socketChannel, long length)
    throws IOException {
    int bufSize = (int) Math.min(1024 * 1024, length);
    ByteBuffer nullReader = ByteBuffer.allocate(bufSize);
    long transfered = 0;
    while (transfered < length) {
      if ((length - transfered) < bufSize) {
        bufSize = (int) (length - transfered);
        nullReader = ByteBuffer.allocate(bufSize);
      }
      transfered = transfered + socketChannel.read(nullReader);
      nullReader.rewind();
    }
  }
}
