package csfs.impl;

import java.io.File;
import java.lang.management.ManagementFactory;
import java.lang.management.MemoryMXBean;
import java.lang.management.MemoryUsage;
import java.net.InetAddress;
import java.net.UnknownHostException;

import org.apache.commons.io.FileSystemUtils;
import org.omg.CORBA.UserException;

import csfs.datatransfer.CopyMethodsInitializer;
import csfs.datatransfer.DataTransferMethod;
import csfs.datatransfer.KnownTransferMethodMap;
import csfs.impl.performancetracker.DataTransferPerformanceTracker;
import csfs.impl.performancetracker.UnavailableInformationException;
import csfs.impl.util.ServerExceptionBuilder;
import csfs.impl.util.SystemProperties;
import csfs.remote.FileServerPOA;
import csfs.remote.RemoteFile;
import csfs.remote.RemoteFileHelper;
import csfs.remote.ServerException;

/**
 * A classe <class>FileServerImpl</class>  a implementao da interface
 * FileServer (CORBA). Essa classe consitui o Servant que representa o servidor
 * de arquivos disponibilizado pelo sistema.
 * 
 * Sua principal funo  a disponibilizao de uma referncia para a raiz
 * exportada pelo servidor atravs de um endereo corbaloc legvel. A referncia
 * para a raiz tambm pode ser obtida ao utilizar o endereo adequado, porm, o
 * uso do mtodo disponibilizado nessa classe aumenta as chances de
 * compatibilidade caso o esquema de identificao de objetos remotos seja
 * alterado.
 * 
 * @version 1.0 27 Jun 2005
 * @author Marcelo Nery dos Santos
 */
public class FileServerImpl extends FileServerPOA {

  MemoryMXBean memoryBean;

  /** O nome figurativo desse servidor, usado apenas para mensagens de log */
  private String name;

  /** O diretorio raiz desse sistema de arquivos */
  private String homeDirectory;

  private String localHost;

  private KnownTransferMethodMap map;

  /**
   * Construtor da classe
   */
  public FileServerImpl(String homeDirectory) {

    this.homeDirectory = homeDirectory;

    this.memoryBean = ManagementFactory.getMemoryMXBean();

    /** Inicializao do nome do servidor */
    name = "UNKNOWN";
    try {
      /** O nome do localhost  recuperado */
      name = InetAddress.getLocalHost().getHostName();
      localHost = name;
    }
    catch (UnknownHostException ex) {
      ex.printStackTrace();
    }
    /**
     * Um nmero  gerado para fazer com que servidores iniciados dentro de um
     * intervalo de tempo de 1000 segundos possuam nomes diferentes
     */
    name = name + " " + ((System.currentTimeMillis() / 1000) % 1000);

    map = new KnownTransferMethodMap();
    CopyMethodsInitializer.initialize(map, localHost);

  }

  /**
   * Retorna o nome do servidor
   * 
   * @return String o nome do servidor, definido pelo nome do host e um inteiro
   *         de 3 dgitos
   */
  public String getName() {
    return name;
  }

  /**
   * Recupera a raiz exportada pelo servidor
   * 
   * @return RemoteFile a referncia para a raiz
   */
  public RemoteFile getRoot() {
    org.omg.CORBA.Object o = null;
    try {
      /**
       * De acordo com a hierarquia de POAs definida na classe de configurao
       * (Server), aps o rootPOA ser retornado pela chamada a the_parent(), o
       * POA RemoteFile  obtido. Ento, uma referncia para a raiz  criada
       * atravs do identificador adequado.
       */
      o = _poa().the_parent().find_POA("RemoteFileDefaultServantPOA", false)
        .create_reference_with_id("/".getBytes(), RemoteFileHelper.type().id());
    }
    catch (UserException ex) {
      /** Caso haja algum problema de configurao */
      ex.printStackTrace();
      throw new RuntimeException("A misconfiguration problem occured!");
    }
    return RemoteFileHelper.narrow(o);
  }

  /**
   * Recupera o servidor NIO que disponibiliza conexes para transferncias
   * usando o mtodo NIO. O sistema assume que o servidor foi corretamente
   * inicializado na porta especificada pelo arquivo de propriedades.
   * 
   * @return String
   */
  public String getCopyServerAddress(String method) {
    return map.getAddress(method);
  }

  /**
   * Mtodo de debug. Caso a propriedade read-only "DEBUG_MODE" do arquivo de
   * configuracao esteja setada para TRUE, o servidor pode ser finalizado pela
   * chamada desse mtodo.
   */
  public void shutdown() {
    if ("TRUE".equals(SystemProperties.getProperty("DEBUG_MODE"))) {
      new Thread(new Runnable() {
        public void run() {
          try {
            Thread.sleep(2000);
          }
          catch (Exception exc) {
            exc.printStackTrace();
          }
          System.exit(0);
        }
      }).start();
    }
  }

  public long getFreeSpace() throws ServerException {
    try {
      return FileSystemUtils.freeSpace(homeDirectory);
    }
    catch (Exception e) {
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  public void setProperty(String key, String value) throws ServerException {
    SystemProperties.setProperty(key, value);
  }

  public String getProperty(String key) throws ServerException {
    return SystemProperties.getProperty(key);
  }

  public long getUsedMemory() throws ServerException {
    memoryBean.gc();
    return memoryBean.getHeapMemoryUsage().getUsed();
  }

  public String getMemoryStatus(boolean callGC) throws ServerException {

    if (callGC) {
      memoryBean.gc();
    }

    MemoryUsage heap = memoryBean.getHeapMemoryUsage();
    MemoryUsage nonHeap = memoryBean.getNonHeapMemoryUsage();

    StringBuffer buf = new StringBuffer();

    buf.append("\tHEAP INIT:");
    buf.append(heap.getInit());
    buf.append("\tHEAP COMMITTED:");
    buf.append(heap.getCommitted());
    buf.append("\tHEAP USED:");
    buf.append(heap.getUsed());
    buf.append("\tHEAP MAX:");
    buf.append(heap.getMax());

    buf.append("\tNON-HEAP INIT:");
    buf.append(nonHeap.getInit());
    buf.append("\tNON-HEAP COMMITTED:");
    buf.append(nonHeap.getCommitted());
    buf.append("\tNON-HEAP USED:");
    buf.append(nonHeap.getUsed());
    buf.append("\tNON-HEAP MAX:");
    buf.append(nonHeap.getMax());

    return buf.toString();

  }

  public double getAverageTransferRateToHost(String fileServerName)
    throws ServerException {
    try {
      return DataTransferPerformanceTracker.getInstance()
        .getAverageTransferRateToSpecificHost(fileServerName);
    }
    catch (UnavailableInformationException exc) {
      System.out.println("Returning 0 to average request");
      return 0;
    }
    catch (Exception exc) {
      throw ServerExceptionBuilder.createServerException(exc);
    }
  }

  public void resetTransferRateData() throws ServerException {
    try {
      DataTransferPerformanceTracker.getInstance().resetTransferRateData();
    }
    catch (Exception exc) {
      throw ServerExceptionBuilder.createServerException(exc);
    }
  }

  public DataTransferMethod getDataTransferMethod(String method) {
    return map.getMethod(method);
  }

  public String getFileSeparator() {
    return File.separator;
  }
}
