package csfs.impl;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.RandomAccessFile;
import java.nio.channels.FileChannel;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Vector;
import java.util.logging.Logger;

import org.omg.CORBA.TypeCode;
import org.omg.CORBA.UserException;
import org.omg.PortableServer.POA;

import csfs.datatransfer.DataTransferMethod;
import csfs.impl.filetracker.FileTracker;
import csfs.impl.filetracker.FileTrackerLeasing;
import csfs.impl.metadata.MetadataManager;
import csfs.impl.performancetracker.DataTransferPerformanceTracker;
import csfs.impl.util.Constants;
import csfs.impl.util.FileLocker;
import csfs.impl.util.IDGenerator;
import csfs.impl.util.NativeFileSystem;
import csfs.impl.util.ServerExceptionBuilder;
import csfs.impl.util.SystemProperties;
import csfs.impl.util.Util;
import csfs.remote.FileAlreadyExistsException;
import csfs.remote.FileInUseException;
import csfs.remote.FileNotFoundException;
import csfs.remote.FileServer;
import csfs.remote.FileServerHelper;
import csfs.remote.InvalidPathException;
import csfs.remote.InvalidStateException;
import csfs.remote.Metadata;
import csfs.remote.MetadataValue;
import csfs.remote.NotDirectoryException;
import csfs.remote.NotEmptyException;
import csfs.remote.NotFileException;
import csfs.remote.RandomAccessChannel;
import csfs.remote.RandomAccessChannelHelper;
import csfs.remote.ReadChannel;
import csfs.remote.ReadChannelHelper;
import csfs.remote.RemoteFile;
import csfs.remote.RemoteFileHelper;
import csfs.remote.RemoteFilePOA;
import csfs.remote.ServerException;
import csfs.remote.WriteChannel;
import csfs.remote.WriteChannelHelper;

/**
 * A classe <class>RemoteFileDefaultServantImpl</class>  a implementao da
 * interface RemoteFile (CORBA). Essa classe consitui o Servant que ser
 * utilizado por todos os objetos remotos dessa interface e portanto nao pode
 * possuir nenhum atributo que nao seja compartilhado entre todos os objetos.
 * 
 * Os diferentes arquivos no sistema de arquivos real so localizados pelo
 * ObjectID do objeto remoto, que  acessvel atravs de uma chamada ao ORB.
 * Dessa forma,  possvel atender vrios clientes sem a necessidade de alocar
 * recursos para cada um deles. Alm disso, no existe a necessidade de se
 * possuir um mecanismo de coleta de lixo distribudo, dado que temos apenas um
 * objeto remoto no servidor (representando a interface RemoteFile), e cada
 * cliente gerencia suas referncias individualmente.
 * 
 * Na documentao interna da classe, os mtodos iniciais apresentam comentrios
 * mais detalhados sobre as operaes realizadas para atender as solicitaes,
 * para evitar repeties, no mtodos subsequentes apenas os novos conceitos
 * sero apresentados.
 * 
 * @version 1.0 22 Jun 2005
 * @author Marcelo Nery dos Santos
 */
public class RemoteFileDefaultServantImpl extends RemoteFilePOA {
  /**
   * Mensagem introduzida nos arquivos que representam os Mount Points.
   */
  private static final String CSFS_MOUNT_HEADER =
    "CSFS Mount - HEADER! DO NOT EDIT THIS FILE!";

  private static final int MAX_CHANNELS = 1000;
  /**
   * O caminho raiz para os arquivos.
   */
  private String homeDirectory;

  /**
   * O servidor de arquivos que oferece as funcionalidades gerais.
   */
  private FileServerImpl fileServer;

  /**
   * O gerenciador de metadados
   */
  private MetadataManager metadataManager;

  /**
   * Classe para controlar o uso dos arquivos atravs dos canais de leitura,
   * escrita e acesso randmico.
   */
  private FileTracker fileTracker;

  /**
   * Classe para controlar o bloqueio e desbloqueio dos arquivos feitos
   * remotamente atravs dos mtodos {@link #tryLock()} e {@link #unlock()}.
   */
  private FileLocker fileLocker;

  /**
   * 
   * Cria o Servant.
   * 
   * @param homeDirectory String A raiz exportada pelo servidor.
   * @param fileServer FileServer O servidor associado aos arquivos.
   */
  public RemoteFileDefaultServantImpl(String homeDirectory,
    FileServerImpl fileServer) {
    this.homeDirectory = homeDirectory;
    this.fileServer = fileServer;
    this.fileTracker = FileTrackerLeasing.getInstance();
    this.metadataManager = new MetadataManager();
    this.fileLocker = new FileLocker();
  }

  /**
   * Recupera o nome do arquivo.
   * 
   * @return String O nome do arquivo.
   * @throws InvalidStateException caso haja algum problema com o arquivo local.
   */
  public String getName() throws InvalidStateException, ServerException {
    try {
      /** O caminho relativo, obtido a partir do ObjectID,  recuperado. */
      String systemPath = getPath();

      /** O arquivo real  buscado pelo seu caminho relativo */
      File localFile = getLocalFile(systemPath);

      /** Caso o arquivo esteja vlido, o nome  retornado */
      return localFile.getName();
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  /**
   * Indica se  um diretorio.
   * 
   * @return boolean
   * @throws InvalidStateException caso haja algum problema com o arquivo local.
   */
  public boolean isDirectory() throws InvalidStateException, ServerException {
    try {
      String systemPath = getPath();
      File localFile = getLocalFile(systemPath);

      return localFile.isDirectory();
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  /**
   * <p>
   * Tenta bloquear o arquivo.
   * </p>
   * Assim, chamadas subsequentes a este mtodo iro retornar <tt>false</tt>
   * enquanto o mtodo {@link #unlock}
   * <p>
   * O arquivo no  realmente bloqueado no servidor, apenas cria-se um marcador
   * indicando que ele est sendo utilizado. Assim, chamadas subsequentes a este
   * mtodo iro retornar <tt>false</tt> enquanto o mtodo {@link #unlock()}
   * no for chamado.
   * </p>
   * 
   * @return <tt>True</tt> se o arquivo foi bloqueado com sucesso, ou
   *         <tt>false</tt> caso o arquivo j estivesse bloqueado ou algum
   *         erro tenha ocorrido.
   * 
   * @throws InvalidStateException Caso haja algum problema com o arquivo local.
   * @throws ServerException Se o arquivo no estivesse bloqueado, porm no foi
   *         possvel criar o lock.
   * @throws NotFileException Se o arquivo a ser bloqueado for um diretrio.
   */
  public boolean tryLock() throws InvalidStateException, ServerException,
    NotFileException {
    try {
      /** O caminho relativo, obtido a partir do ObjectID, recuperado. */
      String systemPath = getPath();

      /** O arquivo real a ser buscado pelo seu caminho relativo */
      File localFile = getLocalFile(systemPath);
      if (!localFile.isFile()) {
        throw new NotFileException(getFullName());
      }
      return fileLocker.tryLock(localFile);
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  /**
   * <p>
   * Desbloqueia um arquivo bloqueado anteriormente por {@link #tryLock()}.
   * </p>
   * <p>
   * O arquivo no est realmente bloqueado no servidor, apenas criou-se um
   * marcador indicando que ele est sendo utilizado. Este mtodo ir remover o
   * marcador permitindo que a prxima chamada ao mtodo {@link #trylock()} crie
   * um novo marcador e retorne <tt>true</tt>
   * </p>
   * 
   * @return <tt>True</tt> se o arquivo foi desbloqueado com sucesso ou
   *         <tt>false</tt> caso o arquivo no estivesse bloqueado ou ocorreu
   *         algum erro.
   * 
   * @throws InvalidStateException Caso haja algum problema com o arquivo local.
   * @throws ServerException Se o arquivo no estivesse bloqueado, porm no foi
   *         possvel criar o lock.
   * @throws NotFileException Se o arquivo a ser desbloqueado for um diretrio.
   */
  public boolean unlock() throws NotFileException, InvalidStateException,
    ServerException {
    try {
      /** O caminho relativo, obtido a partir do ObjectID, recuperado. */
      String systemPath = getPath();

      /** O arquivo real a ser buscado pelo seu caminho relativo */
      File localFile = getLocalFile(systemPath);
      if (!localFile.isFile()) {
        throw new NotFileException(getFullName());
      }
      return fileLocker.unlock(localFile);
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  /**
   * Cria um diretorio no caminho especificado. Caso o caminho possua mais de um
   * elemento, todos os elementos necessarios sao criados.
   * 
   * @param name String[] O caminho do diretrio a ser criado.
   * @return RemoteFile Uma referncia remota ao arquivo criado.
   * @throws InvalidPathException caso o caminho seja invlido conforme
   *         especificao da IDL.
   * @throws InvalidStateException caso haja algum problema com o arquivo local.
   * @throws FileAlreadyExistsException caso j exista um arquivo no caminho
   *         especificado.
   * @throws NotDirectoryException caso o arquivo corrente no seja um
   *         diretrio.
   */
  public RemoteFile createDirectory(String[] name) throws InvalidPathException,
    InvalidStateException, FileAlreadyExistsException, NotDirectoryException,
    ServerException {
    try {
      String systemPath = getPath();

      /**
       * A funco auxiliar createElement cria uma nova representao abastrata
       * para o caminho especificado. Essa funo  compartilhada pelo mtodo de
       * criao de arquivos de dados e possui toda a validao necessria para
       * criar um novo arquivo.
       */
      File localChild = createElement(systemPath, name);

      /** Cria o diretrio no sistema de arquivo real. */
      boolean success = localChild.mkdirs();

      if (!success) {
        /**
         * Caso no haja sucesso aps as validaes feitas anteriormente,
         * assumimos um problema na especificao do novo caminho.
         */
        throw new InvalidPathException(Util.getPathArray(systemPath, name));
      }

      /** Cria uma referncia remota ao arquivo recm criado. */
      return createRemoteFile(name);
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  /**
   * Cria um arquivo de dados no caminho especificado. Caso o caminho possua
   * mais de um elemento, todos os elementos necessarios sao criados.
   * 
   * @param name String[] O caminho do arquivo de dados a ser criado.
   * @return RemoteFile Uma referncia remota ao arquivo criado.
   * @throws InvalidPathException caso o caminho seja invlido conforme
   *         especificao da IDL.
   * @throws InvalidStateException caso haja algum problema com o arquivo local.
   * @throws FileAlreadyExistsException caso j exista um arquivo no caminho
   *         especificado.
   * @throws NotDirectoryException caso o arquivo corrente no seja um
   *         diretrio.
   */
  public RemoteFile createFile(String[] name) throws InvalidPathException,
    InvalidStateException, FileAlreadyExistsException, NotDirectoryException,
    ServerException {
    try {
      String systemPath = getPath();
      File newFile = createElement(systemPath, name);
      boolean created = false;

      try {
        /** Cria o diretrio pai ao arquivo de dados. */
        newFile.getParentFile().mkdirs();

        /** Cria o arquivo de dados */
        created = newFile.createNewFile();
      }
      catch (IOException exc) {
        exc.printStackTrace();

        /**
         * Caso no haja sucesso aps as validaes feitas anteriormente,
         * assumimos um problema na especificao do novo caminho.
         */
        throw new InvalidPathException(Util.getPathArray(systemPath, name));
      }

      if (!created) {
        throw new InvalidPathException(Util.getPathArray(systemPath, name));
      }

      return createRemoteFile(name);
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  /**
   * Cria um Mount Point para um outro arquivo, potencialmente em outro sistema
   * de arquivos. Caso o caminho possua mais de um elemento, todos os elementos
   * necessarios sao criados e o ultimo representa o mount propriamente dito.
   * 
   * @param name String[] O caminho do arquivo de dados a ser criado.
   * @param target RemoteFile O arquivo que representa o destino do mount.
   * @return boolean uma indicao de sucesso.
   * @throws InvalidPathException caso o caminho seja invlido conforme
   *         especificao da IDL.
   * @throws InvalidStateException caso haja algum problema com o arquivo local.
   * @throws FileAlreadyExistsException caso j exista um arquivo no caminho
   *         especificado.
   * @throws NotDirectoryException caso o arquivo corrente no seja um
   *         diretrio.
   */
  public boolean addMountPoint(String[] name, RemoteFile target)
    throws InvalidStateException, NotDirectoryException,
    FileAlreadyExistsException, InvalidPathException, ServerException {
    try {
      String systemPath = getPath();
      File newFile = createElement(systemPath, name);
      boolean created = false;

      try {
        newFile.getParentFile().mkdirs();
        created = newFile.createNewFile();

        /**
         * Aps a criao do novo arquivo, as informaes sobre o destino devem
         * ser escritas.
         */
        BufferedWriter writer = new BufferedWriter(new FileWriter(newFile));

        /** O cabealho  escrito. */
        writer.write(CSFS_MOUNT_HEADER + "\n");

        /** O IOR do destino  escrito */
        writer.write(_orb().object_to_string(target) + "\n");

        /** O arquivo deve ser fechado */
        writer.close();
      }
      catch (IOException exc) {
        /**
         * Caso haja algum problema, o arquivo  removido. Porm, os diretrios
         * pais, caso tenham sido criados, no sero removidos.  controverso se
         * eles deveriam ser removidos, ou no.
         */
        created = false;
        newFile.delete();
        exc.printStackTrace();
        throw new InvalidStateException(
          "error persisting object reference to disk", Util.getPathArray(
            systemPath, name));
      }

      if (!created) {
        throw new InvalidPathException(Util.getPathArray(systemPath, name));
      }

      return created;
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  /**
   * Remove um mount point. Como a operao de "remove" requer uma referencia ao
   * arquivo a ser removido e a tentativa de recuperar o mount retorna o arquivo
   * alvo do mount, nao eh possivel remover um mount point atraves do "remove".
   * Essa funcao remove o mount sem alterar o arquivo alvo.
   * 
   * @param name String[]
   * @return RemoteFile
   * @throws NotDirectoryException caso o arquivo corrente no seja um
   *         direttio.
   * @throws FileNotFoundException caso o mount no seja encontrado.
   * @throws InvalidPathException caso o caminho seja invlido conforme
   *         especificao da IDL.
   * @throws InvalidStateException caso haja algum problema com o arquivo local.
   */
  public RemoteFile removeMountPoint(String[] name)
    throws NotDirectoryException, FileNotFoundException, InvalidPathException,
    InvalidStateException, ServerException {
    try {
      RemoteFile response = null;
      String systemPath = getPath();
      File localFile = getLocalFile(systemPath);

      /** Se o caminho for extenso */
      if (name.length > 1) {
        /** Cria o caminho do pai */
        String[] parent = new String[name.length - 1];

        for (int i = 0; i < parent.length; i++) {
          parent[i] = name[i];
        }

        /** Recupera uma referncia para o pai */
        RemoteFile mountDir = getChild(parent);

        /** Solicita a remoo do mount no servidor que o armazena. */
        response =
          mountDir.removeMountPoint(new String[] { name[name.length - 1] });
      }
      /** Se o caminho  simples, o mount  local */
      else {
        /** Verifica o caminho */
        if (name.length == 0) {
          throw new InvalidPathException(name);
        }

        /** Procura pelo arquivo de mount */
        File probe = new File(localFile, name[0]);

        /** Se o arquivo existe */
        if (probe.exists()) {
          /** tenta resolver o mount */
          response = resolveMountPoint(probe);

          /** caso o arquivo represente realmente um mount */
          if (response != null) {
            /** tenta remover o arquivo */
            if (!probe.delete()) {
              response = null;
            }
          }
        }
      }

      if (response == null) {
        /** caso nao consiga localizar o mount */
        throw new FileNotFoundException(Util.getPathArray(systemPath, name));
      }

      return response;
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  /**
   * Recupera toda a sequencia de elementos necessria para recupertar o
   * arquivo. Esse caminho eh absoluto no contexto do servidor de arquivos e nao
   * expoe a localizacao do arquivo no sistema de arquivos local.
   * 
   * @return String[] O caminho do arquivo.
   * @throws InvalidStateException caso haja algum problema com o arquivo local.
   */
  public String[] getFullName() throws InvalidStateException, ServerException {
    try {
      String systemPath = getPath();

      /**
       * Apesar da chamada a getLocalFile no ser estritamente necessria, o seu
       * uso  recomendado para manter a consistncia das operaes e no
       * retornar o caminho do arquivo com sucesso para um arquivo que foi
       * eventualmente removido.
       */
      getLocalFile(systemPath);

      return Util.stringToStringArray(systemPath);
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  /**
   * Copia o conteudo do arquivo de dados para um outro arquivo de dados. O
   * metodo utilizado na copia eh definido atravs do servidor. Dois metodos
   * estao disponiveis: 1. Metodo CORBA, onde os canais so utilizados para a
   * escrita dos dados. 2. Metodo NIO, onde a transferencia eh realizada atraves
   * de sockets e NIO Channels.
   * 
   * @param destination RemoteFile o arquivo de dados de destino.
   * @return boolean indica o sucesso da copia
   * @throws InvalidStateException caso haja algum problema com o arquivo local.
   * @throws NotFileException caso fonte ou destino no sejam arquivos de dados.
   */
  public boolean copyTo(RemoteFile destination, String method)
    throws InvalidStateException, NotFileException, ServerException {
    try {
      String systemPath = getPath();
      File localFile = getLocalFile(systemPath);

      if (!localFile.isFile()) {
        throw new NotFileException(getFullName());
      }

      int index = method.indexOf('-');
      if (index == -1) {
        index = method.length();
      }
      String methodCore = method.substring(0, index);
      DataTransferMethod bulkTransfer =
        fileServer.getDataTransferMethod(methodCore);
      long startTime = System.currentTimeMillis();
      boolean success = bulkTransfer.copy(localFile, destination, method);

      if (success) {
        long elapsedTime = System.currentTimeMillis() - startTime;

        if (elapsedTime > 0) {
          String destinationHost = destination.getFileServer().getName();
          // int index = destinationHost.indexOf(' ');

          // if (index != -1) {
          // destinationHost = destinationHost.substring(0, index);
          // }

          DataTransferPerformanceTracker.getInstance().update(destinationHost,
            localFile.length(), elapsedTime);
        }
      }

      return success;
    }
    catch (IOException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  public boolean moveTo(RemoteFile destination, String method)
    throws NotFileException, FileInUseException, ServerException,
    InvalidStateException {

    try {
      String systemPath = getPath();
      File localFile = getLocalFile(systemPath);

      if (!localFile.isFile()) {
        throw new NotFileException(getFullName());
      }

      if (fileTracker.inUse(systemPath)) {
        throw new FileInUseException(getFullName());
      }

      boolean success = false;
      if ((_this().getFileServer().getName()).equals(destination
        .getFileServer().getName())) {
        String destinationSystemPath =
          Util.stringArrayToString(destination.getFullName());
        File targetFile = getLocalFile(destinationSystemPath);
        if (targetFile.delete()) {
          success = localFile.renameTo(targetFile);
        }
      }
      else {
        if (_this().copyTo(destination, method)) {
          try {
            success = _this().remove();
          }
          catch (NotEmptyException e) {
            Logger.global.severe("ACESSO CONCORRENTE! O arquivo "
              + _this().getName() + "foi substituido por um diretorio");
          }
        }
      }
      return success;
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  public boolean truncate(long size) throws NotFileException,
    FileInUseException, ServerException, InvalidStateException {
    try {
      String systemPath = getPath();
      File localFile = getLocalFile(systemPath);

      if (!localFile.isFile()) {
        throw new NotFileException(getFullName());
      }

      // colocar verificacao de FileInUse?
      // if (fileTracker.inUse(systemPath)) {
      // throw new FileInUseException(getFullName());
      // }

      FileChannel nioChannel =
        new RandomAccessFile(localFile, "rw").getChannel();
      nioChannel.truncate(size);
      return true; // avaliar um retorno mais "significativo"

    }
    catch (IOException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  public long lastModified() throws ServerException, InvalidStateException {
    try {
      String systemPath = getPath();
      File localFile = getLocalFile(systemPath);
      return localFile.lastModified();
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  /**
   * Remove um arquivo. Caso o arquivo de dados esteja em uso, ou o diretorio
   * nao esteja vazio, uma excecao eh lancada. A validacao do uso corrente do
   * arquivo eh feita no contexto do sistema (sem condiderar o uso por
   * aplicacoes externas). Esse metodo nao remove uma arvore de diretorios. A
   * semantica da remocao recursiva eh dependente do uso dado ao sistema e
   * funcao deve ser implementada externamente.
   * 
   * @return boolean indica o sucesso da remoo.
   * @throws FileInUseException caso o arquivo de dados esteja em uso.
   * @throws NotEmptyException caso o diretrio nao esteja vazio.
   * @throws InvalidStateException caso haja algum problema com o arquivo local.
   */
  public boolean remove() throws FileInUseException, NotEmptyException,
    InvalidStateException, ServerException {
    try {
      boolean response = false;

      String systemPath = getPath();
      File localFile = getLocalFile(systemPath);

      /** verifica se o arquivo est em uso */
      if (fileTracker.inUse(systemPath)) {
        throw new FileInUseException(getFullName());
      }

      /** no caso de arquivo de dados, simplesmente remove. */
      if (localFile.isFile()) {
        response = localFile.delete();
      }

      /** no caso de diretrio */
      if (localFile.isDirectory()) {
        /** recupera a lista de arquivos */
        File[] child = localFile.listFiles();

        /** caso seja vazia, */
        if (child.length == 0) {
          /** verifica se no  a raiz */
          if (systemPath.length() > 1) {
            /** remove o diretrio */
            response = localFile.delete();
          }
          else {
            throw new FileInUseException(getFullName());
          }
        }
        else {
          /** ou lana uma exceo */
          throw new NotEmptyException(getFullName());
        }
      }

      /** retorna o sucesso da operao */
      return response;
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  /**
   * Recupera a lista de arquivos de um diretorio.
   * 
   * @return RemoteFile[] a lista de arquivos
   * @throws NotDirectoryException caso o arquivo corrente no seja um
   *         direttio.
   * @throws InvalidStateException caso haja algum problema com o arquivo local.
   */
  public RemoteFile[] getChildren() throws InvalidStateException,
    NotDirectoryException, ServerException {
    try {
      String systemPath = getPath();
      File localFile = getLocalFile(systemPath);

      if (!localFile.isDirectory()) {
        throw new NotDirectoryException(getFullName());
      }

      /** Recupera os arquivos do diretrio atual e cria a lista de retorno. */
      File[] dirContents = localFile.listFiles();

      RemoteFile mountProbe = null;
      Vector<RemoteFile> response = new Vector<RemoteFile>();

      for (int i = 0; i < dirContents.length; i++) {
        File child = dirContents[i];

        // O cliente no deve ter acesso direto aos metadados serializados.
        if (metadataManager.isMetadataFile(child.getName())) {
          continue;
        }
        // O cliente no deve ter acesso aos locks.
        if (fileLocker.isLockFile(child.getName())) {
          continue;
        }

        if (child.isFile()) {
          mountProbe = resolveMountPoint(child);
        }

        if (mountProbe == null) {
          response.add(createRemoteFile(systemPath + Constants.PATH_SEPARATOR
            + child.getName()));
        }
        else {
          response.add(mountProbe);
        }
      }

      return (RemoteFile[]) response.toArray(new RemoteFile[response.size()]);
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  /**
   * Recupera um filho especifico de um diretorio. O caminho pode conter
   * diretorios e definicoes de Mount Points.
   * 
   * @param name String[]
   * @return RemoteFile
   * @throws InvalidPathException caso o caminho seja invlido conforme
   *         especificao da IDL.
   * @throws InvalidStateException caso haja algum problema com o arquivo local.
   * @throws FileNotFoundException caso o arquivo no seja localizado
   * @throws NotDirectoryException caso o arquivo corrente no seja um
   *         diretrio.
   */
  public RemoteFile getChild(String[] name) throws InvalidPathException,
    InvalidStateException, NotDirectoryException, FileNotFoundException,
    ServerException {
    try {
      RemoteFile response;

      /** Se for um caminho simples */
      if (name.length == 1) {
        /** resolve a chamada localmente */
        response = localGetChild(name[0]);
      }
      else {
        /** inicializa a resposta com o prprio arquivo */
        response = _this();

        for (int i = 0; i < name.length; i++) {
          /**
           * para cada um dos elementos do caminho, uma chamada com um caminho
           * simples  criada, o resultado final  o arquivo desejado.
           */
          response = response.getChild(new String[] { name[i] });
        }
      }

      return response;
    }
    catch (InvalidPathException e) {
      throw e;
    }
    catch (InvalidStateException e) {
      throw e;
    }
    catch (NotDirectoryException e) {
      throw e;
    }
    catch (FileNotFoundException e) {
      throw e;
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  /**
   * Obtem um filho localmente nesse servidor.
   */
  private RemoteFile localGetChild(String name) throws NotDirectoryException,
    FileNotFoundException, InvalidPathException, InvalidStateException,
    ServerException {
    String systemPath = getPath();
    File localFile = getLocalFile(systemPath);

    if (!localFile.isDirectory()) {
      throw new NotDirectoryException(getFullName());
    }

    File existingFile = new File(localFile, name);

    if (!existingFile.exists()) {
      throw new FileNotFoundException(Util.getPathArray(systemPath, name));
    }
    // O cliente no deve ter acesso direto aos metadados serializados.
    if (metadataManager.isMetadataFile(existingFile.getName())) {
      throw new InvalidPathException(Util.getPathArray(systemPath, name));
    }
    // O cliente no deve ter acesso aos locks.
    if (fileLocker.isLockFile(existingFile.getName())) {
      throw new InvalidPathException(Util.getPathArray(systemPath, name));
    }

    if (!Util.isValid(existingFile)) {
      throw new InvalidPathException(Util.getPathArray(systemPath, name));
    }

    /** resolve eventuais mount points encontrados */
    RemoteFile response = resolveMountPoint(existingFile);

    if (response == null) {
      /** Se o mount nao  encontrado */
      if (name.equals(".")) {
        /** verifica se o caminho desejado  o prprio arquivo */
        response = _this();
      }
      else if (name.equals("..")) {
        /** ou se o diretorio pai */
        response =
          createRemoteFile(systemPath.substring(0, systemPath
            .lastIndexOf(Constants.PATH_SEPARATOR)));
      }
      else {
        /**
         * por fim, cria uma nova referencia adicionano o nome do elemento ao
         * caminho atual
         */
        response =
          createRemoteFile(systemPath + Constants.PATH_SEPARATOR + name);
      }
    }

    return response;
  }

  /**
   * Verifica a existencia de um mount point em um arquivo.
   */
  private RemoteFile resolveMountPoint(File file) throws InvalidStateException,
    ServerException {
    try {
      RemoteFile response = null;

      try {
        /**
         * Se o arquivo  um arquivo de dados e se o tamanho  menor que 1K
         * (essa metodo deve ser melhorado para tratar mount points que apontam
         * para caminhos extensos, onde o IOR possui mais que 950 bytes (os
         * demais bytes sao usados no verificador de mount)
         */
        if (file.isFile() && (file.length() < 1024)) {
          /** Abre o arquivo para leitura */
          BufferedReader reader = new BufferedReader(new FileReader(file));

          /** Recupera a primeira linha */
          String line = reader.readLine();

          /** Se a linha  igual ao verificador de mount */
          if (CSFS_MOUNT_HEADER.equals(line)) {
            /** Trata a segunda linha como um IOR */
            String ior = reader.readLine();

            /** Transforma o IOR em um objeto */
            response = RemoteFileHelper.narrow(_orb().string_to_object(ior));
          }

          /** Fecha o arquivo */
          reader.close();
        }
      }
      catch (IOException exc) {
        exc.printStackTrace();
        throw new InvalidStateException("Unable to resolve mount: "
          + exc.getMessage(), getFullName());
      }

      return response;
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  /**
   * Obtem o diretrio pai.
   * 
   * @return RemoteFile uma referencia para o diretrio pai
   * @throws InvalidPathException caso o arquivo atual seja a raiz da rvore.
   * @throws InvalidStateException caso haja algum problema com o arquivo local.
   */
  public RemoteFile getParent() throws InvalidPathException,
    InvalidStateException, ServerException {
    try {
      String systemPath = getPath();

      /** Caso o caminho seja longo */
      if (systemPath.length() > 1) {
        /** verifica a validade do arquivo local */
        getLocalFile(systemPath);
      }
      else {
        /** envia erro indicando que o pai da raiz nao pode ser retornado */
        throw new InvalidPathException(new String[] { ".." });
      }

      return createRemoteFile(systemPath.substring(0, systemPath
        .lastIndexOf(Constants.PATH_SEPARATOR)));
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  /**
   * Recupera o tamanho do arquivo.
   * 
   * @return long o tamanho do arquivo.
   * @throws NotFileException caso a referencia aponte para um diretorio
   * @throws InvalidStateException caso haja algum problema com o arquivo local.
   */
  public long size() throws NotFileException, InvalidStateException,
    ServerException {
    try {
      String systemPath = getPath();
      File localFile = getLocalFile(systemPath);

      if (localFile.isDirectory()) {
        throw new NotFileException(getFullName());
      }

      /** Retorna o tamanho do arquivo */
      return localFile.length();
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  /**
   * Obtem o canal de escrita no arquivo
   * 
   * @return WriteChannel o canal de escrita
   * @throws NotFileException caso a referencia aponte para um diretorio
   * @throws InvalidStateException caso haja algum problema com o arquivo local.
   */
  public WriteChannel getWriteChannel() throws InvalidStateException,
    NotFileException, ServerException {

    if (fileTracker.totalOpenFileDescriptors() >= MAX_CHANNELS) {
      throw new InvalidStateException("MAX OPEN CHANNEL REACHED", getFullName());
    }

    WriteChannel response = null;
    try {

      try {
        String systemPath = getPath();
        File localFile = getLocalFile(systemPath);

        if (!localFile.isFile()) {
          throw new NotFileException(getFullName());
        }

        /**
         * Cria o descritor de arquivo usado para escrever no arquivo local
         */
        OutputStream output = new FileOutputStream(localFile);

        /** Cria uma identificao para o canal */
        String channelId = systemPath + "_" + IDGenerator.nextId();

        /** Associa a identificao do canal com seu descritor de arquivo */
        fileTracker.setFileDescriptor(channelId, output);

        /**
         * solicita a criao de uma referncia para o canal de escrita
         * (observar o mrodo getChannel)
         */
        response =
          WriteChannelHelper.narrow(getChannel("WriteChannelDefaultServantPOA",
            channelId, WriteChannelHelper.type()));
      }
      catch (java.io.FileNotFoundException ex) {
        throw new InvalidStateException("java.io.FileNotFoundException:"
          + ex.getMessage(), getFullName());
      }
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
    catch (Throwable t) {
      System.out.println("t.getMessage()=" + t.getMessage());
      Logger.global.severe(t.getMessage());
      t.printStackTrace();
    }
    return response;
  }

  /**
   * Obtem o canal de leitura do arquivo
   * 
   * @return ReadChannel o canal de leitura
   * 
   * @throws NotFileException caso a referencia aponte para um diretorio
   * @throws InvalidStateException caso haja algum problema com o arquivo local.
   */
  public ReadChannel getReadChannel() throws InvalidStateException,
    NotFileException, ServerException {

    if (fileTracker.totalOpenFileDescriptors() >= MAX_CHANNELS) {
      throw new InvalidStateException("MAX OPEN CHANNEL REACHED", getFullName());
    }

    try {

      ReadChannel response = null;

      try {
        String systemPath = getPath();
        File localFile = getLocalFile(systemPath);

        if (!localFile.isFile()) {
          throw new NotFileException(getFullName());
        }

        /**
         * Cria o descritor de arquivo usado para ler informaes do arquivo
         * local
         */
        InputStream input = new FileInputStream(localFile);

        /** Cria uma identificao para o canal */
        String channelId = systemPath + "_" + IDGenerator.nextId();

        fileTracker.setFileDescriptor(channelId, input);

        /**
         * solicita a criao de uma referncia para o canal de leitura
         * (observar o mrodo getChannel)
         */
        response =
          ReadChannelHelper.narrow(getChannel("ReadChannelDefaultServantPOA",
            channelId, ReadChannelHelper.type()));
      }
      catch (java.io.FileNotFoundException ex) {
        throw new InvalidStateException("java.io.FileNotFoundException:"
          + ex.getMessage(), getFullName());
      }

      return response;
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  /**
   * Obtem o canal de acesso aleatrio ao arquivo
   * 
   * @return RandomAccessChannel o canal de acesso aleatrio
   * 
   * @throws NotFileException caso a referencia aponte para um diretorio
   * @throws InvalidStateException caso haja algum problema com o arquivo local.
   */
  public RandomAccessChannel getRandomAccessChannel()
    throws InvalidStateException, NotFileException, ServerException {

    if (fileTracker.totalOpenFileDescriptors() >= MAX_CHANNELS) {
      throw new InvalidStateException("MAX OPEN CHANNEL REACHED", getFullName());
    }

    try {
      RandomAccessChannel response = null;

      try {
        String systemPath = getPath();
        File localFile = getLocalFile(systemPath);

        if (!localFile.isFile()) {
          throw new NotFileException(getFullName());
        }

        /**
         * Cria o descritor de arquivo usado para acessar informaes
         * aleatoriamente do arquivo local
         */
        RandomAccessFile raf = new RandomAccessFile(localFile, "rw");

        /** Cria uma identificao para o canal */
        String channelId = systemPath + "_" + IDGenerator.nextId();

        fileTracker.setFileDescriptor(channelId, raf);

        /**
         * solicita a criao de uma referncia para o canal de acesso aleatrio
         * (observar o mrodo getChannel)
         */
        response =
          RandomAccessChannelHelper.narrow(getChannel(
            "RandomAccessChannelDefaultServantPOA", channelId,
            RandomAccessChannelHelper.type()));
      }
      catch (java.io.FileNotFoundException ex) {
        throw new InvalidStateException("java.io.FileNotFoundException:"
          + ex.getMessage(), getFullName());
      }

      return response;
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  /**
   * Obtem o hash do arquivo, calculado remotamente pelo servidor.
   * 
   * @return byte[] o hash MD5 do arquivo de dados
   * @throws NotFileException caso a referencia aponte para um diretorio
   * @throws InvalidStateException caso haja algum problema com o arquivo local.
   */
  public byte[] hash() throws NotFileException, InvalidStateException,
    ServerException {
    try {
      byte[] digest = null;

      try {

        String systemPath = getPath();
        File localFile = getLocalFile(systemPath);

        if (!localFile.isFile()) {
          throw new NotFileException(getFullName());
        }

        /** Cria o digester */
        MessageDigest digester = MessageDigest.getInstance("MD5");

        /** cria o buffer de leitura do arquivo */
        byte[] buffer = new byte[4096];

        long read = 0;

        /** obtem o tamanho do arquivo */
        long size = localFile.length();

        /** abre o arquivo para leitura */
        BufferedInputStream input =
          new BufferedInputStream(new FileInputStream(localFile),
            SystemProperties.getIntProperty("BUFFERED_STREAM_SIZE_HASH"));

        /**
         * inicia a leitura e gerao do hash, o loop termina quando todos o
         * numero de bytes lidos for igual ao tamanho do arquivo, ou quando EOF
         *  encontrado
         */
        while (read < size) {
          /**
           * o metodo read pode nao retornar todo o tamanho do buffer em dados,
           * assim,  necessario realizar o nmero de chamadas adequado.
           */
          int localRead = 0;
          int bufferPosition = 0;
          while ((localRead =
            input.read(buffer, bufferPosition, buffer.length - bufferPosition)) > 0) {
            digester.update(buffer, bufferPosition, localRead);
            bufferPosition = bufferPosition + localRead;
            read += localRead;
          }
        }

        /** fecha o arquivo */
        input.close();

        /**
         * Se o nmero de bytes lidos for igual ao tamanho inicial do arquivo
         */
        if (read == size) {
          /** calcula o hash */
          digest = digester.digest();
        }
        else {
          /** lana uma indicao de erro */
          throw new InvalidStateException("Could NOT read expected " + read
            + " bytes", getFullName());
        }
      }
      catch (IOException ex) {
        /** caso haja algum problema interno na leitura */
        throw new InvalidStateException("IOException:" + ex.getMessage(),
          getFullName());
      }
      catch (NoSuchAlgorithmException ex) {
        /** caso nao exista um digester MD5 */
        throw new InvalidStateException("There is no MD5 digester! "
          + ex.getMessage(), getFullName());
      }
      return digest;
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  public void setMetadata(Metadata[] metadata) throws NotFileException,
    InvalidStateException, ServerException {
    try {
      String systemPath = getPath();
      File localFile = getLocalFile(systemPath);

      if (!localFile.isFile()) {
        throw new NotFileException(getFullName());
      }

      try {
        metadataManager.saveMetadataInfo(metadata, localFile);
      }
      catch (IOException ex) {
        throw new InvalidStateException("metadata problem" + ex.getMessage(),
          getFullName());
      }
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  public Metadata[] getMetadata(String[] fields) throws NotFileException,
    InvalidStateException, ServerException {
    try {
      String systemPath = getPath();
      File localFile = getLocalFile(systemPath);

      if (!localFile.isFile()) {
        throw new NotFileException(getFullName());
      }

      Metadata[] response = new Metadata[fields.length];

      for (int i = 0; i < response.length; i++) {
        response[i] = new Metadata(fields[i], new MetadataValue());
      }

      try {
        metadataManager.lookupMetadata(response, localFile);
      }
      catch (IOException ex) {
        throw new InvalidStateException("metadata problem" + ex.getMessage(),
          getFullName());
      }

      return response;
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  public Metadata[] getAllMetadata() throws NotFileException,
    InvalidStateException, ServerException {
    try {
      Metadata[] response = null;
      String systemPath = getPath();
      File localFile = getLocalFile(systemPath);

      if (!localFile.isFile()) {
        throw new NotFileException(getFullName());
      }

      try {
        response = metadataManager.getMetadata(localFile);
      }
      catch (IOException ex) {
        throw new InvalidStateException("metadata problem" + ex.getMessage(),
          getFullName());
      }

      return response;
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  public void updateMetadata(Metadata[] metadata) throws NotFileException,
    InvalidStateException, ServerException {
    try {
      String systemPath = getPath();
      File localFile = getLocalFile(systemPath);

      if (!localFile.isFile()) {
        throw new NotFileException(getFullName());
      }

      try {
        metadataManager.updateMetadata(metadata, localFile);
      }
      catch (IOException ex) {
        throw new InvalidStateException("metadata problem" + ex.getMessage(),
          getFullName());
      }
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  /**
   * Retorna o servidor de arquivos associado a todos os arquivos remotos.
   * Nenhum verificao da validade do arquivo  realizada, permitindo que o
   * cliente obtenha uma referncia vlida ao servidor, mesmo que esteja de
   * posse de uma referncia invlida a um arquivo.
   * 
   * @return FileServer a referncia ao servidor de arquivos
   */
  public FileServer getFileServer() throws ServerException {
    try {
      /**
       * Utiliza o POA corrente para criar uma referncia para um outro arquivo
       */
      org.omg.CORBA.Object o =
        _poa().the_parent().find_POA("FileServerPOA", false)
          .create_reference_with_id("FileServer".getBytes(),
            FileServerHelper.type().id());

      return FileServerHelper.narrow(o);

      // return fileServer;
    }
    catch (Exception e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  /**
   * Habilita a permissao de execucao para sistemas como o UNIX. Melhorar essa
   * implementacao com as novidades do Java 5
   * 
   * @throws InvalidStateException
   */
  public void enableExecutionPermission() throws InvalidStateException,
    ServerException {
    try {
      String systemPath = getPath();
      File localFile = getLocalFile(systemPath);

      if (!System.getProperty("os.arch").equals("Win32")) {
        String filename = localFile.getAbsolutePath();

        try {
          filename = localFile.getCanonicalPath();
          NativeFileSystem.enableExecutionPermission(filename);
        }
        catch (IOException ex) {
          Logger.global.severe("ERRO: atribuindo permissao de execucao para "
            + filename);
          ex.printStackTrace();
        }
      }
    }
    catch (RuntimeException e) {
      Logger.global.severe(e.getMessage());
      e.printStackTrace();
      throw ServerExceptionBuilder.createServerException(e);
    }
  }

  /**
   * Recupera o caminho do arquivo
   * 
   * @return String O caminho do arquivo
   * @throws InvalidStateException
   */
  private String getPath() throws InvalidStateException {
    /**
     * atualmente, o caminho do arquivo  exatamente o object_id, porm existe a
     * possibilidade de realizar verificaes sobre o caminho baseado no
     * timestamp do arquivo local e optamos por deixar esse mtodo isolado.
     */
    return new String(_object_id());
  }

  /**
   * Esse mtodo visa criar uma representao de um arquivo, que pode ser
   * convertido concretizado atravs de um diretrio ou arquivo de dados.
   */
  private File createElement(String systemPath, String[] name)
    throws FileAlreadyExistsException, NotDirectoryException,
    InvalidPathException, InvalidStateException, ServerException {
    File localFile = getLocalFile(systemPath);

    if (!localFile.isDirectory()) {
      throw new NotDirectoryException(getFullName());
    }

    /**
     * valida o caminho fornecido pelo usurio, outros testes podem ser
     * adicionados aqui. Atualmente, apenas os separadores de arquivo sao
     * testados a nvel do servidor
     */
    for (int i = 0; i < name.length; i++) {
      if ((name[i].indexOf('/') != -1) || (name[i].indexOf('\\') != -1)) {
        throw new InvalidPathException(name);
      }
    }
    /**
     * Verifica se  um arquivo de metadado ou lock.
     */
    if (metadataManager.isMetadataFile(name[name.length - 1])
      || fileLocker.isLockFile(name[name.length - 1])) {
      throw new InvalidPathException(name);
    }

    /** Converte o caminho do usurio em uma nica string */
    String systemNewElement = Util.stringArrayToString(name);

    /**
     * Cria um novo arquivo sobre o diretrio corrente, modificando o separador
     * de arquivos do sistema pelo separador de arquivos local
     */
    File newFile =
      new File(localFile, Util
        .systemFileSeparatorToLocalFileSeparator(systemNewElement));

    /**
     * Verifica se o novo arquivo  vlido e compromete o caminho do usurio em
     * caso de falha
     */
    if ((newFile == null) || !Util.isValid(newFile)) {
      throw new InvalidPathException(name);
    }

    /** Verifica a existncia de um arquivo no mesmo local */
    if (newFile.exists()) {
      throw new FileAlreadyExistsException(Util.getPathArray(systemPath, name));
    }

    return newFile;
  }

  /**
   * Cria uma referncia remota para um caminho do usurio
   */
  private RemoteFile createRemoteFile(String[] name)
    throws InvalidStateException, ServerException {
    /**
     * A referncia contm o caminho do arquivo atual, concatenado com o caminho
     * do usurio
     */
    String path =
      getPath() + Constants.PATH_SEPARATOR + Util.stringArrayToString(name);

    /** chama o mtodo para criao da referncia com o caminho completo. */
    return createRemoteFile(path);
  }

  /**
   * Obtem o arquivo associado a um caminho.
   */
  private File getLocalFile(String systemPath) throws InvalidStateException {
    /** Cria o arquivo relativo a raiz exportada pelo servidor */
    File localFile =
      new File(homeDirectory, Util
        .systemFileSeparatorToLocalFileSeparator(systemPath));

    /** Verifica a existncia do arquivo */
    if (!localFile.exists()) {
      throw new InvalidStateException("The file has been erased", Util
        .stringToStringArray(systemPath));
    }

    return localFile;
  }

  /**
   * Cria um canal
   */
  private org.omg.CORBA.Object getChannel(String poaName, String id,
    TypeCode type) throws InvalidStateException, ServerException {
    org.omg.CORBA.Object o = null;

    try {
      /** Localiza o POA responsvel pelo tipo de canal atravs do nome */
      POA poa = _poa().find_POA(poaName, false);

      /** Cria uma referencia com a identificacao do canal */
      o = poa.create_reference_with_id(id.getBytes(), type.id());
    }
    catch (UserException ex) {
      /** Caso haja algum problema de configurao com as polticas do POA */
      throw new InvalidStateException(ex.getMessage(), getFullName());
    }

    return o;
  }

  /**
   * Cria uma referencia remota
   */
  private RemoteFile createRemoteFile(String path)
    throws InvalidStateException, ServerException {
    org.omg.CORBA.Object o = null;

    try {
      /**
       * Utiliza o POA corrente para criar uma referncia para um outro arquivo
       */
      o =
        _poa().create_reference_with_id(path.getBytes(),
          RemoteFileHelper.type().id());
    }
    catch (UserException ex) {
      /** Caso haja algum problema de configurao com as polticas do POA */
      ex.printStackTrace();
      throw new InvalidStateException("Internal misconfiguration problem: "
        + ex.getMessage(), getFullName());
    }

    return RemoteFileHelper.narrow(o);
  }

}
