package csfs.impl;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Properties;
import java.util.logging.FileHandler;
import java.util.logging.Formatter;
import java.util.logging.Handler;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;

import org.omg.CORBA.ORB;
import org.omg.CORBA.Policy;
import org.omg.PortableServer.IdAssignmentPolicyValue;
import org.omg.PortableServer.IdUniquenessPolicyValue;
import org.omg.PortableServer.ImplicitActivationPolicyValue;
import org.omg.PortableServer.LifespanPolicyValue;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.RequestProcessingPolicyValue;
import org.omg.PortableServer.ServantRetentionPolicyValue;
import org.omg.PortableServer.ThreadPolicyValue;

import csfs.impl.channel.NullWriteChannelDefaultServantImpl;
import csfs.impl.channel.RandomAccessChannelDefaultServantImpl;
import csfs.impl.channel.ReadChannelDefaultServantImpl;
import csfs.impl.channel.WriteChannelDefaultServantImpl;
import csfs.impl.util.SystemProperties;
import csfs.impl.util.Util;

/**
 * A classe <class>Server</class> realiza a configurao do ORB e prepara a
 * hierarquia de POAs para atender as solicitaes do clientes. A criao de
 * polticas pode ser realizada de forma mais parcimoniosa, porm, para efeitos
 * de legibilidade, os trechos de cdigo so auto contidos e explicitos.
 * 
 * @version 1.0 27 Jun 2005
 * @author Marcelo Nery dos Santos
 */
public class Server {

  public static void main(String[] args) throws Exception {

    /**
     * Inicializao do ORB. A definio do ORB utilizado  passado como
     * parmetro da mquina virtual java
     */
    Properties orbProperties = new Properties();
    orbProperties
      .load(Server.class.getResourceAsStream("/orb_init.properties"));
    for (int i = 0; i < args.length; i++) {
      if (args[i].startsWith("--")) {
        int index = args[i].indexOf('=');
        orbProperties.setProperty(args[i].substring(2, index), args[i]
          .substring(index + 1));
        SystemProperties.setProperty(args[i].substring(2, index), args[i]
          .substring(index + 1));
      }
    }

    // Verifica se o ROOT_DIR corresponde a um diretrio existente no sistema
    // de arquivos. Em caso positivo, substitui o valor da propriedade pelo
    // canonicalPath. Caso contrrio, cancela a execuo.
    String rootDir = SystemProperties.getProperty("ROOT_DIR");
    if (rootDir == null) {
      System.err.println("ROOT_DIR no foi informado.");
      System.exit(-1);
    }
    String rootDirLocalFileSeparator = Util
      .systemFileSeparatorToLocalFileSeparator(rootDir);
    File rootDirFile = new File(rootDirLocalFileSeparator);
    if (!rootDirFile.exists()) {
      System.err.println("ROOT_DIR: " + rootDirLocalFileSeparator
        + " no existe!!");
      System.exit(-1);
    }
    if (!rootDirFile.isDirectory()) {
      System.err.println("ROOT_DIR: " + rootDirLocalFileSeparator
        + " no  diretrio!!");
      System.exit(-1);
    }
    rootDir = rootDirFile.getCanonicalPath();
    SystemProperties.setProperty("CANONICAL_ROOT_DIR", rootDir);

    /** Inicializao do mecanismo de log do sistema */
    Formatter f = new SimpleFormatter();
    Handler h = new FileHandler(SystemProperties.getProperty("SERVER_LOG"));
    h.setFormatter(f);
    Logger.global.addHandler(h);
    Logger.global.setUseParentHandlers(false);

    ORB orb = ORB.init(args, orbProperties);

    /** Obteno do RootPOA */
    POA rootPOA = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));

    /**
     * Criao das polticas do POA para o FileServer. Todas as polticas esto
     * apresentadas aqui e foram definidas com o objetivo de criar de um objeto
     * persistente, com um identificador de objetos legvel no endereo
     * corbaloc. A ativao do servant  feita explicitamente, e a localizao
     * do servant  feita pelo ORB atravs do mapeamento entre o object ID e o
     * servant.
     */
    Policy[] fileServerPolicies = new Policy[7];
    fileServerPolicies[0] = rootPOA
      .create_thread_policy(ThreadPolicyValue.ORB_CTRL_MODEL);
    fileServerPolicies[1] = rootPOA
      .create_lifespan_policy(LifespanPolicyValue.PERSISTENT);
    fileServerPolicies[2] = rootPOA
      .create_id_uniqueness_policy(IdUniquenessPolicyValue.UNIQUE_ID);
    fileServerPolicies[3] = rootPOA
      .create_id_assignment_policy(IdAssignmentPolicyValue.USER_ID);
    fileServerPolicies[4] = rootPOA
      .create_implicit_activation_policy(ImplicitActivationPolicyValue.NO_IMPLICIT_ACTIVATION);
    fileServerPolicies[5] = rootPOA
      .create_servant_retention_policy(ServantRetentionPolicyValue.RETAIN);
    fileServerPolicies[6] = rootPOA
      .create_request_processing_policy(RequestProcessingPolicyValue.USE_ACTIVE_OBJECT_MAP_ONLY);

    /** Criao do POA FileServer */
    POA fileServerPOA = rootPOA.create_POA("FileServerPOA", null,
      fileServerPolicies);

    /** Destruio das polticas previamente criadas */
    for (int i = 0; i < fileServerPolicies.length; i++) {
      fileServerPolicies[i].destroy();
    }

    /** Criao do servant do FileServer */
    FileServerImpl servant = new FileServerImpl(SystemProperties
      .getProperty("ROOT_DIR"));

    /** Ativao do servant com a um object id sugestivo */
    fileServerPOA.activate_object_with_id("FileServer".getBytes(), servant);

    /** Obteno de uma referncia para o servant recem ativado */
    org.omg.CORBA.Object fileServerObj = fileServerPOA
      .servant_to_reference(servant);

    /**
     * Escrita da referencia em arquivo, para o caso de problemas ao usar o
     * corbaloc
     */
    // PrintWriter pw = new PrintWriter(new FileWriter(
    // SystemProperties.getProperty("IOR_FILE")));
    PrintWriter pw = new PrintWriter(new FileWriter(args[0]));
    pw.println(orb.object_to_string(fileServerObj));
    pw.flush();
    pw.close();

    /**
     * Criao das polticas do POA para o RemoteFile. Todas as polticas esto
     * apresentadas aqui e foram definidas com o objetivo de criar de um objeto
     * persistente, com um identificador til na localizao direta dos arquivos
     * no servidor. A ativao do servant  feita explicitamente, e o default
     * servant  utilizado para atender as solicitaes.
     */
    Policy[] policies = new Policy[7];
    policies[0] = rootPOA
      .create_thread_policy(ThreadPolicyValue.ORB_CTRL_MODEL);
    policies[1] = rootPOA
      .create_lifespan_policy(LifespanPolicyValue.PERSISTENT);
    policies[2] = rootPOA
      .create_id_uniqueness_policy(IdUniquenessPolicyValue.UNIQUE_ID);
    policies[3] = rootPOA
      .create_id_assignment_policy(IdAssignmentPolicyValue.USER_ID);
    policies[4] = rootPOA
      .create_implicit_activation_policy(ImplicitActivationPolicyValue.NO_IMPLICIT_ACTIVATION);
    policies[5] = rootPOA
      .create_servant_retention_policy(ServantRetentionPolicyValue.NON_RETAIN);
    policies[6] = rootPOA
      .create_request_processing_policy(RequestProcessingPolicyValue.USE_DEFAULT_SERVANT);

    /** Criao do POA do RemoteFile */
    POA remoteFileDefaultServantPOA = rootPOA.create_POA(
      "RemoteFileDefaultServantPOA", null, policies);

    /** Destruio das polticas previamente criadas */
    for (int i = 0; i < policies.length; i++) {
      policies[i].destroy();
    }

    /** Criao do servant de RemoteFile */
    RemoteFileDefaultServantImpl remoteFileServant = new RemoteFileDefaultServantImpl(
      SystemProperties.getProperty("ROOT_DIR"), servant);

    /** Atribuio do servant ao POA */
    remoteFileDefaultServantPOA.set_servant(remoteFileServant);

    /**
     * Criao das polticas do POA para os Canais. Todas as polticas esto
     * apresentadas aqui e foram definidas com o objetivo de criar de um objeto
     * transiente, visto que o ciclo de vida do objeto depende de recursos
     * volteis alocados no servidor. O identificado do objeto  capaz de
     * localizar o descritor de arquivo associado ao canal. A ativao do
     * servant  feita explicitamente, e o default servant  utilizado para
     * atender as solicitaes. Os POAs dos canais so criados como filhos do
     * POA RemoteFile devido ao relacionamento entre esses objetos
     */
    Policy[] channelPolicies = new Policy[7];
    channelPolicies[0] = remoteFileDefaultServantPOA
      .create_thread_policy(ThreadPolicyValue.ORB_CTRL_MODEL);
    channelPolicies[1] = remoteFileDefaultServantPOA
      .create_lifespan_policy(LifespanPolicyValue.TRANSIENT);
    channelPolicies[2] = remoteFileDefaultServantPOA
      .create_id_uniqueness_policy(IdUniquenessPolicyValue.UNIQUE_ID);
    channelPolicies[3] = remoteFileDefaultServantPOA
      .create_id_assignment_policy(IdAssignmentPolicyValue.USER_ID);
    channelPolicies[4] = remoteFileDefaultServantPOA
      .create_implicit_activation_policy(ImplicitActivationPolicyValue.NO_IMPLICIT_ACTIVATION);
    channelPolicies[5] = remoteFileDefaultServantPOA
      .create_servant_retention_policy(ServantRetentionPolicyValue.NON_RETAIN);
    channelPolicies[6] = remoteFileDefaultServantPOA
      .create_request_processing_policy(RequestProcessingPolicyValue.USE_DEFAULT_SERVANT);

    /** Criao do POA do ReadChannel */
    POA readChannelDefaultServantPOA = remoteFileDefaultServantPOA.create_POA(
      "ReadChannelDefaultServantPOA", null, channelPolicies);
    /** Criao do servant de ReadChannel */
    ReadChannelDefaultServantImpl readChannelServant = new ReadChannelDefaultServantImpl();
    /** Atribuio do servant ao POA */
    readChannelDefaultServantPOA.set_servant(readChannelServant);

    /** Criao do POA do WriteChannel */
    POA writeChannelDefaultServantPOA = remoteFileDefaultServantPOA.create_POA(
      "WriteChannelDefaultServantPOA", null, channelPolicies);
    /** Criao do servant de ReadChannel */
    WriteChannelDefaultServantImpl writeChannelServant = new WriteChannelDefaultServantImpl();
    /** Atribuio do servant ao POA */
    writeChannelDefaultServantPOA.set_servant(writeChannelServant);

    /** Criao do POA do RandomAccessChannel */
    POA randomAccessChannelDefaultServantPOA = remoteFileDefaultServantPOA
      .create_POA("RandomAccessChannelDefaultServantPOA", null, channelPolicies);
    /** Criao do servant de RandomAccessChannel */
    RandomAccessChannelDefaultServantImpl randomAccessChannelServant = new RandomAccessChannelDefaultServantImpl();
    /** Atribuio do servant ao POA */
    randomAccessChannelDefaultServantPOA
      .set_servant(randomAccessChannelServant);

    /** Criao do POA do WriteChannel */
    POA nullWriteChannelDefaultServantPOA = remoteFileDefaultServantPOA
      .create_POA("NullWriteChannelDefaultServantPOA", null, channelPolicies);
    /** Criao do servant de ReadChannel */
    NullWriteChannelDefaultServantImpl nullWriteChannelServant = new NullWriteChannelDefaultServantImpl();
    /** Atribuio do servant ao POA */
    nullWriteChannelDefaultServantPOA.set_servant(nullWriteChannelServant);

    /** Destruio das polticas previamente criadas */
    for (int i = 0; i < channelPolicies.length; i++) {
      channelPolicies[i].destroy();
    }

    /**
     * Depois da configurao de todos os elementos, cada um dos POAManager deve
     * ser ativado, permitindo que as solicitaes sejam tratadas pelos
     * servants.  possvel ter apenas um Manager associado a todos os POAs,
     * porm o desempenho dessa abordagem ainda no foi avaliado.
     */
    rootPOA.the_POAManager().activate();
    fileServerPOA.the_POAManager().activate();
    remoteFileDefaultServantPOA.the_POAManager().activate();
    readChannelDefaultServantPOA.the_POAManager().activate();
    writeChannelDefaultServantPOA.the_POAManager().activate();
    randomAccessChannelDefaultServantPOA.the_POAManager().activate();
    nullWriteChannelDefaultServantPOA.the_POAManager().activate();

    /** Mantm a execuo do ORB e do processo servidor */
    Logger.global
      .info("ROOT_DIR = " + SystemProperties.getProperty("ROOT_DIR"));
    Logger.global.info("CANONICAL_ROOT_DIR = "
      + SystemProperties.getProperty("CANONICAL_ROOT_DIR"));
    orb.run();

  }

}
