package csfs.impl.channel;

import java.io.RandomAccessFile;
import java.io.IOException;

import org.omg.CORBA.UserException;

import csfs.impl.filetracker.FileTracker;
import csfs.impl.filetracker.FileTrackerLeasing;

import csfs.remote.RemoteFile;
import csfs.remote.RemoteFileHelper;
import csfs.remote.OctetSequenceHolder;
import csfs.remote.RandomAccessChannelPOA;
import csfs.remote.ClosedChannelException;
import csfs.remote.InvalidStateException;
import csfs.remote.ServerException;

/**
 * A classe <class>RandomAccessChannelDefaultServantImpl</class>  a
 * implementao da interface RandomAccessChannel (CORBA). Essa classe consitui
 * o Servant que ser utilizado por todos os canais de acesso aleatrio e
 * portanto nao pode possuir nenhum atributo que nao seja compartilhado entre
 * todos os objetos.
 * 
 * Cada canal aberto possui um descritor de arquivo associado, usado para
 * manipular os dados. Esse descritor  localizado atravs da identificao do
 * objeto, recuperada pelo ORB.
 * 
 * Para evitar a alocao indefinida de recursos, e fazendo com que os canais
 * sejam automaticamente encerrados aps um perodo de tempo, a classe
 * FileTrackerLeasing  utilizada.
 * 
 * Os mtodos dessa classe atendem a especificao da IDL e simplesmente
 * realizam chamadas aos mtodos de manipulao de dados de JAVA. O entendimento
 * sobre a parte de distribuio e CORBA est extensivamente descrito na classe
 * RemoteFileDefaultServantImpl. O mecanismo de leasing est descrito na classe
 * FileTrackerLeasing.
 * 
 * Basicamente, antes de realizar a operao sobre o descritor de arquivo, o
 * objeto deve ser recuperado pelo FileTracker, caso ele no seja encontrado
 * significa que o canal foi encerrado e uma exceo  lanada.
 * 
 * @version 1.0 22 Jun 2005
 * @author Marcelo Nery dos Santos
 */
public class RandomAccessChannelDefaultServantImpl extends
  RandomAccessChannelPOA {

  public RandomAccessChannelDefaultServantImpl() {
  }

  public void seek(long num) throws ClosedChannelException,
    InvalidStateException, ServerException {

    RandomAccessFile raf = getRandomAccessFile();
    if (raf != null) {
      try {
        raf.seek(Math.max(0, num));
      }
      catch (IOException exc) {
        exc.printStackTrace();
        throw new InvalidStateException("Unable to perform seek operation: "
          + exc.getMessage(), getFile().getFullName());
      }
    }
    else {
      throw new ClosedChannelException(_this());
    }
  }

  public long skip(long num) throws ClosedChannelException,
    InvalidStateException, ServerException {

    long seek = -1;
    RandomAccessFile raf = getRandomAccessFile();
    if (raf != null) {
      try {
        long pointer = raf.getFilePointer();
        if (num < (0 - pointer)) {
          seek = 0;
        }
        if (num + pointer > raf.length()) {
          seek = raf.length();
        }
        if (seek == -1) {
          seek = pointer + num;
        }
        raf.seek(seek);
        seek = seek - pointer;
      }
      catch (IOException exc) {
        exc.printStackTrace();
        throw new InvalidStateException("Unable to perform skip operation: "
          + exc.getMessage(), getFile().getFullName());
      }
    }
    else {
      throw new ClosedChannelException(_this());
    }
    return seek;
  }

  public int read(int size, OctetSequenceHolder buffer)
    throws ClosedChannelException, InvalidStateException, ServerException {

    int responseLength = 0;

    RandomAccessFile raf = getRandomAccessFile();
    if (raf != null) {
      try {
        responseLength = (int) Math.min(size, raf.length()
          - raf.getFilePointer());
        byte[] response = new byte[responseLength];
        int read = 0;
        int bytesRead = 0;
        while (read < responseLength && bytesRead != -1) {
          bytesRead = raf.read(response, read, (responseLength - read));
          if (bytesRead != -1) {
            read += bytesRead;
          }
        }
        if (read == responseLength) {
          buffer.value = response;
        }
        else {
          throw new InvalidStateException("Could NOT read expected "
            + responseLength + " bytes", getFile().getFullName());
        }
      }
      catch (IOException exc) {
        exc.printStackTrace();
        throw new InvalidStateException("Unable to perform read operation: "
          + exc.getMessage(), getFile().getFullName());
      }
    }
    else {
      throw new ClosedChannelException(_this());
    }
    return responseLength;
  }

  public int write(int size, byte[] buffer) throws ClosedChannelException,
    InvalidStateException, ServerException {
    int response = 0;
    RandomAccessFile raf = getRandomAccessFile();
    if (raf != null) {
      try {
        raf.write(buffer, 0, size);
        response = size;
      }
      catch (IOException ex) {
        ex.printStackTrace();
      }
    }
    else {
      throw new ClosedChannelException(_this());
    }
    return response;
  }

  public boolean isOpen() throws InvalidStateException, ServerException {
    return (getRandomAccessFile() != null);
  }

  public RemoteFile getFile() throws InvalidStateException, ServerException {
    String channelID = getObjectId();
    String fileID = channelID.substring(0, channelID.lastIndexOf('_'));
    return createRemoteFile(fileID);
  }

  public void close() throws InvalidStateException, ServerException {
    FileTracker tracker = FileTrackerLeasing.getInstance();
    tracker.removeFileDescriptor(getObjectId());
  }

  private RandomAccessFile getRandomAccessFile() throws InvalidStateException,
    ServerException {
    return (RandomAccessFile) FileTrackerLeasing.getInstance()
      .getFileDescriptor(getObjectId());
  }

  private String getObjectId() throws InvalidStateException {
    return new String(_object_id());
  }

  private RemoteFile createRemoteFile(String path)
    throws InvalidStateException, ServerException {
    org.omg.CORBA.Object o = null;
    try {
      o = _poa().the_parent().create_reference_with_id(path.getBytes(),
        RemoteFileHelper.type().id());
    }
    catch (UserException ex) {
      ex.printStackTrace();
      throw new InvalidStateException("Internal misconfiguration problem: "
        + ex.getMessage(), getFile().getFullName());
    }
    return RemoteFileHelper.narrow(o);
  }
}
