package csfs.impl.channel;

import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import org.omg.CORBA.UserException;

import csfs.impl.filetracker.FileTracker;
import csfs.impl.filetracker.FileTrackerLeasing;

import csfs.remote.RemoteFile;
import csfs.remote.RemoteFileHelper;
import csfs.remote.ServerException;
import csfs.remote.WriteChannelPOA;
import csfs.remote.ClosedChannelException;
import csfs.remote.InvalidStateException;

/**
 * A classe <class>WriteChannelDefaultServantImpl</class>  a implementao da
 * interface WriteChannel (CORBA). Essa classe consitui o Servant que ser
 * utilizado por todos os canais de escrita e portanto nao pode possuir nenhum
 * atributo que nao seja compartilhado entre todos os objetos.
 * 
 * Cada canal aberto possui um descritor de arquivo associado, usado para
 * manipular os dados. Esse descritor  localizado atravs da identificao do
 * objeto, recuperada pelo ORB.
 * 
 * Para evitar a alocao indefinida de recursos, e fazendo com que os canais
 * sejam automaticamente encerrados aps um perodo de tempo, a classe
 * FileTrackerLeasing  utilizada.
 * 
 * Os mtodos dessa classe atendem a especificao da IDL e simplesmente
 * realizam chamadas aos mtodos de manipulao de dados de JAVA. O entendimento
 * sobre a parte de distribuio e CORBA est extensivamente descrito na classe
 * RemoteFileDefaultServantImpl. O mecanismo de leasing est descrito na classe
 * FileTrackerLeasing.
 * 
 * Basicamente, antes de realizar a operao sobre o descritor de arquivo, o
 * objeto deve ser recuperado pelo FileTracker, caso ele no seja encontrado
 * significa que o canal foi encerrado e uma exceo  lanada.
 * 
 * @version 1.0 22 Jun 2005
 * @author Marcelo Nery dos Santos
 */
public class WriteChannelDefaultServantImpl extends WriteChannelPOA {

  public WriteChannelDefaultServantImpl() {

  }

  public int write(int size, byte[] buffer) throws ClosedChannelException,
    InvalidStateException, ServerException {

    int response = 0;
    OutputStream outputStream = getOutputStream();
    if (outputStream != null) {
      try {
        outputStream.write(buffer, 0, size);
        outputStream.flush();
        response = size;
      }
      catch (IOException exc) {
        exc.printStackTrace();
        throw new InvalidStateException("Unable to perform write operation: "
          + exc.getMessage(), getFile().getFullName());
      }
    }
    else {
      throw new ClosedChannelException(_this());
    }
    return response;
  }

  public void close() throws InvalidStateException {
    FileTracker tracker = FileTrackerLeasing.getInstance();
    tracker.removeFileDescriptor(getObjectId());
  }

  public RemoteFile getFile() throws InvalidStateException, ServerException {
    String channelID = getObjectId();
    String fileID = channelID.substring(0, channelID.lastIndexOf('_'));
    return createRemoteFile(fileID);
  }

  public boolean isOpen() throws InvalidStateException {
    return (getOutputStream() != null);
  }

  private OutputStream getOutputStream() throws InvalidStateException {
    return (OutputStream) FileTrackerLeasing.getInstance().getFileDescriptor(
      getObjectId());
  }

  private String getObjectId() throws InvalidStateException {
    return new String(_object_id());
  }

  private RemoteFile createRemoteFile(String path)
    throws InvalidStateException, ServerException {
    org.omg.CORBA.Object o = null;
    try {
      o = _poa().the_parent().create_reference_with_id(path.getBytes(),
        RemoteFileHelper.type().id());
    }
    catch (UserException ex) {
      ex.printStackTrace();
      throw new InvalidStateException("Internal misconfiguration problem: "
        + ex.getMessage(), getFile().getFullName());
    }
    return RemoteFileHelper.narrow(o);
  }
}
