package csfs.impl.filetracker;

import java.io.Closeable;
import java.util.Hashtable;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * A classe <class>FileTracker</class> realiza o controle sobre quais arquivos
 * esto em uso pelos clientes, ou seja, para quais arquivos o ssitema mantm
 * recursos alocados e no deve permitir a remoo do arquivo. Ele  usado pela
 * classe RemoteFileDefaultServantImpl.
 * 
 * Apenas uma instncia dessa classe existe no sistema e o padro de projetos
 * Singleton foi utilizado.
 * 
 * @version 1.0 27 Jun 2005
 * @author Marcelo Nery dos Santos
 */
public class FileTracker {

  /** A referncia para o nico FileTracker do sistema */
  private static FileTracker instance;

  /** O indicado de uso dos arquivos */
  private Hashtable fileUsage;

  /** O mapeamento entre identificadores de canal e o descritor de arquivo */
  private Hashtable channelFileDescriptor;

  /**
   * Contrutor da classe, definido como protected por ser chamado exclusivemante
   * pela classe, ou subclasses.
   */
  protected FileTracker() {
    fileUsage = new Hashtable();
    channelFileDescriptor = new Hashtable();
  }

  /** Recupera a instncia do FileTracker */
  public static FileTracker getInstance() {
    /** Verfica a existncia da instncia */
    if (instance == null) {
      /** Caso no exista, cria uma nova instncia */
      instance = new FileTracker();
      /** Indica que uma outra classe mais apropriada est disponvel */
      System.out.println("Why don't you use the FileTrackerLeasing?!");
    }
    return instance;
  }

  /**
   * Adiciona um novo identificador de canal e o respectivo descritor de
   * arquivo. Os descritores de arquivo vlidos so instncias de:
   * RandomAccessFile, BufferedInputStream, e BufferedOutputStream.
   * 
   * @param key String O identificador do canal, composto pelo caminho do
   *        arquivo, um underscore e o nmero de srie do canal.
   * @param fileDescriptor Object O descritor de arquivo.
   */
  public void setFileDescriptor(String key, Object fileDescriptor) {
    /** Se  um tipo vlido de descritor */
    if (fileDescriptor instanceof Closeable) {
      /** Obtem a identificao do arquivo */
      String file = key.substring(0, key.lastIndexOf('_'));
      /** Incrementa o nmero de canais abertos para o arquivo */
      incUsersCount(file);
      /** Adiciona um entrada no mapeamento de descritores */
      channelFileDescriptor.put(key, fileDescriptor);
    }
  }

  /**
   * Recupera o descritor de arquivos associado a uma determinada identificao
   * de canal.
   * 
   * @param key String A identificao do canal
   * @return Object O descritor de arquivo associado
   */
  public Object getFileDescriptor(String key) {
    return channelFileDescriptor.get(key);
  }

  /**
   * Remove o descritor de arquivo associado a um canal
   * 
   * @param key String A identificao do canal.
   */
  public void removeFileDescriptor(String key) {
    /** Verifica a existncia da chave */
    if (channelFileDescriptor.containsKey(key)) {
      /** Obtem a identificao do arquivo */
      String file = key.substring(0, key.lastIndexOf('_'));
      /** Decrementa o nmero de canais abertos para o arquivo */
      decUsersCount(file);
      /** Fecha o descritor associado a chave */
      closeFileDescriptor(key);
      /** Remove a entrada no mapeamento de descritores */
      channelFileDescriptor.remove(key);
    }
  }

  /**
   * Indica se um arquivo est em uso.
   * 
   * @param key String O caminho do arquivo
   * @return boolean true, caso esteja em uso. false, caso contrario.
   */
  public boolean inUse(String key) {
    /** Busca o nmero de usurios do arquivo */
    Object count = fileUsage.get(key);
    /** retorna se o valor  maior que zero */
    return (count != null);
  }

  /**
   * Fecha o descritor de arquivo.
   * 
   * @param key String o identificador do canal
   */
  private void closeFileDescriptor(String key) {
    try {
      Closeable fd = (Closeable) getFileDescriptor(key);
      fd.close();
    }
    catch (java.io.IOException ex) {
      Logger.global.log(Level.SEVERE, ex.getMessage(), ex);
    }
  }

  /**
   * Incrementa o nmero de usurios para um arquivo.
   */
  private void incUsersCount(String key) {
    Integer count = (Integer) fileUsage.get(key);
    if (count == null) {
      count = new Integer(1);
    }
    else {
      count = new Integer(count.intValue() + 1);
    }
    fileUsage.put(key, count);
  }

  /**
   * Decrementa o nmero de usurios para um arquivo.
   */
  private void decUsersCount(String key) {
    Integer count = (Integer) fileUsage.get(key);
    if (count.intValue() == 1) {
      fileUsage.remove(key);
    }
    else {
      fileUsage.put(key, new Integer(count.intValue() - 1));
    }
  }

  public int totalOpenFileDescriptors() {
    return channelFileDescriptor.size();
  }
}
