package csfs.impl.filetracker;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Iterator;

import csfs.impl.util.SystemProperties;
import java.util.logging.Logger;

/**
 * A classe <class>FileTrackerLeasing</class> extende o funcionamento da classe
 * FileTracker mantendo um controle sobre a utilizao dos canais e o indicador
 * da ltima utilizao dos mesmos. Periocicamente, uma thread verifica se algum
 * canal pode ser encerrados, por ter extrapolado o limite de tempo de
 * inatividade.
 * 
 * @version 1.0 27 Jun 2005
 * @author Marcelo Nery dos Santos
 */
public class FileTrackerLeasing extends FileTracker {

  private static FileTracker instance;

  /** A indicao de validade de cada um dos canais */
  private Hashtable leasing;

  /**
   * Thread auxiliar para notificar a classe e realizar a verificao da
   * validade dos canais
   */
  private FileTrackerLeasingThread clock;

  /** Perodo mximo de inatividade */
  private static final int INACTIVITY_LIMIT = SystemProperties
    .getIntProperty("INACTIVITY_LIMIT");

  protected FileTrackerLeasing() {
    super();
    leasing = new Hashtable();
    /** Inicia a thread auxiliar */
    clock = new FileTrackerLeasingThread(this, INACTIVITY_LIMIT);
    clock.start();
  }

  public static FileTracker getInstance() {
    if (instance == null) {
      instance = new FileTrackerLeasing();
    }
    return instance;
  }

  /**
   * Adiciona um novo identificador de canal e o respectivo descritor de
   * arquivo. Os descritores de arquivo vlidos so instncias de:
   * RandomAccessFile, BufferedInputStream, e BufferedOutputStream.
   * 
   * Atualiza a validade desse canal.
   * 
   * @param key String
   * @param fileDescriptor Object
   */
  public synchronized void setFileDescriptor(String key, Object fileDescriptor) {
    /** Realiza o procedimento da superclasse */
    super.setFileDescriptor(key, fileDescriptor);
    /**
     * Determina a validade desse objeto com base na hora local e no perodo de
     * inatividade
     */
    long time = System.currentTimeMillis();
    leasing.put(key, new Long(time + INACTIVITY_LIMIT));
  }

  /**
   * Recupera o descritor de arquivos associado a uma determinada identificao
   * de canal.
   * 
   * Atualiza a validade desse canal.
   * 
   * @param key String A identificao do canal
   * @return Object O descritor de arquivo associado
   */
  public Object getFileDescriptor(String key) {
    Object response = super.getFileDescriptor(key);
    long time = System.currentTimeMillis();
    leasing.put(key, new Long(time + INACTIVITY_LIMIT));
    return response;
  }

  /**
   * Remove o descritor de arquivo associado a um canal
   * 
   * @param key String A identificao do canal.
   */
  public synchronized void removeFileDescriptor(String key) {
    super.removeFileDescriptor(key);
    leasing.remove(key);
  }

  /**
   * Verifica a validade dos canais.
   */
  public synchronized void verifyObjectValidity() {

    /** Verifica a hora atual */
    long currentTime = System.currentTimeMillis();

    /** Define o vetor de objetos a ser coletado */
    Vector collected = new Vector();

    /** Recurepa a lista de canais */
    Iterator iter = leasing.keySet().iterator();
    /** Para cada entrada */
    while (iter.hasNext()) {
      /** Recupera a chave */
      Object key = iter.next();
      /** Recupera a validade */
      Long value = (Long) leasing.get(key);
      /** Verifica se a validade expirou */
      if (value.longValue() < currentTime) {
        /** Marca que o item deve ser coletado */
        collected.add(key);
      }
    }
    /** Para todos os itens que devem ser coletados */
    for (int i = 0; i < collected.size(); i++) {
      /** Chama o procedimento de remoo */
      removeFileDescriptor((String) collected.elementAt(i));
    }
    /** debug purposes */
    if (collected.size() > 0) {
      Logger.global.info("FileTrackerLeasing decided to collect "
        + collected.size() + " objects!");
    }
  }
}
