package csfs.impl.util;

/*
 * $Id$
 */

import java.io.File;

/**
 * <p>
 * Permite gernciar <i>locks</i> persistentes e arquivos que no representem
 * diretrios.
 * </p>
 * <p>
 * Os arquivos no so realmente bloqueados no sistema operacional, apenas 
 * criado um marcador indicando aos que utilizarem esta classe que aquele
 * arquivo j est sendo utilizado.<br>
 * Esses marcadores so diretrios criados no mesmo nvel de cada arquivo a ser
 * bloqueado e tendo como nome o do arquivo concatenado ao sufixo
 * {@value #LOCK_FILE_SUFFIX}.
 * </p>
 * 
 * @author Tecgraf / PUC-Rio
 */
public class FileLocker {

  /**
   * Sufixo a ser utilizado na construo do nome dos <i>locks</i>.
   */
  private static final String LOCK_FILE_SUFFIX = ".lock";

  /**
   * <p>
   * Tenta bloquear um arquivo.
   * </p>
   * <p>
   * O arquivo no  realmente bloqueado no sistema de arquivos, apenas cria-se
   * um marcador indicando que ele est sendo utilizado. Este marcador ir
   * persistir enquanto o mtodo {@link #unlock(File)} no for chamado e
   * qualquer outra chamada a este mtodo nesse meio tempo ir retornar
   * <tt>false</tt>. <br>
   * Esse marcador  um diretrio criado no mesmo nvel do arquivo a ser
   * bloqueado e tendo como nome o do arquivo concatenado ao sufixo
   * {@value #LOCK_FILE_SUFFIX}.
   * </p>
   * 
   * @param file arquivo a ser bloqueado.
   * 
   * @return <tt>true</tt> se o <i>lock</i> foi criado com sucesso ou
   *         <tt>false</tt> caso o arquivo j estivesse bloqueado ou ocorreu
   *         algum erro.
   * 
   * @throws SecurityException Se existe um gerente de segurana e seu
   *         <code>{@link
   *          java.lang.SecurityManager#checkWrite(java.lang.String)}</code>
   *         mtodo no permite que o <i>lock</i>, representado por um
   *         diretrio, seja criado.
   */
  public boolean tryLock(File file) {
    File lock = getLockFile(file);
    return lock.mkdir();
  }

  /**
   * <p>
   * Desbloqueia um arquivo bloqueado por {@link #tryLock(File)}.
   * </p>
   * <p>
   * O arquivo no foi realmente bloqueado no sistema de arquivos, apenas
   * criou-se um marcador indicando que ele est sendo utilizado. Este mtodo
   * ir remover este marcador permitindo que a prxima chamada ao mtodo
   * {@link #tryLock(File)} crie um novo e retorne <tt>true</tt>.<br>
   * Esse marcador  um diretrio criado no mesmo nvel do arquivo a ser
   * bloqueado e tendo como nome o do arquivo concatenado ao sufixo
   * {@value #LOCK_FILE_SUFFIX}.
   * </p>
   * 
   * @param file arquivo a ser desbloqueado.
   * 
   * @return <tt>True</tt> se o arquivo foi desbloqueado com sucesso ou
   *         <tt>false</tt> caso o arquivo no estivesse bloqueado ou ocorreu
   *         algum erro.
   * 
   * @throws SecurityException Se existe um gerente de segurana e seu
   *         <code>{@link
   *          java.lang.SecurityManager#checkDelete}</code> mtodo
   *         no permite que o <i>lock</i>, representado por um diretrio, seja
   *         apagado.
   */
  public boolean unlock(File file) {
    File lock = getLockFile(file);
    return lock.delete();
  }

  /**
   * Obtm o arquivo que representa o <i>lock</i> para o arquivo passado.
   * 
   * @param file arquivo a ser bloqueado.
   * 
   * @return um diretrio, que pode ou no existir no sistema de arquivos,
   *         representando um <i>lock</i> apra o arquivo passado.
   * 
   * @throws IllegalArgumentException se <code>null == file</code> ou se o
   *         arquivo a ser bloqueado representar um diretrio.
   */
  private File getLockFile(File file) {
    if (null == file) {
      throw new IllegalArgumentException("File is null.");
    }
    if (!file.isFile()) {
      throw new IllegalArgumentException(file.getAbsolutePath()
        + " does not exists or is not a normal file.");
    }

    return new File(file.getParent(), file.getName() + LOCK_FILE_SUFFIX);
  }

  /**
   * Verifica se um arquivo representa um <i>lock</i>.
   * 
   * @param fileName nome do arquivo a ser verificado.
   * 
   * @return <tt>True</tt> se o arquivo, cujo nome foi passado como parmetro,
   *         representa um <i>lock</i>.
   */
  public boolean isLockFile(String fileName) {
    return fileName.endsWith(FileLocker.LOCK_FILE_SUFFIX);
  }
}
