package csfs.impl.util;

import java.io.File;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.StringTokenizer;

public class Util {

  /**
   * Verifica se um caminho de arquivo estah dentro da arvore exportada pelo
   * sistema. A definicao de "estar dentro" se baseia em:
   * 
   * Imaginando um usuario logado na raiz do sistema, ele eh capaz de usar
   * comandos como "cd" para localizar o arquivo. A unica restricao eh quanto a
   * saida do usuario da raiz. Durante suas operacoes, ele nao pode sair da raiz
   * em nenhum momento.
   * 
   * Esse metodo deve tratar a presenca de links simbolicos, considerando que um
   * arquivo atingido a partir de um link simbolico eh um arquivo valido.
   * 
   * Esse metodo nao deve falhar caso algum arquivo nao exista. Se o candidato a
   * arquivo for um arquivo valido, ele deve retornar verdadeiro.
   * 
   * @param f - O candidato a ser um arquivo valido
   * @return true - o caminho do arquivo eh acessivel a partir da raiz, sem sair
   *         dela em nenhum momento (exceto para seguir links simbolicos)<br>
   *         false - o caminho do arquivo estah fora da raiz.
   * 
   */
  public static boolean isValid(File f) {
    boolean response;
    try {
      String rootDir = SystemProperties.getProperty("CANONICAL_ROOT_DIR");
      String absolutePath = f.getCanonicalPath();
      response = absolutePath.startsWith(rootDir);
      if (response) {
        if (absolutePath.indexOf("..") != -1) {
          String relativePath = absolutePath.substring(rootDir.length() + 1);
          File probe = new File(rootDir);
          String allowedPath = probe.getCanonicalPath();
          String[] elements = stringToStringArray(relativePath, ""
            + File.separator);
          for (int i = 0; i < elements.length && response; i++) {
            probe = new File(probe, elements[i]);
            String currentPath = probe.getCanonicalPath();
            if (elements[i].equals("..")) {
              // o caminho corrente saiu da area permitida.
              response = currentPath.startsWith(allowedPath);
            }
            else {
              if (!(currentPath.startsWith(allowedPath))) {
                // um link simbolico foi seguido
                allowedPath = currentPath;
              }
            }
          }
        }
      }
    }
    catch (IOException exc) {
      exc.printStackTrace();
      response = false;
    }
    return response;
  }

  public static String[] stringToStringArray(String path) {
    return stringToStringArray(path, "" + Constants.PATH_SEPARATOR);
  }

  public static String[] stringToStringArray(String path, String tokensChars) {
    StringTokenizer tokenizer = new StringTokenizer(path, tokensChars);
    int tokens = tokenizer.countTokens();
    String[] response = new String[tokens];
    for (int i = 0; i < response.length; i++) {
      response[i] = tokenizer.nextToken();
    }
    return response;
  }

  public static String stringArrayToString(String[] path) {
    StringBuffer buf = new StringBuffer();
    for (int i = 0; i < path.length; i++) {
      buf.append(path[i]);
      if (i + 1 < path.length) {
        buf.append(Constants.PATH_SEPARATOR);
      }
    }
    String filePath = buf.toString();
    return filePath;
  }

  public static String localFileSeparatorToSystemFileSeparator(String path) {
    String response = path;
    if (File.separatorChar != Constants.PATH_SEPARATOR) {
      response = path.replace(File.separatorChar, Constants.PATH_SEPARATOR);
    }
    return response;
  }

  public static String systemFileSeparatorToLocalFileSeparator(String path) {
    String response = path;
    if (File.separatorChar != Constants.PATH_SEPARATOR) {
      response = path.replace(Constants.PATH_SEPARATOR, File.separatorChar);
    }
    return response;
  }

  /**
   * #TODO
   * 
   * @param path
   * @param remoteFileSeparator
   * @return
   */
  public static String remoteFileSeparatorToSystemFileSeparator(String path,
    char remoteFileSeparator) {
    String response = path;
    if (remoteFileSeparator != Constants.PATH_SEPARATOR) {
      response = path.replace(remoteFileSeparator, Constants.PATH_SEPARATOR);
    }
    return response;
  }

  public static String[] getPathArray(String systemPath, String[] element) {
    String fullPath = systemPath + Constants.PATH_SEPARATOR
      + Util.stringArrayToString(element);
    return Util.stringToStringArray(fullPath);
  }

  public static String[] getPathArray(String systemPath, String element) {
    String fullPath = systemPath + Constants.PATH_SEPARATOR + element;
    return Util.stringToStringArray(fullPath);
  }

  /**
   * 
   * Metodo para ler uma string de um ByteBuffer. Usa o caracter '\n' como
   * delimitador.
   * 
   * @param buffer
   * @return
   */
  public static String readStringFromBuffer(ByteBuffer buffer) {
    int position = buffer.position();
    int limit = buffer.limit();
    StringBuffer response = new StringBuffer();
    while (position < limit) {
      byte b = buffer.get();
      position++;
      if (b != '\n') {
        response.append((char) b);
      }
      else {
        break;
      }
    }
    return response.toString();
  }

  /**
   * 
   * Metodo otimizado para leitura de um long de um ByteBuffer. Usa o caracter
   * '\n' como delimitador. O resultado eh equivalente a chamar um
   * Long.parseLong(readStringFromBuffer()).
   * 
   * @param buffer
   * @return
   */
  public static long readLongFromBuffer(ByteBuffer buffer) {
    int position = buffer.position();
    int limit = buffer.limit();
    long response = 0;
    while (position < limit) {
      byte b = buffer.get();
      position++;
      if (b != '\n') {
        response = response * 10 + (b - '0');
      }
      else {
        break;
      }
    }
    return response;
  }

}
