package csfs.test;

import java.io.BufferedReader;
import java.io.FileReader;

import java.util.Vector;
import java.util.Arrays;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import org.omg.CORBA.ORB;

import junit.framework.TestCase;

import csfs.remote.RemoteFile;
import csfs.remote.FileServer;
import csfs.remote.FileServerHelper;
import csfs.remote.ReadChannel;
import csfs.remote.ServerException;
import csfs.remote.WriteChannel;
import csfs.remote.RandomAccessChannel;
import csfs.remote.OctetSequenceHolder;
import csfs.remote.ClosedChannelException;
import csfs.remote.InvalidPathException;
import csfs.remote.InvalidStateException;
import csfs.remote.FileAlreadyExistsException;
import csfs.remote.NotDirectoryException;
import csfs.remote.FileNotFoundException;
import csfs.remote.NotEmptyException;
import csfs.remote.FileInUseException;
import csfs.remote.NotFileException;
import csfs.impl.util.SystemProperties;
import csfs.remote.Metadata;
import csfs.remote.MetadataSequenceHolder;

/**
 * A classe <class>TestRemoteFileDefaultServantImpl</class> implementa os
 * testes automatizados do sistema. Os testes possuem uma boa abrangncia,
 * cobrindo os todos mtodos definidos pela interface CORBA, verificando as
 * condies de erro e a corretude das informaes passadas nas excees.
 * 
 * Na documentao interna da classe, os mtodos iniciais apresentam comentrios
 * mais detalhados sobre as operaes realizadas para atender as solicitaes,
 * para evitar repeties, no mtodos subsequentes apenas os novos conceitos
 * sero apresentados.
 * 
 * Atualizado em 25/06/2006: Com as recentes modificacoes no sistema, os testes
 * ainda nao acompanharam a evolucao... Revisar isso! :)
 * 
 * @version 1.0 27 Jun 2005
 * @author Marcelo Nery dos Santos
 */

/** @todo Testar parmetros nulos e mais testes para as funcoes de Metadata */

public class TestRemoteFileDefaultServantImpl extends TestCase {

  /** A raiz para a rvore exportada pelo servidor */
  private RemoteFile root = null;

  public TestRemoteFileDefaultServantImpl() throws Exception {
    ORB orb = ORB.init((String[]) null, null);
    String iorFile = SystemProperties.getProperty("IOR_FILE");
    BufferedReader br = new BufferedReader(new FileReader(iorFile));
    org.omg.CORBA.Object obj = orb.string_to_object(br.readLine());
    br.close();
    FileServer fileServer = FileServerHelper.narrow(obj);
    root = fileServer.getRoot();
  }

  /**
   * Esse mtodo, chamado pelo JUnit antes de cada teste, vefirica se a raiz
   * para a rvore exportada pelo servidor j foi inicializada.
   * 
   * @throws Exception
   */
  protected void setUp() throws Exception {
    super.setUp();
    if (root.getChildren().length > 0) {
      throw new Exception(
        "There are files inside root! Can not perform the test in this situation!!!");
    }
  }

  /**
   * Esse mtodo, chamado pelo JUnit aps cada teste, remove todos os arquivos
   * criados, mantendo o invariante dos testes, que consiste no diretrio raiz
   * vazio.
   * 
   * @throws Exception
   */
  protected void tearDown() throws Exception {
    // remove all files in root directory.
    RemoteFile[] children = root.getChildren();
    for (int i = 0; i < children.length; i++) {
      recursiveRemove(children[i]);
    }
    super.tearDown();
  }

  /**
   * Remove o contedo de um diretrio recursivamente. Detalhes da implementao
   * foram escritos em ingls:
   * 
   * This implementation loads all remote references before deleting any files.
   * It should not be used in an uncontrolled environment due to its greedy
   * approch.
   */
  private void recursiveRemove(RemoteFile treeRoot) throws Exception {
    Vector elements = new Vector();
    elements.add(treeRoot);
    int index = 0;
    while (index < elements.size()) {
      RemoteFile remoteFile = (RemoteFile) elements.elementAt(index);
      index++;
      if (remoteFile.isDirectory()) {
        RemoteFile[] children = remoteFile.getChildren();
        elements.addAll(Arrays.asList(children));
      }
    }
    for (int i = elements.size() - 1; i != -1; i--) {
      RemoteFile remoteFile = (RemoteFile) elements.elementAt(i);
      remoteFile.remove();
    }
  }

  /**
   * In this scenario, receiving null arguments must throw a
   * NullPointerException
   */
  private boolean assertEquals(String[] expected, String[] actual) {
    boolean response = true;
    if (expected.length == actual.length) {
      for (int i = 0; i < expected.length && response; i++) {
        response = expected[i].equals(actual[i]);
      }
    }
    else {
      response = false;
    }
    return response;
  }

  /**
   * Teste para o RF001 - Criar Diretrios
   * 
   * Os passos descritos no documento de Planejamento, Projeto e Execuo de
   * testes esto realizados por esse teste.
   */
  public void testCreateDirectory() {
    try {
      /** Um caminho simples  criado */
      String[] nameA = new String[] { "A" };
      /** O diretrio  criado na raiz com o caminho simples */
      RemoteFile actualReturnA = root.createDirectory(nameA);
      /** Como verificao, o caminho completo do arquivo  solicitado */
      String[] actualNameA = actualReturnA.getFullName();
      /** Verificamos se o arquivo remoto corresponde a um diretrio */
      assertTrue(actualReturnA.isDirectory());
      /** Verificamos se o caminho do arquivo corresponde ao parmetro */
      assertTrue(assertEquals(nameA, actualNameA));

      try {
        /** Tentamos criar um diretrio sobre o diretrio j existente */
        RemoteFile dup = root.createDirectory(nameA);
        /** Caso o sistema nao lane excees, temos uma falha */
        fail();
      }
      catch (FileAlreadyExistsException exc) {
        /**
         * Caso a exceo correta seja lanada, verificamos se o contedo da
         * exceo referencia o arquivo correto
         */
        assertTrue(assertEquals(nameA, exc.name));
      }

      /** Equivalente ao teste anterior, porm com um caminho composto */
      String[] nameA_A1 = new String[] { "A", "A1" };
      RemoteFile actualReturnA_A1 = root.createDirectory(nameA_A1);
      String[] actualNameA_A1 = actualReturnA_A1.getFullName();
      assertTrue(actualReturnA_A1.isDirectory());
      assertTrue(assertEquals(nameA_A1, actualNameA_A1));

      /**
       * Equivalente ao teste de conflito anterior, porm com um caminho
       * composto
       */
      try {
        RemoteFile dup = root.createDirectory(nameA_A1);
        fail();
      }
      catch (FileAlreadyExistsException exc) {
        assertTrue(assertEquals(nameA_A1, exc.name));
      }

      /** Equivalente ao teste anterior, porm com um caminho composto */
      String[] nameB_B1_B11 = new String[] { "B", "B1", "B11" };
      RemoteFile actualReturnB_B1_B11 = root.createDirectory(nameB_B1_B11);
      String[] actualNameB_B1_B11 = actualReturnB_B1_B11.getFullName();
      assertTrue(actualReturnB_B1_B11.isDirectory());
      assertTrue(assertEquals(nameB_B1_B11, actualNameB_B1_B11));

      /**
       * Equivalente ao teste de conflito anterior, porm com um caminho
       * composto
       */
      String[] partialDup = new String[] { "B", "B1" };
      try {
        RemoteFile dup = root.createDirectory(partialDup);
        fail();
      }
      catch (FileAlreadyExistsException exc) {
        assertTrue(assertEquals(partialDup, exc.name));
      }

      /** Verifica a criao de um diretrio fora da raiz */
      String[] outOfRoot = new String[] { ".." };
      try {
        RemoteFile outOfRootFile = root.createDirectory(outOfRoot);
        fail();
      }
      catch (InvalidPathException ex) {
        assertTrue(assertEquals(outOfRoot, ex.name));
      }

      /** Verifica a criao de um diretrio sobre um arquivo */
      String[] file = new String[] { "someDir", "anotherDir", "file" };
      String[] dir = new String[] { "dir" };
      RemoteFile remoteFile = root.createFile(file);
      try {
        remoteFile.createDirectory(dir);
        fail();
      }
      catch (NotDirectoryException ex) {
        assertTrue(assertEquals(file, ex.name));
      }

      /** Verifica a criao de um diretrio sobre um diretrio removido */
      String[] dirH = new String[] { "H" };
      try {
        RemoteFile first = root.createDirectory(dirH);
        RemoteFile grabber = root.getChild(dirH);
        grabber.remove();
        first.createDirectory(new String[] { "dir" });
        fail();
      }
      catch (InvalidStateException ex) {
        assertTrue(assertEquals(dirH, ex.name));
      }
      catch (Exception exc) {
        fail();
        exc.printStackTrace();
      }

      /** Verifica a criao de um diretrio como nomes invlidos */
      if (System.getProperty("os.name").startsWith("Windows")) {
        // this test assumes that the file server operational system is the same
        // of the test runner.
        String[] badName = new String[] { "a?1" };
        try {
          RemoteFile badFile = root.createDirectory(badName);
          fail();
        }
        catch (InvalidPathException ex) {
          assertTrue(assertEquals(badName, ex.name));
        }

        String[] badName2 = new String[] { "someDirWrong", "a?1" };
        try {
          RemoteFile badFile = root.createDirectory(badName2);
          fail();
        }
        catch (InvalidPathException ex) {
          assertTrue(assertEquals(badName2, ex.name));
        }
      }

      /** Verifica a criao de um diretrio contendo separadores de arquivo */
      String[] wrongDirSlash = new String[] { "dir1/dir2" };
      try {
        root.createDirectory(wrongDirSlash);
        fail();
      }
      catch (InvalidPathException ex) {
        assertTrue(assertEquals(wrongDirSlash, ex.name));
      }

      /** Verifica a criao de um diretrio contendo separadores de arquivo */
      String[] wrongDirBackSlash = new String[] { "dir1\\dir2" };
      try {
        root.createDirectory(wrongDirBackSlash);
        fail();
      }
      catch (InvalidPathException ex) {
        assertTrue(assertEquals(wrongDirBackSlash, ex.name));
      }
    }
    catch (Exception exc) {
      exc.printStackTrace();
      fail();
    }
  }

  /**
   * Os casos de testes do RF002 sao semelhantes aos casos de RF001 mostrados
   * nessa classe.
   */
  public void testCreateFile() throws InvalidPathException,
    InvalidStateException, FileAlreadyExistsException, NotDirectoryException,
    ServerException {

    String[] fileA = new String[] { "a" };
    RemoteFile actualReturnFileA = root.createFile(fileA);
    String[] actualFileA = actualReturnFileA.getFullName();
    assertFalse(actualReturnFileA.isDirectory());
    assertTrue(assertEquals(fileA, actualFileA));

    try {
      RemoteFile dup = root.createFile(fileA);
      fail();
    }
    catch (FileAlreadyExistsException exc) {
      assertTrue(assertEquals(fileA, exc.name));
    }

    String[] fileDir1_Dir2_A = new String[] { "dir1", "dir2", "a" };
    RemoteFile actualReturnFileDir1_Dir2_A = root.createFile(fileDir1_Dir2_A);
    String[] actualFileDir1_Dir2_A = actualReturnFileDir1_Dir2_A.getFullName();
    assertFalse(actualReturnFileDir1_Dir2_A.isDirectory());
    assertTrue(assertEquals(fileDir1_Dir2_A, actualFileDir1_Dir2_A));

    try {
      RemoteFile dup = root.createFile(fileDir1_Dir2_A);
      fail();
    }
    catch (FileAlreadyExistsException exc) {
      assertTrue(assertEquals(fileDir1_Dir2_A, exc.name));
    }

    String[] fileDir1_A = new String[] { "dir1", "a" };
    RemoteFile actualReturnFileDir1_A = root.createFile(fileDir1_A);
    String[] actualFileDir1_A = actualReturnFileDir1_A.getFullName();
    assertFalse(actualReturnFileDir1_A.isDirectory());
    assertTrue(assertEquals(fileDir1_A, actualFileDir1_A));

    try {
      RemoteFile dup = root.createFile(fileDir1_A);
      fail();
    }
    catch (FileAlreadyExistsException exc) {
      assertTrue(assertEquals(fileDir1_A, exc.name));
    }

    String[] outOfRoot = new String[] { "..", "root", ".." };
    try {
      RemoteFile outOfRootFile = root.createFile(outOfRoot);
      fail();
    }
    catch (InvalidPathException ex) {
      assertTrue(assertEquals(outOfRoot, ex.name));
    }

    String[] file = new String[] { "someDir", "anotherDir", "file" };
    String[] anotherFile = new String[] { "anotherFile" };
    RemoteFile remoteFile = root.createFile(file);
    try {
      remoteFile.createFile(anotherFile);
      fail();
    }
    catch (NotDirectoryException ex) {
      assertTrue(assertEquals(file, ex.name));
    }

    String[] dirJ = new String[] { "J" };
    try {
      RemoteFile first = root.createDirectory(dirJ);
      RemoteFile grabber = root.getChild(dirJ);
      grabber.remove();
      first.createFile(new String[] { "file" });
      fail();
    }
    catch (InvalidStateException ex) {
      assertTrue(assertEquals(dirJ, ex.name));
    }
    catch (Exception exc) {
      fail();
      exc.printStackTrace();
    }

    if (System.getProperty("os.name").startsWith("Windows")) {
      // this test assumes that the file server operational system is the same
      // of the test runner.
      String[] badName = new String[] { "a?1" };
      try {
        RemoteFile badFile = root.createFile(badName);
        fail();
      }
      catch (InvalidPathException ex) {
        assertTrue(assertEquals(badName, ex.name));
      }

      String[] badName2 = new String[] { "someDirWrong", "a?1" };
      try {
        RemoteFile badFile = root.createFile(badName2);
        fail();
      }
      catch (InvalidPathException ex) {
        assertTrue(assertEquals(badName2, ex.name));
      }
    }

    String[] wrongDirSlash = new String[] { "dir1/dir2" };
    try {
      root.createDirectory(wrongDirSlash);
      fail();
    }
    catch (InvalidPathException ex) {
      assertTrue(assertEquals(wrongDirSlash, ex.name));
    }

    String[] wrongDirBackSlash = new String[] { "dir1\\dir2" };
    try {
      root.createDirectory(wrongDirBackSlash);
      fail();
    }
    catch (InvalidPathException ex) {
      assertTrue(assertEquals(wrongDirBackSlash, ex.name));
    }

  }

  public void testGetChild() throws InvalidPathException,
    InvalidStateException, NotDirectoryException, FileNotFoundException,
    ServerException {

    String[] name = new String[] { "dir1", "dir2", "dir3", "dir4", "dir5",
        "file" };
    try {
      root.createFile(name);
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }

    String[] nameDir1_Dir2 = new String[] { "dir1", "dir2" };
    RemoteFile actualReturnDir1_Dir2 = root.getChild(nameDir1_Dir2);
    String[] actualNameDir1_Dir2 = actualReturnDir1_Dir2.getFullName();
    assertTrue(assertEquals(nameDir1_Dir2, actualNameDir1_Dir2));

    String[] nameDir3_Dir4_Dir5 = new String[] { "dir3", "dir4", "dir5" };
    RemoteFile actualReturnDir3_Dir4_Dir5 = actualReturnDir1_Dir2
      .getChild(nameDir3_Dir4_Dir5);
    String[] actualNameDir3_Dir4_Dir5 = actualReturnDir3_Dir4_Dir5
      .getFullName();
    String[] expectedNameDir3_Dir4_Dir5 = new String[] { "dir1", "dir2",
        "dir3", "dir4", "dir5" };
    assertTrue(assertEquals(expectedNameDir3_Dir4_Dir5,
      actualNameDir3_Dir4_Dir5));

    String[] nameFile = new String[] { "file" };
    RemoteFile actualReturnFile = actualReturnDir3_Dir4_Dir5.getChild(nameFile);
    String[] actualNameFile = actualReturnFile.getFullName();
    assertTrue(assertEquals(name, actualNameFile));

    try {
      actualReturnFile.getChild(new String[] { "another" });
      fail();
    }
    catch (NotDirectoryException ex) {
      assertTrue(assertEquals(name, ex.name));
    }

    RemoteFile parent = actualReturnDir3_Dir4_Dir5
      .getChild(new String[] { ".." });
    String parentName[] = parent.getFullName();
    RemoteFile usualParent = actualReturnDir3_Dir4_Dir5.getParent();
    String usualParentName[] = usualParent.getFullName();
    assertTrue(assertEquals(usualParentName, parentName));

    RemoteFile circle = actualReturnDir3_Dir4_Dir5.getChild(new String[] {
        "..", "..", "..", ".", ".", "dir3", "dir4", "dir5" });
    String circleName[] = circle.getFullName();
    assertTrue(assertEquals(expectedNameDir3_Dir4_Dir5, circleName));

    String[] strangeName = new String[] { "g*1" };
    try {
      RemoteFile strangeFile = root.getChild(strangeName);
      fail();
    }
    catch (FileNotFoundException ex) {
      assertTrue(assertEquals(strangeName, ex.name));
    }

    String[] regularName = new String[] { "nonExistingDir", "nonExistingFile" };
    try {
      RemoteFile regularFile = root.getChild(regularName);
      fail();
    }
    catch (FileNotFoundException ex) {
      // o sistema responde que o primeiro elemento nao pode ser localizado, e
      // ignora todos os demais elementos.
      assertEquals(regularName[0], ex.name[0]);
    }

    String[] regularName2 = new String[] { "dir1", "nonExistingFile" };
    try {
      RemoteFile regularFile2 = root.getChild(regularName2);
      fail();
    }
    catch (FileNotFoundException ex) {
      assertTrue(assertEquals(regularName2, ex.name));
    }

    String[] outOfRoot = new String[] { ".." };
    try {
      RemoteFile outOfRootFile = root.getChild(outOfRoot);
      fail();
    }
    catch (InvalidPathException ex) {
      assertTrue(assertEquals(outOfRoot, ex.name));
    }

    String[] dirH = new String[] { "H" };
    try {
      RemoteFile first = root.createDirectory(dirH);
      RemoteFile grabber = root.getChild(dirH);
      grabber.remove();
      first.getChild(new String[] { "dir" });
      fail();
    }
    catch (InvalidStateException ex) {
      assertTrue(assertEquals(dirH, ex.name));
    }
    catch (Exception exc) {
      fail();
      exc.printStackTrace();
    }
  }

  public void testGetChildren() throws InvalidStateException,
    NotDirectoryException, ServerException {

    RemoteFile dirA = null;
    RemoteFile f = null;
    try {
      dirA = root.createDirectory(new String[] { "dirA" });
      RemoteFile dirB = dirA.createDirectory(new String[] { "dirB" });
      RemoteFile dirC = dirA.createDirectory(new String[] { "dirC" });
      dirC.createFile(new String[] { "D", "E", "F" });
      f = dirA.createFile(new String[] { "file" });
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }

    int total = 0;
    int expected = 8;
    Vector elements = new Vector();
    elements.add(root);

    while (elements.size() > 0) {
      RemoteFile remoteFile = (RemoteFile) elements.elementAt(0);
      if (remoteFile.isDirectory()) {
        RemoteFile[] children = remoteFile.getChildren();
        elements.addAll(Arrays.asList(children));
      }
      elements.remove(0);
      total++;
    }
    assertEquals(expected, total);

    String[] filename = new String[] { "dirA", "file" };
    try {
      f.getChildren();
      fail();
    }
    catch (NotDirectoryException ex) {
      assertTrue(assertEquals(filename, ex.name));
    }

    try {
      f.remove();
      f.getChildren();
      fail();
    }
    catch (InvalidStateException ex) {
      assertTrue(assertEquals(filename, ex.name));
    }
    catch (Exception exc) {
      fail();
      exc.printStackTrace();
    }

  }

  public void testGetFullName() throws InvalidStateException, ServerException {
    RemoteFile file = null;
    try {
      RemoteFile fileA = root
        .createDirectory(new String[] { "A", "B", "C", "D" });
      RemoteFile fileB = fileA.createDirectory(new String[] { "E" });
      file = fileB.createFile(new String[] { "file" });
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }

    String[] name = new String[] { "A", "B", "C", "D", "E", "file" };
    String[] actual = file.getFullName();
    assertTrue(assertEquals(name, actual));

    try {
      file.remove();
      file.getFullName();
      fail();
    }
    catch (InvalidStateException ex) {
      assertTrue(assertEquals(name, ex.name));
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }
  }

  public void testGetName() throws InvalidStateException, ServerException {
    RemoteFile probeDir = null;
    RemoteFile probeFile = null;
    String probeDirStr[] = new String[] { "A", "A1" };
    String probeFileStr[] = new String[] { "B", "B1" };
    try {
      probeDir = root.createDirectory(probeDirStr);
      probeFile = root.createFile(probeFileStr);
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }

    assertEquals("A1", probeDir.getName());
    assertEquals("B1", probeFile.getName());

    try {
      probeFile.remove();
      probeFile.getName();
      fail();
    }
    catch (InvalidStateException ex) {
      assertTrue(assertEquals(probeFileStr, ex.name));
    }
    catch (Exception exc) {
      fail();
      exc.printStackTrace();
    }

    try {
      probeDir.remove();
      probeDir.getName();
      fail();
    }
    catch (InvalidStateException ex) {
      assertTrue(assertEquals(probeDirStr, ex.name));
    }
    catch (Exception exc) {
      fail();
      exc.printStackTrace();
    }
  }

  public void testGetParent() throws InvalidPathException,
    InvalidStateException, ServerException {
    RemoteFile probe = null;
    RemoteFile parent = null;

    String[] probeStr = new String[] { "A", "B" };
    try {
      probe = root.createDirectory(probeStr);
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }

    parent = probe.getParent();
    assertEquals("A", parent.getName());

    parent = parent.getParent();

    try {
      parent = parent.getParent();
      fail();
    }
    catch (InvalidPathException ex) {
      assertTrue(assertEquals(new String[] { ".." }, ex.name));
    }

    try {
      probe.remove();
      probe.getParent();
      fail();
    }
    catch (InvalidStateException ex) {
      assertTrue(assertEquals(probeStr, ex.name));
    }
    catch (Exception exc) {
      fail();
      exc.printStackTrace();
    }
  }

  public void testGetRandomAccessChannel() throws NotFileException,
    InvalidStateException, ServerException {
    RemoteFile file = null;
    String[] fileStr = new String[] { "A", "B", "C" };
    try {
      file = root.createFile(fileStr);
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
    String alphabet = "ABCDEFGHIJKLMNOPQRSTUWXYZ";

    try {
      RandomAccessChannel raf = file.getRandomAccessChannel();
      raf.write(alphabet.length(), alphabet.getBytes());
      OctetSequenceHolder buf = new OctetSequenceHolder(new byte[1]);
      raf.seek(0);
      raf.read(1, buf);
      assertEquals('A', buf.value[0]);
      long skip = raf.skip(3);
      assertEquals(3, skip);
      raf.read(1, buf);
      assertEquals('E', buf.value[0]);
      raf.seek(2);
      raf.read(1, buf);
      assertEquals('C', buf.value[0]);

      RandomAccessChannel another = file.getRandomAccessChannel();
      another.seek(3);
      String corruptAlphabet = "NERY";
      another.write(corruptAlphabet.length(), corruptAlphabet.getBytes());

      raf.seek(0);
      buf.value = new byte[25];
      raf.read(25, buf);

      assertEquals("ABCNERYHIJKLMNOPQRSTUWXYZ", new String(buf.value));
      another.close();
      raf.close();
    }
    catch (ClosedChannelException ex) {
      ex.printStackTrace();
    }

    try {
      RandomAccessChannel raf = file.getRandomAccessChannel();
      raf.seek(-10);
      OctetSequenceHolder buf = new OctetSequenceHolder(new byte[1]);
      raf.read(1, buf);
      assertEquals('A', buf.value[0]);
      raf.seek(50);
      raf.write(2, new byte[] { '%', '!' });
      raf.close();
      RemoteFile existingFile = root.getChild(new String[] { "A", "B", "C" });
      assertEquals(52, existingFile.size());
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    try {
      OctetSequenceHolder buf = new OctetSequenceHolder(new byte[1]);
      RandomAccessChannel raf = file.getRandomAccessChannel();
      raf.skip(3);
      raf.write(4, new byte[] { 'D', 'E', 'F', 'G' });
      raf.skip(-2);
      raf.read(1, buf);
      assertEquals('F', buf.value[0]);
      raf.skip(-3);
      raf.read(1, buf);
      assertEquals('D', buf.value[0]);
      raf.skip(-50);
      raf.read(1, buf);
      assertEquals('A', buf.value[0]);
      raf.skip(100);
      raf.skip(-1);
      raf.read(1, buf);
      assertEquals('!', buf.value[0]);
      raf.close();
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    try {
      OctetSequenceHolder buf = new OctetSequenceHolder(new byte[1]);
      RandomAccessChannel raf = file.getRandomAccessChannel();
      raf.skip(100);
      int read = raf.read(1, buf);
      assertEquals(0, read);
      raf.skip(-1);
      raf.read(1, buf);
      assertEquals('!', buf.value[0]);
      raf.close();
    }
    catch (Exception exc) {
      fail();
      exc.printStackTrace();
    }

    try {
      RandomAccessChannel raf = file.getRandomAccessChannel();
      boolean isOpen;
      isOpen = raf.isOpen();
      assertEquals(true, isOpen);
      raf.close();
      isOpen = raf.isOpen();
      assertEquals(false, isOpen);
      raf.close(); // close is idempotent, there is no problem to call it
                    // twice.
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    try {
      RandomAccessChannel raf = file.getRandomAccessChannel();
      raf.close();
      RemoteFile fileFromRAF = raf.getFile();
      String[] expectedFullName = file.getFullName();
      String[] actualFullName = fileFromRAF.getFullName();
      assertTrue(assertEquals(expectedFullName, actualFullName));
      assertEquals(52, fileFromRAF.size());
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    try {
      RandomAccessChannel raf = file.getRandomAccessChannel();
      raf.close();
      try {
        raf.skip(1);
        fail();
      }
      catch (ClosedChannelException exc) {
        assertEquals("C", exc.ch.getFile().getName());
      }
      try {
        raf.seek(1);
        fail();
      }
      catch (ClosedChannelException exc) {
        assertEquals("C", exc.ch.getFile().getName());
      }
      try {
        raf.read(0, new OctetSequenceHolder(new byte[0]));
        fail();
      }
      catch (ClosedChannelException exc) {
        assertEquals("C", exc.ch.getFile().getName());
      }
      try {
        raf.write(0, new byte[0]);
        fail();
      }
      catch (ClosedChannelException exc) {
        assertEquals("C", exc.ch.getFile().getName());
      }
    }
    catch (Exception exc) {
      fail();
      exc.printStackTrace();
    }

    try {
      file.remove();
      RandomAccessChannel raf = file.getRandomAccessChannel();
      fail();
    }
    catch (InvalidStateException ex) {
      assertTrue(assertEquals(fileStr, ex.name));
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    String[] dirStr = new String[] { "dir1", "dir2", "dir3", "dirXYZ" };
    try {
      RemoteFile dir = root.createDirectory(dirStr);
      dir.getRandomAccessChannel();
      fail();
    }
    catch (NotFileException ex) {
      assertTrue(assertEquals(dirStr, ex.name));
    }
    catch (Exception exc) {
      fail();
      exc.printStackTrace();
    }

  }

  public void testGetReadChannel() throws InvalidStateException,
    NotFileException, ServerException {

    RemoteFile file = null;
    String alphabet = "ABCDEFGHIJKLMNOPQRSTUWXYZ";
    String[] fileStr = new String[] { "A", "B", "C" };

    try {
      file = root.createFile(fileStr);
      WriteChannel writer = file.getWriteChannel();
      writer.write(alphabet.length(), alphabet.getBytes());
      writer.close();
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    try {
      ReadChannel reader = file.getReadChannel();
      ReadChannel another = file.getReadChannel();
      OctetSequenceHolder buf = new OctetSequenceHolder(new byte[1]);
      reader.read(1, buf);
      assertEquals('A', buf.value[0]);
      reader.read(1, buf);
      assertEquals('B', buf.value[0]);
      reader.read(1, buf);
      assertEquals('C', buf.value[0]);
      reader.skip(5);
      reader.read(1, buf);
      assertEquals('I', buf.value[0]);
      buf.value = new byte[16];
      reader.read(16, buf);
      assertEquals("JKLMNOPQRSTUWXYZ", new String(buf.value));
      another.close();
      reader.close();
    }
    catch (ClosedChannelException ex) {
      fail();
      ex.printStackTrace();
    }

    try {
      ReadChannel reader = file.getReadChannel();
      OctetSequenceHolder buf = new OctetSequenceHolder(new byte[1]);
      reader.read(1, buf);
      assertEquals('A', buf.value[0]);
      reader.read(1, buf);
      assertEquals('B', buf.value[0]);
      reader.skip(-1);
      reader.read(1, buf);
      assertEquals('C', buf.value[0]);
      reader.skip(100);
      int read = reader.read(1, buf);
      assertEquals(0, read);
      reader.close();
    }
    catch (Exception exc) {
      fail();
      exc.printStackTrace();
    }

    try {
      ReadChannel reader = file.getReadChannel();
      boolean isOpen;
      isOpen = reader.isOpen();
      assertEquals(true, isOpen);
      reader.close();
      isOpen = reader.isOpen();
      assertEquals(false, isOpen);
      reader.close(); // close is idempotent, there is no problem to call it
                      // twice.
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    try {
      ReadChannel reader = file.getReadChannel();
      reader.close();
      RemoteFile fileFromRAF = reader.getFile();
      String[] expectedFullName = file.getFullName();
      String[] actualFullName = fileFromRAF.getFullName();
      assertTrue(assertEquals(expectedFullName, actualFullName));
      assertEquals(25, fileFromRAF.size());
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    try {
      ReadChannel reader = file.getReadChannel();
      reader.close();
      try {
        reader.skip(1);
        fail();
      }
      catch (ClosedChannelException exc) {
        assertEquals("C", exc.ch.getFile().getName());
      }
      try {
        reader.read(0, new OctetSequenceHolder(new byte[0]));
        fail();
      }
      catch (ClosedChannelException exc) {
        assertEquals("C", exc.ch.getFile().getName());
      }
    }
    catch (Exception exc) {
      fail();
      exc.printStackTrace();
    }

    try {
      file.remove();
      ReadChannel reader = file.getReadChannel();
      fail();
    }
    catch (InvalidStateException ex) {
      for (int i = 0; i < fileStr.length; i++) {
        assertEquals(fileStr[i], ex.name[i]);
      }
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    String[] dirStr = new String[] { "dir1", "dir2", "dir3", "dirXYZ" };
    try {
      RemoteFile dir = root.createDirectory(dirStr);
      dir.getReadChannel();
      fail();
    }
    catch (NotFileException ex) {
      assertTrue(assertEquals(dirStr, ex.name));
    }
    catch (Exception exc) {
      fail();
      exc.printStackTrace();
    }

  }

  public void testGetWriteChannel() throws InvalidStateException,
    NotFileException, ServerException {

    RemoteFile file = null;
    String alphabet = "ABCDEFGHIJKLMNOPQRSTUWXYZ!";

    String[] fileStr = new String[] { "A", "B", "C" };
    try {
      file = root.createFile(fileStr);
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    try {

      WriteChannel writer = file.getWriteChannel();
      WriteChannel another = file.getWriteChannel();

      for (int i = 0; i < alphabet.length(); i++) {
        writer.write(1, new byte[] { (byte) alphabet.charAt(i) });
      }

      writer.write(alphabet.length(), alphabet.getBytes());

      for (int i = 0; i < alphabet.length(); i = i + 2) {
        writer.write(2, new byte[] { (byte) alphabet.charAt(i),
            (byte) alphabet.charAt(i + 1) });
      }

      another.close();
      writer.close();

      ReadChannel reader = file.getReadChannel();
      OctetSequenceHolder buf = new OctetSequenceHolder(new byte[1]);
      for (int i = 0; i < alphabet.length() * 3; i++) {
        reader.read(1, buf);
        assertEquals((byte) alphabet.charAt(i % alphabet.length()),
          buf.value[0]);
      }
      reader.close();

    }
    catch (ClosedChannelException ex) {
      fail();
      ex.printStackTrace();
    }

    try {
      WriteChannel writer = file.getWriteChannel();
      boolean isOpen;
      isOpen = writer.isOpen();
      assertEquals(true, isOpen);
      writer.close();
      isOpen = writer.isOpen();
      assertEquals(false, isOpen);
      writer.close(); // close is idempotent, there is no problem to call it
                      // twice.
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    try {
      WriteChannel writer = file.getWriteChannel();
      writer.close();
      RemoteFile fileFromRAF = writer.getFile();
      String[] expectedFullName = file.getFullName();
      String[] actualFullName = fileFromRAF.getFullName();
      if (expectedFullName.length == 3) { // as defined by the file name of this
                                          // file, this is just an extra check
                                          // as long as getFullName is also
                                          // tested.
        for (int i = 0; i < expectedFullName.length; i++) {
          assertEquals(expectedFullName[i], actualFullName[i]);
        }
      }
      assertEquals(0, fileFromRAF.size()); // the file was truncated because it
                                            // was opened in writing mode
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    try {
      WriteChannel writer = file.getWriteChannel();
      writer.close();
      try {
        writer.write(0, new byte[0]);
        fail();
      }
      catch (ClosedChannelException exc) {
        assertEquals("C", exc.ch.getFile().getName());
      }
    }
    catch (Exception exc) {
      fail();
      exc.printStackTrace();
    }

    try {
      file.remove();
      WriteChannel writer = file.getWriteChannel();
      fail();
    }
    catch (InvalidStateException ex) {
      assertTrue(assertEquals(fileStr, ex.name));
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    String[] dirStr = new String[] { "dir1", "dir2", "dir3", "dirXYZ" };
    try {
      RemoteFile dir = root.createDirectory(dirStr);
      dir.getWriteChannel();
      fail();
    }
    catch (NotFileException ex) {
      assertTrue(assertEquals(dirStr, ex.name));
    }
    catch (Exception exc) {
      fail();
      exc.printStackTrace();
    }

  }

  public void testHash() throws InvalidStateException {

    RemoteFile file = null;
    String alphabet = "ABCDEFGHIJKLMNOPQRSTUWXYZ";
    String[] fileStr = new String[] { "A", "B", "C" };

    try {
      file = root.createFile(fileStr);
      WriteChannel writer = file.getWriteChannel();
      writer.write(alphabet.length(), alphabet.getBytes());
      writer.close();
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    try {
      MessageDigest digester = MessageDigest.getInstance("MD5");
      digester.update(alphabet.getBytes(), 0, alphabet.length());
      byte[] digest = digester.digest();
      byte[] hash = file.hash();
      for (int i = 0; i < digest.length; i++) {
        assertEquals(digest[i], hash[i]);
      }
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    try {
      WriteChannel writer = file.getWriteChannel();
      writer.close();
      try {
        MessageDigest digester = MessageDigest.getInstance("MD5");
        digester.update(new byte[0], 0, 0);
        byte[] digest = digester.digest();
        byte[] hash = file.hash();
        for (int i = 0; i < digest.length; i++) {
          assertEquals(digest[i], hash[i]);
        }
      }
      catch (NoSuchAlgorithmException ex) {
        ex.printStackTrace();
      }
    }
    catch (Exception exc) {
      fail();
      exc.printStackTrace();
    }

    String[] parentFullName = null;
    try {
      RemoteFile parent = file.getParent();
      parentFullName = parent.getFullName();
      parent.hash();
      fail();
    }
    catch (NotFileException ex) {
      assertTrue(assertEquals(parentFullName, ex.name));
    }
    catch (Exception exc) {
      fail();
      exc.printStackTrace();
    }

    try {
      file.remove();
      byte[] hash = file.hash();
      fail();
    }
    catch (InvalidStateException ex) {
      assertTrue(assertEquals(fileStr, ex.name));
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

  }

  public void testIsDirectory() throws InvalidStateException, ServerException {

    RemoteFile file = null;
    String[] fileStr = new String[] { "A", "B", "C" };

    try {
      file = root.createFile(fileStr);

    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    try {
      assertEquals(false, file.isDirectory());
      assertEquals(true, file.getParent().isDirectory());
    }
    catch (InvalidPathException ex) {
      fail();
      ex.printStackTrace();
    }

    try {
      file.remove();
      file.isDirectory();
      fail();
    }
    catch (InvalidStateException ex) {
      assertTrue(assertEquals(fileStr, ex.name));
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }
  }

  public void testRemove() throws FileInUseException, NotEmptyException,
    InvalidStateException, ServerException {

    RemoteFile file = null;
    String[] fileStr = new String[] { "A", "B", "C" };
    try {
      file = root.createFile(fileStr);
      file.remove();
      file = root.getChild(fileStr);
    }
    catch (FileNotFoundException ex) {
      assertTrue(true);
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    ReadChannel reader = null;
    try {
      file = root.createFile(fileStr);
      reader = file.getReadChannel();
      file.remove();
    }
    catch (FileInUseException ex) {
      reader.close();
      assertTrue(assertEquals(fileStr, ex.name));
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    String[] parentFullName = null;
    try {
      RemoteFile parent = file.getParent();
      parentFullName = parent.getFullName();
      parent.remove();
      fail();
    }
    catch (NotEmptyException ex) {
      assertTrue(assertEquals(parentFullName, ex.name));
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    try {
      file.remove();
      file.remove();
      fail();
    }
    catch (InvalidStateException ex) {
      assertTrue(assertEquals(fileStr, ex.name));
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    try {
      recursiveRemove(root);
      fail();
    }
    catch (FileInUseException ex) {
      assertTrue(assertEquals(new String[] {}, ex.name));
    }
    catch (Exception exc) {
      fail();
      exc.printStackTrace();
    }
  }

  public void testSize() throws InvalidStateException, ServerException {

    RemoteFile file = null;
    String alphabet = "ABCDEFGHIJKLMNOPQRSTUWXYZ";

    String[] fileStr = new String[] { "A", "B", "C" };
    try {
      file = root.createFile(fileStr);
      WriteChannel writer = file.getWriteChannel();
      writer.write(alphabet.length(), alphabet.getBytes());
      writer.close();
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    try {
      assertEquals(alphabet.length(), file.size());
    }
    catch (NotFileException ex) {
      fail();
      ex.printStackTrace();
    }

    try {
      root.size();
      fail();
    }
    catch (NotFileException ex) {
      assertTrue(assertEquals(new String[] {}, ex.name));
    }
  }

  public void testAddMountPoint() throws InvalidStateException,
    NotDirectoryException, FileAlreadyExistsException, InvalidPathException,
    ServerException {

    RemoteFile mountDir = root.createDirectory(new String[] { "A", "B", "C" });
    RemoteFile mountPoint = root.createDirectory(new String[] { "D", "E", "F",
        "G", "H" });
    String[] mntStr = new String[] { "M", "mnt" };
    mountDir.addMountPoint(mntStr, mountPoint);

    String[] realFile = new String[] { "D", "E", "F", "G", "H", "file" };
    try {
      String[] mntDir = new String[] { "A", "B", "C", "M", "mnt" };
      RemoteFile file = root.getChild(mntDir);
      RemoteFile fileInMount = file.createFile(new String[] { "file" });

      String[] fileInMountFullName = fileInMount.getFullName();

      assertTrue(assertEquals(realFile, fileInMountFullName));

    }
    catch (FileNotFoundException ex) {
      fail();
      ex.printStackTrace();
    }

    String[] probe = new String[] { "A", "B", "C", "M", "mnt" };
    try {
      root.createFile(probe);
      fail();
    }
    catch (FileAlreadyExistsException ex) {
      assertTrue(true);
      assertTrue(assertEquals(probe, ex.name));
    }

    try {
      root.createDirectory(probe);
      fail();
    }
    catch (FileAlreadyExistsException ex) {
      assertTrue(true);
      assertTrue(assertEquals(probe, ex.name));
    }

    String[] mountPointStr = mountPoint.getFullName();
    try {
      RemoteFile mnt = root.getChild(probe);
      mnt.remove();
      fail();
    }
    catch (NotEmptyException exc) {
      assertTrue(assertEquals(mountPointStr, exc.name));
    }
    catch (Exception exc) {
      fail();
      exc.printStackTrace();
    }

    try {
      RemoteFile fileInMount = root.getChild(realFile);
      String[] fileInMountFullName = fileInMount.getFullName();
      assertTrue(assertEquals(realFile, fileInMountFullName));
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    /** @todo melhorar isso */
    try {
      mountDir.removeMountPoint(mntStr);
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }

  }

  public void testRemoveMountPoint() throws InvalidStateException,
    NotDirectoryException, FileAlreadyExistsException, InvalidPathException,
    ServerException {

    String[] mntStr = new String[] { "A", "mnt" };
    String[] targetStr = new String[] { "B", "C" };
    RemoteFile target = root.createDirectory(targetStr);
    boolean created = root.addMountPoint(mntStr, target);

    try {
      RemoteFile removed = root.removeMountPoint(mntStr);
      String[] expectedName = target.getFullName();
      String[] actualName = removed.getFullName();
      assertTrue(assertEquals(expectedName, actualName));
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    try {
      root.removeMountPoint(new String[] {});
      fail();
    }
    catch (InvalidPathException ex) {
      assertEquals(new String[] {}, ex.name);
    }
    catch (Exception exc) {
      fail();
      exc.printStackTrace();
    }

    String[] regularFile = new String[] { "false_mount" };
    root.createFile(regularFile);
    try {
      root.removeMountPoint(regularFile);
      fail();
    }
    catch (FileNotFoundException ex) {
      assertTrue(assertEquals(regularFile, ex.name));
    }
  }

  public void testCopyTo_UsingNIO() throws InvalidStateException,
    NotFileException, ServerException {
    copyDirTest("NIO");
    copyFile("NIO");
  }

  public void testCopyTo_UsingCORBA() throws InvalidStateException,
    NotFileException, ServerException {
    copyDirTest("CORBA");
    copyFile("CORBA");
  }

  private void copyDirTest(String method) throws InvalidStateException,
    NotFileException {
    String[] dirStr = new String[] { "dir" };
    try {
      RemoteFile dir = root.createDirectory(dirStr);
      RemoteFile target = root.createDirectory(new String[] { "targetDir" });
      dir.copyTo(target, method);
      fail();
    }
    catch (NotFileException ex) {
      assertTrue(assertEquals(dirStr, ex.name));
    }
    catch (Exception exc) {
      fail();
      exc.printStackTrace();
    }
  }

  private void copyFile(String method) throws InvalidStateException,
    NotFileException, ServerException {

    RemoteFile source = null;
    RemoteFile target1 = null;
    RemoteFile target2 = null;
    RemoteFile target3 = null;
    String alphabet = "ABCDEFGHIJKLMNOPQRSTUWXYZ";

    try {
      source = root.createFile(new String[] { "A", "B", "C" });
      WriteChannel writer = source.getWriteChannel();
      writer.write(alphabet.length(), alphabet.getBytes());
      writer.close();
      target1 = root.createFile(new String[] { "T1" });
      target2 = root.createFile(new String[] { "T2" });
      target3 = root.createFile(new String[] { "T3" });
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    // TODO revisar isso

    // source.getFileServer().setTransferMethod(method);
    // assertEquals(method,source.getFileServer().getTransferMethod());
    //
    // source.getFileServer().setTransferBlockSize(1);
    // assertEquals(1,source.getFileServer().getTransferBlockSize());
    // source.copyTo(target1);
    //
    // source.getFileServer().setTransferBlockSize(6);
    // assertEquals(6,source.getFileServer().getTransferBlockSize());
    // source.copyTo(target2);
    //
    // source.getFileServer().setTransferBlockSize(8192);
    // assertEquals(8192,source.getFileServer().getTransferBlockSize());

    source.copyTo(target3, method);

    byte[] sourceHash = source.hash();
    byte[] target1Hash = target1.hash();
    byte[] target2Hash = target2.hash();
    byte[] target3Hash = target3.hash();

    for (int i = 0; i < sourceHash.length; i++) {
      assertEquals(sourceHash[i], target1Hash[i]);
      assertEquals(sourceHash[i], target2Hash[i]);
      assertEquals(sourceHash[i], target3Hash[i]);
    }

    RemoteFile bigOne = null;
    try {
      bigOne = root.createFile(new String[] { "BigOne" });
      RandomAccessChannel raf = bigOne.getRandomAccessChannel();
      byte[] chunk = new byte[1024];
      byte[] small = new byte[] { 'A' };
      for (int i = 0; i < chunk.length; i++) {
        chunk[i] = (byte) ((i % 10) + '0');
        raf.write(1, small);
      }
      raf.write(chunk.length, chunk);
      raf.close();
      WriteChannel writer = bigOne.getWriteChannel();
      for (int i = 0; i < 1024; i++) {
        writer.write(chunk.length, chunk);
      }
      writer.close();
      assertEquals(1024 * chunk.length, bigOne.size()); // os dados do raf sao
                                                        // sobrescritos
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    RemoteFile bigTarget = null;
    try {
      bigTarget = root.createFile(new String[] { "BigTarget" });
      bigOne.copyTo(bigTarget, method);
    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }

    byte[] bigOneHash = bigOne.hash();
    byte[] bigTargetHash = bigTarget.hash();

    if (bigOneHash.length == bigTargetHash.length) {
      for (int i = 0; i < bigOneHash.length; i++) {
        assertEquals(bigOneHash[i], bigTargetHash[i]);
      }
    }
    else {
      fail();
    }

  }

  public void testLeasing() throws InvalidStateException, NotFileException {

    try {

      RemoteFile fileReadChannel = null;
      fileReadChannel = root.createFile(new String[] { "fileReadChannel" });
      fileReadChannel.getReadChannel();

      RemoteFile fileWriteChannel = null;
      fileWriteChannel = root.createFile(new String[] { "fileWriteChannel" });
      fileWriteChannel.getWriteChannel();

      RemoteFile fileRandomAccessChannel = null;
      fileRandomAccessChannel = root
        .createFile(new String[] { "fileRandomAccessChannel" });
      fileRandomAccessChannel.getRandomAccessChannel();

      int excCount = 0;

      try {
        fileReadChannel.remove();
        fail();
      }
      catch (FileInUseException ex) {
        excCount++;
      }

      try {
        fileWriteChannel.remove();
        fail();
      }
      catch (FileInUseException ex) {
        excCount++;
      }

      try {
        fileRandomAccessChannel.remove();
        fail();
      }
      catch (FileInUseException ex) {
        excCount++;
      }

      assertEquals(3, excCount);

      if (excCount > 0) {
        long time = SystemProperties.getIntProperty("INACTIVITY_LIMIT") * 2;
        System.out
          .println("testLeasing: sleeping for a while: " + time + " ms");
        Thread.sleep(time);
      }

      fileReadChannel.remove();
      fileWriteChannel.remove();
      fileRandomAccessChannel.remove();

    }
    catch (Exception ex) {
      fail();
      ex.printStackTrace();
    }
  }

  public void testGetFileServer() throws InvalidStateException, ServerException {
    assertNotNull(root.getFileServer().getName());
  }

  // public void testMetadata() throws InvalidStateException, ServerException {
  //
  // String[] fileStr = new String[]{"dir1","file"};
  // Metadata[] metadata = new Metadata[3];
  // try {
  // RemoteFile file = root.createFile(fileStr);
  // metadata[0] = new Metadata("FIELD_1","teste".getBytes());
  // metadata[1] = new Metadata("HASH",file.hash());
  // metadata[2] = new Metadata("OLD_SIZE",(""+file.size()).getBytes());
  // file.setMetadata(metadata);
  // } catch (Exception exc) {
  // fail();
  // exc.printStackTrace();
  // }
  //
  // try {
  // RemoteFile file = root.getChild(fileStr);
  // Metadata[] allMetadata = file.getAllMetadata();
  // if (metadata.length == allMetadata.length) {
  // for (int i = 0; i < allMetadata.length; i++) {
  // assertEquals(metadata[i].field,allMetadata[i].field);
  // assertTrue(Arrays.equals(metadata[i].value,allMetadata[i].value));
  // }
  // } else {
  // fail();
  // }
  // } catch (Exception exc) {
  // exc.printStackTrace();
  // fail();
  // }
  //
  // try {
  // RemoteFile file = root.getChild(fileStr);
  //
  // Metadata[] response = null;
  //
  // response = file.getMetadata(new String[]{"FIELD_1"});
  // assertEquals("FIELD_1",response[0].field);
  // assertTrue(Arrays.equals("teste".getBytes(),response[0].value));
  //
  // response = file.getMetadata(new String[]{"HASH"});
  // assertEquals("HASH",response[0].field);
  // assertTrue(Arrays.equals(file.hash(),response[0].value));
  //
  // response = file.getMetadata(new String[]{"OLD_SIZE"});
  // assertEquals("OLD_SIZE",response[0].field);
  // assertTrue(Arrays.equals((""+file.size()).getBytes(),response[0].value));
  //
  //
  // response = file.getMetadata(new String[]{"FIELD_1","OLD_SIZE"});
  // assertEquals("FIELD_1",response[0].field);
  // assertTrue(Arrays.equals("teste".getBytes(),response[0].value));
  // assertEquals("OLD_SIZE",response[1].field);
  // assertTrue(Arrays.equals((""+file.size()).getBytes(),response[1].value));
  //
  // /** add more combinations? */
  //
  // /** intentionally out of order */
  //
  // response = file.getMetadata(new String[]{"FIELD_1","OLD_SIZE","HASH"});
  // assertEquals("FIELD_1",response[0].field);
  // assertTrue(Arrays.equals("teste".getBytes(),response[0].value));
  // assertEquals("OLD_SIZE",response[1].field);
  // assertTrue(Arrays.equals((""+file.size()).getBytes(),response[1].value));
  // assertEquals("HASH",response[2].field);
  // assertTrue(Arrays.equals(file.hash(),response[2].value));
  //
  // } catch (Exception exc) {
  // exc.printStackTrace();
  // fail();
  // }
  //
  // try {
  // RemoteFile file = root.getChild(fileStr);
  // Metadata[] update = new Metadata[2];
  // update[0] = new Metadata("EXTRA_FIELD","extra".getBytes());
  // update[1] = new Metadata("FIELD_1","updated".getBytes());
  // file.updateMetadata(update);
  //
  // file.updateMetadata(new Metadata[0]);
  //
  // } catch (Exception exc) {
  // fail();
  // exc.printStackTrace();
  // }
  //
  //
  // try {
  // RemoteFile file = root.getChild(fileStr);
  // Metadata[] allMetadata = file.getAllMetadata();
  // assertEquals("FIELD_1",allMetadata[0].field);
  // assertTrue(Arrays.equals("updated".getBytes(),allMetadata[0].value));
  // assertEquals("HASH",allMetadata[1].field);
  // assertTrue(Arrays.equals(file.hash(),allMetadata[1].value));
  // assertEquals("OLD_SIZE",allMetadata[2].field);
  // assertTrue(Arrays.equals((""+file.size()).getBytes(),allMetadata[2].value));
  // assertEquals("EXTRA_FIELD",allMetadata[3].field);
  // assertTrue(Arrays.equals("extra".getBytes(),allMetadata[3].value));
  // } catch (Exception exc) {
  // fail();
  // exc.printStackTrace();
  // }
  //
  // try {
  // RemoteFile file = root.getChild(fileStr);
  // file.remove();
  //
  // try {
  // file.setMetadata(new Metadata[0]);
  // } catch (InvalidStateException exc) {
  // assertTrue(assertEquals(fileStr, exc.name));
  // }
  //
  // try {
  // file.getAllMetadata();
  // } catch (InvalidStateException exc) {
  // assertTrue(assertEquals(fileStr, exc.name));
  // }
  //
  // try {
  // file.getMetadata(new String[0]);
  // } catch (InvalidStateException exc) {
  // assertTrue(assertEquals(fileStr, exc.name));
  // }
  //
  // try {
  // file.updateMetadata(new Metadata[0]);
  // } catch (InvalidStateException exc) {
  // assertTrue(assertEquals(fileStr, exc.name));
  // }
  //
  // } catch (Exception exc) {
  // exc.printStackTrace();
  // fail();
  // }
  // }

  public void testShutdown() {
    try {
      root.getFileServer().shutdown();
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }
}
