package csfs.fuse;

import java.nio.ByteBuffer;

import fuse.Filesystem;
import fuse.FuseDirEnt;
import fuse.FuseException;
import fuse.FuseFtype;
import fuse.FuseMount;
import fuse.FuseStat;
import fuse.FuseStatfs;
import fuse.util.Log;

import csfs.impl.util.Util;
import csfs.remote.*;
import java.util.Hashtable;
import java.util.Random;

import org.omg.CORBA.ORB;
import java.util.Properties;

public class FileSystemFuseCSFS implements Filesystem {

    private String mountPoint;
    private RemoteFile root;

    private Hashtable channels = new Hashtable();

    private Random random;
    
    private static final Log log = new Log(FileSystemFuseCSFS.class);

    public FileSystemFuseCSFS (String mountPoint, RemoteFile root) {
        this.mountPoint = mountPoint;
        this.root = root;
        this.random = new Random();
    }

    private String[] csfsPath(String path) {
//        path = path.substring(mountPoint.length());
        return Util.stringToStringArray(path);
    }

    public FuseStat getattr(String path) throws FuseException {
        log.info("getattr" + path);
        try {
            RemoteFile child = root.getChild(csfsPath(path));

            FuseStat stat = new FuseStat();

            stat.mode = child.isDirectory() ? FuseFtype.TYPE_DIR | 0777 :
                        FuseFtype.TYPE_FILE | 0777;
            stat.nlink = 0;
            stat.uid = 0;
            stat.gid = 0;
            stat.size = child.isDirectory() ? 0 : child.size();
            stat.atime = stat.mtime = stat.ctime = (int) (child.lastModified() / 1000L);
            stat.blocks = (int) ((stat.size + 511L) / 512L);
            return stat;

        } catch (InvalidStateException ex) {
            throw new FuseException("Invalid File").initErrno(FuseException.EBADF);
        } catch (InvalidPathException ex) {
            throw new FuseException("Unsuported Path").initErrno(FuseException.ENOENT);
        } catch (NotDirectoryException ex) {
            log.error("root must be a directory");
            ex.printStackTrace();
            throw new FuseException("Remote Server Error").initErrno(FuseException.EFAULT);
        } catch (NotFileException ex) {
            log.error("child has been tested and IS a file");
            ex.printStackTrace();
            throw new FuseException("Remote Server Error").initErrno(FuseException.EFAULT);
        }
        catch (FileNotFoundException ex) {
        	throw new FuseException("FileNotFound").initErrno(FuseException.ENOENT);
        } catch (ServerException ex) {
            throw new FuseException("Remote Server Error").initErrno(FuseException.EFAULT);
        } 
    }

    public String readlink(String path) throws FuseException {
        log.info("WARNING!!! readlink invoked!" + path);
        return path;
    }

    public FuseDirEnt[] getdir(String path) throws FuseException {
        log.info("getdir " + path);
        try {
            RemoteFile[] children = (root.getChild(csfsPath(path))).getChildren();
            FuseDirEnt[] response = new FuseDirEnt[children.length];
            for (int i = 0; i < children.length; i++) {
                response[i] = new FuseDirEnt();
                response[i].name = children[i].getName();
                response[i].mode = children[i].isDirectory()? FuseFtype.TYPE_DIR : FuseFtype.TYPE_FILE;
            }
            return response;
        } catch (InvalidPathException ex) {
            throw new FuseException("Unsuported Path").initErrno(FuseException.ENOENT);
        } catch (FileNotFoundException ex) {
            throw new FuseException("FileNotFound").initErrno(FuseException.ENOENT);
        } catch (InvalidStateException ex) {
            throw new FuseException("Invalid File").initErrno(FuseException.EBADF);
        } catch (ServerException ex) {
            throw new FuseException("Remote Server Error").initErrno(FuseException.EFAULT);
        } catch (NotDirectoryException ex) {
            throw new FuseException("Not a Directory").initErrno(FuseException.ENOTDIR);
        }
    }

    public void mknod(String path, int mode, int rdev) throws FuseException {
    	log.info("mknod " + path);
    	try {
    		if ((mode & FuseFtype.TYPE_DIR)>0) {
    			root.createDirectory(csfsPath(path));
    		} else if ((mode & FuseFtype.TYPE_FILE)>0) {
    			root.createFile(csfsPath(path));
    		} else {
    			throw new FuseException("Support only for files and directories").initErrno(FuseException.EACCES);    			
    		}
        } catch (FileAlreadyExistsException ex) {
            // change file date
        } catch (InvalidPathException ex) {
            throw new FuseException("Unsupported Path").initErrno(FuseException.ENOENT);
        } catch (Exception ex) {
            throw new FuseException("Remote Server Error").initErrno(FuseException.EFAULT);
        }
    }

    public void mkdir(String path, int mode) throws FuseException {
        try {
            root.createDirectory(csfsPath(path));
        } catch (FileAlreadyExistsException ex) {
            throw new FuseException("File Already Exists").initErrno(FuseException.EEXIST);
        } catch (InvalidPathException ex) {
            throw new FuseException("Unsupported Path").initErrno(FuseException.ENOENT);
        } catch (Exception ex) {
            throw new FuseException("Remote Server Error").initErrno(FuseException.EFAULT);
        }
    }

    public void unlink(String path) throws FuseException {
        log.info("unlink" + path);
        try {
            root.getChild(csfsPath(path)).remove();
        } catch (InvalidPathException ex) {
            throw new FuseException("Unsupported Path").initErrno(FuseException.ENOENT);
        } catch (FileNotFoundException ex) {
            throw new FuseException("File Not Found").initErrno(FuseException.ENOENT);
        } catch (InvalidStateException ex) {
            throw new FuseException("Concurrent Access, file probably deleted by another process").initErrno(FuseException.ENOENT);
        } catch (NotEmptyException ex) {
            throw new FuseException("Not Empty!").initErrno(FuseException.ENOTEMPTY);
        } catch (FileInUseException ex) {
            throw new FuseException("Not Empty!").initErrno(FuseException.EBUSY);
        } catch (Exception ex) {
            throw new FuseException("Remote Server Error").initErrno(FuseException.EFAULT);
        }
    }

    public void rmdir(String path) throws FuseException {
        log.info("rmdir" + path);
        try {
            root.getChild(csfsPath(path)).remove();
        } catch (InvalidPathException ex) {
            throw new FuseException("Unsupported Path").initErrno(FuseException.ENOENT);
        } catch (FileNotFoundException ex) {
            throw new FuseException("File Not Found").initErrno(FuseException.ENOENT);
        } catch (InvalidStateException ex) {
            throw new FuseException("Concurrent Access, file probably deleted by another process").initErrno(FuseException.ENOENT);
        } catch (NotEmptyException ex) {
            throw new FuseException("Not Empty!").initErrno(FuseException.ENOTEMPTY);
        } catch (FileInUseException ex) {
            throw new FuseException("Not Empty!").initErrno(FuseException.EBUSY);
        } catch (Exception ex) {
            throw new FuseException("Remote Server Error").initErrno(FuseException.EFAULT);
        }
    }

    public void symlink(String from, String to) throws FuseException {
        log.info("symlink");
        throw new FuseException("Unimplemented").initErrno(FuseException.EACCES);
    }

    public void rename(String from, String to) throws FuseException {
        log.info("rename");
        RemoteFile source;
        RemoteFile target;
        try {
        	try {
        		source = root.getChild(csfsPath(from));
        	} catch (FileNotFoundException ex) {
        		throw new FuseException("File Not Found").initErrno(FuseException.ENOENT);        		
        	}
        	try {
        		target = root.getChild(csfsPath(to));
        		if (target.isDirectory()) {
        			target = target.createFile(new String[]{source.getName()});
        		}
        	} catch (FileNotFoundException ex) {
    			target = root.createFile(csfsPath(to));
            }
        	source.moveTo(target);
        } catch (InvalidPathException ex) {
            throw new FuseException("Unsupported Path").initErrno(FuseException.ENOENT);
        } catch (InvalidStateException ex) {
            throw new FuseException("Concurrent Access, file probably deleted by another process").initErrno(FuseException.ENOENT);
        } catch (FileInUseException ex) {
            throw new FuseException("Permission Denied").initErrno(FuseException.EACCES);
        } catch (Exception ex) {
            throw new FuseException("Remote Server Error").initErrno(FuseException.EFAULT);
        }
    }

    public void link(String from, String to) throws FuseException {
        log.info("link");
        throw new FuseException("Unimplemented").initErrno(FuseException.EACCES);
    }

    public void chmod(String path, int mode) throws FuseException {
        log.info("chmod");
        throw new FuseException("Unimplemented").initErrno(FuseException.EACCES);
    }

    public void chown(String path, int uid, int gid) throws FuseException {
        log.info("chown");
        throw new FuseException("Unimplemented").initErrno(FuseException.EACCES);
    }

    public void truncate(String path, long size) throws FuseException {
        log.info("truncate "+path);
        try {
            root.getChild(csfsPath(path)).truncate(size);
        } catch (InvalidPathException ex) {
            throw new FuseException("Unsupported Path").initErrno(FuseException.ENOENT);
        } catch (FileNotFoundException ex) {
            throw new FuseException("File Not Found").initErrno(FuseException.ENOENT);
        } catch (InvalidStateException ex) {
            throw new FuseException("Concurrent Access, file probably deleted by another process").initErrno(FuseException.ENOENT);
        } catch (FileInUseException ex) {
            throw new FuseException("Not Empty!").initErrno(FuseException.EBUSY);
        } catch (Exception ex) {
            throw new FuseException("Remote Server Error").initErrno(FuseException.EFAULT);
        }
    }

    public void utime(String path, int atime, int mtime) throws FuseException {
        log.info("utime");
//        throw new FuseException("Unimplemented").initErrno(FuseException.EACCES);
    }

    public FuseStatfs statfs() throws FuseException {
        FuseStatfs statfs = new FuseStatfs();
        statfs.blocks = 1024;
        statfs.blockSize = 1024*8;
        try {
            statfs.blocksFree = (int) (root.getFileServer().getFreeSpace() /
                                       65536);
        } catch (ServerException ex) {
            ex.printStackTrace();
            statfs.blocksFree = 1024;
        }
        statfs.files = 16;
        statfs.filesFree = 32;
        statfs.namelen = 128;
        return statfs;
    }

    public long open(String path, int flags) throws FuseException {
        log.info("open" + path);
        long key;
        try {
            RandomAccessChannel channel = root.getChild(csfsPath(path)).
                                          getRandomAccessChannel();
            key = Math.abs(random.nextInt());
            channels.put(new Long(key),channel);
        } catch (InvalidPathException ex) {
            throw new FuseException("Unsupported Path").initErrno(FuseException.ENOENT);
        } catch (FileNotFoundException ex) {
            throw new FuseException("File Not Found").initErrno(FuseException.ENOENT);
        } catch (InvalidStateException ex) {
            throw new FuseException("Concurrent Access, file probably deleted by another process").initErrno(FuseException.ENOENT);
        } catch (Exception ex) {
            throw new FuseException("Remote Server Error").initErrno(FuseException.EFAULT);
        }
        log.info("open produced a key:" + key);        
        return key;
    }

    public void read(String path, long fh, ByteBuffer buf, long offset) throws
            FuseException {
//        log.info("read! got a key:" + fh);
        try {
            RandomAccessChannel channel = (RandomAccessChannel)channels.get(new Long(fh));
            if (channel==null || !channel.isOpen()) {
                channel = root.getChild(csfsPath(path)).
                                          getRandomAccessChannel();
                                channels.put(new Long(fh),channel);
            }
            int size = buf.capacity();
            OctetSequenceHolder holder = new OctetSequenceHolder();
            holder.value = new byte[size];
            channel.seek(offset);
            channel.read(size,holder);
            buf.put(holder.value);
//            channel.close();
        } catch (InvalidPathException ex) {
            throw new FuseException("Unsupported Path").initErrno(FuseException.ENOENT);
        } catch (FileNotFoundException ex) {
            throw new FuseException("File Not Found").initErrno(FuseException.ENOENT);
        } catch (InvalidStateException ex) {
            throw new FuseException("Concurrent Access, file probably deleted by another process").initErrno(FuseException.ENOENT);
        } catch (Exception ex) {
            throw new FuseException("Remote Server Error").initErrno(FuseException.EFAULT);
        }
    }

    public void write(String path, long fh, boolean isWritepage,
                      ByteBuffer buf, long offset) throws FuseException {
//        log.info("write! got a key:" + fh);
        try {
            RandomAccessChannel channel = (RandomAccessChannel)channels.get(new Long(fh));
            if (channel==null || !channel.isOpen()) {
                channel = root.getChild(csfsPath(path)).
                                          getRandomAccessChannel();
                                channels.put(new Long(fh),channel);
            }
            channel.seek(offset);
            int dataSize = buf.capacity();
            byte[] data = new byte[dataSize];
            buf.get(data);
            channel.write(dataSize,data);
//            channel.close();
        } catch (InvalidPathException ex) {
            throw new FuseException("Unsupported Path").initErrno(FuseException.ENOENT);
        } catch (FileNotFoundException ex) {
            throw new FuseException("File Not Found").initErrno(FuseException.ENOENT);
        } catch (InvalidStateException ex) {
            throw new FuseException("Concurrent Access, file probably deleted by another process").initErrno(FuseException.ENOENT);
        } catch (Exception ex) {
            throw new FuseException("Remote Server Error").initErrno(FuseException.EFAULT);
        }
    }

    public void flush(String path, long fh) throws FuseException {
        log.info("flush (nothing to be done), key:"+fh);
    }

    public void release(String path, long fh, int flags) throws FuseException {
        log.info("release, key:"+fh);
        RandomAccessChannel channel = (RandomAccessChannel)channels.get(new Long(fh));
        try {
            if (channel != null && channel.isOpen()) {
                channel.close();
            }
        } catch (Exception ex) {
            throw new FuseException("Remote Server Error").initErrno(FuseException.EFAULT);
        }
        channels.remove(new Long(fh));
    }

    public void fsync(String path, long fh, boolean isDatasync) throws
            FuseException {
        log.info("fsync (nothing to be done), key:"+fh);
    }

    public static void main(String[] args) throws Exception {

        Properties props = new Properties();
        props.put("org.omg.CORBA.ORBClass","org.jacorb.orb.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass","org.jacorb.orb.ORBSingleton");
        ORB orb = ORB.init((String[])null, props);
        String corbaloc = args[args.length - 1];
        org.omg.CORBA.Object obj1 = orb.string_to_object(corbaloc);
        FileServer fileServer1 = FileServerHelper.narrow(obj1);
        RemoteFile root = fileServer1.getRoot();

        if (args.length < 1) {
            System.out.println("Must specify a CORBALOC!");
            System.exit( -1);
        }

        String fuseArgs[] = new String[args.length - 1];
        System.arraycopy(args, 0, fuseArgs, 0, fuseArgs.length);

        log.info("entering");

        try {
            FuseMount.mount(fuseArgs, new FileSystemFuseCSFS(fuseArgs[fuseArgs.length - 1],root));
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            log.info("exiting");
        }
    }

}
