package csfs.fuse;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;

import fuse.Filesystem;
import fuse.FuseDirEnt;
import fuse.FuseException;
import fuse.FuseFtype;
import fuse.FuseMount;
import fuse.FuseStat;
import fuse.FuseStatfs;
import fuse.util.Log;

import csfs.impl.util.Util;
import csfs.remote.*;
import java.util.Hashtable;
import java.util.Random;

import org.omg.CORBA.ORB;
import java.util.Properties;

public class TurboLocalFileSystem implements Filesystem {

    private String rootDir;

    private Hashtable fileSize = new Hashtable();
    private Hashtable availableData = new Hashtable();

    private Random random;
    
    private static final Log log = new Log(FileSystemFuseCSFS.class);
    
    public TurboLocalFileSystem (String rootDir) {
        this.rootDir = rootDir;
        random = new Random();
    }

    private long getSize(File f) {
    	try {
    		BufferedReader reader = new BufferedReader(new FileReader(f));
    		String line = reader.readLine();
    		return Long.parseLong(line);
    	}
    	catch (Exception exc) {
    		exc.printStackTrace();
    		return 0;
    	}
    }

    public FuseStat getattr(String path) throws FuseException {
        log.info("getattr" + path);
        	
    	File file = new File(rootDir+path);

    	if (file.exists()) {
	        FuseStat stat = new FuseStat();
	
	        stat.mode = file.isDirectory() ? FuseFtype.TYPE_DIR | 0777 :
	                    FuseFtype.TYPE_FILE | 0777;
	        stat.nlink = 1;
	        stat.uid = 0;
	        stat.gid = 0;
//	        stat.size = file.isDirectory() ? 0 : getSize(file);
	        stat.size = file.isDirectory() ? 0 : file.length();	        
	        stat.atime = stat.mtime = stat.ctime = (int) (file.lastModified() / 1000L);
	        stat.blocks = (int) ((stat.size + 511L) / 512L);
	        return stat;
    	}
    	else {
    		throw new FuseException("FileNotFound").initErrno(FuseException.ENOENT);
    	}
    }

    public String readlink(String path) throws FuseException {
        log.info("WARNING!!! readlink invoked!" + path);
        return path;
    }

    public FuseDirEnt[] getdir(String path) throws FuseException {
        log.info("getdir " + path);
    	File file = new File(rootDir+path);
        File[] children = file.listFiles();
        FuseDirEnt[] response = new FuseDirEnt[children.length];
        for (int i = 0; i < children.length; i++) {
            response[i] = new FuseDirEnt();
            response[i].name = children[i].getName();
            response[i].mode = children[i].isDirectory() ? FuseFtype.TYPE_DIR : FuseFtype.TYPE_FILE;
        }
        return response;
    }

    public void mknod(String path, int mode, int rdev) throws FuseException {
    	log.info("mknod " + path);
    	File file = new File(rootDir+path);
    	try {
    		if ((mode & FuseFtype.TYPE_DIR)>0) {
    			file.mkdirs();
    		} else if ((mode & FuseFtype.TYPE_FILE)>0) {
//            	file.delete();
//            	RandomAccessFile raf = new RandomAccessFile(file,"rw");
//            	raf.writeBytes(""+0+"\n\n");
//            	raf.close();
    			file.createNewFile();
    		} else {
    			throw new FuseException("Support only for files and directories").initErrno(FuseException.EACCES);    			
    		}
        } catch (IOException ex) {
        	throw new FuseException("Remote Server Error").initErrno(FuseException.EFAULT);
        }
    }

    public void mkdir(String path, int mode) throws FuseException {
    	File file = new File(rootDir+path);
    	if (file.exists()) {
    		throw new FuseException("File Already Exists").initErrno(FuseException.EEXIST);        		
    	} else {
        	file.mkdirs();
    	}
    }

    public void unlink(String path) throws FuseException {
        log.info("unlink" + path);
    	File file = new File(rootDir+path);
    	if (!file.delete()) {
    		throw new FuseException("ERROR").initErrno(FuseException.EBADF);    		
    	};
    }

    public void rmdir(String path) throws FuseException {
        log.info("rmdir" + path);
    	File file = new File(rootDir+path);
    	if (!file.delete()) {
    		throw new FuseException("ERROR").initErrno(FuseException.EBADF);    		
    	};
    }

    public void symlink(String from, String to) throws FuseException {
        log.info("symlink");
        throw new FuseException("Unimplemented").initErrno(FuseException.EACCES);
    }

    public void rename(String from, String to) throws FuseException {
        log.info("rename");
        File fileFrom = new File(rootDir+from);
        File fileTo = new File(rootDir+to);
    	if (!fileFrom.renameTo(fileTo)) {
    		throw new FuseException("ERROR").initErrno(FuseException.EBADF);    		
    	};
    }

    public void link(String from, String to) throws FuseException {
        log.info("link");
        throw new FuseException("Unimplemented").initErrno(FuseException.EACCES);
    }

    public void chmod(String path, int mode) throws FuseException {
        log.info("chmod");
        throw new FuseException("Unimplemented").initErrno(FuseException.EACCES);
    }

    public void chown(String path, int uid, int gid) throws FuseException {
        log.info("chown");
        throw new FuseException("Unimplemented").initErrno(FuseException.EACCES);
    }

    public void truncate(String path, long size) throws FuseException {
        log.info("truncate "+path);
        try {
        	File f = new File(rootDir+path);
//        	f.delete();
//        	RandomAccessFile raf = new RandomAccessFile(f,"rw");
//        	raf.writeBytes(""+size+"\n\n");
//        	raf.close();
        	RandomAccessFile raf = new RandomAccessFile(f,"rw");
        	raf.getChannel().truncate(size);
        } catch (Exception ex) {
            throw new FuseException("Error").initErrno(FuseException.EFAULT);
        }
    }

    public void utime(String path, int atime, int mtime) throws FuseException {
        log.info("utime");
//        throw new FuseException("Unimplemented").initErrno(FuseException.EACCES);
    }

    public FuseStatfs statfs() throws FuseException {
        FuseStatfs statfs = new FuseStatfs();
        statfs.blocks = 1024;
        statfs.blockSize = 1024*8;
        statfs.blocksFree = 1024;
        statfs.files = 16;
        statfs.filesFree = 32;
        statfs.namelen = 128;
        return statfs;
    }

    long id = 0;
    
    public long open(String path, int flags) throws FuseException {
        log.info("open" + path);
        long key = -1;
        try {
        	File file = new File(rootDir+path);
        	if (file.isFile() || !file.exists()) {
	            key = Math.abs(random.nextInt());
//	            fileSize.put(new Long(key),new Long(getSize(file)));
//	            availableData.put(new Long(key),new Long(getSize(file)));
	            RandomAccessFile raf = new RandomAccessFile(file,"rw"); 
	            fileSize.put(new Long(key),raf);
        	}
        } catch (Exception ex) {
            throw new FuseException("ERROR").initErrno(FuseException.EFAULT);
        }
        log.info("open produced a key:" + key);        
        return key;
    }

    public void read(String path, long fh, ByteBuffer buf, long offset) throws
            FuseException {
        try {
//            long avaiableDataValue = ((Long)availableData.get(new Long(fh))).longValue();
//            while (offset < avaiableDataValue) {
//            	buf.put((byte)'0');
//            	avaiableDataValue--;
//            }
//            availableData.put(new Long(fh),new Long(avaiableDataValue));
        	RandomAccessFile raf = (RandomAccessFile)fileSize.get(new Long(fh));
        	FileChannel ch = raf.getChannel();
        	ch.position(offset);
        	int read = 0;
        	while (offset + read < raf.length() && buf.position() < buf.capacity()) {
        		read = read + ch.read(buf); 
        	}
        	buf.rewind();
        } catch (Exception ex) {
            throw new FuseException("Remote Server Error").initErrno(FuseException.EFAULT);
        }
    }

    public void write(String path, long fh, boolean isWritepage,
                      ByteBuffer buf, long offset) throws FuseException {
        try {
//            long avaiableDataValue = ((Long)fileSize.get(new Long(fh))).longValue();
//            avaiableDataValue = avaiableDataValue + buf.capacity() - 1;
//            fileSize.put(new Long(fh),new Long(avaiableDataValue));
        	RandomAccessFile raf = (RandomAccessFile)fileSize.get(new Long(fh));
        	FileChannel ch = raf.getChannel();
        	ch.position(offset);
        	buf.rewind();
        	while (buf.position()<buf.limit()) {
        		ch.write(buf);	
        	}
        } catch (Exception ex) {
            throw new FuseException("Remote Server Error").initErrno(FuseException.EFAULT);
        }
    }

    public void flush(String path, long fh) throws FuseException {
    	log.info("flush, key:"+fh);
        try {
//            long avaiableDataValue = ((Long)fileSize.get(new Long(fh))).longValue();
//        	File f = new File(rootDir+path);
//        	f.delete();
//        	RandomAccessFile raf = new RandomAccessFile(f,"rw");
//        	raf.writeBytes(""+avaiableDataValue+"\n\n");
//        	raf.close();
        	RandomAccessFile raf = (RandomAccessFile)fileSize.get(new Long(fh));
        	if (raf!=null) {
        		raf.close();	
        	}
        	fileSize.remove(new Long(fh));
        } catch (Exception ex) {
            throw new FuseException("Remote Server Error").initErrno(FuseException.EFAULT);
        }
    }

    public void release(String path, long fh, int flags) throws FuseException {
        log.info("release, key:"+fh);
        try {
//          long avaiableDataValue = ((Long)fileSize.get(new Long(fh))).longValue();
//        	File f = new File(rootDir+path);
//        	f.delete();
//        	RandomAccessFile raf = new RandomAccessFile(f,"rw");
//        	raf.writeBytes(""+avaiableDataValue+"\n\n");
//        	raf.close();
        	RandomAccessFile raf = (RandomAccessFile)fileSize.get(new Long(fh));
        	if (raf!=null) {
        		raf.close();	
        	}
        	fileSize.remove(new Long(fh));
        } catch (Exception ex) {
        	ex.printStackTrace();
            throw new FuseException("Remote Server Error").initErrno(FuseException.EFAULT);
        }
    }

    public void fsync(String path, long fh, boolean isDatasync) throws
            FuseException {
        log.info("fsync (nothing to be done), key:"+fh);
    }

    public static void main(String[] args) throws Exception {

    	System.out.println("java.library.path="+System.getProperty("java.library.path"));
    	String[] fuseArgs = new String[] {"-f", "-s", "/tmp/fuse2"};
    	try {
            FuseMount.mount(fuseArgs, new TurboLocalFileSystem("/tmp/turbo_fs"));
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            log.info("exiting");
        }
    }

}
