/**
 *   FUSE-J: Java bindings for FUSE (Filesystem in Userspace by Miklos Szeredi (mszeredi@inf.bme.hu))
 *
 *   Copyright (C) 2003 Peter Levart (peter@select-tech.si)
 *
 *   This program can be distributed under the terms of the GNU LGPL.
 *   See the file COPYING.LIB
 */

#include "javafs.h"

//
// JVM allocation/deallocation

static JavaVM *vm;
static JNIEnv *mainEnv;

static jclass_fuse_FuseStat *FuseStat;
static jclass_fuse_FuseFSDirEnt *FuseFSDirEnt;
static jclass_fuse_FuseStatfs *FuseStatfs;
static jclass_fuse_FuseException *FuseException;
static jclass_java_nio_ByteBuffer *ByteBuffer;
static jclass_fuse_FuseFS *FuseFS;

static jobject fuseFS;
static jobject threadGroup;


static void free_fuseFS(JNIEnv *env)
{
   if (fuseFS != NULL) { (*env)->DeleteGlobalRef(env, fuseFS); fuseFS = NULL; }
}


static int retain_fuseFS(JNIEnv *env, jobject util)
{
   fuseFS = (*env)->NewGlobalRef(env, util);

   if ((*env)->ExceptionCheck(env))
   {
      (*env)->ExceptionDescribe(env);
      return 0;
   }

   return 1;
}


static void free_threadGroup(JNIEnv *env)
{
   if (threadGroup != NULL) { (*env)->DeleteGlobalRef(env, threadGroup); threadGroup = NULL; }
}


static int retain_threadGroup(JNIEnv *env, jobject util)
{
   threadGroup = (*env)->NewGlobalRef(env, util);

   if ((*env)->ExceptionCheck(env))
   {
      (*env)->ExceptionDescribe(env);
      return 0;
   }

   return 1;
}


static int alloc_filesystem(JNIEnv *env, char *filesystemClassName)
{
   jclass fsClass = NULL;
   jobject util = NULL;
   jmethodID fsConstructorID;

   while (1)
   {
      fsClass = (*env)->FindClass(env, filesystemClassName);
      if ((*env)->ExceptionCheck(env)) break;

      fsConstructorID = (*env)->GetMethodID(env, fsClass, "<init>", "()V");
      if ((*env)->ExceptionCheck(env)) break;

      util = (*env)->NewObject(env, fsClass, fsConstructorID);
      if ((*env)->ExceptionCheck(env)) break;

      fuseFS = (*env)->NewGlobalRef(env, util);
      if ((*env)->ExceptionCheck(env)) break;

      return 1; /* success */
   }

   // error handler

   if ((*env)->ExceptionCheck(env))
   {
      (*env)->ExceptionDescribe(env);
   }

   free_fuseFS(env);

   if (util != NULL) (*env)->DeleteLocalRef(env, util);
   if (fsClass != NULL) (*env)->DeleteLocalRef(env, fsClass);

   return 0;
}


static void free_classes(JNIEnv *env)
{
   if (FuseStat != NULL)      { free_jclass_fuse_FuseStat(env, FuseStat);           FuseStat = NULL; }
   if (FuseFSDirEnt != NULL)    { free_jclass_fuse_FuseFSDirEnt(env, FuseFSDirEnt);       FuseFSDirEnt = NULL; }
   if (FuseStatfs != NULL)    { free_jclass_fuse_FuseStatfs(env, FuseStatfs);       FuseStatfs = NULL; }
   if (FuseException != NULL) { free_jclass_fuse_FuseException(env, FuseException); FuseException = NULL; }
   if (ByteBuffer != NULL)    { free_jclass_java_nio_ByteBuffer(env, ByteBuffer);   ByteBuffer = NULL; }
   if (FuseFS != NULL)        { free_jclass_fuse_FuseFS(env, FuseFS);               FuseFS = NULL; }
}


static int alloc_classes(JNIEnv *env)
{
   while (1)
   {
      if (!(FuseStat = alloc_jclass_fuse_FuseStat(env))) break;
      if (!(FuseFSDirEnt = alloc_jclass_fuse_FuseFSDirEnt(env))) break;
      if (!(FuseStatfs = alloc_jclass_fuse_FuseStatfs(env))) break;
      if (!(FuseException = alloc_jclass_fuse_FuseException(env))) break;
      if (!(ByteBuffer = alloc_jclass_java_nio_ByteBuffer(env))) break;
      if (!(FuseFS = alloc_jclass_fuse_FuseFS(env))) break;

      return 1;
   }

   // error handler

   if ((*env)->ExceptionCheck(env))
   {
      (*env)->ExceptionDescribe(env);
   }

   free_classes(env);

   return 0;
}


static void free_JVM(JNIEnv *env)
{
   if (vm != NULL)
   {
      (*vm)->DestroyJavaVM(vm);
      vm = NULL;
      mainEnv = NULL;
   }
}


static JNIEnv *alloc_JVM(int argc, char *argv[])
{
   JavaVMInitArgs vm_args;
   int n = argc + 3;
   JavaVMOption options[n];
   int i;
   jint res;

   for (i = 0; i < argc; i++)
      options[i].optionString = argv[i];

   /* options[i++].optionString = "-verbose:jni";                        print JNI-related messages */
   options[i++].optionString = "-Xint";
   options[i++].optionString = "-Xrs";
   options[i++].optionString = "-Xcheck:jni";

   vm_args.version = JNI_VERSION_1_4;
   vm_args.options = options;
   vm_args.nOptions = n;
   vm_args.ignoreUnrecognized = 0;

   res = JNI_CreateJavaVM(&vm, (void **)&mainEnv, &vm_args);
   if (res < 0)
   {
      WARN("Can't create Java VM");
      return NULL;
   }

   TRACE("created JVM @ %p", vm);

   return mainEnv;
}


//
// attaching/detachnig current thread to/from JVM

static JNIEnv *get_env()
{
   JNIEnv *env;
   JavaVMAttachArgs args;

   args.version = JNI_VERSION_1_4;
   args.name = NULL;
   args.group = threadGroup;

   TRACE("will attach thread");

   // attach thread as daemon thread so that JVM can exit after unmounting the fuseFS
   (*vm)->AttachCurrentThreadAsDaemon(vm, (void**)&env, (void*)&args);

   TRACE("did attach thread to env: %p", env);

   return env;
}

static void release_env(JNIEnv *env)
{
   if (env == mainEnv)
   {
      TRACE("will NOT detach main thread from env: %p", env);
   }
   else
   {
      TRACE("will NOT detach thread from env: %p", env);

      // Currently native threads are attached to JVM as daemon threads so we don't need to
      // detach them at return from a Java method call. It is in fact beter not to detach them
      // since then every new Java call would need to atach new Java Thread until max. number of
      // Java threads is exhausted.
      //
      //TRACE("will detach thread from env: %p", env);
      //(*vm)->DetachCurrentThread(vm);
      //TRACE("did detach thread");
   }
}

static int exception_check_errno(JNIEnv *env)
{
   int my_errno = 0;
   jthrowable jException = NULL;

   while ((*env)->ExceptionCheck(env))
   {
      jException = (*env)->ExceptionOccurred(env);

      // Patch by: Daniel Wunsch <the.gray@gmx.net>
      // the Exception has to be cleared before calling any methods on FuseException
      (*env)->ExceptionDescribe(env);
      (*env)->ExceptionClear(env);
      
      if ((*env)->IsInstanceOf(env, jException, FuseException->class))
      {
         my_errno = (*env)->CallIntMethod(env, jException, FuseException->method.getErrno);
      }
      else
      {
         my_errno = ECANCELED;
      }

      break;
   }

   // cleanup

   if (jException != NULL) (*env)->DeleteLocalRef(env, jException);

   release_env(env);
   return -my_errno;
}


//
// javafs API functions

static int javafs_getattr(const char *path, struct stat *stbuf)
{
   JNIEnv *env = get_env();
   jobject jPath = NULL;
   jobject jStat = NULL;

   while (1)
   {
      jPath = (*env)->NewDirectByteBuffer(env, (void *)path, (jlong)strlen(path));
      if ((*env)->ExceptionCheck(env)) break;

      jStat = (*env)->CallObjectMethod(env, fuseFS, FuseFS->method.getattr__Ljava_nio_ByteBuffer_, jPath);
      if ((*env)->ExceptionCheck(env)) break;

      stbuf->st_mode =   (mode_t)((*env)->GetIntField(env, jStat, FuseStat->field.mode));
      stbuf->st_nlink =  (nlink_t)((*env)->GetIntField(env, jStat, FuseStat->field.nlink));
      stbuf->st_uid =    (uid_t)((*env)->GetIntField(env, jStat, FuseStat->field.uid));
      stbuf->st_gid =    (gid_t)((*env)->GetIntField(env, jStat, FuseStat->field.gid));
      stbuf->st_size =   (off_t)((*env)->GetLongField(env, jStat, FuseStat->field.size));
      stbuf->st_atime =  (time_t)((*env)->GetIntField(env, jStat, FuseStat->field.atime));
      stbuf->st_mtime =  (time_t)((*env)->GetIntField(env, jStat, FuseStat->field.mtime));
      stbuf->st_ctime =  (time_t)((*env)->GetIntField(env, jStat, FuseStat->field.ctime));
      stbuf->st_blocks = (blkcnt_t)((*env)->GetIntField(env, jStat, FuseStat->field.blocks));
      break;
   }

   // cleanup

   if (jPath != NULL) (*env)->DeleteLocalRef(env, jPath);
   if (jStat != NULL) (*env)->DeleteLocalRef(env, jStat);

   return exception_check_errno(env);
}


static int javafs_readlink(const char *path, char *buf, size_t size)
{
   JNIEnv *env = get_env();
   jobject jPath = NULL;
   jobject jLink = NULL;
   jint jLinkPosition;

   while (1)
   {
      jPath = (*env)->NewDirectByteBuffer(env, (void *)path, (jlong)strlen(path));
      if ((*env)->ExceptionCheck(env)) break;

      jLink = (*env)->NewDirectByteBuffer(env, buf, (jlong)(size - 1));
      if ((*env)->ExceptionCheck(env)) break;

      (*env)->CallVoidMethod(env, fuseFS, FuseFS->method.readlink__Ljava_nio_ByteBuffer_Ljava_nio_ByteBuffer_, jPath, jLink);
      if ((*env)->ExceptionCheck(env)) break;

      // write a cstring terminator at the end of writen data
      jLinkPosition = (*env)->CallIntMethod(env, jLink, ByteBuffer->method.position);
      buf[jLinkPosition] = '\0';

      break;
   }

   // cleanup

   if (jPath != NULL) (*env)->DeleteLocalRef(env, jPath);
   if (jLink != NULL) (*env)->DeleteLocalRef(env, jLink);

   return exception_check_errno(env);
}


static int javafs_getdir(const char *path, fuse_dirh_t h, fuse_dirfil_t filler)
{
   JNIEnv *env = get_env();
   jobject jPath = NULL;
   jobjectArray jDirEntArray = NULL;
   jobject jDirEnt = NULL;
   jbyteArray jName = NULL;
   jbyte *jNameBytes = NULL;
   jsize jNameLength;
   jint mode, inode;
   jsize i;
   jsize n;
   int res1 = 0;
   int res2;

   while (1)
   {
      jPath = (*env)->NewDirectByteBuffer(env, (void *)path, (jlong)strlen(path));
      if ((*env)->ExceptionCheck(env)) break;

      jDirEntArray = (*env)->CallObjectMethod(env, fuseFS, FuseFS->method.getdir__Ljava_nio_ByteBuffer_, jPath);
      if ((*env)->ExceptionCheck(env)) break;

      n = (*env)->GetArrayLength(env, jDirEntArray);
      for (i = 0; i < n; i++)
      {
         char name[MAX_GETDIR_NAME_LENGTH];
         int nameLength = MAX_GETDIR_NAME_LENGTH - 1;

         jDirEnt = (*env)->GetObjectArrayElement(env, jDirEntArray, i);
         if ((*env)->ExceptionCheck(env)) break;

         mode = (*env)->GetIntField(env, jDirEnt, FuseFSDirEnt->field.mode);
         if ((*env)->ExceptionCheck(env)) break;

         jName = (*env)->GetObjectField(env, jDirEnt, FuseFSDirEnt->field.name);
         if ((*env)->ExceptionCheck(env)) break;

         inode = (*env)->GetIntField(env, jDirEnt, FuseFSDirEnt->field.inode);
         if ((*env)->ExceptionCheck(env)) break;

         jNameBytes = (*env)->GetByteArrayElements(env, jName, NULL);
         if ((*env)->ExceptionCheck(env)) break;

         jNameLength = (*env)->GetArrayLength(env, jName);

         if (nameLength > (int)jNameLength)
            nameLength = (int)jNameLength;

         memcpy(name, jNameBytes, nameLength);
         name[nameLength] = '\0';

         res1 = filler(h, name, IFTODT(mode), inode);
         if(res1 != 0) break;

         (*env)->ReleaseByteArrayElements(env, jName, jNameBytes, JNI_ABORT); jNameBytes = NULL;
         (*env)->DeleteLocalRef(env, jName); jName = NULL;
         (*env)->DeleteLocalRef(env, jDirEnt); jDirEnt = NULL;
      }

      break;
   }

   // cleanup

   if (jNameBytes != NULL) (*env)->ReleaseByteArrayElements(env, jName, jNameBytes, JNI_ABORT);
   if (jName != NULL) (*env)->DeleteLocalRef(env, jName);
   if (jDirEnt != NULL) (*env)->DeleteLocalRef(env, jDirEnt);
   if (jDirEntArray != NULL) (*env)->DeleteLocalRef(env, jDirEntArray);
   if (jPath != NULL) (*env)->DeleteLocalRef(env, jPath);

   res2 = exception_check_errno(env);
   return (res1 == 0)? res2 : res1;
}


static int javafs_mknod(const char *path, mode_t mode, dev_t rdev)
{
   JNIEnv *env = get_env();
   jobject jPath = NULL;

   while (1)
   {
      jPath = (*env)->NewDirectByteBuffer(env, (void *)path, (jlong)strlen(path));
      if ((*env)->ExceptionCheck(env)) break;

      (*env)->CallVoidMethod(env, fuseFS, FuseFS->method.mknod__Ljava_nio_ByteBuffer_II, jPath, (jint)mode, (jint)rdev);
      break;
   }

   // cleanup

   if (jPath != NULL) (*env)->DeleteLocalRef(env, jPath);

   return exception_check_errno(env);
}


static int javafs_mkdir(const char *path, mode_t mode)
{
   JNIEnv *env = get_env();
   jobject jPath = NULL;

   while (1)
   {
      jPath = (*env)->NewDirectByteBuffer(env, (void *)path, (jlong)strlen(path));
      if ((*env)->ExceptionCheck(env)) break;

      (*env)->CallVoidMethod(env, fuseFS, FuseFS->method.mkdir__Ljava_nio_ByteBuffer_I, jPath, (jint)mode);
      break;
   }

   // cleanup

   if (jPath != NULL) (*env)->DeleteLocalRef(env, jPath);

   return exception_check_errno(env);
}


static int javafs_unlink(const char *path)
{
   JNIEnv *env = get_env();
   jobject jPath = NULL;

   while (1)
   {
      jPath = (*env)->NewDirectByteBuffer(env, (void *)path, (jlong)strlen(path));
      if ((*env)->ExceptionCheck(env)) break;

      (*env)->CallVoidMethod(env, fuseFS, FuseFS->method.unlink__Ljava_nio_ByteBuffer_, jPath);
      break;
   }

   // cleanup

   if (jPath != NULL) (*env)->DeleteLocalRef(env, jPath);

   return exception_check_errno(env);
}


static int javafs_rmdir(const char *path)
{
   JNIEnv *env = get_env();
   jobject jPath = NULL;

   while (1)
   {
      jPath = (*env)->NewDirectByteBuffer(env, (void *)path, (jlong)strlen(path));
      if ((*env)->ExceptionCheck(env)) break;

      (*env)->CallVoidMethod(env, fuseFS, FuseFS->method.rmdir__Ljava_nio_ByteBuffer_, jPath);
      break;
   }

   // cleanup

   if (jPath != NULL) (*env)->DeleteLocalRef(env, jPath);

   return exception_check_errno(env);
}


static int javafs_symlink(const char *from, const char *to)
{
   JNIEnv *env = get_env();
   jobject jFrom = NULL;
   jobject jTo = NULL;

   while (1)
   {
      jFrom = (*env)->NewDirectByteBuffer(env, (void *)from, (jlong)strlen(from));
      if ((*env)->ExceptionCheck(env)) break;

      jTo = (*env)->NewDirectByteBuffer(env, (void *)to, (jlong)strlen(to));
      if ((*env)->ExceptionCheck(env)) break;

      (*env)->CallVoidMethod(env, fuseFS, FuseFS->method.symlink__Ljava_nio_ByteBuffer_Ljava_nio_ByteBuffer_, jFrom, jTo);
      break;
   }

   // cleanup

   if (jTo != NULL) (*env)->DeleteLocalRef(env, jTo);
   if (jFrom != NULL) (*env)->DeleteLocalRef(env, jFrom);

   return exception_check_errno(env);
}


static int javafs_rename(const char *from, const char *to)
{
   JNIEnv *env = get_env();
   jobject jFrom = NULL;
   jobject jTo = NULL;

   while (1)
   {
      jFrom = (*env)->NewDirectByteBuffer(env, (void *)from, (jlong)strlen(from));
      if ((*env)->ExceptionCheck(env)) break;

      jTo = (*env)->NewDirectByteBuffer(env, (void *)to, (jlong)strlen(to));
      if ((*env)->ExceptionCheck(env)) break;

      (*env)->CallVoidMethod(env, fuseFS, FuseFS->method.rename__Ljava_nio_ByteBuffer_Ljava_nio_ByteBuffer_, jFrom, jTo);
      break;
   }

   // cleanup

   if (jTo != NULL) (*env)->DeleteLocalRef(env, jTo);
   if (jFrom != NULL) (*env)->DeleteLocalRef(env, jFrom);

   return exception_check_errno(env);
}


static int javafs_link(const char *from, const char *to)
{
   JNIEnv *env = get_env();
   jobject jFrom = NULL;
   jobject jTo = NULL;

   while (1)
   {
      jFrom = (*env)->NewDirectByteBuffer(env, (void *)from, (jlong)strlen(from));
      if ((*env)->ExceptionCheck(env)) break;

      jTo = (*env)->NewDirectByteBuffer(env, (void *)to, (jlong)strlen(to));
      if ((*env)->ExceptionCheck(env)) break;

      (*env)->CallVoidMethod(env, fuseFS, FuseFS->method.link__Ljava_nio_ByteBuffer_Ljava_nio_ByteBuffer_, jFrom, jTo);
      break;
   }

   // cleanup

   if (jTo != NULL) (*env)->DeleteLocalRef(env, jTo);
   if (jFrom != NULL) (*env)->DeleteLocalRef(env, jFrom);

   return exception_check_errno(env);
}


static int javafs_chmod(const char *path, mode_t mode)
{
   JNIEnv *env = get_env();
   jobject jPath = NULL;

   while (1)
   {
      jPath = (*env)->NewDirectByteBuffer(env, (void *)path, (jlong)strlen(path));
      if ((*env)->ExceptionCheck(env)) break;

      (*env)->CallVoidMethod(env, fuseFS, FuseFS->method.chmod__Ljava_nio_ByteBuffer_I, jPath, (jint)mode);
      break;
   }

   // cleanup

   if (jPath != NULL) (*env)->DeleteLocalRef(env, jPath);

   return exception_check_errno(env);
}


static int javafs_chown(const char *path, uid_t uid, gid_t gid)
{
   JNIEnv *env = get_env();
   jobject jPath = NULL;

   while (1)
   {
      jPath = (*env)->NewDirectByteBuffer(env, (void *)path, (jlong)strlen(path));
      if ((*env)->ExceptionCheck(env)) break;

      (*env)->CallVoidMethod(env, fuseFS, FuseFS->method.chown__Ljava_nio_ByteBuffer_II, jPath, (jint)uid, (jint)gid);
      break;
   }

   // cleanup

   if (jPath != NULL) (*env)->DeleteLocalRef(env, jPath);

   return exception_check_errno(env);
}


static int javafs_truncate(const char *path, off_t size)
{
   JNIEnv *env = get_env();
   jobject jPath = NULL;

   while (1)
   {
      jPath = (*env)->NewDirectByteBuffer(env, (void *)path, (jlong)strlen(path));
      if ((*env)->ExceptionCheck(env)) break;

      (*env)->CallVoidMethod(env, fuseFS, FuseFS->method.truncate__Ljava_nio_ByteBuffer_J, jPath, (jlong)size);
      break;
   }

   // cleanup

   if (jPath != NULL) (*env)->DeleteLocalRef(env, jPath);

   return exception_check_errno(env);
}


static int javafs_utime(const char *path, struct utimbuf *buf)
{
   JNIEnv *env = get_env();
   jobject jPath = NULL;

   while (1)
   {
      jPath = (*env)->NewDirectByteBuffer(env, (void *)path, (jlong)strlen(path));
      if ((*env)->ExceptionCheck(env)) break;

      (*env)->CallVoidMethod(env, fuseFS, FuseFS->method.utime__Ljava_nio_ByteBuffer_II, jPath, (jint)(buf->actime), (jint)(buf->modtime));
      break;
   }

   // cleanup

   if (jPath != NULL) (*env)->DeleteLocalRef(env, jPath);

   return exception_check_errno(env);
}


static int javafs_open(const char *path, struct fuse_file_info *ffi)
{
   JNIEnv *env = get_env();
   jobject jPath = NULL;

   while (1)
   {
      jPath = (*env)->NewDirectByteBuffer(env, (void *)path, (jlong)strlen(path));
      if ((*env)->ExceptionCheck(env)) break;

      ffi->fh = (*env)->CallLongMethod(env, fuseFS, FuseFS->method.open__Ljava_nio_ByteBuffer_I, jPath, (jint)(ffi->flags));
      break;
   }

   // cleanup

   if (jPath != NULL) (*env)->DeleteLocalRef(env, jPath);

   return exception_check_errno(env);
}


static int javafs_read(const char *path, char *buf, size_t size, off_t offset, struct fuse_file_info *ffi)
{
   JNIEnv *env = get_env();
   jobject jPath = NULL;
   jobject jBuf = NULL;
   int res1 = 0;
   int res2;

   while (1)
   {
      jPath = (*env)->NewDirectByteBuffer(env, (void *)path, (jlong)strlen(path));
      if ((*env)->ExceptionCheck(env)) break;

      jBuf = (*env)->NewDirectByteBuffer(env, buf, (jlong)size);
      if ((*env)->ExceptionCheck(env)) break;

      (*env)->CallVoidMethod(env, fuseFS, FuseFS->method.read__Ljava_nio_ByteBuffer_JLjava_nio_ByteBuffer_J, jPath, (jlong)(ffi->fh), jBuf, (jlong)offset);
      if ((*env)->ExceptionCheck(env)) break;

      // to obtain # of bytes read, get current position from ByteBuffer
      res1 = (*env)->CallIntMethod(env, jBuf, ByteBuffer->method.position);
      break;
   }

   // cleanup

   if (jBuf != NULL) (*env)->DeleteLocalRef(env, jBuf);
   if (jPath != NULL) (*env)->DeleteLocalRef(env, jPath);

   res2 = exception_check_errno(env);
   return (res2 == 0)? res1 : res2;
}


static int javafs_write(const char *path, const char *buf, size_t size, off_t offset, struct fuse_file_info *ffi)
{
   JNIEnv *env = get_env();
   jobject jPath = NULL;
   jobject jBuf = NULL;
   int res1 = 0;
   int res2;

   while (1)
   {
      jPath = (*env)->NewDirectByteBuffer(env, (void *)path, (jlong)strlen(path));
      if ((*env)->ExceptionCheck(env)) break;

      jBuf = (*env)->NewDirectByteBuffer(env, (void *)buf, (jlong)size);
      if ((*env)->ExceptionCheck(env)) break;

      (*env)->CallVoidMethod(env, fuseFS, FuseFS->method.write__Ljava_nio_ByteBuffer_JZLjava_nio_ByteBuffer_J, jPath, (jlong)(ffi->fh), (ffi->writepage)? JNI_TRUE : JNI_FALSE, jBuf, (jlong)offset);
      if ((*env)->ExceptionCheck(env)) break;

      // to obtain # of bytes writen, get current position from ByteBuffer
      res1 = (*env)->CallIntMethod(env, jBuf, ByteBuffer->method.position);
      break;
   }

   // cleanup

   if (jBuf != NULL) (*env)->DeleteLocalRef(env, jBuf);
   if (jPath != NULL) (*env)->DeleteLocalRef(env, jPath);

   res2 = exception_check_errno(env);
   return (res2 == 0)? res1 : res2;
}


static int javafs_statfs( const char *path, struct statfs *fst)
{
   JNIEnv *env = get_env();
   jobject jStatfs = NULL;

   while (1)
   {
      jStatfs = (*env)->CallObjectMethod(env, fuseFS, FuseFS->method.statfs);
      if ((*env)->ExceptionCheck(env)) break;

      fst->f_bsize   = (*env)->GetIntField(env, jStatfs, FuseStatfs->field.blockSize);
      fst->f_blocks  = (*env)->GetIntField(env, jStatfs, FuseStatfs->field.blocks);
      fst->f_bfree   = (*env)->GetIntField(env, jStatfs, FuseStatfs->field.blocksFree);
      fst->f_files   = (*env)->GetIntField(env, jStatfs, FuseStatfs->field.files);
      fst->f_ffree   = (*env)->GetIntField(env, jStatfs, FuseStatfs->field.filesFree);
      fst->f_namelen = (*env)->GetIntField(env, jStatfs, FuseStatfs->field.namelen);
      break;
   }

   // cleanup

   if (jStatfs != NULL) (*env)->DeleteLocalRef(env, jStatfs);

   return exception_check_errno(env);
}


static int javafs_flush(const char *path, struct fuse_file_info *ffi)
{
   JNIEnv *env = get_env();
   jobject jPath = NULL;

   while (1)
   {
      jPath = (*env)->NewDirectByteBuffer(env, (void *)path, (jlong)strlen(path));
      if ((*env)->ExceptionCheck(env)) break;

      (*env)->CallVoidMethod(env, fuseFS, FuseFS->method.flush__Ljava_nio_ByteBuffer_J, jPath, (jlong)(ffi->fh));
      break;
   }

   // cleanup

   if (jPath != NULL) (*env)->DeleteLocalRef(env, jPath);

   return exception_check_errno(env);
}


static int javafs_release(const char *path, struct fuse_file_info *ffi)
{
   JNIEnv *env = get_env();
   jobject jPath = NULL;

   while (1)
   {
      jPath = (*env)->NewDirectByteBuffer(env, (void *)path, (jlong)strlen(path));
      if ((*env)->ExceptionCheck(env)) break;

      (*env)->CallVoidMethod(env, fuseFS, FuseFS->method.release__Ljava_nio_ByteBuffer_JI, jPath, (jlong)(ffi->fh), (jint)(ffi->flags));
      break;
   }

   // cleanup

   if (jPath != NULL) (*env)->DeleteLocalRef(env, jPath);

   return exception_check_errno(env);
}


static int javafs_fsync(const char *path, int datasync, struct fuse_file_info *ffi)
{
   JNIEnv *env = get_env();
   jobject jPath = NULL;

   while (1)
   {
      jPath = (*env)->NewDirectByteBuffer(env, (void *)path, (jlong)strlen(path));
      if ((*env)->ExceptionCheck(env)) break;

      (*env)->CallVoidMethod(env, fuseFS, FuseFS->method.fsync__Ljava_nio_ByteBuffer_JZ, jPath, (jlong)(ffi->fh), (jint)(ffi->flags), datasync? JNI_TRUE : JNI_FALSE);
      break;
   }

   // cleanup

   if (jPath != NULL) (*env)->DeleteLocalRef(env, jPath);

   return exception_check_errno(env);
}


static struct fuse_operations javafs_oper = {
   getattr:    javafs_getattr,
   readlink:   javafs_readlink,
   getdir:     javafs_getdir,
   mknod:      javafs_mknod,
   mkdir:      javafs_mkdir,
   symlink:    javafs_symlink,
   unlink:     javafs_unlink,
   rmdir:      javafs_rmdir,
   rename:     javafs_rename,
   link:       javafs_link,
   chmod:      javafs_chmod,
   chown:      javafs_chown,
   truncate:   javafs_truncate,
   utime:      javafs_utime,
   open:       javafs_open,
   read:       javafs_read,
   write:      javafs_write,
   statfs:     javafs_statfs,
   flush:      javafs_flush,
   release:    javafs_release,
   fsync:      javafs_fsync,
   // extended attributes are not implemented yet
   setxattr:    NULL,
   getxattr:    NULL,
   listxattr:   NULL,
   removexattr: NULL
};


//
// command line boot-up

int main(int argc, char *argv[])
{
   char *fuseArgv[argc];
   char *javaArgv[argc];
   char *filesystemClassName = NULL;
   int fuseArgc = 0;
   int javaArgc = 0;
   int i;
   JNIEnv *env;

   // split args into fuse & java args
   for (i = 0; i < argc; i++)
   {
      char *arg = argv[i];
      if (!strncmp(arg, "-C", 2))
         filesystemClassName = &(arg[2]);
      else if (!strncmp(arg, "-J", 2))
         javaArgv[javaArgc++] = &(arg[2]);
      else
         fuseArgv[fuseArgc++] = arg;
   }

   if (filesystemClassName == NULL)
   {
      printf("Missing option: -Cfuse.FuseFSClassName\n");
      return -1;
   }

   printf("%d fuse arguments:", fuseArgc);
   for (i = 0; i < fuseArgc; i++)
      printf(" %s", fuseArgv[i]);
   printf("\n");

   printf("%d java arguments:", javaArgc);
   for (i = 0; i < javaArgc; i++)
      printf(" %s", javaArgv[i]);
   printf("\n");

   printf("Java fuseFS%s\n", filesystemClassName);

   if ((env = alloc_JVM(javaArgc, javaArgv)) != NULL)
   {
      if (alloc_classes(env))
      {
         if (alloc_filesystem(env, filesystemClassName))
         {
            // main loop
            fuse_main(fuseArgc, fuseArgv, &javafs_oper);

            // cleanup
            free_fuseFS(env);
         }

         // cleanup
         free_classes(env);
      }

      // cleanup

      if ((*env)->ExceptionCheck(env))
         (*env)->ExceptionClear(env);

      free_JVM(env);
   }

   return 0;
}


//
// JNI boot-up

void Java_fuse_FuseMount_mount(JNIEnv *env, jclass class, jobjectArray jArgs, jobject jFuseFS, jobject jThreadGroup)
{
   if (!((*env)->GetJavaVM(env, &vm)))
   {
      mainEnv = env;
      int i;
      int n = (*env)->GetArrayLength(env, jArgs);
      int fuseArgc = n + 1;
      char *fuseArgv[fuseArgc];

      // fake 1st arbument to be the name of executable
      fuseArgv[0] = "javafs";

      // convert String[] jArgs -> char *fuseArgv[];
      for (i = 0; i < n; i++)
      {
         jstring jArg = (*env)->GetObjectArrayElement(env, jArgs, i);
         const char *arg = (*env)->GetStringUTFChars(env, jArg, NULL);
         char *fuseArg = (char *)malloc(strlen(arg) + 1);
         strcpy(fuseArg, arg);
         (*env)->ReleaseStringUTFChars(env, jArg, arg);
         (*env)->DeleteLocalRef(env, jArg);

         fuseArgv[i + 1] = fuseArg;
      }

      /*
      printf("%d fuse arguments:", fuseArgc);
      for (i = 0; i < fuseArgc; i++)
         printf(" %s", fuseArgv[i]);
      printf("\n");
      */

      if (alloc_classes(env))
      {
         if (retain_fuseFS(env, jFuseFS))
         {
            if (retain_threadGroup(env, jThreadGroup))
            {
               // main loop
               fuse_main(fuseArgc, fuseArgv, &javafs_oper);

               // cleanup
               free_threadGroup(env);
            }

            // cleanup
            free_fuseFS(env);
         }

         // cleanup
         free_classes(env);
      }

      // free char *fuseArgv[] strings
      for (i = 1; i < fuseArgc; i++)
      {
         free(fuseArgv[i]);
      }

      vm = NULL;
      mainEnv = NULL;
   }
}
