/*
 * CSGridServer.java
 */
package csgrid.server;

import csbase.logic.ServerNotificationData;
import csbase.logic.ServerURI;
import csbase.server.Server;
import csbase.server.ServerException;
import csbase.server.ServiceManager;
import csbase.server.services.administrationservice.AdministrationService;
import csbase.server.services.algorithmservice.AlgorithmService;
import csbase.server.services.applicationservice.ApplicationService;
import csbase.server.services.commandpersistenceservice.CommandPersistenceService;
import csbase.server.services.csfsservice.CSFSService;
import csbase.server.services.diskusageservice.DiskUsageService;
import csbase.server.services.eventlogservice.EventLogService;
import csbase.server.services.httpservice.HttpService;
import csbase.server.services.killserverservice.KillServerService;
import csbase.server.services.languageservice.LanguageService;
import csbase.server.services.loginservice.LoginService;
import csbase.server.services.mailservice.MailService;
import csbase.server.services.notificationservice.NotificationService;
import csbase.server.services.openbusservice.OpenBusService;
import csbase.server.services.opendreamsservice.OpenDreamsService;
import csbase.server.services.projectservice.ProjectService;
import csbase.server.services.publicationservice.PublicationService;
import csbase.server.services.schedulerservice.SchedulerService;
import csbase.server.services.sgaservice.SGAService;
import csbase.server.services.sharedobjectservice.SharedObjectService;
import csbase.server.services.trustedserverservice.TrustedServerService;
import csbase.server.services.wioservice.WIOService;

/**
 * A classe <code>CSGridServer</code> representa o servidor.
 */
public class CSGridServer extends Server {

  /**
   * Constri o servidor do CSGrid.
   * 
   * @param args argymentos.
   * @throws ServerException em caso de falha.
   */
  public CSGridServer(final String[] args) throws ServerException {
    super(args);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void createServices() throws ServerException {
    try {
      // LanguageService deve ser o primeiro, para garantir que
      // todos
      // os servios possam obter uma referncia para o servio de
      // internacionalizao na sua criao.
      LanguageService.createService();
      AdministrationService.createService();
      AlgorithmService.createService();
      ApplicationService.createService();
      EventLogService.createService();
      HttpService.createService();
      KillServerService.createService();
      MailService.createService();
      NotificationService.createService();
      PublicationService.createService();
      CSFSService.createService();
      SGAService.createService();
      SchedulerService.createService();
      WIOService.createService();
      TrustedServerService.createService();
      SharedObjectService.createService();
      CommandPersistenceService.createService();
      // O Servio OpenBus precisa ser iniciado antes do ProjectService e do OpenDreams
      OpenBusService.createService();
      ProjectService.createService();
      OpenDreamsService.createService();
      // O servio de monitorao de disco deve ser criado aps o servio de
      // projetos.
      DiskUsageService.createService();

      // NewService.createService();
      // LoginService deve ser o ltimo, para garantir que quem faz login
      // com sucesso obtm o conjunto completo de servios.
      LoginService.createService();
      System.out.println("Servios iniciados...");
    }
    catch (Exception e) {
      throw new ServerException("Erro na criao do servio", e);
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void startCentralServerMonitor() {
    ServerURI sURI = ServerURI.parse(getCentralServerName());
    if (sURI == null)
      throw new IllegalArgumentException("URI do servidor central invlida");
    new CentralServerMonitor(sURI);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void postInitialization() {
    notifyStartToAllUsers();
  }

  /**
   * Notificao de start.
   */
  private void notifyStartToAllUsers() {
    try {
      ServiceManager srvManager = ServiceManager.getInstance();
      if (srvManager == null) {
        throw new csbase.server.ServerException(
          "Gerente de Servios fora do ar");
      }
      NotificationService notSrv = NotificationService.getInstance();
      if (notSrv == null) {
        throw new csbase.server.ServerException(
          "Servio de notificao fora do ar");
      }
      notSrv.notifyAllUsers("CSGridServer", new ServerNotificationData(true));
    }
    catch (Exception e) {
      String errMsg = "Falha de notificao na inicializao!";
      logSevereMessage(errMsg, e);
    }
  }

  /**
   * Inicia o servidor do CSGrid.
   * 
   * @param args No est sendo utilizado.
   */
  public static void main(String[] args) {
    try {
      CSGridServer server = new CSGridServer(args);
      boolean success = server.start();
      if (!success) {
        System.out.println("No foi possvel iniciar o servidor."
          + "\nPor favor consulte o log para maiores detalhes.");
        return;
      }
      ServiceManager srvManager = ServiceManager.getInstance();
      if (srvManager == null) {
        System.out.println("Gerente de Servios fora do ar");
      }
    }
    catch (Exception e) {
      e.printStackTrace();
      System.exit(1);
    }
  }
}
