require "global"
require "getopt"
require "csglobal" 
require "strutil" 
require "cscfg"

-- --------------------------------------------------
-- constantes

-- regex para validao das tags (sem o prefixo do projeto)
local TAG_VALIDATION_REGEX = "_v%d+_%d%d_%d%d_%d%d%d%d_%d%d_%d%d$"

-- regex para validao dos branches (sem o prefixo do projeto)
local BRANCH_VALIDATION_REGEX = "_v%d+_%d%d_%d%d%d%d_%d%d_%d%d$"

-- regex para validao de branches de acordo com o padro antigo
local OLDSTYLE_BRANCH_VALIDATION_REGEX = "_%d%d%d%d_%d%d_%d%d_v%d+_%d%d"

-- regex para validao de branches de acordo com o padro antigo
local OLDSTYLE_TAG_VALIDATION_REGEX = "_%d%d%d%d_%d%d_%d%d_v%d+_%d%d_%d%d"

-- expresso regular para extrair os nmeros de verso (major e minor) a
-- partir do label de um branch
local BRANCH_VERSION_REGEX = "%a+_v(%d)_(%d%d)"

-- expresso regular para extrair os nmeros de verso (major, minor e
-- patch) a partir do label de uma tag
local TAG_VERSION_REGEX = BRANCH_VERSION_REGEX.."_(%d%d)"

-- regex para extrair o ID de um branch (p.ex. MLM_v1_06_2008_03_18)
local BRANCH_ID_REGEX = "(%a+_v%d_%d%d_%d%d%d%d_%d%d_%d%d)"

-- data corrente em um formato que pode ser aproveitado
local TODAY = os.date("%Y_%m_%d")

-- --------------------------------------------------
-- Funes locais

-- Obtm os nmeros de verso (major e minor) de um branch/tag a partir do
-- seu identificador.
local function _getVersion(id, regex)
    local _, _, major, minor, patch = string.find(id, regex)
    return major, minor, patch
end

-- remove uma tag da tabela local
local function _removeTag(tag)
  SVN.existingTags[tag] = nil
  for i, t in ipairs(SVN.existingTags) do
    if tag == t then
      table.remove(SVN.existingTags, i)
      break
    end
  end
end

-- obtem o identificador para a nova tag do branch em questo
local function _askForTag(branch)
    local newTag = SVN:getNextTag(branch)
    newTag = ask("Entre com o identificador da tag", newTag)
    abort(SVN:isValidTag(newTag), [[
Nome de tag invalido. O nome da tag deve estar no seguinte formato:

PRJ_vJJ_NN_TT_AAAA_MM_DD

Onde:
PRJ  : e o prefixo do projeto definido no atributo PRJ_PREFIX do arquivo
       de configuracoes do csmake (csmake.cfg)
JJ   : representa o major do numero da versao (um ou mais digitos)
NN   : representa o minor do numero da versao (dois digitos)
TT   : representa o patch da versao (dois digitos)
AAAA : representa o ano (quatro digitos)
MM   : representa o mes (dois digitos)
DD   : representa o dia (dois digitos)

Exemplos:
WS_v1_00_00_2006_08_10 : tag do WebSintesi (WS) gerado no dia 10/08/2006
    para representar a versao 1.0.0 do sistema
CS_v1_04_01_2006_07_24 : branch do CSBase (CS) gerado no dia 24/07/2006
    para representar a versao 1.4.1 do sistema
]])
    return newTag
end

-- obtem o identificador para um novo branch; sugere como identificador
-- o branch seguinte ao ltimo (minor version + 1)
-- force : aceita qualquer entrada do usurio (no valida com relao ao
-- padro)
local function _askForBranch(force)
    local newBranch = ask("Entre com o identificador do branch",
        SVN:getNextBranch())
    if not force then
        abort(SVN:isValidBranch(newBranch), [[
Nome de branch invalido. O nome do branch deve estar no seguinte formato:

PRJ_vJJ_NN_AAAA_MM_DD

Onde:
PRJ  : e o prefixo do projeto definido no atributo PRJ_PREFIX do arquivo
       de configuracoes do csmake (csmake.cfg)
JJ   : representa o major do numero da versao (um ou mais digitos)
NN   : representa o minor do numero da versao (dois digitos)
AAAA : representa o ano (quatro digitos)
MM   : representa o mes (dois digitos)
DD   : representa o dia (dois digitos)

Exemplos:
WS_v1_00_2006_08_10 : branch do WebSintesi (WS) gerado no dia 10/08/2006
    para representar a versao 1.0 do sistema
CS_v1_04_2006_07_24 : branch do CSBase (CS) gerado no dia 24/07/2006 para
    representar a versao 1.4 do sistema
]])
    end
    return newBranch
end

-- Atualiza o arquivo de verso do CSBase, em uma URL qualquer, com o valor da
-- tag recebida.
local function _updateVersionFile(url, tag, message)
  local tmpDir = "/tmp"
  local resourcesDir = "resources"
  local versionFile = "version.properties"

  local tmpResourcesDir = tmpDir.."/"..resourcesDir

  -- Limpa checkout anterior (pode no existir)
  os.execute("rm -rf "..tmpResourcesDir)

  SVN:resetParams()
  SVN:setCmdParam(url.."/"..resourcesDir)
  SVN:runCommand("checkout", tmpDir, false)

  -- Atualiza o arquivo de verso
  local file, errorMsg = io.open(tmpResourcesDir.."/"..versionFile, "w")
  if not file then
    print("Arquivo de versao nao encontrado - "..errorMsg)
    return
  end

  if not tag then
    tag = ""
  end
  file:write("version="..tag.."\n")
  file:close()

  SVN:resetParams()
  SVN:setCmdParam ("-m", "\""..message.."\"")
  SVN:runCommand("commit", tmpResourcesDir, false)
end

-- Obtm o identificador de uma tag a partir do prefixo do projeto,
-- major version, minor version, patch number e uma data. Caso a data
-- no seja fornecida, usa a data corrente.
local function _getTagId(prj, major, minor, patch, date)
    return string.format("%s_v%d_%02d_%02d_%s", prj,
        tonumber(major), tonumber(minor), tonumber(patch), date or TODAY)
end

-- Obtm o identificador de um branch a partir do prefixo do projeto,
-- major version, minor version e uma data. Caso a data no seja
-- fornecida, usa a data corrente.
local function _getBranchId(prj, major, minor, date)
    return string.format("%s_v%d_%02d_%s", prj,
        tonumber(major), tonumber(minor), date or TODAY)
end

-- Executa o comando que cria ou remove uma tag
-- tag : tag a ser removida/criada
-- branch : branch de onde ser criada a tag
-- message : mensagem para o commit (opcional)
-- delete : flag indicando se a tag ser criada (false) ou removida
--          (true)
-- revision : reviso especfica para a criao da tag (opcional)
local function _execTagCmd (tag, branch, message, delete, revision)
    for _, repository in ipairs (ConfigFile:getRepositories()) do
        print()
        SVN:tag(tag, branch, repository, nil, delete, revision)
    end
end

-- --------------------------------------------------
-- classe SVN

-- 'SVN' deixou de ser local para poder ser utilizada por outros modulos
SVN = OO:new {
    cmdparams = ParamList:new(),
    saved = {},
}

-- reseta parmetros do svn e do comando
function SVN:resetParams ()
    self.cmdparams:reset()
end

-- salva estado atual dos parmetros
function SVN:saveParams ()
    self.cmdparams:save()
end

-- recupera estado previamente salvo
function SVN:restoreParams ()
    self.cmdparams:restore()
end

function SVN:setCmdParam (param, val)
    self.cmdparams:set(param, val)
end

-- executa comando. Assume que os parmetros foram devidamente
-- configurados via setCmdParam()
-- cmd (str) : comando
-- dir (str) : dretrio aonde ser executado comando
-- capture (bool) : define se a sada do comando deve ser capturada (via
--     popen)
function SVN:runCommand (cmd, dir, capture)
    local command, output
    command = string.format("(cd %s; svn %s %s 2>&1)", dir,
        cmd, self.cmdparams:toString())
    cs_log:debug(command)
    if capture then
        output = io.popen(command)
    else
        os.execute(command)
    end
    return output
end

function SVN:list (url)
    SVN:saveParams()

    SVN:resetParams()
    SVN:setCmdParam(url)

    local output = self:runCommand("list", ".", true)

    SVN:restoreParams()

    return output
end

function SVN:copy (sourceUrl, targetUrl, message, revision)
    SVN:saveParams()

    SVN:resetParams()
    SVN:setCmdParam(sourceUrl)
    SVN:setCmdParam(targetUrl)
    SVN:setCmdParam ("-m", "\""..message.."\"")
    if (revision) then
        SVN:setCmdParam ("-r", revision) 
    end

    local out = self:runCommand("copy", ".", true)

    SVN:restoreParams()

    local output = ""
    for l in out:lines() do
        output = string.format("%s%s\n", output, l)
    end
    return output
end

function SVN:delete (url, message)
    SVN:saveParams()
    SVN:resetParams()
    SVN:setCmdParam(url)
    SVN:setCmdParam ("-m", "\""..message.."\"")
    self:runCommand("delete", ".", false)
    SVN:restoreParams()
end

function SVN:info ()
    SVN:saveParams()
    SVN:resetParams()
    local output = self:runCommand("info", ".", true)
    SVN:restoreParams()
    return output
end

-- recupera um determinado mdulo. IMPORTANTE: assume que o diretrio
-- passado como parmetro existe
-- Parmetros opcionais:
--   module = o mdulo que ser recuperado
--   dir = aonde ser feito o checkout
function SVN:checkout (module, dir)
    if not dir then
        cs_log:warn("fazendo checkout no diretorio corrente")
        dir = "."
    end

    local urlPrefix, location = SVN:getRepositoriesInfo()
    local repURL = urlPrefix.."/"..module.rep.name
    local rep = repURL.."/"..location.."/"..module.module

    local logmsg = string.format("diretorio: %-20s repositorio: %s", dir, rep)
    cs_log:info(logmsg)
    SVN:resetParams()
    SVN:setCmdParam(rep)
    self:runCommand("checkout", dir, false)
end

function SVN:status (dir)
    local output = self:runCommand("status", dir, true)

    local result = {
        C = { type = "CONFLITOS no resolvidos" },
        A = { type = "Adicionados (commit pendente)" },
        D = { type = "Removidos (commit pendente)" },
        M = { type = "Modificados" },
        R = { type = "Substitudos (commit pendente)" },
        ["?"] = { type = "Locais" },

        -- LEGENDA:
        -- C : conflitos
        -- A : adicionados ao SVN, commit pendente
        -- D : removidos do SVN, commit pendente
        -- M : arquivos modificados
        -- R : arquivos que foram apagados e novos arquivos de mesmo nome esto sendo inseridos em seus lugares
        -- ? : arquivos locais
    }

    local errorMsg = {}
    for l in output:lines() do
        local _, c = string.gsub(l, "^([CADMR?])[M ][L ][+ ][S ].-(%S+)$", function (op, file)
            if not result[op] then
                cs_log:error("resultado nao-previsto: "..op)
            end
            table.insert(result[op], file)
        end)
        if c == 0 then
            local _, _, revision = string.find (l, ".-(%d+).*$")
            if ( revision ) then
                moduleRevision = revision
            else 
                table.insert(errorMsg, l)
            end
        end
    end
    if table.getn(errorMsg) ~= 0 then
        io.write("\nERROS:\n")
        for _, l in ipairs (errorMsg) do
            io.write(l.."\n")
        end
        print()
    else
        local conflicts = false
        -- se houve conflitos, exibe-os
        if table.getn(result.C) > 0 then
            cs_log:warn("este update gerou conflitos. Digite ENTER para ver a lista dos arquivos: ")
            io.read()
            conflicts = true
        end
        if ( moduleRevision ) then
            print(string.format("Numero de revisao da copia de trabalho : %d\n", moduleRevision))
        end
        -- imprime os demais resultados _sem paginao_
        for _, r in ipairs { "C", "A", "D", "M", "R", "?" } do
            print(string.format("=== %-30s [%s] : %d", result[r].type, r,
                table.getn(result[r])))
            for _, l in ipairs(result[r]) do
                print(r, l)
            end
            if conflicts then
                io.write("\ndigite ENTER para prosseguir: ")
                io.read()
                conflicts = false
                print()
            end
        end
    end
end

-- atualiza diretrio
function SVN:update (dir)
    -- sempre trazemos novos diretrios
    local output = self:runCommand("update", dir, true)
    local result = {
        C = { type = "CONFLITOS" },
        U = { type = "Atualizados" },
        A = { type = "Adicionados" },
        D = { type = "Removidos" },
        R = { type = "Substitudos" },
        G = { type = "Merged" },

        -- LEGENDA:
        -- C : conflitos
        -- U : arquivos atualizados a partir do SVN
        -- A : adicionados ao SVN e recuperados para a copia local
        -- D : removidos do SVN e removidos da copia local
        -- R : arquivos que foram apagados e novos arquivos de mesmo nome esto sendo inseridos em seus lugares na copia local
        -- G : arquivos que sofreram merge
    }

    local errorMsg = {}
    for l in output:lines() do
        local _, c = string.gsub(l, "^([CUADRG])%s+(.+)$", function (op, file)
            if not result[op] then
                cs_log:error("resultado nao-previsto: "..op)
            end
            table.insert(result[op], file)
        end)
        if c == 0 then
            local _, _, revision = string.find (l, ".-(%d+).*$")
            if ( revision ) then
                moduleRevision = revision
            else
                table.insert(errorMsg, l)
            end
        end
    end
    if table.getn(errorMsg) ~= 0 then
        io.write("\nERROS:\n")
        for _, l in ipairs (errorMsg) do
            io.write(l.."\n")
        end
        print()
    else
        local conflicts = false
        -- se houve conflitos, exibe-os
        if table.getn(result.C) > 0 then
            cs_log:warn("este update gerou conflitos. Digite ENTER para ver a lista dos arquivos: ")
            io.read()
            conflicts = true
        end
        if ( moduleRevision ) then
            print(string.format("Numero de revisao da copia de trabalho : %d\n", moduleRevision))
        end
        -- imprime os demais resultados _sem paginao_
        for _, r in ipairs { "C", "U", "A", "D", "R", "G" } do
            print(string.format("=== %-15s [%s] : %d", result[r].type, r,
                table.getn(result[r])))
            for _, l in ipairs(result[r]) do
                print(r, l)
            end
            if conflicts then
                io.write("\ndigite ENTER para prosseguir: ")
                io.read()
                conflicts = false
                print()
            end
        end
    end
end

-- cria uma cpia para representar uma tag de um branch. Essa tag serve
-- como um snapshot de um branch
function SVN:tag (tag, branch, repository, message, delete, revision)
    local action = ""
    if delete then
        action = "removendo"
    else
        action = "criando"
    end
    message = message or action.." tag "..tag
    cs_log:info(message.." em >>> "..repository.name)
    local repURL = SVN:getRepositoriesInfo().."/"..repository.name
    local url = repURL.."/"..repository.tags.."/"..tag
    if delete then
        SVN:delete (url, message)
        _removeTag(tag)
    else
        local sourceUrl = repURL.."/"..repository.branches.."/"..branch
        local output = SVN:copy (sourceUrl, url, message, revision)
        print(output)
    end
end

function SVN:_getTags ()
    self.existingTags = {}
    self.oldStyleTags = {}
    if self.existingTags.wasLoaded then
        return
    end
    local repository = ConfigFile:getMainModule().rep
    local repURL = SVN:getRepositoriesInfo().."/"..repository.name
    local tmp = SVN:list (repURL.."/"..repository.tags)
    for l in tmp:lines() do
        string.gsub(l, "^(.+)/$", function (t)
            if SVN:isValidTag(t) then
                self.existingTags[t] = true
                table.insert(self.existingTags, t)
            elseif SVN:isValidTag(t, OLDSTYLE_TAG_VALIDATION_REGEX) then
                self.oldStyleTags[t] = true
                table.insert(self.oldStyleTags, t)
            end
        end)
    end
    self.existingTags.wasLoaded = true
end

-- valida uma tag: retorna true caso ela exista, nil em caso contrrio
function SVN:checkTag (tag)
    self:_getTags()
    return tag and (self.existingTags[tag] or self.oldStyleTags[tag])
end

-- iterador sobre a lista de tags existentes
function SVN:tagIterator ()
    self:_getTags()
    return coroutine.wrap(function ()
        for _, t in ipairs(self.oldStyleTags) do
            coroutine.yield(t)
        end
        if SVN:hasOldStyleTags() and SVN:hasNewStyleTags() then
            coroutine.yield("-----------------------")
        end
        for _, t in ipairs(self.existingTags) do
            coroutine.yield(t)
        end
    end)
end

-- verifica se um nome eh um nome de tag valido
function SVN:isValidTag(tag, tagValidationRegex)
    tagValidationRegex = tagValidationRegex or TAG_VALIDATION_REGEX
    local PRJ_PREFIX = ConfigFile:getVar("PRJ_PREFIX")
    local firstIndex = string.find(tag, "^" .. PRJ_PREFIX .. tagValidationRegex)
    if firstIndex == nil then
	    return false
    end
    return true
end

-- iterador sobre a lista de tags existentes
function SVN:branchIterator ()
    self:_getBranches()
    return coroutine.wrap(function ()
        for _, b in ipairs(self.oldStyleBranches) do
            coroutine.yield(b)
        end
        if SVN:hasOldStyleBranches() and SVN:hasNewStyleBranches() then
            coroutine.yield("--------------------")
        end
        for _, b in ipairs(self.existingBranches) do
            coroutine.yield(b)
        end
    end)
end

-- sugere o nome da prxima tag associada a um branch. Verifica se
-- j existem tags associadas ao branch; se houver, incrementa o 'patch
-- number'; seno, devolve uma nova tag baseada no branch fornecido, com
-- patch number = 0
function SVN:getNextTag(branch)
    self:_getTags()
    local major, minor = _getVersion(branch, BRANCH_VERSION_REGEX)
    abort(major and minor, "identificador invalido para branch: "..branch)
    -- precisamos verificar se j existe um tag correspondente ao
    -- branch; se existir, pegamos seu patch number e incrementamos.
    -- Seno, sugeriremos o nmero 0
    local PRJ_PREFIX = ConfigFile:getVar("PRJ_PREFIX")
    local versionPrefix = string.format("%s_v%d_%02d", PRJ_PREFIX, major, minor)
    local versionPrefixRegex = versionPrefix .. "_(%d%d)"
    local releaseNumber = 0
    -- percorremos as tags existentes da ltima para a primeira. A
    -- primeira ocorrncia associada ao branch que queremos determinar
    -- o identificador do prximo tag
    for i = table.getn(self.existingTags), 1, -1 do
        local tag = self.existingTags[i]
        local _, _, number = string.find (tag, versionPrefixRegex)
        if number and (tonumber(number) >= releaseNumber) then
            releaseNumber = number + 1
            break
        end
    end
    return _getTagId(PRJ_PREFIX, major, minor, releaseNumber)
end

-- retorna true se existem tags no formato antigo (comeando com
-- PRJ_YYYY_MM_DD...)
function SVN:hasOldStyleTags()
    self:_getTags()
    return table.getn(self.oldStyleTags) > 0
end

function SVN:hasNewStyleTags()
    self:_getTags()
    return table.getn(self.existingTags) > 0
end

-- obtem a ultima tag gerada
function SVN:getLastTag(branch)
    local major, minor = _getVersion(branch, BRANCH_VERSION_REGEX)
    self:_getTags()
    local tags = {}
    for tag in pairs (self.existingTags) do
      if string.find(tag, string.format("v%d_%s", major, minor)) then
        table.insert (tags, tag)
      end
    end
    table.sortIgnoreCase (tags)
    return tags[table.getn(tags)]
end

-- cria um branch
function SVN:branch (branch, repository, message, delete, revision)
    local action = "aplicando"
    if delete then
        action = "removendo"
    end
    if not message then
        message = action.." branch "..branch
    end
    cs_log:info(action.." branch "..branch.." em >>> "..repository.name)
    local repURL = SVN:getRepositoriesInfo().."/"..repository.name
    local url = repURL.."/"..repository.branches.."/"..branch
    if (delete) then
        SVN:delete (url, message)
    else
        local sourceUrl = repURL.."/"..repository.trunk
        local output = SVN:copy(sourceUrl, url, message, revision)
        print(output)
    end
end

function SVN:_getBranches ()
    self.existingBranches = {}
    self.oldStyleBranches = {}
    if self.existingBranches.wasLoaded then
        return
    end
    local repository = ConfigFile:getMainModule().rep
    local repURL = SVN:getRepositoriesInfo().."/"..repository.name
    local tmp = SVN:list (repURL.."/"..repository.branches)
    for l in tmp:lines() do
        string.gsub(l, "^(.+)/$", function (b)
            if SVN:isValidBranch(b) then
                self.existingBranches[b] = true
                table.insert(self.existingBranches, b)
            elseif SVN:isValidBranch(b, OLDSTYLE_BRANCH_VALIDATION_REGEX) then
                self.oldStyleBranches[b] = true
                table.insert(self.oldStyleBranches, b)
            end
        end)
    end
    self.existingBranches.wasLoaded = true
end

-- obtem o ultimo branch gerado
function SVN:getLastBranch()
    self:_getBranches()
    local branches = {}
    for _, value in ipairs (self.existingBranches) do
        table.insert (branches, value)
    end
    table.sortIgnoreCase (branches)
    return branches[table.getn(branches)]
end

-- valida um branch: retorna true caso ele exista, nil em caso contrrio
function SVN:checkBranch (branch)
    self:_getBranches()
    return branch and self.existingBranches[branch]
end

-- verifica se um nome eh um nome de branch valido
function SVN:isValidBranch(branch, validationRegex)
    validationRegex = validationRegex or BRANCH_VALIDATION_REGEX
    local PRJ_PREFIX = ConfigFile:getVar("PRJ_PREFIX")
    local firstIndex = string.find (branch, "^" .. PRJ_PREFIX .. validationRegex)
    return firstIndex ~= nil
end

-- retorna o identificador do prximo branch. Se j existe algum usando
-- a nova nomenclatura, baseia-se nela e incrementa o minor number. Se
-- no existe, sugere o branch 1.0
function SVN:getNextBranch()
    self:_getBranches()
    local releaseMajor = 1
    local releaseMinor = 0
    local branch = SVN:getLastBranch ()
    if branch ~= nil then
        local major, minor = _getVersion(branch, BRANCH_VERSION_REGEX)
        if minor and (tonumber(minor) >= releaseMinor) then
            releaseMinor = minor + 1
        end
        if releaseMinor == 100 then
            releaseMinor = 0
            if major and (tonumber(major) >= releaseMajor) then
                releaseMajor = major + 1
            end
        end
    end
    local PRJ_PREFIX = ConfigFile:getVar("PRJ_PREFIX")
    return _getBranchId(PRJ_PREFIX, releaseMajor, releaseMinor)
end


-- retorna true se existem branches no formato antigo (comeando com
-- PRJ_YYYY_MM_DD...)
function SVN:hasOldStyleBranches()
    self:_getBranches()
    return table.getn(self.oldStyleBranches) > 0
end

-- retorna true se existem branches no formato antigo (comeando com
-- PRJ_YYYY_MM_DD...)
function SVN:hasNewStyleBranches()
    self:_getBranches()
    return table.getn(self.existingBranches) > 0
end

-- obtem informaes sobre os repositrios do sistema.
-- ret1: o prefixo da URL (tudo que aparece antes do nome do sistema).
--         Ex1: https://subversion.tecgraf.puc-rio.br/engdist
--         Ex2: svn+ssh//rodrigoh@ssh.tecgraf.puc-rio.br/svn/engdist
-- ret2: a localizao da URL (trunk, branch, tag) e o nome.
--         Ex1: trunk
--         Ex2: branches/WS_2006_08_10_v1_00
function SVN:getRepositoriesInfo ()
    local output = SVN:info()
    local urlPrefix
    local repoPath = ConfigFile:getMainModule().rep.path or "engdist"
    local urlRegexp = "^URL: (.-/".. repoPath ..")/%w+/(.*)/%w+$"

    for l in output:lines() do
        _, _, urlPrefix, location = string.find(l, urlRegexp)
        if urlPrefix ~= nil then
            break
        end
    end
    return urlPrefix, location
end

--
-- retorna a verso de um diretrio ("branches/..." ou "tags/...") de
-- acordo com uma regex.
-- Retorna as capturas definidas pela regex do branch ou da tag
function SVN:_getCurrentVersion(wdirPrefix, regex)
    local _, wdir = self:getRepositoriesInfo()
    if not string.find(wdir, wdirPrefix, 1, true) then
        return nil
    end
    return _getVersion(wdir, wdirPrefix .. "/" .. regex)
end

-- retorna o identificador de verso da _tag_ corrente (p.ex. para a tag
-- MLM_v1_04_01_2007_08_19 retorna v1_04_01). Retorna nil caso no
-- estejamos em uma tag (i.e. caso 'svn info' no retorne como URL
-- .../tags/...)
-- Reconhece tanto tags no formato PRJ_YYYY_MM_DD_vX_YY_ZZ quanto no
-- formato PRJ_vX_YY_ZZ_YYYY_MM_DD
function SVN:getCurrentTagVersion()
    local major, minor, patch = self:_getCurrentVersion("tags", TAG_VERSION_REGEX)
    if not (major and minor and patch) then
        return nil
    end
    return string.format("v%d_%02d_%02d", major, minor, patch)
end

-- retorna o identificador de verso do _branch_ corrente (p.ex. para o
-- branch MLM_v1_04_2007_08_19 retorna v1_04). Retorna nil caso no
-- estejamos em um branch (i.e. caso 'svn info' no retorne como URL
-- .../branches/...)
-- Reconhece tanto branches no formato PRJ_YYYY_MM_DD_vX_YY quanto no
-- formato PRJ_vX_YY_YYYY_MM_DD
function SVN:getCurrentBranchVersion()
    local major, minor = self:_getCurrentVersion("branches", BRANCH_VERSION_REGEX)
    if not (major and minor) then
        return nil
    end
    return string.format("v%d_%02d", major, minor)
end

-- retorna o identificador do branch corrente (p.ex. MLM_v1_04_2007_08_19).
-- Caso estejamos no trunk, retorna "trunk"
function SVN:getCurrentBranchID()
    local branch = SVN:_getCurrentVersion("branches", BRANCH_ID_REGEX)
    if not branch then
        local _, wdir = self:getRepositoriesInfo()
        if wdir == "trunk" then
            return wdir
        end
    end
    return branch
end

-- --------------------------------------------------
-- classe Checkout

local Checkout = Module:new {
    actionInfo = "checkout",
    info = "traz arvore de diretorios do SVN (checkout)",
    -- ao exportada pelo mdulo
    provides = MODULE.CHECKOUT,
    -- verses exigidas dos utilitrios
    requires = { svn = "*", },
    checkCache = {},
}

-- verifica se todos os mdulos foram extrados para os respectivos
-- diretrios
function Checkout:check ()
    if self.checkOK then return true end
    local missingModule = false
    for _, node, dir, module in ConfigFile:nodes() do
        node.localDir = node.localDir or dir
        dir = node.localDir .. module
        if self.checkCache[dir] == false then
            return false
        elseif self.checkCache[dir] == nil then
            local c1, e1 = isDir(dir)
            local c2, e2
            if c1 then c2, e2 = isDir(dir.."/.svn") end
            cs_log:debug("verificando checkout: "..dir)
            self.checkCache[dir] = c1 and c2
            if not self.checkCache[dir] then
                cs_log:debug(e1 or e2)
                missingModule = true
            end
        end
    end
    self.checkOK = not missingModule
    return self.checkOK
end

-- ao associada ao mdulo
function Checkout:execute (args, params)
    self.getopt:parse(args, string.concat(params, self.params))

    for _, module, dir in ConfigFile:nodes() do
        abort(module.rep, "Repositorio indefinido para o modulo '"..module.module..
          "'.\n        Verifique o parametro 'rep' para este modulo no arquivo csmake.cfg")
        if not self.checkCache[dir..module.module] then
            SVN:checkout(module, dir)
        else
            cs_log:info("modulo "..module.module.." ja recuperado")
        end
        -- registra o dir local aonde o mdulo foi extrado
        module.localDir = dir
    end
end

-- exibe parmetros especficos deste mdulo
function Checkout:help ()
    print([[

Checkout:
----------

Obtem todos os modulos pertencentes a um projeto

]])
end

-- registra o mdulo
cs_registerModule(Checkout)

-- --------------------------------------------------
-- classe Update

local Update = Module:new {
    actionInfo = "svn update",
    info = "atualiza arvore local",
    -- ao exportada pelo mdulo
    provides = MODULE.UPDATE,
    -- verses exigidas dos utilitrios
    requires = { svn = "*", },
}

function Update:check ()
    return false
end

-- exibe parmetros especficos deste mdulo
function Update:help ()
    print([[

Update:
----------
[m1 m2 ...] : modulos afetados pela operacao [1]

[1]   caso nao sejam especificados modulos, atua sobre TODOS os modulos

]])
end

function Update:execute (args, params)
    self.getopt:parse(args, string.concat(params, self.params))
    SVN:resetParams()

    local allModules = (self.getopt:nargs() == 1)

    -- arg[1]  a operao ('update')
    for module, dir, level in self:moduleIterator(allModules, 2, 1) do
        if not allModules or (allModules and level == 1) then
            print()
            cs_log:info("atualizando modulo >>> "..module)
            SVN:update(dir)
        end
    end
    -- se a atualizao envolve todos os mdulos, devemos atualizar
    -- tambm o prprio csmake
    if allModules then
        local act = Action:new(MODULE.AUTOUPDATE)
        local newOpts = self.getopt:leaveOnly(self.getopt:getOptsAsString(),
          "C")
        act:execute(MODULE.AUTOUPDATE.." "..newOpts)
    end
end

-- registra o mdulo
cs_registerModule(Update)

-- --------------------------------------------------
-- classe AutoUpdate

local AutoUpdate = Module:new {
    actionInfo = "auto-atualizacao",
    info = "atualiza csmake",
    -- ao exportada pelo mdulo
    provides = MODULE.AUTOUPDATE,
    -- verses exigidas dos utilitrios
    requires = { svn = "*", },
    moduleName = "csmake",
    moduleDir = "../",
}

function AutoUpdate:check ()
    return false
end

-- exibe parmetros especficos deste mdulo
function AutoUpdate:help ()
    print([[

AutoUpdate:
----------

OBS.:
  * a atualizacao do csmake acontece automaticamente sempre que e feito
    um 'csmake update' aonde todos os modulos sao atualizados

]])
end

function AutoUpdate:execute (args, params)
    self.getopt:parse(args, string.concat(params, self.params))
    SVN:resetParams()
    cs_log:info("atualizando modulo >>> "..self.moduleName)
    SVN:update(self.moduleDir..self.moduleName, self.getopt:opt("s"))
end

-- registra o mdulo
cs_registerModule(AutoUpdate)

-- --------------------------------------------------
-- classe Tag

local Tag = Module:new {
    actionInfo = "svn tag",
    info = "aplica/remove tags simbolicas",
    -- ao exportada pelo mdulo
    provides = MODULE.TAG,
    -- parmetros adicionais aceitos pelo mdulo
    params = "r:d:LR",
    -- verses exigidas dos utilitrios
    requires = { svn = "*", },
}

function Tag:check ()
    return false
end

function Tag:help ()
    print([[

Tag:
----------
-d tag      : remove tag
-L          : lista as tags existentes
-r REV      : especifica a revisao
-R          : reusa a ultima tag (apenas muda a data)

OBS.:
  * atua sobre TODOS os modulos
  * o tag e obtido de forma interativa
  * -d tem precedencia sobre -R

]])
end

function Tag:execute (args, params)
    self.getopt:parse(args, string.concat(params, self.params))
    SVN:resetParams()

    -- apenas lista as tags existentes
    if self.getopt:opt("L") then
        cs_log:info("tags existentes\n")
        for t in SVN:tagIterator() do
            print("       "..t)
        end
        return
    end

    local PRJ_PREFIX = ConfigFile:getVar("PRJ_PREFIX")
    -- usamos o branch corrente
    local branch = SVN:getCurrentBranchID()
    abort(branch ~= "trunk", "Tags nao podem ser criadas a partir do trunk")
    if not branch and SVN:hasOldStyleBranches() then
        -- temos branches mas todos esto no formato antigo
        branch = _getBranchId(PRJ_PREFIX, 1, 0)
    end
    abort(branch, "Nao existe nenhum branch")

    local delete = self.getopt:opt("d")
    local reuseLast = self.getopt:opt("R")
    local revision = self.getopt:opt("r")
    local tag
    if delete then
        -- remover tag
        tag = delete
        abort(SVN:checkTag(tag), "tag inexistente")
    elseif reuseLast then
        -- reusar ltima tag: apagamos a ltima tag e criamos uma nova
        -- com os mesmos nmeros de verso mas com a data corrente
        local oldTag = SVN:getLastTag(branch)
        local major, minor, patch = _getVersion(oldTag, TAG_VERSION_REGEX)
        tag = _getTagId(PRJ_PREFIX, major, minor, patch)
        cs_log:warn("Removendo tag "..oldTag)
        cs_log:warn("Criando tag   "..tag)
        if not confirm("Continua? (s/n)", "s") then
            print()
            cs_log:fatal("operacao abortada pelo usuario")
            os.exit(1)
        end
        -- removemos a tag antiga
        _execTagCmd(oldTag, branch, nil, true, revision)
    else
        -- criar nova tag
        if not SVN:hasNewStyleTags() and SVN:hasOldStyleTags() then
            cs_log:warn("Tags no formato antigo nao serao consideradas.")
            print()
        end
        tag = _askForTag(branch)
    end

    _execTagCmd(tag, branch, nil, delete, revision)

    local csrep = ConfigFile:getRepository("csbase")
    local repositoryPrefix = SVN:getRepositoriesInfo()
    repositoryPrefix = repositoryPrefix.."/"..csrep.name

    if delete then
      tag = SVN:getLastTag(branch)
    end

    -- Atualiza arquivo de verso no branch
    local branchUrl = repositoryPrefix.."/"..csrep.branches.."/"..branch..
        "/csbase"
    _updateVersionFile(branchUrl, tag, "atualizando arquivo de versao no branch")

    -- Atualiza arquivo de verso na tag
    if not delete then
      local tagUrl = repositoryPrefix.."/"..csrep.tags.."/"..tag.."/csbase"
      _updateVersionFile(tagUrl, tag, "atualizando arquivo de versao na tag")
    end
end

-- registra o mdulo
cs_registerModule(Tag)

-- --------------------------------------------------
-- classe Status 

local Status = Module:new {
    actionInfo = "status",
    info = "exibe status",
    -- ao exportada pelo mdulo
    provides = MODULE.STATUS,
    -- parmetros adicionais aceitos pelo mdulo
    params = "l",
    -- verses exigidas dos utilitrios
    requires = { svn = "*", },
}

function Status:check ()
    return false
end

function Status:execute (args, params)
    self.getopt:parse(args, string.concat(params, self.params))
    SVN:resetParams()

    if not self.getopt:opt("l") then
        SVN:setCmdParam("-u")
    end

    local allModules = (self.getopt:nargs() == 1)

    for module, dir, level in self:moduleIterator(allModules, 2, 1) do
        if not allModules or (allModules and level == 1) then
            print()
            cs_log:info("exibindo status do modulo >>> "..module)
            SVN:status (dir)
        end
    end
end

function Status:help ()
    print([[

Status:
----------
[m1 m2 ...] : modulos afetados pela operacao [1]
-l          : modo local

[1] caso nao sejam especificados modulos, atua sobre TODOS os modulos

]])
end

-- registra o mdulo
cs_registerModule(Status)


-- --------------------------------------------------
-- classe Branch 

local Branch = Module:new {
    actionInfo = "svn branch",
    info = "cria, remove ou lista branches",
    -- ao exportada pelo mdulo
    provides = MODULE.BRANCH,
    -- parmetros adicionais aceitos pelo mdulo
    params = "r:d:L",
    -- verses exigidas dos utilitrios
    requires = { svn = "*", },
}

function Branch:check ()
    return false
end

function Branch:execute (args, params)
    self.getopt:parse(args, string.concat(params, self.params))
    SVN:resetParams()

    -- apenas lista os branches existentes
    if self.getopt:opt("L") then
        cs_log:info("branches existentes\n")
        for b in SVN:branchIterator() do
            print("       "..b)
        end
        print()
        return
    end

    local delete = self.getopt:opt("d")
    local branch
    if delete then
        branch = delete
        abort(SVN:checkBranch(branch), "branch inexistente")
    else
        if not SVN:hasNewStyleBranches() and SVN:hasOldStyleBranches() then
            cs_log:warn("Branches no formato antigo nao serao considerados.")
            print()
        end
        local force = self.getopt:opt("f")
        branch = _askForBranch(force)
    end

    local revision = self.getopt:opt("r")
    for _, repository in ipairs(ConfigFile:getRepositories()) do
        print()
        SVN:branch(branch, repository, nil, delete, revision)
    end
end

function Branch:help ()
    print([[

Branch:
----------
-d branch   : remove branch
-L          : lista os branches existentes
-f          : cria o branch sem criticar se o nome esta no padrao de 
              geracao de versao.

OBS.:
  * atua sobre TODOS os modulos
  * o nome do branch e obtido de forma interativa
]])
end

-- registra o mdulo
cs_registerModule(Branch)

-- --------------------------------------------------
-- cadastramento dos mdulos
