-- $Id: getopt.lua 35549 2005-05-19 12:51:42Z costa $

-- Mdulo para processamento de parmetros fornecidos na
-- linha-de-comando. Cdigo baseado no mdulo de mesmo nome criado para
-- o LSpamK

require "global"
require "oo"

GetOpt = OO:new { err = { NOVAL = 5; SYNTAX = 6; INVALID = 7; BADSWITCH = 8 } }

-- ..................................................

-- constructor

function GetOpt:new ()
    return OO.new(self, { switches = {}, valSwitches = {}, cache = {}, })
end

-- ..................................................

-- configures GetOpt options. receives a string with char switches that
-- should be captured. chars followed by ':' correspond to valued params

local function _init (self, initStr)
   self.switches, self.valSwitches = {}, {}  -- reset internal tables
   initStr = string.gsub(initStr, "(%a):", function (opt)
      self.valSwitches[opt] = true
   end) -- end of anon function
   initStr = string.gsub(initStr, "%a", function (opt)
      self.switches[opt] = true
   end) -- end of anon function
   abort(string.len(initStr) == 0, "erro de sintaxe na opcao '"..initStr.."'")
end

-- ..................................................

-- parses a table (argv) according to the configuration set by initStr
-- (which is optional). If initStr is nil, :init() must have been
-- executed before :parse(). Non-switch args can be freely intercalated
-- with switches and will be properly collected on the same order they
-- have been provided. The special switch "--" turns off switch
-- processing, and all remaining args are considered non-switch args.
--
-- data collected by this function has to be extracted with the accessor
-- functions 'opts' and 'args' below
--
-- if 'strict' is not nil, unrecognized parameters will generate errors
--
-- e.g. parse( {"-f", "aaa", "bbb", "-v", "ccc" }, "vf:" ) returns:
-- opts = { ["f"] = "aaa", ["v"] = true },
-- params = { [1] = "bbb", [2] = "ccc" }

local function _parse (self, argv, initStr, strict)
   self.usrOpts = {}  -- user supplied opts
   self.usrArgs = {}  -- user supplied args (non-opts)
   if initStr then
      _init(self, initStr)
   elseif not self.switches then
      -- :init() hasn't been called, all params are considered args
      self.usrArgs = argv
      return
   end
   local isVal, stopProc, optName = false, false, nil
   for _,v in ipairs(argv) do
      if stopProc then  -- we're just collecting args now
         table.insert(self.usrArgs, v)
      else
         local skip = false
         if isVal then
            -- we're expecting a value
            abort(string.sub(v, 1, 1) ~= "-", "parametro '"..v.."' recebido ao inves de valor", 1, true)
            self.usrOpts[optName] = v
            isVal = false
            skip = true
         end
         if not skip then
            local opt, c = string.gsub(v, "^%-([%a%-])$", "%1")
            if c > 0 then
               if opt == "-" then
                  -- stop looking for switches; all remaining args should
                  -- be appended to self.usrArgs table
                  stopProc = true
               elseif self.switches[opt] then
                  self.usrOpts[opt] = true
               elseif self.valSwitches[opt] then
                  -- parameter needs a value
                  isVal = true
                  optName = opt
               elseif strict then
                  -- unrecognized option
                  abort(nil, "parametro invalido '"..v.."'")
               else
                  -- if not in 'strict mode', we collect even the opts
                  -- that have not been requested
                  self.usrOpts[opt] = false
               end
            elseif string.find(v, "^%-") then
               -- neither an option nor a value
               abort(nil, "dado invalido '"..v.."'")
            else
               -- non-opt arg
               table.insert(self.usrArgs, v)
            end
         end
      end
   end
   -- last option was still waiting for a value
   abort(not isVal, "'-"..(optName or "").."' deve receber um valor")
end

-- ..................................................

-- this is just a wrapper for the real function _parse() above. Its
-- purpose is to prepare the argv parameter for _parse, because it
-- expects a table and the user can pass a string
-- If required set (argv, initStr) has already been parsed, use cached
-- values.

function GetOpt:parse (argv, initStr, strict)
    local hash
    if type(argv) == "table" then
        -- uses table.merge to make a shallow copy of argv
        local argv2 = table.merge(argv, {})
        table.sort(argv2)
        -- use argv string signature + initStr as hash value
        hash = table.concat(argv2)..initStr
    else
        hash = string.gsub(argv, " ", "") .. initStr
    end
    if self.cache[hash] then
        self.usrOpts = self.cache[hash].usrOpts
        self.usrArgs = self.cache[hash].usrArgs
        return
    end
    if type(argv) == "string" then
        local strarg = argv
        argv = {}
        string.gsub(strarg, "(%S+)%s*", function (param)
            table.insert(argv, param)
        end)
    else
        abort(type(argv) == "table", "GetOpt:parse : parametro invalido")
    end
    _parse(self, argv, initStr, strict)
    self.cache[hash] = { usrArgs = self.usrArgs, usrOpts = self.usrOpts }
end

-- ..................................................

-- returns nil if no options have been collected or pairs 
-- ((string)optname, optval) otherwise. the '-' prefix is removed from
-- 'optname'. Each call returns one pair.
--
-- e.g. parse( {"-f", "aaa", "bbb", "-v", "ccc" }, "vf:" ) returns:
-- [1st call] "f", "aaa"
-- [2nd call] "v", true

function GetOpt:optPairs ()
   if not self.usrOpts then
      return
   else
      return coroutine.wrap(function ()
         for k,v in pairs(self.usrOpts) do
            coroutine.yield(k, v)
         end
      end)
   end
end

-- ..................................................

-- returns nil if no args have been collected or pairs
-- ((int)argindex, argval) otherwise. Each call returns one pair.
--
-- e.g. parse( {"-f", "aaa", "bbb", "-v", "ccc" }, "vf:" ) returns:
-- [1st call] 1, "aaa"
-- [2nd call] 2, "bbb"

function GetOpt:argPairs ()
   if not self.usrArgs then
      return
   else
      return coroutine.wrap(function ()
         for i,v in ipairs(self.usrArgs) do
            coroutine.yield(i, v)
         end
      end)
   end
end

-- ..................................................

-- returns option associated to parameter 'optname'

function GetOpt:opt (optname)
   return self.usrOpts and self.usrOpts[optname]
end

-- ..................................................

-- returns nth arg (n = [1..N], where N is the number of _args_, not
-- parameters)

function GetOpt:arg (i)
   return self.usrArgs and self.usrArgs[i]
end

-- ..................................................

-- returns the number of arguments passed

function GetOpt:nargs ()
    return self.usrArgs and table.getn(self.usrArgs) or 0
end

-- ..................................................

-- returns args list as string, as well as the number of args. If
-- 'first' is provided, skip all args before it (in this case, nargs is
-- adjusted accordingly). Indexes start at 1.

function GetOpt:getArgsAsString (first)
    if first then
        local str = ""
        local n = 0
        for _, arg in self:argList(first) do
            str = str.." "..arg
            n = n+1
        end
        return string.trim(str), n
    else
        return table.concat(self.usrArgs, " "), table.getn(self.usrArgs)
    end
end

-- ..................................................

-- returns params list as string

function GetOpt:getOptsAsString ()
    local str = ""
    for k,v in pairs(self.usrOpts) do
        str = str.."-"..k.." "
        if type(v) ~= "boolean" then
            str = str..tostring(v).." "
        end
    end
    return string.gsub(str, "(.-)%s$", "%1")
end

-- ..................................................

-- arg list iterator. Accepts starting arg index as a parameter. Indexes
-- start at 1.

function GetOpt:argList (first)
    return coroutine.wrap(function ()
        if table.getn(self.usrArgs) < first then return end
        for k, v in ipairs(self.usrArgs) do
            if k >= first then
                coroutine.yield(k, v)
            end
        end
    end)
end

-- ..................................................

-- removes unwanted params from a given command line. If removeMask
-- contains ':', both parameter AND value are removed.
-- returns the given line without specified parameters/values

function GetOpt:remove (params, removeMask)
    local blackList = {}
    string.gsub(removeMask, "(%a)(:?)", function (opt, hasValue)
        blackList["-"..opt] = { hasValue = (hasValue == ":") }
      end)

    local newParams = ""
    local skipNext = false
    for w in string.gfind(params, "%S+") do
        if blackList[w] ~= nil then
            -- we must skip this value; if parameter has value, next
            -- token is an argument and must be skipped as well
            skipNext = blackList[w].hasValue
        elseif not skipNext then
            newParams = newParams.." "..w
        else
            abort(string.sub(w, 1, 1) ~= "-", "parametro '"..w.."' encontrado, valor era esperado", 1, true)
            -- we have just skipped a value, reset flag
            skipNext = false
        end
    end
    return string.sub(newParams, 2)
end

-- ..................................................

-- complement for GetOpt:removeOpts() : removes all BUT specified
-- parameters
-- ASSERTION: 'params' _must_ be a string

function GetOpt:leaveOnly (params, keepMask)
    local whiteList = {}
    string.gsub(keepMask, "(%a)(:?)", function (opt, hasValue)
        whiteList["-"..opt] = { hasValue = (hasValue == ":") }
      end)

    local newParams = ""
    local keepNext = false
    for w in string.gfind(params, "%S+") do
        if whiteList[w] ~= nil then
            -- we must keep this value; if parameter has value, next
            -- token is a value and must be kept as well
            newParams = newParams.." "..w
            keepNext = whiteList[w].hasValue
        elseif keepNext then
            abort(string.sub(w, 1, 1) ~= "-", "parametro '"..w.."' encontrado, valor era esperado", 1, true)
            newParams = newParams.." "..w
            keepNext = false
        end
    end
    return string.sub(newParams, 2)
end
