package nio.client;

import java.io.File;
import java.io.IOException;
import java.nio.channels.SocketChannel;

import nio.common.SimpleFileTransferProtocol;

/**
 * Classe que define o papel do cliente na execuo do protocolo de transmisso
 * de arquivos.
 *
 * @author Tecgraf
 */
class ClientSimpleFileTransferProtocol extends
  SimpleFileTransferProtocol {
  
  /**
   * Cria o objeto que implementa o protocolo no cliente, indicando a capacidade
   * do buffer a ser utilizado e o canal de comunicao com o servidor.
   * 
   * @param capacity capacidade do buffer a ser utilizado.
   * @param channel canal de comunicao com o servidor.
   */
  ClientSimpleFileTransferProtocol(int capacity, SocketChannel channel) {
    super(capacity, channel);
  }

  /**
   * Envia um arquivo para o servidor.
   * 
   * @param sourcePath caminho de origem para o arquivo no sistema de arquivos 
   *        do cliente.
   * @param targetPath caminho de destino para o arquivo no sistema de arquivos 
   *        do servidor.
   *        
   * @throws IOException
   */
  public void writeFile(String sourcePath, String targetPath) throws IOException {
    super.operation = CLIENT_WRITE;
    super.filePath = targetPath;
    super.filePathSize = filePath.length();
    File file = new File(sourcePath);
    super.fileSize = file.length();
    io.createSource(file);
    processState();
  }
  
  /**
   * Recebe um arquivo do servidor.
   * 
   * @param sourcePath caminho de origem para o arquivo no sistema de arquivos 
   *        do servidor.
   * @param targetPath caminho de destino para o arquivo no sistema de arquivos 
   *        do cliente.
   *        
   * @throws IOException 
   */
  public void readFile(String sourcePath, String targetPath) throws IOException {
    super.operation = CLIENT_READ;
    super.filePath = sourcePath;
    super.filePathSize = filePath.length();
    File file = new File(targetPath);
    io.createTarget(file);
    processState();
  }
  
  /**
   * {@inheritDoc}
   */
  @Override
  protected boolean contentMissing() throws IOException {
    switch (operation) {
      case CLIENT_WRITE:
        return (io.availableForReading() > 0);
      case CLIENT_READ:
        return (totalBytesReceived < fileSize);
      default:
        throw new IllegalStateException("operao "+operation+" desconhecida");
    }    
  }

  /**
   * Envia o cdigo de operao (escrita ou leitura) para o servidor.
   *
   * @return <code>true</code> se o procedimento foi bem-sucedido e o protocolo
   *         puder seguir para a prxima etapa.
   *         
   * @throws IOException
   */
  @Override
  protected boolean processOperation() throws IOException {
    return io.writeByte(operation);
  }  

  /**
   * Envia o tamanho do caminho do arquivo a ser transmitido para o servidor.
   *
   * @return <code>true</code> se o procedimento foi bem-sucedido e o protocolo
   *         puder seguir para a prxima etapa.
   *         
   * @throws IOException
   */
  @Override
  protected boolean processPathSize() throws IOException {
    return io.writeInt(filePathSize);
  }

  /**
   * Envia o caminho do arquivo a ser transmitido para o servidor.
   *
   * @return <code>true</code> se o procedimento foi bem-sucedido e o protocolo
   *         puder seguir para a prxima etapa.
   *         
   * @throws IOException
   */
  @Override
  protected boolean processPath() throws IOException {
    return io.writeString(filePath);
  }

  /**
   * Envia ou recebe o tamanho do arquivo a ser transmitido, dependendo da
   * operao escolhida.
   *
   * @return <code>true</code> se o procedimento foi bem-sucedido e o protocolo
   *         puder seguir para a prxima etapa.
   *         
   * @throws IOException
   */
  @Override
  protected boolean processFileSize() throws IOException {
    switch (operation) {
      case CLIENT_WRITE:
        return io.writeLong(fileSize);
      case CLIENT_READ:
        fileSize = io.readLong();
        return (fileSize != -1);
      default:
        return false;
    }
  }

  /**
   * Envia ou recebe o tamanho de um <b>pacote</b> a ser transmitido, dependendo
   * da operao escolhida.
   *
   * @return <code>true</code> se o procedimento foi bem-sucedido e o protocolo
   *         puder seguir para a prxima etapa.
   *         
   * @throws IOException
   * 
   * @see SimpleFileTransferProtocol para uma definio do termo <b>pacote</p>.
   */
  @Override
  protected boolean processPackSize() throws IOException {
    switch (operation) {
      case CLIENT_WRITE:
        packSize = io.availableForReading();
        packSize = 
          calcPackSizeThatFitsInBuffer(packSize, io.getContentBufferSize());    
        return io.writeInt(packSize);
      case CLIENT_READ:
        packSize = io.readInt();
        if (packSize == -1) {
          // No conseguiu ler tudo ainda. No muda o estado, aguardando a 
          // prxima notificao.
          return false;
        }
        if (packSize > fileSize) {
          //TODO [TESTES] Tratar erro
        }
        return true;
      default:
        return false;
    }
  }

  /**
   * Envia ou recebe um <b>pacote</b> de dados provenientes do arquivo.
   *
   * @return <code>true</code> se o procedimento foi bem-sucedido e o protocolo
   *         puder seguir para a prxima etapa.
   *         
   * @throws IOException
   * 
   * @see SimpleFileTransferProtocol para uma definio do termo <b>pacote</p>.
   */  
  @Override
  protected boolean processPack() throws IOException {
    switch (operation) {
      case CLIENT_WRITE:
        io.writePack(packSize);
        return true;
      case CLIENT_READ:
        long bytesReceived = io.readPack(fileSize, packSize);
        if (bytesReceived == -1) {
          // No conseguiu ler nem um pacote ainda. No muda o estado, aguardando a 
          // prxima notificao.
          return false;
        }
        totalBytesReceived += bytesReceived;
        return true;
      default:
        return false;
    }
  }
  
  /**
   * {@inheritDoc}
   */
  @Override
  protected void closeResources() throws IOException {
    io.closeResources();
  }

  /**
   * {@inheritDoc}. O cliente no precisa nunca interromper o processamento do
   * protocolo, pois seu I/O  bloqueante e por isso bloquear naturalmente se
   * o protocolo estiver aguardando dados para continuar.
   */
  @Override
  protected boolean hasToWait() {
    return false;
  }

}
