package nio.client;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.channels.SocketChannel;

/**
 * Cliente para exportao de arquivos, semelhante a um cliente FTP, mas 
 * usando a API NIO do Java.
 * 
 * Cliente de uma aplicao simples cliente/servidor de transferncia de 
 * arquivos. 
 * 
 * @author Tecgraf
 */
public class NIOClient {
  /** Host default para o servidor. */
  private final static String DEFAULT_SERVER = "localhost";
  /** Porta default em que o servidor aguarda conexes. */
  private final static int DEFAULT_PORT_NUMBER = 7780;
  /** Tamanho default para o buffer de transmisso do contedo do arquivo. */
  private static final int DEFAULT_BUFFER_SIZE = 1048576; // 1MB

  /** Objeto que implementa o protocolo de comunicao no cliente. */
  private ClientSimpleFileTransferProtocol protocol;

  /** Modo de execuo do cliente. */
  private enum Mode { 
    /** Modo de leitura: transferncia sentido servidor -> cliente. */
    READ, 
    /** Modo de escrita: transferncia sentido cliente -> servidor. */
    WRITE };
  
  /**
   * Envia/recebe um arquivo para o/do servidor, uma ou mais vezes seguidas. 
   * 
   * @param server host do servidor.
   * @param port porta do servidor.
   * @param sourcePath caminho para o arquivo a ser transmitido, no sistema de
   *        arquivos do cliente.
   * @param targetPath caminho de destino para o arquivo no sistema de arquivos
   *        do servidor.
   * @param times nmero de repeties do envio do arquivo em srie.
   * @param bufferSize tamanho desejado para o buffer de transmisso do 
   *        contedo do arquivo.
   * @param executionMode modo de execuo do cliente: READ ou WRITE.
   */
  public NIOClient(
    String server, int port, String sourcePath, String targetPath, int times, 
    int bufferSize, Mode executionMode) {
    if (bufferSize <= 0) {
      System.err.println("No  permitido o valor "+bufferSize+" para o buffer."
        +"\nO buffer deve ter tamanho positivo.");
      System.exit(0);
    }
    for (int loop = 0; loop < times; loop++) {
      SocketChannel serverChannel = null;
      try {
        serverChannel = SocketChannel.open();
        serverChannel.connect(new InetSocketAddress(server, port));
      }
      catch (IOException e) {
        System.err.println(
          "No foi possvel conexo com o servidor "+server+" na porta "+port+".");
        e.printStackTrace();
        System.exit(-1);
      }
      protocol = new ClientSimpleFileTransferProtocol(bufferSize, serverChannel);  
      try {
        switch (executionMode) {
          case READ:
            protocol.readFile(sourcePath, targetPath);
            break;
          case WRITE:
            protocol.writeFile(sourcePath, targetPath);
            break;
          default:
            break;
        }
      }
      catch (FileNotFoundException e) {
        System.err.println("No foi possvel abrir o arquivo "+sourcePath+".");
        e.printStackTrace();
      }
      catch (IOException e) {
        System.err.println("No foi possvel transferir o arquivo "+sourcePath+".");
        e.printStackTrace();
      }
      finally {
        if (serverChannel != null) { 
          try { serverChannel.close(); } catch (IOException e) {}
        }
      }
    }
  }

  /**
   * Obtm o host do servidor (default: <code>DEFAULT_SERVER</code>).
   * 
   * @param args argumentos da linha de comando.
   * 
   * @return host do servidor.
   */
  private static String getServer(String[] args) {
    String server = DEFAULT_SERVER;
    for (int i = 0; i < args.length; i++) {
      if (args[i].equals("-s")) {
        if (args.length < i+2) {
          terminate();
        }
        server = args[i+1];
        break;
      }
    }
    System.out.println("Conectando ao servidor: "+server);
    return server;
  }
  
  /**
   * Obtm a porta na qual o servidor dever escutar (default: 
   * <code>DEFAULT_PORT_NUMBER</code>).
   * 
   * @param args argumentos da linha de comando.
   * 
   * @return porta na qual o servidor dever escutar.
   */
  private static int getServerPort(String[] args) {
    int port = DEFAULT_PORT_NUMBER;
    for (int i = 0; i < args.length; i++) {
      if (args[i].equals("-p")) {
        try {
          if (args.length < i+2) {
            terminate();
          }
          port = Integer.parseInt(args[i+1]);
        }
        catch (NumberFormatException ex) {
          terminate();
        }
        break;
      }
    }
    System.out.println("Porta: "+port);
    return port;
  }

  /**
   * Retorna o nmero de vezes que o arquivo deve ser transmitido, em sequncia
   * (uma transmisso s comea quando a anterior  concluda).
   * 
   * @param args parmetros da linha de comando.
   * @return nmero de vezes que o arquivo deve ser transmitido
   */
  private static int getTimes(String[] args) {
    int times = 1;
    for (int i = 0; i < args.length; i++) {
      if (args[i].equals("-t")) {
        try {
          if (args.length < i+2) {
            terminate();
          }
          times = Integer.parseInt(args[i+1]);
        }
        catch (NumberFormatException ex) {
          terminate();
        }
        break;
      }
    }
    System.out.println("N de repeties: "+times);
    return times;
  }
  
  /**
   * Retorna o tamanho do buffer desejado para transmisso do contedo do
   * arquivo.
   * 
   * @param args parmetros da linha de comando.
   * 
   * @return tamanho do buffer desejado para transmisso do contedo do
   *         arquivo.
   */
  private static int getBufferSize(String[] args) {
    int size = DEFAULT_BUFFER_SIZE;
    for (int i = 0; i < args.length; i++) {
      if (args[i].equals("-b")) {
        try {
          if (args.length < i+2) {
            terminate();
          }
          size = Integer.parseInt(args[i+1]);
        }
        catch (NumberFormatException ex) {
          terminate();
        }
        break;
      }
    }
    System.out.println("Tamanho do buffer: "+size);
    return size;
  }
  
  /**
   * Retorna o caminho de destino especificado pelo usurio no sistema de
   * arquivos do servidor.
   * 
   * @param args parmetros da linha de comando.
   * 
   * @return caminho de destino para o arquivo no servidor.
   */
  private static String getTargetPath(String[] args) {
    String path = null;
    for (int i = 0; i < args.length; i++) {
      if (args[i].equals("-d")) {
        if (args.length < i+2) {
          terminate();
        }
        path = args[i+1];
        break;
      }
    }
    if (path == null) {
      terminate();
    }
    System.out.println("Caminho do arquivo de destino: "+path);
    return path;
  }
  
  /**
   * Obtm o caminho para o arquivo a ser exportado.
   * 
   * @param args argumentos da linha de comando.
   * 
   * @return caminho para o arquivo.
   */
  private static String getSourcePath(String[] args) {
    String path = null;
    for (int i = 0; i < args.length; i++) {
      if (args[i].equals("-o")) {
        if (args.length < i+2) {
          terminate();
        }
        path = args[i+1];
        break;
      }
    }
    if (path == null) {
      terminate();
    }
    System.out.println("Caminho do arquivo de origem: "+path);
    return path;
  }

  /**
   * Retorna o modo de execuo do cliente (r - leitura ou w - escrita).
   * 
   * @param args argumentos da linha de comando.
   * 
   * @return modo de execuo do cliente.
   */
  private static Mode getMode(String[] args) {
    char entry = '@';
    for (int i = 0; i < args.length; i++) {
      if (args[i].equals("-m")) {
        if (args.length < i+2) {
          terminate();
        }
        if (args[i+1].length() > 1) {
          terminate();
        }
        entry = args[i+1].charAt(0);
        if (entry != 'r' && entry != 'w') {
          terminate();
        }
        break;
      }
    }
    if (entry == '@') {
      terminate();
    }
    Mode mode = entry == 'r' ? Mode.READ : Mode.WRITE;
    System.out.println("Modo de execuo: "+mode);
    return mode;
  }

  /**
   * Exibe mensagem de erro e aborta a execuo.
   */
  private static void terminate() {
    System.out.println("Uso: nioclient -m <r/w> -o <caminho_arq_origem> -d <caminho_arq_destino> [-s <servidor>] [-p <numero_porta>] [-b <tamanho_buffer>] [-t <numero_repeticoes]");
    System.exit(1);
  }
  
  /**
   * Cria o cliente usando os parmetros especificados na linha de comando.
   * Assume defaults para quase todos os parmetros, caso no sejam fornecidos. 
   * 
   * @param args argumentos da linha de comando.
   */
  public static void main(String[] args) {
    if (args.length < 1) {
      terminate();
    }
    String server = getServer(args);
    int port = getServerPort(args);
    Mode mode = getMode(args);
    int times = getTimes(args);
    int bufSize = getBufferSize(args);
    String sourcePath = getSourcePath(args);
    String targetPath = getTargetPath(args);
    new NIOClient(server, port, sourcePath, targetPath, times, bufSize, mode);
  }
  
}
