package nio.common;

import java.io.IOException;
import java.nio.channels.SocketChannel;

/**
 * <p>
 * Classe abstrata agregando os passos comuns ao protocolo de comunicao entre
 * cliente e servidor.
 * </p>
 * <p>
 * O protocolo obedece a uma seqncia de passos, comum entre cliente e 
 * servidor, definida de acordo com a <b>operao</b> a ser executada. A 
 * <b>operao</b> pode ser de escrita (transmisso no sentido 
 * cliente -> servidor) ou de leitura (sentido servidor -> cliente).
 * </p>
 * <p>
 * O protocolo compreende tambm duas etapas acrescentadas por conta de um bug
 * encontrado na JVM do Linux (vide 
 * http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6253145): transmisso de
 * <b>pacotes</b>, precedidas pelo seu tamanho. Um <b>pacote</b>  uma 
 * subdiviso do arquivo, limitada ao tamanho mximo de 2^32-1 bytes (mximo
 * inteiro, limite em que ocorre o bug). Dessa forma, ao invs de transmitir o
 * arquivo diretamente, o protocolo o transmite em blocos de no mximo 2GB. Para
 * exemplificar, segue a sequncia de etapas da transmisso de um arquivo de
 * 7GB:
 * </p>
 * <ol>
 * <li>Envio da operao;</li>
 * <li>Envio do tamanho do caminho;</li>
 * <li>Envio do caminho;</li>
 * <li>Envio do tamanho do arquivo;</li>
 * <li>Envio do tamanho do pacote 1 (2GB);</li>
 * <li>Envio do pacote 1;</li>
 * <li>Envio do tamanho do pacote 2 (2GB);</li>
 * <li>Envio do pacote 2;</li>
 * <li>Envio do tamanho do pacote 3 (2GB);</li>
 * <li>Envio do pacote 3;</li>
 * <li>Envio do tamanho do pacote 4 (1GB);</li>
 * <li>Envio do pacote 4.</li>
 * 
 * @author Tecgraf
 */
public abstract class SimpleFileTransferProtocol {

  /** Cdigo da <b>operao</b> de escrita (cliente -> servidor). */
  protected static final byte CLIENT_WRITE = 7;
  /** Cdigo da <b>operao</b> de leitura (servidor -> cliente). */
  protected static final byte CLIENT_READ = 13;
  
  /** Estados do protocolo. */
  protected enum State {
    /** Etapa Inicial: iniciando transmisso */
    INITIAL,
    /** Etapa 2: Envio do tamanho do caminho no servidor */
    PATH_SIZE,
    /** Etapa 3: Envio do caminho no servidor */
    PATH,
    /** Etapa 4: Envio do tamanho do arquivo */
    FILE_SIZE,
    /** Etapa 5: Envio do tamanho do <b>pacote</b> (<= tamanho do arquivo) */
    PACK_SIZE,
    /** Etapa 6: Envio do <b>pacote</b> */
    PACK,
    /** Estado final: transmisso concluda */
    FINISHED
  }
  
  /** Etapa em que se encontra a transmisso */
  protected State state;
  /** Objeto que encapsula o tratamento de I/O de baixo nvel */
  protected IOHandler io;
  /** Tamanho do caminho para o arquivo no servidor */
  protected int filePathSize;
  /** Tamanho do arquivo */
  protected long fileSize;
  /** Tamanho do <b>pacote</b> (<= tamanho do arquivo) */
  protected int packSize;
  /** Caminho para o arquivo no servidor */
  protected String filePath;
  /** Cdigo da <b>operao</b> que est sendo realizada */
  protected byte operation;
  /** Total de bytes de um determinado arquivo recebidos at o momento. */
  protected long totalBytesReceived;
  /** Canal do cliente utilizado para a transferncia. */
  protected SocketChannel channel;
  
  /**
   * Cria o objeto que implementa o protocolo no estado inicial, indicando o
   * canal de comunicao com a JVM remota (cliente ou servidor).
   * 
   * @param capacity capacidade do buffer a ser utilizado.
   * @param channel canal de comunicao com a JVM remota
   */
  public SimpleFileTransferProtocol(int capacity, SocketChannel channel) {
    this.channel = channel;
    this.io = new IOHandler(channel, capacity);
    this.state = State.INITIAL;
  }

  /**
   * Retorna o estado que segue ao estado atual, na sequncia do protocolo
   * (mquina de estados).
   * 
   * @return prximo estado na seqncia do protocolo.
   * 
   * @throws IOException
   */
  private State getNextState() throws IOException {
    switch (state) {
    case INITIAL: return State.PATH_SIZE;
    case PATH_SIZE: return State.PATH;
    case PATH: return State.FILE_SIZE;
    case FILE_SIZE: return State.PACK_SIZE;
    case PACK_SIZE: return State.PACK;
    case PACK: 
      return contentMissing() ? State.PACK_SIZE : State.FINISHED;  
    default:
      System.err.println("Erro: estado "+state+" desconhecido");
      return null;
    }
  }
  
  /**
   * Indica se o arquivo foi totalmente transmitido ou se ainda existe alguma
   * parcela a ser transmitida.
   * 
   * @return <code>true</code> se estiver faltando contedo do arquivo a ser
   *         recebido.
   *         
   * @throws IOException
   */
  protected abstract boolean contentMissing() throws IOException;

  /**
   * Executa o passo necessrio do protocolo, no estado atual. Os passos 
   * definidos neste mtodo, embora sigam a mesma seqncia, so diferentes
   * no cliente e no servidor.
   * 
   * @throws IOException
   */
  protected void processState() throws IOException {
    boolean success;
    do {
      switch (state) {
      case INITIAL:
        success = processOperation();
        break;
      case PATH_SIZE:
        success = processPathSize();
        break;
      case PATH:
        success = processPath();
        break;
      case FILE_SIZE:
        success = processFileSize();
        break;
      case PACK_SIZE:
        success = processPackSize();
        break;
      case PACK:
        success = processPack();
        break;
      default:
        System.err.println("Erro: estado "+state+" desconhecido");
        success = false;
        break;
      }
      if (success) {
        state = getNextState();
      }
    } 
    while (state != State.FINISHED && !hasToWait());
    if (isFinished()) {
      closeResources();
    }
  }
  
  /**
   * Define qual o papel de cada JVM (cliente/servidor) na etapa de transmisso
   * da <b>operao</b> a ser executada.
   * 
   * @return <code>true</code> se o procedimento foi bem-sucedido e o protocolo
   *         puder seguir para a prxima etapa.
   *         
   * @throws IOException
   */
  protected abstract boolean processOperation() throws IOException;
  
  /**
   * Define qual o papel de cada JVM (cliente/servidor) na etapa de transmisso
   * do tamanho do caminho do arquivo no servidor.
   * 
   * @return <code>true</code> se o procedimento foi bem-sucedido e o protocolo
   *         puder seguir para a prxima etapa.
   *         
   * @throws IOException
   */  
  protected abstract boolean processPathSize() throws IOException;
  
  /**
   * Define qual o papel de cada JVM (cliente/servidor) na etapa de transmisso
   * do caminho do arquivo no servidor.
   * 
   * @return <code>true</code> se o procedimento foi bem-sucedido e o protocolo
   *         puder seguir para a prxima etapa.
   *         
   * @throws IOException
   */    
  protected abstract boolean processPath() throws IOException;
  
  /**
   * Define qual o papel de cada JVM (cliente/servidor) na etapa de transmisso
   * do tamanho do arquivo.
   * 
   * @return <code>true</code> se o procedimento foi bem-sucedido e o protocolo
   *         puder seguir para a prxima etapa.
   *         
   * @throws IOException
   */    
  protected abstract boolean processFileSize() throws IOException;
  
  /**
   * Define qual o papel de cada JVM (cliente/servidor) na etapa de transmisso
   * do tamanho de um <b>pacote</b> a ser transmitido.
   * 
   * @return <code>true</code> se o procedimento foi bem-sucedido e o protocolo
   *         puder seguir para a prxima etapa.
   *         
   * @throws IOException
   */    
  protected abstract boolean processPackSize() throws IOException;
  
  /**
   * Define qual o papel de cada JVM (cliente/servidor) na etapa de transmisso
   * de um <b>pacote</b>.
   * 
   * @return <code>true</code> se o procedimento foi bem-sucedido e o protocolo
   *         puder seguir para a prxima etapa.
   *         
   * @throws IOException
   */    
  protected abstract boolean processPack() throws IOException;

  
  /**
   * Indica se o estado atual deve aguardar input ou pode seguir com o 
   * protocolo.
   * 
   * @return <code>true</code> se a sequncia do protocolo deva ser interrompida
   *         aguardando input; <code>false</code> se puder seguir.
   */
  protected abstract boolean hasToWait();
  
  /**
   * Define a liberao dos recursos fsicos (canais, arquivos) alocados para a 
   * transmisso.
   *         
   * @throws IOException
   */    
  protected abstract void closeResources() throws IOException;
  

  /**
   * Indica se a transmisso foi encerrada.
   * 
   * @return <code>true</code> se a transmisso tiver sido concluda.
   */
  public boolean isFinished() {
    // TODO Ser que no d pra remover esse cara    
    return state == State.FINISHED;
  }
  
  /**
   * <p>
   * Calcula um tamanho de pacote que seja menor ou mltiplo do tamanho do 
   * buffer.
   * </p>
   * <p>
   * Este mtodo  consequncia do tratamento do bug dos 2GB no Linux (vide
   * http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6253145). 
   * </p>
   * <p>
   * Para evitar que ocorra o bug, a transmisso do arquivo  feita em "pacotes"
   * de no mximo 2GB. O protocolo de transmisso foi estendido e agora informa,
   * alm do tamanho do arquivo esperado, o tamanho dos pacotes em que o arquivo
   * dever ser entregue. A tarefa do servidor  concatenar esses pacotes para 
   * remontar o arquivo.  
   * </p>
   * <p>
   * Entretanto, a transmisso do pacote  feita em pequenos pedaos (buffer) de 
   * tamanho fixo. Como existe um limite (2GB) para o pacote mximo, 
   * mas o buffer pode ter qualquer tamanho,  possvel que o ltimo buffer de 
   * um pacote (aquele que atinge a marca dos 2GB) fique apenas parcialmente 
   * preenchido. Isso gera um problema no outro lado, que consome os bytes 
   * transmitidos via socket sem saber que alguns bytes podem ser lixo (final do 
   * buffer parcialmente preenchido) e devem ser descartados. 
   * </p>
   * <p>
   * Uma soluo seria estender ainda mais o protocolo para informar o tamanho
   * do buffer de envio, ou quantos e quais bytes deveriam ser descartados;
   * entretanto, essa soluo "amarraria" ainda mais o bug na lgica do cdigo.
   * A soluo alternativa escolhida foi limitar ainda mais o tamanho do pacote,
   * em um comprimento que seja mltiplo do tamanho do buffer utilizado. Dessa
   * forma, somente chegaro ao servidor buffers totalmente preenchidos. O 
   * servidor poder ento concatenar tranquilamente todos os bytes recebidos
   * no arquivo, confiante de que todos contm informao til.
   * </p>
   * 
   * @param thePackSize tamanho do pacote.
   * @param bufferSize tamanho do buffer.
   * 
   * @return tamanho de pacote que seja menor ou mltiplo do tamanho do buffer.
   */
  protected int calcPackSizeThatFitsInBuffer(int thePackSize, int bufferSize) {
    if (bufferSize <= 0) {
      throw new IllegalArgumentException("bufferSize <= 0");
    }
    if (bufferSize >= thePackSize) {
      return thePackSize;
    }
    thePackSize -= (thePackSize % bufferSize);
    return thePackSize;
  }

}
