package nio.server;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.nio.channels.ClosedChannelException;
import java.nio.channels.SelectableChannel;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.util.Iterator;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;


/**
 * Servidor para importao de arquivos, semelhante a um servidor FTP, mas 
 * usando a API NIO (com sockets no-bloqueantes) do Java.
 * 
 * @author Leonardo Barros
 */
public class NIOServer {
  /** Porta default em que o servidor aguarda conexes. */
  private final static int DEFAULT_PORT_NUMBER = 7780;
  /** Capacidade default para o buffer de transmisso do contedo do arquivo. */
  private static final int DEFAULT_BUFFER_CAPACITY = 1048576; // 1MB 

  /** Instncia <i>Singleton</i> do servidor. */
  private static NIOServer instance;
  
  /** <i>Timestamp</i> em que comeou a transmisso. */
  private static long initialTime;

  /**
   * Multiplexador (<code>Selector</code>) no qual sero registrados
   * sockets. Sempre que esse multiplexador receber um evento do SO, vai
   * verificar se existe algum socket interessado e disparar a callback 
   * associada ao mesmo.
   */
  private Selector selector;
  /** 
   * Objeto criado para proteger o acesso ao seletor. Sua finalidade 
   * impedir que mais de uma thread possa modificar dados do seletor ao mesmo 
   * tempo.
   */
  private Object guard = new Object();

  /** Pool de threads para a execuo das callbacks de tratamento. */
  private ExecutorService executorsPool;
  
  /**
   * Obtm a porta na qual o servidor dever escutar (default: 
   * <code>DEFAULT_PORT_NUMBER</code>).
   * 
   * @param args argumentos da linha de comando.
   * 
   * @return porta na qual o servidor dever escutar.
   */
  private static int getServerPort(String[] args) {
    int port = DEFAULT_PORT_NUMBER;
    if (args.length > 0) {
      try {
        port = Integer.parseInt(args[0]);
      }
      catch (NumberFormatException ex) {
        System.out.println("Uso: nioserver <numero_porta>");
        System.exit(1);
      }
    }
    System.out.println("Escutando na porta "+port);
    return port;
  }
  
  /**
   * Retorna o tamanho do buffer desejado para transmisso do contedo do
   * arquivo.
   * 
   * @param args parmetros da linha de comando.
   * 
   * @return tamanho do buffer desejado para transmisso do contedo do
   *         arquivo.
   */
  private static int getBufferCapacity(String[] args) {
    int capacity = DEFAULT_BUFFER_CAPACITY;
    for (int i = 0; i < args.length; i++) {
      if (args[i].equals("-b")) {
        try {
          if (args.length < i+2) {
            terminate();
          }
          capacity = Integer.parseInt(args[i+1]);
        }
        catch (NumberFormatException ex) {
          terminate();
        }
        break;
      }
    }
    System.out.println("Tamanho do buffer: "+capacity);
    return capacity;
  }
  
  /**
   * Exibe mensagem de erro e aborta a execuo.
   */
  private static void terminate() {
    System.out.println("Uso: nioserver [-p <numero_porta>] [-b <tamanho_buffer>]");
    System.exit(1);
  }
  
  /**
   * Cria um multiplexador (<code>Selector</code>) no qual sero registrados
   * sockets. Sempre que esse multiplexador receber um evento do SO, vai
   * verificar se existe algum socket interessado e disparar a callback 
   * associada ao mesmo.
   * 
   * @param port porta na qual o servidor dever aguardar conexes.
   * @param bufferCapacity capacidade do buffer de transmisso de contedo.
   */
  private NIOServer(int port, int bufferCapacity) {
    if (instance != null) {
      throw new IllegalStateException("Servidor j est no ar!");
    }
    instance = this;
    // #TODO transformar essa constante em propriedade no CSBase
    executorsPool = Executors.newFixedThreadPool(10);
    try {
      selector = Selector.open();
    }
    catch (IOException e) {
      System.err.println("No foi possvel criar o demultiplexador.");
      e.printStackTrace();
    }
    ServerSocketChannel serverChannel = null;
    try {
      serverChannel = createServerSocketChannel(port);
      serverChannel.register(selector, SelectionKey.OP_ACCEPT);
    }
    catch (ClosedChannelException e) {
      System.err.println("No foi possvel registrar o socket do servidor.");
      e.printStackTrace();
    }
    catch (IOException e) {
      System.err.println("No foi possvel criar o socket do servidor.");
      e.printStackTrace();
    }
    try {
      while (true) {
        synchronized (guard) {
          // Impede que a thread principal invoque a seleo enquanto outra
          // thread tiver acabado de acordar o seletor (wakeup) mas ainda no
          // tiver modificado o interestOps da chave.
        }
        int selectedKeysCount = 0;
        selectedKeysCount = selector.select();
        if (selectedKeysCount == 0) continue;
        Iterator it = selector.selectedKeys().iterator();
        while (it.hasNext()) {
          SelectionKey key = (SelectionKey) it.next();
          if (key.isValid() && key.isAcceptable()) {
            acceptConnection(bufferCapacity, key);
          }
          if (key.isValid() && key.isReadable()) {
            // Necessrio para que esse evento no seja novamente selecionado
            // at que o tratador termine de processar.  responsabilidade do
            // tratador renovar o interesse (se for o caso) aps o 
            // processamento.
            removeInterestOps(key, SelectionKey.OP_READ);
            handleEvent(key);
          }
          if (key.isValid() && key.isWritable()) {
            // Necessrio para que esse evento no seja novamente selecionado
            // at que o tratador termine de processar.  responsabilidade do
            // tratador renovar o interesse (se for o caso) aps o 
            // processamento.
            removeInterestOps(key, SelectionKey.OP_WRITE);
            handleEvent(key);
          }
          it.remove();
        }
      }
    }
    catch (IOException e) {
      System.err.println("No foi possvel tratar um evento do servidor.");
      e.printStackTrace();
    }
    catch (Exception e) {
      System.err.println("No foi possvel tratar um evento do servidor.");
      e.printStackTrace();
    }      
  }

  /**
   * Cria um novo <code>ServerSocketChannel</code>, no-bloqueante, na porta 
   * especificada.
   * 
   * @param port porta na qual o socket vai escutar.
   * 
   * @return <code>ServerSocketChannel</code>
   * 
   * @throws IOException
   */
  private ServerSocketChannel createServerSocketChannel(int port) throws IOException {
    ServerSocketChannel serverChannel = ServerSocketChannel.open();
    ServerSocket serverSocket = serverChannel.socket();
    serverSocket.bind(new InetSocketAddress(port));
    serverChannel.configureBlocking(false);
    return serverChannel;
  }

  /**
   * Estabelece conexo com um socket cliente. Registra esse socket para eventos
   * de leitura (sempre que o socket cliente estiver disponvel para leitura, o
   * multiplexador ser avisado).
   *
   * @param capacity capacidade do buffer de transmisso de contedo.
   * @param key chave contendo o socket do servidor.
   * 
   * @throws IOException
   */
  private void acceptConnection(int capacity, SelectionKey key) 
  throws IOException {
    ServerSocketChannel serverChannel = (ServerSocketChannel)key.channel();
    SocketChannel clientChannel = serverChannel.accept();
    registerChannel(clientChannel, SelectionKey.OP_READ);
    TransferHandler.createHandler(capacity, clientChannel.keyFor(selector));
  }

  /**
   * Registra o canal especificado no multiplexador.
   * 
   * @param channel canal a ser registrado.
   * @param ops eventos (operaes) a observar no canal.
   * 
   * @throws IOException
   */
  private void registerChannel(SelectableChannel channel, int ops) 
  throws IOException {
    if (channel == null) {
      return;
    }
    channel.configureBlocking(false);
    channel.register(selector, ops);
  }

  /**
   * Carrega o <code>ImportHandler</code> adequado para tratar um evento 
   * ocorrido no canal.
   * 
   * @param key chave representando o evento.
   */
  private void handleEvent(SelectionKey key) {
    SocketChannel clientChannel = (SocketChannel)key.channel();
    TransferHandler handler = TransferHandler.getHandler(clientChannel);
    if (handler == null) {
      throw new IllegalStateException("Handler no encontrado!!!");
    }
//    handler.run();
    executorsPool.execute(handler);
  }

  /**
   * Adiciona um ou mais interesses ao conjunto de interesses de uma chave
   * registrada no seletor.
   * 
   * @param key chave a ser modificada.
   * @param addOps conjunto de interesses a ser adicionado.
   */
  public void addInterestOps(SelectionKey key, int addOps) {
    if (!key.isValid()) {
      return;
    }
    synchronized (guard) {
      selector.wakeup();
      key.interestOps(key.interestOps() | addOps);
    }
  }

  /**
   * Remove um ou mais interesses do conjunto de interesses de uma chave
   * registrada no seletor.
   * 
   * @param key chave a ser modificada.
   * @param removeOps conjunto de interesses a ser removido.
   */
  public void removeInterestOps(SelectionKey key, int removeOps) {
    if (!key.isValid()) {
      return;
    }
    synchronized (guard) {
      selector.wakeup();
      key.interestOps(key.interestOps() ^ removeOps);
    }
  }  
  
  /**
   * ATENO!! Mtodo s para teste. No funciona com threads concorrentes.
   * 
   * @return d
   */
  public static long getInitialTime() {
    return initialTime;
  }
   
  /**
   * ATENO!! Mtodo s para teste. No funciona com threads concorrentes.
   */  
  public static void setInitialTime() {
    initialTime = System.currentTimeMillis();
  }
  
  /**
   * Obtm referncia para a instncia nica (<i>Singleton</i>) do servidor.
   * 
   * @return instncia nica do servidor.
   */
  public static NIOServer getInstance() {
    if (instance == null) {
      throw new IllegalStateException("Servidor no inicializado!");
    }
    return instance;
  }
  
  /**
   * Cria o servidor.
   * 
   * @param args
   */
  public static void main(String[] args) {
    int port = getServerPort(args);
    int capacity = getBufferCapacity(args);
    new NIOServer(port, capacity);
  }
  
}
