package nio.server;

import java.io.File;
import java.io.IOException;
import java.nio.channels.SelectionKey;
import java.nio.channels.SocketChannel;

import nio.common.SimpleFileTransferProtocol;

/**
 * Classe que define o papel do servidor no tratamento do protocolo de 
 * transmisso de arquivos.
 *
 * @author Tecgraf
 */
class ServerSimpleFileTransferProtocol extends
  SimpleFileTransferProtocol {

  /** Indica se existem bytes pendentes a serem transmitidos. */
  private boolean nothingToSend = true;
  /** 
   * Conjunto de interesses a serem adicionados quele da chave representando
   * a conexo.
   */
  private int addOps;

  /**
   * Cria o objeto que implementa o protocolo no servidor, indicando o canal de 
   * comunicao com o cliente.
   * 
   * @param capacity capacidade do buffer a ser utilizado.
   * @param channel canal de comunicao com o cliente.
   */
  ServerSimpleFileTransferProtocol(int capacity, SocketChannel channel) {
    super(capacity, channel);
  }
  
  /**
   * {@inheritDoc}
   */
  @Override
  protected boolean contentMissing() throws IOException {
    switch (operation) {
      case CLIENT_READ:
        return (io.availableForReading() > 0);
      case CLIENT_WRITE:
        return (totalBytesReceived < fileSize);
      default:
        throw new IllegalStateException("operao "+operation+" desconhecida");
    }
  }

  /**
   * Aguarda o recebimento de um cdigo de <b>operao</b> vlido para 
   * prosseguir na seqncia do protocolo.
   *
   * @return <code>true</code> se um cdigo de <b>operao</b> vlido tiver sido
   *         recebido.
   *         
   * @throws IOException
   */
  @Override
  protected boolean processOperation() throws IOException {
    addReadInterest();
    operation = io.readByte();
    switch (operation) {
      case -1:
        // No conseguiu ler nem um byte ainda. No muda o estado, aguardando a 
        // prxima notificao.
        return false;
      case CLIENT_WRITE:
      case CLIENT_READ:        
        return true;
      default:
        System.err.println("Erro: operao "+operation+" desconhecida.");
        return false;
    }
  }
  
  /**
   * Afirma o interesse em leitura. ATENO: invocar esse mtodo no altera
   * IMEDIATAMENTE o interesse da chave; ele apenas registra esse interesse,
   * que s vai ser efetivamente adicionado  chave quando o mtodo
   * {@link #doAddInterestOps()} for chamado.
   */
  private void addReadInterest() {
    addOps = SelectionKey.OP_READ;
  }

  /**
   * Afirma o interesse em escrita. ATENO: invocar esse mtodo no altera
   * IMEDIATAMENTE o interesse da chave; ele apenas registra esse interesse,
   * que s vai ser efetivamente adicionado  chave quando o mtodo
   * {@link #doAddInterestOps()} for chamado.
   */
  private void addWriteInterest() {
    addOps = SelectionKey.OP_WRITE;
  }
  
  
  /**
   * Aguarda o recebimento do tamanho do caminho do arquivo para 
   * prosseguir na seqncia do protocolo.
   *
   * @return <code>true</code> se o tamanho do caminho do arquivo tiver sido
   *         recebido com sucesso.
   *         
   * @throws IOException
   */
  @Override
  protected boolean processPathSize() throws IOException {
    addReadInterest();
    // TODO validar o tamanho
    filePathSize = io.readInt();
    return (filePathSize != -1);
  }

  /**
   * Aguarda o recebimento do caminho do arquivo para prosseguir na seqncia do 
   * protocolo. Se estiver recebendo, cria um arquivo vazio no caminho indicado.
   * Se estiver enviando, abre um canal de leitura para transmitir o arquivo.
   *
   * @return <code>true</code> se o caminho do arquivo tiver sido
   *         recebido com sucesso.
   *         
   * @throws IOException
   */  
  @Override
  protected boolean processPath() throws IOException {
    addReadInterest();
    String path = io.readString(filePathSize);
    if (path == null) {
      // No conseguiu ler tudo ainda. No muda o estado, aguardando a prxima
      // notificao.
      return false;
    }
    File file = new File(path);
    switch (operation) {
      case CLIENT_WRITE:
        // TODO E se o arquivo j existir? Se for um diretrio?
        io.createTarget(file);    
        break;
      case CLIENT_READ:
        fileSize = file.length();
        io.createSource(file);
        break;
      default:
        // A operao j foi validada anteriormente.
        break;
    }
    return true;
  }

  /**
   * Aguarda o recebimento do tamanho do arquivo (no caso de estar recebendo)
   * ou envia o tamanho (no caso de estar enviando um arquivo), para 
   * prosseguir na seqncia do protocolo.
   *
   * @return <code>true</code> se o tamanho arquivo tiver sido
   *         recebido com sucesso.
   *         
   * @throws IOException
   */  
  @Override
  protected boolean processFileSize() throws IOException {
    switch (operation) {
      case CLIENT_READ:
        addWriteInterest();
        nothingToSend = io.writeLong(fileSize);
        return nothingToSend;
      case CLIENT_WRITE:
        addReadInterest();
        fileSize = io.readLong();
        return (fileSize != -1);
      default:
        return false;
    }
  }

  /**
   * Aguarda o recebimento (ou envia, no caso de estar enviando um arquivo) do 
   * tamanho de um <b>pacote</b> para prosseguir na seqncia do protocolo.
   *
   * @return <code>true</code> se o tamanho de um <b>pacote</b> tiver sido
   *         recebido com sucesso.
   *         
   * @throws IOException
   */  
  @Override
  protected boolean processPackSize() throws IOException {
    switch (operation) {
      case CLIENT_READ:
        addWriteInterest();
        packSize = io.availableForReading();
        packSize = 
          calcPackSizeThatFitsInBuffer(packSize, io.getContentBufferSize());    
        nothingToSend = io.writeInt(packSize);
        return nothingToSend;
      case CLIENT_WRITE:
        addReadInterest();
        packSize = io.readInt();
        if (packSize == -1) {
          // No conseguiu ler tudo ainda. No muda o estado, aguardando a prxima
          // notificao.
          return false;
        }
        if (packSize > fileSize) {
          //TODO [TESTES] Tratar erro
        }
        return true;
      default:
        return false;
    }
  }

  /**
   * Aguarda o recebimento (ou envia, no caso de estar enviando um arquivo) de 
   * um <b>pacote</b> para prosseguir na seqncia do protocolo.
   *
   * @return <code>true</code> se o <b>pacote</b> tiver sido
   *         recebido com sucesso.
   *         
   * @throws IOException
   */  
  @Override
  protected boolean processPack() throws IOException {
    switch (operation) {
      case CLIENT_READ:
        addWriteInterest();
        nothingToSend = io.writePack(packSize);
        return nothingToSend;
      case CLIENT_WRITE:
        addReadInterest();
        long bytesReceived = io.readPack(fileSize, packSize);
        if (bytesReceived == -1) {
          // No conseguiu ler nem um pacote ainda. No muda o estado, aguardando a 
          // prxima notificao.
          return false;
        }
        totalBytesReceived += bytesReceived;
        return true;
      default:
        return false;
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void closeResources() throws IOException {
    io.closeResources();
    long ms = System.currentTimeMillis() - NIOServer.getInitialTime();
    System.out.println(ms+" milissegundos.");//#LEO#
  }
  
  /**
   * Callback para tratar uma notificao de que ocorreu algum evento no canal
   * de comunicao com o cliente.
   * 
   * @throws IOException
   */
  public void handleEvent() throws IOException {
    synchronized (this) {
      processState();
      doAddInterestOps();
    }
  }

  /**
   * Adiciona (se houver) algum novo interesse ao conjunto de interesses atual
   * da chave. Este mtodo funciona em parceria com os mtodos 
   * {@link #addReadInterest()} e {@link #addWriteInterest()}. A justificativa
   *  a seguinte: como o tratamento de eventos  feito em uma thread separada,
   * ao iniciar o tratamento  removido da chave o interesse no evento que
   * disparou o tratador (leitura ou escrita). Se isso no fosse feito, a thread
   * principal continuaria indefinidamente invocando tratadores para o mesmo
   * evento, que iriam competir entre si e quebrar o protocolo. Dessa forma, o
   * tratador fica livre para tratar o evento, e, ao completar seu tratamento,
   * reafirma o interesse no evento que disparou o tratamento (usando os mtodos
   * {@link #addReadInterest()} e/ou {@link #addWriteInterest()}). No ponto mais
   * adequado, este mtodo  invocado e efetivamente modifica o conjunto de
   * interesses da chave, habilitando o seletor a invocar novamente o tratador
   * sempre que ocorrer o evento em questo, at completar-se o protocolo. 
   */
  private void doAddInterestOps() {
    if (addOps == 0) {
      return;
    }
    TransferHandler handler = TransferHandler.getHandler(channel);
    SelectionKey key = handler.getKey(); 
    NIOServer.getInstance().addInterestOps(key, addOps);
    addOps = 0;
  }

  /**
   * {@inheritDoc}. O servidor, por ser no-bloqueante, precisa de uma 
   * interrupo externa do processamento do protocolo se estiver necessitando
   * receber dados do cliente, caso contrrio pode ficar em um loop 
   * desnecessrio.
   */
  @Override
  protected boolean hasToWait() {
    switch (state) {
      case INITIAL:
      case PATH_SIZE:
      case PATH:
        return true;
      case FILE_SIZE:
        return !nothingToSend;
      case PACK_SIZE:
      case PACK:
        return (operation == CLIENT_WRITE) || !nothingToSend;
      default:
        return false;
    }
  }
  
}
