package nio.server;

import java.io.IOException;
import java.nio.channels.SelectionKey;
import java.nio.channels.SocketChannel;
import java.util.HashMap;
import java.util.Map;

/**
 * Gerenciador de transferncia. Cada conexo aceita com um cliente ser tratada
 * por um objeto desta classe.
 * 
 * @author Tecgraf
 */
public class TransferHandler implements Runnable {
  
  /**
   * Coleo de referncias para os gerenciadores ativos (um para cada 
   * importao). 
   */
  private static Map<SocketChannel, TransferHandler> activeImports = 
    new HashMap<SocketChannel, TransferHandler>();
  
  /** Canal de origem. */
  private SocketChannel sourceChannel;
  /** Objeto que implementa o protocolo de comunicao no servidor. */
  private ServerSimpleFileTransferProtocol protocol;
  /** 
   * Chave representando a ligao entre um canal cliente e o conjunto de 
   * operaes de interesse.
   */
  private SelectionKey key;

  /**
   * Cria um gerenciador associado ao canal especificado. Este  iniciado no
   * estado <code>GettingSize</code> (pegando o tamanho do arquivo).
   * 
   * @param bufferCapacity capacidade do buffer de transmisso de contedo.
   * @param key chave representando a ligao entre o canal com o cliente e o
   *        conjunto de interesses.
   */
  public TransferHandler(int bufferCapacity, SelectionKey key) {
    this.key = key;
    sourceChannel = (SocketChannel)key.channel();
    NIOServer.setInitialTime();//@debug
    protocol = 
      new ServerSimpleFileTransferProtocol(bufferCapacity, sourceChannel);
  }
  
  /**
   * Delega ao objeto que implementa o protocolo o tratamento do evento 
   * ocorrido.
   */
  public void run() {
    try {
      protocol.handleEvent();
    } 
    catch (IOException e) {
      // TODO Logar no CSBase
      e.printStackTrace();
    }
    if (protocol.isFinished()) {
      close();
    }
  }

  /**
   * Finaliza a importao, fechando todos os recursos abertos e removendo o 
   * gerenciador da coleo de gerenciadores ativos.
   */
  public void close() {
    activeImports.remove(sourceChannel);
  }

  /**
   * Obtm a chave representando a ligao entre o canal com o cliente e o
   * conjunto de interesses.
   * 
   * @return chave.
   */
  SelectionKey getKey() {
    return key;
  }
  
  /**
   * Cria um novo gerenciador para tratar eventos de leitura do canal 
   * especificado.
   * 
   * @param capacity capacidade do buffer de transmisso de contedo.
   * @param key chave representando a ligao entre o canal com o cliente e o
   *        conjunto de interesses. 
   */
  public static void createHandler(int capacity, SelectionKey key) {
    activeImports.put(
      (SocketChannel)key.channel(), new TransferHandler(capacity, key));
  }

  /**
   * Obtm o gerenciador associado ao canal especificado.
   * 
   * @param channel canal com dados a serem lidos.
   * 
   * @return gerenciador que ir tratar da leitura no canal especificado.
   */
  public static TransferHandler getHandler(SocketChannel channel) {
    return activeImports.get(channel);
  }
}
