/*
 * ORBHandler.java
 *
 * $Author: clinio $
 * $Revision: 33439 $ - $Date: 2004-02-18 16:57:32 -0300 (Wed, 18 Feb 2004) $
 */

import org.omg.CORBA.*;
import org.omg.PortableServer.*;

/**
 * A classe <code>ORBHandler</code> implementa um servidor
 * CORBA persistente, isto , um servidor cujos objetos
 * TESTE
 * possuem referncias persistentes.
 */
public class ORBHandler extends Thread {

  /**
   * Instncia de ORB utilizada pelo servidor
   */
  private ORB orb = null;

  /**
   * Root POA
   */
  private POA rootPOA = null;

  /**
   * POA persistente para os objetos CORBA ativados pelo servidor
   */
  private POA persistentPOA = null;

  /**
   * Indica se o servidor CORBA est ativo.
   * O servidor CORBA associado a uma instncia de ORBHandler  
   * considerado desativado (por precauo) aps a execuo de um shutdown.
   */
   private boolean isActive;

  /**
   * Constri um servidor CORBA persistente associado ao nome e porta
   * fornecidos como parmetros.
   *
   * Uma instncia de ORB e um POA persistente so criados para este servidor.
   *
   * @param serverPort nmero da porta utilizada pelo servidor CORBA
   * @param serverName nome do servidor persistente.
   */ 
  public ORBHandler(String serverPort, String serverName, String ipAddress) 
  throws Exception {
    orb = null;
    rootPOA = null;
    persistentPOA = null;
    init(serverPort, serverName, ipAddress);
    isActive = true;
  }

  /**
   * Inicializa o servidor CORBA persistente.
   *
   * Instancia um ORB para o servidor e cria um POA persistente
   * para os objetos CORBA que sero associados ao servidor.
   * O nome do servidor persistente, fornecido como parmetro, ser
   * utilizado para nomear o POA persistente. Em alguns ORBs (como
   * o JacORB) tanto o nome do servidor como o nome do POA so
   * utilizados na composio das referncias aos objetos CORBA.
   *
   * A atribuio de uma porta "bem conhecida" ao servidor (fornecida 
   * tambm como parmetro) permite a construo de referncias aos objetos
   * CORBA ativados pelo servidor sem que seja necessria a utilizao de
   * um servio de nomes.
   * 
   * @param  serverPort   nmero da porta do servidor
   * @param  serverName   nome do servidor persistente
   * @throws Exception  
   */
  private void init(String serverPort, String serverName, String ipAddress) 
  throws Exception {

    // Cria as propriedades para configurao do ORB
    java.util.Properties props = new java.util.Properties();
    props.setProperty("org.omg.CORBA.ORBClass", "org.jacorb.orb.ORB");
    props.setProperty("org.omg.CORBA.ORBSingletonClass",
                      "org.jacorb.orb.ORBSingleton");

    // Fixa a porta e o nome do servidor persistente
    props.setProperty("OAPort",serverPort);
    if (ipAddress != null) props.setProperty("OAIAddr",ipAddress);
    props.setProperty("jacorb.implname",serverName);

    // Instancia o ORB e cria um POA persistente
    try {
      String[] args = null;
      orb = ORB.init(args, props);
      rootPOA = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));

      org.omg.CORBA.Policy[] policies = new org.omg.CORBA.Policy[2];
      policies[0] = rootPOA.create_id_assignment_policy(
                           IdAssignmentPolicyValue.USER_ID);
      policies[1] = rootPOA.create_lifespan_policy(
                           LifespanPolicyValue.PERSISTENT);
      persistentPOA = rootPOA.create_POA(serverName + "POA", 
                           rootPOA.the_POAManager(), policies);

      for (int i=0; i<policies.length; i++)
        policies[i].destroy();

      persistentPOA.the_POAManager().activate();
 
      // Cria a thread fornecida ao ORB para atendimento s 
      // requisies de servio aos objetos CORBA
      start();
    }
    catch (org.omg.CORBA.UserException e) {
      throw new Exception("Erro na inicializao de servidor CORBA", e);
    }
  }

  /**
   * Ativa um objeto CORBA associado ao servidor.
   * @param  objectID  identificao do objeto
   * @param  object    implementao do objeto (servant)
   * @throws Exception  
   */
  public void activateObject (String objectID, Servant object) 
  throws Exception {
    // Verifica se o servidor est ativo
    if (!isActive)
      throw new Exception("Servidor CORBA desativado");
      
    // Ativa o objeto
    try {
      persistentPOA.activate_object_with_id(objectID.getBytes(), object);
    }
    catch (org.omg.CORBA.UserException e) {
      throw new Exception("Erro na ativao de um objeto CORBA", e);
    }
  }

  /**
   * Desativa um objeto CORBA associado ao servidor.
   * @param  objectID  identificao do objeto
   * @throws Exception  
   */
  public void deactivateObject (String objectID)
  throws Exception {
    // Verifica se o servidor est ativo
    if (!isActive)
      throw new Exception("Servidor CORBA desativado");
      
    // Desativa o objeto
    try {
      persistentPOA.deactivate_object(objectID.getBytes());
    }
    catch (org.omg.CORBA.UserException e) {
      throw new Exception("Erro na desativao de um objeto CORBA", e);
    }
  }

  /**
   * Dedica uma thread ao procedimento de atendimento a requisies de servio
   * do ORB.
   *
   * Ao receber um comando de shutdown, o ORB devolve o controle. Nesse
   * caso, a instncia de ORB ser destruda.
   * 
   */
  public void run() {
    System.out.println("Thread servidor iniciada");
    orb.run();
    orb.destroy();
    System.out.println("Thread servidor finalizada");
  }

  /**
   *
   */
  public org.omg.CORBA.Object stringToObject(String ior) {
     return orb.string_to_object(ior);
  }

  /**
   * Solicita a desativao do servidor CORBA.
   *
   * Comanda o shutdown do ORB instanciado para o servidor, o que provocar
   * a terminao da thread dedicada ao ORB.
   *
   * @throws Exception  
   */
  public void shutdown() throws Exception {

    // Verifica se o servidor est ativo
    if (!isActive)
      throw new Exception("Servidor CORBA desativado");
      
    // Comanda o shutdown do ORB
    orb.shutdown(false);
    isActive = false;
  }
}
