/**
 * @file
 * 
 */

import sgaidl.*;

import java.util.*;

import org.omg.CORBA.IntHolder;

/**
 * A classe <code>SGABridgeManager</code> implementa a ponte
 * para a gerncia do SGA.
 */
public class SGABridgeManager extends SGAManagerPOA {

  private String id;
  /**
   * Referncia para a implementao do servio SGAManager
   */
  private SGAManager service = null;
  private SGAManager servant = null;

  /**
   * Hashtable que contm os SGA's cadastrados no SSI (servio)
   */
  private Hashtable registeredSGAs = null;

  /**
   * Constri um servant para a interface SGABridgeManager
   *
   * @param srv referncia para a implementao do servio SGABridgeManager
   */
  public SGABridgeManager(SGAManager server) {
     this.service = server;
     this.registeredSGAs = new Hashtable();
  }

  /**
   * SGA avisa ao Manager que est ativo.
   *
   * @param sga referncia para o SGA
   * @param hostname nome da mquina hospedeira do SGA
   * @param staticNodesInfo informaes estticas dos ns
   * @param updateInterval informa de quanto em quanto tempo o SGA
   *        deve dizer que est vivo.
   *
   * @return true se o SGA foi registrado com sucesso junto ao SSI
   */
  public boolean registerSGA(SGAServer sga, String hostname, 
    StaticNodeInfo[] staticNodesInfo, IntHolder updateInterval) {
    SGABridgeServer sgab_new;
    if (service == null) return false;
    try {
       sgab_new = new SGABridgeServer( sga);
       sgab_new.activate();
    }
    catch (Exception e) {
       System.out.println("No foi possvel criar o objeto SGABridgeServer.");
       return false;
    }
    try {
       if (service.registerSGA(sgab_new.getServant(), hostname,
         staticNodesInfo, updateInterval)) {
         // Verifica se o SGA j tinha se registrado
         SGABridgeServer sgab = getSGABridgeServer(hostname);
         if (sgab != null) {
             sgab.deactivate();
             removeSGABridgeServer( hostname);
         }
         putSGABridgeServer( hostname, sgab_new );
         return true;
       }
    }
    catch (Exception e) {
       System.out.println("No foi possvel registrar o objeto SGABridgeServer.");
       return false;
    }
    return false;
  }

  /**
   * SGA avisa ao Manager que sair do ar
   *
   * @param sga referncia para o SGA
   * @param hostname nome da mquina hospedeira do SGA
   */
  public void unregisterSGA(SGAServer sga, String hostname) {
    removeSGABridgeServer( hostname);
    if (service == null) return;
    // Verifica se o SGA j tinha se registrado
    SGABridgeServer sgab = getSGABridgeServer(hostname);
    if (sgab != null) {
      SGAServer registered_sga = sgab.getService();
      if (registered_sga.equals(sga)) {
        sgab.deactivate();
        service.unregisterSGA(sgab.getServant(), hostname);
        removeSGABridgeServer( hostname);
      }
    }
  }

  /**
   * SGA avisa ao Manager que est apto a receber novos comandos
   *
   * @param sga referncia para o SGA
   * @param hostname nome da mquina hospedeira do SGA
   */
  public void setSGAEnabled(SGAServer sgad, String hostname) {
    if (service == null) return;
    SGAServer sga = getSGAServer(hostname);
    if (sga == null) return;
    service.setSGAEnabled(sga, hostname);
  }

  /**
   * SGA avisa ao Manager que no est apto a receber novos comandos,
   * apesar de continuar ativo.
   *
   * @param sga referncia para o SGA
   * @param hostname nome da mquina hospedeira do SGA
   */
  public void setSGADisabled(SGAServer sgad, String hostname) {
    if (service == null) return;
    SGAServer sga = getSGAServer(hostname);
    if (sga == null) return;
    service.setSGADisabled(sga, hostname);
  }

  /**
   * SGA avisa ao manager que um comando foi recuperado.
   * @param sganame nome da mquina hospedeira do SGA
   * @param cmds comandos recuperados pelo SGA.
   * @return true se os comandos foram registrados com sucesso junto ao SSI
   */ 
  public boolean commandRetrieved( String hostName,
    RetrievedInfo[] cmds) {
    if (service == null) return false;
    try {
       SGAServer sga = getSGAServer(hostName);
       if (sga == null) return false;
       for (int i=0; i < cmds.length; i++) {
         SGACommand oldRef = cmds[i].cmdRef;
         SGABridgeCommand cmdb = new SGABridgeCommand(oldRef, sga);
         cmdb.activate();
         SGACommand cmdServant = cmdb.getServant();
         cmds[i].cmdRef = cmdServant;
       }
       return service.commandRetrieved(hostName, cmds);
    } 
    catch (Exception e) {
       System.out.println("No foi possvel criar o objeto " +
         "SGABridgeCommand ao recuperar comando.");
       return false;
    }
  }

  /**
   * SGA avisa ao Manager que um comando foi perdido.
   *
   * @param sga referncia para o SGA
   * @param cmd referncia para o gerenciador do comando
   * @param info informaes sobre o trmino de execuo do comando
   * @return true se o Manager aceitou a notificao
   *         false se o Manager desconhece o SGA 
   *          (pode acontecer se o Manager esteve fora do ar, 
   *           o que exige um novo registro do SGA)
   */
  public boolean commandLost(String hostName, String cmdId,
    String cmdString) {
    if (service == null) return false;
    SGAServer sga = getSGAServer(hostName);
    if (sga == null) return false;
    return service.commandLost(hostName, cmdId, cmdString);
  }

  /**
   * SGA avisa ao Manager que um comando terminou
   *
   * @param sga referncia para o SGA
   * @param cmd referncia para o gerenciador do comando
   * @param info informaes sobre o trmino de execuo do comando
   */
  public boolean commandCompleted(String hostName, SGACommand cmd,
    String cmdId, CompletedCommandInfo info, String cmdString) {
    if (service == null) return false;
    SGABridgeServer sgab = getSGABridgeServer(hostName);
    if (sgab == null) return false;
    SGABridgeCommand cmdb = sgab.getBridgeCommand(cmd);
    // Em caso de falha do SGA, o comando pode j ter terminado.
    if (cmdb == null) {
      try {
        SGAServer sga = getSGAServer(hostName);
        if (sga == null) return false;
        cmdb = new SGABridgeCommand(cmd, sga);
        cmdb.activate();
      }
      catch (Exception e) {
         System.out.println("No foi possvel criar o objeto "+
           "SGABridgeCommand ao terminar comando.");
         return false;
      }
    }
    SGACommand cmdServant = cmdb.getServant();
    //  estranho, mas por enquanto vamos desativar os comandos aqui.
    cmdb.deactivate();
    return service.commandCompleted(hostName, cmdServant, cmdId, info,
      cmdString);
  }

  /**
   * SGA atualiza seus dados dinmicos.
   *
   * @param sga referncia para o SGA
   * @param sgaName nome da mquina hospedeira do SGA
   * @param dynamicInfo informaes dinmicas do SGA
   *
   * @return true
   */
  public boolean updateSGAInfo(SGAServer sgad, String hostname,
    SGADynamicInfo dynamic_data) {
    SGABridgeServer sga = getSGABridgeServer(hostname);
    if (sga == null) return false;
    if (service == null)
       return false;
    if (!sgad.equals(sga.getService())) {
       return false;
    }
    return service.updateSGAInfo(sga.getServant(), hostname,
      dynamic_data);
  }

   /**
    * SGA verifica se o SSI est acessvel e se seu registro est vlido.
    *
    * @param sga referncia para o SGA
    * @param sgaName nome da mquina hospedeira do SGA
    *
    * @return true
    */
   public boolean isRegistered(SGAServer sgad, String hostname) {
     SGABridgeServer sga = getSGABridgeServer(hostname);
     if (sga == null) return false;
     if (service == null)
       return false;
     if (!sgad.equals(sga.getService())) {
        return false;
     }
     return service.isRegistered(sga.getServant(), hostname);
   }

  public SGAServer getSGAServer(String hostName) {
     SGABridgeServer sgab = getSGABridgeServer(hostName);
     if (sgab == null) return null;
     return  sgab.getServant();
  }

  public SGABridgeServer getSGABridgeServer(String hostName) {
     return (SGABridgeServer) registeredSGAs.get(hostName);
  }

  public void putSGABridgeServer(String hostName, SGABridgeServer sgab) {
     registeredSGAs.put(hostName, sgab);
  }

  public void removeSGABridgeServer(String hostName) {
     registeredSGAs.remove(hostName);
  }

  public void activate(String id) throws Exception {
     this.id = id;
     SGABridge.getInstance().activateObjectAtSGAORB(id, this);
     this.servant = SGAManagerHelper.narrow(this._this());
  }

  public void deactivate() {
     try {
       SGABridge.getInstance().deactivateObjectAtSGAORB(this.id);
     }
     catch (Exception e) {
       System.out.println("No foi possvel desativar o servidor "+
         this.id);
     }
     this.servant = null;
  }

  public SGAManager getServant() {
     return this.servant;
  }

}

