/*
 * @file
 *
 */

import sgaidl.*;

import java.util.*;

/**
 * A classe <code>SGABridgeServer</code> implementa a ponte
 * para o servidor SGA.
 */
public class SGABridgeServer extends SGAServerPOA {

  private static Integer id_counter=new Integer(0);
  private String id;
  private Set<SGABridgeCommand> commands;

  /**
   * Referncia para a implementao do servio SGAServer
   */
  private SGAServer service = null;
  private SGAServer servant = null;

  /**
   * Constri um servant para a interface SGABridgeServer
   *
   * @param srv referncia para a implementao do servio SGABridgeServer
   */
  public SGABridgeServer( SGAServer srv ) {
    this.service = srv;
    this.commands = new HashSet<SGABridgeCommand>();
  }

  /**
   * Verifica se o SGA representa um cluster
   *
   * @return true
   */
  public boolean isCluster() {
    if (service == null) return false;
    return service.isCluster();
  }

  /**
   * Verifica se o SGA est ativo
   * @return true
   */
  public boolean isAlive() {
    if (service == null) return false;
    return service.isAlive();
  }

  /**
   * Verifica se o SGA possui um determinado requisito externo
   * que o habilite para execuo de um comando.
   *
   * @param requirement requisito externo a ser verificado.
   * @return true se o SGA possui tal requisito.
   */
  public boolean meetsRequirement(String requirement) {
    if (service == null) return false;
    return service.meetsRequirement(requirement);
  }

  /**
   * Indica se a capacidade de transferncia do SGA deve ser calculada,
   *  ilimitada ou bastante limitada.
   *
   * @return SGANetCapacity.CALC se a capacidade de transferncia do SGA deve 
   *         ser calculada
   *         SGANetCapacity.ALL se a capacidade de transferncia do SGA  
   *         ilimitada
   *         SGANetCapacity.NO se a capacidade de transferncia do SGA  
   *         limitada
   */
  public SGANetCapacity execNetBench() {
    if (service == null) return SGANetCapacity.NO;
    return service.execNetBench();
  }

  /**
   * Verifica se o SGA est apto a receber comandos.
   *
   * @return true se o SGA pode receber requisies de execuo
   */
  public boolean isEnabled() {
    if (service == null) return false;
    return service.isEnabled();
  }

  /**
   * Obtm os nomes das mquinas gerenciadas pelo SGA.
   *
   * @remark Este mtodo deve ser utilizado somente quando
   * tem-se a configurao de cluster.
   * @return uma seqncia com os nomes dos ns (na mesma ordem
   * das seqncias de memria, monitorao e CPU).
   */
  public String[] getNodeNames() {
    if (service == null) return null;
    return service.getNodeNames();
  }

  /**
   * Obtm a identificao da mquina hospedeira
   *
   * @return nome (hostname) da mquina hospedeira do SGA
   */
  public String getHostName() {
    if (service == null) return null;
    return service.getHostName();
  }

  /**
   * Obtm a identificao da mquina que deve ser contactada
   * para transferncias via CSFS.
   *
   * @return nome (hostname) da mquina para transferncias via CSFS
   */
  public String getCSFSHost() {
    if (service == null) return null;
    return service.getCSFSHost();
  }

  /**
   * Obtm o diretrio raz usado para transferncias via CSFS.
   *
   * @return diretrio
   */
  public String getCSFSRootDir() {
    if (service == null) return null;
    return service.getCSFSRootDir();
  }

  /**
   * Requisio de execuo de um comando.
   *
   * @param command comando a ser executado
   * @param cmdId identificador do comando a ser executado
   * @param hostName Nome da mquina hospedeira
   * @param path path de execuo do comando
   * @param outputPath diretorio de output do comando
   * @return referncia para o comando
   */
  public SGACommand executeCommand( String command, String cmdId,
    String hostName, String path, String outputPath) {
    if (service == null) return null;
    try {
       SGACommand cmd = service.executeCommand(command, cmdId, hostName, path, outputPath);
       SGABridgeCommand cmdb = new SGABridgeCommand(cmd, this.getServant());
       cmdb.activate();
       commands.add(cmdb);
       return cmdb.getServant();
    }
    catch (Exception e) {
       System.out.println("No foi possvel criar o objeto "+
         "SGABridgeCommand ao executar comando.");
       return null;
    }
  }
      
  /**
   * Solicita a finalizao do daemon
   */
  public void kill(int status) {
    if (service == null) return;
    service.kill(status);
  }

  public void activate() throws Exception {
     synchronized (id_counter) {
       ++id_counter;
       this.id = id_counter.toString();
     }
     SGABridge.getInstance().activateObjectAtServerORB(id, this);
     this.servant = SGAServerHelper.narrow(this._this());
  }

  public void deactivate() {
     for (SGABridgeCommand cmdb : commands) {
       cmdb.deactivate();
     }
     commands.clear();
     try {
       SGABridge.getInstance().deactivateObjectAtServerORB(this.id);
     }
     catch (Exception e) {
       System.out.println("No foi possvel desativar o sga "+ this.id);
     }
     this.servant = null;
  }

  public SGABridgeCommand getBridgeCommand(SGACommand cmd) {
     for (SGABridgeCommand cmdb : commands) {
       if (cmdb.getService().equals(cmd)) {
         return cmdb;
       }
     }
     return null;
  }

  public SGAServer getServant() {
     return this.servant;
  }

  public SGAServer getService() {
     return this.service;
  }
  
  public boolean checkExistence(String filename) {
    if (service == null) return false;
    return service.checkExistence(filename);
 }
}
