/**
 * TestLoginService.java $Author: taciana $ $Revision: 34156 $ - $Date:
 * 2003/11/24 10:05:38 $
 */
package server.services;

import csbase.logic.*;
import csbase.remote.RemoteLocator;
import csbase.server.services.loginservice.LoginService;

import junit.framework.TestSuite;

import util.*;

import java.rmi.RemoteException;

/**
 * A classe <code>TestLoginService</code>  responsvel por testar o servio de
 * um usurio se logar no sistema e conseguir tambm ser deslogado.
 */
public class TestLoginService extends CSBaseAbstractTestCase {
  /**
   * Os seguintes testes foram realizados:  - logar usurios (vlido) OK -
   * deslogar usurio ( vlido, recem logado ) OK - obter o usurio pela chave
   * ( key vlida ) OK - logar usurio ( com login NULL ) OK - logar usurio (
   * com senha NULL )                            FAILED - logar usurio ( j
   * logado )                                 OK - logar usurio ( login
   * inexistente )                         OK - logar usurio ( login vlido,
   * senha errada )                OK - deslogar usurio ( Key NULL )
   * OK - deslogar usurio ( Key invlida )
   * OK - deslogar usurio ( Key de um usurio nao logado )
   * OK - obter o usurio pela chave ( key NULL ) FAILED - obter o usurio
   * pela chave ( key inexistente ) OK - obter o usurio pela chave ( key de
   * um usurio no logado ) OK
   */
  /** Usurios logados no sistema. */
  private User[] loggedUser;
  private static Object loggedKey;

  public TestLoginService() {}

  public TestLoginService(String test) {
    super(test);
  }

  /**
   *  executado sempre antes de um teste ser rodado.
   *
   * @throws Exception .
   * @throws RemoteException .
   */
  public void setUp() throws Exception, RemoteException {
    initializeUser(ELEMENTS);
    loggedUser = new User[ELEMENTS];
    for (int i = 0; i < ELEMENTS; i++) {
      loggedUser[i] =
        RemoteLocator.loginService.login(
          user[i].getLogin(), user[i].getPasswordDigest());
    }
  }

  /**
   *  executado toda a vez que um teste  terminado.
   *
   * @throws Exception .
   * @throws RemoteException .
   */
  public void tearDown() throws Exception, RemoteException {
    for (int i = 0; i < ELEMENTS; i++) {
      if (loggedUser[i] != null) {
        User user =
          RemoteLocator.loginService.getUserByKey(loggedUser[i].getKey());
        if (user != null) {
          RemoteLocator.loginService.logout(loggedUser[i].getKey());
        }
      }
    }
    cleanUser(ELEMENTS);
  }

  /**
   * Testa se um usurio cadastrado no sistema consegue fazer o login.
   *
   * @throws RemoteException .
   */
  public void testValidLogin() throws RemoteException {
    for (int i = 0; i < ELEMENTS; i++) {
      /*  verifico que a chave foi criada */
      assertNotNull(loggedUser[i].getKey());
      /* certifico que o usurio foi realmente logado */
      loggedUser[i] =
        RemoteLocator.loginService.getUserByKey(loggedUser[i].getKey());
      assertEquals(user[i], loggedUser[i]);
    }
  }

  /**
   * Testa se um usurio recem logado consegue deslogar do sistema.
   *
   * @throws RemoteException .
   */
  public void testValidLogout() throws RemoteException {
    User logoffUser;
    for (int i = 0; i < ELEMENTS; i++) {
      loggedUser[i] =
        RemoteLocator.loginService.getUserByKey(loggedUser[i].getKey());
      RemoteLocator.loginService.logout(loggedUser[i].getKey());
      logoffUser =
        RemoteLocator.loginService.getUserByKey(loggedUser[i].getKey());
      assertNull(logoffUser);
    }
  }

  /**
   * Testa a obteno de um usurio logado dada uma chave de identificao.
   *
   * @throws RemoteException .
   */
  public void testGetValidKey() throws RemoteException {
    User user;
    user = RemoteLocator.loginService.getUserByKey(loggedUser[0].getKey());
    assertNotNull(user);
    assertEquals(user.getName(), loggedUser[0].getName());
    assertEquals(user.getLogin(), loggedUser[0].getLogin());
  }

  /**
   * Tenta logar um usurio passando um login nulo.
   *
   * @throws RemoteException .
   */
  public void testLoginNullUser() throws RemoteException {
    User user = RemoteLocator.loginService.login(null, "abc");
    assertNull(user);
  }

  /**
   * Tenta logar um usurio vlido passando uma senha nula.
   *
   * @throws RemoteException .
   */
  public void testLoginNullPassword() throws RemoteException {
    User user = RemoteLocator.loginService.login("admin", null);
    assertNull(user);
  }

  /**
   * Testa se  possvel um usurio logado consegue se logar novamente. Abrir
   * uma nova seo.
   *
   * @throws RemoteException .
   */
  public void testLoginAlreadyLogged() throws RemoteException {
    User user =
      RemoteLocator.loginService.login(
        loggedUser[0].getLogin(), loggedUser[0].getPasswordDigest());
    assertNotNull(user);
    assertEquals(user.getName(), loggedUser[0].getName());
    assertEquals(user.getLogin(), loggedUser[0].getLogin());
  }

  /**
   * Tenta logar um usurio que nao existe no sistema.
   *
   * @throws RemoteException .
   */
  public void testLoginInvalidUser() throws RemoteException {
    User user = RemoteLocator.loginService.login("palhao", "pipoca");
    assertNull(user);
  }

  /**
   * Tenta logar um usurio vlido com uma senha errada.
   *
   * @throws RemoteException .
   */
  public void testLoginWrongPassword() throws RemoteException {
    User user =
      RemoteLocator.loginService.login(loggedUser[0].getLogin(), "abacaxi");
    assertNull(user);
  }

  /**
   * Tenta realizar um logout passando uma chave de indentificao nula. Teste
   * NAO PASSA. public void testLogoutNullPassword() throws RemoteException {
   * RemoteLocator.loginService.logout( null ); // testo se todos ainda estao
   * logados   testValidLogin(); }
   *
   * @throws RemoteException .
   */
  /**
   * Testa realizar um logout passando uma chave que nao existe.
   *
   * @throws RemoteException .
   */
  public void testLogoutInvalidKey() throws RemoteException {
    Object invalidKey = loggedUser[0].getKey() + "abcv";
    RemoteLocator.loginService.logout(invalidKey);
    /* testo se todos ainda estao logados */
    testValidLogin();
  }

  /**
   * Tento realizar um logout de um usurio no logado.
   *
   * @throws RemoteException .
   */
  public void testLogoutUnloggedUser() throws RemoteException {
    /* deslogo o primeiro usurio */
    RemoteLocator.loginService.logout(loggedUser[0].getKey());
    assertNotNull(loggedUser[0].getKey());
    /* tento deslogar o primeiro usurio novamente */
    RemoteLocator.loginService.logout(loggedUser[0].getKey());
    /* testa se os demais usurios esto logados ainda */
    for (int i = 0; i < ELEMENTS; i++) {
      assertNotNull(loggedUser[i].getKey());
      loggedUser[i] =
        RemoteLocator.loginService.getUserByKey(loggedUser[i].getKey());
      /* sabemos que este usurio no esta mais logado */
      if (i != 0) {
        assertEquals(user[i], loggedUser[i]);
      }
    }
  }

  /**
   * Tento obter um usurio supostamente logado passando uma chave nula. Teste
   * NAO PASSA. public void testGetUserNullKey() throws RemoteException { User
   * user = RemoteLocator.loginService.getUserByKey( null ); assertNull( user
   * ); }
   *
   * @throws RemoteException .
   */
  /**
   * Tento obter um usurio supostamente logado passando uma chave invlida.
   *
   * @throws RemoteException .
   */
  public void testGetUserInvalidKey() throws RemoteException {
    Object key = loggedUser[0].getKey() + "abans";
    User user = RemoteLocator.loginService.getUserByKey(key);
    assertNull(user);
  }

  /**
   * Tento obter um usurio passando uma chave vlida de um usurio que  no
   * esta logado.
   *
   * @throws RemoteException .
   */
  public void testGetUserNonLoggedKey() throws RemoteException {
    Object key = loggedUser[0].getKey();
    RemoteLocator.loginService.logout(key);
    assertNotNull(key);
    User user = RemoteLocator.loginService.getUserByKey(key);
    assertNull(user);
  }

  /**
   * Mantem um usurio logado, para testar se este continuara logado caso  o
   * servidor caia e levante novamente.
   *
   * @throws RemoteException .
   * @throws Exception .
   */
  public void loginBeforeCrash() throws RemoteException, Exception {
    UNInfo unInfoPoli = new UNInfo("Polinesia", "UN Paradisiaca");

    //??Alterado para mudana de permission
    System.out.println("??TestLoginService:282");
    RoleInfo roleInfoLula =
      new RoleInfo("XXX", "Exterminador do Futuro 3", null);
    UN tempUn = UN.createUN(unInfoPoli);
    Role tempRole = Role.createRole(roleInfoLula);
    initializeUser(tempUn, tempRole, 0, 1);
    /* loga o usurio no sistema */
    User user =
      RemoteLocator.loginService.login(
        ((User)userList.get(0)).getLogin(),
        ((User)userList.get(0)).getPasswordDigest());
  }

  /**
   * Testa se o usurio que foi logado antes do servidor sofrer shutdown
   *
   * @throws RemoteException .
   */
  public void loginAfterCrash() throws RemoteException {
    assertNotNull(loggedKey);
    User user = RemoteLocator.loginService.getUserByKey(loggedKey);
    assertNull(user);
  }

  /**
   * Preenche o suite com todos os testes a serem rodados antes do servidor
   * sofrer shutdown.
   *
   * @param suite .
   */
  public void callTestsBeforeCrash(TestSuite suite) {
    suite.addTest(new TestLoginService("testValidLogin"));
    suite.addTest(new TestLoginService("testValidLogout"));
    suite.addTest(new TestLoginService("testGetValidKey"));
    suite.addTest(new TestLoginService("testLoginNullUser"));
    suite.addTest(new TestLoginService("testLoginNullPassword"));
    suite.addTest(new TestLoginService("testLoginAlreadyLogged"));
    suite.addTest(new TestLoginService("testLoginInvalidUser"));
    suite.addTest(new TestLoginService("testLoginWrongPassword"));
    suite.addTest(new TestLoginService("testLogoutNullPassword"));
    suite.addTest(new TestLoginService("testLogoutInvalidKey"));
    suite.addTest(new TestLoginService("testLogoutUnloggedUser"));
    suite.addTest(new TestLoginService("testGetUserNullKey"));
    suite.addTest(new TestLoginService("testGetUserInvalidKey"));
    suite.addTest(new TestLoginService("testGetUserNonLoggedKey"));
    suite.addTest(new TestLoginService("loginBeforeCrash"));
  }
}
