package util;

import csbase.client.ServerMonitor;
import csbase.client.remote.ProjectObserver;
import csbase.logic.*;
import csbase.remote.RemoteLocator;

import junit.framework.TestCase;

import java.io.*;
import java.rmi.RemoteException;
import java.util.*;

/**
 * Classe utilitria para simplificar os testes do CSBase.
 */
public abstract class CSBaseTestCase extends TestCase {
  /** String "dummy" para ser utilizada nos testes */
  protected static final String TEST_KEY = "test";
  private static int testNumber = 0;
  private static String currentTestClass;

  public CSBaseTestCase(String name) {
    super(name);
  }

  public CSBaseTestCase() {}

  private boolean isFirstTest() {
    if (currentTestClass == null) {
      currentTestClass = getClassName();
    }
    else {
      if (!currentTestClass.equals(getClassName())) {
        currentTestClass = getClassName();
        testNumber = 0;
      }
    }
    if (++testNumber == 1) {
      return true;
    }
    return false;
  }

  /**
   * Mtodo chamado antes de cada teste. Exibe o nome do teste sendo executado.
   * Para obter este comportamento, os descendentes devem usar o mtodo
   * initialize e no mais o mtodo setUp do JUnit.
   *
   * @throws Exception .
   */
  protected void setUp() throws Exception {
    if (isFirstTest()) {
      System.out.println("----------> " + getClassName());
    }
    System.out.println("Executando " + getName() + "...");
    assertListSize(0);
    initialize();
  }

  /**
   * Mtodo de inicializao para os descendentes da classe
   *
   * @throws Exception .
   */
  protected void initialize() throws Exception {}

  /**
   * Mtodo chamado depois de cada teste. Este mtodo foi implementado desta
   * forma para garantir coerncia com o mtodo setUp (ver acima), que possui
   * um comportamento desejado em todos seus descendentes. Garante que a
   * devida limpeza de recursos seja feita aps cada teste.
   *
   * @throws Exception .
   */
  protected void tearDown() throws Exception {
    ensureClean();
    terminate();
  }

  /**
   * Mtodo de finalizao para os descendentes da classe
   *
   * @throws Exception .
   */
  protected void terminate() throws Exception {}

  /**
   * Cria uma Unidade de Negcio fictcia para testes, usando para
   * simplificao a chave repassada em todos os campos da UN.
   *
   * @param key Chave que ser usada para criar e armazenar a un na tabela.
   *
   * @return UN A Unidade de Negcio criada.
   *
   * @throws Exception .
   */
  protected static UN createTestUN(String key) throws Exception {
    if (key == null) {
      return null;
    }
    UNInfo unInfo = new UNInfo(key, key);
    UN testUN = UN.createUN(unInfo);
    return testUN;
  }

  /**
   * Cria um Perfil fictcio para testes, usando para simplificao a chave
   * repassada em todos os campos do Perfil.
   *
   * @param key Chave que ser usada para criar e armazenar o perfil na tabela.
   *
   * @return Role O Perfil criado.
   *
   * @throws Exception .
   */
  protected static Role createTestRole(String key) throws Exception {
    if (key == null) {
      return null;
    }

    //??Alterado para mudana de permission
    System.out.println("??CSBaseTestCase:95");
    RoleInfo roleInfo = new RoleInfo(key, key, null);
    Role testRole = Role.createRole(roleInfo);
    return testRole;
  }

  /**
   * Cria uma Plataforma fictcia para testes, usando para simplificao a
   * chave repassada em todos os campos da Plataforma.
   *
   * @param key Chave que ser usada para criar e armazenar a plataforma.
   *
   * @return Platform A Plataforma criada.
   *
   * @throws Exception .
   */
  protected Platform createTestPlatform(String key) throws Exception {
    if (key == null) {
      return null;
    }
    PlatformInfo platInfo = new PlatformInfo(key, key);
    Platform testPlatform = Platform.createPlatform(platInfo);
    return testPlatform;
  }

  /**
   * Cria um Usurio fictcio para testes, usando para simplificao a chave
   * repassada em todos os campos de valores do Usurio, com exceo das
   * referncias  Unidade de Negcio e ao Perfil do Usurio, que devem ser
   * repassadas explicitamente.
   *
   * @param key Chave que ser usada para criar e armazenar o usurio.
   * @param un A Unidade de Negcio associada ao usurio.
   * @param role O Perfil associado ao usurio.
   *
   * @return User O usurio criado.
   *
   * @throws Exception .
   */
  protected static User createTestUser(String key, UN un, Role role)
    throws Exception {
    if ((key == null) || (un == null) || (role == null)) {
      return null;
    }
    return _createUser(key, un.getId(), key, key, role.getId(), key);
  }

  /**
   * Cria um Usurio fictcio para testes, usando para simplificao a chave
   * repassada em todos os campos de valores do Usurio.
   *
   * @param key Chave que ser usada para criar e armazenar o usurio.
   *
   * @return User O usurio criado.
   *
   * @throws Exception .
   */
  protected static User createTestUser(String key) throws Exception {
    if (key == null) {
      return null;
    }
    UN un = createTestUN(key);
    if (un == null) {
      return null;
    }
    Role role = createTestRole(key);
    if (role == null) {
      return null;
    }
    return _createUser(key, un.getId(), key, key, role.getId(), key);
  }

  /**
   * Mtodo interno para facilitar a criao de usurios de teste.
   *
   * @param login .
   * @param unId .
   * @param name .
   * @param mail .
   * @param roleId .
   * @param password .
   *
   * @return .
   *
   * @throws Exception .
   */
  private static User _createUser(
    String login, Object unId, String name, String mail, Object roleId,
    String password) throws Exception {
    //??Alterado para mudana de permission
    System.out.println("??CSBaseTestCase:150");
    UserInfo userInfo = new UserInfo(login, unId, name, mail, null, null);
    userInfo.passwordDigest = password;
    User testUser = User.createUser(userInfo);
    return testUser;
  }

  /**
   * Cria um Projeto fictcio para testes, usando para simplificao a chave
   * repassada em todos os campos do Projeto, com exceo do Usurio, que deve
   * ser repassado explicitamente.
   *
   * @param key Chave que ser usada para criar e armazenar o projeto.
   * @param user O usurio que criou o projeto.
   *
   * @return CommonClientProject O Projeto criado.
   *
   * @throws Exception .
   */
  protected static CommonClientProject createTestProject(String key, User user)
    throws Exception {
    if ((key == null) || (user == null)) {
      return null;
    }
    CommonProjectInfo projInfo = new CommonProjectInfo();
    projInfo.userId = user.getId();
    projInfo.name = key;
    projInfo.description = key;
    CommonClientProject project = CommonClientProject.createProject(projInfo);
    project.installObserver(new ProjectObserver(project));
    return project;
  }

  /**
   * Mtodo utilitrio para facilitar a verificao de arquivos no sistema de
   * arquivos. Caso o arquivo no exista, dispara um AssertionFailedError.
   *
   * @param path Caminho absoluto para o arquivo/diretrio, incluindo o nome do
   *        prprio.
   */
  public void assertFileExists(String path) {
    assertTrue(getFileFromPath(path).exists());
  }

  /**
   * Mtodo interno para reuso dos mtodos assertFileExists e
   * assertFileDoesNotExist. Converte uma String com um caminho abstrato,
   * representado por '|', para um caminho real, utilizando o separador de
   * arquivos do sistema operacional.
   *
   * @param path Caminho absoluto para o arquivo/diretrio, incluindo o nome do
   *        prprio.
   *
   * @return Um File representando o arquivo no sistema de arquivos.
   */
  private File getFileFromPath(String path) {
    String fileName = path.replace('|', File.separator.charAt(0));
    return new File(fileName);
  }

  /**
   * Mtodo utilitrio para facilitar a verificao de arquivos no sistema de
   * arquivos. Caso o arquivo exista, dispara um AssertionFailedError.
   *
   * @param path Caminho absoluto para o arquivo/diretrio, incluindo o nome do
   *        prprio.
   */
  public void assertFileDoesNotExist(String path) {
    assertFalse(getFileFromPath(path).exists());
  }

  /**
   * Garante a contagem de elementos em uma determinada base (qual base 
   * definida pela implementao do mtodo getAllElements.
   *
   * @param expectedSize O nmero esperado de elementos na lista.
   *
   * @throws Exception .
   */
  protected void assertListSize(int expectedSize) throws Exception {
    Vector theList = getAllElements();
    assertEquals(expectedSize, theList.size());
  }

  /**
   * Garante que a determinada lista de elementos da base esteja zerada  aps
   * cada teste (qual base  definida pela implementao do mtodo
   * cleanElements. Caso contrrio, exibe uma mensagem e tenta remover as
   * restantes. Esse mtodo foi criado para evitar que possveis excees nos
   * TestCases fossem perdidas no caso de uma nova exceo ocorrer no
   * terminate().
   */
  private void ensureClean() {
    try {
      Vector remaining = getAllElements();
      if ((remaining != null) && (remaining.size() > 0)) {
        System.err.println(
          "Restaram <" + remaining.size() +
          "> elemento(s) na base. Removendo...");
        cleanElements(remaining);
      }
    }
    catch (Exception e) {
      System.err.println("No foi possvel limpar a base de dados.");
      e.printStackTrace();
    }
  }

  protected Vector getAllElements() throws Exception {
    return null;
  }

  protected void cleanElements(Vector elements) throws Exception {}

  protected String getClassName() {
    return "TEST";
  }
}
